/**
 * JTextAreaOutputStream: output stream which redirects
 * the output to a Jtext area. It's a bit deprecated
 * but very useful if you want to redirect System.out
 * to a PrintStream(thiskindofclass).
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.2 990829: use swing 1.1.1
 * 1.1.1 990801: fixed the internal buffer
 *      a stupid unefficiency (result of a copy-paste)
 *	and add synchronized
 * 1.1: added internal buffer, more
 *      efficient than wrapping this class
 *      with a BufferedOutputStream
 * 1.0: first version
 */

package adavid.io;

import java.io.OutputStream;

import javax.swing.JTextArea;

/**
 * JTextAreaOutputStream
 * output stream JTextArea (from JStringOutputStream)
 * as JTextArea is slow, it could be nice to use this class
 * inside a BufferedOutputStream to enhance performance
 */
public class JTextAreaOutputStream extends OutputStream {

	private final JTextArea _textArea;
	private final int _maxBuffSize;
	private final StringBuffer _stringBuffer = new StringBuffer();

	public JTextAreaOutputStream(JTextArea ta) {
		this(ta, -1);
	}

	public JTextAreaOutputStream(JTextArea ta, int maxBuffSize) {
		_textArea = ta;
		_textArea.setWrapStyleWord(false);
		_textArea.setEditable(false);
		
		_maxBuffSize = maxBuffSize;
	}

	private void flushBufferToTextArea() {
		boolean onLastPosition = getTextArea().getCaretPosition() == getTextArea().getText().length();

		getTextArea().append(_stringBuffer.toString());

		if (onLastPosition) {
			getTextArea().setCaretPosition(getTextArea().getText().length());
		}

		_stringBuffer.setLength(0);
	}

	/**
	 * implements the stream: writes the byte to the string buffer
	 */
	public synchronized void write(int b) {
		if ((_stringBuffer.length() >= _maxBuffSize) || (b == '\n')) 
			flushBufferToTextArea();

		_stringBuffer.append((char) b);
	}

	/**
	 * returns the written string
	 */
	public synchronized String toString() {
		flushBufferToTextArea();

		return getTextArea().getText();
	}

	/**
	 * overrides OutputStream.flush()
	 */
	public synchronized void flush() {
		flushBufferToTextArea();
	}

	public JTextArea getTextArea() {
		return _textArea;
	}

}