/**
 * JTextAreaWriter: writer which redirects
 * the output to a text area.
 * @version 1.1
 * @author Alexandre David
 */

/**
 * version history:
 * 1.0 990801: first version
 * 1.1 990829: use swing 1.1.1
 */

package adavid.io;

import java.io.Writer;

import javax.swing.JTextArea;

/**
 * TextAreaWriter
 * Writer to a TextArea
 */
public class JTextAreaWriter extends Writer {

	private final JTextArea _jta;
	private final int _maxBuffSize;
	private final StringBuffer _stringBuffer = new StringBuffer();

	public JTextAreaWriter(JTextArea ta) {
		this(ta, -1);
	}

	public JTextAreaWriter(JTextArea ta, int maxBuffSize) {
		_jta = ta;

		_jta.setWrapStyleWord(false);
		_maxBuffSize = maxBuffSize;
	}

	private void flushBufferToTextArea() {
		boolean onLastPosition = _jta.getCaretPosition() == _jta.getText().length();

		_jta.append(_stringBuffer.toString());

		if (onLastPosition) {
			_jta.setCaretPosition(_jta.getText().length());
		}

		_stringBuffer.setLength(0);
	}

	/**
	 * returns the written string
	 */
	public synchronized String toString() {
		flushBufferToTextArea();

		return _jta.getText();
	}

	/**
	 * overrides OutputStream.flush()
	 */
	public synchronized void flush() {
		flushBufferToTextArea();
	}
	
   /**
	 * implements the stream: writes the byte to the string buffer
	 */
	public synchronized void write(int b) {
		if ((_stringBuffer.length() >= _maxBuffSize) || (b == '\n')) 
			flushBufferToTextArea();

		_stringBuffer.append((char) b);
	}

	public synchronized void write(char[] cbuf, int off, int len) {
		for (int i = off; i < len; i++) {
			write(cbuf[i]);
		}
	}

	public void close() {
		//nothing
	}

}