package com.geocities.sbridgesgeo.debug;

/*
 * Copyright (C) 2000 Sean Bridges
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.lang.reflect.Field;
import java.util.Iterator;
import java.util.TreeSet;

/**
 * A node created from an object.
 * The most generic type of not primitive node, classes such as ArrayNode and
 * are more specific.
 *
 * @author Sean Bridges
 * @see <a href="http://www.geocities.com/sbridges.geo">more info</a>
 * @version 0.1
 */

class ObjectNode extends ComplexNode {

	//------------------------------
	//instance variables

	private Value value;
	boolean isNull; //mark wether we were created as a null
	Class myClass; //our class, used to check validity
	TreeSet fieldsAndNames = new TreeSet(new FieldAndNameComparator());

	//------------------------------
	//constructors

	ObjectNode(Value value, String name, InspectorNode parent) {

		super(parent, name);
		this.value = value;
		init(value.getValue());
	}
	/**
	 * Create a new Object node with a null instance
	 */
	ObjectNode(String name, InspectorNode parent) {
		super(parent, name);
	}
	//---------------------------
	//child generation

	protected InspectorNode generateChild(int index) {

		Iterator it = fieldsAndNames.iterator();
		int i = 0;
		while ((i < index) && (it.hasNext())) {

			it.next();
			i++;
		}

		if (!it.hasNext()) {
			return null;

		} else {
			FieldAndName fn = (FieldAndName) it.next();
			return createNode(fn.field, this, value, fn.name);
		}
	}
	public Object getValue() {
		return value.getValue();
	}
	//---------------------------
	//printing

	public String getValueString() {

		Object instance = value.getValue();
		if (instance == null) {
			return "<null>";
		}
		return instance.getClass().getName() + "\n" + instance.toString();

	}
	//------------------------------
	//instance methods

	/**
	 * To prevent cycles we need a collection of all the objects that we have already
	 * mapped.
	 */
	private void init(Object instance) {
		if (instance == null) {
			isNull = true;
			return;
		}
		isNull = false;
		myClass = instance.getClass();

		//create our descendents

		String pre = "";
		Class aClass = myClass;
		do {
			Field[] myFields = aClass.getDeclaredFields();

			for (int i = 0; i < myFields.length; i++) {
				fieldsAndNames.add(new FieldAndName(myFields[i], pre));
			} //end for all myFields

			if (aClass != Object.class) {
				aClass = aClass.getSuperclass();
				String newName = aClass.getName();
				int dotPos = newName.lastIndexOf('.');
				if (dotPos < 0)
					dotPos = 0;
				else
					dotPos++;
				newName = newName.substring(dotPos, newName.length());
				pre = newName + "." + pre;
			}

		} while (aClass != Object.class);

		setNumberOfChildren(fieldsAndNames.size());

	} //end init
	public boolean isValid() {
		if (super.isValid()) {
			Object myValue = value.getValue();
			if (myValue == null) {
				return isNull;
			} else {
				return myValue.getClass() == myClass;
			}

		} else {
			return false;
		}
	}

	public int getChildCount() {
		// sometimes super.getChildCount() will return more than fieldsAndNames.size()
		// don't know why?!
		return Math.min(super.getChildCount(), fieldsAndNames.size());
	}
}