package com.ibm.aboutbeaneditor;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.ImageProducer;
import java.beans.*;
import java.net.URL;
import java.util.Vector;

// Referenced classes of package com.ibm.aboutbeaneditor:
//            ExtBaseTabbedPanel, ExtTabPanel, ImageComponent

public class AboutThisBeanEditor extends Panel implements PropertyEditor, ComponentListener {

	static final String COPYRIGHT = "(C)Copyright IBM Corp. 1996, 1998";
	private PropertyChangeSupport propSupport;
	private String aboutThisBeanString;
//	private float xPoints;
//	private float yPoints;
	private ExtTabPanel tabPane;
	private String beanName;
	private ImageComponent infoImage;
	private Label pathLabel1;
	private Label pathLabel2;
	private Label nameLabel;
	private Label versionLabel;
	private ScrollPane wiringPane;
	private boolean onlyImages;

	public AboutThisBeanEditor() {
		propSupport = new PropertyChangeSupport(this);
		beanName = "";
		pathLabel1 = new Label();
		pathLabel2 = new Label();
		nameLabel = new Label();
		versionLabel = new Label();
		wiringPane = new ScrollPane(0);
		onlyImages = false;
		initialize();
	}

	public void addNotify() {
		if (tabPane == null)
			initializeTabPane();
		Object obj;
		for (obj = this; obj != null && !(obj instanceof Window); obj = ((Component) obj).getParent());
		if (obj instanceof Frame) {
			((Frame) obj).setResizable(true);
			((Frame) obj).setTitle("About the " + beanName + " bean");
		} else if (obj instanceof Dialog)
			 ((Dialog) obj).setTitle("About the " + beanName + " bean");
		super.addNotify();
	}

	public void addPropertyChangeListener(PropertyChangeListener propertychangelistener) {
		propSupport.addPropertyChangeListener(propertychangelistener);
	}

	public void componentHidden(ComponentEvent componentevent1) {
	}

	public void componentMoved(ComponentEvent componentevent1) {
	}

	public void componentResized(ComponentEvent componentevent) {
		Dimension dimension = getSize();
		Insets insets = tabPane.getInsets();
		tabPane.setBounds(0, 0, dimension.width, dimension.height);
		dimension = tabPane.getSize();
		for (int i = 0; i < tabPane.getTabsCount(); i++) {
			Component component = tabPane.getTabPanel(i);
			component.setSize(
				(dimension.width - (insets.left + insets.right)) + 1,
				dimension.height - (insets.top + insets.bottom));
		}

		dimension = tabPane.getTabPanel(0).getSize();
		int j = insets.left / 2;
		if (!onlyImages) {
			FontMetrics fontmetrics = nameLabel.getFontMetrics(nameLabel.getFont());
			int l = fontmetrics.getHeight();
			int i1 = fontmetrics.stringWidth(nameLabel.getText()) + 10;
			nameLabel.setBounds(j, j, i1, l);
			FontMetrics fontmetrics1 = versionLabel.getFontMetrics(versionLabel.getFont());
			versionLabel.setBounds(
				j + i1,
				(j + l) - fontmetrics.getDescent() - fontmetrics1.getAscent(),
				dimension.width - 2 * j - i1,
				fontmetrics1.getHeight());
			int j1 = j + l + 5;
			fontmetrics = pathLabel1.getFontMetrics(pathLabel1.getFont());
			l = fontmetrics.getHeight();
			i1 = infoImage.getSize().width;
			pathLabel1.setSize(dimension.width - (2 * j + i1 + 5), l + 2);
			dividePathText();
			if (pathLabel2.isVisible()) {
				pathLabel1.setLocation(j + i1 + 4, dimension.height - 2 * l - 5);
				pathLabel2.setLocation(j + i1 + 4, dimension.height - l - 3);
			} else {
				pathLabel1.setLocation(j + i1 + 4, dimension.height - (j + i1) / 2 - fontmetrics.getAscent());
			}
			infoImage.setLocation(j, pathLabel1.getLocation().y + (l - i1) / 2 + 1);
			wiringPane.setBounds(j, j1, (dimension.width - 2 * j) + 1, pathLabel1.getLocation().y - j1 - j);
			wiringPane.validate();
			wiringPane.doLayout();
			pathLabel1.repaint();
			pathLabel2.repaint();
			nameLabel.repaint();
			versionLabel.repaint();
		}
		for (int k = onlyImages ? 0 : 1; k < tabPane.getTabsCount(); k++) {
			Panel panel = (Panel) tabPane.getTabPanel(k);
			Component component1 = panel.getComponent(0);
			if (panel.getComponentCount() == 1) {
				component1.setBounds(j, j, (dimension.width - j * 2) + 1, dimension.height - j * 2);
			} else {
				component1.setBounds(j, j, dimension.width / 3, dimension.height - j * 2);
				component1 = panel.getComponent(1);
				component1.setBounds(
					j * 2 + dimension.width / 3,
					j,
					(dimension.width - j * 3 - dimension.width / 3) + 3,
					dimension.height - j * 2);
			}
			component1.validate();
			component1.doLayout();
		}

	}

	public void componentShown(ComponentEvent componentevent1) {
	}

	private void dividePathText() {
		String s = pathLabel1.getText();
		if (pathLabel2.getText().length() > 0)
			s = s + " " + pathLabel2.getText();
		FontMetrics fontmetrics = pathLabel1.getFontMetrics(pathLabel1.getFont());
		int i = pathLabel1.getSize().width;
		if (fontmetrics.stringWidth(s) < i) {
			pathLabel1.setText(s);
			pathLabel2.setText("");
			pathLabel2.setVisible(false);
			return;
		}
		int j;
		for (j = s.lastIndexOf(' ');
			fontmetrics.stringWidth(s.substring(0, j)) > i;
			j = s.lastIndexOf(' ', j - 1));
		if (j == -1)
			j = s.indexOf(' ');
		pathLabel1.setText(s.substring(0, j));
		pathLabel2.setText(s.substring(j + 1));
		pathLabel2.setSize(pathLabel1.getSize());
		pathLabel2.setVisible(true);
	}

	public String getAsText() {
		return "About this bean...";
	}

	public Component getCustomEditor() {
		return this;
	}

	public String getJavaInitializationString() {
		return "";
	}

	private String getPageLabel(String s) {
		int i = s.lastIndexOf('.');
		if (i > -1)
			s = s.substring(0, i);
		if (s.indexOf(beanName) == 0)
			s = s.substring(beanName.length());
		if (s.indexOf('_') == 0)
			s = s.substring(1);
		return s.replace('_', ' ');
	}

	private Panel getPanel(String s, Image image) {
		Panel panel = new Panel(null);
		if (s != "" && s != null) {
			TextArea textarea = new TextArea(s, 20, 70, 1);
			textarea.setEditable(false);
			panel.add(textarea);
		}
		if (image != null) {
			ScrollPane scrollpane = new ScrollPane(0);
			scrollpane.setBackground(Color.white);
			scrollpane.add(new ImageComponent(image));
			panel.add(scrollpane);
			scrollpane.getHAdjustable().setUnitIncrement(15);
			scrollpane.getVAdjustable().setUnitIncrement(15);
			scrollpane.setVisible(false);
		}
		return panel;
	}

	public String[] getTags() {
		return null;
	}

	//private String getTextFromFile(Class class1, String s) {
	//   InputStream inputstream = class1.getResourceAsStream("helpresources/" + s);
	//   if (inputstream != null)
	//      try {
	//         byte abyte0[] = new byte[inputstream.available()];
	//         inputstream.read(abyte0);
	//         inputstream.close();
	//         return new String(abyte0);
	//      } catch (Exception _ex) {
	//      }
	//   return "";
	//}

	public Object getValue() {
		return aboutThisBeanString;
	}

	//private int getX(double d) {
	//   return (int) (d * (double) xPoints);
	//}
	//
	//
	//private int getY(double d) {
	//   return (int) (d * (double) yPoints);
	//}

	private void initialize() {
		setLayout(null);
		setBackground(Color.lightGray);
		setSize(570, 330);
		addComponentListener(this);
	}

	private void initializeTabPane() {
		tabPane = new ExtTabPanel();
		add(tabPane);
		int i = aboutThisBeanString.indexOf('^');
		Class class1 = getClass();
		try {
			class1 = Class.forName(aboutThisBeanString.substring(0, i));
		} catch (Exception _ex) {
		}
		beanName = class1.getName().substring(class1.getName().lastIndexOf('.') + 1);
		Vector vector = new Vector();
		int j = aboutThisBeanString.indexOf('^', i + 1);
		if (i > 0 && j > 0 && aboutThisBeanString.substring(j - 4, j).equalsIgnoreCase(".gif")) {
			onlyImages = true;
			do {
				vector.addElement(aboutThisBeanString.substring(i + 1, j));
				i = j;
			} while ((j = aboutThisBeanString.indexOf('^', i + 1)) > 0);
			vector.addElement(aboutThisBeanString.substring(i + 1));
			for (int k = 0; k < vector.size(); k++) {
				String s = (String) vector.elementAt(k);
				Image image = loadImage(class1, s);
				if (image != null)
					tabPane.addTabPanel(getPageLabel(s), true, getPanel("", image));
				if (k == 0) {
					Dimension dimension = (new ImageComponent(image)).getPreferredSize();
					Insets insets = tabPane.getInsets();
					setSize(
						dimension.width + insets.left * 2 + insets.right + 4,
						dimension.height + insets.left + insets.top + insets.bottom + 4);
				}
			}

			tabPane.setCurrentPanelNdx(0);
			componentResized(new ComponentEvent(this, 101));
			return;
		}
		Panel panel = new Panel(null);
		Image image4 = loadImage(getClass(), "Info16.gif");
		if (image4 == null)
			pathLabel2.setText(getClass().getName());
		infoImage = new ImageComponent(image4, false);
		infoImage.setSize(infoImage.getPreferredSize().width, infoImage.getPreferredSize().height);
		panel.add(infoImage);
		if (i > 0 && j > 0)
			nameLabel.setText(aboutThisBeanString.substring(i + 1, j) + " Bean");
		panel.add(nameLabel);
		i = aboutThisBeanString.indexOf('^', j + 1);
		if (i > 0)
			versionLabel.setText("Version " + aboutThisBeanString.substring(j + 1, i));
		while ((j = aboutThisBeanString.indexOf('^', i + 1)) > 0) {
			vector.addElement(aboutThisBeanString.substring(i + 1, j));
			i = j;
		}
		pathLabel1.setText("More info in " + aboutThisBeanString.substring(i + 1));
		panel.add(versionLabel);
		panel.add(pathLabel1);
		panel.add(pathLabel2);
		pathLabel1.setFont(new Font("Dialog", 0, 11));
		pathLabel2.setFont(new Font("Dialog", 0, 11));
		nameLabel.setFont(new Font("TimesRoman", 1, 26));
		versionLabel.setFont(new Font("TimesRoman", 1, 14));
		if (vector.isEmpty()) {
			Image image1 = loadImage(class1, beanName + "About.gif");
			if (image1 != null) {
				wiringPane.setBackground(Color.white);
				wiringPane.add(new ImageComponent(image1));
				wiringPane.getHAdjustable().setUnitIncrement(15);
				wiringPane.getVAdjustable().setUnitIncrement(15);
			}
			panel.add(wiringPane);
			tabPane.addTabPanel("About bean", true, panel);
			image1 = loadImage(class1, beanName + "Features.gif");
			if (image1 != null)
				tabPane.addTabPanel("Features", true, getPanel("", image1));
			image1 = loadImage(class1, beanName + "Wiring.gif");
			if (image1 != null)
				tabPane.addTabPanel("Wiring Example", true, getPanel("", image1));
		} else {
			String s1 = (String) vector.elementAt(0);
			Image image2 = loadImage(class1, s1);
			if (image2 != null) {
				wiringPane.setBackground(Color.white);
				wiringPane.add(new ImageComponent(image2));
				wiringPane.getHAdjustable().setUnitIncrement(15);
				wiringPane.getVAdjustable().setUnitIncrement(15);
			}
			panel.add(wiringPane);
			tabPane.addTabPanel(getPageLabel(s1), true, panel);
			for (int l = 1; l < vector.size(); l++) {
				String s2 = (String) vector.elementAt(l);
				Image image3 = loadImage(class1, s2);
				if (image3 != null)
					tabPane.addTabPanel(getPageLabel(s2), true, getPanel("", image3));
			}

		}
		tabPane.setCurrentPanelNdx(0);
		componentResized(new ComponentEvent(this, 101));
	}

	public boolean isPaintable() {
		return true;
	}

	private Image loadImage(Class class1, String s) {
		try {
			URL url = null;
			if (class1 == getClass())
				url = class1.getResource(s);
			else
				url = class1.getResource("helpresources/" + s);
			Toolkit toolkit = Toolkit.getDefaultToolkit();
			return toolkit.createImage((ImageProducer) url.getContent());
		} catch (Exception _ex) {
			return null;
		}
	}

	public void paintValue(Graphics g, Rectangle rectangle) {
		int i = (rectangle.height - g.getFontMetrics().getHeight()) / 2 + g.getFontMetrics().getAscent();
		g.drawString("About this bean...", rectangle.x + 2, i);
	}

	public void removePropertyChangeListener(PropertyChangeListener propertychangelistener) {
		propSupport.removePropertyChangeListener(propertychangelistener);
	}

	public void setAsText(String s1) {
	}

	public void setValue(Object obj) {
		aboutThisBeanString = (String) obj;
	}

	//private void setXYPoints(Dimension dimension, Point point) {
	//   xPoints = dimension.width / point.x;
	//   yPoints = dimension.height / point.y;
	//}

	public boolean supportsCustomEditor() {
		return true;
	}
}