package com.ibm.aboutbeaneditor;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public abstract class ExtBaseTabbedPanel extends Panel implements MouseListener, ActionListener {

   public static final int TOP = 0;
   public static final int BOTTOM = 1;
   public static final int LEFT = 2;
   public static final int RIGHT = 3;
   public static final int ROUNDED = 0;
   public static final int SQUARE = 1;
   private int TF_LEFT;
   private int TF_RIGHT;
   private int TF_TOP;
   private int TF_BOTTOM;
   private int TF_BTN_HEIGHT;
   private Vector vLabels;
   private Vector vEnabled;
   private Vector vPolys;
   private Vector vNumbers;
   private Vector vLbWidth;
   protected int curIndex;
   private Font fReg;
   private Font fSel;
   private Component userPanel;
   private int iTabsPosition;
   private int iTabsStyle;
   private boolean iMultiLevel;
   private int levelCount;
   private int osAdjustment;
   private int firstVisibleTab;
   private Button dbLeft;
   private Button dbRight;
   private Button dbUp;
   private Button dbDown;
   private Polygon nullPoly;
   private int lastWidth;
   private int lastHeight;

public ExtBaseTabbedPanel() {
   this(0, 0, false);
}


public ExtBaseTabbedPanel(int i) {
   this(i, 0, false);
}


public ExtBaseTabbedPanel(int i, int j) {
   this(i, j, false);
}


public ExtBaseTabbedPanel(int i, int j, boolean flag) {
   TF_LEFT = 9;
   TF_RIGHT = -9;
   TF_TOP = 30;
   TF_BOTTOM = -9;
   TF_BTN_HEIGHT = 20;
   curIndex = -1;
   iTabsPosition = 0;
   iTabsStyle = 0;
   iMultiLevel = false;
   levelCount = 1;
   lastWidth = -1;
   lastHeight = -1;
   vLabels = new Vector();
   vEnabled = new Vector();
   vPolys = new Vector();
   vNumbers = new Vector();
   vLbWidth = new Vector();
   fReg = new Font("Helvetica", 0, 12);
   fSel = new Font("Helvetica", 1, 12);
   if (i == 0 && j == 0)
      iMultiLevel = flag;
   setTabsInfo(i, j);
   if (System.getProperty("os.name").startsWith("S"))
      osAdjustment = -1;
   else
      osAdjustment = 0;
   super.setLayout(null);
   dbLeft = new Button("<");
   dbRight = new Button(">");
   dbUp = new Button("/\\");
   dbDown = new Button("\\/");
   dbLeft.addActionListener(this);
   dbRight.addActionListener(this);
   dbUp.addActionListener(this);
   dbDown.addActionListener(this);
   super.add(dbLeft, -1);
   super.add(dbRight, -1);
   super.add(dbUp, -1);
   super.add(dbDown, -1);
   addMouseListener(this);
   nullPoly = new Polygon();
   nullPoly.addPoint(0, 0);
   nullPoly.addPoint(1, 1);
   nullPoly.addPoint(0, 0);
}


public ExtBaseTabbedPanel(boolean flag) {
   this(0, 0, flag);
}


public void actionPerformed(ActionEvent actionevent) {
   if (actionevent.getSource() == dbLeft || actionevent.getSource() == dbUp)
      if (--firstVisibleTab < 0) {
         firstVisibleTab = 0;
         return;
      } else {
         repaint();
         return;
      }
   int i = vLabels.size();
   if (++firstVisibleTab == i) {
      firstVisibleTab--;
      return;
   } else {
      repaint();
      return;
   }
}


public Component add(Component component) {
   return component;
}


public synchronized Component add(Component component, int i) {
   return component;
}


public synchronized Component add(String s, Component component) {
   return component;
}


public int addTab(String s, boolean flag) {
   vLabels.addElement(s);
   vEnabled.addElement(Boolean.valueOf(flag));
   if (iTabsPosition == 3)
      setRight();
   int i = vLabels.size() - 1;
   if (curIndex == -1 && flag)
      showTab(i);
   return i;
}


public void doLayout() {
   if (iMultiLevel)
      setLevelCount();
   Rectangle rectangle = getBounds();
   int i = (rectangle.width - TF_LEFT) + TF_RIGHT;
   if (i < 0)
      return;
   int j = (rectangle.height - TF_TOP) + TF_BOTTOM;
   if (iTabsPosition == 1)
      j -= TF_BTN_HEIGHT;
   if (j < 0)
      return;
   int k = TF_LEFT;
   int l = 0;
   l = TF_TOP;
   if (userPanel != null) {
      userPanel.setBounds(k + 3, l + 3, i - 6, j - 5);
      userPanel.invalidate();
      userPanel.validate();
      if ((userPanel instanceof Canvas) || (userPanel instanceof Panel)) {
         userPanel.repaint();
         return;
      }
      repaint();
   }
}


public void enableTab(boolean flag, int i) {
   if (i < 0 || i >= vEnabled.size() || i == curIndex)
      return;
   try {
      vEnabled.setElementAt(Boolean.valueOf(flag), i);
      repaint();
      return;
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return;
   }
}


public int getCurrentTabIndex() {
   return curIndex;
}


public Insets getInsets() {
   Insets insets = super.getInsets();
   int i = insets.left + TF_LEFT + 3;
   int j = (insets.right + 6) - TF_RIGHT;
   int k = insets.top;
   int l = insets.bottom;
   if (iTabsPosition == 0) {
      k += TF_TOP + 3;
      l += 5 - TF_BOTTOM;
   } else {
      k += (TF_TOP - TF_BTN_HEIGHT) + 3;
      l += (TF_BTN_HEIGHT + 5) - TF_BOTTOM;
   }
   return new Insets(k, i, l, j);
}


public synchronized String getLabel(int i) {
   if (i < 0 || i >= vLabels.size())
      return "";
   try {
      return (String) vLabels.elementAt(i);
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return "";
   }
}


public Dimension getMinimumSize() {
   if (userPanel != null) {
      Dimension dimension = userPanel.getMinimumSize();
      Insets insets = getInsets();
      return new Dimension(dimension.width + insets.left + insets.right, dimension.height + insets.top + insets.bottom);
   } else {
      return new Dimension(100, 100);
   }
}


public boolean getMultiLevel() {
   return iMultiLevel;
}


public Dimension getPreferredSize() {
   Dimension dimension = getSize();
   Dimension dimension1 = getMinimumSize();
   return new Dimension(Math.max(dimension.width, dimension1.width), Math.max(dimension.height, dimension1.height));
}


public int getTabsPosition() {
   return iTabsPosition;
}


public int getTabsStyle() {
   return iTabsStyle;
}


public void mouseClicked(MouseEvent mouseevent1) {
}


public void mouseEntered(MouseEvent mouseevent1) {
}


public void mouseExited(MouseEvent mouseevent1) {
}


public void mousePressed(MouseEvent mouseevent) {
   int i = vPolys.size();
   for (int j = 0; j < i; j++)
      try {
         Polygon polygon = (Polygon) vPolys.elementAt(j);
         if (polygon != nullPoly && polygon.contains(mouseevent.getX(), mouseevent.getY()))
            showTab(j);
      } catch (ArrayIndexOutOfBoundsException _ex) {
      }

}


public void mouseReleased(MouseEvent mouseevent1) {
}


public synchronized void paint(Graphics g) {
   if (iMultiLevel)
      setLevelCount();
   Rectangle rectangle = getBounds();
   int i1 = rectangle.width + TF_RIGHT;
   FontMetrics fontmetrics = getFontMetrics(fReg);
   FontMetrics fontmetrics1 = getFontMetrics(fSel);
   int j1 = (rectangle.width - TF_LEFT) + TF_RIGHT;
   if (j1 < 0)
      return;
   int k1 = (rectangle.height - TF_TOP) + TF_BOTTOM;
   if (iTabsPosition == 1)
      k1 -= TF_BTN_HEIGHT;
   if (k1 < 0)
      return;
   if (rectangle.width > lastWidth && (iTabsPosition == 0 || iTabsPosition == 1))
      firstVisibleTab = 0;
   if (rectangle.height > lastHeight && (iTabsPosition == 2 || iTabsPosition == 3))
      firstVisibleTab = 0;
   lastWidth = rectangle.width;
   lastHeight = rectangle.height;
   int l1 = TF_LEFT;
   Color color = g.getColor();
   g.setColor(getBackground());
   g.fillRect(0, 0, rectangle.width, rectangle.height);
   int i2 = TF_TOP;
   g.setColor(Color.white);
   g.drawLine(l1, i2, (l1 + j1) - 1, i2);
   g.drawLine(l1, i2, l1, (i2 + k1) - 1);
   g.setColor(Color.gray);
   g.drawLine(l1 + 2, (i2 + k1) - 2, (l1 + j1) - 2, (i2 + k1) - 2);
   g.drawLine((l1 + j1) - 2, i2 + 2, (l1 + j1) - 2, (i2 + k1) - 2);
   g.setColor(Color.black);
   g.drawLine(l1 + 1, (i2 + k1) - 1, (l1 + j1) - 1, (i2 + k1) - 1);
   g.drawLine((l1 + j1) - 1, i2 + 1, (l1 + j1) - 1, (i2 + k1) - 1);
   int j2 = 0;
   int k2 = 0;
   int l2 = 0;
   int i3 = 0;
   int j3 = TF_LEFT + 8;
   int k3 = TF_LEFT;
   int l3 = i2;
   int i4 = 1;
   int j4 = 0;
   int k4 = 0;
   int l4 = 0;
   int k = vLabels.size();
   vPolys.removeAllElements();
   Font font = g.getFont();
   if (iMultiLevel) {
      int i5 = vNumbers.size();
      for (int i = i5 - 1; i >= 0; i--) {
         if (curIndex < ((Integer) vNumbers.elementAt(i)).intValue())
            continue;
         i4 = (i5 - i) % i5 + 1;
         l3 -= i4 * TF_BTN_HEIGHT;
         break;
      }

   }
   int j;
   for (j = 0; j < firstVisibleTab; j++)
      vPolys.addElement(nullPoly);

   if (j > 0)
      k3 += 2;
   if (iTabsPosition == 3)
      k3 = i1;
   for (; j < k; j++) {
      Polygon polygon = new Polygon();
      try {
         String s = (String) vLabels.elementAt(j);
         int l;
         if (j == curIndex)
            l = fontmetrics1.stringWidth(s);
         else
            l = fontmetrics.stringWidth(s);
         if (iMultiLevel) {
            l4 = l;
            l = ((Integer) vLbWidth.elementAt(j)).intValue();
         }
         switch (iTabsPosition) {
            case 0 : // '\0'
               if (!iMultiLevel) {
                  k2 = TF_TOP - TF_BTN_HEIGHT;
                  l2 = TF_TOP - 1;
               } else {
                  k2 = l3;
                  l2 = (k2 + TF_BTN_HEIGHT) - 1;
               }
               if (iTabsStyle == 0) {
                  j2 = k3 + 2;
                  j3 = j2 + l + 13;
               } else {
                  j2 = j3 - 7;
                  j3 = j2 + l + 28;
               }
               break;

            case 1 : // '\001'
               k2 = rectangle.height + TF_BOTTOM + 1;
               l2 = (rectangle.height + TF_BOTTOM) - TF_BTN_HEIGHT;
               if (iTabsStyle == 0) {
                  j2 = k3 + 2;
                  j3 = j2 + l + 13;
               } else {
                  j2 = j3 - 7;
                  j3 = j2 + l + 28;
               }
               break;

            case 3 : // '\003'
               k2 = l3 + 3;
               l2 = k2 + TF_BTN_HEIGHT;
               j2 = i1;
               j3 = j2 + l + 13;
               break;

            case 2 : // '\002'
               k2 = l3 + 3;
               l2 = k2 + 40;
               j2 = 10;
               j3 = TF_LEFT - 1;
               break;
         }
         if ((iTabsPosition == 0 || iTabsPosition == 1) && !iMultiLevel && (j3 + 36) - TF_RIGHT > rectangle.width)
            break;
         if (iMultiLevel && j3 - TF_RIGHT > rectangle.width) {
            if (++i4 > levelCount)
               i4 = 1;
            l3 = i2 - i4 * TF_BTN_HEIGHT;
            k2 = l3;
            l2 = (k2 + TF_BTN_HEIGHT) - 1;
            if (iTabsStyle == 0) {
               j2 = TF_LEFT + 2;
               j3 = j2 + l + 13;
            } else {
               j2 = TF_LEFT + 1;
               j3 = j2 + l + 28;
            }
         }
         if ((iTabsPosition == 2 || iTabsPosition == 3) && (l2 + 36) - TF_BOTTOM > rectangle.height)
            break;
         switch (iTabsPosition) {
            case 0 : // '\0'
               if (iTabsStyle == 0) {
                  if (j == curIndex) {
                     k2 -= 3;
                     j2 -= 2;
                     g.setColor(getBackground());
                     g.drawRect(j2 + 2, k2 + 1, j3 - 2, k2 + 1);
                     g.drawLine(j2 + 2, k2 + 2, j3 - 2, k2 + 2);
                     j4 = j2;
                     k4 = j3;
                  }
                  g.setColor(Color.white);
                  if (curIndex == j + 1 && i4 == 1)
                     g.drawLine(j2 + 2, k2, j3 - 2, k2);
                  else
                     g.drawLine(j2 + 2, k2, j3, k2);
                  if (curIndex != j - 1 || i4 > 1) {
                     if (i4 == 2 && j2 >= j4 && j2 <= k4)
                        g.drawLine(j2, k2 + 2, j2, l2 - 3);
                     else
                        g.drawLine(j2, k2 + 2, j2, l2);
                     i3 = j2;
                  } else {
                     i3 = j2 + 1;
                  }
                  g.drawLine(j2 + 1, k2 + 1, j2 + 1, k2 + 1);
                  if (curIndex != j + 1 || i4 > 1) {
                     if (i4 == 2 && j3 >= j4 && j3 <= k4) {
                        g.setColor(Color.gray);
                        g.drawLine(j3, k2, j3, l2 - 3);
                        g.setColor(Color.black);
                        g.drawLine(j3 + 1, k2 + 2, j3 + 1, l2 - 3);
                     } else {
                        g.setColor(Color.gray);
                        g.drawLine(j3, k2, j3, l2);
                        g.setColor(Color.black);
                        g.drawLine(j3 + 1, k2 + 2, j3 + 1, l2);
                     }
                     k3 = j3;
                  } else {
                     k3 = j3 - 1;
                  }
               } else {
                  g.setColor(Color.gray);
                  g.drawLine(j2 + 9, k2, j3 - 9, k2);
                  g.setColor(Color.black);
                  if (j == 0 || j == curIndex) {
                     g.drawLine(j2, l2, j2 + 9, k2);
                     polygon.addPoint(j2, l2);
                  } else {
                     g.drawLine(j2 + 4, k2 + 9, j2 + 9, k2);
                     polygon.addPoint(j2 + 9, l2);
                     polygon.addPoint(j2 + 4, k2 + 9);
                  }
                  polygon.addPoint(j2 + 9, k2);
                  polygon.addPoint(j3 - 9, k2);
                  if (j + 1 == curIndex) {
                     g.drawLine(j3 - 5, k2 + 9, j3 - 9, k2);
                     polygon.addPoint(j3 - 5, k2);
                     polygon.addPoint(j3 - 9, l2);
                  } else {
                     g.drawLine(j3, l2, j3 - 9, k2);
                     polygon.addPoint(j3, l2);
                  }
                  if (j == 1 || j == curIndex)
                     polygon.addPoint(j2, l2);
                  else
                     polygon.addPoint(j2 + 9, l2);
               }
               break;

            case 2 : // '\002'
               if (j == curIndex) {
                  k2 -= 2;
                  j2 -= 3;
               }
               g.setColor(Color.white);
               g.drawLine(j2, k2 + 2, j2, l2);
               g.drawLine(j2 + 1, k2 + 1, j2 + 1, k2 + 1);
               if (curIndex != j - 1 || j == firstVisibleTab)
                  g.drawLine(j2 + 2, k2, j3, k2);
               if (curIndex != j + 1) {
                  g.setColor(Color.gray);
                  g.drawLine(j2, l2 + 1, j3, l2 + 1);
                  g.setColor(Color.black);
                  g.drawLine(j2 + 1, l2 + 2, j3, l2 + 2);
               }
               l3 = l2;
               break;

            case 3 : // '\003'
               if (j == curIndex) {
                  k2 -= 2;
                  j3 += 3;
               }
               g.setColor(Color.white);
               if (curIndex != j - 1)
                  g.drawLine(j2, k2, j3 - 2, k2);
               else if (j3 - 2 > k3)
                  g.drawLine(k3, k2, j3 - 2, k2);
               k3 = j3 + 2;
               if (curIndex == j) {
                  g.setColor(getBackground());
                  g.drawLine(j2, k2 + 1, j3, k2 + 1);
               }
               g.setColor(Color.gray);
               g.drawLine(j2, l2 + 1, j3, l2 + 1);
               g.drawLine(j3, k2 + 2, j3, l2);
               g.drawLine(j3 - 1, k2 + 1, j3 - 1, k2 + 1);
               g.setColor(Color.black);
               g.drawLine(j2 + 1, l2 + 2, j3 + 1, l2 + 2);
               g.drawLine(j3 + 1, k2 + 3, j3 + 1, l2 + 1);
               l3 = l2;
               break;

            case 1 : // '\001'
               if (iTabsStyle == 1) {
                  g.setColor(Color.gray);
                  g.drawLine(j2 + 9, k2, j3 - 9, k2);
                  g.setColor(Color.black);
                  if (j == 0 || j == curIndex) {
                     g.drawLine(j2, l2, j2 + 9, k2);
                     polygon.addPoint(j2, l2);
                  } else {
                     g.drawLine(j2 + 4, k2 - 9, j2 + 9, k2);
                     polygon.addPoint(j2 + 9, l2);
                     polygon.addPoint(j2 + 4, k2 - 9);
                  }
                  polygon.addPoint(j2 + 9, k2);
                  polygon.addPoint(j3 - 9, k2);
                  if (j + 1 == curIndex) {
                     g.drawLine(j3 - 5, k2 - 9, j3 - 9, k2);
                     polygon.addPoint(j3 - 5, k2);
                     polygon.addPoint(j3 - 9, l2);
                  } else {
                     g.drawLine(j3, l2, j3 - 9, k2);
                     polygon.addPoint(j3, l2);
                  }
                  if (j == 1 || j == curIndex)
                     polygon.addPoint(j2, l2);
                  else
                     polygon.addPoint(j2 + 9, l2);
               } else {
                  if (j == curIndex) {
                     k2 += 3;
                     j2 -= 2;
                  }
                  g.setColor(Color.white);
                  if (curIndex == j + 1)
                     g.drawLine(j2 + 2, k2, j3 - 2, k2);
                  else
                     g.drawLine(j2 + 2, k2, j3, k2);
                  if (curIndex != j - 1) {
                     g.drawLine(j2, k2 - 2, j2, l2);
                     i3 = j2;
                  } else {
                     i3 = j2 + 1;
                  }
                  g.drawLine(j2 + 1, k2 - 1, j2 + 1, k2 - 1);
                  if (curIndex != j + 1) {
                     g.setColor(Color.gray);
                     g.drawLine(j3, k2, j3, l2);
                     g.setColor(Color.black);
                     g.drawLine(j3 + 1, k2 - 2, j3 + 1, l2);
                     k3 = j3;
                  } else {
                     k3 = j3 - 1;
                  }
               }
               break;
         }
         if (j == curIndex) {
            g.setColor(getBackground());
            switch (iTabsPosition) {
               case 0 : // '\0'
                  l2++;
                  g.drawLine(j2 + 1, l2, j3, l2);
                  break;

               case 1 : // '\001'
                  l2--;
                  g.drawLine(j2 + 1, l2, j3, l2);
                  g.drawLine(j2 + 1, l2 - 1, j3, l2 - 1);
                  break;

               case 2 : // '\002'
                  g.drawLine(TF_LEFT, k2, TF_LEFT, l2);
                  break;

               case 3 : // '\003'
                  g.drawLine(j2 - 2, k2, j2 - 2, l2);
                  g.drawLine(j2 - 1, k2, j2 - 1, l2);
                  break;
            }
            g.setFont(fSel);
         } else {
            g.setFont(fReg);
         }
         if (iTabsStyle == 0)
            if (iTabsPosition == 3 || iTabsPosition == 2) {
               polygon.addPoint(j2, k2);
               polygon.addPoint(j3, k2);
               polygon.addPoint(j3, l2);
               polygon.addPoint(j2, l2);
               polygon.addPoint(j2, k2);
            } else {
               polygon.addPoint(i3, l2);
               polygon.addPoint(k3, l2);
               polygon.addPoint(k3, k2);
               polygon.addPoint(i3, k2);
               polygon.addPoint(i3, l2);
            }
         vPolys.addElement(polygon);
         Boolean boolean1 = (Boolean) vEnabled.elementAt(j);
         if (boolean1.booleanValue())
            g.setColor(Color.black);
         else
            g.setColor(Color.gray);
         switch (iTabsPosition) {
            default :
               break;

            case 0 : // '\0'
               if (iTabsStyle == 0) {
                  if (iMultiLevel)
                     g.drawString(s, j2 + ((l + 13) - l4) / 2, k2 + 15 + osAdjustment);
                  else
                     g.drawString(s, j2 + 8, k2 + 15 + osAdjustment);
               } else {
                  g.drawString(s, j2 + 14, (l2 - 4) + osAdjustment);
               }
               break;

            case 3 : // '\003'
               g.drawString(s, j2 + 8, k2 + 15 + osAdjustment);
               break;

            case 1 : // '\001'
               if (iTabsStyle == 0)
                  g.drawString(s, j2 + 8, (k2 - 6) + osAdjustment);
               else
                  g.drawString(s, j2 + 14, (k2 - 4) + osAdjustment);
               break;

            case 2 : // '\002'
               int j5 = s.length();
               try {
                  if (j5 > 6) {
                     int k5 = s.indexOf(" ");
                     if (k5 < 0) {
                        g.drawString(s.substring(0, 6), j2 + 5, k2 + 15 + osAdjustment);
                        g.drawString(s.substring(6, j5 > 12 ? 12 : j5), j2 + 5, k2 + 35 + osAdjustment);
                     } else {
                        g.drawString(s.substring(0, k5 < 7 ? k5 : 6), j2 + 5, k2 + 15 + osAdjustment);
                        g.drawString(s.substring(k5 + 1, j5 > k5 + 8 ? k5 + 7 : j5), j2 + 5, k2 + 35 + osAdjustment);
                     }
                  } else {
                     g.drawString(s, j2 + 5, k2 + 15 + osAdjustment);
                  }
               } catch (StringIndexOutOfBoundsException _ex) {
               }
               break;
         }
      } catch (ArrayIndexOutOfBoundsException _ex) {
      }
   }

   dbLeft.setVisible(false);
   dbRight.setVisible(false);
   dbUp.setVisible(false);
   dbDown.setVisible(false);
   if (firstVisibleTab > 0 || j < k)
      if (iTabsPosition == 0 || iTabsPosition == 1) {
         dbLeft.setVisible(true);
         dbRight.setVisible(true);
         if (firstVisibleTab > 0)
            dbLeft.setEnabled(true);
         else
            dbLeft.setEnabled(false);
         if (j < k)
            dbRight.setEnabled(true);
         else
            dbRight.setEnabled(false);
         if (iTabsPosition == 0) {
            dbLeft.setBounds((rectangle.width - 33) + TF_RIGHT, TF_TOP - 16, 16, 15);
            dbRight.setBounds((rectangle.width - 16) + TF_RIGHT, TF_TOP - 16, 16, 15);
         } else {
            dbLeft.setBounds((rectangle.width - 33) + TF_RIGHT, (rectangle.height + TF_BOTTOM) - TF_BTN_HEIGHT, 16, 15);
            dbRight.setBounds((rectangle.width - 16) + TF_RIGHT, (rectangle.height + TF_BOTTOM) - TF_BTN_HEIGHT, 16, 15);
         }
      } else {
         dbUp.setVisible(true);
         dbDown.setVisible(true);
         if (firstVisibleTab > 0)
            dbUp.setEnabled(true);
         else
            dbUp.setEnabled(false);
         if (j < k)
            dbDown.setEnabled(true);
         else
            dbDown.setEnabled(false);
         if (iTabsPosition == 3) {
            dbUp.setBounds(j2 + 1, (rectangle.height + TF_BOTTOM) - 32, 16, 15);
            dbDown.setBounds(j2 + 1, (rectangle.height + TF_BOTTOM) - 16, 16, 15);
         } else {
            dbUp.setBounds(TF_LEFT - 17, (rectangle.height + TF_BOTTOM) - 32, 16, 15);
            dbDown.setBounds(TF_LEFT - 17, (rectangle.height + TF_BOTTOM) - 16, 16, 15);
         }
      }
   for (; j < k; j++)
      vPolys.addElement(nullPoly);

   g.clearRect(0, 0, 10, 10);
   g.setFont(font);
   g.setColor(color);
}


public synchronized void remove(Component component) {
   if (component == dbLeft || component == dbRight)
      return;
   super.remove(component);
   if (component == userPanel)
      userPanel = null;
}


public synchronized void removeAll() {
   super.removeAll();
   super.add(dbLeft, -1);
   super.add(dbRight, -1);
   userPanel = null;
}


public void removeAllTabs() {
   vLabels = new Vector();
   vEnabled = new Vector();
   vPolys = new Vector();
   curIndex = -1;
   firstVisibleTab = 0;
   lastWidth = -1;
   lastHeight = -1;
   removeAll();
   TF_RIGHT = -9;
   repaint();
}


public void removeTab(int i) {
   if (i < 0 || i >= vEnabled.size() || i == curIndex)
      return;
   try {
      vLabels.removeElementAt(i);
      vEnabled.removeElementAt(i);
      if (iTabsPosition == 3)
         setRight();
      repaint();
      return;
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return;
   }
}


public synchronized void setEnabled(boolean flag, int i) {
   if (i < 0 || i >= vLabels.size())
      return;
   if (i == curIndex && !flag)
      return;
   try {
      vEnabled.setElementAt(Boolean.valueOf(flag), i);
      repaint();
      return;
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return;
   }
}


public synchronized void setLabel(String s, int i) {
   if (i < 0 || i >= vLabels.size())
      return;
   try {
      vLabels.setElementAt(s, i);
      if (iTabsPosition == 3)
         setRight();
      repaint();
      return;
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return;
   }
}


public void setLayout(LayoutManager layoutmanager1) {
}


private void setLevelCount() {
   FontMetrics fontmetrics = getFontMetrics(fReg);
   FontMetrics fontmetrics1 = getFontMetrics(fSel);
   levelCount = 1;
   int i = TF_LEFT + 2;
   int j = getBounds().width;
   vNumbers.removeAllElements();
   vNumbers.addElement(new Integer(0));
   vLbWidth.removeAllElements();
   int j2;
   for (j2 = 0; j2 < vLabels.size(); j2++) {
      String s = (String) vLabels.elementAt(j2);
      int k;
      if (j2 == curIndex)
         k = fontmetrics1.stringWidth(s);
      else
         k = fontmetrics.stringWidth(s);
      vLbWidth.addElement(new Integer(k));
      if (iTabsStyle == 0)
         k += 15;
      else
         k += 30;
      if ((i + k) - TF_RIGHT > j) {
         levelCount++;
         int j1 = (j + TF_RIGHT) - i;
         i = TF_LEFT + 2 + k;
         int k2 = ((Integer) vNumbers.lastElement()).intValue();
         int l1 = j1 / (j2 - k2);
         if (l1 > 0) {
            for (int i3 = k2; i3 < j2; i3++) {
               k = ((Integer) vLbWidth.elementAt(i3)).intValue() + l1;
               vLbWidth.setElementAt(new Integer(k), i3);
            }

         }
         l1 = j1 % (j2 - k2);
         int j3 = j2 - 1;
         for (; l1 > 0; l1--) {
            k = ((Integer) vLbWidth.elementAt(j3)).intValue() + 1;
            vLbWidth.setElementAt(new Integer(k), j3);
            j3--;
         }

         vNumbers.addElement(new Integer(j2));
      } else {
         i += k;
      }
   }

   if (levelCount > 1) {
      int k1 = (j + TF_RIGHT) - i;
      int l2 = ((Integer) vNumbers.lastElement()).intValue();
      int i2 = k1 / (j2 - l2);
      if (i2 > 0) {
         for (int k3 = l2; k3 < j2; k3++) {
            int l = ((Integer) vLbWidth.elementAt(k3)).intValue() + i2;
            vLbWidth.setElementAt(new Integer(l), k3);
         }

      }
      i2 = k1 % (j2 - l2);
      int l3 = j2 - 1;
      for (; i2 > 0; i2--) {
         int i1 = ((Integer) vLbWidth.elementAt(l3)).intValue() + 1;
         vLbWidth.setElementAt(new Integer(i1), l3);
         l3--;
      }

   }
   TF_TOP = 10 + levelCount * TF_BTN_HEIGHT;
}


public void setMultiLevel(boolean flag) {
   if (iTabsPosition == 0 && iTabsStyle == 0 && iMultiLevel != flag) {
      iMultiLevel = flag;
      if (iMultiLevel)
         setLevelCount();
      else
         TF_TOP = 30;
      doLayout();
      repaint();
   }
}


public void setPanel(Component component) {
   removeAll();
   userPanel = component;
   if (userPanel != null) {
      super.add(userPanel, -1);
      userPanel.requestFocus();
   }
}


private void setRight() {
   FontMetrics fontmetrics = getFontMetrics(fSel);
   int j = 0;
   for (int k = 0; k < vLabels.size(); k++) {
      int i = fontmetrics.stringWidth((String) vLabels.elementAt(k));
      if (i > j)
         j = i;
   }

   TF_RIGHT = - (j + 20);
}


public synchronized void setTab(String s, boolean flag, int i) {
   if (i < 0 || i >= vLabels.size())
      return;
   if (i == curIndex && !flag)
      return;
   try {
      vLabels.setElementAt(s, i);
      vEnabled.setElementAt(Boolean.valueOf(flag), i);
      repaint();
      return;
   } catch (ArrayIndexOutOfBoundsException _ex) {
      return;
   }
}


public void setTabsInfo(int i, int j) {
   iTabsPosition = i;
   if (iTabsPosition == 0 || iTabsPosition == 1)
      iTabsStyle = j;
   else
      iTabsStyle = 0;
   if (iTabsStyle == 0)
      TF_BTN_HEIGHT = 20;
   else
      TF_BTN_HEIGHT = 17;
   TF_TOP = iTabsPosition == 0 ? 10 + TF_BTN_HEIGHT : 10;
   if (iTabsPosition == 2)
      TF_LEFT = 60;
   if (iTabsPosition == 3)
      setRight();
   doLayout();
   repaint();
}


public void setTabsPosition(int i) {
   if (iTabsPosition != i) {
      iMultiLevel = false;
      TF_RIGHT = -9;
      TF_LEFT = 9;
      setTabsInfo(i, iTabsStyle);
   }
}


public void setTabsStyle(int i) {
   if (iTabsStyle != i) {
      iMultiLevel = false;
      setTabsInfo(iTabsPosition, i);
   }
}


public void showPanel(Component component) {
   if (userPanel != null)
      userPanel.setVisible(false);
   userPanel = component;
   if (userPanel != null) {
      Component acomponent[] = getComponents();
      int i = acomponent.length;
      int j;
      for (j = 0; j < i; j++)
         if (acomponent[j] == userPanel)
            break;

      if (j == i)
         super.add(userPanel, -1);
      userPanel.setVisible(true);
      userPanel.repaint();
      userPanel.requestFocus();
      validate();
      doLayout();
      repaint();
   }
}


public void showTab(int i) {
   if (i < 0 || i >= vLabels.size() || i == curIndex)
      return;
   if (tabIsEnabled(i)) {
      curIndex = i;
      invalidate();
      validate();
      repaint();
      dispatchEvent(new ActionEvent(this, 1001, null));
   }
}


public boolean tabIsEnabled(int i) {
   if (i < 0 || i >= vLabels.size())
      return false;
   try {
      Boolean boolean1 = (Boolean) vEnabled.elementAt(i);
      if (boolean1.booleanValue())
         return true;
   } catch (ArrayIndexOutOfBoundsException _ex) {
   }
   return false;
}
}