package com.ibm.easyswing;
import java.beans.Beans;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

public class BoundJComboBox extends JComboBox {

	private Vector _currentDataAsVector;
	private Object _currentDataAsArray[];
	private boolean _checkLock;

	public BoundJComboBox() {
		_checkLock = false;
	}

	public BoundJComboBox(Object aobj[]) {
		super(aobj);
		_checkLock = false;
	}

	public BoundJComboBox(Vector vector) {
		super(vector);
		_checkLock = false;
	}

	public BoundJComboBox(ComboBoxModel comboboxmodel) {
		super(comboboxmodel);
		_checkLock = false;
	}

	public void addListDataListener(ListDataListener listdatalistener) {
		super.listenerList.add(ListDataListener.class, listdatalistener);
	}

	private synchronized void checkDataChanges() {
		if (_checkLock)
			return;
		_checkLock = true;
		ComboBoxModel comboboxmodel = getModel();
		if (_currentDataAsVector == null) {
			fillDataVariables();
			_checkLock = false;
			return;
		}
		boolean flag = false;
		if (comboboxmodel.getSize() != _currentDataAsVector.size()) {
			flag = true;
		} else {
			for (int i = 0; i < comboboxmodel.getSize(); i++) {
				if ((comboboxmodel.getElementAt(i) != null || _currentDataAsVector.elementAt(i) == null)
					&& (comboboxmodel.getElementAt(i) == null
						|| comboboxmodel.getElementAt(i).equals(_currentDataAsVector.elementAt(i))))
					continue;
				flag = true;
				break;
			}

		}
		if (!flag) {
			_checkLock = false;
			return;
		}
		int j = _currentDataAsVector.size();
		Vector vector = _currentDataAsVector;
		Object aobj[] = _currentDataAsArray;
		fillDataVariables();
		if (j != _currentDataAsVector.size())
			firePropertyChange("itemCount", new Integer(j), new Integer(_currentDataAsVector.size()));
		firePropertyChange("listDataAsVector", vector, _currentDataAsVector);
		firePropertyChange("listDataAsEnumeration", null, null);
		firePropertyChange("listDataAsArray", aobj, _currentDataAsArray);
		repaint();
		_checkLock = false;
	}

	public void contentsChanged(ListDataEvent listdataevent) {
		super.contentsChanged(listdataevent);
		fireListDataChanged(
			new ListDataEvent(
				this,
				listdataevent.getType(),
				listdataevent.getIndex0(),
				listdataevent.getIndex1()));
		checkDataChanges();
	}

	private void fillDataVariables() {
		ComboBoxModel comboboxmodel = getModel();
		_currentDataAsArray = new Object[comboboxmodel.getSize()];
		for (int i = 0; i < comboboxmodel.getSize(); i++)
			_currentDataAsArray[i] = comboboxmodel.getElementAt(i);

		_currentDataAsVector = new Vector();
		for (int j = 0; j < comboboxmodel.getSize(); j++)
			_currentDataAsVector.addElement(comboboxmodel.getElementAt(j));

	}

	protected void fireListDataChanged(ListDataEvent listdataevent) {
		Object aobj[] = super.listenerList.getListenerList();
		for (int i = aobj.length - 2; i >= 0; i -= 2)
			if (aobj[i] == (ListDataListener.class))
				switch (listdataevent.getType()) {
					case 0 : // '\0'
						 ((ListDataListener) aobj[i + 1]).contentsChanged(listdataevent);
						break;

					case 1 : // '\001'
						 ((ListDataListener) aobj[i + 1]).intervalAdded(listdataevent);
						break;

					case 2 : // '\002'
						 ((ListDataListener) aobj[i + 1]).intervalRemoved(listdataevent);
						break;
				}

	}

	public String getAboutThisBean() {
		return getClass().getName() + "^BoundJComboBox^2.0^/easyswing/docs/index.html";
	}

	protected int getIndexByItem(Object obj) {
		int i = 0;
		for (int j = getModel().getSize(); i < j; i++) {
			Object obj1 = getModel().getElementAt(i);
			if (obj1.equals(obj))
				return i;
		}

		return -1;
	}

	public Object[] getListDataAsArray() {
		return _currentDataAsArray;
	}

	public Enumeration getListDataAsEnumeration() {
		if (_currentDataAsVector != null)
			return new VectorEnumerator(_currentDataAsVector);
		else
			return null;
	}

	public Vector getListDataAsVector() {
		return _currentDataAsVector;
	}

	public ComboBoxModel getModel() {
		return super.getModel();
	}

	public void removeAllItems() {
		if (super.getModel().getSize() == 0) {
			return;
		} else {
			super.removeAllItems();
			return;
		}
	}

	public void removeListDataListener(ListDataListener listdatalistener) {
		super.listenerList.remove(ListDataListener.class, listdatalistener);
	}

	protected void selectedItemChanged() {
		Object obj = super.selectedItemReminder;
		super.selectedItemChanged();
		if (obj == null && getSelectedItem() != null || obj != null && obj != getSelectedItem())
			firePropertyChange("selectedItem", obj, getSelectedItem());
		int i = getIndexByItem(obj);
		int j = getSelectedIndex();
		if (i != j)
			firePropertyChange("selectedIndex", new Integer(i), new Integer(j));
	}

	public void setAboutThisBean(String s) {
	}

	public void setListData(Iterator it) throws IllegalArgumentException {
		if (it == null)
			throw new IllegalArgumentException("data is null");

		Object oldSelectedItem = getSelectedItem();

		DefaultComboBoxModel defaultComboBoxModel = new DefaultComboBoxModel();

		boolean inIterator = false;

		while (it.hasNext()) {
			Object currObj = it.next();
			defaultComboBoxModel.addElement(currObj);
			if (oldSelectedItem == currObj) {
				inIterator = true;
			}
		}

		setModel(defaultComboBoxModel);

		if (inIterator) {
			setSelectedItem(oldSelectedItem);
		}
	}

	public void setListData(List list) throws IllegalArgumentException {
		if (list == null)
			throw new IllegalArgumentException("data is null");

		setListData(list.iterator());
	}

	public void setModel(ComboBoxModel comboboxmodel) {
		if (comboboxmodel == null)
			throw new IllegalArgumentException("model is null");
		if ((comboboxmodel instanceof CBDefModel) && Beans.isDesignTime()) {
			removeAllItems();
			for (int i = 0; i < comboboxmodel.getSize(); i++)
				addItem(comboboxmodel.getElementAt(i));

		} else {
			super.setModel(comboboxmodel);
		}
		checkDataChanges();
	}

	public void setSelectedIndex(int i) {
		if (i < 0) {
			return;
		} else {
			super.setSelectedIndex(i);
			return;
		}
	}
}