package com.ibm.easyswing;
import java.beans.Beans;
import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

// Referenced classes of package com.ibm.easyswing:
//            DefaultListModelSupport, VectorEnumerator

public class BoundJList extends JList implements ListDataListener, ListSelectionListener {

	private static final ListModel STATIC_LIST_MODEL = new DefaultListModel();

	private int selectedIndex;
	private Object selectedValue;
	private boolean setSelIndMethod;
	private boolean flag;

	public BoundJList() {
		this(STATIC_LIST_MODEL);

		addListSelectionListener(this);
	}

	public BoundJList(Object aobj[]) {
		this(DefaultListModelSupport.createDefaultModel(aobj));
	}

	public BoundJList(Vector vector) {
		this(DefaultListModelSupport.createDefaultModel(vector));
	}

	public BoundJList(ListModel listmodel) {
		super(listmodel);
		selectedIndex = -1;
		setSelIndMethod = false;
		flag = false;

		setModel(listmodel);
		if (!Beans.isDesignTime())
			addListSelectionListener(this);
	}

	public void addItem(Object obj) {
		checkDefaultListModel();
		((DefaultListModel) getModel()).addElement(obj);
	}

	public void addListDataListener(ListDataListener listdatalistener) {
		super.listenerList.add(javax.swing.event.ListDataListener.class, listdatalistener);
	}

	protected void checkDefaultListModel() {
		if (!(getModel() instanceof DefaultListModel))
			throw new InternalError("Cannot use this method with a custom data model.");
		else
			return;
	}

	private void checkSelectionIndicesChanges() {

		int j = selectedIndex;
		if (selectedIndex != getSelectedIndex()) {
			selectedIndex = getSelectedIndex();
			if (setSelIndMethod) {
				if (!flag) {
					firePropertyChange("selectedIndex", new Integer(j), new Integer(selectedIndex));
					checkSelectionValueChanges();
					return;
				}
				if (j == -1) {
					firePropertyChange("selectedIndex", new Integer(j), new Integer(selectedIndex));
					checkSelectionValueChanges();
				}
				flag = false;
			} else {

				firePropertyChange("selectedIndex", new Integer(j), new Integer(selectedIndex));
				checkSelectionValueChanges();
			}
		}
	}

	private void checkSelectionValueChanges() {
		if (selectedValue == null
			&& getSelectedValue() != null
			|| selectedValue != null
			&& !selectedValue.equals(getSelectedValue())) {
			Object obj = selectedValue;
			selectedValue = getSelectedValue();
			if (!flag)
				firePropertyChange("selectedValue", obj, selectedValue);
		}
	}

	public void contentsChanged(ListDataEvent listdataevent) {
		fireListDataChanged(
			new ListDataEvent(
				this,
				listdataevent.getType(),
				listdataevent.getIndex0(),
				listdataevent.getIndex1()));
	}

	protected void fireListDataChanged(ListDataEvent listdataevent) {
		Object aobj[] = super.listenerList.getListenerList();
		for (int i = aobj.length - 2; i >= 0; i -= 2)
			if (aobj[i] == (javax.swing.event.ListDataListener.class))
				switch (listdataevent.getType()) {
					case 0 : // '\0'
						 ((ListDataListener) aobj[i + 1]).contentsChanged(listdataevent);
						break;

					case 1 : // '\001'
						 ((ListDataListener) aobj[i + 1]).intervalAdded(listdataevent);
						break;

					case 2 : // '\002'
						 ((ListDataListener) aobj[i + 1]).intervalRemoved(listdataevent);
						break;
				}

	}

	public String getAboutThisBean() {
		return getClass().getName()
			+ "^BoundJList^2.0^BoundJList_About_Bean.gif^BoundJList_Other_Features1.gif^BoundJList_Other_Features2.gif^BoundJList_Expert_Features.gif^BoundJList_Wiring_Example.gif^/easyswing/docs/index.html";
	}

	public Object getItemAt(int i) {
		return getModel().getElementAt(i);
	}

	public int getItemCount() {
		return getModel().getSize();
	}

	public Object[] getListDataAsArray() {
		Object[] array = new Object[getModel().getSize()];
		for (int i = 0; i < getModel().getSize(); i++) {
			array[i] = getModel().getElementAt(i);
		}

		return array;
	}

	public java.util.Iterator getListDataAsIterator() {

		return getListDataAsList().iterator();
	}

	public java.util.List getListDataAsList() {
		java.util.List list = new java.util.ArrayList(getModel().getSize());
		for (int i = 0; i < getModel().getSize(); i++) {
			list.add(getModel().getElementAt(i));
		}

		return list;
	}

	public ListModel getModel() {
		return super.getModel();
	}

	public void insertItemAt(Object obj, int i) {
		checkDefaultListModel();
		((DefaultListModel) getModel()).insertElementAt(obj, i);
	}

	public void intervalAdded(ListDataEvent listdataevent) {
		contentsChanged(listdataevent);
	}

	public void intervalRemoved(ListDataEvent listdataevent) {
		contentsChanged(listdataevent);
	}

	public void removeAllItems() {
		checkDefaultListModel();
		super.setSelectedIndex(-1);
		((DefaultListModel) getModel()).removeAllElements();
	}

	public void removeItem(Object obj) {
		checkDefaultListModel();
		Object aobj[] = new Object[getItemCount()];
		aobj = getListDataAsArray();
		for (int i = 0; i < aobj.length; i++)
			if (aobj[i] == obj) {
				((DefaultListModel) getModel()).removeElement(obj);
				setSelectedIndex(-1);
				repaint();
				return;
			}

		throw new IllegalArgumentException();
	}

	public void removeItemAt(int i) {
		if (i < 0 || i > getItemCount() - 1) {
			throw new IllegalArgumentException();
		} else {
			checkDefaultListModel();
			((DefaultListModel) getModel()).removeElementAt(i);
			return;
		}
	}

	public void removeListDataListener(ListDataListener listdatalistener) {
		super.listenerList.remove(javax.swing.event.ListDataListener.class, listdatalistener);
	}

	public void setAboutThisBean(String s) {
	}

	public void setElementAt(Object obj, int i) {
		if (i < 0 || i > getItemCount() - 1) {
			throw new IllegalArgumentException();
		} else {
			checkDefaultListModel();
			((DefaultListModel) getModel()).setElementAt(obj, i);
			return;
		}
	}

	public void setListData(Object aobj[]) throws IllegalArgumentException {
		if (aobj == null)
			throw new IllegalArgumentException("data is null");
		setModel(DefaultListModelSupport.createDefaultModel(aobj));
	}

	public void setListData(java.util.Iterator it) throws IllegalArgumentException {
		if (it == null)
			throw new IllegalArgumentException("data is null");

		DefaultListModel listModel = new DefaultListModel();
		while (it.hasNext()) {
			listModel.addElement(it.next());
		}

		setModel(listModel);
	}

	public void setListData(java.util.List list) throws IllegalArgumentException {
		if (list == null)
			throw new IllegalArgumentException("data is null");
		setModel(DefaultListModelSupport.createDefaultModel(list));
	}

	public void setModel(ListModel listmodel) {
		if (listmodel == null)
			listmodel = STATIC_LIST_MODEL;

		ListModel listmodel1 = getModel();

		if (!Beans.isDesignTime() && listmodel1 != null)
			listmodel1.removeListDataListener(this);

		if (Beans.isDesignTime()) {
			removeAllItems();
			for (int i = 0; i < listmodel.getSize(); i++)
				addItem(listmodel.getElementAt(i));

			getModel().addListDataListener(this);

		} else {
			super.setModel(listmodel);
		}

		if (!Beans.isDesignTime()) {
			listmodel.addListDataListener(this);
		}

		//invalidate();
		//validate();
		//repaint();
	}

	public void setSelectedIndices(int ai[]) {
		setSelIndMethod = true;
		flag = true;
		int i = getItemCount();
		for (int j = 0; j < ai.length; j++)
			if (ai[j] >= i || ai[j] < 0)
				throw new IllegalArgumentException();

		super.setSelectedIndices(ai);
		setSelIndMethod = false;
	}

	public void valueChanged(ListSelectionEvent listselectionevent) {
		checkSelectionIndicesChanges();
	}
}