package com.jgoodies.animation;

/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

/**
 * This interface describes time based animations. Basically, the #animate
 * method applies an animation effect to an animation target over the
 * animation's duration.
 *
 * @author Karsten Lentzsch
 */
public interface Animation {

    /**
     * Answers the animation's duration.
     * 
     * @return the animation duration
     */
    long duration();

    /**
     * Animates for the given time, that is, applies the animation effect 
     * to the animation target and fires events when the animation is 
     * started, stopped or repeated.
     * 
     * @param time  the time used to determine the animation effect
     */
    void animate(long time);

    /**
     * Adds the given <code>AnimationListener</code> to this animation.
     * 
     * @param listener  the <code>AnimationListener</code> to add
     */
    void addAnimationListener(AnimationListener listener);

    /**
     * Removes the given <code>AnimationListener</code> from this animation.
     * 
     * @param listener  the <code>AnimationListener</code> to remove
     */
    void removeAnimationListener(AnimationListener listener);

}