package com.jgoodies.animation;

/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

/**
 * This interface describes time-based animation functions,
 * by their duration and a mapping from time to animation effects.
 * <p>
 * For each time in the animation's valid time interval,
 * the #valueAt method answers an animation effect.
 *
 * @author Karsten Lentzsch
 */
public interface AnimationFunction {

    /**
     * Returns the length of the valid time interval.
     * 
     * @return the valid time interval
     */
    long duration();

    /**
     * Returns the animation effect for a given time in the valid time interval.
     * 
     * @param time   the time used to determine the animation effect
     * @return the animation effect for the given time
     */
    Object valueAt(long time);

}