package com.jgoodies.animation;

/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.
 * Use is subject to license terms.
 *
 */

import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.SwingUtilities;

/**
 * Provides some behavior useful in the animation framework, 
 * or to implement custom animation functions and animations.
 *
 * @author Karsten Lentzsch
 */
public final class AnimationUtils {

    public static final boolean IS_14_ENVIRONMENT = is14Environment();

    // Override the default constructor; prevents instantiation.
    private AnimationUtils() {}

    public static void invokeOnStop(Animation animation, Runnable runnable) {
        animation.addAnimationListener(new StopListener(runnable));
    }

    /**
     * Checks and answers whether we're on a J2SE 1.4 or later.
     * Therefore tests if we can successfully load a class that
     * has been introduced by 1.4.
     * 
     * @return true in 1.4 or later, false otherwise
     */
    private static boolean is14Environment() {
        try {
            Class.forName("java.nio.Buffer");
            return true;
        } catch (Throwable t) {
            return false;
        }
    }

    /**
     * Answers an array of strings by splitting a given text 
     * into tokens, that are separated by the  '|' character.
     * 
     * @param separatedTexts   a string that encodes a bunch of texts
     * separated by a | character
     * @return an array of the separated texts
     */
    public static String[] splitTexts(String separatedTexts) {
        StringTokenizer tokenizer = new StringTokenizer(separatedTexts, "|");
        List texts = new LinkedList();
        while (tokenizer.hasMoreTokens()) {
            texts.add(tokenizer.nextToken());
        }
        return (String[]) texts.toArray(new String[texts.size()]);
    }

    // Helper class, that performs a runnable at animation stop.
    private static class StopListener extends AnimationAdapter {

        private final Runnable runnable;

        private StopListener(Runnable runnable) {
            this.runnable = runnable;
        }

        public void animationStopped(AnimationEvent e) {
            SwingUtilities.invokeLater(runnable);
        }
    }

}