package com.jgoodies.animation.animations;

/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.  
 * Use is subject to license terms.
 *
 */

import com.jgoodies.animation.AbstractAnimation;
import com.jgoodies.animation.components.GlyphLabel;

/**
 * A text based animation that changes the scaling of the text's 
 * individual glyphs over the time.
 * 
 * @author Karsten Lentzsch
 */
public final class GlyphAnimation extends AbstractAnimation {

    private final GlyphLabel label;
    private final String text;

    /**
     * Constructs an animation that changes the scaling of individual
     * glyphs over the duration.
     * 
     * @param label       the animation target label
     * @param duration    the animation duration
     * @param glyphDelay  a delay used to scale glyphs differently
     * @param text        the text to animation
     */
    public GlyphAnimation(
        GlyphLabel label,
        long duration,
        long glyphDelay,
        String text) {
        super(duration);
        this.label = label;
        this.text = text;
    }

    /**
     * Applies the effect: sets the text and time. 
     * 
     * @param time    the render time position
     */
    protected void applyEffect(long time) {
        label.setText(time == 0 ? " " : text);
        if (time < duration())
            label.setTime(time);
    }

}