package com.jgoodies.animation.components;

/*
 * Copyright (c) 2002 Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.  
 * Use is subject to license terms.
 *
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;

import javax.swing.JComponent;

import com.jgoodies.animation.renderer.BasicTextRenderer;
import com.jgoodies.animation.renderer.HeightMode;

/**
 * A Swing text component that can change the text, x and y scaling, 
 * glyph space, x and y offset and alignment mode.
 *
 * @author Karsten Lentzsch
 */
public final class BasicTextLabel extends JComponent {

    private final BasicTextRenderer renderer;

    /**
     * Constructs a animation text Swing label for the given text.
     */
    public BasicTextLabel(String text) {
        renderer = new BasicTextRenderer(text);
    }

    public Color getColor() {
        return renderer.getColor();
    }
    
    public HeightMode getHeightMode() {
        return renderer.getHeightMode();
    }
    
    public float getScaleX() {
        return renderer.getScaleX();
    }
    
    public float getScaleY() {
        return renderer.getScaleY();
    }
    
    public float getSpace() {
        return renderer.getSpace();
    }
    
    public float getOffsetX() {
        return renderer.getOffsetX();
    }
    
    public float getOffsetY() {
        return renderer.getOffsetY();
    }
    
    public String getText() {
        return renderer.getText();
    }

    public void setColor(Color color) {
        renderer.setColor(color);
        repaint();
    }

    public void setHeightMode(HeightMode heightMode) {
        renderer.setHeightMode(heightMode);
    }

    public void setScale(float scale) {
        renderer.setScaleX(scale);
        renderer.setScaleY(scale);
        repaint();
    }

    public void setScaleX(float scaleX) {
        renderer.setScaleX(scaleX);
        repaint();
    }

    public void setScaleY(float scaleY) {
        renderer.setScaleY(scaleY);
        repaint();
    }

    public void setSpace(float space) {
        renderer.setSpace(space);
        repaint();
    }

    public void setOffsetX(float offsetX) {
        renderer.setOffsetX(offsetX);
        repaint();
    }

    public void setOffsetY(float offsetY) {
        renderer.setOffsetY(offsetY);
        repaint();
    }

    public void setText(String newText) {
        renderer.setText(newText);
        repaint();
    }

    /**
     * Paints the component. Enabled anti-aliasing and sets high quality hints,
     * then renderers the component via the underlying renderer.
     */
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;

        g2.setRenderingHint(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setRenderingHint(
            RenderingHints.KEY_RENDERING,
            RenderingHints.VALUE_RENDER_QUALITY);

        renderer.setFont(getFont());
        renderer.render(g2, getWidth(), getHeight());
    }
}