/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.  
 * Use is subject to license terms.
 *
 */

package com.jgoodies.animation.components;

import java.awt.*;

import javax.swing.JComponent;

/**
 * A Swing component that paints a circle with a given center, radius and color.
 *
 * @author Karsten Lentzsch
 */
public final class CircleComponent extends JComponent {

    private Point center;
    private int   radius;
    private Color color;

    /**
     * Constructs a <code>CircleComponent</code>.
     */
    public CircleComponent() {
        center = new Point(0, 0);
        radius = 30;
        color = Color.black;
    }

    public void setCenter(Point p) {
        this.center = p;
    }
    
    public void setRadius(int radius) {
        this.radius = radius;
    }
    
    public void setColor(Color color) {
        this.color = color;
    }

    /**
     * Sets the bounds and sets the center point.
     */
    public void setBounds(int x, int y, int w, int h) {
        super.setBounds(x, y, w, h);
        setCenter(new Point(x + w / 2, y + h / 2));
    }

    /**
     * Paints the component: enables anti-aliasing and sets high quality hints,
     * then renderers the component via the underlying renderer.
     */
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;

        int diameter = radius * 2;
        g2.setRenderingHint(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g.setColor(color);
        g2.setStroke(new BasicStroke(4));
        g2.drawOval(center.x - radius, center.y - radius, diameter, diameter);
    }

}