package com.jgoodies.animation.components;

/*
 * Copyright (c) 2002 Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of Karsten Lentzsch.  
 * Use is subject to license terms.
 *
 */

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.geom.Point2D;

import javax.swing.JComponent;

import com.jgoodies.animation.renderer.FanRenderer;

/**
 * A Swing component that paints a set of triangles as a fan.
 *
 * @author Karsten Lentzsch
 */
public final class FanComponent extends JComponent {

    private final FanRenderer renderer;

    /**
     * Constructs a <code>FanComponent</code> for the specified 
     * number of triangles and base color.
     */
    public FanComponent(int triangleCount, Color baseColor) {
        this.renderer = new FanRenderer(triangleCount, baseColor);
    }

    // Accessors ************************************************************

    public Point2D getOrigin() {
        return renderer.getOrigin();
    }
    
    public double getRotation() {
        return renderer.getRotation();
    }

    /**
     * Sets a new origin of the fan.
     */
    public void setOrigin(Point2D origin) {
        renderer.setOrigin(origin);
        repaint();
    }

    /**
     * Sets a new rotation.
     */
    public void setRotation(double rotation) {
        renderer.setRotation(rotation);
        repaint();
    }

    /**
     * Delegates painting to the fan renderer. Switches on anti-aliasing
     * and the high quality mode before invoking the renderer.
     */
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;

        g2.setRenderingHint(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setRenderingHint(
            RenderingHints.KEY_RENDERING,
            RenderingHints.VALUE_RENDER_QUALITY);

        Insets insets = getInsets();
        int x = insets.left;
        int y = insets.top;
        int w = getWidth()  - x - insets.right;
        int h = getHeight() - y - insets.bottom;
        g2.translate(x, y);
        renderer.render(g2, w, h);
        g2.translate(-x, -y);
    }

}