/*
 * Copyright (c) 2003 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * This software is the proprietary information of JGoodies Karsten Lentzsch.  
 * Use is subject to license terms.
 *
 */
 
package com.jgoodies.animation.renderer;

import java.awt.Graphics2D;

import com.jgoodies.animation.AnimationUtils;


/**
 * Renders a text with modifiable scaling, color, glyph spacing and position.
 *
 * @author Karsten Lentzsch
 */
public final class BasicTextRenderer extends AbstractTextRenderer {
	
	private float offsetX = 0.0f;
	private float offsetY = 0.0f;
	private float scaleX  = 1.0f;
	private float scaleY  = 1.0f;
	private float space   = 0.0f;


	/**
	 * Constructs a renderer for a text that can be scaled, moved, and change glyph space.
	 */
	public BasicTextRenderer(String text) {    super(text);		}
	
	
	public float getOffsetX()				{ return offsetX;			}
	public float getOffsetY()				{ return offsetY;			}
	public float getSpace() 				{ return space;			}
	public void  setOffsetX(float offsetX){ this.offsetX = offsetX;	}
	public void  setOffsetY(float offsetY){ this.offsetY = offsetY;	}
	public void  setSpace(float space)	{ this.space   = space;	}
	public float getScaleX()				{ return scaleX;			}
	public float getScaleY() 				{ return scaleY;			}
	

	public void setScaleX(float scaleX)	{	
		if (scaleX <= 0.0f) 
			throw new IllegalArgumentException("scaleX must be positive.");
			
		this.scaleX = scaleX;	
	}

	
	public void setScaleY(float scaleY)	{	
		if (scaleY <= 0.0f) 
			throw new IllegalArgumentException("scaleY must be positive.");
			
		this.scaleY = scaleY;	
	}


	/**
	 * Renders the text. Firstly ensures a valid cache, then sets 
	 * the color, and finally paints the cached glyph shapes.
	 */
	public void render(Graphics2D g2, int width, int height) {
		ensureValidCache(g2);

		int   glyphCount  = cachedGlyphShapes.length;
		float totalSpace  = (glyphCount - 1) * space;
		float totalWidth  = cachedTextWidth + totalSpace;
		float textHeight  = getAdjustedAscent();
		float xOffset     = getOffsetX() + (width  - totalWidth * scaleX) / 2.0f;
		float yOffset     = getOffsetY() + (height + textHeight * scaleY) / 2.0f - getAdjustedDescent();
		
		g2.setColor(getColor());
		g2.translate(xOffset, yOffset);
		g2.scale(scaleX, scaleY);
		for (int i = 0; i < glyphCount; i++) {
			if (!AnimationUtils.IS_14_ENVIRONMENT) {
				double glyphX = cachedGlyphVector.getGlyphPosition(i).getX();
				g2.translate(glyphX, 0);
				g2.fill(cachedGlyphShapes[i]);
				g2.translate(space, 0);
				g2.translate(-glyphX, 0);
			} else {
				g2.fill(cachedGlyphShapes[i]);
				g2.translate(space, 0);
			}
		}
		g2.scale(1.0f / scaleX, 1.0f / scaleY);
		g2.translate(-totalSpace -xOffset, -yOffset);
	}
	
}