package de.uniwue.ki.gui.mas.application;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.*;

import de.uniwue.ki.gui.util.PictureDisplayPanel;
import de.uniwue.ki.gui.util.SwingMoreUtilities;


/**
 * @author Reinhard Hatko
 * 
 * Created on 29.12.2003
 */
public class ModelLibraryInternalFrame extends SeSAmDesktopInternalFrame {
	
	private JLabel _descLabel;
	private JLabel _numOfPicLabel;
	private PictureDisplayPanel _picPanel;
	private JTextArea _textArea;
	private JButton _firstPicButton;
	private JButton _previousPicButton;
	private JButton _nextPicButton;
	private JButton _lastPicButton;
	private JButton _openModelButton;
	private JButton _cancelButton;
	private JList _modelList;
	
	private ModelLibraryEntry[] _library;
	private ModelLibraryEntry _displayedModel;
	private DefaultListModel _listModel;
	
	private SeSAmDesktopPane _sesamDesktopPane;
	private int _numOfActualPic = - 1;
	private int _numOfPics;
	
			
	public ModelLibraryInternalFrame(SeSAmDesktopPane pane){
		super("Model Library");
		
		_sesamDesktopPane = pane;
		
		initialize();
		loadLibrary();
		
		setVisible(true);
	}
	
		
	private void initialize(){
		setName("SeSAmDesktopInternalFrame");
		setIconifiable(true);
		setClosable(true);
		setMaximizable(true);
		setResizable(true);
		setSize(600, 600);
				
		_modelList = new JList();
		_modelList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		_modelList.addListSelectionListener( 
			new ListSelectionListener(){
				public void valueChanged(ListSelectionEvent e){
					setDisplayedModel(_library[_modelList.getSelectedIndex()]);
				}
			}
		);

		_descLabel = new JLabel();
		_descLabel.setHorizontalAlignment(SwingConstants.CENTER);
		_descLabel.setVerticalAlignment(SwingConstants.CENTER);
		
		_picPanel = new PictureDisplayPanel();
				
		_openModelButton = new JButton(
			new AbstractAction("Open Model") {
				public void actionPerformed(ActionEvent arg0) {
					dispose();
					_sesamDesktopPane.loadModel(_displayedModel.getModelFile());
				}
			}
		);
		
		_openModelButton.setEnabled(false);
		
		_cancelButton = new JButton(
			new AbstractAction("Close") {
				public void actionPerformed(ActionEvent arg0) {
					dispose();			
				}
			}
		);
		
		_textArea = new JTextArea("-- Select a model on the left --");
		_textArea.setRows(20);
		_textArea.setEditable(false);
		_textArea.setLineWrap(true);
		
		ColumnSpec[] columns = new ColumnSpec[]{
			new ColumnSpec(ColumnSpec.FILL, Sizes.dluX(100), ColumnSpec.NO_GROW), 
			new ColumnSpec(ColumnSpec.DEFAULT, Sizes.DLUX4, 0),
			new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT, ColumnSpec.DEFAULT_GROW)
		};

		RowSpec[] rows = new RowSpec[]{
			new RowSpec(RowSpec.FILL, Sizes.DEFAULT, .1),
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUX4, 0),
			new RowSpec(RowSpec.FILL, Sizes.DEFAULT, 0),
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUX4, 0),
			new RowSpec(RowSpec.FILL, Sizes.DEFAULT, 0),
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUX4, 0),			
			new RowSpec(RowSpec.FILL, Sizes.DEFAULT, .5), //
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUX4, 0),
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUX14, 0)			
		};

		FormLayout layout = new FormLayout(columns, rows);
		PanelBuilder builder = new PanelBuilder(layout);
		builder.setDefaultDialogBorder();
		
		CellConstraints cc = new CellConstraints();
		builder.add(new JScrollPane(_modelList), cc.xywh(1, 1, 1, 7));
		builder.add(_picPanel, 	cc.xy(3,1));
		builder.add(_descLabel, cc.xy(3,3));
		builder.add(getControlPanel(), cc.xy(3,5));
		builder.add(new JScrollPane(_textArea), cc.xy(3,7));
		builder.add(_openModelButton, cc.xy(1, 9, "l,c"));
		builder.add(_cancelButton, cc.xy(3, 9, "r, c"));
		
		getContentPane().add(builder.getPanel());
	}
	
	private void setDisplayedModel(ModelLibraryEntry entry){
		_displayedModel = entry;
		
		_textArea.setText(entry.getDescription());
		_textArea.setCaretPosition(0);
		_numOfPics = entry.getNumberOfPics();
		_numOfActualPic = 0;

		loadPictures(entry.getImages());

		_picPanel.setImage(entry.getImages()[0]);
		_descLabel.setText(entry.getImageDescriptions()[0]);
		
		updateButtonStatus();
	}

	private void refreshPicture(){
		_picPanel.setImage(_displayedModel.getImages()[_numOfActualPic]);
		_descLabel.setText(_displayedModel.getImageDescriptions()[_numOfActualPic]);
		updateButtonStatus();
	}
	
	private void loadPictures(Image[] images){
		for (int i = 0; i < images.length; i++) 
			if (!SwingMoreUtilities.waitForImage(images[i], 400, 300)){
				images[i] = getFailImage(400, 300);
			}
	}
	
	private void loadLibrary(){
		File libraryPath = new File(System.getProperty("user.dir"),"models");
		_library = new ModelLibraryReader(libraryPath).readLibrary();
		
		String[] data = new String[_library.length];

		for (int i = 0; i < _library.length; i++) 
			data[i] = _library[i].getTitle();
		
		_modelList.setListData(data);
	}
	
	private JPanel getControlPanel(){

		_firstPicButton = new JButton(
			new AbstractAction("", SwingMoreUtilities.getJavaSunIcon(16, "media/Rewind16.gif")){
				public void actionPerformed(ActionEvent arg0) {
					_numOfActualPic = 0;				
					refreshPicture();
	
				}
			}
		);
		
		_previousPicButton = new JButton(
			new AbstractAction("", SwingMoreUtilities.getJavaSunIcon(16, "media/StepBack16.gif")){
				public void actionPerformed(ActionEvent arg0) {
					_numOfActualPic--;
					refreshPicture();
				}
			}
		);
				
		_nextPicButton = new JButton(
			new AbstractAction("",SwingMoreUtilities.getJavaSunIcon(16, "media/StepForward16.gif")){
				public void actionPerformed(ActionEvent arg0) {
					_numOfActualPic++;
					refreshPicture();
				}
			}
		);
		
		_lastPicButton = new JButton(
			new AbstractAction("", SwingMoreUtilities.getJavaSunIcon(16, "media/FastForward16.gif")){
				public void actionPerformed(ActionEvent arg0) {
					_numOfActualPic = _numOfPics - 1;
					refreshPicture();
				}
			}
		);
		
		_numOfPicLabel = new JLabel("Picture " + (_numOfActualPic + 1) + " of " + _numOfPics);
		
		_firstPicButton.setEnabled(false);
		_previousPicButton.setEnabled(false);
		_nextPicButton.setEnabled(false);
		_lastPicButton.setEnabled(false);
		
		JPanel controlPanel = new JPanel();		
		controlPanel.add(_firstPicButton);
		controlPanel.add(_previousPicButton);
		controlPanel.add(_numOfPicLabel);
		controlPanel.add(_nextPicButton);
		controlPanel.add(_lastPicButton);		
	
		return controlPanel;
	}
	
	private void updateButtonStatus(){
		
		_openModelButton.setEnabled(true);
		
		if (_numOfActualPic  < 1){
			_firstPicButton.setEnabled(false);
			_previousPicButton.setEnabled(false);
		}else{
			_firstPicButton.setEnabled(true);
			_previousPicButton.setEnabled(true);
		}
		
		if (_numOfActualPic == _numOfPics - 1){
			_lastPicButton.setEnabled(false);
			_nextPicButton.setEnabled(false);
		}
		else{
			_lastPicButton.setEnabled(true);
			_nextPicButton.setEnabled(true);
		}
		
		_numOfPicLabel.setText("Picture " + (_numOfActualPic + 1) + " of " + _numOfPics);
	}
	
	private Image getFailImage(int width, int height){
		Image image = createVolatileImage(width, height);
		Graphics2D g = (Graphics2D)image.getGraphics();
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g.setColor(Color.BLACK);
		g.drawString("Pic not available", width/6 + 50, height/2 + 20);
		g.drawRect(0,0, width - 1, height - 1);
		g.setColor(Color.RED);
		g.setStroke(new BasicStroke(4, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_BEVEL));
		g.drawLine(width/6, height/2, width/6 + 30, height/2 + 40 );
		g.drawLine(width/6 + 30 , height/2, width/6, height/2 + 40 );

		return image;
	}
	
	public void dispose() {
		super.dispose();
		
		for (int i = 0; i < _library.length; i++){
			Image[] images = _library[i].getImages();
			for (int j = 0; j < images.length; j++)
				images[j].flush();
		}
		
	}

}
