package de.uniwue.ki.gui.mas.application;

import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 * @author Reinhard Hatko
 *	
 * Created on 10.01.2004
 */
public class ModelLibraryReader {

	private File _libraryPath;
		
	public ModelLibraryReader(File libaryPath){
		_libraryPath = libaryPath;
	}
	
	public ModelLibraryEntry[] readLibrary(){
		
		if (!_libraryPath.exists())
			return new ModelLibraryEntry[0];
		
		File[] modelPaths = _libraryPath.listFiles(
			new FileFilter(){
				public boolean accept(File pathname) {
					return new File(pathname, "modeldescription.xml").exists();
				}
			}
		);
		
		ArrayList libraryEntrys = new ArrayList(modelPaths.length);
			
		for (int i = 0; i < modelPaths.length; i++) {
			ModelLibraryEntry entry = parseModelDescriptionFile(new File(modelPaths[i], "modeldescription.xml"), modelPaths[i]);
			
			if (entry != null)
				libraryEntrys.add(entry);
		}
		
		return (ModelLibraryEntry[]) libraryEntrys.toArray(new ModelLibraryEntry[0]);
	}
	
	private ModelLibraryEntry parseModelDescriptionFile(File file, File path){
		
		try
		{
		
			Element root = DocumentBuilderFactory.newInstance()
				.newDocumentBuilder().parse(file).getDocumentElement();
			
			File modelFile = new File(path, root.getElementsByTagName("ModelFile").item(0).getFirstChild().getNodeValue());	
			String title = root.getElementsByTagName("Title").item(0).getFirstChild().getNodeValue();
			String description = root.getElementsByTagName("Description").item(0).getFirstChild().getNodeValue();
			
			NodeList imageNodeList = root.getElementsByTagName("Image");
			
			Image[] images = new Image[imageNodeList.getLength()];
			String[] imageDesc = new String[imageNodeList.getLength()];
			
			readImageInfos(imageNodeList, images, imageDesc, path);
			
			return new ModelLibraryEntry(modelFile, title, images, imageDesc, description);
		} 
		catch( Exception e ) {
			System.out.println("Error parsing file: " + file);
			e.printStackTrace();
			return null;
		} 
	}

	private void readImageInfos(NodeList imageNodeList, Image[] images, String[] imageDesc, File modelPath){
		
		int imageCount = images.length;
		
		for (int k = 0; k < imageCount; k++) {
					
			NodeList imageInfos = imageNodeList.item(k).getChildNodes(); 
					
			for (int j = 0; j < imageInfos.getLength(); j++) {						
				if (imageInfos.item(j).getNodeName() == "Path"){
					String fileName = imageInfos.item(j).getFirstChild().getNodeValue(); 
					images[k] = Toolkit.getDefaultToolkit().getImage(new File(modelPath, fileName).toString());
				} 
				else if (imageInfos.item(j).getNodeName() == "ImageTitle"){  
					imageDesc[k] = imageInfos.item(j).getFirstChild().getNodeValue();
				}
			}
		}
	}
	
	
}
