package de.uniwue.ki.gui.mas.application;
import java.awt.*;
import java.util.*;

import javax.swing.*;

import com.jgoodies.animation.*;
import com.jgoodies.animation.animations.BasicTextAnimation;
import com.jgoodies.animation.components.BasicTextLabel;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.util.Utils;

/**
 * Creation date: (15.11.2001 18:28:29)
 * @author Christoph Oechslein
 */
public class SeSAmAboutBoxHelper {
	private static final SeSAmAboutBoxHelper _instance = new SeSAmAboutBoxHelper();
	private JInternalFrame _aboutBoxInternalFrame;

	private static final String[] DESIGNERS =
		{
			"Christoph Oechslein",
			"Franziska Klgl",
			"Rainer Herrler",
			"Manuel Fehler",
			"Steffen Glckselig",
			"Clemens Mhlberger",
			"Reinhard Hatko"};

	private static final String[] LIBRARIES =
		{
			"castor",
			"jgoodies forms",
			"jgoodies animations",
			"JUnit",
			"Eclipse",
			"wrapper",
			"JFreeChart",
			"Xerces",
			"Trove",
			"RngPack",
			"Doug Lea Tools" };

	public SeSAmAboutBoxHelper() {
		super();
	}

	public JInternalFrame getAboutBox(Component parent) {
		if (_aboutBoxInternalFrame == null || _aboutBoxInternalFrame.isClosed()) {

			String windowTitle = "About SeSAm";

			JPanel panel =
				new JPanel(
					new FormLayout(
						"min(200dlu;pref), 5dlu, fill:default:grow",
						"fill:min(30dlu;pref), 10dlu, fill:default:grow, 5dlu, fill:default:grow, 5dlu"));

			CellConstraints cc = new CellConstraints();

			String sesamVersionString = "SeSAm v" + SeSAm.version();
			BasicTextLabel animLabel = new BasicTextLabel(" ");
			animLabel.setFont(new Font("dialog", Font.BOLD, 15));

			panel.add(animLabel, cc.xywh(1, 1, 3, 1));

			JLabel messageLabel = new JLabel(getMessage());
			panel.add(messageLabel, cc.xy(1, 3));

			panel.add(new JLabel(getAboutIcon()), cc.xy(3, 3));

			panel.add(new JSeparator(), cc.xywh(1, 5, 3, 1));

			JOptionPane optionPane =
				new JOptionPane(panel, JOptionPane.PLAIN_MESSAGE, JOptionPane.DEFAULT_OPTION);

			_aboutBoxInternalFrame = optionPane.createInternalFrame(parent, windowTitle);

			_aboutBoxInternalFrame.setClosable(true);
			_aboutBoxInternalFrame.setResizable(false);
			_aboutBoxInternalFrame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

			startTextAnimation(animLabel, sesamVersionString);
		}
		return _aboutBoxInternalFrame;
	}

	private void startTextAnimation(BasicTextLabel animLabel, String sesamVersionString) {
		new Animator(createTextAnimation(animLabel, sesamVersionString), 30).start();
	}

	private Animation createTextAnimation(final BasicTextLabel animLabel, final String sesamVersionString) {
		LinkedList animations = new LinkedList();

		animations.add(presentStrings(animLabel, "coding & design by ", 1000, DESIGNERS));

		animations.add(BasicTextAnimation.defaultFade(animLabel, 5000, sesamVersionString, Color.darkGray));

		animations.add(presentStrings(animLabel, "using libs/tools", 1500, LIBRARIES));

		animations.add(createSpecialAnimation(animLabel, sesamVersionString, 8000, Color.darkGray));

		animations.add(new AbstractAnimation(4000, true) {
			protected void applyEffect(long time) {
				animLabel.setOffsetX(0);
				animLabel.setOffsetY(0);
				animLabel.setSpace(0);
				animLabel.setScale(1);
				animLabel.setColor(Color.darkGray);
				animLabel.setText(sesamVersionString);
			}
		});

		animations.add(createFadeInOrOutAnimation(animLabel, sesamVersionString, 1000, Color.darkGray, true));

		return Animations.sequential(
			BasicTextAnimation.defaultFade(animLabel, 1000, sesamVersionString, Color.darkGray),
			Animations.sequential(
				Animations.repeat(5, Animations.sequential(animations)),
				createFadeInOrOutAnimation(animLabel, sesamVersionString, 1000, Color.darkGray, false)));
	}

	private BasicTextAnimation createSpecialAnimation(
		BasicTextLabel animLabel,
		String sesamVersionString,
		int duration,
		Color color) {

		class DecreaseAnimationFunction implements AnimationFunction {
			private final AnimationFunction _animationFunction;
			private float _decrease = 1.0f;

			public DecreaseAnimationFunction(AnimationFunction animationFunction) {
				_animationFunction = animationFunction;
			}

			public long duration() {
				return _animationFunction.duration();
			}

			public Object valueAt(long time) {
				_decrease *= 0.98;
				return new Double(((Integer)_animationFunction.valueAt(time)).intValue() * _decrease);
			}
		}

		BasicTextAnimation animation =
			new BasicTextAnimation(
				animLabel,
				duration,
				sesamVersionString,
				AnimationFunctions.alphaColor(
					AnimationFunctions.linear(
						duration,
						new Integer[] { new Integer(0), new Integer(255)},
						new float[] { 0.0f, 1.0f }),
					color),
				AnimationFunctions.linear(
					duration,
					new Float[] { new Float(0.5f), new Float(1), new Float(1)}),
				AnimationFunctions.linear(duration, new Float[] { new Float(3f), new Float(1), new Float(1)}),
				AnimationFunctions.fromTo(duration, 2, 0.1f),
				new DecreaseAnimationFunction(AnimationFunctions.random(-8, 8, 0.4f)),
				new DecreaseAnimationFunction(AnimationFunctions.random(-4, 4, 0.8f)));

		animation.setOffsetEnabled(true);

		return animation;
	}

	private BasicTextAnimation createFadeInOrOutAnimation(
		BasicTextLabel animLabel,
		String sesamVersionString,
		int duration,
		Color color,
		boolean fadeOut) {

		BasicTextAnimation animation =
			new BasicTextAnimation(
				animLabel,
				duration,
				sesamVersionString,
				AnimationFunctions
					.alphaColor(
						AnimationFunctions
						.linear(duration, fadeOut ? new Integer[] { new Integer(255), new Integer(0)}
		: new Integer[] { new Integer(0), new Integer(255)}, fadeOut ? new float[] { 1f, 0f }
		: new float[] { 0f, 1f }), color), null, null, null, null, null);

		return animation;
	}

	private Animation presentStrings(
		BasicTextLabel animLabel,
		String introText,
		int presentDuration,
		String[] stringsToPresent) {
		LinkedList animations = new LinkedList();

		animations.add(BasicTextAnimation.defaultSpace(animLabel, 3000, introText, Color.darkGray));

		Collections.shuffle(Arrays.asList(stringsToPresent));
		for (int i = 0; i < stringsToPresent.length; i++) {
			animations.add(
				BasicTextAnimation.defaultSpace(
					animLabel,
					presentDuration,
					stringsToPresent[i],
					Color.darkGray));
		}

		return Animations.sequential(animations);
	}

	private ImageIcon getAboutIcon() {
		ImageIcon splashImageIcon =
			new javax.swing.ImageIcon(
				SeSAmSplashScreen.class.getResource("/de/uniwue/ki/gui/mas/application/SeSAm-startup.gif"));

		splashImageIcon =
			new ImageIcon(
				splashImageIcon.getImage().getScaledInstance(
					-1,
					(int)Math.round(
						100
							* (double)splashImageIcon.getIconHeight()
							/ (double)splashImageIcon.getIconWidth()),
					Image.SCALE_SMOOTH));

		return splashImageIcon;
	}

	public static final SeSAmAboutBoxHelper getInstance() {
		return _instance;
	}

	private String getMessage() {
		StringBuffer message = new StringBuffer();
		message.append("<html>");
		message.append("<center><a href=\"http://www.simsesam.de\">http://www.simsesam.de</a><p><p>");
		message.append("University Wuerzburg<p>");
		message.append("Department of Artificial Intelligence<p>");
		message.append("Java Version:" + System.getProperty("java.version") + "<p>");

		message.append("Memory: ");
		Utils.appendMemoryInfo(message);

		message.append("</html>");

		return message.toString();
	}
}