package de.uniwue.ki.gui.mas.application;
import java.awt.Component;
import java.awt.Cursor;
import java.beans.PropertyVetoException;
import java.io.*;
import java.util.prefs.Preferences;

import javax.swing.*;

import junit.framework.Assert;

import com.jgoodies.forms.factories.Borders;

import de.uniwue.ki.gui.mas.application.importExport.*;
import de.uniwue.ki.gui.mas.application.remote.server.RemoteSimulationControlPanel;
import de.uniwue.ki.gui.mas.basics.feature.user.UserFeatureClassDeclarationListPanel;
import de.uniwue.ki.gui.mas.basics.functionUser.UserFunctionDeclarationListPanel;
import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationListPanel;
import de.uniwue.ki.gui.mas.entity.objectclass.*;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JInternalFrameFactory;
import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreeCellRenderer;
import de.uniwue.ki.gui.mas.sim.analysis.AnalysisDeclarationListPanel;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentDeclarationListPanel;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentListPanel;
import de.uniwue.ki.gui.mas.sim.simulation.SimulationDeclarationListPanel;
import de.uniwue.ki.gui.mas.sim.situation.SituationDeclarationListPanel;
import de.uniwue.ki.gui.mas.sim.test.TestRunner;
import de.uniwue.ki.gui.util.ExtensionsFileFilter;
import de.uniwue.ki.gui.util.JBetterFileChooser;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.mdi.MDIWindowMenu;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.name.INamed;

/**
* Creation date: (10.10.2000 18:53:07)
* @author Christoph Oechslein
*/

final class SeSAmDesktopPaneHelper {

	private final class LoadingThread extends Thread implements Runnable {
		private final Cursor _oldCursor;
		private final ProgressMonitorInputStream _progressMonitorInputStream;
		private final InputStreamReader _inputStreamReader;
		private final File _oldFile;
		private final File _file;

		private LoadingThread(
			Cursor oldCursor,
			ProgressMonitorInputStream progressMonitorInputStream,
			InputStreamReader inputStreamReader,
			File oldFile,
			File file) {

			super("Loading " + file);

			_oldCursor = oldCursor;
			_progressMonitorInputStream = progressMonitorInputStream;
			_inputStreamReader = inputStreamReader;
			_oldFile = oldFile;
			_file = file;
		}

		public void run() {
			try {

				MDIWindowMenu.closeAllFrames(getSeSAmDesktopPane().getDesktopPane());

				getGUISeSAmDeclaration().openModel(_inputStreamReader);
				
				getSeSAmDeclaration().setModelModified(false);
				getSeSAmMainDesktopFrame().addModelFileMenuEntryAndToPrefs(_file);			
				
			} catch (InterruptedIOException e) {//Laden gecancelt
				getSeSAmDeclaration().clear();
				getSeSAmDeclaration().setFile(null);
				getSeSAmDeclaration().setModelModified(false);                           
            } catch (Throwable exception) { //sonstiger Fehler
				getSeSAmDeclaration().getSeSAmDeclarationTreeModel().setEventCreation(true);
				getSeSAmDeclaration().clear();
				getSeSAmDeclaration().setFile(null);
				getSeSAmDeclaration().setModelModified(false);
				getGUISeSAmDeclaration().showException(
					"Error during loading model file",
					exception,
					getSeSAmDesktopPane());

				exception.printStackTrace();

			} finally {
		
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						try {
							getSeSAmMainDesktopFrame()
								.getSeSAmMainDesktopFrameMenuBar()
								.updateCompilerSettings();
							getSeSAmMainDesktopFrame()
								.getSeSAmDeclarationTreePanel()
								.setSeSAmDeclarationTreeModel(
								getGUISeSAmDeclaration().getSeSAmDeclarationTreeModel());

							getSeSAmDeclaration().getSeSAmDeclarationTreeModel().reload();
							getSeSAmDeclaration().getSeSAmDeclarationTreeModel().setEventCreation(true);
							getSeSAmMainDesktopFrame().getSeSAmDeclarationTreePanel().expandFirstCol();
							getSeSAmMainDesktopFrame().getSeSAmDeclarationTreePanel().repaint();

						} finally {
							SwingMoreUtilities.setEnableAllComponentsRecursive(
								getSeSAmMainDesktopFrame().getContentPane(),
								true);
							SwingMoreUtilities.setEnableAllComponentsRecursive(
								getSeSAmMainDesktopFrame().getJMenuBar(),
								true);
							getSeSAmMainDesktopFrame().setCursor(_oldCursor);
						}
					}
				});
			}
		}
	}

	private boolean _makeJIFsInspectable;
	private File _oldDirectory = null;
	private final SeSAmDesktopPane _seSAmDesktopPane;
	private final GUISeSAmDeclaration _guiSeSAmDeclaration;

	private JInternalFrame _userFeatureDeclarationListInternalFrame;
	private JInternalFrame _userInterfaceDeclarationListInternalFrame;
	private JInternalFrame _testRunnerInternalFrame;
	private JInternalFrame _userTypeDeclarationListInternalFrame;
	private JInternalFrame _userFunctionDeclarationListInternalFrame;
	private JInternalFrame _ressourceClassDeclarationListInternalFrame;
	private JInternalFrame _agentClassDeclarationListInternalFrame;
	private JInternalFrame _worldClassDeclarationListInternalFrame;
	private JInternalFrame _simulationDeclarationListInternalFrame;
	private JInternalFrame _situationDeclarationListInternalFrame;
	private JInternalFrame _experimentDeclarationListInternalFrame;

	private JInternalFrame _experimentListInternalFrame;

	private JInternalFrame _analysisDeclarationListInternalFrame;
	private JInternalFrame _modelLibraryInternalFrame;
		

	protected SeSAmDesktopPaneHelper(SeSAmDesktopPane seSAmDesktopPane) {
		super();

		_guiSeSAmDeclaration =
			new GUISeSAmDeclaration(new JInternalFrameFactory(seSAmDesktopPane.getDesktopPane()));
		_seSAmDesktopPane = seSAmDesktopPane;
		
		_oldDirectory = new File(Preferences.userNodeForPackage(getClass())
			.get("LoadSaveDirectory", System.getProperty("user.home")));
	}

	private JInternalFrame addFrame(String title, Component component) {
		return getFrameFactory().addInternalFrame(
			getSeSAmDesktopPane(),
			title,
			component,
			false,
			WindowConstants.HIDE_ON_CLOSE);
	}

	private void closeAllNonRootWindows() {
		for (int i = 0; i < getSeSAmDesktopPane().getDesktopPane().getComponentCount(); i++) {
			Component comp = getSeSAmDesktopPane().getDesktopPane().getComponent(i);
			if ((comp instanceof JInternalFrame)
				&& (comp != _agentClassDeclarationListInternalFrame)
				&& (comp != _ressourceClassDeclarationListInternalFrame)
				&& (comp != _worldClassDeclarationListInternalFrame)
				&& (comp != _simulationDeclarationListInternalFrame)
				&& (comp != _experimentDeclarationListInternalFrame)
				&& (comp != _experimentListInternalFrame)
				&& (comp != _userFeatureDeclarationListInternalFrame)
				&& (comp != _userFunctionDeclarationListInternalFrame)
				&& (comp != _userTypeDeclarationListInternalFrame)
				&& (comp != _situationDeclarationListInternalFrame))
				try {
					((JInternalFrame)comp).setIcon(false);
					((JInternalFrame)comp).setClosed(true);
				} catch (PropertyVetoException exception) {
				}
		}
	}

	private JInternalFrame getAgentClassDeclarationListInternalFrame() {
		if (_agentClassDeclarationListInternalFrame == null) {
			ActivityAgentClassDeclarationListPanel agentClassDeclarationList =
				new ActivityAgentClassDeclarationListPanel();

			agentClassDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration().getSeSAmModelDeclaration().getAgentClassDeclarationListAndFactory());
			agentClassDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_agentClassDeclarationListInternalFrame =
				addFrame(
					agentClassDeclarationList.getDeclarationListModel().getName(),
					agentClassDeclarationList);
			_agentClassDeclarationListInternalFrame.pack();
			_agentClassDeclarationListInternalFrame.setFrameIcon(
				ActivityAgentClassDeclarationListPanel.AGENT_CLASS_LIST_ICON);

		}

		return _agentClassDeclarationListInternalFrame;
	}

	private JInternalFrame getAnalysisDeclarationListInternalFrame() {
		if (_analysisDeclarationListInternalFrame == null) {

			AnalysisDeclarationListPanel analysisDeclarationList = new AnalysisDeclarationListPanel();

			analysisDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getAnalysisDeclarationListAndFactory());
			analysisDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_analysisDeclarationListInternalFrame =
				addFrame(
					analysisDeclarationList.getDeclarationListModel().getName(),
					analysisDeclarationList);
			_analysisDeclarationListInternalFrame.pack();

		}

		return _analysisDeclarationListInternalFrame;
	}

	private JInternalFrame getExperimentDeclarationListInternalFrame() {
		if (_experimentDeclarationListInternalFrame == null) {

			ExperimentDeclarationListPanel experimentDeclarationList = new ExperimentDeclarationListPanel();

			experimentDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getExperimentDeclarationListAndFactory());
			experimentDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_experimentDeclarationListInternalFrame =
				addFrame(
					experimentDeclarationList.getDeclarationListModel().getName(),
					experimentDeclarationList);
			_experimentDeclarationListInternalFrame.pack();

		}

		return _experimentDeclarationListInternalFrame;
	}
	
	private JInternalFrame getUserInterfaceDeclarationListInternalFrame() {
			/*if (_userInterfaceDeclarationListInternalFrame == null) {
				UserInterfaceDeclarationListPanel userInterfaceDeclarationList = new UserInterfaceDeclarationListPanel();

				userInterfaceDeclarationList.setDeclarationListAndFactory(
					getSeSAmDeclaration()
						.getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getUserInterfaceDeclarationListAndFactory());
				userInterfaceDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

				_userInterfaceDeclarationListInternalFrame =
					addFrame(
						userInterfaceDeclarationList.getDeclarationListModel().getName(),
						userInterfaceDeclarationList);
				_userInterfaceDeclarationListInternalFrame.pack();
			}

			return _userInterfaceDeclarationListInternalFrame;*/
	    	return null;
		}

	private JInternalFrame getExperimentListInternalFrame() {
		if (_experimentListInternalFrame == null) {

			ExperimentListPanel experimentList = new ExperimentListPanel();
			experimentList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_experimentListInternalFrame =
				addFrame(((INamed)experimentList.getModel()).getName(), experimentList);
			// _experimentListInternalFrame.pack();

		}

		return _experimentListInternalFrame;
	}

	public JInternalFrameFactory getFrameFactory() {
		return (JInternalFrameFactory)getGUISeSAmDeclaration().getFrameFactory();
	}

	public GUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	private JInternalFrame getRessourceClassDeclarationListInternalFrame() {
		if (_ressourceClassDeclarationListInternalFrame == null) {
			RessourceClassDeclarationListPanel ressourceClassDeclarationList =
				new RessourceClassDeclarationListPanel();

			ressourceClassDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getRessourceClassDeclarationListAndFactory());
			ressourceClassDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_ressourceClassDeclarationListInternalFrame =
				addFrame(
					ressourceClassDeclarationList.getDeclarationListModel().getName(),
					ressourceClassDeclarationList);
			_ressourceClassDeclarationListInternalFrame.pack();
			_ressourceClassDeclarationListInternalFrame.setFrameIcon(
				SeSAmDeclarationTreeCellRenderer.RESSOURCES_CLASS_LIST_ICON);
		}

		return _ressourceClassDeclarationListInternalFrame;
	}

	private ISeSAmDeclaration getSeSAmDeclaration() {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(getGUISeSAmDeclaration());
		Assert.assertNotNull(getGUISeSAmDeclaration().getSeSAmDeclaration());

		return getGUISeSAmDeclaration().getSeSAmDeclaration();
	}

	private final SeSAmDesktopPane getSeSAmDesktopPane() {
		return _seSAmDesktopPane;
	}

	private SeSAmMainDesktopFrame getSeSAmMainDesktopFrame() {
		return getSeSAmDesktopPane().getSeSAmMainDesktopFrame();
	}

	private JInternalFrame getSimulationDeclarationListInternalFrame() {
		if (_simulationDeclarationListInternalFrame == null) {

			SimulationDeclarationListPanel simulationDeclarationList = new SimulationDeclarationListPanel();

			simulationDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getSimulationDeclarationListAndFactory());
			simulationDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_simulationDeclarationListInternalFrame =
				addFrame(
					simulationDeclarationList.getDeclarationListModel().getName(),
					simulationDeclarationList);
			_simulationDeclarationListInternalFrame.pack();
		}

		return _simulationDeclarationListInternalFrame;
	}
	
	protected String getDirectory(){
		try{
			return _oldDirectory.getCanonicalPath();
		}
		catch (Exception e) {
			return System.getProperty("user.dir");
		}
	}

	private JInternalFrame getSituationDeclarationListInternalFrame() {
		if (_situationDeclarationListInternalFrame == null) {
			SituationDeclarationListPanel situationDeclarationList = new SituationDeclarationListPanel();

			situationDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getSituationDeclarationListAndFactory());
			situationDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_situationDeclarationListInternalFrame = addFrame("Situations", situationDeclarationList);
			_situationDeclarationListInternalFrame.pack();
		}

		return _situationDeclarationListInternalFrame;
	}

	private JInternalFrame getUserFeatureDeclarationListInternalFrame() {
		if (_userFeatureDeclarationListInternalFrame == null) {
			UserFeatureClassDeclarationListPanel userFeatureDeclarationListPanel =
				new UserFeatureClassDeclarationListPanel();

			userFeatureDeclarationListPanel.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSeSAmModelBasicsDeclaration()
					.getUserFeatureClassDeclarationListAndFactory());
			userFeatureDeclarationListPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_userFeatureDeclarationListInternalFrame =
				addFrame(
					userFeatureDeclarationListPanel.getDeclarationListModel().getName(),
					userFeatureDeclarationListPanel);
			_userFeatureDeclarationListInternalFrame.pack();
		}

		return _userFeatureDeclarationListInternalFrame;
	}

	private JInternalFrame getUserFunctionDeclarationListInternalFrame() {
		if (_userFunctionDeclarationListInternalFrame == null) {
			UserFunctionDeclarationListPanel userFunctionDeclarationListPanel =
				new UserFunctionDeclarationListPanel();

			userFunctionDeclarationListPanel.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSeSAmModelBasicsDeclaration()
					.getUserFunctionDeclarationListAndFactory());
			userFunctionDeclarationListPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_userFunctionDeclarationListInternalFrame =
				addFrame(
					userFunctionDeclarationListPanel.getDeclarationListModel().getName(),
					userFunctionDeclarationListPanel);
			_userFunctionDeclarationListInternalFrame.pack();
		}

		return _userFunctionDeclarationListInternalFrame;
	}

	private JInternalFrame getUserTypeDeclarationListInternalFrame() {
		if (_userTypeDeclarationListInternalFrame == null) {
			UserTypeDeclarationListPanel userTypeDeclarationListPanel = new UserTypeDeclarationListPanel();

			userTypeDeclarationListPanel.setDeclarationListAndFactory(
				getSeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSeSAmModelBasicsDeclaration()
					.getUserTypeDeclarationListAndFactory());
			userTypeDeclarationListPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_userTypeDeclarationListInternalFrame =
				addFrame(
					userTypeDeclarationListPanel.getDeclarationListModel().getName(),
					userTypeDeclarationListPanel);
			_userTypeDeclarationListInternalFrame.pack();
		}

		return _userTypeDeclarationListInternalFrame;
	}

	private JInternalFrame getWorldClassDeclarationListInternalFrame() {
		if (_worldClassDeclarationListInternalFrame == null) {
			ActivityWorldClassDeclarationListPanel worldClassDeclarationList =
				new ActivityWorldClassDeclarationListPanel();

			worldClassDeclarationList.setDeclarationListAndFactory(
				getSeSAmDeclaration().getSeSAmModelDeclaration().getWorldClassDeclarationListAndFactory());
			worldClassDeclarationList.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_worldClassDeclarationListInternalFrame =
				addFrame(
					worldClassDeclarationList.getDeclarationListModel().getName(),
					worldClassDeclarationList);
			_worldClassDeclarationListInternalFrame.pack();
			_worldClassDeclarationListInternalFrame.setFrameIcon(
				SeSAmDeclarationTreeCellRenderer.WORLD_CLASS_LIST_ICON);
		}

		return _worldClassDeclarationListInternalFrame;
	}
	
	private JInternalFrame getModelLibraryInternalFrame(SeSAmDesktopPane pane){
		if (_modelLibraryInternalFrame == null){
			_modelLibraryInternalFrame = new ModelLibraryInternalFrame(pane); 
		}
		
		return _modelLibraryInternalFrame;
		
	}

	public synchronized boolean loadModel(final File file) {
	
		_oldDirectory = file.getParentFile();
		final File oldFile = getSeSAmDeclaration().getFile();

		try {
			getSeSAmDeclaration().setFile(file);

			SwingMoreUtilities.setEnableAllComponentsRecursive(
				getSeSAmMainDesktopFrame().getContentPane(),
				false);
			SwingMoreUtilities.setEnableAllComponentsRecursive(
				getSeSAmMainDesktopFrame().getJMenuBar(),
				false);

			final Cursor oldCursor = getSeSAmMainDesktopFrame().getCursor();

			getSeSAmMainDesktopFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			getSeSAmDeclaration().getSeSAmDeclarationTreeModel().setEventCreation(false);

			FileInputStream fileInputStream = new FileInputStream(getSeSAmDeclaration().getFile());
			final ProgressMonitorInputStream progressMonitorInputStream =
				new ProgressMonitorInputStream(
					getSeSAmDesktopPane(),
					"Reading File ...",
					new BufferedInputStream(fileInputStream));

			final InputStreamReader inputStreamReader = new InputStreamReader(progressMonitorInputStream);

			Thread loadingThread =
				new LoadingThread(oldCursor, progressMonitorInputStream, inputStreamReader, oldFile, file);

			loadingThread.setPriority(Thread.MAX_PRIORITY);
			loadingThread.start();

			Thread.yield();

			return true;

		} catch (Exception exception2) {
			getGUISeSAmDeclaration().showException(
				"Error during loading model file",
				exception2,
				getSeSAmDesktopPane());
			getSeSAmDeclaration().clear();
			getSeSAmDeclaration().setFile(oldFile);
			return false;
		}
	}

	public void newModel() {
		if (getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified()){
			if(!askToSaveChanges("Your model has unsaved changes. Continue and discard changes?", new String[]{"New model", "Save", "Save as...", "Cancel"}))
				return;
		}
		
		getGUISeSAmDeclaration().clear();
		getSeSAmDeclaration().setFile(null);
		getSeSAmMainDesktopFrame().getSeSAmDeclarationTreePanel().expandFirstCol();
		closeAllNonRootWindows();
		
	}

	public boolean openModel() {
		if (getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified())
			if(!askToSaveChanges("Your model has unsaved changes. Continue and discard changes?", new String[]{"Open model", "Save", "Save as...", "Cancel"}))
				return false;
		
		JFileChooser fileChooser = new JBetterFileChooser();

		fileChooser.addChoosableFileFilter(new ExtensionsFileFilter("SeSAm model (*.xml)", "xml"));

		if (_oldDirectory != null) {
			fileChooser.setCurrentDirectory(_oldDirectory);

		} else if (getSeSAmDeclaration().getFile() != null) {
			fileChooser.setCurrentDirectory(getSeSAmDeclaration().getFile());
		}

		if (fileChooser.showOpenDialog(getSeSAmDesktopPane()) == JFileChooser.APPROVE_OPTION) {
			return loadModel(fileChooser.getSelectedFile());
		} else {
			return false;
		}
		
	}

	public boolean saveAsModel() {

		File oldFile = getSeSAmDeclaration().getFile();
		File newFile =
			choiceFile(
				getSeSAmDeclaration().getFile(),
				new ExtensionsFileFilter("SeSAm model (*.xml)", "xml"));

		if (newFile == null) {
			return false;
		}

		getSeSAmDeclaration().setFile(newFile);
		newFile = getSeSAmDeclaration().getFile();

		if (!newFile.exists()) {
			saveModelIntern(newFile);
			return true;

		} else {
			JOptionPane optionPane =
				new JOptionPane(
					"Replace existing file?",
					JOptionPane.QUESTION_MESSAGE,
					JOptionPane.YES_NO_CANCEL_OPTION);

			getGUISeSAmDeclaration().getFrameFactory().addModalPanel(
				getSeSAmMainDesktopFrame(),
				"Saving into '" + newFile.getName() + "'",
				optionPane);

			if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.YES_OPTION)) {
				saveModelIntern(newFile);

				return true;

			} else if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.NO_OPTION)) {
				return saveAsModel();

			} else {
				getSeSAmDeclaration().setFile(oldFile);
				return false;
			}
		}

	}

	private File choiceFile(File oldFile, ExtensionsFileFilter extensionsFileFilter) {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.addChoosableFileFilter(extensionsFileFilter);

		if (oldFile != null) {
			fileChooser.setSelectedFile(oldFile);

		} else if (_oldDirectory != null) {
			fileChooser.setCurrentDirectory(_oldDirectory);
		}

		if (fileChooser.showSaveDialog(getSeSAmDesktopPane()) != JFileChooser.APPROVE_OPTION) {
			return null;
		} else {
			return fileChooser.getSelectedFile();
		}
	}

	public boolean saveModel() {
		if (getSeSAmDeclaration().getFile() == null)
			return saveAsModel();

		else {
			File file = getSeSAmDeclaration().getFile();
			if (file.exists()) {
				return saveModelIntern(file);

			} else {
				return saveAsModel();
			}
		}
	}

	private boolean saveModelIntern(File file) {
		final Cursor oldCursor = getSeSAmMainDesktopFrame().getCursor();

		try {
			_oldDirectory = file.getParentFile();
			getSeSAmMainDesktopFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

			getGUISeSAmDeclaration().saveModel(file);

			getGUISeSAmDeclaration().getSeSAmDeclaration().setModelModified(false);

			getSeSAmMainDesktopFrame().addModelFileMenuEntryAndToPrefs(file);

			return true;

		} catch (Throwable exception) {
			exception.printStackTrace();
			getGUISeSAmDeclaration().showException(
				"Error during saving model file",
				exception,
				getSeSAmDesktopPane());
			return false;

		} finally {
			getSeSAmMainDesktopFrame().setCursor(oldCursor);
		}
	}

	public void showRemoteSimTaskList() {
		final RemoteSimulationControlPanel remoteSimulationControlPanel = new RemoteSimulationControlPanel();

		remoteSimulationControlPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

		getFrameFactory().addInternalFrame(
			getSeSAmDesktopPane(),
			"Remote Simulation Runs",
			remoteSimulationControlPanel,
			true,
			WindowConstants.DISPOSE_ON_CLOSE);

		new Thread(new Runnable() {
			public void run() {
				remoteSimulationControlPanel.updateClientList();
			}
		}, "Update Client List").start();
	}

	public void showAgentClassDeclarationList() {
		makeVisible(getAgentClassDeclarationListInternalFrame());
	}

	public void showAnalysisDeclarationList() {
		makeVisible(getAnalysisDeclarationListInternalFrame());
	}
	
	public void showUserInterfaceDeclarationList() {
		makeVisible(getUserInterfaceDeclarationListInternalFrame());
	}

	public void showExperimentDeclarationList() {
		makeVisible(getExperimentDeclarationListInternalFrame());
	}

	public void showExperimentList() {
		makeVisible(getExperimentListInternalFrame());
	}

	public void showRessourceClassDeclarationList() {
		makeVisible(getRessourceClassDeclarationListInternalFrame());
	}

	public void showSimulationDeclarationList() {
		makeVisible(getSimulationDeclarationListInternalFrame());
	}

	public void showSituationDeclarationList() {
		makeVisible(getSituationDeclarationListInternalFrame());
	}

	public void showUserFeatureDeclarationList() {
		makeVisible(getUserFeatureDeclarationListInternalFrame());
	}

	public void showUserFunctionDeclarationList() {
		makeVisible(getUserFunctionDeclarationListInternalFrame());
	}

	public void showUserTypeDeclarationList() {
		makeVisible(getUserTypeDeclarationListInternalFrame());
	}

	public void showWorldClassDeclarationList() {
		makeVisible(getWorldClassDeclarationListInternalFrame());
	}
	
	public void showModelLibrary(SeSAmDesktopPane pane){
		makeVisible(getModelLibraryInternalFrame(pane));
	}

	public void makeVisible(JInternalFrame frame) {
		SwingMoreUtilities.makeVisible(frame, getSeSAmDesktopPane().getDesktopPane(), getMakeJIFsInspectable());
	}

	public void showTestRunner() {
		makeVisible(getTestRunnerInternalFrame());
	}

	private JInternalFrame getTestRunnerInternalFrame() {
		//TODO reenable
		//if (_testRunnerInternalFrame == null) {
			TestRunner testRunner = new TestRunner();
		testRunner.setBorder(Borders.createEmptyBorder("2dlu,2dlu,1dlu,2dlu"));

			testRunner.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			_testRunnerInternalFrame = addFrame("Test Runs", testRunner);
			_testRunnerInternalFrame.pack();
		//}

		return _testRunnerInternalFrame;
	}
	
	public boolean askToSaveChanges(String title, String[] options){
		JOptionPane optionPane =
			new JOptionPane(
				title,
				JOptionPane.QUESTION_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION,
				null,
				options,
				options[1]);

		GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
			_seSAmDesktopPane,
			"Unsaved changes",
			optionPane);
				
		if (options[0].equals(optionPane.getValue())) {
			return true;
		}else if (options[1].equals(optionPane.getValue())) {
			return saveModel();
		}else if (options[2].equals(optionPane.getValue())) {
			return saveAsModel();
		}else{
			return false;
		}
	}
	
	protected void setMakeJIFsInspectable(boolean enabled){
		_makeJIFsInspectable = enabled;
		((JInternalFrameFactory) getGUISeSAmDeclaration().getFrameFactory()).setMakeJIFsInspectable(enabled);
	}
	
	
	protected boolean getMakeJIFsInspectable(){
		return _makeJIFsInspectable;	
	}
	
	public void exportModel(){
		
		if (getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified()){
			if(!askToSaveChanges("Your model has unsaved changes. Continue?", new String[]{"Export model", "Save", "Save as...", "Cancel"}))
				return;
		}
		
		makeVisible(new ExportModelInternalFrame(getGUISeSAmDeclaration().getSeSAmDeclaration().getFile(), _oldDirectory));

	}
	
	public void importModel(){
			
	}
	
}