package de.uniwue.ki.gui.mas.application;
import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.DateFormat;
import java.util.Date;
import java.util.prefs.Preferences;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.*;
import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreeCellRenderer;
import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreePanel;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.mdi.MDIWindowMenu;
import de.uniwue.ki.mas.model.*;
import eu.troyer.model.commands.CommandExecutionEvent;
import eu.troyer.view.docking.*;

/**
 * Creation date: (06.10.2000 18:38:31)
 * @author Christoph Oechslein
 */
class SeSAmMainDesktopFrame extends JFrame implements ISeSAmMainDesktopFrame {

	private SeSAmMainDesktopFrameMenuBar _seSAmMainDesktopFrameMenuBar;

	private final EventHandler _eventHandler = new EventHandler();
	private TisContainerPanel _rootTisContainerPanel = null;
	private TisDockablePanelFactoryConfig _tisDockablePanelFactoryConfigTree = null;
	private JPanel _mainPanel = null;
	private SeSAmDesktopPane _seSAmDesktopPane = null;
	private DockablePanelFactoryManager _dockablePanelFactoryManager = null;

	class EventHandler extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			eventCalledExit();
		}
	}

	public SeSAmMainDesktopFrame() {
		super();
		initialize();
	}

	protected DockablePanelFactoryManager createDockablePanelFactoryManager() {
		// instantiate a dockable panel factory:
		DockablePanelFactoryManager aFactoryManager = new DockablePanelFactoryManager("FactoryManagerId");

		return aFactoryManager;
	}

	protected ISeSAmDeclaration createSeSAmDeclaration() {
		return SeSAmDeclarationFactory.createSeSAmDeclaration();
	}

	protected TisDockablePanelFactoryConfig createTisDockablePanelFactoryConfig(DockablePanelFactoryManager aFactoryManager) {
		// instantiate a dockable panel factory configurator:
		TisDockablePanelFactoryConfig aDockableFactoryConfig = new TisDockablePanelFactoryConfig();

		// set configurator's factory name and ID (used for registration and lookup) : 
		aDockableFactoryConfig.setFactoryName("OurPanelFactoryName");
		aDockableFactoryConfig.setFactoryId("OurPanelFactoryID");

		// set the DockablePanelFactoryManager object:
		aDockableFactoryConfig.setFactoryManager(aFactoryManager);

		// set the initial position of appearance (see the note*):
		aDockableFactoryConfig.setInitialPosition(IDockingConstants.POSITION_WEST);

		// set the class name of OurPanel to "become" dockable:
		aDockableFactoryConfig.setPanelClassName("full_package_name. OurPanel");

		return aDockableFactoryConfig;
	}

	protected void eventCalledExit() {
		if (getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified()) {

			Object[] options = { "Quit", "Save", "Cancel" };

			JOptionPane optionPane =
				new JOptionPane(
					"Exit and discard changes?",
					JOptionPane.QUESTION_MESSAGE,
					JOptionPane.OK_CANCEL_OPTION,
					null,
					options,
					options[1]);

			GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
				getDesktopPane(),
				"Quit SeSAm",
				optionPane);

			if (options[0].equals(optionPane.getValue())) {
				savePrefsAndCallExit();

			} else if (options[1].equals(optionPane.getValue())) {
				if (saveModel()) {
					savePrefsAndCallExit();
				}
			}

		} else {
			JOptionPane optionPane =
				new JOptionPane("Are you sure?", JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

			GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
				getDesktopPane(),
				"Quit SeSAm",
				optionPane);

			if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.YES_OPTION)) {
				savePrefsAndCallExit();
			}
		}

	}

	protected void savePrefsAndCallExit() {
		savePrefs();

		if ((getSeSAmDesktopPane() != null) && (getSeSAmDesktopPane().getDesktopPane() != null)) {
			MDIWindowMenu.closeAllFrames(getSeSAmDesktopPane().getDesktopPane());
		}

		dispose();

		try {
			getGUISeSAmDeclaration().getSeSAmDeclaration().remove();

		} catch (Exception e) {
			e.printStackTrace(System.err);
		}

		synchronized (this) {
			try {
				wait(10 * 1000);
			} catch (InterruptedException e) {
			}
		}

		System.exit(0);
	}
	
	protected void savePrefs() {
		Preferences userprefs = getUserPreferences();

		userprefs.putInt("X", getX());
		userprefs.putInt("Y", getY());
		userprefs.putInt("Width", getWidth());
		userprefs.putInt("Height", getHeight());
		userprefs.putInt("ExtendedState", getExtendedState());
		userprefs.put("LoadSaveDirectory", _seSAmDesktopPane.getDirectory());

		Preferences treePrefs = getTreePrefs();
		IDockablePanel dockablePanel = getTisDockablePanelFactoryConfigTree().getDockable();

		if (dockablePanel != null) {
			DockingInfo dockingInfo = dockablePanel.getDockingInfo();

			if (dockingInfo != null) {
				treePrefs.putInt("position", dockingInfo.getCurrentPosition());

				dockablePanel.updateDockingInfo();
				treePrefs.put("dockingInfo", dockingInfo.toString());

				if (dockablePanel.isFloating()) {
					TisDockableFloatingWindow dockableFloatingWindow =
						getTisDockableFloatingWindow(dockablePanel);

					if (dockableFloatingWindow != null) {
						treePrefs.putInt("x", dockableFloatingWindow.getX());
						treePrefs.putInt("y", dockableFloatingWindow.getY());
						treePrefs.putInt("width", dockableFloatingWindow.getWidth());
						treePrefs.putInt("height", dockableFloatingWindow.getHeight());
					}
				}
			}
		}
	}

	protected TisDockableFloatingWindow getTisDockableFloatingWindow(IDockablePanel dockablePanel) {
		Component c = (Component)dockablePanel;

		while (!(c instanceof TisDockableFloatingWindow) && (c != null)) {
			c = c.getParent();
		}

		TisDockableFloatingWindow dockableFloatingWindow = (TisDockableFloatingWindow)c;
		return dockableFloatingWindow;
	}

	protected Preferences getTreePrefs() {
		return getUserPreferences().node("tree");
	}

	public static Dimension getWidthAndHeight(int defaultWidth, int defaultHeight) {
		Preferences userprefs = Preferences.userNodeForPackage(SeSAmMainDesktopFrame.class);
		return new Dimension(
			userprefs.getInt("Width", defaultWidth),
			userprefs.getInt("Height", defaultHeight));
	}

	protected void setUserDefinedDimension(int defaultWidth, int defaultHeight) throws HeadlessException {

		Preferences userprefs = getUserPreferences();

		int extendedState = userprefs.getInt("ExtendedState", Frame.NORMAL);

		if ((extendedState == Frame.NORMAL) || (extendedState == Frame.ICONIFIED)) {
			/* Set Size */
			setSize(
				(int)Math.min(Toolkit.getDefaultToolkit().getScreenSize().getWidth(), defaultWidth),
				(int)Math.min(Toolkit.getDefaultToolkit().getScreenSize().getHeight(), defaultHeight));

			int x = userprefs.getInt("X", -1);
			int y = userprefs.getInt("Y", -1);

			if ((x > -1) && (y > -1)) {
				setLocation(
					(int)Math.min(Toolkit.getDefaultToolkit().getScreenSize().getWidth() - getWidth(), x),
					(int)Math.min(Toolkit.getDefaultToolkit().getScreenSize().getHeight() - getHeight(), y));

			} else {
				/* Center frame on the screen */
				SwingMoreUtilities.centerWindow(this);
			}
		} else {
			setExtendedState(extendedState);
		}

		Preferences treePrefs = getTreePrefs();
		IDockablePanel dockablePanel = getTisDockablePanelFactoryConfigTree().getDockable();

		dockablePanel.updateDockingInfo();
		dockablePanel.setDockingInfo(
			DockingInfo.fromString(treePrefs.get("dockingInfo", dockablePanel.getDockingInfo().toString())));

		if (dockablePanel.isFloating()) {
			TisDockableFloatingWindow dockableFloatingWindow = getTisDockableFloatingWindow(dockablePanel);
			dockableFloatingWindow.setBounds(
				treePrefs.getInt("x", dockableFloatingWindow.getX()),
				treePrefs.getInt("y", dockableFloatingWindow.getY()),
				treePrefs.getInt("width", dockableFloatingWindow.getWidth()),
				treePrefs.getInt("height", dockableFloatingWindow.getHeight()));
		}
	}

	protected static Preferences getUserPreferences() {
		return Preferences.userNodeForPackage(SeSAmMainDesktopFrame.class);
	}

	protected void eventTisCommandSupportCommandExecuted(CommandExecutionEvent event) {
		String commandName = event.getCommandName();
		if (commandName.equals("New")) {
			//
		} else if (commandName.equals("Word Wrap")) {
			/*
			cm.setCommandState(
			   commandName,
			   cm.getCommandState(commandName) == ICommandSupport.STATE_SELECTED
				  ? ICommandSupport.STATE_DEFAULT
				  : ICommandSupport.STATE_SELECTED);
			ordWrap(cm.getCommandState(commandName) == ICommandSupport.STATE_SELECTED);
			*/
		}
	}

	public JDesktopPane getDesktopPane() {
		return getSeSAmDesktopPane().getDesktopPane();
	}

	/**
	 * Return the DockablePanelFactoryManager property value.
	 * @return DockablePanelFactoryManager
	 */
	private DockablePanelFactoryManager getDockablePanelFactoryManager() {
		if (_dockablePanelFactoryManager == null) {
			_dockablePanelFactoryManager = new DockablePanelFactoryManager();
		}

		return _dockablePanelFactoryManager;
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return getSeSAmDesktopPane().getGUISeSAmDeclaration();
	}

	private JPanel getMainPanel() {
		if (_mainPanel == null) {
			_mainPanel = new JPanel();
			_mainPanel.setName("MainPanel");
			_mainPanel.setLayout(new BorderLayout());
			getMainPanel().add(getRootTisContainerPanel(), "Center");
		}
		return _mainPanel;
	}

	private TisContainerPanel getRootTisContainerPanel() {
		if (_rootTisContainerPanel == null) {
			_rootTisContainerPanel = new TisContainerPanel();
			_rootTisContainerPanel.setName("RootTisContainerPanel");
			getRootTisContainerPanel().add(getSeSAmDesktopPane(), "Center");
		}
		return _rootTisContainerPanel;
	}

	protected SeSAmDeclarationTreePanel getSeSAmDeclarationTreePanel() {
		return (SeSAmDeclarationTreePanel)getTisDockablePanelFactoryConfigTree()
			.getDockable()
			.getContentPanel();
	}

	protected SeSAmDesktopInternalFrame getSeSAmDesktopInternalFrameFor(final SeSAmDesktopPane seSAmDesktopPane) {
		return (SeSAmDesktopInternalFrame)seSAmDesktopPane.getRootPane().getParent();
	}

	protected SeSAmDesktopPane getSeSAmDesktopPane() {
		if (_seSAmDesktopPane == null) {
			_seSAmDesktopPane = new SeSAmDesktopPane();
			_seSAmDesktopPane.setName("SeSAmDesktopPane");
			_seSAmDesktopPane.setSeSAmMainDesktopFrame(this);
		}
		return _seSAmDesktopPane;
	}

	SeSAmMainDesktopFrameMenuBar getSeSAmMainDesktopFrameMenuBar() {
		if (_seSAmMainDesktopFrameMenuBar == null) {
			_seSAmMainDesktopFrameMenuBar =
				new SeSAmMainDesktopFrameMenuBar(getSeSAmDesktopPane(), new Runnable() {
				public void run() {
					eventCalledExit();
				}
			});
		}

		return _seSAmMainDesktopFrameMenuBar;
	}

	protected TisDockablePanelFactoryConfig getTisDockablePanelFactoryConfigTree() {
		if (_tisDockablePanelFactoryConfigTree == null) {
			_tisDockablePanelFactoryConfigTree = new TisDockablePanelFactoryConfig();
			_tisDockablePanelFactoryConfigTree.setFactoryName(SeSAmDeclarationTreePanel.class.getName());
			_tisDockablePanelFactoryConfigTree.setFactoryId(SeSAmDeclarationTreePanel.class.getName());

			_tisDockablePanelFactoryConfigTree.setInitialPosition(
				getTreePrefs().getInt("position", IDockingConstants.POSITION_WEST));

			_tisDockablePanelFactoryConfigTree.setPanelClassName(SeSAmDeclarationTreePanel.class.getName());
		}
		return _tisDockablePanelFactoryConfigTree;
	}

	private void initConnections() {
		this.addWindowListener(_eventHandler);

		getTisDockablePanelFactoryConfigTree().setFactoryManager(getDockablePanelFactoryManager());

		getRootTisContainerPanel().setDockablePanelFactoryManager(getDockablePanelFactoryManager());
	}

	private void initialize() {
		setName("SeSamMainDesktopPane");
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		setTitle("SeSAm");
		setIconImage(SeSAmDeclarationTreeCellRenderer.SITUATION_ICON.getImage());
		setSize(610, 338);
		setJMenuBar(getSeSAmMainDesktopFrameMenuBar());
		setContentPane(getMainPanel());
		initConnections();
		_initHelper.userInitialize();
	}

	protected void updateFrameTitle() {
		File file = getGUISeSAmDeclaration().getSeSAmDeclaration().getFile();

		StringBuffer buff = new StringBuffer();

		buff.append("SeSAm v");
		buff.append(SeSAmVersion.getVersionNumber());
		buff.append(" [");

		if (file == null) {
			buff.append("<Untitled>");

		} else {
			buff.append(file.getName());
			if (getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified()) {
				buff.append("*");
			}
		}
		buff.append("]");
		buff.append(" (");
		if (SeSAmVersion.isBuildWithAnt())
			buff.append(
				"build #" + SeSAmVersion.getBuildNumber() + " at " + SeSAmVersion.getAntBuildDateString());
		else
			buff.append(DateFormat.getDateTimeInstance().format(new Date()));
		buff.append(")");

		setTitle(buff.toString());
	}

	private final InitHelper _initHelper = new InitHelper();

	private class InitHelper implements PropertyChangeListener {
		protected void userInitialize() {

			getSeSAmDeclarationTreePanel().setSeSAmMainDesktopFrame(SeSAmMainDesktopFrame.this);
			getSeSAmDeclarationTreePanel().setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			userInitializeAddUpdateFrameTitleListener();

			//Dimension dockTreePanelSize = getTisDockablePanelFactoryConfigTree().getDockable().getSize();
			Dimension treePanelPreferredSize = getSeSAmDeclarationTreePanel().getPreferredSize();

			getTisDockablePanelFactoryConfigTree().getDockable().setSize(
				treePanelPreferredSize.width + 10,
				treePanelPreferredSize.height);
		}

		private void userInitializeAddUpdateFrameTitleListener() {
			updateFrameTitle();

			getGUISeSAmDeclaration().getSeSAmDeclaration().addPropertyChangeListener(this);

			getGUISeSAmDeclaration().getSeSAmDeclaration().addPropertyChangeListener(this);
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if ((evt.getSource() == getGUISeSAmDeclaration().getSeSAmDeclaration())
				&& (("file".equals(evt.getPropertyName())
					|| ("modelModified".equals(evt.getPropertyName()))))) {
				updateFrameTitle();
			}
		}
	}

	public void showAgentClassDeclarationList() {
		getSeSAmDesktopPane().showAgentClassDeclarationList();
	}

	public void showAnalysisDeclarationList() {
		getSeSAmDesktopPane().showAnalysisDeclarationList();
	}

	public void showExperimentDeclarationList() {
		getSeSAmDesktopPane().showExperimentDeclarationList();
	}

	public void showExperimentList() {
		getSeSAmDesktopPane().showExperimentList();
	}

	public void showRessourceClassDeclarationList() {
		getSeSAmDesktopPane().showRessourceClassDeclarationList();
	}

	public void showRemoteSimTaskList() {
		getSeSAmDesktopPane().showRemoteSimTaskList();
	}

	public void showSimulationDeclarationList() {
		getSeSAmDesktopPane().showSimulationDeclarationList();
	}

	public void showSituationDeclarationList() {
		getSeSAmDesktopPane().showSituationDeclarationList();
	}

	public void showUserFeatureDeclarationList() {
		getSeSAmDesktopPane().showUserFeatureDeclarationList();
	}

	public void showUserFunctionDeclarationList() {
		getSeSAmDesktopPane().showUserFunctionDeclarationList();
	}

	public void showUserTypeDeclarationList() {
		getSeSAmDesktopPane().showUserTypeDeclarationList();
	}

	public void showWorldClassDeclarationList() {
		getSeSAmDesktopPane().showWorldClassDeclarationList();
	}

	public boolean saveModel() {
		return getSeSAmDesktopPane().saveModel();
	}

	public boolean openModel() {
		return getSeSAmDesktopPane().openModel();
	}

	public boolean saveAsModel() {
		return getSeSAmDesktopPane().saveAsModel();
	}

	public void loadModel(File modelFile) {
		getSeSAmDesktopPane().loadModel(modelFile);
	}

	void addModelFileMenuEntryAndToPrefs(File modelFile) {
		getSeSAmMainDesktopFrameMenuBar().addModelFileMenuEntryAndToPrefs(modelFile);
	}

	protected void processKeyEvent(KeyEvent e) {
		super.processKeyEvent(e);
	}

public void showUserInterfaceDeclarationList() {
		getSeSAmDesktopPane().showUserInterfaceDeclarationList();
	}	
}