package de.uniwue.ki.gui.mas.application;

import java.awt.event.*;
import java.io.File;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.*;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.basics.functionUser.UserFunctionDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.entity.objectclass.ActivityAgentClassDeclarationListPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.graph.UMLLayoutGraphDeclarationGraphPanelContainer;
import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreeCellRenderer;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentListPanelHelper;
import de.uniwue.ki.gui.mas.sim.test.TestRunner;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.lookAndFeel.LookAndFeelManager;
import de.uniwue.ki.gui.util.mdi.MDIDesktopPane;
import de.uniwue.ki.gui.util.mdi.MDIWindowMenu;
import de.uniwue.ki.mas.basics.action.debugging.FunctionActionDebuggingDeclarationFactory;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.SeSAmDeclarationFactory;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (06.10.2000 18:38:31)
 * 
 * @author Christoph Oechslein
 */

final class SeSAmMainDesktopFrameMenuBar extends JMenuBar {

    public static final ImageIcon NETWORK_ICON = SwingMoreUtilities.createImageIcon("others/network.gif");

    final class EventHandler implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileNewModel())
                newModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileOpenModel())
                openModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileSaveModelAs())
                saveAsModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileSaveModel())
                saveModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemDebugInspectSeSAmDecl())
                inspectSeSAm();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemHelpAbout())
                showAboutBox();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileExit())
                eventCalledExit();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileImport())
				importModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemFileExport())
				exportModel();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelWorlds())
                showWorldClassDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemSimulationRemoteRuns())
                showRemoteSimTaskList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemSimulationRuns())
                showExperimentList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelAgents())
                showAgentClassDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelFunctions())
                showUserFunctionDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelFeatures())
                showUserFeatureDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelSituations())
                showSituationDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelSimulations())
                showSimulationDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemAnalysis())
                showAnalysisDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelTypes())
                showUserTypeDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelRessources())
                showRessourceClassDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemExperiments())
                showExperimentDeclarationList();
            else if (e.getSource() == SeSAmMainDesktopFrameMenuBar.this.getJMenuItemModelLibrary())
                showModelLibrary();

        }
    }

    private JCheckBoxMenuItem _closuresCheckBoxMenuItem;

    private JCheckBoxMenuItem _compileCheckBoxMenuItem;

    private final EventHandler _eventHandler = new EventHandler();

    private final Runnable _exitFunction;

    private JCheckBoxMenuItem _inlineCheckBoxMenuItem;

    private JCheckBoxMenuItem _jCompileAssertCheckBox = null;

    private JMenu _jEditableMenu = null;

    private JCheckBoxMenuItem _jGlobalAssertCheckBox = null;

    private JMenu _jMenuDebug = null;

    private JMenu _jMenuDebugAssert = null;

    private JMenu _jMenuFile = null;

    private JMenu _jMenuHelp = null;

    private JMenuItem _jMenuItemAnalysis = null;

    private JMenuItem _jMenuItemDebugInspectSeSAmDecl = null;

    private JMenuItem _jMenuItemDebugModelGraph = null;

    private JMenuItem _jMenuItemExperiments = null;

    private JMenuItem _jMenuItemFileExit = null;

    private JMenuItem _jMenuItemFileNewModel = null;

    private JMenuItem _jMenuItemFileOpenModel = null;

    private JMenuItem _jMenuItemFileSaveModel = null;

    private JMenuItem _jMenuItemFileSaveModelAs = null;

    private JMenuItem _jMenuItemFileImport = null;
	
	private JMenuItem _jMenuItemFileExport = null;

    private JMenuItem _jMenuItemHelpAbout = null;

    private JMenuItem _jMenuItemModelAgents = null;

    private JMenuItem _jMenuItemTestSimulationRuns = null;

    private JMenuItem _jMenuItemModelFeatures = null;

    private JMenuItem _jMenuItemModelFunctions = null;

    private JMenuItem _jMenuItemModelRessources = null;

    private JMenuItem _jMenuItemModelSimulations = null;

    private JMenuItem _jMenuItemModelSituations = null;

    private JMenuItem _jMenuItemModelTypes = null;

    private JMenuItem _jMenuItemModelWorlds = null;

    private JMenuItem _jMenuItemSimulationRemoteRuns = null;

    private JMenuItem _jMenuItemSimulationRuns = null;

    private JMenuItem _jMenuItemModelLibrary = null;

    private JMenu _jMenuModel = null;

    private JMenu _jMenuModelBasics = null;

    private JMenu _jMenuSimElements = null;

    private JMenu _jMenuSimulationRuns = null;

    private JCheckBoxMenuItem _jRunAssertCheckBox = null;

    private JCheckBoxMenuItem _jMenuItemDebugInspectJIF = null;

    private LinkedList _modelFileList = new LinkedList();

    private JMenu _pluginsMenu = null;

    private final SeSAmDesktopPane _seSAmDesktopPane;

    public SeSAmMainDesktopFrameMenuBar(SeSAmDesktopPane seSAmDesktopPane, Runnable exitFunction) {
        super();

        _seSAmDesktopPane = seSAmDesktopPane;
        _exitFunction = exitFunction;

        initialize();
    }

    private void addFileListMenus() {
        Preferences openModelFileListPreferences = getOpenModelPreferences();

        String[] openModelFiles;
        try {
            openModelFiles = openModelFileListPreferences.keys();

        } catch (BackingStoreException e) {
            return;
        } // remove old keys
        int MAX_FILE_SIZE = 5;
        for (int i = 0; i < openModelFiles.length - MAX_FILE_SIZE; i++) {
            openModelFileListPreferences.remove(openModelFiles[i]);
        }

		for (int i = Math.max(openModelFiles.length - MAX_FILE_SIZE, 0); i < openModelFiles.length; i++) {
            File modelFile = new File(openModelFileListPreferences.get(openModelFiles[i], ""));

            if (modelFile.exists() && !_modelFileList.contains(modelFile)) {
                _modelFileList.add(modelFile);
                addModelFileMenuEntry(modelFile);

            } else {
                openModelFileListPreferences.remove(openModelFiles[i]);
            }
        }
    }

    private void addModelFileMenuEntry(final File modelFile) {

        String menuItemName = modelFile.toString();
        int MAX_LENGTH = 25;

        if (menuItemName.length() > MAX_LENGTH) {
            String fileName = modelFile.getName();
            String pathName = modelFile.getParent();

            if (pathName == null)
                pathName = "";

            int pathPortion = (int) Math.ceil((MAX_LENGTH - 4) * 0.3);
            int filePortion = (MAX_LENGTH - 4) - pathPortion;

            if (pathPortion > pathName.length()) {
                filePortion += pathPortion - pathName.length();
                pathPortion = pathName.length();
            }

            if (filePortion > fileName.length()) {
                pathPortion += filePortion - fileName.length();
                filePortion = fileName.length();
            }

            menuItemName =
                pathName.substring(0, Math.max(0, pathPortion))
                    + "/.../"
                    + fileName.substring(Math.max(0, fileName.length() - filePortion));
        }

        int filePos = getJMenuFile().getItemCount() - 9;

        menuItemName = filePos + ": " + menuItemName;

        JMenuItem fileItem = AbstractBoundAction.createJMenuItem(new AbstractBoundAction(menuItemName) {

            public void doit() {
            	if (getSeSAmDesktopPane().getGUISeSAmDeclaration().getSeSAmDeclaration().isModelModified()){
	                if (getSeSAmDesktopPane().askToSaveChanges("Your model has unsaved changes." +
	                		" Continue and discard changes?", 
	                        new String[]{"Load model", "Save", "Save as...", "Cancel"}))
	                
	                    loadModel(modelFile);
            	}else
                    	loadModel(modelFile);
            	
            }
        }, true);

        fileItem.setMnemonic(String.valueOf(filePos).charAt(0));
        fileItem.setToolTipText(modelFile.toString());

        getJMenuFile().insert(fileItem, getJMenuFile().getItemCount() - 2);

    }

    void addModelFileMenuEntryAndToPrefs(File modelFile) {
        if (!_modelFileList.contains(modelFile)) {
            _modelFileList.add(modelFile);
            Preferences openModelFileListPreferences = getOpenModelPreferences();
            addModelFileMenuEntry(modelFile);
            openModelFileListPreferences.put(Long.toString(System.currentTimeMillis()), modelFile.toString());
        }
    }

    protected void addPluginsMenu() {
        int i = getComponentIndex(_jMenuSimulationRuns);
        _pluginsMenu.setMnemonic('P');

        add(_pluginsMenu, i + 1);
    }

    protected JMenu createCompilerSettingsMenu() {
        JMenu compilerSettingsMenu = new JMenu("Compiler settings");

        compilerSettingsMenu.add(getCompileCheckBoxMenuItem());
        compilerSettingsMenu.add(getInlineCheckBoxMenuItem());
        compilerSettingsMenu.add(getClosuresCheckBoxMenuItem());

        return compilerSettingsMenu;
    }

    private JMenuItem createEditMenuItem(String name, KeyStroke keyStroke, Action action, Icon defaultIcon) {
        Icon icon = (Icon) action.getValue(Action.SMALL_ICON);

        if (icon == null)
            icon = defaultIcon;

        JMenuItem menuItem = new JMenuItem(name, icon);

        menuItem.setAccelerator(keyStroke);

        return menuItem;
    }

    protected JMenu createLoggerLevelMenu() {
        JMenu loggerLevelMenu = new JMenu("Logging level");

        ButtonGroup buttonGroup = new ButtonGroup();

        JCheckBoxMenuItem checkBoxMenuItem;

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.OFF);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.SEVERE);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.WARNING);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.INFO);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.CONFIG);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.FINE);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.FINER);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.FINEST);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        checkBoxMenuItem = createLoggerLevelMenuItem(Level.ALL);
        loggerLevelMenu.add(checkBoxMenuItem);
        buttonGroup.add(checkBoxMenuItem);

        return loggerLevelMenu;
    }

    protected JCheckBoxMenuItem createLoggerLevelMenuItem(final Level level) {
        JCheckBoxMenuItem checkBoxMenuItem = new JCheckBoxMenuItem(level.toString());

        AbstractBoundAction.setActionAndFont(checkBoxMenuItem, new AbstractBoundAction() {

            public void doit() {
                Logger.getLogger("").setLevel(level);
            }
        }, true);

        checkBoxMenuItem.setSelected(Logger.getLogger("").getLevel() == level);

        return checkBoxMenuItem;
    }

    protected ISeSAmDeclaration createSeSAmDeclaration() {
        return SeSAmDeclarationFactory.createSeSAmDeclaration();
    }

    protected void eventCalledExit() {
        _exitFunction.run();
    }

    protected JInternalFrame getAboutBox() {
        return SeSAmAboutBoxHelper.getInstance().getAboutBox(getDesktopPane());
    }

    private JCheckBoxMenuItem getClosuresCheckBoxMenuItem() {
        if (_closuresCheckBoxMenuItem == null) {
            _closuresCheckBoxMenuItem = new JCheckBoxMenuItem("Closures");
            AbstractBoundAction.setActionAndFont(_closuresCheckBoxMenuItem, new AbstractBoundAction() {

                public void doit() {
                    AbstractSpecificFunctionType.setSupportClosures(
                        getClosuresCheckBoxMenuItem().isSelected());
                }
            }, true);

        }
        return _closuresCheckBoxMenuItem;
    }

    private JCheckBoxMenuItem getCompileCheckBoxMenuItem() {
        if (_compileCheckBoxMenuItem == null) {
            _compileCheckBoxMenuItem = new JCheckBoxMenuItem("Optimize");
            AbstractBoundAction.setActionAndFont(_compileCheckBoxMenuItem, new AbstractBoundAction() {

                public void doit() {
                    FunctionDeclarationCompiler.setDefaultCompileOn(
                        getCompileCheckBoxMenuItem().isSelected());
                }
            }, true);
        }
        return _compileCheckBoxMenuItem;
    }

    public JDesktopPane getDesktopPane() {
        return getSeSAmDesktopPane().getDesktopPane();
    }

    public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
        return getSeSAmDesktopPane().getGUISeSAmDeclaration();
    }

    private JCheckBoxMenuItem getInlineCheckBoxMenuItem() {
        if (_inlineCheckBoxMenuItem == null) {
            _inlineCheckBoxMenuItem = new JCheckBoxMenuItem("Inline");
            AbstractBoundAction.setActionAndFont(_inlineCheckBoxMenuItem, new AbstractBoundAction() {

                public void doit() {
                    FunctionDeclarationCompiler.setDefaultInlineOn(getInlineCheckBoxMenuItem().isSelected());
                }
            }, true);
        }
        return _inlineCheckBoxMenuItem;
    }

    protected JCheckBoxMenuItem getJCompileAssertFunctionCheckBox() {
        if (_jCompileAssertCheckBox == null) {

            _jCompileAssertCheckBox =
                new JCheckBoxMenuItem(new AbstractBoundAction("Compile Debug Function") {

                public void doit() {
                    FunctionActionDebuggingDeclarationFactory.setCompileDebugFunctions(
                        getJCompileAssertFunctionCheckBox().isSelected());

                    getJRunAssertFunctionCheckBox().setEnabled(
                        FunctionActionDebuggingDeclarationFactory.isCompileDebugFunctions());
                    getJRunAssertFunctionCheckBox().setSelected(
                        FunctionActionDebuggingDeclarationFactory.isRunDebugFunctions()
                            && FunctionActionDebuggingDeclarationFactory.isCompileDebugFunctions());
                }
            });
        }
        return _jCompileAssertCheckBox;
    }

    private JMenu getJEditableMenu() {
        if (_jEditableMenu == null) {

            _jEditableMenu = new JMenu();
            _jEditableMenu.setName("JEditableMenu");
            _jEditableMenu.setMnemonic('e');
            _jEditableMenu.setText("Edit");

            _jEditableMenu.add(
                createEditMenuItem(
                    "Cut",
                    KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK),
                    TransferHandler.getCutAction(),
                    SwingMoreUtilities.getJavaSunIcon(16, "general/Cut16.gif")));
            _jEditableMenu.add(
                createEditMenuItem(
                    "Copy",
                    KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK),
                    TransferHandler.getCopyAction(),
                    SwingMoreUtilities.getJavaSunIcon(16, "general/Copy16.gif")));
            _jEditableMenu.add(
                createEditMenuItem(
                    "Paste",
                    KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK),
                    TransferHandler.getPasteAction(),
                    SwingMoreUtilities.getJavaSunIcon(16, "general/Paste16.gif")));
        }

        return _jEditableMenu;
    }

    protected JCheckBoxMenuItem getJGlobalAssertCheckBox() {
        if (_jGlobalAssertCheckBox == null) {

            _jGlobalAssertCheckBox = new JCheckBoxMenuItem(new AbstractBoundAction("Global Assert") {

                public void doit() {
                    Assert.setAssertionFlag(getJGlobalAssertCheckBox().isSelected());
                }
            });
        }
        return _jGlobalAssertCheckBox;
    }

    private JMenu getJMenuDebug() {
        if (_jMenuDebug == null) {

            _jMenuDebug = new JMenu();
            _jMenuDebug.setName("JMenuDebug");
            _jMenuDebug.setMnemonic('d');
            _jMenuDebug.setText("Debug");
            _jMenuDebug.add(getJMenuItemDebugInspectSeSAmDecl());
            _jMenuDebug.add(getJMenuItemDebugInspectJIF());
            _jMenuDebug.add(getJMenuDebugAssert());
            _jMenuDebug.add(createCompilerSettingsMenu());
            _jMenuDebug.add(createLoggerLevelMenu());

        }
        return _jMenuDebug;
    }

    private JMenu getJMenuDebugAssert() {
        if (_jMenuDebugAssert == null) {

            _jMenuDebugAssert = new JMenu();
            _jMenuDebugAssert.setMnemonic('a');
            _jMenuDebugAssert.setText("Assert");
            _jMenuDebugAssert.add(getJGlobalAssertCheckBox());
            _jMenuDebugAssert.add(getJCompileAssertFunctionCheckBox());
            _jMenuDebugAssert.add(getJRunAssertFunctionCheckBox());
        }
        return _jMenuDebugAssert;
    }

    private JMenu getJMenuFile() {
        if (_jMenuFile == null) {

            _jMenuFile = new JMenu();
            _jMenuFile.setName("JMenuFile");
            _jMenuFile.setMnemonic('f');
            _jMenuFile.setText("File");
            _jMenuFile.add(getJMenuItemFileNewModel());
            _jMenuFile.add(getJMenuItemFileOpenModel());
            _jMenuFile.add(getJMenuItemFileSaveModel());
            _jMenuFile.add(getJMenuItemFileSaveModelAs());
            _jMenuFile.add(getJMenuItemModelLibrary());
			//_jMenuFile.addSeparator();
			//_jMenuFile.add(getJMenuItemFileImport());
			//_jMenuFile.add(getJMenuItemFileExport());
            _jMenuFile.addSeparator();
            _jMenuFile.add(getLookAndFeelMenu());
            _jMenuFile.addSeparator();
            _jMenuFile.addSeparator();
            _jMenuFile.add(getJMenuItemFileExit());
            addFileListMenus();
        }
        return _jMenuFile;
    }

    private JMenu getJMenuHelp() {
        if (_jMenuHelp == null) {

            _jMenuHelp = new JMenu();
            _jMenuHelp.setName("JMenuHelp");
            _jMenuHelp.setMnemonic('h');
            _jMenuHelp.setText("Help");
            _jMenuHelp.add(getJMenuItemHelpAbout());

        }
        return _jMenuHelp;
    }

    protected JMenuItem getJMenuItemAnalysis() {
        if (_jMenuItemAnalysis == null) {

            _jMenuItemAnalysis = new JMenuItem();
            _jMenuItemAnalysis.setName("JMenuItemAnalysis");
            _jMenuItemAnalysis.setIcon(SeSAmDeclarationTreeCellRenderer.ANALYSIS_ICON);
            _jMenuItemAnalysis.setMnemonic('a');
            _jMenuItemAnalysis.setText("Analysis ...");

        }
        return _jMenuItemAnalysis;
    }

    protected JMenuItem getJMenuItemDebugInspectSeSAmDecl() {
        if (_jMenuItemDebugInspectSeSAmDecl == null) {

            _jMenuItemDebugInspectSeSAmDecl = new JMenuItem();
            _jMenuItemDebugInspectSeSAmDecl.setName("JMenuItemDebugInspectSeSAmDecl");
            _jMenuItemDebugInspectSeSAmDecl.setIcon(
                SwingMoreUtilities.getJavaSunIcon(16, "general/Zoom16.gif"));
            _jMenuItemDebugInspectSeSAmDecl.setMnemonic('i');
            _jMenuItemDebugInspectSeSAmDecl.setText("Inspect ...");

        }
        return _jMenuItemDebugInspectSeSAmDecl;
    }

    private JCheckBoxMenuItem getJMenuItemDebugInspectJIF() {
        if (_jMenuItemDebugInspectJIF == null) {

            _jMenuItemDebugInspectJIF =
                new JCheckBoxMenuItem(new AbstractBoundAction("Inspect InternalFrames") {
                public void doit() {
                    getSeSAmDesktopPane().setMakeJIFsInspectable(_jMenuItemDebugInspectJIF.isSelected());
                }
            });
        }

        return _jMenuItemDebugInspectJIF;

    }

    protected JMenuItem getJMenuItemDebugModelGraph() {
        if (_jMenuItemDebugModelGraph == null) {

            _jMenuItemDebugModelGraph = new JMenuItem(new AbstractBoundAction("Model Graph ...") {

                public void doit() {
                    showUMLGraphPanel();
                }
            });
            // _jMenuItemDebugModelGraph.setIcon(SwingMoreUtilities.getJavaSunIcon(16,
            // "general/Zoom16.gif"));
            _jMenuItemDebugModelGraph.setMnemonic('g');
        }
        return _jMenuItemDebugModelGraph;
    }

    protected JMenuItem getJMenuItemModelLibrary() {
        if (_jMenuItemModelLibrary == null) {

            _jMenuItemModelLibrary = new JMenuItem();
            _jMenuItemModelLibrary.setName("JMenuItemModelLibrary");
            _jMenuItemModelLibrary.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "navigation/Home16.gif"));
            _jMenuItemModelLibrary.setMnemonic('o');
            _jMenuItemModelLibrary.setText("Open Model Library...");

        }
        return _jMenuItemModelLibrary;
    }

    protected JMenuItem getJMenuItemExperiments() {
        if (_jMenuItemExperiments == null) {

            _jMenuItemExperiments = new JMenuItem();
            _jMenuItemExperiments.setName("JMenuItemExperiments");
            _jMenuItemExperiments.setMnemonic('e');
            _jMenuItemExperiments.setText("Experiments ...");

        }
        return _jMenuItemExperiments;
    }

    protected JMenuItem getJMenuItemFileExit() {
        if (_jMenuItemFileExit == null) {

            _jMenuItemFileExit = new JMenuItem();
            _jMenuItemFileExit.setName("JMenuItemFileExit");
            _jMenuItemFileExit.setMnemonic('x');
            _jMenuItemFileExit.setText("Exit");
            //_jMenuItemFileExit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
            // InputEvent.CTRL_MASK));
        }
        return _jMenuItemFileExit;
    }

    /**
     * Return the JMenuItem6 property value.
     * 
     * @return JMenuItem
     */
    protected JMenuItem getJMenuItemFileNewModel() {
        if (_jMenuItemFileNewModel == null) {

            _jMenuItemFileNewModel = new JMenuItem();
            _jMenuItemFileNewModel.setName("JMenuItemFileNewModel");
            _jMenuItemFileNewModel.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/New16.gif"));
            _jMenuItemFileNewModel.setMnemonic('n');
            _jMenuItemFileNewModel.setText("New Model");
            _jMenuItemFileNewModel.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));

        }
        return _jMenuItemFileNewModel;
    }

    /**
     * Return the JMenuItem2 property value.
     * 
     * @return JMenuItem
     */
    protected JMenuItem getJMenuItemFileOpenModel() {
        if (_jMenuItemFileOpenModel == null) {

            _jMenuItemFileOpenModel = new JMenuItem();
            _jMenuItemFileOpenModel.setName("JMenuItemFileOpenModel");
            _jMenuItemFileOpenModel.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/Open16.gif"));
            _jMenuItemFileOpenModel.setMnemonic('o');
            _jMenuItemFileOpenModel.setText("Open Model ...");
            _jMenuItemFileOpenModel.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));

        }
        return _jMenuItemFileOpenModel;
    }

    protected JMenuItem getJMenuItemFileSaveModel() {
        if (_jMenuItemFileSaveModel == null) {

            _jMenuItemFileSaveModel = new JMenuItem();
            _jMenuItemFileSaveModel.setName("JMenuItemFileSaveModel");
            _jMenuItemFileSaveModel.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/Save16.gif"));
            _jMenuItemFileSaveModel.setMnemonic('s');
            _jMenuItemFileSaveModel.setText("Save Model");
            _jMenuItemFileSaveModel.setAccelerator(
                KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
        }
        return _jMenuItemFileSaveModel;
    }

    protected JMenuItem getJMenuItemFileSaveModelAs() {
        if (_jMenuItemFileSaveModelAs == null) {

            _jMenuItemFileSaveModelAs = new JMenuItem();
            _jMenuItemFileSaveModelAs.setName("JMenuItemFileSaveModelAs");
            _jMenuItemFileSaveModelAs.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/SaveAs16.gif"));
            _jMenuItemFileSaveModelAs.setMnemonic('a');
            _jMenuItemFileSaveModelAs.setText("Save Model As ...");

        }
        return _jMenuItemFileSaveModelAs;
    }
    
	protected JMenuItem getJMenuItemFileImport(){
		if (_jMenuItemFileImport == null) {
			
			_jMenuItemFileImport = new JMenuItem();
			_jMenuItemFileImport.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/Import16.gif"));
			_jMenuItemFileImport.setMnemonic('i');
			_jMenuItemFileImport.setText("Import...");
			
		}
		
		return _jMenuItemFileImport;
	}
	
	protected JMenuItem getJMenuItemFileExport(){
		if (_jMenuItemFileExport == null) {
		
			_jMenuItemFileExport = new JMenuItem();
			_jMenuItemFileExport.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/Export16.gif"));
			_jMenuItemFileExport.setMnemonic('e');
			_jMenuItemFileExport.setText("Export...");
		
		}
	
		return _jMenuItemFileExport;
	}

    protected JMenuItem getJMenuItemTestSimulationRuns() {
        if (_jMenuItemTestSimulationRuns == null) {
            _jMenuItemTestSimulationRuns =
                new JMenuItem(new AbstractBoundAction("Test Simulation Runs ...") {

                public void doit() {
                    showTestRunner();
                }
            });
            _jMenuItemTestSimulationRuns.setIcon(TestRunner.POSITIVE_TEST_ICON);
            _jMenuItemTestSimulationRuns.setMnemonic('t');
        }
        return _jMenuItemTestSimulationRuns;
    }

    protected JMenuItem getJMenuItemHelpAbout() {
        if (_jMenuItemHelpAbout == null) {

            _jMenuItemHelpAbout = new JMenuItem();
            _jMenuItemHelpAbout.setName("JMenuItemHelpAbout");
            _jMenuItemHelpAbout.setIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/About16.gif"));
            _jMenuItemHelpAbout.setMnemonic('a');
            _jMenuItemHelpAbout.setText("About ...");

        }
        return _jMenuItemHelpAbout;
    }

    /**
     * Return the JMenuItem11 property value.
     * 
     * @return JMenuItem
     */
    protected JMenuItem getJMenuItemModelAgents() {
        if (_jMenuItemModelAgents == null) {

            _jMenuItemModelAgents = new JMenuItem();
            _jMenuItemModelAgents.setName("JMenuItemModelAgents");
            _jMenuItemModelAgents.setIcon(ActivityAgentClassDeclarationListPanel.AGENT_CLASS_LIST_ICON);
            _jMenuItemModelAgents.setMnemonic('a');
            _jMenuItemModelAgents.setText("Agents ...");

        }
        return _jMenuItemModelAgents;
    }

    protected JMenuItem getJMenuItemModelFeatures() {
        if (_jMenuItemModelFeatures == null) {

            _jMenuItemModelFeatures = new JMenuItem();
            _jMenuItemModelFeatures.setName("JMenuItemModelFeatures");
            _jMenuItemModelFeatures.setIcon(SeSAmDeclarationTreeCellRenderer.FEATURE_CLASS_LIST_ICON);
            _jMenuItemModelFeatures.setMnemonic('f');
            _jMenuItemModelFeatures.setText("User Features ...");

        }
        return _jMenuItemModelFeatures;
    }

    protected JMenuItem getJMenuItemModelFunctions() {
        if (_jMenuItemModelFunctions == null) {

            _jMenuItemModelFunctions = new JMenuItem();
            _jMenuItemModelFunctions.setName("JMenuItemModelFunctions");
            _jMenuItemModelFunctions.setIcon(UserFunctionDeclarationCellRendererAdapter.PUBLIC_ICON);
            _jMenuItemModelFunctions.setMnemonic('u');
            _jMenuItemModelFunctions.setText("User Functions ...");

        }
        return _jMenuItemModelFunctions;
    }

    protected JMenuItem getJMenuItemModelRessources() {
        if (_jMenuItemModelRessources == null) {

            _jMenuItemModelRessources = new JMenuItem();
            _jMenuItemModelRessources.setName("JMenuItemModelRessources");
            _jMenuItemModelRessources.setIcon(SeSAmDeclarationTreeCellRenderer.RESSOURCES_CLASS_LIST_ICON);
            _jMenuItemModelRessources.setMnemonic('r');
            _jMenuItemModelRessources.setText("Ressources ...");

        }
        return _jMenuItemModelRessources;
    }

    protected JMenuItem getJMenuItemModelSimulations() {
        if (_jMenuItemModelSimulations == null) {

            _jMenuItemModelSimulations = new JMenuItem();
            _jMenuItemModelSimulations.setName("JMenuItemModelSimulations");
            _jMenuItemModelSimulations.setMnemonic('s');
            _jMenuItemModelSimulations.setText("Simulations ...");

        }
        return _jMenuItemModelSimulations;
    }

    protected JMenuItem getJMenuItemModelSituations() {
        if (_jMenuItemModelSituations == null) {

            _jMenuItemModelSituations = new JMenuItem();
            _jMenuItemModelSituations.setName("JMenuItemModelSituations");
            _jMenuItemModelSituations.setIcon(SeSAmDeclarationTreeCellRenderer.SITUATION_ICON);
            _jMenuItemModelSituations.setMnemonic('t');
            _jMenuItemModelSituations.setText("Situations ...");

        }
        return _jMenuItemModelSituations;
    }

    protected JMenuItem getJMenuItemModelTypes() {
        if (_jMenuItemModelTypes == null) {

            _jMenuItemModelTypes = new JMenuItem();
            _jMenuItemModelTypes.setName("JMenuItemModelTypes");
            _jMenuItemModelTypes.setIcon(UserTypeDeclarationCellRendererAdapter.COMPOSED_ICON);
            _jMenuItemModelTypes.setMnemonic('t');
            _jMenuItemModelTypes.setText("User Types ...");

        }
        return _jMenuItemModelTypes;
    }

    protected JMenuItem getJMenuItemModelWorlds() {
        if (_jMenuItemModelWorlds == null) {

            _jMenuItemModelWorlds = new JMenuItem();
            _jMenuItemModelWorlds.setIcon(SeSAmDeclarationTreeCellRenderer.WORLD_CLASS_LIST_ICON);
            _jMenuItemModelWorlds.setMnemonic('w');
            _jMenuItemModelWorlds.setText("Worlds ...");

        }
        return _jMenuItemModelWorlds;
    }

    protected JMenuItem getJMenuItemSimulationRemoteRuns() {
        if (_jMenuItemSimulationRemoteRuns == null) {
            _jMenuItemSimulationRemoteRuns = new JMenuItem();
            _jMenuItemSimulationRemoteRuns.setIcon(NETWORK_ICON);
            _jMenuItemSimulationRemoteRuns.setMnemonic('r');
            _jMenuItemSimulationRemoteRuns.setText("Remote Simulation Runs ...");
        }
        return _jMenuItemSimulationRemoteRuns;
    }

    protected JMenuItem getJMenuItemSimulationRuns() {
        if (_jMenuItemSimulationRuns == null) {
            _jMenuItemSimulationRuns = new JMenuItem();
            _jMenuItemSimulationRuns.setMnemonic('s');
            _jMenuItemSimulationRuns.setIcon(ExperimentListPanelHelper.SIMULATION_RUNNING_ICON);
            _jMenuItemSimulationRuns.setText("Simulation Runs ...");
        }
        return _jMenuItemSimulationRuns;
    }

    private JMenu getJMenuModel() {
        if (_jMenuModel == null) {

            _jMenuModel = new JMenu();
            _jMenuModel.setMnemonic('m');
            _jMenuModel.setText("Model");
            _jMenuModel.add(getJMenuItemModelRessources());
            _jMenuModel.add(getJMenuItemModelAgents());
            _jMenuModel.add(getJMenuItemModelWorlds());
            _jMenuModel.addSeparator();
            _jMenuModel.add(getJMenuItemDebugModelGraph());
            //_jMenuModel.add(getJMenuItemModelLibrary());

        }
        return _jMenuModel;
    }

    /**
     * Return the JMenuModelBasics property value.
     * 
     * @return JMenu
     */
    private JMenu getJMenuModelBasics() {
        if (_jMenuModelBasics == null) {

            _jMenuModelBasics = new JMenu();
            _jMenuModelBasics.setName("JMenuModelBasics");
            _jMenuModelBasics.setMnemonic('b');
            _jMenuModelBasics.setText("Basics");
            _jMenuModelBasics.add(getJMenuItemModelFunctions());
            _jMenuModelBasics.add(getJMenuItemModelFeatures());
            _jMenuModelBasics.add(getJMenuItemModelTypes());

        }
        return _jMenuModelBasics;
    }

    JMenu getJMenuPlugins() {

        if (_pluginsMenu == null) {

            _pluginsMenu = new JMenu("Plugins");

            _pluginsMenu.getPopupMenu().addContainerListener(new ContainerAdapter() {

                public void componentAdded(ContainerEvent e) {
                    addPluginsMenu();
                }
            });
        }

        return _pluginsMenu;
    }

    /**
     * Return the JMenuSimElements property value.
     * 
     * @return JMenu
     */
    private JMenu getJMenuSimElements() {
        if (_jMenuSimElements == null) {

            _jMenuSimElements = new JMenu();
            _jMenuSimElements.setName("JMenuSimElements");
            _jMenuSimElements.setMnemonic('s');
            _jMenuSimElements.setText("Simulation");
            _jMenuSimElements.add(getJMenuItemModelSituations());
            _jMenuSimElements.add(getJMenuItemAnalysis());
            _jMenuSimElements.addSeparator();
            _jMenuSimElements.add(getJMenuItemModelSimulations());
            _jMenuSimElements.add(getJMenuItemExperiments());

        }
        return _jMenuSimElements;
    }

    private JMenu getJMenuSimulationRuns() {
        if (_jMenuSimulationRuns == null) {

            _jMenuSimulationRuns = new JMenu();
            _jMenuSimulationRuns.setName("JMenuSimulationRuns");
            _jMenuSimulationRuns.setMnemonic('r');
            _jMenuSimulationRuns.setText("Runs");
            _jMenuSimulationRuns.add(getJMenuItemSimulationRuns());
            _jMenuSimulationRuns.add(getJMenuItemTestSimulationRuns());
            _jMenuSimulationRuns.add(getJMenuItemSimulationRemoteRuns());

        }
        return _jMenuSimulationRuns;
    }

    protected JCheckBoxMenuItem getJRunAssertFunctionCheckBox() {
        if (_jRunAssertCheckBox == null) {

            _jRunAssertCheckBox = new JCheckBoxMenuItem(new AbstractBoundAction("Run Debug Function") {

                public void doit() {
                    FunctionActionDebuggingDeclarationFactory.setRunDebugFunctions(
                        getJRunAssertFunctionCheckBox().isSelected());
                }
            });
        }
        return _jRunAssertCheckBox;
    }

    private JMenu getLookAndFeelMenu() {
        LookAndFeelManager lookAndFeelManager = LookAndFeelManager.getDefaultLookAndFeelManager();

        lookAndFeelManager.registerFrame(_seSAmDesktopPane.getSeSAmMainDesktopFrame());
        lookAndFeelManager.setUpdateImmediately(true);

        JMenu lookAndFeelMenu = lookAndFeelManager.getLookAndFeelMenu();

        lookAndFeelMenu.setMnemonic('l');
        lookAndFeelMenu.setText("Look&Feel");

        return lookAndFeelMenu;
    }

    private Preferences getOpenModelPreferences() {
        return Preferences.userNodeForPackage(getClass()).node("openModelFileList");
    }

    protected final SeSAmDesktopPane getSeSAmDesktopPane() {
        return _seSAmDesktopPane;
    }

    private void initConnections() {

        getJMenuItemHelpAbout().addActionListener(_eventHandler);
        getJMenuItemModelAgents().addActionListener(_eventHandler);
        getJMenuItemModelWorlds().addActionListener(_eventHandler);
        getJMenuItemFileOpenModel().addActionListener(_eventHandler);
        getJMenuItemFileSaveModelAs().addActionListener(_eventHandler);
        getJMenuItemFileNewModel().addActionListener(_eventHandler);
		getJMenuItemFileExport().addActionListener(_eventHandler);
		getJMenuItemFileImport().addActionListener(_eventHandler);
        getJMenuItemSimulationRuns().addActionListener(_eventHandler);
        getJMenuItemSimulationRemoteRuns().addActionListener(_eventHandler);
        getJMenuItemFileExit().addActionListener(_eventHandler);
        getJMenuItemFileSaveModel().addActionListener(_eventHandler);
        getJMenuItemModelSituations().addActionListener(_eventHandler);
        getJMenuItemModelSimulations().addActionListener(_eventHandler);
        getJMenuItemModelFunctions().addActionListener(_eventHandler);
        getJMenuItemModelFeatures().addActionListener(_eventHandler);
        getJMenuItemDebugInspectSeSAmDecl().addActionListener(_eventHandler);
        getJMenuItemAnalysis().addActionListener(_eventHandler);
        getJMenuItemModelTypes().addActionListener(_eventHandler);
        getJMenuItemModelRessources().addActionListener(_eventHandler);
        getJMenuItemExperiments().addActionListener(_eventHandler);
        getJMenuItemModelLibrary().addActionListener(_eventHandler);
    }

    private void initialize() {
        setName("SeSamMainDesktopPaneJMenuBar");
        add(getJMenuFile());
        add(getJEditableMenu());
        add(getJMenuModelBasics());
        add(getJMenuModel());
        add(getJMenuSimElements());
        add(getJMenuSimulationRuns());
        // das Plugins-Men wird standardmssig nicht von anfang an angezeigt,
        // weil man nicht
        // sicher sein kann, dass ein Plugin eingetragen wird und ansonsten das
        // men leer bliebe -> cf. getJMenuPlugins()
        if (getDesktopPane() instanceof MDIDesktopPane) {
            MDIWindowMenu mdiWindowMenu = new MDIWindowMenu((MDIDesktopPane) getDesktopPane());
            mdiWindowMenu.setMnemonic('w');
            add(mdiWindowMenu);
        }

        add(getJMenuDebug());
        add(getJMenuHelp());

        initConnections();

        updateCompilerSettings();
    }

    protected void inspectSeSAm() {
        Utils.inspect(getSeSAmDesktopPane().getGUISeSAmDeclaration().getSeSAmDeclaration());
    }

    public void loadModel(File modelFile) {
        getSeSAmDesktopPane().loadModel(modelFile);
    }

    public void newModel() {
        getSeSAmDesktopPane().newModel();
    }

    public boolean openModel() {
        return getSeSAmDesktopPane().openModel();
    }

    public boolean saveAsModel() {
        return getSeSAmDesktopPane().saveAsModel();
    }

    public boolean saveModel() {
        return getSeSAmDesktopPane().saveModel();
    }
	
	public void exportModel(){
		getSeSAmDesktopPane().exportModel();	
	}
	
	public void importModel(){
		getSeSAmDesktopPane().importModel();	
	}

    public void showAboutBox() {
        getAboutBox().setVisible(true);
    }

    public void showAgentClassDeclarationList() {
        getSeSAmDesktopPane().showAgentClassDeclarationList();
    }

    public void showAnalysisDeclarationList() {
        getSeSAmDesktopPane().showAnalysisDeclarationList();
    }

    public void showExperimentDeclarationList() {
        getSeSAmDesktopPane().showExperimentDeclarationList();
    }

    public void showExperimentList() {
        getSeSAmDesktopPane().showExperimentList();
    }

    public void showUMLGraphPanel() {

        UMLLayoutGraphDeclarationGraphPanelContainer container =
            new UMLLayoutGraphDeclarationGraphPanelContainer(getSeSAmDesktopPane().getGUISeSAmDeclaration());

        //        JRootPane pane =
        getSeSAmDesktopPane().addPanel(getSeSAmDesktopPane(), "UMLLayoutGraph", container, true);

    }

    public void showRemoteSimTaskList() {
        getSeSAmDesktopPane().showRemoteSimTaskList();
    }

    public void showRessourceClassDeclarationList() {
        getSeSAmDesktopPane().showRessourceClassDeclarationList();
    }

    public void showSimulationDeclarationList() {
        getSeSAmDesktopPane().showSimulationDeclarationList();
    }

    public void showSituationDeclarationList() {
        getSeSAmDesktopPane().showSituationDeclarationList();
    }

    public void showTestRunner() {
        getSeSAmDesktopPane().showTestRunner();
    }

    public void showUserFeatureDeclarationList() {
        getSeSAmDesktopPane().showUserFeatureDeclarationList();
    }

    public void showUserFunctionDeclarationList() {
        getSeSAmDesktopPane().showUserFunctionDeclarationList();
    }

    public void showUserTypeDeclarationList() {
        getSeSAmDesktopPane().showUserTypeDeclarationList();
    }

    public void showWorldClassDeclarationList() {
        getSeSAmDesktopPane().showWorldClassDeclarationList();
    }

    public void showModelLibrary() {
        getSeSAmDesktopPane().showModelLibrary();

    }

    public void updateCompilerSettings() {
        getJCompileAssertFunctionCheckBox().setSelected(
            FunctionActionDebuggingDeclarationFactory.isCompileDebugFunctions());
        getJRunAssertFunctionCheckBox().setSelected(
            FunctionActionDebuggingDeclarationFactory.isRunDebugFunctions()
                && FunctionActionDebuggingDeclarationFactory.isCompileDebugFunctions());
        getJRunAssertFunctionCheckBox().setEnabled(
            FunctionActionDebuggingDeclarationFactory.isCompileDebugFunctions());
        getJGlobalAssertCheckBox().setSelected(Assert.ASSERTION_ON);
        getJGlobalAssertCheckBox().setEnabled(Assert.isAssertionFlagSettable());
        getCompileCheckBoxMenuItem().setSelected(FunctionDeclarationCompiler.isDefaultCompileOn());
        getInlineCheckBoxMenuItem().setSelected(FunctionDeclarationCompiler.isDefaultInlineOn());
        getClosuresCheckBoxMenuItem().setSelected(AbstractSpecificFunctionType.isSupportClosures());
    }
}