package de.uniwue.ki.gui.mas.application.importExport;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;

import javax.swing.JTree;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeCellRenderer;

/**
 * @author Reinhard Hatko
 * 
 * Created on 07.03.2004
 * 
 */
public class CheckTreeCellEditor
	extends CheckTreeCellRenderer
	implements TreeCellEditor, ActionListener {

	protected TreeCellRenderer _renderer;
	protected CheckableTreeNode _node;
	protected ArrayList _listeners;

	public CheckTreeCellEditor(JTree tree, TreeCellRenderer renderer) {
		super(tree);
		_renderer = renderer;
		_checkBox.addActionListener(this);
	}
	
	public CheckTreeCellEditor(JTree tree){
		this(tree, null);
	}
	
	public Component getTreeCellEditorComponent(
		JTree tree,Object value,
		boolean selected,boolean expanded,boolean leaf,int row) {
		
		if (value instanceof CheckableTreeNode)
			_node  = (CheckableTreeNode) value;
		
		return getTreeCellRendererComponent(
			tree, value, true, expanded,
			leaf, row, true);
	}

	public boolean stopCellEditing() {
		((DefaultTreeModel) _tree.getModel()).nodeChanged(_node);
		_node.setSelected(_checkBox.isSelected(), true, true);
		return true;
	}

	public Object getCellEditorValue() {
		return _node.getUserObject();
	}

	public boolean isCellEditable(EventObject event) {
		
		if (event instanceof MouseEvent) {
			return ((MouseEvent) event).getClickCount() == 1;
		}
		return false;
	}

	public boolean shouldSelectCell(EventObject event) {
		return true;
	}
	
	public void cancelCellEditing() {
		fireEditingCancelled();
	}
	
	public void addCellEditorListener(CellEditorListener listener) {
		if (_listeners == null)
			_listeners = new ArrayList();
		
		_listeners.add(listener);
	}

	public void removeCellEditorListener(CellEditorListener listener) {
		if (_listeners == null)
			return;
		
		_listeners.remove(listener);
	}

	protected void fireEditingStopped() {
		if (_listeners == null) return;
		
		Iterator it = _listeners.iterator();
		ChangeEvent ev = new ChangeEvent(this);
		
		while (it.hasNext()) 
			((CellEditorListener) it.next()).editingStopped(ev);		
				
	}

	protected void fireEditingCancelled() {
		if (_listeners == null) return;
		
		Iterator it = _listeners.iterator();
		ChangeEvent ev = new ChangeEvent(this);
		
		while (it.hasNext()) 
			((CellEditorListener) it.next()).editingCanceled(ev);		
		
	}
	
	public void actionPerformed(ActionEvent event) {
		fireEditingStopped();
		stopCellEditing();
	}
}
