package de.uniwue.ki.gui.mas.application.importExport;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * @author Reinhard Hatko
 * 
 * Created on 15.03.2004
 * 
 */
public class CheckableItem implements Checkable {

	private boolean _selected;
	private final Object _userObject;
	private Checkable[] _dependentObjects;
	private Checkable _father;
	private ArrayList _listenerList;
	
	public CheckableItem(Object userObject, Checkable[] dependentObjects){
		_userObject = userObject;
		setChildren(dependentObjects);
	}
	
	public CheckableItem(Object userObject) {
		this(userObject, null);
	}

	public boolean isSelected() {
		return _selected;
	}

	public void setSelected(boolean b) {
		if (_selected != b){
			_selected = b;
			fireStateChanged();
		}	
		
		_selected = b;
	}

	public void setSelected(boolean selected, boolean propagateUp, boolean propagateDown) {
		
		setSelected(selected);
			
		if (_dependentObjects != null && propagateDown)
			for (int i = 0; i < _dependentObjects.length; i++) 
				_dependentObjects[i].setSelected(selected, false, true);
		
		if (_father != null && propagateUp)
			_father.setSelected(selected, true, false);	
	
	}

	public Object getUserObject(){
		return _userObject;
	}

	public Checkable[] getChildren() {
		return _dependentObjects;
	}

	public void setChildren(Checkable[] children) {
		_dependentObjects = children;
		
		if(children != null)
			for (int i = 0; i < children.length; i++) 
				children[i].setFather(this);
	}

	public Checkable getFather() {
		return _father;
	}
	
	public void setFather(Checkable father) {
		_father = father;	
	}

	public void getUserObjectsRecursively(List list) {
		if (_selected){
			list.add(_userObject);
			
			if (_dependentObjects != null)
				for (int i = 0; i < _dependentObjects.length; i++) 
					_dependentObjects[i].getUserObjectsRecursively(list);				
		
		}
			
	}
	
	private boolean hasSelectedChildren(){
		boolean hasSelectedChildren = false;
			
		if (_dependentObjects != null)
			for (int i = 0; i < _dependentObjects.length; i++) 
				hasSelectedChildren |= _dependentObjects[i].isSelected();
		
		return hasSelectedChildren;
	}
	
	public void addChangeListener(ChangeListener listener) {
		if (_listenerList == null)
			_listenerList = new ArrayList();
		
		_listenerList.add(listener);
	}

	public void removeChangeListener(ChangeListener listener) {
		if (_listenerList == null)
			return;
		
		_listenerList.remove(listener);
	}
	
	private void fireStateChanged(){
		if (_listenerList == null)
			return;
		
		Iterator it = _listenerList.iterator();
		ChangeEvent ev = new ChangeEvent(this);
		
		while (it.hasNext()) 
			((ChangeListener)it.next()).stateChanged(ev);
		
	}
}
