package de.uniwue.ki.gui.mas.application.importExport;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;


/**
 * @author Reinhard Hatko
 * 
 * Created on 02.03.2004
 * 
 */
public class CheckableTreeNode extends DefaultMutableTreeNode implements Checkable{
	
	private boolean _selected;
	private Checkable[] _dependentObjects;
	private ArrayList _listenerList;
	
	public CheckableTreeNode(Object userObject, Checkable[] children){
		super(userObject);
		setChildren(children);
	}	
			
	public CheckableTreeNode(Object userObject){
		this(userObject, null);
	}
 	
	public boolean isSelected() {
		return _selected;
	}

	public void setSelected(boolean b) {
		if (_selected != b){
			_selected = b;
			fireStateChanged();
		}	
		_selected = b;
	}

	public void setSelected(
	  boolean selected, boolean propagateUp, boolean propagateDown) {
	  	
		if (propagateDown)
			propagateDownwards(selected);
					
		if(!selected && propagateUp && hasSelectedChildren())
			return;
		
		setSelected(selected);
				
		if (propagateUp)
			propagateUpwards(selected);
		
	}

	public void propagateDownwards(boolean state){
				
		if (_dependentObjects != null)
			for (int i = 0; i < _dependentObjects.length; i++) 
				_dependentObjects[i].setSelected(state, false, true);
			
		Enumeration enum = children();
		while(enum.hasMoreElements())
			((Checkable) enum.nextElement()).setSelected(state, false, true);				 
	}
	
	public void propagateUpwards(boolean state){
				
		CheckableTreeNode father = (CheckableTreeNode) getParent();
		
		if (father != null)
			father.setSelected(state, true, false);
		
	}
	
	public Checkable[] getChildren() {
		return _dependentObjects;
	}

	public void setChildren(Checkable[] children) {
		_dependentObjects = children;
		
		if(children != null)
			for (int i = 0; i < children.length; i++) 
				children[i].setFather(this);
			
	}
	
	public Checkable getFather() {
		return (CheckableTreeNode) getParent();
	}

	public void setFather(Checkable father) {
			
	}
	
	public void getUserObjectsRecursively(List list) {
		if (_selected){
			list.add(userObject);
			
			if (_dependentObjects != null)
				for (int i = 0; i < _dependentObjects.length; i++) 
					_dependentObjects[i].getUserObjectsRecursively(list);				
			
			Enumeration enum = children();
			
			while (enum.hasMoreElements()) {
				 ((Checkable) enum.nextElement()).getUserObjectsRecursively(list);
				
			}
			
		}
	}
	
	private boolean hasSelectedChildren(){
		boolean hasSelectedChildren = false;
		
		Enumeration enum = children();
		
		while(enum.hasMoreElements())
			hasSelectedChildren |= ((Checkable) enum.nextElement()).isSelected();				 
		
		if (_dependentObjects != null)
			for (int i = 0; i < _dependentObjects.length; i++) 
				hasSelectedChildren |= _dependentObjects[i].isSelected();
		
		return hasSelectedChildren;
	}
	
	public void addChangeListener(ChangeListener listener) {
		if (_listenerList == null)
			_listenerList = new ArrayList();
		
		_listenerList.add(listener);
	}

	public void removeChangeListener(ChangeListener listener) {
		if (_listenerList == null)
			return;
		
		_listenerList.remove(listener);
	}
	
	private void fireStateChanged(){
		if (_listenerList == null)
			return;
		
		Iterator it = _listenerList.iterator();
		ChangeEvent ev = new ChangeEvent(this);
		
		while (it.hasNext()) 
			((ChangeListener)it.next()).stateChanged(ev);
		
	}

}
