package de.uniwue.ki.gui.mas.application.importExport;

import java.io.*;
import java.util.Arrays;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.JOptionPane;

/**
 * @author Reinhard Hatko
 * 
 * Created on 18.03.2004
 * 
 */
public class ExportModelHelper {

	private ExportModelInternalFrame _frame;
	
	public ExportModelHelper(ExportModelInternalFrame frame) {
		_frame = frame;
	}
	
	protected CheckableTreeNode readDirectory(File directory, Map fileMap){
		
		File[] files = directory.listFiles(
			new FileFilter(){
				public boolean accept(File pathname) {
					return pathname.isFile();
				}
			}
		);
		
		Arrays.sort(files);
		
		if (files == null)
			return null;
		
		Checkable[] checkableFiles = new CheckableItem[files.length];
		
		for (int i = 0; i < files.length; i++)
			checkableFiles[i]  = new CheckableItem(files[i]);
			
		fileMap.put(directory, checkableFiles);
		
		CheckableTreeNode node = new CheckableTreeNode(directory, checkableFiles);
		
		File[] subDirs = directory.listFiles(
			new FileFilter(){
				public boolean accept(File pathname) {
					return pathname.isDirectory();
				}
			}
		);
		
		Arrays.sort(subDirs);
		
		for (int i = subDirs.length - 1; i > -1; i--){
			CheckableTreeNode child = readDirectory(subDirs[i], fileMap);
			if (child != null)
				node.insert(child, 0);
		}			
				
		return node;
	}
	
	
	protected void writeFileToZip(ZipOutputStream out, File file, String zipEntryName){
		byte[] buf = new byte[4096];
		ZipEntry zipEntry = new ZipEntry(zipEntryName);
		int len;
		
		try{												
			FileInputStream in = new FileInputStream(file);
																
			out.putNextEntry(zipEntry);
								
			while ((len = in.read(buf)) > 0) {
			  out.write(buf, 0, len);
			}
			in.close();
		}catch(Exception e){
			e.printStackTrace();
			JOptionPane.showInternalMessageDialog(_frame,
				"Error accessing file " + file.getPath() + ".",
				"Error", JOptionPane.ERROR_MESSAGE);
		}

	}
	
	protected String getRelativePath(File parent, File file){
		String result = null;
		
		try{
			result = file.getCanonicalPath().substring(parent.getCanonicalPath().length());	
		}
		catch(Exception e){
			e.printStackTrace();
		}
		
		return result;
	}


	protected boolean askToOverwriteFile(File file){
		
		return JOptionPane.YES_OPTION == JOptionPane.showInternalConfirmDialog(_frame,
			 "Target file: " + file.getPath() + " already exists. Overwrite?",
			  "Overwrite File", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				
	}
	
	protected boolean checkTargetFile(File targetFile){
		if (targetFile.exists())
			return askToOverwriteFile(targetFile);
		else{
			try{
				targetFile.getParentFile().mkdirs();
				return targetFile.createNewFile();
			}catch(Exception e){
				JOptionPane.showInternalMessageDialog(_frame,
				  "Error accessing file "+ targetFile.getPath() + ".",
				  "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
	}
	
	protected void removeSourceFileFromList(File file, Map map){
		Object obj = map.remove(file.getParentFile());
		
		if (obj instanceof Checkable[]){
			Checkable[] cfiles = (Checkable[]) obj;
			Checkable[] newcfiles = new Checkable[cfiles.length - 1];
			
			int i = 0;
			
			for (; i < cfiles.length; i++)
				if (((File) cfiles[i].getUserObject()).equals(file))
					break;

			System.arraycopy(cfiles, 0, newcfiles, 0, i);
						
			System.arraycopy(cfiles, i+1, newcfiles, i, newcfiles.length - i);
			
			map.put(file.getParentFile(), newcfiles);
									
		}
	}
	
}
