package de.uniwue.ki.gui.mas.application.importExport;

import java.awt.event.*;
import java.io.*;
import java.text.DateFormat;
import java.util.*;
import java.util.zip.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.jfree.ui.ExtensionFileFilter;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.*;


import de.uniwue.ki.gui.mas.application.SeSAmDesktopInternalFrame;
import de.uniwue.ki.gui.util.JBetterFileChooser;
import de.uniwue.ki.gui.util.SwingMoreUtilities;


/**
 * @author Reinhard Hatko
 *
 * Created on 27.01.2004
 */

public class ExportModelInternalFrame extends SeSAmDesktopInternalFrame {
	
	private File _directory;
	private File _sourceFile;
	private HashMap _fileMap;
	private CheckableTreeNode _root;
	private ExportModelHelper _helper;

	private JTree _tree;
	private JList _list;
	private JTextArea _commentArea;
	private JTextField _targetFileNameField;
	private JTextField _sourceFileNameField;
		
	private JButton _browseTargetButton;
	private JButton _browseSourceButton;
	private JButton _exportButton;
	private JButton _cancelButton;
	private JButton _clearCommentButton;
	
	private EventHandler _eventHandler = new EventHandler();
	
	private class EventHandler implements ActionListener{
		public void actionPerformed(ActionEvent e) {
		
			if (e.getSource() == getBrowseTargetButton()){
				eventTargetButtonPressed();
				checkIfReadyToExport();
			}else if (e.getSource() == getBrowseSourceButton()){
				eventSourceButtonPressed();
				checkIfReadyToExport();
			}else if (e.getSource() == getCancelButton()) {
				dispose();
			}else if (e.getSource() == getExportButton()) {
				export();
			}else if (e.getSource() == getClearCommentButton()) {
				_commentArea.setText("");
			}
		}
	}
	
	public ExportModelInternalFrame(File modelFile, File directory){
		super("Export Model");
		_directory = directory;
		_helper = new ExportModelHelper(this);
		_fileMap = new HashMap();
		setSourceFile(modelFile);
				
		initialize();
	}
	
	private void initialize(){
		setName("SeSAmDesktopInternalFrame");
		setIconifiable(true);
		setClosable(true);
		setMaximizable(true);
		setResizable(true);
		setSize(450, 550);
		setFrameIcon(SwingMoreUtilities.getJavaSunIcon(16, "general/Export16.gif"));
		
		ColumnSpec[] columns = new ColumnSpec[]{
			new ColumnSpec(ColumnSpec.LEFT, Sizes.DEFAULT, 	ColumnSpec.NO_GROW), 
			new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT,	ColumnSpec.DEFAULT_GROW),
			new ColumnSpec(ColumnSpec.FILL, Sizes.DLUX3, 	ColumnSpec.NO_GROW),
			new ColumnSpec(ColumnSpec.FILL, Sizes.DEFAULT,	ColumnSpec.DEFAULT_GROW),
			new ColumnSpec(ColumnSpec.FILL, Sizes.DLUX7,	ColumnSpec.NO_GROW),
			new ColumnSpec(ColumnSpec.FILL, Sizes.MINIMUM, 	ColumnSpec.NO_GROW)
		};

		RowSpec[] rows = new RowSpec[]{
			new RowSpec(RowSpec.DEFAULT, Sizes.DEFAULT, RowSpec.NO_GROW),
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY4, 	RowSpec.NO_GROW),
			new RowSpec(RowSpec.DEFAULT, Sizes.DEFAULT, RowSpec.NO_GROW),  //sep 3
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY4, 	RowSpec.NO_GROW),
			new RowSpec(RowSpec.FILL,	 Sizes.DEFAULT, RowSpec.DEFAULT_GROW),  // tree, list 5
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY4, 	RowSpec.NO_GROW),
			new RowSpec(RowSpec.DEFAULT, Sizes.DEFAULT, RowSpec.NO_GROW), // target
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY4, 	RowSpec.NO_GROW),
			new RowSpec(RowSpec.DEFAULT, Sizes.DEFAULT, RowSpec.NO_GROW), //commentsep
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY4, 	RowSpec.NO_GROW),		
			new RowSpec(RowSpec.TOP, 	 Sizes.PREFERRED, RowSpec.DEFAULT_GROW), //comment
			new RowSpec(RowSpec.DEFAULT, Sizes.DLUY11, 	RowSpec.NO_GROW),
			new RowSpec(RowSpec.DEFAULT, Sizes.DEFAULT, RowSpec.NO_GROW)			
		};
				
		
		JPanel barBuilder = new JPanel();
		barBuilder.add(getExportButton());
		barBuilder.add(getCancelButton());
		
		PanelBuilder builder = new PanelBuilder(new FormLayout(columns, rows));
		builder.setDefaultDialogBorder();
		
		CellConstraints cc = new CellConstraints();
				
		builder.addLabel("Model file: ",				cc.xy(1, 1));
		builder.add(getSourceFileNameField(), 			cc.xywh(2, 1, 3, 1));
		builder.add(getBrowseSourceButton(),			cc.xywh(6, 1, 1, 1));
		builder.addSeparator("Resources to include",	cc.xywh(1, 3, 6, 1));
		builder.add(new JScrollPane(getTree()),			cc.xywh(1, 5, 2, 1));
		builder.add(new JScrollPane(getList()), 		cc.xywh(4, 5, 3, 1));
		builder.add(getTargetFileNameField(), 			cc.xywh(2, 7, 3, 1));
		builder.add(getBrowseTargetButton(),			cc.xywh(6, 7, 1, 1));
		builder.add(new JScrollPane(getCommentArea()),	cc.xywh(1, 11, 4, 1));
		builder.add(getClearCommentButton(),			cc.xy(6, 11));
		builder.addLabel("To zip file: ",				cc.xy(1, 7));
		builder.addSeparator("Comment",					cc.xywh(1, 9, 6, 1));
		builder.addSeparator("",						cc.xywh(1, 12, 6, 1));
		builder.add(barBuilder,							cc.xywh(4, 13, 3, 1, CellConstraints.RIGHT, CellConstraints.DEFAULT ));
		
		getContentPane().add(builder.getPanel());		
						
	}

	private JButton getBrowseTargetButton() {
		if (_browseTargetButton == null){
			_browseTargetButton = new JButton("Browse...");
			_browseTargetButton.addActionListener(_eventHandler);
		}
				
		return _browseTargetButton;
	}
	
	private JButton getBrowseSourceButton() {
		if (_browseSourceButton == null){
			_browseSourceButton = new JButton("Browse...");
			_browseSourceButton.addActionListener(_eventHandler);
		}
				
		return _browseSourceButton;
	}

	private JButton getCancelButton() {
		if (_cancelButton == null){
			_cancelButton = new JButton("Cancel");
			_cancelButton.addActionListener(_eventHandler);
		}
			
		return _cancelButton;
	}
	
	private JButton getClearCommentButton(){
		if (_clearCommentButton == null) {
			_clearCommentButton = new JButton("Clear");
			_clearCommentButton.addActionListener(_eventHandler);
		}
		
		return _clearCommentButton;
	}

	private JTextArea getCommentArea() {
		if (_commentArea == null){
			_commentArea = new JTextArea();
			_commentArea.setRows(7);
						
			String sep = System.getProperty("line.separator");
			String name = System.getProperty("user.name", "");
			Date date = new Date(System.currentTimeMillis());
			
			String dateString = DateFormat.getDateInstance().format(date); 
			String timeString = DateFormat.getTimeInstance().format(date);
			
			StringBuffer comment = new StringBuffer();
			comment.append("Author: " + name + sep);
			comment.append("Date: " + dateString + sep);
			comment.append("Time: " + timeString + sep);
			comment.append("Version: " + sep);
						
			_commentArea.setText(comment.toString());
		}
		
		return _commentArea;
	}

	private JButton getExportButton() {
		if (_exportButton == null){
			_exportButton = new JButton("Export");
			_exportButton.addActionListener(_eventHandler);
			_exportButton.setEnabled(false);
		}
		
		return _exportButton;
	}
	
	private JTextField getTargetFileNameField() {
		if (_targetFileNameField == null){
			_targetFileNameField = new JTextField();
			_targetFileNameField.addKeyListener(
				new KeyAdapter(){
					public void keyReleased(KeyEvent e) {
						checkIfReadyToExport();
					}
				}
			);
		}
		
		return _targetFileNameField;
	}
	
	private JTextField getSourceFileNameField() {
		if (_sourceFileNameField == null){
			_sourceFileNameField = new JTextField();
			_sourceFileNameField.setEditable(false);
			_sourceFileNameField.addActionListener(_eventHandler);
			if (_sourceFile != null)
				_sourceFileNameField.setText(_sourceFile.toString());
			
		}
		
		return _sourceFileNameField;
	}

	private JTree getTree() {
		if (_tree == null){
			_tree = new JTree(new DefaultTreeModel(null));
			
			_tree.setShowsRootHandles(true);
			_tree.setCellRenderer(new CheckTreeCellRenderer(_tree));
			_tree.setEditable(true);
						
			CheckTreeCellEditor editor = new CheckTreeCellEditor(_tree); 
			editor.addCellEditorListener(
				new CellEditorListener(){
					public void editingCanceled(ChangeEvent e) {
						getList().repaint();
					}
					
					public void editingStopped(ChangeEvent e) {
						editingCanceled(e);
					}
				}
			);
			
			_tree.setCellEditor(editor);
			
			_tree.addTreeSelectionListener(
				new TreeSelectionListener(){
					public void valueChanged(TreeSelectionEvent e) {
						TreePath path = e.getNewLeadSelectionPath();
						
						if (path != null)
							updateList((CheckableTreeNode) path.getLastPathComponent());
						
					}
				}
			);
									
		}
		
		return _tree;
	}
	
	private JList getList(){
		if (_list == null){
			_list = new JList();
			
			_list.setCellRenderer(new FileListCellRenderer(_list));
			_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			_list.addMouseListener(
				new MouseAdapter() {
					public void mouseClicked(MouseEvent e) {
						getTree().repaint();
					}
				}
			);
		}
				
		return _list;
	}
	
	private void updateList(CheckableTreeNode selectedTreeNode){
		File directory = (File) selectedTreeNode.getUserObject();
		Checkable[] cfiles = (Checkable[]) _fileMap.get(directory);
		getList().setListData(cfiles);
	}
	
	private void export(){
		
		File targetFile = new File(_targetFileNameField.getText());
		
		if(!_helper.checkTargetFile(targetFile))
			return;
		
		ArrayList listOfFilesToExport = new ArrayList();
		_root.getUserObjectsRecursively(listOfFilesToExport);
		
		listOfFilesToExport.add(_sourceFile);
		
		try {						
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(targetFile));
						
			File parentDir = (File) _root.getUserObject();
					
			Iterator it = listOfFilesToExport.iterator();
			
			while (it.hasNext()) {
				File file = (File) it.next();
				
				if (file.isFile()){
					String zipEntryName = _helper.getRelativePath(parentDir, file);//.getParentFile()) + File.separator + file.getName();
					_helper.writeFileToZip(out, file, zipEntryName);
				}
			}
			
			out.setComment(getCommentForZip());
			out.close();
		}catch(Exception e){
			e.printStackTrace();
		}

		//dispose();
	}		
	

	private void eventSourceButtonPressed(){
		JFileChooser fileChooser = new JBetterFileChooser(_directory);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setSelectedFile(_sourceFile);
		
		fileChooser.setFileFilter(new ExtensionFileFilter("SeSAm model (*.xml)","xml"));
		fileChooser.setAcceptAllFileFilterUsed(false);
		
		if (fileChooser.showOpenDialog(ExportModelInternalFrame.this) == JFileChooser.APPROVE_OPTION){
			setSourceFile(fileChooser.getSelectedFile());	
			_sourceFileNameField.setText(fileChooser.getSelectedFile().toString());
		}
				
	}
	
	private void eventTargetButtonPressed(){
		JFileChooser fileChooser = new JBetterFileChooser(_directory);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setFileFilter(new ExtensionFileFilter("Exported SeSAm model (*.zip)","zip"));

		if (fileChooser.showSaveDialog(ExportModelInternalFrame.this) == JFileChooser.APPROVE_OPTION) 
			_targetFileNameField.setText(fileChooser.getSelectedFile().toString());
				
	}
	
	private void setSourceFile(File file){
		if(file != null){
			_sourceFile = file;
			_fileMap.clear();
			_root = _helper.readDirectory(_sourceFile.getParentFile(), _fileMap);
			_helper.removeSourceFileFromList(_sourceFile, _fileMap);
			getTree().setModel(new DefaultTreeModel(_root));
			_tree.setSelectionRow(0);
		}

	}
	
	private void checkIfReadyToExport() {
		boolean enabled = 
			!getSourceFileNameField().getText().equals("")
			&& !getTargetFileNameField().getText().equals("");
			
		getExportButton().setEnabled(enabled);
				
	}
	
	private String getCommentForZip(){
		String sep = System.getProperty("line.separator");
		StringBuffer comment = new StringBuffer();
		comment.append("$$SeSAm model file");
		comment.append(sep);
		comment.append("$$");
		comment.append(_sourceFile.getName());
		comment.append(sep);
		comment.append(getCommentArea().getText());
		
//TODO check length
		return comment.toString();
	}
}
