package de.uniwue.ki.gui.mas.application.importExport;

import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.HashMap;

import javax.swing.*;
import javax.swing.filechooser.FileSystemView;


/**
 * @author Reinhard Hatko
 * 
 * Created on 08.03.2004
 * 
 */
public class FileListCellRenderer extends DefaultListCellRenderer{

	private static FileSystemView fsv = FileSystemView.getFileSystemView();
	
	private JCheckBox _checkBox;
	private JLabel _iconLabel;
	private JLabel _nameLabel;
	private JPanel _panel;
	private JList _list;
	private HashMap _iconMap;
	
	public FileListCellRenderer(JList list) {
		super();
		
		initialize();
		_list = list;
		_list.addMouseListener(
			new MouseAdapter(){
				public void mousePressed(MouseEvent e) {
					int index = _list.locationToIndex(e.getPoint());
		  			if (index == -1)
		  				return;
		  			
		  			Object element = _list.getModel().getElementAt(index);
		  			
		  			if(element instanceof Checkable){
			  			Checkable file = (Checkable) element;
						file.setSelected(!file.isSelected(), true, true);
						_list.repaint(_list.getCellBounds(index, index));
		  			}
				}
			}
		);
	}
	
	private void initialize(){
		_iconMap = new HashMap();
		_panel = new JPanel();
		_nameLabel = new JLabel();
		_iconLabel = new JLabel();
		_checkBox = new JCheckBox();
		
		_nameLabel.setOpaque(true);		 
		_checkBox.setOpaque(false);
		_panel.setOpaque(false);
		_panel.setLayout(new BoxLayout(_panel, BoxLayout.X_AXIS));

		_panel.add(Box.createHorizontalStrut(2));
		_panel.add(_checkBox);
		_panel.add(Box.createHorizontalStrut(3));
		_panel.add(_iconLabel);
		_panel.add(Box.createHorizontalStrut(3));
		_panel.add(_nameLabel);	
	}
	
	public Component getListCellRendererComponent(
	  JList list, Object value, int index,
	  boolean isSelected, boolean cellHasFocus) {
		
		File file = (File) ((Checkable) value).getUserObject();
		String name = file.getName(); 	
		_nameLabel.setFont(list.getFont());
		_nameLabel.setText(name);
		_iconLabel.setIcon(getIcon(file));
		
		if (isSelected) {
			_nameLabel.setBackground(list.getSelectionBackground());
			_nameLabel.setForeground(list.getSelectionForeground());
		}else {
			_nameLabel.setBackground(list.getBackground());
			_nameLabel.setForeground(list.getForeground());
		}
		
		_checkBox.setSelected(((CheckableItem)value).isSelected());

		_panel.setEnabled(list.isEnabled());
		
		return _panel;
	}
	
	private Icon getIcon(File file){
				
		String name = file.getName();
		int suffixStart = name.lastIndexOf(".");
		
		if(suffixStart == -1)
			return fsv.getSystemIcon(file);
		
		String suffix = name.substring(suffixStart).toLowerCase();

		Icon icon = (Icon) _iconMap.get(suffix);
		
		if(icon == null){
			icon = fsv.getSystemIcon(file);
			_iconMap.put(suffix, icon);	
		}
		
		return icon;
	}

}
