package de.uniwue.ki.gui.mas.application.remote;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.net.*;
import java.util.prefs.Preferences;

import javax.swing.*;

import snoozesoft.plaf.windows.systray.SysTrayException;
import snoozesoft.plaf.windows.systray.SysTrayMenu;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.application.remote.*;
import de.uniwue.ki.mas.application.remote.server.RemoteSimulationControl;
import de.uniwue.ki.mas.application.remote.server.RemoteSimulationControlConnection;
import de.uniwue.ki.mas.xml.castor.v01.remote.XMLRemoteInfo;
import de.uniwue.ki.mas.xml.castor.v01.remote.XMLRemoteTimeSchedule;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: 21.10.2002, 13:39:17
 * @author oechslein
 */
public class ClientControlPanel extends JPanel {

	private static final int CLIENT_PORT = IRemoteSimulationConstants.DEFAULT_CONN_SERVER_PORT + 1;
	private static final String SIMULATION_ALLOWED_STRING = "   Simulation Allowed   ";
	private static final String SIMULATION_DISALLOWED_STRING = "Simulation Disallowed";

	private static final String _frameTitle = "ClientControl";

	private InetAddress _destination;

	private static boolean _systrayPossible = true;

	protected static ServerSocket _serverSocket;
	private final int _serverID = (int) (Math.random() * Integer.MAX_VALUE);

	private final XMLRemoteTimeSchedule _xmlRemoteTimeSchedule = new XMLRemoteTimeSchedule();

	protected RemoteSimulationControlConnection _remoteSimulationControlConnection;
	private boolean _openAtStartup = false;

	private JTextField _statusTextField;
	private JTextField _infoTextField;
	private JToggleButton _allowSimulationButton;
	private JCheckBox _allowAtExitCheckBox;
	private JCheckBox _disallowAtStartupCheckBox;
	private JCheckBox _automaticScheduleCheckBox;

	private static final int INSET_TOP_BUTTONS = 30;

	public ClientControlPanel() {
		super();

		System.out.println(_serverID);

		initialize();
	}

	private void initialize() {
		initializeComponents();
	}

	private void initializeComponents() {
		setLayout(new GridBagLayout());

		initializeInfoAndStatusBoxes();

		initialzeAllowButtons();

		initializeTimeButtons();

		initializeControlButtons();

	}

	private void initializeControlButtons() {
		GridBagConstraints constraints;

		constraints = new GridBagConstraints();
		constraints.gridx = 2;
		constraints.gridy = 4;
		constraints.gridwidth = 1;
		constraints.weightx = 0.5;
		constraints.insets.top = INSET_TOP_BUTTONS;
		constraints.anchor = GridBagConstraints.EAST;
		add(new JButton(new AbstractBoundAction("Refresh") {
			public void doit() {
				refresh();
			}
		}), constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 3;
		constraints.gridy = 4;
		constraints.gridwidth = 1;
		constraints.weightx = 0.5;
		constraints.insets.top = INSET_TOP_BUTTONS;
		constraints.anchor = GridBagConstraints.EAST;
		add(new JButton(new AbstractBoundAction("Quit") {
			public void doit() {
				quit(0);
			}
		}), constraints);
	}

	private void initializeTimeButtons() {
		GridBagConstraints constraints;

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 3;
		constraints.weightx = 1;
		constraints.gridwidth = 2;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets.top = 15;
		add(new JButton(new AbstractBoundAction("Set times") {
			public void doit() {
				eventSetTimeButtonPressed();
			}
		}), constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 2;
		constraints.gridy = 3;
		constraints.weightx = 0;
		constraints.gridwidth = 2;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.insets.top = 15;
		add(getAutomaticScheduleCheckBox(), constraints);

	}

	private void initialzeAllowButtons() {
		GridBagConstraints constraints;

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.weightx = 1;
		constraints.gridwidth = 2;
		constraints.anchor = GridBagConstraints.WEST;
		add(getAllowSimulationButton(), constraints);

		Box box = Box.createVerticalBox();

		box.add(getDisallowAtStartupCheckBox());
		box.add(getUnpauseAtExitCheckBox());

		constraints = new GridBagConstraints();
		constraints.gridx = 2;
		constraints.gridy = 2;
		constraints.gridwidth = 2;
		constraints.anchor = GridBagConstraints.WEST;
		add(box, constraints);
	}

	private void initializeInfoAndStatusBoxes() {
		GridBagConstraints constraints = new GridBagConstraints();

		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.gridwidth = 4;
		constraints.weightx = 1;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.HORIZONTAL;

		int widthOfLongestLabel = new JLabel("Status: ").getPreferredSize().width;

		add(createHorBox("Status: ", getStatusTextField(), widthOfLongestLabel), constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 4;
		constraints.weightx = 1;
		constraints.insets.top = 3;
		constraints.insets.bottom = 5;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		add(createHorBox("Info: ", getInfoTextField(), widthOfLongestLabel), constraints);
	}

	protected void eventSetTimeButtonPressed() {
		XMLRemoteTimeSchedule remoteTimeSchedule = new XMLRemoteTimeSchedule();
		TimeSettingsPanel.copyInto(_xmlRemoteTimeSchedule, remoteTimeSchedule);

		if (TimeSettingsPanel.showTimeSettingsDialog(this, remoteTimeSchedule)) {
			updateTimeSchedule(remoteTimeSchedule, true);
		}
	}

	protected void updateTimeSchedule(XMLRemoteTimeSchedule remoteTimeSchedule, boolean withSend) {
		TimeSettingsPanel.copyInto(remoteTimeSchedule, _xmlRemoteTimeSchedule);
		getAutomaticScheduleCheckBox().setSelected(remoteTimeSchedule.getAutomaticSchedule());
		if (withSend)
			sendTimeScheduleToClient();
	}

	protected void sendTimeScheduleToClient() {
		try {
			_xmlRemoteTimeSchedule.setAutomaticSchedule(getAutomaticScheduleCheckBox().isSelected());

			_remoteSimulationControlConnection.setXMLTimeSchedule(_xmlRemoteTimeSchedule);
			refresh();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private Box createHorBox(String name, JTextField textField, int width) {
		Box box = Box.createHorizontalBox();

		JLabel label = new JLabel(name);
		label.setPreferredSize(new Dimension(width, label.getPreferredSize().height));
		box.add(label);
		box.add(textField);
		return box;
	}

	private JTextField getStatusTextField() {
		if (_statusTextField == null) {
			_statusTextField = new JTextField("Starting Application ...");
			_statusTextField.setEditable(false);
		}

		return _statusTextField;
	}

	private JTextField getInfoTextField() {
		if (_infoTextField == null) {
			_infoTextField = new JTextField("");
			_infoTextField.setEditable(false);

			_infoTextField.setPreferredSize(new Dimension(100, 20)); // wildes Rumgehacke
			_infoTextField.setHorizontalAlignment(SwingConstants.LEFT);
		}

		return _infoTextField;
	}

	private JToggleButton getAllowSimulationButton() {
		if (_allowSimulationButton == null) {
			_allowSimulationButton = new JToggleButton(new AbstractBoundAction(SIMULATION_ALLOWED_STRING) {
				public void doit() {
					eventAllowSimChanged();
				}
			});
		}

		return _allowSimulationButton;
	}

	private JCheckBox getUnpauseAtExitCheckBox() {
		if (_allowAtExitCheckBox == null) {
			_allowAtExitCheckBox = new JCheckBox("Allow at exit");
		}

		return _allowAtExitCheckBox;
	}

	protected JCheckBox getDisallowAtStartupCheckBox() {
		if (_disallowAtStartupCheckBox == null) {
			_disallowAtStartupCheckBox = new JCheckBox("Disallow at startup");
		}

		return _disallowAtStartupCheckBox;
	}

	private JCheckBox getAutomaticScheduleCheckBox() {
		if (_automaticScheduleCheckBox == null) {
			_automaticScheduleCheckBox = new JCheckBox("Use automatic schedule");
			_automaticScheduleCheckBox.addActionListener(new AbstractBoundAction() {
				public void doit() {
					sendTimeScheduleToClient();
				}
			});
		}

		return _automaticScheduleCheckBox;
	}

	private void setDestinationInetAddress(InetAddress destination) {
		_destination = destination;
	}

	private void initializeClientSocket(InetAddress destination) throws IOException {
		setDestinationInetAddress(destination);

		Thread thread = new Thread(new Runnable() {
			public void run() {
				try {
					Socket clientSocket;

					clientSocket = getClientSocket();

					_remoteSimulationControlConnection = new RemoteSimulationControlConnection(clientSocket);

					XMLRemoteTimeSchedule xmlTimeSchedule = getXMLTimeSchedule();

					if (xmlTimeSchedule != null) {
						updateTimeSchedule(xmlTimeSchedule, false);
					}

					XMLRemoteInfo remoteInfo = getXMLRemoteInfo();
					if (remoteInfo != null) {
						if ((remoteInfo.getState()
							== IRemoteProtocolConstants.DISALLOW_SIMULATION_STATE.getId())
							|| getDisallowAtStartupCheckBox().isSelected()) {

							//							getAllowSimulationButton().setSelected(true);
							eventAllowSimChanged();
						}
					}

					refresh();

					synchronized (ClientControlPanel.this) {
						ClientControlPanel.this.notify();
					}

				} catch (SocketTimeoutException e) {
					JOptionPane.showMessageDialog(
						ClientControlPanel.this,
						"Service not running on local host.");
					quit(1);

				} catch (IOException e) {
					e.printStackTrace();
					quit(1);
				}
			}

			private Socket getClientSocket() throws IOException, BindException, SocketException {
				initializeServerSocket();

				_serverSocket.setSoTimeout(1 * 1000);

				Socket clientSocket = _serverSocket.accept();

				synchronized (ClientControlPanel.this) {
					ClientControlPanel.this.notifyAll();
				}

				return clientSocket;
			}

		});

		thread.start();

		synchronized (this) {
			try {
				wait(100);

				RemoteSimulationControl.sendDatagram(destination, CLIENT_PORT, _serverID);

				while (thread.isAlive()) {
					wait(100);
				}

			} catch (InterruptedException e) {
			}
		}
	}

	protected static void initializeServerSocket() throws IOException, BindException {
		if (_serverSocket == null)
			_serverSocket = new ServerSocket(CLIENT_PORT);
	}

	protected void eventSysTray(ActionEvent e, JFrame frame) {
		if ("Quit".equals(e.getActionCommand())) {
			quit(0);

		} else if ("Open".equals(e.getActionCommand())) {
			if ((_remoteSimulationControlConnection != null)
				&& (_remoteSimulationControlConnection.isConnected())) {

				frame.show();
				frame.toFront();

			} else {
				_openAtStartup = true;
			}
		}
	}

	protected synchronized void eventAllowSimChanged() {
		eventAllowSimChanged(!getAllowSimulationButton().isSelected());
	}

	private void eventAllowSimChanged(boolean allowed) {
		if (!allowed) {
			getAllowSimulationButton().setText(SIMULATION_DISALLOWED_STRING);
			changeSysTrayIcon(false);

		} else {
			getAllowSimulationButton().setText(SIMULATION_ALLOWED_STRING);
			changeSysTrayIcon(true);
		}

		updateClientSettings(allowed);
	}

	private void changeSysTrayIcon(boolean enabled) {
		if (_systrayPossible) {
			try {
				SysTrayMenu.changeIcon(enabled ? 0 : 1);

			} catch (SysTrayException e) {
				e.printStackTrace();
			}
		}
	}

	private void updateClientSettings(boolean allowed) {
		if ((_remoteSimulationControlConnection != null)
			&& (_remoteSimulationControlConnection.isConnected())) {

			try {
				XMLRemoteInfo remoteInfo = getXMLRemoteInfo();

				if (remoteInfo != null) {

					boolean disallowed =
						remoteInfo.getState() == IRemoteProtocolConstants.DISALLOW_SIMULATION_STATE.getId();

					if (allowed == disallowed) {
						_remoteSimulationControlConnection.setAllowSimulation(allowed);
					}

					refresh();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	protected void refresh() {
		try {
			XMLRemoteInfo remoteInfo = getXMLRemoteInfo();

			refresh(remoteInfo);

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	protected XMLRemoteInfo getXMLRemoteInfo() throws IOException {
		if ((_remoteSimulationControlConnection != null)
			&& (_remoteSimulationControlConnection.isConnected())) {

			return _remoteSimulationControlConnection.getXMLRemoteInfo();

		} else {
			return null;
		}
	}

	protected XMLRemoteTimeSchedule getXMLTimeSchedule() throws IOException {
		if ((_remoteSimulationControlConnection != null)
			&& (_remoteSimulationControlConnection.isConnected())) {

			return _remoteSimulationControlConnection.getXMLTimeSchedule();

		} else {
			return null;
		}
	}

	protected void refresh(XMLRemoteInfo remoteInfo) throws IOException {
		if (remoteInfo == null) {
			getStatusTextField().setText("");

		} else {

			setTextFieldText(getStatusTextField(), RemoteClientState.getStateName(remoteInfo.getState()));

			StringBuffer buff = new StringBuffer();
			if (remoteInfo.getSimulationName() != null) {
				buff.append(remoteInfo.getSimulationName());
				if (remoteInfo.getExperimentName() != null) {
					buff.append(" (");
					buff.append(remoteInfo.getExperimentName());
					buff.append(")");
				}
				if (remoteInfo.getTicks() != -1) {
					buff.append("; ticks: ");
					buff.append(remoteInfo.getTicks());
				}
			}

			setTextFieldText(getInfoTextField(), buff.toString());
		}
	}

	private void setTextFieldText(JTextField field, String labelText) {
		field.setText(labelText);
		field.setToolTipText(labelText);
	}

	protected synchronized void quit(int status) {
		if (_remoteSimulationControlConnection != null) {
			closeConnection();

			savePrefs((JFrame) SwingMoreUtilities.getRootPaneContainer(this));
			if (getUnpauseAtExitCheckBox().isSelected()) {
				eventAllowSimChanged(true);
				if (_systrayPossible) {
					// just show icon change
					try {
						wait(500);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
		}

		if (_systrayPossible) {
			SysTrayMenu.destroy();
		}
		System.exit(status);
	}

	protected void closeConnection() {
		try {
			_remoteSimulationControlConnection.closeConnection();

			_serverSocket.close();

			while (!_serverSocket.isClosed()) {

				try {
					Thread.sleep(500);
				} catch (InterruptedException timeout) {
				}
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] arguments) {

		Utils.setEnglishLocale();

		try {
			initializeServerSocket();

		} catch (BindException e2) {
			switchToOrRemoveOldClient();

		} catch (IOException e) {

			e.printStackTrace();
			System.exit(1);
		}

		JFrame frame = new JFrame(_frameTitle);

		InetAddress destination = null;
		try {
			if (arguments.length == 1) {
				destination = InetAddress.getByName(arguments[0]);
			} else {
				destination = InetAddress.getLocalHost();
			}
		} catch (UnknownHostException e) {
		}

		//TODO systray funktioniert nicht immer, daher ausgeschaltet
		_systrayPossible = false;
		initializeClientControl(frame, destination);
		//		try {
		//			System.loadLibrary("stmenu");
		//			_systrayPossible = true;
		//			initializeClientControlWithSysTray(frame, destination);
		//
		//		} catch (UnsatisfiedLinkError e) {
		//			System.out.println("java.library.path = " + System.getProperty("java.library.path"));
		//			e.printStackTrace();
		//
		//			_systrayPossible = false;
		//			initializeClientControl(frame, destination);
		//		}
	}

	private static void switchToOrRemoveOldClient() {

		try {
			Object[] options = { "Show old client", "Replace old Client", "Cancel" };

			int selectedIndex =
				JOptionPane.showOptionDialog(
					null,
					"Client Control already running.",
					"Question",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);

			if (selectedIndex == 0) {
				connectToControlClientServer(false);
				System.exit(0);

			} else if (selectedIndex == 1) {
				connectToControlClientServer(true);

			} else {
				System.exit(0);
			}

		} catch (IOException ee) {
			ee.printStackTrace();
			System.exit(1);
		}

	}

	private static final int CONTROL_CLIENT_PORT = IRemoteSimulationConstants.DEFAULT_CONN_SERVER_PORT + 7;
	private static final byte SHUTDOWN_COMMAND = 0;
	private static final byte SHOWPANEL_COMMAND = 1;

	public static void connectToControlClientServer(boolean shutDown) throws IOException {
		Socket clientSocket = new Socket(InetAddress.getLocalHost(), CONTROL_CLIENT_PORT);

		clientSocket.setSoTimeout(10 * 1000);

		clientSocket.getOutputStream().write(shutDown ? SHUTDOWN_COMMAND : SHOWPANEL_COMMAND);
		clientSocket.getOutputStream().flush();
		clientSocket.getInputStream().read();
		clientSocket.close();
	}

	private static void initializeControlClientServer(final ClientControlPanel clientControlPanel) {
		Thread thread = new Thread(new Runnable() {
			public void run() {
				ServerSocket sock;
				try {
					sock = new ServerSocket(CONTROL_CLIENT_PORT);

				} catch (IOException e) {
					e.printStackTrace();
					System.exit(1);
					return;
				}

				while (true) {
					try {

						Socket clientSocket = sock.accept();
						clientSocket.setSoTimeout(10 * 1000);

						int c = clientSocket.getInputStream().read();

						if (c == -1) {
							// communication problem
						} else if ((byte) c == SHUTDOWN_COMMAND) {
							clientControlPanel.closeConnection();
							clientSocket.getOutputStream().write((byte) 0);
							clientSocket.close();

							clientControlPanel.quit(1);

						} else if ((byte) c == SHOWPANEL_COMMAND) {
							Window win = SwingUtilities.getWindowAncestor(clientControlPanel);
							win.show();
							win.toFront();
						}

						clientSocket.getOutputStream().write((byte) 0);
						clientSocket.close();

					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}, "ShutdownServer");

		thread.setDaemon(true);
		thread.start();
	}

	public static void initializeClientControlWithSysTray(
		final JFrame frame,
		final InetAddress destination) {
		new Thread(new Runnable() {
			public void run() {
				initializeClientControl(frame, destination);
			}
		}).start();

		// call sysTray must be the last due to documentation (and does not return)
		SysTrayMenu.init("SeSAm Client Control", new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (frame.getContentPane() instanceof ClientControlPanel)
					 ((ClientControlPanel) frame.getContentPane()).eventSysTray(e, frame);
			}
		},
			"Open",
			new String[] { "Quit", SysTrayMenu.SEPARATOR, "Open" },
			new String[] { "Quit", null, "Open" },
			new String[] {
				getAbsoluteIconPath("SysTrayIcon.ico"),
				getAbsoluteIconPath("SysTrayIconDisabled.ico")});
	}

	protected static void initializeClientControl(final JFrame frame, InetAddress destination) {
		frame.setResizable(false);

		final ClientControlPanel clientControlPanel = new ClientControlPanel();
		clientControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		frame.setContentPane(clientControlPanel);

		frame.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

		frame.addWindowListener(new WindowAdapter() {
			public void windowIconified(WindowEvent e) {
				frame.hide();
			}

		});

		if (ClientControlPanel.getPrefs().getInt("panelX", -1) == -1)
			SwingMoreUtilities.centerWindow(frame);

		frame.pack();

		SwingMoreUtilities.setEnableAllComponentsRecursive(frame, false);

		frame.setLocation(
			getPrefs().getInt("panelX", frame.getX()),
			getPrefs().getInt("panelY", frame.getY()));
		frame.setExtendedState(getPrefs().getInt("extendedState", frame.getExtendedState()));

		if (!_systrayPossible) {
			frame.show();

			clientControlPanel.getStatusTextField().setText("Connecting to client ...");
			frame.repaint();
		}

		try {
			clientControlPanel.initializeClientSocket(destination);
			if (clientControlPanel._openAtStartup)
				frame.show();

		} catch (IOException e) {
			e.printStackTrace();
			clientControlPanel.quit(1);
		}

		clientControlPanel.setFrameTitle(frame);

		frame.pack();

		SwingMoreUtilities.setEnableAllComponentsRecursive(frame, true);

		initializeControlClientServer(clientControlPanel);

		clientControlPanel.sendTimeScheduleToClient();
	}

	private void setFrameTitle(JFrame frame) {
		try {
			frame.setTitle(_frameTitle + " for " //connected to
			+_destination.getHostAddress()
				+ " ("
				+ (_destination.equals(InetAddress.getLocalHost()) ? "" : "non-")
				+ "local)");
		} catch (UnknownHostException e) {
		}
	}

	private static String getAbsoluteIconPath(String iconName) {
		File file =
			new File(ClientControlPanel.class.getPackage().getName().replace('.', '/') + "/" + iconName);
		if (file.exists())
			return file.getAbsolutePath();
		else {
			// maybe class file is in a jar
			return new File("./icons/" + iconName).getAbsolutePath();
		}
	}

	private static Preferences getPrefs() {
		return Preferences.userNodeForPackage(ClientControlPanel.class);
	}

	private void savePrefs(JFrame frame) {
		Preferences prefs = getPrefs();

		prefs.putBoolean("allowAtExit", getUnpauseAtExitCheckBox().isSelected());

		prefs.putBoolean("disallowAtStartup", getDisallowAtStartupCheckBox().isSelected());

		prefs.putBoolean("automaticSchedule", getAutomaticScheduleCheckBox().isSelected());

		prefs.putInt("panelX", frame.getX());
		prefs.putInt("panelY", frame.getY());
		prefs.putInt("extendedState", frame.getExtendedState());
	}
}