/**
 * Created on 04.11.2002
 *
 * This class is responsible for //insert purpose here//
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.application.remote;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.*;
import java.util.Calendar;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.text.DateFormatter;
import javax.swing.text.DefaultFormatterFactory;

import de.uniwue.ki.gui.util.KeyAdapterReactOnEnter;
import de.uniwue.ki.mas.xml.castor.v01.remote.XMLRemoteTimeSchedule;
import de.uniwue.ki.mas.xml.castor.v01.remote.XMLRemoteTimeSlice;

public class TimeSettingsPanel extends JTabbedPane {

	private static class StartEndTextFieldPair {
		private JFormattedTextField _endTextField;
		private JFormattedTextField _startTextField;

		public void setEndTextField(JFormattedTextField endTextField) {
			_endTextField = endTextField;
		}

		public JFormattedTextField getEndTextField() {
			return _endTextField;
		}

		public void setStartTextField(JFormattedTextField startTextField) {
			_startTextField = startTextField;
		}

		public JFormattedTextField getStartTextField() {
			return _startTextField;
		}
	}

	private static final class DowStartEndTextFieldPair extends StartEndTextFieldPair {
		private int _dow;

		public void setDow(int dow) {
			_dow = dow;
		}

		public int getDow() {
			return _dow;
		}
	}

	private static final DateFormat DATE_FORMAT = new SimpleDateFormat("kk:mm");
	private static final DefaultFormatterFactory DATE_FORMATTER_FACTORY =
		new DefaultFormatterFactory(new DateFormatter(DATE_FORMAT));

	private StartEndTextFieldPair _weekdaysTextFieldPair;
	private StartEndTextFieldPair _weekendsTextFieldPair;

	private DowStartEndTextFieldPair[] _longTextFieldPair = new DowStartEndTextFieldPair[7];

	private XMLRemoteTimeSchedule _settings;

	public TimeSettingsPanel(XMLRemoteTimeSchedule xmlRemoteTimeSchedule) {
		super();

		layoutShortSettingsPane();
		layoutDowSettingsPane();

		setXMLRemoteTimeSchedule(xmlRemoteTimeSchedule);
	}

	private void setXMLRemoteTimeSchedule(XMLRemoteTimeSchedule xmlRemoteTimeSchedule) {
		_settings = xmlRemoteTimeSchedule;
		loadTimesSettings();
	}

	private void layoutShortSettingsPane() {
		JPanel shortSettingsPanel = new JPanel();
		shortSettingsPanel.setLayout(new GridBagLayout());

		addNamedTimeFields(shortSettingsPanel, 0, "Weekdays", getWeekdaysTextFieldPair());
		addNamedTimeFields(shortSettingsPanel, 1, "Weekends", getWeekendsTextFieldPair());

		add("Short", shortSettingsPanel);
	}

	private void layoutDowSettingsPane() {
		JPanel dowSettingsPanel = new JPanel();

		dowSettingsPanel.setLayout(new GridBagLayout());

		addNamedTimeFields(dowSettingsPanel, 0, "Monday", getStartEndTextFieldPairFor(Calendar.MONDAY));
		addNamedTimeFields(dowSettingsPanel, 1, "Tuesday", getStartEndTextFieldPairFor(Calendar.TUESDAY));
		addNamedTimeFields(dowSettingsPanel, 2, "Wednesday", getStartEndTextFieldPairFor(Calendar.WEDNESDAY));
		addNamedTimeFields(dowSettingsPanel, 3, "Thursday", getStartEndTextFieldPairFor(Calendar.THURSDAY));
		addNamedTimeFields(dowSettingsPanel, 4, "Friday", getStartEndTextFieldPairFor(Calendar.FRIDAY));
		addNamedTimeFields(dowSettingsPanel, 5, "Saturday", getStartEndTextFieldPairFor(Calendar.SATURDAY));
		addNamedTimeFields(dowSettingsPanel, 6, "Sunday", getStartEndTextFieldPairFor(Calendar.SUNDAY));

		add("Long", dowSettingsPanel);
	}

	private void addNamedTimeFields(
		JPanel panel,
		int row,
		String dayOfWeek,
		StartEndTextFieldPair startEndTextFieldPair) {

		addToRow(panel, row, 0, new JLabel(dayOfWeek + "  "));
		addToRow(panel, row, 1, startEndTextFieldPair.getStartTextField());
		addToRow(panel, row, 2, new JLabel(" to "));
		addToRow(panel, row, 3, startEndTextFieldPair.getEndTextField());
	}

	private void addToRow(JPanel panel, int row, int col, Component component) {
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.top = 5;
		constraints.gridx = col;
		constraints.gridy = row;
		constraints.anchor = GridBagConstraints.WEST;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		panel.add(component, constraints);
	}

	protected void updateShortSettingsTimeSettings() {
		updateSlice(getWeekdaysTextFieldPair(), getSliceFor(Calendar.MONDAY));
		updateSlice(getWeekdaysTextFieldPair(), getSliceFor(Calendar.TUESDAY));
		updateSlice(getWeekdaysTextFieldPair(), getSliceFor(Calendar.WEDNESDAY));
		updateSlice(getWeekdaysTextFieldPair(), getSliceFor(Calendar.THURSDAY));
		updateSlice(getWeekdaysTextFieldPair(), getSliceFor(Calendar.FRIDAY));
		updateSlice(getWeekendsTextFieldPair(), getSliceFor(Calendar.SATURDAY));
		updateSlice(getWeekendsTextFieldPair(), getSliceFor(Calendar.SUNDAY));

		loadTimesSettings();
	}

	protected void updateSlice(StartEndTextFieldPair startEndTextFieldPair, XMLRemoteTimeSlice slice) {
		slice.setStart(transformTimeToInt(startEndTextFieldPair.getStartTextField().getText()));
		slice.setEnd(transformTimeToInt(startEndTextFieldPair.getEndTextField().getText()));
	}

	protected void updateEnabledState() {
		boolean sameEntries = weekdaysAndEndHasSameEntries();

		setEnabledAt(0, sameEntries);

		if (!sameEntries) {
			setSelectedIndex(1);
		}
	}

	private boolean weekdaysAndEndHasSameEntries() {
		int weekdaysStart = getWeekdaysRepresenterSlice().getStart();
		int weekdaysEnd = getWeekdaysRepresenterSlice().getEnd();

		boolean weekdaysSameEntries =
			sliceHasSameStartAndEnd(Calendar.MONDAY, weekdaysStart, weekdaysEnd)
				&& sliceHasSameStartAndEnd(Calendar.TUESDAY, weekdaysStart, weekdaysEnd)
				&& sliceHasSameStartAndEnd(Calendar.WEDNESDAY, weekdaysStart, weekdaysEnd)
				&& sliceHasSameStartAndEnd(Calendar.THURSDAY, weekdaysStart, weekdaysEnd)
				&& sliceHasSameStartAndEnd(Calendar.FRIDAY, weekdaysStart, weekdaysEnd);

		int weekendsStart = getWeekendsRepresenterSlice().getStart();
		int weekendsEnd = getWeekendsRepresenterSlice().getEnd();

		boolean weekendsSameEntries =
			sliceHasSameStartAndEnd(Calendar.SUNDAY, weekendsStart, weekendsEnd)
				&& sliceHasSameStartAndEnd(Calendar.SATURDAY, weekendsStart, weekendsEnd);

		boolean sameEntries = weekdaysSameEntries && weekendsSameEntries;
		return sameEntries;
	}

	private boolean sliceHasSameStartAndEnd(int dow, int start, int end) {
		return (getSliceFor(dow).getStart() == start) && (getSliceFor(dow).getEnd() == end);
	}

	private void loadTimesSettings() {
		loadSlice(getWeekdaysRepresenterSlice(), getWeekdaysTextFieldPair());
		loadSlice(getWeekendsRepresenterSlice(), getWeekendsTextFieldPair());

		for (int dow = Calendar.SUNDAY; dow <= Calendar.SATURDAY; dow++) {
			loadSlice(getSliceFor(dow), getStartEndTextFieldPairFor(dow));
		}

		updateEnabledState();
	}

	private static void loadSlice(XMLRemoteTimeSlice slice, StartEndTextFieldPair startEndTextFieldPair) {
		startEndTextFieldPair.getStartTextField().setText(transformIntToTime(slice.getStart()));
		startEndTextFieldPair.getEndTextField().setText(transformIntToTime(slice.getEnd()));
	}

	private XMLRemoteTimeSlice getWeekendsRepresenterSlice() {
		return getSliceFor(Calendar.SUNDAY);
	}

	private XMLRemoteTimeSlice getWeekdaysRepresenterSlice() {
		return getSliceFor(Calendar.MONDAY);
	}

	protected XMLRemoteTimeSlice getSliceFor(int dow) {
		for (int i = 0; i < _settings.getSliceCount(); i++) {
			if (_settings.getSlice(i).getDow() == dow) {
				return _settings.getSlice(i);
			}
		}
		XMLRemoteTimeSlice xmlRemoteTimeSlice = new XMLRemoteTimeSlice();
		xmlRemoteTimeSlice.setDow(dow);
		xmlRemoteTimeSlice.setStart(0);
		xmlRemoteTimeSlice.setEnd(0);

		_settings.addSlice(xmlRemoteTimeSlice);

		return xmlRemoteTimeSlice;
	}

	private StartEndTextFieldPair createStartEndTextFieldPair() {
		StartEndTextFieldPair pair = new StartEndTextFieldPair();
		pair.setStartTextField(createShortSettingsFormattedTextField());
		pair.setEndTextField(createShortSettingsFormattedTextField());
		return pair;
	}

	private JFormattedTextField createShortSettingsFormattedTextField() {
		JFormattedTextField textField = createTextField();

		textField.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				updateShortSettingsTimeSettings();
			}
		});

		return textField;
	}

	private JFormattedTextField createTextField() {
		JFormattedTextField textField = new JFormattedTextField(DATE_FORMATTER_FACTORY);

		textField.addKeyListener(new KeyAdapterReactOnEnter(textField));
		textField.setEditable(true);
		textField.setHorizontalAlignment(SwingConstants.RIGHT);
		textField.setColumns(5);

		return textField;
	}

	private JFormattedTextField createDowSettingsFormattedTextField(
		final StartEndTextFieldPair startEndTextFieldPair,
		final int dow) {

		JFormattedTextField textField = createTextField();

		textField.addKeyListener(new KeyAdapterReactOnEnter(textField));

		textField.addPropertyChangeListener("value", new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				updateSlice(startEndTextFieldPair, getSliceFor(dow));
				updateEnabledState();
			}
		});

		textField.setEditable(true);
		textField.setHorizontalAlignment(SwingConstants.RIGHT);

		return textField;
	}

	private static int transformTimeToInt(String time) {
		String pre = time.substring(0, time.indexOf(':'));
		String post = time.substring(time.indexOf(':') + 1, time.length());

		return Integer.parseInt(pre + post);
	}

	private static final NumberFormat TIME_FORMAT = new DecimalFormat("00");

	private static String transformIntToTime(int time) {
		return TIME_FORMAT.format((time / 100)) + ":" + TIME_FORMAT.format((time % 100));
	}

	public static boolean showTimeSettingsDialog(Component owner, XMLRemoteTimeSchedule remoteTimeSchedule) {
		TimeSettingsPanel timeSettingPanel = new TimeSettingsPanel(remoteTimeSchedule);

		int selectedOption =
			JOptionPane.showConfirmDialog(
				owner,
				timeSettingPanel,
				"Input dissallow times",
				JOptionPane.OK_CANCEL_OPTION,
				JOptionPane.PLAIN_MESSAGE);

		return selectedOption == JOptionPane.OK_OPTION;
	}

	public static void copyInto(
		XMLRemoteTimeSchedule srcTimeSchedule,
		XMLRemoteTimeSchedule dstTimeSchedule) {
		dstTimeSchedule.setAutomaticSchedule(srcTimeSchedule.getAutomaticSchedule());
		dstTimeSchedule.clearSlice();

		Iterator it = srcTimeSchedule.getSliceAsReference().iterator();
		while (it.hasNext()) {
			XMLRemoteTimeSlice slice = (XMLRemoteTimeSlice) it.next();
			XMLRemoteTimeSlice newSlice = new XMLRemoteTimeSlice();
			newSlice.setDow(slice.getDow());
			newSlice.setStart(slice.getStart());
			newSlice.setEnd(slice.getEnd());
			dstTimeSchedule.addSlice(newSlice);
		}
	}
	private StartEndTextFieldPair getWeekendsTextFieldPair() {
		if (_weekendsTextFieldPair == null) {
			_weekendsTextFieldPair = createStartEndTextFieldPair();
		}

		return _weekendsTextFieldPair;
	}

	private StartEndTextFieldPair getWeekdaysTextFieldPair() {
		if (_weekdaysTextFieldPair == null) {
			_weekdaysTextFieldPair = createStartEndTextFieldPair();
		}

		return _weekdaysTextFieldPair;
	}

	private StartEndTextFieldPair getStartEndTextFieldPairFor(int dow) {
		DowStartEndTextFieldPair startEndTextFieldPair = _longTextFieldPair[dow - 1];

		if (startEndTextFieldPair == null) {
			startEndTextFieldPair = new DowStartEndTextFieldPair();
			startEndTextFieldPair.setDow(dow);

			startEndTextFieldPair.setStartTextField(
				createDowSettingsFormattedTextField(startEndTextFieldPair, dow));
			startEndTextFieldPair.setEndTextField(
				createDowSettingsFormattedTextField(startEndTextFieldPair, dow));

			_longTextFieldPair[dow - 1] = startEndTextFieldPair;
		}

		return startEndTextFieldPair;
	}

}
