package de.uniwue.ki.gui.mas.application.remote.server;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.JEditMutableListPanel;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.bound.BoundJIntegerTextField;
import de.uniwue.ki.gui.util.dnd.ObjectDataFlavor;
import de.uniwue.ki.mas.application.remote.server.*;
import de.uniwue.ki.mas.xml.writer.XMLWriteException;

/**
 * Creation date: 10.09.2002, 18:11:38
 * @author oechslein
 */
public class RemoteSimulationAutomaticControlPanel extends JPanelWithGUISeSAmDeclaration {
	private static final String PREFS_AUTOM_NAME = "autom";
	private static final String PREFS_MODEL_FILE_NAME_NAME = "modelFile";
	private static final String PREFS_PERIOD_NAME = "period";
	private static final String PREFS_SCHEDULE_NAME = "schedule";
	private static final String PREFS_SIMTASK_FILE_NAME_NAME = "simTaskFile";
	private static final String PREFS_SIMTASKS_NAME = "simTasks";
	private static final String PREFS_SIMULATION_NAME = "simulationName";

	protected JCheckBox _automaticUpdateCheckBox;
	protected BoundJIntegerTextField _delayTextField;

	private String _oldModelFileName = "";
	private String _oldSimulationName = "";

	protected final RemoteSimulationAutomaticControl _remoteSimulationAutomaticControl;
	private RemoteSimulationAutomaticControl.ScheduleListener _scheduleListener =
		new RemoteSimulationAutomaticControl.ScheduleListener() {

		public void scheduleFinished() {
			setStatusBarText("");
		}

		public void scheduleStarted() {
			setStatusBarText("Scheduling ...");
		}

		public void simulationStarted(String name) {
			setStatusBarText(getStatusBarText() + " " + name + " started ");
		}
	};
	private JEditMutableListPanel _simTasksListPanel;
	private JButton _updateButton;

	public RemoteSimulationAutomaticControlPanel(RemoteSimulationAutomaticControl remoteSimulationAutomaticControl) {
		super();

		_remoteSimulationAutomaticControl = remoteSimulationAutomaticControl;
		initialize();
		remoteSimulationAutomaticControl.addScheduleListener(_scheduleListener);
	}
	protected Iterator createNewSimTasks() {

		return createRemoteModelAsStringSimulationTasks();
		//		if (getGUISeSAmDeclaration() != null) {
		//
		//			int result = JOptionPane.showConfirmDialog(this, "Local Simulation?");
		//
		//			if (result == JOptionPane.YES_OPTION) {
		//				return createRemoteModelAsStringSimulationTask();
		//
		//			} else if (result == JOptionPane.NO_OPTION) {
		//				return createRemoteModelAsFileSimulationTask();
		//
		//			} else {
		//				return null;
		//			}
		//
		//		} else {
		//			return createRemoteModelAsFileSimulationTask();
		//		}
	}

	//	private IRemoteModelSimulationTask createRemoteModelAsFileSimulationTask() {
	//
	//		JFileChooser fileChooser = new JFileChooser();
	//		fileChooser.setSelectedFile(new File(getOldModelFileName()));
	//
	//		fileChooser.addChoosableFileFilter(new ExtensionsFileFilter("SeSAm model (*.xml)", "xml"));
	//
	//		if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
	//			final File modelFile = fileChooser.getSelectedFile();
	//
	//			if (modelFile.exists()) {
	//				final String modelFileString = modelFile.toString();
	//
	//				setOldModelFileName(modelFileString);
	//
	//				String simulationName =
	//					JOptionPane.showInputDialog(this, "Simulation Name", getOldSimulationName());
	//
	//				if (simulationName != null) {
	//					setOldSimulationName(simulationName);
	//
	//					return new RemoteModelAsFileSimulationTask(modelFile, simulationName);
	//				}
	//			}
	//		}
	//
	//		return null;
	//	}

	private Iterator createRemoteModelAsStringSimulationTasks() {
		List tasks = new LinkedList();
		Iterator simNames =
			RemoteSimulationControlPanelPopupListener.choiceSimulationNames(getGUISeSAmDeclaration(), this);

		while (simNames.hasNext()) {
			String simName = (String) simNames.next();
			try {
				tasks.add(
					new RemoteModelAsGZipStringSimulationTask(
						getGUISeSAmDeclaration().getSeSAmDeclaration(),
						simName));

			} catch (IOException e) {
				showException(e);
			} catch (XMLWriteException e) {
				showException(e);
			}
		}

		return tasks.iterator();
	}

	public void dispose() {
		_remoteSimulationAutomaticControl.removeScheduleListener(_scheduleListener);

		try {
			synchronized (RemoteSimulationAutomaticControlPanel.class) {
				writeOldModelFileAndSimName();
				writeUpdateSettings();
				writeSimTasks();
			}
		} catch (BackingStoreException e) {
			e.printStackTrace();
		}

		super.dispose();
	}

	private JCheckBox getAutomaticUpdateCheckBox() {
		if (_automaticUpdateCheckBox == null) {
			_automaticUpdateCheckBox = new JCheckBox("Aut. Schedule");
			_automaticUpdateCheckBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					_remoteSimulationAutomaticControl.setAutomaticSchedule(
						_automaticUpdateCheckBox.isSelected());
				}
			});
			_automaticUpdateCheckBox.setSelected(_remoteSimulationAutomaticControl.isAutomaticSchedule());
		}

		return _automaticUpdateCheckBox;
	}

	private BoundJIntegerTextField getDelayTextField() {
		if (_delayTextField == null) {
			_delayTextField = new BoundJIntegerTextField();
			_delayTextField.setColumns(6);
			_delayTextField.addPropertyChangeListener("integer", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					_remoteSimulationAutomaticControl.setSchedulePeriodInSecs(
						_delayTextField.getIntegerValue());
				}
			});
			_delayTextField.setIntegerValue(_remoteSimulationAutomaticControl.getSchedulePeriodInSecs());
		}

		return _delayTextField;
	}

	public String getOldModelFileName() {
		return _oldModelFileName;
	}

	public String getOldSimulationName() {
		return _oldSimulationName;
	}

	private RemoteSimulationControlPanel getRemoteSimulationControlPanel() {
		Component parent = getParent();
		while (parent != null) {
			if (parent instanceof RemoteSimulationControlPanel)
				return (RemoteSimulationControlPanel) parent;

			parent = parent.getParent();
		}
		return null;
	}

	private JEditMutableListPanel getSimTasksListPanel() {
		if (_simTasksListPanel == null) {
			_simTasksListPanel = new JEditMutableListPanel() {
				protected void addNewItem(ActionEvent evt) {
					Iterator it = createNewSimTasks();
					while (it.hasNext()) {
						getMutableListModel().add(it.next());
					}
				}
			};
			_simTasksListPanel.setEditable(true);
			_simTasksListPanel.setArrowsAvailible(true);
			_simTasksListPanel.setCellRenderer(
				new NamedCellRendererAdapter(_simTasksListPanel.getCellRenderer()));
			_simTasksListPanel.setMutableListModel(_remoteSimulationAutomaticControl.getSimulationTaskList());
			_simTasksListPanel.enableDnDSupport(
				ObjectDataFlavor.getDataFlavor(IRemoteModelSimulationTask.class));

		}

		return _simTasksListPanel;
	}

	protected String getStatusBarText() {
		return getRemoteSimulationControlPanel().getStatusBarText();
	}

	private JButton getUpdateButton() {
		if (_updateButton == null) {
			_updateButton = new JButton("Schedule");
			_updateButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					_remoteSimulationAutomaticControl.startSimOnAllFreeConnections();
					Thread.yield();
				}
			});
		}

		return _updateButton;
	}

	private void initialize() {
		try {
			synchronized (RemoteSimulationAutomaticControlPanel.class) {
				readOldModelFileAndSimName();
				readUpdateSettings();
				readSimTasks();
			}
		} catch (BackingStoreException e) {
			e.printStackTrace();
		}

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		add(getSimTasksListPanel());

		JPanel autoSchedulePanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		autoSchedulePanel.add(getAutomaticUpdateCheckBox());
		autoSchedulePanel.add(new JLabel("Delay (in sec): "));
		autoSchedulePanel.add(getDelayTextField());
		autoSchedulePanel.add(new JLabel("  "));

		add(autoSchedulePanel);

		JPanel schedulePanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 0));

		schedulePanel.add(getUpdateButton());

		add(schedulePanel);
		add(Box.createVerticalStrut(5));
	}

	private void readOldModelFileAndSimName() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class);

		setOldModelFileName(prefs.get(PREFS_MODEL_FILE_NAME_NAME, getOldModelFileName()));
		setOldSimulationName(prefs.get(PREFS_SIMULATION_NAME, getOldSimulationName()));
	}

	private void readSimTasks() throws BackingStoreException {
		Preferences simTasksPrefs =
			Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class).node(PREFS_SIMTASKS_NAME);

		try {
			String simTaskFileString = simTasksPrefs.get(PREFS_SIMTASK_FILE_NAME_NAME, null);
			if (simTaskFileString == null)
				return;

			File simTaskFile = new File(simTaskFileString);
			if (simTaskFile.exists()) {

				ObjectInputStream ois = new ObjectInputStream(new FileInputStream(simTaskFile));

				List list = null;
				try {
					list = (List) ois.readObject();

				} catch (ClassNotFoundException e) {
					e.printStackTrace();

				} catch (ClassCastException e) {
					e.printStackTrace();
				}

				if (list != null)
					_remoteSimulationAutomaticControl.getSimulationTaskList().addAll(list);

				ois.close();

				simTaskFile.deleteOnExit();
				simTaskFile.delete();
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void readUpdateSettings() throws BackingStoreException {
		Preferences updatePrefs =
			Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class).node(PREFS_SCHEDULE_NAME);

		_remoteSimulationAutomaticControl.setSchedulePeriodInSecs(
			updatePrefs.getInt(
				PREFS_PERIOD_NAME,
				_remoteSimulationAutomaticControl.getSchedulePeriodInSecs()));
		getDelayTextField().setIntegerValue(_remoteSimulationAutomaticControl.getSchedulePeriodInSecs());

		_remoteSimulationAutomaticControl.setAutomaticSchedule(
			updatePrefs.getBoolean(
				PREFS_AUTOM_NAME,
				_remoteSimulationAutomaticControl.isAutomaticSchedule()));
		getAutomaticUpdateCheckBox().setSelected(_remoteSimulationAutomaticControl.isAutomaticSchedule());
	}

	public void setOldModelFileName(String oldModelFileName) {
		_oldModelFileName = oldModelFileName;
	}

	public void setOldSimulationName(String oldSimulationName) {
		_oldSimulationName = oldSimulationName;
	}

	protected void setStatusBarText(String text) {
		getRemoteSimulationControlPanel().setStatusBarText(text);
	}

	private void showException(Exception e) {
		GUISeSAmDeclaration.showException(getGUISeSAmDeclaration(), "ERROR", e, this);
	}

	private void writeOldModelFileAndSimName() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class);

		prefs.put(PREFS_MODEL_FILE_NAME_NAME, getOldModelFileName());
		prefs.put(PREFS_SIMULATION_NAME, getOldSimulationName());
	}

	private synchronized void writeSimTasks() throws BackingStoreException {
		if (!_remoteSimulationAutomaticControl.getSimulationTaskList().isEmpty()) {
			Preferences simTasksPrefs =
				Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class).node(
					PREFS_SIMTASKS_NAME);

			try {
				File simTaskFile = File.createTempFile("SimTaksFile", null);

				simTasksPrefs.put(PREFS_SIMTASK_FILE_NAME_NAME, simTaskFile.toString());

				ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(simTaskFile));

				oos.writeObject(new ArrayList(_remoteSimulationAutomaticControl.getSimulationTaskList()));

				oos.close();

			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void writeUpdateSettings() throws BackingStoreException {
		Preferences updatePrefs =
			Preferences.userNodeForPackage(RemoteSimulationAutomaticControl.class).node(PREFS_SCHEDULE_NAME);
		updatePrefs.putInt(PREFS_PERIOD_NAME, _remoteSimulationAutomaticControl.getSchedulePeriodInSecs());
		updatePrefs.putBoolean(PREFS_AUTOM_NAME, _remoteSimulationAutomaticControl.isAutomaticSchedule());
	}
}
