package de.uniwue.ki.gui.mas.application.remote.server;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.table.DefaultTableCellRenderer;

import org.jfree.ui.SortableTable;

import de.uniwue.ki.gui.mas.model.*;
import de.uniwue.ki.mas.application.remote.server.RemoteSimulationControl;
import de.uniwue.ki.util.table.DisableColumnsTableModel;

/**
 * Creation date: 10.09.2002, 15:25:38
 * @author oechslein
 */
public class RemoteSimulationControlPanel extends JPanelWithGUISeSAmDeclaration {
	private JPanel _buttonPanel;
	private JComponent _mainPanel;
	private SortableTable _remoteConnectionTablePanel;

	private SimulationControlConnectionsTableModel _remoteControlConnectionTableModel;
	private RemoteSimulationAutomaticControlPanel _remoteSimulationAutomaticControlPanel;

	private final RemoteSimulationControl _remoteSimulationControl;
	private final RemoteSimulationControlPanelPopupListener _remoteSimulationControlPanelHelper;
	private JScrollPane _scrollTablePanel;
	private JLabel _statusbarJLabel;

	public RemoteSimulationControlPanel() {
		super();

		_remoteSimulationControl = new RemoteSimulationControl();
		_remoteSimulationControlPanelHelper =
			new RemoteSimulationControlPanelPopupListener(this, _remoteSimulationControl);

		initialize();
	}

	public void dispose() {
		getRemoteSimulationAutomaticControlPanel().dispose();
		try {
			writeSettings();
		} catch (BackingStoreException e) {
			GUISeSAmDeclaration.showException(getGUISeSAmDeclaration(), "ERROR", e, this);
		}

		_remoteSimulationControl.remove();

		super.dispose();
	}

	private JPanel getButtonPanel() {
		if (_buttonPanel == null) {
			_buttonPanel = new JPanel();
			_buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

			JButton button = new JButton(RemoteSimulationControlPanelPopupListener.REFRESH_ALL_STRING);
			button.addActionListener(_remoteSimulationControlPanelHelper);
			_buttonPanel.add(button);

			button = new JButton(RemoteSimulationControlPanelPopupListener.UPDATE_CLIENT_LIST_STRING);
			button.addActionListener(_remoteSimulationControlPanelHelper);
			_buttonPanel.add(button);
		}

		return _buttonPanel;
	}

	private JPanel getLeftPanel() {
		JPanel leftPanel = new JPanel(new BorderLayout());
		leftPanel.add(getScrollTablePanel(), BorderLayout.CENTER);
		leftPanel.add(getButtonPanel(), BorderLayout.SOUTH);
		return leftPanel;
	}

	private JComponent getMainPanel() {
		if (_mainPanel == null) {
			JTabbedPane tabbedPane = new JTabbedPane(SwingConstants.LEFT);
			_mainPanel = tabbedPane;

			tabbedPane.add(getLeftPanel(), "Connections");
			tabbedPane.add(getRightPanel(), "Sim Tasks");
		}

		return _mainPanel;
	}

	private SortableTable getRemoteConnectionTablePanel() {
		if (_remoteConnectionTablePanel == null) {
			DisableColumnsTableModel disableColumnsTableModel =
				new DisableColumnsTableModel(getRemoteControlConnectionTableModel());
			disableColumnsTableModel.setDisabled(3, true);
			disableColumnsTableModel.setDisabled(7, true);
			disableColumnsTableModel.setDisabled(9, true);
			_remoteConnectionTablePanel = new SortableTable(disableColumnsTableModel);
			//_remoteConnectionTablePanel.setMinimumSize(new Dimension(100, 50));
			_remoteConnectionTablePanel.setMinimumSize(new Dimension(65535, 65535));
			//_remoteConnectionTablePanel.setPreferredSize(new Dimension(100, 50));
			_remoteConnectionTablePanel.setDefaultRenderer(String.class, new DefaultTableCellRenderer() {
				public Component getTableCellRendererComponent(
					JTable table,
					Object value,
					boolean isSelected,
					boolean hasFocus,
					int row,
					int column) {
					Component c =
						super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
					if (c instanceof JComponent) {
						((JComponent)c).setToolTipText(value + "");
					}

					return c;
				}

			});
			_remoteConnectionTablePanel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			_remoteConnectionTablePanel.setAutoResizeMode(JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS);
			_remoteConnectionTablePanel.addMouseListener(new MouseAdapter() {

				public void mouseClicked(MouseEvent e) {
					showPopupIfPopupTrigger(e);
				}

				public void mousePressed(MouseEvent e) {
					showPopupIfPopupTrigger(e);
				}

				public void mouseReleased(MouseEvent e) {
					showPopupIfPopupTrigger(e);
				}
				private void showPopupIfPopupTrigger(MouseEvent evt) {
					if (SwingUtilities.isRightMouseButton(evt) && evt.isPopupTrigger()) {
						try {
							showPopup(evt.getPoint());
						} catch (IOException e) {
							GUISeSAmDeclaration.showException(
								getGUISeSAmDeclaration(),
								"ERROR",
								e,
								RemoteSimulationControlPanel.this);
						}
					}
				}
			});
		}

		return _remoteConnectionTablePanel;
	}

	private SimulationControlConnectionsTableModel getRemoteControlConnectionTableModel() {
		if (_remoteControlConnectionTableModel == null) {
			_remoteControlConnectionTableModel =
				new SimulationControlConnectionsTableModel(_remoteSimulationControl.getConnectionListModel());
		}

		return _remoteControlConnectionTableModel;
	}

	private RemoteSimulationAutomaticControlPanel getRemoteSimulationAutomaticControlPanel() {
		if (_remoteSimulationAutomaticControlPanel == null) {
			_remoteSimulationAutomaticControlPanel =
				new RemoteSimulationAutomaticControlPanel(
					_remoteSimulationControl.getRemoteSimulationAutomaticControl());
		}
		return _remoteSimulationAutomaticControlPanel;
	}

	private JPanel getRightPanel() {
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.X_AXIS));
		rightPanel.add(getRemoteSimulationAutomaticControlPanel());
		rightPanel.add(Box.createHorizontalGlue());
		rightPanel.add(Box.createHorizontalGlue());
		return rightPanel;
	}

	private JScrollPane getScrollTablePanel() {
		if (_scrollTablePanel == null) {
			_scrollTablePanel = new JScrollPane();
			_scrollTablePanel.setViewportView(getRemoteConnectionTablePanel());
		}

		return _scrollTablePanel;
	}

	private Iterator getSelectedConnections() {
		int[] rows = getRemoteConnectionTablePanel().getSelectedRows();
		java.util.List connectionList = new ArrayList(rows.length);
		for (int i = 0; i < rows.length; i++) {
			connectionList.add(_remoteSimulationControl.getConnectionListModel().getElementAt(rows[i]));
		}

		return connectionList.iterator();
	}

	private JLabel getStatusbarJLabel() {
		if (_statusbarJLabel == null) {
			_statusbarJLabel = new JLabel();
			_statusbarJLabel.setText(" ");
			_statusbarJLabel.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
		}

		return _statusbarJLabel;
	}

	public String getStatusBarText() {
		return getStatusbarJLabel().getText();
	}

	private void initialize() {
		initializeContent();

		_remoteSimulationControl.startServer();
		try {
			readSettings();
		} catch (BackingStoreException e) {
			GUISeSAmDeclaration.showException(getGUISeSAmDeclaration(), "ERROR", e, this);
		}

		registerKeyStrokes();
	}

	private void initializeContent() {
		setLayout(new GridBagLayout());

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;

		add(getMainPanel(), constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.fill = GridBagConstraints.HORIZONTAL;

		add(getStatusbarJLabel(), constraints);
	}

	private void readSettings() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationControlPanel.class);
		Preferences framePrefs = prefs.node("frame");
		int x = framePrefs.getInt("x", getX());
		int y = framePrefs.getInt("y", getY());
		int width = framePrefs.getInt("width", getWidth());
		int height = framePrefs.getInt("height", getHeight());
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(
			(int)Math.max(0, Math.min(screenSize.getWidth(), width)),
			(int)Math.max(0, Math.min(screenSize.getHeight(), height)));
		setLocation(
			(int)Math.max(0, Math.min(screenSize.getWidth() - getWidth(), x)),
			(int)Math.max(0, Math.min(screenSize.getHeight() - getHeight(), y)));
		//		int dividerLocation = prefs.getInt("dividerLocation", -1);
		//		if (dividerLocation != -1)
		//			getMainPanel().setDividerLocation(dividerLocation);
		StringTokenizer stringTokenizer = new StringTokenizer(prefs.get("table", ""));
		int i = 0;
		while (stringTokenizer.hasMoreTokens()) {
			String widthString = stringTokenizer.nextToken();

			if (i >= getRemoteConnectionTablePanel().getColumnCount())
				break;

			int convertedIndex = getRemoteConnectionTablePanel().convertColumnIndexToModel(i);

			if (convertedIndex >= getRemoteConnectionTablePanel().getColumnCount())
				break;

			try {
				getRemoteConnectionTablePanel().getColumnModel().getColumn(convertedIndex).setPreferredWidth(
					Integer.parseInt(widthString));
			} catch (NumberFormatException e) {
				Exception e1 = e;
				GUISeSAmDeclaration.showException(getGUISeSAmDeclaration(), "ERROR", e1, this);
			}
			i++;
		}

		_remoteSimulationControlPanelHelper.readSettings();
	}

	private void registerKeyStrokes() {
		KeyStroke f5KeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0);
		_remoteConnectionTablePanel.registerKeyboardAction(
			_remoteSimulationControlPanelHelper,
			RemoteSimulationControlPanelPopupListener.REFRESH_ALL_STRING,
			f5KeyStroke,
			JComponent.WHEN_IN_FOCUSED_WINDOW);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		getRemoteSimulationAutomaticControlPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);

		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void setStatusBarText(String text) {
		if (text.length() == 0)
			getStatusbarJLabel().setText(" ");
		else
			getStatusbarJLabel().setText(text);
	}

	protected void showPopup(Point pt) throws IOException {
		if (getRemoteConnectionTablePanel().getSelectedRowCount() == 0) {
			int row = getRemoteConnectionTablePanel().rowAtPoint(pt);
			if (row == -1)
				return;
			else
				getRemoteConnectionTablePanel().setRowSelectionInterval(row, row);
		}

		_remoteSimulationControlPanelHelper.showPopup(
			SwingUtilities.convertPoint(getRemoteConnectionTablePanel(), pt, this),
			getSelectedConnections());
	}

	public void updateClientList() {
		_remoteSimulationControlPanelHelper.updateClientList();
	}

	private void writeSettings() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationControlPanel.class);
		Preferences framePrefs = prefs.node("frame");

		//SwingMoreUtilities

		framePrefs.putInt("x", getX());
		framePrefs.putInt("y", getY());
		framePrefs.putInt("width", getWidth());
		framePrefs.putInt("height", getHeight());
		//		prefs.putInt("dividerLocation", getMainPanel().getDividerLocation());

		StringBuffer buff = new StringBuffer();
		for (int i = 0; i < getRemoteConnectionTablePanel().getColumnCount(); i++) {
			buff.append(
				getRemoteConnectionTablePanel()
					.getColumnModel()
					.getColumn(getRemoteConnectionTablePanel().convertColumnIndexToModel(i))
					.getWidth());
			buff.append('\n');
		}

		prefs.put("table", buff.toString());

		_remoteSimulationControlPanelHelper.writeSettings();
	}
}