package de.uniwue.ki.gui.mas.application.remote.server;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.util.*;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import java.util.zip.ZipException;

import javax.swing.*;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.application.remote.TimeSettingsPanel;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentOrSimulationChooserPanel;
import de.uniwue.ki.gui.util.ExtensionsFileFilter;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.application.remote.*;
import de.uniwue.ki.mas.application.remote.server.*;
import de.uniwue.ki.mas.xml.castor.v01.remote.*;
import de.uniwue.ki.mas.xml.writer.XMLWriteException;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.file.FileUtilities;
import de.uniwue.ki.util.file.ZipUtilities;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: 03.12.2002, 20:03:26
 * @author oechslein
 */
final class RemoteSimulationControlPanelPopupListener implements ActionListener {

	private interface IDoWithFunction extends IProgressFunction {
		boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
			throws IOException;
	}

	private interface IProgressFunction {
	}

	private interface IResultFunction extends IProgressFunction {
		Object calcResults(StateCacheSimulationControlConnection remoteSimulationControlConnection)
			throws IOException;
	}

	private final class ProgressRunner implements Runnable {
		private final JDialog _dialog;
		private final String _failString;
		private final JProgressBar _jProgressBar;
		private final IProgressFunction _progressFunction;
		private final String _successString;
		private final String _title;
		private final boolean _withRefresh;

		private ProgressRunner(
			JDialog dialog,
			String title,
			JProgressBar jProgressBar,
			IProgressFunction progressFunction,
			boolean withRefresh,
			String failString,
			String successString) {
			super();

			_dialog = dialog;
			_title = title;
			_jProgressBar = jProgressBar;
			_progressFunction = progressFunction;
			_withRefresh = withRefresh;
			_failString = failString;
			_successString = successString;
		}
		private void appendString(
			StringBuffer buff,
			StateCacheSimulationControlConnection remoteSimulationControlConnection,
			String str) {

			buff.append(str);
			buff.append(" by client ");
			try {
				buff.append(getConnectionName(remoteSimulationControlConnection));

			} catch (IOException e2) {
				buff.append(
					remoteSimulationControlConnection
						.getSimulationControlConnection()
						.getInetAddress()
						.getHostName());
			}
			buff.append('\n');
		}

		private void doWithProgressbarIntern() {

			StringBuffer buff = new StringBuffer();
			Iterator connections = getSavedSelectedConnections();
			int index = 0;
			while (connections.hasNext()) {
				StateCacheSimulationControlConnection remoteSimulationControlConnection =
					(StateCacheSimulationControlConnection)connections.next();

				try {
					_jProgressBar.setString(
						getConnectionName(remoteSimulationControlConnection)
							+ " ("
							+ index
							+ " of "
							+ _jProgressBar.getMaximum()
							+ ")");

					if (_progressFunction instanceof IDoWithFunction) {
						IDoWithFunction doWithFunction = (IDoWithFunction)_progressFunction;
						boolean result = doWithFunction.doWith(remoteSimulationControlConnection);

						if (!result) {
							appendString(buff, remoteSimulationControlConnection, _failString);

						} else if (_successString != null) {
							appendString(buff, remoteSimulationControlConnection, _successString);
						}
					} else {
						IResultFunction resultFunction = (IResultFunction)_progressFunction;
						Object result = resultFunction.calcResults(remoteSimulationControlConnection);

						if (result == null) {
							appendString(buff, remoteSimulationControlConnection, _failString);

						} else if (_successString != null) {
							appendString(buff, remoteSimulationControlConnection, _successString + result);
						}
					}

					if (_withRefresh)
						remoteSimulationControlConnection.refresh();

				} catch (IOException e) {
					Exception e1 = e;
					GUISeSAmDeclaration.showException(
						_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
						"ERROR",
						e1,
						_remoteSimulationControlPanel);

					appendString(buff, remoteSimulationControlConnection, _failString);
				}

				_jProgressBar.setValue(++index);
			}

			if (buff.length() > 0)
				showJOptionPaneFromOtherThread(buff.toString());

		}

		public void run() {
			String oldText = _remoteSimulationControlPanel.getStatusBarText();
			_remoteSimulationControlPanel.setStatusBarText(_title + " ...");
			try {
				doWithProgressbarIntern();

			} finally {
				_dialog.setVisible(false);
				_dialog.dispose();
				_remoteSimulationControlPanel.setStatusBarText(oldText);
			}
		}
	}
	private static final String ALLOW_SIM_STRING = "Allow Simulation";
	private static final String DISALLOW_SIM_STRING = "Disallow Simulation";

	private static final String GET_ANALYSIS_STRING = "Get Analysis List";
	private static final String GET_INFO_STRING = "Get Info";
	private static final String GET_VAR_STRING = "Get World Var Values";
	private static final String LOAD_MODEL_STRING = "Load Model";
	private static final String PAUSE_STRING = "Pause";
	private static final String QUIT_STRING = "Quit";
	public static final String REFRESH_ALL_STRING = "Refresh All";
	private static final String REFRESH_STRING = "Refresh";
	private static final String REMOVE_ANALYSIS_STRING = "Remove Analysis List";
	private static final String REMOVE_UNFINISHED_ANALYSIS_STRING = "Remove Unfinished Analysis";
	private static final String RESTART_STRING = "Restart";
	private static final String SET_DISALLOW_TIMES_STRING = "Set Disallow Times";
	private static final String START_LOCAL_SIM_STRING = "Start Simulation (from local)";
	private static final String START_SIM_STRING = "Start Simulation";
	private static final String STOP_SIM_STRING = "Stop Simulation";
	private static final String UNPAUSE_STRING = "Unpause";
	public static final String UPDATE_CLIENT_LIST_STRING = "Update Client List";

	public static String choiceSimulationName(IGUISeSAmDeclaration guiSeSAmDeclaration, Component parent) {
		Iterator it = choiceSimulationNames(guiSeSAmDeclaration, parent, true);
		if (it.hasNext())
			return (String)it.next();
		else
			return null;
	}

	public static Iterator choiceSimulationNames(
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component parent) {
		return choiceSimulationNames(guiSeSAmDeclaration, parent, false);
	}

	private static Iterator choiceSimulationNames(
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component parent,
		boolean onlyOneSelection) {

		return CollectionsHelper
			.map(choiceSimulations(guiSeSAmDeclaration, parent, onlyOneSelection), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				return ((INamed)obj).getName();
			}
		});
	}

	public static Iterator choiceSimulations(
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component parent,
		boolean onlyOneSelection) {
		ExperimentOrSimulationChooserPanel experimentOrSimulationChooserPanel =
			new ExperimentOrSimulationChooserPanel(onlyOneSelection);

		experimentOrSimulationChooserPanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		JOptionPane optionPane =
			new JOptionPane(
				experimentOrSimulationChooserPanel,
				JOptionPane.QUESTION_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);

		GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
			parent,
			"Choice Simulation",
			optionPane);

		if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
			return experimentOrSimulationChooserPanel.getSelectedTreeNodes();
		}

		return CollectionsHelper.EMPTY_ITERATOR;
	}
	private File _oldAnalysisDir = null;
	private File _oldJarFile = null;

	protected final RemoteSimulationControl _remoteSimulationControl;
	protected final RemoteSimulationControlPanel _remoteSimulationControlPanel;
	private final List _savedSelectedConnections = new LinkedList();

	private final XMLRemoteTimeSchedule _timeSchedule = new XMLRemoteTimeSchedule();

	public RemoteSimulationControlPanelPopupListener(
		RemoteSimulationControlPanel remoteSimulationControlPanel,
		RemoteSimulationControl remoteSimulationControl) {

		_remoteSimulationControlPanel = remoteSimulationControlPanel;
		_remoteSimulationControl = remoteSimulationControl;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals(LOAD_MODEL_STRING)) {
			eventLoadModel();

		} else if (e.getActionCommand().equals(START_SIM_STRING)) {
			eventStartSimulation();

		} else if (e.getActionCommand().equals(START_LOCAL_SIM_STRING)) {
			eventStartSimulationFromLocal();

		} else if (e.getActionCommand().equals(GET_INFO_STRING)) {
			eventGetInfo();

		} else if (e.getActionCommand().equals(GET_VAR_STRING)) {
			eventGetWorldVarValues();

		} else if (e.getActionCommand().equals(PAUSE_STRING)) {
			eventPause();

		} else if (e.getActionCommand().equals(STOP_SIM_STRING)) {
			eventStopSimulation();

		} else if (e.getActionCommand().equals(UNPAUSE_STRING)) {
			eventUnpause();

		} else if (e.getActionCommand().equals(QUIT_STRING)) {
			eventQuit();

		} else if (e.getActionCommand().equals(RESTART_STRING)) {
			eventRestart();

		} else if (e.getActionCommand().equals(REFRESH_STRING)) {
			eventRefresh();

		} else if (e.getActionCommand().equals(ALLOW_SIM_STRING)) {
			eventAllowSimulation(true);

		} else if (e.getActionCommand().equals(DISALLOW_SIM_STRING)) {
			eventAllowSimulation(false);

		} else if (e.getActionCommand().equals(SET_DISALLOW_TIMES_STRING)) {
			eventSetDissallowTimes();

		} else if (e.getActionCommand().equals(REFRESH_ALL_STRING)) {
			eventRefreshAll();

		} else if (e.getActionCommand().equals(UPDATE_CLIENT_LIST_STRING)) {
			eventUpdateClientList();

		} else if (e.getActionCommand().equals(REMOVE_ANALYSIS_STRING)) {
			eventRemoveAnalysisList();

		} else if (e.getActionCommand().equals(REMOVE_UNFINISHED_ANALYSIS_STRING)) {
			eventRemoveUnfinishedAnalysisList();

		} else if (e.getActionCommand().equals(GET_ANALYSIS_STRING)) {
			eventGetAnalysisList();
		}
	}

	private void addPopupMenuItem(JMenu popupMenu, String label) {
		JMenuItem item = new JMenuItem(label);
		AbstractBoundAction.setFont(item, false);
		popupMenu.add(item);
		item.addActionListener(this);
	}

	private void addPopupMenuItem(JPopupMenu popupMenu, String label) {
		addPopupMenuItem(popupMenu, label, false);
	}

	private void addPopupMenuItem(JPopupMenu popupMenu, String label, boolean font) {
		JMenuItem item = new JMenuItem(label);
		AbstractBoundAction.setFont(item, font);
		popupMenu.add(item);
		item.addActionListener(this);
	}

	private boolean allSelectedConnectionsHaveOneOfStates(RemoteClientState[] states) throws IOException {
		Iterator it = getSavedSelectedConnections();
		while (it.hasNext()) {
			ISimulationControlConnection simulationControlConnection =
				(ISimulationControlConnection)it.next();
			XMLRemoteInfo xmlRemoteInfo = simulationControlConnection.getXMLRemoteInfo();
			if (!connectionIsInOneOfStates(xmlRemoteInfo, states)) {
				return false;
			}
		}

		return true;
	}

	private boolean connectionIsInOneOfStates(XMLRemoteInfo xmlRemoteInfo, RemoteClientState[] states) {
		if (xmlRemoteInfo == null)
			return false;
		int currState = xmlRemoteInfo.getState();
		for (int i = 0; i < states.length; i++) {
			if (states[i].getId() == currState) {
				return true;
			}
		}

		return false;
	}

	private String createConnectionsAsHTMLList(String title) {
		if (isSavedSelectionEmpty()) {
			return null;
		}

		StringBuffer buff = new StringBuffer();
		buff.append("<html>");
		buff.append(title);
		buff.append("<ul>");
		Iterator connections = getSavedSelectedConnections();
		while (connections.hasNext()) {
			ISimulationControlConnection remoteSimulationControlConnection =
				(ISimulationControlConnection)connections.next();
			buff.append("<li>");
			try {
				buff.append(getConnectionName(remoteSimulationControlConnection));
			} catch (IOException e) {
				Exception e1 = e;
				GUISeSAmDeclaration.showException(
					_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
					"ERROR",
					e1,
					_remoteSimulationControlPanel);
				buff.append("ERROR");
			}
		}

		buff.append("</ul>");

		return buff.toString();
	}

	private void doIndeterminateProgressbar(final String title, final Runnable runnable) {
		RemoteSimulationControlPanel parent = _remoteSimulationControlPanel;

		final JDialog dialog = SwingMoreUtilities.createIndeterminateProgressbarDialog(parent, title);

		dialog.setModal(false);

		SwingMoreUtilities.centerWindow(dialog);
		dialog.show();

		SwingMoreUtilities.invokeNotInSwingThread("Progressbar Thread", new Runnable() {
			public void run() {
				try {
					String oldText = _remoteSimulationControlPanel.getStatusBarText();
					_remoteSimulationControlPanel.setStatusBarText(title + " ...");

					runnable.run();

					_remoteSimulationControlPanel.setStatusBarText(oldText);

				} finally {
					dialog.setVisible(false);
					dialog.dispose();
				}
			}
		});
	}

	//	private void addPopupMenuItemWithAsk(JPopupMenu popupMenu, final String label) {
	//		JMenuItem item = new JMenuItem(label);
	//		AbstractBoundAction.setFont(item, false);
	//		popupMenu.add(item);
	//		item.addActionListener(new ActionListener() {
	//			public void actionPerformed(ActionEvent e) {
	//				if (e.getActionCommand().equals(label)) {
	//					if (JOptionPane.showConfirmDialog(_remoteSimulationControlPanel, "Really " + label)
	//						== JOptionPane.YES_OPTION)
	//						RemoteSimulationControlPanelPopupHelper.this.actionPerformed(e);
	//				}
	//			}
	//		});
	//	}

	private void doIndeterminateProgressbarShowResult(
		final String title,
		final IResultFunction resultFunction) {
		final StateCacheSimulationControlConnection remoteSimulationControlConnection =
			(StateCacheSimulationControlConnection)getSavedSelectedConnection();
		if (remoteSimulationControlConnection != null) {
			doIndeterminateProgressbar(
				title + getConnectionNameWithoutException(remoteSimulationControlConnection),
				new Runnable() {
				public void run() {
					try {
						String oldText = _remoteSimulationControlPanel.getStatusBarText();
						_remoteSimulationControlPanel.setStatusBarText(title + " ...");

						showJOptionPaneFromOtherThread(
							resultFunction.calcResults(remoteSimulationControlConnection));

						_remoteSimulationControlPanel.setStatusBarText(oldText);

					} catch (IOException e) {
						GUISeSAmDeclaration.showException(
							_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
							"ERROR",
							e,
							_remoteSimulationControlPanel);
					}
				}
			});
		}
	}

	private void doWithProgressbar(
		final String title,
		final String successString,
		final String failString,
		final boolean withRefresh,
		final IProgressFunction progressFunction) {

		final JProgressBar jProgressBar = new JProgressBar();
		jProgressBar.setStringPainted(true);
		jProgressBar.setMinimum(0);
		jProgressBar.setValue(0);
		jProgressBar.setMaximum(_savedSelectedConnections.size());
		//jProgressBar.setIndeterminate(true);

		//JOptionPane jOptionPane = new JOptionPane(jProgressBar,JOptionPane.INFORMATION_MESSAGE,JOptionPane.NO_OPTION);
		JOptionPane jOptionPane = new JOptionPane(jProgressBar);

		final JDialog dialog = jOptionPane.createDialog(_remoteSimulationControlPanel, title);

		dialog.setModal(false);

		SwingMoreUtilities.invokeNotInSwingThread(
			"Progress Thread",
			new ProgressRunner(
				dialog,
				title,
				jProgressBar,
				progressFunction,
				withRefresh,
				failString,
				successString));

		dialog.show();
	}

	private void eventAllowSimulation(final boolean allowSimulation) {
		if (!isSavedSelectionEmpty()) {

			doWithProgressbar(
				allowSimulation ? "Allow Simulation " : "Disallow Simulation ",
				null,
				"Failed to " + (allowSimulation ? "disallow" : "allow") + "simulation ",
				true,
				new IDoWithFunction() {
				public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
					throws IOException {
					return remoteSimulationControlConnection.setAllowSimulation(allowSimulation);
				}
			});
		}
	}

	private void eventGetAnalysisList() {
		if (!isSavedSelectionEmpty()) {
			JFileChooser chooser = new JFileChooser();
			if (_oldAnalysisDir == null) {
				File modelFile =
					_remoteSimulationControlPanel.getGUISeSAmDeclaration().getSeSAmDeclaration().getFile();
				if (modelFile != null) {
					_oldAnalysisDir = modelFile.getParentFile();
				}
			}

			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

			if (_oldAnalysisDir != null) {
				chooser.setCurrentDirectory(_oldAnalysisDir);
			}

			if (chooser.showSaveDialog(_remoteSimulationControlPanel) == JFileChooser.APPROVE_OPTION) {

				final File dir = chooser.getSelectedFile();
				dir.mkdirs();

				if (dir.exists() && dir.isDirectory()) {
					_oldAnalysisDir = dir;

					doWithProgressbar(
						GET_ANALYSIS_STRING,
						"Successfully saved Analysis to '" + dir + "': ",
						"Failed to get analysis list",
						true,
						new IResultFunction() {

						public Object calcResults(StateCacheSimulationControlConnection remoteSimulationControlConnection)
							throws IOException {
							XMLRemoteInfo remoteInfo = remoteSimulationControlConnection.getXMLRemoteInfo();
							if (remoteInfo != null) {
								long byteAmount =
									eventGetAnalysisListForConnection(
										remoteSimulationControlConnection.getSimulationControlConnection(),
										dir);
								if (byteAmount != -1) {
									double kiloBytes = Math.round(byteAmount / 1024d * 100d) / 100d;

									if (kiloBytes > 1024)
										return (Math.round((kiloBytes / 1024d * 100d)) / 100d) + "MB";
									else
										return kiloBytes + "KB";

								} else {
									return null;
								}
							}

							return null;
						}
					});
				}
			}
		}
	}

	protected long eventGetAnalysisListForConnection(
		RemoteSimulationControlConnection connection,
		final File dir)
		throws IOException, ZipException {

		RemoteAnswer answer = connection.getAnalysisAsRemoteAnswer();
		if ((answer == null) || !answer.isSuccessful()) {
			return -1;

		} else if (answer.getAnswer() == null) {
			return 0;

		} else {
			File file = connection.getAnalysisAsZipFile(answer);
			if ((file != null) && (file.exists())) {
				file.deleteOnExit();

				File newDir = dir; // new File(dir, remoteInfo.getMachineName());

				newDir.mkdirs();

				if (newDir.exists() && newDir.isDirectory()) {
					ZipUtilities.unzip(file, newDir);

				} else {
					ZipUtilities.unzip(file, dir);
				}
			}
			return file.length();
		}
	}

	private void eventGetInfo() {
		doIndeterminateProgressbarShowResult("Get Info for ", new IResultFunction() {
			public Object calcResults(StateCacheSimulationControlConnection remoteSimulationControlConnection)
				throws IOException {
				XMLDetailedRemoteInfo xmlDetailedRemoteInfo =
					remoteSimulationControlConnection.getXMLDetailedRemoteInfo();

				if (xmlDetailedRemoteInfo == null)
					return null;
				else
					return xmlDetailedRemoteInfo.getDetailedInfo();
			}
		});
	}

	private void eventGetWorldVarValues() {
		doIndeterminateProgressbarShowResult("Get WorldVars for ", new IResultFunction() {
			public Object calcResults(StateCacheSimulationControlConnection remoteSimulationControlConnection)
				throws IOException {

				XMLDetailedRemoteInfo xmlDetailedRemoteInfo =
					remoteSimulationControlConnection.getXMLDetailedRemoteInfo();

				if (xmlDetailedRemoteInfo == null)
					return null;

				Object[][] rowData = new Object[xmlDetailedRemoteInfo.getVariableInfoCount()][2];

				for (int i = 0; i < xmlDetailedRemoteInfo.getVariableInfoCount(); i++) {
					XMLRemoteVariableInfo xmlRemoteVariableInfo = xmlDetailedRemoteInfo.getVariableInfo(i);

					rowData[i][0] = xmlRemoteVariableInfo.getName();
					rowData[i][1] = xmlRemoteVariableInfo.getValue();
				}

				JTable table = new JTable(rowData, new Object[] { "Name", "Value" });

				table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

				return new JScrollPane(table);
			}
		});
	}
	private void eventLoadModel() {
		if (!isSavedSelectionEmpty()) {

			JFileChooser fileChooser = new JFileChooser();

			fileChooser.addChoosableFileFilter(new ExtensionsFileFilter("SeSAm model (*.xml)", "xml"));

			if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
				final File modelFile = fileChooser.getSelectedFile();
				if (modelFile.exists()) {
					final String modelFileString = modelFile.toString();

					try {
						final String fileContents =
							FileUtilities.getContentsAsString(new FileReader(modelFile));

						doWithProgressbar(
							"Loading " + modelFileString,
							null,
							"Error during loading of " + modelFile,
							true,
							new IDoWithFunction() {
							public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
								throws IOException {
								return remoteSimulationControlConnection.loadModelStream(
									modelFileString,
									new StringReader(fileContents));
							}
						});
					} catch (IOException e) {
						GUISeSAmDeclaration.showException(
							_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
							"ERROR",
							e,
							_remoteSimulationControlPanel);
					}
				}
			}
		}
	}

	private void eventPause() {
		if (!isSavedSelectionEmpty()) {

			doWithProgressbar("Pausing ", null, "Failed to pause simulation", true, new IDoWithFunction() {
				public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
					throws IOException {
					return remoteSimulationControlConnection.pause();
				}
			});
		}
	}

	private void eventQuit() {
		if (reallyDoIt("Really quit ")) {
			Iterator connections = getSavedSelectedConnections();

			while (connections.hasNext()) {
				ISimulationControlConnection remoteSimulationControlConnection =
					(ISimulationControlConnection)connections.next();
				try {
					remoteSimulationControlConnection.quit();

				} catch (IOException e) {
					Exception e1 = e;
					GUISeSAmDeclaration.showException(
						_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
						"ERROR",
						e1,
						_remoteSimulationControlPanel);
				}
			}
		}
	}

	private void eventReallyRestart(final File jarFile) {
		final LinkedList inetAdressList = new LinkedList();
		doWithProgressbar(
			(jarFile == null) ? RESTART_STRING : "Restart with new jar: " + jarFile,
			null,
			"Failed restarting",
			false,
			new IDoWithFunction() {
			public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
				throws IOException {
				inetAdressList.add(
					remoteSimulationControlConnection.getSimulationControlConnection().getInetAddress());
				remoteSimulationControlConnection.restart(jarFile);
				return true;
			}
		});

		final javax.swing.Timer timer = new javax.swing.Timer(100, new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				Iterator it = CollectionsHelper.copyCollection(inetAdressList).iterator();
				_remoteSimulationControl.sendDatagrams(it);

				((javax.swing.Timer)evt.getSource()).stop();
			}
		});

		timer.start();
	}

	private void eventRefresh() {
		doWithProgressbar("Refreshing connections", null, null, false, new IDoWithFunction() {
			public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
				throws IOException {
				remoteSimulationControlConnection.refresh();
				return true;
			}
		});
	}

	private void eventRefreshAll() {
		SwingMoreUtilities.invokeNotInSwingThread("Refresh Connections", new Runnable() {
			public void run() {
				for (int i = 0; i < _remoteSimulationControl.getConnectionListModel().size(); i++) {
					try {
						((StateCacheSimulationControlConnection)_remoteSimulationControl
							.getConnectionListModel()
							.get(i))
							.refresh();
					} catch (IOException e) {
					}
				}
			}
		});
	}

	private void eventRemoveAnalysisList() {
		if (reallyDoIt("Really remove last received analysis ")) {
			Iterator connections = getSavedSelectedConnections();

			while (connections.hasNext()) {
				ISimulationControlConnection remoteSimulationControlConnection =
					(ISimulationControlConnection)connections.next();
				try {
					remoteSimulationControlConnection.removeAnalysisFromLastZipFile();

				} catch (IOException e) {
					Exception e1 = e;
					GUISeSAmDeclaration.showException(
						_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
						"ERROR",
						e1,
						_remoteSimulationControlPanel);
				}
			}
		}
	}

	private void eventRemoveUnfinishedAnalysisList() {
		if (reallyDoIt("Really remove unfinished analysis ")) {
			Iterator connections = getSavedSelectedConnections();

			while (connections.hasNext()) {
				ISimulationControlConnection remoteSimulationControlConnection =
					(ISimulationControlConnection)connections.next();
				try {
					remoteSimulationControlConnection.removeAllUnfinishedFiles();

				} catch (IOException e) {
					Exception e1 = e;
					GUISeSAmDeclaration.showException(
						_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
						"ERROR",
						e1,
						_remoteSimulationControlPanel);
				}
			}
		}
	}

	private void eventRestart() {
		String str = createConnectionsAsHTMLList("Really Restart (with or without new jar)?");
		if (str != null) {
			Object[] options = { "Restart with new Jar", "Restart with old Jar", "Cancel" };

			int selectedIndex =
				JOptionPane.showOptionDialog(
					_remoteSimulationControlPanel,
					str,
					RESTART_STRING,
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[0]);

			if (selectedIndex == 0) {
				JFileChooser fileChooser = new JFileChooser();
				if ((_oldJarFile != null) && _oldJarFile.exists()) {
					fileChooser.setSelectedFile(_oldJarFile);
				} else if (
					(_remoteSimulationControlPanel.getGUISeSAmDeclaration() != null)
						&& (_remoteSimulationControlPanel
							.getGUISeSAmDeclaration()
							.getSeSAmDeclaration()
							.getFile()
							!= null)) {
					fileChooser.setCurrentDirectory(
						_remoteSimulationControlPanel
							.getGUISeSAmDeclaration()
							.getSeSAmDeclaration()
							.getFile()
							.getParentFile());
				}

				if (fileChooser.showOpenDialog(_remoteSimulationControlPanel)
					== JFileChooser.APPROVE_OPTION) {
					if (fileChooser.getSelectedFile().exists()) {
						_oldJarFile = fileChooser.getSelectedFile();
						eventReallyRestart(fileChooser.getSelectedFile());
					}
				}

			} else if (selectedIndex == 1) {
				eventReallyRestart(null);
			}
		}
	}

	private void eventSetDissallowTimes() {
		if (!isSavedSelectionEmpty()) {

			if (_savedSelectedConnections.size() == 1) {
				ISimulationControlConnection conn = getSavedSelectedConnection();
				try {
					XMLRemoteTimeSchedule schedule = conn.getXMLTimeSchedule();
					if (schedule != null) {
						TimeSettingsPanel.copyInto(schedule, _timeSchedule);
					}
				} catch (IOException e) {
					e.printStackTrace();
				}
			}

			String str = createConnectionsAsHTMLList("Really set automatic time dissallow schedule ?");
			if (str != null) {
				Object[] options = { "Enable dissallow schedule", "Disable dissallow schedule", "Cancel" };

				int selectedIndex =
					JOptionPane.showOptionDialog(
						_remoteSimulationControlPanel,
						str,
						RESTART_STRING,
						JOptionPane.OK_CANCEL_OPTION,
						JOptionPane.QUESTION_MESSAGE,
						null,
						options,
						options[0]);

				if (selectedIndex == 0) {
					XMLRemoteTimeSchedule remoteTimeSchedule = new XMLRemoteTimeSchedule();
					TimeSettingsPanel.copyInto(_timeSchedule, remoteTimeSchedule);

					if (TimeSettingsPanel
						.showTimeSettingsDialog(_remoteSimulationControlPanel, remoteTimeSchedule)) {

						TimeSettingsPanel.copyInto(remoteTimeSchedule, _timeSchedule);
						_timeSchedule.setAutomaticSchedule(true);
						sendTimeScheduleToSelectedClients();
					}

				} else if (selectedIndex == 1) {
					_timeSchedule.setAutomaticSchedule(false);
					sendTimeScheduleToSelectedClients();
				}
			}
		}
	}

	private void eventStartSimulation() {
		if (!isSavedSelectionEmpty()) {
			final String simName = JOptionPane.showInputDialog(_remoteSimulationControlPanel, "Simulation");

			if (simName != null) {

				doWithProgressbar(
					"Start sim " + simName + " at client ",
					null,
					"Failed to start" + " simulation of " + simName,
					true,
					new IDoWithFunction() {
					public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
						throws IOException {
						return remoteSimulationControlConnection.startSimulation(simName);
					}
				});
			}
		}
	}

	private void eventStartSimulationFromLocal() {
		if (!isSavedSelectionEmpty() && (_remoteSimulationControlPanel.getGUISeSAmDeclaration() != null)) {
			String simName =
				RemoteSimulationControlPanelPopupListener.choiceSimulationName(
					_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
					_remoteSimulationControlPanel);

			if (simName != null) {
				startSimulation(simName);
			}
		}
	}

	private void eventStopSimulation() {
		if (!isSavedSelectionEmpty()) {

			doWithProgressbar("Stopping ", null, "Failed to stop simulation", true, new IDoWithFunction() {
				public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
					throws IOException {
					return remoteSimulationControlConnection.stopSimulation();
				}
			});
		}
	}

	private void eventUnpause() {
		if (!isSavedSelectionEmpty()) {

			doWithProgressbar(
				"Unpausing ",
				null,
				"Failed to unpause simulation",
				true,
				new IDoWithFunction() {
				public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
					throws IOException {
					return remoteSimulationControlConnection.unpause();
				}
			});
		}
	}

	private void eventUpdateClientList() {
		updateClientList();
	}

	protected String getConnectionName(ISimulationControlConnection remoteSimulationControlConnection)
		throws IOException {
		XMLRemoteInfo remoteInfo = remoteSimulationControlConnection.getXMLRemoteInfo();
		if (remoteInfo == null)
			return remoteSimulationControlConnection.getName();
		else
			return remoteInfo.getMachineName();
	}

	private String getConnectionNameWithoutException(ISimulationControlConnection remoteSimulationControlConnection) {
		try {
			return getConnectionName(remoteSimulationControlConnection);

		} catch (IOException e) {
			GUISeSAmDeclaration.showException(
				_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
				"ERROR",
				e,
				_remoteSimulationControlPanel);
			return null;
		}
	}

	private ISimulationControlConnection getSavedSelectedConnection() {
		if (isSavedSelectionEmpty()) {
			return null;

		} else {
			return (ISimulationControlConnection)_savedSelectedConnections.get(0);
		}
	}

	protected Iterator getSavedSelectedConnections() {
		return _savedSelectedConnections.iterator();
	}

	private boolean isSavedSelectionEmpty() {
		return _savedSelectedConnections.isEmpty();
	}

	public void readSettings() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationControlPanel.class);

		String oldJarFileString = prefs.get("restartJarFile", null);
		if (oldJarFileString != null) {
			_oldJarFile = new File(oldJarFileString);
		}
		String oldAnalysisDirString = prefs.get("analysisDir", null);
		if (oldAnalysisDirString != null) {
			_oldAnalysisDir = new File(oldAnalysisDirString);
		}
	}

	private boolean reallyDoIt(String reallyDoString) {
		String str = createConnectionsAsHTMLList(reallyDoString);
		if (str != null)
			return JOptionPane.showConfirmDialog(_remoteSimulationControlPanel, str)
				== JOptionPane.YES_OPTION;
		else
			return false;
	}

	private void saveSelectedConnections(Iterator selectedConnections) {
		_savedSelectedConnections.clear();
		CollectionsHelper.addAll(_savedSelectedConnections, selectedConnections);
	}

	private void sendTimeScheduleToSelectedClients() {
		sortTimeSchedule();
		Iterator it = getSavedSelectedConnections();
		while (it.hasNext()) {
			StateCacheSimulationControlConnection element = (StateCacheSimulationControlConnection)it.next();
			try {
				if (element.setXMLTimeSchedule(_timeSchedule)) {
					element.refresh();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void sortTimeSchedule() {
		// dow -1 = index
		XMLRemoteTimeSlice[] newSliceArray = new XMLRemoteTimeSlice[7];
		for (int dow = 0; dow < 7; dow++) {
			XMLRemoteTimeSlice slice = getSliceFor(dow + 1);
			Assert.assertNotNull(slice);
			newSliceArray[dow] = slice;
		}
		_timeSchedule.setSlice(newSliceArray);
	}

	private XMLRemoteTimeSlice getSliceFor(int dow) {
		for (int i = 0; i < _timeSchedule.getSliceCount(); i++) {
			XMLRemoteTimeSlice slice = _timeSchedule.getSlice(i);
			if (slice.getDow() == dow) {
				return slice;
			}
		}
		return null;
	}

	protected void showJOptionPaneFromOtherThread(final Object message) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JOptionPane.showMessageDialog(_remoteSimulationControlPanel, message);
			}
		});
	}

	public void showPopup(Point pt, Iterator selectedConnections) throws IOException {
		saveSelectedConnections(selectedConnections);

		JPopupMenu popupMenu = new JPopupMenu();
		boolean bold = true;
		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.PAUSED_STATE })) {
			addPopupMenuItem(popupMenu, UNPAUSE_STRING, bold);
			bold = false;
		}

		if ((_remoteSimulationControlPanel.getGUISeSAmDeclaration() != null)
			&& allSelectedConnectionsHaveOneOfStates(
				new RemoteClientState[] {
					IRemoteProtocolConstants.NO_MODEL_STATE,
					IRemoteProtocolConstants.HAS_MODEL_STATE,
					IRemoteProtocolConstants.PAUSED_STATE,
					IRemoteProtocolConstants.FINISHED_STATE,
					IRemoteProtocolConstants.FINISHED_WITH_ERRORS_STATE })) {
			addPopupMenuItem(popupMenu, START_LOCAL_SIM_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.HAS_MODEL_STATE,
			IRemoteProtocolConstants.PAUSED_STATE,
			IRemoteProtocolConstants.FINISHED_STATE })) {
			addPopupMenuItem(popupMenu, START_SIM_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.NO_MODEL_STATE,
			IRemoteProtocolConstants.HAS_MODEL_STATE,
			IRemoteProtocolConstants.PAUSED_STATE,
			IRemoteProtocolConstants.FINISHED_STATE,
			IRemoteProtocolConstants.FINISHED_WITH_ERRORS_STATE })) {
			addPopupMenuItem(popupMenu, LOAD_MODEL_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.RUNNING_STATE })) {
			addPopupMenuItem(popupMenu, PAUSE_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.RUNNING_STATE,
			IRemoteProtocolConstants.FINISHED_STATE,
			IRemoteProtocolConstants.PAUSED_STATE })) {
			addPopupMenuItem(popupMenu, GET_VAR_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.RUNNING_STATE,
			IRemoteProtocolConstants.FINISHED_STATE,
			IRemoteProtocolConstants.FINISHED_WITH_ERRORS_STATE,
			IRemoteProtocolConstants.PAUSED_STATE,
			IRemoteProtocolConstants.DISALLOW_SIMULATION_STATE })) {
			addPopupMenuItem(popupMenu, STOP_SIM_STRING, bold);
			bold = false;
		}

		if (allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.DISALLOW_SIMULATION_STATE })) {
			addPopupMenuItem(popupMenu, ALLOW_SIM_STRING, bold);
			bold = false;
		}

		JMenu moreItems = new JMenu("More");
		AbstractBoundAction.setFont(moreItems, false);

		addPopupMenuItem(moreItems, GET_ANALYSIS_STRING);
		addPopupMenuItem(moreItems, REMOVE_ANALYSIS_STRING);
		addPopupMenuItem(moreItems, REMOVE_UNFINISHED_ANALYSIS_STRING);
		popupMenu.addSeparator();
		addPopupMenuItem(moreItems, GET_INFO_STRING);
		if (!allSelectedConnectionsHaveOneOfStates(new RemoteClientState[] { IRemoteProtocolConstants
			.DISALLOW_SIMULATION_STATE })) {
			addPopupMenuItem(moreItems, DISALLOW_SIM_STRING);
		}

		addPopupMenuItem(moreItems, RESTART_STRING);
		addPopupMenuItem(moreItems, SET_DISALLOW_TIMES_STRING);
		addPopupMenuItem(moreItems, QUIT_STRING);

		popupMenu.add(moreItems);

		addPopupMenuItem(popupMenu, REFRESH_STRING);

		popupMenu.show(_remoteSimulationControlPanel, pt.x, pt.y);
	}

	private void startSimulation(String simName) {
		try {
			final RemoteModelAsStringSimulationTask simTask =
				new RemoteModelAsStringSimulationTask(
					_remoteSimulationControlPanel.getGUISeSAmDeclaration().getSeSAmDeclaration(),
					simName);

			startSimulation(simName, simTask);

		} catch (IOException e) {
			GUISeSAmDeclaration.showException(
				_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
				"ERROR",
				e,
				_remoteSimulationControlPanel);

		} catch (XMLWriteException e) {
			GUISeSAmDeclaration.showException(
				_remoteSimulationControlPanel.getGUISeSAmDeclaration(),
				"ERROR",
				e,
				_remoteSimulationControlPanel);
		}
	}

	private void startSimulation(String simName, final IRemoteModelSimulationTask simTask) {
		doWithProgressbar(
			"Start sim " + simName + " at client ",
			null,
			"Failed to start" + " simulation of " + simName,
			true,
			new IDoWithFunction() {
			public boolean doWith(StateCacheSimulationControlConnection remoteSimulationControlConnection)
				throws IOException {
				return (
					simTask.loadModelAndSimulate(remoteSimulationControlConnection)
						== IRemoteModelSimulationTask.LOAD_MODEL_START_SIM_SUCCESS);
			}
		});
	}

	public void updateClientList() {
		doIndeterminateProgressbar("Refreshing Client List", new Runnable() {
			public void run() {

				_remoteSimulationControl.refreshList();

				synchronized (_remoteSimulationControl) {
					try {
						_remoteSimulationControl.wait(500);
					} catch (InterruptedException e) {
					}
				}
			}
		});
	}

	public void writeSettings() throws BackingStoreException {
		Preferences prefs = Preferences.userNodeForPackage(RemoteSimulationControlPanel.class);
		if (_oldJarFile != null) {
			prefs.put("restartJarFile", _oldJarFile.toString());
		}
		if (_oldAnalysisDir != null) {
			prefs.put("analysisDir", _oldAnalysisDir.toString());
		}
	}
}