package de.uniwue.ki.gui.mas.basics.feature;
import java.util.Iterator;

import javax.swing.JPopupMenu;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationListPanel;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationListToDeclarationListPanel;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (18.01.2002 11:03:09)
 * @author Christoph Oechslein
 */
public class FeatureClassDeclarationListToFeatureClassDeclarationList
	extends DeclarationListToDeclarationListPanel {

	private IObjectClassDeclaration _objectClassDeclaration = null;

	public FeatureClassDeclarationListToFeatureClassDeclarationList() {
		super();

		initialize();
	}

	public void dispose() {
		setObjectClassDeclaration(null);

		super.dispose();
	}

	private IFeatureClassDeclarationList getAvailibleFeatureClassDeclarationList() {
		if (getGUISeSAmDeclaration() == null)
			return null;
		else if (getObjectClassDeclaration() == null)
			return getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getAvailibleFeatureClassDeclarationList();
		else
			return getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getAvailibleFeatureClassDeclarationListFor(
				getObjectClassDeclaration());
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	class AbstractBoundSeeFeatureClassAction extends AbstractBoundAction {
		final IFeatureClassDeclaration _decl;
		AbstractBoundSeeFeatureClassAction(IFeatureClassDeclaration decl) {
			super("See details",InspectAction.INSPECT_ICON);
			_decl = decl;
		}
		public void doit() {
			Assert.assertNotNull(getGUISeSAmDeclaration());
			getGUISeSAmDeclaration().getFrameFactory().addPanel(
				null,
				"System feature",
				new SystemFeatureClassDeclarationPanel(_decl),
				true);
		}
	}

	class FeatureClassDeclarationListPanel extends DeclarationListPanel {
		protected JPopupMenu getDefaultPopupFor(final IDeclaration declaration) {
			IFeatureClassDeclaration featureClass = ((IFeatureClassDeclaration)declaration);
			if (featureClass != null) {
				JPopupMenu popupMenu = new JPopupMenu();
				popupMenu.add(
					AbstractBoundAction.createJMenuItem(
						new AbstractBoundSeeFeatureClassAction(featureClass),
						true));
				return popupMenu;
			}
			return null;
		}
	}

	private void initialize() {
		setName("FeatureClassDeclarationListToFeatureClassDeclarationList");
		setSize(421, 234);
		setRightDeclarationListPanelEditable(true);
		setRightDeclarationListPanelButtonPaneAvailible(false);

		setRightDeclarationListPanelText("Added Features");
		setLeftDeclarationListPanelText("All availible Features");

		setLeftDeclarationListPanel(new FeatureClassDeclarationListPanel());
	}

	protected Object mapLeftToRightButton(Object obj) {
		if (getObjectClassDeclaration() == null) {
			return null;

		} else if (containsFeatureClassDeclaration((IFeatureClassDeclaration)obj)) {
			return null;

		} else {
			return ((IFeatureClassDeclaration)obj).createObjectFeatureClassDeclaration(
				getObjectClassDeclaration());
		}
	}

	protected boolean containsFeatureClassDeclaration(IFeatureClassDeclaration featureClassDeclaration) {
		Iterator it = getObjectClassDeclaration().getObjectFeatureClassDeclarationList().iterator();
		while (it.hasNext()) {
			IObjectFeatureClassDeclaration objectFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration)it.next();
			if (objectFeatureClassDeclaration.getFeatureClassDeclaration() == featureClassDeclaration) {
				return true;
			}
		}

		return false;
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		updateLeftAndRightDeclarationListPanelDeclarationListModel();
	}

	public void setObjectClassDeclaration(IObjectClassDeclaration objectClassDeclaration) {
		IObjectClassDeclaration oldValue = _objectClassDeclaration;

		_objectClassDeclaration = objectClassDeclaration;

		firePropertyChange("objectClassDeclaration", oldValue, objectClassDeclaration);

		updateLeftAndRightDeclarationListPanelDeclarationListModel();
	}

	private void updateLeftAndRightDeclarationListPanelDeclarationListModel() {
		setLeftDeclarationListPanelDeclarationListModel(getAvailibleFeatureClassDeclarationList());
		if (getObjectClassDeclaration() == null) {
			setRightDeclarationListPanelDeclarationListModel(null);
		} else {
			setRightDeclarationListPanelDeclarationListModel(
				getObjectClassDeclaration().getObjectFeatureClassDeclarationList());
		}
	}
}