package de.uniwue.ki.gui.mas.basics.feature.evolution;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.plaf.basic.BasicArrowButton;

import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;

import de.uniwue.ki.mas.basics.feature.evolution.EvolutionObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.list.IMutableListModel;
/**
 * Insert the type's description here.
 * Creation date: (18.01.2002 11:34:19)
 * @author Christoph Oechslein
 */
public class EvolutionObjectFeatureClassDeclarationPanel extends AbstractDeclarationPanel {
	private EvolutionObjectFeatureClassDeclaration _evolutionObjectFeatureClassDeclaration;
	private JTable ivjScrollPaneTable = null;
	private JCheckBox ivjJCheckBoxDiploid = null;
	private JComboBox _dominantTypeJComboBox;
	private JList _orderList;

	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private final class IvjEventHandler implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == EvolutionObjectFeatureClassDeclarationPanel.this.getJCheckBoxDiploid())
				eventDiploidButtonPressed();
			else if (
				e.getSource() == EvolutionObjectFeatureClassDeclarationPanel.this.getDominantTypeJSpinner())
				eventDominantTypePressed();
		}
	}

	public EvolutionObjectFeatureClassDeclarationPanel() {
		super();
		initialize();
	}

	protected void eventDiploidButtonPressed() {
		if (getEvolutionObjectFeatureClassDeclaration() != null)
			getEvolutionObjectFeatureClassDeclaration().setDiploid(getJCheckBoxDiploid().isSelected());
	}

	protected void eventDominantTypePressed() {
		if (getEvolutionObjectFeatureClassDeclaration() != null)
			getEvolutionObjectFeatureClassDeclaration().setDominantType(
				(EvolutionObjectFeatureClassDeclaration.DominantType)getDominantTypeJSpinner()
					.getSelectedItem());
	}

	public IDeclaration getDeclaration() {
		return getEvolutionObjectFeatureClassDeclaration();
	}

	public EvolutionObjectFeatureClassDeclaration getEvolutionObjectFeatureClassDeclaration() {
		return _evolutionObjectFeatureClassDeclaration;
	}

	protected javax.swing.JCheckBox getJCheckBoxDiploid() {
		if (ivjJCheckBoxDiploid == null) {
			ivjJCheckBoxDiploid = new javax.swing.JCheckBox();
			ivjJCheckBoxDiploid.setText("Diploid (Start Setting)");
		}
		return ivjJCheckBoxDiploid;
	}

	private JTable getScrollPaneTable() {
		if (ivjScrollPaneTable == null) {
			ivjScrollPaneTable = new JTable();
			ivjScrollPaneTable.setAutoCreateColumnsFromModel(true);
		}

		return ivjScrollPaneTable;
	}

	private void initConnections() {
		getJCheckBoxDiploid().addActionListener(ivjEventHandler);
		getDominantTypeJSpinner().addActionListener(ivjEventHandler);
	}

	private void initialize() {
		setSize(468, 427);

		setLayout(new GridBagLayout());

		GridBagConstraints constraints;

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.insets.bottom = 5;
		constraints.insets.left = 3;
		constraints.anchor = GridBagConstraints.WEST;

		Box diploBox = Box.createHorizontalBox();

		diploBox.add(getJCheckBoxDiploid());
		diploBox.add(Box.createHorizontalStrut(5));
		diploBox.add(getDominantTypeJSpinner());

		add(diploBox, constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.fill = GridBagConstraints.BOTH;

		add(
			new JScrollPane(
				getScrollPaneTable(),
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS),
			constraints);

		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 2;
		constraints.weightx = 1;
		constraints.weighty = 0.75;
		constraints.fill = GridBagConstraints.BOTH;

		add(getOrderPanel(), constraints);

		initConnections();
	}

	private JPanel getOrderPanel() {
		JPanel orderPanel = new JPanel(new BorderLayout());

		orderPanel.setBorder(BorderFactory.createTitledBorder("Order in BitString"));

		orderPanel.add(new JScrollPane(getOrderJList()), BorderLayout.CENTER);

		Box updownPanel = getUpDownArrowsPanel();

		orderPanel.add(updownPanel, BorderLayout.EAST);

		return orderPanel;
	}

	private Box getUpDownArrowsPanel() {
		Box updownPanel = Box.createVerticalBox();

		BasicArrowButton upBasicArrowButton = new BasicArrowButton(SwingConstants.NORTH);
		BasicArrowButton downBasicArrowButton = new BasicArrowButton(SwingConstants.SOUTH);

		upBasicArrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeOrder(-1);
			}
		});

		downBasicArrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeOrder(1);
			}
		});

		updownPanel.add(upBasicArrowButton);
		updownPanel.add(downBasicArrowButton);

		return updownPanel;
	}

	protected void changeOrder(int delta) {
		int oldIndex = getOrderJList().getSelectedIndex();
		if (oldIndex != -1) {
			IMutableListModel mutableList = (IMutableListModel)getOrderJList().getModel();

			if ((oldIndex + delta < mutableList.size()) && (oldIndex + delta >= 0)) {
				Object selectedValue = getOrderJList().getSelectedValue();
				mutableList.changeIndex(selectedValue, oldIndex + delta);
				getOrderJList().setSelectedValue(selectedValue, true);
			}
		}
	}

	private JList getOrderJList() {
		if (_orderList == null) {
			_orderList = new JList();
			_orderList.setCellRenderer(new NamedCellRendererAdapter(_orderList.getCellRenderer()));
		}

		return _orderList;
	}

	private static final EvolutionObjectFeatureClassDeclaration.DominantType[] DOMINANT_TYPES =
		{
			EvolutionObjectFeatureClassDeclaration.CO_DOMINANT,
			EvolutionObjectFeatureClassDeclaration.BITSTRING_DOMINANT };

	protected JComboBox getDominantTypeJSpinner() {
		if (_dominantTypeJComboBox == null) {
			_dominantTypeJComboBox = new JComboBox(DOMINANT_TYPES);
			_dominantTypeJComboBox.setRenderer(
				new NamedCellRendererAdapter(_dominantTypeJComboBox.getRenderer()));
		}

		return _dominantTypeJComboBox;
	}

	public void setDeclaration(IDeclaration declaration) {
		setEvolutionObjectFeatureClassDeclaration((EvolutionObjectFeatureClassDeclaration)declaration);
	}

	public void setEvolutionObjectFeatureClassDeclaration(EvolutionObjectFeatureClassDeclaration newEvolutionObjectFeatureClassDeclaration) {
		_evolutionObjectFeatureClassDeclaration = newEvolutionObjectFeatureClassDeclaration;

		if (newEvolutionObjectFeatureClassDeclaration != null) {
			getScrollPaneTable().setModel(
				newEvolutionObjectFeatureClassDeclaration
					.getEvolutionObjectFeatureClassDeclarationTableModel());
			getJCheckBoxDiploid().setSelected(newEvolutionObjectFeatureClassDeclaration.isDiploid());
			getOrderJList().setModel(newEvolutionObjectFeatureClassDeclaration.getEvoEntryListModel());
			getDominantTypeJSpinner().setSelectedItem(
				newEvolutionObjectFeatureClassDeclaration.getDominantType());

		} else {
			// getScrollPaneTable().setModel(null);
		}
	}
}