package de.uniwue.ki.gui.mas.basics.feature.evolution;

import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.evolution.EvolutionObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.mas.xml.castor.v01.model.types.XMLDominantType;
import de.uniwue.ki.mas.xml.reader.*;

/**
 * @author oechslein
 * 
 * Creation date: 21.05.2003
 */
public final class XMLEvolutionFeatureReader extends AbstractXMLFeatureReader
        implements IXMLFeatureReader {

        public XMLEvolutionFeatureReader() {
    }

    public boolean isResponsibleFor(XMLFeature xmlFeature) {
        return xmlFeature instanceof XMLEvolutionFeature;
    }

    public boolean isResponsibleFor(XMLFeatureInstance xmlFeatureinstance) {
        return false;
    }

    public IObjectFeatureClassDeclaration createObjectFeatureClassDeclaration(
            XMLModelReader xmlModelReader,
            IObjectClassDeclaration objectClassDeclaration,
            XMLFeature xmlFeature) throws XMLReadException {

        XMLEvolutionFeature xmlEvolutionSettings = (XMLEvolutionFeature) xmlFeature;

        EvolutionObjectFeatureClassDeclaration evolutionObjectFeatureClassDeclaration = new EvolutionObjectFeatureClassDeclaration(
                objectClassDeclaration);

        if (!xmlEvolutionSettings.hasDiploid()) {
            evolutionObjectFeatureClassDeclaration.setDiploid(false);
        } else {
            evolutionObjectFeatureClassDeclaration
                    .setDiploid(xmlEvolutionSettings.getDiploid());
        }

        if ((xmlEvolutionSettings.getDominantType() == null)
                || (xmlEvolutionSettings.getDominantType()
                        .equals(XMLDominantType.VALUE_3))) {
            evolutionObjectFeatureClassDeclaration
                    .setDominantType(EvolutionObjectFeatureClassDeclaration.CO_DOMINANT);
        } else if ((xmlEvolutionSettings.getDominantType()
                .equals(XMLDominantType.VALUE_0))
                || (xmlEvolutionSettings.getDominantType()
                        .equals(XMLDominantType.VALUE_1))
                || (xmlEvolutionSettings.getDominantType()
                        .equals(XMLDominantType.VALUE_2))) {
            evolutionObjectFeatureClassDeclaration
                    .setDominantType(EvolutionObjectFeatureClassDeclaration.BITSTRING_DOMINANT);
        } else {
            throw new XMLReadException(
                    "Error while reading evo settings, dominant type '"
                            + xmlEvolutionSettings.getDominantType()
                            + "' not known.");
        }

        for (int i = 0; i < xmlEvolutionSettings.getSettingCount(); i++) {

            IBodyVariableClassDeclaration bodyVariableClassDeclaration = xmlModelReader.getXMLModelObjectReader()
                    .getRealBodyVariableClassDeclaration(
                            objectClassDeclaration,
                            (XMLBodyVariableClass) xmlEvolutionSettings
                                    .getSetting(i).getBodyVariableClass());

            EvolutionObjectFeatureClassDeclaration.EvoEntry evoEntry = new EvolutionObjectFeatureClassDeclaration.EvoEntry(
                    bodyVariableClassDeclaration);

            if (xmlEvolutionSettings.getSetting(i).hasBitLength()) {
                evoEntry.setMaxValue((1 << xmlEvolutionSettings.getSetting(i)
                        .getBitLength()) - 1);
                evoEntry.setMinValue(0);
            }

            if (xmlEvolutionSettings.getSetting(i).hasMinValue()) {
                evoEntry.setMinValue(xmlEvolutionSettings.getSetting(i)
                        .getMinValue());
            }

            if (xmlEvolutionSettings.getSetting(i).hasMaxValue()) {
                evoEntry.setMaxValue(xmlEvolutionSettings.getSetting(i)
                        .getMaxValue());
            }

            evolutionObjectFeatureClassDeclaration.getEvoEntryListModel().add(
                    evoEntry);
        }

        return evolutionObjectFeatureClassDeclaration;
    }

    public IObjectFeatureInstanceDeclaration createObjectFeatureInstanceDeclaration(
            XMLModelReader xmlModelReader,
            IObjectClassDeclaration objectClassDeclaration,
            XMLFeatureInstance xmlFeatureInstance, IObjectFeatureClassDeclaration featureClassParaent) throws XMLReadException {
        return null;
    }

}
