package de.uniwue.ki.gui.mas.basics.feature.evolution;

import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.evolution.EvolutionObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.mas.xml.castor.v01.model.types.XMLDominantType;
import de.uniwue.ki.mas.xml.writer.*;

/**
 * @author oechslein
 * 
 * Creation date: 21.05.2003
 */
public final class XMLEvolutionFeatureWriter 
        implements IXMLFeatureWriter {

    public boolean isResponsibleFor(
            IObjectFeatureClassDeclaration objectFeatureClassDeclaration) {
        return objectFeatureClassDeclaration instanceof EvolutionObjectFeatureClassDeclaration;
    }

    public XMLFeature createXMLObjectFeatureClassDeclaration(
            XMLModelWriter modelWriter,
            XMLModelObjectWriter xmlModelObjectWriter,
            IObjectFeatureClassDeclaration objectFeatureClassDeclaration)
            throws XMLWriteException {

        return createXMLEvolutionFeature(
                xmlModelObjectWriter,
                (EvolutionObjectFeatureClassDeclaration) objectFeatureClassDeclaration);
    }

    private XMLEvolutionFeature createXMLEvolutionFeature(
            XMLModelObjectWriter xmlModelObjectWriter,
            EvolutionObjectFeatureClassDeclaration evolutionObjectFeatureClassDeclaration)
            throws XMLWriteException {
        XMLEvolutionFeature xmlEvolutionSettings = new XMLEvolutionFeature();

        xmlEvolutionSettings.setType("BitString");
        if (evolutionObjectFeatureClassDeclaration.isDiploid()) {
            xmlEvolutionSettings.setDiploid(true);
        }

        if (EvolutionObjectFeatureClassDeclaration.BITSTRING_DOMINANT
                .equals(evolutionObjectFeatureClassDeclaration
                        .getDominantType())) {
            xmlEvolutionSettings.setDominantType(XMLDominantType.VALUE_2);

        } else if (EvolutionObjectFeatureClassDeclaration.CO_DOMINANT
                .equals(evolutionObjectFeatureClassDeclaration
                        .getDominantType())) {
            xmlEvolutionSettings.setDominantType(XMLDominantType.VALUE_3);

        } else {
            Assert.fail("Unknown dominant type: "
                    + evolutionObjectFeatureClassDeclaration.getDominantType());
        }

        Iterator it = evolutionObjectFeatureClassDeclaration
                .getEvoEntryListModel().iterator();
        while (it.hasNext()) {
            EvolutionObjectFeatureClassDeclaration.EvoEntry evoEntry = (EvolutionObjectFeatureClassDeclaration.EvoEntry) it
                    .next();
            XMLEvolutionSetting xmlEvolutionSetting = new XMLEvolutionSetting();

            xmlEvolutionSetting.setBodyVariableClass(xmlModelObjectWriter
                    .getXMLBodyVariableClass(evoEntry
                            .getBodyVariableClassDeclaration()));
            xmlEvolutionSetting.setMinValue(evoEntry.getMinValue());
            xmlEvolutionSetting.setMaxValue(evoEntry.getMaxValue());

            xmlEvolutionSettings.addSetting(xmlEvolutionSetting);
        }

        return xmlEvolutionSettings;
    }


    public XMLFeatureInstance createXMLObjectFeatureInstanceDeclaration(XMLModelWriter modelWriter, XMLModelObjectWriter modelObjectWriter, IObjectFeatureInstanceDeclaration currentFeatureInstanceDeclaration) throws XMLWriteException {
        //returns nothing because feature cannot be configured at instance-level
        return null;
    }

}
