package de.uniwue.ki.gui.mas.basics.feature.spatial;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.basics.type.argument.IArgumentDeclarationEditor;
import de.uniwue.ki.gui.mas.basics.type.argument.IArgumentDeclarationEditorParameter;
import de.uniwue.ki.mas.basics.feature.spatial.ImageType;

/**
 * Creation date: (05.10.2001 16:22:38)
 * @author Christoph Oechslein
 */
public class ImageTypeArgumentEditorPanel extends javax.swing.JButton implements IArgumentDeclarationEditor {
	private boolean _disposed = false;

	private File _imageFile;

	private final class IvjEventHandler implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			if (e.getSource() == ImageTypeArgumentEditorPanel.this)
				connEtoC1(e);
		}
	}
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();

	public ImageTypeArgumentEditorPanel() {
		super();
		initialize();
	}

	private void connEtoC1(java.awt.event.ActionEvent arg1) {
		try {
			this.eventButtonPressed();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	public void dispose() {
		_disposed = true;
	}

	protected void eventButtonPressed() {
		javax.swing.JFileChooser fileChooser = new javax.swing.JFileChooser(getImageFile());

		fileChooser.addChoosableFileFilter(
			new de.uniwue.ki.gui.util.ExtensionsFileFilter(
				"Image (*.gif;*.jpg;*.png)",
				new String[] { "gif", "jpg", "png" }));

		if (fileChooser.showOpenDialog(null) == javax.swing.JFileChooser.APPROVE_OPTION) {
			setImageFile(fileChooser.getSelectedFile());
		}
		setPreferredSize(new Dimension(getIcon().getIconWidth()+20, getIcon().getIconHeight()+40));
	}

	public de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration getArgumentDeclaration() {
		if (getImageFile() == null) {
			return null;
		} else {

			return de
				.uniwue
				.ki
				.mas
				.basics
				.type
				.argument
				.ArgumentDeclarationFactory
				.getInstance()
				.createArgumentDeclaration(ImageType.getInstance(), getImageFile());
		}
	}

	public File getImageFile() {
		return _imageFile;
	}

	public String getTitle() {
		return "Choice Image";
	}

	private void handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}

	private void initConnections() {
		this.addActionListener(ivjEventHandler);
	}

	private void initialize() {
		try {
			setName("ImageTypeArgumentEditorPanel");
			setText("no image");
			setSize(217, 25);
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	public boolean isDisposed() {
		return _disposed;
	}

	public static void main(String[] args) {
		try {
			final javax.swing.JFrame frame = new javax.swing.JFrame();
			final ImageTypeArgumentEditorPanel aImageTypeArgumentEditorPanel;
			aImageTypeArgumentEditorPanel = new ImageTypeArgumentEditorPanel();
			frame.setContentPane(aImageTypeArgumentEditorPanel);
			frame.setSize(aImageTypeArgumentEditorPanel.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			final java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			aImageTypeArgumentEditorPanel.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent arg0) {
					frame.setSize(
					aImageTypeArgumentEditorPanel.getIcon().getIconWidth() + insets.left + insets.right,
					aImageTypeArgumentEditorPanel.getIcon().getIconHeight() + insets.top + insets.bottom);
					System.out.println(arg0.getPropertyName());
					System.out.println(frame.getWidth() +","+
					frame.getHeight());
					frame.repaint();
					frame.invalidate();
				}
			});
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println(
				"Exception occurred in main() of de.uniwue.ki.gui.mas.model.basics.feature.spatial.ImageTypeArgumentEditorPanel");
			exception.printStackTrace(System.out);
		}
	}

	public void setArgumentDeclarationEditorParameter(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
		Object obj = argumentDeclarationEditorParameter.getArgumentDeclaration().getObject();

		if (obj == null) {

		} else if (obj instanceof File) {
			setImageFile((File)obj);

		} else if (obj instanceof String) {
			setImageFile(new File((String)obj));

		} else {
			Assert.fail();
		}
	}

	public void setImageFile(File newImageFile) {
		_imageFile = newImageFile;

		javax.swing.ImageIcon imageIcon = new javax.swing.ImageIcon(newImageFile.toString());

		if (imageIcon.getImage() != null) {
			setIcon(imageIcon);
			setText("");
		} else {
			setIcon(null);
			setText("no image");
		}

		invalidate();
	}

	public boolean inModalPanel() {
		return true;
	}
}