/*
 * Created on 27.06.2003
 */
package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;

import javax.swing.JFrame;

import de.uniwue.ki.mas.basics.feature.spatial.CircularSpatialInfo;
import de.uniwue.ki.mas.basics.feature.spatial.DefaultSpatialMapModel;

/**
 * @author muehlberger
 *
 */
public class JGridSpatialMapPanel extends JSpatialMapPanel {
	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			final DefaultSpatialMapModel defaultSMM = new DefaultSpatialMapModel();
			final CircularSpatialInfo info1 = createSpatialInfo(0, 0, 0);
			final CircularSpatialInfo info2 = createSpatialInfo(10, 10, 70.5);
			final CircularSpatialInfo info3 = createSpatialInfo(400, 300, 20);
			defaultSMM.addSpatialInfo(info1);
			defaultSMM.addSpatialInfo(info2);
			defaultSMM.addSpatialInfo(info3);

			defaultSMM.setWidth(400);
			defaultSMM.setHeight(400);
			defaultSMM.setTorus(true);
			defaultSMM.setDrawGrid(true);
			defaultSMM.setDiscrete(true);

			final JGridSpatialMapPanel spatialMapPanel = new JGridSpatialMapPanel(defaultSMM);

			spatialMapPanel.setZoomFactor(0.9);

			frame.setContentPane(spatialMapPanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			frame.setSize(200, 200);

			new Thread(new Runnable() {
				public void run() {
					for (int i = 0; i < 10000; i++) {
						defaultSMM.changePosition(info1, info1.getPosition().x + 1, info1.getPosition().y + 1);
						defaultSMM.changeDirection(info1, (info1.getDirection() + 0.01) % (2 * Math.PI));
						defaultSMM.changeDirection(info2, info1.getDirection());
						defaultSMM.changePosition(info3, info3.getPosition().x - 1, info3.getPosition().y + 1);
						defaultSMM.changeDirection(info3, -info1.getDirection());
						synchronized (this) {
							try {
								wait(1 * 10);
							} catch (InterruptedException e) {
							}

						}
					}
				}
			}, "Moving").start();

			frame.show();

		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JRectangularMap");
			exception.printStackTrace(System.out);
		}
	}

	public JGridSpatialMapPanel() {
		super();
	}

	public JGridSpatialMapPanel(DefaultSpatialMapModel spatialMapModel) {
		super(spatialMapModel);
	}

	protected Rectangle2D calculateRectangle(double posX, double posY, double radius) {

		int factorX = (int) Math.round(getGridWidth() * getGridWidth()) * 10;
		int factorY = (int) Math.round(getGridHeight() * getGridHeight()) * 10;

		if (!getSpatialMapModel().isDiscrete()) {
			return super.calculateRectangle(posX, posY, radius);

		} else {
			// Rundungsfehler werden hier (bis zu einem gewissen Grad) durch den Faktor "factorX" bzw. "factorY" eliminiert:
			double col =
				Math.floor(Math.round(convertMapPositionX(posX) * factorX / getGridWidth()) / factorX) 
					* getGridWidth();
			double row =
				Math.floor(Math.round(convertMapPositionY(posY) * factorY / getGridHeight()) / factorY)
					* getGridHeight();

			if (radius == 0) {
				return new Rectangle2D.Double(col, row, getGridWidth(), getGridHeight());

			} else {
				double colWidth = Math.ceil(convertMapPositionX(radius) / getGridWidth()) * getGridWidth();
				double colHeight = Math.ceil(convertMapPositionY(radius) / getGridHeight()) * getGridHeight();

				return new Rectangle2D.Double(col, row, colWidth, colHeight);
			}
		}
	}

	private void drawGrid(Graphics2D g2d) {
		Rectangle2D.Double totalRect = new Rectangle2D.Double(0, 0, getWidth() - 1, getHeight() - 1);
		drawGrid(g2d, totalRect);
		g2d.draw(totalRect);
	}

	// draws all Grids within Rectangle rect  
	public void drawGrid(Graphics2D g2d, Rectangle2D rect) {
		double colBegin = rect.getX() / getGridWidth();
		double rowBegin = rect.getY() / getGridHeight();
		double colEnd = rect.getWidth() / getGridWidth();
		double rowEnd = rect.getHeight() / getGridHeight();
		
		Color oldColor = g2d.getColor();
		g2d.setColor(new Color(0, 0, 0, 32));
		
		Double currRect = new Rectangle2D.Double(0, 0, getGridWidth(), getGridHeight());

		for (double i = colBegin; i < colEnd; i++) {
			currRect.x = i * getGridWidth();
			for (double j = rowBegin; j < rowEnd; j++) {
				currRect.y = j * getGridHeight();
				g2d.draw(currRect);
			}
		}
		g2d.setColor(oldColor);
	}

	private double getGridHeight() {
		if (getSpatialMapModel().getGridRows() < 2)
			return getHeight();
		else
			return (getHeight() + 0.0) / getSpatialMapModel().getGridRows();
	}

	private double getGridWidth() {
		if (getSpatialMapModel().getGridCols() < 2)
			return getWidth();
		else
			return (getWidth() + 0.0) / getSpatialMapModel().getGridCols();
	}

	protected void paintComponent(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;

		super.paintComponent(g2d);

		if (getSpatialMapModel().isDrawGrid())
			drawGrid(g2d);
	}
}
