package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.BorderLayout;
import java.util.Iterator;

import javax.swing.JSplitPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.mas.entity.object.ObjectInstanceDeclarationListWithSelectionPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.entity.object.ObjectInstanceDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.comparator.NamedComparator;

/**
 * @author oechslein
 * 
 * Creation date: 24.06.2003
 */
public class JSpatialMapBuildinObjectFeatureInstDeclPanel extends AbstractDeclarationPanel {
	private SpatialMapObjectFeatureInstDeclaration _spatialMapBuildinObjectFeatureInstanceDeclaration;

	private JSpatialMapPanelWithAddingRemovingToolbar _spatialMapPanelWithToolbar;
	private ObjectInstanceDeclarationListWithSelectionPanel _objectInstanceDeclarationListWithSelectionPanel;
	private ISpatialMapModelListener _mapModelListener = new ISpatialMapModelListener(){
			public void spatialInfoAdded(SpatialMapModelEvent spatialMapModelEvent) {
				eventSelectionChanged();
			}
			public void spatialInfoChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
				eventSelectionChanged();
			}
			public void spatialInfoRemoved(SpatialMapModelEvent spatialMapModelEvent) {
				eventSelectionChanged();
			}
			public void spatialMapModelChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
				getSpatialMapPanelWithToolbar().doEqualSize();
			}

			public void spatialInfosMoved(
				ISpatialInfo[] movedSpatialInfos,
				Position2D[] oldPosition) {

			}

		};

	public JSpatialMapBuildinObjectFeatureInstDeclPanel() {
		super();
		initialize();
	}

	public void dispose(){
		getSpatialMapPanelWithToolbar().getSpatialMapModel().removeSpatialMapModelListener(_mapModelListener);
		getObjectGroupObjectSettingPanel().dispose();
		getSpatialMapPanelWithToolbar().dispose();

		super.dispose();
	}

	protected void eventSelectionChanged() {
		ObjectInstanceDeclarationList objectInstanceDeclarationList = new ObjectInstanceDeclarationList();

		Iterator it =
			getSpatialMapPanelWithToolbar().getSelectSpatialMapPanelState().getSelectedSpatialInfos();

		while (it.hasNext()) {
			ISpatialInfo element = (ISpatialInfo)it.next();
			objectInstanceDeclarationList.add(element.getObject());
		}

		objectInstanceDeclarationList.sort(NamedComparator.getInstance());

		getObjectGroupObjectSettingPanel().setObjectInstanceDeclarationList(objectInstanceDeclarationList);
	}
	
	protected void eventSpatialInfoMoved(){
		ObjectInstanceDeclarationList objectInstanceDeclarationList = new ObjectInstanceDeclarationList();
		objectInstanceDeclarationList.add(getSpatialMapPanelWithToolbar().getMoveState().getSpatialInfoToDisplay().getObject());
		getObjectGroupObjectSettingPanel().setObjectInstanceDeclarationList(objectInstanceDeclarationList);
	}

	public IDeclaration getDeclaration() {
		return _spatialMapBuildinObjectFeatureInstanceDeclaration;
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return getSpatialMapPanelWithToolbar().getGUISeSAmDeclaration();
	}

	private ObjectInstanceDeclarationListWithSelectionPanel getObjectGroupObjectSettingPanel() {
		if (_objectInstanceDeclarationListWithSelectionPanel == null) {
			_objectInstanceDeclarationListWithSelectionPanel =
				new ObjectInstanceDeclarationListWithSelectionPanel();
			_objectInstanceDeclarationListWithSelectionPanel.setOrientation(JSplitPane.VERTICAL_SPLIT);
			_objectInstanceDeclarationListWithSelectionPanel.setButtonPaneAvailible(false);
		}
		return _objectInstanceDeclarationListWithSelectionPanel;
	}

	public ISpatialMapModel getSpatialMapModel() {
		return getSpatialMapPanelWithToolbar().getSpatialMapModel();
	}

	private JSpatialMapPanelWithAddingRemovingToolbar getSpatialMapPanelWithToolbar() {
		if (_spatialMapPanelWithToolbar == null) {
			_spatialMapPanelWithToolbar = new JSpatialMapPanelWithAddingRemovingToolbar();
			
			_spatialMapPanelWithToolbar
				.getSelectSpatialMapPanelState()
				.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					eventSelectionChanged();
				}
			});
			
			_spatialMapPanelWithToolbar
				.getMoveState()
				.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					eventSpatialInfoMoved();
				}
			});
			
			
			_spatialMapPanelWithToolbar.setAutomaticUpdate(true);
		}
				  
		return _spatialMapPanelWithToolbar;
	}

	private void initialize() {
		JSplitPane splitPane =
			new JSplitPane(
				JSplitPane.HORIZONTAL_SPLIT,
				getSpatialMapPanelWithToolbar(),
				getObjectGroupObjectSettingPanel());

		splitPane.setOneTouchExpandable(true);
		//splitPane.setResizeWeight(1);
		splitPane.setBorder(null);
		splitPane.resetToPreferredSizes();

		setLayout(new BorderLayout());
		add(splitPane, BorderLayout.CENTER);
	}

	public void setDeclaration(IDeclaration declaration) {
		_spatialMapBuildinObjectFeatureInstanceDeclaration =
			(SpatialMapObjectFeatureInstDeclaration)declaration;

		if (declaration != null)
			setSpatialMapModel(((SpatialMapObjectFeatureInstDeclaration)declaration).getSpatialMapModel());
		else
			setSpatialMapModel(null);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		getSpatialMapPanelWithToolbar().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getObjectGroupObjectSettingPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		if (spatialMapModel != null){
			spatialMapModel.addSpatialMapModelListener(_mapModelListener);
		}
		
		ISpatialMapModel model = getSpatialMapPanelWithToolbar().getSpatialMapModel();
		
		if (spatialMapModel == null && model != null){
			model.removeSpatialMapModelListener(_mapModelListener);
		}
		
		getSpatialMapPanelWithToolbar().setSpatialMapModel(spatialMapModel);
		
	}

	public void setSpatialMapPanelState(ISpatialMapPanelState spatialMapPanelState) {
		getSpatialMapPanelWithToolbar().setSpatialMapPanelState(spatialMapPanelState);
	}
}
