//$Id: JSpatialMapPanel.java,v 1.2 2004/02/03 13:24:34 herrler Exp $

package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.*;
import java.awt.geom.*;
import java.io.File;
import java.util.*;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.math.MathHelper;

public class JSpatialMapPanel extends JPanelWithGUISeSAmDeclaration implements ISpatialMapPanel, Scrollable {

	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			final DefaultSpatialMapModel defaultSMM = new DefaultSpatialMapModel();
			final CircularSpatialInfo info1 = createSpatialInfo(0, 0, 10);
			final CircularSpatialInfo info2 = createSpatialInfo(0, 0, 20);
			final CircularSpatialInfo info3 = createSpatialInfo(400, 300, 5);
			File agentGif = new File("c:/temp/agent.gif");
			if (agentGif.exists())
				info3.setImagePath(agentGif);
			defaultSMM.addSpatialInfo(info1);
			defaultSMM.addSpatialInfo(info2);
			defaultSMM.addSpatialInfo(info3);

			defaultSMM.setWidth(400);
			defaultSMM.setHeight(400);
			defaultSMM.setTorus(true);
			defaultSMM.setRotateImages(true);

			final JSpatialMapPanel spatialMapPanel = new JSpatialMapPanel(defaultSMM);

			spatialMapPanel.setZoomFactor(0.9);

			frame.setContentPane(spatialMapPanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			frame.setSize(200, 200);

			new Thread(new Runnable() {
				public void run() {
					for (int i = 0; i < 10000; i++) {
						defaultSMM.changePosition(
							info1,
							info1.getPosition().x + 1,
							info1.getPosition().y + 1);
						defaultSMM.changeDirection(info1, (info1.getDirection() + 0.01) % (2 * Math.PI));
						defaultSMM.changeDirection(info2, info1.getDirection());
						defaultSMM.changePosition(
							info3,
							info3.getPosition().x - 1,
							info3.getPosition().y + 1);
						defaultSMM.changeDirection(info3, -info1.getDirection());
						synchronized (this) {
							try {
								wait(1 * 10);
							} catch (InterruptedException e) {
							}

						}
					}
				}
			}, "Moving").start();

			frame.show();

		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JRectangularMap");
			exception.printStackTrace(System.out);
		}
	}

	protected static CircularSpatialInfo createSpatialInfo(int x, int y, double radius) {
		final CircularSpatialInfo info = new CircularSpatialInfo(new Object());
		info.setRadius(radius);
		info.getPosition().x = x;
		info.getPosition().y = y;
		info.setImage(ImageType.getDefaultAgentImage());
		return info;
	}

	private boolean _automaticUpdate = true;
	private double _zoomFactor = 1.0;
	private Image _backgroundImage;
	private ISpatialMapModel _spatialMapModel;
	private ISpatialMapPanelState _spatialMapPanelState;

	private ISpatialMapModelListener _spatialMapModelListener = new ISpatialMapModelListener() {

		public void spatialInfosMoved(
			ISpatialInfo[] movedSpatialInfos,
			Position2D[] oldPosition) {
		
		
		}

		public void spatialInfoAdded(SpatialMapModelEvent spatialMapModelEvent) {
			if (isAutomaticUpdate())
				repaintSpatialInfo(spatialMapModelEvent.getSpatialInfo());
		}

		public void spatialInfoChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
			if (isAutomaticUpdate()) {
				ISpatialInfo spatialInfo = spatialMapModelEvent.getSpatialInfo();
				Position2D pos = spatialInfo.getPosition();
				double x = pos.getX();
				double y = pos.getY();
				double radius = spatialInfo.getRadius();
				double direction = spatialInfo.getDirection();

				if (spatialMapModelEvent.getChangedType() == SpatialMapModelChangeEvent.POS_CHANGED_TYPE) {
					repaintSpatialInfo(
						direction,
						spatialMapModelEvent.getOldPosX(),
						spatialMapModelEvent.getOldPosY(),
						radius);
					repaintSpatialInfo(direction, x, y, radius);

				} else if (
					spatialMapModelEvent.getChangedType()
						== SpatialMapModelChangeEvent.DIRECTION_CHANGED_TYPE) {
					repaintSpatialInfo(spatialMapModelEvent.getOldValue(), x, y, radius);
					repaintSpatialInfo(direction, x, y, radius);

				} else if (
					spatialMapModelEvent.getChangedType()
						== SpatialMapModelChangeEvent.RADIUS_CHANGED_TYPE) {
					repaintSpatialInfo(direction, x, y, Math.max(spatialMapModelEvent.getOldValue(), radius));

				} else {
					repaintSpatialInfo(direction, x, y, radius);
				}
			}
		}

		public void spatialInfoRemoved(SpatialMapModelEvent spatialMapModelEvent) {
			if (isAutomaticUpdate())
				repaintSpatialInfo(spatialMapModelEvent.getSpatialInfo());
		}

		public void spatialMapModelChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
			if (getSpatialMapModel().getBackgroundImageFile() != null) {
				_backgroundImage =
					getToolkit().getImage(getSpatialMapModel().getBackgroundImageFile().toString());
			} else {
				_backgroundImage = null;
			}
			repaint();
		}
		
		
	};
	
	private Comparator _spatialInfoSorter = new Comparator(){
		
		public int compare(Object o1, Object o2){

			ISpatialInfo si1 = (ISpatialInfo) o1;
			ISpatialInfo si2 = (ISpatialInfo) o2;
			
			if (si1.getDrawPriority() < si2.getDrawPriority())
				return -1;
			else if (si1.getDrawPriority() > si2.getDrawPriority())
				return 1;
			else {
				if (si1.getRadius() > si2.getRadius())
					return -1;
				else if (si1.getRadius() < si2.getRadius())
					return 1;
				else 
				return 0;
					
			}
		}
	
		};

	public JSpatialMapPanel() {
		super();
		initialize();
	}

	public JSpatialMapPanel(DefaultSpatialMapModel spatialMapModel) {
		this();

		setSpatialMapModel(spatialMapModel);
	}

	protected Rectangle2D calculateRectangle(double posX, double posY, double radius) {
		if (radius == 0) {
			radius = 5;
		}

		double x = convertMapPositionX(posX - radius);
		double y = convertMapPositionY(posY - radius);
		double width = Math.max(10, convertMapPositionX(radius * 2));
		double height = Math.max(10, convertMapPositionY(radius * 2));

		return new Rectangle2D.Double(x, y, width, height);
	}

	public double convertMapPositionX(double mapPosX) {
		if (getSpatialMapModel() == null)
			return mapPosX;
		else
			return mapPosX * getWidth() / getSpatialMapModel().getWidth();
	}

	public double convertMapPositionY(double mapPosY) {
		if (getSpatialMapModel() == null)
			return mapPosY;
		else
			return mapPosY * getHeight() / getSpatialMapModel().getHeight();
	}

	public Position2D convertMapPosition(Point2D point){
		return new Position2D(convertMapPositionX(point.getX()), convertMapPositionY(point.getY()));
		
	}

	public double convertPanelPositionX(int panelPosX) {
		if (getSpatialMapModel() == null)
			return panelPosX;
		else
			return panelPosX * getSpatialMapModel().getWidth() / getWidth();
	}

	public double convertPanelPositionY(int panelPosY) {
		if (getSpatialMapModel() == null)
			return panelPosY;
		else
			return panelPosY * getSpatialMapModel().getHeight() / getHeight();
	}
	
	public Position2D convertPanelPosition(Point2D point){
		return new Position2D(convertPanelPositionX((int) point.getX()), convertPanelPositionY((int) point.getY()));
		
	}

	private AffineTransform createRotateTransform(Rectangle2D rectangle, double direction) {
		return AffineTransform.getRotateInstance(direction, rectangle.getCenterX(), rectangle.getCenterY());
	}

	public void dispose() {
		setSpatialMapModel(null);

		super.dispose();
	}

	private void drawBackground(Graphics2D g2d) {
		if (_backgroundImage != null)
			g2d.drawImage(_backgroundImage, 0, 0, getWidth(), getHeight(), this);
	}

	private void drawImage(Graphics2D g2d, Image image, Rectangle2D rectangle) {
		g2d.drawImage(
			image,
			(int)Math.floor(rectangle.getX()),
			(int)Math.floor(rectangle.getY()),
			(int)Math.ceil(rectangle.getWidth()),
			(int)Math.ceil(rectangle.getHeight()),
			this);
	}

	private void drawSpatialInfo(
		ISpatialInfo spatialInfo,
		Graphics2D g2d,
		Rectangle2D rectangle,
		Shape oldClip) {

		if (getSpatialMapModel().isRotateImages()) {
			AffineTransform rotateTransform = createRotateTransform(rectangle, spatialInfo.getDirection());
			Shape rotatedShape = rotateTransform.createTransformedShape(rectangle);

			if (oldClip == null || oldClip.intersects(rotatedShape.getBounds2D())) {
				AffineTransform oldTransform = g2d.getTransform();

				rotateTransform.preConcatenate(oldTransform);

				g2d.setTransform(rotateTransform);

				drawImage(g2d, spatialInfo.getImage(), rectangle);

				g2d.setTransform(oldTransform);
			}
		} else {
			if (oldClip == null || oldClip.intersects(rectangle)) {
				drawImage(g2d, spatialInfo.getImage(), rectangle);
			}
		}
	}

	private void drawSpatialInfo(ISpatialInfo spatialInfo, Graphics2D g2d, Shape oldClip) {
		double radius = spatialInfo.getRadius();
		Position2D pos = spatialInfo.getPosition();
		Rectangle2D rectangle = calculateRectangle(pos.getX(), pos.getY(), radius);

		if (getSpatialMapModel().isTorus() && !getBounds().contains(rectangle)) {
			Rectangle2D.Double currRectangle2D =
				new Rectangle2D.Double(0, 0, rectangle.getWidth(), rectangle.getHeight());
			for (int x = -1; x <= 1; x++) {
				currRectangle2D.x = x * getWidth() + rectangle.getX();
				for (int y = -1; y <= 1; y++) {
					currRectangle2D.y = y * getHeight() + rectangle.getY();
					drawSpatialInfo(spatialInfo, g2d, currRectangle2D, oldClip);
				}
			}
		} else {
			drawSpatialInfo(spatialInfo, g2d, rectangle, oldClip);
		}
	}

	private int getIncrement(int orientation, double increment) {
		if (getSpatialMapModel() == null) {
			return 0;
		} else if (orientation == SwingConstants.HORIZONTAL) {
			return (int) (getSpatialMapModel().getWidth() * increment);
		} else {
			return (int) (getSpatialMapModel().getHeight() * increment);
		}
	}

	public Dimension getPreferredScrollableViewportSize() {
		//updateComponentSize();
		Dimension preferredSize = getPreferredSize();
		return new Dimension(Math.max(300, preferredSize.width), Math.max(300, preferredSize.height));
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		return getIncrement(orientation, 0.1);
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		return getIncrement(orientation, 0.01);
	}

	//	public Dimension getPreferredSize() {
	//		if (getSpatialMapModel() == null)
	//			return super.getPreferredSize();
	//		else
	//			return new Dimension((int)getSpatialMapModel().getWidth(), (int)getSpatialMapModel().getHeight());
	//	}

	public ISpatialMapModel getSpatialMapModel() {
		return _spatialMapModel;
	}

	public final ISpatialMapPanelState getSpatialMapPanelState() {
		return _spatialMapPanelState;
	}

	public double getZoomFactor() {
		return _zoomFactor;
	}

	private void initialize() {
		setOpaque(true);
		setDoubleBuffered(true);
		setSize(300, 300);
	}

	public boolean isAutomaticUpdate() {
		return _automaticUpdate;
	}

	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		Graphics2D g2d = (Graphics2D)g;

		//						g2d.setRenderingHint(
		//							java.awt.RenderingHints.KEY_ALPHA_INTERPOLATION,
		//							java.awt.RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);

		g2d.setRenderingHint(
			java.awt.RenderingHints.KEY_TEXT_ANTIALIASING,
			java.awt.RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		g2d.setRenderingHint(
			java.awt.RenderingHints.KEY_ANTIALIASING,
			java.awt.RenderingHints.VALUE_ANTIALIAS_ON);

		//				g2d.setRenderingHint(
		//					java.awt.RenderingHints.KEY_INTERPOLATION,
		//					java.awt.RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		//
		//		g2d.setRenderingHint(
		//			java.awt.RenderingHints.KEY_RENDERING,
		//			java.awt.RenderingHints.VALUE_RENDER_QUALITY);
		//
		//				g2d.setRenderingHint(
		//					java.awt.RenderingHints.KEY_COLOR_RENDERING,
		//					java.awt.RenderingHints.VALUE_COLOR_RENDER_QUALITY);
		//				g2d.setRenderingHint(
		//					java.awt.RenderingHints.KEY_DITHERING,
		//					java.awt.RenderingHints.VALUE_DITHER_ENABLE);

		g2d.draw(new Rectangle2D.Double(0,0,getWidth()-1,getHeight()-1));
		Shape oldClip = g2d.getClip();

		drawBackground(g2d);

		Object[] list = getSpatialMapModel().getAllSpatialInfoList().toArray(); 
		Arrays.sort(list, _spatialInfoSorter);
		
		
		Iterator it = Arrays.asList(list).iterator();
		
		while (it.hasNext()) {
			drawSpatialInfo((ISpatialInfo)it.next(), g2d, oldClip);
		}

		if (getSpatialMapPanelState() != null) {
			getSpatialMapPanelState().paintComponent(g2d);
		}
	}

	public void refresh() {
		repaint();
	}

	private void repaintSpatialInfo(double direction, double posX, double posY, double radius) {
		Rectangle2D rectangle = calculateRectangle(posX, posY, radius);
		rectangle.add(rectangle.getWidth() + 1, rectangle.getHeight() + 1);

		Rectangle rect;

		if (getSpatialMapModel().isRotateImages()) {
			rect = createRotateTransform(rectangle, direction).createTransformedShape(rectangle).getBounds();
		} else {
			rect = rectangle.getBounds();
		}

		if (getSpatialMapModel().isTorus() && !getBounds().contains(rect)) {
			for (int x = -1; x <= 1; x++) {
				for (int y = -1; y <= 1; y++) {
					repaint(x * getWidth() + rect.x, y * getHeight() + rect.y, rect.width, rect.height);
				}
			}
		} else {
			repaint(
				Math.max(0, rect.x),
				Math.max(0, rect.y),
				Math.min(getWidth(), rect.width),
				Math.min(getHeight(), rect.height));
		}
	}

	protected void repaintSpatialInfo(ISpatialInfo spatialInfo) {
		Position2D pos = spatialInfo.getPosition();
		repaintSpatialInfo(spatialInfo.getDirection(), pos.getX(), pos.getY(), spatialInfo.getRadius());
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		if (automaticUpdate != _automaticUpdate) {
			_automaticUpdate = automaticUpdate;

			if (getSpatialMapModel() != null) {
				if (automaticUpdate)
					getSpatialMapModel().addSpatialMapModelListener(_spatialMapModelListener);
				else
					getSpatialMapModel().removeSpatialMapModelListener(_spatialMapModelListener);
			}
		}
	}

	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		ISpatialMapModel oldSpatialMapModel = getSpatialMapModel();

		if (oldSpatialMapModel != null) {
			oldSpatialMapModel.removeSpatialMapModelListener(_spatialMapModelListener);
		}

		_spatialMapModel = spatialMapModel;

		if ((spatialMapModel != null) && isAutomaticUpdate()) {
			spatialMapModel.addSpatialMapModelListener(_spatialMapModelListener);
		}

		if (spatialMapModel != null) {
			setZoomFactor(300 / spatialMapModel.getWidth());

			updateComponentSize();
			
			/********* added by Jrg Freudenstein, Aachen, 28.9.03 **/
			if (getSpatialMapModel().getBackgroundImageFile() != null) {
				_backgroundImage = getToolkit().getImage(getSpatialMapModel().getBackgroundImageFile().toString());
			} else {
				_backgroundImage = null;
			}
			repaint();
			/********* end added */
		}
	}

	public final void setSpatialMapPanelState(ISpatialMapPanelState spatialMapPanelState) {
		ISpatialMapPanelState oldSpatialMapPanelState = _spatialMapPanelState;
		if (oldSpatialMapPanelState != spatialMapPanelState) {
			_spatialMapPanelState = spatialMapPanelState;

			if (oldSpatialMapPanelState != null) {
				oldSpatialMapPanelState.setSpatialMapPanel(null);
				removeMouseListener(oldSpatialMapPanelState);
			}

			if (spatialMapPanelState != null) {
				spatialMapPanelState.setSpatialMapPanel(this);
				addMouseListener(spatialMapPanelState);
			}
		}
	}

	public void setZoomFactor(double zoomFactor) {
		if ((zoomFactor != 0) && !MathHelper.doubleEqual(zoomFactor, _zoomFactor)) {
			_zoomFactor = zoomFactor;
			updateComponentSize();
		}
	}

	protected void updateComponentSize() {
		if (getSpatialMapModel() == null)
			return;

		Insets insets = getInsets();

		double width = insets.left + insets.right + _zoomFactor * getSpatialMapModel().getWidth();
		double height = insets.top + insets.bottom + _zoomFactor * getSpatialMapModel().getHeight();

		Dimension newDimension = new Dimension((int)Math.ceil(width), (int)Math.ceil(height));
		setSize(newDimension);
		setPreferredSize(newDimension);
	}
}
