package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;

import de.uniwue.ki.gui.mas.entity.objectclass.ObjectClassDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationComboBox;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;
import de.uniwue.ki.mas.basics.feature.spatial.SpatialInfoObjectFeatureHelper;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.util.declaration.DeclarationListEvent;
import de.uniwue.ki.mas.util.declaration.DeclarationListListener;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/**
 * @author oechslein
 * 
 * Creation date: 23.07.2003
 */
public class JSpatialMapPanelWithAddingRemovingToolbar extends JSpatialMapPanelWithToolbar {
	private AddSpatialInfoSpatialMapPanelState _addState;
	private RemoveSpatialInfoSpatialMapPanelState _removeState;
	private MoveSpatialInfoSpatialMapPanelState _moveState;

	private DeclarationComboBox _comboBox;
	private JToggleButton _addButton;
	private JToggleButton _removeButton;
	private JToggleButton _moveButton;
	//private PropertyChangeListener _propertyChangeListener;
	private JButton _undoButton;
	private JButton _redoButton;
	private SpatialMapUndoManager _undoMan;
	
	private DeclarationListListener _declListListener = new DeclarationListListener() {
        public void declarationAdded(DeclarationListEvent event) {
			refreshComboBox();
        }

        public void declarationRemoved(DeclarationListEvent event) {
			refreshComboBox();
        }
    };
	
	private UndoableEditListener _undoableEditlistener = new UndoableEditListener(){
		public void undoableEditHappened(UndoableEditEvent e) {
			updateUndoRedoButtonStatus();
		}
  
	};
	
/*
	private PropertyChangeListener getComboBoxPropertyChangeListener() {
		if (_propertyChangeListener == null) {
			_propertyChangeListener = new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent e) {
					Object source = e.getSource();
					if ((source == getComboBox())
						&& (e.getNewValue() instanceof AbstractObjectClassDeclaration)) {
							getAddState().setItem(e.getNewValue());
							getComboBox().setSelectedItem(e.getNewValue());
					}
				}
			};
		}
		return _propertyChangeListener;
	}
*/	
	public JSpatialMapPanelWithAddingRemovingToolbar(){
		super();
		_undoMan = new SpatialMapUndoManager();
		_undoMan.addUndoableEditListener(_undoableEditlistener);
	}

	protected void addToolBarStateButtons(JToolBar toolBar, ButtonGroup toolBarButtonGroup) {

		super.addToolBarStateButtons(toolBar, toolBarButtonGroup);

		toolBar.add(getMoveToggleButton(toolBarButtonGroup));
		toolBar.add(getAddToggleButton(toolBarButtonGroup));
		toolBar.add(getRemoveToggleButton(toolBarButtonGroup));
		
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(getComboBox());
		toolBar.add(panel);
				
		toolBar.add(getUndoButton());
		toolBar.add(getRedoButton());
	
		toolBar.addSeparator();
	}

	private JToggleButton getAddToggleButton(ButtonGroup toolBarButtonGroup) {
		if (_addButton == null) {
			_addButton = new JToggleButton(getAddState().getIcon());
			_addButton.setToolTipText("Add Agent");
			
			_addButton.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					_comboBox.setEnabled(_addButton.isSelected());
					if (_addButton.isSelected()) {
						if (getComboBox().getSelectedItem() == null) {
							IDeclarationList declarationListModel = getComboBox().getDeclarationListModel();
							if (declarationListModel.isEmpty()) {
								getComboBox().setSelectedItem(
									getGUISeSAmDeclaration()
										.getSeSAmModelDeclaration()
										.getAgentClassDeclarationListAndFactory()
										.getFirstDeclarationOrCreateNewIfEmpty());
							} else {
								getComboBox().setSelectedItem(declarationListModel.get(0));
							}
						}
						setSpatialMapPanelState(getAddState());
					}
				}
			});

			toolBarButtonGroup.add(_addButton);
			_addButton.setMargin(new Insets(0, 0, 0, 0));

		}
		return _addButton;
	}

	private JToggleButton getRemoveToggleButton(ButtonGroup toolBarButtonGroup) {
		if (_removeButton == null) {
			_removeButton = new JToggleButton(getRemoveState().getIcon());
			_removeButton.setToolTipText("Remove Agent");
			_removeButton.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (_removeButton.isSelected())
						setSpatialMapPanelState(getRemoveState());
				}
			});

			toolBarButtonGroup.add(_removeButton);
			_removeButton.setMargin(new Insets(0, 0, 0, 0));

		}
		return _removeButton;
	}
	
	private JToggleButton getMoveToggleButton(ButtonGroup toolBarButtonGroup) {
		if (_moveButton == null) {
			_moveButton = new JToggleButton(getMoveState().getIcon());
			_moveButton.setToolTipText("Move Agent");
			
			_moveButton.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					if (_moveButton.isSelected())
						setSpatialMapPanelState(getMoveState());
				}
		
			});

			toolBarButtonGroup.add(_moveButton);
			_moveButton.setMargin(new Insets(0, 0, 0, 0));

		}
		return _moveButton;
	}
	
	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		if(getGUISeSAmDeclaration() != null){
			getGUISeSAmDeclaration()
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getAgentClassDeclarationListAndFactory()
				.getAgentClassDeclarationList()
				.removeDeclarationListListener(_declListListener);
			
			getGUISeSAmDeclaration()
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getRessourceClassDeclarationListAndFactory()
				.getRessourceClassDeclarationList()
				.removeDeclarationListListener(_declListListener);
		}
		
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null) {
			
			guiSeSAmDeclaration
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getAgentClassDeclarationListAndFactory()
				.getAgentClassDeclarationList()
				.addDeclarationListListener(_declListListener);
			
			guiSeSAmDeclaration
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getRessourceClassDeclarationListAndFactory()
				.getRessourceClassDeclarationList()
				.addDeclarationListListener(_declListListener);
			
			ISeSAmModelDeclaration seSAmModelDeclaration = getGUISeSAmDeclaration().getSeSAmModelDeclaration();
		
			if (seSAmModelDeclaration != null) {
				getComboBox().setDeclarationFactory(
					seSAmModelDeclaration.getAgentClassDeclarationListAndFactory().getDeclarationFactory());
			}
			
            getComboBox().setDeclarationListModel(
                SpatialInfoObjectFeatureHelper
                    .getAllObjectClassesWithSpatialInfoFeature(
                    	seSAmModelDeclaration));
		}
			
			
	}

	public DeclarationComboBox getComboBox() {
		if (_comboBox == null) {
			_comboBox = new DeclarationComboBox();
			_comboBox.setEnabled(false);
			_comboBox.setEditable(false);
			_comboBox.setPrototypeDisplayValue("Untitled Ressource Class");
			_comboBox.setRenderer(new ObjectClassDeclarationCellRendererAdapter(_comboBox.getRenderer()));
			
			_comboBox.addItemListener(
				new ItemListener() {
	                public void itemStateChanged(ItemEvent e) {
						if (e.getStateChange() == ItemEvent.SELECTED)
							getComboBox().setSelectedItem(e.getItem());
							getAddState().setItem(e.getItem());

	                }
	            }
			);
			
			
		}
		return _comboBox;
	}

	private RemoveSpatialInfoSpatialMapPanelState getRemoveState() {
		if (_removeState == null) {
			_removeState = new RemoveSpatialInfoSpatialMapPanelState();
		}
		return _removeState;
	}

	private AddSpatialInfoSpatialMapPanelState getAddState() {
		if (_addState == null) {
			_addState = new AddSpatialInfoSpatialMapPanelState();
		}
		return _addState;
	}
	
	protected MoveSpatialInfoSpatialMapPanelState getMoveState() {
		if (_moveState == null) 
			_moveState = new MoveSpatialInfoSpatialMapPanelState(getSelectSpatialMapPanelState());
		return _moveState;
	}
	
	private JButton getUndoButton(){
		if (_undoButton == null){
			_undoButton = new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/Undo24.gif"));
			_undoButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					 _undoMan.undo();
					 updateUndoRedoButtonStatus();
					 getJSpatialMapPanel().repaint();
				}
			});
			_undoButton.setEnabled(false);
			_undoButton.setToolTipText("Undo");
		}
		return _undoButton;
	}
	
	private JButton getRedoButton(){
		if (_redoButton == null){
			_redoButton = new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/Redo24.gif"));
			_redoButton.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e){
					_undoMan.redo();
					updateUndoRedoButtonStatus();
					getJSpatialMapPanel().repaint();
				}
			});
			_redoButton.setEnabled(false);
			_redoButton.setToolTipText("Redo");
		}
		return _redoButton;
	}
	
	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		if(getSpatialMapModel() != null){
			getSpatialMapModel().removeSpatialMapModelListener(_undoMan);
		
		}
		super.setSpatialMapModel(spatialMapModel);
		
		if (spatialMapModel != null){
			spatialMapModel.addSpatialMapModelListener(_undoMan);
		
		}
		
	}
	
	protected void updateUndoRedoButtonStatus(){
		getUndoButton().setEnabled(_undoMan.canUndo());
		getRedoButton().setEnabled(_undoMan.canRedo());
		getUndoButton().setToolTipText("Undo: " + _undoMan.getUndoPresentationName());
		getRedoButton().setToolTipText("Redo: " + _undoMan.getRedoPresentationName());
		
	}
	
	protected void refreshComboBox(){
        IDeclarationList objectsWithSpatialInfo =
            SpatialInfoObjectFeatureHelper
                .getAllObjectClassesWithSpatialInfoFeature(
               		getGUISeSAmDeclaration().getSeSAmModelDeclaration());
               		
		getComboBox().setDeclarationListModel(objectsWithSpatialInfo);
		
		if (!objectsWithSpatialInfo.isEmpty()){
			getComboBox().setSelectedItem(objectsWithSpatialInfo.get(0));
			getAddState().setItem(objectsWithSpatialInfo.get(0));
		}else{
			getComboBox().setSelectedItem(null);
			getAddState().setItem(null);
		}
	}

    public void dispose() {
		IGUISeSAmDeclaration seSAmDeclaration = getGUISeSAmDeclaration();
		if (seSAmDeclaration != null){
			seSAmDeclaration
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getAgentClassDeclarationListAndFactory()
				.getAgentClassDeclarationList()
				.removeDeclarationListListener(_declListListener);
			
			seSAmDeclaration
				.getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getRessourceClassDeclarationListAndFactory()
				.getRessourceClassDeclarationList()
				.removeDeclarationListListener(_declListListener);
			
		}
        super.dispose();
    }
	
}
