package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.BorderLayout;
import java.util.Iterator;

import javax.swing.JSplitPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.entity.object.ObjectGroupObjectSettingPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.ObjectFeaturePanel;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.list.DefaultMutableListModel;

/**
 * @author oechslein
 * 
 * Creation date: 24.06.2003
 */
public class JSpatialMapPanelWithSelectedIObjects extends ObjectFeaturePanel implements ISpatialMapPanel {
	private ObjectGroupObjectSettingPanel _objectGroupObjectSettingPanel;
	private JSpatialMapPanelWithToolbar _spatialMapPanelWithToolbar;

	public JSpatialMapPanelWithSelectedIObjects() {
		super();

		initialize();
	}

	protected void eventSelectionChanged() {
		DefaultMutableListModel model = new DefaultMutableListModel();
		Iterator it =
			getSpatialMapPanelWithToolbar().getSelectSpatialMapPanelState().getSelectedSpatialInfos();

		while (it.hasNext()) {
			ISpatialInfo element = (ISpatialInfo)it.next();
			model.add(element.getObject());
		}

		model.sort(NamedComparator.getInstance());
		
		getObjectGroupObjectSettingPanel().setObjectGroup(model);
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return getSpatialMapPanelWithToolbar().getGUISeSAmDeclaration();
	}

	private ObjectGroupObjectSettingPanel getObjectGroupObjectSettingPanel() {
		if (_objectGroupObjectSettingPanel == null) {
			_objectGroupObjectSettingPanel = new ObjectGroupObjectSettingPanel();
			_objectGroupObjectSettingPanel.setOrientation(JSplitPane.VERTICAL_SPLIT);
		}
		
		return _objectGroupObjectSettingPanel;
	}

	public ISpatialMapModel getSpatialMapModel() {
		return getSpatialMapPanelWithToolbar().getSpatialMapModel();
	}

	private JSpatialMapPanelWithToolbar getSpatialMapPanelWithToolbar() {
		if (_spatialMapPanelWithToolbar == null) {
			_spatialMapPanelWithToolbar = new JSpatialMapPanelWithToolbar();
			_spatialMapPanelWithToolbar
				.getSelectSpatialMapPanelState()
				.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					eventSelectionChanged();
				}
			});
		}
		return _spatialMapPanelWithToolbar;
	}

	private void initialize() {
		JSplitPane splitPane =
			new JSplitPane(
				JSplitPane.HORIZONTAL_SPLIT,
				getSpatialMapPanelWithToolbar(),
				getObjectGroupObjectSettingPanel());

		splitPane.setOneTouchExpandable(true);
		//splitPane.setResizeWeight(1);
		splitPane.setBorder(null);
		splitPane.resetToPreferredSizes();

		setLayout(new BorderLayout());
		add(splitPane, BorderLayout.CENTER);
	}

	public boolean isAutomaticUpdate() {
		return getSpatialMapPanelWithToolbar().isAutomaticUpdate();
	}

	public void refresh() {
		getSpatialMapPanelWithToolbar().refresh();
		getObjectGroupObjectSettingPanel().refresh();
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		getSpatialMapPanelWithToolbar().setAutomaticUpdate(automaticUpdate);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		getSpatialMapPanelWithToolbar().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getObjectGroupObjectSettingPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		getSpatialMapPanelWithToolbar().setSpatialMapModel(spatialMapModel);
	}

	public void setSpatialMapPanelState(ISpatialMapPanelState spatialMapPanelState) {
		getSpatialMapPanelWithToolbar().setSpatialMapPanelState(spatialMapPanelState);
	}
	
	public void setObjectFeature(IObjectFeature feature) {
		super.setObjectFeature(feature);
		
		Assert.assertTrue(feature instanceof ISpatialMapObjectFeature);
		setSpatialMapModel(((ISpatialMapObjectFeature)feature).getSpatialMapModel());
	}


	public void dispose() {
		getObjectGroupObjectSettingPanel().dispose();
		getSpatialMapPanelWithToolbar().dispose();

		super.dispose();
	}
}
