package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.JDisposeablePanel;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.feature.spatial.*;

/**
 * @author oechslein
 * 
 * Creation date: 24.06.2003
 */
public class JSpatialMapPanelWithToolbar extends JDisposeablePanel implements ISpatialMapPanel {
	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			final DefaultSpatialMapModel defaultSMM = new DefaultSpatialMapModel();
			final CircularSpatialInfo info1 = new CircularSpatialInfo(new Object());
			info1.setRadius(60.0);
			info1.setImage(ImageType.getDefaultAgentImage());
			final CircularSpatialInfo info2 = new CircularSpatialInfo(new Object());
			info2.setRadius(70.5);
			info2.setImage(ImageType.getDefaultAgentImage());
			defaultSMM.addSpatialInfo(info1);
			defaultSMM.addSpatialInfo(info2);

			defaultSMM.setWidth(200);
			defaultSMM.setHeight(200);
			defaultSMM.setTorus(true);

			final JSpatialMapPanelWithToolbar spatialMapPanel = new JSpatialMapPanelWithToolbar();
			spatialMapPanel.setSpatialMapModel(defaultSMM);

			frame.setContentPane(spatialMapPanel);
			frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

			frame.setSize(400, 300);

			new Thread(new Runnable() {
				public void run() {
					for (int i = 0; i < 100; i++) {
						defaultSMM.changePosition(info1, i * 10, i * 10);
						synchronized (this) {
							try {
								wait(1 * 1000);
							} catch (InterruptedException e) {
							}

						}
					}
				}
			}, "Moving").start();

			frame.show();

		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JRectangularMap");
			exception.printStackTrace(System.out);
		}
	}

	private JScrollPane _scrollPane;

	private SelectSpatialMapPanelState _selectSpatialMapPanelState;
	private JToggleButton _selectToggleButton;
	private JGridSpatialMapPanel _spatialMapPanel;
	private ButtonGroup _spatialMapPanelStateButtonGroup;
	private JToolBar _toolBar;

	public JSpatialMapPanelWithToolbar() {
		super();
		initialize();
	}

	public void dispose() {
		getJSpatialMapPanel().dispose();

		super.dispose();
	}

	protected void doEqualSize() {
		if (getSpatialMapModel() != null) {
			Insets scrollInsets = getJScrollPane().getInsets();
			Insets mapInsets = getJSpatialMapPanel().getInsets();

			double scrollHeight =
				getJScrollPane().getHeight()
					- scrollInsets.top
					- scrollInsets.bottom
					- mapInsets.top
					- mapInsets.bottom
					- (getJScrollPane().getHorizontalScrollBar().isVisible()
						? getJScrollPane().getHorizontalScrollBar().getHeight()
						: 0);
			double scrollWidth =
				getJScrollPane().getWidth()
					- scrollInsets.left
					- scrollInsets.right
					- mapInsets.left
					- mapInsets.right
					- (getJScrollPane().getVerticalScrollBar().isVisible()
						? getJScrollPane().getVerticalScrollBar().getWidth()
						: 0);

			double newHeight = scrollHeight / getSpatialMapModel().getHeight();
			double newWidth = scrollWidth / getSpatialMapModel().getWidth();

			getJSpatialMapPanel().setZoomFactor(Math.min(newHeight, newWidth));
		}
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return getJSpatialMapPanel().getGUISeSAmDeclaration();
	}

	protected JScrollPane getJScrollPane() {
		if (_scrollPane == null) {
			_scrollPane = new JScrollPane(getJSpatialMapPanel());
		}
		return _scrollPane;
	}

	protected JSpatialMapPanel getJSpatialMapPanel() {
		if (_spatialMapPanel == null) {
			_spatialMapPanel = new JGridSpatialMapPanel();
		}
		return _spatialMapPanel;
	}

	protected JToolBar getJToolBar() {
		if (_toolBar == null) {

			_toolBar = new JToolBar();
			_toolBar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);

			_toolBar.setFloatable(false);
			_toolBar.setBorderPainted(false);

			addToolBarStateButtons(_toolBar, getSpatialMapPanelStateButtonGroup());

			addOtherToolBarButtons(_toolBar);

			for (int i = 0; i < _toolBar.getComponentCount(); i++) {
				if (_toolBar.getComponentAtIndex(i) instanceof AbstractButton)
					 ((AbstractButton)_toolBar.getComponentAtIndex(i)).setMargin(new Insets(0, 0, 0, 0));
			}
		}

		return _toolBar;
	}

	protected void addOtherToolBarButtons(JToolBar toolBar) {
		JButton displayWholeMap = new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/Zoom24.gif"));
		displayWholeMap.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				doEqualSize();
			}
		});
		displayWholeMap.setToolTipText("Display whole Map");
		
		JButton zoomIn = new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/ZoomIn24.gif"));
		zoomIn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				_spatialMapPanel.setZoomFactor(_spatialMapPanel.getZoomFactor() * 1.1);
			}
		});
		zoomIn.setToolTipText("Zoom in");
		
		JButton zoomOut = new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/ZoomOut24.gif"));
		zoomOut.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				_spatialMapPanel.setZoomFactor(_spatialMapPanel.getZoomFactor() * 0.9);
			}
		});
		zoomOut.setToolTipText("Zoom out");
		
		toolBar.add(displayWholeMap);
		toolBar.add(zoomIn);
		toolBar.add(zoomOut);
		
		toolBar.addSeparator();

		final JButton settingsButton =
			new JButton(SwingMoreUtilities.getJavaSunIcon(24, "general/Preferences24.gif"));

		settingsButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				settingsButton.setEnabled(false);
				JSpatialMapSettingsPanel settingsPanel =
					new JSpatialMapSettingsPanel(_spatialMapPanel.getSpatialMapModel());

				JPanel panel = new JPanel();
				panel.add(settingsPanel);
				panel.setBorder(Borders.createEmptyBorder("0dlu, 1dlu, 1dlu, 1dlu"));

				getGUISeSAmDeclaration()
					.getFrameFactory()
					.addPanel(
						JSpatialMapPanelWithToolbar.this,
						"Map Settings",
						panel,
						true,
						WindowConstants.DISPOSE_ON_CLOSE,
						new Runnable() {
					public void run() {
						settingsButton.setEnabled(true);
					}
				});
			}
		});
		settingsButton.setToolTipText("Map Settings");
		
		toolBar.add(settingsButton);
	}

	protected void addToolBarStateButtons(JToolBar toolBar, ButtonGroup spatialMapPanelStateButtonGroup) {
		toolBar.add(getSelectToggleButton());
	}

	public SelectSpatialMapPanelState getSelectSpatialMapPanelState() {
		if (_selectSpatialMapPanelState == null) {
			_selectSpatialMapPanelState = new SelectSpatialMapPanelState();
		}
		
		return _selectSpatialMapPanelState;
	}

	private JToggleButton getSelectToggleButton() {
		if (_selectToggleButton == null) {
			_selectToggleButton =
				new JToggleButton(new AbstractBoundAction(getSelectSpatialMapPanelState().getIcon()) {
				public void doit() {
					setSpatialMapPanelState(getSelectSpatialMapPanelState());
				}
			});
			
			_selectToggleButton.setToolTipText("Select Agents");
			getSpatialMapPanelStateButtonGroup().add(_selectToggleButton);
			_selectToggleButton.setSelected(true);
			setSpatialMapPanelState(getSelectSpatialMapPanelState());
			_selectToggleButton.setMargin(new Insets(0, 0, 0, 0));
		}

		return _selectToggleButton;
	}

	public ISpatialMapModel getSpatialMapModel() {
		return getJSpatialMapPanel().getSpatialMapModel();
	}

	public ISpatialMapPanelState getSpatialMapPanelState() {
		return getJSpatialMapPanel().getSpatialMapPanelState();
	}

	private ButtonGroup getSpatialMapPanelStateButtonGroup() {
		if (_spatialMapPanelStateButtonGroup == null) {
			_spatialMapPanelStateButtonGroup = new ButtonGroup();
		}
		return _spatialMapPanelStateButtonGroup;
	}

	private void initialize() {
		setLayout(new FormLayout("default:grow", "default, default:grow"));

		CellConstraints cc = new CellConstraints();

		add(getJToolBar(), cc.xy(1, 1));
		add(getJScrollPane(), cc.xy(1, 2, "fill, fill"));

		addAncestorListener(new AncestorListener() {
			public void ancestorAdded(AncestorEvent event) {
				doEqualSize();
				removeAncestorListener(this);
			}

			public void ancestorMoved(AncestorEvent event) {
				doEqualSize();
				removeAncestorListener(this);
			}

			public void ancestorRemoved(AncestorEvent event) {
				doEqualSize();
				removeAncestorListener(this);
			}
		});
	}

	public boolean isAutomaticUpdate() {
		return getJSpatialMapPanel().isAutomaticUpdate();
	}

	public void refresh() {
		getJSpatialMapPanel().refresh();
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		getJSpatialMapPanel().setAutomaticUpdate(automaticUpdate);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		getJSpatialMapPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		getJSpatialMapPanel().setSpatialMapModel(spatialMapModel);
		doEqualSize();
	}

	public void setSpatialMapPanelState(ISpatialMapPanelState spatialMapPanelState) {
		getJSpatialMapPanel().setSpatialMapPanelState(spatialMapPanelState);
	}
}
