/*
 * Created on 09.07.2003
 *
 */
package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.BorderLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.extras.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.util.bound.BoundJDoubleTextField;
import de.uniwue.ki.gui.util.bound.BoundJIntegerTextField;
import de.uniwue.ki.mas.basics.feature.spatial.DefaultSpatialMapModel;
import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;

/**
 * @author muehlberger
 *
 */
public class JSpatialMapSettingsPanel extends JPanel {

	public static void main(String[] args) {
		DefaultSpatialMapModel mapModel = new DefaultSpatialMapModel();
		JFrame frame = new JFrame("SeSAm Panel");

		JPanel panel = new JPanel(new BorderLayout());

		JSpatialMapSettingsPanel changeSpatialMapPanel = new JSpatialMapSettingsPanel(mapModel);

		JSpatialMapPanelWithToolbar mapPanel = new JSpatialMapPanelWithToolbar();
		mapPanel.setSpatialMapModel(mapModel);

		panel.add(changeSpatialMapPanel, BorderLayout.NORTH);
		panel.add(mapPanel, BorderLayout.CENTER);

		frame.setSize(600, 600);
		frame.getContentPane().add(panel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}

//	public static void makeFrame(JSpatialMapSettingsPanel changeSpatialMapPanel, DefaultSpatialMapModel mapModel) {
//		JFrame frame = new JFrame("Change Map Panel");
//
//		JPanel panel = new JPanel(new BorderLayout());
//
//		//		JSpatialMapPanel mapPanel = new JSpatialMapPanel();
//		//		mapPanel.setSpatialMapModel(mapModel);
//
//		panel.add(changeSpatialMapPanel, BorderLayout.NORTH);
//		//		panel.add(mapPanel, BorderLayout.CENTER);
//
//		frame.setSize(600, 600);
//		frame.getContentPane().add(panel);
//		frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
//		frame.pack();
//		frame.setVisible(true);
//	}

	private JCheckBox _torusCheckBox;
	private JCheckBox _discreteCheckBox;
	private JCheckBox _centeredPosCheckBox;
	private JCheckBox _gridCheckBox;
	private JCheckBox _rotateCheckBox;
	private BoundJIntegerTextField _rowsField;
	private BoundJIntegerTextField _colsField;
	private BoundJDoubleTextField _heightField;
	private BoundJDoubleTextField _widthField;

	private ISpatialMapModel _spatialMapModel;

	private final ItemListener _itemListener = new ItemListener() {
		public void itemStateChanged(ItemEvent e) {

			Object source = e.getItemSelectable();
			if (source == _torusCheckBox) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getSpatialMapModel().setTorus(true);
				} else {
					getSpatialMapModel().setTorus(false);
				}
			} else if (source == _discreteCheckBox) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getSpatialMapModel().setDiscrete(true);
					_centeredPosCheckBox.setEnabled(true);
				} else {
					getSpatialMapModel().setDiscrete(false);
					_centeredPosCheckBox.setEnabled(false);
				}
			} else if (source == _centeredPosCheckBox) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getSpatialMapModel().setCenteredPos(true);
				} else {
					getSpatialMapModel().setCenteredPos(false);
				}
			} else if (source == _gridCheckBox) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getSpatialMapModel().setDrawGrid(true);
				} else {
					getSpatialMapModel().setDrawGrid(false);
				}
			} else if (source == _rotateCheckBox) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					getSpatialMapModel().setRotateImages(true);
				} else {
					getSpatialMapModel().setRotateImages(false);
				}
			}
		}
	};

	private final PropertyChangeListener _changeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
			Object source = e.getSource();

			if (source == _rowsField) {
				getSpatialMapModel().setGridRows(_rowsField.getIntegerValue());
			} else if (source == _colsField) {
				getSpatialMapModel().setGridCols(_colsField.getIntegerValue());
			} else if (source == _heightField) {
				getSpatialMapModel().setHeight(_heightField.getDoubleValue());
			} else if (source == _widthField) {
				getSpatialMapModel().setWidth(_widthField.getDoubleValue());
			}
		}
	};

	public JSpatialMapSettingsPanel() {
		this(new DefaultSpatialMapModel());
	}

	public JSpatialMapSettingsPanel(ISpatialMapModel model) {

		setSpatialMapModel(model);

		PanelBuilder builder =
			new PanelBuilder(
				this,
				new FormLayout(
					"max(10dlu;pref), 2dlu, "
						+ "max(10dlu;pref), 2dlu, "
						+ "max(10dlu;pref), 2dlu, "
						+ "max(10dlu;pref), 2dlu, "
						+ "max(10dlu;pref)",
					"default, default"));

		_torusCheckBox = new JCheckBox("Torus", getSpatialMapModel().isTorus());
		_discreteCheckBox = new JCheckBox("Discrete", getSpatialMapModel().isDiscrete());
		_centeredPosCheckBox = new JCheckBox("center Pos?", getSpatialMapModel().isCenteredPos());
		_centeredPosCheckBox.setEnabled(getSpatialMapModel().isDiscrete());
		_gridCheckBox = new JCheckBox("Grid", getSpatialMapModel().isDrawGrid());
		_rotateCheckBox = new JCheckBox("Rotate", getSpatialMapModel().isRotateImages());

		_rowsField = new BoundJIntegerTextField();
		_colsField = new BoundJIntegerTextField();
		_heightField = new BoundJDoubleTextField();
		_widthField = new BoundJDoubleTextField();

		_rowsField.setText(getSpatialMapModel().getGridRows() + "");
		_colsField.setText(getSpatialMapModel().getGridCols() + "");
		_heightField.setText(getSpatialMapModel().getHeight() + "");
		_widthField.setText(getSpatialMapModel().getWidth() + "");

		_torusCheckBox.addItemListener(_itemListener);
		_discreteCheckBox.addItemListener(_itemListener);
		_centeredPosCheckBox.addItemListener(_itemListener);
		_gridCheckBox.addItemListener(_itemListener);
		_rotateCheckBox.addItemListener(_itemListener);

		_rowsField.addPropertyChangeListener("value", _changeListener);
		_colsField.addPropertyChangeListener("value", _changeListener);
		_heightField.addPropertyChangeListener("value", _changeListener);
		_widthField.addPropertyChangeListener("value", _changeListener);

		builder.add(_discreteCheckBox, new CellConstraints(1, 1, 1, 1));
		builder.add(_centeredPosCheckBox, new CellConstraints(3, 1, 1, 1));
		builder.add(_torusCheckBox, new CellConstraints(5, 1, 1, 1));
		builder.add(_gridCheckBox, new CellConstraints(7, 1, 1, 1));
		builder.add(_rotateCheckBox, new CellConstraints(9, 1, 1, 1));

		builder.nextLine();

		builder.add(makePanel(new JPanel(), "Rows:", _rowsField), new CellConstraints(1, 2, 1, 1));
		builder.add(makePanel(new JPanel(), "Cols:", _colsField), new CellConstraints(3, 2, 1, 1));
		builder.add(makePanel(new JPanel(), "Height(total):", _heightField), new CellConstraints(5, 2, 1, 1));
		builder.add(makePanel(new JPanel(), "Width(total):", _widthField), new CellConstraints(7, 2, 1, 1));

	}

	private JPanel makePanel(JPanel panel, String labelText, JFormattedTextField field) {
		DefaultFormBuilder builder =
			new DefaultFormBuilder(panel, new FormLayout("max(10dlu;pref), 2dlu, max(25dlu;pref)", "default"));
		field.setColumns(5);
		builder.append(labelText, field);
		return builder.getPanel();
	}

	private void setSpatialMapModel(ISpatialMapModel model) {
		_spatialMapModel = model;
	}

	private ISpatialMapModel getSpatialMapModel() {
		return _spatialMapModel;
	}

}
