package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.Icon;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.event.StateChangeMulticaster;


/**
 * @author Hatko
 * Created on 13.11.2003
 */
public class MoveSpatialInfoSpatialMapPanelState extends AbstractSpatialMapPanelState implements MouseMotionListener{
	private transient ChangeListener _changeListener;
	private boolean _moving;
	private boolean _exited;
	private Position2D[] _oldPositions;
	private ISpatialInfo[] _spatialInfosToMove;
	private ISpatialInfo _spatialInfoToDisplayInSpatialInfoPanel;
	private SelectSpatialMapPanelState _selectState;
	private Point _startPoint;
	
		
	public MoveSpatialInfoSpatialMapPanelState(SelectSpatialMapPanelState select){
		_selectState = select;
		_startPoint = new Point();
		_spatialInfosToMove = new CircularSpatialInfo[0];
	}
	
	public Icon getIcon() {
		return SwingMoreUtilities.getJavaSunIcon(24, "general/Move24.gif");
	}

	public String getName() {
		return "Move";
	}
	
	public void addChangeListener(ChangeListener changeListener) {
		_changeListener = StateChangeMulticaster.add(_changeListener, changeListener);
	}
	
	private void fireStateChanged() {
		if (_changeListener != null) 
			_changeListener.stateChanged(new ChangeEvent(this));
	}
	
	public void mouseExited(MouseEvent e) {
		if(_moving)
			_exited = true;
	}
	
	private void clearMoveMode(){
		//_spatialInfosToMove = new CircularSpatialInfo[0];
		_spatialInfoToDisplayInSpatialInfoPanel = null;
		_moving = false;
		_exited = false;
		getSpatialMapPanel().repaint();	
		
	}

	public void mousePressed(MouseEvent e) {
		LinkedList selectedSpatialInfos;
		
		if (checkIfClickedOnAgentInGivenIterator(_selectState.getSelectedSpatialInfosAsList().iterator(), e.getPoint())){
			selectedSpatialInfos = _selectState.getSelectedSpatialInfosAsList();
		}
		else if(checkIfClickedOnAgentInGivenIterator(getModel().getAllSpatialInfos(), e.getPoint())){
			selectedSpatialInfos = new LinkedList();
			selectedSpatialInfos.add(_spatialInfoToDisplayInSpatialInfoPanel);
			_selectState.clearSelection();
		}
		else{
			_selectState.clearSelection();			
			return;
		}
		
		if (!selectedSpatialInfos.isEmpty()){
		    _oldPositions = new Position2D[selectedSpatialInfos.size()];
		    _spatialInfosToMove = (ISpatialInfo[]) selectedSpatialInfos.toArray(new ISpatialInfo[0]);
		   
		   	for (int i = 0; i < _spatialInfosToMove.length; i++) 
		   		_oldPositions[i] = (Position2D) _spatialInfosToMove[i].getPosition().clone(); 
			
			startMoveMode(e.getPoint());		
		}
	}
	
	public void mouseDragged(MouseEvent e) {
		
		if (_moving){
			Position2D distance = getSpatialMapPanel().convertPanelPosition(
				new Position2D(e.getPoint().x - _startPoint.x, e.getPoint().y -_startPoint.y));
				
			if (!getModel().isDiscrete()){
				for (int i = 0; i < _spatialInfosToMove.length; i++) {			
					_spatialInfosToMove[i].setPosition(Position2D.add(
						new Position2D(_oldPositions[i]), distance));
				}
			}
			else{
				double gridWidth = getModel().getWidth() / getModel().getGridCols();
				double gridHeight = getModel().getHeight() / getModel().getGridRows();
				
				int cellsMovedX = (int) ((distance.x) / gridWidth);
				int cellsMovedY =  (int) ((distance.y) / gridHeight);
				
				distance.setLocation((cellsMovedX * gridWidth), (cellsMovedY * gridHeight));
							
				for (int i = 0; i < _spatialInfosToMove.length; i++) {			
					_spatialInfosToMove[i].setPosition(Position2D.add(new Position2D(_oldPositions[i]), distance));
				}
			}
				
			getSpatialMapPanel().repaint();
			fireStateChanged();
		}		
	}
	
	public void mouseReleased(MouseEvent e) {
		
		if (!_moving){
			_spatialInfosToMove = new ISpatialInfo[0];
			getSpatialMapPanel().repaint();
			return;			
		}
		if (_exited){
			resetToOldPos();
			clearMoveMode();
			return;			
		}
		if (_oldPositions[0].equals(_spatialInfosToMove[0].getPosition())){
			clearMoveMode();
			return;
		}
			

		double width = getModel().getWidth();
		double height = getModel().getHeight();
		
		if(getModel().isTorus()){			
			for (int i = 0; i < _spatialInfosToMove.length; i++) {
				if (_spatialInfosToMove[i].getPosition().x > width || _spatialInfosToMove[i].getPosition().y > height){
					Position2D pos = _spatialInfosToMove[i].getPosition();
					Position2D correctedPos = new Position2D(pos.x % width, pos.y % height);
					_spatialInfosToMove[i].setPosition(correctedPos);
				}
				if (_spatialInfosToMove[i].getPosition().x < 0){
					Position2D pos = _spatialInfosToMove[i].getPosition();
					Position2D correctedPos = new Position2D((pos.x % width) + width, pos.y);
					_spatialInfosToMove[i].setPosition(correctedPos);				
				}
				if (_spatialInfosToMove[i].getPosition().y < 0){
					Position2D pos = _spatialInfosToMove[i].getPosition();
					Position2D correctedPos = new Position2D(pos.x, (pos.y % height) + height);
					_spatialInfosToMove[i].setPosition(correctedPos);
				}
			}
		}
		else{	
			for (int i = 0; i < _spatialInfosToMove.length; i++) {	
				if (_spatialInfosToMove[i].getPosition().x > width || _spatialInfosToMove[i].getPosition().x < 0)
					_spatialInfosToMove[i].setPosition(_oldPositions[i]);
				else if (_spatialInfosToMove[i].getPosition().y > height || _spatialInfosToMove[i].getPosition().y < 0)
					_spatialInfosToMove[i].setPosition(_oldPositions[i]);
				
			}
		}
		
		getModel().fireSpatialInfosMoved(_spatialInfosToMove, _oldPositions);
		clearMoveMode();
	}

	public void setSpatialMapPanel(JSpatialMapPanel spatialMapPanel) {
		if (getSpatialMapPanel() != null) 
			getSpatialMapPanel().removeMouseMotionListener(this);
		
		super.setSpatialMapPanel(spatialMapPanel);

		if (spatialMapPanel != null) 
			spatialMapPanel.addMouseMotionListener(this);
	}
	
	private DefaultSpatialMapModel getModel() {
		return (DefaultSpatialMapModel) getSpatialMapPanel().getSpatialMapModel();
	}
	
	protected ISpatialInfo getSpatialInfoToDisplay(){
		 return _spatialInfoToDisplayInSpatialInfoPanel;
	}
	
	public void mouseEntered(MouseEvent e) {
		_exited = false;
	}

	public void mouseMoved(MouseEvent e) {}

	public void paintComponent(Graphics2D g2d) {
		Color oldColor = g2d.getColor();
		g2d.setColor(Color.blue);
		
		for (int i = 0; i < _spatialInfosToMove.length; i++) {
			ISpatialInfo spInfo = _spatialInfosToMove[i];
			g2d.draw(getSpatialMapPanel().calculateRectangle(spInfo.getPosition().x, spInfo.getPosition().y, spInfo.getRadius()));
		}
		g2d.setColor(oldColor);
	}

	private void startMoveMode(Point point){
		_startPoint.setLocation(point);
		_moving = true;
		fireStateChanged();
	}
	
	private void resetToOldPos(){
		for (int i = 0; i < _spatialInfosToMove.length; i++)			
			_spatialInfosToMove[i].setPosition(_oldPositions[i]);
	}
	
	private boolean checkIfClickedOnAgentInGivenIterator(Iterator it, Point point){
		double minDistance = Double.MAX_VALUE;
						
		while (it.hasNext()){
			ISpatialInfo spatialInfo = (ISpatialInfo) it.next();
						
			if (isClickOnSpatialInfo(spatialInfo, point)){
				double distanceOfActualSpatialInfo = point.distance(spatialInfo.getPosition()); 
				if (distanceOfActualSpatialInfo < minDistance){
					minDistance = distanceOfActualSpatialInfo;
					_spatialInfoToDisplayInSpatialInfoPanel = spatialInfo;
				}								
			}
		}
				
		return _spatialInfoToDisplayInSpatialInfoPanel != null;
	}	

	private boolean isClickOnSpatialInfo(ISpatialInfo spatialInfo, Point point){
		if (getModel().isDiscrete()){
			Rectangle2D rect = getSpatialMapPanel().calculateRectangle(
				spatialInfo.getPosition().x,
				spatialInfo.getPosition().y,
				spatialInfo.getRadius());
			return rect.contains(point);
		}
		else{
			Position2D convertedPos = getSpatialMapPanel().convertPanelPosition(point);
			double radius = Math.max(spatialInfo.getRadius(), 10);	
			return Math.abs(convertedPos.x - spatialInfo.getPosition().x) < radius 
					&& Math.abs(convertedPos.y - spatialInfo.getPosition().y)< radius;
		}
	}
}
