package de.uniwue.ki.gui.mas.basics.feature.spatial;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.jgoodies.forms.extras.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.bound.BoundJDoubleTextField;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.util.Position2D;

/**
 * Creation date: (05.02.2001 12:59:29)
 * @author Christoph Oechslein
 */
public class SpatialInfoPanel extends JPanelWithGUISeSAmDeclaration {

	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			SpatialInfoPanel aSpatialInfoPanel;
			aSpatialInfoPanel = new SpatialInfoPanel();
			frame.setContentPane(aSpatialInfoPanel);
			frame.setSize(aSpatialInfoPanel.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of javax.swing.JPanel");
			exception.printStackTrace(System.out);
		}
	}

	private boolean _automaticUpdate = false;
	private final transient ISpatialMapModelListener _spatialMapModelListener =
		new ISpatialMapModelListener() {

		public void spatialInfoAdded(SpatialMapModelEvent spatialMapModelEvent) {
		}
		public void spatialInfoChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
			if (isAutomaticUpdate() && (spatialMapModelEvent.getSpatialInfo() == getSpatialInfo())) {
				eventSpatialInfoChanged();
			}
		}

		public void spatialInfoRemoved(SpatialMapModelEvent spatialMapModelEvent) {
		}

		public void spatialMapModelChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
		}
		public void spatialInfosMoved(
			ISpatialInfo[] movedSpatialInfos,
			Position2D[] oldPosition) {
		}

	};

	protected BoundJDoubleTextField ivjJTextFieldDirection = null;
	protected BoundJDoubleTextField ivjJTextFieldPosX = null;
	protected BoundJDoubleTextField ivjJTextFieldPosY = null;
	protected BoundJDoubleTextField ivjJTextFieldRadius = null;
	protected BoundJDoubleTextField ivjJTextFieldSpeed = null;
	private ISpatialInfo ivjSpatialInfoIntern = null;
	private ISpatialMapModel ivjSpatialMapModelIntern = null;

	/**
	 * SpatialInfoFeatureInstanceDeclarationPanel constructor comment.
	 */
	public SpatialInfoPanel() {
		super();
		initialize();
	}

	/**
	 * connEtoC2:  (SpatialInfo.this --> SpatialInfoFeatureInstanceDeclarationPanel.spatialInfoChanged(Lde.uniwue.ki.mas.feature.spatial.ISpatialInfo;)V)
	 * @param value de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialInfo
	 */
	private void connEtoC2(ISpatialInfo value) {
		try {
					this.eventSpatialInfoChanged();
				} catch (Throwable ivjExc) {
					handleException(ivjExc);
		}
	}

	public void dispose() {
		if (getSpatialMapModel() != null)
			getSpatialMapModel().removeSpatialMapModelListener(_spatialMapModelListener);

		super.dispose();

	}

	protected void eventSpatialInfoChanged() {
		ISpatialInfo spatialInfo = getSpatialInfo();

		if (spatialInfo == null) {
			setEnabled(false);

		} else {
			setEnabled(true);
			Position2D pos = spatialInfo.getPosition();

			getJTextFieldPosX().setDoubleValue(Math.round(pos.x * 10.0) / 10.0);
			getJTextFieldPosY().setDoubleValue(Math.round(pos.y * 10.0) / 10.0);

			getJTextFieldDirection().setDoubleValue(
				Math.round(Math.toDegrees(spatialInfo.getDirection()) * 10.0) / 10.0);
			getJTextFieldSpeed().setDoubleValue(Math.round(spatialInfo.getSpeed() * 10.0) / 10.0);
			getJTextFieldRadius().setDoubleValue(Math.round(spatialInfo.getRadius() * 10.0) / 10.0);

		}
	}

	/**
	 * Return the JTextFieldDirection property value.
	 * @return BoundJDoubleTextField
	 */
	protected BoundJDoubleTextField getJTextFieldDirection() {
		if (ivjJTextFieldDirection == null) {
			try {
				ivjJTextFieldDirection = new BoundJDoubleTextField();
				ivjJTextFieldDirection.setName("JTextFieldDirection");
				ivjJTextFieldDirection.setEditable(false);
				//ivjJTextFieldDirection.setMinimumSize(new java.awt.Dimension(44, 20));
				ivjJTextFieldDirection.setColumns(3);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldDirection;
	}

	/**
	 * Return the JTextFieldPosX property value.
	 * @return BoundJDoubleTextField
	 */
	protected BoundJDoubleTextField getJTextFieldPosX() {
		if (ivjJTextFieldPosX == null) {
			try {
				ivjJTextFieldPosX = new BoundJDoubleTextField();
				ivjJTextFieldPosX.setName("JTextFieldPosX");
				ivjJTextFieldPosX.setEditable(false);
				//ivjJTextFieldPosX.setMinimumSize(new java.awt.Dimension(44, 20));
				ivjJTextFieldPosX.setColumns(6);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldPosX;
	}

	/**
	 * Return the JTextFieldPosY property value.
	 * @return BoundJDoubleTextField
	 */
	protected BoundJDoubleTextField getJTextFieldPosY() {
		if (ivjJTextFieldPosY == null) {
			try {
				ivjJTextFieldPosY = new BoundJDoubleTextField();
				ivjJTextFieldPosY.setName("JTextFieldPosY");
				ivjJTextFieldPosY.setEditable(false);
				//ivjJTextFieldPosY.setMinimumSize(new java.awt.Dimension(44, 20));
				ivjJTextFieldPosY.setColumns(6);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldPosY;
	}

	/**
	 * Return the JTextFieldRadius property value.
	 * @return BoundJDoubleTextField
	 */
	protected BoundJDoubleTextField getJTextFieldRadius() {
		if (ivjJTextFieldRadius == null) {
			try {
				ivjJTextFieldRadius = new BoundJDoubleTextField();
				ivjJTextFieldRadius.setName("JTextFieldRadius");
				ivjJTextFieldRadius.setEditable(false);
				//ivjJTextFieldRadius.setMinimumSize(new java.awt.Dimension(44, 20));
				ivjJTextFieldRadius.setColumns(3);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldRadius;
	}

	/**
	 * Return the JTextFieldSpeed property value.
	 * @return BoundJDoubleTextField
	 */
	protected BoundJDoubleTextField getJTextFieldSpeed() {
		if (ivjJTextFieldSpeed == null) {
			try {
				ivjJTextFieldSpeed = new BoundJDoubleTextField();
				ivjJTextFieldSpeed.setName("JTextFieldSpeed");
				ivjJTextFieldSpeed.setEditable(false);
				//ivjJTextFieldSpeed.setMinimumSize(new java.awt.Dimension(44, 20));
				ivjJTextFieldSpeed.setColumns(3);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldSpeed;
	}

	/**
	 * Method generated to support the promotion of the spatialInfo attribute.
	 * @return de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialInfo
	 */
	public ISpatialInfo getSpatialInfo() {
		return getSpatialInfoIntern();
	}

	/**
	 * Return the SpatialInfo property value.
	 * @return de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialInfo
	 */
	private ISpatialInfo getSpatialInfoIntern() {
		return ivjSpatialInfoIntern;
	}

	/**
	 * Method generated to support the promotion of the SpatialMapModel attribute.
	 * @return de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialMapModel
	 */
	public ISpatialMapModel getSpatialMapModel() {
		return getSpatialMapModelIntern();
	}

	/**
	 * Return the SpatialMapModel property value.
	 * @return de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialMapModel
	 */
	private ISpatialMapModel getSpatialMapModelIntern() {
		return ivjSpatialMapModelIntern;
	}

	/**
	 * Called whenever the part throws an exception.
	 * @param exception Throwable
	 */
	private void handleException(Throwable exception) {

		exception.printStackTrace(System.out);
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		addComponents();

		updateSpatialMapListener();
	}

	private JPanel _posPanel;

	protected JPanel getPosPanel() {
		if (_posPanel == null) {
			_posPanel = new JPanel();

			DefaultFormBuilder builder =
				new DefaultFormBuilder(
					_posPanel,
					new FormLayout("fill:default, 1dlu, pref, 2dlu, fill:default"));

			builder.append(getJTextFieldPosX());
			builder.append(",", getJTextFieldPosY());
		}

		return _posPanel;
	}

	protected void addComponents() {
		FormLayout formLayout = new FormLayout(getFormLayoutString());

		formLayout.addGroupedColumn(3);
		formLayout.addGroupedColumn(7);

		addComponents(new DefaultFormBuilder(this, formLayout));
	}

	protected String getFormLayoutString() {
		return "right:pref, 1dlu, fill:default, 4dlu " + ", right:pref, 4dlu, fill:default";
	}

	protected void addComponents(DefaultFormBuilder builder) {
		builder.append("Position:", getPosPanel());
		builder.append("Radius:", getJTextFieldRadius());
		builder.nextLine();
		builder.append("Direction:", getJTextFieldDirection());
		builder.append("Speed:", getJTextFieldSpeed());
	}

	public boolean isAutomaticUpdate() {
		return _automaticUpdate;

	}

	public void refresh() {
		eventSpatialInfoChanged();
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		_automaticUpdate = automaticUpdate;
		updateSpatialMapListener();
	}

	public void setEnabled(boolean isEnabled) {
		super.setEnabled(isEnabled);

		getJTextFieldPosX().setEnabled(isEnabled);
		getJTextFieldPosY().setEnabled(isEnabled);
		getJTextFieldSpeed().setEnabled(isEnabled);
		getJTextFieldDirection().setEnabled(isEnabled);
		getJTextFieldRadius().setEnabled(isEnabled);
	}

	/**
	 * Method generated to support the promotion of the spatialInfo attribute.
	 * @param arg1 de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialInfo
	 */
	public void setSpatialInfo(ISpatialInfo arg1) {
		setSpatialInfoIntern(arg1);
	}

	/**
	 * Set the SpatialInfo to a new value.
	 * @param newValue de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialInfo
	 */
	private void setSpatialInfoIntern(ISpatialInfo newValue) {
		if (ivjSpatialInfoIntern != newValue) {
			try {
				ISpatialInfo oldValue = getSpatialInfoIntern();
				ivjSpatialInfoIntern = newValue;
				connEtoC2(ivjSpatialInfoIntern);
				firePropertyChange("spatialInfo", oldValue, newValue);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
	}

	/**
	 * Method generated to support the promotion of the SpatialMapModel attribute.
	 * @param arg1 de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialMapModel
	 */
	public void setSpatialMapModel(ISpatialMapModel arg1) {
		setSpatialMapModelIntern(arg1);
		eventSpatialInfoChanged();
	}

	/**
	 * Set the SpatialMapModelIntern to a new value.
	 * @param newValue de.uniwue.ki.mas.model.basics.feature.spatial.ISpatialMapModel
	 */
	private void setSpatialMapModelIntern(ISpatialMapModel newValue) {
		if (ivjSpatialMapModelIntern != newValue) {
			try {
				ISpatialMapModel oldValue = getSpatialMapModelIntern();
				ivjSpatialMapModelIntern = newValue;
				firePropertyChange("spatialMapModel", oldValue, newValue);
						updateSpatialMapListener(oldValue);
					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
	}

	private void updateSpatialMapListener() {
		if (getSpatialMapModel() != null) {
			getSpatialMapModel().removeSpatialMapModelListener(_spatialMapModelListener);
			if (isAutomaticUpdate()) {
				getSpatialMapModel().addSpatialMapModelListener(_spatialMapModelListener);
			}
		}
	}

	private void updateSpatialMapListener(ISpatialMapModel oldSpatialMapModel) {
		if (oldSpatialMapModel != null)
			oldSpatialMapModel.removeSpatialMapModelListener(_spatialMapModelListener);

		updateSpatialMapListener();
	}
}