package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.util.Position2D;

/**
 * @author Reinhard Hatko
 * 
 * 02.12.2003
 */
public class SpatialMapUndoManager implements ISpatialMapModelListener {

	private UndoManager _undoMan;
	private LinkedList _undoListener;
	private boolean _inProgress;
	
	public SpatialMapUndoManager(){
		_undoMan = new UndoManager();
		_undoListener = new LinkedList();
		_inProgress = false;
	}

	public void spatialInfoChanged(SpatialMapModelChangeEvent e) {
		if(!_inProgress){
			UndoableEdit edit;
			
			if (e.getChangedType() == SpatialMapModelChangeEvent.POS_CHANGED_TYPE){
				edit = new UndoableSpatialInfosMovedEdit(new ISpatialInfo[]{e.getSpatialInfo()}, new Position2D[]{new Position2D(e.getOldPosX(), e.getOldPosY())});
				
				
			}else {
				edit = new UndoableSpatialInfoChangedEdit(e);
			}
			undoableMapEventHappened(edit);

		}
	}

	public void spatialInfosMoved(ISpatialInfo[] movedSpatialInfos,	Position2D[] oldPosition) {
		if(!_inProgress){
			UndoableEdit edit = new UndoableSpatialInfosMovedEdit(movedSpatialInfos, oldPosition);
			undoableMapEventHappened(edit);			
		}
	}



	public void spatialInfoAdded(SpatialMapModelEvent e) {
		if(!_inProgress){
			UndoableEdit edit = new UndoableSpatialInfoAddedEdit(e.getSpatialInfo(), e.getSpatialMapModel());
			undoableMapEventHappened(edit);
		}
	}


	public void spatialInfoRemoved(SpatialMapModelEvent e) {
		if(!_inProgress){
			UndoableEdit edit = new UndoableSpatialInfoRemovedEdit(e.getSpatialInfo(), e.getSpatialMapModel());
			undoableMapEventHappened(edit);
		}
	}


	public void spatialMapModelChanged(SpatialMapModelChangeEvent e) {

	}
	
	private void undoableMapEventHappened(UndoableEdit edit){
		_undoMan.addEdit(edit);
		for (Iterator it = _undoListener.iterator(); it.hasNext();) {
			((UndoableEditListener) it.next()).undoableEditHappened(new UndoableEditEvent(this, edit));
		}
		
	}
	
	public void undo(){
		_inProgress = true;
		_undoMan.undo();
		_inProgress = false;
	}
	
	public void redo(){
		_inProgress = true;		
		_undoMan.redo();
		_inProgress = false;
	}
	
	public boolean canUndo(){
		return _undoMan.canUndo();
	}
	
	public boolean canRedo(){
		return _undoMan.canRedo();	
	}
	
	public void addUndoableEditListener(UndoableEditListener listener){
		_undoListener.add(listener);
	}
	
	public void removeUndoableEditListener(UndoableEditListener listener){
		_undoListener.remove(listener);
	}

	public String getRedoPresentationName() {
		return _undoMan.getRedoPresentationName();
	}

	public String getUndoPresentationName() {
		return _undoMan.getUndoPresentationName();
	}

}
