package de.uniwue.ki.gui.mas.basics.feature.spatial;

import java.awt.Image;
import java.io.File;

import javax.swing.undo.*;

import de.uniwue.ki.mas.basics.feature.spatial.ISpatialInfo;
import de.uniwue.ki.mas.basics.feature.spatial.SpatialMapModelChangeEvent;

/**
 * @author Reinhard Hatko
 * 
 * Created on 07.12.2003
 */
public class UndoableSpatialInfoChangedEdit extends AbstractUndoableEdit {
	private String _undoPresentation;
	private String _redoPresentation;
	private int _changeType;
	private ISpatialInfo _spatialInfo;
	private double _oldValue;
	private double _newValue;
	private Image _oldImage;
	private Image _newImage;
	private File _oldImagePath;
	private File _newImagePath;

	protected UndoableSpatialInfoChangedEdit(SpatialMapModelChangeEvent event) {
		_changeType = event.getChangedType();
		_spatialInfo = event.getSpatialInfo();

		switch (_changeType) {
			case SpatialMapModelChangeEvent.DIRECTION_CHANGED_TYPE :
				_oldValue = event.getOldValue();
				_newValue = _spatialInfo.getDirection();
				_undoPresentation = "Change direction";
				break;

			case SpatialMapModelChangeEvent.DRAWPRIORITY_CHANGED_TYPE :
				_oldValue = event.getOldValue();
				_newValue = _spatialInfo.getDrawPriority();
				_undoPresentation = "Change draw priority";
				break;

			case SpatialMapModelChangeEvent.RADIUS_CHANGED_TYPE :
				_oldValue = event.getOldValue();
				_newValue = _spatialInfo.getRadius();
				_undoPresentation = "Change radius";
				break;
				
			case SpatialMapModelChangeEvent.SPEED_CHANGED_TYPE :
				_oldValue = event.getOldValue();
				_newValue = _spatialInfo.getSpeed();
				_undoPresentation = "Change speed";
				break;
				
			case SpatialMapModelChangeEvent.IMAGE_CHANGED_TYPE :
				_oldImage = event.getOldImage();
				_oldImagePath = event.getOldImagePath();
				_newImage = _spatialInfo.getImage();
				_newImagePath = _spatialInfo.getImagePath();
				_undoPresentation = "Change image";
				break;
		}
	}

	public String getRedoPresentationName() {
		return _redoPresentation;
	}
	
	public String getPresentationName() {
		return _undoPresentation;
	}

	public void undo() throws CannotUndoException {
		super.undo();
		
		switch (_changeType) {
			case SpatialMapModelChangeEvent.DIRECTION_CHANGED_TYPE :
				_spatialInfo.setDirection(_oldValue);
				break;

			case SpatialMapModelChangeEvent.DRAWPRIORITY_CHANGED_TYPE :
				_spatialInfo.setDrawPriority(_oldValue);
				break;

			case SpatialMapModelChangeEvent.RADIUS_CHANGED_TYPE :
				_spatialInfo.setRadius(_oldValue);
				break;
				
			case SpatialMapModelChangeEvent.IMAGE_CHANGED_TYPE :
				_spatialInfo.setImage(_oldImage);
				_spatialInfo.setImagePath(_oldImagePath);
				break;
				
			case SpatialMapModelChangeEvent.SPEED_CHANGED_TYPE :
				_spatialInfo.setSpeed(_oldValue);
				break;
		}
	}
	
	public void redo() throws CannotRedoException {
		super.redo();
		
		switch (_changeType) {
			case SpatialMapModelChangeEvent.DIRECTION_CHANGED_TYPE :
				_spatialInfo.setDirection(_newValue);
				break;

			case SpatialMapModelChangeEvent.DRAWPRIORITY_CHANGED_TYPE :
				_spatialInfo.setDrawPriority(_newValue);
				break;

			case SpatialMapModelChangeEvent.RADIUS_CHANGED_TYPE :
				_spatialInfo.setRadius(_newValue);
				break;
				
			case SpatialMapModelChangeEvent.IMAGE_CHANGED_TYPE :
				_spatialInfo.setImage(_newImage);
				_spatialInfo.setImagePath(_newImagePath);
				break;
				
			case SpatialMapModelChangeEvent.SPEED_CHANGED_TYPE :
				_spatialInfo.setSpeed(_newValue);
				break;
		}
	}
}
