package de.uniwue.ki.gui.mas.basics.functionCall;

import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.AbstractDelegateFunctionDeclarationListAble;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationListAble;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: 31.07.2002, 15:38:00
 * @author oechslein
 */
public class ExpertFilterFunctionDeclarationListAble
	extends AbstractDelegateFunctionDeclarationListAble
	implements IPredicateFunction, IExpertFilterFunctionDeclarationListAble {

	private boolean _expert = false;
	private IObjectClassDeclaration _objectClassDeclaration;
	private ISeSAmModelDeclaration _seSAmModelDeclaration;
	private IPredicateFunction _filterPredicate = null;

	public ExpertFilterFunctionDeclarationListAble() {
		super(null, true);
	}

	public boolean isExpert() {
		return _expert;
	}

	public void setExpert(boolean expert) {
		_expert = expert;

		stateChanged();
	}

	/*
	 */
	public void setAdditionalFilter(IPredicateFunction predicate) {
		_filterPredicate = predicate;

		stateChanged();
	}

	public boolean isElementOk(Object obj) throws Exception {
		// sorted out by the additional filter
		if ((_filterPredicate != null) && (!_filterPredicate.isElementOk(obj)))
			return false;

		if (isExpert())
			return true;
		else
			return !((IFunctionDeclaration)obj).isExpertFunctionDeclaration();
	}

	public Iterator getFunctionDeclarations() {
		if (getDelegateFunctionDeclarationListAble() != null)
			return CollectionsHelper.select(
				getDelegateFunctionDeclarationListAble().getFunctionDeclarations(),
				this);
		else
			return CollectionsHelper.EMPTY_ITERATOR;
	}

	// must return a newly created instance
	protected IFunctionDeclarationListAble calculateFunctionDeclarationList() {
		if (getSeSAmModelDeclaration() == null) {
			return null;

		} else if (getObjectClassDeclaration() != null) {
			return getSeSAmModelDeclaration().getAvailibleFunctionDeclarationListFor(
				getObjectClassDeclaration());

		} else {
			return getSeSAmModelDeclaration().getAvailibleFunctionDeclarationList();
		}
	}

	public ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return _seSAmModelDeclaration;
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	public void setObjectClassDeclaration(IObjectClassDeclaration objectClassDeclaration) {
		if (objectClassDeclaration == _objectClassDeclaration)
			return;

		Assert.assertNotNull(getSeSAmModelDeclaration());

		_objectClassDeclaration = objectClassDeclaration;

		updateDelegate();
	}

	public void setSeSAmModelDeclaration(ISeSAmModelDeclaration seSAmModelDeclaration) {
		if (seSAmModelDeclaration == _seSAmModelDeclaration)
			return;

		_seSAmModelDeclaration = seSAmModelDeclaration;

		if (seSAmModelDeclaration != null)
			updateDelegate();
	}

	protected void updateDelegate() {
		setDelegateFunctionDeclarationListAble(calculateFunctionDeclarationList());
	}

	public ICopyable copy(Map objectMapping) {
		ExpertFilterFunctionDeclarationListAble newExpertFunAble = createInstance();

		copyInto(newExpertFunAble, objectMapping);

		return newExpertFunAble;
	}

	protected ExpertFilterFunctionDeclarationListAble createInstance() {
		Assert.assertSame(ExpertFilterFunctionDeclarationListAble.class, getClass());
		return new ExpertFilterFunctionDeclarationListAble();
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		ExpertFilterFunctionDeclarationListAble dstExpertFunAble =
			(ExpertFilterFunctionDeclarationListAble)dstObject;

		dstExpertFunAble.setExpert(isExpert());
		dstExpertFunAble.setAdditionalFilter(_filterPredicate);
		//dstExpertFunAble.setDelegateFunctionDeclarationListAble(getDelegateFunctionDeclarationListAble());
		dstExpertFunAble.setSeSAmModelDeclaration(getSeSAmModelDeclaration());
		dstExpertFunAble.setObjectClassDeclaration(getObjectClassDeclaration());
	}

	public Iterator getBodyVariableClassDeclarations() {
		if (getObjectClassDeclaration() != null) {
			return getObjectClassDeclaration()
				.getBodyClassDeclaration()
				.getBodyVariableClassDeclarationListAndFactory()
				.getBodyVariableClassDeclarationList()
				.iterator();

		} else {
			return null;
		}
	}

	public Iterator getFeatureVariableClassDeclarations() {
		return null;
	}
}