package de.uniwue.ki.gui.mas.basics.functionCall;
import java.awt.Container;
import java.util.Iterator;

import de.uniwue.ki.gui.mas.model.AbstractDeclarationPanelAndActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.IDeclarationPanelFactory;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.function.rest.FunctionDeclarationRestFactory;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.VoidType;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Rainer Herrler
 */

public final class FunctionCallDeclarationPanelActionFactory
	extends AbstractDeclarationPanelAndActionFactory {

	private static final IAtomicFunctionCallInternDeclaration falseArgument =
		FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(null,BooleanType.getInstance().createArgumentDeclaration(new Boolean(false)));
	private static final IFunctionCallInternDeclaration noopDecl =
		VoidType.getInstance().createDefaultFunctionCallInternDeclaration();
	private static final IGenericFunctionDeclaration ifThenElseDecl =
		FunctionDeclarationRestFactory.IF_THEN_ELSE_GENERIC_FUNCTION_DECLARATION;

	public static boolean isComment(IFunctionCallDeclaration funCallDecl) {
		if ((!funCallDecl.getMostGeneralOutputType().equals(VoidType.getInstance()))
			|| (!(funCallDecl.getFunctionCallInternDeclaration()
				instanceof IComposedFunctionCallInternDeclaration)))
			return false;

		IComposedFunctionCallInternDeclaration cfcid =
			(IComposedFunctionCallInternDeclaration)funCallDecl.getFunctionCallInternDeclaration();

		if ((!(ifThenElseDecl
			.equals(cfcid.getSpecificFunctionDeclaration().getGenericFunctionDeclaration())))
			|| (!falseArgument.isomorphicEquals(cfcid.getChild(0)))
			|| (!noopDecl.isomorphicEquals(cfcid.getChild(2))))
			return false;

		return true;
	}

	class CommentFunctionCallAction extends AbstractBoundAction {
		private IGUISeSAmDeclaration _guiSeSAmDeclaration;
		private Container _parent;
		private IFunctionCallDeclaration _declaration;

		public CommentFunctionCallAction(
			Container parent,
			IDeclaration anDeclaration,
			IGUISeSAmDeclaration guiSeSAmDeclaration) {
			super("Comment", SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif"));
			_guiSeSAmDeclaration = guiSeSAmDeclaration;
			_parent = parent;
			_declaration = (IFunctionCallDeclaration)anDeclaration;
		}

		public boolean isEnabled() {
			return !isComment(_declaration);
		}

		public void doit() {
			IFunctionCallInternDeclaration id = (IFunctionCallInternDeclaration)_declaration.getFunctionCallInternDeclaration().copy(null);
			_declaration.setFunctionCallInternDeclaration(
				FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
					null,
					ifThenElseDecl.createFunctionDeclaration(
						VoidType.getInstance(),
						new DependentTypeBindings()),
					CollectionsHelper.createIterator(
						FunctionCallDeclarationFactory
							.getInstance()
							.createAtomicFunctionCallInternDeclaration(
							null,
							BooleanType.getInstance(),
							new Boolean(false)),
						id,
						noopDecl)));
			_declaration.checkInvariants();
		}
	}

	class UnCommentFunctionCallAction extends AbstractBoundAction {
		private IGUISeSAmDeclaration _guiSeSAmDeclaration;
		private Container _parent;
		private IFunctionCallDeclaration _declaration;

		public UnCommentFunctionCallAction(
			Container parent,
			IDeclaration anDeclaration,
			IGUISeSAmDeclaration guiSeSAmDeclaration) {
			super("Uncomment", SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif"));
			_guiSeSAmDeclaration = guiSeSAmDeclaration;
			_parent = parent;
			_declaration = (IFunctionCallDeclaration)anDeclaration;
		}

		public boolean isEnabled() {
			return isComment(_declaration);
		}

		public void doit() {
			IFunctionCallInternDeclaration newInternDecl = (IFunctionCallInternDeclaration)(
				(IComposedFunctionCallInternDeclaration)_declaration.getFunctionCallInternDeclaration()).getChild(1).copy(null);
			newInternDecl.setParent(null);

			_declaration.setFunctionCallInternDeclaration(newInternDecl);
		}
	}

	public FunctionCallDeclarationPanelActionFactory() {
		super();
	}

	public FunctionCallDeclarationPanelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super();

		setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	protected boolean getCanProduceDeclarationActions(Container parent, IDeclaration anDeclaration) {
		return (anDeclaration instanceof IFunctionCallDeclaration);
	}

	public IDeclarationPanelFactory getDeclarationPanelFactory(
		Container parent,
		IDeclaration anDeclaration) {
		return null;
	}

	protected Iterator getImportantDeclarationActions(Container parent, IDeclaration anDeclaration) {
		return CollectionsHelper.createIterator(
			new CommentFunctionCallAction(parent, anDeclaration, getGUISeSAmDeclaration()),
			new UnCommentFunctionCallAction(parent, anDeclaration, getGUISeSAmDeclaration()));
	}
}