package de.uniwue.ki.gui.mas.basics.functionCall;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JTree;
import javax.swing.tree.TreePath;

import de.uniwue.ki.gui.util.dnd.IDnDListenerAdapter;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;

/**
 * Creation date: (08.05.2001 16:04:50)
 * @author Christoph Oechslein
 */
final class FunctionCallDeclarationTreeListPanelDnDListenerAdapter implements IDnDListenerAdapter {
	private final DropTarget _dropTarget;
	private final DragSource _dragSource;

	private final FunctionCallDeclarationTreeListPanel _functionCallDeclarationTreeListPanel;

	public FunctionCallDeclarationTreeListPanelDnDListenerAdapter(
		FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel,
		JTree tree) {
		this(functionCallDeclarationTreeListPanel, tree, DnDConstants.ACTION_COPY);

	}

	private FunctionCallDeclarationTreeListPanelDnDListenerAdapter(
		FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel,
		JTree tree,
		int defaultDragGestureRecognizerActions) {

		super();

		_dragSource = new DragSource();

		_dragSource.createDefaultDragGestureRecognizer(tree, defaultDragGestureRecognizerActions, this);
		_dropTarget = new DropTarget(tree, this);

		_functionCallDeclarationTreeListPanel = functionCallDeclarationTreeListPanel;
	}

	public void dragDropEnd(DragSourceDropEvent dsde) {
		if (dsde.getDropSuccess()) {
			dragDropEndWithSuccess(dsde);
		}
	}

	private void dragDropEndWithSuccess(DragSourceDropEvent dsde) {
	}

	public void dragEnter(DragSourceDragEvent e) {
		int myaction = e.getDropAction();
		if ((myaction & DnDConstants.ACTION_COPY) != 0) {
			e.getDragSourceContext().setCursor(DragSource.DefaultCopyDrop);

		} else {
			e.getDragSourceContext().setCursor(DragSource.DefaultCopyNoDrop);
		}
	}

	public void dragEnter(DropTargetDragEvent dtde) {
		if (isDropOk(dtde)) {
			dtde.acceptDrag(DnDConstants.ACTION_COPY);

		} else {
			dtde.rejectDrag();
		}
	}

	public void dragExit(DragSourceEvent e) {
	}

	public void dragExit(DropTargetEvent dte) {
	}

	public void dragGestureRecognized(DragGestureEvent dge) {
		IFunctionCallInternDeclaration dragObject = getGestureRecognizedObject(dge);
		if (dragObject != null) {
			Transferable transferable = getTransferable(dge, dragObject);

			getDragSource().startDrag(dge, getDragCursor(dge), transferable, this);
		}
	}

	public void dragOver(DragSourceDragEvent dsde) {
	}

	public void dragOver(DropTargetDragEvent dtde) {
		if (isDropOk(dtde)) {
			dtde.acceptDrag(DnDConstants.ACTION_COPY);
		} else {
			dtde.rejectDrag();
		}
	}

	public void drop(DropTargetDropEvent event) {
		try {
			Transferable transferable = event.getTransferable();

			if (isTransferableOk(event, transferable)) {
				event.acceptDrop(DnDConstants.ACTION_COPY);

				IFunctionCallInternDeclaration object = getFunctionCallInternDeclaration(event, transferable);
				if (object != null) {
					functionCallInternDeclarationDropped(event, object);
					return;
				}
			}

			event.dropComplete(false);

		} catch (IOException exception) {
			exception.printStackTrace();
			System.err.println("Exception" + exception.getMessage());
			event.dropComplete(false);

		} catch (UnsupportedFlavorException ufException) {
			ufException.printStackTrace();
			System.err.println("Exception" + ufException.getMessage());
			event.dropComplete(false);
		}
	}

	public void dropActionChanged(DragSourceDragEvent dsde) {
	}

	public void dropActionChanged(DropTargetDragEvent dtde) {
	}

	private void functionCallInternDeclarationDropped(
		DropTargetDropEvent event,
		IFunctionCallInternDeclaration transferedFunctionCallInternDeclaration) {

		if (transferedFunctionCallInternDeclaration == null) {
			event.dropComplete(false);
		} else {
			IFunctionCallInternDeclaration targetFunctionCallInternDeclaration =
				getCurrentTargetFunctionCallInternDeclaration(event);

			if ((event.getDropAction() & DnDConstants.ACTION_COPY) != 0) {
				functionCallInternDeclarationDroppedCopy(
					event,
					transferedFunctionCallInternDeclaration,
					targetFunctionCallInternDeclaration);

			} else {
				event.dropComplete(false);
			}

		}
	}

	private void functionCallInternDeclarationDroppedCopy(
		DropTargetDropEvent event,
		IFunctionCallInternDeclaration transferedFunctionCallInternDeclaration,
		IFunctionCallInternDeclaration targetFunctionCallInternDeclaration) {

		IFunctionCallDeclaration targetFunctionCallDeclaration = getFunctionCallDeclaration();

		if (!targetFunctionCallInternDeclaration
			.getMostGeneralSpecificOutputType()
			.isMoreSpecific(transferedFunctionCallInternDeclaration.getMostGeneralSpecificOutputType())) {
			event.dropComplete(false);

		} else if (
			transferedFunctionCallInternDeclaration.isomorphicEquals(targetFunctionCallInternDeclaration)) {
			event.dropComplete(true);

		} else {

			IFunctionCallInternDeclaration copyOfTransferedFunctionCallDeclaration =
				(IFunctionCallInternDeclaration)transferedFunctionCallInternDeclaration.copy(new HashMap());

			copyOfTransferedFunctionCallDeclaration.setParent(
				targetFunctionCallInternDeclaration.getParent());

			targetFunctionCallDeclaration.replaceFunctionCallInternDeclarationTo(
				copyOfTransferedFunctionCallDeclaration,
				targetFunctionCallInternDeclaration);

			event.dropComplete(true);
		}
	}

	private IFunctionCallInternDeclaration getCurrentTargetFunctionCallInternDeclaration(DropTargetDragEvent event) {
		return getCurrentTargetFunctionCallInternDeclaration(event.getLocation());
	}

	private IFunctionCallInternDeclaration getCurrentTargetFunctionCallInternDeclaration(DropTargetDropEvent event) {
		return getCurrentTargetFunctionCallInternDeclaration(event.getLocation());
	}

	private IFunctionCallInternDeclaration getCurrentTargetFunctionCallInternDeclaration(
		java.awt.Point dropPoint) {
		TreePath path = getJTree().getPathForLocation(dropPoint.x, dropPoint.y);

		if (path == null) {
			return null;
		} else {

			return (IFunctionCallInternDeclaration)path.getLastPathComponent();
		}
	}

	private java.awt.Cursor getDragCursor(DragGestureEvent dge) {
		if ((DnDConstants.ACTION_MOVE & dge.getDragAction()) != 0) {
			return DragSource.DefaultMoveNoDrop;

		} else if ((DnDConstants.ACTION_COPY & dge.getDragAction()) != 0) {
			return DragSource.DefaultCopyDrop;

		} else
			return DragSource.DefaultCopyNoDrop;
	}

	public final java.awt.dnd.DragSource getDragSource() {
		return _dragSource;
	}

	public final java.awt.dnd.DropTarget getDropTarget() {
		return _dropTarget;
	}

	private final IFunctionCallDeclaration getFunctionCallDeclaration() {
		return getFunctionCallDeclarationTreeListPanel().getFunctionCallDeclaration();
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (04.12.2001 13:57:25)
	 * @return de.uniwue.ki.gui.mas.model.basics.functionCall.FunctionCallDeclarationTreeListPanel
	 */
	private final FunctionCallDeclarationTreeListPanel getFunctionCallDeclarationTreeListPanel() {
		return _functionCallDeclarationTreeListPanel;
	}

	/**
	  * Decapsulate object from transferable, called after isTransferableOk.
	  *
	  * @see #objectDropped(DropTargetDropEvent, Object)
	  * @see #getTransferable(DragGestureEvent, Object)
	  * @see #isTransferableOk(DropTargetDropEvent, Transferable)
	  */
	private IFunctionCallInternDeclaration getFunctionCallInternDeclaration(
		DropTargetDropEvent event,
		Transferable transferable)
		throws IOException, UnsupportedFlavorException {

		return (IFunctionCallInternDeclaration)transferable.getTransferData(
			getFunctionCallInternDeclDataFlavor(getCurrentTargetFunctionCallInternDeclaration(event)));
	}

	private DataFlavor getFunctionCallInternDeclDataFlavor(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		return new FunctionCallPanelTransferHandler.FunctionCallInternDeclarationDataFlavor(
			functionCallInternDeclaration.getMostGeneralSpecificOutputType());
	}

	private IFunctionCallInternDeclaration getGestureRecognizedObject(DragGestureEvent dge) {
		return getCurrentTargetFunctionCallInternDeclaration(dge.getDragOrigin());
	}

	private final JTree getJTree() {
		return (JTree)_dropTarget.getComponent();
	}

	/**
	  * Creates a new Transferable instance which includes object.
	  *
	  * @see #objectDropped(DropTargetDropEvent, Object)
	  * @see #getObject(DropTargetDropEvent, Transferable)
	  * @see #isTransferableOk(DropTargetDropEvent, Transferable)
	  */
	private Transferable getTransferable(DragGestureEvent dge, IFunctionCallInternDeclaration transferable) {
		return new FunctionCallPanelTransferHandler.FunctionCallInternDeclarationTransferable(transferable);
	}

	private boolean isDropOk(DropTargetDragEvent event) {
		return (getCurrentTargetFunctionCallInternDeclaration(event) != null)
			&& (event
				.isDataFlavorSupported(
					getFunctionCallInternDeclDataFlavor(
						getCurrentTargetFunctionCallInternDeclaration(event))))
			&& ((event.getSourceActions() & DnDConstants.ACTION_COPY) != 0);
	}

	/**
	  * Returns true if transferable can be dropped on this component.
	  *
	  * @see #objectDropped(DropTargetDropEvent, Object)
	  * @see #getObject(DropTargetDropEvent, Transferable)
	  * @see #getTransferable(DragGestureEvent, Object)
	  */
	private boolean isTransferableOk(DropTargetDropEvent event, Transferable transferable) {
		return (getCurrentTargetFunctionCallInternDeclaration(event) != null)
			&& transferable.isDataFlavorSupported(
				getFunctionCallInternDeclDataFlavor(getCurrentTargetFunctionCallInternDeclaration(event)));
	}
}