package de.uniwue.ki.gui.mas.basics.functionCall;
import java.awt.Component;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;

/**
 * Creation date: (17.05.00 12:46:35)
 * @author Christoph Oechslein
 */
class FunctionCallInternDeclarationCellRenderer extends DefaultTreeCellRenderer {
	private static final int MAX_SIZE = 100;

	public FunctionCallInternDeclarationCellRenderer() {
		super();
	}

	private static String getInputNameFor(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		IComposedFunctionCallInternDeclaration parent = functionCallInternDeclaration.getParent();
		if (parent == null)
			return null;

		int index = parent.getIndexOfChild(functionCallInternDeclaration);
		if (index == -1)
			return "(wrong parent)";
		else
			return parent.getSpecificFunctionDeclaration().getSpecificInputTypeNameAt(index);
	}

	protected static String getLongNameOf(
		IFunctionCallInternDeclaration functionCallInternDeclaration,
		boolean expanded) {
		String name;
		if (functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration) {
			IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration =
				(IComposedFunctionCallInternDeclaration)functionCallInternDeclaration;

			name = composedFunctionCallInternDeclaration.getName();

		} else {
			name = functionCallInternDeclaration.getName();
		}

		if (name == null)
			return "no name";
		else
			return NamedCellRendererAdapter.insertLineBreaks(name);
	}

	protected static String getNameOf(
		IFunctionCallInternDeclaration functionCallInternDeclaration,
		boolean expanded) {
		String name;
		if (expanded && functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration)
			name =
				((IComposedFunctionCallInternDeclaration)functionCallInternDeclaration)
					.getSpecificFunctionDeclaration()
					.getName();
		else
			name = functionCallInternDeclaration.getName();

		if (name == null)
			return "no name";
		else
			return name;
	}

	public Component getTreeCellRendererComponent(
		JTree tree,
		Object value,
		boolean sel,
		boolean expanded,
		boolean leaf,
		int row,
		boolean cellHasFocus) {

		if (value == null)
			return super.getTreeCellRendererComponent(tree, null, sel, expanded, leaf, row, cellHasFocus);

		else {
			if (!(value instanceof IFunctionCallInternDeclaration)) {
				return super.getTreeCellRendererComponent(
					tree,
					value,
					sel,
					expanded,
					leaf,
					row,
					cellHasFocus);

			} else {

				IFunctionCallInternDeclaration functionCallInternDeclaration =
					(IFunctionCallInternDeclaration)value;

				String name = getNameOf(functionCallInternDeclaration, expanded);

				String prefix = getInputNameFor(functionCallInternDeclaration);

				if (prefix != null) {
					if (functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration)
						name =
							"<html>"
								+ NamedCellRendererAdapter.getQuotedString(name)
								+ " <strong>(= "
								+ NamedCellRendererAdapter.getQuotedString(prefix)
								+ ")</strong>";
					else
						name =
							"<html><strong>"
								+ NamedCellRendererAdapter.getQuotedString(prefix)
								+ ": </strong>"
								+ NamedCellRendererAdapter.getQuotedString(name);
				}

				String shortName =
					(name.length() < MAX_SIZE)
						? name
						: name.substring(0, MAX_SIZE - " ...".length()) + " ...";

				//				if (prefix != null)
				//					shortName += "</html>";

				java.awt.Component comp =
					super.getTreeCellRendererComponent(
						tree,
						shortName,
						sel,
						expanded,
						leaf,
						row,
						cellHasFocus);

				//workaround for swing html bug see Bug Id 4743195 (will not be fixed!!)
				if ((!getTextSelectionColor().equals(getTextNonSelectionColor()))) {
					//					if (sel)
					//						comp.setForeground(getTextNonSelectionColor());
					//better but slower
					if (shortName.startsWith("<html>"))
						firePropertyChange("text", null, shortName);
				}

				if (comp instanceof JComponent) {
					((JComponent)comp).setToolTipText(getLongNameOf(functionCallInternDeclaration, expanded));
				}

				return comp;
			}
		}
	}

	public void setText(String text) {
		super.setText(text);
	}
}