package de.uniwue.ki.gui.mas.basics.functionCall;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.basics.type.dependent.DependentTypeBindingsDialog;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListHelper;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;

/**
 * Creation date: (02.06.00 10:13:51)
 * @author Christoph Oechslein
 */
public class FunctionCallInternDeclarationGUIFactory {

	private FunctionCallInternDeclarationGUIFactory() {
		super();
	}

	public static IComposedFunctionCallInternDeclaration createFunctionCallInternDeclaration(
		IFunctionDeclaration functionDeclaration,
		IFunctionCallInternDeclaration currentFunctionCallInternDecl,
		FunctionCallDeclarationTreeListPanel onComponent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		return createFunctionCallInternDeclaration(
			functionDeclaration,
			currentFunctionCallInternDecl,
			onComponent,
			true,
			guiSeSAmDeclaration);
	}

	public static IComposedFunctionCallInternDeclaration createFunctionCallInternDeclaration(
		IFunctionDeclaration functionDeclaration,
		IFunctionCallInternDeclaration currentFunctionCallInternDecl,
		FunctionCallDeclarationTreeListPanel onComponent,
		boolean shouldReUseOldCall,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		if (functionDeclaration instanceof ISpecificFunctionDeclaration)
			return createFunctionCallInternDeclarationWithSpecificFuncDecl(
				(ISpecificFunctionDeclaration) functionDeclaration,
				currentFunctionCallInternDecl,
				onComponent,
				shouldReUseOldCall,
				guiSeSAmDeclaration);
		else
			return createFunctionCallInternDeclarationWithGenericFuncDecl(
				(IGenericFunctionDeclaration) functionDeclaration,
				currentFunctionCallInternDecl,
				onComponent,
				shouldReUseOldCall,
				guiSeSAmDeclaration);
	}

	public static IComposedFunctionCallInternDeclaration createFunctionCallInternDeclarationWithGenericFuncDecl(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		IFunctionCallInternDeclaration currentFunctionCallInternDecl,
		FunctionCallDeclarationTreeListPanel onComponent,
		boolean shouldReUseOldCall,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		ISpecificType outputSpecificType;

		IComposedFunctionCallInternDeclaration parent = currentFunctionCallInternDecl.getParent();
		if (parent != null) {
			int indexOfChild = parent.getIndexOfChild(currentFunctionCallInternDecl);

			if (indexOfChild != -1) {
				outputSpecificType =
					parent.getSpecificFunctionDeclaration().getSpecificInputTypeAt(indexOfChild);
			} else {
				// shouldn't be the case ...
				outputSpecificType = currentFunctionCallInternDecl.getSpecificOutputType();
			}
		} else {
			outputSpecificType = onComponent.getFunctionCallDeclaration().getMostGeneralOutputType();
		}

		outputSpecificType =
			FunctionDeclarationListHelper.getOutputTypeWithCanCreateFunctionDeclaration(
				genericFunctionDeclaration,
				outputSpecificType);

		if (Assert.ASSERTION_ON) {
			Assert.assertNotNull(outputSpecificType);
			Assert.assertTrue(genericFunctionDeclaration.getName()+" can not return "+outputSpecificType.getName(),
				genericFunctionDeclaration.canCreateFunctionDeclarationWith(guiSeSAmDeclaration.getSeSAmDeclaration(), outputSpecificType));
		}

		DependentTypeBindings bindings =
			DependentTypeBindingsDialog.createDependentTypeBindings(
				genericFunctionDeclaration,
				currentFunctionCallInternDecl.getSpecificOutputType(),
				onComponent,
				getTypes(onComponent),
				guiSeSAmDeclaration);

		if (bindings == null)
			return null;

		else
			return createFunctionCallInternDeclarationWithSpecificFuncDecl(
				genericFunctionDeclaration.createFunctionDeclaration(outputSpecificType, bindings),
				currentFunctionCallInternDecl,
				onComponent,
				shouldReUseOldCall,
				guiSeSAmDeclaration);
	}

	public static IComposedFunctionCallInternDeclaration createFunctionCallInternDeclarationWithSpecificFuncDecl(
		ISpecificFunctionDeclaration functionDeclaration,
		IFunctionCallInternDeclaration currentFunctionCallInternDecl,
		FunctionCallDeclarationTreeListPanel onComponent,
		boolean shouldReUseOldCall,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		Assert.assertNotNull(functionDeclaration);

		if (shouldReUseOldCall
			&& (currentFunctionCallInternDecl instanceof IComposedFunctionCallInternDeclaration)) {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				currentFunctionCallInternDecl.getParent(),
				functionDeclaration,
				(IComposedFunctionCallInternDeclaration) currentFunctionCallInternDecl);

		} else {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				currentFunctionCallInternDecl.getParent(),
				functionDeclaration);
		}
	}

	private static List getTypes(FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel) {
		return functionCallDeclarationTreeListPanel.getTypeList();
	}
}