package de.uniwue.ki.gui.mas.basics.functionCall;

import java.awt.datatransfer.*;
import java.io.IOException;
import java.util.HashMap;

import javax.swing.JComponent;
import javax.swing.TransferHandler;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.dnd.ObjectDataFlavor;
import de.uniwue.ki.gui.util.dnd.ObjectTransferable;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;

/**
 * Creation date: 01.10.2002, 19:23:40
 * @author oechslein
 */
public class FunctionCallPanelTransferHandler extends TransferHandler {
	static final class FunctionCallInternDeclarationDataFlavor extends ObjectDataFlavor {
		private final ISpecificType _specificType;

		protected FunctionCallInternDeclarationDataFlavor(ISpecificType specificType) {
			super(IFunctionCallInternDeclaration.class);

			_specificType = specificType;

			if (Assert.ASSERTION_ON) {
				Assert.assertNotNull(specificType);
				Assert.assertTrue(equals(this));
			}
		}

		public boolean equals(DataFlavor dataFlavor) {
			if (dataFlavor == this)
				return true;

			if (!super.equals(dataFlavor)) {
				return false;
			}

			if (dataFlavor instanceof FunctionCallInternDeclarationDataFlavor) {
				FunctionCallInternDeclarationDataFlavor functionCallInternDeclarationDataFlavor =
					(FunctionCallInternDeclarationDataFlavor)dataFlavor;

				return getSpecificOutputType().isMoreSpecific(
					functionCallInternDeclarationDataFlavor.getSpecificOutputType());
			}

			return false;
		}

		public final ISpecificType getSpecificOutputType() {
			return _specificType;
		}

		private static FunctionCallInternDeclarationDataFlavor getDataFlavor(ISpecificType specificType) {
			if (!hasDataFlavor(specificType)) {
				synchronized (ObjectDataFlavor.class) {
					if (!hasDataFlavor(specificType)) {
						setDataFlavor(
							specificType,
							new FunctionCallInternDeclarationDataFlavor(specificType));
					}
				}
			}

			return (FunctionCallInternDeclarationDataFlavor)objectDataFlavorMap.get(specificType);
		}

		private static boolean hasDataFlavor(ISpecificType specificType) {
			return objectDataFlavorMap.containsKey(specificType);
		}

		private static void setDataFlavor(ISpecificType specificType, ObjectDataFlavor objectDataFlavor) {
			objectDataFlavorMap.put(specificType, objectDataFlavor);
		}
	}

	static final class FunctionCallInternDeclarationTransferable extends ObjectTransferable {
		public FunctionCallInternDeclarationTransferable(IFunctionCallInternDeclaration functionCallInternDeclaration) {
			super(
				FunctionCallInternDeclarationDataFlavor.getDataFlavor(
					getSpecificOutputType(functionCallInternDeclaration)),
				functionCallInternDeclaration.copy(new HashMap()));
		}

		public boolean isDataFlavorSupported(DataFlavor dataFlavor) {
			if (!super.isDataFlavorSupported(dataFlavor)) {
				return false;
			}

			if (dataFlavor instanceof FunctionCallInternDeclarationDataFlavor) {
				FunctionCallInternDeclarationDataFlavor functionCallInternDeclarationDataFlavor =
					(FunctionCallInternDeclarationDataFlavor)dataFlavor;

				ISpecificType transferedSpecificType = getTransferedSpecificType();
				ISpecificType dataSpecificType =
					functionCallInternDeclarationDataFlavor.getSpecificOutputType();

				return dataSpecificType.isMoreSpecific(transferedSpecificType);
			}

			return false;
		}

		protected ISpecificType getTransferedSpecificType() {
			return getSpecificOutputType(getTransferredFunctionCallInternDeclarationWithoutCopy());
		}

		protected Object getTransferredData() {
			return ((IFunctionCallInternDeclaration)super.getTransferredData()).copy(new HashMap());
		}

		private IFunctionCallInternDeclaration getTransferredFunctionCallInternDeclarationWithoutCopy() {
			return (IFunctionCallInternDeclaration)super.getTransferredData();
		}

		public void lostOwnership(Clipboard clipboard, Transferable contents) {
			Assert.assertSame(this, contents);

			lostOwnershipIntern();

			super.lostOwnership(clipboard, this);
		}

		private void lostOwnershipIntern() {
			if (getTransferredFunctionCallInternDeclarationWithoutCopy() != null)
				getTransferredFunctionCallInternDeclarationWithoutCopy().remove();
		}

		protected void finalize() throws Throwable {
			// since the TransferHandler#exportToClipboard do not allow to add a clipboard listener
			lostOwnershipIntern();
			super.finalize();
		}
	}

	public FunctionCallPanelTransferHandler() {
		super();
	}

	protected Transferable createTransferable(JComponent c) {
		while (!(c instanceof FunctionCallDeclarationTreeListPanel)) {
			Assert.assertNotNull(c);
			c = (JComponent)c.getParent();
		}

		FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel =
			(FunctionCallDeclarationTreeListPanel)c;

		IFunctionCallInternDeclaration selectedFunCallInternDecl =
			functionCallDeclarationTreeListPanel.getCurrentSelectedFunctionCallInternDeclaration();

		if (selectedFunCallInternDecl == null)
			return null;
		else
			return new FunctionCallInternDeclarationTransferable(selectedFunCallInternDecl);
	}

	public boolean importData(JComponent c, Transferable t) {
		while (!(c instanceof FunctionCallDeclarationTreeListPanel)) {
			Assert.assertNotNull(c);
			c = (JComponent)c.getParent();
		}

		FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel =
			(FunctionCallDeclarationTreeListPanel)c;

		try {
			IFunctionCallInternDeclaration selectedFunCallInternDecl =
				functionCallDeclarationTreeListPanel.getCurrentSelectedFunctionCallInternDeclaration();

			if (selectedFunCallInternDecl == null)
				return false;

			FunctionCallInternDeclarationDataFlavor funCallDataFlavor =
				FunctionCallInternDeclarationDataFlavor.getDataFlavor(
					getSpecificOutputType(selectedFunCallInternDecl));

			if (!t.isDataFlavorSupported(funCallDataFlavor))
				return false;

			IFunctionCallInternDeclaration transferedFunctionCallDecl =
				(IFunctionCallInternDeclaration)t.getTransferData(funCallDataFlavor);

			return pasteFunctionCallInternDeclaration(
				transferedFunctionCallDecl,
				functionCallDeclarationTreeListPanel);

		} catch (UnsupportedFlavorException e) {
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
	}

	public int getSourceActions(JComponent c) {
		return COPY_OR_MOVE;
	}

	private boolean pasteFunctionCallInternDeclaration(
		IFunctionCallInternDeclaration transferedFunctionCallInternDeclaration,
		FunctionCallDeclarationTreeListPanel functionCallDeclarationTreeListPanel)
		throws IOException, UnsupportedFlavorException {

		IFunctionCallDeclaration functionCallDeclaration =
			functionCallDeclarationTreeListPanel.getFunctionCallDeclaration();
		IFunctionCallInternDeclaration functionCallInternDeclaration =
			functionCallDeclarationTreeListPanel.getCurrentSelectedFunctionCallInternDeclaration();

		if (transferedFunctionCallInternDeclaration != functionCallInternDeclaration) {

			IComposedFunctionCallInternDeclaration parent = functionCallInternDeclaration.getParent();

			if (parent == null) {
				if (TypeManager
					.getInstance()
					.isMoreGeneral(
						functionCallDeclaration.getMostGeneralOutputType(),
						getSpecificOutputType(transferedFunctionCallInternDeclaration))) {

					transferedFunctionCallInternDeclaration.setParent(null);

					functionCallDeclaration.setFunctionCallInternDeclaration(
						transferedFunctionCallInternDeclaration);

					return true;

				} else {
					transferedFunctionCallInternDeclaration.remove();
					return false;
				}

			} else {

				int index = parent.getIndexOfChild(functionCallInternDeclaration);

				if (index != -1) {

					ISpecificFunctionDeclaration specificFunctionDeclaration =
						parent.getSpecificFunctionDeclaration();

					if (TypeManager
						.getInstance()
						.isMoreGeneral(
							specificFunctionDeclaration.getSpecificInputTypeAt(index),
							getSpecificOutputType(transferedFunctionCallInternDeclaration))) {

						transferedFunctionCallInternDeclaration.setParent(parent);

						functionCallDeclaration.replaceFunctionCallInternDeclarationTo(
							transferedFunctionCallInternDeclaration,
							functionCallInternDeclaration);
						return true;

					} else {
						transferedFunctionCallInternDeclaration.remove();
						return false;
					}
				}
			}
		}

		return false;
	}

	protected static ISpecificType getSpecificOutputType(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		IComposedFunctionCallInternDeclaration parent = functionCallInternDeclaration.getParent();
		if (parent != null) {
			int index = parent.getIndexOfChild(functionCallInternDeclaration);
			if (index != -1) {
				return parent.getSpecificFunctionDeclaration().getSpecificInputTypeAt(index);
			}
		}

		return functionCallInternDeclaration.getSpecificOutputType();
	}
}
