package de.uniwue.ki.gui.mas.basics.functionCall;
import java.util.Iterator;

import javax.swing.ListCellRenderer;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListModel;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: (01.12.2001 10:04:39)
 * @author Christoph Oechslein
 */
public class FunctionDeclarationCellRendererAdapter extends NamedCellRendererAdapter {

	private IGUISeSAmDeclaration _guiSeSAmDeclaration;
	private ISpecificType _outputType;

	public FunctionDeclarationCellRendererAdapter(ListCellRenderer listCellRenderer) {
		super(listCellRenderer);
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	protected String getName(INamed value) {
		ISpecificType outputType = (getOutputType() == null) ? DoubleType.getInstance() : getOutputType();

		if (value instanceof IFunctionDeclaration) {
			IFunctionDeclaration functionDeclaration = (IFunctionDeclaration)value;
			String name =
				functionDeclaration.getName()
					+ " "
					+ FunctionDeclarationListModel.getSignatureWithoutNameAndOutputType(
						functionDeclaration,
						outputType);

			if (functionDeclaration instanceof IUserFunctionDeclaration) {
				IUserFeatureClassDeclaration userFeatureClassDeclaration =
					searchUserFeatureClassDeclaration((IUserFunctionDeclaration)functionDeclaration);
				if (userFeatureClassDeclaration == null)
					return name + " <U>";
				else
					return name + " <" + userFeatureClassDeclaration.getName() + ">";
			} else {
				return name;
			}

		} else {
			return super.getName(value);
		}
	}

	public ISpecificType getOutputType() {
		return _outputType;
	}

	private IUserFeatureClassDeclaration searchUserFeatureClassDeclaration(IUserFunctionDeclaration userFunctionDeclaration) {

		Iterator it =
			getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserFeatureClassDeclarationListAndFactory()
				.getDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IUserFeatureClassDeclaration userFeatureClassDeclaration =
				(IUserFeatureClassDeclaration)it.next();

			if (userFeatureClassDeclaration
				.getUserFunctionDeclarationListAndFactory()
				.getDeclarationList()
				.contains(userFunctionDeclaration)) {
				return userFeatureClassDeclaration;
			}
		}

		return null;
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		_guiSeSAmDeclaration = guiSeSAmDeclaration;
	}

	public void setOutputType(ISpecificType newOutputType) {
		_outputType = newOutputType;
	}
}