package de.uniwue.ki.gui.mas.basics.type;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.*;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.NamedTreeCellRenderer;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.util.tree.IObjectTreeNode;

/**
 * Creation date: (30.05.00 14:01:01)
 * @author Christoph Oechslein
 */
public abstract class AbstractTreeTypeEditor extends JPanelWithGUISeSAmDeclaration {

	private IObjectTreeNode _selectedTreeNode;
	private JTree ivjJTree = null;
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private JScrollPane ivjJScrollPane1 = null;

	private final class IvjEventHandler implements javax.swing.event.TreeSelectionListener {
		public void valueChanged(javax.swing.event.TreeSelectionEvent e) {
			if (e.getSource() == AbstractTreeTypeEditor.this.getJTree())
				connEtoC1(e);
		}
	}

	public AbstractTreeTypeEditor() {
		super();
		initialize();
	}

	private void connEtoC1(javax.swing.event.TreeSelectionEvent arg1) {
		try {
			this.eventValueChanged(arg1);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}


	protected final void createAndSetTreeModel() {
		DefaultTreeModel defaultTreeModel = createTreeModel();

		getJTree().setModel(defaultTreeModel);

		int topLevelCount = ((IObjectTreeNode) defaultTreeModel.getRoot()).getChildCount();
		for (int i = topLevelCount - 1; i >= 0; i--) {
			getJTree().expandRow(i);
		}

		IObjectTreeNode leaf = checkIfOnlyOneLeaf((IObjectTreeNode) defaultTreeModel.getRoot());
		if (leaf != null)
			SwingMoreUtilities.setSelectionPath(getJTree(), new TreePath(leaf.getPath()), true);
	}

	protected IObjectTreeNode checkIfOnlyOneLeaf(IObjectTreeNode objectTreeNode) {
		List childs = new LinkedList();
		checkIfOnlyOneLeaf(objectTreeNode, childs);
		if (childs.size() == 1)
			return (IObjectTreeNode) childs.get(0);
		else
			return null;
	}

	protected void checkIfOnlyOneLeaf(IObjectTreeNode objectTreeNode, List childs) {
		if (objectTreeNode.isLeaf()) {
			childs.add(objectTreeNode);

		} else {
			for (int i = 0; i < objectTreeNode.getChildCount(); i++) {
				IObjectTreeNode childTreeNode = (IObjectTreeNode) objectTreeNode.getChildAt(i);
				checkIfOnlyOneLeaf(childTreeNode, childs);
			}
		}
	}

	protected abstract DefaultTreeModel createTreeModel();

	protected abstract boolean isValidForSelection(Object userObject);

	protected final void eventValueChanged(TreeSelectionEvent treeSelectionEvent) {
		if (treeSelectionEvent.getPath() != null) {
			IObjectTreeNode node = (IObjectTreeNode) treeSelectionEvent.getPath().getLastPathComponent();

			if (isValidForSelection(node.getUserObject())) {
				setSelectedTreeNode(node);

			} else if (getSelectedTreeNode() != null) {
				SwingMoreUtilities.setSelectionPath(getJTree(), new TreePath(getSelectedTreeNode().getPath()), true);

			} else {
				getJTree().clearSelection();
			}
		}
	}

	
	private JScrollPane getJScrollPane1() {
		if (ivjJScrollPane1 == null) {
			try {
				ivjJScrollPane1 = new JScrollPane();
				ivjJScrollPane1.setName("JScrollPane1");
				getJScrollPane1().setViewportView(getJTree());
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJScrollPane1;
	}

	private JTree getJTree() {
		if (ivjJTree == null) {
			try {
				javax.swing.tree.DefaultTreeSelectionModel ivjLocalSelectionModel;
				ivjLocalSelectionModel = new DefaultTreeSelectionModel();
				ivjLocalSelectionModel.setRowMapper(new FixedHeightLayoutCache());
				ivjLocalSelectionModel.setSelectionMode(1);
				ivjJTree = new javax.swing.JTree();
				ivjJTree.setName("JTree");
				ivjJTree.setShowsRootHandles(true);
				ivjJTree.setCellRenderer(new NamedTreeCellRenderer());
				ivjJTree.setBounds(0, 0, 29, 54);
				ivjJTree.setRootVisible(false);
				ivjJTree.setSelectionModel(ivjLocalSelectionModel);
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJTree;
	}

	public abstract String getTitle();

	private void handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}

	private void initConnections() {
		getJTree().addTreeSelectionListener(ivjEventHandler);
	}

	private void initialize() {
		try {
			setName("ArgumentEditor");
			setSize(229, 193);
			add(getJScrollPane1(), getJScrollPane1().getName());
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	protected final void setSelectedTreeNodeTo(Object obj) {
		if (Assert.ASSERTION_ON) {
			isValidForSelection(obj);
		}

		getJTree().clearSelection();

		IObjectTreeNode rootNode = (IObjectTreeNode) getJTree().getModel().getRoot();

		setSelectedTreeNodeTo(obj, rootNode);
	}

	protected final boolean setSelectedTreeNodeTo(Object obj, IObjectTreeNode node) {
		if (node.getUserObject() == obj) {
			SwingMoreUtilities.setSelectionPath(getJTree(), new TreePath(node.getPath()), true);
			return true;

		} else {
			for (int i = 0; i < node.getChildCount(); i++) {
				if (setSelectedTreeNodeTo(obj, (IObjectTreeNode) node.getChildAt(i))) {
					return true;
				}
			}

			return false;
		}
	}

	protected void setSelectedTreeNode(IObjectTreeNode selectedTreeNode) {
		_selectedTreeNode = selectedTreeNode;
	}

	protected IObjectTreeNode getSelectedTreeNode() {
		return _selectedTreeNode;
	}
}