package de.uniwue.ki.gui.mas.basics.type;

import java.awt.*;
import java.util.Enumeration;
import java.util.HashMap;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.*;

import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationCellRendererAdapter;
import de.uniwue.ki.mas.basics.search.SearchChildrenHelper;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;

/**
 * @author Reinhard Hatko
 * 
 * 16.01.2004
 */
public class ComposedTypeHierarchicalViewPanel extends JPanel{
	
	private class OwnTreeCellRenderer extends DefaultTreeCellRenderer{
		
		public Component getTreeCellRendererComponent(
			JTree tree,Object value,boolean sel,boolean expanded,boolean leaf,int row,boolean hasFocus) {
		
			super.getTreeCellRendererComponent(tree, value,	sel, expanded, leaf, row, hasFocus);
			String name = ((ISpecificComposedType)((DefaultMutableTreeNode) value).getUserObject()).getName();
			setText(name);
			setToolTipText(name);
			setIcon(UserTypeDeclarationCellRendererAdapter.COMPOSED_ICON);
							
			return this;
		}
	}
	
	private JTree _tree;
	private JTable _table;
	private ISpecificComposedType _comType;
	private ISeSAmDeclaration _sesamDecl;
	private static final String[] _tableHeader = new String[]{"Writeable", "Name", "Type", "Defined in"};
		
	private transient ChangeListener _changeListener = new ChangeListener(){
		public void stateChanged(ChangeEvent e) {
			if (e instanceof ComposedTypeChangeEvent)
			    updateTree();
			
			TreePath path = _tree.getSelectionPath();
			
			if (path != null)
				updateTableContent((DefaultMutableTreeNode) path.getLastPathComponent());
		}
	};

	public ComposedTypeHierarchicalViewPanel(ISpecificComposedType comType, ISeSAmDeclaration sesamDeclaration){
		_comType = comType;
		_sesamDecl = sesamDeclaration;
		
		initialize();
	}
	
	private void initialize(){

	    DefaultMutableTreeNode node = SearchChildrenHelper.buildTree(_comType, _sesamDecl); 
			    
		addListener((DefaultMutableTreeNode) node.getRoot());

		initializeTable();
		initializeTree(node);
		
		setLayout(new GridLayout(1, 1));

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, 
			new JScrollPane(_tree), new JScrollPane(_table)); 
			
		splitPane.setOneTouchExpandable(true);
		
		add(splitPane);

	}
	
	public void dispose(){
	    removeListener((DefaultMutableTreeNode) _tree.getModel().getRoot());
	    
	}
	
	private void updateTableContent(DefaultMutableTreeNode selectedTreeNode){
		
		ISpecificComposedType comType = (ISpecificComposedType) selectedTreeNode.getUserObject();
		
		int size = comType.getAttributeListSize();
		
		Object[][] data = new Object[size][4];
		
		HashMap map = SearchChildrenHelper.getAttributeDefiners(comType);
		
		for (int i = 0; i < size; i++) {
			IComposedTypeAttribute ctAttr = comType.getAttributeAt(i);
			data[i][0] = new Boolean(ctAttr.isWriteable());
			data[i][1] = ctAttr.getAttributeName();
			data[i][2] = ctAttr.getAttributeType().getName();
			data[i][3] = (String) map.get(ctAttr); 
		}
		
		class ModifiedTableModel extends DefaultTableModel{
		    
		    ModifiedTableModel(Object[][] data, Object[] header){
		        super(data, header);
		    }
			public Class getColumnClass(int c) {
				return getValueAt(0, c).getClass();
			}

			public boolean isCellEditable(int row, int column) {
                return false;
            }
		}
		
		_table.setModel(new ModifiedTableModel(data, _tableHeader));
	}
	

	private void initializeTree(DefaultMutableTreeNode node){
		
		_tree = new JTree(node.getRoot());
		_tree.setCellRenderer(new OwnTreeCellRenderer());
		
		_tree.addTreeSelectionListener(
			new TreeSelectionListener() {
				public void valueChanged(TreeSelectionEvent e) {
					
					TreePath path = e.getNewLeadSelectionPath();
					if (path != null)  
						updateTableContent((DefaultMutableTreeNode)	path.getLastPathComponent());
				}
			}
		);
		
		for (int i = 0; i < _tree.getRowCount(); i++) 
			_tree.expandRow(i);

		_tree.getSelectionModel()
			.setSelectionMode(DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
		_tree.setSelectionPath(new TreePath(node.getPath()));
	}

	private void initializeTable(){
		
		_table = new JTable();
		_table.setCellSelectionEnabled(false);
		
	}
	
	private void addListener(DefaultMutableTreeNode node){
		Object userObject = node.getUserObject();
		
		if (userObject instanceof SpecificComposedType) 
            ((SpecificComposedType) userObject).addChangeListener(_changeListener);
		        
		Enumeration enum = node.children();
		
		while (enum.hasMoreElements())
			addListener((DefaultMutableTreeNode) enum.nextElement());

	}
	
	private void removeListener(DefaultMutableTreeNode node){
		Object userObject = node.getUserObject();
		
		if (userObject instanceof SpecificComposedType) 
		    ((SpecificComposedType) userObject).removeChangeListener(_changeListener);

		Enumeration enum = node.children();
		
		while (enum.hasMoreElements())
			removeListener((DefaultMutableTreeNode) enum.nextElement());

	}
	
	private void updateTree(){
	    removeListener((DefaultMutableTreeNode) _tree.getModel().getRoot());
	    
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) SearchChildrenHelper.buildTree(_comType, _sesamDecl);
		
		addListener((DefaultMutableTreeNode) node.getRoot());
		
		_tree.setModel(new DefaultTreeModel(node.getRoot()));
		
		for (int i = 0; i < _tree.getRowCount(); i++) 
			_tree.expandRow(i);
	}
	
		
}