package de.uniwue.ki.gui.mas.basics.type;

import java.util.Iterator;
import java.util.List;

import javax.swing.tree.DefaultTreeModel;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.functions.IPredicateFunction;
import de.uniwue.ki.util.tree.*;

/**
 * Creation date: (30.05.00 14:01:01)
 * @author Christoph Oechslein
 */
public class GenericComposedTypeAttributeTypeDefinePanel
	extends AbstractTreeTypeEditor
	implements IGenericTypeDefinePanel {

	private List _typeList;

	private ISpecificType _specificType;

	private ISpecificType _mostGeneralOutputType = null;

	public GenericComposedTypeAttributeTypeDefinePanel() {
		super();
	}

	protected void addUserDefinedTypesToTreeNode(DefaultObjectTreeNode rootNode, Iterator userDefinedTypes) {
		addComposedTypesToTreeNode(rootNode, getComposedSpecificTypes(userDefinedTypes));
	}

	private static Iterator getComposedSpecificTypes(Iterator userDefinedTypes) {
		return CollectionsHelper.map(CollectionsHelper.select(userDefinedTypes, new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				return ((IUserTypeDeclaration)obj).isComposedType();
			}
		}), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				return ((IUserTypeDeclaration)obj).getSpecificComposedType();
			}
		});
	}

	// should replace addUserDefinedTypesToTreeNode
	protected void addComposedTypesToTreeNode(DefaultObjectTreeNode rootNode, Iterator composedTypes) {
		while (composedTypes.hasNext()) {
			ISpecificComposedType composedType = (ISpecificComposedType)composedTypes.next();

			DefaultObjectTreeNode composedTypeNode = new DefaultObjectTreeNode(rootNode, composedType);

			addComposedTypeClassesToTreeNode(composedTypeNode, composedType);

			if (composedTypeNode.getChildCount() > 0)
				rootNode.addChild(composedTypeNode);
		}
	}

	protected void addComposedTypeClassesToTreeNode(
		DefaultObjectTreeNode parentNode,
		ISpecificComposedType composedType) {

		Iterator composedTypeAttributes = composedType.getOwnAttributes();

		while (composedTypeAttributes.hasNext()) {
			IComposedTypeAttribute composedTypeAttribute =
				(IComposedTypeAttribute)composedTypeAttributes.next();

			if (composedTypeAttribute.isWriteable()
				&& ((_mostGeneralOutputType.equals(VoidType.getInstance()))
				//this happens when setComposedTypeValue opens the dialog
					|| _mostGeneralOutputType == null)) {
				addNode(parentNode, composedTypeAttribute);
			} else
				//if ((SpecificComposedTypeAttributeType)getSpecificTypeIntern()).isWriteable()
				if (TypeManager
					.getInstance()
					.isMoreGeneral(_mostGeneralOutputType, composedTypeAttribute.getAttributeType())) {
					addNode(parentNode, composedTypeAttribute);
				}
		}
	}

	private void addNode(DefaultObjectTreeNode parentNode, IComposedTypeAttribute composedTypeAttribute) {
		DefaultObjectTreeNodeLeaf composedTypeAttributeNode =
			new DefaultObjectTreeNodeLeaf(parentNode, composedTypeAttribute);

		parentNode.addChild(composedTypeAttributeNode);
	}

	protected DefaultTreeModel createTreeModel() {
		DefaultObjectTreeNode rootNode = new DefaultObjectTreeNode(null, "root");

		addComposedTypesToTreeNode(
			rootNode,
			TypeManager.getInstance().getAllComposedTypes(getGUISeSAmDeclaration().getSeSAmDeclaration()));

		return new DefaultTreeModel(rootNode);
	}

	public ISpecificType getSpecificType() {
		if ((getSelectedTreeNode() != null)
			&& (getSelectedTreeNode().getUserObject() instanceof IComposedTypeAttribute)) {
			return (
				(GenericComposedTypeAttributeType)getSpecificTypeIntern()
					.getGenericType())
					.createSpecificComposedTypeAttributeType(
				(IComposedTypeAttribute)getSelectedTreeNode().getUserObject(),
				(ISpecificComposedType) ((IObjectTreeNode)getSelectedTreeNode().getParent()).getUserObject());
		} else {
			return getSpecificTypeIntern();
		}
	}

	protected ISpecificType getSpecificTypeIntern() {
		return _specificType;
	}

	public String getTitle() {
		return "Choice User Type Attribute";
	}

	public void setSpecificType(ISpecificType specificType) {
		_specificType = specificType;

		createAndSetTreeModel();

		Assert.assertTrue(specificType instanceof SpecificComposedTypeAttributeType);

		SpecificComposedTypeAttributeType specificIComposedTypeAttributeType =
			(SpecificComposedTypeAttributeType)specificType;

		if ((_mostGeneralOutputType != null) && (!_mostGeneralOutputType.equals(VoidType.getInstance()))) {
			if (!(specificIComposedTypeAttributeType
				.getComposedTypeAttribute()
				.getAttributeType()
				.isMoreSpecific(_mostGeneralOutputType)))
				_specificType =
					GenericComposedTypeAttributeType.createDefaultSpecificType(
						getGUISeSAmDeclaration().getSeSAmDeclaration(),
						_mostGeneralOutputType);
		}

		if (specificIComposedTypeAttributeType.getComposedType() != null) {
			setSelectedTreeNodeTo(specificIComposedTypeAttributeType.getComposedTypeAttribute());
		}

		revalidate();
	}

	public void setDefaultSpecificType() {
		setSpecificType(
			GenericComposedTypeAttributeType.createDefaultSpecificType(
				getGUISeSAmDeclaration().getSeSAmDeclaration(),
				_mostGeneralOutputType));
	}

	public List getTypeList() {
		return _typeList;
	}

	public void setTypeList(List typesList) {
		_typeList = typesList;
	}

	protected boolean isValidForSelection(Object userObject) {
		return true;
	}

	// sets the most general outputType for the attributes
	public void setMostGeneralAttributeOutputType(ISpecificType outputType) {
		_mostGeneralOutputType = outputType;
	}

}