package de.uniwue.ki.gui.mas.basics.type;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;

import junit.framework.Assert;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationComboBox;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclarationList;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/**
 * Creation date: (10.07.2001 10:58:46)
 * @author Christoph Oechslein
 */
final class GenericSimObjectTypeDefinePanel
	extends JPanelWithGUISeSAmDeclaration
	implements IGenericTypeDefinePanel {

	private final ButtonGroup _buttonGroup = new ButtonGroup();
	private DeclarationComboBox _comboBoxFeatureClassDecl;
	private DeclarationComboBox _comboBoxObjectClassDecl;
	private JRadioButton _radioButtonAll;
	private JRadioButton _radioButtonFeatureClassDecl;
	private JRadioButton _radioButtonObjectClassDecl;

	private List _typeList;

	public GenericSimObjectTypeDefinePanel() {
		super();
		initialize();
	}

	private DeclarationComboBox getComboBoxFeatureClassDecl() {
		if (_comboBoxFeatureClassDecl == null) {
			_comboBoxFeatureClassDecl = new DeclarationComboBox();
			_comboBoxFeatureClassDecl.setEditable(false);
			_comboBoxFeatureClassDecl.setEnabled(false);
		}
		return _comboBoxFeatureClassDecl;
	}

	private DeclarationComboBox getComboBoxObjectClassDecl() {
		if (_comboBoxObjectClassDecl == null) {
			_comboBoxObjectClassDecl = new DeclarationComboBox();
			_comboBoxObjectClassDecl.setEnabled(false);
			_comboBoxObjectClassDecl.setEditable(false);
		}
		return _comboBoxObjectClassDecl;
	}

	private JRadioButton getRadioButtonAll() {
		if (_radioButtonAll == null) {
			_radioButtonAll = new JRadioButton(new AbstractBoundAction("All") {
				public void doit() {
					getComboBoxFeatureClassDecl().setEnabled(!getRadioButtonAll().isEnabled());
					getComboBoxObjectClassDecl().setEnabled(!getRadioButtonAll().isEnabled());
				}
			});
			_buttonGroup.add(_radioButtonAll);
			_radioButtonAll.setSelected(true);
		}
		return _radioButtonAll;
	}

	private JRadioButton getRadioButtonFeatureClassDecl() {
		if (_radioButtonFeatureClassDecl == null) {
			_radioButtonFeatureClassDecl = new JRadioButton(new AbstractBoundAction("Features") {
				public void doit() {
					if (getComboBoxFeatureClassDecl().getDeclarationListModel().isEmpty()) {
						getRadioButtonAll().setSelected(true);
					} else {
						getComboBoxFeatureClassDecl().setEnabled(
							getRadioButtonFeatureClassDecl().isEnabled());
						getComboBoxObjectClassDecl().setEnabled(
							!getRadioButtonFeatureClassDecl().isEnabled());
					}
				}
			});
			_buttonGroup.add(_radioButtonFeatureClassDecl);
		}
		return _radioButtonFeatureClassDecl;
	}

	private JRadioButton getRadioButtonObjectClassDecl() {
		if (_radioButtonObjectClassDecl == null) {
			_radioButtonObjectClassDecl = new JRadioButton(new AbstractBoundAction("ObjectClass") {
				public void doit() {
					if (getComboBoxObjectClassDecl().getDeclarationListModel().isEmpty()) {
						getRadioButtonAll().setSelected(true);
					} else {
						getComboBoxFeatureClassDecl().setEnabled(
							!getRadioButtonObjectClassDecl().isEnabled());
						getComboBoxObjectClassDecl().setEnabled(getRadioButtonObjectClassDecl().isEnabled());
					}
				}
			});
			_buttonGroup.add(_radioButtonObjectClassDecl);
		}
		return _radioButtonObjectClassDecl;
	}

	public ISpecificType getSpecificType() {
		if (getRadioButtonAll().isSelected()) {
			return GenericSimObjectType.getGeneralSimObjectType();
		} else if (getRadioButtonObjectClassDecl().isSelected()) {
			IObjectClassDeclaration objectClassDecl =
				(IObjectClassDeclaration)getComboBoxObjectClassDecl().getSelectedItem();
			if (objectClassDecl == null)
				return null;
			else
				return GenericSimObjectType.getSimObjectType(objectClassDecl);
		} else {
			Assert.assertTrue(getRadioButtonFeatureClassDecl().isSelected());
			IUserFeatureClassDeclaration featureClassDecl =
				(IUserFeatureClassDeclaration)getComboBoxFeatureClassDecl().getSelectedItem();
			if (featureClassDecl == null)
				return null;
			else
				return GenericSimObjectType.getSimObjectType(featureClassDecl);
		}
	}

	public List getTypeList() {
		return _typeList;
	}

	private void initialize() {
		FormLayout formLayout =
			new FormLayout("default, 7dlu, min(60dlu;default)", "default, 3dlu, default, 3dlu, default");
		setLayout(formLayout);

		formLayout.addGroupedRow(1);
		formLayout.addGroupedRow(3);
		formLayout.addGroupedRow(5);

		CellConstraints cc = new CellConstraints();
		add(getRadioButtonAll(), cc.xy(1, 1));

		add(getRadioButtonObjectClassDecl(), cc.xy(1, 3));
		add(getComboBoxObjectClassDecl(), cc.xy(3, 3));

		add(getRadioButtonFeatureClassDecl(), cc.xy(1, 5));
		add(getComboBoxFeatureClassDecl(), cc.xy(3, 5));
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null) {
			ISeSAmModelDeclaration seSAmModelDeclaration = guiSeSAmDeclaration.getSeSAmModelDeclaration();

			IObjectClassDeclarationList objectClassDeclarationList =
				seSAmModelDeclaration.getAvailibleObjectClassDeclarationList();
			getComboBoxObjectClassDecl().setDeclarationListModel(objectClassDeclarationList);
			getRadioButtonObjectClassDecl().setEnabled(!objectClassDeclarationList.isEmpty());

			IDeclarationList userFeatureClassDeclarationList =
				seSAmModelDeclaration
					.getSeSAmModelBasicsDeclaration()
					.getUserFeatureClassDeclarationListAndFactory()
					.getDeclarationList();
			getComboBoxFeatureClassDecl().setDeclarationListModel(userFeatureClassDeclarationList);
			getRadioButtonFeatureClassDecl().setEnabled(!userFeatureClassDeclarationList.isEmpty());

			revalidate();
		}
	}

	public void setSpecificType(ISpecificType specificType) {
		ISimObjectType newSimObjectType = (ISimObjectType)specificType;
		if (newSimObjectType.getFeatureClassDeclaration() != null) {
			getRadioButtonFeatureClassDecl().setSelected(true);
			getComboBoxFeatureClassDecl().setSelectedItem(newSimObjectType.getFeatureClassDeclaration());
			getComboBoxFeatureClassDecl().setEnabled(true);

		} else if (newSimObjectType.getObjectClassDeclaration() != null) {
			getRadioButtonObjectClassDecl().setSelected(true);
			getComboBoxObjectClassDecl().setSelectedItem(newSimObjectType.getObjectClassDeclaration());
			getComboBoxObjectClassDecl().setEnabled(true);

		} else {
			getRadioButtonAll().setSelected(true);
		}
	}

	public void setTypeList(List typeList) {
		_typeList = typeList;
	}

	public void dispose() {
		getComboBoxFeatureClassDecl().dispose();
		getComboBoxObjectClassDecl().dispose();

		super.dispose();
	}
}