package de.uniwue.ki.gui.mas.basics.type;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.ArrayList;

import javax.swing.*;
import javax.swing.event.*;

import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.bound.BestBoundJTextField;
import de.uniwue.ki.gui.util.bound.BetterBoundJList;
import de.uniwue.ki.gui.util.event.KeyAndMouseAdapter;
import de.uniwue.ki.util.event.ListDataListenerAdapter;
import de.uniwue.ki.util.functions.IPredicateFunction;
import de.uniwue.ki.util.list.MuteableFilerableListModel;
import de.uniwue.ki.util.name.INamed;

/**
 * @author Reinhard Hatko
 * 
 * Created on: 07.05.2004
 */
public class JListWithFastTypeField extends JComponent {

	private BetterBoundJList _list;
	private BestBoundJTextField _textField;
	private JScrollPane _listScrollPane;
	private boolean _clearSelectionAfterAction = true;
	
	private EventHandler _eventHandler;
	private FilterFunction _filterFunction;
	
	private ArrayList _registeredActionListeners;
	
	private ListDataListener _listDataListener = new ListDataListenerAdapter() {
        public void contentsChanged(ListDataEvent e) {
            if (getList().getModel().getSize() < getList().getSelectedIndex())
                getList().clearSelection();
                
                if(getList().getModel().getSize() > 0)
                    getList().setSelectedIndex(0);
                
        }
    };
	
	private class EventHandler extends KeyAndMouseAdapter{
		
		public void keyPressed(KeyEvent e) {
			
			if (e.getSource() == getTextField()) {
				if (e.getKeyCode() == KeyEvent.VK_UP || e.getKeyCode() == KeyEvent.VK_KP_UP)
					moveOneUpInList();
				else if (e.getKeyCode() == KeyEvent.VK_DOWN || e.getKeyCode() == KeyEvent.VK_KP_DOWN) 
					moveOneDownInList();
				else if (e.getKeyCode() == KeyEvent.VK_HOME){
				    moveToFirstInList();
				    e.consume();
				}
				else if (e.getKeyCode() == KeyEvent.VK_END){
				    moveToLastInList();
				    e.consume();
				} 
				else if (e.getKeyCode() == KeyEvent.VK_ENTER)
				    fireActionPerformed();
				
			} else if (e.getSource() == getList()) {
				int type = Character.getType(e.getKeyChar());
				
				if (Character.isLetterOrDigit(e.getKeyChar())
					|| (type == Character.MATH_SYMBOL)
					|| (type == Character.OTHER_PUNCTUATION)
					|| (type == Character.CURRENCY_SYMBOL)
					|| (type == Character.START_PUNCTUATION)
					|| (type == Character.DASH_PUNCTUATION)) 
					getTextField().replaceSelection(Character.toString(e.getKeyChar()));					
				else if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE)
				    removeLastChar();
				else if (e.getKeyCode() == KeyEvent.VK_ENTER)
				    fireActionPerformed();
			}
		}
		
		public void mouseClicked(MouseEvent e) {
		    if(e.getClickCount() > 1)
		        fireActionPerformed();
        }
		
		
	}
	
	private class FilterFunction implements IPredicateFunction{
		private String _filterString;
		
		FilterFunction(String filter){
			_filterString = filter;
		}

		public boolean isElementOk(Object obj) throws Exception {
			if (!(obj instanceof INamed))
				return false;
			String name = ((INamed)obj).getName();
		
			return name.toLowerCase().startsWith(
				getTextField().getText().toLowerCase());
		}
		
		public String getFilterString() {
			return _filterString;
		}

		public void setFilterString(String filterString) {
			_filterString = filterString;
		}
	}
	

	public JListWithFastTypeField() {
		initialize();
	}
	
	private void initialize() {
		setLayout(new BorderLayout());
		_eventHandler = new EventHandler();
		_filterFunction = new FilterFunction("");
		setFilterFunction(_filterFunction);
		add(new JScrollPane(getList()), BorderLayout.CENTER);
		add(getTextField(), BorderLayout.SOUTH);
		
		/*
addActionListener(new ActionListener() {
    public void actionPerformed(ActionEvent e) {
   System.out.println("Actionperformed " + ((INamed) getSelectedValue()).getName()); 
    }
});
		*/
		
	}
	
	
	protected BetterBoundJList getList() {
		if (_list == null) {
			_list = new BetterBoundJList();
			// remove?
			_list.setCellRenderer(
				new NamedCellRendererAdapter(_list.getCellRenderer()));
			//
			_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			_list.setModel(new MuteableFilerableListModel());
			_list.addKeyListener(_eventHandler);
			_list.addMouseListener(_eventHandler);
			_list.addListDataListener(_listDataListener);
			/*			
			for (int i = 0; i < objs.length; i++) 
				((MuteableFilerableListModel)_list.getModel()).add(objs[i]);			
				*/
		}
		return _list;
		
	}
	protected BestBoundJTextField getTextField() {
		if (_textField == null) {
			_textField = new BestBoundJTextField();
			_textField.addKeyListener(_eventHandler);
			
			_textField.getDocument().addDocumentListener(new DocumentListener() {
                public void changedUpdate(DocumentEvent e) {}
                
                public void insertUpdate(DocumentEvent e) {
                    SwingUtilities.invokeLater(
                        new Runnable() {
                            public void run() {
                                ((MuteableFilerableListModel) getList().getModel()).updateFilteredList();
                                if (getList().getModel().getSize() == 0) 
                                    removeLastChar();
                                
                                
                            }
                        }
                    );
      
                }

                public void removeUpdate(DocumentEvent e) {
                    insertUpdate(e);
                }
            });
			
			
		}
		return _textField;
	}
	
	public MuteableFilerableListModel getModel() {
		return (MuteableFilerableListModel) _list.getModel();
	}

	public void setModel(MuteableFilerableListModel listmodel) {
		_list.setModel(listmodel);
	}
	
	protected void moveOneDownInList() {
		int selectedIndex = getList().getSelectedIndex();
		int size = getList().getModel().getSize();
		
		if (size == 0 || selectedIndex == size - 1)
		    return;		    
		else if (selectedIndex == -1)
		    selectedIndex = 0;
		else if (size > selectedIndex + 1)
			selectedIndex++;
		else
			selectedIndex = 0;
			
		selectAndScrollToIndex(selectedIndex);
		
	}
	
	protected void moveOneUpInList() {
		int selectedIndex = getList().getSelectedIndex();
		int size = getList().getModel().getSize();

		if (size == 0 || selectedIndex == 0)
		    return;
		else if (selectedIndex == -1)
		    selectedIndex = size - 1;
		else if (selectedIndex > 0)
			selectedIndex--;
		else
			selectedIndex = 0;
		
		selectAndScrollToIndex(selectedIndex);		
	}
	
	protected void moveToFirstInList(){
	    if (getList().getModel().getSize() > 0)
	        selectAndScrollToIndex(0);	  
	}
	
	protected void moveToLastInList(){
	    int size = getList().getModel().getSize();
	    if (size > 0)
	        selectAndScrollToIndex(size - 1);
	    
	}
	
	private void selectAndScrollToIndex(int index){
		getList().setSelectedIndex(index);
		getList().ensureIndexIsVisible(index);
	}
	
	public void setFilterFunction(IPredicateFunction predicateFunction) {
		((MuteableFilerableListModel)getList().getModel()).setFilter(predicateFunction);		
	}	
	

	public void addActionListener(ActionListener listener){
	    if (_registeredActionListeners == null)
	        _registeredActionListeners = new ArrayList();
	    
	    _registeredActionListeners.add(listener);
	    
	}
	
    public boolean isClearSelectionAfterAction() {
        return _clearSelectionAfterAction;
    }

    public void setClearSelectionAfterAction(boolean clearSelectionAfterAction) {
        _clearSelectionAfterAction = clearSelectionAfterAction;
    }
	
	public void removeActionListener(ActionListener listener){
	    if (_registeredActionListeners == null)
	        return;
	    
	    _registeredActionListeners.remove(listener);
	}
	
	private void removeLastChar(){
	    String content = getTextField().getText();
	    if (content.length() > 0)
	        getTextField().setText(
                content.substring(0, content.length() - 1));
	}
	
	protected void fireActionPerformed(){
        if (getSelectedValue() == null || _registeredActionListeners == null) 
            return;
          
        for (int i = 0, size = _registeredActionListeners.size(); i < size; i++){
            ((ActionListener) _registeredActionListeners.get(i)).actionPerformed(new ActionEvent(JListWithFastTypeField.this, ActionEvent.ACTION_PERFORMED, "Object selected"));
        }
        
        if (_clearSelectionAfterAction)
            getTextField().setText("");
        
	}
	
	// Delegate methods
	
	public void addListSelectionListener(ListSelectionListener listener) {
		_list.addListSelectionListener(listener);
	}
		
	public void removeListSelectionListener(ListSelectionListener listener) {
		_list.removeListSelectionListener(listener);
	}
	
	public int getSelectedIndex() {
		return _list.getSelectedIndex();
	}

	public Object getSelectedValue() {
		return _list.getSelectedValue();
	}
	
    public void setSelectedIndex(int index) {
        _list.setSelectedIndex(index);
    }
    
    public void setSelectedValue(Object anObject, boolean shouldScroll) {
        _list.setSelectedValue(anObject, shouldScroll);
    }
    
    public void setSelectedValue(Object newObject) {
        _list.setSelectedValue(newObject);
    }
    
    public ListCellRenderer getCellRenderer() {
        return _list.getCellRenderer();
    }
    
    public void setCellRenderer(ListCellRenderer cellRenderer) {
        _list.setCellRenderer(cellRenderer);
    }
    
    
    
	
	
	
	
	
	
	
	
	
	
	
	

/*
	
	Object[] objs = new Object[] {
			new INamed() {
				public String getName() {
					return "a";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "as";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "asd";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "asdf";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "asdfg";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "y";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "yx";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "yxc";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "yxcv";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "yxcvb";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "q";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "qw";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "qwe";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "qwer";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 
			new INamed() {
				public String getName() {
					return "qwert";
				}

				public void appendName(StringBuffer buff, int currDepth,
						int maxDepth) {
				}
			}, 

			
	};

	
*/
}
