package de.uniwue.ki.gui.mas.basics.type;
import java.awt.event.*;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JOptionPane;

import junit.framework.Assert;

import com.ibm.easyswing.BoundJComboBox;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.*;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.GenericComposedType;
import de.uniwue.ki.mas.basics.type.enum.GenericEnumType;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.basics.type.user.UserTypeDeclarationListAndFactory;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (19.12.2001 14:53:31)
 * @author Christoph Oechslein
 */
public class SpecificTypeDefinePanel extends JPanelWithGUISeSAmDeclaration implements ITypeDefinePanel {
	private boolean _inSetSpecificType = false;
	private List _typeList;
	private ISpecificType _specificTypeIntern = null;
	private BoundJComboBox _boundJComboBoxTypes = null;
	private JButton _jButtonSettings = null;

	private final IvjEventHandler _eventHandler = new IvjEventHandler();

	private final class IvjEventHandler implements ActionListener, ItemListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == SpecificTypeDefinePanel.this.getJButtonSettings()) {
				eventMoreSettingsButtonPressed();
			}
		}
		public void itemStateChanged(ItemEvent e) {
			if (e.getSource() == SpecificTypeDefinePanel.this.getBoundJComboBoxTypes()) {
				eventSelectionChanged();
			}
		}
	};

	public SpecificTypeDefinePanel() {
		super();
		initialize();
	}

	protected void eventMoreSettingsButtonPressed() {
		IType selectedType = (IType)getBoundJComboBoxTypes().getSelectedItem();

		if (selectedType != null) {

			IGenericTypeDefinePanel genericTypeDefinePanel =
				GenericArgumentDefinePanelFactory.createGenericTypeDefinePanelFor(
					(IGenericType)selectedType,
					getGUISeSAmDeclaration(),
					getTypeList());

			if (genericTypeDefinePanel == null)
				return;

			genericTypeDefinePanel.setSpecificType(getSpecificType());

			JOptionPane optionPane =
				new JOptionPane(
					genericTypeDefinePanel,
					JOptionPane.QUESTION_MESSAGE,
					JOptionPane.OK_CANCEL_OPTION);

			GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
				this,
				"More Settings for " + selectedType.getName(),
				optionPane);

			if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
				ISpecificType specType = genericTypeDefinePanel.getSpecificType();
				if (specType != null)
			    	setSpecificType(genericTypeDefinePanel.getSpecificType());
			}
		}
	}

	protected void eventSelectionChanged() {
		IType selectedType = (IType)getBoundJComboBoxTypes().getSelectedItem();

		getJButtonSettings().setEnabled(isEnabled() && hasMoreSettings(selectedType));

		if (!_inSetSpecificType) {

			if (selectedType instanceof ISpecificType) {
				setSpecificType((ISpecificType)selectedType);

			} else {
				IGenericType selectedGenericType = (IGenericType)selectedType;

				if ((getGUISeSAmDeclaration() == null) || (selectedGenericType == null)) {
					setSpecificType(null);

				} else if (
					((selectedGenericType == GenericEnumType.getInstance())
						|| (selectedType == GenericComposedType.getInstance()))) {

					boolean composed = (selectedType == GenericComposedType.getInstance());

					UserTypeDeclarationListAndFactory userTypeListAndFactory =
						getGUISeSAmDeclaration()
							.getSeSAmModelDeclaration()
							.getSeSAmModelBasicsDeclaration()
							.getUserTypeDeclarationListAndFactory();

					ISpecificType specificType;

					Iterator it =
						(composed)
							? TypeManager.getInstance().getAllComposedTypes(
								getGUISeSAmDeclaration().getSeSAmDeclaration())
							: TypeManager.getInstance().getAllEnumTypes(
								getGUISeSAmDeclaration().getSeSAmDeclaration());
					if (it.hasNext()) {
						specificType = (ISpecificType)it.next();
					} else {
						IUserTypeDeclaration newUserDefinedTypeDeclaration =
							userTypeListAndFactory.getUserTypeCreateIfEmpty(composed);
						specificType = newUserDefinedTypeDeclaration.getSpecificType();
					}
					setSpecificType(specificType);
				} else {
					setSpecificType(selectedGenericType.createDefaultSpecificType(getGUISeSAmDeclaration().getSeSAmDeclaration()));
				}
			}
		}
	}

	protected void eventSpecificTypeChanged(ISpecificType newSpecificType) {
		if (getTypeList().contains(newSpecificType)) {
			if (!Utils.equals(getBoundJComboBoxTypes().getSelectedItem(), newSpecificType))
				getBoundJComboBoxTypes().setSelectedItem(newSpecificType);
		} else {
			for (int i = 0; i < getTypeList().size(); i++) {
				IType type = (IType)getTypeList().get(i);
				if ((type instanceof IGenericType) && (((IGenericType)type).canCreate(newSpecificType))) {
					if (!Utils.equals(getBoundJComboBoxTypes().getSelectedItem(), type))
						getBoundJComboBoxTypes().setSelectedItem(type);
					return;
				}
			}
		}
	}

	private BoundJComboBox getBoundJComboBoxTypes() {
		if (_boundJComboBoxTypes == null) {
			_boundJComboBoxTypes = new BoundJComboBox();
			_boundJComboBoxTypes.setRenderer(
				new NamedCellRendererAdapter(_boundJComboBoxTypes.getRenderer()));
		}
		return _boundJComboBoxTypes;
	}

	private JButton getJButtonSettings() {
		if (_jButtonSettings == null) {
			_jButtonSettings = new JButton();
			_jButtonSettings.setName("JButtonSettings");
			_jButtonSettings.setText("Settings ...");
			_jButtonSettings.setEnabled(false);
		}
		return _jButtonSettings;
	}

	public ISpecificType getSpecificType() {
		return getSpecificTypeIntern();
	}

	private ISpecificType getSpecificTypeIntern() {
		return _specificTypeIntern;
	}

	public List getTypeList() {
		return _typeList;
	}

	protected boolean hasMoreSettings(IType type) {
		if (type == null)
			return false;

		if (type instanceof IGenericType)
			return true;

		//		ISpecificType specificType = (ISpecificType)type;
		//
		//		return (specificType.getGenericType() != null);
		return false;
	}

	private void initConnections() {
		getJButtonSettings().addActionListener(_eventHandler);
		getBoundJComboBoxTypes().addItemListener(_eventHandler);
	}

	private void initialize() {
		setLayout(new FormLayout("default:grow, 4dlu, default", "pref:grow"));

		CellConstraints cc = new CellConstraints();

		add(getBoundJComboBoxTypes(), cc.xy(1, 1));
		add(getJButtonSettings(), cc.xy(3, 1));

		initConnections();

		setTypeList(CollectionsHelper.asLinkedList(TypeManager.getInstance().getAllTypes()));
	}

	public void setEnabled(boolean isEnabled) {
		if (isEnabled() != isEnabled) {
			super.setEnabled(isEnabled);

			IType selectedType = (IType)getBoundJComboBoxTypes().getSelectedItem();

			getJButtonSettings().setEnabled(isEnabled && hasMoreSettings(selectedType));
		}
	}

	private void setSpecificTypeIntern(ISpecificType newValue) {
		if (_specificTypeIntern != newValue) {
			ISpecificType oldValue = getSpecificTypeIntern();
			_specificTypeIntern = newValue;
			eventSpecificTypeChanged(getSpecificTypeIntern());
			firePropertyChange("specificType", oldValue, newValue);
		}
	}

	public void setTypeList(List newTypeList) {
		if (newTypeList.contains(TypeManager.getInstance().getDefaultType()))
			setTypeListAndSelectedItem(newTypeList, TypeManager.getInstance().getDefaultType());
		else
			setTypeListAndSelectedItem(newTypeList, (ISpecificType)newTypeList.get(0));
	}

	public void setTypeListAndSelectedItem(List newTypeList, ISpecificType defaultType) {
		_typeList = newTypeList;
		if (newTypeList != null) {
			if (Assert.ASSERTION_ON)
				Assert.assertTrue(!newTypeList.isEmpty());
			//ISpecificType defaultType = TypeManager.getInstance().getDefaultType();

			if (Assert.ASSERTION_ON)
				Assert.assertTrue(
					"Type list '"
						+ newTypeList
						+ "' doesn't contain default type '"
						+ defaultType.getName()
						+ "'",
					newTypeList.contains(defaultType) || newTypeList.contains(defaultType.getGenericType()));

			getBoundJComboBoxTypes().setListData(newTypeList);
			setSpecificType(defaultType);

			if (Assert.ASSERTION_ON) {
				Object selectedItem = getBoundJComboBoxTypes().getSelectedItem();
				if (selectedItem instanceof ISpecificType)
					Assert.assertEquals(defaultType, selectedItem);
				else
					Assert.assertEquals(defaultType.getGenericType(), selectedItem);
			}
		}
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null) {
			setTypeList(guiSeSAmDeclaration.getSeSAmModelDeclaration().getAvailibleTypeList());
		}
	}

	public void setSpecificType(ISpecificType arg1) {
		if (!_inSetSpecificType) {
			try {
				_inSetSpecificType = true;
				setSpecificTypeIntern(arg1);
			} finally {
				_inSetSpecificType = false;
			}
		}
	}

	public void dispose() {
		getBoundJComboBoxTypes().setSelectedItem(null);
		getBoundJComboBoxTypes().setListData(CollectionsHelper.EMPTY_ITERATOR);

		super.dispose();
	}
}