package de.uniwue.ki.gui.mas.basics.type.argument;
import java.util.*;

import javax.swing.tree.DefaultTreeModel;

import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.name.INamed;
import de.uniwue.ki.util.tree.DefaultObjectTreeNode;
import de.uniwue.ki.util.tree.DefaultObjectTreeNodeLeaf;

/**
 * Creation date: (30.05.00 14:01:01)
 * @author Christoph Oechslein
 */
public class ActivityDeclarationArgumentDeclarationEditor
	extends AbstractTreeArgumentDeclarationEditor {

	public ActivityDeclarationArgumentDeclarationEditor() {
		super();
	}

	protected void addActivityClassesToTreeNode(
		DefaultObjectTreeNode agentClassDeclarationNode,
		Iterator objectClasses) {
		while (objectClasses.hasNext()) {
			IObjectClassDeclaration currIObjectClassDeclaration =
				(IObjectClassDeclaration) objectClasses.next();

			if (currIObjectClassDeclaration instanceof IActivityAgentClassDeclaration) {
				IActivityAgentClassDeclaration activityAgentClassDeclaration =
					(IActivityAgentClassDeclaration) currIObjectClassDeclaration;

				Iterator it =
					activityAgentClassDeclaration
						.getReasoningEngineDeclarationListAndFactory()
						.getDeclarationList()
						.iterator();

				while (it.hasNext()) {
					IActivityReasoningEngineDeclaration activityReasoningEngineDeclaration =
						(IActivityReasoningEngineDeclaration) it.next();
					addActivitiesClassesToTreeNode(
						agentClassDeclarationNode,
						activityAgentClassDeclaration,
						activityReasoningEngineDeclaration.getRootActivityGraphDeclaration());
				}
			}
		}
	}

	protected void addActivitiesClassesToTreeNode(
		DefaultObjectTreeNode parentDeclarationNode,
		INamed parent,
		IActivityGraphDeclaration composedActivityDeclaration) {

		Iterator activities = composedActivityDeclaration.getActivityGraphNodeDeclarationsWithoutEmergencyAtAll();

		List activitiesToAdd = new LinkedList();

		while (activities.hasNext()) {
			IActivityGraphNodeDeclaration activityDeclaration =
				(IActivityGraphNodeDeclaration) activities.next();

			if ((activityDeclaration instanceof IEmergencyActivityGraphLeafDeclaration)
				|| (activityDeclaration instanceof IEntryActivityGraphLeafDeclaration)
				|| (activityDeclaration instanceof IExitActivityGraphLeafDeclaration)) {
			} else {
				activitiesToAdd.add(activityDeclaration);
			}

		}

		if (!activitiesToAdd.isEmpty()) {
			DefaultObjectTreeNode currObjectClassNode =
				new DefaultObjectTreeNode(parentDeclarationNode, parent);
			parentDeclarationNode.addChild(currObjectClassNode);

			Collections.sort(activitiesToAdd, NamedComparator.getInstance());

			Iterator it = activitiesToAdd.iterator();
			while (it.hasNext()) {
				IActivityGraphNodeDeclaration activityDeclaration =
					(IActivityGraphNodeDeclaration) it.next();

				if (activityDeclaration instanceof IActivityGraphDeclaration) {
					IActivityGraphDeclaration composedActivity =
						(IActivityGraphDeclaration) activityDeclaration;

					addActivitiesClassesToTreeNode(
						currObjectClassNode,
						composedActivity,
						composedActivity);

				} else {
					DefaultObjectTreeNodeLeaf currActivityClassNode =
						new DefaultObjectTreeNodeLeaf(currObjectClassNode, activityDeclaration);

					currObjectClassNode.addChild(currActivityClassNode);
				}
			}
		}
	}

	protected DefaultTreeModel createTreeModel() {
		DefaultObjectTreeNode rootNode = new DefaultObjectTreeNode(null, "root");

		if (getGUISeSAmDeclaration() != null) {

			addActivityClassesToTreeNode(
				rootNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getAgentClassDeclarationListAndFactory()
					.getAgentClassDeclarationList()
					.iterator());

			addActivityClassesToTreeNode(
				rootNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getWorldClassDeclarationListAndFactory()
					.getWorldClassDeclarationList()
					.iterator());

		}

		return new DefaultTreeModel(rootNode);
	}

	public String getTitle() {
		return "Choice Activity";
	}

	protected boolean isValidForSelection(Object userObject) {
		return userObject instanceof IActivityGraphNodeDeclaration;
	}
}