package de.uniwue.ki.gui.mas.basics.type.argument;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.AncestorListener;

import junit.framework.Assert;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.ButtonBarFactory;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.*;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.SpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.enum.ISpecificEnumType;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (30.05.00 14:20:41)
 * @author Christoph Oechslein
 */
public class ArgumentDeclarationEditorPaneFactoryManager {
	public interface ISetArgumentDeclarationFunction {
		void setArgumentDeclaration(IArgumentDeclaration argumentDeclaration);
	}

	private static final java.util.List _argumentDeclarationEditorPaneFactories = new ArrayList();

	public interface ArgumentDeclarationEditorPaneFactory {
		boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter);

		IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter);
	}

	public static class PrototypeArgumentDeclarationEditorPaneFactory
		implements ArgumentDeclarationEditorPaneFactory {
		private final ISpecificType _prototype;
		private final Class _argumentDeclarationEditorClass;

		public PrototypeArgumentDeclarationEditorPaneFactory(
			ISpecificType prototype,
			Class argumentDeclarationEditorClass) {
			super();

			if (Assert.ASSERTION_ON)
				Assert.assertTrue(
					IArgumentDeclarationEditor.class.isAssignableFrom(argumentDeclarationEditorClass));

			_prototype = prototype;
			_argumentDeclarationEditorClass = argumentDeclarationEditorClass;
		}

		public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
			return _prototype.equals(
				argumentDeclarationEditorParameter.getArgumentDeclaration().getSpecificType());
		}

		public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
			return (IArgumentDeclarationEditor)Utils.newInstanceOf(_argumentDeclarationEditorClass);
		}
	}

	static {
		initialize();
	}

	private ArgumentDeclarationEditorPaneFactoryManager() {
		super();
	}

	public static void addArgumentDeclarationEditorPaneFactory(ArgumentDeclarationEditorPaneFactory typeEditorPaneFactory) {

		_argumentDeclarationEditorPaneFactories.add(typeEditorPaneFactory);
	}

	public static void addArgumentDeclarationEditorPaneFactory(
		ISpecificType prototype,
		Class argumentDeclarationEditorClass) {

		addArgumentDeclarationEditorPaneFactory(
			new PrototypeArgumentDeclarationEditorPaneFactory(prototype, argumentDeclarationEditorClass));
	}

	private static void askForArgumentDeclarationInNonModalPanel(
		final IArgumentDeclarationEditor editorPanel,
		IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter,
		JComponent onComponent,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		final ISetArgumentDeclarationFunction doitFunction) {

		editorPanel.setArgumentDeclarationEditorParameter(argumentDeclarationEditorParameter);

		final JDisposeablePanel panel = new JDisposeablePanel(new BorderLayout()) {
			public void dispose() {
				editorPanel.dispose();

				super.dispose();
			}
		};

		panel.add(createInnerPanel(editorPanel), BorderLayout.CENTER);
		panel.add(createButtonBarPanel(editorPanel, doitFunction, panel), BorderLayout.SOUTH);

		GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addPanel(
			onComponent,
			editorPanel.getTitle(),
			panel,
			true);
	}

	private static JPanel createInnerPanel(final IArgumentDeclarationEditor editorPanel) {
		JPanel innerPanel = new JPanel(new BorderLayout());

		innerPanel.add((Component)editorPanel);

		innerPanel.setBorder(
			BorderFactory.createCompoundBorder(
				BorderFactory.createBevelBorder(BevelBorder.LOWERED),
				Borders.createEmptyBorder("2dlu,2dlu,2dlu,2dlu")));

		return innerPanel;
	}

	private static JPanel createButtonBarPanel(
		final IArgumentDeclarationEditor editorPanel,
		final ISetArgumentDeclarationFunction doitFunction,
		final JDisposeablePanel panel) {

		JButton cancelButton = new JButton(new AbstractBoundAction("Cancel") {
			public void doit() {
				SwingMoreUtilities.closeRootWindow(panel);
			}
		});
		cancelButton.setMnemonic('c');
		final JButton okButton = new JButton(new AbstractBoundAction("Ok") {
			public void doit() {
				doitFunction.setArgumentDeclaration(editorPanel.getArgumentDeclaration());
				SwingMoreUtilities.closeRootWindow(panel);
			}
		});
		okButton.setMnemonic('o');

		final JPanel buttonBarPanel = ButtonBarFactory.buildOKCancelBar(okButton, cancelButton);

		buttonBarPanel.setBorder(Borders.DLU2_BORDER);

		final AncestorListener ancestorListener = new AncestorListener() {
			private void setDefaultButton() {
				SwingUtilities.getRootPane(buttonBarPanel).setDefaultButton(okButton);
				buttonBarPanel.removeAncestorListener(this);
			}
			public void ancestorAdded(javax.swing.event.AncestorEvent event) {
				setDefaultButton();
			}
			public void ancestorMoved(javax.swing.event.AncestorEvent event) {
				setDefaultButton();
			}
			public void ancestorRemoved(javax.swing.event.AncestorEvent event) {
			}
		};

		buttonBarPanel.addAncestorListener(ancestorListener);

		return buttonBarPanel;
	}

	private static void askForArgumentDeclarationInModalPanel(
		IArgumentDeclarationEditor editorPanel,
		IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter,
		JComponent onComponent,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		ISetArgumentDeclarationFunction doitFunction) {

		editorPanel.setArgumentDeclarationEditorParameter(argumentDeclarationEditorParameter);

		JOptionPane optionPane =
			//new JOptionPane(editorPanel, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);
			new ResizeableOptionPane((Component)editorPanel);

		GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
			onComponent,
			editorPanel.getTitle(),
			optionPane);

		if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
			doitFunction.setArgumentDeclaration(editorPanel.getArgumentDeclaration());
		}

		editorPanel.dispose();
	}

	public static void askArgumentDeclarationFor(
		IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter,
		JComponent onComponent,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		ISetArgumentDeclarationFunction doitFunction) {

		Assert.assertNotNull(argumentDeclarationEditorParameter.getArgumentDeclaration());

		IArgumentDeclarationEditor editorPanel =
			getArgumentDeclarationEditorFor(argumentDeclarationEditorParameter);

		if (editorPanel == null) {
			ISpecificType specificType =
				argumentDeclarationEditorParameter.getArgumentDeclaration().getSpecificType();
			doitFunction.setArgumentDeclaration(
				specificType.createArgumentDeclaration(specificType.getArgumentDeclarationDefaultValue()));

		} else if (!editorPanel.inModalPanel()) {
			askForArgumentDeclarationInNonModalPanel(
				editorPanel,
				argumentDeclarationEditorParameter,
				onComponent,
				guiSeSAmDeclaration,
				doitFunction);

		} else {
			askForArgumentDeclarationInModalPanel(
				editorPanel,
				argumentDeclarationEditorParameter,
				onComponent,
				guiSeSAmDeclaration,
				doitFunction);
		}
	}

	public static IArgumentDeclarationEditor getArgumentDeclarationEditorFor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

		Iterator it = _argumentDeclarationEditorPaneFactories.iterator();
		while (it.hasNext()) {
			ArgumentDeclarationEditorPaneFactory currPaneFactory =
				(ArgumentDeclarationEditorPaneFactory)it.next();

			if (currPaneFactory.canEditArgumentDeclaration(argumentDeclarationEditorParameter)) {
				return currPaneFactory.getArgumentDeclarationEditor(argumentDeclarationEditorParameter);
			}
		}

		return null;
	}

	private static void initialize() {

		addArgumentDeclarationEditorPaneFactory(
			StringType.getInstance(),
			StringArgumentDeclarationEditor.class);

		addArgumentDeclarationEditorPaneFactory(
			BooleanType.getInstance(),
			BooleanArgumentDeclarationEditor.class);

		addArgumentDeclarationEditorPaneFactory(
			DoubleType.getInstance(),
			DoubleArgumentDeclarationEditor.class);

		addArgumentDeclarationEditorPaneFactory(
			IntegerType.getInstance(),
			IntegerArgumentDeclarationEditor.class);

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				return (
					argumentDeclarationEditorParameter.getArgumentDeclaration().getSpecificType()
						== FeatureClassDeclarationType.getInstance())
					&& !FeatureClassDeclarationArgumentDeclarationEditor
						.getFeatureClassDeclarationList(
							argumentDeclarationEditorParameter.getGUISeSAmDeclaration())
						.isEmpty();
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new FeatureClassDeclarationArgumentDeclarationEditor();
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				IArgumentDeclaration argumentDeclaration =
					argumentDeclarationEditorParameter.getArgumentDeclaration();
				ISpecificType type = argumentDeclaration.getSpecificType();

				return (type == ObjectClassDeclarationType.getInstance())
					&& !(argumentDeclarationEditorParameter
						.getGUISeSAmDeclaration()
						.getSeSAmModelDeclaration()
						.getAvailibleObjectClassDeclarationList()
						.isEmpty());
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new ObjectClassDeclarationArgumentDeclarationEditor();
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				IArgumentDeclaration argumentDeclaration =
					argumentDeclarationEditorParameter.getArgumentDeclaration();
				ISpecificType type = argumentDeclaration.getSpecificType();

				return (type == ActivityType.getInstance());
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new ActivityDeclarationArgumentDeclarationEditor();
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				IArgumentDeclaration argumentDeclaration =
					argumentDeclarationEditorParameter.getArgumentDeclaration();
				ISpecificType type = argumentDeclaration.getSpecificType();

				return (type == ObjectInstanceDeclarationType.getInstance())
					&& !(argumentDeclarationEditorParameter
						.getGUISeSAmDeclaration()
						.getSeSAmModelDeclaration()
						.getAvailibleObjectClassDeclarationList()
						.isEmpty());
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new ObjectInstanceDeclarationArgumentDeclarationEditor();
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				ISpecificType type =
					argumentDeclarationEditorParameter.getArgumentDeclaration().getSpecificType();
				return GenericVariableClassDeclarationType.getInstance(true, true).canCreate(type)
					|| GenericVariableClassDeclarationType.getInstance(true, false).canCreate(type)
					|| GenericVariableClassDeclarationType.getInstance(false, true).canCreate(type)
					|| GenericVariableClassDeclarationType.getInstance(false, false).canCreate(type);
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				ISpecificType type =
					argumentDeclarationEditorParameter.getArgumentDeclaration().getSpecificType();
				if (GenericVariableClassDeclarationType.getInstance(true, true).canCreate(type)
					|| GenericVariableClassDeclarationType.getInstance(true, false).canCreate(type)) {
					return new ExternalVariableClassDeclarationArgumentDeclarationEditor();

				} else {
					return new VariableClassDeclarationArgumentDeclarationEditor();
				}
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				IArgumentDeclaration argumentDeclaration =
					argumentDeclarationEditorParameter.getArgumentDeclaration();
				ISpecificType type = argumentDeclaration.getSpecificType();

				return (type instanceof SpecificFunctionType)
					&& !(argumentDeclaration instanceof IFunctionArgumentDeclaration)
					&& ((argumentDeclaration.getObject() == null)
						|| (argumentDeclaration.getObject()
							instanceof SpecificFunctionDeclarationWithArgDeclBinds));
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new FunctionArgumentDeclarationEditor();
			}
		});

		addArgumentDeclarationEditorPaneFactory(new ArgumentDeclarationEditorPaneFactory() {
			public boolean canEditArgumentDeclaration(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {

				IArgumentDeclaration argumentDeclaration =
					argumentDeclarationEditorParameter.getArgumentDeclaration();
				ISpecificType type = argumentDeclaration.getSpecificType();

				return (type instanceof ISpecificEnumType);
				/**
					&& !(argumentDeclarationEditorParameter
						.getGUISeSAmDeclaration()
						.getSeSAmModelDeclaration()
						.getSeSAmModelBasicsDeclaration()
						.getUserTypeDeclarationListAndFactory()
						.getUserTypeDeclarationList()
						.isEmpty());
				*/
			}

			public IArgumentDeclarationEditor getArgumentDeclarationEditor(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
				return new EnumTypeArgumentDeclarationEditor();
			}
		});

	}
}