package de.uniwue.ki.gui.mas.basics.type.argument;
import java.util.List;

import de.uniwue.ki.gui.util.JEditMutableListPanel;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.enum.ISpecificEnumType;
import de.uniwue.ki.util.list.DefaultMutableListModel;

/**
 * Creation date: (10.07.2001 10:58:46)
 * @author Christoph Oechslein
 */
public class EnumTypeArgumentDeclarationEditor
	extends JEditMutableListPanel
	implements IArgumentDeclarationEditor {

	private ISpecificEnumType _specificEnumType;

	public EnumTypeArgumentDeclarationEditor() {
		super();
		initialize();

		setCellRenderer(new NamedCellRendererAdapter(getCellRenderer()));
	}

	public IArgumentDeclaration getArgumentDeclaration() {
		if (getSelectedValue() != null) {
			return ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				_specificEnumType,
				getSelectedValue());
		} else {
			return null;
		}
	}

	protected List getEnumItemList() {
		return _specificEnumType.getItemList();
	}

	public String getTitle() {
		return "Choice Enum item";
	}

	/**
	 * Called whenever the part throws an exception.
	 * @param exception Throwable
	 */
	private void handleException(Throwable exception) {

		exception.printStackTrace(System.out);
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
					setName("ObjectClassDeclarationArgumentDeclarationEditor");
			setSize(294, 312);
			setEditable(false);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	protected void initializeMutableList() {
		setMutableListModel(new DefaultMutableListModel(getEnumItemList()));
		//TODO rausgefallen... Wenn kein EnumItem definiert ist erzeuge eines
		if (getEnumItemList().size() > 0)
			setSelectedValue(getEnumItemList().get(0));
	}

	public static void main(String[] args) {
		try {
			javax.swing.JFrame frame = new javax.swing.JFrame();
			EnumTypeArgumentDeclarationEditor aObjectClassDeclarationArgumentDeclarationEditor;
			aObjectClassDeclarationArgumentDeclarationEditor = new EnumTypeArgumentDeclarationEditor();
			frame.setContentPane(aObjectClassDeclarationArgumentDeclarationEditor);
			frame.setSize(aObjectClassDeclarationArgumentDeclarationEditor.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JPanelWithGUISeSAmDeclaration");
			exception.printStackTrace(System.out);
		}
	}

	public void setArgumentDeclarationEditorParameter(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
		IArgumentDeclaration argumentDeclaration =
			argumentDeclarationEditorParameter.getArgumentDeclaration();

		_specificEnumType = (ISpecificEnumType) argumentDeclaration.getSpecificType();

		if (argumentDeclaration.getObject() != null) {
			setSelectedValue(argumentDeclaration.getObject());
		}

		initializeMutableList();
	}

	public boolean inModalPanel() {
		return true;
	}
}