package de.uniwue.ki.gui.mas.basics.type.argument;
import java.util.*;

import javax.swing.tree.DefaultTreeModel;

import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.type.SpecificVariableClassDeclarationType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.tree.DefaultObjectTreeNode;
import de.uniwue.ki.util.tree.DefaultObjectTreeNodeLeaf;

/**
 * Creation date: (30.05.00 14:01:01)
 * @author Christoph Oechslein
 */
public class ExternalVariableClassDeclarationArgumentDeclarationEditor
	extends AbstractTreeArgumentDeclarationEditor {

	public ExternalVariableClassDeclarationArgumentDeclarationEditor() {
		super();
	}

	protected void addFeatureClassesToTreeNode(
		DefaultObjectTreeNode featureClassDeclarationNode,
		Iterator features) {

		while (features.hasNext()) {
			IUserFeatureClassDeclaration currIFeatureClassDeclaration =
				(IUserFeatureClassDeclaration) features.next();

			List variablesToAdd = new LinkedList();

			{
				Iterator variables =
					currIFeatureClassDeclaration
						.getFeatureVariableClassDeclarationListAndFactory()
						.getVariableClassDeclarationList()
						.iterator();

				while (variables.hasNext()) {
					IVariableClassDeclaration currIVariableClassDeclaration =
						(IVariableClassDeclaration) variables.next();

					if (isCorrectVariable(currIVariableClassDeclaration)) {
						variablesToAdd.add(currIVariableClassDeclaration);
					}
				}
			}

			if (!variablesToAdd.isEmpty()) {

				DefaultObjectTreeNode currFeatureNode =
					new DefaultObjectTreeNode(featureClassDeclarationNode, currIFeatureClassDeclaration);
				featureClassDeclarationNode.addChild(currFeatureNode);

				Collections.sort(variablesToAdd, NamedComparator.getInstance());

				Iterator it = variablesToAdd.iterator();
				while (it.hasNext()) {
					IVariableClassDeclaration currIVariableClassDeclaration =
						(IVariableClassDeclaration) it.next();

					DefaultObjectTreeNodeLeaf currVariableNode =
						new DefaultObjectTreeNodeLeaf(currFeatureNode, currIVariableClassDeclaration);
					currFeatureNode.addChild(currVariableNode);
				}
			}
		}
	}

	protected void addObjectClassesToTreeNode(
		DefaultObjectTreeNode objectClassDeclarationNode,
		Iterator objectClasses) {
		while (objectClasses.hasNext()) {
			IObjectClassDeclaration currIObjectClassDeclaration =
				(IObjectClassDeclaration) objectClasses.next();

			Iterator variables =
				currIObjectClassDeclaration
					.getBodyClassDeclaration()
					.getBodyVariableClassDeclarationListAndFactory()
					.getBodyVariableClassDeclarationList()
					.iterator();

			List variablesToAdd = new LinkedList();

			while (variables.hasNext()) {
				IBodyVariableClassDeclaration currIBodyVariableClassDeclaration =
					(IBodyVariableClassDeclaration) variables.next();

				if (isCorrectVariable(currIBodyVariableClassDeclaration.getVariableClassDeclaration())) {
					variablesToAdd.add(currIBodyVariableClassDeclaration);
				}
			}

			if (!variablesToAdd.isEmpty()) {

				DefaultObjectTreeNode currObjectClassNode =
					new DefaultObjectTreeNode(objectClassDeclarationNode, currIObjectClassDeclaration);
				objectClassDeclarationNode.addChild(currObjectClassNode);

				Collections.sort(variablesToAdd, NamedComparator.getInstance());

				Iterator it = variablesToAdd.iterator();
				while (it.hasNext()) {
					IBodyVariableClassDeclaration currIBodyVariableClassDeclaration =
						(IBodyVariableClassDeclaration) it.next();

					DefaultObjectTreeNodeLeaf currVariableNode =
						new DefaultObjectTreeNodeLeaf(currObjectClassNode, currIBodyVariableClassDeclaration);

					currObjectClassNode.addChild(currVariableNode);
				}
			}
		}
	}

	protected DefaultTreeModel createTreeModel() {
		DefaultObjectTreeNode rootNode = new DefaultObjectTreeNode(null, "root");

		DefaultObjectTreeNode objectClassDeclarationNode = new DefaultObjectTreeNode(rootNode, "Objects");
		DefaultObjectTreeNode featureClassDeclarationNode = new DefaultObjectTreeNode(rootNode, "Features");

		rootNode.addChild(objectClassDeclarationNode);
		rootNode.addChild(featureClassDeclarationNode);

		if (getGUISeSAmDeclaration() != null) {

			addObjectClassesToTreeNode(
				objectClassDeclarationNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getRessourceClassDeclarationListAndFactory()
					.getRessourceClassDeclarationList()
					.iterator());

			addObjectClassesToTreeNode(
				objectClassDeclarationNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getAgentClassDeclarationListAndFactory()
					.getAgentClassDeclarationList()
					.iterator());

			addObjectClassesToTreeNode(
				objectClassDeclarationNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getWorldClassDeclarationListAndFactory()
					.getWorldClassDeclarationList()
					.iterator());

			addFeatureClassesToTreeNode(
				featureClassDeclarationNode,
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSeSAmModelBasicsDeclaration()
					.getUserFeatureClassDeclarationListAndFactory()
					.getUserFeatureClassDeclarationList()
					.iterator());
		}

		return new DefaultTreeModel(rootNode);
	}

	protected SpecificVariableClassDeclarationType getSpecificVariableClassDeclarationType() {
		if (getArgumentDeclarationIntern() == null) {
			return null;
		} else {
			return (SpecificVariableClassDeclarationType) getArgumentDeclarationIntern().getSpecificType();
		}
	}

	public String getTitle() {
		return "Choice Variable";
	}

	protected boolean isCorrectVariable(IVariableClassDeclaration variableClassDeclaration) {
		if (getArgumentDeclarationIntern() == null)
			return true;

		SpecificVariableClassDeclarationType type =
			(SpecificVariableClassDeclarationType) getArgumentDeclarationIntern().getSpecificType();

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(
				type.getVariableClassDeclarationType(),
				variableClassDeclaration.getSpecificType())) {
			return false;
		}

		if (type.isExternal() && !variableClassDeclaration.isExternal()) {
			return false;
		}

		if (type.isWriteable())
			return variableClassDeclaration.isWriteable();
		else
			return true;
	}

	protected boolean isValidForSelection(Object userObject) {
		return (userObject instanceof IVariableClassDeclaration)
			|| (userObject instanceof IBodyVariableClassDeclaration);
	}

}