package de.uniwue.ki.gui.mas.basics.type.argument;
import java.awt.event.*;

import javax.swing.*;

import junit.framework.Assert;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.sim.simulation.SimulationWithNameFunctionCallDeclDeclarationPanel;
import de.uniwue.ki.gui.mas.sim.situation.SituationDeclarationPanel;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationComboBox;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclarationList;
import de.uniwue.ki.mas.basics.type.SimulationDeclarationType;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.sim.simulation.*;

/**
 * Creation date: (10.07.2001 10:58:46)
 * @author Christoph Oechslein
 */
public class SimulationDeclarationArgumentDeclarationEditor
	extends JPanelWithGUISeSAmDeclaration
	implements IArgumentDeclarationEditor {

	private ISimulationWithNameFunctionCallDeclDeclaration _simulationWithNameFunctionCallDeclDeclaration;

	private boolean _usedCreatedSimDecl = false;

	private final ButtonGroup _buttonGroup = new ButtonGroup();

	private JRadioButton ivjJRadioButton = null; //
	private JRadioButton ivjJRadioButton1 = null; //
	private DeclarationComboBox ivjDeclarationComboBox = null;
	private JButton ivjJButton = null;

	private SituationDeclarationPanel ivjSituationDeclarationPanel;

	private SimulationWithNameFunctionCallDeclDeclarationPanel ivjSimulationWithNameFunctionCallDeclDeclarationPanel;

	public SimulationDeclarationArgumentDeclarationEditor() {
		super();
		initialize();
	}

	public IArgumentDeclaration getArgumentDeclaration() {
		Assert.assertFalse(isDisposed());

		ISimulationDeclaration simulationDeclaration;
		boolean oldSim = getJRadioButtonOldSim().isSelected();
		if (oldSim) {
			simulationDeclaration = (ISimulationDeclaration)getDeclarationComboBox().getSelectedItem();
		} else {
			_usedCreatedSimDecl = true;
			simulationDeclaration = getSimulationWithNameFunctionCallDeclDeclaration();
		}

		if (simulationDeclaration != null) {
			return ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				SimulationDeclarationType.getInstance(),
				simulationDeclaration);
		} else {
			return null;
		}
	}

	protected SimulationDeclarationList getSimulationDeclarationList() {
		return getGUISeSAmDeclaration()
			.getSeSAmModelDeclaration()
			.getSimulationElementsDeclaration()
			.getSimulationDeclarationListAndFactory()
			.getSimulationDeclarationList();
	}

	public String getTitle() {
		return "Choice Simulation";
	}

	public void setArgumentDeclarationEditorParameter(IArgumentDeclarationEditorParameter argumentDeclarationEditorParameter) {
		setGUISeSAmDeclaration(argumentDeclarationEditorParameter.getGUISeSAmDeclaration());
		getSituationDeclarationPanel().setGUISeSAmDeclaration(
			argumentDeclarationEditorParameter.getGUISeSAmDeclaration());
		getSimulationWithNameFunctionCallDeclDeclarationPanel().setGUISeSAmDeclaration(
			argumentDeclarationEditorParameter.getGUISeSAmDeclaration());

		IFunctionArgumentDeclarationList outerFunctionArgumentDeclarationList =
			argumentDeclarationEditorParameter
				.getFunctionCallDeclarationTreeListPanel()
				.getAllFunctionArgumentDeclarationList();

		getSituationDeclarationPanel().setOuterFunctionArgumentDeclarationList(
			outerFunctionArgumentDeclarationList);
		getSimulationWithNameFunctionCallDeclDeclarationPanel().setOuterFunctionArgumentDeclarationList(
			outerFunctionArgumentDeclarationList);

		getDeclarationComboBox().setDeclarationListModel(getSimulationDeclarationList());
		if (getSimulationDeclarationList().size() > 0)
			getDeclarationComboBox().setSelectedItem(getSimulationDeclarationList().get(0));

		ISimulationDeclaration simulationDeclaration =
			(ISimulationDeclaration)argumentDeclarationEditorParameter.getArgumentDeclaration().getObject();

		if (simulationDeclaration != null) {
			if (simulationDeclaration instanceof ISimulationWithNameFunctionCallDeclDeclaration) {
				Assert.assertFalse(getSimulationDeclarationList().contains(simulationDeclaration));
				setSimulationWithNameFunctionCallDeclDeclaration(
					(ISimulationWithNameFunctionCallDeclDeclaration)simulationDeclaration);
				getJRadioButtonNewSim().setSelected(true);

			} else {
				Assert.assertTrue(getSimulationDeclarationList().contains(simulationDeclaration));
				getDeclarationComboBox().setSelectedItem(simulationDeclaration);
				getJRadioButtonOldSim().setSelected(true);
			}
		}

		getSimulationWithNameFunctionCallDeclDeclarationPanel()
			.setSimulationWithNameFunctionCallDeclDeclaration(
			getSimulationWithNameFunctionCallDeclDeclaration());
		getSituationDeclarationPanel().setSituationDeclaration(
			getSimulationWithNameFunctionCallDeclDeclaration().getSituationDeclaration());

		updateEnabledState();
	}

	protected void eventEditButtonPressed() {
		getGUISeSAmDeclaration().getFrameFactory().addPanel(
			this,
			"Edit Situation",
			getSituationDeclarationPanel(),
			true,
			WindowConstants.HIDE_ON_CLOSE);
	}

	private SituationDeclarationPanel getSituationDeclarationPanel() {
		if (ivjSituationDeclarationPanel == null) {
			ivjSituationDeclarationPanel = new SituationDeclarationPanel();
		}
		return ivjSituationDeclarationPanel;
	}

	protected void updateEnabledState() {
		if (isEnabled()) {
			boolean oldSimIsSelected = getJRadioButtonOldSim().isSelected();
			getDeclarationComboBox().setEnabled(oldSimIsSelected);
			getJButton().setEnabled(!oldSimIsSelected);
			getSimulationWithNameFunctionCallDeclDeclarationPanel().setEnabled(!oldSimIsSelected);
		} else {
			getDeclarationComboBox().setEnabled(false);
			getJButton().setEnabled(false);
			getSimulationWithNameFunctionCallDeclDeclarationPanel().setEnabled(false);
		}
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		PanelBuilder builder =
			new PanelBuilder(
				this,
				new FormLayout(
					"default, 1dlu, default, fill:default:grow",
					"default, 1dlu, default, 1dlu, fill:default:grow"));

		builder.add(getJRadioButtonOldSim());
		builder.nextColumn(2);
		builder.add(getDeclarationComboBox());
		builder.nextLine(2);
		builder.add(getJRadioButtonNewSim());
		builder.nextColumn(2);
		builder.add(getJButton());
		builder.nextLine(2);
		builder.setColumnSpan(4);
		builder.add(getSimulationWithNameFunctionCallDeclDeclarationPanel());

		_buttonGroup.add(getJRadioButtonOldSim());
		_buttonGroup.add(getJRadioButtonNewSim());

		getJRadioButtonOldSim().setSelected(true);
	}

	private SimulationWithNameFunctionCallDeclDeclarationPanel getSimulationWithNameFunctionCallDeclDeclarationPanel() {
		if (ivjSimulationWithNameFunctionCallDeclDeclarationPanel == null) {
			ivjSimulationWithNameFunctionCallDeclDeclarationPanel =
				new SimulationWithNameFunctionCallDeclDeclarationPanel();
		}
		return ivjSimulationWithNameFunctionCallDeclDeclarationPanel;
	}

	/**
	 * This method initializes ivjJRadioButton
	 * 
	 * @return JRadioButton
	 */
	private JRadioButton getJRadioButtonOldSim() {
		if (ivjJRadioButton == null) {
			ivjJRadioButton = new JRadioButton();
			ivjJRadioButton.setText("Old Simulations: ");
			ivjJRadioButton.setName("");
			ivjJRadioButton.setSelected(true);

			ivjJRadioButton.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					updateEnabledState();
				}
			});
		}
		return ivjJRadioButton;
	}
	/**
	 * This method initializes ivjJRadioButton1
	 * 
	 * @return JRadioButton
	 */
	private JRadioButton getJRadioButtonNewSim() {
		if (ivjJRadioButton1 == null) {
			ivjJRadioButton1 = new JRadioButton();
			ivjJRadioButton1.setText("New Simulation: ");
		}
		return ivjJRadioButton1;
	}
	/**
	 * This method initializes ivjDeclarationComboBox
	 * 
	 * @return DeclarationComboBox
	 */
	private DeclarationComboBox getDeclarationComboBox() {
		if (ivjDeclarationComboBox == null) {
			ivjDeclarationComboBox = new DeclarationComboBox();
			ivjDeclarationComboBox.setEditable(false);
		}
		return ivjDeclarationComboBox;
	}
	/**
	 * This method initializes ivjJButton
	 * 
	 * @return JButton
	 */
	private JButton getJButton() {
		if (ivjJButton == null) {
			ivjJButton = new JButton();
			ivjJButton.setText("Edit Situation ...");
			ivjJButton.setEnabled(false);
			ivjJButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					eventEditButtonPressed();
				}
			});
		}
		return ivjJButton;
	}

	public void setEnabled(boolean b) {
		super.setEnabled(b);

		updateEnabledState();
	}

	private ISimulationWithNameFunctionCallDeclDeclaration getSimulationWithNameFunctionCallDeclDeclaration() {
		if (_simulationWithNameFunctionCallDeclDeclaration == null) {

			Assert.assertNotNull(getGUISeSAmDeclaration());

			SimulationDeclarationFactory simulationDeclarationFactory =
				getGUISeSAmDeclaration()
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getSimulationDeclarationListAndFactory()
					.getSimulationDeclarationFactory();

			if (getDeclarationComboBox().getSelectedItem() != null) {
				setSimulationWithNameFunctionCallDeclDeclaration(
					simulationDeclarationFactory.createSimulationWithNameFunctionCallDeclDeclaration(
						(ISimulationDeclaration)getDeclarationComboBox().getSelectedItem()));

			} else {
				setSimulationWithNameFunctionCallDeclDeclaration(
					simulationDeclarationFactory.createSimulationWithNameFunctionCallDeclDeclaration());
			}
		}

		return _simulationWithNameFunctionCallDeclDeclaration;
	}

	private void setSimulationWithNameFunctionCallDeclDeclaration(ISimulationWithNameFunctionCallDeclDeclaration simulationWithNameFunctionCallDeclDeclaration) {
		_simulationWithNameFunctionCallDeclDeclaration = simulationWithNameFunctionCallDeclDeclaration;
	}

	public void dispose() {
		getDeclarationComboBox().dispose();

		if ((_simulationWithNameFunctionCallDeclDeclaration != null) && (!_usedCreatedSimDecl)) {
			getSimulationWithNameFunctionCallDeclDeclaration().remove();
		}

		getSituationDeclarationPanel().dispose();

		super.dispose();
	}

	public boolean inModalPanel() {
		return false;
	}
}
