package de.uniwue.ki.gui.mas.basics.type.dependent;
import java.awt.Component;
import java.util.List;

import javax.swing.JOptionPane;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.basics.type.*;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (02.06.00 10:08:30)
 * @author Christoph Oechslein
 */
public class DependentTypeBindingsDialog {

	private DependentTypeBindingsDialog() {
		super();
	}

	public static DependentTypeBindings createDependentTypeBindings(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType outputSpecificType,
		Component onComponent,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		List dependentTypeList =
			CollectionsHelper.asLinkedList(genericFunctionDeclaration.getDependentTypes(outputSpecificType));

		int size = dependentTypeList.size();

		if (size == 0) {
			return DependentTypeBindings.EMPTY_BINDINGS;

		} else if (size == 1) {
			IDependentType typeT = (IDependentType)dependentTypeList.get(0);

			//			if (typeT instanceof ConstrainedToSubtypesDependentType) {
			//				((ConstrainedToSubtypesDependentType)typeT).setOutputType(outputSpecificType);
			//			}

			ISpecificType choicedType =
				askForSpecificTypeFor(typeT, typeList, guiSeSAmDeclaration, onComponent, outputSpecificType);

			if (choicedType != null) {
				DependentTypeBindings bindings = new DependentTypeBindings(dependentTypeList.iterator());
				bindings.setBindingFor(typeT, choicedType);
				return bindings;

			} else {
				return null;
			}

		} else {
			DependentTypeBindings bindings = new DependentTypeBindings(dependentTypeList.iterator());

			DependentTypeBindingsPanel panel = new DependentTypeBindingsPanel();

			panel.setGUISeSAmDeclaration(guiSeSAmDeclaration);

			panel.setTypeList(typeList);

			panel.setBindingTableModelDependentTypeBindings(bindings);
			panel.setBindingTableModelDependentTypes(dependentTypeList);

			JOptionPane optionPane =
				new JOptionPane(panel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

			GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
				onComponent,
				"Set Types",
				optionPane);

			if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
				return bindings;

			} else {
				return null;
			}
		}
	}

	public static ISpecificType askForSpecificTypeFor(
		final IDependentType dependentType,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component onComponent) {
		return askForSpecificTypeFor(dependentType, typeList, guiSeSAmDeclaration, onComponent, null);
	}

	public static ISpecificType askForSpecificTypeFor(
		final IDependentType dependentType,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component onComponent,
		ISpecificType outputType) {

		if (dependentType instanceof UnconstraintDependentType) {
			return askForSpecificTypeForDependentType(
				dependentType,
				typeList,
				guiSeSAmDeclaration,
				onComponent);

		} else if (dependentType instanceof ConstrainedToSubtypesDependentType) {
			((ConstrainedToSubtypesDependentType)dependentType).setOutputType(outputType);

			return askForSpecificTypeForDependentType(
				dependentType,
				TypeManager.getSubTypes(outputType),
				guiSeSAmDeclaration,
				onComponent);

		} else if (dependentType instanceof ConstraintGenericDependentType) {
			return askForSpecificTypeForConstraintGenericDependentType(
				(ConstraintGenericDependentType)dependentType,
				typeList,
				guiSeSAmDeclaration,
				onComponent,
				outputType);

		} else {
			Assert.fail("Not possible for: " + dependentType);

			return null;
		}
	}

	private static ISpecificType askForSpecificTypeForDependentType(
		IDependentType unconstraintDependentType,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component onComponent) {

		return askForSpecificTypeFor(
			new SpecificTypeDefinePanelWithName(),
			unconstraintDependentType,
			typeList,
			guiSeSAmDeclaration,
			"Edit " + unconstraintDependentType.getName(),
			onComponent);
	}

	private static ISpecificType askForSpecificTypeForConstraintGenericDependentType(
		ConstraintGenericDependentType constraintGenericDependentType,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Component onComponent,
		ISpecificType outputType) {

		IGenericType genericType = constraintGenericDependentType.getGenericType();

		IGenericTypeDefinePanel genericTypeDefinePanel =
			GenericArgumentDefinePanelFactory.createGenericTypeDefinePanelFor(
				genericType,
				guiSeSAmDeclaration,
				typeList);
		if (genericTypeDefinePanel instanceof GenericComposedTypeAttributeTypeDefinePanel) {
			((GenericComposedTypeAttributeTypeDefinePanel)genericTypeDefinePanel).setMostGeneralAttributeOutputType(
				outputType);
		}

		return askForSpecificTypeFor(
			genericTypeDefinePanel,
			constraintGenericDependentType,
			typeList,
			guiSeSAmDeclaration,
			"Settings for " + genericType.getName(),
			onComponent);
	}

	private static ISpecificType askForSpecificTypeFor(
		ITypeDefinePanel typeDefinePanel,
		IDependentType dependentType,
		List typeList,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		String title,
		Component onComponent) {

		if (typeDefinePanel == null)
			return null;

		typeDefinePanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);
		typeDefinePanel.setTypeList(typeList);

		ISpecificType preSelected =
			dependentType.getDefaultSpecificType(guiSeSAmDeclaration.getSeSAmDeclaration());
		
		if (preSelected != null) 
			typeDefinePanel.setSpecificType(
				dependentType.getDefaultSpecificType(guiSeSAmDeclaration.getSeSAmDeclaration()));
		

		//TODO unterdrcken wenn keine Optionen?
		JOptionPane optionPane =
			new JOptionPane(typeDefinePanel, JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

		GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
			onComponent,
			title,
			optionPane);

		if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
			return typeDefinePanel.getSpecificType();

		} else {
			return null;
		}
	}
}