package de.uniwue.ki.gui.mas.basics.type.user;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.basics.type.SpecificTypeDefinePanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.border.BetterTitledBorder;
import de.uniwue.ki.gui.util.bound.BetterBoundJTextField;
import de.uniwue.ki.mas.basics.search.SearchReferencesHelper;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.refactoring.AddUserTypeAttribute;
import de.uniwue.ki.mas.refactoring.ChangeUserTypeAttributeRepair;
import de.uniwue.ki.mas.refactoring.RemoveUserTypeAttribute;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: (03.12.2001 18:35:38)
 * @author Rainer Herrler
 */
public class ComposedTypeDeclarationPanel extends AbstractDeclarationPanel {
	private JCheckBox _writeableJCheckBox;
	private IUserTypeDeclaration _userTypeDeclaration;

	private JPanel ivjAttributeDefinitionPanel = null;
	private JPanel ivjInheritanceDefinitionPanel = null;
	private JButton ivjJButtonRemove = null;
	private JButton ivjJButtonNew = null;
	private JLabel ivjJLabelAttributes = null;
	private JPanel ivjAttributeListPanel = null;
	private IComposedTypeAttribute ivjMySelectedAttribute = null;
	private JTextField ivjAttributeDefinitionNameLabel = null;
	private SpecificComposedType ivjMyComposedType = null;
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private JList ivjUserTypeAttributeList = null;
	private boolean interactiveChange = true;
	private SpecificTypeDefinePanel ivjSpecificTypeDefinePanel = null;
	private JComboBox ivjComboBoxFather = null;

	class IvjEventHandler implements ActionListener, PropertyChangeListener, ListSelectionListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == ComposedTypeDeclarationPanel.this.getJButtonNew())
				connEtoCNew(e);
			if (e.getSource() == ComposedTypeDeclarationPanel.this.getJButtonRemove())
				connEtoCRemove(e);
		}

		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			if (evt.getSource() == ComposedTypeDeclarationPanel.this.getSpecificTypeDefinePanel()
				&& (evt.getPropertyName().equals("specificType")))
				connEtoCAttributeAltered(evt);
		}
		public void valueChanged(ListSelectionEvent e) {
			if (e.getSource() == ComposedTypeDeclarationPanel.this.getUserTypeAttributeList())
				connEtoCAttributeSelectionChanged(e);
		}
	}

	public ComposedTypeDeclarationPanel() {
		super();
		initialize();
	}

	public ComposedTypeDeclarationPanel(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super();
		setGUISeSAmDeclaration(guiSeSAmDeclaration);
		initialize();
	}

	public void applyButton_ActionPerformed(ActionEvent actionEvent) {
	}

	private void connEtoCAttributeSelectionChanged(javax.swing.event.ListSelectionEvent arg1) {
		try {
			this.eventAttributeSelectionChanged(arg1);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	private void connEtoCNew(ActionEvent arg1) {
		try {
			this.eventNew(arg1);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	private void connEtoCRemove(ActionEvent arg1) {
		try {
			this.eventRemove(arg1);
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	private void connEtoCAttributeAltered(java.beans.PropertyChangeEvent arg1) {
		try {
			this.eventAttributeAltered(getSpecificTypeDefinePanel().getSpecificType());
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	public void eventAttributeAltered(ISpecificType arg1) {
		// i dont know about the int-parameters
		eventAttributeAltered();
	}

	public void eventAttributeAltered(ActionEvent actionEvent) {
		eventAttributeAltered();
	}

	public void eventAttributeAltered() {
		if (interactiveChange) {
			if (ivjMySelectedAttribute != null) {
				if (askAreYouSure()) {
				ivjMySelectedAttribute.setAttributeName(getAttributeDefinitionNameLabel().getText());
				ivjMySelectedAttribute.setAttributeType(getSpecificTypeDefinePanel().getSpecificType());
				ivjMyComposedType.attributeAltered(ivjMySelectedAttribute);

				ChangeUserTypeAttributeRepair.execute(getGUISeSAmDeclaration().getSeSAmDeclaration(),ivjMySelectedAttribute);
				}
			}

			getUserTypeAttributeList().invalidate();
			getUserTypeAttributeList().repaint();
		}
	}

	public void eventAttributeSelectionChanged(javax.swing.event.ListSelectionEvent listSelectionEvent) {
		eventAttributeSelectionChanged();
	}

	public void eventAttributeSelectionChanged() {
		Object selectedObj = getUserTypeAttributeList().getSelectedValue();

		Assert.assertTrue(((selectedObj == null) || (selectedObj instanceof IComposedTypeAttribute)));

		interactiveChange = false;
		setMySelectedAttribute((IComposedTypeAttribute)getUserTypeAttributeList().getSelectedValue());
		interactiveChange = true;
	}

	public void eventFatherSelectionChanged() {
		Object selectedObj = getComboBoxFather().getSelectedItem();

		Assert.assertTrue(((selectedObj == null) || (selectedObj instanceof ISpecificComposedType)));

// somehow this is not working
//		if (interactiveChange)
//		   if (!askAreYouSure()) {
//			getComboBoxFather().setSelectedItem(ivjMyComposedType.getFather());
//			return;
//		}

		ivjMyComposedType.setFather((ISpecificComposedType)selectedObj);
	}

	public void eventNew(ActionEvent actionEvent) {
		if (getMyComposedType() == null)
			return;

		if (interactiveChange)
			if (!askAreYouSure())
				return;

		AddUserTypeAttribute.execute(getGUISeSAmDeclaration().getSeSAmDeclaration(),getMyComposedType());
		
		Object[] attributeArray = CollectionsHelper.asArray(getMyComposedType().getOwnAttributes());
		getUserTypeAttributeList().setListData(
			attributeArray);
		getUserTypeAttributeList().setSelectedValue(attributeArray[attributeArray.length-1], true);
	}

	private boolean askAreYouSure() {
		if (SearchReferencesHelper
			.findReferences(getUserTypeDeclaration(), getGUISeSAmDeclaration().getSeSAmDeclaration())
			.hasNext()) {
			final JOptionPane dialog = new JOptionPane();
			dialog.setMessage(
				new JLabel(
					"<html>The composed type is referenced by other declarations.<br>"
						+ "This might cause model inconsitency. Some declarations are going <br>" +
								"to be modified automatically. Do you want to continue?</html>"));

			dialog.setVisible(true);
			dialog.setOptionType(JOptionPane.YES_NO_OPTION);
			getGUISeSAmDeclaration().getFrameFactory().addModalPanel(null, "Warning", dialog);

			if ((!((dialog.getValue() == null) || (!(dialog.getValue() instanceof Integer))))
				&& (((Integer)dialog.getValue()).intValue() == JOptionPane.NO_OPTION))
				return false;
		}
		return true;
	}

	public void eventRemove(ActionEvent actionEvent) {
		Object selectedObj = getUserTypeAttributeList().getSelectedValue();
		if (selectedObj == null)
			return;
		if (!askAreYouSure())
			return;

		Assert.assertTrue((selectedObj instanceof IComposedTypeAttribute));

		//getMyComposedType().removeAttribute((IComposedTypeAttribute)selectedObj);
		RemoveUserTypeAttribute.execute(getGUISeSAmDeclaration().getSeSAmDeclaration(), (IComposedTypeAttribute)selectedObj);
		getUserTypeAttributeList().setListData(
			CollectionsHelper.asArray(getMyComposedType().getOwnAttributes()));
	}

	private JTextField getAttributeDefinitionNameLabel() {
		if (ivjAttributeDefinitionNameLabel == null) {
			try {
				ivjAttributeDefinitionNameLabel = new BetterBoundJTextField();
				ivjAttributeDefinitionNameLabel.setName("AttributeDefinitionNameLabel");
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjAttributeDefinitionNameLabel;
	}

	private JPanel getAttributeDefinitionPanel() {
		if (ivjAttributeDefinitionPanel == null) {
			try {
				ivjAttributeDefinitionPanel = new JPanel();
				ivjAttributeDefinitionPanel.setName("AttributeDefinitionPanel");
				ivjAttributeDefinitionPanel.setBorder(
					BorderFactory.createCompoundBorder(
						new BetterTitledBorder("Selected Attribute"),
						BorderFactory.createEmptyBorder(0, 3, 0, 3)));
				ivjAttributeDefinitionPanel.setLayout(new java.awt.GridBagLayout());

				Box nameBox = Box.createHorizontalBox();
				nameBox.add(new JLabel("Name:  "));
				nameBox.add(getAttributeDefinitionNameLabel());

				nameBox.add(Box.createHorizontalStrut(3));
				nameBox.add(Box.createHorizontalGlue());
				nameBox.add(getWriteableJCheckBox());

				GridBagConstraints constraintsAttributeDefinitionNameLabel_2 = new GridBagConstraints();
				constraintsAttributeDefinitionNameLabel_2.gridx = 0;
				constraintsAttributeDefinitionNameLabel_2.gridy = 0;
				constraintsAttributeDefinitionNameLabel_2.fill = GridBagConstraints.HORIZONTAL;
				constraintsAttributeDefinitionNameLabel_2.weightx = 1.0;
				constraintsAttributeDefinitionNameLabel_2.anchor = GridBagConstraints.NORTHWEST;

				getAttributeDefinitionPanel().add(nameBox, constraintsAttributeDefinitionNameLabel_2);

				GridBagConstraints constraintsSpecificTypeDefinePanel_2 = new GridBagConstraints();
				constraintsSpecificTypeDefinePanel_2.gridx = 0;
				constraintsSpecificTypeDefinePanel_2.gridy = 2;
				constraintsSpecificTypeDefinePanel_2.weightx = 1.0;
				constraintsSpecificTypeDefinePanel_2.weighty = 1.0;
				constraintsSpecificTypeDefinePanel_2.anchor = GridBagConstraints.NORTHWEST;
				constraintsSpecificTypeDefinePanel_2.insets.top = 11;
				getAttributeDefinitionPanel().add(
					getSpecificTypeDefinePanel(),
					constraintsSpecificTypeDefinePanel_2);

				SwingMoreUtilities.setEnableAllComponentsRecursive(getAttributeDefinitionPanel(), false);

				//				getSpecificTypeDefinePanel().setEnabled(false);
				//				//getAttributeDefinitionNameLabel().setEditable(false);
				//				getAttributeDefinitionNameLabel().setEnabled(false);
				//				getWriteableJCheckBox().setEnabled(false);

				getAttributeDefinitionNameLabel()
					.addPropertyChangeListener("text", new PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent evt) {
						eventAttributeAltered();
					}
				});
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjAttributeDefinitionPanel;
	}

	private JPanel getInheritanceDefinitionPanel() {
		if (ivjInheritanceDefinitionPanel == null) {
			try {
				ivjInheritanceDefinitionPanel = new JPanel();
				ivjInheritanceDefinitionPanel.setName("InheritanceDefinitionPanel");
				ivjInheritanceDefinitionPanel.setBorder(
					BorderFactory.createCompoundBorder(
						new BetterTitledBorder("Inheritance"),
						BorderFactory.createEmptyBorder(0, 3, 0, 3)));
				ivjInheritanceDefinitionPanel.setLayout(new FlowLayout());

				Assert.assertNotNull(getGUISeSAmDeclaration());

				ivjComboBoxFather = getComboBoxPossibleFathers();
				ivjComboBoxFather.setRenderer(new NamedCellRendererAdapter(ivjComboBoxFather.getRenderer()));
				ivjComboBoxFather.setSelectedItem(RootComposedType.getInstance());

				Box nameBox = Box.createHorizontalBox();
				nameBox.add(new JLabel("Father:  "));
				nameBox.add(ivjComboBoxFather);
				getInheritanceDefinitionPanel().add(nameBox);

				ivjComboBoxFather.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						eventFatherSelectionChanged();
					}
				});
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjInheritanceDefinitionPanel;
	}

	private JComboBox getComboBoxPossibleFathers() {
		if (ivjComboBoxFather == null) {
			ivjComboBoxFather = new JComboBox();
			updateComboBoxFather(getMyComposedType());
		}
		return ivjComboBoxFather;
	}

	private void updateComboBoxFather(final ISpecificComposedType son) {
		Object selectedItem = ivjComboBoxFather.getSelectedItem();
		ivjComboBoxFather
			.setModel(new DefaultComboBoxModel(
				CollectionsHelper
				.asArray(
					CollectionsHelper
					.select(
						TypeManager.getInstance().getAllComposedTypes(
							getGUISeSAmDeclaration().getSeSAmDeclaration()),
						new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				return (
					(son == null) || ((son != obj) && (!(son.isMoreSpecific((ISpecificComposedType)obj)))));
			}
		}))));
		if (((DefaultComboBoxModel)ivjComboBoxFather.getModel()).getIndexOf(selectedItem) > -1)
			ivjComboBoxFather.setSelectedItem(selectedItem);
	}

	/**
	 * Set the MyComposedType to a new value.
	 */
	public IDeclaration getDeclaration() {
		return getUserTypeDeclaration();
	}

	private JButton getJButtonRemove() {
		if (ivjJButtonRemove == null) {
			try {
				ivjJButtonRemove = new JButton();
				ivjJButtonRemove.setName("JButton1");
				ivjJButtonRemove.setText("Remove");
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJButtonRemove;
	}

	private JButton getJButtonNew() {
		if (ivjJButtonNew == null) {
			try {
				ivjJButtonNew = new JButton();
				ivjJButtonNew.setName("JButton2");
				ivjJButtonNew.setText("New");
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJButtonNew;
	}

	private JLabel getJLabelAttributes() {
		if (ivjJLabelAttributes == null) {
			try {
				ivjJLabelAttributes = new JLabel();
				ivjJLabelAttributes.setName("JLabel2");
				ivjJLabelAttributes.setText("Attributes:");
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJLabelAttributes;
	}

	private JPanel getAttributeListPanel() {
		if (ivjAttributeListPanel == null) {
			try {

				ivjAttributeListPanel = new JPanel();
				ivjAttributeListPanel.setName("JPanel4");
				//ivjJPanel4.setPreferredSize(new java.awt.Dimension(200, 245));
				ivjAttributeListPanel.setLayout(new java.awt.GridBagLayout());

				GridBagConstraints constraintsJLabel2 = new GridBagConstraints();
				constraintsJLabel2.gridx = 0;
				constraintsJLabel2.gridy = 2;
				constraintsJLabel2.anchor = GridBagConstraints.WEST;
				getAttributeListPanel().add(getJLabelAttributes(), constraintsJLabel2);

				GridBagConstraints constraintsUserTypeAttributeList = new GridBagConstraints();
				constraintsUserTypeAttributeList.gridx = 0;
				constraintsUserTypeAttributeList.gridy = 3;
				constraintsUserTypeAttributeList.fill = GridBagConstraints.BOTH;
				constraintsUserTypeAttributeList.insets.top = 3;
				constraintsUserTypeAttributeList.weightx = 1.0;
				constraintsUserTypeAttributeList.weighty = 1.0;
				getAttributeListPanel().add(
					new JScrollPane(getUserTypeAttributeList()),
					constraintsUserTypeAttributeList);

				JPanel buttonPanel = new JPanel();
				buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 0, 0));

				buttonPanel.add(getJButtonNew());
				buttonPanel.add(Box.createHorizontalStrut(3));
				buttonPanel.add(getJButtonRemove());

				GridBagConstraints constraintsJButtonPanel = new GridBagConstraints();
				constraintsJButtonPanel.gridx = 0;
				constraintsJButtonPanel.gridy = 4;
				constraintsJButtonPanel.weightx = 1;
				constraintsJButtonPanel.fill = GridBagConstraints.HORIZONTAL;
				constraintsJButtonPanel.anchor = GridBagConstraints.SOUTHEAST;
				constraintsJButtonPanel.insets.top = 3;
				getAttributeListPanel().add(buttonPanel, constraintsJButtonPanel);

			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjAttributeListPanel;
	}

	private SpecificComposedType getMyComposedType() {
		return ivjMyComposedType;
	}

	private IComposedTypeAttribute getMySelectedAttribute() {
		return ivjMySelectedAttribute;
	}

	private de.uniwue.ki.gui.mas.basics.type.SpecificTypeDefinePanel getSpecificTypeDefinePanel() {
		if (ivjSpecificTypeDefinePanel == null) {
			try {
				ivjSpecificTypeDefinePanel = new SpecificTypeDefinePanel();
				ivjSpecificTypeDefinePanel.setName("SpecificTypeDefinePanel");
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjSpecificTypeDefinePanel;
	}

	private JList getUserTypeAttributeList() {
		if (ivjUserTypeAttributeList == null) {
			try {
				ivjUserTypeAttributeList = new JList();
				ivjUserTypeAttributeList.setName("UserTypeAttributeList");
				ivjUserTypeAttributeList.setCellRenderer(
					new NamedCellRendererAdapter(ivjUserTypeAttributeList.getCellRenderer()));

				ivjUserTypeAttributeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjUserTypeAttributeList;
	}

	private void handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}

	private void initConnections() {
		getUserTypeAttributeList().addListSelectionListener(ivjEventHandler);
		getJButtonNew().addActionListener(ivjEventHandler);
		getJButtonRemove().addActionListener(ivjEventHandler);
		getSpecificTypeDefinePanel().addPropertyChangeListener(ivjEventHandler);
	}

	private void initialize() {
		try {
			setName("ComposedTypeDefineDefinePanel");
			setLayout(new GridBagLayout());
			setSize(443, 233);

			GridBagConstraints constraintsAttributeListPanel = new GridBagConstraints();
			constraintsAttributeListPanel.gridx = 0;
			constraintsAttributeListPanel.gridy = 0;
			constraintsAttributeListPanel.gridheight = 2;
			constraintsAttributeListPanel.fill = GridBagConstraints.BOTH;
			constraintsAttributeListPanel.weightx = 1.0;
			constraintsAttributeListPanel.weighty = 1.0;
			constraintsAttributeListPanel.insets.left = 3;
			constraintsAttributeListPanel.insets.right = 3;
			add(getAttributeListPanel(), constraintsAttributeListPanel);

			GridBagConstraints constraintsAttributeDefinitionPanel = new GridBagConstraints();
			constraintsAttributeDefinitionPanel.gridx = 1;
			constraintsAttributeDefinitionPanel.gridy = 0;
			constraintsAttributeDefinitionPanel.fill = GridBagConstraints.BOTH;
			constraintsAttributeDefinitionPanel.weightx = 1.0;
			constraintsAttributeDefinitionPanel.weighty = 1.0;
			constraintsAttributeDefinitionPanel.insets.left = 3;
			constraintsAttributeDefinitionPanel.insets.right = 3;
			add(getAttributeDefinitionPanel(), constraintsAttributeDefinitionPanel);

			GridBagConstraints constraintsInheritanceDefinitionPanel = new GridBagConstraints();
			constraintsInheritanceDefinitionPanel.gridx = 1;
			constraintsInheritanceDefinitionPanel.gridy = 1;
			constraintsInheritanceDefinitionPanel.fill = GridBagConstraints.BOTH;
			constraintsInheritanceDefinitionPanel.weightx = 1.0;
			constraintsInheritanceDefinitionPanel.gridwidth = 1;
			constraintsInheritanceDefinitionPanel.weighty = 1.0;
			constraintsInheritanceDefinitionPanel.insets.left = 3;
			constraintsInheritanceDefinitionPanel.insets.right = 3;
			add(getInheritanceDefinitionPanel(), constraintsInheritanceDefinitionPanel);

			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	public void setDeclaration(IDeclaration decl) {
		boolean memorize = interactiveChange;
		interactiveChange=false;
		setUserTypeDeclaration((IUserTypeDeclaration)decl);
		interactiveChange=memorize;
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		getSpecificTypeDefinePanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	private void setMyComposedType(SpecificComposedType newValue) {
		if (ivjMyComposedType != newValue) {
			try {
				ivjMyComposedType = newValue;
				if (newValue != null) {
					getUserTypeAttributeList().setListData(
						CollectionsHelper.asArray(newValue.getOwnAttributes()));
					if (newValue.getAttributeListSize() > 0)
						getUserTypeAttributeList().setSelectedIndex(0);
					if (newValue.getFather() != null) {
						getComboBoxFather().setSelectedItem(newValue.getFather());
					}
				} else {
					eventAttributeSelectionChanged();
				}
				updateComboBoxFather(newValue);
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
	}

	private void setMySelectedAttribute(IComposedTypeAttribute newValue) {
		if (ivjMySelectedAttribute != newValue) {
			try {
				ivjMySelectedAttribute = newValue;
				SwingMoreUtilities.setEnableAllComponentsRecursive(
					getAttributeDefinitionPanel(),
					newValue != null);

				if (newValue == null) {
					//					//getAttributeDefinitionComboBox().setEditable(false);
					//					getSpecificTypeDefinePanel().setEnabled(false);
					//					//getAttributeDefinitionNameLabel().setEditable(false);
					//					getAttributeDefinitionNameLabel().setEnabled(false);
					//					getWriteableJCheckBox().setEnabled(false);

				} else {
					//					//getAttributeDefinitionComboBox().setEditable(true);
					//					getSpecificTypeDefinePanel().setEnabled(true);
					//					//getAttributeDefinitionNameLabel().setEditable(true);
					//					getAttributeDefinitionNameLabel().setEnabled(true);
					//					getWriteableJCheckBox().setEnabled(true);

					getSpecificTypeDefinePanel().setSpecificType(newValue.getAttributeType());
					getAttributeDefinitionNameLabel().setText(newValue.getAttributeName());
					getWriteableJCheckBox().setSelected(newValue.isWriteable());
				}
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
	}

	protected JCheckBox getWriteableJCheckBox() {
		if (_writeableJCheckBox == null) {
			_writeableJCheckBox = new JCheckBox(new AbstractBoundAction("writeable") {
				public void doit() {
					if (getMySelectedAttribute() != null){
						getMySelectedAttribute().setWriteable(getWriteableJCheckBox().isSelected());
						eventAttributeAltered();	
					}
				}
			});
		}

		return _writeableJCheckBox;
	}

	public IUserTypeDeclaration getUserTypeDeclaration() {
		return _userTypeDeclaration;
	}

	public void setUserTypeDeclaration(IUserTypeDeclaration userTypeDeclaration) {
		_userTypeDeclaration = userTypeDeclaration;

		if (userTypeDeclaration != null) {
			Assert.assertNotNull(userTypeDeclaration.getSpecificComposedType());
			setMyComposedType(userTypeDeclaration.getSpecificComposedType());
		}
	}

	public JComboBox getComboBoxFather() {
		return ivjComboBoxFather;
	}

}