package de.uniwue.ki.gui.mas.basics.type.user;

import java.util.Iterator;

import javax.swing.*;

import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeAttribute;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedType;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.util.name.INamed;

/**
 * @author herrler
 */

public class UserTypeDeclarationCellRendererAdapter extends NamedCellRendererAdapter {
	public static final ImageIcon COMPOSED_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-composed-type.gif");

	public static final ImageIcon ENUM_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-enum-type.gif");

	public UserTypeDeclarationCellRendererAdapter(ListCellRenderer cellRenderer) {
		super(cellRenderer);
	}

	protected static String getMarkers(IUserTypeDeclaration userTypeDeclaration) {
		StringBuffer marker = new StringBuffer();

		if (userTypeDeclaration.isComposedType()) {
			marker.append("composed");
		} else {
			marker.append("enumeration");
		}

		return marker.toString();
	}

	protected String getName(INamed value) {
		if (value instanceof IUserTypeDeclaration) {
			return getUserTypeDeclarationText((IUserTypeDeclaration) value);

		} else {
			return super.getName(value);
		}
	}

	public static String getUserTypeDeclarationToolTipText(IUserTypeDeclaration userTypeDeclaration) {
		StringBuffer buff = new StringBuffer();

		buff.append("<strong>");
		NamedCellRendererAdapter.appendQuotedString(userTypeDeclaration.getName(), buff);
		buff.append("</strong> (");
		NamedCellRendererAdapter.appendQuotedString(getMarkers(userTypeDeclaration), buff);
		buff.append(")");

		if (userTypeDeclaration.isComposedType()) {
			SpecificComposedType compType = userTypeDeclaration.getSpecificComposedType();
			Iterator it = compType.getAttributes();

			buff.append("<P>");
			while (it.hasNext()) {
				IComposedTypeAttribute element = (IComposedTypeAttribute) it.next();
				NamedCellRendererAdapter.appendQuotedString(element.getName(), buff);
				buff.append("<BR>");
			}

		}

		return "<html>" + buff.toString();
	}

	public static String getUserTypeDeclarationText(IUserTypeDeclaration userTypeDeclaration) {
		StringBuffer buff = new StringBuffer();

		buff.append("<strong>");
		NamedCellRendererAdapter.appendQuotedString(userTypeDeclaration.getName(), buff);
		buff.append("</strong> (");
		NamedCellRendererAdapter.appendQuotedString(getMarkers(userTypeDeclaration), buff);
		buff.append(")");

		return "<html>" + buff.toString();
	}

	protected String getToolTipText(INamed value, String name) {
		if (value instanceof IUserTypeDeclaration) {
			return getUserTypeDeclarationToolTipText((IUserTypeDeclaration) value);

		} else {
			return super.getName(value);
		}
	}

	protected Icon getIcon(INamed value) {
		if (value instanceof IUserTypeDeclaration) {
			return getMarkerIcon((IUserTypeDeclaration) value);
		} else {
			return super.getIcon(value);
		}
	}

	public static Icon getMarkerIcon(IUserTypeDeclaration userTypeDeclaration) {
		if (userTypeDeclaration.isComposedType()) {
			return COMPOSED_ICON;
		} else {
			return ENUM_ICON;
		}
	}
}
