package de.uniwue.ki.gui.mas.behaviour.activityGraph;
import java.awt.Container;
import java.util.*;

import de.uniwue.ki.gui.mas.model.DefaultDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.gui.mas.util.declaration.action.RemoveMutableListModelAction;
import de.uniwue.ki.gui.mas.util.declaration.action.RenameDocumentableDeclarationAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;

/**
 * Creation date: (05.02.2002 11:38:36)
 * @author Christoph Oechslein
 */

public class ActionActivityGraphLeafDeclarationPanelActionFactory
	extends DefaultDeclarationPanelActionFactory {

	private final class ActionAtomicActivityDeclarationPanelFactory extends AbstractDeclarationPanelFactory {
		private final IActivityAgentClassDeclaration _activityAgentClassDeclaration;

		public ActionAtomicActivityDeclarationPanelFactory(IActivityAgentClassDeclaration activityAgentClassDeclaration) {
			super();

			_activityAgentClassDeclaration = activityAgentClassDeclaration;
		}

		public IDeclarationPanel createDeclarationPanel(final IDeclaration declaration) {
			ActionActivityGraphLeafDeclarationPanel actionAtomicActivityDeclarationPanel =
				new ActionActivityGraphLeafDeclarationPanel();

			actionAtomicActivityDeclarationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
			actionAtomicActivityDeclarationPanel.setActivityAgentClassDeclaration(
				_activityAgentClassDeclaration);

			return actionAtomicActivityDeclarationPanel;
		}

		public String getDeclarationTitle(IDeclaration declaration) {
			return "Action Activity";
		}
	}

	public ActionActivityGraphLeafDeclarationPanelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super(guiSeSAmDeclaration);
	}

	public boolean getCanProduceDeclarationActions(Container parent, IDeclaration anObject) {
		if (!(anObject instanceof IActivityGraphLeafDeclaration))
			return false;

		return getSeSAmDeclarationTreeModel().getUserObjectParentOf(
			getSeSAmDeclarationTreeModel().getUserObjectParentOf(anObject))
			instanceof IActivityGraphDeclaration;
	}

	protected Iterator getMoreDeclarationActions(Container parent, IDeclaration declaration) {
		List actions = new LinkedList();

		if ((declaration instanceof IActionActivityGraphLeafDeclaration)
			&& (declaration instanceof IDocumentableDeclaration)) {
			actions.add(new RenameDocumentableDeclarationAction(parent, declaration));
		}

		if (declaration.canBeRemoved()) {

			Object parentOfDeclaration = getSeSAmDeclarationTreeModel().getUserObjectParentOf(declaration);

			if (parentOfDeclaration instanceof IMutableListModel) {
				actions.add(
					new RemoveMutableListModelAction(declaration, (IMutableListModel)parentOfDeclaration));

			} else if (parentOfDeclaration instanceof IMutableListModelAndFactory) {
				actions.add(
					new RemoveMutableListModelAction(
						declaration,
						((IMutableListModelAndFactory)parentOfDeclaration).getMutableListModel()));
			}

			/*
			if (parent instanceof SeSAmDeclarationTreePanel)
			    actions.add(DefaultMutableListModelPanelActionFactory.createRefreshAction(getSeSAmDeclarationTreeModel(), declaration));
			*/
		}

		actions.add(new InspectAction(declaration));

		return actions.iterator();
	}

	public IDeclarationPanelFactory getDeclarationPanelFactory(Container parent, IDeclaration anObject) {
		if (!(anObject instanceof IActionActivityGraphLeafDeclaration))
			return null;

		else {
			IActivityGraphDeclaration composedActivityDeclaration =
				(IActivityGraphDeclaration)getSeSAmDeclarationTreeModel().getUserObjectParentOf(
					getSeSAmDeclarationTreeModel().getUserObjectParentOf(anObject));

			IActivityAgentClassDeclaration activityAgentClassDeclaration =
				searchActivityAgentClassDeclaration(composedActivityDeclaration);

			return new ActionAtomicActivityDeclarationPanelFactory(activityAgentClassDeclaration);
		}
	}

	private IActivityAgentClassDeclaration searchActivityAgentClassDeclaration(IActivityGraphDeclaration activityGraphDeclaration) {
		if (activityGraphDeclaration.getParentActivityGraphComposedNodeDeclaration() == null)
			return (IActivityAgentClassDeclaration)getSeSAmDeclarationTreeModel().getUserObjectParentOf(
				getSeSAmDeclarationTreeModel().getUserObjectParentOf(activityGraphDeclaration));
		else
			return searchActivityAgentClassDeclaration(
				activityGraphDeclaration
					.getParentActivityGraphComposedNodeDeclaration()
					//.getInnerActivityGraphDeclaration()  //caused an endless recursion - rh 11.Juni.2004
					.getActivityGraphDeclaration()
					);
					
	}
}