package de.uniwue.ki.gui.mas.behaviour.activityGraph;
import java.awt.datatransfer.*;
import java.beans.Beans;
import java.io.IOException;

import javax.swing.*;

import mygale.mcge.cmd.*;
import mygale.mcge.core.*;
import mygale.mcge.fig.IFigEdgeFactory;
import mygale.mcge.fig.IFigNodeFactory;
import mygale.mcge.mode.ModeMouseLocation;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.fig.FigActivityGraphNodeDeclarationFactory;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.fig.FigActivityGraphRuleDeclarationFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.dnd.ObjectDataFlavor;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;

/**
 * Creation date: (21.03.2001 17:09:20)
 * @author Christoph Oechslein
 */

public class ActivityGraphDeclarationGraph extends JGraph {
	private static final IActivityGraphDeclaration _dummyComposedActivityDeclaration =
		ActivityGraphDeclarationFactory.createActivityGraphDeclaration();

	public ActivityGraphDeclarationGraph(JGraphEditor graphEditor) {

		this(
			graphEditor,
			FigActivityGraphNodeDeclarationFactory.getInstance(),
			FigActivityGraphRuleDeclarationFactory.getInstance());
	}

	public ActivityGraphDeclarationGraph(
		JGraphEditor graphEditor,
		IFigNodeFactory figNodeFactory,
		IFigEdgeFactory figEdgeFactory) {
		super(graphEditor, figNodeFactory, figEdgeFactory);
		initialize();
	}
	
//	//TODO remove
//	public JGraphEditor getGraphEditor() {
//		return super.getGraphEditor();
//	}

	public void setGraphEditor(JGraphEditor newJGraphEditor) {
		setGraphEditor(newJGraphEditor);

		setNetGraphModel(getActivityGraphDeclaration());
	}

	public IActivityAgentClassDeclaration getActivityAgentClassDeclaration() {
		if (getActivityGraphDeclarationGraphEditor() != null) {
			return getActivityGraphDeclarationGraphEditor().getActivityAgentClassDeclaration();
		} else {
			return null;
		}
	}

	public IActivityGraphDeclaration getActivityGraphDeclaration() {
		return (IActivityGraphDeclaration)getNetGraphModel();
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		if (getActivityGraphDeclarationGraphEditor() != null) {
			return getActivityGraphDeclarationGraphEditor().getGUISeSAmDeclaration();
		} else {
			return null;
		}
	}

	public void setActivityGraphDeclaration(IActivityGraphDeclaration composedActivityDeclaration) {
		setNetGraphModel(composedActivityDeclaration);
	}

	protected ActivityGraphDeclarationGraphEditor getActivityGraphDeclarationGraphEditor() {
		return (ActivityGraphDeclarationGraphEditor)getGraphEditor();
	}

	private void initialize() {
		setName("ComposedActivityDeclarationGraph");
		setSize(10, 10);

		setNetGraphModel(getActivityGraphDeclaration());

	}

	protected void makeGraphDiagram() {
		if (Beans.isDesignTime())
			setNetGraphModel(_dummyComposedActivityDeclaration);
	}

	protected void makeModeManager() {
		super.makeModeManager();
		getModeManager().beginMode(new ModeMouseLocation(getModeManager()));
	}

	// TODO Print-Button inserted for printing ActivityGraph; Mhlberger 10.10.03
	protected JToolBar createPersonalizedToolBar() {
		JToolBar toolbar = new JGraphToolBar();
		CmdDelete cmdDelete = new CmdDelete(getGraphEditor(), 24);
		getSelection().addCmdToInform(cmdDelete);

		toolbar.setFloatable(false);

		toolbar.add(new CmdSelect(getGraphEditor(), 24));
		toolbar.add(cmdDelete);
		
		toolbar.addSeparator();
		toolbar.add(new CmdSaveAs(getGraphEditor(), 24));
		toolbar.add(new CmdPrintGraph(getGraphEditor(), 24));  // Print-Button
		
		toolbar.addSeparator();

		toolbar.add(new CmdZoomIn(getGraphEditor(), 24));
		toolbar.add(new CmdZoomStandard(getGraphEditor(), 24));
		toolbar.add(new CmdZoomOut(getGraphEditor(), 24));

		toolbar.addSeparator();

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add an action activity",
				24,
				"general/Node",
				new ActionActivityGraphLeafDeclarationFactory(getActivityGraphDeclaration())));
		toolbar.add(
			new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add a rule",
				24,
				"general/EdgeArrow",
				ActivityGraphRuleDeclarationFactory.getInstance(),
				_figEdgeFactory));
		toolbar.add(
			new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add an otherwise rule",
				24,
				"general/EdgeDashedArrow",
				OtherwiseActivityEdgeDeclarationFactory.getInstance(),
				_figEdgeFactory));

		toolbar.addSeparator();

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add an activity graph node",
				24,
				"general/ComplexNode",
				new ActivityGraphComposedNodeDeclarationFactory(true)));

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add a decision node",
				24,
				"general/RhombusNode",
				new DecisionGraphLeafDeclarationFactory(getActivityGraphDeclaration())));

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add a signal out node",
				24,
				"general/SignalOutNode",
				new ActionActivityGraphLeafDeclarationFactory(
					getActivityGraphDeclaration(),
					IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE)));

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add a signal in node",
				24,
				"general/SignalInNode",
				new ActionActivityGraphLeafDeclarationFactory(
					getActivityGraphDeclaration(),
					IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE)));

		toolbar.addSeparator();

		toolbar.add(
			new CmdCreateNode(
				getGraphEditor(),
				null,
				"add a documentation node",
				24,
				"general/DocuNode",
				new GraphDocumentationLeafDeclarationFactory(getActivityGraphDeclaration())));

		toolbar.add(
			new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add documentation edge",
				24,
				"general/EdgeDotted",
				DocumentEdgeDeclerationFactory.getInstance(),
				_figEdgeFactory));

		return toolbar;
	}

	protected void makeToolBar() {
		setToolbar(createPersonalizedToolBar());
		((JToolBar)getToolbar()).setBorder(BorderFactory.createEmptyBorder(4, 3, 5, 3));
	}

	public JPopupMenu getPopupMenu() {
		JPopupMenu popupMenu = super.getPopupMenu();

		Clipboard clipboard = getGUISeSAmDeclaration().getClipboard();
		final Transferable transferable = clipboard.getContents(null);

		if (transferable != null) {
			popupMenu.add(new AbstractBoundAction("Paste") {
				public void doit() {

					try {
						IActivityGraphNodeDeclaration newActivityDeclaration =
							(IActivityGraphNodeDeclaration)transferable.getTransferData(
								ObjectDataFlavor.getDataFlavor(IActivityGraphNodeDeclaration.class));
						getActivityGraphDeclaration().addActivityGraphNodeDeclaration(newActivityDeclaration);

					} catch (IOException exception) {
						throw new RuntimeException(exception);
					} catch (UnsupportedFlavorException exception) {
					}
				}
			});
		}

		return popupMenu;
	}
}