package de.uniwue.ki.gui.mas.behaviour.activityGraph.action;
import java.util.*;

import javax.swing.Icon;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphEdgeDeclaration;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphTransitionEdgeDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (09.11.2001 16:55:20)
 * @author Christoph Oechslein
 */
public class ComposeActivitiesToAComposedActivityDeclarationAction extends AbstractBoundAction {
	private final List _activityDeclarationList;
	private final IActivityGraphDeclaration _parentActivityGraphDeclaration;

	public ComposeActivitiesToAComposedActivityDeclarationAction(List activityDeclarationList) {
		this(activityDeclarationList, "Compose");
	}

	public ComposeActivitiesToAComposedActivityDeclarationAction(List activityDeclarationList, String name) {
		super(name);

		if (Assert.ASSERTION_ON)
			Assert.assertTrue(!activityDeclarationList.isEmpty());

		_activityDeclarationList = activityDeclarationList;
		_parentActivityGraphDeclaration =
			((IActivityGraphNodeDeclaration)_activityDeclarationList.get(0)).getActivityGraphDeclaration();

		if (Assert.ASSERTION_ON) {
			checkSameParents();
		}
	}

	public ComposeActivitiesToAComposedActivityDeclarationAction(
		List activityDeclarationList,
		String name,
		Icon icon) {
		super(name, icon);

		Assert.assertTrue(!activityDeclarationList.isEmpty());

		_activityDeclarationList = activityDeclarationList;
		_parentActivityGraphDeclaration =
			((IActivityGraphNodeDeclaration)_activityDeclarationList.get(0)).getActivityGraphDeclaration();

		if (Assert.ASSERTION_ON) {
			checkSameParents();
		}
	}

	private void checkSameParents() {
		Iterator it = _activityDeclarationList.iterator();
		while (it.hasNext()) {
			IActivityGraphNodeDeclaration currActivityDeclaration = (IActivityGraphNodeDeclaration)it.next();

			Assert.assertSame(
				_parentActivityGraphDeclaration,
				currActivityDeclaration.getActivityGraphDeclaration());
		}
	}

	public IActivityGraphComposedNodeDeclaration compose() {
		Assert.assertTrue(isPossibleToCompose());

		IActivityGraphNodeDeclaration startActivityDeclaration =
			getActivityWithOneIncomingOrOutgoingLink(true);
		IActivityGraphNodeDeclaration endActivityDeclaration =
			getActivityWithOneIncomingOrOutgoingLink(false);

		IActivityGraphComposedNodeDeclaration newActivityGraphComposedNodeDeclaration =
			ActivityGraphComposedNodeDeclarationFactory.createActivityGraphComposedNodeDeclaration();

		IActivityGraphDeclaration newActivityGraphDeclaration =
			newActivityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration();

		_parentActivityGraphDeclaration.addActivityGraphNodeDeclaration(
			newActivityGraphComposedNodeDeclaration);

		List ruleEdges = new LinkedList();

		// move start-activity
		if (startActivityDeclaration != null) {
			List edges = CollectionsHelper.asLinkedList(startActivityDeclaration.getNodeEdges());
			_parentActivityGraphDeclaration.removeDeclarationNetNode(
				startActivityDeclaration,
				false);
			newActivityGraphDeclaration.addActivityGraphNodeDeclaration(startActivityDeclaration);

			//create edge from start        
			newActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
				newActivityGraphDeclaration.getEntryActivityGraphLeafDeclaration(),
				startActivityDeclaration);

			ruleEdges.addAll(edges);
		}

		// move end-activity
		if (endActivityDeclaration != null) {
			List edges = CollectionsHelper.asArrayList(endActivityDeclaration.getNodeEdges());

			if (startActivityDeclaration != endActivityDeclaration) {
				_parentActivityGraphDeclaration.removeDeclarationNetNode(
					endActivityDeclaration,
					false);
				newActivityGraphDeclaration.addActivityGraphNodeDeclaration(endActivityDeclaration);
			}

			// create edge to exit
			/* newComposedActivityDeclaration.addChainRuleDeclaration(
			    endActivityDeclaration,
			    newComposedActivityDeclaration.getExitActivityDeclaration()); */

			if (startActivityDeclaration != endActivityDeclaration) {
				ruleEdges.addAll(edges);
			}
		}

		//remove activities 
		Iterator it = _activityDeclarationList.iterator();
		while (it.hasNext()) {
			IActivityGraphNodeDeclaration currIActivityDeclaration = (IActivityGraphNodeDeclaration)it.next();

			if ((currIActivityDeclaration != startActivityDeclaration)
				&& (currIActivityDeclaration != endActivityDeclaration)) {
				List edges = CollectionsHelper.asArrayList(currIActivityDeclaration.getNodeEdges());

				_parentActivityGraphDeclaration.removeDeclarationNetNode(
					currIActivityDeclaration,
					false);

				newActivityGraphDeclaration.addActivityGraphNodeDeclaration(currIActivityDeclaration);

				ruleEdges.addAll(edges);
			}
		}

		relinkEdges(
			newActivityGraphComposedNodeDeclaration,
			startActivityDeclaration,
			endActivityDeclaration,
			ruleEdges);

		return newActivityGraphComposedNodeDeclaration;
	}

	public void doit() {
		if (isPossibleToCompose()) {
			compose();
		}
	}

	private IActivityGraphNodeDeclaration getActivityWithOneIncomingOrOutgoingLink(boolean inOrOutOf) {
		Iterator it = _activityDeclarationList.iterator();

		while (it.hasNext()) {
			IActivityGraphNodeDeclaration currActivityDeclaration = (IActivityGraphNodeDeclaration)it.next();

			if (hasLinkOfActivityInOrOutOfList(currActivityDeclaration, inOrOutOf)) {
				return currActivityDeclaration;
			}
		}

		return null;
	}

	private IActivityGraphEdgeDeclaration getLinkOfActivityInOrOutOfList(
		IActivityGraphNodeDeclaration currIActivityDeclaration,
		boolean inOrOutOf) {

		Iterator rules =
			inOrOutOf
				? currIActivityDeclaration.getIncomingNodeEdges()
				: currIActivityDeclaration.getOutgoingNodeEdges();

		while (rules.hasNext()) {
			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)rules.next();

			Assert.assertSame(
				currIActivityDeclaration,
				inOrOutOf
					? currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration()
					: currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration());

			IActivityGraphNodeDeclaration otherActivityDeclaration =
				inOrOutOf
					? currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
					: currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration();

			if (!_activityDeclarationList.contains(otherActivityDeclaration)) {
				return currIActivityEdgeDeclaration;
			}
		}

		return null;
	}

	private final boolean hasLinkOfActivityInOrOutOfList(
		IActivityGraphNodeDeclaration currIActivityDeclaration,
		boolean inOrOutOf) {

		return getLinkOfActivityInOrOutOfList(currIActivityDeclaration, inOrOutOf) != null;
	}

	private boolean hasOneOrLowerLinkInOrOutOfList(boolean inOrOutOf) {
		Iterator it = _activityDeclarationList.iterator();
		boolean alreadyFoundOneLink = false;

		while (it.hasNext()) {
			IActivityGraphNodeDeclaration currActivityDeclaration = (IActivityGraphNodeDeclaration)it.next();

			if (!alreadyFoundOneLink) {
				alreadyFoundOneLink = hasLinkOfActivityInOrOutOfList(currActivityDeclaration, inOrOutOf);

			} else if (hasLinkOfActivityInOrOutOfList(currActivityDeclaration, inOrOutOf)) {
				return false;
			}

		}

		return true;
	}

	public boolean isPossibleToCompose() {
		if (_activityDeclarationList
			.contains(_parentActivityGraphDeclaration.getEntryActivityGraphLeafDeclaration())
			|| _activityDeclarationList.contains(
				_parentActivityGraphDeclaration.getExitActivityGraphLeafDeclaration())
			|| _activityDeclarationList.contains(
				_parentActivityGraphDeclaration.getEmergencyActivityGraphLeafDeclaration())) {
			return false;
		}

		if (!hasOneOrLowerLinkInOrOutOfList(true)) {
			return false;
		}

		if (!hasOneOrLowerLinkInOrOutOfList(false)) {
			return false;
		}

		return true;
	}

	private void relinkEdge(
		IActivityGraphComposedNodeDeclaration newActivityGraphComposedNodeDeclaration,
		IActivityGraphNodeDeclaration startActivityDeclaration,
		IActivityGraphNodeDeclaration endActivityDeclaration,
		IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration) {

		IActivityGraphDeclaration newActivityGraphDeclaration =
			newActivityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration();

		if ((currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration() == endActivityDeclaration)
			&& (!_activityDeclarationList
				.contains(currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration()))) {

			//from one make two
			_parentActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
				newActivityGraphComposedNodeDeclaration,
				currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration());

			currIActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(
				newActivityGraphDeclaration.getExitActivityGraphLeafDeclaration());

			newActivityGraphDeclaration.addActivityGraphEdgeDeclaration(currIActivityEdgeDeclaration);

			/*
			newComposedActivityDeclaration.addChainRuleDeclaration(
			    endActivityDeclaration,
			    newComposedActivityDeclaration.getExitActivityDeclaration());
			
			currIActivityEdgeDeclaration.setFromActivityDeclaration(
			    newComposedActivityDeclaration);
			
			_parentComposedActivityDeclaration.addChainRuleDeclaration(
			    (IActivityEdgeDeclaration) currIActivityEdgeDeclaration.copy());
			*/

		} else if (
			(currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration() == startActivityDeclaration)
				&& (!_activityDeclarationList
					.contains(currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()))) {
			currIActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(
				newActivityGraphComposedNodeDeclaration);

			_parentActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
				(IActivityGraphTransitionEdgeDeclaration)currIActivityEdgeDeclaration.copy(new HashMap()));

		} else {
			newActivityGraphDeclaration.addActivityGraphEdgeDeclaration(currIActivityEdgeDeclaration);
		}
	}

	private void relinkEdges(
		IActivityGraphComposedNodeDeclaration newActivityGraphComposedNodeDeclaration,
		IActivityGraphNodeDeclaration startActivityDeclaration,
		IActivityGraphNodeDeclaration endActivityDeclaration,
		List edgeList) {

		Iterator it = edgeList.iterator();
		while (it.hasNext()) {
			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)it.next();
			relinkEdge(
				newActivityGraphComposedNodeDeclaration,
				startActivityDeclaration,
				endActivityDeclaration,
				currIActivityEdgeDeclaration);
		}
	}
}