package de.uniwue.ki.gui.mas.behaviour.activityGraph.action;
import java.util.*;

import javax.swing.Icon;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (09.11.2001 16:55:20)
 * @author Christoph Oechslein
 */
public class DeComposeComposedActivityDeclarationToActivitiesAction extends AbstractBoundAction {
	private final IActivityGraphComposedNodeDeclaration _activityGraphComposedNodeDeclaration;

	public DeComposeComposedActivityDeclarationToActivitiesAction(IActivityGraphComposedNodeDeclaration activityGraphComposedNodeDeclaration) {
		this(activityGraphComposedNodeDeclaration, "Decompose");
	}

	public DeComposeComposedActivityDeclarationToActivitiesAction(
		IActivityGraphComposedNodeDeclaration activityGraphComposedNodeDeclaration,
		String name) {
		super(name);

		_activityGraphComposedNodeDeclaration = activityGraphComposedNodeDeclaration;
	}

	public DeComposeComposedActivityDeclarationToActivitiesAction(
		IActivityGraphComposedNodeDeclaration activityGraphComposedNodeDeclaration,
		String name,
		Icon icon) {
		super(name, icon);

		_activityGraphComposedNodeDeclaration = activityGraphComposedNodeDeclaration;
	}

	private IActivityGraphNodeDeclaration createEntryActionAtomicActivityDeclaration() {
		IActivityGraphDeclaration parentActivityGraphDeclaration =
			getActivityGraphComposedNodeDeclaration().getActivityGraphDeclaration();

		IActionActivityGraphLeafDeclaration newEntryActionAtomicActivityDeclaration =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
				parentActivityGraphDeclaration);

		newEntryActionAtomicActivityDeclaration.setShortDescription("Old Entry Activity");
		newEntryActionAtomicActivityDeclaration.setUsesShortDescription(true);

		parentActivityGraphDeclaration.addActivityGraphNodeDeclaration(
			newEntryActionAtomicActivityDeclaration);

		return newEntryActionAtomicActivityDeclaration;
	}

	private final IActivityGraphDeclaration getInnerActivityGraphDeclaration() {
		return getActivityGraphComposedNodeDeclaration().getInnerActivityGraphDeclaration();
	}

	private IActivityGraphNodeDeclaration createEntryActionAtomicActivityDeclarationIfNeeded() {

		List rulesFromEntry =
			CollectionsHelper.asLinkedList(
				getInnerActivityGraphDeclaration()
					.getEntryActivityGraphLeafDeclaration()
					.getActivityGraphEdgeDeclarations());

		if (rulesFromEntry.size() == 1) {
			IActivityGraphTransitionEdgeDeclaration activityArgumentRuleDeclaration =
				(IActivityGraphTransitionEdgeDeclaration)rulesFromEntry.get(0);

			if (isTrue(activityArgumentRuleDeclaration)) {
				getInnerActivityGraphDeclaration().removeDeclarationNetEdge(
					activityArgumentRuleDeclaration,
					false);
				return activityArgumentRuleDeclaration.getToActivityGraphNodeDeclaration();
			}
		}

		return createEntryActionAtomicActivityDeclaration();
	}

	private IActivityGraphNodeDeclaration createExitActionAtomicActivityDeclaration() {
		IActivityGraphDeclaration parentComposedActivityDeclaration =
			getActivityGraphComposedNodeDeclaration().getActivityGraphDeclaration();

		IActionActivityGraphLeafDeclaration newExitActionAtomicActivityDeclaration =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
				parentComposedActivityDeclaration);

		newExitActionAtomicActivityDeclaration.setShortDescription("Old Exit Activity");
		newExitActionAtomicActivityDeclaration.setUsesShortDescription(true);

		parentComposedActivityDeclaration.addActivityGraphNodeDeclaration(
			newExitActionAtomicActivityDeclaration);

		return newExitActionAtomicActivityDeclaration;
	}

	private IActivityGraphNodeDeclaration createExitActionAtomicActivityDeclarationIfNeeded() {
		//precalculation
		List rulesToExit =
			CollectionsHelper.asLinkedList(
				getInnerActivityGraphDeclaration()
					.getExitActivityGraphLeafDeclaration()
					.getIncomingNodeEdges());
		List rulesFromExit =
			CollectionsHelper.asArrayList(getActivityGraphComposedNodeDeclaration().getOutgoingNodeEdges());
		IActivityGraphTransitionEdgeDeclaration ruleFromExit = null;
		IActivityGraphTransitionEdgeDeclaration ruleToExit = null;

		//different cases
		if (rulesToExit.size() == 0) {
			return null;
		} else {
			ruleToExit = (IActivityGraphTransitionEdgeDeclaration)rulesToExit.get(0);
		}

		if ((rulesToExit.size() == 1) && (isTrue(ruleToExit))) {
			getInnerActivityGraphDeclaration().removeDeclarationNetEdge(ruleToExit, false);
			return ruleToExit.getFromActivityGraphNodeDeclaration();
		}

		if (rulesFromExit.size() == 0) {
			createExitActionAtomicActivityDeclaration();
		} else {
			ruleFromExit = (IActivityGraphTransitionEdgeDeclaration)rulesFromExit.get(0);
		}

		if ((rulesToExit.size() == 1) && (rulesFromExit.size() == 1) && (isTrue(ruleFromExit))) {
			//memorize exit
			IActivityGraphNodeDeclaration returnValue = ruleToExit.getFromActivityGraphNodeDeclaration();
			//remove both rules
			getInnerActivityGraphDeclaration().removeDeclarationNetEdge(ruleToExit, false);
			getActivityGraphComposedNodeDeclaration()
				.getActivityGraphDeclaration()
				.removeDeclarationNetEdge(
				ruleFromExit,
				false);

			//relink 
			ruleToExit.setFromActivityGraphNodeDeclaration(
				ruleFromExit.getFromActivityGraphNodeDeclaration());
			ruleToExit.setToActivityGraphNodeDeclaration(ruleFromExit.getToActivityGraphNodeDeclaration());

			//add by parent
			getActivityGraphComposedNodeDeclaration()
				.getActivityGraphDeclaration()
				.addActivityGraphEdgeDeclaration(
				(IActivityGraphTransitionEdgeDeclaration)ruleToExit.copy(new HashMap()));

			return returnValue;
		}

		return createExitActionAtomicActivityDeclaration();
	}

	public void decompose() {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(isPossibleToDecompose());

		IActivityGraphDeclaration parentComposedActivityDeclaration =
			getActivityGraphComposedNodeDeclaration().getActivityGraphDeclaration();

		IActivityGraphNodeDeclaration newEntryActivityDeclaration =
			createEntryActionAtomicActivityDeclarationIfNeeded();

		IActivityGraphNodeDeclaration newExitActivityDeclaration =
			createExitActionAtomicActivityDeclarationIfNeeded();

		List chainRules = new ArrayList();

		Iterator it =
			CollectionsHelper
				.asLinkedList(
					getInnerActivityGraphDeclaration().getActivityGraphNodeDeclarationList().iterator())
				.iterator();

		while (it.hasNext()) {
			IActivityGraphNodeDeclaration currIActivityDeclaration = (IActivityGraphNodeDeclaration)it.next();

			removeAllChainRulesWithoutCallingRemove(currIActivityDeclaration, chainRules);

			if ((currIActivityDeclaration
				!= getInnerActivityGraphDeclaration().getEntryActivityGraphLeafDeclaration())
				&& (currIActivityDeclaration
					!= getInnerActivityGraphDeclaration().getExitActivityGraphLeafDeclaration())
				&& (currIActivityDeclaration
					!= getInnerActivityGraphDeclaration().getEmergencyActivityGraphLeafDeclaration())) {

				parentComposedActivityDeclaration.addActivityGraphNodeDeclaration(currIActivityDeclaration);
			}
		}

		// interne Regeln
		modifyChainRulesAtEntryAndExit(newEntryActivityDeclaration, newExitActivityDeclaration, chainRules);

		it = chainRules.iterator();
		while (it.hasNext()) {
			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)it.next();

			parentComposedActivityDeclaration.addActivityGraphEdgeDeclaration(currIActivityEdgeDeclaration);
		}

		// externe Regeln
		Iterator rules =
			CollectionsHelper
				.asArrayList(getActivityGraphComposedNodeDeclaration().getNodeEdges())
				.iterator();

		parentComposedActivityDeclaration.removeDeclarationNetNode(
			getActivityGraphComposedNodeDeclaration(),
			false);

		while (rules.hasNext()) {
			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)rules.next();

			if (currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
				== getActivityGraphComposedNodeDeclaration()) {

				currIActivityEdgeDeclaration.setFromActivityGraphNodeDeclaration(newExitActivityDeclaration);

				parentComposedActivityDeclaration.addActivityGraphEdgeDeclaration(
					currIActivityEdgeDeclaration);

			} else {
				Assert.assertSame(
					getActivityGraphComposedNodeDeclaration(),
					currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration());

				currIActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(newEntryActivityDeclaration);

				parentComposedActivityDeclaration.addActivityGraphEdgeDeclaration(
					currIActivityEdgeDeclaration);
			}

		}
	}

	public void doit() {
		if (isPossibleToDecompose()) {
			decompose();
		}
	}

	public boolean isPossibleToDecompose() {
		if (getActivityGraphComposedNodeDeclaration().getActivityGraphDeclaration() == null) {
			return false;
		}

		if (!CollectionsHelper
			.isEmpty(
				getInnerActivityGraphDeclaration()
					.getEmergencyActivityGraphLeafDeclaration()
					.getActivityGraphEdgeDeclarations())) {
			return false;
		}

		return true;
	}

	/* Is this just a "true" rule ? */
	private boolean isTrue(IActivityGraphTransitionEdgeDeclaration edge) {
		if (edge instanceof IActivityGraphRuleDeclaration) {
			IActivityGraphRuleDeclaration rule = (IActivityGraphRuleDeclaration)edge;
			return (
				FunctionCallDeclarationFactory.getInstance().isDefaultAtomicFunctionCallDeclaration(
					rule.getConditionFunctionCallDeclaration()));
		} else {
			return false;
		}
	}

	private void modifyChainRulesAtEntryAndExit(
		IActivityGraphNodeDeclaration newEntryActionAtomicActivityDeclaration,
		IActivityGraphNodeDeclaration newExitActionAtomicActivityDeclaration,
		List chainRules) {

		LinkedList modifiedRules = new LinkedList();
		Iterator it = chainRules.iterator();
		while (it.hasNext()) {

			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)it.next();

			if (currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
				== getInnerActivityGraphDeclaration().getEntryActivityGraphLeafDeclaration()) {
				//currIActivityEdgeDeclaration.setFromActivityDeclaration(newEntryActionAtomicActivityDeclaration);
				currIActivityEdgeDeclaration.setBeginNode(newEntryActionAtomicActivityDeclaration);

				it.remove();
				if (currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
					!= currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration()) {
					modifiedRules.add(
						currIActivityEdgeDeclaration.copy(new HashMap()));
				}
			}

			if (currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration()
				== getInnerActivityGraphDeclaration().getExitActivityGraphLeafDeclaration()) {
				//currIActivityEdgeDeclaration.setToActivityDeclaration(newExitActionAtomicActivityDeclaration);
				currIActivityEdgeDeclaration.setEndNode(newExitActionAtomicActivityDeclaration);

				it.remove();
				if (currIActivityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
					!= currIActivityEdgeDeclaration.getToActivityGraphNodeDeclaration()) {
					modifiedRules.add(currIActivityEdgeDeclaration.copy(new HashMap()));
				}
			}
		}

		chainRules.addAll(modifiedRules);
	}

	private void removeAllChainRulesWithoutCallingRemove(
		IActivityGraphNodeDeclaration activityDeclaration,
		List chainRules) {
		IActivityGraphDeclaration parentComposedActivityDeclaration =
			activityDeclaration.getActivityGraphDeclaration();

		Iterator it = CollectionsHelper.asLinkedList(activityDeclaration.getNodeEdges()).iterator();
		while (it.hasNext()) {
			IActivityGraphEdgeDeclaration currIActivityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)it.next();

			parentComposedActivityDeclaration.removeDeclarationNetEdge(
				currIActivityEdgeDeclaration,
				false);

			chainRules.add(currIActivityEdgeDeclaration);
		}
	}

	private IActivityGraphComposedNodeDeclaration getActivityGraphComposedNodeDeclaration() {
		return _activityGraphComposedNodeDeclaration;
	}
}