package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.*;
import java.util.Iterator;

import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;

import junit.framework.Assert;
import mygale.mcge.core.JGraph;
import mygale.mcge.fig.IFigObject;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActionActivityGraphLeafDeclarationPanel;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActivityGraphDeclarationGraph;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationList;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActionActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * Creation date: (14.03.2002 14:32:40)
 * @author Christoph Oechslein
 */
public class FigActionActivityGraphLeafDeclaration extends FigTypeableActivityGraphNodeDeclaration {

	//	the font of the declaration-lists is just "smallerFont"-percent of the original fontsize
	private static final float SMALLER_FONT_FACTOR = 0.9f;

	public FigActionActivityGraphLeafDeclaration() {
		super();
	}

	public void editProperties(Container parent) {
		final ActivityGraphDeclarationGraph composedActivityDeclarationGraph =
			(ActivityGraphDeclarationGraph) parent;

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(composedActivityDeclarationGraph.getActivityAgentClassDeclaration());

		IDeclarationFrameFactory declarationFrameFactory =
			GUISeSAmDeclaration.getDeclarationFrameFactory(
				composedActivityDeclarationGraph.getGUISeSAmDeclaration());

		declarationFrameFactory
			.addPanel(
				composedActivityDeclarationGraph,
				getActionAtomicActivityDeclaration(),
				new AbstractDeclarationPanelFactory() {
			public IDeclarationPanel createDeclarationPanel(IDeclaration declaration) {
				ActionActivityGraphLeafDeclarationPanel actionAtomicActivityDeclarationPanel =
					new ActionActivityGraphLeafDeclarationPanel();

				actionAtomicActivityDeclarationPanel.setGUISeSAmDeclaration(
					composedActivityDeclarationGraph.getGUISeSAmDeclaration());
				actionAtomicActivityDeclarationPanel.setActivityAgentClassDeclaration(
					composedActivityDeclarationGraph.getActivityAgentClassDeclaration());

				return actionAtomicActivityDeclarationPanel;
			}

			public String getDeclarationTitle(IDeclaration declaration) {
				return "Activity";
			}

		}, true, true);
	}

	protected IActionActivityGraphLeafDeclaration getActionAtomicActivityDeclaration() {
		return (IActionActivityGraphLeafDeclaration) getActivityDeclaration();
	}

	protected Iterator getPopupMenuImportantActions(final JGraph graph, boolean forOnlyOneFig) {
		if (forOnlyOneFig) {
			IFigObject currFigObject = (IFigObject) graph.getSelection().getAllFigs().next();
			final IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration =
				(IActionActivityGraphLeafDeclaration) currFigObject.getNetObject();

			JCheckBoxMenuItem jcheckBoxMenuItem = new JCheckBoxMenuItem("Instantly");

			jcheckBoxMenuItem.setSelected(actionAtomicActivityDeclaration.isInstantly());

			AbstractBoundAction.setActionAndFont(jcheckBoxMenuItem, new AbstractBoundAction() {
				public void doit() {
					actionAtomicActivityDeclaration.setInstantly(
						!actionAtomicActivityDeclaration.isInstantly());
				}
			}, false);

			return CollectionsHelper.createConsIterator(
				jcheckBoxMenuItem,
				super.getPopupMenuImportantActions(graph, forOnlyOneFig));

		} else {

			Action changeInstantlyAction = new AbstractBoundAction("Change Instantly") {
				public void doit() {
					Iterator it = graph.getSelection().getAllFigs();
					while (it.hasNext()) {
						IFigObject currFigObject = (IFigObject) it.next();
						if (currFigObject.getNetObject() instanceof IActionActivityGraphLeafDeclaration) {
							IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration =
								(IActionActivityGraphLeafDeclaration) currFigObject.getNetObject();
							actionAtomicActivityDeclaration.setInstantly(
								!actionAtomicActivityDeclaration.isInstantly());
						}

					}
				}
			};

			return CollectionsHelper.createConsIterator(
				changeInstantlyAction,
				super.getPopupMenuImportantActions(graph, forOnlyOneFig));
		}
	}

	protected Rectangle paintBottom(Graphics2D g2d, Rectangle interiorPaintRect) {

		Font oldFont = g2d.getFontMetrics().getFont();
		Font smallFont = oldFont.deriveFont(oldFont.getSize() * SMALLER_FONT_FACTOR);

		g2d.setFont(smallFont);

		Font smallItalicFont = smallFont.deriveFont(Font.ITALIC);

		FunctionActionCallDeclarationList functionActionCallDeclarationListEntry =
			getActionAtomicActivityDeclaration()
				.getEntryFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList();

		FunctionActionCallDeclarationList functionActionCallDeclarationList =
			getActionAtomicActivityDeclaration()
				.getFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList();

		FunctionActionCallDeclarationList functionActionCallDeclarationListExit =
			getActionAtomicActivityDeclaration()
				.getExitFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList();

		boolean hasNormalActions = !functionActionCallDeclarationList.isEmpty();
		boolean hasEntryActions = !functionActionCallDeclarationListEntry.isEmpty();
		boolean hasExitActions = !functionActionCallDeclarationListExit.isEmpty();

		int amountOfLines = 0;

		if (hasNormalActions)
			amountOfLines++;
		if (hasEntryActions)
			amountOfLines++;
		if (hasExitActions)
			amountOfLines++;

		if (amountOfLines == 0) {
			// nothing to do
		} else if (amountOfLines == 1) {

			if (hasNormalActions)
				drawFunctionActionCallDeclarationList(
					g2d,
					interiorPaintRect,
					functionActionCallDeclarationList);
			else if (hasEntryActions)
				drawFunctionActionCallDeclarationList(
					g2d,
					interiorPaintRect,
					functionActionCallDeclarationListEntry,
					smallItalicFont);
			else if (hasExitActions)
				drawFunctionActionCallDeclarationList(
					g2d,
					interiorPaintRect,
					functionActionCallDeclarationList,
					smallFont,
					functionActionCallDeclarationListExit,
					smallItalicFont);

		} else if (amountOfLines == 2) {
			FunctionActionCallDeclarationList functionActionCallDeclarationList1 =
				functionActionCallDeclarationList;
			FunctionActionCallDeclarationList functionActionCallDeclarationList2 =
				functionActionCallDeclarationList;

			Font font1 = smallFont;
			Font font2 = smallFont;

			if (hasEntryActions) {
				functionActionCallDeclarationList1 = functionActionCallDeclarationListEntry;
				font1 = smallItalicFont;
			}

			if (hasExitActions) {
				functionActionCallDeclarationList2 = functionActionCallDeclarationListExit;
				font2 = smallItalicFont;
			}

			drawFunctionActionCallDeclarationList(
				g2d,
				interiorPaintRect,
				functionActionCallDeclarationList1,
				font1,
				functionActionCallDeclarationList2,
				font2);

		} else {
			Assert.assertEquals(3, amountOfLines);

			drawFunctionActionCallDeclarationList(
				g2d,
				interiorPaintRect,
				functionActionCallDeclarationListEntry,
				smallItalicFont,
				functionActionCallDeclarationList,
				smallFont,
				functionActionCallDeclarationListExit,
				smallItalicFont);
		}

		g2d.setFont(oldFont);

		return interiorPaintRect;
	}

	private void drawFunctionActionCallDeclarationList(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		FunctionActionCallDeclarationList functionActionCallDeclarationList1,
		Font font1,
		FunctionActionCallDeclarationList functionActionCallDeclarationList2,
		Font font2,
		FunctionActionCallDeclarationList functionActionCallDeclarationList3,
		Font font3) {

		double meanFontHeight =
			Math.ceil(
				(g2d.getFontMetrics(font1).getHeight()
					+ g2d.getFontMetrics(font2).getHeight()
					+ g2d.getFontMetrics(font3).getHeight())
					/ 3.0);

		int numberActions1 = functionActionCallDeclarationList1.size();
		int numberActions2 = functionActionCallDeclarationList2.size();
		int numberActions3 = functionActionCallDeclarationList3.size();

		Rectangle copyInteriorPaintRect = new Rectangle(interiorPaintRect);

		int oldHeight = copyInteriorPaintRect.height;

		if (copyInteriorPaintRect.getHeight() < 4 * meanFontHeight) {

			copyInteriorPaintRect.height = (int) Math.round(copyInteriorPaintRect.height / 3.0);

			drawFunctionActionCallDeclarationList(
				g2d,
				copyInteriorPaintRect,
				functionActionCallDeclarationList1,
				font1);

			drawLine(g2d, copyInteriorPaintRect);

		} else if (
			copyInteriorPaintRect.getHeight()
				>= (numberActions1 + numberActions2 + numberActions3 + 1) * meanFontHeight) {

			copyInteriorPaintRect.height =
				(int) Math.max(
					1.5 * g2d.getFontMetrics().getHeight(),
					Math.round((numberActions1 + 0.5) * meanFontHeight));

			drawFunctionActionCallDeclarationList(
				g2d,
				copyInteriorPaintRect,
				functionActionCallDeclarationList1,
				font1);

			drawLine(g2d, copyInteriorPaintRect);

			//			copyInteriorPaintRect.y = copyInteriorPaintRect.y + copyInteriorPaintRect.height;
			//
			//			copyInteriorPaintRect.height =
			//				(int) Math.round(
			//					(numberActions2 + 0.5) * g2d.getFontMetrics().getHeight());

		} else {
			// copyInteriorPaintRect.height = (int) Math.round(copyInteriorPaintRect.height / 3.0);
			int newHeight =
				(int) Math.max(
					1.5 * meanFontHeight,
					Math.round(
						copyInteriorPaintRect.height
							* ((numberActions1 + 0.5)
								/ (numberActions1 + numberActions2 + numberActions3 + 1.5))));

			copyInteriorPaintRect.height = newHeight;

			drawFunctionActionCallDeclarationList(
				g2d,
				copyInteriorPaintRect,
				functionActionCallDeclarationList1,
				font1);

			drawLine(g2d, copyInteriorPaintRect);

		}

		copyInteriorPaintRect.y = copyInteriorPaintRect.y + copyInteriorPaintRect.height;
		copyInteriorPaintRect.height = oldHeight - copyInteriorPaintRect.height;

		drawFunctionActionCallDeclarationList(
			g2d,
			copyInteriorPaintRect,
			functionActionCallDeclarationList2,
			font2,
			functionActionCallDeclarationList3,
			font3);
	}

	private void drawFunctionActionCallDeclarationList(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		FunctionActionCallDeclarationList functionActionCallDeclarationList1,
		Font font1,
		FunctionActionCallDeclarationList functionActionCallDeclarationList2,
		Font font2) {

		double meanFontHeight =
			Math.ceil((g2d.getFontMetrics(font1).getHeight() + g2d.getFontMetrics(font2).getHeight()) / 2.0);

		int numberActions1 = functionActionCallDeclarationList1.size();
		int numberActions2 = functionActionCallDeclarationList2.size();

		Rectangle copyInteriorPaintRect = new Rectangle(interiorPaintRect);

		int oldHeight = copyInteriorPaintRect.height;

		if (copyInteriorPaintRect.getHeight() >= (numberActions1 + numberActions2 + 0.5) * meanFontHeight) {

			copyInteriorPaintRect.height =
				(int) Math.max(1.5 * meanFontHeight, Math.round((numberActions1 + 0.5) * meanFontHeight));

		} else {
			// copyInteriorPaintRect.height = (int) Math.round(copyInteriorPaintRect.height / 2.0);
			int newHeight =
				(int) Math.max(
					1.5 * meanFontHeight,
					Math.round(
						copyInteriorPaintRect.height
							* ((numberActions1 + 0.5) / (numberActions1 + numberActions2 + 1))));

			if (Math.abs(copyInteriorPaintRect.height - newHeight) < (1 * meanFontHeight)) {
				copyInteriorPaintRect.height = (int) Math.round(copyInteriorPaintRect.height / 2.0);

			} else {
				copyInteriorPaintRect.height = newHeight;
			}

		}

		drawFunctionActionCallDeclarationList(
			g2d,
			copyInteriorPaintRect,
			functionActionCallDeclarationList1,
			font1);

		drawLine(g2d, copyInteriorPaintRect);

		copyInteriorPaintRect.y = copyInteriorPaintRect.y + copyInteriorPaintRect.height;
		copyInteriorPaintRect.height = oldHeight - copyInteriorPaintRect.height;

		drawFunctionActionCallDeclarationList(
			g2d,
			copyInteriorPaintRect,
			functionActionCallDeclarationList2,
			font2);
	}

	private void drawFunctionActionCallDeclarationList(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		FunctionActionCallDeclarationList functionActionCallDeclarationList,
		Font font) {

		Font oldFont = g2d.getFontMetrics().getFont();

		if (oldFont == font) {
			drawFunctionActionCallDeclarationList(g2d, interiorPaintRect, functionActionCallDeclarationList);

		} else {
			g2d.setFont(font);
			drawFunctionActionCallDeclarationList(g2d, interiorPaintRect, functionActionCallDeclarationList);
			g2d.setFont(oldFont);
		}
	}

	private void drawFunctionActionCallDeclarationList(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		FunctionActionCallDeclarationList functionActionCallDeclarationList) {

		double y = interiorPaintRect.getY() + g2d.getFontMetrics().getHeight();

		Iterator it =
			CollectionsHelper.map(functionActionCallDeclarationList.iterator(), new IMapFunction() {
			public Object map(Object obj) {
				return ((IFunctionCallDeclaration) obj).getName();
			}
		});

		drawStringIteratorInRect(g2d, interiorPaintRect, y, it);
	}

}
