/**
 * Created on 04.12.2002
 *
 * This class is responsible for //insert purpose here//
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;

import java.awt.*;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.plaf.basic.BasicHTML;
import javax.swing.text.View;

import mygale.mcge.util.prefs.Prefs;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActivityGraphDeclarationGraph;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.GraphDocumentationLeafDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 19.12.2002, 11:40:55
 * @author glueckselig
 */
class FigActivityGraphDocumentationLeafDeclaration extends FigInstantlyActivityGraphNodeDeclaration {

	private int _perc;

	protected void updatePolygon() {

		int width = getWidth();
		int height = getHeight();

		setPercent(15); // where to paint the dog-ear
		setPolygon(new Polygon());

		getPolygon().addPoint(getX() + getPercent(), getY());
		getPolygon().addPoint(getX() + width, getY());
		getPolygon().addPoint(getX() + width, getY() + height);
		getPolygon().addPoint(getX(), getY() + height);
		getPolygon().addPoint(getX(), getY() + getPercent());
	}

	public void editProperties(Container parent) {

		GraphDocumentationLeafDeclaration documentNode =
			((GraphDocumentationLeafDeclaration) getActivityDeclaration());

		CommentDialogPanel cDial =
			new CommentDialogPanel(documentNode.getName(), documentNode.getDocumentation());

		JOptionPane optionPane =
			new JOptionPane(cDial, JOptionPane.PLAIN_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

		((ActivityGraphDeclarationGraph) parent).getGUISeSAmDeclaration().getFrameFactory().addModalPanel(
			parent,
			"Edit Comment",
			optionPane);

		if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
			documentNode.setShortDescription(cDial.getTitle());
			documentNode.setUsesShortDescription(true);
			documentNode.setDocumentation(cDial.getComment());
		}

	}

	protected Rectangle calculateInteriorPaintRect() {
		final int BORDER = 2;

		Rectangle rectForString = new Rectangle();

		rectForString.setBounds(
			Math.round(getX() + BORDER),
			Math.round(getY() + BORDER),
			Math.round(getWidth() - 2 * BORDER),
			Math.round(getHeight() - 2 * BORDER));

		return rectForString;
	}

	protected void paintSymbol(Graphics2D g2d, Prefs prefs) {
		// Paint the background
		g2d.setColor(getBackgroundColor(prefs));
		g2d.fill(getPolygon());

		// Paint the frame
		g2d.setColor(prefs.getFigForegroundColor());
		g2d.draw(getPolygon());

		g2d.drawLine(getX(), getY() + getPercent(), getX() + getPercent(), getY() + getPercent());
		g2d.drawLine(getX() + getPercent(), getY() + getPercent(), getX() + getPercent(), getY());
		g2d.drawLine(getX() + getPercent() + 1, getY() + getPercent() - 1, getX() + getPercent() + 1, getY());
	}

	protected Rectangle paintTitle(Graphics2D g2d, Rectangle interiorPaintRect) {
		Rectangle titleRect = new Rectangle(interiorPaintRect);
		titleRect.x += getPercent();
		titleRect.width -= getPercent();

		Rectangle rectForString = paintTitle(g2d, titleRect, true, false);

		interiorPaintRect.y += rectForString.getHeight();
		interiorPaintRect.height -= rectForString.getHeight();

		drawSeparator(g2d, rectForString);

		return interiorPaintRect;
	}

	protected void drawSeparator(Graphics2D g2d, Rectangle rectForString) {
		int offset = 3;

		int heightText = (int) rectForString.getHeight();

		if (heightText >= getPercent()) {
			g2d.drawLine(
				getX(),
				getY() + heightText + offset,
				getX() + getWidth(),
				getY() + heightText + offset);
		} else {
			g2d.drawLine(
				getX() + getPercent(),
				getY() + heightText + offset,
				getX() + getWidth(),
				getY() + heightText + offset);
		}

	}

	protected Rectangle paintBottom(Graphics2D g2d, Rectangle interiorPaintRect) {
		Rectangle rectForString = paintComment(g2d, interiorPaintRect, true, true);

		interiorPaintRect.y += rectForString.getHeight();
		interiorPaintRect.height -= rectForString.getHeight();

		return interiorPaintRect;
	}

	protected final Rectangle paintComment(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		boolean centerHoriz,
		boolean centerVert) {

		String caption = ((GraphDocumentationLeafDeclaration) getActivityDeclaration()).getDocumentation();
		FontMetrics fm = g2d.getFontMetrics();

		Rectangle rectForString;

		rectForString = calculateRectForStringTopLocation(g2d, fm, caption, interiorPaintRect);

		if (BasicHTML.isHTMLString(caption)) {

			JComponent dummyComp = new JLabel();
			Font font = g2d.getFont();
			View view = BasicHTML.createHTMLView(dummyComp, caption);
			view.paint(g2d, rectForString);
			g2d.setFont(font);

		} else {

			caption = conserveNewlinesInStringForStringTokenizer(caption);

			//	ATT Ersetzen von tabs mit Leerzeichen hardcoded!
			caption = caption.replaceAll("\t", "    ");

			StringTokenizer tokenizer = new StringTokenizer(caption, "\n", false);

			drawStringIteratorInRect(
				g2d,
				interiorPaintRect,
				interiorPaintRect.getY() + g2d.getFontMetrics().getHeight(),
				CollectionsHelper.enumerationToIterator(tokenizer));
		}
		return rectForString;
	}

	protected String conserveNewlinesInStringForStringTokenizer(String caption) {
		String temporaryString = caption.replaceFirst("\n\n", "\n \n");
		while (!temporaryString.equals(caption)) {

			caption = temporaryString;
			temporaryString = caption.replaceFirst("\n\n", "\n \n");
		}
		return caption;
	}

	private void setPercent(int perc) {
		_perc = perc;
	}

	private int getPercent() {
		return _perc;
	}
}
