package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import junit.framework.Assert;
import mygale.mcge.fig.AbstractFigNodeFactory;
import mygale.mcge.fig.IFigNode;
import mygale.mcge.net.INetNode;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;

/**
 * Creation date: (11.06.2001 13:20:17)
 * @author Christoph Oechslein
 */

public class FigActivityGraphNodeDeclarationFactory extends AbstractFigNodeFactory {
	private static final FigActivityGraphNodeDeclarationFactory _instance = new FigActivityGraphNodeDeclarationFactory();

	public static final FigActivityGraphNodeDeclarationFactory getInstance() {
		return _instance;
	}

	private FigActivityGraphNodeDeclarationFactory() {
		super();
	}

	private IFigNode createFigActivityDeclaration(IActivityGraphNodeDeclaration activityDeclaration) {

		if (activityDeclaration instanceof IEmergencyActivityGraphLeafDeclaration) {
			return new FigEmergencyActivityGraphLeafDeclaration();

		} else if (activityDeclaration instanceof IEntryActivityGraphLeafDeclaration) {
			return new FigEntryActivityGraphLeafDeclaration();

		} else if (activityDeclaration instanceof IExitActivityGraphLeafDeclaration) {
			return new FigExitActivityGraphLeafDeclaration();

		} else if (activityDeclaration instanceof IActionActivityGraphLeafDeclaration) {
			return new FigActionActivityGraphLeafDeclaration();

		} else if (activityDeclaration instanceof IActivityGraphComposedNodeDeclaration) {
			return new FigActivityGraphComposedNodeDeclaration();

		} else if (activityDeclaration instanceof IDecisionGraphLeafDeclaration) {
			return new FigDecisionActivityGraphNodeDeclaration();

		} else if (activityDeclaration instanceof GraphDocumentationLeafDeclaration) {
			return new FigActivityGraphDocumentationLeafDeclaration();

//		} else if (activityDeclaration instanceof IActivityGraphLeafDeclaration) {
//			return new FigActivityGraphNodeDeclaration();
//
		} else {
			Assert.fail("no fig for " + activityDeclaration);
			return null;
		}
	}

	public IFigNode createFigNode(INetNode netObject) {

		IFigNode figObject = createFigActivityDeclaration((IActivityGraphNodeDeclaration) netObject);

		figObject.setNetObject(netObject);

		// java.awt.Dimension dim = getGraphEditor().getScrollPane().getSize();
		// java.awt.Rectangle rect = _graph.getWorkspaceRect();
		// int maxWidth = rect.width; // 500;
		// int maxHeight = rect.height; // 500;

		int maxWidth = 500;
		int maxHeight = 300;

		figObject.setLocation(
			(int) (Math.random() * (maxWidth - figObject.getWidth())),
			(int) (Math.random() * (maxHeight - figObject.getHeight())));

		return figObject;
	}
}
