package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.Container;
import java.awt.datatransfer.*;
import java.io.IOException;
import java.util.*;

import javax.swing.*;

import junit.framework.Assert;
import mygale.mcge.core.JGraph;
import mygale.mcge.fig.FigEdgeSuperLinear;
import mygale.mcge.fig.IFigObject;
import mygale.mcge.fig.arrow.Arrow;
import mygale.mcge.fig.arrow.ArrowFilledArrow;
import mygale.mcge.net.INetGraphModel;
import mygale.mcge.net.INetObject;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActivityGraphDeclarationGraph;
import de.uniwue.ki.gui.mas.behaviour.rule.ActivityRuleDeclarationPanel;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.gui.mas.util.declaration.action.CopyToClipboardAction;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.gui.util.dnd.ObjectDataFlavor;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IDeclarationNetEdge;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IDeclarationNetNode;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IEmergencyActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IEntryActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IExitActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IDoitFunction;

/**
 * Creation date: (22.03.2001 13:43:30)
 * 
 * @author Christoph Oechslein
 */
public class FigActivityGraphRuleDeclaration extends FigEdgeDeclaration {

	public FigActivityGraphRuleDeclaration() {
		this(new ArrowFilledArrow());
	}

	protected FigActivityGraphRuleDeclaration(Arrow arrow) {
		super();

		setEndArrow(arrow);
	}

	public void editProperties(Container parent) {
		final ActivityGraphDeclarationGraph composedActivityDeclarationGraph = (ActivityGraphDeclarationGraph) parent;

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(composedActivityDeclarationGraph
					.getActivityAgentClassDeclaration());

		IActivityGraphRuleDeclaration activityArgumentRuleDeclaration = (IActivityGraphRuleDeclaration) getNetObject();

		IDeclarationFrameFactory declarationFrameFactory = GUISeSAmDeclaration
				.getDeclarationFrameFactory(composedActivityDeclarationGraph
						.getGUISeSAmDeclaration());

		declarationFrameFactory.addPanel(composedActivityDeclarationGraph,
				activityArgumentRuleDeclaration,
				new AbstractDeclarationPanelFactory() {
					public IDeclarationPanel createDeclarationPanel(
							IDeclaration declaration) {
						ActivityRuleDeclarationPanel activityArgumentRuleDeclarationPanel = new ActivityRuleDeclarationPanel();

						activityArgumentRuleDeclarationPanel
								.setGUISeSAmDeclaration(composedActivityDeclarationGraph
										.getGUISeSAmDeclaration());
						activityArgumentRuleDeclarationPanel
								.setActivityAgentClassDeclaration(composedActivityDeclarationGraph
										.getActivityAgentClassDeclaration());

						return activityArgumentRuleDeclarationPanel;
					}

					public String getDeclarationTitle(IDeclaration declaration) {
						return "Activity Rule";
					}

				}, true, true);

	}

	protected Iterator getPopupMenuMoreActions(JGraph graph,
			boolean forOnlyOneFig) {
		ActivityGraphDeclarationGraph activityGraph = (ActivityGraphDeclarationGraph) graph;

		if (forOnlyOneFig) {
			LinkedList list = new LinkedList();
			Iterator it = (activityGraph).getSelection().getAllFigs();
			while (it.hasNext()) {
				IDeclaration selectedDeclaration = (IDeclaration) ((IFigObject) it
						.next()).getNetObject();

				if (selectedDeclaration instanceof IActivityGraphRuleDeclaration) {
					final IActivityGraphRuleDeclaration selectedActivityGraphRuleDeclaration = (IActivityGraphRuleDeclaration) selectedDeclaration;
					list.add(new CopyToClipboardAction(selectedDeclaration,
							activityGraph.getGUISeSAmDeclaration()
									.getClipboard()));

					list.add(new InspectAction(this));

					addPasteAction(selectedActivityGraphRuleDeclaration,
							activityGraph, list);
				}
				break;
			}

			CollectionsHelper.addAll(list, super.getPopupMenuMoreActions(
					activityGraph, forOnlyOneFig));

			return list.iterator();

		} else {
			return super.getPopupMenuMoreActions(activityGraph, forOnlyOneFig);
		}
	}

	protected Iterator createPossibleFromDeclarations(
			IDeclarationNetNode toDeclaration) {
		IExitActivityGraphLeafDeclaration exit = ((IActivityGraphNodeDeclaration) toDeclaration)
				.getActivityGraphDeclaration()
				.getExitActivityGraphLeafDeclaration();
		return CollectionsHelper.remove(super
				.createPossibleFromDeclarations(toDeclaration), exit);
	}

	protected Iterator createPossibleToDeclarations(
			IDeclarationNetNode fromDeclaration) {
		IEntryActivityGraphLeafDeclaration entry = ((IActivityGraphNodeDeclaration) fromDeclaration)
				.getActivityGraphDeclaration()
				.getEntryActivityGraphLeafDeclaration();
		IEmergencyActivityGraphLeafDeclaration emergency = ((IActivityGraphNodeDeclaration) fromDeclaration)
				.getActivityGraphDeclaration()
				.getEmergencyActivityGraphLeafDeclaration();

		return CollectionsHelper.remove(CollectionsHelper.remove(super
				.createPossibleToDeclarations(fromDeclaration), entry),
				emergency);
	}

	private void addPasteAction(
			final IActivityGraphRuleDeclaration selectedActivityGraphRuleDeclaration,
			final ActivityGraphDeclarationGraph activityGraph, LinkedList list) {
		Clipboard clipboard = activityGraph.getGUISeSAmDeclaration()
				.getClipboard();
		final Transferable transferable = clipboard.getContents(null);

		if (transferable != null) {
			list.add(new AbstractBoundAction("Paste") {
				public void doit() {

					try {
						IActivityGraphRuleDeclaration newActivityGraphRuleDeclaration = (IActivityGraphRuleDeclaration) transferable
								.getTransferData(ObjectDataFlavor
										.getDataFlavor(IActivityGraphRuleDeclaration.class));

						newActivityGraphRuleDeclaration
								.setFromActivityGraphNodeDeclaration(null);
						newActivityGraphRuleDeclaration
								.setToActivityGraphNodeDeclaration(null);

						newActivityGraphRuleDeclaration.copyInto(
								selectedActivityGraphRuleDeclaration,
								new HashMap());

						selectedActivityGraphRuleDeclaration
								.setFromActivityGraphNodeDeclaration((IActivityGraphNodeDeclaration) getBeginFigNode()
										.getNetObject());
						selectedActivityGraphRuleDeclaration
								.setToActivityGraphNodeDeclaration((IActivityGraphNodeDeclaration) getEndFigNode()
										.getNetObject());

					} catch (IOException exception) {
						throw new RuntimeException(exception);
					} catch (UnsupportedFlavorException exception) {
					}
				}
			});
		}
	}
}