package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.Container;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.ButtonGroup;
import javax.swing.JMenu;
import javax.swing.JRadioButtonMenuItem;

import mygale.mcge.core.JGraph;
import mygale.mcge.fig.FigEdgeSuperLinear;
import mygale.mcge.fig.IFigObject;
import mygale.mcge.fig.arrow.Arrow;
import mygale.mcge.fig.arrow.ArrowFilledArrow;
import mygale.mcge.net.INetGraphModel;
import mygale.mcge.net.INetObject;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IDeclarationNetEdge;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IDeclarationNetNode;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IDoitFunction;

/**
 * @author Rainer Herrler
 */
public class FigEdgeDeclaration extends FigEdgeSuperLinear {

	public FigEdgeDeclaration() {
		this(new ArrowFilledArrow());
	}

	protected FigEdgeDeclaration(Arrow arrow) {
		super();
		
		setEndArrow(arrow);
	}

	public void cleanUp() {
		if (getNetObject() != null)
			 ((IDeclaration) getNetObject()).removeChangeListener(this);
	}

	protected JMenu createChangeFromDeclarationFromMenu(
		final JGraph graph,
		final IDeclarationNetEdge edgeDeclaration) {
		IDeclarationNetNode fromDeclaration = edgeDeclaration.getFromDeclarationNetNode();
		IDeclarationNetNode toDeclaration = edgeDeclaration.getToDeclarationNetNode();

		final INetGraphModel parentNetModel = fromDeclaration.getNetGraphModel();

		Iterator possibleFromDeclarations = createPossibleFromDeclarations(toDeclaration);

		if (!possibleFromDeclarations.hasNext())
			return null;

		IDoitFunction fromDoit = new IDoitFunction() {
			public void doit(Object obj) {
				IDeclarationNetNode newFromDeclaration =
					(IDeclarationNetNode) obj;

				parentNetModel.removeEdge(
				edgeDeclaration);
				edgeDeclaration.setFromDeclarationNetNode(newFromDeclaration);
				edgeDeclaration.setBeginNode(newFromDeclaration);
				parentNetModel.addEdge(edgeDeclaration);
			}
		};

		JMenu changeFromDeclarationFromMenu =
			createDeclarationSelectionMenu(
				"Change From",
				possibleFromDeclarations,
				fromDeclaration,
				fromDoit);

		if (changeFromDeclarationFromMenu == null)
			return null;

		AbstractBoundAction.setFont(changeFromDeclarationFromMenu, false);

		return changeFromDeclarationFromMenu;
	}

	protected JMenu createChangeToDeclarationFromMenu(
		final JGraph graph,
		final IDeclarationNetEdge edgeDeclaration) {

		IDeclarationNetNode fromDeclaration =
			edgeDeclaration.getFromDeclarationNetNode();
		IDeclarationNetNode toDeclaration =
			edgeDeclaration.getToDeclarationNetNode();

		final INetGraphModel parentNetGraphModel =
			fromDeclaration.getNetGraphModel();

		Iterator toDeclarations = createPossibleToDeclarations(fromDeclaration);

		ButtonGroup toButtonGroup = new ButtonGroup();

		IDoitFunction toDoit = new IDoitFunction() {
			public void doit(Object obj) {
				IDeclarationNetNode newToDeclaration = (IDeclarationNetNode) obj;

				parentNetGraphModel.removeEdge(
					edgeDeclaration);
				edgeDeclaration.setToDeclarationNetNode(newToDeclaration);
				edgeDeclaration.setEndNode(newToDeclaration);
				parentNetGraphModel.addEdge(edgeDeclaration);
			}
		};

		JMenu changeToDeclarationToMenu =
			createDeclarationSelectionMenu("Change To", toDeclarations, toDeclaration, toDoit);

		if (changeToDeclarationToMenu == null)
			return null;

		AbstractBoundAction.setFont(changeToDeclarationToMenu, false);

		return changeToDeclarationToMenu;
	}

	private JMenu createDeclarationSelectionMenu(
		String menuName,
		Iterator it,
		IDeclaration selectedDeclaration,
		final IDoitFunction doitFunction) {
			
		JMenu toDeclarationMenu = new JMenu(menuName);

		int count = 0;

		while (it.hasNext()) {
			final IDeclaration currDeclaration =
				(IDeclaration) it.next();

			String currName = currDeclaration.getName();

			JRadioButtonMenuItem jRadioButtonMenuItem =
				new JRadioButtonMenuItem(currName.length() > 0 ? currName : "    ");

			AbstractBoundAction.setActionAndFont(jRadioButtonMenuItem, new AbstractBoundAction() {
				public void doit() {
					try {
						doitFunction.doit(currDeclaration);

					} catch (Exception exception) {
						throw new RuntimeException(exception);
					}
				}
			}, false);

			jRadioButtonMenuItem.setSelected(currDeclaration == selectedDeclaration);

			toDeclarationMenu.add(jRadioButtonMenuItem);
			count++;
		}

		if (count <= 1)
			return null;
		else
			return toDeclarationMenu;
	}
	
	protected Iterator createPossibleFromDeclarations(IDeclarationNetNode toDeclaration) {
		return CollectionsHelper.remove(
				toDeclaration.getNetGraphModel().getNodes(),
				toDeclaration);
	}

	protected Iterator createPossibleToDeclarations(IDeclarationNetNode fromDeclaration) {
		final INetGraphModel parentNetGraphModel =
			fromDeclaration.getNetGraphModel();

		return CollectionsHelper.remove(
			fromDeclaration.getNetGraphModel().getNodes(),
			fromDeclaration);
	}

	

	public void editProperties(Container parent) {
		/*
		 * Overwrite to edit
		 */
	}

	

	public String getName() {
		IDeclaration declaration =
			(IDeclaration) getNetObject();

		return declaration.getName();
	}

	protected Iterator getPopupMenuImportantActions(JGraph graph, boolean forOnlyOneFig) {
		LinkedList list = new LinkedList();

		if (forOnlyOneFig) {
			IFigObject currFigObject = (IFigObject) graph.getSelection().getAllFigs().next();

			IDeclarationNetEdge edgeDeclaration =
				(IDeclarationNetEdge) currFigObject.getNetObject();

			JMenu fromMenu = createChangeFromDeclarationFromMenu(graph, edgeDeclaration);
			if (fromMenu != null)
				list.add(fromMenu);

			JMenu toMenu = createChangeToDeclarationFromMenu(graph, edgeDeclaration);
			if (toMenu != null)
				list.add(toMenu);

			return CollectionsHelper.concatenate(
				super.getPopupMenuImportantActions(graph, forOnlyOneFig),
				list.iterator());

		} else {
			return super.getPopupMenuImportantActions(graph, forOnlyOneFig);
		}
	}


	

	public void setNetObject(INetObject netObject) {
		IDeclaration oldDeclaration = (IDeclaration) getNetObject();
		if (oldDeclaration != null) {
			oldDeclaration.removeChangeListener(this);
		}

		super.setNetObject(netObject);

		IDeclaration newDeclaration = (IDeclaration) netObject;
		if (newDeclaration != null) {
			newDeclaration.addChangeListener(this);
		}
	}

	public String toString() {
	    return super.toString() + " ["+ getName() + "]";
	}
}