package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.*;
import java.util.Iterator;

import javax.swing.JMenuItem;

import mygale.mcge.core.JGraph;
import mygale.mcge.fig.IFigObject;
import mygale.mcge.util.prefs.Prefs;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.action.DuplicateActivityDeclarationAction;
import de.uniwue.ki.gui.mas.util.declaration.action.RenameDocumentableDeclarationAction;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (03.05.2001 11:09:31)
 * @author Christoph Oechslein
 */
public abstract class FigInstantlyActivityGraphNodeDeclaration extends FigActivityGraphNodeDeclaration {

	public FigInstantlyActivityGraphNodeDeclaration() {
		super();
	}

	private static JMenuItem createRenameMenuItem(
		final JGraph graph,
		IActivityGraphNodeDeclaration activityDeclaration) {

		return AbstractBoundAction.createJMenuItem(
			new RenameDocumentableDeclarationAction(graph, activityDeclaration),
			false);
	}

	private static JMenuItem createDuplicateMenuItem(
		final JGraph graph,
		IActivityGraphNodeDeclaration activityDeclaration) {

		if (activityDeclaration.getActivityGraphDeclaration() == null)
			return null;
		else
			return AbstractBoundAction.createJMenuItem(
				new DuplicateActivityDeclarationAction(activityDeclaration),
				false);
	}

	protected Iterator getPopupMenuImportantActions(final JGraph graph, boolean forOnlyOneFig) {
		if (forOnlyOneFig) {
			IFigObject currFigObject = (IFigObject) graph.getSelection().getAllFigs().next();
			final IActivityGraphNodeDeclaration activityDeclaration =
				(IActivityGraphNodeDeclaration) currFigObject.getNetObject();

			return CollectionsHelper.createConsIterator(
				createRenameMenuItem(graph, activityDeclaration),
				createDuplicateMenuItem(graph, activityDeclaration),
				super.getPopupMenuImportantActions(graph, forOnlyOneFig));

		} else {
			return super.getPopupMenuImportantActions(graph, forOnlyOneFig);
		}
	}

	protected void paintSymbol(Graphics2D g2d, Prefs prefs) {
		// Paint the background
		g2d.setColor(getBackgroundColor(prefs));
		g2d.fill(getShape());

		// Paint the frame
		g2d.setColor(prefs.getFigForegroundColor());
		g2d.draw(getShape());
	}

	

	protected void drawStringIteratorInRect(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		double y,
		Iterator it) {

		FontMetrics fmSmall = g2d.getFontMetrics();

		while (it.hasNext()) {

			String str = (String) it.next();

			Rectangle rectForString = calculateRectForStringTopLocation(g2d, fmSmall, str, interiorPaintRect);

			if ((y < (interiorPaintRect.getY() + interiorPaintRect.getHeight() - fmSmall.getHeight() * 1.25))
				|| !it.hasNext()) {

				drawStringInRect(g2d, rectForString, fmSmall, str, (int) Math.round(y), false);

				y += rectForString.getHeight();

			} else {

				drawStringInRect(g2d, rectForString, fmSmall, "...", (int) Math.round(y), false);

				break;
			}
		}
	}

}