package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.*;

import javax.swing.ImageIcon;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.SwingMoreUtilities;

/**
 * Creation date: (22.03.2001 13:42:40)
 * @author Christoph Oechslein
 */
final class FigInstantlyIconHelper {
	public static final int INSTANTLY = 0;
	public static final int NOT_INSTANTLY = 1;
	public static final int UNKNOWN_IF_INSTANTLY = 2;

	private static final int _distanceToIcon = 3;
	private static final int _iconBorder = 1;

	private static final ImageIcon _notInstanlyIcon =
		SwingMoreUtilities.createImageIcon("others/stopwatch.gif");

	private static final ImageIcon _questionmarkInstanlyIcon =
		SwingMoreUtilities.createImageIcon("others/questionmark stopwatch.gif");

	private static final ImageIcon _instanlyIcon =
		SwingMoreUtilities.createImageIcon("others/stroked stopwatch.gif");

	private FigInstantlyIconHelper() {
		super();
	}

	public static void drawInstantlyIcon(Graphics2D g2d, int rightX, int topY, int instantlyType) {

		ImageIcon icon = getIcon(instantlyType);

		if (icon != null) {
			int width = getInstantlyIconWidth(icon);
			int height = getInstantlyIconHeight(icon);

			g2d.drawImage(
				icon.getImage(),
				rightX - width - _iconBorder,
				topY + _iconBorder,
				width,
				height,
				icon.getImageObserver());
		}
	}

	private static int getInstantlyIconWidth(ImageIcon icon) {
		return 16;
		//return icon.getIconWidth();
	}

	private static int getInstantlyIconHeight(ImageIcon icon) {
		return 16;
		//return icon.getIconHeight();
	}

	public static void drawInstantlyIconCenterTop(Graphics2D g2d, Rectangle rect, int instantlyType) {

		ImageIcon icon = getIcon(instantlyType);

		if (icon != null) {

			int width = getInstantlyIconWidth(icon);
			int height = getInstantlyIconHeight(icon);

			int x = (int)Math.round(rect.getX() + (rect.getWidth() - width) * 0.5);
			int y = (int)Math.round(rect.getY() + rect.getHeight() * 0.07);

			g2d.drawImage(
				icon.getImage(),
				x + _iconBorder,
				y + _iconBorder,
				width - 2 * _iconBorder,
				height - 2 * _iconBorder,
				icon.getImageObserver());
		}
	}

	private static ImageIcon getIcon(int instantlyType) {
		ImageIcon icon;

		if (instantlyType == INSTANTLY) {
			icon = _instanlyIcon;

		} else if (instantlyType == NOT_INSTANTLY) {
			icon = _notInstanlyIcon;

		} else {
			Assert.assertEquals(UNKNOWN_IF_INSTANTLY, instantlyType);
			icon = _questionmarkInstanlyIcon;
		}

		if (icon.getImageLoadStatus() == MediaTracker.COMPLETE)
			return icon;
		else
			return null;
	}

	public static void setDrawInstantlyIconBounds(Graphics2D g2d, Rectangle rectForString) {
		rectForString.setBounds(
			rectForString.x,
			rectForString.y,
			rectForString.width - getInstantlyIconWidth(_notInstanlyIcon) - _distanceToIcon - _iconBorder,
			Math.max(getInstantlyIconHeight(_notInstanlyIcon) + 2 * _iconBorder, rectForString.height));
	}
}