package de.uniwue.ki.gui.mas.behaviour.activityGraph.fig;
import java.awt.*;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.*;

import junit.framework.Assert;
import mygale.mcge.core.JGraph;
import mygale.mcge.fig.IFigObject;
import mygale.mcge.util.prefs.Prefs;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.action.ChangeActivityTypeAction;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclarationTypeable;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (22.03.2001 13:42:40)
 * @author Christoph Oechslein
 */
public class FigTypeableActivityGraphNodeDeclaration extends FigInstantlyActivityGraphNodeDeclaration {
	private static final int _signalArrowLengthAbsolut = 15;

	public FigTypeableActivityGraphNodeDeclaration() {
		super();
	}

	static JMenu createChangeActivityTypeActionMenu(JGraph graph) {
		return createChangeActivityTypeActionMenu(graph, (IActivityGraphNodeDeclarationTypeable.Type) null);
	}

	static JMenu createChangeActivityTypeActionMenu(
		final JGraph graph,
		IActivityGraphNodeDeclarationTypeable.Type selectedType) {

		JMenu menu = new JMenu("Change Type");

		AbstractBoundAction.setFont(menu, false);

		ButtonGroup buttonGroup = new ButtonGroup();

		menu.add(
			createChangeActivityTypeActionMenuItem(
				graph,
				IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE,
				buttonGroup,
				selectedType));

		menu.add(
			createChangeActivityTypeActionMenuItem(
				graph,
				IActivityGraphNodeDeclarationTypeable.STATE_LIKE,
				buttonGroup,
				selectedType));

		menu.add(
			createChangeActivityTypeActionMenuItem(
				graph,
				IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE,
				buttonGroup,
				selectedType));

		menu.add(
			createChangeActivityTypeActionMenuItem(
				graph,
				IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE,
				buttonGroup,
				selectedType));

		return menu;
	}

	static JMenu createChangeActivityTypeActionMenu(
		JGraph graph,
		IActivityGraphNodeDeclarationTypeable activityDeclarationTypeable) {
		return createChangeActivityTypeActionMenu(graph, activityDeclarationTypeable.getType());
	}

	private static JCheckBoxMenuItem createChangeActivityTypeActionMenuItem(
		final JGraph graph,
		IActivityGraphNodeDeclarationTypeable.Type type,
		ButtonGroup buttonGroup,
		IActivityGraphNodeDeclarationTypeable.Type selectedType) {

		JCheckBoxMenuItem checkBoxMenuItem = new JCheckBoxMenuItem(type.getName());

		AbstractBoundAction.setActionAndFont(
			checkBoxMenuItem,
			new ChangeActivityTypeAction(type, graph),
			false);

		checkBoxMenuItem.setSelected(selectedType == type);

		buttonGroup.add(checkBoxMenuItem);

		return checkBoxMenuItem;
	}

	protected IActivityGraphNodeDeclarationTypeable getActivityDeclarationTypeable() {
		return (IActivityGraphNodeDeclarationTypeable) getActivityDeclaration();
	}

	protected Iterator getPopupMenuImportantActions(final JGraph graph, boolean forOnlyOneFig) {
		LinkedList list = new LinkedList();

		if (forOnlyOneFig) {
			IFigObject currFigObject = (IFigObject) graph.getSelection().getAllFigs().next();

			final IActivityGraphNodeDeclarationTypeable activityDeclarationTypeable =
				(IActivityGraphNodeDeclarationTypeable) currFigObject.getNetObject();

			list.add(createChangeActivityTypeActionMenu(graph, activityDeclarationTypeable));

		} else {
			list.add(createChangeActivityTypeActionMenu(graph));
		}

		return CollectionsHelper.concatenate(
			list.iterator(),
			super.getPopupMenuImportantActions(graph, forOnlyOneFig));
	}

	protected void paintSymbol(Graphics2D g2d, Prefs prefs) {
		if ((getActivityDeclarationTypeable().getType() == IActivityGraphNodeDeclarationTypeable.STATE_LIKE)
			|| (getActivityDeclarationTypeable().getType()
				== IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE)) {

			paintRoundRect(g2d, prefs);

		} else {
			super.paintSymbol(g2d, prefs);
		}
	}

	private void paintRoundRect(Graphics2D g2d, Prefs prefs) {
		int arcWidth = getArcWidthAndHeight();
		int arcHeight = getArcWidthAndHeight();

		// Paint the background
		g2d.setColor(getBackgroundColor(prefs));

		g2d.fillRoundRect(getX(), getY(), getWidth(), getHeight(), arcWidth, arcHeight);

		// Paint the frame
		g2d.setColor(prefs.getFigForegroundColor());

		g2d.drawRoundRect(getX(), getY(), getWidth(), getHeight(), arcWidth, arcHeight);
	}

	private int getArcWidthAndHeight() {
		if (getActivityDeclarationTypeable().getType() == IActivityGraphNodeDeclarationTypeable.STATE_LIKE)
			return 20;
		else if (
			getActivityDeclarationTypeable().getType() == IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE)
			return 50;
		else
			return 0;
	}

	protected Rectangle calculateInteriorPaintRect() {
		Rectangle rectForString = super.calculateInteriorPaintRect();

		if (getActivityDeclarationTypeable().getType()
			== IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE) {
			rectForString.x += _signalArrowLengthAbsolut * 1.2;
			rectForString.width -= 1 * _signalArrowLengthAbsolut * 1.2;

		} else if (
			getActivityDeclarationTypeable().getType()
				== IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE) {
			rectForString.x += _signalArrowLengthAbsolut * 1.1;
			rectForString.width -= 1 * _signalArrowLengthAbsolut * 1.1;

		} else {
			rectForString.x += getArcWidthAndHeight() / 7.0;
			rectForString.width -= 2 * getArcWidthAndHeight() / 7.0;
		}

		rectForString.y += getArcWidthAndHeight() / 20.0;
		rectForString.height -= 2 * getArcWidthAndHeight() / 20.0;

		return rectForString;
	}

	protected Rectangle paintTitle(Graphics2D g2d, Rectangle interiorPaintRect) {

		String caption = getName();
		FontMetrics fm = g2d.getFontMetrics();

		Rectangle rectForString = calculateRectForStringTopLocation(g2d, fm, caption, interiorPaintRect);

		FigInstantlyIconHelper.setDrawInstantlyIconBounds(g2d, rectForString);

		drawStringInRectCentered(g2d, rectForString, fm, caption);

		FigInstantlyIconHelper.drawInstantlyIcon(
			g2d,
			interiorPaintRect.x + interiorPaintRect.width,
			interiorPaintRect.y,
			getActivityDeclaration().isInstantly()
				? FigInstantlyIconHelper.INSTANTLY
				: ((getActivityDeclaration() instanceof IActivityGraphDeclaration)
					? FigInstantlyIconHelper.UNKNOWN_IF_INSTANTLY
					: FigInstantlyIconHelper.NOT_INSTANTLY));

		drawLine(g2d, rectForString);

		interiorPaintRect.y += rectForString.getHeight() + 1;
		interiorPaintRect.height -= rectForString.getHeight() + 1;

		return interiorPaintRect;
	}

	protected void drawLine(Graphics2D g2d, Rectangle rectForString) {
		int lineY = (int) (rectForString.y + rectForString.getHeight());

		double y0 = getBounds().getY();
		double y1 = y0 + getHeight() / 2.0;

		int schnittpunktX = (int) Math.round(((lineY - y0) * _signalArrowLengthAbsolut) / (y1 - y0));

		if (getActivityDeclarationTypeable().getType()
			== IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE) {
			if (lineY <= y1) {
				g2d.drawLine(getX() + schnittpunktX, lineY, getX() + getWidth(), lineY);
			} else {
				g2d.drawLine(
					(getX() + _signalArrowLengthAbsolut) - (schnittpunktX - _signalArrowLengthAbsolut),
					lineY,
					getX() + getWidth(),
					lineY);
			}
		} else if (
			getActivityDeclarationTypeable().getType()
				== IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE) {
			if (lineY <= y1) {
				g2d.drawLine(
					(getX() + _signalArrowLengthAbsolut) - schnittpunktX,
					lineY,
					getX() + getWidth(),
					lineY);
			} else {
				g2d.drawLine(
					getX() - _signalArrowLengthAbsolut + schnittpunktX,
					lineY,
					getX() + getWidth(),
					lineY);
			}
		} else {
			g2d.drawLine(getX(), lineY, getX() + getWidth(), lineY);
		}
	}

	protected void updatePolygon() {
		if (getActivityDeclarationTypeable().getType()
			== IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE) {
			updateActivityPolygon();

		} else if (
			getActivityDeclarationTypeable().getType() == IActivityGraphNodeDeclarationTypeable.STATE_LIKE) {
			updateStatePolygon();

		} else if (
			getActivityDeclarationTypeable().getType()
				== IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE) {
			updateSignalInPolygon();

		} else if (
			getActivityDeclarationTypeable().getType()
				== IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE) {
			updateSignalOutPolygon();

		} else {
			Assert.fail();
		}
	}

	private void updateRectPolygon() {
		setPolygon(new Polygon());
		getPolygon().addPoint(getX(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY() + getHeight());
		getPolygon().addPoint(getX(), getY() + getHeight());
	}

	private void updateSignalInPolygon() {
		setPolygon(new Polygon());
		getPolygon().addPoint(getX() + _signalArrowLengthAbsolut, getY() + getHeight() / 2);
		getPolygon().addPoint(getX(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY() + getHeight());
		getPolygon().addPoint(getX(), getY() + getHeight());
	}

	private void updateSignalOutPolygon() {
		setPolygon(new Polygon());
		getPolygon().addPoint(getX(), getY() + getHeight() / 2);
		getPolygon().addPoint(getX() + _signalArrowLengthAbsolut, getY());
		getPolygon().addPoint(getX() + getWidth(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY() + getHeight());
		getPolygon().addPoint(getX() + _signalArrowLengthAbsolut, getY() + getHeight());
	}

	private void updateActivityPolygon() {
		updateRectPolygon();
	}

	private void updateStatePolygon() {
		updateRectPolygon();
	}
}