package de.uniwue.ki.gui.mas.behaviour.reasoning;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JFrame;
import javax.swing.JPanel;

import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActivityGraphDeclarationGraphPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.IDeclarationPanel;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (16.06.2000 19:11:40)
 * @author Christoph Oechslein
 */

public class ActivityReasoningEngineDeclarationPanel extends JPanel implements IDeclarationPanel {

	private IActivityReasoningEngineDeclaration _activityReasoningEngineDeclarationIntern;
	private final EventHandler _eventHandler = new EventHandler();
	private ActivityGraphDeclarationGraphPanel _composedActivityDeclarationGraphPanel;
	private boolean _connPtoP1Aligning;

	private final class EventHandler implements PropertyChangeListener {
		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getSource()
				== ActivityReasoningEngineDeclarationPanel.this.getActivityReasoningEngineDeclarationIntern()
				&& (evt.getPropertyName().equals("composedActivityDeclaration")))
				connPtoP1SetTarget();
			if (evt.getSource()
				== ActivityReasoningEngineDeclarationPanel.this.getComposedActivityDeclarationGraphPanel()
				&& (evt.getPropertyName().equals("gUISeSAmDeclaration")))
				connEtoC1(evt);
			if (evt.getSource()
				== ActivityReasoningEngineDeclarationPanel.this.getComposedActivityDeclarationGraphPanel()
				&& (evt.getPropertyName().equals("activityAgentClassDeclaration")))
				connEtoC2(evt);
		}
	}

	public ActivityReasoningEngineDeclarationPanel() {
		super();
		initialize();
	}

	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			ActivityReasoningEngineDeclarationPanel aActivityReasoningEngineDeclarationPanel;
			aActivityReasoningEngineDeclarationPanel = new ActivityReasoningEngineDeclarationPanel();
			frame.setContentPane(aActivityReasoningEngineDeclarationPanel);
			frame.setSize(aActivityReasoningEngineDeclarationPanel.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				}
			});
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of javax.swing.JPanel");
			exception.printStackTrace(System.out);
		}
	}

	public void dispose() {
		getComposedActivityDeclarationGraphPanel().removePropertyChangeListener(_eventHandler);

		getComposedActivityDeclarationGraphPanel().dispose();
	}

	public IActivityAgentClassDeclaration getActivityAgentClassDeclaration() {
		return getComposedActivityDeclarationGraphPanel().getActivityAgentClassDeclaration();
	}

	public IActivityReasoningEngineDeclaration getActivityReasoningEngineDeclaration() {
		return getActivityReasoningEngineDeclarationIntern();
	}

	public de.uniwue.ki.mas.util.declaration.IDeclaration getDeclaration() {
		return getActivityReasoningEngineDeclaration();
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return getComposedActivityDeclarationGraphPanel().getGUISeSAmDeclaration();
	}

	public boolean isDisposed() {
		return getComposedActivityDeclarationGraphPanel().isDisposed();
	}

	public void setActivityAgentClassDeclaration(IActivityAgentClassDeclaration arg1) {
		getComposedActivityDeclarationGraphPanel().setActivityAgentClassDeclaration(arg1);
	}

	public void setActivityReasoningEngineDeclaration(IActivityReasoningEngineDeclaration arg1) {
		setActivityReasoningEngineDeclarationIntern(arg1);
	}

	public void setDeclaration(IDeclaration declaration) {
		setActivityReasoningEngineDeclaration((IActivityReasoningEngineDeclaration)declaration);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration arg1) {
		getComposedActivityDeclarationGraphPanel().setGUISeSAmDeclaration(arg1);
	}

	private IActivityReasoningEngineDeclaration getActivityReasoningEngineDeclarationIntern() {

		return _activityReasoningEngineDeclarationIntern;
	}

	private void setActivityReasoningEngineDeclarationIntern(IActivityReasoningEngineDeclaration newValue) {

		if (_activityReasoningEngineDeclarationIntern != newValue) {
			try {
				IActivityReasoningEngineDeclaration oldValue = getActivityReasoningEngineDeclarationIntern();
				/* Stop listening for events from the current object */
				if (_activityReasoningEngineDeclarationIntern != null) {
					_activityReasoningEngineDeclarationIntern.removePropertyChangeListener(_eventHandler);
				}
				_activityReasoningEngineDeclarationIntern = newValue;

				/* Listen for events from the new object */
				if (_activityReasoningEngineDeclarationIntern != null) {
					_activityReasoningEngineDeclarationIntern.addPropertyChangeListener(_eventHandler);
				}
				connPtoP1SetTarget();
				firePropertyChange("activityReasoningEngineDeclaration", oldValue, newValue);

			} catch (Throwable ivjExc) {

				handleException(ivjExc);
			}
		}

	}

	private ActivityGraphDeclarationGraphPanel getComposedActivityDeclarationGraphPanel() {
		if (_composedActivityDeclarationGraphPanel == null) {
				_composedActivityDeclarationGraphPanel = new ActivityGraphDeclarationGraphPanel();
		}
		return _composedActivityDeclarationGraphPanel;
	}

	private void connEtoC1(PropertyChangeEvent arg1) {
		try {

			this.firePropertyChange("gUISeSAmDeclaration", arg1.getOldValue(), arg1.getNewValue());

		} catch (Throwable ivjExc) {

			handleException(ivjExc);
		}
	}

	private void connEtoC2(PropertyChangeEvent arg1) {
		try {

			this.firePropertyChange("activityAgentClassDeclaration", arg1.getOldValue(), arg1.getNewValue());

		} catch (Throwable ivjExc) {

			handleException(ivjExc);
		}
	}

	private void connPtoP1SetTarget() {

		try {
			if (_connPtoP1Aligning == false) {

				_connPtoP1Aligning = true;
				if ((getActivityReasoningEngineDeclarationIntern() != null)) {
					getComposedActivityDeclarationGraphPanel().setComposedActivityDeclaration(
						getActivityReasoningEngineDeclarationIntern().getRootActivityGraphDeclaration());
				}

				_connPtoP1Aligning = false;
			}
		} catch (Throwable ivjExc) {
			_connPtoP1Aligning = false;

			handleException(ivjExc);
		}
	}

	private void handleException(Throwable exception) {

		exception.printStackTrace(System.out);
	}

	private void initConnections() {

		getComposedActivityDeclarationGraphPanel().addPropertyChangeListener(_eventHandler);
		connPtoP1SetTarget();
	}

	private void initialize() {
		try {

			setName("ActivityReasoningEngineDeclarationPanel");
			setLayout(new java.awt.BorderLayout());
			setSize(323, 319);
			add(getComposedActivityDeclarationGraphPanel(), "Center");
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}

	}
}
