package de.uniwue.ki.gui.mas.entity.object;
import java.awt.FlowLayout;
import java.util.List;

import javax.swing.*;

import de.uniwue.ki.gui.mas.basics.feature.spatial.ObjectWithSpatialInfoPanel;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.JEditMutableListPanel;
import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.list.DefaultMutableListModel;
import de.uniwue.ki.util.list.IMutableListModel;

/**
 * Creation date: (11.10.2000 15:02:47)
 * @author Christoph Oechslein
 */
public class ObjectGroupPanel extends JEditMutableListPanel {
	private IWorld _world;
	private IMutableListModel _objectGroup;
	private ISpatialMapModel _spatialMapModel = null;
	private IGUISeSAmDeclaration _guiSeSAmDeclarationIntern = null;
	private boolean _sortedList = true;

	public ObjectGroupPanel() {
		super();
		initialize();
	}

	protected Object createItem() {
		return null;
	}

	public void dispose() {
		setObjectGroup(null);
		_world = null;
		_spatialMapModel = null;
	}

	public void editItem(Object obj) {
		editObject((IObject) obj);
	}

	protected void editObject(IObject object) {
		final ObjectWithSpatialInfoPanel objectPanel = new ObjectWithSpatialInfoPanel();

		objectPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
		objectPanel.setSpatialMapModel(getSpatialMapModel());
		objectPanel.setObject(object);
		objectPanel.setAutomaticUpdate(true);

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		buttonPanel.add(new JButton(new AbstractBoundAction("Refresh") {
			public void doit() {
				objectPanel.refresh();
			}
		}));

		Box box = Box.createVerticalBox();
		box.add(objectPanel);
		box.add(Box.createVerticalStrut(3));
		box.add(new JSeparator());
		box.add(Box.createVerticalStrut(3));
		box.add(buttonPanel);
		box.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));

		GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
			this,
			"Edit Object '" + object.getName() + "'",
			box,
			true);
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclarationIntern;
	}

	private void initialize() {
		setName("WorldList");
		setCellRenderer(new NamedCellRendererAdapter(getCellRenderer()));
		setSize(188, 237);
		setEditable(false);
		setModel(getObjectGroup());
	}

	protected void popupItemIntern() {
		if (getClickedValue() != null) {
			int x = getClickedPoint().x;
			int y = getClickedPoint().y;

			JPopupMenu jPopupMenu =
				getGUISeSAmDeclaration().getPanelActionFactoryManager().createPopup(this, getClickedValue());

			jPopupMenu.show(getJList(), x, y);
		}

		//			int x = getClickedPoint().x;
		//			int y = getClickedPoint().y;
		//
		//			JPopupMenu jPopupMenu = new JPopupMenu();
		//
		//			jPopupMenu.add(AbstractBoundAction.createJMenuItem(new AbstractAction("Properties") {
		//				public void actionPerformed(ActionEvent evt) {
		//					editItem(getClickedValue());
		//				}
		//			}, true));
		//
		//			jPopupMenu.addSeparator();
		//
		//			jPopupMenu.add(AbstractBoundAction.createJMenuItem(new AbstractAction("Inspect") {
		//				public void actionPerformed(ActionEvent evt) {
		//					de.uniwue.ki.util.Utils.inspect(getClickedValue());
		//				}
		//			}, true));
		//			
		//			jPopupMenu.show(this, x, y);
		//		}
	}

	public void refresh() {
		if (getWorld() != null) {
			Object oldValue = null;

			if (getModel().getSize() > 0)
				oldValue = getSelectedValue();

			updateObjectGroup();

			if ((oldValue != null) && (getObjectGroup().contains(oldValue))) {
				setSelectedValue(oldValue);

			} else {
				clearSelection();
			}
		}

		repaint();
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newValue) {
		if (_guiSeSAmDeclarationIntern != newValue) {
			IGUISeSAmDeclaration oldValue = getGUISeSAmDeclaration();
			_guiSeSAmDeclarationIntern = newValue;
			firePropertyChange("gUISeSAmDeclaration", oldValue, newValue);
		}
	}

	private void updateObjectGroup() {
		if (getWorld() != null) {

			List newList;
			if (isSortedList()) {
				newList =
					CollectionsHelper.asSortedArrayList(
						getWorld().getAllObjects(false, false),
						NamedComparator.getInstance());
			} else {
				newList = CollectionsHelper.asLinkedList(getWorld().getAllObjects(false, false));
			}

			setObjectGroup(new DefaultMutableListModel(newList));
		}
	}

	public ISpatialMapModel getSpatialMapModel() {
		return _spatialMapModel;
	}

	public void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		_spatialMapModel = spatialMapModel;
	}

	public IWorld getWorld() {
		return _world;
	}

	public void setWorld(IWorld world) {
		_world = world;

		updateObjectGroup();
	}

	public void setObjectGroup(IMutableListModel objectGroup) {
		_objectGroup = objectGroup;

		setModel(_objectGroup);

		if ((objectGroup != null) && (objectGroup.size() > 0)) {
			setSelectedIndex(0);
		}
	}

	public IMutableListModel getObjectGroup() {
		return _objectGroup;
	}

	public void setSortedList(boolean sortedList) {
		_sortedList = sortedList;
	}

	public boolean isSortedList() {
		return _sortedList;
	}
}