package de.uniwue.ki.gui.mas.entity.object;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.extras.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.entity.body.BodyPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (12.10.2000 13:46:18)
 * @author Christoph Oechslein
 */
public class ObjectPanel extends AbstractObjectPanel {
	private boolean _automaticUpdate = false;

	private IObject ivjObjectIntern = null;
	private BodyPanel ivjBodyPanel = null;
	private IObjectInstanceDeclaration ivjObjectInstanceDeclaration = null;
	private JTextField ivjJTextFieldAction = null;
	private JTextField ivjJTextFieldActivity = null;

	/**
	 * ObjectPanel constructor comment.
	 */
	public ObjectPanel() {
		super();
		initialize();
	}

	/**
	 * connEtoC1:  (ObjectIntern.this --> ObjectPanel.eventObjectChanged()V)
	 * @param value de.uniwue.ki.mas.model.entity.object.IObject
	 */
	private void connEtoC1(IObject value) {
		try {
					this.eventObjectChanged();
				} catch (Throwable ivjExc) {
					handleException(ivjExc);
		}
	}

	/**
	 * connEtoM1:  (ObjectIntern.this --> ObjectClassDeclaration.this)
	 * @param value de.uniwue.ki.mas.model.entity.objectclass.IObject
	 */
	private void connEtoM1(IObject value) {
		try {
					if ((getObjectIntern() != null)) {
				setObjectInstanceDeclaration(getObjectIntern().getObjectInstanceDeclaration());
			}
				} catch (Throwable ivjExc) {
					handleException(ivjExc);
		}
	}

	/**
	 * connEtoM3:  (ObjectIntern.this --> BodyPanel.body)
	 * @param value de.uniwue.ki.mas.model.entity.object.IObject
	 */
	private void connEtoM3(IObject value) {
		try {
					if ((getObjectIntern() != null)) {
				getBodyPanel().setBody(getObjectIntern().getBody());
			}
				} catch (Throwable ivjExc) {
					handleException(ivjExc);
		}
	}

	public void dispose() {
		super.dispose();
	}

	protected void eventObjectChanged() {
		updateTextFieldEnabledState();

		if (getObject() instanceof IAgent) {
			IAgent agent = (IAgent)getObject();

			String actionName = getName(agent.getCurrentAction());
			if (!actionName.equals(getJTextFieldAction().getText()))
				getJTextFieldAction().setText(actionName);

			if (agent instanceof IActivityAgent) {
				IActivityAgent activityAgent = (IActivityAgent)agent;

				String activityName = getName(activityAgent.getCurrentActivityGraphLeafs());
				if (!activityName.equals(getJTextFieldActivity().getText()))
					getJTextFieldActivity().setText(activityName);
			}

		} else {

			getJTextFieldActivity().setText("");
			getJTextFieldAction().setText("");
		}
	}
	protected void updateTextFieldEnabledState() {
		boolean enabled = isEnabled() && ((getObject() == null) || (getObject() instanceof IAgent));

		getJTextFieldActivity().setEnabled(enabled);
		getJTextFieldAction().setEnabled(enabled);
	}

	/**
	 * Return the BodyPanel property value.
	 * @return de.uniwue.ki.gui.mas.model.entity.body.BodyPanel
	 */
	private BodyPanel getBodyPanel() {
		if (ivjBodyPanel == null) {
			try {
				ivjBodyPanel = new BodyPanel();
				ivjBodyPanel.setName("BodyPanel");
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjBodyPanel;
	}

	/**
	 * Return the JTextFieldAction property value.
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JTextField getJTextFieldAction() {
		if (ivjJTextFieldAction == null) {
			try {
				ivjJTextFieldAction = new javax.swing.JTextField();
				ivjJTextFieldAction.setName("JTextFieldAction");
				ivjJTextFieldAction.setEditable(false);
				ivjJTextFieldAction.setColumns(10);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldAction;
	}

	/**
	 * Return the JTextFieldActivity property value.
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JTextField getJTextFieldActivity() {
		if (ivjJTextFieldActivity == null) {
			try {
				ivjJTextFieldActivity = new javax.swing.JTextField();
				ivjJTextFieldActivity.setName("JTextFieldActivity");
				ivjJTextFieldActivity.setEditable(false);
				ivjJTextFieldActivity.setColumns(10);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjJTextFieldActivity;
	}

	private final String getName(INamed named) {
		if (named == null)
			return "<null>";
		else
			return named.getName();
	}

	private final String getName(INamed[] named) {
		if (named == null) {
			return "<null>";

		} else if (named.length == 1) {
			return getName(named[0]);

		} else {
			StringBuffer buff = new StringBuffer("[");
			for (int i = 0; i < named.length; i++) {
				if (named[i] != null) {
					named[i].appendName(buff, 0, 10);
				} else {
					buff.append("null");
				}
				if (i < named.length - 1) {
					buff.append(", ");
				}
			}
			buff.append("]");
			return buff.toString();
		}
	}

	/**
	 * Method generated to support the promotion of the object attribute.
	 * @return de.uniwue.ki.mas.model.entity.objectclass.IObject
	 */
	public IObject getObject() {
		return getObjectIntern();
	}

	/**
	 * Return the ObjectIntern property value.
	 * @return de.uniwue.ki.mas.model.entity.objectclass.IObject
	 */
	private IObject getObjectIntern() {
		return ivjObjectIntern;
	}

	/**
	 * Called whenever the part throws an exception.
	 * @param exception Throwable
	 */
	private void handleException(Throwable exception) {

		exception.printStackTrace(System.out);
	}

	protected void ignoreNullPointerException(Throwable exception) {
		if (!(exception instanceof NullPointerException))
			handleException(exception);
	}

	/**
	 * Initializes connections
	 * @exception Exception The exception description.
	 */
	private void initConnections() {
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {

		JPanel panel = new JPanel();

		DefaultFormBuilder builder =
			new DefaultFormBuilder(panel, new FormLayout("right:default, 1dlu, fill:default:grow"));

		builder.append("Action:", getJTextFieldAction());
		builder.nextLine();
		builder.append("Activity:", getJTextFieldActivity());

		PanelBuilder builder2 =
			new PanelBuilder(
				this,
				new FormLayout("fill:default:grow", "default, 2dlu, fill:default:grow"));

		builder2.add(panel);
		builder2.nextLine(2);
		builder2.add(getBodyPanel());

		initConnections();
	}

	public static void main(String[] args) {
		try {
			JFrame frame = new JFrame();
			ObjectPanel aObjectPanel;
			aObjectPanel = new ObjectPanel();
			frame.setContentPane(aObjectPanel);
			frame.setSize(aObjectPanel.getSize());
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				};
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of javax.swing.JPanel");
			exception.printStackTrace(System.out);
		}
	}

	public void refresh() {
		eventObjectChanged();
		connEtoM1(getObject());
		connEtoM3(getObject());
		connEtoC1(getObject());

		getBodyPanel().refresh();
	}

	/**
	 * Method generated to support the promotion of the object attribute.
	 * @param arg1 de.uniwue.ki.mas.model.entity.objectclass.IObject
	 */
	public void setObject(IObject arg1) {
		if (arg1 != getObject())
			setObjectIntern(arg1);
	}

	/**
	 * Set the ObjectInstanceDeclaration to a new value.
	 * @param newValue de.uniwue.ki.mas.model.entity.object.IObjectInstanceDeclaration
	 */
	private void setObjectInstanceDeclaration(IObjectInstanceDeclaration newValue) {
		if (ivjObjectInstanceDeclaration != newValue) {
			try {
				ivjObjectInstanceDeclaration = newValue;
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		};
	}

	/**
	 * Set the ObjectIntern to a new value.
	 * @param newValue de.uniwue.ki.mas.model.entity.objectclass.IObject
	 */
	private void setObjectIntern(IObject newValue) {
		if (ivjObjectIntern != newValue) {
			try {
				IObject oldValue = getObjectIntern();
				ivjObjectIntern = newValue;
				connEtoM1(ivjObjectIntern);
				connEtoM3(ivjObjectIntern);
				connEtoC1(ivjObjectIntern);
				firePropertyChange("object", oldValue, newValue);
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		};
	}

	public boolean isAutomaticUpdate() {
		return _automaticUpdate;
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		_automaticUpdate = automaticUpdate;
	}

	/**
	 * Method generated to support the promotion of the gUISeSAmDeclaration attribute.
	 * @param arg1 de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration
	 */
	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration arg1) {
		super.setGUISeSAmDeclaration(arg1);
		getBodyPanel().setGUISeSAmDeclaration(arg1);
	}

	public void setEnabled(boolean b) {
		super.setEnabled(b);

		updateTextFieldEnabledState();
	}
}