package de.uniwue.ki.gui.mas.entity.objectclass;
import gnu.trove.TObjectIntHashMap;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.*;

import javax.swing.*;

import de.uniwue.ki.gui.mas.entity.bodyclass.BodyClassDeclarationSettingsWithTabsPanel;
import de.uniwue.ki.gui.mas.model.IDeclarationPanelAndActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (16.06.2000 16:51:25)
 * @author Christoph Oechslein
 */
public class ObjectClassDeclarationPanel extends AbstractDeclarationPanel {
	private IObjectClassDeclaration _objectClassDeclaration;

	private final DeclarationListListener _objectFeatureClassDeclarationList =
		new DeclarationListListener() {
		public void declarationAdded(DeclarationListEvent event) {
			insertFeatureTabs();
		}

		public void declarationRemoved(DeclarationListEvent event) {
			insertFeatureTabs();
		}
	};

	private TObjectIntHashMap _oldObjectFeatureClassDeclarationMap = new TObjectIntHashMap();
	private BodyClassDeclarationSettingsWithTabsPanel ivjBodyClassDeclarationSettingsPanel = null;
	private ObjectFeatureListSettingsPanel ivjFeatureClassDeclarationListToFeatureClassDeclarationList = null;
	private JPanel ivjJPanel1 = null;
	private JPanel ivjJPanelBodyFeature = null;
	private JPanel ivjJPanelBodyVariables = null;
	private JTabbedPane ivjJTabbedPane = null;
	private NameDeclarationWithDocumentationPanel ivjNameDeclarationWithDocumentationPanel = null;

	public ObjectClassDeclarationPanel() {
		super();
		initialize();
	}

	public void dispose() {
		getBodyClassDeclarationSettingsPanel().dispose();
		getFeatureClassDeclarationListToFeatureClassDeclarationList().dispose();

		super.dispose();
	}

	private BodyClassDeclarationSettingsWithTabsPanel getBodyClassDeclarationSettingsPanel() {
		if (ivjBodyClassDeclarationSettingsPanel == null) {
			ivjBodyClassDeclarationSettingsPanel = new BodyClassDeclarationSettingsWithTabsPanel();
		}
		return ivjBodyClassDeclarationSettingsPanel;
	}

	public IDeclaration getDeclaration() {
		return getObjectClassDeclaration();
	}

	private ObjectFeatureListSettingsPanel getFeatureClassDeclarationListToFeatureClassDeclarationList() {
		if (ivjFeatureClassDeclarationListToFeatureClassDeclarationList == null) {
			ivjFeatureClassDeclarationListToFeatureClassDeclarationList =
				new ObjectFeatureListSettingsPanel();
		}
		return ivjFeatureClassDeclarationListToFeatureClassDeclarationList;
	}

	private JPanel getJPanel1() {
		if (ivjJPanel1 == null) {
			ivjJPanel1 = new JPanel();
			ivjJPanel1.setName("JPanel1");
			ivjJPanel1.setLayout(new BoxLayout(getJPanel1(), BoxLayout.X_AXIS));
			getJPanel1().add(
				getBodyClassDeclarationSettingsPanel(),
				getBodyClassDeclarationSettingsPanel().getName());
			//getJPanel1().add(new HorizontalGlue());
		}
		return ivjJPanel1;
	}

	private JPanel getBodyFeaturePanel() {
		if (ivjJPanelBodyFeature == null) {
			ivjJPanelBodyFeature = new JPanel();
			ivjJPanelBodyFeature.setLayout(new BoxLayout(getBodyFeaturePanel(), BoxLayout.Y_AXIS));
			getBodyFeaturePanel().add(
				getFeatureClassDeclarationListToFeatureClassDeclarationList(),
				getFeatureClassDeclarationListToFeatureClassDeclarationList().getName());
		}
		return ivjJPanelBodyFeature;
	}

	private JPanel getBodyVariablePanel() {
		if (ivjJPanelBodyVariables == null) {
			ivjJPanelBodyVariables = new JPanel();
			ivjJPanelBodyVariables.setLayout(new BoxLayout(getBodyVariablePanel(), BoxLayout.Y_AXIS));
			getBodyVariablePanel().add(getJPanel1(), getJPanel1().getName());
			//getJPanelBodyVariables().add(new VerticalGlue(), new VerticalGlue().getName());
		}
		return ivjJPanelBodyVariables;
	}

	private JTabbedPane getJTabbedPane() {
		if (ivjJTabbedPane == null) {
			ivjJTabbedPane = new JTabbedPane();

			insertTabs();
		}
		return ivjJTabbedPane;
	}

	private NameDeclarationWithDocumentationPanel getNameDeclarationWithDocumentationPanel() {
		if (ivjNameDeclarationWithDocumentationPanel == null) {
			ivjNameDeclarationWithDocumentationPanel = new NameDeclarationWithDocumentationPanel();
		}
		return ivjNameDeclarationWithDocumentationPanel;
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	public IFunctionArgumentDeclarationList getOuterFunctionArgumentDeclarationList() {
		return getBodyClassDeclarationSettingsPanel().getOuterFunctionArgumentDeclarationList();
	}

	private void initialize() {
		setLayout(new BorderLayout());
		add(getJTabbedPane(), BorderLayout.CENTER);
		add(getNameDeclarationWithDocumentationPanel(), BorderLayout.NORTH);
	}

	private void insertFeatureTabs() {
		if (getObjectClassDeclaration() != null) {

			List oldObjectFeatureClassDeclarations =
				Arrays.asList(_oldObjectFeatureClassDeclarationMap.keys());
			Iterator it =
				CollectionsHelper.setDifference(
					oldObjectFeatureClassDeclarations.iterator(),
					getObjectClassDeclaration().getObjectFeatureClassDeclarationList().iterator());
			while (it.hasNext()) {
				IObjectFeatureClassDeclaration objectFeatureClassDeclaration =
					(IObjectFeatureClassDeclaration)it.next();
				ivjJTabbedPane.removeTabAt(
					_oldObjectFeatureClassDeclarationMap.get(objectFeatureClassDeclaration));
				_oldObjectFeatureClassDeclarationMap.remove(objectFeatureClassDeclaration);
			}

			it =
				CollectionsHelper.setDifference(
					getObjectClassDeclaration().getObjectFeatureClassDeclarationList().iterator(),
					oldObjectFeatureClassDeclarations.iterator());
			
			while (it.hasNext()) {
				IObjectFeatureClassDeclaration objectFeatureClassDeclaration =
					(IObjectFeatureClassDeclaration)it.next();
		
				IDeclarationPanelAndActionFactory panelActionFactory =
					(IDeclarationPanelAndActionFactory)getGUISeSAmDeclaration()
						.getPanelActionFactoryManager()
						.getPanelActionFactoryFor(this, objectFeatureClassDeclaration);

				if ((panelActionFactory != null) && (!(objectFeatureClassDeclaration instanceof IUserFeatureClassDeclaration))) {
					IDeclarationPanelFactory declarationPanelFactory =
						panelActionFactory.getDeclarationPanelFactory(this, objectFeatureClassDeclaration);
					if (declarationPanelFactory != null) {
						IDeclarationPanel declarationPanel =
							declarationPanelFactory.createDeclarationPanel(objectFeatureClassDeclaration);
						declarationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
						declarationPanel.setDeclaration(objectFeatureClassDeclaration);
						_oldObjectFeatureClassDeclarationMap.put(
							objectFeatureClassDeclaration,
							ivjJTabbedPane.getComponentCount());
						insertTab(
							declarationPanelFactory.getDeclarationTitle(objectFeatureClassDeclaration),
							null,
							(Component)declarationPanel,
							"",
							ivjJTabbedPane.getComponentCount());
					}
				}
			}
		}
	}

	public void insertTab(String name, Icon icon, Component component, String tip, int index) {
		ivjJTabbedPane.insertTab(name, icon, component, tip, index);
	}

	protected void insertTabs() {
		removeAllTabs();

		insertTab("Variables", null, getBodyVariablePanel(), null, 0);
		insertTab("Features", null, getBodyFeaturePanel(), null, 1);
	}

	protected void removeAllTabs() {
		for (int i = 0; i < ivjJTabbedPane.getComponentCount(); i++) {
			ivjJTabbedPane.remove(0);
		}
	}

	public void setDeclaration(IDeclaration declaration) {
		setObjectClassDeclaration((IObjectClassDeclaration)declaration);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		getFeatureClassDeclarationListToFeatureClassDeclarationList().setGUISeSAmDeclaration(
			guiSeSAmDeclaration);
		getBodyClassDeclarationSettingsPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getNameDeclarationWithDocumentationPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void setObjectClassDeclaration(IObjectClassDeclaration newValue) {
		IObjectClassDeclaration oldValue = getObjectClassDeclaration();
		if (oldValue != newValue) {
			if (oldValue != null) {
				oldValue.getObjectFeatureClassDeclarationList().removeDeclarationListListener(
					_objectFeatureClassDeclarationList);
			}

			_objectClassDeclaration = newValue;

			getFeatureClassDeclarationListToFeatureClassDeclarationList().setObjectClassDeclaration(newValue);
			getBodyClassDeclarationSettingsPanel().setObjectClassDeclaration(newValue);
			getNameDeclarationWithDocumentationPanel().setDeclaration(newValue);

			insertFeatureTabs();

			if (newValue != null) {
				newValue.getObjectFeatureClassDeclarationList().addDeclarationListListener(
					_objectFeatureClassDeclarationList);
			}
		}
	}

	public void setOuterFunctionArgumentDeclarationList(IFunctionArgumentDeclarationList functionArgumentDeclarationList) {
		getBodyClassDeclarationSettingsPanel().setOuterFunctionArgumentDeclarationList(
			functionArgumentDeclarationList);
	}
}