package de.uniwue.ki.gui.mas.entity.objectclass.factory;

import java.awt.Container;
import java.util.Iterator;

import de.uniwue.ki.gui.mas.basics.search.SearchDeclarationPanelAction;
import de.uniwue.ki.gui.mas.model.AbstractDeclarationPanelAndActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.IDeclarationPanelFactory;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (10.10.2001 11:09:49)
 * @author Christoph Oechslein
 */

public class ActivityAgentClassDeclarationPanelActionFactory extends AbstractDeclarationPanelAndActionFactory {
	private final EditActivityAgentClassDeclarationPanelFactory _editPropertiesDeclarationPanelFactory =
		new EditActivityAgentClassDeclarationPanelFactory();

	private final EditReasoningActivityAgentClassDeclarationPanelFactory _editReasoningDeclarationPanelFactory =
		new EditReasoningActivityAgentClassDeclarationPanelFactory();

	private final EditVariablesObjectClassDeclarationPanelFactory _editVariablesDeclarationPanelFactory =
		new EditVariablesObjectClassDeclarationPanelFactory();

	private final EditFeaturesObjectClassDeclarationPanelFactory _editFeaturesDeclarationPanelFactory =
		new EditFeaturesObjectClassDeclarationPanelFactory();

	public ActivityAgentClassDeclarationPanelActionFactory() {
		super();
	}

	public ActivityAgentClassDeclarationPanelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {

		super();

		setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	protected boolean getCanProduceDeclarationActions(Container parent, IDeclaration anDeclaration) {
		return anDeclaration instanceof IActivityAgentClassDeclaration;
	}

	protected Iterator getImportantDeclarationActions(Container parent, IDeclaration anDeclaration) {
		return CollectionsHelper.createIterator(
			new DeclarationPanelAction(
				parent,
				"Reasoning",
				SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif"),
				anDeclaration,
				_editReasoningDeclarationPanelFactory,
				false),
			new DeclarationPanelAction(
				parent,
				"Variables",
				SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif"),
				anDeclaration,
				_editVariablesDeclarationPanelFactory,
				false),
			new DeclarationPanelAction(
				parent,
				"Features",
				SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif"),
				anDeclaration,
				_editFeaturesDeclarationPanelFactory,
				false),
			new SearchDeclarationPanelAction(parent, anDeclaration, getGUISeSAmDeclaration()));
	}

	public IDeclarationPanelFactory getDeclarationPanelFactory(
		Container parent,
		IDeclaration anDeclaration) {
		return _editPropertiesDeclarationPanelFactory;
	}

	protected boolean shouldUseOkCancelDeclarationPanel(Container parent, IDeclaration anDeclaration) {
		return false;
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newGUISeSAmDeclaration) {
		super.setGUISeSAmDeclaration(newGUISeSAmDeclaration);

		_editPropertiesDeclarationPanelFactory.setGUISeSAmDeclaration(newGUISeSAmDeclaration);
		_editReasoningDeclarationPanelFactory.setGUISeSAmDeclaration(newGUISeSAmDeclaration);
		_editVariablesDeclarationPanelFactory.setGUISeSAmDeclaration(newGUISeSAmDeclaration);
		_editFeaturesDeclarationPanelFactory.setGUISeSAmDeclaration(newGUISeSAmDeclaration);
	}
}
