package de.uniwue.ki.gui.mas.entity.variable;
import java.awt.FontMetrics;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.TableColumn;

import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectTypePair;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (13.10.2000 13:42:28)
 * @author Christoph Oechslein
 */
public class BodyVariableTablePanel extends JPanelWithGUISeSAmDeclaration {
	private JTable ivjScrollPaneTable = null;
	private TableColumn ivjTableColumnDeclarationName = null;
	private TableColumn ivjTableColumnDeclarationType = null;
	private TableColumn ivjTableColumnValue = null;
	private TableModelVariableListAdapter ivjTableModelVariableListAdapterFactory = null;
	private IBody ivjBodyIntern = null;
	private JScrollPane ivjJScrollPane = null;
	IvjEventHandler eventHandler = new IvjEventHandler(this);

	class IvjEventHandler extends MouseAdapter implements MouseListener {
		private BodyVariableTablePanel _parent = null;

		public IvjEventHandler(BodyVariableTablePanel parent) {
			_parent = parent;
		}

		private JPopupMenu getPopupMenu(ISpecificType type, Object object)
			throws SimulationRuntimeException {
			return _parent.getGUISeSAmDeclaration().getPanelActionFactoryManager().createPopup(
				_parent,
				new ObjectTypePair(type, object));
		}

		private Object getItemOfEvent(MouseEvent mouseEvent) throws SimulationRuntimeException {

			IBodyVariable bodyVariable = getBodyVariableOf(mouseEvent);

			return bodyVariable.getValue();
		}

		private IBodyVariable getBodyVariableOf(MouseEvent mouseEvent) {
			JTable table = ((JTable)mouseEvent.getSource());

			int rowIndex = table.rowAtPoint(mouseEvent.getPoint());

			return getBody().getBodyVariableAt(rowIndex);
		}

		private ISpecificType getTypeOfEvent(MouseEvent mouseEvent) throws SimulationRuntimeException {
			return getBody()
				.getBodyVariableInstanceDeclarationFor(getBodyVariableOf(mouseEvent))
				.getBodyVariableClassDeclaration()
				.getVariableClassDeclaration()
				.getSpecificType();
		}

		/**
		* Invoked when the mouse has been clicked on a component.
		*/
		public void mouseClicked(MouseEvent mouseEvent) {
			//rechte Maustaste
			if (mouseEvent.getModifiers() == 4) {
				try {
					JPopupMenu popup = getPopupMenu(getTypeOfEvent(mouseEvent), getItemOfEvent(mouseEvent));
					if (popup != null)
						popup.show(mouseEvent.getComponent(), mouseEvent.getX(), mouseEvent.getY());

				} catch (SimulationRuntimeException e) {
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * VariableList constructor comment.
	 */
	public BodyVariableTablePanel() {
		super();
		initialize();
	}

	/**
	 * connEtoM3:  (BodyIntern.this --> ScrollPaneTable.model)
	 * @param value de.uniwue.ki.mas.model.entity.body.IBody
	 */
	private void connEtoM3(IBody value) {
		TableModelVariableListAdapter localValue = null;
		try {
					getScrollPaneTable().setModel(localValue = new TableModelVariableListAdapter(getBodyIntern()));
				} catch (Throwable ivjExc) {
					handleException(ivjExc);
		}
		setTableModelVariableListAdapterFactory(localValue);
	}

	/**
	 * Method generated to support the promotion of the body attribute.
	 * @return de.uniwue.ki.mas.model.entity.body.IBody
	 */
	public IBody getBody() {
		return getBodyIntern();
	}

	/**
	 * Return the BodyIntern property value.
	 * @return de.uniwue.ki.mas.model.entity.body.IBody
	 */
	private IBody getBodyIntern() {
		return ivjBodyIntern;
	}

	/**
	 * Return the JScrollPane1 property value.
	 * @return JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (ivjJScrollPane == null) {
			try {
				ivjJScrollPane = new JScrollPane();
				ivjJScrollPane.setName("JScrollPane");
				getJScrollPane().setViewportView(getScrollPaneTable());
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJScrollPane;
	}

	/**
	 * Return the ScrollPaneTable property value.
	 * @return JTable
	 */
	private JTable getScrollPaneTable() {
		if (ivjScrollPaneTable == null) {
			try {
				ivjScrollPaneTable = new JTable();
				ivjScrollPaneTable.setName("ScrollPaneTable");
				//getJScrollPane().setColumnHeaderView(ivjScrollPaneTable.getTableHeader());
				ivjScrollPaneTable.setBounds(0, 0, 200, 200);
				ivjScrollPaneTable.setAutoCreateColumnsFromModel(false);
				ivjScrollPaneTable.addColumn(getTableColumnDeclarationName());
				ivjScrollPaneTable.addColumn(getTableColumnValue());
				ivjScrollPaneTable.addColumn(getTableColumnDeclarationType());
						//				ivjScrollPaneTable.setPreferredScrollableViewportSize(
				//					new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE));
					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjScrollPaneTable;
	}

	/**
	 * Return the TableColumnDeclarationName property value.
	 * @return TableColumn
	 */
	private TableColumn getTableColumnDeclarationName() {
		if (ivjTableColumnDeclarationName == null) {
			try {
				ivjTableColumnDeclarationName = new TableColumn();
				ivjTableColumnDeclarationName.setModelIndex(0);
				ivjTableColumnDeclarationName.setHeaderValue("Name");
						ivjTableColumnDeclarationName.setCellRenderer(new BodyVariableTablePanelCellRenderer(true));
					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjTableColumnDeclarationName;
	}

	/**
	 * Return the TableColumnDeclarationType property value.
	 * @return TableColumn
	 */
	private TableColumn getTableColumnDeclarationType() {
		if (ivjTableColumnDeclarationType == null) {
			try {
				ivjTableColumnDeclarationType = new TableColumn();
				ivjTableColumnDeclarationType.setModelIndex(2);
				ivjTableColumnDeclarationType.setHeaderValue("Type");
						ivjTableColumnDeclarationType.setCellRenderer(new BodyVariableTablePanelCellRenderer(false));
					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjTableColumnDeclarationType;
	}

	/**
	     * Return the TableColumnValue property value.
	     * @return TableColumn
	     */
	private TableColumn getTableColumnValue() {
		if (ivjTableColumnValue == null) {
			try {
				ivjTableColumnValue = new TableColumn();
				ivjTableColumnValue.setModelIndex(1);
				ivjTableColumnValue.setHeaderValue("Value");
						ivjTableColumnValue.setCellRenderer(new BodyVariableTablePanelCellRenderer(true));
					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
		return ivjTableColumnValue;
	}

	/**
	 * Return the TableModelVariableListAdapterFactory property value.
	 * @return entity.variable2.TableModelVariableListAdapter
	 */
	private TableModelVariableListAdapter getTableModelVariableListAdapterFactory() {
		return ivjTableModelVariableListAdapterFactory;
	}

	/**
	 * Called whenever the bean information class throws an exception.
	 * @param exception Throwable
	 */
	private void handleException(Throwable exception) {

		exception.printStackTrace(System.out);
	}

	/**
	 * Initializes connections
	 * @exception Exception The exception description.
	 */
	private void initConnections() {
		getScrollPaneTable().addMouseListener(eventHandler);
	}

	/**
	     * Initialize the class.
	     */
	private void initialize() {
		try {
					setName("VariableList");
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setSize(322, 318);
			add(getJScrollPane(), getJScrollPane().getName());
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
		ToolTipManager.sharedInstance().registerComponent(getScrollPaneTable());
	}

	public static void main(String[] args) {
		try {
			JFrame frame = new JFrame();
			BodyVariableTablePanel aBodyVariableListPanel;
			aBodyVariableListPanel = new BodyVariableTablePanel();
			frame.setContentPane(aBodyVariableListPanel);
			frame.setSize(aBodyVariableListPanel.getSize());
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JPanelWithGUISeSAmDeclaration");
			exception.printStackTrace(System.out);
		}
	}

	public void refresh() {
		repaint();
	}

	/**
	 * Method generated to support the promotion of the body attribute.
	 * @param arg1 de.uniwue.ki.mas.model.entity.body.IBody
	 */
	public void setBody(IBody arg1) {
		setBodyIntern(arg1);
	}

	/**
	 * Set the BodyIntern to a new value.
	 * @param newValue de.uniwue.ki.mas.model.entity.body.IBody
	 */
	private void setBodyIntern(IBody newValue) {
		if (ivjBodyIntern != newValue) {
			try {
				IBody oldValue = getBodyIntern();
				ivjBodyIntern = newValue;
				connEtoM3(ivjBodyIntern);
				firePropertyChange("body", oldValue, newValue);
		
				if (newValue != null) {
					getScrollPaneTable().setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

					FontMetrics fm = getFontMetrics(getScrollPaneTable().getFont());

					for (int col = 0; col < getScrollPaneTable().getColumnCount(); col++) {
						TableColumn tableColumn = getScrollPaneTable().getColumnModel().getColumn(col);
						int maxWidth = tableColumn.getPreferredWidth();
						for (int row = 0; row < getScrollPaneTable().getRowCount(); row++) {
							int width =
								fm.stringWidth(
									BodyVariableTablePanelCellRenderer.getStringValue(
										getScrollPaneTable().getModel(),
										row,
										col))
									+ 6;
							if (width > maxWidth)
								maxWidth = width;
						}
						tableColumn.setPreferredWidth(maxWidth);
					}
				}

					} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
	}

	/**
	 * Set the TableModelVariableListAdapterFactory to a new value.
	 * @param newValue entity.variable2.TableModelVariableListAdapter
	 */
	private void setTableModelVariableListAdapterFactory(TableModelVariableListAdapter newValue) {
		if (ivjTableModelVariableListAdapterFactory != newValue) {
			try {
				ivjTableModelVariableListAdapterFactory = newValue;
							} catch (Throwable ivjExc) {
								handleException(ivjExc);
			}
		}
	}

	public String getToolTipText(MouseEvent event) {
		int columnIndex = getScrollPaneTable().columnAtPoint(event.getPoint());
		int rowIndex = getScrollPaneTable().rowAtPoint(event.getPoint());

		if ((columnIndex != -1) && (rowIndex != -1))
			return String.valueOf(
				getTableModelVariableListAdapterFactory().getValueAt(rowIndex, columnIndex));
		else
			return null;
	}

	public void setGUISeSAmDeclaration(de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration arg1) {
		super.setGUISeSAmDeclaration(arg1);
	}
}