package de.uniwue.ki.gui.mas.entity.variable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (13.10.2000 13:51:38)
 * @author Christoph Oechslein
 */
public class TableModelVariableListAdapter extends AbstractTableModel implements TableModel {

	private final IBody _body;

	public TableModelVariableListAdapter(IBody body) {
		super();

		_body = body;
	}

	public int getColumnCount() {
		return 3;
	}

	//TODO gendert wegen NullPointerException beim "Reset"; Mhlberger 01.08.2003
	public int getRowCount() {
		if (_body != null) {
			return _body.getBodyVariablesLength();
		} else {
			return 0;
		}
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		switch (columnIndex) {
			case 0 :
				IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclarationAt(rowIndex);

				if (variableClassDeclaration == null)
					return "var name at index " + rowIndex + "vanished";
				else
					return variableClassDeclaration;

			case 1 :
				return getVariableAt(rowIndex);

			case 2 :
				IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
					getVariableInstanceDeclarationAt(rowIndex);

				if (bodyVariableInstanceDeclaration == null)
					return "var type at index " + rowIndex + "vanished";
				else
					return bodyVariableInstanceDeclaration;

		}

		return null;
	}

	IBodyVariable getVariableAt(int index) {
		return _body.getBodyVariableAt(index);
	}

	IVariableClassDeclaration getVariableClassDeclarationAt(int index) {
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
			getVariableInstanceDeclarationAt(index);

		if (bodyVariableInstanceDeclaration == null)
			return null;

		else
			return bodyVariableInstanceDeclaration
				.getBodyVariableClassDeclaration()
				.getVariableClassDeclaration();
	}

	protected IBodyVariableInstanceDeclaration getVariableInstanceDeclarationAt(int index) {
		try {
			if ((index < _body.getBodyVariablesLength()) && (index != -1))
				return _body.getBodyVariableInstanceDeclarationFor(_body.getBodyVariableAt(index));
			else
				return null;

		} catch (IndexOutOfBoundsException e) {
			System.out.println(
				"didn't find variable " + _body.getBodyVariableAt(index) + "(" + index + ") in ");
			// e.printStackTrace(System.err);

			return null;
		}
	}

	public Object getVariableValueAt(int index) {
		try {
			IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclarationAt(index);

			if (variableClassDeclaration == null)
				return "var value at index " + index + "vanished";

			else {
				return getVariableAt(index).getValue();
			}
		} catch (SimulationRuntimeException exception) {
			return exception.getMessage();
		}
	}

	public String getVariableValueNameAt(int index) {
		try {
			IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclarationAt(index);

			if (variableClassDeclaration == null)
				return "var value at index " + index + "vanished";

			else {
				StringBuffer buff = new StringBuffer();
				variableClassDeclaration.getSpecificType().appendNameForArgumentObject(
					buff,
					getVariableAt(index).getValue(),
					0,
					5);
				return buff.toString();
			}
		} catch (SimulationRuntimeException exception) {
			return exception.getMessage();
		}
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		// return columnIndex == 1;
		return false;
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(columnIndex == 1);

		try {
			getVariableAt(rowIndex).setValue(aValue);

		} catch (SimulationRuntimeException exception) {
		    System.out.println(exception);
		}
	}

	private static final String[] COLUMN_NAMES = { "Name", "Value", "Type" };

	public String getColumnName(int columnIndex) {
		return COLUMN_NAMES[columnIndex];
	}

}