package de.uniwue.ki.gui.mas.entity.variable.viewer;

import java.awt.BorderLayout;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.table.*;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.entity.variable.BodyVariableTablePanelCellRenderer;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeValue;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * @author herrler
 *
 */
public class ObjectValueViewerComposedType
	extends JPanelWithGUISeSAmDeclaration
	implements IObjectValueViewer {

	private JTable _table;
	private IvjEventHandler _eventHander = new IvjEventHandler(this);
	private final IComposedTypeValue _composedTypeValue;
	private final ISpecificComposedType _composedType;

	private final class ComposedTypeTableModel extends AbstractTableModel implements TableModel {

		ComposedTypeTableModel() {
			super();
		}

		public String getColumnName(int columnIndex) {
			if (columnIndex == 0)
				return "Attribute";
			if (columnIndex == 1)
				return "Value";
			Assert.fail();
			return "";
		}

		public int getColumnCount() {
			return 2;
		}

		public int getRowCount() {
			return _composedTypeValue.getComposedType().getAttributeListSize();
		}

		/**
		 * @see javax.swing.table.TableModel#getValueAt(int, int)
		 */
		public Object getValueAt(int rowIndex, int columnIndex) {
			try {
				if (columnIndex == 0)
					return _composedTypeValue.getComposedType().getAttributeAt(rowIndex).getAttributeName();

				if (columnIndex == 1) {
					StringBuffer buff = new StringBuffer();
					_composedType.getAttributeAt(rowIndex).getAttributeType().appendNameForArgumentObject(
						buff,
						_composedTypeValue.getAttributeValue(rowIndex),
						0,
						5);
					return buff.toString();
				}

			} catch (SimulationRuntimeException e) {
				e.printStackTrace();
			}

			return "ERROR";
		}
	}

	class IvjEventHandler extends MouseAdapter implements MouseListener {
		private ObjectValueViewerComposedType _parent = null;

		public IvjEventHandler(ObjectValueViewerComposedType parent) {
			_parent = parent;
		}

		private JPopupMenu getPopupMenu(ISpecificType type, Object object) {
			return _parent.getGUISeSAmDeclaration().getPanelActionFactoryManager().createPopup(
				_parent,
				new ObjectTypePair(type, object));
		}

		private Object getItemOfEvent(MouseEvent mouseEvent) throws SimulationRuntimeException {
			JTable table = ((JTable) mouseEvent.getSource());

			int rowIndex = table.rowAtPoint(mouseEvent.getPoint());

			return _composedTypeValue.getAttributeValue(rowIndex);
		}

		private ISpecificType getTypeOfEvent(MouseEvent mouseEvent) {
			JTable table = ((JTable) mouseEvent.getSource());

			int rowIndex = table.rowAtPoint(mouseEvent.getPoint());

			return _composedType.getAttributeAt(rowIndex).getAttributeType();
		}

		/**
		* Invoked when the mouse has been clicked on a component.
		*/
		public void mouseClicked(MouseEvent mouseEvent) {
			//rechte Maustaste
			if (mouseEvent.getModifiers() == 4) {
				JPopupMenu popup;
				try {
					popup = getPopupMenu(getTypeOfEvent(mouseEvent), getItemOfEvent(mouseEvent));
					if (popup != null)
						popup.show(mouseEvent.getComponent(), mouseEvent.getX(), mouseEvent.getY());

				} catch (SimulationRuntimeException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public String getContextMenuString() {
		return "View Composed Type";
	}

	public boolean canShow(ISpecificType type, Object obj) {
		return (type instanceof ISpecificComposedType) && (obj instanceof IComposedTypeValue);
	}

	public ObjectValueViewerComposedType(IGUISeSAmDeclaration seSAmDeclaration) {
		super();

		_composedType = null;
		_composedTypeValue = null;

		setGUISeSAmDeclaration(seSAmDeclaration);
	}

	public ObjectValueViewerComposedType(
		IGUISeSAmDeclaration seSAmDeclaration,
		ISpecificComposedType composedType,
		IComposedTypeValue composedTypeValue) {

		super();

		_composedType = composedType;
		_composedTypeValue = composedTypeValue;

		setGUISeSAmDeclaration(seSAmDeclaration);

		getTable().addMouseListener(_eventHander);
		JLabel label = new JLabel("DataType: " + composedType.getName());

		getTable().setModel(new ComposedTypeTableModel());
		//_tableModel = (table,composedType);

		this.setBorder(new EtchedBorder());
		this.setLayout(new BorderLayout());
		this.add(label, BorderLayout.NORTH);
		this.add(_table, BorderLayout.CENTER);
	}

	private JTable getTable() {
		if (_table != null)
			return _table;

		_table = new JTable();
		_table.setAutoCreateColumnsFromModel(false);
		TableColumn columnAttribute = new TableColumn();
		columnAttribute.setModelIndex(0);
		columnAttribute.setHeaderValue("Attribute");
		TableColumn columnValue = new TableColumn();
		columnValue.setModelIndex(1);
		columnValue.setHeaderValue("Value");
		columnValue.setCellRenderer(new BodyVariableTablePanelCellRenderer(true));
		_table.addColumn(columnAttribute);
		_table.addColumn(columnValue);
		return _table;
	}

	public final JPanelWithGUISeSAmDeclaration getBodyVariableValueViewerPanel(
		ISpecificType type,
		Object obj) {

		return new ObjectValueViewerComposedType(
			getGUISeSAmDeclaration(),
			(ISpecificComposedType) type,
			(IComposedTypeValue) obj);
	}
}
