package de.uniwue.ki.gui.mas.entity.variable.viewer;

import java.awt.Container;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.*;
import de.uniwue.ki.gui.mas.util.AbstractActionFactory;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author herrler
 */
public class ObjectValueViewerToActionFactoryAdapter extends AbstractActionFactory {
	IObjectValueViewer _viewer;

	class ViewCloslyAction extends AbstractBoundAction {
		private final ObjectTypePair _objectTypePair;
		private final Container _parent;

		public ViewCloslyAction(Container parent, String contextEntry, ObjectTypePair objectTypePair) {
			super(contextEntry, SwingMoreUtilities.getJavaSunIcon(16, "general/Zoom16.gif"));
			_parent = parent;
			_objectTypePair = objectTypePair;
		}

		public void doit() {
			Object object = _objectTypePair.getObject();
			ISpecificType specificType = _objectTypePair.getSpecificType();
			if (Assert.ASSERTION_ON) {
				Assert.assertTrue(
					"Viewer for " + object + " not found",
					_viewer.canShow(specificType, object));
			}

			String frametitle;

			if (object == null)
				frametitle = "null";
			else
				frametitle = "View " + specificType.getName() + " - " + object;

			JPanelWithGUISeSAmDeclaration panel =
				_viewer.getBodyVariableValueViewerPanel(specificType, object);

			GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
				_parent,
				frametitle,
				panel,
				true);
		}
	}

	public ObjectValueViewerToActionFactoryAdapter(
		IGUISeSAmDeclaration sesamDeclaration,
		IObjectValueViewer viewer) {
		super(sesamDeclaration);
		_viewer = viewer;
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		if (anObject instanceof ObjectTypePair)
			return _viewer.canShow(
				((ObjectTypePair) anObject).getSpecificType(),
				((ObjectTypePair) anObject).getObject());
		else
			return false;
	}

	public Iterator getMoreActions(Container parent, Object anObject) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(anObject instanceof ObjectTypePair);

		return CollectionsHelper.createIterator(
			new ViewCloslyAction(parent, _viewer.getContextMenuString(), (ObjectTypePair) anObject),
			new InspectAction(anObject));
	}

}
