package de.uniwue.ki.gui.mas.entity.variableclass;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.ListCellRenderer;

import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (22.11.2001 16:54:03)
 * @author Christoph Oechslein
 */
public class BodyVariableClassDeclarationCellRendererAdapter extends VariableClassDeclarationCellRendererAdapter {
	private IObjectClassDeclaration _objectClassDeclaration;

	public BodyVariableClassDeclarationCellRendererAdapter(ListCellRenderer cellRenderer) {
		super(cellRenderer);
	}

	public static String getBodyVariableClassDeclarationToolTipText(
		IBodyVariableClassDeclaration bodyVariableClassDeclaration,
		IObjectClassDeclaration objectClassDeclaration) {
		StringBuffer buff = new StringBuffer();

		appendBodyVariableClassDeclarationName(
			bodyVariableClassDeclaration,
			objectClassDeclaration,
			true,
			buff);

		IVariableClassDeclaration variableClassDeclaration =
			bodyVariableClassDeclaration.getVariableClassDeclaration();

		if (variableClassDeclaration.getDocumentation().length() > 0) {
			buff.append("<P>");
			buff.append(
				NamedCellRendererAdapter.getQuotedString(variableClassDeclaration.getDocumentation()));
		}

		return "<html>" + buff.toString();
	}

	public static String getBodyVariableClassDeclarationText(
		IBodyVariableClassDeclaration bodyVariableClassDeclaration,
		IObjectClassDeclaration objectClassDeclaration) {
		StringBuffer buff = new StringBuffer();

		appendBodyVariableClassDeclarationName(
			bodyVariableClassDeclaration,
			objectClassDeclaration,
			false,
			buff);

		return "<html>" + buff.toString();
	}

	protected static void appendBodyVariableClassDeclarationName(
		IBodyVariableClassDeclaration bodyVariableClassDeclaration,
		IObjectClassDeclaration objectClassDeclaration,
		boolean withFeatureName,
		StringBuffer buff) {

		IVariableClassDeclaration variableClassDeclaration =
			bodyVariableClassDeclaration.getVariableClassDeclaration();

		buff.append("<strong>");

		NamedCellRendererAdapter.appendQuotedString(variableClassDeclaration.getName(), buff);
		buff.append("</strong> (");
		NamedCellRendererAdapter.appendQuotedString(
			variableClassDeclaration.getSpecificType().getName(),
			buff);
		buff.append(", ");
		buff.append(VariableClassDeclarationCellRendererAdapter.getMarkers(variableClassDeclaration));
		if (bodyVariableClassDeclaration instanceof IBodyFeatureVariableClassDeclaration) {
			buff.append(", ");
			if (withFeatureName)
				appendBodyFeatureVariableClassDeclarationName(
					(IBodyFeatureVariableClassDeclaration) bodyVariableClassDeclaration,
					objectClassDeclaration,
					buff);
			else
				buff.append("F");
		}
		buff.append(")");
	}

	private static void appendBodyFeatureVariableClassDeclarationName(
		IBodyFeatureVariableClassDeclaration bodyFeatureVariableClassDeclaration,
		IObjectClassDeclaration objectClassDeclaration,
		StringBuffer buff) {

		IObjectFeatureClassDeclaration featureClassDeclaration =
			searchFeatureClassDeclaration(bodyFeatureVariableClassDeclaration, objectClassDeclaration);

		if (featureClassDeclaration != null) {
			buff.append(" '");
			buff.append(featureClassDeclaration.getName());
			buff.append("'");
		}
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	protected String getName(INamed value) {
		if (value instanceof IBodyVariableClassDeclaration) {
			return getBodyVariableClassDeclarationText(
				(IBodyVariableClassDeclaration) value,
				getObjectClassDeclaration());

		} else {
			return super.getName(value);
		}
	}

	protected String getToolTipText(INamed value, String name) {
		if (value instanceof IBodyVariableClassDeclaration) {
			return getBodyVariableClassDeclarationToolTipText(
				(IBodyVariableClassDeclaration) value,
				getObjectClassDeclaration());

		} else {
			return super.getName(value);
		}
	}

	private static IObjectFeatureClassDeclaration searchFeatureClassDeclaration(
		IBodyFeatureVariableClassDeclaration bodyFeatureVariableClassDeclaration,
		IObjectClassDeclaration objectClassDeclaration) {

		if (objectClassDeclaration == null) {
			return null;
		}

		IVariableClassDeclaration variableClassDeclaration =
			bodyFeatureVariableClassDeclaration.getVariableClassDeclaration();

		Iterator it = objectClassDeclaration.getObjectFeatureClassDeclarationList().iterator();

		while (it.hasNext()) {
			IObjectFeatureClassDeclaration currIObjectFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration) it.next();

			if (CollectionsHelper
				.contains(
					currIObjectFeatureClassDeclaration.getVariableClassDeclarations(),
					variableClassDeclaration)) {
				return currIObjectFeatureClassDeclaration;
			}
		}

		return null;
	}

	public void setObjectClassDeclaration(IObjectClassDeclaration newObjectClassDeclaration) {
		_objectClassDeclaration = newObjectClassDeclaration;
	}

	protected Icon getIcon(INamed value) {
		if (value instanceof IBodyVariableClassDeclaration)
			return VariableClassDeclarationCellRendererAdapter.getMarkerIcon(
				((IBodyVariableClassDeclaration) value).getVariableClassDeclaration());
		else
			return super.getIcon(value);
	}
}