package de.uniwue.ki.gui.mas.entity.variableclass;
import javax.swing.*;

import de.uniwue.ki.gui.util.NamedCellRendererAdapter;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (22.11.2001 16:54:03)
 * @author Christoph Oechslein
 */
public class VariableClassDeclarationCellRendererAdapter extends NamedCellRendererAdapter {
	public static final ImageIcon PUBLIC_ICON =
		SwingMoreUtilities.createImageIcon("eclipse/public-marker.gif");

	public static final ImageIcon PRIVATE_ICON =
		SwingMoreUtilities.createImageIcon("eclipse/private-marker.gif");

	public VariableClassDeclarationCellRendererAdapter(ListCellRenderer cellRenderer) {
		super(cellRenderer);
	}

	protected static String getMarkers(IVariableClassDeclaration variableClassDeclaration) {
		StringBuffer marker = new StringBuffer();

		if (variableClassDeclaration.isWriteable()) {
			marker.append("write");
		} else {
			marker.append("const");
		}

		return marker.toString();
	}

	protected String getName(INamed value) {
		if (value instanceof IVariableClassDeclaration) {
			return getVariableClassDeclarationText((IVariableClassDeclaration) value);

		} else {
			return super.getName(value);
		}
	}

	public static String getVariableClassDeclarationToolTipText(IVariableClassDeclaration variableClassDeclaration) {
		StringBuffer buff = new StringBuffer();

		appendVariableClassDeclarationName(variableClassDeclaration, buff);

		if (variableClassDeclaration.getDocumentation().length() > 0) {
			buff.append("<P>");
			buff.append(NamedCellRendererAdapter.getQuotedString(variableClassDeclaration.getDocumentation()));
		}

		return "<html>" + buff.toString();
	}

	public static String getVariableClassDeclarationText(IVariableClassDeclaration variableClassDeclaration) {
		StringBuffer buff = new StringBuffer();

		appendVariableClassDeclarationName(variableClassDeclaration, buff);

		return "<html>" + buff.toString();
	}

	private static void appendVariableClassDeclarationName(
		IVariableClassDeclaration variableClassDeclaration,
		StringBuffer buff) {

		buff.append("<strong>");

		NamedCellRendererAdapter.appendQuotedString(variableClassDeclaration.getName(), buff);
		buff.append("</strong> (");
		NamedCellRendererAdapter.appendQuotedString(variableClassDeclaration.getSpecificType().getName(), buff);
		buff.append(", ");
		buff.append(getMarkers(variableClassDeclaration));
		buff.append(")");
	}

	protected String getToolTipText(INamed value, String name) {
		if (value instanceof IVariableClassDeclaration) {
			return getVariableClassDeclarationToolTipText((IVariableClassDeclaration) value);

		} else {
			return super.getName(value);
		}
	}

	protected Icon getIcon(INamed value) {
		if (value instanceof IVariableClassDeclaration) {
			return getMarkerIcon((IVariableClassDeclaration) value);
		} else {
			return super.getIcon(value);
		}
	}

	public static Icon getMarkerIcon(IVariableClassDeclaration variableClassDeclaration) {
		if (variableClassDeclaration.isExternal()) {
			return PUBLIC_ICON;
		} else {
			return PRIVATE_ICON;
		}
	}
}