package de.uniwue.ki.gui.mas.model;
import java.awt.Container;
import java.util.*;

import javax.swing.Icon;

import de.uniwue.ki.gui.mas.basics.search.SearchDeclarationPanelAction;
import de.uniwue.ki.gui.mas.util.declaration.IDeclarationPanelFactory;
import de.uniwue.ki.gui.mas.util.declaration.action.*;
import de.uniwue.ki.gui.util.IObjectPanelFactory;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.basics.search.SearchReferencesHelper;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;

/**
 * Creation date: (10.10.2001 10:38:07)
 * @author Christoph Oechslein
 */
public abstract class AbstractDeclarationPanelAndActionFactory implements IDeclarationPanelAndActionFactory{

	protected class DeclarationPanelAction extends AbstractDeclarationPanelAction {
		public DeclarationPanelAction(
			Container parent,
			String name,
			Icon icon,
			IDeclaration declaration,
			IDeclarationPanelFactory declarationPanelFactory) {

			super(parent, name, icon, declaration, declarationPanelFactory, true);
		}

		public DeclarationPanelAction(
			Container parent,
			String name,
			Icon icon,
			IDeclaration declaration,
			IDeclarationPanelFactory declarationPanelFactory,
			boolean shouldUseOkCancelDeclarationPanel) {

			super(
				parent,
				name,
				icon,
				declaration,
				declarationPanelFactory,
				true,
				shouldUseOkCancelDeclarationPanel);
		}

		public DeclarationPanelAction(
			Container parent,
			String name,
			Icon icon,
			IDeclaration declaration,
			IDeclarationPanelFactory declarationPanelFactory,
			boolean shouldTheFrameBeShown,
			boolean shouldUseOkCancelDeclarationPanel) {

			super(
				parent,
				name,
				icon,
				declaration,
				declarationPanelFactory,
				shouldTheFrameBeShown,
				shouldUseOkCancelDeclarationPanel);
		}

		public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
			return AbstractDeclarationPanelAndActionFactory.this.getGUISeSAmDeclaration();
		}
	}
	private IGUISeSAmDeclaration _guiSeSAmDeclaration;

	public AbstractDeclarationPanelAndActionFactory() {

		super();
	}

	public AbstractDeclarationPanelAndActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {

		super();

		setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public final boolean getCanProduceActions(Container parent, Object anObject) {
		return (getGUISeSAmDeclaration() != null)
			&& (anObject instanceof IDeclaration)
			&& getCanProduceDeclarationActions(parent, (IDeclaration)anObject);
	}

	protected abstract boolean getCanProduceDeclarationActions(Container parent, IDeclaration anDeclaration);

	public abstract IDeclarationPanelFactory getDeclarationPanelFactory(
		Container parent,
		IDeclaration anDeclaration);

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	public final Iterator getImportantActions(Container parent, Object anObject) {
		return getImportantDeclarationActions(parent, (IDeclaration)anObject);
	}

	protected Iterator getImportantDeclarationActions(Container parent, IDeclaration anDeclaration) {
		if (SearchReferencesHelper.isSearchAbleDeclaration(anDeclaration) && (getGUISeSAmDeclaration() != null))
			return CollectionsHelper.createIterator(
				new SearchDeclarationPanelAction(parent, anDeclaration, getGUISeSAmDeclaration()));
		else
			return CollectionsHelper.EMPTY_ITERATOR;
	}

	public final Iterator getMoreActions(Container parent, Object anObject) {
		return getMoreDeclarationActions(parent, (IDeclaration)anObject);
	}

	protected Iterator getMoreDeclarationActions(Container parent, IDeclaration declaration) {
		List actions = new LinkedList();

		if (declaration instanceof IObjectInstanceDeclaration) {
			Object parentOfDeclaration = getSeSAmDeclarationTreeModel().getUserObjectParentOf(declaration);
			if (parentOfDeclaration instanceof IMutableListModel)
			actions.add(new CopyObjectInstanceDeclarationAction(declaration, (IMutableListModel)parentOfDeclaration));	
		}
		
		if (declaration instanceof IDocumentableDeclaration) {
			actions.add(new RenameDocumentableDeclarationAction(parent, declaration));
		}
		
		
		if (declaration.canBeRemoved()) {
			Object parentOfDeclaration = getSeSAmDeclarationTreeModel().getUserObjectParentOf(declaration);

			if (parentOfDeclaration instanceof IMutableListModel) {
				actions.add(
					new RemoveMutableListModelAction(declaration, (IMutableListModel)parentOfDeclaration));

			} else if (parentOfDeclaration instanceof IMutableListModelAndFactory) {
				actions.add(
					new RemoveMutableListModelAction(
						declaration,
						((IMutableListModelAndFactory)parentOfDeclaration).getMutableListModel()));
			}

			/*
			if (parent instanceof SeSAmDeclarationTreePanel)
			    actions.add(DefaultMutableListModelPanelActionFactory.createRefreshAction(getSeSAmDeclarationTreeModel(), declaration));
			*/
		}

		actions.add(new InspectAction(declaration));

		return actions.iterator();
	}

	public final AbstractBoundAction getPropertyAction(Container parent, Object anObject) {
		return getPropertyDeclarationAction(parent, (IDeclaration)anObject);
	}

	protected AbstractBoundAction getPropertyDeclarationAction(
		Container parent,
		IDeclaration anDeclaration) {

		IDeclarationPanelFactory declarationPanelFactory = getDeclarationPanelFactory(parent, anDeclaration);
		if (declarationPanelFactory == null)
			return null;
		else
			return new DeclarationPanelAction(
				parent,
				"Properties",
				PropertiesDeclarationAction.PROPERTIES_ICON,
				anDeclaration,
				declarationPanelFactory,
				shouldUseOkCancelDeclarationPanel(parent, anDeclaration));
	}

	protected boolean shouldUseOkCancelDeclarationPanel(Container parent, IDeclaration anDeclaration) {
		return true;
	}

	protected ISeSAmDeclaration getSeSAmDeclaration() {
		return getGUISeSAmDeclaration().getSeSAmDeclaration();
	}

	protected ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel() {
		return getGUISeSAmDeclaration().getSeSAmDeclarationTreeModel();
	}

	protected ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return getGUISeSAmDeclaration().getSeSAmModelDeclaration();
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newGuiSeSAmDeclaration) {
		_guiSeSAmDeclaration = newGuiSeSAmDeclaration;
	}
	
	public IObjectPanelFactory getPanelFactory(Container parent, Object anDeclaration) {
		return getDeclarationPanelFactory(parent, (IDeclaration)anDeclaration);
	}
}