package de.uniwue.ki.gui.mas.model;
import java.awt.Container;
import java.util.*;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreePanel;
import de.uniwue.ki.gui.mas.util.IActionFactory;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.list.IMutableListModel;

/**
 * Creation date: (10.10.2001 10:38:07)
 * @author Christoph Oechslein
 */
public class DefaultMutableListModelActionFactory implements IActionFactory {
	protected static final Icon _propertiesIcon =
		SwingMoreUtilities.getJavaSunIcon(16, "general/Properties16.gif");
	protected static final Icon _newIcon = SwingMoreUtilities.getJavaSunIcon(16, "general/New16.gif");
	protected static final Icon _clearIcon = SwingMoreUtilities.getJavaSunIcon(16, "general/Delete16.gif");
	protected static final Icon _refreshIcon = SwingMoreUtilities.getJavaSunIcon(16, "general/Refresh16.gif");
	protected static final Icon _sortIcon = SwingMoreUtilities.createImageIcon("eclipse/alphab_sort_co.gif");

	private IGUISeSAmDeclaration _guiSeSAmDeclaration;

	public DefaultMutableListModelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {

		super();

		setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public static AbstractBoundAction createClearAction(
		final Container parent,
		final IMutableListModel mutableListModel,
		final IGUISeSAmDeclaration guiSeSAmDeclaration) {

		if (mutableListModel instanceof IDeclarationList) {
			boolean canAllBeRemoved = true;

			Iterator it = mutableListModel.iterator();
			while (it.hasNext()) {
				IDeclaration currIDeclaration = (IDeclaration) it.next();

				if (!currIDeclaration.canBeRemoved())
					canAllBeRemoved = false;
			}

			if (!canAllBeRemoved)
				return null;
		}

		return new AbstractBoundAction("Clear", _clearIcon) {
			public void doit() {
				JOptionPane optionPane =
					new JOptionPane(
						"Really deleted all declarations?",
						JOptionPane.QUESTION_MESSAGE,
						JOptionPane.YES_NO_OPTION);

				GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
					parent,
					"Clear",
					optionPane);

				if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.YES_OPTION)) {
					mutableListModel.clear();
				}
			}
		};
	}

	public static Action createRefreshAction(
		final ISeSAmDeclarationTreeModel seSAmDeclarationTreeModel,
		final Object anObject) {
		return new AbstractBoundAction("Refresh", _refreshIcon) {
			public void doit() {
				seSAmDeclarationTreeModel.reloadNode(anObject);
			}
		};
	}

	public static AbstractBoundAction createSortAction(final IMutableListModel mutableListModel) {

		return new AbstractBoundAction("Sort", _sortIcon) {
			public void doit() {
				mutableListModel.sort(NamedComparator.getInstance());
			}
		};
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		return (getGUISeSAmDeclaration() != null) && (anObject instanceof IMutableListModel);
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	public Iterator getImportantActions(Container parent, Object anObject) {
		return CollectionsHelper.EMPTY_ITERATOR;
	}

	public Iterator getMoreActions(Container parent, Object anObject) {
		IMutableListModel mutableListModel = (IMutableListModel) anObject;

		List actions = new LinkedList();

		if (!mutableListModel.isEmpty()) {
			Action clearAction = createClearAction(parent, mutableListModel, getGUISeSAmDeclaration());
			if (clearAction != null)
				actions.add(clearAction);

			actions.add(createSortAction(mutableListModel));
		}

		if (parent instanceof SeSAmDeclarationTreePanel)
			actions.add(createRefreshAction(getSeSAmDeclarationTreeModel(), mutableListModel));

		actions.add(new InspectAction(mutableListModel));

		return actions.iterator();
	}

	public AbstractBoundAction getPropertyAction(final Container parent, Object anObject) {
		return null;
	}

	protected ISeSAmDeclaration getSeSAmDeclaration() {
		return getGUISeSAmDeclaration().getSeSAmDeclaration();
	}

	protected ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel() {
		return getGUISeSAmDeclaration().getSeSAmDeclarationTreeModel();
	}

	protected ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return getGUISeSAmDeclaration().getSeSAmModelDeclaration();
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newGuiSeSAmDeclaration) {
		_guiSeSAmDeclaration = newGuiSeSAmDeclaration;
	}
}