package de.uniwue.ki.gui.mas.model;
import java.awt.Container;
import java.util.*;

import javax.swing.Action;

import de.uniwue.ki.gui.mas.model.tree.SeSAmDeclarationTreePanel;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;

/**
 * Creation date: (10.10.2001 10:38:07)
 * @author Christoph Oechslein
 */
public class DefaultMutableListModelAndFactoryPanelActionFactory
	extends DefaultMutableListModelActionFactory {
	public DefaultMutableListModelAndFactoryPanelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super(guiSeSAmDeclaration);
	}

	protected AbstractBoundAction createNewAction(
		final Container parent,
		final IMutableListModelAndFactory mutableListModelAndFactory) {

		return new AbstractBoundAction("New", _newIcon) {
			public void doit() {
				createNewObjectAndAdd(parent, mutableListModelAndFactory);
			}
		};
	}

	protected Object createNewObject(
		Container parent,
		IMutableListModelAndFactory mutableListModelAndFactory)
		throws Exception {

		return mutableListModelAndFactory.getObjectFactory().createObject();
	}

	protected void createNewObjectAndAdd(
		Container parent,
		IMutableListModelAndFactory mutableListModelAndFactory) {
		try {
			Object newObject = createNewObject(parent, mutableListModelAndFactory);

			if (newObject != null) {
				mutableListModelAndFactory.getMutableListModel().add(newObject);
			}

		} catch (Throwable exception) {
			getGUISeSAmDeclaration().showException("Error during Creation", exception, parent);
			exception.printStackTrace();
		}
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		return (getGUISeSAmDeclaration() != null) && (anObject instanceof IMutableListModelAndFactory);
	}

	public Iterator getMoreActions(Container parent, Object anObject) {
		IMutableListModelAndFactory mutableListModelAndFactory = (IMutableListModelAndFactory) anObject;

		List actions = new LinkedList();

		AbstractBoundAction newAction = createNewAction(parent, mutableListModelAndFactory);

		if (newAction != null)
			actions.add(newAction);

		if (!mutableListModelAndFactory.getMutableListModel().isEmpty()) {
			Action clearAction =
				createClearAction(
					parent,
					mutableListModelAndFactory.getMutableListModel(),
					getGUISeSAmDeclaration());
			if (clearAction != null)
				actions.add(clearAction);

			actions.add(createSortAction(mutableListModelAndFactory.getMutableListModel()));
		}

		if (parent instanceof SeSAmDeclarationTreePanel)
			actions.add(createRefreshAction(getSeSAmDeclarationTreeModel(), mutableListModelAndFactory));

		actions.add(new InspectAction(mutableListModelAndFactory));

		return actions.iterator();
	}

	public AbstractBoundAction getPropertyAction(final Container parent, Object anObject) {
		AbstractBoundAction action = super.getPropertyAction(parent, anObject);

		if (action != null) {
			return action;

		} else {
			final IMutableListModelAndFactory mutableListModelAndFactory =
				(IMutableListModelAndFactory) anObject;

			ISeSAmModelDeclaration seSAmModelDeclaration =
				getGUISeSAmDeclaration().getSeSAmModelDeclaration();

			if (!(parent instanceof SeSAmDeclarationTreePanel)) {
				return null;
			}

			final SeSAmDeclarationTreePanel seSAmDeclarationTreePanel = (SeSAmDeclarationTreePanel) parent;

			if (mutableListModelAndFactory
				== seSAmModelDeclaration
					.getSeSAmModelBasicsDeclaration()
					.getUserFunctionDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel
							.getSeSAmMainDesktopFrame()
							.showUserFunctionDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showUserFeatureDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSeSAmModelBasicsDeclaration()
						.getUserTypeDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showUserTypeDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration.getRessourceClassDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel
							.getSeSAmMainDesktopFrame()
							.showRessourceClassDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration.getAgentClassDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showAgentClassDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration.getWorldClassDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showWorldClassDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSimulationElementsDeclaration()
						.getSimulationDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showSimulationDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSimulationElementsDeclaration()
						.getAnalysisDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showAnalysisDeclarationList();
					}
				};
				
			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSimulationElementsDeclaration()
						.getUserInterfaceDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showUserInterfaceDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSimulationElementsDeclaration()
						.getSituationDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showSituationDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== seSAmModelDeclaration
						.getSimulationElementsDeclaration()
						.getExperimentDeclarationListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showExperimentDeclarationList();
					}
				};

			} else if (
				mutableListModelAndFactory
					== getGUISeSAmDeclaration().getSeSAmDeclaration().getExperimentListAndFactory()) {
				return new AbstractBoundAction("Properties", _propertiesIcon) {
					public void doit() {
						seSAmDeclarationTreePanel.getSeSAmMainDesktopFrame().showExperimentList();
					}
				};
				/*
				      } else if (mutableListModelAndFactory instanceof IDeclarationListAndFactory) {
				          return new AbstractBoundAction("Properties", _propertiesIcon) {
				              public void doit() {
				                  DeclarationListPanel declarationListPanel = new DeclarationListPanel();
				
				                  declarationListPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
				
				                  declarationListPanel.setDeclarationListAndFactory((IDeclarationListAndFactory) mutableListModelAndFactory);
				
				                  de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
				                      parent,
				                      mutableListModelAndFactory.getName(),
				                      declarationListPanel,
				                      true);
				              }
				          };
				          */
			}

			return null;
		}
	}
}