package de.uniwue.ki.gui.mas.model;
import java.awt.Component;
import java.awt.datatransfer.Clipboard;
import java.io.File;
import java.io.IOException;
import java.io.Reader;

import javax.swing.JOptionPane;

import junit.framework.Assert;
import mygale.mcge.fig.DefaultFigMapper;
import mygale.mcge.fig.IFigMapper;
import de.uniwue.ki.gui.mas.basics.feature.ObjectFeatureInstanceDeclarationListActionFactory;
import de.uniwue.ki.gui.mas.basics.feature.user.UserFeatureClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.basics.feature.user.UserFeatureFunctionAndVariableDeclarationListAndFactoryActionFactory;
import de.uniwue.ki.gui.mas.basics.feature.user.UserFeatureFunctionDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.basics.feature.user.UserFeatureVariableClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.basics.functionCall.FunctionCallDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.basics.functionCall.FunctionCallInternDeclarationActionFactory;
import de.uniwue.ki.gui.mas.basics.functionUser.UserFunctionDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.basics.type.user.ComposedTypeAttributeActionFactory;
import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationListActionFactory;
import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.behaviour.activityGraph.ActionActivityGraphLeafDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.behaviour.rule.ActivityRuleDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.entity.object.ObjectInstanceDeclarationListActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.ActivityAgentClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.ActivityGraphDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.ObjectBodyVariableDeclarationListAndFactoryActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.ObjectBodyVariablesClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.ObjectFeatureFeatueClassDeclarationListActionFactory;
import de.uniwue.ki.gui.mas.entity.objectclass.factory.RessourceClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.entity.variable.BodyVariableInstanceDeclarationListActionFactory;
import de.uniwue.ki.gui.mas.entity.variable.viewer.IObjectValueViewer;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerComposedType;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerEnumItem;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerImage;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerListType;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerManager;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerSimObject;
import de.uniwue.ki.gui.mas.entity.variable.viewer.ObjectValueViewerToActionFactoryAdapter;
import de.uniwue.ki.gui.mas.model.graph.UMLLayoutGraphDeclarationGraphPanelActionFactory;
import de.uniwue.ki.gui.mas.sim.analysis.AnalysisDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentActionFactory;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentListActionFactory;
import de.uniwue.ki.gui.mas.sim.simulation.SimulationActionFactory;
import de.uniwue.ki.gui.mas.sim.simulation.SimulationDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.sim.situation.SituationClassDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.sim.userInterface.UserInterfaceDeclarationPanelActionFactory;
import de.uniwue.ki.gui.mas.util.declaration.IDeclarationFrameFactory;
import de.uniwue.ki.gui.mas.util.declaration.JDeclarationFrameFactory;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.model.SeSAmDeclarationFactory;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.mas.sim.simulation.SimulationEvent;
import de.uniwue.ki.mas.sim.simulation.StartPauseFinishEvent;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.mas.xml.XMLReader;
import de.uniwue.ki.mas.xml.XMLWriter;
import de.uniwue.ki.mas.xml.reader.XMLReadException;
import de.uniwue.ki.mas.xml.writer.XMLWriteException;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (26.06.2001 14:56:53)
 * @author Christoph Oechslein
 */
public class GUISeSAmDeclaration extends AbstractDeclaration implements IGUISeSAmDeclaration {
	private final PanelActionFactoryManager _panelActionFactoryManager;
	private final ObjectValueViewerManager _objectValueViewerManager = new ObjectValueViewerManager();

	private IDeclarationFrameFactory _frameFactory = null;
	private ISeSAmDeclaration _seSAmDeclaration = null;
	private final IFigMapper _figMapper = new DefaultFigMapper();
	private final Clipboard _clipboard = new Clipboard("SeSAm");

	public GUISeSAmDeclaration() {
		this(SeSAmDeclarationFactory.createSeSAmDeclaration(), JDeclarationFrameFactory.getInstance());
	}

	public GUISeSAmDeclaration(IDeclarationFrameFactory frameFactory) {
		this(SeSAmDeclarationFactory.createSeSAmDeclaration(), frameFactory);
	}

	public GUISeSAmDeclaration(ISeSAmDeclaration seSAmDeclaration) {
		this(seSAmDeclaration, JDeclarationFrameFactory.getInstance());
	}

	public GUISeSAmDeclaration(ISeSAmDeclaration seSAmDeclaration, IDeclarationFrameFactory frameFactory) {
		super();

		_panelActionFactoryManager = new PanelActionFactoryManager(this);
		_seSAmDeclaration = seSAmDeclaration;
		_frameFactory = frameFactory;

		initializePanelActionFactories();
		initializeObjectValueViewer();
	}

	public final Clipboard getClipboard() {
		return _clipboard;
	}

	public static final IDeclarationFrameFactory getDeclarationFrameFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		if (guiSeSAmDeclaration != null)
			return guiSeSAmDeclaration.getFrameFactory();
		else
			return JDeclarationFrameFactory.getInstance();
	}

	public final IFigMapper getFigMapper() {
		return _figMapper;
	}

	public IDeclarationFrameFactory getFrameFactory() {
		return _frameFactory;
	}

	public String getName() {
		return getSeSAmDeclaration().getName();
	}

	public final PanelActionFactoryManager getPanelActionFactoryManager() {
		return _panelActionFactoryManager;
	}

	public final ObjectValueViewerManager getObjectValueViewerManager() {
		return _objectValueViewerManager;
	}

	public ISeSAmDeclaration getSeSAmDeclaration() {
		return _seSAmDeclaration;
	}

	public ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel() {
		return getSeSAmDeclaration().getSeSAmDeclarationTreeModel();
	}

	public ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return getSeSAmDeclaration().getSeSAmModelDeclaration();
	}

	private void initializeObjectValueViewer() {
		addObjectValueViewer(new ObjectValueViewerImage(this));
		addObjectValueViewer(new ObjectValueViewerListType(this));
		addObjectValueViewer(new ObjectValueViewerEnumItem(this));
		addObjectValueViewer(new ObjectValueViewerComposedType(this));
		addObjectValueViewer(new ObjectValueViewerSimObject(this));
	}

	/*
	 * Public because this is called from the iniFileReader to add new Viewers
	 */
	public void addObjectValueViewer(IObjectValueViewer viewer) {
		//remember Valueviewers and 
		_objectValueViewerManager.addObjectValueViewer(viewer);
		//create ActionFactories with an Adapter
		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectValueViewerToActionFactoryAdapter(this, viewer));
	}

	private void initializePanelActionFactories() {
		_panelActionFactoryManager.addPanelActionFactory(
			new FunctionCallInternDeclarationActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new DefaultDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new DefaultMutableListModelActionFactory(this));
		_panelActionFactoryManager.addPanelActionFactory(
			new DefaultMutableListModelAndFactoryPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectBodyVariablesClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectBodyVariableDeclarationListAndFactoryActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ActivityGraphDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectFeatureFeatueClassDeclarationListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ActivityAgentClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ActionActivityGraphLeafDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new RessourceClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new SimulationDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new UserInterfaceDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new UserFeatureClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new UserFunctionDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new UserTypeDeclarationListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new UserFeatureFunctionDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new UserFeatureVariableClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectFeatureInstanceDeclarationListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new SituationClassDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new ExperimentDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new SimulationActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new ExperimentActionFactory(this));
		_panelActionFactoryManager.addPanelActionFactory(new ExperimentListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new AnalysisDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new UserTypeDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new ComposedTypeAttributeActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new UserFeatureFunctionAndVariableDeclarationListAndFactoryActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new ObjectInstanceDeclarationListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(
			new BodyVariableInstanceDeclarationListActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new ActivityRuleDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new FunctionCallDeclarationPanelActionFactory(this));

		_panelActionFactoryManager.addPanelActionFactory(new UMLLayoutGraphDeclarationGraphPanelActionFactory(this));
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void openModel(File file) throws XMLReadException, IOException {
		XMLReader.readModelInto(file, getSeSAmDeclaration(), getFigMapper(), true);
	}

	public void openModel(Reader reader) throws XMLReadException, IOException {
		XMLReader.readModelInto(reader, getSeSAmDeclaration(), getFigMapper(), true);
	}

	public void saveModel(File file) throws XMLWriteException, IOException {
		File tmpFile = File.createTempFile("SeSAm-", ".xml.tmp", file.getParentFile());
		tmpFile.deleteOnExit();

		XMLWriter.writeModel(tmpFile, getSeSAmDeclaration(), getFigMapper());

		if (file.exists()) {
			File backupOldFile = new File(file.toString() + "-old.bck");
			backupOldFile.deleteOnExit();

			backupOldFile.delete();
			file.renameTo(backupOldFile);
		}

		boolean result = tmpFile.renameTo(file);

		Assert.assertTrue(result);
	}

	public void setSeSAmDeclaration(ISeSAmDeclaration seSAmDeclaration) {
		ISeSAmDeclaration oldValue = _seSAmDeclaration;
		_seSAmDeclaration = seSAmDeclaration;
		firePropertyChange("seSAmDeclaration", oldValue, seSAmDeclaration);
	}

	public void showException(StartPauseFinishEvent startPauseFinishEvent, Component onComponent) {

		if (startPauseFinishEvent instanceof SimulationEvent) {
			showSimulationEventException((SimulationEvent)startPauseFinishEvent, onComponent);

		} else {
			showException("Error", startPauseFinishEvent.getException(), onComponent);
		}
	}

	public void showException(String title, String preamble, Throwable exception, Component onComponent) {
		showException(this, title, preamble, exception, onComponent);
	}

	public void showException(String title, Throwable exception, Component onComponent) {
		showException(this, title, exception, onComponent);
	}

	public static void showException(
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		String title,
		String preamble,
		Throwable exception,
		Component onComponent) {

		if (guiSeSAmDeclaration != null) {
			JOptionPane optionPane = SwingMoreUtilities.getExceptionOptionPane(preamble, exception);
			guiSeSAmDeclaration.getFrameFactory().addModalPanel(onComponent, title, optionPane);

		} else {
			SwingMoreUtilities.showException(title, preamble, exception, onComponent);
		}
	}

	public static void showException(
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		String title,
		Throwable exception,
		Component onComponent) {

		if (guiSeSAmDeclaration != null) {
			JOptionPane optionPane = SwingMoreUtilities.getExceptionOptionPane(exception);
			guiSeSAmDeclaration.getFrameFactory().addModalPanel(onComponent, title, optionPane);

		} else {
			SwingMoreUtilities.showException(title, exception, onComponent);
		}
	}

	private void showSimulationEventException(SimulationEvent simulationEvent, Component onComponent) {

		StringBuffer buff = new StringBuffer("Error during Simulation!\n");

		IObject simObject = simulationEvent.getObject();

		if (simObject != null) {

			buff.append("Object: ");
			buff.append(simObject.getName());

			if (simObject instanceof IAgent) {
				IAgent agent = (IAgent)simObject;

				if (simObject instanceof IActivityAgent) {
					IActivityAgent activityAgent = (IActivityAgent)agent;

					buff.append("\nActivity: ");
					appendName(buff, activityAgent.getCurrentActivityGraphLeafs());
				}

				if (agent.getCurrentAction() != null) {
					buff.append("\nAction: ");
					buff.append(agent.getCurrentAction().getName());
				}
			}

			buff.append("\n");
		}

		showException(
			"Error during Simulation",
			buff.toString(),
			simulationEvent.getException(),
			onComponent);
	}

	private final void appendName(StringBuffer buff, INamed named) {
		if (named == null) {
			buff.append("<null>");
		} else {
			named.appendName(buff, 0, 10);
		}
	}

	private final void appendName(StringBuffer buff, INamed[] named) {
		if (named == null) {
			buff.append("<null>");

		} else if (named.length == 1) {
			appendName(buff, named[0]);

		} else {
			buff.append("[");
			for (int i = 0; i < named.length; i++) {
				appendName(buff, named[i]);
				if (i < named.length - 1) {
					buff.append(", ");
				}
			}
			buff.append("]");
		}
	}

	public void clear() {
		getFigMapper().clear();
		getSeSAmDeclaration().clear();
	}
}