package de.uniwue.ki.gui.mas.model;

import java.awt.*;
import java.util.HashMap;

import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.gui.util.*;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.DefaultThreadFactory;
import de.uniwue.ki.util.Pair;
import edu.oswego.cs.dl.util.concurrent.LinkedQueue;
import edu.oswego.cs.dl.util.concurrent.QueuedExecutor;

/**
 * 
 * @author Christoph Oechslein
 */
public class JInternalFrameFactory implements IDeclarationFrameFactory {

    private final JDesktopPane _jDesktopPane;

    private final HashMap _declarationToFrameMap = new HashMap();

    private final QueuedExecutor _disposeExecutor = new QueuedExecutor(
            new LinkedQueue());
    
    private boolean _makeJIFsInspectable;

    private static final class JDeclarationInternalFrame extends JInternalFrame {

        private IDeclarationPanel _declarationPanel;

        private boolean _disposed = false;

        public boolean isDisposed() {
            return _disposed;
        }

        public void dispose() {
            super.dispose();
            _disposed = true;
        }

        public IDeclarationPanel getDeclarationPanel() {
            return _declarationPanel;
        }

        public void setDeclarationPanel(IDeclarationPanel declarationPanel) {
            _declarationPanel = declarationPanel;
        }

    }

    public JInternalFrameFactory(JDesktopPane jDesktopPane) {
        super();

        _jDesktopPane = jDesktopPane;
        _disposeExecutor.setThreadFactory(new DefaultThreadFactory(
                "Disposing Frame", Thread.MIN_PRIORITY, true));
    }

    public JInternalFrame addInternalFrame(Component parent, String title,
            Component comp, boolean shouldTheFrameBeShown,
            int windowCloseOperation) {

        return addFrameIntern(parent, title, comp, shouldTheFrameBeShown, null,
                windowCloseOperation, null);
    }

    public JRootPane addPanel(Component parent, String title, Component comp,
            boolean shouldTheFrameBeShown, int windowCloseOperation) {

        return addPanel(parent, title, comp, shouldTheFrameBeShown,
                windowCloseOperation, null);
    }

    public JRootPane addPanel(Component parent, String title, Component comp,
            boolean shouldTheFrameBeShown, int windowCloseOperation,
            Runnable windowCloseFunction) {

        return addFrameIntern(parent, title, comp, shouldTheFrameBeShown, null,
                windowCloseOperation, windowCloseFunction).getRootPane();
    }

    private JDeclarationInternalFrame addFrameIntern(final Component parent,
            String title, Component comp, boolean shouldTheFrameBeShown,
            Object constraint, int windowCloseOperation,
            final Runnable windowCloseFunction) {

        JDeclarationInternalFrame jInternalFrame = createInternalFrame(parent,
                title, comp);

        jInternalFrame.setDefaultCloseOperation(windowCloseOperation);

        if (windowCloseFunction != null) {
            jInternalFrame.addInternalFrameListener(new InternalFrameAdapter() {

                public void internalFrameClosed(InternalFrameEvent e) {
                    windowCloseFunction.run();
                }
            });
        }

        if ((windowCloseOperation == WindowConstants.DISPOSE_ON_CLOSE)
                && (comp instanceof IDisposeable)) {
            final IDisposeable disposeablePanel = (IDisposeable) comp;
            jInternalFrame.addInternalFrameListener(new InternalFrameAdapter() {

                public void internalFrameClosed(InternalFrameEvent e) {
                    try {
                        _disposeExecutor.execute(new Runnable() {

                            public void run() {
                                disposeablePanel.dispose();
                                if (Assert.ASSERTION_ON)
                                        Assert.assertEquals(
                                                "IDisposeable wasn't disposed: "
                                                        + disposeablePanel,
                                                true, disposeablePanel
                                                        .isDisposed());
                            }
                        });
                    } catch (InterruptedException e2) {
                        e2.printStackTrace();
                        Thread.currentThread().interrupt();
                    }
                }
            });
        }

        jInternalFrame.addInternalFrameListener(new InternalFrameAdapter() {

            public void internalFrameClosed(InternalFrameEvent e) {

                RootPaneContainer rootPaneContainer = SwingMoreUtilities
                        .getRootPaneContainer(parent);
                if (rootPaneContainer instanceof JFrame) {
                    JFrame frame = (JFrame) rootPaneContainer;
                    frame.show();
                } else if (rootPaneContainer instanceof JInternalFrame) {
                    JInternalFrame frame = (JInternalFrame) rootPaneContainer;
                    frame.show();
                }
            }
        });

        Rectangle visibleRect = _jDesktopPane.getVisibleRect();
        Dimension s = visibleRect.getSize();
        Dimension f = jInternalFrame.getSize();

        int w = Math.max(s.width - f.width, 0);
        int h = Math.max(s.height - f.height, 0);
        int x = (int) (Math.random() * w) + visibleRect.x;
        int y = (int) (Math.random() * h) + visibleRect.y;

        jInternalFrame.setLocation(x, y);

        _jDesktopPane.add(jInternalFrame, constraint);

		if (_makeJIFsInspectable)
			SwingMoreUtilities.addInspectAction(jInternalFrame);

        if (shouldTheFrameBeShown) {
            jInternalFrame.show();
        }

        return jInternalFrame;
    }

    public void addModalPanel(Component parent, String title,
            JOptionPane optionPane) {

        final JDialog dialog = optionPane.createDialog(parent, title);

        dialog.setResizable(true);

        dialog.pack();

        // dialog.getSize()
        // dialog.getPreferredSize()

        if (dialog.getSize() != null) {
            Dimension maxSize = Toolkit.getDefaultToolkit().getScreenSize();
            dialog.setSize((int) Math
                    .min(dialog.getWidth(), maxSize.getWidth()), (int) Math
                    .min(dialog.getHeight(), maxSize.getHeight()));
        }

        dialog.setVisible(true);

        // due to problems with internal modal dialogs
        /*
         * 
         * Dimension maxSize = _jDesktopPane.getSize();
         * 
         * optionPane.setMaximumSize(maxSize);
         * 
         * _modalDepth++;
         * 
         * try {
         * 
         * if (_modalDepth > 1) { JDialog dialog =
         * optionPane.createDialog(parent, title);
         * 
         * dialog.setVisible(true); } else {
         * 
         * JRootPane rootPane = SwingUtilities.getRootPane(_jDesktopPane);
         * 
         * SwingMoreUtilities.setEnableAllComponentsRecursive(_jDesktopPane,
         * false);
         * 
         * try { IModalFrame modal =
         * ModalInternalFrame.createModalInternalFrame(title, rootPane,
         * _jDesktopPane, _jDesktopPane, optionPane);
         * 
         * modal.setVisible(true);
         * 
         * optionPane.selectInitialValue();
         * 
         * modal.startModal(); } finally {
         * SwingMoreUtilities.setEnableAllComponentsRecursive(_jDesktopPane,
         * true); } } } finally { _modalDepth--; }
         *  
         */
    }

    private JDeclarationInternalFrame addNewDeclarationPanel(Component parent,
            IDeclaration declaration,
            IDeclarationPanelFactory declarationPanelFactory,
            boolean shouldTheFrameBeShown,
            boolean shouldUseOkCancelDeclarationPanel,
            final Pair declarationAndDeclarationFactoryPair) {

        IDeclarationPanel declarationPanel = declarationPanelFactory
                .createDeclarationPanel(declaration);

        if (declarationPanel == null) { return null; }

        Component componentToShow;

        if (shouldUseOkCancelDeclarationPanel) {

            OkCancelDeclarationPanel okCancelDeclarationPanel = new OkCancelDeclarationPanel() {

                public void dispose() {
                    super.dispose();

                    _declarationToFrameMap
                            .remove(declarationAndDeclarationFactoryPair);
                }
            };

            okCancelDeclarationPanel.setDeclarationPanel(declarationPanel);
            okCancelDeclarationPanel.setDeclaration(declaration);

            componentToShow = okCancelDeclarationPanel;
            declarationPanel = okCancelDeclarationPanel;

        } else {
            declarationPanel.setDeclaration(declaration);

            final IDeclarationPanel finalDeclarationPanel = declarationPanel;

            JDisposeablePanel panel = new JDisposeablePanel() {

                public void dispose() {
                    finalDeclarationPanel.dispose();

                    super.dispose();

                    _declarationToFrameMap
                            .remove(declarationAndDeclarationFactoryPair);
                }
            };

            panel.setLayout(new BorderLayout());

            panel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
            panel.add((Component) declarationPanel, BorderLayout.CENTER);

            componentToShow = panel;
        }

        String title = "Edit "
                + declarationPanelFactory.getDeclarationTitle(declaration);

        JDeclarationInternalFrame declarationInternalFrame = addFrameIntern(
                parent, title, componentToShow, shouldTheFrameBeShown, null,
                WindowConstants.DISPOSE_ON_CLOSE, null);
                
		Icon icon = declarationPanelFactory.getIcon(declaration);
		
		if (icon != null)
			declarationInternalFrame.setFrameIcon(icon);

        if (shouldUseOkCancelDeclarationPanel)
                declarationInternalFrame.setClosable(false);

        declarationInternalFrame.setDeclarationPanel(declarationPanel);

        return declarationInternalFrame;
    }

    public JRootPane addPanel(final Component parent,
            final IDeclaration declaration,
            final IDeclarationPanelFactory declarationPanelFactory,
            final boolean shouldTheFrameBeShown,
            final boolean shouldUseOkCancelDeclarationPanel) {

        return addPanelIntern(parent, declaration, declarationPanelFactory,
                shouldTheFrameBeShown, shouldUseOkCancelDeclarationPanel);
    }

    public JRootPane addPanel(Component parent, String title, Component comp,
            boolean shouldTheFrameBeShown) {
        return addPanel(parent, title, comp, shouldTheFrameBeShown,
                WindowConstants.DISPOSE_ON_CLOSE);
    }

    private JRootPane addPanelIntern(Component parent,
            IDeclaration declaration,
            IDeclarationPanelFactory declarationPanelFactory,
            boolean shouldTheFrameBeShown,
            boolean shouldUseOkCancelDeclarationPanel) {

        Pair pair = new Pair(declaration, declarationPanelFactory);

        if (!_declarationToFrameMap.containsKey(pair)) {
            return reallyAddPanelIntern(parent, declaration,
                    declarationPanelFactory, shouldTheFrameBeShown,
                    shouldUseOkCancelDeclarationPanel, pair);

        } else {

            JDeclarationInternalFrame declarationInternalFrame = (JDeclarationInternalFrame) _declarationToFrameMap
                    .get(pair);

            if (declarationInternalFrame.isDisposed()) {

                _declarationToFrameMap.remove(pair);
                return reallyAddPanelIntern(parent, declaration,
                        declarationPanelFactory, shouldTheFrameBeShown,
                        shouldUseOkCancelDeclarationPanel, pair);

            } else {

                // the pair is equal so
                Assert.assertSame(declarationInternalFrame
                        .getDeclarationPanel().getDeclaration(), declaration);
                //				if
                // (declarationInternalFrame.getDeclarationPanel().getDeclaration()
                // != declaration) {
                //					declarationInternalFrame.getDeclarationPanel().setDeclaration(declaration);
                //				}
				
				if (_makeJIFsInspectable)
					SwingMoreUtilities.addInspectAction(declarationInternalFrame);
				

                if (shouldTheFrameBeShown) {
                    declarationInternalFrame.setVisible(true);
                    declarationInternalFrame.show();
					try {
                        declarationInternalFrame.setIcon(false);
                        declarationInternalFrame.setSelected(true);
                    } catch (java.beans.PropertyVetoException exception) {
                    }
                    declarationInternalFrame.setVisible(true);
                    declarationInternalFrame.show();
                }

                return declarationInternalFrame.getRootPane();
            }
        }
    }

    private JRootPane reallyAddPanelIntern(Component parent,
            IDeclaration declaration,
            IDeclarationPanelFactory declarationPanelFactory,
            boolean shouldTheFrameBeShown,
            boolean shouldUseOkCancelDeclarationPanel, Pair pair) {
        JDeclarationInternalFrame declarationInternalFrame = addNewDeclarationPanel(
                parent, declaration, declarationPanelFactory,
                shouldTheFrameBeShown, shouldUseOkCancelDeclarationPanel, pair);

        if (declarationInternalFrame != null) {
            _declarationToFrameMap.put(pair, declarationInternalFrame);
            return declarationInternalFrame.getRootPane();

        } else {
            return null;
        }
    }

    protected JDeclarationInternalFrame createInternalFrame(Component parent,
            String title, Component comp) {
        return createInternalFrame(parent, title, comp, _jDesktopPane.getSize());
    }

    static JDeclarationInternalFrame createInternalFrame(Component parent,
            String titel, Component comp, Dimension maxSize) {

        Assert.assertNotNull(titel);
        Assert.assertNotNull(comp);

        JDeclarationInternalFrame jInternalFrame = new JDeclarationInternalFrame();

        jInternalFrame.setIconifiable(true);
        jInternalFrame.setClosable(true);
        jInternalFrame.setMaximizable(true);
        jInternalFrame.setResizable(true);

        jInternalFrame.setTitle(titel);

        jInternalFrame.getContentPane()
                .setLayout(
                        new BoxLayout(jInternalFrame.getContentPane(),
                                BoxLayout.Y_AXIS));

        jInternalFrame.getContentPane().add(comp);

        // jInternalFrame.setNextFocusableComponent(jInternalFrame.getContentPane());

        //jInternalFrame.setMaximumSize(maxSize);

        jInternalFrame.pack();

        jInternalFrame.setSize((int) Math.min(jInternalFrame.getWidth(),
                maxSize.getWidth()), (int) Math.min(jInternalFrame.getHeight(),
                maxSize.getHeight()));

        return jInternalFrame;

    }
    
	public boolean getMakeJIFsInspectable() {
		return _makeJIFsInspectable;
	}

	public void setMakeJIFsInspectable(boolean b) {
		_makeJIFsInspectable = b;
	}

}