package de.uniwue.ki.gui.mas.model;
import java.awt.Container;
import java.awt.Cursor;
import java.util.*;

import javax.swing.*;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.refactoring.MarkAsTODOAction;
import de.uniwue.ki.gui.mas.refactoring.RefactoringMenuFactory;
import de.uniwue.ki.gui.mas.util.IActionFactory;
import de.uniwue.ki.gui.util.IObjectPanelFactory;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (10.10.2001 14:03:51)
 * @author Christoph Oechslein
 */
public final class PanelActionFactoryManager implements IActionFactory {

	private final List _panelActionFactories = new ArrayList();
	private final IGUISeSAmDeclaration seSAmDeclaration;

	PanelActionFactoryManager(IGUISeSAmDeclaration declaration) {
		super();
		seSAmDeclaration = declaration;
	}

	private static boolean addJMenuItems(JPopupMenu popupComponent, Iterator it) {
		boolean addOneAction = false;

		while (it.hasNext()) {
			Object currObject = it.next();
			if (currObject == null)
				popupComponent.addSeparator();

			else {
				popupComponent.add(createJMenuItem(currObject, false));
				addOneAction = true;
			}
		}

		return addOneAction;
	}

	public void addPanelActionFactory(IActionFactory panelActionFactory) {
		_panelActionFactories.add(panelActionFactory);
	}

	public boolean callPropertyAction(Container parent, Object anObject) {
		return callPropertyAction(parent, anObject, getPanelActionFactoryFor(parent, anObject));
	}

	public static boolean callPropertyAction(
		Container parent,
		Object anObject,
		IActionFactory panelActionFactory) {

		if (panelActionFactory != null) {

			AbstractBoundAction propertyAction = panelActionFactory.getPropertyAction(parent, anObject);

			if (propertyAction != null) {
				Cursor oldCursor = parent.getCursor();

				try {
					parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					propertyAction.doit();

				} finally {
					parent.setCursor(oldCursor);
				}

				return true;
			}
		}
		return false;
	}

	private static JMenuItem createJMenuItem(Object anObject, boolean strong) {
		if (anObject instanceof JMenuItem)
			return (JMenuItem) anObject;

		else {
			return AbstractBoundAction.createJMenuItem((Action) anObject, strong);
		}
	}

	public JPopupMenu createPopup(Container parent, Object anObject) {
		return createPopup(parent, anObject, getPanelActionFactoryFor(parent, anObject));
	}

	public JPopupMenu createPopup(
		Container parent,
		Object anObject,
		IActionFactory panelActionFactory) {

		JPopupMenu popupComponent = new JPopupMenu();

		if (panelActionFactory != null) {

			boolean addOneAction = false;

			AbstractBoundAction propertyAction = panelActionFactory.getPropertyAction(parent, anObject);
			if (propertyAction != null) {
				popupComponent.add(AbstractBoundAction.createJMenuItem(propertyAction, true));
				addOneAction = true;
			}

			addOneAction =
				addJMenuItems(popupComponent, panelActionFactory.getImportantActions(parent, anObject))
					|| addOneAction;

			if (addOneAction)
				popupComponent.addSeparator();

			Iterator it = panelActionFactory.getMoreActions(parent, anObject);
			if (it.hasNext()) {
				addJMenuItems(popupComponent, it);
				popupComponent.addSeparator();
			}
											
			popupComponent.add(RefactoringMenuFactory.getInstance().createMenu(parent, getSeSAmDeclaration(),anObject));
			
			if (anObject instanceof IDeclaration) {
				IDeclaration decl = (IDeclaration)anObject;
				popupComponent.add(new MarkAsTODOAction(decl));
			}
		}

		return popupComponent;
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		for (int i = _panelActionFactories.size() - 1; i >= 0; i--) {
			IActionFactory currPanelActionFactory = (IActionFactory) _panelActionFactories.get(i);
			if (currPanelActionFactory.getCanProduceActions(parent, anObject)) {
				return true;
			}
		}
		return false;
	}

	public Iterator getImportantActions(Container parent, Object anObject) {
		for (int i = _panelActionFactories.size() - 1; i >= 0; i--) {
			IActionFactory currPanelActionFactory = (IActionFactory) _panelActionFactories.get(i);
			if (currPanelActionFactory.getCanProduceActions(parent, anObject)) {
				return currPanelActionFactory.getImportantActions(parent, anObject);
			}
		}

		Assert.fail();
		return null;
	}

	public Iterator getMoreActions(Container parent, Object anObject) {
		for (int i = _panelActionFactories.size() - 1; i >= 0; i--) {
			IActionFactory currPanelActionFactory = (IActionFactory) _panelActionFactories.get(i);
			if (currPanelActionFactory.getCanProduceActions(parent, anObject)) {
				return currPanelActionFactory.getMoreActions(parent, anObject);
			}
		}

		Assert.fail();
		return null;
	}

	public IActionFactory getPanelActionFactoryFor(Container parent, Object anObject) {
		for (int i = _panelActionFactories.size() - 1; i >= 0; i--) {
			IActionFactory currPanelActionFactory = (IActionFactory) _panelActionFactories.get(i);
			if (currPanelActionFactory.getCanProduceActions(parent, anObject)) {
				return currPanelActionFactory;
			}
		}
		return null;
	}
	
	public IObjectPanelFactory getPanelFactory(Container parent, Object anObject) {
	    IActionFactory actionFactory=getPanelActionFactoryFor(parent, anObject);
	    if (actionFactory instanceof IObjectPanelAndActionFactory) {
	        return ((IObjectPanelAndActionFactory)actionFactory).getPanelFactory(parent, anObject);
	    }
	    return null;
	}

	public AbstractBoundAction getPropertyAction(Container parent, Object anObject) {
		for (int i = _panelActionFactories.size() - 1; i >= 0; i--) {
			IActionFactory currPanelActionFactory = (IActionFactory) _panelActionFactories.get(i);
			if (currPanelActionFactory.getCanProduceActions(parent, anObject)) {
				return currPanelActionFactory.getPropertyAction(parent, anObject);
			}
		}

		Assert.fail();
		return null;
	}

	private IGUISeSAmDeclaration getSeSAmDeclaration() {
		return seSAmDeclaration;
	}
}