package de.uniwue.ki.gui.mas.model;

import java.awt.Container;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.uniwue.ki.gui.mas.basics.type.argument.ArgumentDeclarationEditorPaneFactoryManager;
import de.uniwue.ki.gui.mas.entity.variable.viewer.IObjectValueViewer;
import de.uniwue.ki.gui.mas.model.plugin.ISeSAmMenuPlugin;
import de.uniwue.ki.gui.mas.util.IActionFactory;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.xml.XMLIniFileException;
import de.uniwue.ki.mas.xml.XMLIniReader;
import de.uniwue.ki.mas.xml.castor.v01.ini.XMLIniFileType;

/**
 * @author oechslein
 * 
 * Creation date: 21.05.2003
 */
public class XMLIniReaderForGUI extends XMLIniReader {

	private final IGUISeSAmDeclaration _guiSeSAmDeclaration;
	private final Container _parent;
	private final JMenu _pluginMenu;

	public XMLIniReaderForGUI(
		ClassLoader classLoader,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		Container parent,
		JMenu pluginMenu) {

		super(classLoader);

		_guiSeSAmDeclaration = guiSeSAmDeclaration;
		_parent = parent;
		_pluginMenu = pluginMenu;
	}

	protected void parseIniFileIntern()
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			NoSuchMethodException,
			XMLIniFileException {

		super.parseIniFileIntern();

		parseObjectActionSection();
		parseObjectViewerSection();
		parseMenuPluginSection();
	}

	private final IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	private void parseObjectViewerSection()
		throws
			XMLIniFileException,
			IllegalArgumentException,
			SecurityException,
			ClassNotFoundException,
			InstantiationException,
			IllegalAccessException,
			InvocationTargetException,
			NoSuchMethodException {

		if (_xmlIniFileGroup.getObjectViewerSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getObjectViewerSection().getObjectViewerCount(); i++) {
				String className =
					_xmlIniFileGroup.getObjectViewerSection().getObjectViewer(i).getClassName();
				getGUISeSAmDeclaration().getObjectValueViewerManager().addObjectValueViewer(
					(IObjectValueViewer) callGUISeSAmDeclarationConstructor(className));
			}
		}
	}

	private void parseObjectActionSection()
		throws
			IllegalArgumentException,
			SecurityException,
			InstantiationException,
			IllegalAccessException,
			InvocationTargetException,
			NoSuchMethodException,
			ClassNotFoundException {

		if (_xmlIniFileGroup.getObjectActionSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getObjectActionSection().getObjectActionCount(); i++) {

				String className =
					_xmlIniFileGroup.getObjectActionSection().getObjectAction(i).getClassName();
				getGUISeSAmDeclaration().getPanelActionFactoryManager().addPanelActionFactory(
					(IActionFactory) callGUISeSAmDeclarationConstructor(className));
			}
		}
	}

	private Object callGUISeSAmDeclarationConstructor(String className)
		throws
			ClassNotFoundException,
			InstantiationException,
			IllegalAccessException,
			InvocationTargetException,
			NoSuchMethodException {

		Class anClass = _classLoader.loadClass(className);
		Class[] classArray = { IGUISeSAmDeclaration.class };
		Object[] objectArray = { getGUISeSAmDeclaration()};

		return anClass.getConstructor(classArray).newInstance(objectArray);
	}

	private void parseMenuPluginSection()
		throws XMLIniFileException, InstantiationException, IllegalAccessException, ClassNotFoundException {

		if (_xmlIniFileGroup.getMenuPluginSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getMenuPluginSection().getClassNameCount(); i++) {

				String className = _xmlIniFileGroup.getMenuPluginSection().getClassName(i);

				Class menuPluginClass = _classLoader.loadClass(className);

				ISeSAmMenuPlugin menuPlugin = (ISeSAmMenuPlugin) menuPluginClass.newInstance();
				menuPlugin.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
				menuPlugin.setParent(_parent);

				JMenuItem menuItem = menuPlugin.createJMenuItemForPlugin();
				_pluginMenu.add(menuItem);
			}
		}
	}

	protected void parseArgumentDeclarationEditorPane(
		XMLIniFileType xmlIniFileType,
		ISpecificType specificType)
		throws XMLIniFileException, InstantiationException, IllegalAccessException, ClassNotFoundException {

		String argumentDeclarationEditorPaneFactoryClassName =
			xmlIniFileType.getArgumentDeclarationEditorPaneFactoryClass();

		String argumentDeclarationEditorClassName = xmlIniFileType.getArgumentDeclarationEditorClass();

		// use reflection to allow non GUI Application not to import GUI Classes
		if (argumentDeclarationEditorClassName != null) {

			if (argumentDeclarationEditorPaneFactoryClassName != null) {
				throw new XMLIniFileException(
					"Error in TypeSection for Type '"
						+ xmlIniFileType.getClassName()
						+ "' can specify both argumentDeclarationEditorPaneFactoryClassName and argumentDeclarationEditorClassName");
			}

			Class argumentDeclarationEditorClass = _classLoader.loadClass(argumentDeclarationEditorClassName);

			ArgumentDeclarationEditorPaneFactoryManager.addArgumentDeclarationEditorPaneFactory(
				specificType,
				argumentDeclarationEditorClass);

		} else if (argumentDeclarationEditorPaneFactoryClassName != null) {

			Object newInstance =
				_classLoader.loadClass(argumentDeclarationEditorPaneFactoryClassName).newInstance();

			ArgumentDeclarationEditorPaneFactoryManager.addArgumentDeclarationEditorPaneFactory(
				(ArgumentDeclarationEditorPaneFactoryManager
					.ArgumentDeclarationEditorPaneFactory) newInstance);
		}
	}
}
