/**
 * Created on 03.02.2004
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.model.graph;

import java.awt.Point;

import mygale.mcge.fig.FigNodePolygon;
import mygale.mcge.net.INetNode;

public abstract class FigRubberGraphNode extends FigNodePolygon {

    private boolean _isFixed;
    private double _vX;
    private double _vY;

    public void setFixed(boolean value) {
        _isFixed = value;

        setVX(0);
        setVY(0);

        fireStateChanged();
    }

    public boolean isFixed() {
        return _isFixed;
    }

    private final double INTERVAL = 5;
    private DoubleRingList xList;
    private DoubleRingList yList;

    protected int _minRingSizeBeforeSumUp;

    public void fix() {

        setFixed(true);
    }

    public void checkState() {

        xList.add(getLocation().x);
        yList.add(getLocation().y);

        if (xList.size() > _minRingSizeBeforeSumUp) {
            double xAvg = xList.sumUp() / xList.size();
            double yAvg = yList.sumUp() / yList.size();

            if (Math.abs(xAvg - getLocation().x) < INTERVAL && Math.abs(yAvg - getLocation().y) < INTERVAL)
                setFixed(true);
        }
    }

    protected void initRubber(int ringSize) {

        setFixed(false);

        initRingSize(ringSize);

        setMinRingSizeBeforeSum(30);
    }

    public void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, y, width, height);

        setFixed(true);
    }

    // the vX-methods change the speed in which the node moves for JRubberGraph
    public double getVX() {
        return _vX;
    }

    public void setVX(double vx) {
        _vX = vx;
    }

    public double getVY() {
        return _vY;
    }

    public void setVY(double vy) {
        _vY = vy;
    }

    public void changeLocation(double dx, double dy) {
        Point point = getLocation();

        setLocation(point.x + (int) dx, point.y + (int) dy);

    }

    public void initRingSize(int size) {
        xList = new DoubleRingList(size);
        yList = new DoubleRingList(size);

        xList.add(Integer.MAX_VALUE);
        yList.add(Integer.MAX_VALUE);
    }
    public void setMinRingSizeBeforeSum(int min) {
        _minRingSizeBeforeSumUp = min;
    }

    public INetNode getNetNode() {

        return (INetNode) getNetObject();
    }

}
