/**
 * Created on 24.07.2003
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.model.graph;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.fig.*;
import mygale.mcge.net.*;
import mygale.mcge.util.prefs.Prefs;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;

public class FigUMLGraphReflectiveEdge extends FigUMLGraphEdge {

	private int _lineHeight = 10;
	private final int _horOff = 3;
	
	private int offx = 100; //length of the longest horizontal line //topmost
	private int offy = 150; //length of the longest vertical line // (rightmost)

	public FigUMLGraphReflectiveEdge() {

		super();
	}

	private void calculatePoints() {

		IFigNode node = getBeginFigNode();

		Point point = node.getLocation();

		int width = node.getWidth();
		
		removeAllCtrlPoints();

		int nTh = getPositionOfThisReflectiveEdge(node);

		Point begin = point;
		Point end = point;

		int xLeftmost = begin.x - (offx / 2) + width - nTh * _horOff;
		int xBottom = begin.x + (offx / 2) + width + _lineHeight * nTh;
		int yLeftmost = begin.y - (offy / 2) - nTh * _horOff;
		int yRightmost = begin.y + (offy / 2) + nTh * _horOff;
		
		insertCtrlPointAt(0, new Point(xLeftmost, end.y));		
		insertCtrlPointAt(0, new Point(xLeftmost, yLeftmost));
		insertCtrlPointAt(0, new Point(xBottom, yLeftmost));
		insertCtrlPointAt(0, new Point(xBottom, yRightmost));
		insertCtrlPointAt(0, new Point(begin.x + width, yRightmost));

	}

	private int getPositionOfThisReflectiveEdge(IFigNode node) {

		Iterator allEdges = ((INetNode) node.getNetObject()).getNodeEdges();
		((DefaultNetNode) node.getNetObject()).getGraphModel();
		Iterator reflEdges = CollectionsHelper.select(allEdges, new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				INetEdge netEdge = (INetEdge) obj;
				return (netEdge.getBeginNode() == netEdge.getEndNode());
			}
		});

		int index = 0;
		while (reflEdges.hasNext()) {

			INetEdge element = (INetEdge) reflEdges.next();
			if (element == this.getNetObject())
				return index;

			index++;
		}

		Assert.fail(); // mustn't be
		return 0;
	}

	private void removeAllCtrlPoints() {

		clearCtrlPoints();
	}

	public int getMaxAllowedCtrlPoints() {
		return 3;
	}

	/**
	 * Method to paint this FigEdge.
	 * 
	 * @param g2d
	 *            the graphic context.
	 */
	public void paintContent(Graphics2D g2d, Prefs prefs, int dx, int dy, boolean shadowModeOn) {

		Stroke oldStroke = g2d.getStroke();

		g2d.setStroke(getStroke());

		paintPolyLine(g2d, prefs, dx, dy, shadowModeOn, oldStroke);

		g2d.setStroke(oldStroke);

		if (shadowModeOn
			&& (_shadowMoveMode == IFigEdge.DST_PORTS_SHADOW_MOVE)
			&& (getCtrlPointSize() > 3)) {
			// 2->3
			return;
		}

		Point begin;
		Point end;

		begin = getCtrlPointAt(1); //0->1
		end = getCtrlPointAt(2); //1->2

		String str = getName();

		FontMetrics fm = g2d.getFontMetrics();
		_lineHeight = fm.getHeight();

		int arrowWidth = (getEndArrow() == null) ? 0 : getEndArrow().getWidth();
		int border = 1;

		int strWidth = fm.stringWidth(str);
		int distance = (int) begin.distance(end) - 2 * border - arrowWidth;

		if (strWidth > distance) {
			str = FigNodePolygon.trimToFit(str, distance, fm, "...");
			strWidth = fm.stringWidth(str);
		}

		if (strWidth <= distance) {
			// zeichne Bezeichnung auf Kante
			if (end.x < begin.x)
				arrowWidth = -arrowWidth;

			int x = (int) Math.floor((end.x + begin.x - strWidth - arrowWidth) * 0.5);
			int y = (int) Math.floor((end.y + begin.y - 5) * 0.5);

			g2d.setColor(prefs.getFigForegroundColor());

			AffineTransform oldAf = g2d.getTransform();

			if (_shadowMoveMode == IFigEdge.BOTH_PORTS_SHADOW_MOVE) {
				g2d.setTransform(createRotateTransform(oldAf, g2d, str, begin, end, dx, dy));
				g2d.drawString(str, x + dx, y + dy);
			} else {
				g2d.setTransform(createRotateTransform(oldAf, g2d, str, begin, end, 0, 0));
				g2d.drawString(str, x, y);
			}

			g2d.setTransform(oldAf);
		}

	}

	private void paintPolyLine(
		Graphics2D g2d,
		Prefs prefs,
		int dx,
		int dy,
		boolean shadowModeOn,
		Stroke oldStroke) {

		calculatePoints();

		g2d.setColor(prefs.getFigEdgeColor());

		int size = getCtrlPointSize();

		if (_shadowMoveMode == IFigEdge.BOTH_PORTS_SHADOW_MOVE) {

			for (int i = 0; i < (size - 1); i++) {
				drawLine(g2d, getCtrlPointAt(i), getCtrlPointAt(i + 1), dx, dy);
			}

			drawLine(g2d, _shadowHelper._secondLast, _shadowHelper._last);

			paintArrows(g2d, prefs, dx, dy, shadowModeOn);

		} else {
			_shadowHelper.calculatePoints(dx, dy, shadowModeOn);

			drawLine(g2d, _shadowHelper._first, _shadowHelper._second);

			for (int i = 1; i < (size - 2); i++) {
				drawLine(g2d, getCtrlPointAt(i), getCtrlPointAt(i + 1));
			}

			drawLine(g2d, _shadowHelper._secondLast, _shadowHelper._last);

			paintArrows(g2d, prefs, 0, 0, shadowModeOn);
		}

	}

}
