package de.uniwue.ki.gui.mas.model.graph;

/*
 * Copyright (c) 1994-1996 Sun Microsystems, Inc. All Rights Reserved.
 * @modified 96/04/24 Jim Hagen : changed stressColor
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for NON-COMMERCIAL or COMMERCIAL purposes and without fee is
 * hereby granted. Please refer to the file
 * http://java.sun.com/copy_trademarks.html for further important copyright and
 * trademark information and to http://java.sun.com/licensing.html for further
 * important licensing information for the Java (tm) Technology.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR
 * ITS DERIVATIVES.
 * 
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE PERFORMANCE,
 * SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT NAVIGATION OR
 * COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE SUPPORT MACHINES, OR
 * WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE SOFTWARE COULD LEAD DIRECTLY TO
 * DEATH, PERSONAL INJURY, OR SEVERE PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH
 * RISK ACTIVITIES"). SUN SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED
 * WARRANTY OF FITNESS FOR HIGH RISK ACTIVITIES.
 */

import java.awt.*;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JPanel;

class Node {
    double _x;
    double _y;

    double _dx;
    double _dy;

    boolean _fixed; // true => Knoten hat feste Position

    String _lbl;

    private final double INTERVAL = 5;

    private DoubleRingList xList = new DoubleRingList();
    private DoubleRingList yList = new DoubleRingList();

    public void checkState() {

        xList.add(_x);
        yList.add(_y);

        if (xList.size() > 90) {
            double xAvg = xList.sumUp() / xList.size();
            double yAvg = yList.sumUp() / yList.size();

            if (Math.abs(xAvg - _x) < INTERVAL
                && Math.abs(yAvg - _y) < INTERVAL)
                _fixed = true;
        }
    }

    public Node() {

        xList.add(Integer.MAX_VALUE);
        yList.add(Integer.MAX_VALUE);
    }
}

class Edge {
    int _from;
    int _to;

    double _len;
}

class Graph extends Panel implements Runnable {
    GraphPanel _graph;
    int _nnodes;
    Node _nodes[] = new Node[100];

    int _nedges;
    Edge _edges[] = new Edge[200];

    Thread _relaxer;

    Graph(GraphPanel graph) {
        this._graph = graph;
    }

    int findNode(String lbl) {
        for (int i = 0; i < _nnodes; i++) {
            if (_nodes[i]._lbl.equals(lbl)) {
                return i;
            }
        }
        return addNode(lbl);
    }
    int addNode(String lbl) {
        Node n = new Node();
        n._x = 10 + 380 * Math.random();
        n._y = 10 + 380 * Math.random();
        n._lbl = lbl;
        _nodes[_nnodes] = n;
        return _nnodes++;
    }
    void addEdge(String from, String to, int len) {
        Edge e = new Edge();
        e._from = findNode(from);
        e._to = findNode(to);
        e._len = len;
        _edges[_nedges++] = e;
    }

    public void run() {
        while (true) {

            checkNodes();
            relax();

            //			 bringt (sehr unwahrscheinliche) Bewegung in's Spiel - dadurch
            // ordnen sich die Knoten schneller
            if ((Math.random() < 0.03)) {
                Node n = _nodes[(int) (Math.random() * _nnodes)];
                if (!n._fixed) {
                    n._x += 100 * Math.random() - 50;
                    n._y += 100 * Math.random() - 50;
                }
            }
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                break;
            }
        }
    }

    private void checkNodes() {

        for (int i = 0; i < _nnodes; i++) {

            _nodes[i].checkState();
        }
    }

    // die einzig wirklich interessante Methode: der Algorithmus zum
    // 'Entwirren'
    synchronized void relax() {
        for (int i = 0; i < _nedges; i++) {
            Edge e = _edges[i];
            double vx = _nodes[e._to]._x - _nodes[e._from]._x;
            double vy = _nodes[e._to]._y - _nodes[e._from]._y;
            double len = Math.sqrt(vx * vx + vy * vy); // lnge der kante
            double f = (_edges[i]._len - len) / (len * 3);
            double dx = f * vx;
            double dy = f * vy;

            // berechnet geschwindigkeit, mit der Knoten aufeinanderzu gezogen
            // werden - je lnger die kante zwischen ihnen ist, desto grer
            // die geschwindigkeit
            _nodes[e._to]._dx += dx;
            _nodes[e._to]._dy += dy;
            _nodes[e._from]._dx += -dx;
            _nodes[e._from]._dy += -dy;
        }

        // Knoten stoen sich gegenseitig ab
        for (int i = 0; i < _nnodes; i++) {
            Node n1 = _nodes[i];
            double dx = 0;
            double dy = 0;

            for (int j = 0; j < _nnodes; j++) {
                if (i == j) {
                    continue;
                }
                Node n2 = _nodes[j];
                double vx = n1._x - n2._x;
                double vy = n1._y - n2._y;
                double len = vx * vx + vy * vy;
                if (len == 0) {
                    dx += Math.random();
                    dy += Math.random();
                } else if (len < 100 * 100) {
                    dx += vx / len;
                    dy += vy / len;
                }
            }
            double dlen = dx * dx + dy * dy;
            if (dlen > 0) {
                dlen = Math.sqrt(dlen) / 2;
                n1._dx += dx / dlen;
                n1._dy += dy / dlen;
            }
        }

        Dimension d = getSize();

        // Knoten bewegen sich immer ein wenig - bringt bewegung ins spiel
        for (int i = 0; i < _nnodes; i++) {
            Node n = _nodes[i];
            if (!n._fixed) {
                n._x += Math.max(-5, Math.min(5, n._dx));
                n._y += Math.max(-5, Math.min(5, n._dy));

                if (n._x < 0) {
                    n._x = 0;
                } else if (n._x > d.width) {
                    n._x = d.width;
                }
                if (n._y < 0) {
                    n._y = 0;
                } else if (n._y > d.height) {
                    n._y = d.height;
                }
            }
            n._dx /= 2;
            n._dy /= 2;
        }
        repaint();
    }

    Node _pick;
    boolean _pickfixed;
    Image _offscreen;
    Dimension _offscreensize;
    Graphics _offgraphics;

    final Color fixedColor = Color.red;
    final Color selectColor = Color.pink;
    final Color edgeColor = Color.black;
    final Color nodeColor = new Color(250, 220, 100);
    final Color stressColor = Color.darkGray;
    final Color arcColor1 = Color.black;
    final Color arcColor2 = Color.pink;
    final Color arcColor3 = Color.red;

    public void paintNode(Graphics g, Node n, FontMetrics fm) {
        int x = (int) n._x;
        int y = (int) n._y;
        g.setColor(
            (n == _pick) ? selectColor : (n._fixed ? fixedColor : nodeColor));
        int w = fm.stringWidth(n._lbl) + 10;
        int h = fm.getHeight() + 4;
        g.fillRect(x - w / 2, y - h / 2, w, h);
        g.setColor(Color.black);
        g.drawRect(x - w / 2, y - h / 2, w - 1, h - 1);
        g.drawString(
            n._lbl,
            x - (w - 10) / 2,
            (y - (h - 4) / 2) + fm.getAscent());
    }

    public synchronized void update(Graphics g) {
        Dimension d = getSize();
        if ((_offscreen == null)
            || (d.width != _offscreensize.width)
            || (d.height != _offscreensize.height)) {
            _offscreen = createImage(d.width, d.height);
            _offscreensize = d;
            _offgraphics = _offscreen.getGraphics();
            _offgraphics.setFont(getFont());
        }

        _offgraphics.setColor(getBackground());
        _offgraphics.fillRect(0, 0, d.width, d.height);
        for (int i = 0; i < _nedges; i++) {
            Edge e = _edges[i];
            int x1 = (int) _nodes[e._from]._x;
            int y1 = (int) _nodes[e._from]._y;
            int x2 = (int) _nodes[e._to]._x;
            int y2 = (int) _nodes[e._to]._y;
            int len =
                (int) Math.abs(
                    Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2))
                        - e._len);
            _offgraphics.setColor(
                (len < 10) ? arcColor1 : (len < 20 ? arcColor2 : arcColor3));
            _offgraphics.drawLine(x1, y1, x2, y2);

        }

        FontMetrics fm = _offgraphics.getFontMetrics();
        for (int i = 0; i < _nnodes; i++) {
            paintNode(_offgraphics, _nodes[i], fm);
        }

        g.drawImage(_offscreen, 0, 0, null);
    }

    public synchronized boolean mouseDown(Event evt, int x, int y) {
        double bestdist = Double.MAX_VALUE;
        for (int i = 0; i < _nnodes; i++) {
            Node n = _nodes[i];
            double dist = (n._x - x) * (n._x - x) + (n._y - y) * (n._y - y);
            if (dist < bestdist) {
                _pick = n;
                bestdist = dist;
            }
        }
        _pickfixed = _pick._fixed;
        _pick._fixed = true;
        _pick._x = x;
        _pick._y = y;
        repaint();
        return true;
    }

    public synchronized boolean mouseDrag(Event evt, int x, int y) {
        _pick._x = x;
        _pick._y = y;
        repaint();
        return true;
    }

    public synchronized boolean mouseUp(Event evt, int x, int y) {
        _pick._x = x;
        _pick._y = y;
        _pick._fixed = _pickfixed;
        _pick = null;

        repaint();
        return true;
    }

    public void start() {
        _relaxer = new Thread(this);
        _relaxer.start();
    }

    public void stop() {
        _relaxer.stop();
    }
}

public class GraphPanel extends JPanel {
    Graph _panel;

    public static void main(String[] args) {

        JFrame frame = new JFrame();
        GraphPanel g = new GraphPanel();
        g.init();
        frame.getContentPane().add(g);
        frame.setSize(400, 400);
        frame.show();

        g.start();
    }

    public Graph getPanel() {

        return _panel;
    }

    public Dimension size() {

        return new Dimension(400, 400);
    }

    public void init() {
        setLayout(new BorderLayout());

        _panel = new Graph(this);
        add("Center", _panel);
        Panel p = new Panel();
        add("South", p);
        p.add(new Button("Scramble"));
        p.add(new Button("Shake"));

        String edges =
            "a1-b1,a1-a2,a2-a3,a3-a4,a4-a5,a5-a6,b1-b2,b2-b3,b3-b4,b4-b5,b5-b6,c1-c2,c2-c3,c3-c4,c4-c5,c5-c6,x-a1,x-b1,x-c1,x-a6,x-b6,x-c6";
        for (StringTokenizer t = new StringTokenizer(edges, ",");
            t.hasMoreTokens();
            ) {
            String str = t.nextToken();
            int i = str.indexOf('-');
            if (i > 0) {
                int len = 50;
                int j = str.indexOf('/');
                if (j > 0) {
                    len = Integer.valueOf(str.substring(j + 1)).intValue();
                    str = str.substring(0, j);
                }
                _panel.addEdge(str.substring(0, i), str.substring(i + 1), len);
            }
        }

        Dimension d = size();
        Node n = _panel._nodes[_panel.findNode("x")];
        n._x = d.width / 2;
        n._y = d.height / 2;
        n._fixed = true;

    }

    public void start() {
        _panel.start();
    }
    public void stop() {
        _panel.stop();
    }
    public boolean action(Event evt, Object arg) {

        if ("Scramble".equals(arg)) {
            Dimension d = size();
            for (int i = 0; i < _panel._nnodes; i++) {
                Node n = _panel._nodes[i];
                if (!n._fixed) {
                    n._x = 10 + (d.width - 20) * Math.random();
                    n._y = 10 + (d.height - 20) * Math.random();
                }
            }
            return true;
        }
        if ("Shake".equals(arg)) {
            //Dimension d = getSize();
            for (int i = 0; i < _panel._nnodes; i++) {
                Node n = _panel._nodes[i];
                if (!n._fixed) {
                    n._x += 80 * Math.random() - 40;
                    n._y += 80 * Math.random() - 40;
                }
            }
            return true;
        }
        return false;
    }
}
