/**
 * Created on 14.01.2004
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.model.graph;

import java.awt.*;
import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.core.JGraph;
import mygale.mcge.core.JGraphEditor;
import mygale.mcge.fig.*;
import mygale.mcge.net.*;
import de.uniwue.ki.mas.model.graph.RubberGraphDeclarationWithNetModel;
import de.uniwue.ki.mas.model.graph.UMLLayoutGraphDeclaration;

public class RubberGraph extends JGraph {

    protected RubberGraphBrain _brain;

    public RubberGraph(
        JGraphEditor graphEditor,
        IFigNodeFactory figNodeFactory,
        IFigEdgeFactory figEdgeFactory) {

        super(graphEditor, figNodeFactory, figEdgeFactory);

        setMinimumSize(new Dimension(1000, 800));

    }

    public void remove(IFigLayered fig) {
        if (!(fig instanceof IFigObject)) {
            getLayerManager().remove(fig);
        } else {
            INetObject netobject = ((IFigObject) fig).getNetObject();
            if (netobject instanceof INetNode) {
                remove((INetNode) netobject);
            } else if (netobject instanceof INetEdge) {
                remove((INetEdge) netobject);
            } else {
               
            }
        }
    }

    public boolean isFixed() {
        // return whether all nodes are fixed
        return getNetGraphModelIntern().isFixed();
    }

    public void paint(Graphics g) {
        super.paint(g);

        if (!isFixed()) {

            //		getModeManager().beginMode(new ModeMove());

            rubberize();

            //		getModeManager().endMode();
        }
    }

    protected void makeGraphDiagram() {
        // this overwrites the makeGraphDiagram() from JGraph.
        // this is necessary to get the right NetGraphModel into the graph
        // otherwise it would be a simple DefaultNetGraphModel, causing
        // ClassCastExceptions
        //TODO I don't think this is clean --steffen
        //        setNetGraphModel(new UMLLayoutGraphDeclaration());
    }

    public void setNetGraphModel(INetGraphModel newNetGraphModel) {

        if (newNetGraphModel != null && !(newNetGraphModel instanceof UMLLayoutGraphDeclaration))
            Assert.fail(
                "Expected UMLLayoutGraphDeclaration but found " + newNetGraphModel.getClass().getName());

        super.setNetGraphModel(newNetGraphModel);

        if (newNetGraphModel != null)
            init();
        // notwendig, beim disposen des Panels wird ein NULL-netGraphModel
        // gesetzt => sonst: NullPointerExction
    }

    private RubberGraphDeclarationWithNetModel getNetGraphModelIntern() {

        return (RubberGraphDeclarationWithNetModel) super.getNetGraphModel();
    }

    private void init() {
        _brain = new RubberGraphBrain(this);

        // depending on the number of nodes in the graph - the nodes get
        // more
        // or less time to arrange themselves.

        Iterator nodes = getNetGraphModel().getNodes();
        while (nodes.hasNext()) {
            FigUMLGraphNode node = ((FigUMLGraphNode) getFigMapper().getFigFor((INetObject) nodes.next()));

            node.initRingSize(_nodeC * 10 * 4);
            node.setMinRingSizeBeforeSum(_nodeC * 10 * 2);
        }
    }

    protected int _nodeC = 0; // counting number of nodes in this model

    protected void addFigForNode(INetNode netnode, boolean val) {
        super.addFigForNode(netnode, val);
        _nodeC++;
    }

    private void rubberize() {

        checkStateOfNodes();

        attract();

        repel();

        move();

        checkStateOfGraph();
    }

    private void checkStateOfGraph() {
        _brain.checkState();
    }

    private boolean isFloating() {

        // return whether the graph is floating into one direction

        return _brain.isFloating();
    }

    private boolean _allFixed;
    protected void checkStateOfNodes() {

        _allFixed = true;
        Iterator nodes = getNetGraphModelIntern().getNodes();
        while (nodes.hasNext()) {
            // let nodes check whether they should change their state to
            // 'fixed'
            FigUMLGraphNode node = ((FigUMLGraphNode) getFigFor((INetObject) nodes.next()));

            node.checkState();
            _allFixed &= node.isFixed();

        }

        if (isFloating())
            fixAllNodes();

    }
    protected void fixAllNodes() {

        _allFixed = true;
        Iterator nodes = getNetGraphModelIntern().getNodes();
        while (nodes.hasNext()) {

            FigUMLGraphNode node = ((FigUMLGraphNode) getFigFor((INetObject) nodes.next()));

            node.fix();
        }
    }

    private void move() {
        // hier werden die Knoten bewegt, falls sie nicht schon fest
        // positioniert sind.
        Iterator nodes3 = getNetGraphModel().getNodes();
        while (nodes3.hasNext()) {
            FigUMLGraphNode n = (FigUMLGraphNode) getFigFor((INetObject) nodes3.next());

            if (!n.isFixed()) {

                moveInLayer(n, (int) n.getVX(), (int) n.getVY());
            }

            // nachdem die Figur einmal bewegt wurde, wird ihr
            // Vorwrtsdrang
            // reduziert
            n.setVX(n.getVX() / 2);
            n.setVY(n.getVY() / 2);

        }
    }

    private void repel() {

        // Knoten stoen sich gegenseitig ab
        Iterator nodes2 = getNetGraphModel().getNodes();
        while (nodes2.hasNext()) {
            FigUMLGraphNode n1 = (FigUMLGraphNode) getFigFor((INetObject) nodes2.next());

            double dx = 0;
            double dy = 0;

            Iterator nodes2_intern = getNetGraphModel().getNodes();
            while (nodes2_intern.hasNext()) {
                FigUMLGraphNode n2 = (FigUMLGraphNode) getFigFor((INetObject) nodes2_intern.next());

                if (n1 == n2)
                    continue;

                double vx = n1.getLocation().x - n2.getLocation().x;
                double vy = n1.getLocation().y - n2.getLocation().y;
                double len = vx * vx + vy * vy;

                int sizeNode = Math.max(n1.getHeight(), n2.getWidth()) + 100;
                if (len < sizeNode * sizeNode) {

                    dx += vx * 3 / len;
                    dy += vy * 3 / len;
                }

            }

            double dlen = dx * dx + dy * dy; // delta
            if (dlen > 0) {
                dlen = Math.sqrt(dlen) / 10;

                n1.setVX(n1.getVX() + dx / dlen);
                n1.setVY(n1.getVY() + dy / dlen);
            }

        }
    }

    private void attract() {

        Iterator edges = getNetGraphModel().getEdges();
        while (edges.hasNext()) {

            FigUMLGraphEdge edge = (FigUMLGraphEdge) getFigFor((INetObject) edges.next());

            if (edge instanceof FigUMLGraphReflectiveEdge)
                continue;

            FigUMLGraphNode fromNode = (FigUMLGraphNode) edge.getBeginFigNode();
            FigUMLGraphNode toNode = (FigUMLGraphNode) edge.getEndFigNode();

            Point fromLoc = fromNode.getLocation();
            Point toLoc = toNode.getLocation();

            double vx = fromLoc.x - toLoc.x;
            double vy = fromLoc.y - toLoc.y;
            double len = Math.sqrt(vx * vx + vy * vy); // lnge der
            // kante
            double f = len * len / 10000000;

            double dx = f * vx;
            double dy = f * vy;

            // berechnet geschwindigkeit, mit der Knoten aufeinanderzu
            // gezogen
            // werden - je lnger die kante zwischen ihnen ist, desto
            // grer
            // die geschwindigkeit
            toNode.setVX(toNode.getVX() + dx);
            toNode.setVY(toNode.getVY() + dy);
            fromNode.setVX(fromNode.getVX() - dx);
            fromNode.setVY(fromNode.getVY() - dy);
        }
    }

    private void moveInLayer(FigUMLGraphNode fig, int dx, int dy) {

        fig.changeLocation(dx, dy);

        udpateEdges(fig, dx, dy);

        damaged((Rectangle) null); // der gesamte Graphenbereich wird neu
        // gezeichnet
        Point loc = fig.getLocation();
        scrollToShow(loc.x, loc.y);
    }

    private void udpateEdges(FigUMLGraphNode fig, int dx, int dy) {
        Iterator inEdges = ((INetNode) fig.getNetObject()).getIncomingNodeEdges();
        Iterator outEdges = ((INetNode) fig.getNetObject()).getOutgoingNodeEdges();

        while (inEdges.hasNext()) {

            FigUMLGraphEdge edge = (FigUMLGraphEdge) getFigFor((INetObject) inEdges.next());
            edge.updateEndNode(dx, dy);
            //			Rectangle bounds = edge.getBounds();
            //			
            //			Point p = edge.getCtrlPointAt(edge.getCtrlPointSize() - 1);
            //			edge.setCtrlPointAt(edge.getCtrlPointSize() - 1, new
            // Point(p.x +
            // dx, p.y + dy));

        }

        while (outEdges.hasNext()) {

            FigUMLGraphEdge edge = (FigUMLGraphEdge) getFigFor((INetObject) outEdges.next());

            edge.updateStartNode(dx, dy);
            //			Point p = edge.getCtrlPointAt(0);
            //			edge.setCtrlPointAt(0, new Point(p.x + dx, p.y + dy));

        }

    }

}
