/**
 * Created on 04.02.2004
 *
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.model.graph;

import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.net.INetObject;

public class RubberGraphBrain {
    // eigentlich knnten von dieser Klasse FigUMLGraphNode und RubberGraph
    // erben, damit sie ihre 'Gummieigenschaft' haben. Leider ist
    // Mehrfachvererbung in Java aber nicht mglich. Daher ein Objekt dieser
    // Klasse als Member.
    // Jetzt ist RubberGraphBrain auf RubberGraph zugeschnitten (siehe
    // checkState())...

    private static final int RING_SIZE = 200;
    private RubberGraph _graph;

    public RubberGraphBrain(RubberGraph graph) {
        _graph = graph;

        init(RING_SIZE);
    }

    private final double INTERVAL = 2;
    private DoubleRingList xList;
    private DoubleRingList yList;

    protected int _minRingSizeBeforeSumUp;

    public void checkState() {
        int count = 0;
        double sumX = 0;
        double sumY = 0;

        Iterator nodes = _graph.getNetGraphModel().getNodes();
        while (nodes.hasNext()) {
            FigUMLGraphNode n = (FigUMLGraphNode) _graph.getFigMapper().getFigFor((INetObject) nodes.next());

            if (n == null)
                continue;

            sumX += n.getVX();
            sumY += n.getVY();
            count++;
        }

        xList.add(sumX / count);
        yList.add(sumY / count);

        if (xList.size() > _minRingSizeBeforeSumUp) {
            double xAvg = xList.sumUp() / xList.size();
            double yAvg = yList.sumUp() / yList.size();

            if (Math.abs(xAvg) < INTERVAL && Math.abs(yAvg) < INTERVAL)
                setGraphFloats(true);
        }
    }

    private boolean _floating;
    private void setGraphFloats(boolean b) {
        // the graph floats if it moves constantly into one direction (all
        // nodes move in the same direction)
        _floating = b;
    }

    public boolean isFloating() {

        return _floating;
    }

    protected void init(int ringSize) {

        setGraphFloats(false);

        initRingSize(ringSize);

        setMinRingSizeBeforeSum(100);
    }

    public void initRingSize(int size) {
        xList = new DoubleRingList(size);
        yList = new DoubleRingList(size);

        xList.add(Integer.MAX_VALUE);
        yList.add(Integer.MAX_VALUE);
    }

    public void setMinRingSizeBeforeSum(int min) {
        Assert.assertTrue(min < RING_SIZE);

        _minRingSizeBeforeSumUp = min;
    }

}
