package de.uniwue.ki.gui.mas.model.tree;
import java.awt.*;

import javax.swing.Icon;
/**
 * Creation date: (28.03.2002 10:18:52)
 * @author Christoph Oechslein
 */
public class InsetsIcon implements Icon {
	private final Icon _icon;
	private Insets _insets = null;
	public InsetsIcon(Icon icon) {
		super();

		_icon = icon;
	}

	public InsetsIcon(Icon icon, int border) {
		this(icon, new Insets(border, border, border, border));
	}

	public InsetsIcon(Icon icon, Insets insets) {
		super();

		_icon = icon;
		setInsets(insets);
	}

	public Insets getInsets() {
		return _insets;
	}

	public void setInsets(Insets newInsets) {
		_insets = newInsets;
	}

	/**
	 * Returns the icon's height.
	 *
	 * @return an int specifying the fixed height of the icon.
	 */
	public int getIconHeight() {
		return getTop() + _icon.getIconHeight() + getBottom();
	}

	/**
	 * Returns the icon's width.
	 *
	 * @return an int specifying the fixed width of the icon.
	 */
	public int getIconWidth() {
		return getLeft() + _icon.getIconWidth() + getRight();
	}

	/**
	 * Draw the icon at the specified location.  Icon implementations
	 * may use the Component argument to get properties useful for 
	 * painting, e.g. the foreground or background color.
	 */
	public void paintIcon(Component c, Graphics g, int x, int y) {
		_icon.paintIcon(c, g, x + getLeft(), y + getTop());
	}

	private int getBottom() {
		return getInsets() == null ? 0 : getInsets().bottom;
	}

	private int getLeft() {
		return getInsets() == null ? 0 : getInsets().left;
	}

	private int getRight() {
		return getInsets() == null ? 0 : getInsets().right;
	}

	private int getTop() {
		return getInsets() == null ? 0 : getInsets().top;
	}
}