package de.uniwue.ki.gui.mas.model.tree;
import java.awt.Component;
import java.awt.Graphics;
import java.util.Iterator;
import java.util.List;

import javax.swing.Icon;

import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (28.03.2002 10:18:52)
 * @author Christoph Oechslein
 */
public class ManyIconsAsOneIcon implements Icon {
    private final List _iconList;
    private int _gap = 1;
    private boolean _horizontal = true;



	public ManyIconsAsOneIcon(List iconList) {
	    super();
	
	    _iconList = iconList;
	}



	public ManyIconsAsOneIcon(Icon iconA, Icon iconB) {
	    this(CollectionsHelper.createList(iconA, iconB));
	}



	public int getGap() {
	    return _gap;
	}



	public void setGap(int newGap) {
	    _gap = newGap;
	}



	public boolean isHorizontal() {
	    return _horizontal;
	}



	public void setHorizontal(boolean newHorizontal) {
	    _horizontal = newHorizontal;
	}



	/**
	 * Returns the icon's height.
	 *
	 * @return an int specifying the fixed height of the icon.
	 */
	public int getIconHeight() {
	    int height = 0;
	
	    Iterator it = _iconList.iterator();
	
	    if (isHorizontal()) {
	        while (it.hasNext()) {
	            Icon currIcon = (Icon) it.next();
	
	            height = Math.max(currIcon.getIconHeight(), height);
	        }
	
	    } else {
	        while (it.hasNext()) {
	            Icon currIcon = (Icon) it.next();
	
	            height += currIcon.getIconHeight();
	
	            if (it.hasNext())
	                height += getGap();
	        }
	    }
	
	    return height;
	}



	/**
	 * Returns the icon's width.
	 *
	 * @return an int specifying the fixed width of the icon.
	 */
	public int getIconWidth() {
	    int width = 0;
	
	    Iterator it = _iconList.iterator();
	
	    if (!isHorizontal()) {
	        while (it.hasNext()) {
	            Icon currIcon = (Icon) it.next();
	
	            width = Math.max(currIcon.getIconWidth(), width);
	        }
	
	    } else {
	        while (it.hasNext()) {
	            Icon currIcon = (Icon) it.next();
	
	            width += currIcon.getIconWidth();
	
	            if (it.hasNext())
	                width += getGap();
	        }
	    }
	
	    return width;
	}



	/**
	 * Draw the icon at the specified location.  Icon implementations
	 * may use the Component argument to get properties useful for 
	 * painting, e.g. the foreground or background color.
	 */
	public void paintIcon(Component c, Graphics g, int x, int y) {
	    int currX = x;
	    int currY = y;
	
	    Iterator it = _iconList.iterator();
	    while (it.hasNext()) {
	        Icon currIcon = (Icon) it.next();
	
	        currIcon.paintIcon(c, g, currX, currY);
	
	        if (_horizontal) {
	            currX += currIcon.getIconWidth();
	
	            if (it.hasNext())
	                currX += getGap();
	
	        } else {
	            currY += currIcon.getIconHeight();
	
	            if (it.hasNext())
	                currY += getGap();
	        }
	    }
	}
}