package de.uniwue.ki.gui.mas.model.tree;
import java.awt.Color;
import java.awt.Component;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.tree.TreeNode;

import junit.framework.Assert;
import de
	.uniwue
	.ki
	.gui
	.mas
	.basics
	.functionArgument
	.SpecificFunctionDeclarationWithArgDeclBindsCellRendererAdapter;
import de.uniwue.ki.gui.mas.basics.functionUser.UserFunctionDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.basics.type.user.UserTypeDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.entity.objectclass.ActivityAgentClassDeclarationListPanel;
import de.uniwue.ki.gui.mas.entity.objectclass.ObjectClassDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.entity.variableclass.BodyVariableClassDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.entity.variableclass.VariableClassDeclarationCellRendererAdapter;
import de.uniwue.ki.gui.mas.sim.experiment.ExperimentListPanelHelper;
import de.uniwue.ki.gui.util.*;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRuleDeclaration;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphTransitionEdgeDeclaration;
import de.uniwue.ki.mas.entity.objectclass.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.sim.analysis.IAnalysisDeclaration;
import de.uniwue.ki.mas.sim.experiment.ExperimentDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.experiment.IExperiment;
import de.uniwue.ki.mas.sim.simulation.SimulationDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.situation.ISituation;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.name.IDocumentable;
import de.uniwue.ki.util.name.INamed;
import de.uniwue.ki.util.tree.IObjectTreeNode;

/**
 * Creation date: (28.03.2002 09:52:36)
 * 
 * @author Christoph Oechslein
 */

public class SeSAmDeclarationTreeCellRenderer extends NamedTreeCellRenderer {
	
	public static final ImageIcon WORLD_CLASS_LIST_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-worlds.gif");
	public static final ImageIcon RESSOURCES_CLASS_LIST_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-resources.gif");
	public static final ImageIcon ANALYSIS_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-analysis.gif");
	public static final ImageIcon SITUATION_ICON =
		SwingMoreUtilities.createImageIcon("others/sesam-situation-class-icon.gif");
	public static final ImageIcon FEATURE_CLASS_LIST_ICON =
		SwingMoreUtilities.createImageIcon("others/feature.gif");
	public static final ImageIcon AGENT_CLASS_ICON = 
		SwingMoreUtilities.createImageIcon("others/User.gif");
		
	public interface ISeSAmTreeNodeRenderer {
		String getText(IObjectTreeNode objectTreeNode, String defaultText);
		String getToolTipText(IObjectTreeNode objectTreeNode, String defaultText, String defaultToolTipText);
		Icon getIcon(
			IObjectTreeNode objectTreeNode,
			Icon defaultIcon,
			boolean expanded,
			boolean leaf,
			boolean disabled);
	}

	public static class AbstractSeSAmTreeNodeRenderer implements ISeSAmTreeNodeRenderer {
		public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
			return defaultText;
		}

		public String getToolTipText(
			IObjectTreeNode objectTreeNode,
			String defaultText,
			String defaultToolTipText) {
			return defaultToolTipText;
		}

		public Icon getIcon(
			IObjectTreeNode objectTreeNode,
			Icon defaultIcon,
			boolean expanded,
			boolean leaf,
			boolean disabled) {
			return defaultIcon;
		}
	}

	private static final LinkedList _classList = new LinkedList();

	static {
		initialize();
	}

	public SeSAmDeclarationTreeCellRenderer() {
		super();
	}

	private static void add(Class aClass, ISeSAmTreeNodeRenderer seSAmTreeNodeRenderer) {
		_classList.add(aClass);
		_classList.add(seSAmTreeNodeRenderer);
	}

	private static void initialize() {
		add(IBodyVariableClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			private IObjectClassDeclaration getObjectClassDeclaration(TreeNode node) {
				IObjectTreeNode objectTreeNode = (IObjectTreeNode)node;
				if (objectTreeNode == null)
					return null;
				else if (objectTreeNode.getUserObject() instanceof IObjectClassDeclaration)
					return (IObjectClassDeclaration)objectTreeNode.getUserObject();
				else
					return getObjectClassDeclaration(objectTreeNode.getParent());
			}

			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				return BodyVariableClassDeclarationCellRendererAdapter.getBodyVariableClassDeclarationText(
					(IBodyVariableClassDeclaration)objectTreeNode.getUserObject(),
					getObjectClassDeclaration(objectTreeNode));
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				return BodyVariableClassDeclarationCellRendererAdapter
					.getBodyVariableClassDeclarationToolTipText(
					(IBodyVariableClassDeclaration)objectTreeNode.getUserObject(),
					getObjectClassDeclaration(objectTreeNode));
			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				Assert.assertTrue(
					"getIcon expected IVariableClassDeclaration",
					objectTreeNode.getUserObject() instanceof IBodyVariableClassDeclaration);

				IBodyVariableClassDeclaration bodyVariableClassDeclaration =
					((IBodyVariableClassDeclaration)objectTreeNode.getUserObject());

				if (bodyVariableClassDeclaration.getVariableClassDeclaration().isExternal())
					return VariableClassDeclarationCellRendererAdapter.PUBLIC_ICON;
				else
					return VariableClassDeclarationCellRendererAdapter.PRIVATE_ICON;
			}
		});

		add(IFunctionCallInternDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {

				IFunctionCallInternDeclaration callDeclaration =
					(IFunctionCallInternDeclaration)objectTreeNode.getUserObject();

				if (callDeclaration instanceof IAtomicFunctionCallInternDeclaration)
					return ((IAtomicFunctionCallInternDeclaration)callDeclaration).getName();
				else
					return ((IComposedFunctionCallInternDeclaration)callDeclaration)
						.getSpecificFunctionDeclaration()
						.getName();
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				return defaultText;
			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return defaultIcon;
			}
		});

		add(IExperiment.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				return defaultText;
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {

				IExperiment experiment = (IExperiment)objectTreeNode.getUserObject();
				if ((experiment.getCurrentSimulation() != null)
					&& (experiment.getCurrentSimulation().getSituation() != null)) {
					ISituation situation = experiment.getCurrentSimulation().getSituation();
					int count =
						(situation.getWorld() != null)
							? situation.getWorld().getObjectCount(false, false)
							: 0;
					return defaultToolTipText
						+ " [ticks: "
						+ situation.getTimeDurationInTicks()
						+ "; count: "
						+ count
						+ "]";
				} else {
					return defaultToolTipText;
				}

			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				if (Assert.ASSERTION_ON) {
					Assert.assertTrue(
						"getIcon expected IExperiment",
						objectTreeNode.getUserObject() instanceof IExperiment);
				}

				return ExperimentListPanelHelper.getMarkerIcon((IExperiment)objectTreeNode.getUserObject());
			}

		});

		add(IVariableClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				return VariableClassDeclarationCellRendererAdapter.getVariableClassDeclarationText(
					(IVariableClassDeclaration)objectTreeNode.getUserObject());
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				return VariableClassDeclarationCellRendererAdapter.getVariableClassDeclarationToolTipText(
					(IVariableClassDeclaration)objectTreeNode.getUserObject());
			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				Assert.assertTrue(
					"getIcon expected IVariableClassDeclaration",
					objectTreeNode.getUserObject() instanceof IVariableClassDeclaration);

				return VariableClassDeclarationCellRendererAdapter.getMarkerIcon(
					((IVariableClassDeclaration)objectTreeNode.getUserObject()));
			}

		});

		add(IUserFunctionDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				return SpecificFunctionDeclarationWithArgDeclBindsCellRendererAdapter
					.getSpecificFunctionDeclarationWithArgDeclBindsName(
					(IUserFunctionDeclaration)objectTreeNode.getUserObject());
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {

				IUserFunctionDeclaration userFunDeclaration =
					(IUserFunctionDeclaration)objectTreeNode.getUserObject();
				if (userFunDeclaration.getDocumentation().length() > 0)
					return "<html><strong>"
						+ NamedCellRendererAdapter.getQuotedString(defaultText)
						+ "</strong><P>"
						+ NamedCellRendererAdapter.getQuotedString(userFunDeclaration.getDocumentation());
				else
					return NamedCellRendererAdapter.getQuotedString(defaultText);

			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				Assert.assertTrue(
					"getIcon expected IFunctionDeclaration",
					objectTreeNode.getUserObject() instanceof IUserFunctionDeclaration);

				return UserFunctionDeclarationCellRendererAdapter.getMarkerIcon(
					(IUserFunctionDeclaration)objectTreeNode.getUserObject());
			}
		});

		add(IUserFeatureClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				Assert.assertTrue(
					"getIcon expected IUserFeatureDeclaration",
					objectTreeNode.getUserObject() instanceof IUserFeatureClassDeclaration);
				return FEATURE_CLASS_LIST_ICON;
			}
		});

		add(IActivityGraphTransitionEdgeDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			private String getText(IActivityGraphTransitionEdgeDeclaration activityEdgeDeclaration) {
				return "<strong>"
					+ activityEdgeDeclaration.getName()
					+ "</strong> ("
					+ activityEdgeDeclaration.getFromActivityGraphNodeDeclaration().getName()
					+ " -> "
					+ activityEdgeDeclaration.getToActivityGraphNodeDeclaration().getName()
					+ ")";
			}

			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				return "<html>"
					+ getText((IActivityGraphTransitionEdgeDeclaration)objectTreeNode.getUserObject());
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				IActivityGraphTransitionEdgeDeclaration activityEdgeDeclaration =
					(IActivityGraphTransitionEdgeDeclaration)objectTreeNode.getUserObject();

				String text = defaultText;

				if (activityEdgeDeclaration instanceof IActivityGraphRuleDeclaration) {
					IActivityGraphRuleDeclaration activityRuleDeclaration =
						(IActivityGraphRuleDeclaration)activityEdgeDeclaration;
					if (activityRuleDeclaration.getUsesShortDescription()) {
						text += "<P>"
							+ activityRuleDeclaration.getConditionFunctionCallDeclaration().getName();
					}
				}

				if (activityEdgeDeclaration instanceof IDocumentable) {
					IDocumentable documentable = (IDocumentable)activityEdgeDeclaration;
					if (documentable.getDocumentation() != null)
						text += "<P>" + documentable.getDocumentation();
				}

				return "<html>" + text;
			}
		});

		add(IUserTypeDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getText(IObjectTreeNode objectTreeNode, String defaultText) {
				if (objectTreeNode.getUserObject() instanceof IUserTypeDeclaration)
					return UserTypeDeclarationCellRendererAdapter.getUserTypeDeclarationText(
						(IUserTypeDeclaration)objectTreeNode.getUserObject());
				return defaultText;
			}

			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {

				IUserTypeDeclaration userTypDeclaration =
					(IUserTypeDeclaration)objectTreeNode.getUserObject();
				if (userTypDeclaration.getDocumentation().length() > 0)
					return "<html><strong>"
						+ NamedCellRendererAdapter.getQuotedString(defaultText)
						+ "</strong><P>"
						+ NamedCellRendererAdapter.getQuotedString(userTypDeclaration.getDocumentation());
				else
					return NamedCellRendererAdapter.getQuotedString(defaultText);
				/*
				 * if (objectTreeNode.getUserObject() instanceof
				 * IUserTypeDeclaration) return
				 * UserTypeDeclarationCellRendererAdapter.getUserTypeDeclarationToolTipText(
				 * (IUserTypeDeclaration) objectTreeNode.getUserObject());
				 */
			}

			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				if (objectTreeNode.getUserObject() instanceof IUserTypeDeclaration)
					return UserTypeDeclarationCellRendererAdapter.getMarkerIcon(
						(IUserTypeDeclaration)objectTreeNode.getUserObject());
				return defaultIcon;
			}
		});

		add(IWorldClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return ObjectClassDeclarationCellRendererAdapter.getWorldIcon();
			}
		});

		add(IRessourceClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return ObjectClassDeclarationCellRendererAdapter.getRessourceIcon();
			}
		});

		add(IActivityAgentClassDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return ObjectClassDeclarationCellRendererAdapter.getActivityAgentIcon();
			}
		});

		add(IWorldClassDeclarationListAndFactory.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return WORLD_CLASS_LIST_ICON;
			}
		});

		add(IAgentClassDeclarationListAndFactory.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return ActivityAgentClassDeclarationListPanel.AGENT_CLASS_LIST_ICON;
			}
		});

		add(IRessourceClassDeclarationListAndFactory.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return RESSOURCES_CLASS_LIST_ICON;
			}
		});

		add(IAnalysisDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return ANALYSIS_ICON;
			}
		});

		add(ISituationDeclaration.class, new AbstractSeSAmTreeNodeRenderer() {
			public Icon getIcon(
				IObjectTreeNode objectTreeNode,
				Icon defaultIcon,
				boolean expanded,
				boolean leaf,
				boolean disabled) {
				return SITUATION_ICON;
			}
		});

		add(SimulationDeclarationListAndFactory.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				return "<html>"
					+ "Simulations consist of a situation, one or more <br>"
					+ "analysis items and finish rules. Tests extend <br>"
					+ "simulations that invariant rules can be defined.<br>"
					+ "</html>";
			}
		});

		add(ExperimentDeclarationListAndFactory.class, new AbstractSeSAmTreeNodeRenderer() {
			public String getToolTipText(
				IObjectTreeNode objectTreeNode,
				String defaultText,
				String defaultToolTipText) {
				return "<html>"
					+ "Experiment scripts can create simulations, analyse sensitivity to <br>"
					+ "certain parameters or define complex tests. In addition to the <br>"
					+ "default set of SeSAm primitives the following addional primitives <br>"
					+ "are available:<br>"
					+ "<ul> "
					+ "<li> <code> Void Simulate(Simulation) </code>"
					+ "<li> <code> Simulation CreateSimulation(SimulationDeclaration)</code></ul>"
					+ "The first action starts a simulation run and the second function<br>"
					+ "creates a simulation run from a simulation declaration. The initial<br>"
					+ "parameters for the szenario can also be defined.<br> "
					+ "<blockquote>ForElements([x : Double]<br>"
					+ "Simulate(CreateSimulation(TestWorld [maxEnergy = x]))"
					+ "</blockquote>"
					+ "</html>";
			}
		});

	}

	/**
	 * Configures the renderer based on the passed in components. The value is
	 * set from messaging value with toString(). The foreground color is set
	 * based on the selection and the icon is set based on on leaf and
	 * expanded.
	 */
	public Component getTreeCellRendererComponent(
		JTree tree,
		Object value,
		boolean sel,
		boolean expanded,
		boolean leaf,
		int row,
		boolean cellHasFocus) {

		SeSAmDeclarationTreeCellRenderer c =
			(SeSAmDeclarationTreeCellRenderer)super.getTreeCellRendererComponent(
				tree,
				value,
				selected,
				expanded,
				leaf,
				row,
				cellHasFocus);

		if (value instanceof IObjectTreeNode) {
			IObjectTreeNode objectTreeNode = (IObjectTreeNode)value;
			String text =
				(objectTreeNode.getUserObject() instanceof INamed)
					? ((INamed)objectTreeNode.getUserObject()).getName()
					: objectTreeNode.getUserObject() + "";

			//TODOFLAG Blau
			if ((objectTreeNode.getUserObject() instanceof IDeclaration))
				if (((IDeclaration)objectTreeNode.getUserObject()).hasTODOFlag())
					c.setForeground(Color.BLUE);

			c.setText(text);

			Object userObject = ((IObjectTreeNode)value).getUserObject();
			if (userObject instanceof IDocumentable) {
				IDocumentable documentable = (IDocumentable)userObject;
				if (documentable.getDocumentation().length() > 0)
					text =
						"<html><strong>"
							+ NamedCellRendererAdapter.getQuotedString(text)
							+ "</strong><P>"
							+ NamedCellRendererAdapter.getQuotedString(documentable.getDocumentation());
			}

			c.setToolTipText(text);

			applySeSAmTreeCellRenderer(c, objectTreeNode, !tree.isEnabled(), expanded, leaf);
		}

		return c;
	}

	private boolean applySeSAmTreeCellRenderer(
		SeSAmDeclarationTreeCellRenderer c,
		IObjectTreeNode objectTreeNode,
		boolean disabled,
		boolean expanded,
		boolean leaf) {

		Object userObject = objectTreeNode.getUserObject();

		Iterator it = _classList.iterator();

		while (it.hasNext()) {

			Class clazz = (Class)it.next();
			ISeSAmTreeNodeRenderer seSAmTreeNodeRenderer = (ISeSAmTreeNodeRenderer)it.next();

			if (clazz.isInstance(userObject)) {
				c.setText(seSAmTreeNodeRenderer.getText(objectTreeNode, c.getText()));
				c.setToolTipText(
					seSAmTreeNodeRenderer.getToolTipText(objectTreeNode, c.getText(), c.getToolTipText()));

				if (disabled) {
					setDisabledIcon(
						seSAmTreeNodeRenderer.getIcon(
							objectTreeNode,
							c.getDisabledIcon(),
							expanded,
							leaf,
							true));
				} else {
					setIcon(
						seSAmTreeNodeRenderer.getIcon(objectTreeNode, c.getIcon(), expanded, leaf, false));
				}

				return true;
			}
		}

		return false;
	}
}