package de.uniwue.ki.gui.mas.model.tree;
import java.awt.BorderLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;

import javax.swing.*;
import javax.swing.tree.TreePath;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.ISeSAmMainDesktopFrame;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.util.tree.IObjectTreeNode;

/**
 * Creation date: (05.10.2001 09:53:43)
 * @author Christoph Oechslein
 */
public class SeSAmDeclarationTreePanel extends JPanelWithGUISeSAmDeclaration {
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private JTree ivjJTree = null;
	private ISeSAmDeclarationTreeModel _seSAmDeclarationTreeModel;
	private ISeSAmMainDesktopFrame _seSAmMainDesktopFrame;

	private final class IvjEventHandler implements MouseListener, java.beans.PropertyChangeListener {
		public void mouseClicked(MouseEvent e) {
			if ((e.getClickCount() == 2) && (SwingUtilities.isLeftMouseButton(e))) {

				TreePath path = getJTree().getPathForLocation(e.getX(), e.getY());

				if ((path != null)
					&& eventMouseDoubleClicked(
						((IObjectTreeNode) path.getLastPathComponent()).getUserObject(),
						path)) {
					e.consume();
				}
			}
		}
		public void mouseEntered(MouseEvent e) {
		}
		public void mouseExited(MouseEvent e) {
		}
		public void mousePressed(MouseEvent e) {
			if (e.isPopupTrigger()) {
				eventMousePopup(e);
			}
		}
		public void mouseReleased(MouseEvent e) {
			if (e.isPopupTrigger()) {
				eventMousePopup(e);
			}
		}
		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getPropertyName().equals("gUISeSAmDeclaration"))
				if (getGUISeSAmDeclaration() != null) {
					setSeSAmDeclarationTreeModel(getGUISeSAmDeclaration().getSeSAmDeclarationTreeModel());
					expandFirstCol();
				}
		}
	}

	public SeSAmDeclarationTreePanel() {
		super();
		initialize();
	}

	protected boolean eventMouseDoubleClicked(Object anObject, TreePath path) {
		if (getGUISeSAmDeclaration() != null) {
			if (getGUISeSAmDeclaration()
				.getPanelActionFactoryManager()
				.getCanProduceActions(this, anObject)) {

				final AbstractBoundAction propAction =
					getGUISeSAmDeclaration()
						.getPanelActionFactoryManager()
						.getPanelActionFactoryFor(this, anObject)
						.getPropertyAction(this, anObject);

				if (propAction != null) {

					if (getJTree().isExpanded(path)) {
						getJTree().collapsePath(path);
					} else {
						getJTree().expandPath(path);
					}

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							propAction.doit();
						}
					});

					return true;
				}
			}
		}

		return false;
	}

	protected boolean eventMousePopup(Object anObject, int x, int y) {
		if (getGUISeSAmDeclaration() != null) {
			if (getGUISeSAmDeclaration()
				.getPanelActionFactoryManager()
				.getCanProduceActions(this, anObject)) {

				getGUISeSAmDeclaration().getPanelActionFactoryManager().createPopup(this, anObject).show(
					getJTree(),
					x,
					y);
				return true;
			}
		}

		return false;
	}

	protected void eventMousePopup(MouseEvent mouseEvent) {
		int x = mouseEvent.getX();
		int y = mouseEvent.getY();

		TreePath path = getJTree().getPathForLocation(x, y);

		if (path != null) {
			if (eventMousePopup(((IObjectTreeNode) path.getLastPathComponent()).getUserObject(), x, y)) {
				mouseEvent.consume();
			}
		} else {
			JPopupMenu popupComponent = new JPopupMenu();

			popupComponent.add(AbstractBoundAction.createJMenuItem(new AbstractBoundAction("Refresh") {
				public void doit() {
					getSeSAmDeclarationTreeModel().reload();
				}
			}, false));

			popupComponent.show(getJTree(), x, y);
		}
	}

	private JTree getJTree() {
		if (ivjJTree == null) {
			ivjJTree = new JTree();
			ivjJTree.setName("JTree");
			ivjJTree.setCellRenderer(new SeSAmDeclarationTreeCellRenderer());
			ivjJTree.setShowsRootHandles(true);
			ivjJTree.setLargeModel(true);
			ivjJTree.setBounds(0, 0, 78, 72);
			ivjJTree.setRootVisible(false);

			ivjJTree.setTransferHandler(null); // no drag and drop
			ivjJTree.putClientProperty("JTree.lineStyle", "Angled"); // I like this look
			ToolTipManager.sharedInstance().registerComponent(getJTree());
		}
		return ivjJTree;
	}

	protected ISeSAmDeclaration getSeSAmDeclaration() {
		if (getGUISeSAmDeclaration() != null)
			return getGUISeSAmDeclaration().getSeSAmDeclaration();
		else
			return null;
	}

	public ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel() {
		return _seSAmDeclarationTreeModel;
	}

	public ISeSAmMainDesktopFrame getSeSAmMainDesktopFrame() {
		return _seSAmMainDesktopFrame;
	}

	private void initConnections() {
		addPropertyChangeListener(ivjEventHandler);
		getJTree().addMouseListener(ivjEventHandler);
	}

	private void initialize() {
		setName("SeSAmDeclarationTreePanel");
		setLayout(new BorderLayout());
		setSize(221, 338);
		add(new JScrollPane(getJTree()), "Center");
		initConnections();
	}

	public void setSeSAmMainDesktopFrame(ISeSAmMainDesktopFrame newSeSAmMainDesktopFrame) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newSeSAmMainDesktopFrame);

		_seSAmMainDesktopFrame = newSeSAmMainDesktopFrame;
	}

	public void expandFirstCol() {
		for (int i = getJTree().getRowCount() - 1; i >= 0; i--) {
			getJTree().expandRow(i);
		}
	}

	public void setSeSAmDeclarationTreeModel(ISeSAmDeclarationTreeModel newSeSAmDeclarationTreeModel) {
		_seSAmDeclarationTreeModel = newSeSAmDeclarationTreeModel;

		getJTree().setModel(newSeSAmDeclarationTreeModel);
	}
}