package de.uniwue.ki.gui.mas.refactoring;

import de.uniwue.ki.mas.basics.functionArgument.SpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.functionCall.IAtomicFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.refactoring.ExtractMethod;
import de.uniwue.ki.mas.refactoring.ExtractHigherOrderFunctionArgument;

/**
 * @author Michael Scherer
 *
 * To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
public class TestExtractMethod extends AbstractModelTestCase {
	
	private ExtractMethod refactoring;
	
	/*
	 * @see TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		super.setUp();

		// Lade Testmodell
		try {
			loadTestModel(getFullPath("TestExtractMethod.xml"));
		} catch(Throwable ex) {
			fail("Loading model file failed");
		}
	}

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		super.tearDown();		
	}
	
	
	
	
	
	public void testExtractConstant() throws Throwable {
		// Wende Refactoring an
		extractConstant();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutput1.xml"));
	}
	
	private void extractConstant() throws Throwable {
		IComposedFunctionCallInternDeclaration andNode = getAndNode();
		
		ExtractMethod refactoring = 
			(ExtractMethod) getRefactoring(
					andNode.getChild(0), 
					ExtractMethod.class
			);
		assertNotNull(refactoring);

		// Wende Refactoring an
		((ExtractMethod) refactoring).execute(new DummyRefactoringUI(), 
				"ReturnTrue");
	}
	
	
	
	
	
	public void testExtractComposedFunctionAcllInternDeclaration() throws Throwable {
		// Wende Refactoring an
		extractComposedFunctionCallInternDeclaration();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutput2.xml"));
	}
	
	private void extractComposedFunctionCallInternDeclaration() throws Throwable {
		IComposedFunctionCallInternDeclaration equalNode = 
		(IComposedFunctionCallInternDeclaration) getAndNode().getChild(1);
		
		IComposedFunctionCallInternDeclaration plusNode = 
		(IComposedFunctionCallInternDeclaration) equalNode.getChild(0);

		ExtractMethod refactoring = 
		(ExtractMethod) getRefactoring(
				plusNode, 
				ExtractMethod.class
		);
		assertNotNull(refactoring);

		// Wende Refactoring an
		((ExtractMethod) refactoring).execute(new DummyRefactoringUI(), 
				"ComposedFunction");
	}
	
	
	
	
	
	public void testExtractAnonymousFunctionDeclaration() throws Throwable {
		// Wende Refactoring an
		extractAnonymousFunctionCallInternDeclaration();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutput3.xml"));
	}
	
	private void extractAnonymousFunctionCallInternDeclaration() throws Throwable {
		IComposedFunctionCallInternDeclaration equalNode = 
		(IComposedFunctionCallInternDeclaration) getAndNode().getChild(2);
		
		IComposedFunctionCallInternDeclaration doWithNode = 
		(IComposedFunctionCallInternDeclaration) equalNode.getChild(0);
		
		IAtomicFunctionCallInternDeclaration functionReference = 
		(IAtomicFunctionCallInternDeclaration) doWithNode.getChild(0);
		
		SpecificFunctionDeclarationWithArgDeclBinds function =
		(SpecificFunctionDeclarationWithArgDeclBinds) functionReference
		.getArgumentDeclaration()
		.getObject();
		
		ExtractHigherOrderFunctionArgument refactoring = 
		(ExtractHigherOrderFunctionArgument) getRefactoring(
				functionReference, 
				ExtractHigherOrderFunctionArgument.class
		);
		assertNotNull(refactoring);

		// Wende Refactoring an
		((ExtractHigherOrderFunctionArgument) refactoring).execute(new DummyRefactoringUI(), 
				"AnonymousFunction");
	}
	
	
	
	
	
	public void testExtractSimpleFunctionArgumentDeclaration() throws Throwable {
		// Wende Refactoring an
		extractSimpleFunctionWithArgumentDeclaration();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutput4.xml"));
	}
		
	private void extractSimpleFunctionWithArgumentDeclaration() throws Throwable {
		IComposedFunctionCallInternDeclaration equalNode = 
			(IComposedFunctionCallInternDeclaration) getAndNode().getChild(3);
		
		IComposedFunctionCallInternDeclaration doWithNode = 
			(IComposedFunctionCallInternDeclaration) equalNode.getChild(0);
		
		IAtomicFunctionCallInternDeclaration functionReference = 
			(IAtomicFunctionCallInternDeclaration) doWithNode.getChild(0);
		
		SpecificFunctionDeclarationWithArgDeclBinds function =
			(SpecificFunctionDeclarationWithArgDeclBinds) functionReference
			.getArgumentDeclaration()
			.getObject();
		
		// Suche "Input"-Knoten in anonymer Funktion
		IComposedFunctionCallInternDeclaration minusNode = 
		(IComposedFunctionCallInternDeclaration) function
		.getParametricFunctionCallDeclaration()
		.getFunctionCallDeclaration().getFunctionCallInternDeclaration();

		IAtomicFunctionCallInternDeclaration functionArgument = 
		(IAtomicFunctionCallInternDeclaration) minusNode.getChild(0);

		ExtractMethod refactoring = 
		(ExtractMethod) getRefactoring(
				functionArgument, 
				ExtractMethod.class
		);
		assertNotNull(refactoring);

		// Wende Refactoring an
		((ExtractMethod) refactoring).execute(new DummyRefactoringUI(), 
				"SimpleFunctionWithInputArgument");
	}
	
	
	
	
	
	public void testExtractComplexFunctionWithArgumentDeclaration() throws Throwable {
		// Wende Refactoring an
		extractComplexFunctionWithArgumentDeclaration();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutput5.xml"));
	}
	
	private void extractComplexFunctionWithArgumentDeclaration() throws Throwable {
		IComposedFunctionCallInternDeclaration equalNode = 
		(IComposedFunctionCallInternDeclaration) getAndNode().getChild(4);
		
		IComposedFunctionCallInternDeclaration doWithNode = 
		(IComposedFunctionCallInternDeclaration) equalNode.getChild(0);
		
		IAtomicFunctionCallInternDeclaration functionReference = 
		(IAtomicFunctionCallInternDeclaration) doWithNode.getChild(0);
		
		SpecificFunctionDeclarationWithArgDeclBinds function =
		(SpecificFunctionDeclarationWithArgDeclBinds) functionReference
		.getArgumentDeclaration()
		.getObject();
		
		// Suche "Plus"-Knoten in anonymer Funktion
		IComposedFunctionCallInternDeclaration minusNode = 
		(IComposedFunctionCallInternDeclaration) function
		.getParametricFunctionCallDeclaration()
		.getFunctionCallDeclaration().getFunctionCallInternDeclaration();

		IComposedFunctionCallInternDeclaration plusNode = 
		(IComposedFunctionCallInternDeclaration) minusNode.getChild(0);

		ExtractMethod refactoring = 
		(ExtractMethod) getRefactoring(
				plusNode, 
				ExtractMethod.class
		);
		assertNotNull(refactoring);

		// Wende Refactoring an
		((ExtractMethod) refactoring).execute(new DummyRefactoringUI(), 
				"ComplexFunctionWithInputArgument");
	}
	
	
	
	
	
	public void testDoAll() throws Throwable {
		// Wende Refactorings an
		extractConstant();
		extractComposedFunctionCallInternDeclaration();
		extractAnonymousFunctionCallInternDeclaration();
		extractSimpleFunctionWithArgumentDeclaration();
		extractComplexFunctionWithArgumentDeclaration();

		// Teste gendertes Modell
		runTestModel();
		
		// Speichere gendertes Modell zur Verifikation
		saveModelFile(getFullPath("TestOutputDoAll.xml"));
	}
}
