package de.uniwue.ki.gui.mas.sim.analysis;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;

import com.magelang.box.HorizontalStrut;

import de.uniwue.ki.gui.mas.basics.functionCall.FunctionCallDeclarationTreeListPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.IDisposeable;
import de.uniwue.ki.gui.util.bound.BetterBoundJTextField;
import de.uniwue.ki.mas.sim.analysis.IAnalysisItemDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (28.11.2001 17:34:47)
 * @author Christoph Oechslein
 */
final class AnalysisItemDeclarationPanelManager
	extends JPanelWithGUISeSAmDeclaration
	implements IDisposeable {
	private final List _allAnalysisItemDeclarationPanel =
		CollectionsHelper.createList(
			new CSVAnalysisItemDeclarationPanel(),
			new ChartAnalysisItemDeclarationPanel());

	private IAnalysisItemDeclarationPanel _currentAnalysisItemDeclarationPanel;
	private JPanel _cardLayoutPanel;
	private FunctionCallDeclarationTreeListPanel _valueFunctionCallDeclarationPanel;
	private BetterBoundJTextField _betterBoundJTextField;
	private JPanel _namePanel;

	public AnalysisItemDeclarationPanelManager() {
		super();

		userInitialize();
	}

	private void changeToAnalysisItemDeclarationPanel(IAnalysisItemDeclarationPanel analysisItemDeclarationPanel) {

		IAnalysisItemDeclarationPanel oldValue = getCurrentAnalysisItemDeclarationPanel();

		if (!Utils.equals(oldValue, analysisItemDeclarationPanel)) {

			if (analysisItemDeclarationPanel != null) {

				((CardLayout) getCardLayoutPanel().getLayout()).show(
					getCardLayoutPanel(),
					analysisItemDeclarationPanel.getName());

				if (analysisItemDeclarationPanel.hasSpecialSettings()) {
					getCardLayoutPanel().setVisible(true);
				} else {
					getCardLayoutPanel().setVisible(false);
				}

				_currentAnalysisItemDeclarationPanel = analysisItemDeclarationPanel;
			}

			firePropertyChange(
				"currentAnalysisItemDeclarationPanel",
				oldValue,
				_currentAnalysisItemDeclarationPanel);
			firePropertyChange(
				"currentAnalysisItemDeclaration",
				(oldValue == null) ? null : oldValue.getAnalysisItemDeclaration(),
				(_currentAnalysisItemDeclarationPanel == null)
					? null
					: _currentAnalysisItemDeclarationPanel.getAnalysisItemDeclaration());

		}
	}

	private void changeToAnalysisItemDeclarationPanel(IAnalysisItemDeclaration analysisItemDeclaration) {
		Iterator it = _allAnalysisItemDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisItemDeclarationPanel currIAnalysisItemDeclarationPanel =
				(IAnalysisItemDeclarationPanel) it.next();

			if (currIAnalysisItemDeclarationPanel.canEdit(analysisItemDeclaration)) {
				currIAnalysisItemDeclarationPanel.setAnalysisItemDeclaration(analysisItemDeclaration);

				changeToAnalysisItemDeclarationPanel(currIAnalysisItemDeclarationPanel);
			}
		}
	}

	public void dispose() {
		Iterator it = _allAnalysisItemDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisItemDeclarationPanel currIAnalysisItemDeclarationPanel =
				(IAnalysisItemDeclarationPanel) it.next();

			currIAnalysisItemDeclarationPanel.dispose();
		}

		getValueFunctionCallDeclarationPanel().dispose();
		getBetterBoundJTextField().dispose();

		super.dispose();
	}

	private void eventNamedPanelTextChanged(String newValue) {
		if ((getCurrentAnalysisItemDeclaration() != null)
			&& (!Utils.equals(getCurrentAnalysisItemDeclaration().getName(), newValue))) {
			getCurrentAnalysisItemDeclaration().setName(newValue);
		}
	}

	public Iterator getAnalysisItemDeclarationPanels() {
		return _allAnalysisItemDeclarationPanel.iterator();
	}

	private BetterBoundJTextField getBetterBoundJTextField() {
		if (_betterBoundJTextField == null) {
			_betterBoundJTextField = new BetterBoundJTextField();

			_betterBoundJTextField.addPropertyChangeListener("text", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					eventNamedPanelTextChanged((String) evt.getNewValue());
				}
			});
		}

		return _betterBoundJTextField;
	}

	private JPanel getCardLayoutPanel() {
		if (_cardLayoutPanel == null) {
			_cardLayoutPanel = new JPanel();

			CardLayout cardLayout = new CardLayout();

			_cardLayoutPanel.setLayout(cardLayout);

			Iterator it = _allAnalysisItemDeclarationPanel.iterator();
			while (it.hasNext()) {
				IAnalysisItemDeclarationPanel currIAnalysisItemDeclarationPanel =
					(IAnalysisItemDeclarationPanel) it.next();

				_cardLayoutPanel.add(
					(Component) currIAnalysisItemDeclarationPanel,
					currIAnalysisItemDeclarationPanel.getName());
			}

			_cardLayoutPanel.setBorder(BorderFactory.createTitledBorder("Special Settings"));
		}

		return _cardLayoutPanel;
	}

	public IAnalysisItemDeclaration getCurrentAnalysisItemDeclaration() {
		return getCurrentAnalysisItemDeclarationPanel().getAnalysisItemDeclaration();
	}

	public IAnalysisItemDeclarationPanel getCurrentAnalysisItemDeclarationPanel() {
		return _currentAnalysisItemDeclarationPanel;
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return ((IAnalysisItemDeclarationPanel) _allAnalysisItemDeclarationPanel.get(0))
			.getGUISeSAmDeclaration();
	}

	private JPanel getNamePanel() {
		if (_namePanel == null) {
			_namePanel = new JPanel(null);

			_namePanel.setLayout(new BoxLayout(_namePanel, BoxLayout.X_AXIS));

			_namePanel.add(new JLabel("Name:"));
			_namePanel.add(new HorizontalStrut());
			_namePanel.add(getBetterBoundJTextField());

		}

		return _namePanel;
	}

	private FunctionCallDeclarationTreeListPanel getValueFunctionCallDeclarationPanel() {
		if (_valueFunctionCallDeclarationPanel == null) {
			_valueFunctionCallDeclarationPanel = new FunctionCallDeclarationTreeListPanel();
		}

		return _valueFunctionCallDeclarationPanel;
	}

	public void setCurrentAnalysisItemDeclaration(IAnalysisItemDeclaration newAnalysisItemDeclaration) {
		changeToAnalysisItemDeclarationPanel(newAnalysisItemDeclaration);

		if (newAnalysisItemDeclaration != null) {
			getValueFunctionCallDeclarationPanel().setFunctionCallDeclaration(
				newAnalysisItemDeclaration.getValueFunctionCallDeclaration());
			getBetterBoundJTextField().setText(newAnalysisItemDeclaration.getName());
		}
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		Iterator it = _allAnalysisItemDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisItemDeclarationPanel currIAnalysisItemDeclarationPanel =
				(IAnalysisItemDeclarationPanel) it.next();

			currIAnalysisItemDeclarationPanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);
		}

		getValueFunctionCallDeclarationPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	private void userInitialize() {

		setLayout(new GridBagLayout());

		add(
			getNamePanel(),
			new GridBagConstraints(
				0,
				0,
				1,
				1,
				1.0,
				0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL,
				new Insets(0, 0, 4, 0),
				0,
				0));

		add(
			getCardLayoutPanel(),
			new GridBagConstraints(
				0,
				1,
				1,
				1,
				1.0,
				0.0,
				GridBagConstraints.NORTH,
				GridBagConstraints.HORIZONTAL,
				new Insets(0, 0, 0, 0),
				0,
				0));

		JPanel panel = new JPanel(null);

		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

		panel.add(getValueFunctionCallDeclarationPanel());

		panel.setBorder(BorderFactory.createTitledBorder("Value Function Call"));

		add(
			panel,
			new GridBagConstraints(
				0,
				2,
				1,
				1,
				1.0,
				1.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0),
				0,
				0));

	}
}