package de.uniwue.ki.gui.mas.sim.analysis;
import java.awt.Container;
import java.util.Iterator;

import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.chart.renderer.CategoryItemRenderer;
import org.jfree.chart.renderer.XYItemRenderer;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.*;
import de.uniwue.ki.mas.sim.analysis.*;

/**
 * Creation date: (30.11.2001 09:35:05)
 * @author Christoph Oechslein
 */
public class AnalysisListPanel extends JEditMutableListPanel implements IDisposeable {
	private IGUISeSAmDeclaration _guiSeSAmDeclaration;

	public AnalysisListPanel() {
		super();

		setEditable(false);
		setCellRenderer(new NamedCellRendererAdapter(getCellRenderer()));

		// enableDnDSupport();
	}

	public static boolean canEdit(IAnalysis analysis) {
		return (analysis instanceof TableModelAnalysis)
			|| (analysis instanceof BlockChartAnalysis)
			|| (analysis instanceof SeriesChartAnalysis);
	}

	public static void editItem(
		Container parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		IAnalysis analysis) {
			
		if (analysis instanceof TableModelAnalysis) {
			TableModelAnalysisPanel tableModelAnalysisPanel = new TableModelAnalysisPanel();

			tableModelAnalysisPanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);
			tableModelAnalysisPanel.setTableModelAnalysis((TableModelAnalysis) analysis);

			GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addPanel(
				parent,
				analysis.getName(),
				tableModelAnalysisPanel,
				true);

		} else {
			JFreeChart jFreeChart = null;

			if (analysis instanceof BlockChartAnalysis) {
				BlockChartAnalysis blockChartAnalysis = (BlockChartAnalysis) analysis;

				jFreeChart =
					ChartFactory.createBarChart3D(
						analysis.getName(),
						"",
						"",
						blockChartAnalysis,
						PlotOrientation.VERTICAL,
						true,
						true,
						true);
						
				setSeriesPaints(analysis, ((CategoryPlot)jFreeChart.getPlot()).getRenderer());

			} else if (analysis instanceof SeriesChartAnalysis) {
				SeriesChartAnalysis seriesChartAnalysis = (SeriesChartAnalysis) analysis;

				jFreeChart =
					ChartFactory.createLineXYChart(
						analysis.getName(),
						"",
						"",
						seriesChartAnalysis,
						true,
						true,
						true);
						
				setSeriesPaints(analysis, ((XYPlot)jFreeChart.getPlot()).getRenderer());
			}

			if (jFreeChart != null) {
				ChartPanel jfreeChartPanel = new ChartPanel(jFreeChart);


				GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addPanel(
					parent,
					analysis.getName(),
					jfreeChartPanel,
					true);
			}
		}
	}

	private static void setSeriesPaints(IAnalysis analysis, XYItemRenderer renderer) {
		AnalysisItemDeclarationList analysisItemDeclarationList =
			analysis
				.getAnalysisDeclaration()
				.getAnalysisItemDeclarationListAndFactory()
				.getAnalysisItemDeclarationList();

		Iterator it = analysisItemDeclarationList.iterator();

		int i = 0;
		while (it.hasNext()) {
			IAnalysisItemDeclaration element = (IAnalysisItemDeclaration) it.next();
			if (element instanceof ColorAnalysisItemDeclaration) {
				renderer.setSeriesPaint(i, ((ColorAnalysisItemDeclaration) element).getColor());
				i++;
			}
		}
	}

	private static void setSeriesPaints(IAnalysis analysis, CategoryItemRenderer renderer) {
		AnalysisItemDeclarationList analysisItemDeclarationList =
			analysis
				.getAnalysisDeclaration()
				.getAnalysisItemDeclarationListAndFactory()
				.getAnalysisItemDeclarationList();

		Iterator it = analysisItemDeclarationList.iterator();

		int i = 0;
		while (it.hasNext()) {
			IAnalysisItemDeclaration element = (IAnalysisItemDeclaration) it.next();
			if (element instanceof ColorAnalysisItemDeclaration) {
				renderer.setSeriesPaint(i, ((ColorAnalysisItemDeclaration) element).getColor());
				i++;
			}
		}
	}
	public void editItem(Object obj) {
		editItem(this, getGUISeSAmDeclaration(), (IAnalysis) obj);
	}

	public AnalysisList getAnalysisList() {
		return (AnalysisList) getModel();
	}

	public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	public void setAnalysisList(AnalysisList analysisList) {
		setModel(analysisList);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newGUISeSAmDeclaration) {
		_guiSeSAmDeclaration = newGUISeSAmDeclaration;
	}
}