package de.uniwue.ki.gui.mas.sim.analysis;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;

import com.ibm.easyswing.BoundJComboBox;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.IDisposeable;
import de.uniwue.ki.mas.sim.analysis.IAnalysisTypeDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (28.11.2001 17:34:47)
 * @author Christoph Oechslein
 */
final class AnalysisTypeDeclarationPanelManager
	extends JPanelWithGUISeSAmDeclaration
	implements IDisposeable {

	private final List _allAnalysisTypeDeclarationPanel =
		CollectionsHelper.createList(
			new CSVAnalysisTypeDeclarationPanel(),
			new BlockChartAnalysisTypeDeclarationPanel(),
			new SeriesChartAnalysisTypeDeclarationPanel(),
			new TableModelAnalysisTypeDeclarationPanel());

	private IAnalysisTypeDeclarationPanel _currentAnalysisTypeDeclarationPanel;
	private JPanel _cardLayoutPanel;
	private JPanel _comboBoxPanel;
	private BoundJComboBox _boundJComboBox;

	public AnalysisTypeDeclarationPanelManager() {
		super();

		userInitialize();
	}

	private void changeToAnalysisTypeDeclarationPanel(IAnalysisTypeDeclarationPanel analysisTypeDeclarationPanel) {

		IAnalysisTypeDeclarationPanel oldValue = getCurrentAnalysisTypeDeclarationPanel();

		if (!Utils.equals(oldValue, analysisTypeDeclarationPanel)) {

			if (analysisTypeDeclarationPanel != null) {
				((CardLayout)getCardLayoutPanel().getLayout()).show(
					getCardLayoutPanel(),
					analysisTypeDeclarationPanel.getName());

				_currentAnalysisTypeDeclarationPanel = analysisTypeDeclarationPanel;
			}

			firePropertyChange(
				"currentAnalysisTypeDeclarationPanel",
				oldValue,
				_currentAnalysisTypeDeclarationPanel);
			firePropertyChange(
				"currentAnalysisTypeDeclaration",
				(oldValue == null) ? null : oldValue.getAnalysisTypeDeclaration(),
				(_currentAnalysisTypeDeclarationPanel == null)
					? null
					: _currentAnalysisTypeDeclarationPanel.getAnalysisTypeDeclaration());

		}
	}

	private void changeToAnalysisTypeDeclarationPanel(IAnalysisTypeDeclaration analysisTypeDeclaration) {
		Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
				(IAnalysisTypeDeclarationPanel)it.next();

			if (currIAnalysisTypeDeclarationPanel.canEdit(analysisTypeDeclaration)) {
				currIAnalysisTypeDeclarationPanel.setAnalysisTypeDeclaration(analysisTypeDeclaration);

				changeToAnalysisTypeDeclarationPanel(currIAnalysisTypeDeclarationPanel);
			}
		}
	}

	private void changeToAnalysisTypeDeclarationPanel(String name) {
		Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
				(IAnalysisTypeDeclarationPanel)it.next();

			if (name.equals(currIAnalysisTypeDeclarationPanel.getAnalysisTypeDeclaration().getName())) {
				changeToAnalysisTypeDeclarationPanel(currIAnalysisTypeDeclarationPanel);
			}
		}
	}

	public void dispose() {
		Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
				(IAnalysisTypeDeclarationPanel)it.next();

			currIAnalysisTypeDeclarationPanel.dispose();
		}

		super.dispose();
	}

	public Iterator getAnalysisTypeDeclarationPanels() {
		return _allAnalysisTypeDeclarationPanel.iterator();
	}

	private JPanel getCardLayoutPanel() {
		if (_cardLayoutPanel == null) {
			_cardLayoutPanel = new JPanel();

			CardLayout cardLayout = new CardLayout();

			_cardLayoutPanel.setLayout(cardLayout);

			Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
			while (it.hasNext()) {
				IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
					(IAnalysisTypeDeclarationPanel)it.next();

				_cardLayoutPanel.add(
					(Component)currIAnalysisTypeDeclarationPanel,
					currIAnalysisTypeDeclarationPanel.getName());
			}
		}

		return _cardLayoutPanel;
	}

	private JComboBox getComboBox() {
		if (_boundJComboBox == null) {
			DefaultComboBoxModel defaultComboBoxModel = new DefaultComboBoxModel();
			_boundJComboBox = new BoundJComboBox(defaultComboBoxModel);

			Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
			while (it.hasNext()) {
				IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
					(IAnalysisTypeDeclarationPanel)it.next();

				defaultComboBoxModel.addElement(
					currIAnalysisTypeDeclarationPanel.getAnalysisTypeDeclaration().getName());
			}

			defaultComboBoxModel.setSelectedItem(null);

			_boundJComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					changeToAnalysisTypeDeclarationPanel(
						(String) ((BoundJComboBox)e.getSource()).getSelectedItem());
				}
			});
		}

		return _boundJComboBox;
	}

	private JPanel getComboBoxPanel() {
		if (_comboBoxPanel == null) {
			_comboBoxPanel = new JPanel(null);

			_comboBoxPanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT, 0, 0));

			_comboBoxPanel.add(getComboBox());

		}

		return _comboBoxPanel;
	}

	public IAnalysisTypeDeclaration getCurrentAnalysisTypeDeclaration() {
		return getCurrentAnalysisTypeDeclarationPanel().getAnalysisTypeDeclaration();
	}

	public IAnalysisTypeDeclarationPanel getCurrentAnalysisTypeDeclarationPanel() {
		return _currentAnalysisTypeDeclarationPanel;
	}

	public void setCurrentAnalysisTypeDeclaration(IAnalysisTypeDeclaration newAnalysisTypeDeclaration) {
		changeToAnalysisTypeDeclarationPanel(newAnalysisTypeDeclaration);

		_boundJComboBox.setSelectedItem(newAnalysisTypeDeclaration.getName());
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		Iterator it = _allAnalysisTypeDeclarationPanel.iterator();
		while (it.hasNext()) {
			IAnalysisTypeDeclarationPanel currIAnalysisTypeDeclarationPanel =
				(IAnalysisTypeDeclarationPanel)it.next();

			currIAnalysisTypeDeclarationPanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);
		}
	}

	private void userInitialize() {

		setLayout(new GridBagLayout());

		add(
			getComboBoxPanel(),
			new GridBagConstraints(
				0,
				0,
				1,
				1,
				1.0,
				0.0,
				GridBagConstraints.NORTHWEST,
				GridBagConstraints.NONE,
				new java.awt.Insets(0, 0, 4, 0),
				0,
				0));

		// add(new BoxLayoutVerticalStrut());

		add(
			getCardLayoutPanel(),
			new GridBagConstraints(
				0,
				1,
				1,
				1,
				1.0,
				0.0,
				GridBagConstraints.NORTHWEST,
				GridBagConstraints.NONE,
				new java.awt.Insets(0, 0, 0, 0),
				0,
				0));

		getComboBox().setSelectedIndex(0);

	}
}