package de.uniwue.ki.gui.mas.sim.analysis;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.ExtensionsFileFilter;
import de.uniwue.ki.gui.util.bound.BetterBoundJTextField;
import de.uniwue.ki.mas.sim.analysis.*;

/**
 * Creation date: (28.11.2001 17:50:02)
 * @author Christoph Oechslein
 */

public class CSVAnalysisTypeDeclarationPanel
	extends JPanelWithGUISeSAmDeclaration
	implements IAnalysisTypeDeclarationPanel {

	private static class AdvancedSettingsPanel extends JPanel {
		private JLabel _separatorLabel = null;
		private BetterBoundJTextField _separatorTextField = null;
		private JLabel _timecolumnLabel = null;
		private BetterBoundJTextField _timecolumnTextField = null;

		public AdvancedSettingsPanel() {
			GridBagConstraints constraintsSeparatorLabel = new GridBagConstraints();
			constraintsSeparatorLabel.gridx = 0;
			constraintsSeparatorLabel.gridy = 0;
			constraintsSeparatorLabel.insets = new Insets(0, 0, 0, 4);
			add(getSeparatorLabel(), constraintsSeparatorLabel);

			GridBagConstraints constraintsSeparatorTextField = new GridBagConstraints();
			constraintsSeparatorTextField.gridx = 1;
			constraintsSeparatorTextField.gridy = 0;
			constraintsSeparatorTextField.anchor = GridBagConstraints.WEST;
			//constraintsSeparatorTextField.fill = GridBagConstraints.HORIZONTAL;
			//constraintsSeparatorTextField.weightx = 0.5;
			add(getSeparatorTextField(), constraintsSeparatorTextField);

			GridBagConstraints constraintsTimeColumnLabel = new GridBagConstraints();
			constraintsTimeColumnLabel.gridx = 2;
			constraintsTimeColumnLabel.gridy = 0;
			constraintsTimeColumnLabel.insets = new Insets(0, 20, 0, 4);
			add(getTimeColumnNameLabel(), constraintsTimeColumnLabel);

			GridBagConstraints constraintsTimecolumnTextField = new GridBagConstraints();
			constraintsTimecolumnTextField.gridx = 3;
			constraintsTimecolumnTextField.gridy = 0;
			constraintsTimecolumnTextField.anchor = GridBagConstraints.WEST;
			//constraintsTimecolumnTextField.fill = GridBagConstraints.HORIZONTAL;
			//constraintsTimecolumnTextField.weightx = 0.5;
			add(getTimeColumnNameTextField(), constraintsTimecolumnTextField);

			getSeparatorTextField().addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ((evt.getSource() == getSeparatorTextField())
						&& evt.getPropertyName().equals("text")
						&& (getSeparatorTextField().getText().length() == 0)) {

						String oldText = (String) evt.getOldValue();
						if ((oldText != null) && (oldText.length() > 0)) {
							getSeparatorTextField().setText(oldText);
						}
					}
				}
			});
		}

		private BetterBoundJTextField getSeparatorTextField() {
			if (_separatorTextField == null) {

				_separatorTextField = new BetterBoundJTextField();
				_separatorTextField.setEditable(true);
				_separatorTextField.setMinimumSize(new Dimension(75, 20));
				_separatorTextField.setColumns(4);
				_separatorTextField.setHorizontalAlignment(SwingConstants.CENTER);

				_separatorTextField.setToolTipText("Use \\t for tab.");
			}

			return _separatorTextField;
		}

		private JLabel getSeparatorLabel() {
			if (_separatorLabel == null) {
				_separatorLabel = new JLabel();
				_separatorLabel.setToolTipText("String separating entries in output-file");
				_separatorLabel.setText("Separator:");
			}

			return _separatorLabel;
		}

		private JLabel getTimeColumnNameLabel() {
			if (_timecolumnLabel == null) {
				_timecolumnLabel = new JLabel("First column:");
			}

			return _timecolumnLabel;

		}

		private BetterBoundJTextField getTimeColumnNameTextField() {
			if (_timecolumnTextField == null) {
				_timecolumnTextField = new BetterBoundJTextField();
				_timecolumnTextField.setEditable(true);
				_timecolumnTextField.setMinimumSize(new Dimension(75, 20));
				_timecolumnTextField.setColumns(10);
			}

			return _timecolumnTextField;
		}
	}

	private static final String TOKEN_REPLACER_LEGEND =
		CSVAnalysis.TOKEN_REPLACER.getTokenLegend(
			"E.g.: \"c:\\protokolls\\%E_%d_m_%Y\\protokoll_%H_%S_%s.csv\"<br>"
				+ "would be expanded to \"c:\\protokolls\\my simulation_01_01_2002\\protokoll_20_15_01.csv\"<p>");

	private BetterBoundJTextField _fileStringTextField = null;
	private EventHandler _eventHandler = new EventHandler();
	private JButton _browseButton = null;
	private JButton _legendButton = null;
	private JLabel _fileLabel = null;
	private CSVAnalysisTypeDeclaration _csvAnalysisTypeDeclaration = null;

	private JButton _advancedButton = null;

	class EventHandler implements ActionListener, PropertyChangeListener {

		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == getBrowseButton()) {
				eventBrowseButtonPressed();

			} else if (e.getSource() == getLegendButton()) {
				GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
					CSVAnalysisTypeDeclarationPanel.this,
					"Possible Tokens",
					new JOptionPane(TOKEN_REPLACER_LEGEND, JOptionPane.INFORMATION_MESSAGE));

			} else if (e.getSource() == getAdvancedButton()) {
				eventAdvancedButtonPressed();
			}
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if ((getCSVAnalysisTypeDeclaration() != null)
				&& (evt.getSource() == getFileStringTextField())
				&& evt.getPropertyName().equals("text")) {

				getCSVAnalysisTypeDeclaration().setFileString(getFileStringTextField().getText());
			}
		}
	}

	public CSVAnalysisTypeDeclarationPanel() {
		super();
		initialize();
	}

	public boolean canEdit(IAnalysisTypeDeclaration newAnalysisTypeDeclaration) {
		return newAnalysisTypeDeclaration instanceof CSVAnalysisTypeDeclaration;
	}

	protected void eventBrowseButtonPressed() {
		JFileChooser fileChooser = new JFileChooser(getCSVAnalysisTypeDeclaration().getFileString());

		fileChooser.setSelectedFile(new File(getCSVAnalysisTypeDeclaration().getFileString()));

		fileChooser.addChoosableFileFilter(
			new ExtensionsFileFilter("Comma seperated (*.csv)", new String[] { "csv" }));

		if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
			return;

		String selectedFile = fileChooser.getSelectedFile().toString();

		if (!selectedFile.endsWith(".csv")) {
			selectedFile += ".csv";
		}

		getCSVAnalysisTypeDeclaration().setFileString(selectedFile);
	}

	protected void eventAdvancedButtonPressed() {
		if (getCSVAnalysisTypeDeclaration() != null) {

			AdvancedSettingsPanel advancedSettingsPanel = new AdvancedSettingsPanel();

			advancedSettingsPanel.getTimeColumnNameTextField().setText(
				getCSVAnalysisTypeDeclaration().getTimeColumnName());

			advancedSettingsPanel.getSeparatorTextField().setText(
				getCSVAnalysisTypeDeclaration().getSeparatorString());

			if (JOptionPane
				.showConfirmDialog(
					this,
					advancedSettingsPanel,
					"Advanced Settings for " + getCSVAnalysisTypeDeclaration().getName(),
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.PLAIN_MESSAGE)
				== JOptionPane.OK_OPTION) {

				getCSVAnalysisTypeDeclaration().setTimeColumnName(
					advancedSettingsPanel.getTimeColumnNameTextField().getText());
				getCSVAnalysisTypeDeclaration().setSeparatorString(
					advancedSettingsPanel.getSeparatorTextField().getText());
			}
		}
	}

	public IAnalysisTypeDeclaration getAnalysisTypeDeclaration() {
		return getCSVAnalysisTypeDeclaration();
	}

	private BetterBoundJTextField getFileStringTextField() {
		if (_fileStringTextField == null) {
			_fileStringTextField = new BetterBoundJTextField();
			_fileStringTextField.setEditable(true);
			_fileStringTextField.setMinimumSize(new Dimension(330, 20));
			_fileStringTextField.setColumns(40);
		}

		return _fileStringTextField;
	}

	private CSVAnalysisTypeDeclaration getCSVAnalysisTypeDeclaration() {
		if (_csvAnalysisTypeDeclaration == null) {
			_csvAnalysisTypeDeclaration = new CSVAnalysisTypeDeclaration();
		}
		return _csvAnalysisTypeDeclaration;
	}

	private JButton getAdvancedButton() {
		if (_advancedButton == null) {
			_advancedButton = new JButton();
			_advancedButton.setText("Advanced ...");
		}

		return _advancedButton;
	}

	private JButton getBrowseButton() {
		if (_browseButton == null) {
			_browseButton = new JButton();
			_browseButton.setText("Browse ...");
		}

		return _browseButton;
	}

	private JButton getLegendButton() {

		if (_legendButton == null) {

			_legendButton = new JButton();
			_legendButton.setToolTipText("Legend of possible Tokens");
			_legendButton.setText("?");
		}

		return _legendButton;
	}

	private JLabel getFileLabel() {

		if (_fileLabel == null) {

			_fileLabel = new JLabel();
			_fileLabel.setText("File:");
		}

		return _fileLabel;
	}

	private void initConnections() {

		getBrowseButton().addActionListener(_eventHandler);
		getLegendButton().addActionListener(_eventHandler);
		getAdvancedButton().addActionListener(_eventHandler);

		getFileStringTextField().addPropertyChangeListener(_eventHandler);
	}

	private void initialize() {
		setName("CSVAnalysisDeclarationTypePanel");
		setLayout(new GridBagLayout());

		GridBagConstraints constraintsJLabel2 = new GridBagConstraints();
		constraintsJLabel2.gridx = 0;
		constraintsJLabel2.gridy = 0;
		constraintsJLabel2.insets = new Insets(0, 0, 0, 4);
		constraintsJLabel2.anchor = GridBagConstraints.WEST;
		add(getFileLabel(), constraintsJLabel2);

		GridBagConstraints constraintsFileStringTextField = new GridBagConstraints();
		constraintsFileStringTextField.gridx = 1;
		constraintsFileStringTextField.gridy = 0;
		constraintsFileStringTextField.fill = GridBagConstraints.HORIZONTAL;
		constraintsFileStringTextField.weightx = 1.0;
		add(getFileStringTextField(), constraintsFileStringTextField);

		GridBagConstraints constraintsJButtonLegend = new GridBagConstraints();
		constraintsJButtonLegend.gridx = 4;
		constraintsJButtonLegend.gridy = 0;
		constraintsJButtonLegend.insets = new Insets(0, 2, 0, 0);
		add(getLegendButton(), constraintsJButtonLegend);

		GridBagConstraints constraintsJButton1 = new GridBagConstraints();
		constraintsJButton1.gridx = 5;
		constraintsJButton1.gridy = 0;
		constraintsJButton1.insets = new Insets(0, 4, 0, 0);
		add(getBrowseButton(), constraintsJButton1);

		GridBagConstraints constraintsJAdvancedButton = new GridBagConstraints();
		constraintsJAdvancedButton.gridx = 6;
		constraintsJAdvancedButton.gridy = 0;
		constraintsJAdvancedButton.insets = new Insets(0, 7, 0, 0);
		add(getAdvancedButton(), constraintsJAdvancedButton);

		initConnections();

	}

	public void setAnalysisTypeDeclaration(IAnalysisTypeDeclaration newAnalysisTypeDeclaration) {
		setCSVAnalysisTypeDeclaration((CSVAnalysisTypeDeclaration) newAnalysisTypeDeclaration);
	}

	private void setCSVAnalysisTypeDeclaration(CSVAnalysisTypeDeclaration newValue) {
		if (_csvAnalysisTypeDeclaration != newValue) {
			_csvAnalysisTypeDeclaration = newValue;

			if (newValue != null) {
				getFileStringTextField().setText(newValue.getFileString());
			}
		}
	}
}
