package de.uniwue.ki.gui.mas.sim.experiment;
import java.awt.Container;
import java.util.*;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.sim.simulation.ThreadedSimulationPanel;
import de.uniwue.ki.gui.mas.sim.simulation.ThreadedSimulationWithNamePanel;
import de.uniwue.ki.gui.mas.util.IActionFactory;
import de.uniwue.ki.gui.mas.util.declaration.action.PropertiesDeclarationAction;
import de.uniwue.ki.gui.mas.util.declaration.action.RemoveMutableListModelAction;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.sim.experiment.*;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (22.11.2001 09:48:43)
 * @author Chripauseh Oechslein
 */
public class ExperimentActionFactory implements IActionFactory {
	private static final Icon _startIcon = SwingMoreUtilities.getJavaSunIcon(16, "media/Play16.gif");
	private static final Icon _pauseIcon = SwingMoreUtilities.getJavaSunIcon(16, "media/Pause16.gif");

	private final IGUISeSAmDeclaration _guiSeSAmDeclaration;

	public ExperimentActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super();

		_guiSeSAmDeclaration = guiSeSAmDeclaration;
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		return anObject instanceof IExperiment;
	}

	private JMenu getCurrentSimJMenu(Container parent, IThreadedSimulation threadedSimulation) {

		JMenu menu = new JMenu("Current Sim.");

		menu.add(
			AbstractBoundAction.createJMenuItem(
				getSimulationPanelAction(
					parent,
					threadedSimulation,
					"Properties",
					PropertiesDeclarationAction.PROPERTIES_ICON),
				true));

		menu.add(
			AbstractBoundAction.createJMenuItem(
				createShowAnalysisPanelAction(
					parent,
					threadedSimulation,
					"Show Analysis",
					ThreadedSimulationPanel.ANALYSIS_ICON)));

		menu.addSeparator();

		menu.add(AbstractBoundAction.createJMenuItem(createPauseOrStartAction(threadedSimulation)));

		menu.addSeparator();

		menu.add(AbstractBoundAction.createJMenuItem(new InspectAction(threadedSimulation)));

		return menu;
	}

	private Action createPauseOrStartAction(final IStartPauseFinishInThreadAble threadAble) {
		if (threadAble.isRunning()) {
			return new AbstractBoundAction("Pause", _pauseIcon) {
				public void doit() {
					threadAble.pauseInThread();
				}
			};

		} else {
			return new AbstractBoundAction("Start", _startIcon) {
				public void doit() {
					if (!threadAble.isFinished() && !threadAble.isFinishedWithErrors())
						threadAble.startInThread();
				}
			};
		}
	}

	private AbstractBoundAction createExperimentPanelAction(
		final Container parent,
		final IExperiment experiment,
		String actionName,
		Icon actionIcon) {

		return new AbstractBoundAction(actionName, actionIcon) {
			public void doit() {
				ExperimentPanel experimentPanel = new ExperimentPanel();

				experimentPanel.setExperiment(experiment);
				experimentPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

				GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
					parent,
					"Show Simulation Runs",
					experimentPanel,
					true);
			}
		};
	}

	private AbstractBoundAction createShowAnalysisPanelAction(
		final Container parent,
		final ISimulation simulation,
		String actionName,
		Icon actionIcon) {

		return new AbstractBoundAction(actionName, actionIcon) {
			public void doit() {
				ThreadedSimulationPanel.showAnalysisPanel(parent, simulation, _guiSeSAmDeclaration);
			}
		};
	}

	protected final IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	public Iterator getImportantActions(final Container parent, Object anObject) {

		final IExperiment experiment = (IExperiment) anObject;

		List actions = new LinkedList();

		if (experiment instanceof OneSimulationExperiment) {
			//			actions.add(
			//				getExperimentPanelAction(
			//					parent,
			//					experiment,
			//					"Experiment",
			//					PropertiesDeclarationAction.PROPERTIES_ICON));
			actions.add(
				createShowAnalysisPanelAction(
					parent,
					((OneSimulationExperiment) experiment).getThreadedSimulation(),
					"Show Analysis",
					ThreadedSimulationPanel.ANALYSIS_ICON));
		} else if (
			(experiment.getCurrentSimulation() != null)
				&& (experiment.getCurrentSimulation() instanceof IThreadedSimulation)) {
			actions.add(getCurrentSimJMenu(parent, (IThreadedSimulation) experiment.getCurrentSimulation()));
		}

		actions.add(null);

		actions.add(createPauseOrStartAction(experiment));

		return actions.iterator();
	}

	public Iterator getMoreActions(Container parent, Object anObject) {
		final IExperiment experiment = (IExperiment) anObject;

		return CollectionsHelper
			.createIterator(new AbstractBoundAction("Remove", RemoveMutableListModelAction.REMOVE_ICON) {
			public void doit() {
				((ExperimentListAndFactory) getGUISeSAmDeclaration()
					.getSeSAmDeclarationTreeModel()
					.getUserObjectParentOf(experiment))
					.getExperimentList()
					.remove(experiment);
				experiment.remove();
			}
		}, new InspectAction(anObject));
	}

	public AbstractBoundAction getPropertyAction(Container parent, Object anObject) {
		IExperiment experiment = (IExperiment) anObject;

		if (experiment instanceof OneSimulationExperiment) {
			return getSimulationPanelAction(
				parent,
				((OneSimulationExperiment) experiment).getThreadedSimulation(),
				"Properties",
				PropertiesDeclarationAction.PROPERTIES_ICON);

		} else {
			return createExperimentPanelAction(
				parent,
				experiment,
				"Properties",
				PropertiesDeclarationAction.PROPERTIES_ICON);
		}
	}

	private AbstractBoundAction getSimulationPanelAction(
		final Container parent,
		final IThreadedSimulation simulation,
		String actionName,
		Icon actionIcon) {

		return new AbstractBoundAction(actionName, actionIcon) {
			public void doit() {
				ThreadedSimulationWithNamePanel simulationPanel = new ThreadedSimulationWithNamePanel();

				simulationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
				simulationPanel.setThreadedSimulation(simulation);
				
				GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
					parent,
					"Show Simulation",
					simulationPanel,
					true);
			}
		};

	}
}