package de.uniwue.ki.gui.mas.sim.experiment;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.mas.basics.functionCall.FunctionCallDeclarationTreeListPanel;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.gui.util.ExtensionsFileFilter;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.bound.BetterBoundJTextField;
import de.uniwue.ki.mas.sim.experiment.AbstractExperiment;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
/**
 * Creation date: (28.01.2002 18:29:56)
 * 
 * @author Christoph Oechslein
 */
public class ExperimentDeclarationPanel extends AbstractDeclarationPanel {
    private static final String TOKEN_REPLACER_LEGEND =
        AbstractExperiment.TOKEN_REPLACER.getTokenLegend(null);
    private IExperimentDeclaration _experimentDeclarationIntern = null;
    private final EventHandler _eventHandler = new EventHandler();
    private FunctionCallDeclarationTreeListPanel _functionCallDeclarationTreeListPanel1 = null;
    private JLabel _jLabelConfigFileName = null;
    private JPanel _jPanelConfigFileName = null;
    private BetterBoundJTextField _jTextFieldConfigFileName = null;
    private JButton _jButtonLegend = null;
    private JButton _jButtonBrowse = null;
    private JCheckBox _checkBoxTest;

    private final class EventHandler implements ActionListener, PropertyChangeListener, ChangeListener {

        public void actionPerformed(ActionEvent e) {
            if (e.getSource() == ExperimentDeclarationPanel.this.getJButtonLegend())
                connEtoC1(e);
            if (e.getSource() == ExperimentDeclarationPanel.this.getJButtonBrowse())
                eventShowFileChooser(e);
        }

        public void propertyChange(PropertyChangeEvent evt) {
            if (evt.getSource() == ExperimentDeclarationPanel.this.getJTextFieldConfigFileName()
                && (evt.getPropertyName().equals("text")))
                connPtoP4SetSource();
        }

        public void stateChanged(ChangeEvent e) {

            if (e.getSource() == ExperimentDeclarationPanel.this.getExperimentDeclaration()) {
                // connects the textField to the Filechooser --steffen
                getJTextFieldConfigFileName().setText(
                    ExperimentDeclarationPanel.this.getExperimentDeclaration().getConfigurationFileName());
            }
        }
    }

    public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
        super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

        getFunctionCallDeclarationTreeListPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
    }

    public ExperimentDeclarationPanel() {
        super();
        initialize();
    }

    private void connEtoC1(ActionEvent arg1) {
        eventQuestionMarkButtonPressed();

    }

    private void eventShowFileChooser(ActionEvent arg1) {
        showFileChooser();
    }

    private void connPtoP1SetTarget() {
        if ((getExperimentDeclaration() != null)) {
            getFunctionCallDeclarationTreeListPanel().setFunctionCallDeclaration(
                getExperimentDeclaration().getSimulationActionFunctionCallDeclaration());
        }

    }

    private void connPtoP4SetSource() {
        if ((getExperimentDeclaration() != null)) {
            getExperimentDeclaration().setConfigurationFileName(getJTextFieldConfigFileName().getText());
        }

    }

    private void connPtoP4SetTarget() {
        if ((getExperimentDeclaration() != null)) {
            getJTextFieldConfigFileName().setText(getExperimentDeclaration().getConfigurationFileName());
        }

    }

    protected void showFileChooser() {
        JFileChooser fileChooser = new JFileChooser(getExperimentDeclaration().getConfigurationFileName());

        fileChooser.setSelectedFile(new File(getExperimentDeclaration().getConfigurationFileName()));

        fileChooser.addChoosableFileFilter(
            new ExtensionsFileFilter("Comma seperated (*.csv)", new String[] { "csv" }));

        if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
            return;

        String selectedFile = fileChooser.getSelectedFile().toString();

        if (!selectedFile.endsWith(".csv")) {
            selectedFile += ".csv";
        }

        getExperimentDeclaration().setConfigurationFileName(selectedFile);
    }

    protected void eventQuestionMarkButtonPressed() {
        JOptionPane optionPane = new JOptionPane(TOKEN_REPLACER_LEGEND, JOptionPane.INFORMATION_MESSAGE);
        GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addModalPanel(
            this,
            "Possible Tokens",
            optionPane);
    }

    public final IDeclaration getDeclaration() {
        return getExperimentDeclaration();
    }

    public IExperimentDeclaration getExperimentDeclaration() {
        return _experimentDeclarationIntern;
    }

    private FunctionCallDeclarationTreeListPanel getFunctionCallDeclarationTreeListPanel() {
        if (_functionCallDeclarationTreeListPanel1 == null) {
            _functionCallDeclarationTreeListPanel1 = new FunctionCallDeclarationTreeListPanel();
            _functionCallDeclarationTreeListPanel1.setName("FunctionCallDeclarationTreeListPanel1");
        }
        return _functionCallDeclarationTreeListPanel1;
    }

    public void dispose() {
        getFunctionCallDeclarationTreeListPanel().dispose();

        super.dispose();
    }

    private JButton getJButtonBrowse() {
        if (_jButtonBrowse == null) {
            _jButtonBrowse = new JButton();
            _jButtonBrowse.setText("Browse ...");

        }
        return _jButtonBrowse;
    }

    private JButton getJButtonLegend() {
        if (_jButtonLegend == null) {
            _jButtonLegend = new JButton();
            _jButtonLegend.setToolTipText("Legend of possible Tokens");
            _jButtonLegend.setText("?");
        }
        return _jButtonLegend;
    }

    private JLabel getJLabelConfigFileName() {
        if (_jLabelConfigFileName == null) {
            _jLabelConfigFileName = new JLabel();
            _jLabelConfigFileName.setText("Configuration File:");

        }
        return _jLabelConfigFileName;
    }

    private JPanel getJPanelConfigFileName() {
        if (_jPanelConfigFileName == null) {
            _jPanelConfigFileName = new JPanel();
            _jPanelConfigFileName.setLayout(new GridBagLayout());

            GridBagConstraints constraintsJLabelConfigFileName = new GridBagConstraints();
            constraintsJLabelConfigFileName.gridx = 0;
            constraintsJLabelConfigFileName.gridy = 0;
            constraintsJLabelConfigFileName.insets = new Insets(0, 0, 0, 4);
            getJPanelConfigFileName().add(getJLabelConfigFileName(), constraintsJLabelConfigFileName);

            GridBagConstraints constraintsJTextFieldConfigFileName = new GridBagConstraints();
            constraintsJTextFieldConfigFileName.gridx = 1;
            constraintsJTextFieldConfigFileName.gridy = 0;
            constraintsJTextFieldConfigFileName.fill = GridBagConstraints.HORIZONTAL;
            constraintsJTextFieldConfigFileName.weightx = 1.0;
            constraintsJTextFieldConfigFileName.insets = new Insets(0, 0, 0, 10);
            getJPanelConfigFileName().add(getJTextFieldConfigFileName(), constraintsJTextFieldConfigFileName);

            GridBagConstraints constraintsJButtonLegend = new GridBagConstraints();
            constraintsJButtonLegend.gridx = 2;
            constraintsJButtonLegend.gridy = 0;
            constraintsJButtonLegend.anchor = GridBagConstraints.EAST;
            constraintsJButtonLegend.insets = new Insets(0, 0, 0, 3);
            getJPanelConfigFileName().add(getJButtonLegend(), constraintsJButtonLegend);

            GridBagConstraints constraintsJButtonBrowse = new GridBagConstraints();
            constraintsJButtonBrowse.gridx = 3;
            constraintsJButtonBrowse.gridy = 0;
            constraintsJButtonBrowse.anchor = GridBagConstraints.EAST;
            constraintsJButtonBrowse.insets = new Insets(0, 0, 0, 5);
            getJPanelConfigFileName().add(getJButtonBrowse(), constraintsJButtonBrowse);

        }
        return _jPanelConfigFileName;
    }

    private BetterBoundJTextField getJTextFieldConfigFileName() {
        if (_jTextFieldConfigFileName == null) {
            _jTextFieldConfigFileName = new BetterBoundJTextField();
            _jTextFieldConfigFileName.setName("JTextFieldConfigFileName");
            _jTextFieldConfigFileName.setColumns(10);

        }
        return _jTextFieldConfigFileName;
    }

    private void initConnections() {
        getJTextFieldConfigFileName().addPropertyChangeListener(_eventHandler);
        getJButtonLegend().addActionListener(_eventHandler);
        getJButtonBrowse().addActionListener(_eventHandler);
    }

    private void initialize() {
        setLayout(new GridBagLayout());
        setSize(435, 445);

        GridBagConstraints constraintsJPanelConfigFileName = new GridBagConstraints();
        constraintsJPanelConfigFileName.gridx = 0;
        constraintsJPanelConfigFileName.gridy = 0;
        constraintsJPanelConfigFileName.weightx = 1.0;
        constraintsJPanelConfigFileName.fill = GridBagConstraints.HORIZONTAL;
        add(getJPanelConfigFileName(), constraintsJPanelConfigFileName);

        GridBagConstraints constraintsJCheckBoxTest = new GridBagConstraints();
        constraintsJCheckBoxTest.gridx = 1;
        constraintsJCheckBoxTest.gridy = 0;
        constraintsJCheckBoxTest.anchor = GridBagConstraints.EAST;
        add(getCheckBoxTest(), constraintsJCheckBoxTest);

        GridBagConstraints constraintsFunctionCallDeclarationTreeListPanel1 = new GridBagConstraints();
        constraintsFunctionCallDeclarationTreeListPanel1.gridx = 0;
        constraintsFunctionCallDeclarationTreeListPanel1.gridy = 1;
        constraintsFunctionCallDeclarationTreeListPanel1.gridwidth = 2;
        constraintsFunctionCallDeclarationTreeListPanel1.fill = GridBagConstraints.BOTH;
        constraintsFunctionCallDeclarationTreeListPanel1.weightx = 1.0;
        constraintsFunctionCallDeclarationTreeListPanel1.weighty = 1.0;
        constraintsFunctionCallDeclarationTreeListPanel1.insets = new Insets(4, 0, 0, 0);
        add(getFunctionCallDeclarationTreeListPanel(), constraintsFunctionCallDeclarationTreeListPanel1);
        initConnections();
    }

    private JCheckBox getCheckBoxTest() {
        if (_checkBoxTest == null) {
            _checkBoxTest = new JCheckBox(new AbstractBoundAction("Test") {
                public void doit() {
                    if (getExperimentDeclaration() != null) {
                        getExperimentDeclaration().setTestSimulationDeclaration(
                            getCheckBoxTest().isSelected());
                    }
                }
            });
        }

        return _checkBoxTest;
    }

    public void setDeclaration(IDeclaration declaration) {
        setExperimentDeclaration((IExperimentDeclaration) declaration);
    }

    public void setExperimentDeclaration(IExperimentDeclaration newValue) {
        if (_experimentDeclarationIntern != newValue) {
            IExperimentDeclaration oldValue = getExperimentDeclaration();
            _experimentDeclarationIntern = newValue;
            connPtoP1SetTarget();
            connPtoP4SetTarget();

            if (newValue != null)
                newValue.addChangeListener(_eventHandler);

            firePropertyChange("experimentDeclaration", oldValue, newValue);
            if (newValue != null) {
                getCheckBoxTest().setSelected(newValue.isTestSimulationDeclaration());
            }
        }
    }
}