package de.uniwue.ki.gui.mas.sim.experiment;
import java.awt.Container;
import java.util.Iterator;

import de.uniwue.ki.gui.mas.basics.type.argument.ArgumentDeclarationEditorPaneFactoryManager;
import de.uniwue.ki.gui.mas.basics.type.argument.SimulationDeclarationArgumentDeclarationEditor;
import de.uniwue.ki.gui.mas.model.AbstractDeclarationPanelAndActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.gui.mas.util.declaration.action.NewDeclarationAction;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.basics.type.SimulationDeclarationType;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.experiment.ExperimentFactory;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (11.10.2001 15:13:45)
 * @author Christoph Oechslein
 */
public class ExperimentDeclarationPanelActionFactory extends AbstractDeclarationPanelAndActionFactory {
	private final IDeclarationPanelFactory _editPropertiesDeclarationPanelFactory =
		new EditPropertiesDeclarationPanelFactory();

	private class EditPropertiesDeclarationPanelFactory extends AbstractDeclarationPanelFactory {

		public IDeclarationPanel createDeclarationPanel(final IDeclaration declaration) {
			ExperimentDeclarationPanel experimentDeclarationPanel = new ExperimentDeclarationPanel();

			NameDeclarationWithDocumentationAdapterPanel nameDeclarationWithDocumentationAdapterPanel =
				new NameDeclarationWithDocumentationAdapterPanel();

			nameDeclarationWithDocumentationAdapterPanel.setLinePanel(true);

			nameDeclarationWithDocumentationAdapterPanel.setDeclarationPanel(experimentDeclarationPanel);

			nameDeclarationWithDocumentationAdapterPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());

			return nameDeclarationWithDocumentationAdapterPanel;
		}
		public String getDeclarationTitle(IDeclaration declaration) {
			return "Experiment";
		}
	}

	static {
		ArgumentDeclarationEditorPaneFactoryManager.addArgumentDeclarationEditorPaneFactory(
			SimulationDeclarationType.getInstance(),
			SimulationDeclarationArgumentDeclarationEditor.class);
	}

	public ExperimentDeclarationPanelActionFactory() {
		super();
	}

	public ExperimentDeclarationPanelActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super(guiSeSAmDeclaration);
	}

	protected boolean getCanProduceDeclarationActions(Container parent, IDeclaration anDeclaration) {
		return anDeclaration instanceof IExperimentDeclaration;
	}

	public IDeclarationPanelFactory getDeclarationPanelFactory(
		Container parent,
		IDeclaration anDeclaration) {
		return _editPropertiesDeclarationPanelFactory;
	}

	protected Iterator getImportantDeclarationActions(
		final Container parent,
		final IDeclaration anDeclaration) {

		return CollectionsHelper
			.createConsIterator(new AbstractBoundAction(
				"Create Simulation Run",
				NewDeclarationAction.NEW_ICON) {
			public void doit() {
				try {
					ISeSAmDeclaration seSAmDeclaration = getGUISeSAmDeclaration().getSeSAmDeclaration();
					seSAmDeclaration.getExperimentListAndFactory().getExperimentList().add(
						ExperimentFactory.createExperiment(
							seSAmDeclaration,
							(IExperimentDeclaration)anDeclaration));

				} catch (ModelCompilingException e) {
					getGUISeSAmDeclaration().showException("During simualtion run creation", e, parent);
				}

			}
		}, super.getImportantDeclarationActions(parent, anDeclaration));
	}
}