package de.uniwue.ki.gui.mas.sim.experiment;
import java.awt.Container;
import java.util.Iterator;
import java.util.List;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import de.uniwue.ki.gui.mas.model.DefaultMutableListModelAndFactoryPanelActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.model.ISimulationElementsDeclaration;
import de.uniwue.ki.mas.sim.experiment.ExperimentListAndFactory;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;
/**
 * Creation date: (21.01.2002 20:09:18)
 * @author Christoph Oechslein
 */
public class ExperimentListActionFactory extends DefaultMutableListModelAndFactoryPanelActionFactory {
	public ExperimentListActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super(guiSeSAmDeclaration);
	}

	protected void createNewObjectAndAdd(
		Container parent,
		IMutableListModelAndFactory mutableListModelAndFactory) {
		ExperimentListPanelHelper.addExperimentInThread(parent, getGUISeSAmDeclaration());
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		return (getGUISeSAmDeclaration() != null) && (anObject instanceof ExperimentListAndFactory);
	}

	public Iterator getMoreActions(Container parent, Object anObject) {

		List actions = CollectionsHelper.asLinkedList(super.getMoreActions(parent, anObject));
		addNewActions(actions, 1, parent);
		return actions.iterator();
	}

	private void addNewActions(List actions, int position, final Container parent) {
		ISimulationElementsDeclaration simulationElementsDeclaration =
			getGUISeSAmDeclaration().getSeSAmModelDeclaration().getSimulationElementsDeclaration();

		boolean newSitMenuAdded =
			addNewMenu(
				parent,
				actions,
				position,
				"New (Situations)",
				simulationElementsDeclaration.getSituationDeclarationListAndFactory().getDeclarationList());
		boolean newSimMenuAdded =
			addNewMenu(
				parent,
				actions,
				position,
				"New (Simulations)",
				simulationElementsDeclaration.getSimulationDeclarationListAndFactory().getDeclarationList());
		boolean newExpMenuAdded =
			addNewMenu(
				parent,
				actions,
				position,
				"New (Experiments)",
				simulationElementsDeclaration
					.getExperimentDeclarationListAndFactory()
					.getExperimentDeclarationList());

		int newPosition = position;
		if (newSitMenuAdded)
			newPosition++;
		if (newSimMenuAdded)
			newPosition++;
		if (newExpMenuAdded)
			newPosition++;

		if (newPosition != position) {
			actions.add(newPosition, null);
		}
	}

	private boolean addNewMenu(
		final Container parent,
		List actions,
		int position,
		String menuName,
		IDeclarationList declarationList) {

		if (declarationList.isEmpty()) {
			return false;

		} else {
			JMenu menu = new JMenu(menuName);
			menu.setIcon(_newIcon);

			Iterator it = declarationList.iterator();
			while (it.hasNext()) {
				final IDocumentableDeclaration decl = (IDocumentableDeclaration) it.next();
				JMenuItem menuItem = new JMenuItem(new AbstractBoundAction(decl.getName(), _newIcon) {
					public void doit() {
						ExperimentListPanelHelper.addExperimentInThread(parent, getGUISeSAmDeclaration(), decl);
					}
				});

				menuItem.setToolTipText(decl.getDocumentation());
				menu.add(menuItem);
			}

			actions.add(position, menu);
			return true;
		}
	}

}