package de.uniwue.ki.gui.mas.sim.experiment;

import java.awt.Component;
import java.awt.Container;

import javax.swing.Icon;
import javax.swing.JOptionPane;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationListPanel;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISimulationElementsDeclaration;
import de.uniwue.ki.mas.sim.experiment.*;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

/**
 * @author oechslein
 * 
 * Creation date: 14.05.2003
 */
public class ExperimentListPanelHelper {

	public static final Icon SIMULATION_RUNNING_ICON =
		SwingMoreUtilities.createImageIcon("eclipse/sesam-simrunning.gif");
	public static final Icon SIMULATION_PAUSED_ICON =
		SwingMoreUtilities.createImageIcon("eclipse/sesam-simpaused.gif");
	public static Icon getMarkerIcon(IExperiment experiment) {
		return (((IExperiment)experiment).isRunning()) ? SIMULATION_RUNNING_ICON : SIMULATION_PAUSED_ICON;
	}

	private static IExperimentDeclaration choiceExperimentDeclaration(
		Component parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		return (IExperimentDeclaration)DeclarationListPanel.choiceDeclarationList(
			parent,
			guiSeSAmDeclaration,
			getSimulationElementsDeclaration(guiSeSAmDeclaration)
				.getExperimentDeclarationListAndFactory()
				.getExperimentDeclarationList(),
			"Choice a Experiment",
			"Which Experiment:");
	}

	private static ISimulationElementsDeclaration getSimulationElementsDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		return guiSeSAmDeclaration
			.getSeSAmDeclaration()
			.getSeSAmModelDeclaration()
			.getSimulationElementsDeclaration();
	}

	private static ISimulationDeclaration choiceSimulationDeclaration(
		Component parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		return (ISimulationDeclaration)DeclarationListPanel.choiceDeclarationList(
			parent,
			guiSeSAmDeclaration,
			getSimulationElementsDeclaration(guiSeSAmDeclaration)
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationList(),
			"Choice a Simulation",
			"Which Simulation:");
	}
	private static ISituationDeclaration choiceSituationDeclaration(
		Component parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		return (ISituationDeclaration)DeclarationListPanel.choiceDeclarationList(
			parent,
			guiSeSAmDeclaration,
			getSimulationElementsDeclaration(guiSeSAmDeclaration)
				.getSituationDeclarationListAndFactory()
				.getSituationDeclarationList(),
			"Choice a Situation",
			"Which Situation:");
	}

	private static IDocumentableDeclaration choiceExperiment(
		Component parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		ISimulationElementsDeclaration simulationElementsDeclaration =
			guiSeSAmDeclaration.getSeSAmModelDeclaration().getSimulationElementsDeclaration();

		int sizeOfSituationList =
			simulationElementsDeclaration
				.getSituationDeclarationListAndFactory()
				.getSituationDeclarationList()
				.size();

		int sizeOfSimulationList =
			simulationElementsDeclaration
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationList()
				.size();

		int sizeOfExperimentList =
			simulationElementsDeclaration
				.getExperimentDeclarationListAndFactory()
				.getExperimentDeclarationList()
				.size();

		if ((sizeOfExperimentList == 0) && (sizeOfSimulationList == 0) && (sizeOfSituationList == 0)) {
			return (ISituationDeclaration)simulationElementsDeclaration
				.getSituationDeclarationListAndFactory()
				.getFirstDeclarationOrCreateNewIfEmpty();

		} else if (
			((sizeOfExperimentList > 0) && (sizeOfSimulationList > 0))
				|| ((sizeOfSimulationList > 0) && (sizeOfSituationList > 0))
				|| ((sizeOfExperimentList > 0) && (sizeOfSituationList > 0))) {
			return choiceExperimentWithTree(parent, guiSeSAmDeclaration);

		} else if (sizeOfExperimentList > 0) {
			return choiceExperimentDeclaration(parent, guiSeSAmDeclaration);

		} else if (sizeOfSimulationList > 0) {
			return choiceSimulationDeclaration(parent, guiSeSAmDeclaration);

		} else if (sizeOfSituationList > 0) {
			if (sizeOfSituationList == 1) {
				return (ISituationDeclaration)simulationElementsDeclaration
					.getSituationDeclarationListAndFactory()
					.getSituationDeclarationList()
					.get(0);

			} else {
				return choiceSituationDeclaration(parent, guiSeSAmDeclaration);
			}

		} else {
			Assert.fail();
		}

		return null;
	}

	private static IDocumentableDeclaration choiceExperimentWithTree(
		Component parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration) {

		ExperimentOrSimulationChooserPanel experimentOrSimulationChooserPanel =
			new ExperimentOrSimulationChooserPanel();

		experimentOrSimulationChooserPanel.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		JOptionPane optionPane =
			new JOptionPane(
				experimentOrSimulationChooserPanel,
				JOptionPane.QUESTION_MESSAGE,
				JOptionPane.OK_CANCEL_OPTION);

		GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration).addModalPanel(
			parent,
			"Choice",
			optionPane);

		ISeSAmDeclaration seSAmDeclaration = guiSeSAmDeclaration.getSeSAmDeclaration();

		if (SwingMoreUtilities.userChoiceOption(optionPane, JOptionPane.OK_OPTION)) {
			if (experimentOrSimulationChooserPanel.getExperimentDeclaration() != null) {
				return experimentOrSimulationChooserPanel.getExperimentDeclaration();

			} else if (experimentOrSimulationChooserPanel.getSimulationDeclaration() != null) {
				return experimentOrSimulationChooserPanel.getSimulationDeclaration();

			} else if (experimentOrSimulationChooserPanel.getSituationDeclaration() != null) {
				ISimulationDeclaration simulationDeclaration =
					seSAmDeclaration
						.getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getSimulationDeclarationListAndFactory()
						.getSimulationDeclarationFactory()
						.createSimulationDeclaration(
							experimentOrSimulationChooserPanel.getSituationDeclaration());

				return simulationDeclaration;

			}
		}

		return null;
	}

	public static void addExperimentInThread(
		final Container parent,
		final IGUISeSAmDeclaration guiSeSAmDeclaration,
		final IDocumentableDeclaration decl) {

		SwingMoreUtilities
			.invokeWithIndeterminateProgressbar(
				parent,
				"Creating Experiment",
				"Creating Experiment " + decl.getName(),
				new Runnable() {
			public void run() {
				addExperiment(parent, guiSeSAmDeclaration, decl);
			}
		});
	}

	public static void addExperiment(
		Container parent,
		IGUISeSAmDeclaration guiSeSAmDeclaration,
		IDocumentableDeclaration decl) {

		ExperimentList experimentList =
			guiSeSAmDeclaration.getSeSAmDeclaration().getExperimentListAndFactory().getExperimentList();
		try {
			if (decl instanceof ISimulationDeclaration) {
				experimentList.add(
					ExperimentFactory.createExperiment(
						guiSeSAmDeclaration.getSeSAmDeclaration(),
						(ISimulationDeclaration)decl));
			} else if (decl instanceof IExperimentDeclaration) {
				experimentList.add(
					ExperimentFactory.createExperiment(
						guiSeSAmDeclaration.getSeSAmDeclaration(),
						(IExperimentDeclaration)decl));
			} else {
				experimentList.add(
					ExperimentFactory.createExperiment(
						guiSeSAmDeclaration.getSeSAmDeclaration(),
						(ISituationDeclaration)decl));
			}

		} catch (Throwable exception) {
			guiSeSAmDeclaration.showException("Compile Error", exception, parent);
			exception.printStackTrace();
		}
	}

	public static void addExperiment(Container parent, IGUISeSAmDeclaration guiSeSAmDeclaration) {
		IDocumentableDeclaration newExperiment = choiceExperiment(parent, guiSeSAmDeclaration);
		if (newExperiment != null)
			addExperiment(parent, guiSeSAmDeclaration, newExperiment);
	}

	public static void addExperimentInThread(Container parent, IGUISeSAmDeclaration guiSeSAmDeclaration) {
		IDocumentableDeclaration newExperiment = choiceExperiment(parent, guiSeSAmDeclaration);
		if (newExperiment != null) {
			addExperimentInThread(parent, guiSeSAmDeclaration, newExperiment);
		}
	}
}
