package de.uniwue.ki.gui.mas.sim.experiment;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.*;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.*;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.tree.JScrollableTree;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.name.INamed;

/**
 * Insert the type's description here. Creation date: (01.02.2002 16:16:52)
 * 
 * @author Christoph Oechslein
 */
public class ExperimentOrSimulationChooserPanel extends JScrollableTree {

    private static class NamedDefaultMutableTreeNode extends DefaultMutableTreeNode implements INamed {
        public NamedDefaultMutableTreeNode(String name) {
            super(name, true);
        }
        public NamedDefaultMutableTreeNode(INamed named) {
            super(named, true);
        }
        public String getName() {
            if (getUserObject() instanceof String)
                return (String) getUserObject();
            else
                return ((INamed) getUserObject()).getName();
        }

        public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
            if (getUserObject() instanceof String) {
                buff.append((String) getUserObject());
            } else {
                ((INamed) getUserObject()).appendName(buff, currDepth, maxDepth);
            }
        }

        public boolean isLeaf() {
            return false;
        }
    }

    private static class NamedDefaultMutableTreeNodeLeaf extends DefaultMutableTreeNode implements INamed {
        public NamedDefaultMutableTreeNodeLeaf(INamed named) {
            super(named, false);
        }

        public String getName() {
            return ((INamed) getUserObject()).getName();
        }

        public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
            ((INamed) getUserObject()).appendName(buff, currDepth, maxDepth);
        }

        public boolean isLeaf() {
            return true;
        }
    }
    private IGUISeSAmDeclaration _guiSeSAmDeclaration;
    private final EventHandler _eventHandler = new EventHandler();
    private JTree _jTreeIntern = null;

    private final class EventHandler implements TreeSelectionListener {
        public void valueChanged(TreeSelectionEvent e) {
            if (e.getSource() == ExperimentOrSimulationChooserPanel.this.getjTreeIntern())
                connEtoC1(e);
        }
    }

    public ExperimentOrSimulationChooserPanel() {
        this(true);
    }

    public ExperimentOrSimulationChooserPanel(boolean onlyOneSelection) {
        super();
        initialize();

        getJTree().getSelectionModel().setSelectionMode(
            onlyOneSelection
                ? TreeSelectionModel.SINGLE_TREE_SELECTION
                : TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
    }

    protected boolean addToTreeNode(NamedDefaultMutableTreeNode simulationDeclarationNode, Iterator it) {

        List list = CollectionsHelper.asArrayList(it);

        Collections.sort(list, NamedComparator.getInstance());

        it = list.iterator();

        while (it.hasNext()) {
            INamed currINamed = (INamed) it.next();

            NamedDefaultMutableTreeNodeLeaf currNode = new NamedDefaultMutableTreeNodeLeaf(currINamed);
            simulationDeclarationNode.add(currNode);
        }

        return !list.isEmpty();
    }

    /**
     * connEtoC1: (ExperimentOrSimulationChooserPanel.treeSelection.valueChanged(TreeSelectionEvent) -->
     * ExperimentOrSimulationChooserPanel.eventValueChanged(LTreeSelectionEvent;)V)
     * 
     * @param arg1
     *            TreeSelectionEvent
     */

    private void connEtoC1(TreeSelectionEvent arg1) {
        try {

            this.eventValueChanged(arg1);

        } catch (Throwable ivjExc) {

            handleException(ivjExc);
        }
    }

    private void connPtoP1SetTarget() {
        try {
            setjTreeIntern(this.getJTree());

        } catch (Throwable ivjExc) {

            handleException(ivjExc);
        }
    }

    protected void createAndSetTreeModel() {
        DefaultTreeModel defaultTreeModel = createTreeModel();

        getJTree().setModel(defaultTreeModel);

        //		for (int i = defaultTreeModel.getChildCount(defaultTreeModel.getRoot()) - 1; i >= 0; i--) {
        //			getJTree().expandRow(i);
        //		}

        getJTree().expandRow(0);

        getJTree().setSelectionRow(1);
    }

    protected DefaultTreeModel createTreeModel() {
        NamedDefaultMutableTreeNode rootNode = new NamedDefaultMutableTreeNode("root");

        NamedDefaultMutableTreeNode experimentClassDeclarationNode =
            new NamedDefaultMutableTreeNode("Experiments");
        NamedDefaultMutableTreeNode simulationClassDeclarationNode =
            new NamedDefaultMutableTreeNode("Simulations");
        NamedDefaultMutableTreeNode situationClassDeclarationNode =
            new NamedDefaultMutableTreeNode("Situations");

        if (getGUISeSAmDeclaration() != null) {

            if (addToTreeNode(experimentClassDeclarationNode,
                getGUISeSAmDeclaration()
                    .getSeSAmModelDeclaration()
                    .getSimulationElementsDeclaration()
                    .getExperimentDeclarationListAndFactory()
                    .getExperimentDeclarationList()
                    .iterator())) {

                rootNode.add(experimentClassDeclarationNode);
            }

            if (addToTreeNode(simulationClassDeclarationNode,
                getGUISeSAmDeclaration()
                    .getSeSAmModelDeclaration()
                    .getSimulationElementsDeclaration()
                    .getSimulationDeclarationListAndFactory()
                    .getSimulationDeclarationList()
                    .iterator())) {

                rootNode.add(simulationClassDeclarationNode);
            }

            if (addToTreeNode(situationClassDeclarationNode,
                getGUISeSAmDeclaration()
                    .getSeSAmModelDeclaration()
                    .getSimulationElementsDeclaration()
                    .getSituationDeclarationListAndFactory()
                    .getSituationDeclarationList()
                    .iterator())) {

                rootNode.add(situationClassDeclarationNode);
            }

        }

        return new DefaultTreeModel(rootNode);
    }

    protected void eventValueChanged(TreeSelectionEvent treeSelectionEvent) {
    }

    public IExperimentDeclaration getExperimentDeclaration() {
        if ((getSelectedTreeNode() != null)
            && (getSelectedTreeNode().getUserObject() instanceof IExperimentDeclaration))
            return (IExperimentDeclaration) getSelectedTreeNode().getUserObject();

        else
            return null;
    }

    public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
        return _guiSeSAmDeclaration;
    }

    private JTree getjTreeIntern() {

        return _jTreeIntern;
    }

    private VariableHeightLayoutCache getLocalSelectionModelVariableHeightLayoutCache() {
        VariableHeightLayoutCache _localSelectionModelVariableHeightLayoutCache = null;
        try {
            _localSelectionModelVariableHeightLayoutCache = new VariableHeightLayoutCache();
            _localSelectionModelVariableHeightLayoutCache.setModel(new DefaultTreeModel(null));
            _localSelectionModelVariableHeightLayoutCache.setRootVisible(true);
            _localSelectionModelVariableHeightLayoutCache.setSelectionModel(new DefaultTreeSelectionModel());
        } catch (Throwable ivjExc) {
            handleException(ivjExc);
        }
        return _localSelectionModelVariableHeightLayoutCache;
    }

    public ISimulationDeclaration getSimulationDeclaration() {
        if ((getSelectedTreeNode() != null)
            && (getSelectedTreeNode().getUserObject() instanceof ISimulationDeclaration))
            return (ISimulationDeclaration) getSelectedTreeNode().getUserObject();

        else
            return null;
    }

    public ISituationDeclaration getSituationDeclaration() {
        if ((getSelectedTreeNode() != null)
            && (getSelectedTreeNode().getUserObject() instanceof ISituationDeclaration))
            return (ISituationDeclaration) getSelectedTreeNode().getUserObject();

        else
            return null;
    }

    private void handleException(Throwable exception) {

        exception.printStackTrace(System.out);
    }

    private void initConnections() {

        connPtoP1SetTarget();
    }

    private void initialize() {
        try {

            setName("ExperimentOrSimulationChooserPanel");
            setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
            setSize(192, 224);
            initConnections();
        } catch (Throwable ivjExc) {
            handleException(ivjExc);
        }

        DefaultTreeSelectionModel localSelectionModel;
        localSelectionModel = new DefaultTreeSelectionModel();
        localSelectionModel.setRowMapper(getLocalSelectionModelVariableHeightLayoutCache());
        localSelectionModel.setSelectionMode(1);
        getJTree().setShowsRootHandles(true);
        getJTree().setCellRenderer(new de.uniwue.ki.gui.util.NamedTreeCellRenderer());
        getJTree().setRootVisible(false);
        getJTree().setSelectionModel(localSelectionModel);

    }

    public static void main(String[] args) {
        try {
            JFrame frame = new JFrame();
            ExperimentOrSimulationChooserPanel aExperimentOrSimulationChooserPanel;
            aExperimentOrSimulationChooserPanel = new ExperimentOrSimulationChooserPanel();
            frame.setContentPane(aExperimentOrSimulationChooserPanel);
            frame.setSize(aExperimentOrSimulationChooserPanel.getSize());
            frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
            frame.show();
            Insets insets = frame.getInsets();
            frame.setSize(
                frame.getWidth() + insets.left + insets.right,
                frame.getHeight() + insets.top + insets.bottom);
            frame.setVisible(true);
        } catch (Throwable exception) {
            System.err.println(
                "Exception occurred in main() of de.uniwue.ki.gui.mas.model.sim.experiment.ExperimentOrSimulationChooserPanel");
            exception.printStackTrace(System.out);
        }
    }

    public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
        _guiSeSAmDeclaration = guiSeSAmDeclaration;

        createAndSetTreeModel();
    }

    private void setjTreeIntern(JTree newValue) {
        if (_jTreeIntern != newValue) {
            try {
                if (_jTreeIntern != null) {
                    _jTreeIntern.removeTreeSelectionListener(_eventHandler);
                }
                _jTreeIntern = newValue;

                if (_jTreeIntern != null) {
                    _jTreeIntern.addTreeSelectionListener(_eventHandler);
                }

            } catch (Throwable ivjExc) {

                handleException(ivjExc);
            }
        }

    }

    private DefaultMutableTreeNode getSelectedTreeNode() {
        TreePath selectionPath = getJTree().getSelectionPath();
        if (selectionPath == null)
            return null;
        else {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) selectionPath.getLastPathComponent();
            if (testNode(node))
                return node;
            else
                return null;
        }
    }

    public Iterator getSelectedTreeNodes() {
        TreePath[] selectionPaths = getJTree().getSelectionPaths();
        if (selectionPaths == null)
            return CollectionsHelper.EMPTY_ITERATOR;

        List nodes = new ArrayList(selectionPaths.length);
        for (int i = 0; i < selectionPaths.length; i++) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) selectionPaths[i].getLastPathComponent();
            if (testNode(node))
                nodes.add(node);
        }

        return nodes.iterator();
    }

    private boolean testNode(DefaultMutableTreeNode node) {
        return (node.getUserObject() instanceof IExperimentDeclaration)
            || (node.getUserObject() instanceof ISimulationDeclaration)
            || (node.getUserObject() instanceof ISituationDeclaration);
    }
}