package de.uniwue.ki.gui.mas.sim.simulation;
import java.awt.Container;
import java.util.*;

import javax.swing.Icon;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.IActionFactory;
import de.uniwue.ki.gui.mas.util.declaration.action.PropertiesDeclarationAction;
import de.uniwue.ki.gui.mas.util.declaration.action.RemoveMutableListModelAction;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.sim.experiment.OneSimulationExperiment;
import de.uniwue.ki.mas.sim.simulation.IThreadedSimulation;
import de.uniwue.ki.util.list.IMutableListModel;

/**
 * Creation date: (22.11.2001 09:48:43)
 * @author Chripauseh Oechslein
 */
public class SimulationActionFactory implements IActionFactory {
	private static final Icon _startIcon = SwingMoreUtilities.getJavaSunIcon(16, "media/Play16.gif");
	private static final Icon _pauseIcon = SwingMoreUtilities.getJavaSunIcon(16, "media/Pause16.gif");

	private final IGUISeSAmDeclaration _guiSeSAmDeclaration;

	public SimulationActionFactory(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super();

		_guiSeSAmDeclaration = guiSeSAmDeclaration;
	}

	public boolean getCanProduceActions(Container parent, Object anObject) {
		return (anObject instanceof IThreadedSimulation);
	}

	protected final IGUISeSAmDeclaration getGUISeSAmDeclaration() {
		return _guiSeSAmDeclaration;
	}

	public Iterator getImportantActions(Container parent, Object anObject) {
		final IThreadedSimulation simulation =
			(anObject instanceof IThreadedSimulation)
				? (IThreadedSimulation) anObject
				: ((OneSimulationExperiment) anObject).getThreadedSimulation();

		List actions = new LinkedList();

		actions.add(null);

		if (simulation.isRunning()) {
			actions.add(new AbstractBoundAction("Pause", _pauseIcon) {
				public void doit() {
					simulation.pauseInThread();
				}
			});
		} else {
			actions.add(new AbstractBoundAction("Start", _startIcon) {
				public void doit() {
					simulation.startInThread();
				}
			});
		}

		return actions.iterator();
	}

	public Iterator getMoreActions(Container parent, final Object anObject) {
		List actions = new LinkedList();

		actions.add(
			new RemoveMutableListModelAction(
				anObject,
				(IMutableListModel) getGUISeSAmDeclaration().getSeSAmDeclarationTreeModel().getUserObjectParentOf(anObject)));

		actions.add(new InspectAction(anObject));

		return actions.iterator();
	}

	public AbstractBoundAction getPropertyAction(final Container parent, Object anObject) {

		final IThreadedSimulation simulation = (IThreadedSimulation) anObject;

		return new AbstractBoundAction("Properties", PropertiesDeclarationAction.PROPERTIES_ICON) {
			public void doit() {
				ThreadedSimulationWithNamePanel simulationPanel = new ThreadedSimulationWithNamePanel();

				simulationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
				simulationPanel.setThreadedSimulation(simulation);

				de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration.getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
					parent,
					"Show Simulation",
					simulationPanel,
					true);
			}
		};
	}
}