package de.uniwue.ki.gui.mas.sim.simulation;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;

import javax.swing.*;

import de.uniwue.ki.gui.mas.basics.functionCall.ExpertFilterFunctionDeclarationListAble;
import de.uniwue.ki.gui.mas.basics.functionCall.FunctionCallDeclarationListWithSelectionPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.sim.analysis.AnalysisDeclarationListToAnalysisDeclarationList;
import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationComboBox;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.border.BetterEmptyBorder;
import de.uniwue.ki.mas.basics.action.debugging.FunctionActionDebuggingDeclarationFactory;
import de.uniwue.ki.mas.basics.action.rest.FunctionActionRestDeclarationFactory;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclarationList;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.NamedFunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.VoidType;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.sim.situation.SituationDeclarationListAndFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationFactory;

/**
 * Creation date: (30.10.2000 16:32:34)
 * @author Christoph Oechslein
 */
public class SimulationDeclarationPanel extends AbstractDeclarationPanel {

	private static final NamedFunctionCallDeclarationFactory ASSERT_FUNCTIONCALL_DECL_FACTORY =
		new NamedFunctionCallDeclarationFactory(
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				FunctionActionDebuggingDeclarationFactory.ASSERT_SPECIFIC_FUNCTION_DECL));

	private static ExpertFilterFunctionDeclarationListAble createExpertFilterFunctionDeclarationListAble() {
		ExpertFilterFunctionDeclarationListAble able = new ExpertFilterFunctionDeclarationListAble() {

			public boolean isElementOk(Object obj) throws Exception {
				IFunctionDeclaration functionDeclaration = (IFunctionDeclaration)obj;

				if (functionDeclaration
					== FunctionActionDebuggingDeclarationFactory.ASSERT_SPECIFIC_FUNCTION_DECL)
					return true;

				if (functionDeclaration == FunctionActionRestDeclarationFactory.NOOP_SPECIFIC_FUNCTION_DECL)
					return true;

				if (!super.isElementOk(obj))
					return false;

				if (functionDeclaration instanceof IGenericFunctionDeclaration) {
					return !(
						(IGenericFunctionDeclaration)functionDeclaration).canCreateFunctionDeclarationWith(
						VoidType.getInstance());
				} else {
					return !((ISpecificFunctionDeclaration)functionDeclaration)
						.getSpecificOutputType()
						.isMoreSpecific(
						VoidType.getInstance());
				}
			}
		};
		return able;
	}

	public static FunctionCallDeclarationListWithSelectionPanel createTestActionFunctionCallDeclListWithSelectionPanel() {
		FunctionCallDeclarationListWithSelectionPanel functionCallDeclarationListWithSelectionPanel =
			new FunctionCallDeclarationListWithSelectionPanel();
		functionCallDeclarationListWithSelectionPanel.setSpecificType(VoidType.getInstance());
		functionCallDeclarationListWithSelectionPanel.setDeclarationFactory(ASSERT_FUNCTIONCALL_DECL_FACTORY);

		updateExpertFilterFunctionDeclarationListAble(functionCallDeclarationListWithSelectionPanel);
		return functionCallDeclarationListWithSelectionPanel;
	}

	private static void updateExpertFilterFunctionDeclarationListAble(FunctionCallDeclarationListWithSelectionPanel functionCallDeclarationListWithSelectionPanel) {
		ExpertFilterFunctionDeclarationListAble expertFilterFunctionDeclarationListAble =
			createExpertFilterFunctionDeclarationListAble();

		functionCallDeclarationListWithSelectionPanel.getExpertFilterFunctionDeclarationListAble().copyInto(
			expertFilterFunctionDeclarationListAble,
			new HashMap());

		functionCallDeclarationListWithSelectionPanel.setExpertFilterFunctionDeclarationListAble(
			expertFilterFunctionDeclarationListAble);
	}
	private FunctionCallDeclarationListWithSelectionPanel _afterTestActionFunctionCallDeclListWithSelectionPanel =
		null;
	private AnalysisDeclarationListToAnalysisDeclarationList _analysisDeclListToAnalysisDeclList = null;
	private JCheckBox _checkBoxTest;

	private FunctionCallDeclarationListWithSelectionPanel _duringTestActionFunctionCallDeclListWithSelectionPanel =
		null;
	private FunctionCallDeclarationListWithSelectionPanel _endFunctionCallDeclListWithSelectionPanel = null;

	private ISimulationDeclaration _simulationDeclaration = null;
	private DeclarationComboBox _situationDeclarationComboBox = null;
	private JTabbedPane _tabbedPane = null;

	public SimulationDeclarationPanel() {
		super();
		initialize();
	}

	public void dispose() {
		super.dispose();

		getSituationDeclarationComboBox().dispose();
		getAnalysisDeclarationListToAnalysisDeclarationList().dispose();
		getEndFunctionCallDeclarationListWithSelectionPanel().dispose();
		getAfterTestActionFunctionCallDeclListWithSelectionPanel().dispose();
		getDuringTestActionFunctionCallDeclListWithSelectionPanel().dispose();
	}

	protected void eventEditSituation() {
		if ((getGUISeSAmDeclaration() != null) && (getSimulationDeclaration() != null)) {
			getGUISeSAmDeclaration().getPanelActionFactoryManager().callPropertyAction(
				this,
				getSimulationDeclaration().getSituationDeclaration());
		}
	}

	private FunctionCallDeclarationListWithSelectionPanel getAfterTestActionFunctionCallDeclListWithSelectionPanel() {
		if (_afterTestActionFunctionCallDeclListWithSelectionPanel == null) {
			_afterTestActionFunctionCallDeclListWithSelectionPanel =
				createTestActionFunctionCallDeclListWithSelectionPanel();
		}
		return _afterTestActionFunctionCallDeclListWithSelectionPanel;
	}

	private AnalysisDeclarationListToAnalysisDeclarationList getAnalysisDeclarationListToAnalysisDeclarationList() {
		if (_analysisDeclListToAnalysisDeclList == null) {
			_analysisDeclListToAnalysisDeclList = new AnalysisDeclarationListToAnalysisDeclarationList();
			_analysisDeclListToAnalysisDeclList.setBorder(new BetterEmptyBorder());

		}
		return _analysisDeclListToAnalysisDeclList;
	}

	private JCheckBox getCheckBoxTest() {
		if (_checkBoxTest == null) {
			_checkBoxTest = new JCheckBox(new AbstractBoundAction("Test") {
				public void doit() {
					if (getSimulationDeclaration() != null) {
						getSimulationDeclaration().setTestSimulationDeclaration(
							getCheckBoxTest().isSelected());
					}

					updateTestTabsEnabledState();
				}
			});
		}

		return _checkBoxTest;
	}

	public IDeclaration getDeclaration() {
		return getSimulationDeclaration();
	}

	private FunctionCallDeclarationListWithSelectionPanel getDuringTestActionFunctionCallDeclListWithSelectionPanel() {
		if (_duringTestActionFunctionCallDeclListWithSelectionPanel == null) {
			_duringTestActionFunctionCallDeclListWithSelectionPanel =
				createTestActionFunctionCallDeclListWithSelectionPanel();
		}
		return _duringTestActionFunctionCallDeclListWithSelectionPanel;
	}

	private FunctionCallDeclarationListWithSelectionPanel getEndFunctionCallDeclarationListWithSelectionPanel() {
		if (_endFunctionCallDeclListWithSelectionPanel == null) {
			_endFunctionCallDeclListWithSelectionPanel = new FunctionCallDeclarationListWithSelectionPanel();
			_endFunctionCallDeclListWithSelectionPanel.setSpecificType(BooleanType.getInstance());
		}
		return _endFunctionCallDeclListWithSelectionPanel;
	}

	private JTabbedPane getJTabbedPane() {
		if (_tabbedPane == null) {
			_tabbedPane = new JTabbedPane();

			_tabbedPane.addTab("End Functions", getEndFunctionCallDeclarationListWithSelectionPanel());
			_tabbedPane.addTab("Analysis", getAnalysisDeclarationListToAnalysisDeclarationList());
			_tabbedPane.addTab(
				"During Run Tests",
				getDuringTestActionFunctionCallDeclListWithSelectionPanel());
			_tabbedPane.addTab("After Run Tests", getAfterTestActionFunctionCallDeclListWithSelectionPanel());
		}
		return _tabbedPane;
	}

	public IFunctionArgumentDeclarationList getOuterFunctionArgumentDeclarationList() {
		return getEndFunctionCallDeclarationListWithSelectionPanel()
			.getOuterFunctionArgumentDeclarationList();
	}

	public ISimulationDeclaration getSimulationDeclaration() {
		return _simulationDeclaration;
	}

	private DeclarationComboBox getSituationDeclarationComboBox() {
		if (_situationDeclarationComboBox == null) {
			_situationDeclarationComboBox = new DeclarationComboBox();
			_situationDeclarationComboBox.setEditable(false);

			_situationDeclarationComboBox
				.addPropertyChangeListener("selectedItem", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ((getSimulationDeclaration() != null)) {
						getSimulationDeclaration().setSituationDeclaration(
							(ISituationDeclaration)getSituationDeclarationComboBox().getSelectedItem());
					}
				}
			});

		}
		return _situationDeclarationComboBox;
	}

	protected IDeclarationFactory getSituationDeclarationFactory() {
		if (getGUISeSAmDeclaration() == null) {
			return null;

		} else {
			return getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSituationDeclarationListAndFactory()
				.getSituationDeclarationFactory();
		}
	}

	private void initialize() {

		setLayout(new java.awt.GridBagLayout());

		java.awt.GridBagConstraints constraintsJLabelSituation = new java.awt.GridBagConstraints();
		constraintsJLabelSituation.gridx = 0;
		constraintsJLabelSituation.gridy = 0;
		constraintsJLabelSituation.anchor = java.awt.GridBagConstraints.WEST;
		constraintsJLabelSituation.insets = new java.awt.Insets(0, 0, 0, 4);
		add(new JLabel("Situation:"), constraintsJLabelSituation);

		java.awt.GridBagConstraints constraintsDeclarationComboBox1 = new java.awt.GridBagConstraints();
		constraintsDeclarationComboBox1.gridx = 1;
		constraintsDeclarationComboBox1.gridy = 0;
		constraintsDeclarationComboBox1.anchor = java.awt.GridBagConstraints.WEST;
		constraintsDeclarationComboBox1.insets = new java.awt.Insets(0, 0, 0, 4);
		add(getSituationDeclarationComboBox(), constraintsDeclarationComboBox1);

		java.awt.GridBagConstraints constraintsJButtonSituation = new java.awt.GridBagConstraints();
		constraintsJButtonSituation.gridx = 2;
		constraintsJButtonSituation.gridy = 0;
		constraintsJButtonSituation.anchor = java.awt.GridBagConstraints.WEST;
		constraintsJButtonSituation.insets = new java.awt.Insets(0, 3, 0, 0);
		add(new JButton(new AbstractBoundAction("Edit Situation ...") {
			public void doit() {
				eventEditSituation();
			}
		}), constraintsJButtonSituation);

		java.awt.GridBagConstraints constraintsJCheckBoxTest = new java.awt.GridBagConstraints();
		constraintsJCheckBoxTest.gridx = 3;
		constraintsJCheckBoxTest.gridy = 0;
		constraintsJCheckBoxTest.anchor = java.awt.GridBagConstraints.EAST;
		constraintsJCheckBoxTest.insets = new java.awt.Insets(0, 3, 0, 3);
		add(getCheckBoxTest(), constraintsJCheckBoxTest);

		java.awt.GridBagConstraints constraintsJTabbedPane = new java.awt.GridBagConstraints();
		constraintsJTabbedPane.gridx = 0;
		constraintsJTabbedPane.gridy = 1;
		constraintsJTabbedPane.gridwidth = 4;
		constraintsJTabbedPane.fill = java.awt.GridBagConstraints.BOTH;
		constraintsJTabbedPane.weightx = 1.0;
		constraintsJTabbedPane.weighty = 1.0;
		constraintsJTabbedPane.insets = new java.awt.Insets(2, 0, 0, 0);
		add(getJTabbedPane(), constraintsJTabbedPane);

		updateTestTabsEnabledState();
	}

	public void setDeclaration(IDeclaration declaration) {
		setSimulationDeclaration((ISimulationDeclaration)declaration);
	}

	public void setEnabled(boolean newEnabled) {
		super.setEnabled(newEnabled);

		updateTestTabsEnabledState();
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null) {
			SituationDeclarationListAndFactory situationDeclarationListAndFactory =
				guiSeSAmDeclaration
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getSituationDeclarationListAndFactory();
			getSituationDeclarationComboBox().setDeclarationListModel(
				situationDeclarationListAndFactory.getSituationDeclarationList());
			getSituationDeclarationComboBox().setDeclarationFactory(
				situationDeclarationListAndFactory.getSituationDeclarationFactory());
		}
		getEndFunctionCallDeclarationListWithSelectionPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getAnalysisDeclarationListToAnalysisDeclarationList().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getAfterTestActionFunctionCallDeclListWithSelectionPanel().setGUISeSAmDeclaration(
			guiSeSAmDeclaration);
		getDuringTestActionFunctionCallDeclListWithSelectionPanel().setGUISeSAmDeclaration(
			guiSeSAmDeclaration);
	}

	public void setOuterFunctionArgumentDeclarationList(IFunctionArgumentDeclarationList functionArgumentDeclarationList) {
		getEndFunctionCallDeclarationListWithSelectionPanel().setOuterFunctionArgumentDeclarationList(
			functionArgumentDeclarationList);
	}

	public void setSimulationDeclaration(ISimulationDeclaration newValue) {
		ISimulationDeclaration oldValue = getSimulationDeclaration();

		if (oldValue != newValue) {
			_simulationDeclaration = newValue;

			if (newValue != null) {
				getCheckBoxTest().setSelected(newValue.isTestSimulationDeclaration());
				getSituationDeclarationComboBox().setSelectedItem(
					getSimulationDeclaration().getSituationDeclaration());
				getAnalysisDeclarationListToAnalysisDeclarationList().setAnalysisDeclarationList(
					getSimulationDeclaration().getAnalysisDeclarationList());

				getEndFunctionCallDeclarationListWithSelectionPanel().setDeclarationListAndFactory(
					newValue.getEndFunctionDeclarationListAndFactory());
				getDuringTestActionFunctionCallDeclListWithSelectionPanel().setDeclarationListModel(
					newValue.getDuringRunTestActionDefinitionList());
				getAfterTestActionFunctionCallDeclListWithSelectionPanel().setDeclarationListModel(
					newValue.getAfterRunTestActionDefinitionList());
			}

			updateTestTabsEnabledState();

			firePropertyChange("simulationDeclaration", oldValue, newValue);
		}
	}

	private void updateTestTabsEnabledState() {
		getJTabbedPane().setEnabledAt(2, getCheckBoxTest().isSelected() && isEnabled());
		getJTabbedPane().setEnabledAt(3, getCheckBoxTest().isSelected() && isEnabled());

		if (getJTabbedPane().getSelectedIndex() >= 2) {
			getJTabbedPane().setSelectedIndex(0);
		}
	}
}