package de.uniwue.ki.gui.mas.sim.situation;
import java.awt.Component;
import java.util.*;

import javax.swing.JTabbedPane;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.entity.object.ObjectInstanceDeclarationListWithSelectionPanel;
import de.uniwue.ki.gui.mas.entity.object.ObjectInstanceDeclarationPanel;
import de.uniwue.ki.gui.mas.model.IDeclarationPanelAndActionFactory;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.*;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.featureList.ObjectFeatureInstanceDeclarationList;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclarationList;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclarationFactory;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.WorldClassDeclarationList;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/**
 * Creation date: (30.10.2000 16:33:00)
 * @author Christoph Oechslein
 */
public class SituationDeclarationPanel extends AbstractDeclarationPanel {
	private ObjectInstanceDeclarationPanel ivjJPanelWorldInstanceDeclPanel = null;
	private ISituationDeclaration ivjSituationDeclarationIntern = null;
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private ObjectInstanceDeclarationListWithSelectionPanel ivjObjectInstanceDeclarationListWithSelectionPanel =
		null;
	private JTabbedPane ivjJTabbedPane = null;
	private boolean _eventSituationDeclarationChangeInProgress = false;

	private final List _featureDeclPanelList = new LinkedList();

	private final class IvjEventHandler implements java.beans.PropertyChangeListener {
		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			if (evt.getSource() == SituationDeclarationPanel.this.getJPanelWorldInstanceDeclPanel()
				&& (evt.getPropertyName().equals("objectInstanceDeclaration")))
				connEtoC1(evt);
		}
	}

	public SituationDeclarationPanel() {
		super();
		initialize();
	}

	private void connEtoC1(java.beans.PropertyChangeEvent arg1) {
		eventWorldInstanceDeclarationChanged(
			getJPanelWorldInstanceDeclPanel().getObjectInstanceDeclaration());
	}

	private void connEtoC2(ISituationDeclaration value) {
		eventSituationDeclarationChanged();
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		getJPanelWorldInstanceDeclPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		getObjectInstanceDeclarationListWithSelectionPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	public void dispose() {
		getJPanelWorldInstanceDeclPanel().dispose();
		getObjectInstanceDeclarationListWithSelectionPanel().dispose();

		Iterator it = _featureDeclPanelList.iterator();
		while (it.hasNext()) {
			IDeclarationPanel declarationPanel = (IDeclarationPanel)it.next();
			declarationPanel.dispose();
		}

		super.dispose();
	}

	protected void eventSituationDeclarationChanged() {
		if(_eventSituationDeclarationChangeInProgress)
			return;
		
		_eventSituationDeclarationChangeInProgress = true;		
		
		if (getSituationDeclaration() != null) {

			removeAllObjectFeatureInstDeclTabs();
			
			IWorldInstanceDeclaration worldInstanceDeclaration = getSituationDeclaration().getWorldInstanceDeclaration();
			
			getJPanelWorldInstanceDeclPanel().setObjectInstanceDeclaration(worldInstanceDeclaration);

			getObjectInstanceDeclarationListWithSelectionPanel().setObjectInstanceDeclarationList(worldInstanceDeclaration.getObjectInstanceDeclarationList());
			

			//removeAllObjectFeatureInstDeclTabs();

			insertObjectFeatureInstDeclTabs(
				worldInstanceDeclaration.getObjectFeatureInstanceDeclarationList());
		}
		_eventSituationDeclarationChangeInProgress = false;
	}

	private void removeAllObjectFeatureInstDeclTabs() {
		Iterator it = _featureDeclPanelList.iterator();
		while (it.hasNext()) {
			IDeclarationPanel declarationPanel = (IDeclarationPanel)it.next();
			getJTabbedPane().remove((Component)declarationPanel);
			declarationPanel.dispose();
			it.remove();
		}
	}

	private void insertObjectFeatureInstDeclTabs(ObjectFeatureInstanceDeclarationList objectFeatureInstanceDeclarationList) {
		Iterator it = objectFeatureInstanceDeclarationList.iterator();
		while (it.hasNext()) {
			IObjectFeatureInstanceDeclaration objectFeatureInstanceDeclaration =
				(IObjectFeatureInstanceDeclaration)it.next();
			IDeclarationPanelAndActionFactory panelActionFactory =
				(IDeclarationPanelAndActionFactory)getGUISeSAmDeclaration()
					.getPanelActionFactoryManager()
					.getPanelActionFactoryFor(this, objectFeatureInstanceDeclaration);

			if (panelActionFactory != null) {
				IDeclarationPanelFactory declarationPanelFactory =
					panelActionFactory.getDeclarationPanelFactory(this, objectFeatureInstanceDeclaration);
				if (declarationPanelFactory != null) {
					IDeclarationPanel declarationPanel =
						declarationPanelFactory.createDeclarationPanel(objectFeatureInstanceDeclaration);
					declarationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
					declarationPanel.setDeclaration(objectFeatureInstanceDeclaration);
					getJTabbedPane().insertTab(
						declarationPanelFactory.getDeclarationTitle(objectFeatureInstanceDeclaration),
						null,
						(Component)declarationPanel,
						"",
						0);
					_featureDeclPanelList.add(declarationPanel);
				}
			}
		}
	}

	private void eventWorldInstanceDeclarationChanged(IObjectInstanceDeclaration newObjectInstanceDeclaration) {
		if ((newObjectInstanceDeclaration != null) && (getSituationDeclaration() != null)) {
			IWorldInstanceDeclaration newWorldInstanceDeclaration =
				(IWorldInstanceDeclaration)newObjectInstanceDeclaration;
			IWorldInstanceDeclaration oldWorldInstanceDeclaration =
				getSituationDeclaration().getWorldInstanceDeclaration();

			if (newWorldInstanceDeclaration != oldWorldInstanceDeclaration) {

				getSituationDeclaration().setWorldInstanceDeclaration(newWorldInstanceDeclaration);
				getObjectInstanceDeclarationListWithSelectionPanel().setObjectInstanceDeclarationList(
					newWorldInstanceDeclaration.getObjectInstanceDeclarationList());
				oldWorldInstanceDeclaration.remove();
			}

			eventSituationDeclarationChanged();
		}
	}

	public IDeclaration getDeclaration() {
		return getSituationDeclaration();
	}

	private ObjectInstanceDeclarationPanel getJPanelWorldInstanceDeclPanel() {
		if (ivjJPanelWorldInstanceDeclPanel == null) {
			ivjJPanelWorldInstanceDeclPanel = new ObjectInstanceDeclarationPanel();
			ivjJPanelWorldInstanceDeclPanel.setChangeClassButtonEnabled(true);
			ivjJPanelWorldInstanceDeclPanel.setMaximumSize(new java.awt.Dimension(3999, 3999));
			ivjJPanelWorldInstanceDeclPanel.setShowObjectInstanceDeclarationFeatures(false);
		}
		return ivjJPanelWorldInstanceDeclPanel;
	}

	private javax.swing.JTabbedPane getJTabbedPane() {
		if (ivjJTabbedPane == null) {
			ivjJTabbedPane = new javax.swing.JTabbedPane();
			ivjJTabbedPane.setName("JTabbedPane");
			ivjJTabbedPane.insertTab("World", null, getJPanelWorldInstanceDeclPanel(), null, 0);
			ivjJTabbedPane.insertTab(
				"Objects",
				null,
				getObjectInstanceDeclarationListWithSelectionPanel(),
				null,
				1);
		}
		return ivjJTabbedPane;
	}

	protected IObjectInstanceDeclarationFactory getObjectInstanceDeclarationFactory(ISeSAmModelDeclaration seSAmModelDeclaration) {
		return seSAmModelDeclaration
			.getAgentClassDeclarationListAndFactory()
			.getAgentClassDeclarationFactory()
			.createAgentInstanceDeclarationFactory();
	}

	protected IDeclarationList getObjectInstanceDeclarationList(ISituationDeclaration situationDeclaration) {
		return situationDeclaration.getWorldInstanceDeclaration().getObjectInstanceDeclarationList();
	}

	private ObjectInstanceDeclarationListWithSelectionPanel getObjectInstanceDeclarationListWithSelectionPanel() {
		if (ivjObjectInstanceDeclarationListWithSelectionPanel == null) {
			ivjObjectInstanceDeclarationListWithSelectionPanel =
				new ObjectInstanceDeclarationListWithSelectionPanel();
		}
		return ivjObjectInstanceDeclarationListWithSelectionPanel;
	}

	public ISituationDeclaration getSituationDeclaration() {
		return ivjSituationDeclarationIntern;
	}

	protected WorldClassDeclarationList getWorldClassDeclarationList() {
		if (getGUISeSAmDeclaration() != null) {
			return getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getWorldClassDeclarationListAndFactory()
				.getWorldClassDeclarationList();
		} else {
			return null;
		}
	}

	private void initConnections() {
		getJPanelWorldInstanceDeclPanel().addPropertyChangeListener(ivjEventHandler);
		this.addPropertyChangeListener(ivjEventHandler);
		getObjectInstanceDeclarationListWithSelectionPanel().addPropertyChangeListener(ivjEventHandler);
	}

	private void initialize() {
		setLayout(new java.awt.GridBagLayout());

		java.awt.GridBagConstraints constraintsJTabbedPane = new java.awt.GridBagConstraints();
		constraintsJTabbedPane.gridx = 0;
		constraintsJTabbedPane.gridy = 1;
		constraintsJTabbedPane.fill = java.awt.GridBagConstraints.BOTH;
		constraintsJTabbedPane.weightx = 1.0;
		constraintsJTabbedPane.weighty = 1.0;
		add(getJTabbedPane(), constraintsJTabbedPane);
		initConnections();
	}

	public void setDeclaration(IDeclaration declaration) {
		setSituationDeclaration((ISituationDeclaration)declaration);
	}

	public void setSituationDeclaration(ISituationDeclaration arg1) {
		setSituationDeclarationIntern(arg1);
	}

	private void setSituationDeclarationIntern(ISituationDeclaration newValue) {
		ISituationDeclaration oldValue = getSituationDeclaration();
		if (oldValue != newValue) {
			ivjSituationDeclarationIntern = newValue;
			connEtoC2(ivjSituationDeclarationIntern);
			firePropertyChange("situationDeclaration", oldValue, newValue);
		}
	}

	protected void worldInstanceDeclarationPanelChanged(IWorldInstanceDeclaration worldInstanceDeclaration) {
		if ((worldInstanceDeclaration != null)
			&& (getSituationDeclaration() != null)
			&& (worldInstanceDeclaration != getSituationDeclaration().getWorldInstanceDeclaration())) {

			getSituationDeclaration().setWorldInstanceDeclaration(worldInstanceDeclaration);
			eventSituationDeclarationChanged();
		}
	}

	public IFunctionArgumentDeclarationList getOuterFunctionArgumentDeclarationList() {
		Assert.assertSame(
			getObjectInstanceDeclarationListWithSelectionPanel().getOuterFunctionArgumentDeclarationList(),
			getJPanelWorldInstanceDeclPanel().getOuterFunctionArgumentDeclarationList());
		return getObjectInstanceDeclarationListWithSelectionPanel().getOuterFunctionArgumentDeclarationList();
	}

	public void setOuterFunctionArgumentDeclarationList(IFunctionArgumentDeclarationList functionArgumentDeclarationList) {
		getObjectInstanceDeclarationListWithSelectionPanel().setOuterFunctionArgumentDeclarationList(
			functionArgumentDeclarationList);
		getJPanelWorldInstanceDeclPanel().setOuterFunctionArgumentDeclarationList(
			functionArgumentDeclarationList);
	}
}