package de.uniwue.ki.gui.mas.sim.situation;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.*;

import com.ibm.easyswing.BoundJCheckBox;

import de.uniwue.ki.gui.mas.entity.object.ObjectGroupObjectSettingPanel;
import de.uniwue.ki.gui.mas.entity.object.ObjectPanel;
import de.uniwue.ki.gui.mas.model.*;
import de.uniwue.ki.gui.util.IObjectPanelFactory;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;
import de.uniwue.ki.mas.basics.feature.spatial.SpatialMapFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.situation.*;

/**
 * Creation date: (12.10.2000 11:10:07)
 * @author Christoph Oechslein
 */
public class SituationPanel extends de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration {
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private ISituation ivjSituationIntern = null;
	private ObjectGroupObjectSettingPanel ivjObjectGroupObjectSettingPanel = null;
	private IWorld ivjWorld = null;
	private JButton ivjJButtonRefresh = null;
	private JPanel ivjJPanelRefresh = null;
	private IWorldInstanceDeclaration ivjWorldInstanceDeclaration = null;
	//	private ISpatialMapPanel ivjSpatialMapPanel = null;
	private JTabbedPane ivjJTabbedPane = null;
	private ObjectPanel ivjWorldObjectPanel = null;
	private BoundJCheckBox ivjJCheckBoxAutoUpdate = null;
	private LinkedList insertedFeatureTabs = new LinkedList();

	private final SituationListener _situationListener = new SituationListener() {
		public void nextTimeStep(SituationEvent event) {
			synchronized (getJCheckBoxAutoUpdate()) {
				/* can occur if event hasn't fired, but this method gets called by an other task
				 * Assert.assertTrue(getJCheckBoxAutoUpdate().isSelected()); */

				eventNextTimeStep(event);
			}
		}
	};

	private final class IvjEventHandler implements ActionListener, java.beans.PropertyChangeListener {
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == SituationPanel.this.getJButtonRefresh()) {
				refresh();
			}
		}
		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			if (evt.getSource() == SituationPanel.this.getJCheckBoxAutoUpdate()
				&& (evt.getPropertyName().equals("selected"))) {
				eventAutoUpdateSettingChanged(getJCheckBoxAutoUpdate().isSelected());
			}
		}
	};

	public SituationPanel() {
		super();
		initialize();
	}

	public void dispose() {
		super.dispose();

		getWorldInstanceDeclaration().removePropertyChangeListener(ivjEventHandler);

		getWorldObjectPanel().dispose();
		Iterator it = insertedFeatureTabs.iterator();
		while (it.hasNext()) {
			ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
			element.dispose();
		}
		//getSpatialMapPanel().dispose();
		getObjectGroupObjectSettingPanel().dispose();
	}

	protected void eventAutoUpdateSettingChanged(boolean oldState) {
		if ((getSituation() != null) && (oldState == getJCheckBoxAutoUpdate().isSelected())) {

			getWorldObjectPanel().setAutomaticUpdate(isAutomaticUpdate());
			Iterator it = insertedFeatureTabs.iterator();
			while (it.hasNext()) {
				ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
				element.setAutomaticUpdate(isAutomaticUpdate());
			}
			//getSpatialMapPanel().setAutomaticUpdate(isAutomaticUpdate());
			getObjectGroupObjectSettingPanel().setAutomaticUpdate(isAutomaticUpdate());

			if (isAutomaticUpdate()) {
				getSituation().addSituationListener(_situationListener);

			} else {
				getSituation().removeSituationListener(_situationListener);
			}
		}
	}

	protected void eventNextTimeStep(SituationEvent event) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (isAutomaticUpdate())
					refresh();
			}
		});
	}

	protected void eventSituationChanged() {

	}

	protected void eventWorldChanged() {
		insertFeatureTabs();
		//		if (SpatialMapFeatureClassDeclaration.hasSpatialMapModel(getWorld())) {
		//			//			getJTabbedPane().setEnabledAt(0, true);
		//			//			getSpatialMapFeaturePanel().setEnabled(true);
		//
		//			//getSpatialMapPanel().setSpatialMapModel(getSpatialMapModel());
		//			getObjectGroupObjectSettingPanel().setSpatialMapModel(getSpatialMapModel());
		//			//getWorldObjectPanel().setDiscretRectangularSpatialMapModel(getDiscretRectangularSpatialMapModel());
		//
		//		} else {
		//			getJTabbedPane().removeTabAt(0);
		//			//			getJTabbedPane().setEnabledAt(0, false);
		//			//			getSpatialMapFeaturePanel().setEnabled(false);
		//			//			getJTabbedPane().setSelectedIndex(1);
		//		}
	}

	protected ISpatialMapModel getSpatialMapModel() {
		try {
			return SpatialMapFeatureClassDeclaration.getSpatialMapModel(getWorld());

		} catch (SimulationRuntimeException e) {
			e.printStackTrace();
			return null;
		}
	}

	private javax.swing.JButton getJButtonRefresh() {
		if (ivjJButtonRefresh == null) {

			ivjJButtonRefresh = new javax.swing.JButton();
			ivjJButtonRefresh.setName("JButtonRefresh");
			ivjJButtonRefresh.setText("Refresh");
		}
		return ivjJButtonRefresh;
	}

	private BoundJCheckBox getJCheckBoxAutoUpdate() {
		if (ivjJCheckBoxAutoUpdate == null) {

			ivjJCheckBoxAutoUpdate = new BoundJCheckBox();
			ivjJCheckBoxAutoUpdate.setName("JCheckBoxAutoUpdate");
			ivjJCheckBoxAutoUpdate.setSelected(false);
			ivjJCheckBoxAutoUpdate.setText("aut. Refresh");

			getWorldObjectPanel().setAutomaticUpdate(isAutomaticUpdate());
			Iterator it = insertedFeatureTabs.iterator();
			while (it.hasNext()) {
				ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
				element.setAutomaticUpdate(isAutomaticUpdate());
			}
			//getSpatialMapPanel().setAutomaticUpdate(isAutomaticUpdate());
			getObjectGroupObjectSettingPanel().setAutomaticUpdate(isAutomaticUpdate());
		}
		return ivjJCheckBoxAutoUpdate;
	}

	private JPanel getJPanelRefresh() {
		if (ivjJPanelRefresh == null) {
			ivjJPanelRefresh = new javax.swing.JPanel();
			ivjJPanelRefresh.setName("JPanelRefresh");
			ivjJPanelRefresh.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
			getJPanelRefresh().add(getJCheckBoxAutoUpdate(), getJCheckBoxAutoUpdate().getName());
			getJPanelRefresh().add(getJButtonRefresh(), getJButtonRefresh().getName());
		}
		return ivjJPanelRefresh;
	}

	private JTabbedPane getJTabbedPane() {
		if (ivjJTabbedPane == null) {
			ivjJTabbedPane = new javax.swing.JTabbedPane();
			ivjJTabbedPane.setName("JTabbedPane");
			//ivjJTabbedPane.insertTab("Map", null, ((Component)getSpatialMapPanel()), null, 0);
			ivjJTabbedPane.insertTab("World", null, getWorldObjectPanel(), null, 0);
			ivjJTabbedPane.insertTab("Objects", null, getObjectGroupObjectSettingPanel(), null, 1);
		}
		return ivjJTabbedPane;
	}

	private ObjectGroupObjectSettingPanel getObjectGroupObjectSettingPanel() {
		if (ivjObjectGroupObjectSettingPanel == null) {
			ivjObjectGroupObjectSettingPanel = new ObjectGroupObjectSettingPanel();
			ivjObjectGroupObjectSettingPanel.setName("ObjectGroupObjectSettingPanel");
		}
		return ivjObjectGroupObjectSettingPanel;
	}

	public ISituation getSituation() {
		return getSituationIntern();
	}

	private ISituation getSituationIntern() {
		return ivjSituationIntern;
	}

	//	protected ISpatialMapPanel getSpatialMapPanel() {
	//		if (ivjSpatialMapPanel == null) {
	//			//ivjSpatialMapPanel = new DiscretRectangularSpatialMapPanel();
	//			ivjSpatialMapPanel = new JSpatialMapPanelWithSelectedIObjects();
	//		}
	//		return ivjSpatialMapPanel;
	//	}

	private IWorld getWorld() {
		return ivjWorld;
	}

	protected IWorldClassDeclaration getWorldClassDeclaration(ISituation situation) {
		return (IWorldClassDeclaration)situation.getWorld().getObjectInstanceDeclaration();
	}

	private IWorldInstanceDeclaration getWorldInstanceDeclaration() {
		return ivjWorldInstanceDeclaration;
	}

	private ObjectPanel getWorldObjectPanel() {
		if (ivjWorldObjectPanel == null) {
			ivjWorldObjectPanel = new ObjectPanel();
			ivjWorldObjectPanel.setName("WorldObjectPanel");
		}
		return ivjWorldObjectPanel;
	}

	private void initConnections() {
		getObjectGroupObjectSettingPanel().addPropertyChangeListener(ivjEventHandler);
		getJButtonRefresh().addActionListener(ivjEventHandler);
		addPropertyChangeListener(ivjEventHandler);
		getWorldObjectPanel().addPropertyChangeListener(ivjEventHandler);
		getJCheckBoxAutoUpdate().addPropertyChangeListener(ivjEventHandler);
	}

	private void initialize() {
		setLayout(new BorderLayout());
		setSize(397, 406);
		add(getJTabbedPane(), BorderLayout.CENTER);
		add(getJPanelRefresh(), BorderLayout.SOUTH);

		initConnections();
		setAutomaticUpdate(true);
	}

	public void refresh() {
		getWorldObjectPanel().refresh();
		getObjectGroupObjectSettingPanel().refresh();
		Iterator it = insertedFeatureTabs.iterator();
		while (it.hasNext()) {
			ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
			element.refresh();
		}
		//getSpatialMapPanel().refresh();
		repaint();
	}

	public void setSituation(ISituation arg1) {
		setSituationIntern(arg1);
		//insertFeatureTabs();
	}

	private void setSituationIntern(ISituation newValue) {
		ISituation oldValue = getSituationIntern();
		if (oldValue != newValue) {
			if (oldValue != null)
				oldValue.removeSituationListener(_situationListener);

			ivjSituationIntern = newValue;

			if (newValue != null) {
				setWorld(newValue.getWorld());
			}

			firePropertyChange("situation", oldValue, newValue);

			if ((newValue != null) && (getJCheckBoxAutoUpdate().isSelected())) {
				newValue.addSituationListener(_situationListener);
			}
		}
	}

	private void setWorld(IWorld newValue) {
		if (ivjWorld != newValue) {
			ivjWorld = newValue;

			if (newValue != null) {
				setWorldInstanceDeclaration(
					(IWorldInstanceDeclaration)getWorld().getObjectInstanceDeclaration());

			} else {
				setWorldInstanceDeclaration(null);
			}

			getObjectGroupObjectSettingPanel().setWorld(newValue);
			getWorldObjectPanel().setObject(getWorld());

			eventWorldChanged();
		}
	}

	private void setWorldInstanceDeclaration(IWorldInstanceDeclaration newValue) {
		ivjWorldInstanceDeclaration = newValue;
	}

	public boolean isAutomaticUpdate() {
		return getJCheckBoxAutoUpdate().isSelected();
	}

	public void setAutomaticUpdate(boolean automaticUpdate) {
		getJCheckBoxAutoUpdate().setSelected(automaticUpdate);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		getWorldObjectPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
		Iterator it = insertedFeatureTabs.iterator();
		while (it.hasNext()) {
			ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
			element.setGUISeSAmDeclaration(guiSeSAmDeclaration);
		}
		getObjectGroupObjectSettingPanel().setGUISeSAmDeclaration(guiSeSAmDeclaration);
	}

	private void insertFeatureTabs() {
		removeAllFeatureTabs();

		if (getWorldInstanceDeclaration() != null) {
			Iterator it = getSituation().getWorld().getObjectFeatures();

			while (it.hasNext()) {
				IObjectFeature objectFeature = (IObjectFeature)it.next();

				ObjectFeaturePanel panel = insertFeatureTabIfAny(objectFeature);
				if (panel != null)
					insertedFeatureTabs.add(panel);
			}
		}
	}

	private void removeAllFeatureTabs() {
		Iterator it = insertedFeatureTabs.iterator();

		while (it.hasNext()) {
			ObjectFeaturePanel element = (ObjectFeaturePanel)it.next();
			for (int i = 0; i < ivjJTabbedPane.getTabCount(); i++) {
				if (element == ivjJTabbedPane.getComponent(i));
				{
					ivjJTabbedPane.removeTabAt(i);
					element.dispose();
				}
			}
		}
		insertedFeatureTabs = new LinkedList();
	}

	private ObjectFeaturePanel insertFeatureTabIfAny(IObjectFeature objectFeature) {
		IObjectPanelAndActionFactory panelActionFactory =
			(IObjectPanelAndActionFactory)getGUISeSAmDeclaration()
				.getPanelActionFactoryManager()
				.getPanelActionFactoryFor(this, objectFeature);

		if (panelActionFactory != null) {
			IObjectPanelFactory declarationPanelFactory =
				panelActionFactory.getPanelFactory(this, objectFeature);
			if (declarationPanelFactory != null) {
				ObjectFeaturePanel objectFeaturePanel =
					(ObjectFeaturePanel)declarationPanelFactory.createObjectPanel(objectFeature);
				objectFeaturePanel.setObjectFeature(objectFeature);
				ivjJTabbedPane.insertTab(
					declarationPanelFactory.getObjectTitle(objectFeature),
					null,
					(Component)objectFeaturePanel,
					"",
					(declarationPanelFactory.getObjectTitle(objectFeature).equals("Map")) ? 0: ivjJTabbedPane.getComponentCount());
				return objectFeaturePanel;
			}
		}
		return null;
	}
}