package de.uniwue.ki.gui.mas.sim.test;

import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.mas.sim.test.TestRunnerTableModel.Entry;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.experiment.*;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.mas.sim.situation.SituationListener;
import de.uniwue.ki.mas.util.declaration.ITestableDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.event.StateChangeMulticaster;
/**
 * @author oechslein
 * 
 * Creation date: 28.08.2003
 */
final class AllTestsStartPauseFinish
	extends AbstractStartPauseFinishAble
	implements IStartPauseFinishInThreadAble {

	private SituationListener _situationListener;
	private transient ChangeListener _changeListener;
	private IStartPauseFinishInThreadAble _currStartPauseFinishInThreadAble = null;

	private int _total = 0;
	private int _errorCount = 0;
	private int _runCount = 0;

	private boolean _paused = false;
	private boolean _started = false;

	private final TestRunnerTableModel _testRunnerTableModel;
	private Iterator _tests = CollectionsHelper.EMPTY_ITERATOR;

	private final StartPauseFinishListener startPauseFinishListener = new StartPauseFinishListener() {
		public void finished(StartPauseFinishEvent event) {

			ITestableDeclaration testableDeclaration;

			IStartPauseFinishInThreadAble startPauseFinishInThreadAble =
				(IStartPauseFinishInThreadAble)event.getSource();
			testableDeclaration = getTestableDeclarationFor(startPauseFinishInThreadAble);

			update(testableDeclaration, event.getException());

			startNextInTests();
			fireStateChanged();
		}

		public void paused(StartPauseFinishEvent event) {
			_paused = true;
			firePaused();
		}

		public void started(StartPauseFinishEvent event) {
			if (!_started) {
				fireStarted();
				_started = true;
			}
			fireStateChanged();
		}
	};

	private ITestableDeclaration getTestableDeclarationFor(IStartPauseFinishInThreadAble startPauseFinishInThreadAble) {
		ITestableDeclaration testableDeclaration;
		if (startPauseFinishInThreadAble instanceof ISimulation)
			testableDeclaration = ((ISimulation)startPauseFinishInThreadAble).getSimulationDeclaration();
		else
			testableDeclaration = ((IExperiment)startPauseFinishInThreadAble).getExperimentDeclaration();
		return testableDeclaration;
	}

	public ITestableDeclaration getCurrentTestableDeclaration() {
		if (_currStartPauseFinishInThreadAble == null)
			return null;
		else
			return getTestableDeclarationFor(_currStartPauseFinishInThreadAble);
	}

	public SituationListener getSituationListener() {
		return _situationListener;
	}

	public void setSituationListener(SituationListener situationListener) {
		_situationListener = situationListener;
	}

	public AllTestsStartPauseFinish(TestRunnerTableModel testRunnerTableModel) {
		_testRunnerTableModel = testRunnerTableModel;
	}

	public void addChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.add(_changeListener, newListener);
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		append(_tests, buff, currDepth, maxDepth);
	}

	protected void fireStateChanged() {
		if (_changeListener != null) {
			_changeListener.stateChanged(new ChangeEvent(this));
		}
	}

	public int getErrorCount() {
		return _errorCount;
	}

	public String getName() {
		return getNameIntern();
	}

	public int getRunCount() {
		return _runCount;
	}

	public int getTotal() {
		return _total;
	}

	public boolean isFinished() {
		return false; //_tests.hasNext() && (_started = true);
	}

	public boolean isFinishedWithErrors() {
		return false; //_errorCount > 0 && (_started = true);
	}

	public boolean isPaused() {
		return _paused;
	}

	public boolean isRunning() {
		return (_currStartPauseFinishInThreadAble != null) && (_currStartPauseFinishInThreadAble.isRunning());
	}

	public void pauseInThread() {
		if (_currStartPauseFinishInThreadAble != null) {
			_currStartPauseFinishInThreadAble.pauseInThread();
		}
		_paused = true;
	}

	public void remove() {
		terminateInThread();
		_changeListener = null;
	}

	public void removeChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, newListener);
	}

	private void setCurrStartPauseFinishInThreadAble(IStartPauseFinishInThreadAble newStartPauseFinishInThreadAble) {
		IStartPauseFinishInThreadAble oldStartPauseFinishInThreadAble = _currStartPauseFinishInThreadAble;

		if (oldStartPauseFinishInThreadAble == _currStartPauseFinishInThreadAble) {
			_currStartPauseFinishInThreadAble = newStartPauseFinishInThreadAble;
			if (newStartPauseFinishInThreadAble != null) {
				newStartPauseFinishInThreadAble.addStartPauseFinishListener(startPauseFinishListener);
			}

			if (oldStartPauseFinishInThreadAble != null) {
				oldStartPauseFinishInThreadAble.removeStartPauseFinishListener(startPauseFinishListener);
				oldStartPauseFinishInThreadAble.remove();
			}
		}
	}

	public void startInThread() {
		//TODO asserts anschalten !!

		if (_paused && (_currStartPauseFinishInThreadAble != null) && (_tests.hasNext())) {
			_paused = false;

		} else {
			_errorCount = 0;
			_runCount = 0;
			reset();
			LinkedList testRunList =
				CollectionsHelper.asLinkedList(_testRunnerTableModel.getActiveTestDeclarations(true));
			_total = testRunList.size();
			_tests = testRunList.iterator();
			startNextInTests();
		}
	}

	void reset() {
		_started = false;
		_paused = false;
	}

	private void startNextInTests() {
		if (_tests.hasNext()) {
			IStartPauseFinishInThreadAble startPauseFinishInThreadAble;
			ITestableDeclaration testableDeclaration = (ITestableDeclaration)_tests.next();

			try {
				if (testableDeclaration instanceof ISimulationDeclaration) {
					IThreadedSimulation simulation =
						SimulationFactory.createThreadedSimulation(
							_testRunnerTableModel.getSeSAmDeclaration(),
							(ISimulationDeclaration)testableDeclaration);

					if (getSituationListener() != null)
						simulation.getSituation().addSituationListener(getSituationListener());

					startPauseFinishInThreadAble = simulation;

				} else {
					IExperiment experiment =
						ExperimentFactory.createExperiment(
							_testRunnerTableModel.getSeSAmDeclaration(),
							(IExperimentDeclaration)testableDeclaration);

					if (getSituationListener() != null) {
						experiment.addExperimentListener(_experimentListener);
					}

					startPauseFinishInThreadAble = experiment;
				}
			} catch (ModelCompilingException e) {

				update(testableDeclaration, e);

				startNextInTests();
				return;
			}

			setCurrStartPauseFinishInThreadAble(startPauseFinishInThreadAble);
			_currStartPauseFinishInThreadAble.startInThread();

		} else {
			setCurrStartPauseFinishInThreadAble(null);
			fireFinished();
		}
	}

	private final ExperimentListener _experimentListener = new ExperimentListener() {
		public void currentSimulationChanged(ExperimentCurrentSimulationChangedEvent event) {
			if (getSituationListener() != null) {
				if (event.getOldSimulation() != null)
					event.getOldSimulation().getSituation().removeSituationListener(getSituationListener());

				if (event.getNewSimulation() != null)
					event.getNewSimulation().getSituation().addSituationListener(getSituationListener());
			}
		}

		public void finished(StartPauseFinishEvent event) {
		}

		public void paused(StartPauseFinishEvent event) {
		}

		public void started(StartPauseFinishEvent event) {
		}
	};

	public void terminateInThread() {
		_tests = CollectionsHelper.EMPTY_ITERATOR;
		if (_currStartPauseFinishInThreadAble != null) {
			_currStartPauseFinishInThreadAble.terminateInThread();
			setCurrStartPauseFinishInThreadAble(null);

		} else {
			fireFinished();
		}
	}

	private void update(ITestableDeclaration testableDeclaration, Throwable exception) {
		Entry entry = _testRunnerTableModel.getEntry(testableDeclaration);
		if (exception != null) {
			if (entry != null) {
				entry.setLastRunError(exception);
			}
			_errorCount++;
		}

		if (entry != null) {
			entry.setRunFinished(true);
		}

		_runCount++;
	}
}