package de.uniwue.ki.gui.mas.sim.test;

import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.*;
import javax.swing.event.*;

import junit.swingui.StatusLine;

import org.jfree.ui.SortableTable;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.sim.simulation.StartPauseFinishControlPanel;
import de.uniwue.ki.gui.mas.sim.test.TestRunnerTableModel.Entry;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.mas.sim.simulation.StartPauseFinishEvent;
import de.uniwue.ki.mas.sim.simulation.StartPauseFinishListener;
import de.uniwue.ki.mas.sim.situation.SituationEvent;
import de.uniwue.ki.mas.sim.situation.SituationListener;
import de.uniwue.ki.mas.util.declaration.ITestableDeclaration;

public class TestRunner extends JPanelWithGUISeSAmDeclaration {
	
	public static final ImageIcon CURRENT_TEST_ICON = SwingMoreUtilities.createImageIcon("eclipse/currentTest.gif"); 
	public static final ImageIcon POSITIVE_TEST_ICON = SwingMoreUtilities.createImageIcon("eclipse/positiveTest.gif"); 
	public static final ImageIcon NEGATIVE_TEST = SwingMoreUtilities.createImageIcon("eclipse/negativeTest.gif");
	
	
	static class CounterPanel extends JPanel {
		private Icon fErrorIcon = SwingMoreUtilities.createImageIcon("eclipse/negativeTest.gif");
		private JTextField fNumberOfErrors;
		private JTextField fNumberOfRuns;

		private int fTotal;

		public CounterPanel() {
			super(new GridBagLayout());
			fNumberOfErrors = createOutputField(5);
			fNumberOfRuns = createOutputField(9);

			addToGrid(
				this,
				new JLabel(
					"Runs : ",
					SwingMoreUtilities.createImageIcon("eclipse/positiveTest.gif"),
					SwingConstants.LEFT),
				0,
				0,
				1,
				1,
				0.0,
				0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.NONE,
				new Insets(0, 0, 0, 0));
			addToGrid(
				this,
				fNumberOfRuns,
				1,
				0,
				1,
				1,
				0.33,
				0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL,
				new Insets(0, 8, 0, 0));

			addToGrid(
				this,
				new JLabel("Errors:", fErrorIcon, SwingConstants.LEFT),
				2,
				0,
				1,
				1,
				0.0,
				0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.NONE,
				new Insets(0, 8, 0, 0));
			addToGrid(
				this,
				fNumberOfErrors,
				3,
				0,
				1,
				1,
				0.33,
				0.0,
				GridBagConstraints.CENTER,
				GridBagConstraints.HORIZONTAL,
				new Insets(0, 8, 0, 0));
		}

		private JTextField createOutputField(int width) {
			JTextField field = new JTextField("0", width);
			// force a fixed layout to avoid accidental hiding on relayout
			field.setMinimumSize(field.getPreferredSize());
			field.setMaximumSize(field.getPreferredSize());
			field.setHorizontalAlignment(SwingConstants.LEFT);
			field.setFont(StatusLine.BOLD_FONT);
			field.setEditable(false);
			field.setBorder(BorderFactory.createEmptyBorder());
			return field;
		}
		public void reset() {
			setLabelValue(fNumberOfErrors, 0);
			setLabelValue(fNumberOfRuns, 0);
			fTotal = 0;
		}

		public void setErrorValue(int value) {
			setLabelValue(fNumberOfErrors, value);
		}

		private void setLabelValue(JTextField label, int value) {
			label.setText(Integer.toString(value));
		}

		public void setRunValue(int value) {
			fNumberOfRuns.setText(Integer.toString(value) + "/" + fTotal);
		}

		public void setTotal(int value) {
			fTotal = value;
		}
	}

	/**
	* A progress bar showing the green/red status
	*/
	static class ProgressBar extends JProgressBar {
		boolean fError = false;

		public ProgressBar() {
			super();
			setForeground(getStatusColor());
		}

		private Color getStatusColor() {
			if (fError)
				return Color.red;
			return Color.green;
		}

		public void reset() {
			fError = false;
			setForeground(getStatusColor());
			setValue(0);
		}

		public void start(int total) {
			setMaximum(total);
			reset();
		}

		public void step(int value, boolean successful) {
			setValue(value);
			if (!fError && !successful) {
				fError = true;
				setForeground(getStatusColor());
			}
		}
	}

	private static void addToGrid(
		Container parent,
		Component comp,
		int gridx,
		int gridy,
		int gridwidth,
		int gridheight,
		double weightx,
		double weighty,
		int anchor,
		int fill,
		Insets insets) {

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.gridx = gridx;
		constraints.gridy = gridy;
		constraints.gridwidth = gridwidth;
		constraints.gridheight = gridheight;
		constraints.weightx = weightx;
		constraints.weighty = weighty;
		constraints.anchor = anchor;
		constraints.fill = fill;
		constraints.insets = insets;
		parent.add(comp, constraints);
	}

	public static void main(String[] args) {
		final TestRunner testRunner = new TestRunner();
		JFrame frame = new JFrame("JUnit");

		frame.getContentPane().setLayout(new BorderLayout(0, 0));

		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				testRunner.dispose();
			}
		});

		frame.setContentPane(testRunner);
		frame.pack();
		frame.setLocation(200, 200);

		frame.setVisible(true);
	}

	private JTextArea _errorTextArea;

	private ProgressBar _progressIndicator;
	private StartPauseFinishControlPanel _startPauseFinishControlPanel;
	private SortableTable _testTable;
	private CounterPanel _counterPanel;
	private StatusLine _statusLine;

	public TestRunner() {
		initialize();
	}

	private JSplitPane createSplitPane() {
		JSplitPane splitPane =
			new JSplitPane(
				JSplitPane.VERTICAL_SPLIT,
				new JScrollPane(getTestTable()),
				new JScrollPane(
					getErrorTextArea(),
					ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
					ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED));
		splitPane.setOneTouchExpandable(true);
		splitPane.setResizeWeight(0.5);
		splitPane.setDividerLocation(0.5);
		return splitPane;
	}

	public void dispose() {
		getTestRunnerTableModel().remove();
		getAllTestsStartPauseFinish().remove();

		super.dispose();
	}

	private void clearStatus() {
		getStatusLine().clear();
	}

	private StatusLine getStatusLine() {
		if (_statusLine == null) {
			_statusLine = new StatusLine(380);
		}

		return _statusLine;
	}

	private AllTestsStartPauseFinish getAllTestsStartPauseFinish() {
		return (AllTestsStartPauseFinish)getStartPauseFinishControlPanel().getStartPauseFinishInThreadAble();
	}

	private CounterPanel getCounterPanel() {
		if (_counterPanel == null) {
			_counterPanel = new CounterPanel();
		}
		return _counterPanel;
	}

	private JTextArea getErrorTextArea() {
		if (_errorTextArea == null) {
			_errorTextArea = new JTextArea();
		}
		return _errorTextArea;
	}

	private ProgressBar getProgressIndicator() {
		if (_progressIndicator == null) {
			_progressIndicator = new ProgressBar();
		}
		return _progressIndicator;
	}

	private StartPauseFinishControlPanel getStartPauseFinishControlPanel() {
		if (_startPauseFinishControlPanel == null) {
			_startPauseFinishControlPanel = new StartPauseFinishControlPanel();
			final AllTestsStartPauseFinish allTestsStartPauseFinish =
				new AllTestsStartPauseFinish(getTestRunnerTableModel());
			allTestsStartPauseFinish.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					getCounterPanel().setErrorValue(getAllTestsStartPauseFinish().getErrorCount());
					getCounterPanel().setRunValue(getAllTestsStartPauseFinish().getRunCount());
					getTestRunnerTableModel().setCurrEntry(
						getTestRunnerTableModel().getEntry(
							getAllTestsStartPauseFinish().getCurrentTestableDeclaration()));

					getProgressIndicator().step(
						getAllTestsStartPauseFinish().getRunCount(),
						getAllTestsStartPauseFinish().getErrorCount() == 0);

					getTestTable().repaint();
				}
			});
			allTestsStartPauseFinish.setSituationListener(new SituationListener() {
				public void nextTimeStep(final SituationEvent event) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							ITestableDeclaration currentTestableDeclaration =
								getAllTestsStartPauseFinish().getCurrentTestableDeclaration();

							if (currentTestableDeclaration == null)
								return;

							getStatusLine().showInfo(
								"Running '"
									+ currentTestableDeclaration.getName()
									+ "' "
									+ event.getTimeDurationInTicks()
									+ " ticks!");
						}
					});
				}
			});
			allTestsStartPauseFinish.addStartPauseFinishListener(new StartPauseFinishListener() {
				public void finished(StartPauseFinishEvent event) {
					getStatusLine().showInfo("Finished Tests");
					allTestsStartPauseFinish.reset();
					getStartPauseFinishControlPanel().refresh();
				}

				public void paused(StartPauseFinishEvent event) {
				}

				public void started(StartPauseFinishEvent event) {
					getStatusLine().showInfo("Starting Tests");

					getCounterPanel().reset();
					getProgressIndicator().reset();

					getCounterPanel().setTotal(getAllTestsStartPauseFinish().getTotal());
					getProgressIndicator().start(getAllTestsStartPauseFinish().getTotal());

					getErrorTextArea().setText("");
					getTestTable().clearSelection();
				}
			});
			_startPauseFinishControlPanel.setStartPauseFinishInThreadAble(allTestsStartPauseFinish);
		}

		return _startPauseFinishControlPanel;
	}

	private TestRunnerTableModel getTestRunnerTableModel() {
		return (TestRunnerTableModel)getTestTable().getModel();
	}

	private SortableTable getTestTable() {
		if (_testTable == null) {
			_testTable = new SortableTable(new TestRunnerTableModel());
			_testTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
			_testTable.setShowHorizontalLines(true);
			_testTable.setShowVerticalLines(false);
			_testTable.setRowSelectionAllowed(true);
			_testTable.getColumnModel().getColumn(0).setMaxWidth(
				_testTable.getColumnModel().getColumn(0).getPreferredWidth());
			_testTable.getColumnModel().getColumn(2).setMaxWidth(
				_testTable.getColumnModel().getColumn(2).getPreferredWidth());
			_testTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			_testTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					if (e.getValueIsAdjusting())
						return;

					Entry entry = getTestRunnerTableModel().getEntry(getTestTable().getSelectedRow());
					getErrorTextArea().setText("");
					if ((entry != null) && (entry.getLastRunError() != null)) {
						StringWriter stringWriter = new StringWriter();
						stringWriter.write(entry.getLastRunError().toString() + "\n");
						entry.getLastRunError().printStackTrace(new PrintWriter(stringWriter));
						getErrorTextArea().setText(stringWriter.toString());
					}
				}
			});
		}
		return _testTable;
	}

	private void initialize() {
		PanelBuilder panelBuilder =
			new PanelBuilder(
				this,
				new FormLayout(
					"fill:default:grow",
					"default, 2dlu, default, 3dlu, default, 4dlu, default, 2dlu, default, 4dlu, default, 2dlu, fill:default:grow, 0dlu, default"));

		panelBuilder.addSeparator("Process");
		panelBuilder.nextLine(2);
		panelBuilder.add(getProgressIndicator());
		panelBuilder.nextLine(2);
		panelBuilder.add(getCounterPanel(), "1, 5, left, center");
		panelBuilder.nextLine(2);
		panelBuilder.addSeparator("Control");
		panelBuilder.nextLine(2);
		panelBuilder.add(getStartPauseFinishControlPanel());
		panelBuilder.nextLine(2);
		panelBuilder.addSeparator("Tests");
		panelBuilder.nextLine(2);
		panelBuilder.add(createSplitPane());
		panelBuilder.nextLine(2);
		panelBuilder.add(getStatusLine());
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null)
			getTestRunnerTableModel().setSeSAmDeclaration(guiSeSAmDeclaration.getSeSAmDeclaration());
		else
			getTestRunnerTableModel().setSeSAmDeclaration(null);
	}
}