package de.uniwue.ki.gui.mas.sim.userInterface;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JLabel;

import de.uniwue.ki.gui.mas.entity.object.ObjectInstanceDeclarationPanel;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.AbstractDeclarationPanel;
import de.uniwue.ki.gui.mas.util.declaration.DeclarationComboBox;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.model.userInterface.IUserInterfaceDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.simulation.SimulationDeclarationListAndFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationFactory;

/**
 * Creation date: (30.10.2000 16:32:34)
 * @author Christoph Oechslein
 */
public class UserInterfaceDeclarationPanel extends AbstractDeclarationPanel {

	private ObjectInstanceDeclarationPanel ivjJPanelWorldInstanceDeclPanel = null;
	private IUserInterfaceDeclaration _userInterfaceDeclaration = null;
	private DeclarationComboBox _simulationDeclarationComboBox = null;

	public UserInterfaceDeclarationPanel() {
		super();
		initialize();
	}

	public void dispose() {
		super.dispose();

		getSimulationDeclarationComboBox().dispose();
	}

	protected void eventEditSimulation() {
		if ((getGUISeSAmDeclaration() != null) && (getUserInterfaceDeclaration() != null)) {
			getGUISeSAmDeclaration().getPanelActionFactoryManager().callPropertyAction(
				this,
				getUserInterfaceDeclaration().getSimulationDeclaration());
		}
	}

	public IDeclaration getDeclaration() {
		return getUserInterfaceDeclaration();
	}

	public IUserInterfaceDeclaration getUserInterfaceDeclaration() {
		return _userInterfaceDeclaration;
	}

	private DeclarationComboBox getSimulationDeclarationComboBox() {
		if (_simulationDeclarationComboBox == null) {
			_simulationDeclarationComboBox = new DeclarationComboBox();
			_simulationDeclarationComboBox.setEditable(false);

			_simulationDeclarationComboBox
				.addPropertyChangeListener("selectedItem", new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if ((getUserInterfaceDeclaration() != null)) {
						getUserInterfaceDeclaration().setSimulationDeclaration(
							(ISimulationDeclaration)getSimulationDeclarationComboBox().getSelectedItem());
					}
				}
			});

		}
		return _simulationDeclarationComboBox;
	}

	protected IDeclarationFactory getSimulationDeclarationFactory() {
		if (getGUISeSAmDeclaration() == null) {
			return null;

		} else {
			return getGUISeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationFactory();
		}
	}

	private void initialize() {

		setLayout(new java.awt.GridBagLayout());

		java.awt.GridBagConstraints constraintsJLabelSimulation = new java.awt.GridBagConstraints();
		constraintsJLabelSimulation.gridx = 0;
		constraintsJLabelSimulation.gridy = 0;
		constraintsJLabelSimulation.anchor = java.awt.GridBagConstraints.WEST;
		constraintsJLabelSimulation.insets = new java.awt.Insets(0, 0, 0, 4);
		add(new JLabel("Simulation:"), constraintsJLabelSimulation);

		java.awt.GridBagConstraints constraintsDeclarationComboBox1 = new java.awt.GridBagConstraints();
		constraintsDeclarationComboBox1.gridx = 1;
		constraintsDeclarationComboBox1.gridy = 0;
		constraintsDeclarationComboBox1.anchor = java.awt.GridBagConstraints.WEST;
		constraintsDeclarationComboBox1.insets = new java.awt.Insets(0, 0, 0, 4);
		add(getSimulationDeclarationComboBox(), constraintsDeclarationComboBox1);

		java.awt.GridBagConstraints constraintsJButtonSimulation = new java.awt.GridBagConstraints();
		constraintsJButtonSimulation.gridx = 2;
		constraintsJButtonSimulation.gridy = 0;
		constraintsJButtonSimulation.anchor = java.awt.GridBagConstraints.WEST;
		constraintsJButtonSimulation.insets = new java.awt.Insets(0, 3, 0, 0);
		add(new JButton(new AbstractBoundAction("Edit Simulation ...") {
			public void doit() {
				eventEditSimulation();
			}
		}), constraintsJButtonSimulation);
	}

	public void setDeclaration(IDeclaration declaration) {
		setUserInterfaceDeclaration((IUserInterfaceDeclaration)declaration);
	}

	public void setEnabled(boolean newEnabled) {
		super.setEnabled(newEnabled);
	}

	public void setGUISeSAmDeclaration(IGUISeSAmDeclaration guiSeSAmDeclaration) {
		super.setGUISeSAmDeclaration(guiSeSAmDeclaration);

		if (guiSeSAmDeclaration != null) {
			SimulationDeclarationListAndFactory simulationDeclarationListAndFactory =
				guiSeSAmDeclaration
					.getSeSAmModelDeclaration()
					.getSimulationElementsDeclaration()
					.getSimulationDeclarationListAndFactory();
			getSimulationDeclarationComboBox().setDeclarationListModel(
				simulationDeclarationListAndFactory.getSimulationDeclarationList());
			getSimulationDeclarationComboBox().setDeclarationFactory(
				simulationDeclarationListAndFactory.getSimulationDeclarationFactory());
		}
	}

	public void setUserInterfaceDeclaration(IUserInterfaceDeclaration newValue) {
		IUserInterfaceDeclaration oldValue = getUserInterfaceDeclaration();

		if (oldValue != newValue) {
			_userInterfaceDeclaration = newValue;

			if (newValue != null) {
				getSimulationDeclarationComboBox().setSelectedItem(
					getUserInterfaceDeclaration().getSimulationDeclaration());
			}
			firePropertyChange("userInterfaceDeclaration", oldValue, newValue);
		}
	}

}