package de.uniwue.ki.gui.mas.util;

import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;

import junit.framework.Assert;

/**
 * Creation date: (02.07.2001 13:56:15)
 * @author Christoph Oechslein
 */
public class JFrameFactory implements IFrameFactory {
	private static final JFrameFactory instance = new JFrameFactory();

	public static final JFrameFactory getInstance() {
		return instance;
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown) {

		return addFrame(parent, title, component, shouldTheFrameBeShown, WindowConstants.HIDE_ON_CLOSE, null)
			.getRootPane();
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown,
		int windowCloseOperation) {

		return addFrame(parent, title, component, shouldTheFrameBeShown, windowCloseOperation, null)
			.getRootPane();
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown,
		int windowCloseOperation,
		Runnable windowCloseFunction) {

		return addFrame(
			parent,
			title,
			component,
			shouldTheFrameBeShown,
			windowCloseOperation,
			windowCloseFunction)
			.getRootPane();
	}

	public JFrame createFrame(Component parent, String titel, Component comp, Dimension maxSize) {

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(titel);
		Assert.assertNotNull(comp);

		JFrame jFrame = new JFrame();

		jFrame.setTitle(titel);

		if (comp instanceof Container) {
			jFrame.setContentPane((Container)comp);

		} else {

			JPanel jPanel = new JPanel(new BorderLayout());

			jPanel.add(comp, BorderLayout.CENTER);
			// jPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 11, 11));

			jFrame.setContentPane(jPanel);
		}

		jFrame.pack();

		java.awt.Dimension panelSize = null;

		try {
			panelSize = comp.getPreferredSize();
		} catch (Exception e) {
			e.printStackTrace();
			try {
				panelSize = comp.getMinimumSize();
			} catch (Exception e2) {
				e2.printStackTrace();
			}
		}

		if (panelSize != null) {
			if (maxSize == null) {
				jFrame.setSize(panelSize.getSize());
			} else {
				jFrame.setSize(
					(int)Math.min(panelSize.getWidth(), maxSize.getWidth()),
					(int)Math.min(panelSize.getHeight(), maxSize.getHeight()));
			}
		}

		return jFrame;

	}

	protected JFrameFactory() {
		super();
	}

	private JFrame addFrame(
		Component parent,
		String title,
		java.awt.Component component,
		boolean shouldTheFrameBeShown,
		int windowCloseOperation,
		final Runnable windowCloseFunction) {

		JFrame frame = createFrame(parent, title, component);

		frame.setDefaultCloseOperation(windowCloseOperation);

		if (windowCloseFunction != null) {
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosed(WindowEvent e) {
					windowCloseFunction.run();
				}
			});
		}

		if (shouldTheFrameBeShown) {
			frame.show();
		}

		return frame;
	}

	protected JFrame createFrame(Component parent, String title, Component comp) {
		return createFrame(parent, title, comp, Toolkit.getDefaultToolkit().getScreenSize());
	}

	public void addModalPanel(Component parent, String title, JOptionPane optionPane) {
		Dimension maxSize = Toolkit.getDefaultToolkit().getScreenSize();

		optionPane.setMaximumSize(maxSize);

		JDialog dialog = optionPane.createDialog(parent, title);

		dialog.show();

		// JOptionPane.showOptionDialog(parent, comp, title, JOptionPane.DEFAULT_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);
	}
}