/**
 * Created on 26.01.2004
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.mas.util.declaration;

import java.awt.BorderLayout;

import javax.swing.*;

import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.mas.util.declaration.*;

public abstract class AbstractSplittedDeclarationPanel extends AbstractDeclarationPanel {

    private JSplitPane _splitPane;

    private NameDeclarationWithDocumentationPanel _namePanel;

    protected void initialize(IDeclarationListAndFactory lFac) {

        getListPanel().setDeclarationListAndFactory(lFac);

        // if there is no graph yet, add a new one.
        if (getListPanel().getModel().getSize() == 0) {

            IDeclaration decl = lFac.getDeclarationFactory().createDeclaration();
            setDeclaration(decl);
            getListPanel().addItem(decl);
        }

        add(getSplitPane());

    }

    protected void initialize(IDeclarationFactory factory) {

        getListPanel().setDeclarationFactory(factory);

        add(getSplitPane());
    }

    protected JSplitPane getSplitPane() {
        if (_splitPane == null) {

            JPanel rightPanel = new JPanel(new BorderLayout());

            rightPanel.add(getNamePanel(), BorderLayout.NORTH);
            rightPanel.add((JPanel) getDeclarationPanel(), BorderLayout.CENTER);
            rightPanel.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));

            _splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, getListPanel(), rightPanel);

            _splitPane.setOneTouchExpandable(true);
            _splitPane.setBorder(BorderFactory.createEmptyBorder(3, 0, 0, 0));

        }
        
        return _splitPane;
    }

    protected NameDeclarationWithDocumentationPanel getNamePanel() {
        if (_namePanel == null) {
            _namePanel = new NameDeclarationWithDocumentationPanel();
        }

        return _namePanel;
    }

    protected abstract IDeclarationPanel getDeclarationPanel();

    protected abstract DeclarationListPanel getListPanel();

    public void setDeclaration(IDeclaration declaration) {
        getNamePanel().setDeclaration(declaration);
        getNamePanel().setEditable(true);
        getNamePanel().setEnabled(true);

        getDeclarationPanel().setDeclaration(declaration);

        if (declaration != null) {
            checkSplitPaneDividerLocation();
        }

    }

    public void dispose() {

        getDeclarationPanel().dispose();
        getListPanel().dispose();

        super.dispose();
    }

    protected void checkSplitPaneDividerLocation() {
        if (getListPanel().getDeclarationListModel().size() <= 1) {
            getSplitPane().setDividerLocation(0);
        }
    }

    public void setGUISeSAmDeclaration(IGUISeSAmDeclaration decl) {
        getDeclarationPanel().setGUISeSAmDeclaration(decl);

        getListPanel().setGUISeSAmDeclaration(decl);
    }

}
