package de.uniwue.ki.gui.mas.util.declaration;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.mas.model.GUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.model.IGUISeSAmDeclaration;
import de.uniwue.ki.gui.mas.util.declaration.action.*;
import de.uniwue.ki.gui.util.*;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.mas.util.declaration.*;

/**
 * Creation date: (27.06.2001 19:22:08)
 * 
 * @author Christoph Oechslein
 */
public class DeclarationListPanel extends JEditMutableListPanel {

    private IDeclarationPanelFactory _declarationPanelFactory = null;

    private final class MethodDeclarationPanelFactory extends
            AbstractDeclarationPanelFactory {

        public IDeclarationPanel createDeclarationPanel(IDeclaration declaration) {
            IDeclarationPanel declarationPanel = DeclarationListPanel.this
                    .createDeclarationPanel(declaration);

            if (declarationPanel != null) {
                declarationPanel
                        .setGUISeSAmDeclaration(getGUISeSAmDeclaration());
                return declarationPanel;

            } else {
                return null;
            }
        }

        public String getDeclarationTitle(IDeclaration declaration) {
            return DeclarationListPanel.this
                    .getDeclarationFrameTitle(declaration);
        }

    }

    private final IvjEventHandler ivjEventHandler = new IvjEventHandler();

    private IGUISeSAmDeclaration _guiSeSAmDeclarationIntern = null;

    private final class IvjEventHandler implements PropertyChangeListener {

        public void propertyChange(java.beans.PropertyChangeEvent evt) {
            if (evt.getSource() == DeclarationListPanel.this
                    && (evt.getPropertyName().equals("selectedValue")))
                    connEtoC1(evt);
        }
    }

    private final transient ChangeListener _changeListener = new ChangeListener() {

        public void stateChanged(ChangeEvent e) {
            repaint();
        }
    };

    public DeclarationListPanel() {
        super();
        initialize();
    }

    public final void addPanel(IDeclaration declaration,
            IDeclarationPanelFactory declarationPanelFactory) {
        addPanel(declaration, declarationPanelFactory, true,
                shouldUseOkCancelDeclarationPanelFor(declaration));
    }

    public final void addPanel(IDeclaration declaration,
            IDeclarationPanelFactory declarationPanelFactory,
            boolean shouldTheFrameBeShown,
            boolean shouldUseOkCancelDeclarationPanel) {

        GUISeSAmDeclaration
                .getDeclarationFrameFactory(getGUISeSAmDeclaration()).addPanel(
                        this, declaration, declarationPanelFactory,
                        shouldTheFrameBeShown,
                        shouldUseOkCancelDeclarationPanel);
    }

    private void connEtoC1(PropertyChangeEvent propertyChangeEvent) {
        firePropertyChange("selectedDeclaration", propertyChangeEvent
                .getOldValue(), propertyChangeEvent.getNewValue());
    }

    protected IDeclarationPanel createDeclarationPanel(IDeclaration declaration) {
        return null;
        // NOT
        // declarationPanel.setGUISeSAmDeclaration(getGUISeSAmDeclaration());
        // NOT declarationPanel.setDeclaration(declaration);
    }

    public void editItem(Object anObject) {
        IDeclaration declaration = (IDeclaration) anObject;

        if (isEditable()) {
            AbstractBoundAction propertyAction = null;
            if (getGUISeSAmDeclaration() != null) {
                propertyAction = getGUISeSAmDeclaration()
                        .getPanelActionFactoryManager().getPropertyAction(this,
                                declaration);
            }

            if (propertyAction != null) {
                propertyAction.doit();

            } else {
                if (_declarationPanelFactory == null) {
                    _declarationPanelFactory = new MethodDeclarationPanelFactory();
                }

                addPanel(declaration, _declarationPanelFactory);
            }
        }
    }

    public IDeclaration getClickedDeclaration() {
        return (IDeclaration) getClickedValue();
    }

    public IDeclarationFactory getDeclarationFactory() {
        return (IDeclarationFactory) getObjectFactory();
    }

    protected String getDeclarationFrameTitle(IDeclaration declaration) {
        return "Untitled";
    }

    public IDeclarationList getDeclarationListModel() {
        return (IDeclarationList) getMutableListModel();
    }

    protected JPopupMenu getDefaultPopup() {
        if (!isEditable()) {
            return null;

        } else {
            JPopupMenu popupComponent = new JPopupMenu();

            if (isButtonPaneAvailible()) {
                popupComponent.add(AbstractBoundAction
                        .createJMenuItem(new NewDeclarationAction(this)));
                popupComponent.add(AbstractBoundAction
                        .createJMenuItem(new RemoveAllDeclarationAction(this)));
            }
            popupComponent.add(AbstractBoundAction
                    .createJMenuItem(new InspectAction(this)));

            return popupComponent;
        }
    }

    protected JPopupMenu getDefaultPopupFor(
            final IDeclaration declaration) {
        return getGUISeSAmDeclaration().getPanelActionFactoryManager()
                .createPopup(this, declaration);
    }

    public IGUISeSAmDeclaration getGUISeSAmDeclaration() {
        return _guiSeSAmDeclarationIntern;
    }

    public IDeclaration getSelectedDeclaration() {
        return (IDeclaration) getSelectedValue();
    }

    private void initConnections() {
        addPropertyChangeListener(ivjEventHandler);
    }

    private void initialize() {
        setCellRenderer(new NamedCellRendererAdapter(getCellRenderer()));
        initConnections();
        setModel(new DefaultDeclarationList());
        registerKeyStrokes();
    }

    public void popupItem(Object value, int x, int y) {
        JPopupMenu popupMenu;

        if (value == null) {
            popupMenu = getDefaultPopup();
        } else {
            popupMenu = getDefaultPopupFor((IDeclaration) value);
        }

        if (popupMenu != null) {
            popupMenu.show(this, x, y);
        }
    }

    protected void registerKeyStrokes() {
        KeyStroke insertKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_INSERT,
                0);
        KeyStroke delKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);
        KeyStroke f2KeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0);

        ActionListener newButtonActionListener = new AbstractBoundAction() {

            public void doit() {
                try {
                    addNewItem(false);

                } catch (Exception exception) {
                    exception.printStackTrace();
                }
            }
        };

        ActionListener removeButtonActionListener = new AbstractBoundAction() {

            public void doit() {
                Object[] selectedValues = getSelectedValues();
                for (int i = 0; i < selectedValues.length; i++) {
                    removeItem(selectedValues[ i]);
                }
            }
        };

        ActionListener renameButtonActionListener = new AbstractRenameDocumentableDeclarationAction(
                DeclarationListPanel.this) {

            public IDeclaration getDeclaration() {
                return DeclarationListPanel.this.getSelectedDeclaration();
            }

            public void doit() {
                IDeclaration declaration = getDeclaration();

                if ((declaration != null)
                        && (declaration instanceof IDocumentableDeclaration)) {

                    JOptionPane optionPane = new JOptionPane(null,
                            JOptionPane.QUESTION_MESSAGE,
                            JOptionPane.OK_CANCEL_OPTION);

                    optionPane.setWantsInput(true);
                    optionPane.setInitialSelectionValue(declaration.getName());

                    GUISeSAmDeclaration.getDeclarationFrameFactory(
                            getGUISeSAmDeclaration()).addModalPanel(
                            DeclarationListPanel.this, "Rename", optionPane);

                    if (SwingMoreUtilities.userChoiceOption(optionPane,
                            JOptionPane.OK_OPTION)) {

                        String newName = (String) optionPane.getInputValue();
                        if (newName != null) {
                            ((IDocumentableDeclaration) declaration)
                                    .setUsesShortDescription(true);
                            ((IDocumentableDeclaration) declaration)
                                    .setShortDescription(newName);
                        }
                    }
                }
            }
        };

        registerKeyboardAction(newButtonActionListener, insertKeyStroke,
                JComponent.WHEN_IN_FOCUSED_WINDOW);
        registerKeyboardAction(removeButtonActionListener, delKeyStroke,
                JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        registerKeyboardAction(renameButtonActionListener, f2KeyStroke,
                JComponent.WHEN_IN_FOCUSED_WINDOW);
    }

    public void removeItem(Object obj) {
        if (isEditable()) {

            IDeclaration declaration = (IDeclaration) obj;
            if (getDeclarationListModel().canBeRemoved(declaration)) {
                getDeclarationListModel().remove(declaration);
            }
        }
    }

    public void setDeclarationFactory(IDeclarationFactory newDeclarationFactory) {
        IDeclarationFactory oldDeclarationFactory = getDeclarationFactory();

        setObjectFactory(newDeclarationFactory);

        firePropertyChange("declarationFactory", oldDeclarationFactory,
                newDeclarationFactory);
    }

    public void setDeclarationListAndFactory(
            IDeclarationListAndFactory declarationListAndFactory) {
        if (declarationListAndFactory != null) {
            setDeclarationListModel(declarationListAndFactory
                    .getDeclarationList());
            setDeclarationFactory(declarationListAndFactory
                    .getDeclarationFactory());
        } else {
            setDeclarationListModel(null);
            setDeclarationFactory(null);
        }
    }

    public void setDeclarationListModel(IDeclarationList newDeclarationList) {
        IDeclarationList oldDeclarationList = getDeclarationListModel();

        if (oldDeclarationList != null)
                oldDeclarationList.removeChangeListener(_changeListener);

        setMutableListModel(newDeclarationList);

        if (newDeclarationList != null) {
            newDeclarationList.addChangeListener(_changeListener);

            if (!newDeclarationList.isEmpty()) setSelectedIndex(0);
        }

        firePropertyChange("declarationListModel", oldDeclarationList,
                newDeclarationList);
    }

    public void setGUISeSAmDeclaration(IGUISeSAmDeclaration newValue) {
        if (_guiSeSAmDeclarationIntern != newValue) {
            IGUISeSAmDeclaration oldValue = getGUISeSAmDeclaration();
            _guiSeSAmDeclarationIntern = newValue;
            firePropertyChange("gUISeSAmDeclaration", oldValue, newValue);
        }
    }

    public void setSelectedDeclaration(IDeclaration newDeclaration) {
        setSelectedValue(newDeclaration);
    }

    protected boolean shouldUseOkCancelDeclarationPanelFor(
            IDeclaration declaration) {
        return true;
    }

    public static IDeclaration choiceDeclarationList(Component parent,
            IGUISeSAmDeclaration guiSeSAmDeclaration,
            IDeclarationList declarationList, String optionPaneTitle,
            String windowTitle) {

        if (declarationList.size() == 1) {
            return (IDeclaration) declarationList.get(0);

        } else {
            JList list = new JList(declarationList);

            list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

            list.setCellRenderer(new NamedCellRendererAdapter(list
                    .getCellRenderer()));

            list.setSelectedValue(declarationList.get(0), true);

            JOptionPane optionPane = new JOptionPane(list,
                    JOptionPane.QUESTION_MESSAGE, JOptionPane.OK_CANCEL_OPTION);

            GUISeSAmDeclaration.getDeclarationFrameFactory(guiSeSAmDeclaration)
                    .addModalPanel(parent, windowTitle, optionPane);

            if (SwingMoreUtilities.userChoiceOption(optionPane,
                    JOptionPane.OK_OPTION)) {
                return (IDeclaration) list.getSelectedValue();

            } else {
                return null;
            }
        }
    }
}