package de.uniwue.ki.gui.mas.util.declaration;

import java.awt.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import junit.framework.Assert;
import de.uniwue.ki.gui.mas.model.JPanelWithGUISeSAmDeclaration;
import de.uniwue.ki.gui.util.SwingMoreUtilities;
import de.uniwue.ki.gui.util.actions.AbstractBoundAction;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.util.name.IDocumentable;

/**
 * Creation date: (21.08.2001 16:23:09)
 * @author Christoph Oechslein
 */

public class DocumentableDeclarationButton extends JPanelWithGUISeSAmDeclaration {
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private JButton ivjJButton = null;

	public static final ImageIcon DOCUTHERE =
		SwingMoreUtilities.createImageIcon(
			NameDeclarationWithDocumentationPanel.class,
			"documentation-ok.gif");

	public static final ImageIcon DOCUMISSING =
		SwingMoreUtilities.createImageIcon(
			NameDeclarationWithDocumentationPanel.class,
			"documentation-missing.gif");

	private IDocumentableDeclaration _documentableDeclaration;

	private static class DocumentationDialog extends JDialog {
		private final IDocumentable _documentable;
		public DocumentationDialog(Frame frame, IDocumentable documentable) {
			super(frame, "Edit Documentation", true);
			_documentable = documentable;

			initialize();
		}

		private void initialize() {
			getContentPane().setLayout(new BorderLayout());

			final JEditorPane editorPane = addTabbedPanel();

			addButtons(editorPane);

			if (JDialog.isDefaultLookAndFeelDecorated()) {
				boolean supportsWindowDecorations = UIManager.getLookAndFeel().getSupportsWindowDecorations();
				if (supportsWindowDecorations) {
					setUndecorated(true);
					getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);
				}
			}
		}

		public void show() {
			pack();
			SwingMoreUtilities.centerWindow(this);
			super.show();
		}

		private void addButtons(final JEditorPane editorPane) {
			JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
			JButton okButton = new JButton(new AbstractBoundAction("Ok") {
				public void doit() {
					_documentable.setDocumentation(transformHTML(editorPane.getText()));
					hide();
					dispose();
				}
			});
			okButton.setMnemonic('O');
			buttonPanel.add(okButton);
			JButton cancelButton = new JButton(new AbstractBoundAction("Cancel") {
				public void doit() {
					hide();
					dispose();
				}
			});
			cancelButton.setMnemonic('c');
			buttonPanel.add(cancelButton);

			getRootPane().setDefaultButton(okButton);

			getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		}

		private JEditorPane addTabbedPanel() {
			final JEditorPane editorPane = new JEditorPane();

			editorPane.setEditorKit(new HTMLEditorKit());
			HTMLDocument htmlDocument = (HTMLDocument)editorPane.getDocument();
			htmlDocument.setPreservesUnknownTags(true);

			editorPane.setText(transformHTML(_documentable.getDocumentation()));

			final JEditorPane codeEditorPane =
				new JEditorPane("text/plain", transformPlain(_documentable.getDocumentation(), false));

			class MyDocumentListener implements DocumentListener {
				private boolean _inEvent = false;
				public void changedUpdate(DocumentEvent e) {
					changed(e.getDocument());
				}
				private void changed(Document document) {
					if (!_inEvent) {
						_inEvent = true;
						if (document == editorPane.getDocument()) {
							codeEditorPane.setText(transformPlain(editorPane.getText(), true));

						} else {
							Assert.assertSame(document, codeEditorPane.getDocument());
							editorPane.setText(transformHTML(codeEditorPane.getText()));
						}
						_inEvent = false;
					}
				}

				public void insertUpdate(DocumentEvent e) {
					changed(e.getDocument());
				}
				public void removeUpdate(DocumentEvent e) {
					changed(e.getDocument());
				}
			}

			MyDocumentListener myDocumentListener = new MyDocumentListener();

			editorPane.getDocument().addDocumentListener(myDocumentListener);
			codeEditorPane.getDocument().addDocumentListener(myDocumentListener);

			JTabbedPane tabbedPane = new JTabbedPane();
			tabbedPane.add("Normal", new JScrollPane(editorPane));
			tabbedPane.add("HTML", new JScrollPane(codeEditorPane));

			getContentPane().add(tabbedPane, BorderLayout.CENTER);
			return codeEditorPane;
		}

		private String transformPlain(String text, boolean afterHTMLEdit) {
			Matcher matcher =
				Pattern
					.compile(
						".*<html>(.*<head>)?(.*</head>)?(\n|\t|\r| )*.*<body>(.*)</body>.*</html>.*",
						Pattern.DOTALL)
					.matcher(text);
			if (!matcher.find())
				return text;
			else if (afterHTMLEdit)
				return transformPlain(matcher.group(4), afterHTMLEdit).trim().replaceAll("\\n", "<br>\n");
			else
				return transformPlain(matcher.group(4), afterHTMLEdit).trim();
		}

		private String transformHTML(String text) {
			if (text.startsWith("<html>"))
				return text;
			else
				return "<html><body>" + text.trim() + "</body></html>";
		}
	}

	private final class IvjEventHandler implements java.awt.event.ActionListener {
		public void actionPerformed(java.awt.event.ActionEvent e) {
			if (e.getSource() == DocumentableDeclarationButton.this.getJButton())
				connEtoC1(e);
		}
	}

	public DocumentableDeclarationButton() {
		super();
		initialize();
	}

	public static void main(String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			DocumentableDeclarationButton aDocumentableDeclarationButton;
			aDocumentableDeclarationButton = new DocumentableDeclarationButton();
			frame.setContentPane(aDocumentableDeclarationButton);
			frame.setSize(aDocumentableDeclarationButton.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				};
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println(
				"Exception occurred in main() of de.uniwue.ki.gui.mas.model.util.declaration.DocumentableDeclarationButton");
			exception.printStackTrace(System.out);
		}
	}

	public IDeclaration getDeclaration() {
		return getDocumentableDeclaration();
	}

	public void setDeclaration(IDeclaration arg1) {
		setDocumentableDeclaration((IDocumentableDeclaration)arg1);
	}

	private JButton getJButton() {
		if (ivjJButton == null) {
			try {
				ivjJButton = new javax.swing.JButton();
				ivjJButton.setName("Docubutton");
				updateDocubutton();
			} catch (Throwable ivjExc) {
				handleException(ivjExc);
			}
		}
		return ivjJButton;
	}

	private void updateDocubutton() {
		if ((getDocumentableDeclaration() != null)
			&& getDocumentableDeclaration().getDocumentation() != null
			&& getDocumentableDeclaration().getDocumentation() != "") {
			ivjJButton.setIcon(DOCUTHERE);
			ivjJButton.setToolTipText(getDocumentableDeclaration().getDocumentation());
		} else {
			ivjJButton.setIcon(DOCUMISSING);
			ivjJButton.setToolTipText("Enter Documentation");
		}
	}

	private void connEtoC1(java.awt.event.ActionEvent arg1) {
		try {
			this.editDocumentation();
			updateDocubutton();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	protected void editDocumentation() {
		if (getDocumentableDeclaration() != null) {
			new DocumentationDialog(
				(Frame)SwingUtilities.getWindowAncestor(this),
				getDocumentableDeclaration())
				.show();
		}
	}

	private void handleException(Throwable exception) { /* Uncomment the following lines to print uncaught exceptions to stdout */
		exception.printStackTrace(System.out);
	}

	private void initConnections() {
		getJButton().addActionListener(ivjEventHandler);
	}

	private void initialize() {
		try {
			setName("DocumentableDeclarationButton2");
			setLayout(new BoxLayout(this, javax.swing.BoxLayout.X_AXIS));
			setSize(140, 32);
			add(getJButton(), getJButton().getName());
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	public void setEnabled(boolean newSettings) {
		super.setEnabled(newSettings);
		getJButton().setEnabled(newSettings);
	}

	public IDocumentableDeclaration getDocumentableDeclaration() {
		return _documentableDeclaration;
	}

	public void setDocumentableDeclaration(IDocumentableDeclaration documentableDeclaration) {
		_documentableDeclaration = documentableDeclaration;
		updateDocubutton();
	}
}