package de.uniwue.ki.gui.mas.util.declaration;
import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.*;

import de.uniwue.ki.gui.mas.util.JFrameFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (17.09.2001 10:24:25)
 * @author Christoph Oechslein
 */
public class JDeclarationFrameFactory implements IDeclarationFrameFactory {
	private static final JDeclarationFrameFactory _instance = new JDeclarationFrameFactory();

	protected JDeclarationFrameFactory() {
		super();
	}

	public void addModalPanel(Component parent, String title, JOptionPane component) {
		JFrameFactory.getInstance().addModalPanel(parent, title, component);
	}

	public JRootPane addPanel(
		Component parent,
		IDeclaration declaration,
		IDeclarationPanelFactory declarationPanelFactory,
		boolean shouldTheFrameBeShown,
		boolean shouldUseOkCancelDeclarationPanel) {

		IDeclarationPanel declarationPanel = declarationPanelFactory.createDeclarationPanel(declaration);

		if (declarationPanel == null) {
			return null;
		}

		Component componentToShow;

		if (shouldUseOkCancelDeclarationPanel) {

			OkCancelDeclarationPanel okCancelDeclarationPanel = new OkCancelDeclarationPanel();

			okCancelDeclarationPanel.setDeclarationPanel(declarationPanel);
			okCancelDeclarationPanel.setDeclaration(declaration);

			componentToShow = okCancelDeclarationPanel;

		} else {
			declarationPanel.setDeclaration(declaration);

			JPanel panel = new JPanel(new BorderLayout());
			panel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
			panel.add((Component)declarationPanel, BorderLayout.CENTER);

			componentToShow = panel;
		}

		String title = "Edit " + declarationPanelFactory.getDeclarationTitle(declaration);

		JRootPane rootPane = addPanel(parent, title, componentToShow, shouldTheFrameBeShown);

		return rootPane;
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown) {
		return JFrameFactory.getInstance().addPanel(parent, title, component, shouldTheFrameBeShown);
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown,
		int windowCloseOperation) {
		return JFrameFactory.getInstance().addPanel(
			parent,
			title,
			component,
			shouldTheFrameBeShown,
			windowCloseOperation);
	}

	public JRootPane addPanel(
		Component parent,
		String title,
		Component component,
		boolean shouldTheFrameBeShown,
		int windowCloseOperation,
		Runnable windowCloseFunction) {

		return JFrameFactory.getInstance().addPanel(
			parent,
			title,
			component,
			shouldTheFrameBeShown,
			windowCloseOperation,
			windowCloseFunction);
	}

	public static final JDeclarationFrameFactory getInstance() {
		return _instance;
	}
}