package de.uniwue.ki.gui.util;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.dnd.Autoscroll;

import javax.swing.JTree;

/**
 * Creation date: 04.09.2002, 19:14:08
 * @author oechslein
 */
public class JAutoscrollTree extends JTree implements Autoscroll {
    private int _autoScrollMargin = 12;

    /**
     * JDragableTree constructor comment.
     */
    public JAutoscrollTree() {
        super();

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param value java.lang.Object[]
     */
    public JAutoscrollTree(java.lang.Object[] value) {
        super(value);

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param value java.util.Hashtable
     */
    public JAutoscrollTree(java.util.Hashtable value) {
        super(value);

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param value java.util.Vector
     */
    public JAutoscrollTree(java.util.Vector value) {
        super(value);

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param newModel javax.swing.tree.TreeModel
     */
    public JAutoscrollTree(javax.swing.tree.TreeModel newModel) {
        super(newModel);

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param root javax.swing.tree.TreeNode
     */
    public JAutoscrollTree(javax.swing.tree.TreeNode root) {
        super(root);

        initialize();
    }


    /**
     * JDragableTree constructor comment.
     * @param root javax.swing.tree.TreeNode
     * @param asksAllowsChildren boolean
     */
    public JAutoscrollTree(javax.swing.tree.TreeNode root, boolean asksAllowsChildren) {
        super(root, asksAllowsChildren);

        initialize();
    }


    // Ok, we?ve been told to scroll because the mouse cursor is in our
    // scroll zone.
    public void autoscroll(Point p) {
        // Figure out which row we?re on.
        int realrow = getRowForLocation(p.x, p.y);
        Rectangle outer = getBounds();
        // Now decide if the row is at the top of the screen or at the
        // bottom. We do this to make the previous row (or the next
        // row) visible as appropriate. If we?re at the absolute top or
        // bottom, just return the first or last row respectively.
        realrow = (p.y + outer.y <= _autoScrollMargin ? realrow < 1 ? 0 : realrow - 1 : realrow < getRowCount() - 1 ? realrow + 1 : realrow);
        scrollRowToVisible(realrow);
    }


    /**
     * @return Insets
     * @see java.awt.dnd.Autoscroll#getAutoscrollInsets()
     */
    // Calculate the insets for the *JTREE*, not the viewport
    // the tree is in. This makes it a bit messy.
    public Insets getAutoscrollInsets() {
        Rectangle outer = getBounds();
        Rectangle inner = getParent().getBounds();
        return new Insets(
            inner.y - outer.y + _autoScrollMargin,
            inner.x - outer.x + _autoScrollMargin,
            outer.height - inner.height - inner.y + outer.y + _autoScrollMargin,
            outer.width - inner.width - inner.x + outer.x + _autoScrollMargin);
    }



    /**
     * Called whenever the part throws an exception.
     * @param exception java.lang.Throwable
     */
    private void handleException(java.lang.Throwable exception) {

        /* Uncomment the following lines to print uncaught exceptions to stdout */
        System.out.println("--------- UNCAUGHT EXCEPTION ---------");
        exception.printStackTrace(System.out);
    }


private void initialize() {
    try {
        // user code begin {1}
        // user code end
        setName("JDragableTree");
        setSize(152, 180);
    } catch (java.lang.Throwable ivjExc) {
        handleException(ivjExc);
    }
    // user code begin {2}
    setAutoscrolls(true);
    // user code end
}
}