package de.uniwue.ki.gui.util;
import java.awt.Component;
import java.awt.Container;
import java.awt.LayoutManager;

import javax.swing.JPanel;

import junit.framework.Assert;

/**
 * Creation date: (13.07.2001 14:29:12)
 * @author Christoph Oechslein
 */
public class JDisposeablePanel extends JPanel implements IDisposeable {
	private boolean _disposed = false;

	public JDisposeablePanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}

	public JDisposeablePanel(LayoutManager layout) {
		super(layout);
	}

	public JDisposeablePanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}

	public JDisposeablePanel() {
		super();
	}


	public void dispose() {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(false, _disposed);
		_disposed = true;
	}


	protected void disposeAllComponents() {
		disposeAllComponents(this);
	}

	protected static void disposeAllComponents(Container aContainer) {
		for (int i = 0; i < aContainer.getComponentCount(); i++) {
			Component currComponent = aContainer.getComponent(i);

			if (currComponent instanceof IDisposeable) {
				((IDisposeable) currComponent).dispose();
			}

			if (currComponent instanceof JDisposeablePanel) {
				return;
			}

			if (currComponent instanceof Container) {
				disposeAllComponents((Container) currComponent);
			}
		}
	}


	public boolean isDisposed() {
		return _disposed;
	}
}