package de.uniwue.ki.gui.util;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListModel;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.TransferHandler;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.plaf.basic.BasicArrowButton;

import junit.framework.Assert;

import com.jgoodies.forms.factories.Borders;
import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.factories.FormFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.ColumnSpec;
import com.jgoodies.forms.layout.FormLayout;

import de.uniwue.ki.gui.util.bound.ClickedOnElementEvent;
import de.uniwue.ki.gui.util.bound.ClickedOnElementListener;
import de.uniwue.ki.gui.util.dnd.JDnDBetterBoundJList;
import de.uniwue.ki.gui.util.dnd.ObjectDataFlavor;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.factory.IObjectFactory;
import de.uniwue.ki.util.list.DefaultMutableListModel;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;

/**
 * Insert the type's description here.
 * Creation date: (11.10.2000 15:02:47)
 * @author Christoph Oechslein
 */

public class JEditMutableListPanel extends JDisposeablePanel {

	private static final class JEditMutableListPanelTransferHandler extends TransferHandler {
		private static final class CopyArrayTransferable implements Transferable, ClipboardOwner {
			private Object[] _oArray;
			private final DataFlavor[] _flavors;

			public CopyArrayTransferable(DataFlavor dataFlavor, Object[] object) {
				_oArray = deepCopyArray(object);
				_flavors = new DataFlavor[] { dataFlavor };
			}

			private static Object[] deepCopyArray(Object[] oArray) {
				Object[] copyArray = (Object[])oArray.clone();
				Map map = new HashMap();
				for (int i = 0; i < oArray.length; i++) {
					copyArray[i] = ((ICopyable)oArray[i]).copy(map);
				}
				return copyArray;
			}

			public synchronized Object getTransferData(DataFlavor flavor)
				throws UnsupportedFlavorException, IOException {

				if (isDataFlavorSupported(flavor)) {
					return deepCopyArray(_oArray);
				} else {
					throw new UnsupportedFlavorException(flavor);
				}
			}

			public synchronized DataFlavor[] getTransferDataFlavors() {
				return _flavors;
			}

			public boolean isDataFlavorSupported(DataFlavor flavor) {
				return flavor.equals(_flavors[0]);
			}

			public void lostOwnership(Clipboard clipboard, Transferable contents) {
				Assert.assertSame(this, contents);

				lostOwnershipIntern();
			}

			private void lostOwnershipIntern() {
				if (_oArray != null)
					removeCopiedArray(_oArray);

				_oArray = null;
			}

			private void removeCopiedArray(Object[] oArray) {
				for (int i = 0; i < oArray.length; i++) {
					if (oArray[i] instanceof IRemoveable) {
						((IRemoveable)oArray[i]).remove();
					}
				}
			}

			protected void finalize() throws Throwable {
				// since the TransferHandler#exportToClipboard do not allow to add a clipboard listener
				lostOwnershipIntern();
				super.finalize();
			}
		}

		private final DataFlavor _dataFlavor;

		public JEditMutableListPanelTransferHandler(DataFlavor dataFlavor) {

			super();

			_dataFlavor = dataFlavor;
		}

		public JEditMutableListPanelTransferHandler(Class aClass) {

			this(ObjectDataFlavor.getDataFlavor(aClass));

			Assert.assertTrue(aClass.isArray());
		}

		protected Transferable createTransferable(JComponent c) {
			while (!(c instanceof JEditMutableListPanel)) {
				Assert.assertNotNull(c);
				c = (JComponent)c.getParent();
			}

			JEditMutableListPanel editMutableListPanel = (JEditMutableListPanel)c;

			if (editMutableListPanel.isSelectionEmpty())
				return null;

			Object[] selectedValues = editMutableListPanel.getSelectedValues();

			return new CopyArrayTransferable(_dataFlavor, selectedValues);
		}

		public boolean importData(JComponent c, Transferable t) {
			while (!(c instanceof JEditMutableListPanel)) {
				Assert.assertNotNull(c);
				c = (JComponent)c.getParent();
			}

			JEditMutableListPanel dstEditMutableListPanel = (JEditMutableListPanel)c;

			try {
				if (!t.isDataFlavorSupported(_dataFlavor))
					return false;

				Object[] oArray = (Object[])t.getTransferData(_dataFlavor);

				for (int i = 0; i < oArray.length; i++) {
					dstEditMutableListPanel.getMutableListModel().add(oArray[i]);
				}

				return true;

			} catch (UnsupportedFlavorException e) {
				e.printStackTrace(System.err);
				return false;
			} catch (IOException e) {
				e.printStackTrace(System.err);
				return false;
			}
		}

		public boolean canImport(JComponent c, DataFlavor[] transferFlavors) {
			return super.canImport(c, transferFlavors);
		}

		public int getSourceActions(JComponent c) {
			return COPY_OR_MOVE;
		}
	}

	private IObjectFactory _objectFactory;

	private ListSelectionListener selectionListener;

	/* A ListSelectionListener that forwards ListSelectionEvents from 
	 * the selectionModel to the JList ListSelectionListeners.  The 
	 * forwarded events only differ from the originals in that their 
	 * source is the JList instead of the selectionModel itself.
	 */
	private class ListSelectionHandler implements ListSelectionListener, java.io.Serializable {
		public void valueChanged(ListSelectionEvent e) {
			fireSelectionValueChanged(e.getFirstIndex(), e.getLastIndex(), e.getValueIsAdjusting());
		}
	}

	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private JButton ivjJButtonNew = null;
	private JButton ivjJButtonRemove = null;
	private JPanel ivjJPanelButtons = null;
	private JDnDBetterBoundJList ivjJList = null;
	private JScrollPane ivjJScrollPaneJList = null;
	private boolean fieldButtonPaneAvailible = true;
	private boolean _arrowsAvailible = false;

	class IvjEventHandler
		implements ClickedOnElementListener, ActionListener, PropertyChangeListener, ListSelectionListener {
		/**
		 * @param e ActionEvent
		 * @see ActionListener#actionPerformed(ActionEvent)
		 */
		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == JEditMutableListPanel.this.getJButtonNew()) {

				addNewItem(e);
			} else if (e.getSource() == JEditMutableListPanel.this.getJButtonRemove())
				pressedRemoveButton();
		}
		/**
		 * @param event ClickedOnElementEvent
		 * @see ClickedOnElementListener#doubleClick(ClickedOnElementEvent)
		 */
		public void doubleClick(ClickedOnElementEvent event) {
			if (event.getSource() == JEditMutableListPanel.this.getJList())
				editItemIntern();
		}
		/**
		 * @param event ClickedOnElementEvent
		 * @see ClickedOnElementListener#popupClick(ClickedOnElementEvent)
		 */
		public void popupClick(ClickedOnElementEvent event) {
			if (event.getSource() == JEditMutableListPanel.this.getJList())
				popupItemIntern();
		}
		/**
		 * @param evt PropertyChangeEvent
		 * @see PropertyChangeListener#propertyChange(PropertyChangeEvent)
		 */
		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("dnDListenerAdapter"))) {
				PropertyChangeEvent arg12 = evt;
				firePropertyChange("dnDListenerAdapter", arg12.getOldValue(), arg12.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("cellRenderer"))) {
				PropertyChangeEvent arg13 = evt;
				firePropertyChange("cellRenderer", arg13.getOldValue(), arg13.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("clickedPoint"))) {
				PropertyChangeEvent arg11 = evt;
				firePropertyChange("clickedPoint", arg11.getOldValue(), arg11.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("selectedIndex"))) {
				PropertyChangeEvent arg12 = evt;
				firePropertyChange("selectedIndex", arg12.getOldValue(), arg12.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("model"))) {
				PropertyChangeEvent arg12 = evt;
				firePropertyChange("model", arg12.getOldValue(), arg12.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("selectedValue"))) {
				PropertyChangeEvent arg11 = evt;
				firePropertyChange("selectedValue", arg11.getOldValue(), arg11.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("clickedValue"))) {
				PropertyChangeEvent arg11 = evt;
				firePropertyChange("clickedValue", arg11.getOldValue(), arg11.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("selectionEmpty"))) {
				PropertyChangeEvent arg1 = evt;
				firePropertyChange("selectionEmpty", arg1.getOldValue(), arg1.getNewValue());
			}
			if (evt.getSource() == JEditMutableListPanel.this.getJList()
				&& (evt.getPropertyName().equals("selectionModel")))
				firePropertyChange("selectionModel", evt.getOldValue(), evt.getNewValue());
		}

		public void valueChanged(ListSelectionEvent e) {
			if (e.getSource() == JEditMutableListPanel.this.getJList())
				eventJListValueChanged(e);
		}
	}

	public int getVisibleRowCount() {
		return ivjJList.getVisibleRowCount();
	}

	public void setVisibleRowCount(int visibleRowCount) {
		ivjJList.setVisibleRowCount(visibleRowCount);
	}

	public JEditMutableListPanel() {
		super();

		initialize();
	}

	public void addItem(Object newObject) {
		if (isEditable()) {
			getMutableListModel().add(newObject);
			setSelectedValue(newObject);
		}
	}

	/**
	 * Add a listener to the list that's notified each time a change
	 * to the selection occurs.  Listeners added directly to the JList
	 * will have their ListSelectionEvent.getSource() == this JList
	 * (instead of the ListSelectionModel).
	 *
	 * @param listener The ListSelectionListener to add.
	 * @see #getSelectionModel
	 */
	public void addListSelectionListener(ListSelectionListener listener) {
		if (selectionListener == null) {
			selectionListener = new ListSelectionHandler();
			getSelectionModel().addListSelectionListener(selectionListener);
		}

		listenerList.add(ListSelectionListener.class, listener);
	}

	protected void addNewItem(ActionEvent evt) {
		try {
			addNewItem((evt.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void addNewItem(boolean editAfterCreate) throws Exception {
		if (isEditable()) {
			Object newItem = createItem();

			if (newItem != null) {
				addItem(newItem);

				if (editAfterCreate) {
					editItem(newItem);
				}
			}
		}
	}

	public void clearSelection() {
		getJList().clearSelection();
	}

	protected Object createItem() throws Exception {
		if (getObjectFactory() != null)
			return getObjectFactory().createObject();
		else
			return null;
	}

	public void dispose() {
		getJList().dispose();

		super.dispose();
	}

	public void editItem(Object obj) {
	}

	protected void editItemIntern() {
		editItem(getJList().getClickedValue());
	}

	public void enableDnDSupport(DataFlavor dataFlavor) {
		// for copy & paste
		getJList().setTransferHandler(new JEditMutableListPanelTransferHandler(dataFlavor));

		getJList().enableDnDSupport(dataFlavor);
	}

	protected void eventJListValueChanged(ListSelectionEvent listSelectionEvent) {
		return;
	}

	/**
	 * This method notifies JList ListSelectionListeners that
	 * the selection model has changed.  It's used to forward
	 * ListSelectionEvents from the selectionModel to the
	 * ListSelectionListeners added directly to the JList.
	 *
	 * @see #addListSelectionListener
	 * @see #removeListSelectionListener
	 * @see EventListenerList
	 */
	protected void fireSelectionValueChanged(int firstIndex, int lastIndex, boolean isAdjusting) {
		Object[] listeners = listenerList.getListenerList();
		ListSelectionEvent e = null;

		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == ListSelectionListener.class) {
				if (e == null) {
					e = new ListSelectionEvent(this, firstIndex, lastIndex, isAdjusting);
				}
				((ListSelectionListener)listeners[i + 1]).valueChanged(e);
			}
		}
	}

	/**
	 * Method generated to support the promotion of the cellRenderer attribute.
	 * @return javax.swing.ListCellRenderer
	 */
	public javax.swing.ListCellRenderer getCellRenderer() {
		return getJList().getCellRenderer();
	}

	public java.awt.Point getClickedPoint() {
		return SwingUtilities.convertPoint(getJList(), getJList().getClickedPoint(), this);
	}

	public java.lang.Object getClickedValue() {
		return getJList().getClickedValue();
	}

	public DataFlavor getDataFlavor() {
		return getJList().getDataFlavor();
	}

	public int getItemCount() {
		return getJList().getItemCount();
	}
	private JButton getJButtonNew() {
		if (ivjJButtonNew == null) {
			ivjJButtonNew = new javax.swing.JButton();
			ivjJButtonNew.setMnemonic('n');
			ivjJButtonNew.setText("New");
		}

		return ivjJButtonNew;
	}

	private JButton getJButtonRemove() {
		if (ivjJButtonRemove == null) {
			ivjJButtonRemove = new javax.swing.JButton();
			ivjJButtonRemove.setMnemonic('r');
			ivjJButtonRemove.setText("Remove");
		}
		return ivjJButtonRemove;
	}

	protected JDnDBetterBoundJList getJList() {
		if (ivjJList == null) {
			ivjJList = new JDnDBetterBoundJList();
			setVisibleRowCount(10);
		}
		return ivjJList;
	}

	private javax.swing.JPanel getJPanelButtons() {
		if (ivjJPanelButtons == null) {

			ivjJPanelButtons = ButtonBarFactory.buildAddRemoveRightBar(getJButtonNew(), getJButtonRemove());
			ivjJPanelButtons.setBorder(Borders.createEmptyBorder("1dlu,2dlu,2dlu,2dlu"));

			//			ivjJPanelButtons =
			//				new javax.swing.JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT, 5, 2));
			//			ivjJPanelButtons.setName("JPanelButtons");
			//
			//			getJPanelButtons().add(getJButtonNew(), getJButtonNew().getName());
			//			getJPanelButtons().add(getJButtonRemove(), getJButtonRemove().getName());

		}
		return ivjJPanelButtons;
	}

	private void removeArrowsToJPanelButtons() {
		getJPanelButtons().remove(2);
	}

	private void addArrowsToJPanelButtons() {
		BasicArrowButton upBasicArrowButton = new BasicArrowButton(SwingConstants.NORTH);
		BasicArrowButton downBasicArrowButton = new BasicArrowButton(SwingConstants.SOUTH);

		upBasicArrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeOrder(-1);
			}
		});

		downBasicArrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeOrder(1);
			}
		});

		Box updownPanel = Box.createVerticalBox();

		updownPanel.add(upBasicArrowButton);
		updownPanel.add(downBasicArrowButton);

		updownPanel.setPreferredSize(
			new Dimension(
				updownPanel.getPreferredSize().width,
				getJButtonNew().getPreferredSize().height - getJButtonNew().getInsets().bottom / 2));

		((FormLayout)getJPanelButtons().getLayout()).appendColumn(FormFactory.RELATED_GAP_COLSPEC);
		((FormLayout)getJPanelButtons().getLayout()).appendColumn(new ColumnSpec("pref"));
		getJPanelButtons().add(updownPanel, new CellConstraints(6, 1));
	}

	protected void changeOrder(int delta) {
		if (!isSelectionEmpty()) {
			IMutableListModel mutableList = getMutableListModel();
			Object[] selectedValues = getSelectedValues();

			if (delta > 0) {
				for (int i = 0; i < selectedValues.length * 0.5; i++) {
					Object dummy = selectedValues[i];
					selectedValues[i] = selectedValues[selectedValues.length - i - 1];
					selectedValues[selectedValues.length - i - 1] = dummy;
				}
			}

			for (int i = 0; i < selectedValues.length; i++) {
				Object selectedValue = selectedValues[i];
				int oldIndex = mutableList.indexOf(selectedValue);

				if (oldIndex + delta >= mutableList.size()) {
					mutableList.changeIndex(selectedValue, 0);
				} else if (oldIndex + delta < 0) {
					mutableList.changeIndex(selectedValue, mutableList.size() - 1);
				} else {
					mutableList.changeIndex(selectedValue, oldIndex + delta);
				}
			}

			boolean shouldScroll = true;
			setSelectedValues(selectedValues, shouldScroll);
		}
	}

	public void setSelectedValues(Object[] selectedValues, boolean shouldScroll) {
		int[] selectedIndices = new int[selectedValues.length];
		IMutableListModel mutableList = getMutableListModel();
		int smallestIndex = Integer.MAX_VALUE;
		for (int i = 0; i < selectedIndices.length; i++) {
			int selectedIndex = mutableList.indexOf(selectedValues[i]);
			selectedIndices[i] = selectedIndex;
			if (smallestIndex > selectedIndex)
				smallestIndex = selectedIndex;
		}
		getJList().setSelectedIndices(selectedIndices);
		if (shouldScroll && (selectedIndices.length > 0) && (smallestIndex<selectedIndices.length))
			getJList().ensureIndexIsVisible(selectedIndices[smallestIndex]);
	}

	private javax.swing.JScrollPane getJScrollPaneJList() {
		if (ivjJScrollPaneJList == null) {
			ivjJScrollPaneJList = new javax.swing.JScrollPane(getJList());
		}
		return ivjJScrollPaneJList;
	}

	public java.lang.Object[] getListDataAsArray() {
		return getJList().getListDataAsArray();
	}

	public javax.swing.ListModel getModel() {
		return getJList().getModel();
	}

	public IMutableListModel getMutableListModel() {
		ListModel listModel = getModel();

		if (listModel instanceof IMutableListModel)
			return (IMutableListModel)listModel;
		else
			return null;
	}

	public IObjectFactory getObjectFactory() {
		return _objectFactory;
	}

	public int getSelectedIndex() {
		return getJList().getSelectedIndex();
	}

	public Object getSelectedValue() {
		return getJList().getSelectedValue();
	}

	public Object[] getSelectedValues() {
		return getJList().getSelectedValues();
	}

	public boolean isSelectionEmpty() {
		return getJList().isSelectionEmpty();
	}

	public int getSelectionMode() {
		return getJList().getSelectionMode();
	}

	public javax.swing.ListSelectionModel getSelectionModel() {
		return getJList().getSelectionModel();
	}

	private void initConnections() {

		getJButtonNew().addActionListener(ivjEventHandler);
		getJButtonRemove().addActionListener(ivjEventHandler);
		getJList().addPropertyChangeListener(ivjEventHandler);
		getJList().addClickedOnElementListener(ivjEventHandler);
		getJList().addListSelectionListener(ivjEventHandler);
	}

	private void initialize() {
		setLayout(new FormLayout("default:grow", "fill:default:grow, default"));

		CellConstraints cc = new CellConstraints();

		add(getJScrollPaneJList(), cc.xy(1, 1, "fill, fill"));
		add(getJPanelButtons(), cc.xy(1, 2));

		initConnections();

		setModel(new DefaultMutableListModel());

		ToolTipManager.sharedInstance().registerComponent(getJList());
		ToolTipManager.sharedInstance().registerComponent(this);

	}

	public boolean isButtonPaneAvailible() {
		return fieldButtonPaneAvailible;
	}

	public boolean isEditable() {
		return getJList().isEditable();
	}

	public int locationToIndex(java.awt.Point point) {
		return getJList().locationToIndex(point);
	}

	public void popupItem(Object clickedObject, int x, int y) {
	}

	protected void popupItemIntern() {
		if (isEditable()) {
			Point point = getClickedPoint();
			popupItem(getClickedValue(), point.x, point.y);
		}
	}

	protected void pressedRemoveButton() {
		int firstMinSelectionIndex = getJList().getMinSelectionIndex();

		Object[] selectedValues = getJList().getSelectedValues();

		for (int i = 0; i < selectedValues.length; i++) {
			removeItem(selectedValues[i]);
		}

		if (!getJList().isSelectionEmpty()) {
			//
		} else if (getModel().getSize() == 0) {
			clearSelection();
		} else {
			setSelectedValue(
				getModel().getElementAt(
					Math.min(getModel().getSize() - 1, Math.max(firstMinSelectionIndex - 1, 0))));
		}
	}

	public void removeItem(Object obj) {
		if (isEditable()) {
			getMutableListModel().remove(obj);
		}
	}

	public void removeListSelectionListener(ListSelectionListener listener) {
		listenerList.remove(ListSelectionListener.class, listener);
	}

	public void setButtonPaneAvailible(boolean buttonPaneAvailible) {
		if (buttonPaneAvailible != fieldButtonPaneAvailible) {
			if (buttonPaneAvailible) {
				add(getJPanelButtons(), new CellConstraints(1, 2));
				invalidate();
			} else {
				remove(getJPanelButtons());
				invalidate();
			}

			boolean oldValue = fieldButtonPaneAvailible;
			fieldButtonPaneAvailible = buttonPaneAvailible;
			firePropertyChange(
				"buttonPaneAvailible",
				Boolean.valueOf(oldValue),
				Boolean.valueOf(buttonPaneAvailible));
		}
	}

	public void setCellRenderer(javax.swing.ListCellRenderer arg1) {
		getJList().setCellRenderer(arg1);
	}

	public void setEditable(boolean editable) {
		boolean oldValue = isEditable();

		getJList().setEditable(editable);

		setButtonPaneAvailible(editable);

		firePropertyChange("editable", Boolean.valueOf(oldValue), Boolean.valueOf(editable));
	}

	public void setListData(java.util.Iterator it) throws java.lang.IllegalArgumentException {
		getJList().setListData(it);
	}

	public void setListData(java.util.List list) throws java.lang.IllegalArgumentException {
		getJList().setListData(list);
	}

	public void setListDataAsIterator(java.util.Iterator arg1) throws java.lang.IllegalArgumentException {
		getJList().setListData(arg1);
	}

	public void setModel(javax.swing.ListModel arg1) {
		getJList().setModel(arg1);
	}

	public void setMutableListModel(IMutableListModel newMutableListModel) {
		setModel(newMutableListModel);
	}

	public void setMutableListModelAndFactory(IMutableListModelAndFactory mutableListModelAndFactory) {
		setMutableListModel(mutableListModelAndFactory.getMutableListModel());
		setObjectFactory(mutableListModelAndFactory.getObjectFactory());
	}

	public void setObjectFactory(IObjectFactory newObjectFactory) {
		_objectFactory = newObjectFactory;
	}

	public void setSelectedIndex(int arg1) {
		getJList().setSelectedIndex(arg1);
	}

	public void setSelectedValue(java.lang.Object arg1) {
		getJList().setSelectedValue(arg1);
	}

	public void setSelectionMode(int arg1) {
		getJList().setSelectionMode(arg1);
	}

	public void setSelectionModel(javax.swing.ListSelectionModel arg1) {
		getJList().setSelectionModel(arg1);
	}

	public void setArrowsAvailible(boolean arrowsAvailible) {
		boolean oldValue = isArrowsAvailible();

		if (oldValue != arrowsAvailible) {
			if (arrowsAvailible) {
				addArrowsToJPanelButtons();
			} else {
				removeArrowsToJPanelButtons();
			}

			_arrowsAvailible = arrowsAvailible;
		}
	}

	public boolean isArrowsAvailible() {
		return _arrowsAvailible;
	}
}