package de.uniwue.ki.gui.util;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

import de.uniwue.ki.gui.util.lookAndFeel.LookAndFeelManagerAuxiliary;
import de.uniwue.ki.util.name.IDocumentable;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: (17.05.00 12:46:35)
 * @author Christoph Oechslein
 */
public class NamedCellRendererAdapter implements ListCellRenderer {
	private final ListCellRenderer _listCellRenderer;

	public NamedCellRendererAdapter(ListCellRenderer listCellRenderer) {
		super();

		_listCellRenderer = listCellRenderer;
	}

	public Component getListCellRendererComponent(
		JList list,
		Object value,
		int index,
		boolean isSelected,
		boolean cellHasFocus) {

		Component comp;
		if (!(value instanceof INamed)) {
			comp =
				_listCellRenderer.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

		} else {
			INamed named = (INamed)value;

			comp =
				getListCellRendererComponentForNamed(
					list,
					getName(named),
					named,
					index,
					isSelected,
					cellHasFocus);

			if (comp instanceof JLabel) {
				((JLabel)comp).setIcon(getIcon(named));
			}
		}

		if ((comp instanceof JComponent) && LookAndFeelManagerAuxiliary.isCurrentKunststoffLookAndFeel()) {
			((JComponent)comp).setOpaque(isSelected);
		}

		//workaround for swing html bug see Bug Id 4615452 (will not be fixed!!)
		if ((comp instanceof JLabel) && !(list.getSelectionForeground().equals(list.getForeground()))) {
			if (((JLabel)comp).getText().startsWith("<html>")) {
				String oldText = ((JLabel)comp).getText();
				((JLabel)comp).setText("");
				((JLabel)comp).setText(oldText);
			}
		}

		return comp;
	}

	protected Component getListCellRendererComponentForNamed(
		JList list,
		String name,
		INamed value,
		int index,
		boolean isSelected,
		boolean cellHasFocus) {

		if (name == null)
			name = "<no name>";

		String shortName;

		int MAX_SIZE = 100;

		if (name.length() > MAX_SIZE) {
			shortName = name.substring(0, MAX_SIZE - 3) + "...";

		} else {
			shortName = name;
		}

		Component comp =
			_listCellRenderer.getListCellRendererComponent(list, shortName, index, isSelected, cellHasFocus);

		if ((comp instanceof JComponent)) {
			((JComponent)comp).setToolTipText(getToolTipText(value, name));
		}

		return comp;
	}

	protected String getName(INamed value) {
		return value.getName();
	}

	protected String getToolTipText(INamed value, String name) {
		String nameStrong =
			"<html><strong>" + NamedCellRendererAdapter.insertLineBreaks(name, false) + "</strong>";

		if (value instanceof IDocumentable) {
			String documentation = getDocumentation((IDocumentable)value);

			if ((documentation != null) && (documentation.length() > 0))
				return nameStrong + "<P>" + NamedCellRendererAdapter.insertLineBreaks(documentation, false);
			else
				return nameStrong;

		} else {
			return nameStrong;
		}
	}

	protected String getDocumentation(IDocumentable documentable) {
		return documentable.getDocumentation();
	}

	protected Icon getIcon(INamed value) {
		return null;
	}

	public static final String insertLineBreaks(String text) {
		return insertLineBreaks(text, true);
	}

	public static final String insertLineBreaks(String text, boolean insertHTML) {
		return insertLineBreaks(text, 100, insertHTML);
	}

	public static final String insertLineBreaks(String text, int delta) {
		return insertLineBreaks(text, delta, true);
	}

	public static String insertLineBreaks(String text, int delta, boolean insertHTML) {
		if (false && text.length() < delta) {
			return getQuotedString(text);

		} else {
			StringBuffer buff = new StringBuffer();

			if (insertHTML)
				buff.append("<html>");

			for (int i = 0; i < text.length(); i += delta) {
				appendQuotedString(text.substring(i, Math.min(text.length(), i + delta)), buff);

				if (i + delta < text.length())
					buff.append("<br>\n");
			}

			//buff.append("</html>");

			return buff.toString();
		}
	}

	public static String getQuotedString(String str) {
		if (str.startsWith("<html>"))
			return str;
		else {
			StringBuffer buff = new StringBuffer();
			appendQuotedString(str, buff);

			return buff.toString();
		}
	}

	public static void appendQuotedString(String str, StringBuffer buff) {
		if (str.startsWith("<html>")) {
			buff.append(str);
		} else {
			for (int i = 0; i < str.length(); i++) {
				switch (str.charAt(i)) {
					case '\n' :
						buff.append("<br>");
						break;

					case '<' :
						buff.append("&lt;");
						break;

					case '>' :
						buff.append("&gt;");
						break;

					case '"' :
						buff.append("&quot;");
						break;

					//					case '/' :
					//						buff.append("&nbsp;");
					//						break;
					
					//6 Februar Marcus: Primitiv "/" wurde nicht gerendert
					case '/' :
						buff.append("&#047;");
						break;
					//

					case '&' :
						buff.append("&amp;");
						break;
					default :
						buff.append(str.charAt(i));
						break;
				}
			}
		}
	}

}