package de.uniwue.ki.gui.util;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.ImageObserver;
import java.beans.PropertyVetoException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.util.EventListener;
import java.util.LinkedList;

import javax.swing.DesktopManager;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.RootPaneContainer;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.EventListenerList;
import javax.swing.plaf.basic.BasicInternalFrameUI;
import javax.swing.tree.TreePath;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.actions.InspectAction;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (22.06.2001 18:14:08)
 * @author Christoph Oechslein
 */

public class SwingMoreUtilities {

	private static final class NotifyImageObserver implements ImageObserver {
		private final int _successFlags;

		private NotifyImageObserver(int successFlags) {
			super();
			_successFlags = successFlags;
		}

		public boolean imageUpdate(Image img, int infoflags, int x, int y, int width, int height) {
			if ((infoflags & _successFlags) != 0) {
				synchronized (this) {
					this.notifyAll();
				}
				return false;
			} else if ((infoflags & (ABORT | ERROR)) != 0) {
				// error during loading
				synchronized (this) {
					this.notifyAll();
				}
				return false;
			} else {
				return true;
			}
		}
	}

	public static boolean waitForImage(Image image, int width, int height) {

		Toolkit defaultToolkit = Toolkit.getDefaultToolkit();

		if ((defaultToolkit.checkImage(image, width, height, null) & ImageObserver.ALLBITS) != 0) {
			return true;
		}

		ImageObserver imageObserver = new NotifyImageObserver(ImageObserver.ALLBITS);

		defaultToolkit.prepareImage(image, width, height, imageObserver);
		do {
			int flags = defaultToolkit.checkImage(image, width, height, imageObserver);

			if ((flags & ImageObserver.ALLBITS) != 0) {
				return true;
			} else if ((flags & (ImageObserver.ABORT | ImageObserver.ERROR)) != 0) {
				return false;
			}
			synchronized (imageObserver) {
				try {
					imageObserver.wait();
				} catch (InterruptedException e) {
				}
			}
		} while (true);
	}

	public static Image scaleImageIfToLarge(
		Image image,
		int maxWidth,
		int maxHeight,
		ImageObserver observer) {
		int imageWidth = image.getWidth(observer);
		int imageHeight = image.getHeight(observer);

		if (imageWidth > maxWidth) {
			if (imageHeight > maxHeight) {
				if (Math.abs(imageWidth - maxWidth) > Math.abs(imageHeight - maxHeight)) {
					return image.getScaledInstance(maxWidth, -1, Image.SCALE_SMOOTH);

				} else {
					return image.getScaledInstance(-1, maxHeight, Image.SCALE_SMOOTH);
				}

			} else {
				return image.getScaledInstance(maxWidth, -1, Image.SCALE_SMOOTH);
			}

		} else if (imageHeight > maxHeight) {
			Assert.assertTrue(imageWidth <= maxWidth);

			return image.getScaledInstance(-1, maxHeight, Image.SCALE_SMOOTH);

		} else {
			return image;
		}
	}

	public static Image scaleImage(Image image, int width, int height, ImageObserver observer) {
		// int imageWidth = image.getWidth(NULL_IMAGE_OBSERVER);
		// int imageHeight = image.getHeight(NULL_IMAGE_OBSERVER);

		return image.getScaledInstance(width, height, Image.SCALE_SMOOTH);

		//				if (Math.abs(imageWidth - width) > Math.abs(imageHeight - height)) {
		//					return image.getScaledInstance(width, -1, Image.SCALE_SMOOTH);
		//		
		//				} else {
		//					return image.getScaledInstance(-1, height, Image.SCALE_SMOOTH);
		//				}

	}

	public static void closeRootWindow(Component aComponent) {
		RootPaneContainer rootPaneContainer = getRootPaneContainer(aComponent);

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(rootPaneContainer);

		if (rootPaneContainer instanceof JFrame) {
			JFrame jFrame = (JFrame)rootPaneContainer;
			int closeOperation = jFrame.getDefaultCloseOperation();
			switch (closeOperation) {
				case WindowConstants.DO_NOTHING_ON_CLOSE :
					return;
					// break;
				case WindowConstants.DISPOSE_ON_CLOSE :
					jFrame.dispose();
					return;
					// break;
				case WindowConstants.HIDE_ON_CLOSE :
					jFrame.hide();
					return;
					// break;
				case WindowConstants.EXIT_ON_CLOSE :
					System.exit(0);
					return;
					// break;
				default :
					Assert.fail();
			}

		} else if (rootPaneContainer instanceof JInternalFrame) {
			try {
				((JInternalFrame)rootPaneContainer).setClosed(true);
			} catch (java.beans.PropertyVetoException exception) {
			}
			return;
		} else if (rootPaneContainer instanceof Window) {
			((Window)rootPaneContainer).dispose();
			return;
		} else
			Assert.fail();
	}

	public static ImageIcon createImageIcon(String fileName) {
		return createImageIcon(SwingUtilities.class, "/icons/" + fileName, null);
	}

	public static ImageIcon createImageIcon(Class anClass, String fileName) {
		return createImageIcon(anClass, fileName, null);
	}

	public static ImageIcon createImageIcon(Class anClass, String fileName, String description) {
		URL url = anClass.getResource(fileName);
		if (url == null) {
			Assert.fail(
				"Could not load Icon: fileName='" + fileName + "'; class='" + anClass.getName() + "'");
			return null;
		} else if (description == null) {
			return new ImageIcon(url);
		} else {
			return new ImageIcon(url, description);
		}
	}

	public static JOptionPane getExceptionOptionPane(String preamble, Throwable exception) {
		StringWriter stringWriter = new StringWriter();

		PrintWriter printWriter = new PrintWriter(stringWriter);

		if (preamble != null) {
			printWriter.println(preamble);
			printWriter.println();
		}

		printWriter.println(exception.toString());
		printWriter.println();
		exception.printStackTrace(printWriter);

		JScrollPane scrollPane = new JScrollPane();
		JTextArea textArea = new JTextArea();

		printWriter.flush();
		String text = stringWriter.toString(); // stringWriter.getBuffer().toString();

		textArea.setText(text);
		textArea.setColumns(80);
		textArea.setEditable(false);
		textArea.setLineWrap(false);
		textArea.setRows(20);
		//   textArea.setPreferredSize(new Dimension(480, 320));
		//   textArea.setMaximumSize(new Dimension(480, 320));

		scrollPane.setViewportView(textArea);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		//   scrollPane.setMaximumSize(new Dimension(480, 320));
		//scrollPane.setPreferredSize(new Dimension(480, 320));

		return new JOptionPane(scrollPane);
	}

	public static JOptionPane getExceptionOptionPane(Throwable exception) {
		return getExceptionOptionPane(null, exception);
	}

	public static ImageIcon getJavaSunIcon(int size, String name) {
		if (!name.endsWith(".gif")) {
			if (!name.endsWith(size + ""))
				name += Integer.toString(size);

			name += ".gif";
		}

		URL iconURL = SwingMoreUtilities.class.getResource("/icons/jlfgr/toolbarButtonGraphics/" + name);

		Assert.assertNotNull("Image not found: /icons/jlfgr/toolbarButtonGraphics/" + name, iconURL);

		Image image = Toolkit.getDefaultToolkit().getImage(iconURL);

		Image scaledImage = image.getScaledInstance(size, size, Image.SCALE_SMOOTH);

		return new ImageIcon(scaledImage);
	}

	public static EventListener[] getListener(
		EventListenerList eventListenerList,
		Class eventListenerClass) {
		java.util.List list = new LinkedList();
		Object[] listenerList = eventListenerList.getListenerList();

		for (int i = 0; i < listenerList.length; i = i + 2) {
			if (listenerList[i] == eventListenerClass) {
				list.add(listenerList[i + 1]);
			}
		}

		return (EventListener[])list.toArray(new EventListener[list.size()]);
	}

	public static RootPaneContainer getRootPaneContainer(Component aComponent) {
		if (aComponent==null) return null;
		for (Container p = aComponent.getParent(); p != null; p = p.getParent()) {
			if (p instanceof RootPaneContainer) 
				return (RootPaneContainer)p;
			if (p==null) return null;
		}
		return null;
	}

	public static void makeVisible(JInternalFrame internalFrame, JDesktopPane desktopPane) {
		try {
			if (internalFrame.getDesktopPane() == null) {
				desktopPane.add(internalFrame);
			}

			Assert.assertSame(desktopPane, internalFrame.getDesktopPane());

			DesktopManager desktopManager = desktopPane.getDesktopManager();

			internalFrame.setClosed(false);
			internalFrame.show();
			internalFrame.setVisible(true);

			desktopManager.deiconifyFrame(internalFrame);
			desktopManager.openFrame(internalFrame);
			desktopManager.activateFrame(internalFrame);

		} catch (PropertyVetoException e) {
		}
	}

	public final static void centerWindow(Window window) {
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		Insets insets = window.getInsets();

		int realWidth = window.getWidth() + insets.left + insets.right;
		int realHeight = window.getHeight() + insets.top + insets.bottom;

		if (realWidth > screenSize.width)
			realWidth = screenSize.width;

		if (realHeight > screenSize.height)
			realHeight = screenSize.height;

		window.setLocation((screenSize.width - realWidth) / 2, (screenSize.height - realHeight) / 2);

		window.setSize(realWidth - insets.left - insets.right, realHeight - insets.top - insets.bottom);
	}

	/**
	* Positions the specified frame at a relative position in the screen, where 50% is considered
	* to be the center of the screen.
	* @param comp Component
	* @param horizontalPercent The relative horizontal position of the frame (0.0 to 1.0, where 0.5
	*                          is the center of the screen).
	* @param verticalPercent The relative vertical position of the frame (0.0 to 1.0, where 0.5 is
	*                        the center of the screen).
	*/
	public static void positionFrameOnScreen(
		Component comp,
		double horizontalPercent,
		double verticalPercent) {

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		int w = Math.max(screenSize.width - comp.getWidth(), 0);
		int h = Math.max(screenSize.height - comp.getHeight(), 0);
		int x = (int) (horizontalPercent * w);
		int y = (int) (verticalPercent * h);
		comp.setLocation(x, y);
	}

	/**
	 * Positions the specified frame at a random location on the screen while ensuring that the
	 * entire frame is visible (provided that the frame is smaller than the screen).
	 * @param comp Component
	 */
	public static void positionFrameRandomly(Component comp) {
		positionFrameOnScreen(comp, Math.random(), Math.random());
	}

	public static void setEnableAllComponentsRecursive(Container aContainer, boolean isEnabled) {
		for (int i = 0; i < aContainer.getComponentCount(); i++) {
			Component currComponent = aContainer.getComponent(i);
			if (currComponent.isEnabled() != isEnabled) {
				if (currComponent instanceof Container) {
					setEnableAllComponentsRecursive((Container)currComponent, isEnabled);
				}
				currComponent.setEnabled(isEnabled);
			}
		}
	}

	public static void showException(String title, Throwable exception, Component onComponent) {
		showException(title, null, exception, onComponent);
	}

	public static void showException(
		final String title,
		final String preamble,
		final Throwable exception,
		final Component onComponent) {

		if (SwingUtilities.isEventDispatchThread()) {
			showExceptionIntern(title, preamble, exception, onComponent);

		} else {
			try {

				SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						showExceptionIntern(title, preamble, exception, onComponent);
					}
				});

			} catch (InvocationTargetException exception2) {
				throw new RuntimeException(exception2.getTargetException());

			} catch (InterruptedException exception2) {
				throw new RuntimeException(exception2);
			}

		}
	}

	private static void showExceptionIntern(
		String title,
		String preamble,
		Throwable exception,
		Component onComponent) {

		JOptionPane optionPane = getExceptionOptionPane(preamble, exception);

		JDialog dialog = optionPane.createDialog(onComponent, title);

		dialog.setVisible(true);
	}

	public static boolean userChoiceOption(JOptionPane optionPane, int option) {
		Object result = optionPane.getValue();

		if ((result == null) || (result == JOptionPane.UNINITIALIZED_VALUE)) {
			return false;
		} else {
			return option == ((Integer)result).intValue();
		}
	}

	public final static void setSelectionPath(JTree tree, TreePath treePath, boolean shouldScroll) {
		tree.setSelectionPath(treePath);
		if (shouldScroll)
			tree.scrollPathToVisible(treePath);
	}

	public static void invokeNotInSwingThread(String threadName, Runnable runnable) {
		if (SwingUtilities.isEventDispatchThread()) {
			Utils.invokeLater(threadName, runnable);

		} else {
			runnable.run();
		}
	}

	public static JDialog createIndeterminateProgressbarDialog(Component parent, String title) {

		return createIndeterminateProgressbarDialog(
			parent,
			title,
			JOptionPane.DEFAULT_OPTION,
		    JOptionPane.INFORMATION_MESSAGE,			
			null,
			new Object[] { UIManager.getString("OptionPane.cancelButtonText")});
	}

	public static JDialog createIndeterminateProgressbarDialog(
		Component parent,
		String title,
		int optionType,
		int messageType,
		Icon icon,
		Object[] options) {

		JProgressBar jProgressBar = new JProgressBar();
		jProgressBar.setIndeterminate(true);

		JOptionPane jOptionPane = new JOptionPane(jProgressBar, messageType, optionType, icon, options);

		return jOptionPane.createDialog(parent, title);
	}

	public static void invokeWithIndeterminateProgressbar(
		final Container parent,
		String dialogTitle,
		String threadName,
		final Runnable runnable) {
		final JDialog dialog = SwingMoreUtilities.createIndeterminateProgressbarDialog(parent, dialogTitle);

		dialog.setModal(false);

		SwingMoreUtilities.centerWindow(dialog);
		dialog.show();

		SwingMoreUtilities.invokeNotInSwingThread(threadName, new Runnable() {
			public void run() {
				try {
					runnable.run();
				} finally {
					dialog.setVisible(false);
					dialog.hide();
				}
			}
		});
	}
	
	public static void makeVisible(JInternalFrame internalFrame, JDesktopPane desktopPane, boolean addInspectAction) {
		if(addInspectAction)
			addInspectAction(internalFrame);
		makeVisible(internalFrame, desktopPane);
	}
	
	public static void addInspectAction(final JInternalFrame frame){		
		((BasicInternalFrameUI) frame.getUI()).getNorthPane()
			.addMouseListener(
			new MouseAdapter() {
				public void mouseClicked(MouseEvent e) {			
					if (e.getButton() == 3){				
						JPopupMenu popup = new JPopupMenu();
						popup.add(new InspectAction(frame));
						popup.show(frame, e.getX(), e.getY());
					}			
				}
			}
		);	
	}	
	
}