package de.uniwue.ki.gui.util.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JMenuItem;
import javax.swing.SwingConstants;

/**
 * Creation date: (29.05.00 12:55:12)
 * @author Christoph Oechslein
 */
public abstract class AbstractBoundAction extends AbstractAction {

	public AbstractBoundAction() {
		super();
	}

	public AbstractBoundAction(String name) {
		super(name);
	}

	public AbstractBoundAction(String name, Icon icon) {
		super(name, icon);
	}

	public AbstractBoundAction(Icon icon) {
		super(null, icon);
	}

	public final void actionPerformed(ActionEvent evt) {
		doit();
	}

	public static JMenuItem createJMenuItem(Action action) {
		return createJMenuItem(action, false);
	}

	public static JMenuItem createJMenuItem(Action action, boolean isFontBold) {
		JMenuItem menuItem =
			new JMenuItem((String)action.getValue(Action.NAME), (Icon)action.getValue(Action.SMALL_ICON));

		setActionAndFont(menuItem, action, isFontBold);

		return menuItem;
	}

	public abstract void doit();

	public String getDescription() {
		return (String)getValue(Action.LONG_DESCRIPTION);
	}

	public Icon getIcon() {
		return (Icon)getValue(Action.SMALL_ICON);
	}

	public String getName() {
		return (String)getValue(Action.NAME);
	}

	public String getShortDescription() {
		return (String)getValue(Action.SHORT_DESCRIPTION);
	}

	public static void setActionAndFont(JMenuItem menuItem, ActionListener action, boolean isFontBold) {
		menuItem.addActionListener(action);

		setFont(menuItem, isFontBold);
	}

	public static void setActionAndFont(JMenuItem menuItem, Action action, boolean isFontBold) {
		menuItem.setEnabled(action.isEnabled());
		menuItem.addActionListener(action);

		setFont(menuItem, isFontBold);
	}

	public void setDescription(String name) {
		putValue(Action.LONG_DESCRIPTION, name);
	}

	public static void setFont(JMenuItem menuItem, boolean isFontBold) {
		menuItem.setHorizontalTextPosition(SwingConstants.RIGHT);
		menuItem.setVerticalTextPosition(SwingConstants.CENTER);

		java.awt.Font menuItemFont = menuItem.getFont();
		if (isFontBold && !menuItemFont.isBold()) {
			menuItem.setFont(
				new java.awt.Font(
					menuItemFont.getName(),
					menuItemFont.getStyle() & java.awt.Font.BOLD,
					menuItemFont.getSize()));
		} else if (!isFontBold && menuItemFont.isBold()) {
			menuItem.setFont(
				new java.awt.Font(
					menuItemFont.getName(),
					menuItemFont.getStyle() & java.awt.Font.PLAIN,
					menuItemFont.getSize()));
		}
	}

	public void setIcon(Icon icon) {
		putValue(Action.SMALL_ICON, icon);
	}

	public void setName(String name) {
		putValue(Action.NAME, name);
	}

	public void setShortDescription(String name) {
		putValue(Action.SHORT_DESCRIPTION, name);
	}
}