 package de.uniwue.ki.gui.util.bound;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JFormattedTextField;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.IDisposeable;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (08.12.2000 15:22:10)
 * @author Christoph Oechslein
 */
public class BestBoundJTextField extends JFormattedTextField implements IDisposeable {

	private boolean _disposed = false;
	private EventHandler _eventHandler = new EventHandler();
	private String ivjOldValue = null;

	class EventHandler implements ActionListener, KeyListener {
		
		public void keyPressed(KeyEvent arg0) {
		}
		
		public void keyTyped(KeyEvent arg0) {
		}
		
		public void keyReleased(KeyEvent arg0) {
			if (arg0.getSource() == BestBoundJTextField.this)
				setOldValueAndFirePropertyChangeEvent();
		}

		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == BestBoundJTextField.this)
				setOldValueAndFirePropertyChangeEvent();
		}

		public void focusGained(FocusEvent e) {

			if (e.getSource() == BestBoundJTextField.this)
				selectAll();
		}

		public void focusLost(FocusEvent e) {

			if (e.getSource() == BestBoundJTextField.this)
				setOldValueAndFirePropertyChangeEvent();
		}
	}

	public BestBoundJTextField() {
		super();
		initialize();
	}

	public void dispose() {

		ivjOldValue = "";

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(false, isDisposed());

		_disposed = true;
	}

	private String getOldValue() {

		return ivjOldValue;
	}

	private void initConnections() {
		addKeyListener(_eventHandler);
		addActionListener(_eventHandler);
	}

	private void initialize() {

		setName("BetterBoundJTextField");
		setSize(202, 31);

		initConnections();
	}

	public boolean isDisposed() {

		return _disposed;
	}

	/**
	 * Selects all the text in the TextComponent.  Does nothing on a null
	 * or empty document.
	 */
	public void selectAll() {

		if (getDocument() != null) {

			setCaretPosition(getDocument().getLength());
			moveCaretPosition(0);
		}
	}

	private void setOldValue(java.lang.String newValue) {

		if (ivjOldValue != newValue) {

			String oldValue = getOldValue();
			ivjOldValue = newValue;

			firePropertyChange("text", oldValue, newValue);
		}

	}

	protected void setOldValueAndFirePropertyChangeEvent() {

		String oldValue = getOldValue();
		String newValue = getText();

		if (!Utils.equals(oldValue, newValue)) {

			setOldValue(newValue);

			firePropertyChange("text", oldValue, newValue);
		}
	}

	public void setText(String newValue) {

		super.setText(newValue);
		setOldValue(newValue);
	}
}