package de.uniwue.ki.gui.util.bound;
import junit.framework.Assert;

import com.ibm.easyswing.BoundJList;

import de.uniwue.ki.gui.util.IDisposeable;
/**
 * Creation date: (16.06.2000 17:47:39)
 * @author Christoph Oechslein
 */
public class BetterBoundJList extends BoundJList implements IDisposeable {
	private boolean _emptySelection = true;
	private boolean _disposed = false;
	private java.lang.Object fieldClickedValue = null;
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	protected transient ClickedOnElementListener aClickedOnElementListener = null;
	private java.awt.Point fieldClickedPoint = new java.awt.Point();
	private final class IvjEventHandler
		implements java.awt.event.MouseListener, javax.swing.event.ListSelectionListener {
		/**
		 * @param e java.awt.event.MouseEvent
		 * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
		 */
		public void mouseClicked(java.awt.event.MouseEvent e) {
			if (e.getSource() == BetterBoundJList.this)
				connEtoC1(e);
		}
		/**
		 * @param e java.awt.event.MouseEvent
		 * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
		 */
		public void mouseEntered(java.awt.event.MouseEvent e) {
		}
		public void mouseExited(java.awt.event.MouseEvent e) {
		}
		/**
		 * @param e java.awt.event.MouseEvent
		 * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
		 */
		public void mousePressed(java.awt.event.MouseEvent e) {
			if (e.getSource() == BetterBoundJList.this)
				connEtoC2(e);
		}
		/**
		 * @param e java.awt.event.MouseEvent
		 * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
		 */
		public void mouseReleased(java.awt.event.MouseEvent e) {
			if (e.getSource() == BetterBoundJList.this)
				connEtoC3(e);
		}
		/**
		 * @param e javax.swing.event.ListSelectionEvent
		 * @see javax.swing.event.ListSelectionListener#valueChanged(ListSelectionEvent)
		 */
		public void valueChanged(javax.swing.event.ListSelectionEvent e) {
			if (e.getSource() == BetterBoundJList.this)
				connEtoC4(e);
		}
	}

	/**
	 * BetterBoundJList constructor comment.
	 */
	public BetterBoundJList() {
		super();
		initialize();
	}

	/**
	 * BetterBoundJList constructor comment.
	 * @param arg1 java.lang.Object[]
	 */
	public BetterBoundJList(java.lang.Object[] arg1) {
		super(arg1);
	}

	/**
	 * BetterBoundJList constructor comment.
	 * @param arg1 java.util.Vector
	 */
	public BetterBoundJList(java.util.Vector arg1) {
		super(arg1);
	}

	/**
	 * BetterBoundJList constructor comment.
	 * @param arg1 javax.swing.ListModel
	 */
	public BetterBoundJList(javax.swing.ListModel arg1) {
		super(arg1);
	}

	/**
	 *
	 * @param newListener ClickedOnElementListener
	 */
	public void addClickedOnElementListener(ClickedOnElementListener newListener) {
		aClickedOnElementListener =
			ClickedOnElementEventMulticaster.add(aClickedOnElementListener, newListener);
		return;
	}

	private void connEtoC1(java.awt.event.MouseEvent arg1) {
		try {
			this.doubleClicked(arg1);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	private void connEtoC2(java.awt.event.MouseEvent arg1) {
		try {
			this.popupClicked(arg1);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	private void connEtoC3(java.awt.event.MouseEvent arg1) {
		try {
			this.popupClicked(arg1);
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
	}

	/**
	 * connEtoC4:  (BetterBoundJList.listSelection.valueChanged(javax.swing.event.ListSelectionEvent) --> BetterBoundJList.eventValueChanged(Ljavax.swing.event.ListSelectionEvent;)V)
	 * @param arg1 javax.swing.event.ListSelectionEvent
	 */
	/* WARNING: THIS METHOD WILL BE REGENERATED. */
	private void connEtoC4(javax.swing.event.ListSelectionEvent arg1) {
		try {
			// user code begin {1}
			// user code end
			this.eventValueChanged(arg1);
			// user code begin {2}
			// user code end
		} catch (java.lang.Throwable ivjExc) {
			// user code begin {3}
			// user code end
			handleException(ivjExc);
		}
	}

	/**
	 * @see de.uniwue.ki.gui.util.IDisposeable#dispose()
	 */
	public void dispose() {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(false, isDisposed());

		setClickedValue(null);
		// runs in trouble if listModel was removed (is empty)
		// setSelectedValue(null);

		_disposed = true;
	}

	protected void doubleClicked(java.awt.event.MouseEvent mouseEvent) {
		if (!mouseEvent.isConsumed()) {
			if (javax.swing.SwingUtilities.isLeftMouseButton(mouseEvent)
				&& mouseEvent.getClickCount() == 2) {
				updateClickedValue(mouseEvent);
				if (getClickedValue() != null) {
					fireDoubleClick(new ClickedOnElementEvent(this, mouseEvent));
				}
			}
		}
	}

	/**
	 * @param obj Object
	 */
	public void ensureObjectIsVisible(Object obj) {
		ensureIndexIsVisible(indexOfObject(obj));
	}

	protected void eventValueChanged(javax.swing.event.ListSelectionEvent listSelectionEvent) {
		boolean oldValue = _emptySelection;

		_emptySelection = isSelectionEmpty();

		firePropertyChange("selectionEmpty", oldValue, _emptySelection);

	}

	/**
	 * Method to support listener events.
	 * @param event ClickedOnElementEvent
	 */
	protected void fireDoubleClick(ClickedOnElementEvent event) {
		if (aClickedOnElementListener != null) {
			aClickedOnElementListener.doubleClick(event);
		}
	}

	/**
	 * Method to support listener events.
	 * @param event ClickedOnElementEvent
	 */
	protected void firePopupClick(ClickedOnElementEvent event) {
		if (aClickedOnElementListener != null) {
			aClickedOnElementListener.popupClick(event);
		}
	}

	/**
	 * Support for reporting bound property changes.  This method can be called
	 * when a bound property has changed and it will send the appropriate
	 * PropertyChangeEvent to any registered PropertyChangeListeners.
	 */
	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		super.firePropertyChange(propertyName, oldValue, newValue);
	}

	/**
	 * Gets the clickedPoint property (java.awt.Point) value.
	 * @return The clickedPoint property value.
	 * @see #setClickedPoint
	 */
	public java.awt.Point getClickedPoint() {
		return fieldClickedPoint;
	}

	/**
	 * Gets the clickedValue property (java.lang.Object) value.
	 * @return The clickedValue property value.
	 * @see #setClickedValue
	 */
	public java.lang.Object getClickedValue() {
		return fieldClickedValue;
	}

	/**
	 * Called whenever the part throws an exception.
	 * @param exception java.lang.Throwable
	 */
	private void handleException(java.lang.Throwable exception) {

		System.out.println("--------- UNCAUGHT EXCEPTION ---------");
		exception.printStackTrace(System.out);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (16.06.2000 17:48:34)
	 * @author Christoph Oechslein
	 * @param object Object
	 * @return int
	 */
	public int indexOfObject(Object object) {
		javax.swing.ListModel listModel = getModel();
		for (int i = 0; i < listModel.getSize(); i++) {
			if (listModel.getElementAt(i) == object) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Initializes connections
	 * @exception java.lang.Exception The exception description.
	 */
	/* WARNING: THIS METHOD WILL BE REGENERATED. */
	private void initConnections() throws java.lang.Exception {
		// user code begin {1}
		// user code end
		this.addMouseListener(ivjEventHandler);
		this.addListSelectionListener(ivjEventHandler);
	}

	/**
	 * Initialize the class.
	 */
	/* WARNING: THIS METHOD WILL BE REGENERATED. */
	private void initialize() {
		try {
			// user code begin {1}
			// user code end
			setName("BetterBoundJList");
			setSize(160, 120);
			initConnections();
		} catch (java.lang.Throwable ivjExc) {
			handleException(ivjExc);
		}
		// user code begin {2}
		// user code end
	}

	/**
	 * @return boolean
	 * @see de.uniwue.ki.gui.util.IDisposeable#isDisposed()
	 */
	public boolean isDisposed() {
		return _disposed;
	}

	/**
	 * main entrypoint - starts the part when it is run as an application
	 * @param args java.lang.String[]
	 */
	public static void main(java.lang.String[] args) {
		try {
			javax.swing.JFrame frame = new javax.swing.JFrame();
			BetterBoundJList aBetterBoundJList;
			aBetterBoundJList = new BetterBoundJList();
			frame.setContentPane(aBetterBoundJList);
			frame.setSize(aBetterBoundJList.getSize());
			frame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of de.uniwue.ki.gui.util.BetterBoundJList");
			exception.printStackTrace(System.out);
		}
	}

	protected void popupClicked(java.awt.event.MouseEvent mouseEvent) {
		if (!mouseEvent.isConsumed()) {
			if (mouseEvent.isPopupTrigger()) {
				updateClickedValue(mouseEvent);
				firePopupClick(new ClickedOnElementEvent(this, mouseEvent));
			}

		}
	}

	/**
	 *
	 * @param newListener ClickedOnElementListener
	 */
	public void removeClickedOnElementListener(ClickedOnElementListener newListener) {
		aClickedOnElementListener =
			ClickedOnElementEventMulticaster.remove(aClickedOnElementListener, newListener);
		return;
	}

	/**
	 * Sets the clickedPoint property (java.awt.Point) value.
	 * @param clickedPoint The new value for the property.
	 * @see #getClickedPoint
	 */
	protected void setClickedPoint(java.awt.Point clickedPoint) {
		java.awt.Point oldValue = fieldClickedPoint;
		fieldClickedPoint = clickedPoint;
		firePropertyChange("clickedPoint", oldValue, clickedPoint);
	}

	/**
	 * Sets the clickedValue property (java.lang.Object) value.
	 * @param clickedValue The new value for the property.
	 * @see #getClickedValue
	 */
	protected void setClickedValue(java.lang.Object clickedValue) {
		Object oldValue = fieldClickedValue;
		fieldClickedValue = clickedValue;
		firePropertyChange("clickedValue", oldValue, clickedValue);
	}

	/**
	 * @param newIndex int
	 */
	public void setSelectedIndex(int newIndex) {
		super.setSelectedIndex(newIndex);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (16.06.2000 17:48:34)
	 * @author Christoph Oechslein
	 * @param newObject Object
	 */
	public void setSelectedValue(Object newObject) {
		/*    super.setSelectedValue(newObject, true);
		    /*
		    Object oldObject = getSelectedValue();
		    int index = indexOfObject(newObject);
		    if (index != -1) {
		        setSelectedIndex(index);
		        // firePropertyChange("selectedValue", oldObject, newObject)
		        // setSelectedIndex should initiate this
		    }
		    */

		// it seems that the Property event is not correctly handled if newObject == null or oldObject == null
		Object oldObject = getSelectedValue();

		super.setSelectedValue(newObject, true);

		if ((newObject == null) || (oldObject == null)) {
			firePropertyChange("selectedValue", oldObject, newObject);
		}
	}

	protected void updateClickedValue(java.awt.event.MouseEvent mouseEvent) {
		setClickedPoint(mouseEvent.getPoint());
		int index = locationToIndex(mouseEvent.getPoint());
		if (index >= 0) {
			javax.swing.ListModel listModel = getModel();
			Object element = listModel.getElementAt(index);
			setClickedValue(element);
			mouseEvent.consume();
		} else {
			setClickedValue(null);
		}
	}
}