package de.uniwue.ki.gui.util.bound;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.JTextArea;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.IDisposeable;
import de.uniwue.ki.util.Utils;
/**
 * Creation date: (08.12.2000 15:22:10)
 * @author Christoph Oechslein
 */
public class BetterBoundJTextArea extends JTextArea implements IDisposeable {

	private boolean _disposed = false;
	private EventHandler _eventHandler = new EventHandler();
	private String ivjOldValue = null;

	class EventHandler implements ActionListener, FocusListener {

		public void actionPerformed(ActionEvent e) {

			if (e.getSource() == BetterBoundJTextArea.this)
				setOldValueAndFirePropertyChangeEvent();
		}

		public void focusGained(FocusEvent e) {

			if (e.getSource() == BetterBoundJTextArea.this)
				selectAll();
		}

		public void focusLost(FocusEvent e) {

			if (e.getSource() == BetterBoundJTextArea.this)
				setOldValueAndFirePropertyChangeEvent();
		}
	}

	public BetterBoundJTextArea() {

		super();
		initialize();
	}

	public void dispose() {
		ivjOldValue = "";

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(false, isDisposed());

		_disposed = true;
	}

	private String getOldValue() {

		return ivjOldValue;
	}

	private void initConnections() {

		addFocusListener(_eventHandler);
		//addActionListener(_eventHandler);
	}

	private void initialize() {

		setName("BetterBoundJTextField");
		setSize(202, 31);

		initConnections();
	}

	public boolean isDisposed() {

		return _disposed;
	}

	/**
	 * Selects all the text in the TextComponent.  Does nothing on a null
	 * or empty document.
	 */
	public void selectAll() {

		if (getDocument() != null) {

			setCaretPosition(getDocument().getLength());
			moveCaretPosition(0);
		}
	}

	private void setOldValue(java.lang.String newValue) {

		if (ivjOldValue != newValue) {

			String oldValue = getOldValue();
			ivjOldValue = newValue;

			firePropertyChange("text", oldValue, newValue);
		}

	}

	protected void setOldValueAndFirePropertyChangeEvent() {

		String oldValue = getOldValue();
		String newValue = getText();

		if (!Utils.equals(oldValue, newValue)) {

			setOldValue(newValue);

			firePropertyChange("text", oldValue, newValue);
		}
	}

	public void setText(java.lang.String newValue) {

		super.setText(newValue);
		setOldValue(newValue);
	}
}