package de.uniwue.ki.gui.util.bound;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.Format;
import java.text.NumberFormat;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;

import junit.framework.Assert;

/**
 * Creation date: 19.07.2002, 10:10:25
 * 
 * @author oechslein
 */
public class BoundJDoubleTextField extends AbstractBoundTextField {
	private static final NumberFormat _numberFormat;

	static {
		_numberFormat = (NumberFormat) NumberFormat.getNumberInstance().clone();
		_numberFormat.setGroupingUsed(false);
		_numberFormat.setMaximumFractionDigits(Integer.MAX_VALUE);
		_numberFormat.setMaximumIntegerDigits(Integer.MAX_VALUE);
	}

	private final PropertyChangeListener _propertyChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			if ("value".equals(evt.getPropertyName())) {
				Double oldDouble = transformDouble(evt.getOldValue());
				Double newDouble = transformDouble(evt.getNewValue());

				firePropertyChange("double", oldDouble, newDouble);

				firePropertyChange("doubleValue", oldDouble, newDouble);
			}
		}
	};
	
	public BoundJDoubleTextField() {
		this(_numberFormat);

		setInputVerifier(new FormattedTextFieldVerifier());
	}

	public BoundJDoubleTextField(Format format) {
		super(format);

		setValue(new Double(0.0));
		setHorizontalAlignment(SwingConstants.RIGHT);

		addPropertyChangeListener("value", _propertyChangeListener);
	}

	public double getDoubleValue() {
		return transformSmallDouble(getValue());
	}

	public Double getDouble() {
		return transformDouble(getValue());
	}

	public void setDoubleValue(double newValue) {
		setValue(new Double(newValue));
	}

	public void setDouble(Double newValue) {
		setValue(newValue);
	}

	public void selectAll() {
		getCaret().setDot(0);
		getCaret().moveDot(getText().length());
	}

	private static Double transformDouble(Object value) {
		if (value instanceof Double)
			return (Double) value;
		else if (value == null)
			return null;
		else {
			Assert.assertTrue(value instanceof Number);
			return new Double(((Number) value).doubleValue());
		}
	}

	private static double transformSmallDouble(Object value) {
		Double valueSmallDouble = transformDouble(value);

		if (valueSmallDouble == null)
			return 0;
		else
			return valueSmallDouble.doubleValue();
	}

	protected boolean processKeyBinding(KeyStroke ks, KeyEvent e,
			int condition, boolean pressed) {
		boolean processed = super.processKeyBinding(ks, e, condition, pressed);

		if (processed && condition != JComponent.WHEN_IN_FOCUSED_WINDOW
				&& KeyStroke.getKeyStroke("ENTER").equals(ks)) {
			// Returning false will allow further processing
			// of the bindings, eg our parent Containers will get a
			// crack at them.
			return false;
		}
		return processed;
	}
	
	private boolean firstTimeShown = false;
	public void paint(Graphics g) {
		//selectAll();
		super.paint(g);
		if (!firstTimeShown) {
			firstTimeShown = true;
			selectAll();
		}
	}
	
	public static void main(String[] arguments) {
		BoundJDoubleTextField doubleText = new BoundJDoubleTextField();
		doubleText.setDouble(new Double(3.0));
		JFrame frame = new JFrame();
		frame.getContentPane().add(doubleText);
		frame.setSize(new Dimension(50, 50));
		frame.show();
		//doubleText.selectAll();
	}
}