package de.uniwue.ki.gui.util.bound;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.Format;
import java.text.NumberFormat;

import javax.swing.SwingConstants;

/**
 * Creation date: 19.07.2002, 10:10:25
 * @author oechslein
 */
public class BoundJIntegerTextField extends AbstractBoundTextField {

	private static final NumberFormat _numberFormat;

	static {
		_numberFormat = (NumberFormat)NumberFormat.getIntegerInstance().clone();
		_numberFormat.setGroupingUsed(false);
	}

	private final PropertyChangeListener _propertyChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent evt) {
			if ("value".equals(evt.getPropertyName())) {
				Integer oldInteger = transformInteger(evt.getOldValue());
				Integer newInteger = transformInteger(evt.getNewValue());

				firePropertyChange("integer", oldInteger, newInteger);

				firePropertyChange("integerValue", oldInteger, newInteger);
			}
		}
	};

	public BoundJIntegerTextField() {
		this(_numberFormat);

		setInputVerifier(new FormattedTextFieldVerifier());
	}

	public BoundJIntegerTextField(Format format) {
		super(format);

		setValue(new Integer(0));
		setHorizontalAlignment(SwingConstants.RIGHT);

		addPropertyChangeListener("value", _propertyChangeListener);
	}

	public int getIntegerValue() {
		return transformInt(getValue());
	}

	public Integer getInteger() {
		return transformInteger(getValue());
	}

	public void setIntegerValue(int newValue) {
		setValue(new Integer(newValue));
	}

	public void setInteger(Integer newValue) {
		setValue(newValue);
	}

	private static Integer transformInteger(Object value) {
		if (value instanceof Integer)
			return (Integer)value;
		else if (value instanceof Number)
			return new Integer(((Number)value).intValue());
		else if (value == null)
			return null;
		else
			return Integer.valueOf((String)value);
	}

	private static int transformInt(Object value) {
		Integer valueInt = transformInteger(value);

		if (valueInt == null)
			return 0;
		else
			return valueInt.intValue();
	}
}
