package de.uniwue.ki.gui.util.bound;/**
 * This is the event multicaster class to support the ClickedOnElementListenerinterface.
 * @author oechslein
 */
public class ClickedOnElementEventMulticaster
   extends java.awt.AWTEventMulticaster
   implements ClickedOnElementListener {

	/**
	 * Add new listener to support multicast events.
	 * @param a ClickedOnElementListener
	 * @param b ClickedOnElementListener
	 * @return ClickedOnElementListener
	 */
	public static ClickedOnElementListener add(ClickedOnElementListener a, ClickedOnElementListener b) {
	   return (ClickedOnElementListener) addInternal(a, b);
	}

	/**
	 * Remove listener to support multicast events.
	 * @param l ClickedOnElementListener
	 * @param oldl ClickedOnElementListener
	 * @return ClickedOnElementListener
	 */
	public static ClickedOnElementListener remove(ClickedOnElementListener l, ClickedOnElementListener oldl) {
	   if (l == oldl || l == null)
	      return null;
	   if (l instanceof ClickedOnElementEventMulticaster)
	      return (ClickedOnElementListener) ((ClickedOnElementEventMulticaster) l).remove(oldl);
	   return l;
	}

	/**
	 *
	 * @param event ClickedOnElementEvent
	 * @see de.uniwue.ki.gui.util.bound.ClickedOnElementListener#doubleClick(ClickedOnElementEvent)
	 */
	public void doubleClick(ClickedOnElementEvent event) {
	   ((ClickedOnElementListener) a).doubleClick(event);
	   ((ClickedOnElementListener) b).doubleClick(event);
	}

	/**
	 *
	 * @param event ClickedOnElementEvent
	 * @see de.uniwue.ki.gui.util.bound.ClickedOnElementListener#popupClick(ClickedOnElementEvent)
	 */
	public void popupClick(ClickedOnElementEvent event) {
	   ((ClickedOnElementListener) a).popupClick(event);
	   ((ClickedOnElementListener) b).popupClick(event);
	}

	/**
	 * Constructor to support multicast events.
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected ClickedOnElementEventMulticaster(java.util.EventListener a, java.util.EventListener b) {
	   super(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 * @return java.util.EventListener
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected static java.util.EventListener addInternal(java.util.EventListener a, java.util.EventListener b) {
	   if (a == null)
	      return b;
	   if (b == null)
	      return a;
	   return new ClickedOnElementEventMulticaster(a, b);
	}

	/**
	 *
	 * @return java.util.EventListener
	 * @param oldl ClickedOnElementListener
	 */
	protected java.util.EventListener remove(ClickedOnElementListener oldl) {
	   if (oldl == a)
	      return b;
	   if (oldl == b)
	      return a;
	   java.util.EventListener a2 = removeInternal(a, oldl);
	   java.util.EventListener b2 = removeInternal(b, oldl);
	   if (a2 == a && b2 == b)
	      return this;
	   return addInternal(a2, b2);
	}
}