package de.uniwue.ki.gui.util.dnd;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetListener;
/**
 * Creation date: (08.05.2001 15:09:12)
 * @author Christoph Oechslein
 */
public class DragAndDropSupport {

   private final DragSourceListener _dragSourceListener;
   private final DropTargetListener _dropTargetListener;
   private final DragGestureListenerAdapter _dragGestureListener;

   private final DropTarget _dropTarget;
   private final DragSource _dragSource;

	/**
	 * @param comp java.awt.Component
	 */
	public DragAndDropSupport(java.awt.Component comp) {
	   this(comp, new DragSourceListenerAdapter(), new DropTargetListenerAdapter(), new DragGestureListenerAdapter());
	}

	/**
	 * @param comp java.awt.Component
	 * @param defaultDragGestureRecognizerActions int
	 * @param dragSourceListener DragSourceListener
	 * @param dropTargetListener DropTargetListener
	 * @param dragGestureListener DragGestureListenerAdapter
	 */
	public DragAndDropSupport(
	   java.awt.Component comp,
	   int defaultDragGestureRecognizerActions,
	   DragSourceListener dragSourceListener,
	   DropTargetListener dropTargetListener,
	   DragGestureListenerAdapter dragGestureListener) {
	
	   super();
	
	   _dragSourceListener = dragSourceListener;
	   _dropTargetListener = dropTargetListener;
	   _dragGestureListener = dragGestureListener;
	
	   _dropTarget = new DropTarget(comp, _dropTargetListener);
	   _dragSource = new DragSource();
	
	   _dragGestureListener.setDragAndDropSupport(this);
	   _dragSource.createDefaultDragGestureRecognizer(comp, defaultDragGestureRecognizerActions, _dragGestureListener);
	}

	public DragAndDropSupport(
	   java.awt.Component comp,
	   DragSourceListener dragSourceListener,
	   DropTargetListener dropTargetListener,
	   DragGestureListenerAdapter dragGestureListener) {
	
	   this(comp, DnDConstants.ACTION_COPY_OR_MOVE, dragSourceListener, dropTargetListener, dragGestureListener);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (08.05.2001 15:28:30)
	 * @return de.uniwue.ki.gui.generic.dnd.DragGestureListenerAdapter
	 */
	public final DragGestureListenerAdapter getDragGestureListener() {
	   return _dragGestureListener;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (08.05.2001 15:28:30)
	 * @return java.awt.dnd.DragSource
	 */
	public final java.awt.dnd.DragSource getDragSource() {
	   return _dragSource;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (08.05.2001 15:28:30)
	 * @return java.awt.dnd.DragSourceListener
	 */
	public final java.awt.dnd.DragSourceListener getDragSourceListener() {
	   return _dragSourceListener;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (08.05.2001 15:28:30)
	 * @return java.awt.dnd.DropTarget
	 */
	public final java.awt.dnd.DropTarget getDropTarget() {
	   return _dropTarget;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (08.05.2001 15:28:30)
	 * @return java.awt.dnd.DropTargetListener
	 */
	public final java.awt.dnd.DropTargetListener getDropTargetListener() {
	   return _dropTargetListener;
	}
}