package de.uniwue.ki.gui.util.dnd;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;
/**
 * Creation date: (08.05.2001 14:43:06)
 * @author Christoph Oechslein
 */
public class DropTargetListenerAdapter implements DropTargetListener {

	/**
	 * DropTargetListenerAdapter constructor comment.
	 */
	public DropTargetListenerAdapter() {
	   super();
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener
	 */
	public void dragEnter(DropTargetDragEvent dtde) {
	   dtde.acceptDrag(DnDConstants.ACTION_COPY_OR_MOVE);
	}

	/**
	 * @see java.awt.dnd.DropTargetListener
	 */
	public void dragExit(DropTargetEvent dte) {
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener
	 */
	public void dragOver(DropTargetDragEvent dtde) {
	}

	/**
	 * @param event DropTargetDropEvent
	 * @see java.awt.dnd.DropTargetListener
	 */
	public void drop(DropTargetDropEvent event) {
	   try {
	      Transferable transferable = event.getTransferable();
	
	      // we accept only Strings
	      if (isTransferableOk(event, transferable)) {
	
	         event.acceptDrop(DnDConstants.ACTION_MOVE);
	
	         Object object = getObject(event, transferable);
	         if (object != null) {
	            if (objectDropped(event, object)) {
	               event.getDropTargetContext().dropComplete(true);
	            }
	         }
	      } else {
	         event.rejectDrop();
	      }
	   } catch (IOException exception) {
	      exception.printStackTrace();
	      System.err.println("Exception" + exception.getMessage());
	      event.rejectDrop();
	   } catch (UnsupportedFlavorException ufException) {
	      ufException.printStackTrace();
	      System.err.println("Exception" + ufException.getMessage());
	      event.rejectDrop();
	   }
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener
	 */
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}

	protected Object getObject(DropTargetDropEvent event, Transferable transferable)
	   throws IOException, UnsupportedFlavorException {
	
	   return (String) transferable.getTransferData(DataFlavor.stringFlavor);
	}

	protected boolean isTransferableOk(DropTargetDropEvent event, Transferable transferable) {
	   return transferable.isDataFlavorSupported(DataFlavor.stringFlavor);
	}

	protected boolean objectDropped(DropTargetDropEvent event, Object object) {
	   return true;
	}
}