package de.uniwue.ki.gui.util.dnd;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.io.IOException;
import java.util.Hashtable;

import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.list.IMutableListModel;
/**
 * Creation date: (08.05.2001 16:04:50)
 * @author Christoph Oechslein
 */
public class MutableListModelInJDnDBetterBoundJListDnDListenerAdapter implements IDnDListenerAdapter {
	private final DropTarget _dropTarget;
	private final DragSource _dragSource;

	public MutableListModelInJDnDBetterBoundJListDnDListenerAdapter(JDnDBetterBoundJList dnDBetterBoundJList) {
		this(dnDBetterBoundJList, DnDConstants.ACTION_COPY_OR_MOVE);

	}

	public MutableListModelInJDnDBetterBoundJListDnDListenerAdapter(
		JDnDBetterBoundJList dnDBetterBoundJList,
		int defaultDragGestureRecognizerActions) {

		super();

		_dragSource = new DragSource();

		_dragSource.createDefaultDragGestureRecognizer(
			dnDBetterBoundJList,
			defaultDragGestureRecognizerActions,
			this);
		_dropTarget = new DropTarget(dnDBetterBoundJList, this);
	}

	private boolean objectDroppedCopy(
		ICopyable transferedObject,
		IMutableListModel transferedMutableListModel,
		int newIndex) {

		IMutableListModel targetMutableListModel = getMutableListModel();

		Object copyOfTransferedObject = transferedObject.copy(new Hashtable());

		if (!getMutableListModel().accept(copyOfTransferedObject)) {
			return false;

		} else {

			targetMutableListModel.add(newIndex, copyOfTransferedObject);

			return true;
		}
	}

	private boolean objectDroppedMove(
		ICopyable transferedObject,
		IMutableListModel transferedMutableListModel,
		int newIndex) {

		IMutableListModel targetMutableListModel = getMutableListModel();

		if (!getMutableListModel().accept(transferedObject)) {
			return false;

		} else if (transferedMutableListModel == targetMutableListModel) {
			targetMutableListModel.changeIndex(transferedObject, newIndex);

			return true;

		} else {

			transferedMutableListModel.moveTo(transferedObject, targetMutableListModel, newIndex);

			return true;
		}
	}

	/**
	 * @param dsde DragSourceDropEvent
	 * @see java.awt.dnd.DragSourceListener#dragDropEnd(DragSourceDropEvent)
	 */
	public void dragDropEnd(DragSourceDropEvent dsde) {
		if (dsde.getDropSuccess()) {
			dragDropEndWithSuccess(dsde);
		}
	}

	private void dragDropEndWithSuccess(DragSourceDropEvent dsde) {
	}

	/**
	 * @param e DragSourceDragEvent
	 * @see java.awt.dnd.DragSourceListener#dragEnter(DragSourceDragEvent)
	 */
	public void dragEnter(DragSourceDragEvent e) {

		DragSourceContext context = e.getDragSourceContext();
		//intersection of the users selected action, and the source and target actions
		int myaction = e.getDropAction();
		if ((myaction & DnDConstants.ACTION_COPY) != 0) {
			context.setCursor(DragSource.DefaultCopyDrop);

		} else if ((myaction & DnDConstants.ACTION_MOVE) != 0) {
			context.setCursor(DragSource.DefaultMoveDrop);

		} else {
			context.setCursor(DragSource.DefaultCopyNoDrop);
		}
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener#dragEnter(DropTargetDragEvent)
	 */
	public void dragEnter(DropTargetDragEvent dtde) {

		dtde.acceptDrag(dtde.getDropAction());

		if (isDropOk(dtde)) {
			dtde.acceptDrag(dtde.getDropAction());
		} else {
			dtde.rejectDrag();
		}
	}

	/**
	 * @param e DragSourceEvent
	 * @see java.awt.dnd.DragSourceListener#dragExit(DragSourceEvent)
	 */
	public void dragExit(DragSourceEvent e) {
	}

	/**
	 * @param dte DropTargetEvent
	 * @see java.awt.dnd.DropTargetListener#dragExit(DropTargetEvent)
	 */
	public void dragExit(DropTargetEvent dte) {
	}

	/**
	 * @param dge DragGestureEvent
	 * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(DragGestureEvent)
	 */
	public void dragGestureRecognized(DragGestureEvent dge) {
		Object dragObject = getGestureRecognizedObject(dge);
		if (dragObject != null) {
			Transferable transferable = getTransferable(dge, dragObject);

			if (DragSource.isDragImageSupported()) {
				getDragSource().startDrag(dge, getDragCursor(dge), transferable, this);

			} else {
				getDragSource().startDrag(dge, getDragCursor(dge), transferable, this);
			}
		}
	}

	/**
	 * @param dsde DragSourceDragEvent
	 * @see java.awt.dnd.DragSourceListener#dragOver(DragSourceDragEvent)
	 */
	public void dragOver(DragSourceDragEvent dsde) {
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener#dragOver(DropTargetDragEvent)
	 */
	public void dragOver(DropTargetDragEvent dtde) {
		if (isDropOk(dtde)) {
			dtde.acceptDrag(dtde.getDropAction());
		} else {
			dtde.rejectDrag();
		}
	}

	/**
	 * @param event DropTargetDropEvent
	 * @see java.awt.dnd.DropTargetListener#drop(DropTargetDropEvent)
	 */
	public void drop(DropTargetDropEvent event) {
		try {
			Transferable transferable = event.getTransferable();

			if (isTransferableOk(event, transferable)) {
				event.acceptDrop(event.getDropAction());

				Object[] object = getObject(event, transferable);
				if (object != null) {
					if (objectDropped(event, object)) {
						event.getDropTargetContext().dropComplete(true);

						return;
					}
				}

				event.getDropTargetContext().dropComplete(false);

			} else {
				event.rejectDrop();
			}

		} catch (IOException exception) {
			exception.printStackTrace();
			System.err.println("Exception" + exception.getMessage());
			event.rejectDrop();

		} catch (UnsupportedFlavorException ufException) {
			ufException.printStackTrace();
			System.err.println("Exception" + ufException.getMessage());
			event.rejectDrop();
		}
	}

	/**
	 * @param dsde DragSourceDragEvent
	 * @see java.awt.dnd.DragSourceListener#dropActionChanged(DragSourceDragEvent)
	 */
	public void dropActionChanged(DragSourceDragEvent dsde) {
	}

	/**
	 * @param dtde DropTargetDragEvent
	 * @see java.awt.dnd.DropTargetListener#dropActionChanged(DropTargetDragEvent)
	 */
	public void dropActionChanged(DropTargetDragEvent dtde) {
	}

	private final JDnDBetterBoundJList getDnDBetterBoundJList() {
		return (JDnDBetterBoundJList)_dropTarget.getComponent();
	}

	private java.awt.Cursor getDragCursor(DragGestureEvent dge) {
		if ((DnDConstants.ACTION_MOVE & dge.getDragAction()) != 0) {
			return DragSource.DefaultMoveDrop;

		} else if ((DnDConstants.ACTION_COPY & dge.getDragAction()) != 0) {
			return DragSource.DefaultCopyDrop;

		} else
			return DragSource.DefaultCopyNoDrop;
	}

	/**
	 * @return DragSource
	 */
	public final DragSource getDragSource() {
		return _dragSource;
	}

	/**
	 * @return DropTarget
	 */
	public final DropTarget getDropTarget() {
		return _dropTarget;
	}

	private Object getGestureRecognizedObject(DragGestureEvent dge) {
		java.awt.Point dragPoint = dge.getDragOrigin();

		int index = getDnDBetterBoundJList().locationToIndex(dragPoint);
		if (index >= 0) {
			return getMutableListModel().get(index);
		} else {
			return null;
		}
	}

	private final IMutableListModel getMutableListModel() {
		return ((IMutableListModel)getDnDBetterBoundJList().getModel());
	}

	private Transferable getTransferable(DragGestureEvent dge, Object transferable) {
		return new ObjectWithMutableListModelTransferable(
			getDnDBetterBoundJList().getDataFlavor(),
			transferable,
			getMutableListModel());
	}

	private boolean isDropOk(DropTargetDragEvent e) {
		return getDnDBetterBoundJList().isEditable()
			&& (e.isDataFlavorSupported(getDnDBetterBoundJList().getDataFlavor()))
			&& ((e.getSourceActions() & DnDConstants.ACTION_COPY_OR_MOVE) != 0);
	}

	private boolean isTransferableOk(DropTargetDropEvent event, Transferable transferable) {
		return transferable.isDataFlavorSupported(getDnDBetterBoundJList().getDataFlavor());
	}

	private Object[] getObject(DropTargetDropEvent event, Transferable transferable)
		throws IOException, UnsupportedFlavorException {

		return (Object[])transferable.getTransferData(getDnDBetterBoundJList().getDataFlavor());
	}

	private boolean objectDropped(DropTargetDropEvent event, Object[] objectArray) {
		if (objectArray == null)
			return false;

		if (objectArray.length != 2)
			return false;

		ICopyable transferedObject = (ICopyable)objectArray[0];
		IMutableListModel transferedMutableListModel = (IMutableListModel)objectArray[1];

		java.awt.Point dropPoint = event.getLocation();
		int index = getDnDBetterBoundJList().locationToIndex(dropPoint);

		if (index == -1) {
			index = Math.max(0, getMutableListModel().size() - 1);
		}

		if ((event.getDropAction() & DnDConstants.ACTION_MOVE) != 0) {
			return objectDroppedMove(transferedObject, transferedMutableListModel, index);

		} else if ((event.getDropAction() & DnDConstants.ACTION_COPY) != 0) {
			return objectDroppedCopy(transferedObject, transferedMutableListModel, index);

		} else {
			return false;
		}

	}
}