package de.uniwue.ki.gui.util.dnd;
import java.awt.datatransfer.DataFlavor;
import java.io.Serializable;
import java.util.Map;
import java.util.WeakHashMap;

import junit.framework.Assert;
/**
 * Creation date: (10.05.2001 13:55:32)
 * @author Christoph Oechslein
 */
public class ObjectDataFlavor extends DataFlavor {
	protected static transient final Map objectDataFlavorMap = new WeakHashMap();

	/**
	 * @param representationClass Class
	 */
	protected ObjectDataFlavor(Class representationClass) {
		super(
			javaJVMLocalObjectMimeType + ";class=" + representationClass.getName(),
			representationClass.getName() + " DataFlavor");

		if (Assert.ASSERTION_ON)
			Assert.assertTrue(Serializable.class.isAssignableFrom(representationClass));
	}

	/**
	 * Compare this DataFlavor against another DataFlavor object
	 * both getMimeType() and representationClass are considered
	 *
	 * @return if the DataFlavors represent the same type.
	 */

	public boolean equals(DataFlavor dataFlavor) {
		if (dataFlavor == this)
			return true;

		if (dataFlavor == null)
			return false;
		if (getRepresentationClass() == null) {
			if (dataFlavor.getRepresentationClass() != null) {
				return false;
			} else if (getMimeType() == null) {
				return dataFlavor.getMimeType() == null;
			} else
				return isMimeTypeEqual(dataFlavor);

		} else if (getMimeType() == null) {
			return dataFlavor.getMimeType() == null
				&& getRepresentationClass().isAssignableFrom(dataFlavor.getRepresentationClass());
		}
		return isMimeTypeEqual(dataFlavor.getMimeType())
			&& getRepresentationClass().isAssignableFrom(dataFlavor.getRepresentationClass());
	}

	/**
	 * @param forClass Class
	 * @return ObjectDataFlavor
	 */
	public static ObjectDataFlavor getDataFlavor(Class forClass) {
		if (!hasDataFlavor(forClass)) {
			synchronized (ObjectDataFlavor.class) {
				if (!hasDataFlavor(forClass)) {
					setDataFlavor(forClass, new ObjectDataFlavor(forClass));
				}
			}
		}

		return (ObjectDataFlavor) objectDataFlavorMap.get(forClass);
	}

	private static boolean hasDataFlavor(Class forClass) {
		return objectDataFlavorMap.containsKey(forClass);
	}

	private static void setDataFlavor(Class forClass, ObjectDataFlavor objectDataFlavor) {
		objectDataFlavorMap.put(forClass, objectDataFlavor);
	}
}