package de.uniwue.ki.gui.util.dnd;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import junit.framework.Assert;
/**
 * Creation date: 04.09.2002, 19:14:08
 * @author oechslein
 */
public class ObjectTransferable implements Transferable, ClipboardOwner {

	private Object _object;
	private final DataFlavor[] _flavors;

	/**
	 * @param dataFlavor DataFlavor
	 * @param object Object
	 */
	public ObjectTransferable(DataFlavor dataFlavor, Object object) {
		super();

		_flavors = new DataFlavor[] { dataFlavor };
		_object = object;
	}

	/**
	 * @param flavor DataFlavor
	 * @return Object
	 * @throws UnsupportedFlavorException
	 * @throws IOException
	 * @see java.awt.datatransfer.Transferable#getTransferData(DataFlavor)
	 */
	public synchronized Object getTransferData(DataFlavor flavor)
		throws UnsupportedFlavorException, IOException {
		if (isDataFlavorSupported(flavor)) {
			return getTransferredData();
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	/**
	 * @return DataFlavor[]
	 * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
	 */
	public synchronized DataFlavor[] getTransferDataFlavors() {
		return _flavors;
	}

	protected Object getTransferredData() {
		return _object;
	}

	/**
	 * @param flavor DataFlavor
	 * @return boolean
	 * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(DataFlavor)
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		return flavor.equals(_flavors[0]);
	}

	/**
	 * @param clipboard Clipboard
	 * @param contents Transferable
	 * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(Clipboard, Transferable)
	 */
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		Assert.assertSame(this, contents);

		lostOwnershipIntern();
	}

	private void lostOwnershipIntern() {
		_object = null;
	}

	protected void finalize() throws Throwable {
		// since the TransferHandler#exportToClipboard do not allow to add a clipboard listener
		lostOwnershipIntern();
		super.finalize();
	}
}