package de.uniwue.ki.gui.util.hexagon;
import de.uniwue.ki.util.math.MathHelper;

/**
 * Creation date: (22.05.00 11:30:19)
 * @author Christoph Oechslein
 */
public class HexCoor {

	public static int arrayToHexX(int xa, int ya) {
		return xa - floor2(ya);
	}

	public static int arrayToHexY(int xa, int ya) {
		return xa + ceil2(ya);
	}

	private static int ceil2(int x) {
		if (x >= 0)
			return (x + 1) >> 1;
		else
			return x / 2;
	}

	private static int floor2(int x) {
		if (x >= 0)
			return x >> 1;
		else
			return (x - 1) / 2;
	}

	public static int hexToArrayX(int x, int y) {
		return floor2(x + y);
	}

	public static int hexToArrayY(int x, int y) {
		return y - x;
	}

	public static HexCoor screenToHex(int screenX, int screenY, int hexXSpacing, int hexYSpacing) {
		int x;
		int y;
		// System.out.println("");
		// System.out.println("screenX: " + screenX + " screenY: " + screenY + " Spac " + hexXSpacing + " " + hexYSpacing);
		int whichRow = (int)Math.floor(screenY / (0.75 * hexYSpacing));
		// System.out.println("whichRow: " + whichRow);
		int whichCol;
		boolean evenP = whichRow % 2 == 0;
		if (evenP) {
			whichCol = (int)Math.floor(screenX / (1.0 * hexXSpacing));
		} else {
			whichCol = (int)Math.floor((screenX - 0.5 * hexXSpacing) / (1.0 * hexXSpacing));
		}
		// System.out.println("whichCol: " + whichCol);
		if (whichRow * 0.75 * hexYSpacing + 0.25 * hexYSpacing > screenY) {
			// System.out.println("im dreieck");
			double startX =
				evenP ? (whichCol * hexXSpacing + 0.5 * hexXSpacing) : ((whichCol + 1) * hexXSpacing);
			if (startX > screenX) {
				// System.out.println("links");
				boolean isUpperLeft =
					searchAsQuadUpperLeft(
						screenX,
						screenY,
						(int)Math.floor(startX - 0.5 * hexXSpacing),
						(int)Math.floor(whichRow * 0.75 * hexYSpacing),
						(int)Math.floor(0.5 * hexXSpacing),
						(int)Math.floor(0.25 * hexYSpacing));
				if (isUpperLeft) {
					// System.out.println("oben links");
					y = whichRow - 1;
					x = evenP ? (whichCol - 1) : whichCol;
				} else {
					x = whichCol;
					y = whichRow;
				}
			} else {
				// System.out.println("rechts");
				boolean isUpperRight =
					searchAsQuadUpperRight(
						screenX,
						screenY,
						(int)Math.floor(startX),
						(int)Math.floor(whichRow * 0.75 * hexYSpacing),
						(int)Math.floor(0.5 * hexXSpacing),
						(int)Math.floor(0.25 * hexYSpacing));
				if (isUpperRight) {
					// System.out.println("oben rechts");
					y = whichRow - 1;
					x = evenP ? whichCol : (whichCol + 1);
				} else {
					y = whichRow;
					x = whichCol;
				}
			}
		} else {
			y = whichRow;
			x = whichCol;
		}
		// System.out.println("x: " + x + " y:" + y);
		return new HexCoor(arrayToHexX(x, y), arrayToHexY(x, y));
	}

	private static boolean searchAsQuadUpperLeft(
		int screenX,
		int screenY,
		double x,
		double y,
		double width,
		double height) {
		double midX = x + width * 0.5;
		double midY = y + height * 0.5;
		if (Math.abs(midX - screenX) < 1 && Math.abs(midY - screenY) < 1)
			return true;
		if (screenX > midX) {
			if (screenY > midY) {
				return false; // lowerright
			} else {
				return searchAsQuadUpperLeft(screenX, screenY, midX, y, width * 0.5, height * 0.5);
			}
		} else {
			if (screenY > midY) {
				return searchAsQuadUpperLeft(screenX, screenY, x, midY, width * 0.5, height * 0.5);
			} else {
				return true; // upperleft
			}
		}
	}

	private static boolean searchAsQuadUpperRight(
		int screenX,
		int screenY,
		double x,
		double y,
		double width,
		double height) {
		double midX = x + width * 0.5;
		double midY = y + height * 0.5;
		if (Math.abs(midX - screenX) < 1 && Math.abs(midY - screenY) < 1)
			return true;
		if (screenX > midX) {
			if (screenY > midY) {
				return searchAsQuadUpperRight(screenX, screenY, midX, midY, width * 0.5, height * 0.5);
			} else {
				return true; // upperright
			}
		} else {
			if (screenY > midY) {
				return false; // lowerleft
			} else {
				return searchAsQuadUpperRight(screenX, screenY, x, y, width * 0.5, height * 0.5);
			}
		}
	}

	public static void test() {
		// HexCoor srcCoor = new HexCoor(arrayToHexX(1, 4), arrayToHexY(1, 4));
		// HexCoor dstCoor = new HexCoor(arrayToHexX(5, 5), arrayToHexY(5, 5));
		//
		HexCoor srcCoor = new HexCoor(3, 3);
		HexCoor dstCoor = new HexCoor(-1, 2);
		//
		IProcessFunction printProcessFunction = new IProcessFunction() {
			public void process(long x, long y) {
				System.out.println("process(" + x + ", " + y + ")");
			}
		};
		srcCoor.straightLineTo(dstCoor, printProcessFunction);
	}
	private int _x;
	private int _y;

	public HexCoor() {
		super();
	}

	public HexCoor(int x, int y) {
		super();
		_x = x;
		_y = y;
	}

	public int distance(HexCoor hCoor) {
		int dx = hCoor._x - _x;
		int dy = hCoor._y - _y;
		//
		if (MathHelper.sign(dx) == MathHelper.sign(dy))
			return Math.max(Math.abs(dx), Math.abs(dy));
		else
			return Math.abs(dx) + Math.abs(dy);
	}

	public int getX() {
		return _x;
	}

	public int getY() {
		return _y;
	}

	public int hexToArrayX() {
		return floor2(_x + _y);
	}

	public int hexToArrayY() {
		return _y - _x;
	}

	void process(IProcessFunction processFunction, int arrayX, int arrayY) {
		processFunction.process(arrayX, arrayY);
	}

	public void straightLineTo(HexCoor hCoor, IProcessFunction processFunction) {
		final int dx = hCoor._x - _x;
		final int dy = hCoor._y - _y;
		//
		final boolean sig = MathHelper.sign(dx) == MathHelper.sign(dx);
		final int xOne = (dx < 0) ? -1 : 1; // unit increments
		final int yOne = (dy < 0) ? -1 : 1; // unit increments
		final int dxAbs;
		final int dyAbs;
		if (dx % 2 != 0) {
			dxAbs = Math.abs(dx * 2);
			dyAbs = Math.abs(dy * 2);
		} else {
			dxAbs = Math.abs(dx);
			dyAbs = Math.abs(dy);
		}
		final int dstX = hCoor._x;
		final int dstY = hCoor._y;
		int curX = _x;
		int curY = _y;
		int factor = dxAbs / 2; // should start at 0.5, which ist just (dxAbs/2)/dxAbs
		//
		process(processFunction, hexToArrayX(curX, curY), hexToArrayY(curX, curY));
		// note: passing array coors
		while (curX != dstX || curY != dstY) {
			factor += dyAbs;
			//
			if (factor >= dxAbs) {
				// Make a diagonal move in grid
				factor -= dxAbs;
				if (sig) {
					curX += xOne; // add 1 in the appropriate sign
					curY += yOne; // add 1 in the appropriate sign
				} else {
					curX += xOne; // doesn't matter which increases first
					process(processFunction, floor2(curX + curY), curY - curX); // note: passing array coors
					curY += yOne; // add 1 in the appropriate sign
				}
			} else {
				curX += xOne;
			}
			process(processFunction, floor2(curX + curY), curY - curX); // note: passing array coors
		}
	}
}