package de.uniwue.ki.gui.util.hexagon;/**
 * This is the event multicaster class to support the HexagonalMapModelListenerinterface.
 * @author oechslein
 */
public class HexagonalMapModelEventMulticaster
	extends java.awt.AWTEventMulticaster
	implements HexagonalMapModelListener {

	/**
	 * Constructor to support multicast events.
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected HexagonalMapModelEventMulticaster(java.util.EventListener a, java.util.EventListener b) {
	    super(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 * @return de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelListener
	 * @param a de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelListener
	 * @param b de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelListener
	 */
	public static HexagonalMapModelListener add(HexagonalMapModelListener a, HexagonalMapModelListener b) {
	    return (HexagonalMapModelListener) addInternal(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 * @return java.util.EventListener
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected static java.util.EventListener addInternal(java.util.EventListener a, java.util.EventListener b) {
	    if (a == null)
	        return b;
	    if (b == null)
	        return a;
	    return new HexagonalMapModelEventMulticaster(a, b);
	}

	/**
	 *
	 * @param event de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelEvent
	 * @see de.uniwue.ki.gui.util.hexagon.HexagonalMapModelListener#contentChanged(HexagonalMapModelEvent)
	 */
	public void contentChanged(HexagonalMapModelEvent event) {
	    ((HexagonalMapModelListener) a).contentChanged(event);
	    ((HexagonalMapModelListener) b).contentChanged(event);
	}

	/**
	 *
	 * @param event de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelEvent
	 * @see de.uniwue.ki.gui.util.hexagon.HexagonalMapModelListener#listOfTilesUpdate(HexagonalMapModelEvent)
	 */
	public void listOfTilesUpdate(HexagonalMapModelEvent event) {
	    ((HexagonalMapModelListener) a).listOfTilesUpdate(event);
	    ((HexagonalMapModelListener) b).listOfTilesUpdate(event);
	}

	/**
	 *
	 * @return java.util.EventListener
	 * @param oldl HexagonalMapModelListener
	 */
	protected java.util.EventListener remove(HexagonalMapModelListener oldl) {
	    if (oldl == a)
	        return b;
	    if (oldl == b)
	        return a;
	    java.util.EventListener a2 = removeInternal(a, oldl);
	    java.util.EventListener b2 = removeInternal(b, oldl);
	    if (a2 == a && b2 == b)
	        return this;
	    return addInternal(a2, b2);
	}

	/**
	 * Remove listener to support multicast events.
	 * @param l HexagonalMapModelListener
	 * @param oldl HexagonalMapModelListener
	 * @return HexagonalMapModelListener
	 */
	public static HexagonalMapModelListener remove(HexagonalMapModelListener l, HexagonalMapModelListener oldl) {
	    if (l == oldl || l == null)
	        return null;
	    if (l instanceof HexagonalMapModelEventMulticaster)
	        return (HexagonalMapModelListener) ((HexagonalMapModelEventMulticaster) l).remove(oldl);
	    return l;
	}

	/**
	 *
	 * @param event de.uniwue.ki.gui.generic.hexagon.HexagonalMapModelEvent
	 * @see de.uniwue.ki.gui.util.hexagon.HexagonalMapModelListener#tileUpdate(HexagonalMapModelEvent)
	 */
	public void tileUpdate(HexagonalMapModelEvent event) {
	    ((HexagonalMapModelListener) a).tileUpdate(event);
	    ((HexagonalMapModelListener) b).tileUpdate(event);
	}
}