package de.uniwue.ki.gui.util.hexagon;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
/**
 * Insert the type's description here. Creation date: (22.05.00 14:05:54)
 * @author Christoph Oechslein
 */
public class HexagonalTilesPanel extends JPanel implements Scrollable {
	private int _diamondWidth = 32;
	private int _diamondHeight = 32;
	private IvjEventHandler _eventHandler = new IvjEventHandler();
	private IHexagonalMapModel _hexagonalMapModel = null;
	private ITileRenderer _tileRenderer = null;

	private final class IvjEventHandler implements HexagonalMapModelListener, MouseListener {
		public void contentChanged(HexagonalMapModelEvent event) {
			if (event.getSource() == HexagonalTilesPanel.this.getanHexagonalMapModel())
				connEtoC2(event);
		}

		public void listOfTilesUpdate(HexagonalMapModelEvent event) {
			if (event.getSource() == HexagonalTilesPanel.this.getanHexagonalMapModel())
				connEtoC3(event);
		}

		/**
		 * @param e MouseEvent
		 * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
		 */
		public void mouseClicked(MouseEvent e) {
			if (e.getSource() == HexagonalTilesPanel.this)
				connEtoC1(e);
		}

		/**
		 * @param e MouseEvent
		 * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
		 */
		public void mouseEntered(MouseEvent e) {
		}

		/**
		 * @param e MouseEvent
		 * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
		 */
		public void mouseExited(MouseEvent e) {
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		/**
		 * @param event HexagonalMapModelEvent
		 * @see HexagonalMapModelListener#tileUpdate(HexagonalMapModelEvent)
		 */
		public void tileUpdate(HexagonalMapModelEvent event) {
			if (event.getSource() == HexagonalTilesPanel.this.getanHexagonalMapModel())
				connEtoC4(event);
		}
	}

	/** HexagonalTilesPanel constructor comment. */
	public HexagonalTilesPanel() {
		super();
		initialize();
	}

	/**
	     * HexagonalTilesPanel constructor comment.
	     * @param layout java.awt.LayoutManager
	     */
	public HexagonalTilesPanel(LayoutManager layout) {
		super(layout);
	}

	/**
	     * HexagonalTilesPanel constructor comment.
	     * @param layout java.awt.LayoutManager
	     * @param isDoubleBuffered boolean
	     */
	public HexagonalTilesPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}

	/**
	     * HexagonalTilesPanel constructor comment.
	     * @param isDoubleBuffered boolean
	     */
	public HexagonalTilesPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}

	/**
	     * main entrypoint - starts the part when it is run as an application
	     * @param args java.lang.String[]
	     */
	public static void main(String[] args) {
		try {
			JFrame frame = new JFrame();
			HexagonalTilesPanel aHexagonalTilesPanel;
			aHexagonalTilesPanel = new HexagonalTilesPanel();
			frame.setContentPane(aHexagonalTilesPanel);
			frame.setSize(aHexagonalTilesPanel.getSize());
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			});
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of javax.swing.JPanel");
			exception.printStackTrace(System.out);
		}
	}

	/**
	     * Insert the method's description here. Creation date: (23.05.00 09:47:31)
	     * @author Christoph Oechslein
	     * @return long
	     */
	public int getDiamondHeight() {
		return _diamondHeight;
	}

	/**
	     * Insert the method's description here. Creation date: (23.05.00 09:47:31)
	     * @author Christoph Oechslein
	     * @param newDiamondHeight long
	     */
	public void setDiamondHeight(int newDiamondHeight) {
		_diamondHeight = newDiamondHeight;
		updateComponentSize();
	}

	/**
	     * Insert the method's description here. Creation date: (23.05.00 09:47:31)
	     * @author Christoph Oechslein
	     * @return long
	     */
	public int getDiamondWidth() {
		return _diamondWidth;
	}

	/**
	     * Insert the method's description here. Creation date: (23.05.00 09:47:31)
	     * @author Christoph Oechslein
	     * @param newDiamondWidth long
	     */
	public void setDiamondWidth(int newDiamondWidth) {
		_diamondWidth = newDiamondWidth;
		invalidate();
		updateComponentSize();
	}

	/**
	     * Method generated to support the promotion of the anHexagonalMapModelThis attribute.
	     * @return IHexagonalMapModel
	     */
	public IHexagonalMapModel getAnHexagonalMapModelThis() {
		return getanHexagonalMapModel();
	}

	/**
	     * Method generated to support the promotion of the anTileRendererThis attribute.
	     * @return ITileRenderer
	     */
	public ITileRenderer getAnTileRendererThis() {
		return getanTileRenderer();
	}

	/** getPreferredScrollableViewportSize method comment. * @return Dimension
	 * @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	/** getScrollableBlockIncrement method comment. * @param visibleRect Rectangle
	 * @param orientation int
	 * @param direction int
	 * @return int
	 * @see javax.swing.Scrollable#getScrollableBlockIncrement(Rectangle, int, int)
	 */
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL)
			return visibleRect.width - getDiamondWidth();
		else
			return (int) (visibleRect.height - getDiamondHeight() * 0.75);
	}

	/** getScrollableTracksViewportHeight method comment. * @return boolean
	 * @see javax.swing.Scrollable#getScrollableTracksViewportHeight()
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	/** getScrollableTracksViewportWidth method comment. * @return boolean
	 * @see javax.swing.Scrollable#getScrollableTracksViewportWidth()
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/** getScrollableUnitIncrement method comment. * @param visibleRect Rectangle
	 * @param orientation int
	 * @param direction int
	 * @return int
	 * @see javax.swing.Scrollable#getScrollableUnitIncrement(Rectangle, int, int)
	 */
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL) {
			return getDiamondWidth();
		} else {
			return (int) (getDiamondHeight() * 0.75);
		}
	}

	/** Comment * @param componentEvent ComponentEvent
	 */
	public void hexagonalTilesPanel_ComponentResized(ComponentEvent componentEvent) {
		System.out.println(componentEvent.getComponent());
		return;
	}

	/** Comment */
	public void hexagonalTilesPanel_MouseClicked(MouseEvent mouseEvent) {
		if (!mouseEvent.isConsumed()) {
			HexCoor hCoor = hexagonalTilesPanel_MouseCoorToHexArray(mouseEvent.getX(), mouseEvent.getY());
			System.out.println(
				"x: "
					+ hCoor.hexToArrayX()
					+ " y: "
					+ hCoor.hexToArrayY()
					+ " hexX: "
					+ hCoor.getX()
					+ " hexY: "
					+ hCoor.getY());
			mouseEvent.consume();
		}
	}

	/** Comment * @param mouseX int
	 * @param mouseY int
	 * @return HexCoor
	 */
	public HexCoor hexagonalTilesPanel_MouseCoorToHexArray(int mouseX, int mouseY) {
		System.out.println("mouseX: " + mouseX + " mouseY: " + mouseY);
		return HexCoor.screenToHex(mouseX, mouseY, getDiamondWidth(), getDiamondHeight());
	}

	/** Comment * @param mouseEvent MouseEvent
	 */
	public void hexagonalTilesPanel_MousePressedOrReleased(MouseEvent mouseEvent) {
		if (!mouseEvent.isConsumed()) {
			// HexCoor hCoor = hexagonalTilesPanel_MouseCoorToHexArray(mouseEvent.getX(), mouseEvent.getY());
			mouseEvent.consume();
		}
	}

	/**
	     * Method generated to support the promotion of the anHexagonalMapModelThis attribute.
	     * @param arg1 IHexagonalMapModel
	     */
	public void setAnHexagonalMapModelThis(IHexagonalMapModel arg1) {
		setanHexagonalMapModel(arg1);
	}

	/**
	     * Method generated to support the promotion of the anTileRendererThis attribute.
	     * @param arg1 ITileRenderer
	     */
	public void setAnTileRendererThis(ITileRenderer arg1) {
		setanTileRenderer(arg1);
	}

	protected static int calculateEndX(Rectangle oldClip, int diamondWidth, int colsCount, int startX) {
		int endX = 1 + (int)Math.floor(((double) (oldClip.x + oldClip.width)) / diamondWidth);
		return Math.max(Math.min(endX, colsCount), startX);
	}

	protected static int calculateEndY(
		Rectangle oldClip,
		int diamondHeight_0_75,
		int rowsCount,
		int startY) {
		int endY = 1 + (int)Math.floor(((double) (oldClip.y + oldClip.height)) / (double)diamondHeight_0_75);
		return Math.max(Math.min(endY, rowsCount), startY);
	}

	protected static int calculateStartX(Rectangle oldClip, int diamondWidth) {
		return Math.max(0, (int) (Math.ceil((double)oldClip.x / (double)diamondWidth) - 1));
	}

	protected static int calculateStartY(Rectangle oldClip, int diamondHeight_0_75) {
		return (int)Math.floor((1.0 * oldClip.y) / (double)diamondHeight_0_75);
	}

	/**
	     * Return the anHexagonalMapModel property value.
	     * @return IHexagonalMapModel
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private IHexagonalMapModel getanHexagonalMapModel() {
		// user code begin {1}
		if (_hexagonalMapModel == null) {
			_hexagonalMapModel = new AbstractHexagonalMapModel() {
				public int getColsCount() {
					return 3;
				}
				public int getRowsCount() {
					return 3;
				}
				public ITile getTileAt(int x, int y) {
					return null;
				}
			};
			_hexagonalMapModel.addHexagonalMapModelListener(_eventHandler);
			updateComponentSize();
		}
		// user code end
		return _hexagonalMapModel;
	}

	/**
	     * Set the anHexagonalMapModel to a new value.
	     * @param newValue com.ibm.uvm.abt.edit.DeletedClassView
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void setanHexagonalMapModel(IHexagonalMapModel newValue) {
		if (_hexagonalMapModel != newValue) {
			try {
				IHexagonalMapModel oldValue = getanHexagonalMapModel();

				/* Stop listening for events from the current object */

				if (_hexagonalMapModel != null) {
					_hexagonalMapModel.removeHexagonalMapModelListener(_eventHandler);
				}
				_hexagonalMapModel = newValue;

				/* Listen for events from the new object */

				if (_hexagonalMapModel != null) {
					_hexagonalMapModel.addHexagonalMapModelListener(_eventHandler);
				}
				firePropertyChange("anHexagonalMapModelThis", oldValue, newValue);
				// user code begin {1}
				updateComponentSize();
				// user code end
			} catch (Throwable ivjExc) {
				// user code begin {2}
				// user code end
				handleException(ivjExc);
			}
		}
		// user code begin {3}
		// user code end
	}

	/**
	     * Return the anTileRenderer property value.
	     * @return ITileRenderer
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private ITileRenderer getanTileRenderer() {
		// user code begin {1}
		if (_tileRenderer == null)
			_tileRenderer = new NullTileRenderer();
		// user code end
		return _tileRenderer;
	}

	/**
	     * Set the anTileRenderer to a new value.
	     * @param newValue com.ibm.uvm.abt.edit.DeletedClassView
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void setanTileRenderer(ITileRenderer newValue) {
		if (_tileRenderer != newValue) {
			try {
				ITileRenderer oldValue = getanTileRenderer();
				_tileRenderer = newValue;
				firePropertyChange("anTileRendererThis", oldValue, newValue);
				// user code begin {1}
				updateComponentSize();
				// user code end
			} catch (Throwable ivjExc) {
				// user code begin {2}
				// user code end
				handleException(ivjExc);
			}
		}
		// user code begin {3}
		// user code end
	}

	/**
	     * connEtoC1:  (HexagonalTilesPanel.mouse.mouseClicked(java.awt.event.MouseEvent) -->
	     * HexagonalTilesPanel.hexagonalTilesPanel_MouseClicked(Ljava.awt.event.MouseEvent;)V)
	     * @param arg1 java.awt.event.MouseEvent
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void connEtoC1(MouseEvent arg1) {
		try {
			// user code begin {1}
			// user code end
			this.hexagonalTilesPanel_MouseClicked(arg1);
			// user code begin {2}
			// user code end
		} catch (Throwable ivjExc) {
			// user code begin {3}
			// user code end
			handleException(ivjExc);
		}
	}

	/**
	     * connEtoC2:  (anHexagonalMapModel.hexagonalMapModel.contentChanged(HexagonalMapModelEvent)
	     * --> HexagonalTilesPanel.updateListOfTiles(Ljava.util.Iterator;)V)
	     * @param arg1 HexagonalMapModelEvent
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void connEtoC2(HexagonalMapModelEvent arg1) {
		try {
			// user code begin {1}
			// user code end
			this.updateContentChanged();
			// user code begin {2}
			// user code end
		} catch (Throwable ivjExc) {
			// user code begin {3}
			// user code end
			handleException(ivjExc);
		}
	}

	/**
	     * connEtoC3:  (anHexagonalMapModel.hexagonalMapModel.listOfTilesUpdate(HexagonalMapModelEvent)
	     * --> HexagonalTilesPanel.updateListOfTiles(Ljava.util.Iterator;)V)
	     * @param arg1 HexagonalMapModelEvent
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void connEtoC3(HexagonalMapModelEvent arg1) {
		try {
			// user code begin {1}
			// user code end
			this.updateListOfTiles(arg1.getListOfTiles());
			// user code begin {2}
			// user code end
		} catch (Throwable ivjExc) {
			// user code begin {3}
			// user code end
			handleException(ivjExc);
		}
	}

	/**
	     * connEtoC4:  (anHexagonalMapModel.hexagonalMapModel.contentChanged(HexagonalMapModelEvent)
	     * --> HexagonalTilesPanel.updateTile(Lde.uniwue.ki.hexagonaltiles.HexCoor;)V)
	     * @param arg1 HexagonalMapModelEvent
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void connEtoC4(HexagonalMapModelEvent arg1) {
		try {
			// user code begin {1}
			// user code end
			this.updateTile(arg1.getHexCoor());
			// user code begin {2}
			// user code end
		} catch (Throwable ivjExc) {
			// user code begin {3}
			// user code end
			handleException(ivjExc);
		}
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected Polygon createPolygon() {
		//
		final int diamondWidth = getDiamondWidth();
		final int diamondHeight = getDiamondHeight();
		//
		Polygon polygon = new Polygon();
		polygon.addPoint(0 * diamondWidth / 4, 1 * diamondHeight / 4);
		polygon.addPoint(2 * diamondWidth / 4, 0 * diamondHeight / 4);
		polygon.addPoint(4 * diamondWidth / 4, 1 * diamondHeight / 4);
		polygon.addPoint(4 * diamondWidth / 4, 3 * diamondHeight / 4);
		polygon.addPoint(2 * diamondWidth / 4, 4 * diamondHeight / 4);
		polygon.addPoint(0 * diamondWidth / 4, 3 * diamondHeight / 4);
		//
		return polygon;
	}

	/**
	     * Called whenever the part throws an exception.
	     * @param exception java.lang.Throwable
	     */
	private void handleException(Throwable exception) {
		System.out.println("--------- UNCAUGHT EXCEPTION ---------");
		exception.printStackTrace(System.out);
	}

	/**
	     * Initializes connections
	     * @exception java.lang.Exception The exception description.
	     */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void initConnections() {
		// user code begin {1}
		// user code end
		this.addMouseListener(_eventHandler);
	}

	/** Initialize the class. */

	/* WARNING: THIS METHOD WILL BE REGENERATED. */

	private void initialize() {
		try {
			// user code begin {1}
			// user code end
			setName("HexagonalTilesPanel");
			setOpaque(true);
			setLayout(new BorderLayout());
			setDoubleBuffered(true);
			setSize(537, 508);
			initConnections();
		} catch (Throwable ivjExc) {
			handleException(ivjExc);
		}
		// user code begin {2}
		// user code end
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void paintCell(Graphics2D g2d, int x, int y, Polygon polygon, Rectangle polygonBounds) {
		ITile tile = getanHexagonalMapModel().getTileAt(x, y);
		getanTileRenderer().drawTile(g2d, polygon, tile, x, y);
		//
		final Component component =
			getanTileRenderer().getTileRendererComponent(this, polygon, tile, x, y, false, false);
		if (component == null)
			return;
		//
		if (component.getParent() != this) {
			this.add(component);
		}
		component.setBounds(polygonBounds);
		component.validate();
		boolean wasDoubleBuffered = false;
		if ((component instanceof JComponent) && ((JComponent)component).isDoubleBuffered()) {
			wasDoubleBuffered = true;
			((JComponent)component).setDoubleBuffered(false);
		}
		//
		SwingUtilities.paintComponent(g2d, component, this, polygonBounds);
		//
		if ((component instanceof JComponent) && wasDoubleBuffered) {
			((JComponent)component).setDoubleBuffered(true);
		}
		if (component instanceof JComponent) {
			((JComponent)component).setDoubleBuffered(wasDoubleBuffered);
		}
		component.setBounds(-polygonBounds.width, -polygonBounds.height, 0, 0);
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void paintCol(Graphics2D g2d, Polygon polygon, Rectangle oldClip, int x, int y) {
		final Rectangle polygonBounds = polygon.getBounds();
		if (oldClip.intersects(polygonBounds)) {
			g2d.drawPolygon(polygon);
			// set Clip and draw component
			//g.setClip(polygon); // this is to slow!
			g2d.clipRect(polygonBounds.x, polygonBounds.y, polygonBounds.width, polygonBounds.height);
			// g.clipRect(oldClip.x, oldClip.y, oldClip.width, oldClip.height);
			paintCell(g2d, x, y, polygon, polygonBounds);
			g2d.setClip(oldClip);
		}
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void paintComponent(Graphics g) {
		// final boolean printTime = false;

		super.paintComponent(g);
		//
		Graphics2D g2d = (Graphics2D)g;

		// long startTime = System.currentTimeMillis();

		final Polygon polygon = createPolygon();
		final Insets insets = getInsets();

		final IHexagonalMapModel hexagonalMapModel = getAnHexagonalMapModelThis();
		final int colsCount = hexagonalMapModel.getColsCount();
		final int rowsCount = hexagonalMapModel.getRowsCount();

		final Rectangle oldClip = g2d.getClipBounds();

		final int diamondWidth = getDiamondWidth();
		final int diamondHeight = getDiamondHeight();
		final int diamondHeight_0_75 = (int)Math.rint(diamondHeight * 0.75);
		final int diamondWidth_0_5 = (int)Math.rint(diamondWidth * 0.5);

		final int startX = calculateStartX(oldClip, diamondWidth);
		final int endX = calculateEndX(oldClip, diamondWidth, colsCount, startX);
		final int startY = calculateStartY(oldClip, diamondHeight_0_75);
		final int endY = calculateEndY(oldClip, diamondHeight_0_75, rowsCount, startY);

		// move startX, startY
		polygon.translate(insets.left + startX * diamondWidth, insets.top + startY * diamondHeight_0_75);
		// move deltaWidth if even
		polygon.translate(((startY % 2 == 0) ? diamondWidth_0_5 : 0), 0);

		g2d.setFont(getFont().deriveFont((float) (polygon.getBounds().getHeight() * 0.25)));

		for (int y = startY; y < endY; ++y) {
			// move to deltaWidth, minus if even
			polygon.translate(((y % 2 == 0) ? -1 : +1) * diamondWidth_0_5, 0);
			int addContainer = 0;
			for (int x = startX; x < endX; ++x) {
				paintCol(g2d, polygon, oldClip, x, y);
				polygon.translate(diamondWidth, 0);
				addContainer += diamondWidth;
			}
			// move down from right to left and one row down
			polygon.translate(-addContainer, diamondHeight_0_75);
		}
		/*
		if (printTime) {
		   long endTime = System.currentTimeMillis();
		   System.out.println("time: " + (endTime - startTime));
		}
		*/
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void translatePolygon(Polygon polygon, long dx, long dy) {
		polygon.translate((int) (dx * getDiamondWidth()), (int) (dy * getDiamondHeight()));
	}

	/**
	     * Insert the method's description here. Creation date: (23.05.00 14:49:34)
	     * @author Christoph Oechslein
	     */
	protected void updateComponentSize() {
		Insets insets = getInsets();
		int x =
			insets.left
				+ insets.right
				+ 1
				+ (int)Math.ceil((getDiamondWidth() * (getanHexagonalMapModel().getColsCount() + 0.5)));
		int y =
			insets.top
				+ insets.bottom
				+ 1
				+ (int)Math.ceil((getDiamondHeight() * 0.75 * (getanHexagonalMapModel().getRowsCount() + 0.3)));
		Dimension newDim = new Dimension(x, y);
		setPreferredSize(newDim);
		setMaximumSize(newDim);
		// setSize(newDim);
		revalidate();
	}

	/** Comment */
	protected void updateContentChanged() {
		invalidate();
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void updateListOfTiles(Iterator it) {
		while (it.hasNext())
			updateTile((HexCoor)it.next());
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void updateTile(int x, int y) {
		IHexagonalMapModel model = getanHexagonalMapModel();
		updateTile(model.getTileAt(x, y), x, y);
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void updateTile(HexCoor hCoor) {
		updateTile(hCoor.hexToArrayX(), hCoor.hexToArrayY());
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void updateTile(ITile tile, long x, long y) {
		Polygon polygon = createPolygon();
		translatePolygon(polygon, x, y);
		repaint(polygon.getBounds());
	}
}