/**
 * Created on 17.04.2003
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.util.lookAndFeel;

import java.io.File;
import java.io.FileFilter;
import java.io.FilenameFilter;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;

import de.uniwue.ki.util.jarclassloader.JarClassLoader;

public class LookAndFeelManagerAuxiliary {

	private static final String LOOK_N_FEEL__ROOT_CLASSNAME = "javax.swing.LookAndFeel";
	private static final String LnF_DIRECTORY_NAME = "lookAndFeels";
	private static final String SKINS_DIRECTORY_NAME = "skins";

	public final static String DEFAULT__SKIN_NAME = "themepack";

	private static LookAndFeelManagerAuxiliary _landfJarFinder;
	private static JarClassLoader _jarClassLoader;

	private LookAndFeelRepository _lnfRepository;

	private LookAndFeelManagerAuxiliary() {

		init();

		getRepository().writeToDisk(getRepositoryLocation());
	}

	public static LookAndFeelManagerAuxiliary getInstance() {
		if (_landfJarFinder == null) {
			_landfJarFinder = new LookAndFeelManagerAuxiliary();
		}

		return _landfJarFinder;
	}

	public URL getResource(String name) {

		return getJarClassLoader().getResource(name);
	}

	private JarClassLoader getJarClassLoader() {

		if (_jarClassLoader == null) {

			_jarClassLoader = JarClassLoader.getInstance();
		}

		return _jarClassLoader;
	}

	private void init() {

		initExclusionList();

		initializeLookAndFeels();
		initialzeSkins();
	}

	public LookAndFeelRepository getRepository() {

		if (_lnfRepository == null) {

			_lnfRepository = LookAndFeelRepository.loadFromDisk(getRepositoryLocation());

			if (_lnfRepository == null) {
				System.out.println("LookAndFeelManagerAuxiliary.getRepository() - Error loading repository.");
				_lnfRepository = new LookAndFeelRepository();
			}

		}

		return _lnfRepository;
	}

	private ArrayList _xclusionList;

	private void initExclusionList() {

		_xclusionList = new ArrayList();
		_xclusionList.add("LinuxLookAndFeel");

	}

	private boolean excludeLnF(String className) {

		Iterator iterator = _xclusionList.iterator();
		while (iterator.hasNext()) {
			String element = (String)iterator.next();

			if (className.endsWith(element)) {

				return true;
			}
		}

		return false;
	}

	private void initialzeSkins() {

		// TODO ist das noch ntig (dass die Skins auch hier 'geladen' werden)?
		File sFdirectory = new File(getSkinDirectory());

		if (sFdirectory.isDirectory()) {

			File[] landfs = sFdirectory.listFiles(new FileFilter() {
				public boolean accept(File pathname) {

					return pathname.getName().endsWith(".zip");
				}
			});

			for (int index = 0; index < landfs.length; index++) {

				File skinF = landfs[index];
				String skinFileString = skinF.getAbsolutePath();

				registerArchive(skinFileString, false);
			}
		}

	}

	private void initializeLookAndFeels() {
		File lFdirectory = new File(getLandFDirectory());

		if (lFdirectory.isDirectory()) {

			File[] landfs = lFdirectory.listFiles(new FileFilter() {
				public boolean accept(File pathname) {

					return pathname.getName().endsWith(".zip") || pathname.getName().endsWith(".jar");
				}
			});

			for (int index = 0; index < landfs.length; index++) {

				File landf = landfs[index];
				String landfileString = landf.getAbsolutePath();

				if (!getRepository().isArchiveAndLnFContained(landfileString)) {
					loadLookAndFeel(landfileString);
				}
			}
		}
	}

	private void loadLookAndFeel(String landfileString) {
		System.out.println(
			"LookAndFeelManagerAuxiliary.registerLookAndFeels() - loading archive completely: "
				+ landfileString);

		registerArchive(landfileString, true);

		List list =
			getJarClassLoader().getClassnamesOfDescendantsFromArchiveGivenByLocation(
				LOOK_N_FEEL__ROOT_CLASSNAME,
				landfileString);

		Iterator iter = list.iterator();
		while (iter.hasNext()) {
			String className = (String)iter.next();

			if (!excludeLnF(className)) {

				try {
					getRepository().addEntry(
						new LookAndFeelEntry(
							landfileString,
							className,
							((LookAndFeel)getJarClassLoader().loadClass(className).newInstance()).getName()));
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static String getLandFDirectory() {
		return System.getProperty("user.dir") + '/' + LnF_DIRECTORY_NAME;
	}

	public static String getRepositoryLocation() {
		return System.getProperty("java.io.tmpdir", "") + '/' + "LookAndFeelRepository.ser";
		// return getLandFDirectory() + '/' + "LookAndFeelRepository.ser";
	}

	public static String getSkinDirectory() {

		return getLandFDirectory() + '/' + SKINS_DIRECTORY_NAME;
	}

	public static String getDefaultLandFSkin() {

		return getSkinDirectory() + '/' + DEFAULT__SKIN_NAME + ".zip";
	}

	private void registerArchive(String jarFilename, boolean deflateArchive) {

		getJarClassLoader().registerArchive(jarFilename, deflateArchive);
	}

	public static String[] getThemepackNames() {

		//		URL url = this.getClass().getResource("");
		File landfDirectory = new File(getSkinDirectory());
		if (landfDirectory.exists()) {

			return landfDirectory.list(new FilenameFilter() {

				public boolean accept(File dir, String name) {

					return name.endsWith("zip");
				}
			});
		} else {

			return new String[] {
			};
		}
	}

	public static boolean isCurrentKunststoffLookAndFeel() {
		return UIManager.getLookAndFeel().getID().equals("Kunststoff");
	}
}
