/**
 * Created on 06.05.2003
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.gui.util.lookAndFeel;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.uniwue.ki.util.jarclassloader.JarClassLoader;

public class LookAndFeelRepository implements Serializable {

	private ArrayList _entries;

	public LookAndFeelRepository() {

		_entries = new ArrayList();
	}

	public boolean writeToDisk(String destination) {

		try {
			File destFile = new File(destination);

			if (destFile.getParentFile() != null)
				destFile.getParentFile().mkdirs();

			FileOutputStream fs = new FileOutputStream(destFile);
			ObjectOutputStream os = new ObjectOutputStream(fs);

			os.writeObject(_entries);
			os.close();

		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}

	public void setData(ArrayList data) {

		_entries = data;
	}

	public static LookAndFeelRepository loadFromDisk(String source) {

		LookAndFeelRepository readObject = new LookAndFeelRepository();

		try {
			ObjectInputStream is = new ObjectInputStream(new FileInputStream(source));
			ArrayList data = (ArrayList) is.readObject();

			for (int index = 0; index < data.size(); index++) {
				LookAndFeelEntry entry = (LookAndFeelEntry) data.get(index);

				if (!new File(entry.getArchiveName()).exists()) {

					System.out.println("removing: " + entry.getArchiveName());
					data.remove(index);
					index--;
				}
			}

			readObject.setData(data);

			is.close();
		} catch (FileNotFoundException e) {
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return readObject;
	}

	public boolean addEntry(LookAndFeelEntry entry) {

		if (!isArchiveAndLnFContained(entry.getArchiveName(), entry.getClassName())) {

			_entries.add(entry);
			return true;
		}

		return false;
	}

	public Iterator iterator() {

		ArrayList names = new ArrayList();

		for (Iterator iter = _entries.iterator(); iter.hasNext();) {
			LookAndFeelEntry element = (LookAndFeelEntry) iter.next();

			names.add(element.getName());
		}

		return names.iterator();
	}

	public List asList() {

		return _entries;
	}

	/**
	 * Checks whether the given archive-name is the name of one from a
	 * LookAndFeelEntry. This method is mainly used to avoid dublicated
	 * entries.
	 */
	public boolean isArchiveAndLnFContained(String aName, String cName) {

		boolean result = false;

		for (Iterator iter = _entries.iterator(); iter.hasNext();) {
			LookAndFeelEntry element = (LookAndFeelEntry) iter.next();

			result |= element.getArchiveName().equals(aName) && element.getClassName().equals(cName);
		}

		return result;
	}

	public boolean isArchiveAndLnFContained(String aName) {

		boolean result = false;

		for (Iterator iter = _entries.iterator(); iter.hasNext();) {
			LookAndFeelEntry element = (LookAndFeelEntry) iter.next();

			result |= element.getArchiveName().equals(aName);
		}

		return result;
	}

	/**
	 * Causes the JarClassLoader to load the archive containing the given main
	 * LnF-class by registering the archive with the loader.
	 */
	public boolean installLookAndFeelByMainClass(String mainClass) {

		boolean result = false;

		for (Iterator iter = _entries.iterator(); iter.hasNext();) {
			LookAndFeelEntry entry = (LookAndFeelEntry) iter.next();

			if (entry.getClassName().equals(mainClass)) {

				JarClassLoader.getInstance().loadArchive(entry.getArchiveName());
				result = true;
				break;
			}
		}

		return result;
	}

}
