package de.uniwue.ki.gui.util.mdi;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.DefaultDesktopManager;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
/**
 * Private class used to replace the standard DesktopManager for JDesktopPane.
 * Used to provide scrollbar functionality.
 */
class MDIDesktopManager extends DefaultDesktopManager {
	private boolean _inResize = false;
	private final MDIDesktopPane _desktop;

	public MDIDesktopManager(MDIDesktopPane desktop) {
		_desktop = desktop;
	}

	public void endDraggingFrame(JComponent f) {
		super.endDraggingFrame(f);
		resizeDesktop();
		scrollToComponent(f);
	}

	public void openFrame(JInternalFrame f) {
		super.openFrame(f);
		resizeDesktop();
	}

	public void closeFrame(JInternalFrame f) {
		super.closeFrame(f);
		resizeDesktop();
	}

	public void deactivateFrame(JInternalFrame f) {
		super.deactivateFrame(f);
		resizeDesktop();
	}

	public void activateFrame(JInternalFrame f) {
		super.activateFrame(f);
		resizeDesktop();
	}

	public void minimizeFrame(JInternalFrame f) {
		super.minimizeFrame(f);
		resizeDesktop();
	}

	public void iconifyFrame(JInternalFrame f) {
		super.iconifyFrame(f);
		resizeDesktop();
	}

	public void deiconifyFrame(JInternalFrame f) {
		super.deiconifyFrame(f);
		resizeDesktop();
	}

	public void maximizeFrame(JInternalFrame f) {
		super.maximizeFrame(f);
		resizeDesktop();
	}

	public void endResizingFrame(JComponent f) {
		super.endResizingFrame(f);
		resizeDesktop();
		scrollToComponent(f);
	}

	private void scrollToComponent(JComponent f) {
		// f.scrollRectToVisible(f.getBounds());
	}

	private JScrollPane getScrollPane() {
		if (_desktop.getParent() instanceof JViewport) {
			JViewport viewPort = (JViewport)_desktop.getParent();
			if (viewPort.getParent() instanceof JScrollPane)
				return (JScrollPane)viewPort.getParent();
		}
		return null;
	}

	private Insets getScrollPaneInsets() {
		JScrollPane scrollPane = getScrollPane();
		if (scrollPane == null)
			return new Insets(0, 0, 0, 0);
		else
			return getScrollPane().getBorder().getBorderInsets(scrollPane);
	}

	public void resizeDesktop() {
		if (!_inResize) {
			_inResize = true;

			resizeDesktopIntern();

			_inResize = false;
		}
	}

	private void resizeDesktopIntern() {
		JScrollPane scrollPane = getScrollPane();

		if (scrollPane != null) {

			Rectangle workspace = new Rectangle(0, 0);

			JInternalFrame allFrames[] = _desktop.getAllFrames();
			for (int i = 0; i < allFrames.length; i++) {
				if (allFrames[i].isVisible())
					workspace.add(allFrames[i].getBounds());
			}

			int minX = (int)Math.ceil(workspace.getX());
			int minY = (int)Math.ceil(workspace.getY());

			int newWidth = (int)Math.floor(workspace.getWidth()) - minX;
			int newHeight = (int)Math.floor(workspace.getHeight()) - minY;

			Dimension scrollPanelVisibleSize = scrollPane.getVisibleRect().getSize();
			if (scrollPane.getBorder() != null) {
				Insets scrollInsets = getScrollPaneInsets();
				scrollPanelVisibleSize.setSize(
					scrollPanelVisibleSize.getWidth() - scrollInsets.left - scrollInsets.right,
					scrollPanelVisibleSize.getHeight() - scrollInsets.top - scrollInsets.bottom);
			}

			if (newWidth <= scrollPanelVisibleSize.getWidth())
				newWidth = ((int)scrollPanelVisibleSize.getWidth()) - 20;
			if (newHeight <= scrollPanelVisibleSize.getHeight())
				newHeight = ((int)scrollPanelVisibleSize.getHeight()) - 20;

			if ((newWidth == _desktop.getMaximumSize().getWidth())
				&& (newHeight == _desktop.getMaximumSize().getHeight()))
				return;

			Point oldViewPosition = scrollPane.getViewport().getViewPosition();

			_desktop.setAllSize(newWidth, newHeight);

			for (int i = 0; i < allFrames.length; i++) {
				allFrames[i].setLocation(
					allFrames[i].getX() - Math.min(0, minX),
					allFrames[i].getY() - Math.min(0, minY));
			}

			scrollPane.invalidate();
			scrollPane.validate();

			scrollPane.getViewport().setViewPosition(oldViewPosition);
		}
	}

	public void setNormalSize() {
		JScrollPane scrollPane = getScrollPane();
		int x = 0;
		int y = 0;
		Insets scrollInsets = getScrollPaneInsets();

		if (scrollPane != null) {
			Dimension d = scrollPane.getVisibleRect().getSize();
			if (scrollPane.getBorder() != null) {
				d.setSize(
					d.getWidth() - scrollInsets.left - scrollInsets.right,
					d.getHeight() - scrollInsets.top - scrollInsets.bottom);
			}

			d.setSize(d.getWidth() - 20, d.getHeight() - 20);
			_desktop.setAllSize(x, y);
			scrollPane.invalidate();
			scrollPane.validate();
		}
	}
}