package de.uniwue.ki.gui.util.mdi;
import java.awt.Component;
import java.awt.Dimension;
import java.beans.PropertyVetoException;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
/**
 * An extension of WDesktopPane that supports often used MDI functionality. This
 * class also handles setting scroll bars for when windows move too far to the left or
 * bottom, providing the MDIDesktopPane is in a ScrollPane.
 */
public class MDIDesktopPane extends JDesktopPane {

	public MDIDesktopPane() {
	    setDesktopManager(new MDIDesktopManager(this));
	    //setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
	}

	public Component add(JInternalFrame frame) {
	    Component retval = super.add(frame);
	
	    /*
	    JInternalFrame[] array = getAllFrames();
	    Point p;
	    int w;
	    int h;
	    
	    checkDesktopSize();
	    if (array.length > 0) {
	        p = array[0].getLocation();
	        p.x = p.x + FRAME_OFFSET;
	        p.y = p.y + FRAME_OFFSET;
	    }
	    else {
	        p = new Point(0, 0);
	    }
	    frame.setLocation(p.x, p.y);
	    if (frame.isResizable()) {
	        w = getWidth() - (getWidth()/3);
	        h = getHeight() - (getHeight()/3);
	        if (w < frame.getMinimumSize().getWidth()) w = (int)frame.getMinimumSize().getWidth();
	        if (h < frame.getMinimumSize().getHeight()) h = (int)frame.getMinimumSize().getHeight();
	        frame.setSize(w, h);
	    }
	    */
	
	    moveToFront(frame);
	    frame.setVisible(true);
	    try {
	        frame.setSelected(true);
	    } catch (PropertyVetoException e) {
	        frame.toBack();
	    }
	    return retval;
	}

	private void checkDesktopSize() {
	    if (getParent() != null && isVisible())
	        getMDIDesktopManager().resizeDesktop();
	}

	public final MDIDesktopManager getMDIDesktopManager() {
	    return (MDIDesktopManager) getDesktopManager();
	}


	public void remove(Component c) {
	    super.remove(c);
	    checkDesktopSize();
	}


	public void setAllSize(int width, int height) {
	    setAllSize(new Dimension(width, height));
	}


	public void setAllSize(Dimension d) {
	    setMinimumSize(d);
	    setMaximumSize(d);
	    setPreferredSize(d);
	}


	public void setBounds(int x, int y, int w, int h) {
	    super.setBounds(x, y, w, h);
	    checkDesktopSize();
	}
}