package de.uniwue.ki.gui.util.rectangular;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

import junit.framework.Assert;
import de.uniwue.ki.gui.util.JDisposeablePanel;
import de.uniwue.ki.util.rectangular.IRectangularMapModel;
import de.uniwue.ki.util.rectangular.event.RectangularMapChangedEvent;
import de.uniwue.ki.util.rectangular.event.RectangularMapChangedListener;
/**
 * Creation date: (15.11.2000 15:55:43)
 * @author Christoph Oechslein
 */
public class JRectangularMap extends JDisposeablePanel implements Scrollable {
	private int _rectangleWidth;
	private int _rectangleHeight;
	private boolean ivjConnPtoP1Aligning = false;
	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private IRectangularMapModel ivjRectangularMapModel = null;
	private IRectangularMapCellRenderer _rectangularMapCellRenderer = null;
	private boolean _drawingGrid = false;
	private IRectangularMapPanelState ivjRectangularMapStateIntern = null;
	private java.awt.Point _selectedPoint = new Point();
	private boolean _automaticUpdate = true;

	private final class IvjEventHandler
		implements RectangularMapChangedListener, java.beans.PropertyChangeListener {

		public void mapChanged(RectangularMapChangedEvent event) {
			if (event.getSource() == JRectangularMap.this.getRectangularMapModel())
				rectangularMapModelMapChanged(event);
		}

		public void positionChanged(RectangularMapChangedEvent event) {
			if (event.getSource() == JRectangularMap.this.getRectangularMapModel()) {
				//this.rectangularMapModelMapChanged(arg1);
			}
		}

		public void propertyChange(java.beans.PropertyChangeEvent evt) {
			if (evt.getSource() == JRectangularMap.this
				&& (evt.getPropertyName().equals("rectangularMapModel")))
				if (ivjConnPtoP1Aligning == false) {

					ivjConnPtoP1Aligning = true;
					setRectangularMapModel(getRectangularMapModel());

					ivjConnPtoP1Aligning = false;
				}
			if (evt.getSource() == JRectangularMap.this.getRectangularMapModel()
				&& (evt.getPropertyName().equals("colsCount"))) {
				updateComponentSize();
			}
			if (evt.getSource() == JRectangularMap.this.getRectangularMapModel()
				&& (evt.getPropertyName().equals("rowsCount")))
				updateComponentSize();
		}
	}

	public JRectangularMap() {
		super();
		initialize();
	}

	public JRectangularMap(IRectangularMapModel rectangularMapModel) {
		super();

		initialize();
		setRectangularMapModel(rectangularMapModel);
	}

	protected final int calculateEndX(Rectangle clipBounds, int colsCount, int startX) {
		int endX = 1 + (int)Math.floor(clipBounds.getMaxX() / getRectangleWidth());

		return Math.max(Math.min(endX, colsCount), startX);
	}

	protected final int calculateEndY(Rectangle clipBounds, int rowsCount, int startY) {
		int endY =
			1 + (int)Math.floor(((double) (clipBounds.y + clipBounds.height)) / (double)getRectangleHeight());
		return Math.max(Math.min(endY, rowsCount), startY);
	}

	protected final int calculateStartX(Rectangle clipBounds) {
		return Math.max(0, (int) (Math.ceil((double)clipBounds.x / (double)getRectangleWidth()) - 1));
	}

	protected final int calculateStartY(Rectangle clipBounds) {
		return Math.max(0, (int) (Math.ceil((double)clipBounds.y / (double)getRectangleHeight()) - 1));
		// return (int) Math.floor((1.0 * clipBounds.y) / (double) getRectangleHeight());
	}

	public final Rectangle convertPositionToRectangle(int col, int row) {
		Rectangle rectangle = new Rectangle();
		convertPositionToRectangle(col, row, rectangle);
		return rectangle;
	}

	public void convertPositionToRectangle(int col, int row, Rectangle rectangle) {
		Insets insets = getInsets();

		int startX = insets.left + col * getRectangleWidth();
		int startY = insets.top + row * getRectangleHeight();

		rectangle.setBounds(startX, startY, getRectangleWidth(), getRectangleHeight());
	}

	public void dispose() {
		if (getRectangularMapModel() != null) {
			getRectangularMapModel().removePropertyChangeListener(ivjEventHandler);
			getRectangularMapModel().removeRectangularMapChangedListener(ivjEventHandler);
			//getRectangularMapModel().removeRectangularMapObjectAddedListener(ivjEventHandler);
			//getRectangularMapModel().removeRectangularMapObjectRemovedListener(ivjEventHandler);
		}

		super.dispose();
	}

	public int getColumnIndexAtX(int mouseX) {

		Insets insets = getInsets();

		int col = (mouseX - insets.left) / getRectangleWidth();

		if (col > getRectangularMapModel().getColsCount() || col < 0)
			return -1;
		else
			return col;
	}

	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	public final int getRectangleHeight() {
		return _rectangleHeight;
	}

	public final int getRectangleWidth() {
		return _rectangleWidth;
	}

	public IRectangularMapCellRenderer getRectangularMapCellRenderer() {
		if (_rectangularMapCellRenderer == null)
			_rectangularMapCellRenderer = new DefaultRectangularMapCellRenderer();

		return _rectangularMapCellRenderer;
	}

	public final IRectangularMapModel getRectangularMapModel() {
		return ivjRectangularMapModel;
	}

	public IRectangularMapPanelState getRectangularMapPanelState() {
		return ivjRectangularMapStateIntern;
	}

	public int getRowIndexAtX(int mouseX) {

		Insets insets = getInsets();

		int row = (mouseX - insets.bottom) / getRectangleHeight();

		if (row > getRectangularMapModel().getRowsCount() || row < 0)
			return -1;
		else
			return row;
	}

	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL)
			return visibleRect.width - getRectangleWidth();
		else
			return (int) (visibleRect.height - getRectangleHeight() * 0.75);
	}

	public boolean getScrollableTracksViewportHeight() {
		return false;
	}

	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		if (orientation == SwingConstants.HORIZONTAL) {
			return getRectangleWidth();
		} else {
			return (int) (getRectangleHeight() * 0.75);
		}
	}

	/**
	 * Gets the selectedPoint property (java.awt.Point) value.
	 * @return The selectedPoint property value.
	 * @see #setSelectedPoint
	 */
	public java.awt.Point getSelectedPoint() {
		return _selectedPoint;
	}

	/**
	 * Initializes connections
	 * @exception java.lang.Exception The exception description.
	 */

	private void initConnections() {
		addPropertyChangeListener(ivjEventHandler);
		if (ivjConnPtoP1Aligning == false) {

			ivjConnPtoP1Aligning = true;
			setRectangularMapModel(getRectangularMapModel());

			ivjConnPtoP1Aligning = false;
		}
	}

	/**
	 * Initialize the class.
	 */

	private void initialize() {
		setRectangleWidth(30);
		setRectangleHeight(30);

		setOpaque(true);
		setBorder(BorderFactory.createTitledBorder(""));
		setDoubleBuffered(true);
		setSize(151, 120);
		initConnections();
	}

	/**
	 * Gets the automaticUpdate property (boolean) value.
	 * @return The automaticUpdate property value.
	 * @see #setAutomaticUpdate
	 */
	public boolean isAutomaticUpdate() {
		return _automaticUpdate;
	}

	/**
	 * Gets the drawingGrid property (boolean) value.
	 * @return The drawingGrid property value.
	 * @see #setDrawingGrid
	 */
	public boolean isDrawingGrid() {
		return _drawingGrid;
	}

	/**
	 * main entrypoint - starts the part when it is run as an application
	 * @param args java.lang.String[]
	 */
	public static void main(java.lang.String[] args) {
		try {
			JFrame frame = new javax.swing.JFrame();
			JRectangularMap aJRectangularMap;
			aJRectangularMap = new JRectangularMap();
			frame.setContentPane(aJRectangularMap);
			frame.setSize(aJRectangularMap.getSize());
			frame.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			});
			frame.show();
			java.awt.Insets insets = frame.getInsets();
			frame.setSize(
				frame.getWidth() + insets.left + insets.right,
				frame.getHeight() + insets.top + insets.bottom);
			frame.setVisible(true);
		} catch (Throwable exception) {
			System.err.println("Exception occurred in main() of JRectangularMap");
			exception.printStackTrace(System.out);
		}
	}

	protected void paintCell(Graphics2D g2d, Rectangle rectangle, int currentCol, int currentRow) {

		boolean isSelectedCell;

		if (getSelectedPoint() != null
			&& getSelectedPoint().x == currentCol
			&& getSelectedPoint().y == currentRow)
			isSelectedCell = true;
		else
			isSelectedCell = false;

		// Graphics2D.clip uses interior of rectangle
		rectangle.grow(1, 1);
		g2d.clip(rectangle);
		rectangle.grow(-1, -1);

		/*
		if (isSelectedCell) {
		        g2d.setColor(Color.red);
		        g2d.fill(rectangle);
		        g2d.setColor(Color.blue);
		        g2d.draw(rectangle);
		        g2d.setColor(Color.black);
		    }
		    /*else {
		        g2d.setColor(Color.white);
		        g2d.fill(rectangle);
		        g2d.setColor(Color.green);
		        g2d.draw(rectangle);
		        g2d.setColor(Color.black);
		    }
		    */

		Component component =
			getRectangularMapCellRenderer().getRectangularMapCellRendererComponent(
				this,
				getRectangularMapModel().getObjectsAt(currentCol, currentRow),
				isSelectedCell,
				false,
				currentCol,
				currentRow);

		if (component == null) {
			getRectangularMapCellRenderer().renderCellInGraphics(
				this,
				rectangle,
				g2d,
				isSelectedCell,
				false,
				currentCol,
				currentRow);
		} else {

			if (component.getParent() != this) {
				this.add(component);
			}

			component.setBounds(rectangle);
			component.validate();

			if (component instanceof JComponent) {

				JComponent jComponent = (JComponent)component;
				boolean wasDoubleBuffered = jComponent.isDoubleBuffered();

				jComponent.setDoubleBuffered(false);

				SwingUtilities.paintComponent(g2d, component, this, rectangle);

				jComponent.setDoubleBuffered(wasDoubleBuffered);

			} else
				SwingUtilities.paintComponent(g2d, component, this, rectangle);

			component.setBounds(-rectangle.width, -rectangle.height, 0, 0);
		}
	}

	/**
	     * Insert the method's description here. Creation date: (22.05.00 14:12:55)
	     * @author Christoph Oechslein
	     * @param:
	     * @return:
	     */
	protected void paintCol(
		Graphics2D g2d,
		Rectangle rectangle,
		Shape oldClip,
		int currentCol,
		int currentRow) {

		if (oldClip.intersects(rectangle)) {

			try {
				if (isDrawingGrid()) {

					g2d.drawRect(rectangle.x, rectangle.y, rectangle.width, rectangle.height);

					rectangle.grow(-1, -1);
					//rectangle.translate(1, 1);

					paintCell(g2d, rectangle, currentCol, currentRow);
					rectangle.grow(1, 1);
					//rectangle.translate(-1, -1);

				} else {
					rectangle.setSize(rectangle.width - 1, rectangle.height - 1);
					paintCell(g2d, rectangle, currentCol, currentRow);
					rectangle.setSize(rectangle.width + 1, rectangle.height + 1);
				}

			} finally {
				g2d.setClip(oldClip);
			}
		}
	}

	protected void paintComponent(Graphics g) {

		super.paintComponent(g);

		if (getRectangularMapModel() == null)
			return;

		Graphics2D g2d = (Graphics2D)g;

		final int colsCount = getRectangularMapModel().getColsCount();
		final int rowsCount = getRectangularMapModel().getRowsCount();

		final Shape oldClip = g2d.getClip();
		final Rectangle clipBounds = oldClip.getBounds();

		final int startCol = calculateStartX(clipBounds);
		final int endCol = calculateEndX(clipBounds, colsCount, startCol);
		final int startRow = calculateStartY(clipBounds);
		final int endRow = calculateEndY(clipBounds, rowsCount, startRow);

		final Rectangle rectangle = convertPositionToRectangle(startCol, startRow);

		for (int currentRow = startRow; currentRow < endRow; ++currentRow) {
			int addContainer = 0;
			for (int currentCol = startCol; currentCol < endCol; ++currentCol) {
				paintCol(g2d, rectangle, oldClip, currentCol, currentRow);
				rectangle.translate(getRectangleWidth(), 0);
				addContainer += getRectangleWidth();
			}
			// move down from right to left and one row down
			rectangle.translate(-addContainer, getRectangleHeight());
		}
	}

	protected void rectangularMapModelMapChanged(RectangularMapChangedEvent event) {
		if (event.getType() == RectangularMapChangedEvent.POSITION_CHANGED)
			updatePositionChanged(event);
		else
			repaint();
	}

	protected void setAllSizes(Dimension newDimension) {
		setMinimumSize(newDimension);
		setPreferredSize(newDimension);
		setSize(newDimension);
		setMaximumSize(newDimension);
	}

	/**
	 * @param automaticUpdate boolean
	 */
	public void setAutomaticUpdate(boolean automaticUpdate) {
		if (isAutomaticUpdate() != automaticUpdate) {
			if (getRectangularMapModel() != null) {
				if (automaticUpdate) {
					getRectangularMapModel().addRectangularMapChangedListener(ivjEventHandler);
				} else {
					getRectangularMapModel().removeRectangularMapChangedListener(ivjEventHandler);
				}
			}

			_automaticUpdate = automaticUpdate;
		}
	}

	/**
	 * Sets the drawingGrid property (boolean) value.
	 * @param drawingGrid The new value for the property.
	 * @see #getDrawingGrid
	 */
	public void setDrawingGrid(boolean drawingGrid) {
		if (drawingGrid != _drawingGrid) {
			boolean oldValue = _drawingGrid;
			_drawingGrid = drawingGrid;
			firePropertyChange("drawingGrid", Boolean.valueOf(oldValue), Boolean.valueOf(drawingGrid));
			repaint();
		}
	}

	/**
	 * Sets the rectangleHeight property (int) value.
	 * @param rectangleHeight The new value for the property.
	 * @see #getRectangleHeight
	 */
	public void setRectangleHeight(int rectangleHeight) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(rectangleHeight > 0);

		int oldValue = _rectangleHeight;
		_rectangleHeight = rectangleHeight;
		updateComponentSize();
		firePropertyChange("rectangleHeight", oldValue, rectangleHeight);
	}

	/**
	 * Sets the rectangleWidth property (int) value.
	 * @param rectangleWidth The new value for the property.
	 * @see #getRectangleWidth
	 */
	public void setRectangleWidth(int rectangleWidth) {
		Assert.assertTrue(rectangleWidth > 0);

		int oldValue = _rectangleWidth;
		_rectangleWidth = rectangleWidth;
		updateComponentSize();
		firePropertyChange("rectangleWidth", oldValue, rectangleWidth);
	}

	/**
	 * Sets the rectangularMapCellRenderer property (IRectangularMapCellRenderer) value.
	 * @param rectangularMapCellRenderer The new value for the property.
	 * @see #getRectangularMapCellRenderer
	 */
	public void setRectangularMapCellRenderer(IRectangularMapCellRenderer rectangularMapCellRenderer) {
		if (rectangularMapCellRenderer != _rectangularMapCellRenderer) {
			IRectangularMapCellRenderer oldValue = _rectangularMapCellRenderer;
			_rectangularMapCellRenderer = rectangularMapCellRenderer;
			firePropertyChange("rectangularMapCellRenderer", oldValue, rectangularMapCellRenderer);
			repaint();
		}
	}

	public void setRectangularMapModel(IRectangularMapModel newValue) {
		IRectangularMapModel oldValue = getRectangularMapModel();
		if (oldValue != newValue) {

			if (oldValue != null) {
				oldValue.removePropertyChangeListener(ivjEventHandler);
				oldValue.removeRectangularMapChangedListener(ivjEventHandler);
			}

			ivjRectangularMapModel = newValue;

			if (newValue != null) {
				newValue.addPropertyChangeListener(ivjEventHandler);
				if (isAutomaticUpdate())
					newValue.addRectangularMapChangedListener(ivjEventHandler);
			}

			firePropertyChange("rectangularMapModel", oldValue, newValue);

			updateComponentSize();
		}
	}

	public void setRectangularMapState(IRectangularMapPanelState arg1) {
		setRectangularMapStateIntern(arg1);
	}

	private void setRectangularMapStateIntern(IRectangularMapPanelState newValue) {
		IRectangularMapPanelState oldValue = getRectangularMapPanelState();
		if (oldValue != newValue) {
			ivjRectangularMapStateIntern = newValue;

			if (oldValue != null) {
				removeMouseListener(oldValue);
				newValue.setRectangularMap(null);
			}
			if (newValue != null) {
				addMouseListener(newValue);
				newValue.setRectangularMap(this);
			}

			firePropertyChange("rectangularMapState", oldValue, newValue);
		}
	}

	public void setSelectedPoint(Point point) {
		if (point.x == getSelectedPoint().x && point.y == getSelectedPoint().y)
			return;

		if (point.x >= getRectangularMapModel().getColsCount()
			|| point.y >= getRectangularMapModel().getRowsCount())
			return;

		if (point.x < 0 || point.y < 0)
			return;

		Point oldPoint = getSelectedPoint();
		_selectedPoint = point;
		firePropertyChange("selectedPoint", oldPoint, point);

		updatePositionChanged(point.x, point.y);
		updatePositionChanged(oldPoint.x, oldPoint.y);
	}

	protected void updateComponentSize() {
		if (getRectangularMapModel() == null)
			return;

		Insets insets = getInsets();

		double width =
			insets.left + insets.right + (getRectangleWidth() * getRectangularMapModel().getColsCount());
		double height =
			insets.top + insets.bottom + (getRectangleHeight() * getRectangularMapModel().getRowsCount());

		setAllSizes(new Dimension((int)Math.ceil(width), (int)Math.ceil(height)));

		// already called by setSize
		// revalidate();
		// repaint();
	}

	protected void updatePositionChanged(int col, int row) {
		repaint(convertPositionToRectangle(col, row));
	}

	protected void updatePositionChanged(RectangularMapChangedEvent event) {
		updatePositionChanged(event.getCol(), event.getRow());
	}
}