package de.uniwue.ki.gui.util.timetable;


import java.util.Calendar;

import junit.framework.Assert;

/**
 * @author herrler
 *
 * herrler@ki.informatik.uni-wuerzburg.de
 */
public class DateHelper {
	private static final String[] WEEKDAYS = new String[] { "Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"};
	
	public static String getWeekDescription(Calendar day) {
		Calendar endDay = nextDay(day,6);
		return getDateDescription(day)+" - "+getDateDescription(endDay);
	}
	
	public static String getDateDescription(Calendar cal) {
	
		return ((cal.get(Calendar.DATE)<10) ? "0" : "") +cal.get(Calendar.DATE)+"."+
		(((cal.get(Calendar.MONTH)+1)<10) ? "0" : "")+ (cal.get(Calendar.MONTH)+1)+"."+(cal.get(Calendar.YEAR));
	}
	
	public static String getTimeDescription(Calendar cal) {
	
			return ((cal.get(Calendar.HOUR_OF_DAY)<10) ? "0" : "") +cal.get(Calendar.HOUR_OF_DAY)+":"+
			(((cal.get(Calendar.MINUTE))<10) ? "0" : "")+ (cal.get(Calendar.MINUTE));
	}
	public static Calendar getFirstDayOfWeek(Calendar cal) {
		if (cal.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY) //monday to saturday
			return DateHelper.previousDay(cal, cal.get(Calendar.DAY_OF_WEEK) - Calendar.MONDAY);
		if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.SUNDAY) // sunday
			return  DateHelper.previousDay(cal, 6);
		Assert.fail("Error in getFirstDayOfWeek()");
		return null;
	}

	public static boolean isDayEqual(Calendar cal1, Calendar cal2) {
		
		return (
			(cal1.get(Calendar.YEAR) == cal2.get(Calendar.YEAR))
				&& (cal1.get(Calendar.MONTH) == cal2.get(Calendar.MONTH))
				&& (cal1.get(Calendar.DATE) == cal2.get(Calendar.DATE)));
	}
	
	public static Calendar nextDay(final Calendar day) {
		Calendar newcal = (Calendar)day.clone();
		newcal.add(Calendar.DAY_OF_MONTH,1);
		return newcal;
	}
	
	public static Calendar previousDay(final Calendar day) {
		Calendar newcal = (Calendar)day.clone();
		newcal.add(Calendar.DAY_OF_MONTH,-1);
		return newcal;
	}
	
	public static Calendar previousDay(Calendar day, int i) { 
		Calendar newcal = (Calendar)day.clone();
		newcal.add(Calendar.DAY_OF_MONTH,-i);
		return newcal;
	}
	
	public static Calendar nextWeek(Calendar day) {
		return nextDay(day,7); }
	
	public static String getWeekdayVerbalisation(int i) {
		if (Assert.ASSERTION_ON) {
		Assert.assertTrue("getWeekdayVerbalisation got value "+i,-1<i);
		Assert.assertTrue("getWeekdayVerbalisation got value "+i,i<7); }
		return WEEKDAYS[i];
	}
		
	public static Calendar nextDay(Calendar day, int i) { 
		Calendar newcal = (Calendar)day.clone();
		newcal.add(Calendar.DAY_OF_MONTH,i);
		return newcal;
	}
	
}
