/*
 * Created on 30.04.2004
 * 
 * To change this generated comment go to Window>Preferences>Java>Code
 * Generation>Code and Comments
 */
package de.uniwue.ki.gui.util.timetable;

import java.util.Calendar;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.event.StateChangeMulticaster;

/**
 * @author herrler
 *  
 */
public class DefaultTimeTable implements ITimeTable {

    private transient ChangeListener _changeListener = null;

    private List appointments = new LinkedList();

    public void addAppointment(IAppointment appointment) {
        appointments.add(appointment);
        stateChanged();
    }

    public Iterator getAppointments() {
        return appointments.iterator();
    }
    
    public Iterator getSortedAppointments() {
        Collections.sort(appointments);
        return appointments.iterator();
    }

    private void stateChanged() {
        if (_changeListener!=null) {
        _changeListener.stateChanged(new ChangeEvent("Appointments chnaged"));
        }
    }

    public void addChangeListener(ChangeListener newListener) {
        _changeListener = StateChangeMulticaster.add(_changeListener,
                newListener);
    }

    public void removeChangeListener(ChangeListener newListener) {
        _changeListener = StateChangeMulticaster.remove(_changeListener,
                newListener);
    }

    public DefaultTimeTable(List appointments) {
        super();
        this.appointments = appointments;
    }

    public DefaultTimeTable() {
        super();
    }

    public synchronized boolean overwriteAppointments(List list) {
        // Sort by startTime
        Collections.sort(list);
        Collections.sort(appointments);

        Iterator coveringIntervals = getAppointmentsBetween(appointments,
                ((IAppointment) list.get(0)).getStartTime(),
                ((IAppointment) list.get(list.size() - 1)).getEndTime());

        // overwrite something
        if (coveringIntervals != null && coveringIntervals.hasNext()) {
            removeAppointments(coveringIntervals);
        }
        
        addAppointments(list.iterator());
        Collections.sort(appointments);
        return (coveringIntervals != null && coveringIntervals.hasNext());
   }

    private void addAppointments(Iterator it) {
        while (it.hasNext()) {
            IAppointment element = (IAppointment) it.next();
            addAppointment(element); 
            }
    }

    private void removeAppointments(Iterator it) {
        while (it.hasNext()) {
            IAppointment element = (IAppointment) it.next();
            removeAppointment(element); 
            }
    }
    
    public void removeAppointment(IAppointment element) {
        appointments.remove(element);
        stateChanged();
    }

    public Iterator getAppointmentsBetween(List appointments, Calendar start,
            Calendar end) {
        List result = new LinkedList();
        List appointmentList = appointments;
        int slicestart = 0;

        // Check if all intervals are before given TimePeriod
        if (start.after(((IAppointment) appointmentList.get(appointmentList
                .size() - 1)).getEndTime())) {
            slicestart = appointmentList.size();
        }
        // Find index of first interval which is not completely before given
        // period
        else {
            int i;
            for (i = 0; i < appointmentList.size(); i++) {
                IAppointment slice = (IAppointment) appointmentList.get(i);
                if (!slice.getStartTime().before(start)) break;
            }

            // Look, if one slice overlaps begin of period
            if (i != 0
                    && ((IAppointment) appointmentList.get(i - 1)).getEndTime()
                            .after(start)) {
                slicestart = i - 1;
            } else {
                slicestart = i;
            }
        }

        //		 Add slices to result list starting from computed slicestart ending at
        // period end
        for (int j = slicestart; j < appointmentList.size(); j++) {
            IAppointment slice = (IAppointment) appointmentList.get(j);
            if (slice.getStartTime().before(end)) {
                result.add(slice);
                continue;
            } else {
                break;
            }
        }
        return result.iterator();
    }
}