package de.uniwue.ki.gui.util.timetable;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.gui.util.JDisposeablePanel;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author herrler, schoerger
 * 
 * This class can keep more than one WeekSchedulePanel, but displayed is only
 * one. Via ComboBox you can navigate between the different WeekSchedulePanels.
 * 
 * It is recommended that this panel is set into a scrollpane in cases of a
 * small number for vertical pixels ( < 1000).
 */

public class SchedulePanel extends JDisposeablePanel implements ISchedulePanel {

    private AnnotedTimeTableModel _annotedTimeTableModel = new AnnotedTimeTableModel();

    private WeekSchedulePanel _scheduleWeekPanel;
    private LinkedList _contextActionList = new LinkedList();
    private JLabel _descriptionLabel;
    private JComboBox _comboBox;
    private JScrollPane scrollPane=null;
    
    private final transient ChangeListener changeListener = new ChangeListener() {
        public void stateChanged(ChangeEvent arg0) {
            if ((_comboBox!=null) && ((Calendar)_comboBox.getSelectedItem()!=null)) 
            updateStateChanged((Calendar)_comboBox.getSelectedItem());            
        }};

    class WeekSelectEventHandler implements ActionListener {

        SchedulePanel _myBoss = null;

        WeekSelectEventHandler(SchedulePanel boss) {
            super();
            _myBoss = boss;
        }

        public void actionPerformed(ActionEvent e) {
            JComboBox combo = (JComboBox) e.getSource();

            if (combo.getSelectedItem() instanceof Calendar) {
                Calendar week = (Calendar) combo.getSelectedItem();

                _myBoss.updateStateChanged(week);
            }
        }
    }

    /**
     * Updates the view according to the model, the week has to choosen
     */
    private void updateStateChanged(Calendar week) {
        _descriptionLabel.setText("Angezeigter Zeitraum: "
                + DateHelper.getWeekDescription(week));
        if (_scheduleWeekPanel!=null) remove(_scheduleWeekPanel);
        _scheduleWeekPanel = createAndInitializeWeekSchedulePanel(week);
        add(_scheduleWeekPanel, BorderLayout.SOUTH);
        validate();
        repaint();
    }

    class WeekRenderer extends DefaultListCellRenderer implements
            ListCellRenderer {

        public Component getListCellRendererComponent(JList list, Object value,
                int index, boolean isSelected, boolean cellHasFocus) {
            return super.getListCellRendererComponent(list, DateHelper
                    .getWeekDescription((Calendar) value), index, isSelected,
                    cellHasFocus);
        }
    }

    public SchedulePanel(ITimeTable table) {
        super();
        table.addChangeListener(changeListener);
        _annotedTimeTableModel.setTimeTable(table);
        initialize();
    }
    
    
    public void dispose() {
        super.dispose();
        _annotedTimeTableModel.getTimeTable().removeChangeListener(changeListener);
        System.out.println("Juhu dispose in SchedulePanel wird aufgerufen");
    }

    public SchedulePanel(AnnotedTimeTableModel sc) {
        super();
        _annotedTimeTableModel = sc;
        _annotedTimeTableModel.getTimeTable().addChangeListener(changeListener);
        initialize();
    }

    private WeekSchedulePanel createAndInitializeWeekSchedulePanel(Calendar week) {
        _annotedTimeTableModel.setStartDay(week);
        
        WeekSchedulePanel scheduleWeekPanel = new WeekSchedulePanel(
                _annotedTimeTableModel);
        scheduleWeekPanel.setContextActions(_contextActionList);

        return scheduleWeekPanel;
    }

    public void updateGUI() {
        // UpdateGUI
        _scheduleWeekPanel.updateChildPanels();
        _scheduleWeekPanel.revalidate();

    }

    public void addContextAction(IAppointmentContextAction action) {
        _contextActionList.add(action);
        //_scheduleWeekPanel.addContextAction(action);
    }

    private void initialize() {
        java.util.List _weeks = updateWeeks();

       this.setLayout(new BorderLayout());
        JPanel headpanel = new JPanel();
        headpanel.setLayout(new BorderLayout());
        headpanel.add(getDescriptionLabel(),BorderLayout.WEST);
        headpanel.add(getComboBox(), BorderLayout.EAST);

        add(headpanel, BorderLayout.NORTH);
        updateStateChanged((Calendar)_weeks.get(0));
        updateWeeksComboBox();
    }

    private JLabel getDescriptionLabel() {
        if (_descriptionLabel ==null) {
        _descriptionLabel = new JLabel("Angezeigter Zeitraum: -leer-"); }
        return _descriptionLabel;
    }

    protected java.util.List updateWeeks() {
        LinkedList weeks=new LinkedList();
        
        // Create startDays for given number of weeks
        Iterator it = ((DefaultTimeTable) _annotedTimeTableModel.getTimeTable())
                .getSortedAppointments();
        IAppointment appointment = (it.hasNext()) ? ((IAppointment) it.next())
                : null;

        Calendar firstDay = null;
        Calendar lastDay = null;
        if (appointment == null) {
            firstDay = GregorianCalendar.getInstance();
            firstDay.set(GregorianCalendar.HOUR,0);
            firstDay.set(GregorianCalendar.MINUTE,0);
            firstDay.set(GregorianCalendar.SECOND,1);
            lastDay = GregorianCalendar.getInstance();
        } else {
            firstDay = appointment.getStartTime();
            for (; it.hasNext(); appointment = (IAppointment) it.next())
                ;
            lastDay = (Calendar)appointment.getEndTime().clone();
            lastDay.set(Calendar.HOUR,23);
            lastDay.set(Calendar.MINUTE,59);
            lastDay.set(Calendar.SECOND,59);
        }
        
        for (Calendar currentWeek = DateHelper.getFirstDayOfWeek(firstDay); currentWeek
                .before(lastDay); currentWeek = DateHelper
                .nextWeek(currentWeek)) {
            weeks.add(currentWeek);
        }
        return weeks;
    }
    
    private void updateWeeksComboBox() {
        updateWeeksComboBox(updateWeeks());
    }


    private void updateWeeksComboBox(java.util.List weeks) {
        DefaultComboBoxModel weekListModel = new DefaultComboBoxModel(
                CollectionsHelper.asArray(weeks.iterator()));
        getComboBox().setModel(weekListModel);
        getComboBox().setSelectedIndex(0);
    }

    private JComboBox getComboBox() {
        if (_comboBox == null) {
            _comboBox = new JComboBox();
            _comboBox.setRenderer(new WeekRenderer());
            _comboBox.addActionListener(new WeekSelectEventHandler(this));
        }
        return _comboBox;
    }

    public static void main(String[] args) {
        try {
            DefaultTimeTable timeTable = new DefaultTimeTable();
            timeTable.addAppointment(new SimpleAppointment(new GregorianCalendar(
                    2004, 4, 10, 8, 0, 0), new GregorianCalendar(2004, 4, 10,
                    8, 15, 0), "Eins"));
            timeTable.addAppointment(new SimpleAppointment(new GregorianCalendar(
                    2004, 4, 10, 8, 15, 0), new GregorianCalendar(2004, 4, 10,
                    8, 30, 0), "Eins zwei"));
            timeTable.addAppointment(new SimpleAppointment(new GregorianCalendar(
                    2004, 4, 9, 9, 30, 0), new GregorianCalendar(2004, 4, 11,
                    10, 30, 0), "Zwei"));

            JFrame frame = new JFrame();
            SchedulePanel panel = new SchedulePanel(timeTable);
            frame.setContentPane(panel);
            frame.setSize(panel.getSize());
            frame.addWindowListener(new WindowAdapter() {

                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
            frame.pack();
            frame.show();
            Insets insets = frame.getInsets();
            frame.setSize(frame.getWidth() + insets.left + insets.right, frame
                    .getHeight()
                    + insets.top + insets.bottom);
            frame.setVisible(true);
            
            // nachtrgliches einfgen
            timeTable.addAppointment(new SimpleAppointment(new GregorianCalendar(
                    2004, 4, 9, 11, 30, 0), new GregorianCalendar(2004, 4, 11,
                    12, 30, 0), "Nigel neu"));

            
        } catch (Throwable exception) {
            System.err.println("Exception occurred in main() of SchedulePanel");
            exception.printStackTrace();
        }
    }

    public double getScalefactor() {
        return _annotedTimeTableModel.getScaleFactor();
    }

    public void setScalefactor(double scalefactor) {
        _annotedTimeTableModel.setScaleFactor(scalefactor);
    }

    public int getColumnWidth() {
        return _annotedTimeTableModel.getColumnWidth();
    }

    public void setColumnWidth(int columnWidth) {
        _annotedTimeTableModel.setColumnWidth(columnWidth);
    }

    public IAppointmentRenderer getRenderer() {
        return _annotedTimeTableModel.getAppointmentRenderer();
    }

    public void setRenderer(IAppointmentRenderer renderer) {
        _annotedTimeTableModel.setAppointmentRenderer(renderer);
        //_scheduleWeekPanel.setRenderer(renderer);
    }

    public void setStartHour(int _startHour) {
        _annotedTimeTableModel.setStartHourInTheMorning(_startHour);
        //_scheduleWeekPanel.setStartHour(_startHour);
    }

    public int getStartHour() {
        return _annotedTimeTableModel.getStartHourInTheMorning();
    }

    public void setEndHour(int _endHour) {
        _annotedTimeTableModel.setEndHourinTheEvening(_endHour);
        //_scheduleWeekPanel.setEndHour(_endHour);
    }

    public int getEndHour() {
        return _annotedTimeTableModel.getEndHourinTheEvening();
    }

    public Iterator getAppointments() {
        return _annotedTimeTableModel.getTimeTable().getAppointments();
    }

    public java.util.List getAppointmentList() {
        //TODO: make it obsolete and remove
        return CollectionsHelper.asArrayList(_annotedTimeTableModel
                .getTimeTable().getAppointments());
    }

    public int getPixelPerMinute() {
        return _annotedTimeTableModel.getPixelPerMinute();
    }

    public void setPixelPerMinute(int i) {
        _annotedTimeTableModel.setPixelPerMinute(i);
    }

    // nchste funktion: fr drag'n drop
    public WeekSchedulePanel getScheduleWeekPanel() {
        return _scheduleWeekPanel;
    }

    public AnnotedTimeTableModel getAnnotedTimeTableModel() {
        return _annotedTimeTableModel;
    }

    public void setAnnotedTimeTableModel(
            AnnotedTimeTableModel annotedTimeTableModel) {
        this._annotedTimeTableModel = annotedTimeTableModel;
    }
}