package de.uniwue.ki.gui.util.timetable;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;

import de.uniwue.ki.gui.util.actions.AbstractBoundAction;

/**
 * @author herrler, schoerger
 * 
 * This class shows the timetable of one week starting on Monday. For the seven
 * days of the week, in which the start day lies, all the appointments during
 * this time are displayed.
 * 
 * The size of the this panel and ist component can be adjusted by a
 * scale_factor and vlaue, representing the pixels per minute.
 * 
 * It is recommended that this panel is set into a scrollpane in cases of a
 * small number for vertical pixels ( < 1000).
 */

public class WeekSchedulePanel extends JPanel implements ISchedulePanel {

    private LinkedList _contextActionList = new LinkedList();

    private IAppointmentRenderer _renderer = new DefaultAppointmentRenderer();

    private IvjEventHandler _ivjeventhandler;

    private AnnotedTimeTableModel _annotedTimeTableModel = new AnnotedTimeTableModel();

    private Calendar firstDay;

    private Calendar lastDay;

    class IvjEventHandler extends MouseAdapter implements MouseListener {

        private WeekSchedulePanel _parent = null;

        public IvjEventHandler(WeekSchedulePanel parent) {
            _parent = parent;
        }

        private JPopupMenu getPopupMenu(Object appointment) {
            JPopupMenu popup = new JPopupMenu();
            Iterator it = _contextActionList.iterator();
            if (!it.hasNext()) return null; // kein popup
            while (it.hasNext()) {
                IAppointmentContextAction element = (IAppointmentContextAction) it
                        .next();
                if (element.canFire(appointment)) {
                    AbstractBoundAction action = element
                            .getAbstractBoundAction(appointment);
                    popup.add(AbstractBoundAction
                            .createJMenuItem(action, false));
                }
            }
            return popup;
        }

        public void mouseClicked(MouseEvent mouseEvent) {

            if (mouseEvent.getModifiers() == 4) {

                Object o = mouseEvent.getComponent();
                JPopupMenu popup = getPopupMenu(o);

                if (popup != null)
                        popup.show(mouseEvent.getComponent(),
                                mouseEvent.getX(), mouseEvent.getY());
            }

        }
    }

    /*
     * weekday: one day of the week to display
     */
    public WeekSchedulePanel(Calendar weekday, java.util.List appointments) {
        super();

        // add additional information to constraints
        _annotedTimeTableModel.setStartDay(weekday);
        _annotedTimeTableModel.setAppointmentList(appointments);

        firstDay = DateHelper.getFirstDayOfWeek(_annotedTimeTableModel.getStartDay());
        lastDay = DateHelper.nextDay(firstDay, 6);

        _ivjeventhandler = new IvjEventHandler(this);

        updateChildPanels();
    }

    public WeekSchedulePanel(AnnotedTimeTableModel annotedTimeTableModel) {
        super();
        _annotedTimeTableModel = annotedTimeTableModel;

        firstDay = DateHelper.getFirstDayOfWeek(_annotedTimeTableModel.getStartDay());
        lastDay = DateHelper.nextDay(firstDay, 6);

        _ivjeventhandler = new IvjEventHandler(this);

        updateChildPanels();
    }

    /*
     * weekday: one day of the week to display
     */
    public WeekSchedulePanel(Calendar weekday, java.util.List appointments,
            int beginHour, int endHour) {
        super();

        // add additional information to constraints
        _annotedTimeTableModel.setStartDay(weekday);
        _annotedTimeTableModel.setAppointmentList(appointments);
        _annotedTimeTableModel.setStartHourInTheMorning(beginHour);
        _annotedTimeTableModel.setEndHourinTheEvening(endHour);

        firstDay = DateHelper.getFirstDayOfWeek(_annotedTimeTableModel.getStartDay());
        lastDay = DateHelper.nextDay(firstDay, 6);

        _ivjeventhandler = new IvjEventHandler(this);

        updateChildPanels();
    }

    public void updateChildPanels() {
        this.removeAll();
        GridBagLayout grodBafLayout = new GridBagLayout();
        setLayout(grodBafLayout);

        GridBagConstraints defaultConstraints = new GridBagConstraints();

        { // head
            defaultConstraints.gridx = 0;
            defaultConstraints.gridy = 0;

            JLabel headLabel = new JLabel("Zeit", SwingConstants.CENTER);
            headLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(headLabel, defaultConstraints);
            add(headLabel);
        }

        //      Paint head
        //        JLabel label1 = new JLabel(DateHelper.getWeekdayVerbalisation(cal
        //                .get(Calendar.DAY_OF_WEEK) - 1), SwingConstants.CENTER);
        //        label1.setPreferredSize(new Dimension(60, 25));
        //        layout.setConstraints(label1, c);
        //        panel.add(label1);

        {
            defaultConstraints.gridx = 1;
            JLabel dayLabel = new JLabel("Monday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(dayLabel, defaultConstraints);
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 2;
            JLabel dayLabel = new JLabel("Thuesday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(dayLabel, defaultConstraints);
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 3;
            JLabel dayLabel = new JLabel("Wednesday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(dayLabel, defaultConstraints);
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 4;
            JLabel dayLabel = new JLabel("Thursday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(dayLabel, defaultConstraints);
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 5;
            JLabel dayLabel = new JLabel("Friday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            grodBafLayout.setConstraints(dayLabel, defaultConstraints);
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 6;
            JLabel dayLabel = new JLabel("Saturday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            add(dayLabel);
        }
        {
            defaultConstraints.gridx = 7;
            JLabel dayLabel = new JLabel("Sunday", SwingConstants.CENTER);
            dayLabel.setPreferredSize(new Dimension(60, 25));
            add(dayLabel);
        }

        defaultConstraints.gridy = 1;
        defaultConstraints.gridx = 0;
        JComponent firstpanel = getTimePanel();
        grodBafLayout.setConstraints(firstpanel, defaultConstraints);
        add(firstpanel);//add(firstpanel);

        for (Calendar currentDate = firstDay; currentDate.before(DateHelper
                .nextDay(lastDay)); currentDate = DateHelper
                .nextDay(currentDate)) {

            defaultConstraints.gridx++;
            JComponent dayPanel = getDayPanel(currentDate);
            grodBafLayout.setConstraints(dayPanel, defaultConstraints);
            this.add(dayPanel);
        }
    }

    public void addContextAction(IAppointmentContextAction action) {
        _contextActionList.add(action);
    }

    public void setContextActions(LinkedList actionList) {
        _contextActionList = actionList;
    }

    private LinkedList getSortedAppointmentsOfDay(Calendar day) {
        LinkedList ret = new LinkedList();
        Iterator it = _annotedTimeTableModel.getTimeTable().getAppointments();
        while (it.hasNext()) {
            IAppointment element = (IAppointment) it.next();
            if (DateHelper.isDayEqual(element.getStartTime(), day))
                    ret.add(element);
        }

        Collections.sort(ret, new Comparator() {

            public int compare(Object o1, Object o2) {
                if (((IAppointment) o1).getStartTime().before(
                        ((IAppointment) o2).getStartTime())) return -1;
                if (((IAppointment) o1).getStartTime().after(
                        ((IAppointment) o2).getStartTime())) return 1;
                return 0;
            }

            public boolean equals(Object obj) {
                return false;
            }
        });

        return ret;
    }

    private JComponent getTimePanel() {
        JPanel timePanel = new JPanel();
        GridBagLayout layout = new GridBagLayout();
        timePanel.setLayout(layout);
        GridBagConstraints defaultLayoutConstraint = new GridBagConstraints();
        defaultLayoutConstraint.gridwidth = GridBagConstraints.REMAINDER;

        //		{ // head
        //			JLabel headLabel = new JLabel("Zeit", SwingConstants.CENTER);
        //			headLabel.setPreferredSize(new Dimension(60, 25));
        //			layout.setConstraints(headLabel, defaultLayoutConstraint);
        //			timePanel.add(headLabel);
        //		}

        for (int hour = getStartHour(); hour < getEndHour(); hour++) {
            for (int min = 0; min < _annotedTimeTableModel.getSlotSize() + 1; min = min
                    + _annotedTimeTableModel.getSlotSize()) {
                JLabel timeLabel = new JLabel(((hour < 10) ? "0" : "") + hour
                        + ":" + ((min < 10) ? "0" : "") + min,
                        SwingConstants.CENTER);
                timeLabel.setBorder(new EtchedBorder());

                Dimension dimension = new Dimension(60, (int) (_annotedTimeTableModel
                        .getSlotSize()
                        * _annotedTimeTableModel.getPixelPerMinute() * _annotedTimeTableModel.getScaleFactor()));
                timeLabel.setPreferredSize(dimension);
                timeLabel.setMinimumSize(dimension);
                layout.setConstraints(timeLabel, defaultLayoutConstraint);
                timePanel.add(timeLabel);
            }
        }
        return timePanel;
    }

    private JComponent getDayPanel(Calendar cal) {

        JPanel panel = new JPanel();
        GridBagLayout layout = new GridBagLayout();
        panel.setLayout(layout);
        GridBagConstraints c = new GridBagConstraints();
        //c.weighty = 1;
        c.gridwidth = GridBagConstraints.REMAINDER;

        // Paint body
        int curpos = 0;
        Iterator it = getSortedAppointmentsOfDay(cal).iterator();

        while (it.hasNext()) {
            IAppointment element = (IAppointment) it.next();

            // Fill empty space
            if (getYPosOfAppointment(element) > curpos) {
                JLabel label = new JLabel();

                Dimension dimension = new Dimension(getColumnWidth(),
                        getYPosOfAppointment(element) - curpos);
                label.setPreferredSize(dimension);
                label.setMinimumSize(dimension);
                label.setBorder(new EtchedBorder());
                label.setBackground(Color.LIGHT_GRAY);
                layout.setConstraints(label, c);
                panel.add(label);
            }
            // Paint Appointment
            JComponent appointmentComponent = getRenderer()
                    .getAppointmentComponent(element);
            appointmentComponent.addMouseListener(_ivjeventhandler);

            Dimension dimension = new Dimension(getColumnWidth(),
                    getYLengthOfAppointment(element));
            appointmentComponent.setPreferredSize(dimension);
            appointmentComponent.setMinimumSize(dimension);
            layout.setConstraints(appointmentComponent, c);
            panel.add(appointmentComponent);
            curpos = getYPosOfEndDate(element.getEndTime());
        }

        // Fill left space
        if (curpos < getYPosOfEndTime(getEndHour(), 0)) {
            JLabel label = new JLabel();

            Dimension dimension = new Dimension(getColumnWidth(),
                    getYPosOfEndTime(getEndHour(), 0) - curpos);
            label.setPreferredSize(dimension);
            label.setMinimumSize(dimension);
            label.setBorder(new EtchedBorder());
            label.setBackground(Color.LIGHT_GRAY);
            layout.setConstraints(label, c);
            panel.add(label);
        }

        return panel;
    }

    public static void main(String[] args) {
        try {
            LinkedList appointmentlist = new LinkedList();
            appointmentlist.add(new SimpleAppointment(new GregorianCalendar(
                    2002, 10, 26, 8, 30, 0), new GregorianCalendar(2002, 10,
                    26, 9, 30, 0), "Eins"));
            appointmentlist.add(new SimpleAppointment(new GregorianCalendar(
                    2002, 10, 26, 10, 30, 0), new GregorianCalendar(2002, 11,
                    26, 11, 19, 0), "Eins zwei"));
            appointmentlist.add(new SimpleAppointment(new GregorianCalendar(
                    2002, 10, 27, 9, 30, 0), new GregorianCalendar(2002, 10,
                    27, 10, 30, 0), "Zwei"));
            appointmentlist.add(new SimpleAppointment(new GregorianCalendar(
                    2002, 10, 27, 10, 30, 0), new GregorianCalendar(2002, 10,
                    27, 11, 30, 0), "Zwei"));
            appointmentlist.add(new SimpleAppointment(new GregorianCalendar(
                    2002, 10, 28, 9, 30, 0), new GregorianCalendar(2002, 10,
                    28, 11, 30, 0), "Drei"));
            JFrame frame = new JFrame();
            JPanel panel = new WeekSchedulePanel(new GregorianCalendar(2002,
                    10, 25, 8, 30, 0), appointmentlist);
            //ScrollPane com = new ScrollPane();
            //com.add(panel);
            frame.setContentPane(panel);
            frame.setSize(panel.getSize());
            frame.addWindowListener(new java.awt.event.WindowAdapter() {

                public void windowClosing(java.awt.event.WindowEvent e) {
                    System.exit(0);
                }
            });
            frame.pack();
            frame.show();
            Insets insets = frame.getInsets();
            frame.setSize(frame.getWidth() + insets.left + insets.right, frame
                    .getHeight()
                    + insets.top + insets.bottom);
            frame.setVisible(true);
        } catch (Throwable exception) {
            System.err.println("Exception occurred in main() of SchedulePanel");
            exception.printStackTrace();
        }
    }

    /*
     * Time-Calculations
     */

    /**
     * If end time ends at 0:00 pm of the next day, use 24 instead of 0 for the
     * hour value
     * 
     * @param hour
     *            Hour_of_Day
     * @param minute
     * @return int y position in pixels
     */
    private int getYPosOfEndTime(int hour, int minute) {
        if (hour == 0 && minute == 0)
            return (int) ((((24 - getStartHour()) * 60) + minute)
                    * _annotedTimeTableModel.getPixelPerMinute() * _annotedTimeTableModel.getScaleFactor());
        else
            return getYPosOfTime(hour, minute);
    }

    private int getYPosOfTime(int hour, int minute) {
        return (int) ((((hour - getStartHour()) * 60) + minute)
                * _annotedTimeTableModel.getPixelPerMinute() * _annotedTimeTableModel.getScaleFactor());
    }

    private int getYPosOfEndDate(Calendar cal) {
        return getYPosOfEndTime(cal.get(Calendar.HOUR_OF_DAY), cal
                .get(Calendar.MINUTE));
    }

    private int getYPosOfDate(Calendar cal) {
        return getYPosOfTime(cal.get(Calendar.HOUR_OF_DAY), cal
                .get(Calendar.MINUTE));
    }

    private int getYPosOfAppointment(IAppointment appointment) {
        return getYPosOfDate(appointment.getStartTime());
    }

    public int getYLengthOfAppointment(IAppointment appointment) {
        return ((getYPosOfEndDate(appointment.getEndTime()) - getYPosOfDate(appointment
                .getStartTime())));
    }

    /*
     * Methods for implementing ISchedulePanel
     */
    
    public int getColumnWidth() {
        return _annotedTimeTableModel.getColumnWidth();
    }

    public void setColumnWidth(int _columnWidth) {
        _annotedTimeTableModel.setColumnWidth(_columnWidth);
    }

    public IAppointmentRenderer getRenderer() {
        return _annotedTimeTableModel.getAppointmentRenderer();
    }

    public void setRenderer(IAppointmentRenderer _renderer) {
        _annotedTimeTableModel.setAppointmentRenderer(_renderer);
    }

    public void setStartHour(int _startHour) {
        _annotedTimeTableModel.setStartHourInTheMorning(_startHour);

    }

    public int getStartHour() {
        return _annotedTimeTableModel.getStartHourInTheMorning();
    }

    public void setEndHour(int _endHour) {
        _annotedTimeTableModel.setEndHourinTheEvening(_endHour);

    }

    public int getEndHour() {
        return _annotedTimeTableModel.getEndHourinTheEvening();
    }

}