package de.uniwue.ki.mas.application;

import java.io.*;
import java.util.Arrays;
import java.util.Iterator;

import ViolinStrings.Strings;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.SeSAmDeclarationFactory;
import de.uniwue.ki.mas.xml.XMLIniFileException;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.comparator.NamedComparator;

/**
 * @author oechslein
 * 
 * Creation date: 06.09.2003
 */
public class AllPrimitiveOutput {

	private static final String[] REPLACE_SIGNATURE_ARRAY =
		{ "Number<Double>", "Number", "Object", "T", "SimT", "SimObject" };

	private static final String[] REPLACE_DOCU_ARRAY =
		{
			"^2",
			"$^2$",
			"&",
			"\\&",
			"<",
			"$<$",
			">",
			"$>$",
			"%",
			"\\%",
			"~",
			"$\\sim$",
			"Arg1",
			"\\textit{Arg1}",
			"Arg2",
			"\\textit{Arg2}",
			"Arg3",
			"\\textit{Arg3}",
			"Arg4",
			"\\textit{Arg4}",
			"Arg5",
			"\\textit{Arg5}",
			"Arg6",
			"\\textit{Arg6}" };

	public static void main(String[] args) {
		try {
			SeSAmCmd.loadPluginIniFiles();

			File outFile =
				new File("C:/Dokumente und Einstellungen/oechslein/Eigene Dateien/Diss/anhang/FunktionenOutput.tex");

			Writer writer = new BufferedWriter(new FileWriter(outFile));

			writer.write("\\begin{xtabular}{|p{\\textwidth}|}\n");

			ISeSAmDeclaration seSAmDeclaration = SeSAmDeclarationFactory.createSeSAmDeclaration(false);

			Iterator it =
				CollectionsHelper
					.asSortedArrayList(
						seSAmDeclaration
							.getSeSAmModelDeclaration()
							.getAvailibleFunctionDeclarationList()
							.getFunctionDeclarations(),
						NamedComparator.getInstance())
					.iterator();
			while (it.hasNext()) {
				IFunctionDeclaration element = (IFunctionDeclaration)it.next();

				String functionSignature =
					replace(replace(element.getFullSignature(), REPLACE_SIGNATURE_ARRAY), REPLACE_DOCU_ARRAY);

				String docu = replace(element.getDocumentation(), REPLACE_DOCU_ARRAY);

				writer.write("\\normalsize\\function{");
				writer.write(functionSignature);
				writer.write("}: \\vspace*{0.4em} \\newline \n\\small ");

				writer.write(docu);
				writer.write(" \\\\ \\hline\n\n");
			}

			writer.write("\\end{xtabular}\n\\normalsize\n");
			writer.close();

			Iterator types = seSAmDeclaration.getSeSAmModelDeclaration().getAvailibleTypeList().iterator();
			while (types.hasNext()) {
				IType element = (IType)types.next();
				if (element instanceof IGenericType)
					System.out.println(element.getName());
			}
			System.out.println();
			System.out.println();
			types = seSAmDeclaration.getSeSAmModelDeclaration().getAvailibleTypeList().iterator();
			while (types.hasNext()) {
				IType element = (IType)types.next();
				if (element instanceof ISpecificType)
					System.out.println(element.getName());
			}

		} catch (XMLIniFileException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static String replace(String result, String[] strings) {
		Iterator replayes = Arrays.asList(strings).iterator();

		while (replayes.hasNext()) {
			String oldStr = (String)replayes.next();
			String newStr = (String)replayes.next();
			result = Strings.change(result, oldStr, newStr);
		}
		return result;
	}

	/*	private static String insertLineBreaks(String string, int len) {
			if (string.length() > len) {
				return string.substring(0, len) + " \\newline\n  " + insertLineBreaks(string.substring(len), len);
			} else {
				return string;
			}
		}*/
}
