package de.uniwue.ki.mas.application.remote;

import java.io.*;
import java.net.InetAddress;
import java.net.UnknownHostException;

public class RemoteCommand implements Serializable {
	private static int _id = -1;
	private static int _randomNumber = (int) (Math.random() * Integer.MAX_VALUE);

	private String _messageId;
	private String _commandId;
	private Object _argument;

	private RemoteCommand(String commandId, Object argument, String messageId) {
		super();

		_messageId = messageId;
		_commandId = commandId;
		_argument = argument;
	}

	public RemoteCommand(String commandId, Object argument) {
		this(commandId, argument, createNextMessageId());
	}

	public RemoteCommand(String commandId) {
		this(commandId, null);
	}

	private static String createNextMessageId() {
		_id++;
		try {
			return InetAddress.getLocalHost().getHostAddress() + "_" + _randomNumber + "_" + _id;

		} catch (UnknownHostException e) {
			return _randomNumber + "_" + _id;
		}
	}

	public String getMessageId() {
		return _messageId;
	}

	public Object getCommandId() {
		return _commandId;
	}

	public Object getArgument() {
		return _argument;
	}

	public void writeObject(DataOutputStream out) throws IOException {
		RemoteReaderWriter.outputObject(getCommandId(), out);
		RemoteReaderWriter.outputObject(getArgument(), out);
		RemoteReaderWriter.outputObject(getMessageId(), out);
	}

	public static RemoteCommand readObject(DataInputStream in) throws IOException {
		return new RemoteCommand(
			(String) RemoteReaderWriter.inputObject(in),
			RemoteReaderWriter.inputObject(in),
			(String) RemoteReaderWriter.inputObject(in));
	}
}
