package de.uniwue.ki.mas.application.remote;

import java.io.*;

import junit.framework.Assert;

import org.exolab.castor.xml.*;

import de.uniwue.ki.mas.xml.castor.XMLRootObject;

/**
 * @author oechslein
 * 
 * Creation date: 26.03.2003
 */
final class RemoteReaderWriter {
	private static final int BooleanType = 3;
	private static final int ByteArrayType = 1;
	private static final int NullType = 5;
	private static final int StringArrayType = 4;
	private static final int StringType = 2;
	private static final int XMLRootObjectType = 0;

	public static Object inputObject(DataInputStream in) throws IOException {
		int type = in.readInt();

		if (type == XMLRootObjectType) {
			String className = readString(in);
			String answerString = readString(in);
			try {
				return Unmarshaller.unmarshal(Class.forName(className), new StringReader(answerString));
			} catch (MarshalException e) {
				throw new IOException(e.getMessage());
			} catch (ValidationException e) {
				throw new IOException(e.getMessage());
			} catch (ClassNotFoundException e) {
				throw new IOException(e.getMessage());
			}

		} else if (type == ByteArrayType) {
			int length = in.readInt();
			byte[] answerArray = new byte[length];
			int readLen = 0;
			while (readLen < length) {
				int currReadLen = in.read(answerArray, readLen, length - readLen);
				if (currReadLen == -1) {
					throw new IOException("Error during byte[] reading.");
				} else {
					readLen += currReadLen;
				}
			}
			return answerArray;

		} else if (type == StringType) {
			return readString(in);

		} else if (type == BooleanType) {
			return Boolean.valueOf(in.readBoolean());

		} else if (type == NullType) {
			return null;

		} else if (type == StringArrayType) {
			int length = in.readInt();
			String[] answerArray = new String[length];
			for (int i = 0; i < length; i++) {
				answerArray[i] = readString(in);
			}
			return answerArray;

		} else {
			Assert.fail("wrong type: " + type);
			return null;
		}
	}

	private static String readString(DataInputStream in) throws IOException {
		int length = in.readInt();
		StringBuffer buff = new StringBuffer();
		for (int i = 0; i < length; i++) {
			buff.append(in.readChar());
		}
		return buff.toString();
	}

	public static void outputObject(Object obj, DataOutputStream out) throws IOException {
		if (obj instanceof XMLRootObject) {
			out.writeInt(XMLRootObjectType);

			writeString(out, obj.getClass().getName());
			StringWriter writer = new StringWriter();
			try {
				((XMLRootObject) obj).marshal(writer);
			} catch (MarshalException e) {
				throw new IOException(e.getMessage());
			} catch (ValidationException e) {
				throw new IOException(e.getMessage());
			}
			writeString(out, writer.toString());

		} else if (obj instanceof byte[]) {
			out.writeInt(ByteArrayType);
			out.writeInt(((byte[]) obj).length);
			out.write((byte[]) obj);

		} else if (obj instanceof String) {
			out.writeInt(StringType);
			writeString(out, (String) obj);

		} else if (obj instanceof Boolean) {
			out.writeInt(BooleanType);
			out.writeBoolean(((Boolean) obj).booleanValue());

		} else if (obj instanceof String[]) {
			out.writeInt(StringArrayType);
			String[] str = (String[]) obj;
			out.writeInt(str.length);
			for (int i = 0; i < str.length; i++) {
				writeString(out, str[i]);
			}

		} else if (obj == null) {
			out.writeInt(NullType);

		} else {
			Assert.fail("Not possible to output " + obj);
		}
		
		out.flush();
	}

	private static void writeString(DataOutputStream out, String str) throws IOException {
		out.writeInt(str.length());
		out.writeChars(str);
	}
}
