package de.uniwue.ki.mas.application.remote.server;

import java.io.*;
import java.util.Map;

import de.uniwue.ki.mas.application.remote.ISimulationControlConnection;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: 18.09.2002, 10:06:19
 * @author oechslein
 */
public class RemoteModelAsFileSimulationTask extends AbstractCopyable implements IRemoteModelSimulationTask {
	private final File _modelFile;
	private final String _simulationName;

	public RemoteModelAsFileSimulationTask(File modelFile, String simulationName) {
		super();

		_modelFile = modelFile;
		_simulationName = simulationName;
	}

	public String getName() {
		return getSimulationName() + " [" + getModelFileName() + "]";
	}

	public ICopyable copy(Map objectMapping) {
		return new RemoteModelAsFileSimulationTask(getModelFile(), getSimulationName());
	}

	public String getSimulationName() {
		return _simulationName;
	}

	public String getModelFileName() {
		return _modelFile.toString();
	}

	private File getModelFile() {
		return _modelFile;
	}

	public int loadModelAndSimulate(ISimulationControlConnection simulationControlConnection)
		throws IOException {
		if (simulationControlConnection
			.loadModelStream(getModelFileName().toString(), new FileReader(getModelFile()))) {
			if (simulationControlConnection.startSimulation(getSimulationName())) {
				return LOAD_MODEL_START_SIM_SUCCESS;
			} else {
				return START_SIMULATION_ERROR;
			}

		} else {
			return MODEL_FILE_LOAD_ERROR;
		}
	}
}
