package de.uniwue.ki.mas.application.remote.server;

import java.io.*;
import java.util.Map;

import de.uniwue.ki.mas.application.remote.ISimulationControlConnection;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.xml.XMLWriter;
import de.uniwue.ki.mas.xml.writer.XMLWriteException;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.file.FileUtilities;

/**
 * Creation date: 18.09.2002, 10:06:19
 * @author oechslein
 */
public class RemoteModelAsStringSimulationTask
	extends AbstractCopyable
	implements IRemoteModelSimulationTask {

	private final String _modelAsString;
	private final String _modelName;
	private final String _simulationName;

	public RemoteModelAsStringSimulationTask(File modelFile, String simulationName)
		throws FileNotFoundException, IOException {

		this(modelFile.toString(), new FileReader(modelFile), simulationName);
	}

	public RemoteModelAsStringSimulationTask(ISeSAmDeclaration seSAmDeclaration, String simulationName)
		throws IOException, XMLWriteException {

		this(
			seSAmDeclaration.getFile() + "",
			XMLWriter.writeXMLString(seSAmDeclaration, null),
			simulationName);
	}

	public RemoteModelAsStringSimulationTask(String modelName, Reader modelReader, String simulationName)
		throws IOException {

		this(modelName, FileUtilities.getContentsAsString(modelReader), simulationName);
	}

	public RemoteModelAsStringSimulationTask(String modelName, String modelAsString, String simulationName) {
		_modelName = modelName;
		_modelAsString = modelAsString;
		_simulationName = simulationName;
	}

	public String getName() {
		return getSimulationName() + " [" + getModelFileName() + "]";
	}

	public ICopyable copy(Map objectMapping) {
		return new RemoteModelAsStringSimulationTask(_modelName, _modelAsString, _simulationName);
	}

	public String getSimulationName() {
		return _simulationName;
	}

	public String getModelFileName() {
		return _modelName;
	}

	public int loadModelAndSimulate(ISimulationControlConnection simulationControlConnection)
		throws IOException {
		boolean loadModelResult =
			((StateCacheSimulationControlConnection) simulationControlConnection).loadModelStream(
				getModelFileName().toString(),
				new StringReader(_modelAsString));

		if (loadModelResult) {
			boolean startSimResult = simulationControlConnection.startSimulation(getSimulationName());
			if (startSimResult) {
				return LOAD_MODEL_START_SIM_SUCCESS;

			} else {
				return START_SIMULATION_ERROR;
			}

		} else {
			return MODEL_FILE_LOAD_ERROR;
		}
	}
}
