package de.uniwue.ki.mas.application.remote.server;

import java.io.*;
import java.util.HashMap;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.application.remote.ISimulationControlConnection;
import de.uniwue.ki.mas.xml.castor.v01.remote.*;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.event.StateChangeMulticaster;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: 04.09.2002, 19:19:00
 * @author oechslein
 */
public class StateCacheSimulationControlConnection
	extends AbstractNamed
	implements ISimulationControlConnection {

	private static final Map _remoteConnectionAutoScheduleMap = new HashMap();

	private boolean _autSchedule = true;
	private transient ChangeListener _changeListener = null;
	private final RemoteSimulationControlConnection _delegate;

	protected XMLRemoteInfo _xmlRemoteInfo;

	public StateCacheSimulationControlConnection(RemoteSimulationControlConnection delegate)
		throws IOException {
		super();

		_delegate = delegate;

		if (_remoteConnectionAutoScheduleMap.containsKey(delegate.getInetAddress())) {
			_autSchedule =
				((Boolean) _remoteConnectionAutoScheduleMap.get(delegate.getInetAddress())).booleanValue();
		} else {
			setAutSchedule(true);
		}

		refresh();
	}

	public void addChangeListener(ChangeListener changeListenerToAdd) {
		_changeListener = StateChangeMulticaster.add(_changeListener, changeListenerToAdd);
	}

	public void removeChangeListener(ChangeListener changeListenerToRemove) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, changeListenerToRemove);
	}

	protected void fireStateChanged() {
		if (_changeListener != null)
			_changeListener.stateChanged(new ChangeEvent(this));
	}

	public void refresh() throws IOException {
		_xmlRemoteInfo = getSimulationControlConnection().getXMLRemoteInfo();
		fireStateChanged();
	}

	protected boolean _alreadyRefreshing = false;

	private void refreshLater() throws IOException {
		if (!_alreadyRefreshing) {
			_alreadyRefreshing = true;
			Utils.invokeLater("Refresh " + getName(), new Runnable() {
				public synchronized void run() {
					try {
						try {
							wait(1 * 1000);
						} catch (InterruptedException e) {
						}

						try {
							_xmlRemoteInfo = getSimulationControlConnection().getXMLRemoteInfo();

						} catch (IOException e) {
						}

						fireStateChanged();
					} finally {
						_alreadyRefreshing = false;
					}
				}
			});
		}
	}

	public void quit() throws IOException {
		getSimulationControlConnection().quit();
	}

	public void closeConnection() throws IOException {
		getSimulationControlConnection().closeConnection();
	}

	public void restart(File jarFile) throws IOException {
		getSimulationControlConnection().restart(jarFile);
	}

	public boolean isConnected() {
		return getSimulationControlConnection().isConnected();
	}

	public XMLRemoteInfo getXMLRemoteInfo() throws IOException {
		return _xmlRemoteInfo;
	}

	public XMLDetailedRemoteInfo getXMLDetailedRemoteInfo() throws IOException {
		return getSimulationControlConnection().getXMLDetailedRemoteInfo();
	}

	public boolean loadModelFile(File modelFile) throws IOException {
		refreshLater();
		return getSimulationControlConnection().loadModelFile(modelFile);
	}

	public boolean loadModelStream(String remoteModelFileName, Reader reader) throws IOException {
		refreshLater();
		return getSimulationControlConnection().loadModelStream(remoteModelFileName, reader);
	}

	public boolean startSimulation(String name) throws IOException {
		refreshLater();
		return getSimulationControlConnection().startSimulation(name);
	}

	public boolean stopSimulation() throws IOException {
		refreshLater();
		return getSimulationControlConnection().stopSimulation();
	}

	public boolean pause() throws IOException {
		refreshLater();
		return getSimulationControlConnection().pause();
	}

	public boolean setAllowSimulation(boolean allowSimulation) throws IOException {
		refreshLater();
		return getSimulationControlConnection().setAllowSimulation(allowSimulation);
	}

	public boolean unpause() throws IOException {
		refreshLater();
		return getSimulationControlConnection().unpause();
	}

	public String getName() {
		return getSimulationControlConnection().getName();
	}

	public RemoteSimulationControlConnection getSimulationControlConnection() {
		return _delegate;
	}

	public boolean isAutSchedule() {
		return _autSchedule;
	}

	public void setAutSchedule(boolean autSchedule) {
		_autSchedule = autSchedule;
		_remoteConnectionAutoScheduleMap.put(
			getSimulationControlConnection().getInetAddress(),
			Boolean.valueOf(autSchedule));
	}

	public File getAnalysisAsZipFile() throws IOException {
		return getSimulationControlConnection().getAnalysisAsZipFile();
	}

	public void removeAnalysisFromLastZipFile() throws IOException {
		getSimulationControlConnection().removeAnalysisFromLastZipFile();
	}

	public void removeAllUnfinishedFiles() throws IOException {
		getSimulationControlConnection().removeAllUnfinishedFiles();
	}

	public boolean setXMLTimeSchedule(XMLRemoteTimeSchedule xmlRemoteTimeSchedule) throws IOException {
		return getSimulationControlConnection().setXMLTimeSchedule(xmlRemoteTimeSchedule);
	}

	public XMLRemoteTimeSchedule getXMLTimeSchedule() throws IOException {
		return getSimulationControlConnection().getXMLTimeSchedule();
	}
}