package de.uniwue.ki.mas.basics.action;
import java.util.*;

import de.uniwue.ki.mas.basics.action.rest.FunctionActionNoop;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * <BR>Creation date: (30.03.00 10:02:32)
 * @author Christoph Oechslein
 */

public class FunctionActionFactory {

	public static final IFunction createFunctionAction(IFunction arg) {
		if (isNoopFunction(arg)) {
			return getNoopFunction();

		} else {
			return arg;
		}
	}

	public static final IFunction createFunctionAction(IFunction arg1, IFunction arg2) {
		if (isNoopFunction(arg1)) {
			return createFunctionAction(arg2);

		} else {
			if (isNoopFunction(arg2)) {
				return createFunctionAction(arg1);

			} else {
				return FunctionActionTwoFactory.createFunctionAction(arg1, arg2);
			}
		}
	}

	public static final IFunction createFunctionAction(IFunction arg1, IFunction arg2, IFunction arg3) {
		return createFunctionAction(createFunctionAction(arg1, arg2), arg3);
	}

	public static final IFunction createFunctionAction(List actions) {
		switch (actions.size()) {
			case 0 :
				return getNoopFunction();
			case 1 :
				return createFunctionAction((IFunction) actions.get(0));
			case 2 :
				return createFunctionAction((IFunction) actions.get(0), (IFunction) actions.get(1));
			case 3 :
				return createFunctionAction(
					(IFunction) actions.get(0),
					(IFunction) actions.get(1),
					(IFunction) actions.get(2));
			case 4 :
				return createFunctionAction(
					createFunctionAction((IFunction) actions.get(0), (IFunction) actions.get(1)),
					createFunctionAction((IFunction) actions.get(2), (IFunction) actions.get(3)));
			default :
				return FunctionActionList.createFunctionActionList(actions);
		}
	}

	public static final IFunction getNoopFunction() {
		return FunctionActionNoop.getInstance();
	}

	public static final boolean isNoopFunction(IFunction function) {
		if (function instanceof FunctionActionNoop) {
			return true;
		}

		if (function instanceof IFunctionActionTwo) {
			return isNoopFunction(((IFunctionActionTwo) function).getFunctionAction1())
				&& isNoopFunction(((IFunctionActionTwo) function).getFunctionAction2());
		}

		return false;
	}


	public static final IFunction createFunctionAction(
		FunctionActionCallDeclarationList functionActionCallDeclarationList,
		IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		Iterator it = functionActionCallDeclarationList.iterator();
		ArrayList actions = null;

		while (it.hasNext()) {
			IFunctionCallDeclaration functionActionCallDeclaration = (IFunctionCallDeclaration) it.next();
			if (actions == null) {
				actions = new ArrayList();
			}
			actions.add(functionActionCallDeclaration.createFunction(objectCreateFunctionArgs));
		}
		if (actions == null) {
			return getNoopFunction();

		} else {
			// actions.trimToSize();
			return createFunctionAction(actions);
		}
	}
}