package de.uniwue.ki.mas.basics.action;
import java.util.*;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionActionList extends AbstractFunctionAction implements IFunction {
	static IFunction createFunctionActionList(Iterator functionActionIterator) {

		List functionActionList = simplify(CollectionsHelper.asArrayList(functionActionIterator));

		if (functionActionList == null) {
			return FunctionActionFactory.getNoopFunction();

		} else if (functionActionList.size() == 0) {
			return FunctionActionFactory.getNoopFunction();

		} else if (functionActionList.size() == 1) {
			return FunctionActionFactory.createFunctionAction((IFunction) functionActionList.get(0));

		} else if (functionActionList.size() == 2) {
			return FunctionActionFactory.createFunctionAction(
				(IFunction) functionActionList.get(0),
				(IFunction) functionActionList.get(1));

		} else if (functionActionList.size() == 3) {
			return FunctionActionFactory.createFunctionAction(
				(IFunction) functionActionList.get(0),
				(IFunction) functionActionList.get(1),
				(IFunction) functionActionList.get(2));

		} else {
			return new FunctionActionList(functionActionList);
		}
	}

	static IFunction createFunctionActionList(List functionActions) {
		return createFunctionActionList(functionActions.iterator());
	}

	private static List simplify(ArrayList functionActionList) {
		int size = functionActionList.size();

		int i = 0;
		while (i < size) {
			IFunction currentfAction = (IFunction) functionActionList.get(i);

			if (currentfAction instanceof FunctionActionList) {
				functionActionList.remove(i);

				FunctionActionList currentfActionList = (FunctionActionList) currentfAction;
				List currList = Arrays.asList(currentfActionList._functionActionArray);

				if (i == size - 1) {
					functionActionList.addAll(currList);
				} else {
					functionActionList.addAll(i, currList);
				}
				size += currList.size() - 1;
				i += currList.size();

			} else if (FunctionActionFactory.isNoopFunction(currentfAction)) {
				functionActionList.remove(i);
				size--;

			} else {
				++i;
			}
		}

		if (functionActionList.isEmpty()) {
			return null;
		} else {
			functionActionList.trimToSize();
			return functionActionList;
		}
	}

	private IFunction[] _functionActionArray;

	private FunctionActionList(List functionActionList) {
		super();

		_functionActionArray = (IFunction[]) functionActionList.toArray(new IFunction[functionActionList.size()]);
	}

	/**
	 * @param executeFunctionArgs IExecuteFunctionArgs
	 * @throws SimulationRuntimeException
	 * @see de.uniwue.ki.mas.basics.function.IFunction#act(IExecuteFunctionArgs)
	 */
	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		for (int i = 0; i < _functionActionArray.length; i++) {
			try {
				_functionActionArray[i].act(executeFunctionArgs);

			} catch (EmptyIteratorSimulationRuntimeException exception) {
			}
		}
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		StringBuffer buff = new StringBuffer();

		buff.append("[");

		for (int i = 0; i < _functionActionArray.length; i++) {
			buff.append(_functionActionArray[i].getName());

			if (i < _functionActionArray.length - 1) {
				buff.append(", ");
			}
		}

		buff.append("]");

		return buff.toString();
	}
}