package de.uniwue.ki.mas.basics.action;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.rest.FunctionActionNoop;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Creation date: (27.07.2001 15:54:44)
 * @author Christoph Oechslein
 */public final class FunctionActionTwoFactory {
	protected static final IFunction _noopFunction = FunctionActionFactory.getNoopFunction();

	static class AllNoopFunctionActionTwo extends AbstractFunctionAction implements IFunctionActionTwo {

		private static final AllNoopFunctionActionTwo _instance = new AllNoopFunctionActionTwo();

		protected AllNoopFunctionActionTwo() {
			super();
		}

		public static final AllNoopFunctionActionTwo getInstance() {
			return _instance;
		}

		public boolean equals(Object anObject) {
			return (anObject instanceof IFunction)
				&& (FunctionActionFactory.isNoopFunction((IFunction)anObject));
		}

		public int hashCode() {
			return FunctionActionFactory.getNoopFunction().hashCode();
		}

		public IFunction getFunctionAction1() {
			return _noopFunction;
		}

		public IFunction getFunctionAction2() {
			return _noopFunction;
		}

		public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		}

		public String getName() {
			return _noopFunction.getName();
		}
	}

	static final class FirstNoopFunctionActionTwo extends AllNoopFunctionActionTwo {
		private final IFunction _functionAction2;

		public FirstNoopFunctionActionTwo(IFunction function2) {
			super();

			if (Assert.ASSERTION_ON)
				Assert.assertTrue(!FunctionActionFactory.isNoopFunction(function2));
			_functionAction2 = function2;
		}

		public boolean equals(Object anObject) {
			if (anObject instanceof FirstNoopFunctionActionTwo) {
				FirstNoopFunctionActionTwo firstNoopFunctionActionTwo = (FirstNoopFunctionActionTwo)anObject;
				return firstNoopFunctionActionTwo.getFunctionAction2().equals(getFunctionAction2());
			} else {
				return getFunctionAction2().equals(anObject);
			}
		}

		public int hashCode() {
			return getFunctionAction2().hashCode();
		}

		public IFunction getFunctionAction2() {
			return _functionAction2;
		}

		public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			_functionAction2.act(executeFunctionArgs);
		}

		public String getName() {
			return _functionAction2.getName();
		}
	}

	static final class SecondNoopFunctionActionTwo extends AllNoopFunctionActionTwo {
		private final IFunction _functionAction1;

		public SecondNoopFunctionActionTwo(IFunction function1) {
			super();

			Assert.assertTrue(!FunctionActionFactory.isNoopFunction(function1));
			_functionAction1 = function1;
		}

		public boolean equals(Object anObject) {
			if (anObject instanceof SecondNoopFunctionActionTwo) {
				SecondNoopFunctionActionTwo secondNoopFunctionActionTwo =
					(SecondNoopFunctionActionTwo)anObject;
				return secondNoopFunctionActionTwo.getFunctionAction1().equals(getFunctionAction1());

			} else {
				return getFunctionAction1().equals(anObject);
			}
		}

		public int hashCode() {
			return getFunctionAction1().hashCode();
		}

		public IFunction getFunctionAction1() {
			return _functionAction1;
		}

		public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			_functionAction1.act(executeFunctionArgs);
		}

		public String getName() {
			return _functionAction1.getName();
		}
	}

	public static final class FunctionActionTwo
		extends AllNoopFunctionActionTwo
		implements IFunctionActionTwo {
		private final IFunction _functionAction1;
		private final IFunction _functionAction2;

		public FunctionActionTwo(IFunction fAction1, IFunction fAction2) {
			super();

			Assert.assertNotNull(fAction1);
			Assert.assertNotNull(fAction2);
			Assert.assertTrue(!FunctionActionFactory.isNoopFunction(fAction1));
			Assert.assertTrue(!FunctionActionFactory.isNoopFunction(fAction2));

			_functionAction1 = fAction1;
			_functionAction2 = fAction2;
		}

		public boolean equals(Object anObject) {
			if (anObject instanceof FunctionActionTwo) {
				FunctionActionTwo functionActionTwo = (FunctionActionTwo)anObject;
				return functionActionTwo.getFunctionAction1().equals(getFunctionAction1())
					&& functionActionTwo.getFunctionAction2().equals(getFunctionAction2());

			} else {
				return false;
			}
		}

		public int hashCode() {
			return 3 * getFunctionAction2().hashCode() + getFunctionAction1().hashCode();
		}

		public final IFunction getFunctionAction1() {
			return _functionAction1;
		}

		public final IFunction getFunctionAction2() {
			return _functionAction2;
		}

		public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			try {
				getFunctionAction1().act(executeFunctionArgs);

			} catch (EmptyIteratorSimulationRuntimeException exception) {
			}

			try {
				getFunctionAction2().act(executeFunctionArgs);

			} catch (EmptyIteratorSimulationRuntimeException exception) {
			}
		}

		public String getName() {
			return"["+_functionAction1.getName() +", "+_functionAction2.getName() +"]";
}
}

public FunctionActionTwoFactory() {
	super();
}

public static final IFunctionActionTwo createFunctionAction(IFunction arg1, IFunction arg2) {
	if (arg1 instanceof FunctionActionNoop) {
		if (arg2 instanceof FunctionActionNoop) {
			return AllNoopFunctionActionTwo.getInstance();

		} else {
			return new FirstNoopFunctionActionTwo(arg2);
		}

	} else {
		if (arg2 instanceof FunctionActionNoop) {
			return new SecondNoopFunctionActionTwo(arg1);

		} else {
			return new FunctionActionTwo(arg1, arg2);
		}
	}
}
}