package de.uniwue.ki.mas.basics.action.debugging;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;

/**
 * Creation date: (30.07.2001 14:42:36)
 * @author Christoph Oechslein
 */

public class FunctionActionDebuggingDeclarationFactory {
	private static boolean _compileDebugFunctions = true;
	private static boolean _runDebugFunctions = true;

	public static final ISpecificFunctionDeclaration ASSERT_SPECIFIC_FUNCTION_DECL =
		new AssertSpecificFunctionDeclaration();
	public static final ISpecificFunctionDeclaration FAIL_SPECIFIC_FUNCTION_DECL =
		new FailSpecificFunctionDeclaration();

	private static IFunctionDeclarationList _cachedFunctionDeclarationList = new FunctionDeclarationList();

	static {
		// Assert(Boolean, String)
		_cachedFunctionDeclarationList.add(ASSERT_SPECIFIC_FUNCTION_DECL);

		// Fail(String)
		_cachedFunctionDeclarationList.add(FAIL_SPECIFIC_FUNCTION_DECL);

		// print(object)
		_cachedFunctionDeclarationList.add(new PrintSpecificFunctionDelaration());

		// inspect(object)
		_cachedFunctionDeclarationList.add(new InspectSpecificFunctionDeclaration());
	}

	public static void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
		functionDeclarationList.addAll(_cachedFunctionDeclarationList);
	}

	public static void setCompileDebugFunctions(boolean compileDebugFunctions) {
		_compileDebugFunctions = compileDebugFunctions;
	}

	public static boolean isCompileDebugFunctions() {
		return _compileDebugFunctions;
	}

	public static void setRunDebugFunctions(boolean runDebugFunctions) {
		_runDebugFunctions = runDebugFunctions;
	}

	public static boolean isRunDebugFunctions() {
		return _runDebugFunctions;
	}
}