package de.uniwue.ki.mas.basics.action.rest;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * ForElements: T1 ForElements (fun<(T2) T1>, iterator<T2>)
 * T2 is unspecific
 * Creation date: (04.09.2001 16:53:35)
 * @author Christoph Oechslein
 */

final class ForElementsInIteratorGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("iterator");

	public ForElementsInIteratorGenericFunctionDeclaration() {
		super("ForElements", GenericType.getInstance(), CollectionsHelper.createList(typeDependentT), false);

		setDocumentation("Uses function (Arg1) on every element of iterator (Arg2)");
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final ISpecificFunctionType specificFunctionType,
		final SpecificIteratorType specificSetType) {

		// ForElements: T1 ForElements (fun<(T2) T1>, iterator<T2>)
		// T2 is unspecific

		return new ForElementsInIteratorSpecificFunctionDeclaration(
			this,
			specificFunctionType,
			specificSetType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// ForElements: T1 ForElements (fun<(T2) T1>, iterator<T2>)
		// T2 is unspecific

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(
			GenericFunctionType.getInstance().createSpecificFunctionType(
				outputType,
				CollectionsHelper.createList(typeT)),
			GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		// ForElements: T1 ForElements (fun<(T2) T1>, iterator<T2>)
		// T2 is unspecific

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeB instanceof SpecificIteratorType)
			|| !(specificTypeA instanceof SpecificFunctionType))
			return null;

		SpecificIteratorType setTypeT = (SpecificIteratorType) specificTypeB;
		SpecificFunctionType functionTypeT = (SpecificFunctionType) specificTypeA;

		if (functionTypeT.getSpecificInputTypesLength() != 1)
			return null;

		if (!functionTypeT.getSpecificInputTypeAt(0).isMoreSpecific(setTypeT.getIteratorSpecificType()))
			return null;

		return createFunctionDeclaration(functionTypeT, setTypeT);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		// ForElements: T1 ForElements (fun<(T2) T1>, iterator<T2>)
		// T2 is unspecific

		return "(Fun<(T) " + outputType.getName() + "> , Iterator<T>)";
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return true;
	}

}