package de.uniwue.ki.mas.basics.action.rest;
import java.util.logging.Level;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 19.11.2002, 12:29:32
 * @author oechslein
 */
public final class ForElementsInIteratorSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final ISpecificFunctionType _specificFunctionType;
	private final SpecificIteratorType _specificSetType;

	public ForElementsInIteratorSpecificFunctionDeclaration(
		ForElementsInIteratorGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificFunctionType specificFunctionType,
		SpecificIteratorType specificSetType) {

		super(genericFunctionDeclaration, 2);

		_specificFunctionType = specificFunctionType;
		_specificSetType = specificSetType;
	}
	public ISpecificType getSpecificOutputType() {
		return _specificFunctionType.getSpecificOutputType();
	}
	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return _specificFunctionType;

		} else if (index == 1) {
			return _specificSetType;

		} else {
			Assert.fail();
			return null;
		}

	}
	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());

		return new FunctionForElements(functions.getFunction(0), functions.getFunction(1));
	}

	private IFunction createConstantFunction(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionCallInternDeclaration listFunCall,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		try {
			IExecuteIterator iterator =
				ExecuteIteratorFactory.createExecuteIterator(
					createFunctionArgs.createCreateAndExecuteFunctionArgs(),
					listFunCall.createFunction(createFunctionArgs));

			if (CollectionsHelper.isEmpty(iterator)) {
				if (Assert.ASSERTION_ON && FunctionDeclarationCompiler.LOGGER.isLoggable(Level.INFO)) {
					FunctionDeclarationCompiler.LOGGER.info(
						"Removed redundant action since list is empty in ForElements: "
							+ composedFunctionCallInternDeclaration.getName());
				}

				return getSpecificInputTypeAt(1).createDefaultFunctionCallInternDeclaration().createFunction(
					createFunctionArgs);

			} else {
				return new FunctionForElements(
					composedFunctionCallInternDeclaration.getChild(0).createFunction(createFunctionArgs),
					_specificSetType.createArgument(createFunctionArgs, iterator));

			}

		} catch (SimulationRuntimeException e) {
			throw new ModelCompilingException(e);
		}

	}
	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		Assert.assertEquals(2, composedFunctionCallInternDeclaration.getChildCount());

		IFunctionCallInternDeclaration listFunCall = composedFunctionCallInternDeclaration.getChild(1);

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& listFunCall.isConstant(createFunctionArgs)) {
			return createConstantFunction(
				createFunctionArgs,
				listFunCall,
				composedFunctionCallInternDeclaration);
		} else {
			return super.createFunction(createFunctionArgs, composedFunctionCallInternDeclaration);
		}
	}
}