package de.uniwue.ki.mas.basics.action.rest;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * ForTimes: T ForTimes (fun<(Number) T>, Number, Number)
 * Creation date: (04.09.2001 16:53:35)
 * @author Christoph Oechslein
 */

final class ForTimesGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {
	public ForTimesGenericFunctionDeclaration() {
		super("ForTimes", GenericType.getInstance(), false);

		setDocumentation("Repeats function (Arg1) ((Arg3) - (Arg2))-times often, Arg2 and Arg3 are rounded.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType ouputType) {

		// ForTimes: T ForTimes (fun<(Number) T>, Number, Number)

		return new ForTimesSpecificFunctionDeclaration(this, ouputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		// ForTimes: T ForTimes (fun<(Number) T>, Number, Number)

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);
		ISpecificType specificTypeC = getNextSpecificType(inputTypes);

		if ((specificTypeA == null)
			|| (specificTypeB == null)
			|| (specificTypeC == null)
			|| inputTypes.hasNext())
			return null;

		if (!DoubleType.getInstance().isMoreSpecific(specificTypeB)
			|| !DoubleType.getInstance().isMoreSpecific(specificTypeC)
			|| !(specificTypeA instanceof SpecificFunctionType))
			return null;

		SpecificFunctionType functionTypeT = (SpecificFunctionType) specificTypeA;
		if (functionTypeT.getSpecificInputTypesLength() != 1)
			return null;

		if (!DoubleType.getInstance().isMoreSpecific(functionTypeT.getSpecificInputTypeAt(0)))
			return null;

		return createFunctionDeclaration(functionTypeT.getSpecificOutputType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		// ForTimes: T ForTimes (fun<(Number) T>, Number, Number)
		return "(Fun<(Number) " + outputType.getName() + "> , Number, Number)";
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return true;
	}

}