package de.uniwue.ki.mas.basics.action.rest;
import java.util.Collections;

import de.uniwue.ki.mas.basics.action.AbstractFunctionActionDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;

/**
 * Creation date: (30.07.2001 14:42:36)
 * @author Christoph Oechslein
 */

public class FunctionActionRestDeclarationFactory {
	public static final ISpecificFunctionDeclaration BLOCK_SPECIFIC_FUNCTION_DECL =
		new BlockSpecificFunctionDeclaration();
	public static final IGenericFunctionDeclaration FOR_ELEMENTS_GENERIC_FUNCTION_DECL =
		new ForElementsInIteratorGenericFunctionDeclaration();
	public static final IGenericFunctionDeclaration FOR_TIMES_GENERIC_FUNCTION_DECL =
		new ForTimesGenericFunctionDeclaration();
	public static final IGenericFunctionDeclaration WHILE_GENERIC_FUNCTION_DECL =
		new WhileGenericFunctionDeclaration();
	public static final IGenericFunctionDeclaration PROGN_GENERIC_FUNCTION_DECLARATION =
		new PrognGenericFunctionDeclaration();
	public static final IGenericFunctionDeclaration PROG1_GENERIC_FUNCTION_DECLARATION =
		new Prog1GenericFunctionDeclaration();

	public static final ISpecificFunctionDeclaration NOOP_SPECIFIC_FUNCTION_DECL =
		new AbstractFunctionActionDeclaration("Noop", Collections.EMPTY_LIST) {
			{
				setDocumentation("Does nothing");
			}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList function) {
			return FunctionActionNoop.getInstance();
		}
	};

	private static IFunctionDeclarationList _cachedFunctionDeclarationList = new FunctionDeclarationList();

	static {
		_cachedFunctionDeclarationList.add(NOOP_SPECIFIC_FUNCTION_DECL);
		NOOP_SPECIFIC_FUNCTION_DECL.setDocumentation("Does nothing");

		// Block(action1, action2, ...)
		_cachedFunctionDeclarationList.add(BLOCK_SPECIFIC_FUNCTION_DECL);

		// T ForElements(fun<(T) T>, set<T>)
		_cachedFunctionDeclarationList.add(FOR_ELEMENTS_GENERIC_FUNCTION_DECL);

		// T ForTimes(fun<(Number) T>, Number, Number)
		_cachedFunctionDeclarationList.add(FOR_TIMES_GENERIC_FUNCTION_DECL);

		// T While(fun<() Boolean>,fun<() T>)
		_cachedFunctionDeclarationList.add(WHILE_GENERIC_FUNCTION_DECL);

		// InheritFeatureVars(Feature, BodyImage)
		_cachedFunctionDeclarationList.add(new InheritFeatureVariablesSpecificFunctionDeclaration());

		// <T> Progn(void, <T>)
		_cachedFunctionDeclarationList.add(PROGN_GENERIC_FUNCTION_DECLARATION);

		// <T> Prog1(<T>, void)
		_cachedFunctionDeclarationList.add(PROG1_GENERIC_FUNCTION_DECLARATION);
	}

	public static void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
		functionDeclarationList.addAll(_cachedFunctionDeclarationList);
	}

}