package de.uniwue.ki.mas.basics.action.rest;
import java.util.ConcurrentModificationException;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionForElements extends AbstractFunctionDynamic {
	private final IFunction _functionFunction;
	private final IFunction _functionSet;
	private final SingletonObjectFunctionList _singletonFunctionList = new SingletonObjectFunctionList();

	public FunctionForElements(IFunction functionFunction, IFunction functionSet) {
		super();

		_functionFunction = functionFunction;
		_functionSet = functionSet;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forElementFunction = _functionFunction.execute(executeFunctionArgs);
		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			Object result = null;
			while (setIterator.hasNext()) {
				_singletonFunctionList.setFunction(setIterator.nextWithException());

				result =
					AbstractSpecificFunctionType.getParametricFunction(forElementFunction).execute(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}
			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object forElementFunction = _functionFunction.execute(executeFunctionArgs);
		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			double result = 0;
			while (setIterator.hasNext()) {
				_singletonFunctionList.setFunction(setIterator.nextWithException());

				result =
					AbstractSpecificFunctionType.getParametricFunction(forElementFunction).executeForDouble(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}
			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else
				throw new RuntimeException(e);
		}
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forElementFunction = _functionFunction.execute(executeFunctionArgs);
		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			int result = 0;
			while (setIterator.hasNext()) {
				_singletonFunctionList.setFunction(setIterator.nextWithException());

				result =
					AbstractSpecificFunctionType.getParametricFunction(forElementFunction).executeForInt(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}
			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else
				throw new RuntimeException(e);
		}
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object forElementFunction = _functionFunction.execute(executeFunctionArgs);
		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			boolean result = false;
			while (setIterator.hasNext()) {
				_singletonFunctionList.setFunction(setIterator.nextWithException());

				result =
					AbstractSpecificFunctionType.getParametricFunction(forElementFunction).executeForBoolean(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}
			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else
				throw new RuntimeException(e);
		}
	}
	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forElementFunction = _functionFunction.execute(executeFunctionArgs);
		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			while (setIterator.hasNext()) {
				_singletonFunctionList.setFunction(setIterator.nextWithException());

				AbstractSpecificFunctionType.getParametricFunction(forElementFunction).act(
					createAndExecuteFunctionArgs,
					_singletonFunctionList);
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;

			else if (e instanceof ConcurrentModificationException) {
				e.printStackTrace();
				throw new SimulationRuntimeException("Changed list during for elements is not allowed!");

			} else
				throw new RuntimeException(e);
		}
	}
	public String getName() {
		return "ForElements(" + _functionFunction.getName() + ", " + _functionSet.getName() + ")";
	}
}