package de.uniwue.ki.mas.basics.action.rest;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionForTimes extends AbstractFunctionAction {
	private final IFunction _functionFunction;
	private final IFunction _functionStartNumber;
	private final IFunction _functionEndNumber;
	private final SingletonObjectFunctionList _singletonFunctionList = new SingletonObjectFunctionList();

	public FunctionForTimes(
		IFunction functionFunction,
		IFunction functionStartNumber,
		IFunction functionEndNumber) {
		super();

		_functionFunction = functionFunction;
		_functionStartNumber = functionStartNumber;
		_functionEndNumber = functionEndNumber;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forFunction = _functionFunction.execute(executeFunctionArgs);

		int startValue = (int) Math.round(_functionStartNumber.executeForDouble(executeFunctionArgs));
		int endValue = (int) Math.round(_functionEndNumber.executeForDouble(executeFunctionArgs));

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			for (int i = startValue; i < endValue; i++) {
				_singletonFunctionList.setFunction(new Double(i));

				AbstractSpecificFunctionType.getParametricFunction(forFunction).act(
					createAndExecuteFunctionArgs,
					_singletonFunctionList);
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forFunction = _functionFunction.execute(executeFunctionArgs);

		int startValue = (int) Math.round(_functionStartNumber.executeForDouble(executeFunctionArgs));
		int endValue = (int) Math.round(_functionEndNumber.executeForDouble(executeFunctionArgs));

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			Object result = null;
			for (int i = startValue; i < endValue; i++) {
				_singletonFunctionList.setFunction(new Double(i));

				result =
					AbstractSpecificFunctionType.getParametricFunction(forFunction).execute(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}

			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object forFunction = _functionFunction.execute(executeFunctionArgs);

		int startValue = (int) Math.round(_functionStartNumber.executeForDouble(executeFunctionArgs));
		int endValue = (int) Math.round(_functionEndNumber.executeForDouble(executeFunctionArgs));

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			int result = 0;
			for (int i = startValue; i < endValue; i++) {
				_singletonFunctionList.setFunction(new Double(i));

				result =
					AbstractSpecificFunctionType.getParametricFunction(forFunction).executeForInt(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}

			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object forFunction = _functionFunction.execute(executeFunctionArgs);

		int startValue = (int) Math.round(_functionStartNumber.executeForDouble(executeFunctionArgs));
		int endValue = (int) Math.round(_functionEndNumber.executeForDouble(executeFunctionArgs));

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			double result = 0;
			for (int i = startValue; i < endValue; i++) {
				_singletonFunctionList.setFunction(new Double(i));

				result =
					AbstractSpecificFunctionType.getParametricFunction(forFunction).executeForDouble(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}

			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object forFunction = _functionFunction.execute(executeFunctionArgs);

		int startValue = (int) Math.round(_functionStartNumber.executeForDouble(executeFunctionArgs));
		int endValue = (int) Math.round(_functionEndNumber.executeForDouble(executeFunctionArgs));

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {
			boolean result = false;
			for (int i = startValue; i < endValue; i++) {
				_singletonFunctionList.setFunction(new Double(i));

				result =
					AbstractSpecificFunctionType.getParametricFunction(forFunction).executeForBoolean(
						createAndExecuteFunctionArgs,
						_singletonFunctionList);
			}

			return result;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public String getName() {
		return "ForTimes("
			+ _functionFunction.getName()
			+ ", "
			+ _functionStartNumber.getName()
			+ ", "
			+ _functionEndNumber.getName()
			+ ")";
	}
}