package de.uniwue.ki.mas.basics.action.rest;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.body.BodyVariableNotInitialized;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.agent.IAgentExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionInheritFeatureVariables extends AbstractFunctionAction {
	private final IFunction _functionFeature;
	private final IFunction _functionObject;

	public FunctionInheritFeatureVariables(IFunction functionFeature, IFunction functionObject) {
		super();

		_functionFeature = functionFeature;
		_functionObject = functionObject;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IUserFeatureClassDeclaration featureClassDeclaration =
			(IUserFeatureClassDeclaration) _functionFeature.execute(executeFunctionArgs);

		IObject child = (IObject) _functionObject.execute(executeFunctionArgs);

		IObject simObject = ((IAgentExecuteFunctionArgs) executeFunctionArgs).getObject();

		Iterator it =
			featureClassDeclaration
				.getFeatureVariableClassDeclarationListAndFactory()
				.getVariableClassDeclarationList()
				.iterator();
		while (it.hasNext()) {
			setVariableValue(
				simObject,
				child.getBody(),
				(IVariableClassDeclaration) it.next(),
				executeFunctionArgs);
		}
	}

	public Iterator getChildrens() {
		return CollectionsHelper.createIterator(_functionFeature, _functionObject);
	}

	public String getFunctionName() {
		return "InheritFeatureVars";
	}

	public String getName() {
		return getFunctionName() + "(" + _functionFeature.getName() + ", " + _functionObject.getName() + ")";
	}

	private static void setVariableValue(
		IObject simObject,
		IBody child,
		IVariableClassDeclaration variableDeclaration,
		IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		ISpecificType type = variableDeclaration.getSpecificType();

		IBody objectBody = simObject.getBody();

		try {

			int indexChild =
				child.getBodyInstanceDeclaration().getBodyClassDeclaration().getBodyVariableIndexOf(
					variableDeclaration);
			int indexObject =
				objectBody.getBodyInstanceDeclaration().getBodyClassDeclaration().getBodyVariableIndexOf(
					variableDeclaration);

			if (Assert.ASSERTION_ON && !objectBody.isBodyVariableInitialized(indexObject))
				throw new BodyVariableNotInitialized(objectBody, indexObject);

			Object value = objectBody.getBodyVariableAt(indexObject).getValue();

			Object copiedValue = type.copyArgumentObject(value, null);

			if (Assert.ASSERTION_ON && !child.isBodyVariableInitialized(indexChild))
				throw new BodyVariableNotInitialized(child, indexChild);

			child.getBodyVariableAt(indexChild).setValue(copiedValue);

		} catch (ModelCompilingException exception) {
			throw new SimulationRuntimeException(exception);
		}
	}
}