package de.uniwue.ki.mas.basics.action.rest;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionWhile extends AbstractFunctionAction {
	private final IFunction _testFunction;
	private final IFunction _doFunction;

	public FunctionWhile(IFunction testFunction, IFunction doFunction) {
		super();

		_testFunction = testFunction;
		_doFunction = doFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		int counter = 0;
		while (_testFunction.executeForBoolean(executeFunctionArgs)) {
			_doFunction.act(executeFunctionArgs);

			if (Assert.ASSERTION_ON) {
				if (counter > 10000)
					throw new SimulationRuntimeException("Endless loop detected (more than 10000 iterations)");

				counter++;
			}
		}
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		int counter = 0;
		Object result = null;
		while (_testFunction.executeForBoolean(executeFunctionArgs)) {
			result = _doFunction.execute(executeFunctionArgs);

			if (Assert.ASSERTION_ON) {
				if (counter > 10000)
					throw new SimulationRuntimeException("Endless loop detected (more than 10000 iterations)");

				counter++;
			}
		}

		return result;
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
			
		int counter = 0;
		double result = 0;
		while (_testFunction.executeForBoolean(executeFunctionArgs)) {
			result = _doFunction.executeForDouble(executeFunctionArgs);

			if (Assert.ASSERTION_ON) {
				if (counter > 10000)
					throw new SimulationRuntimeException("Endless loop detected (more than 10000 iterations)");

				counter++;
			}
		}

		return result;
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		int counter = 0;
		int result = 0;
		while (_testFunction.executeForBoolean(executeFunctionArgs)) {
			result = _doFunction.executeForInt(executeFunctionArgs);

			if (Assert.ASSERTION_ON) {
				if (counter > 10000)
					throw new SimulationRuntimeException("Endless loop detected (more than 10000 iterations)");

				counter++;
			}
		}

		return result;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		int counter = 0;
		boolean result = false;
		while (_testFunction.executeForBoolean(executeFunctionArgs)) {
			result = _doFunction.executeForBoolean(executeFunctionArgs);

			if (Assert.ASSERTION_ON) {
				if (counter > 10000)
					throw new SimulationRuntimeException("Endless loop detected (more than 10000 iterations)");

				counter++;
			}
		}

		return result;
	}

	public String getName() {
		return "While(" + _testFunction.getName() + ", " + _doFunction.getName() + ")";
	}
}