package de.uniwue.ki.mas.basics.action.rest;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;

/**
 * @author herrler
 */
final class Prog1GenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {

	public Prog1GenericFunctionDeclaration() {
		super("Prog1", GenericType.getInstance(), true);

		setDocumentation("Executes a block of two instructions and returns the value of the first instruction");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return super.canCreateFunctionDeclarationWith(outputType)
			&& (TypeManager.getInstance().isInAllTypes(outputType)
				|| (GenericIteratorType.getInstance().canCreate(outputType)));

	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		outputType = convertIteratorType(outputType);

		return new Prog1SpecificFunctionDeclaration(this, outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		/* <T> prog1(<T>, void) */

		ISpecificType specificFirstType = getNextSpecificType(inputArgs);
		if (!((TypeManager.getInstance().isInAllTypes(specificFirstType))
			|| (specificFirstType instanceof ISpecificComposedType)
			|| (GenericIteratorType.getInstance().canCreate(specificFirstType))))
			return null;

		ISpecificType specificVoidType = getNextSpecificType(inputArgs);
		if (specificVoidType != VoidType.getInstance())
			return null;

		return createFunctionDeclaration(specificFirstType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		outputType = convertIteratorType(outputType);

		return "(Prog1<" + outputType.getName() + ">)";
	}

	/** 
	 * was macht das denn ??
	 */
	private ISpecificType convertIteratorType(ISpecificType outputType) {
		if (outputType instanceof SpecificIteratorType) {
			SpecificIteratorType specificIteratorType = (SpecificIteratorType) outputType;
			return GenericListType.getInstance().createSpecificListType(
				specificIteratorType.getIteratorSpecificType());
		} else {
			return outputType;
		}
	}
}