package de.uniwue.ki.mas.basics.action.rest;
import java.util.logging.Level;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (01.10.2001 16:25:22)
 * @author Christoph Oechslein
 */

final class WhileSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {
	private final ISpecificType _outputType;

	public WhileSpecificFunctionDeclaration(
		WhileGenericFunctionDeclaration whileGenericFunctionDeclaration,
		ISpecificType outputType) {

		super(whileGenericFunctionDeclaration, 2);

		_outputType = outputType;

		setDocumentation("Executes the instruction (Arg2) while condition (Arg1) is 'true'");
		Assert.assertTrue(isExpertFunctionDeclaration());
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());

		return new FunctionWhile(functions.getFunction(0), functions.getFunction(1));
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return BooleanType.getInstance();

		} else {
			Assert.assertEquals(1, index);
			return _outputType;
		}
	}

	public ISpecificType getSpecificOutputType() {
		return _outputType;
	}

	private IFunction createConstantFunction(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionCallInternDeclaration clauseFunCall,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		try {
			boolean clauseValue =
				clauseFunCall.createFunction(createFunctionArgs).executeForBoolean(
					createFunctionArgs.createCreateAndExecuteFunctionArgs());

			if (clauseValue == true) {
				throw new ModelCompilingException(
					"Clause in While would be always true: " + clauseFunCall.getName());

			} else {

				IFunction defaultValueFunction =
					getSpecificInputTypeAt(1).createDefaultFunctionCallInternDeclaration().createFunction(
						createFunctionArgs);

				if (Assert.ASSERTION_ON && FunctionDeclarationCompiler.LOGGER.isLoggable(Level.INFO)) {
					FunctionDeclarationCompiler.LOGGER.info(
						"Removed redundant clause (= false) in While: "
							+ composedFunctionCallInternDeclaration.getName()
							+ " to "
							+ defaultValueFunction.getName());
				}

				return defaultValueFunction;
			}

		} catch (SimulationRuntimeException e) {
			throw new ModelCompilingException(e);
		}

	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		Assert.assertEquals(2, composedFunctionCallInternDeclaration.getChildCount());

		IFunctionCallInternDeclaration clauseFunCall = composedFunctionCallInternDeclaration.getChild(0);

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& clauseFunCall.isConstant(createFunctionArgs)) {
			return createConstantFunction(
				createFunctionArgs,
				clauseFunCall,
				composedFunctionCallInternDeclaration);
		} else {
			return super.createFunction(createFunctionArgs, composedFunctionCallInternDeclaration);
		}
	}
}