package de.uniwue.ki.mas.basics.action.simulation;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ObjectInstanceDeclarationType;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public final class FunctionActionCreateObject extends AbstractFunctionAction implements IFunction {
	static final String NAME = "CreateObject";
	private final IFunction _functionObjectInstanceClassDeclaration;

	public FunctionActionCreateObject(IFunction functionObjectInstanceClassDeclaration) {
		super();

		_functionObjectInstanceClassDeclaration = functionObjectInstanceClassDeclaration;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs =
			(ISimulationExecuteFunctionArgs) executeFunctionArgs;

		IWorld world = simulationExecuteFunctionArgs.getSimulation().getSituation().getWorld();

		try {
			IObject newObject =
				ObjectInstanceDeclarationType.createObject(
					_functionObjectInstanceClassDeclaration,
					(IObjectExecuteFunctionArgs) executeFunctionArgs);

			world.addObjectInSimulation(newObject, simulationExecuteFunctionArgs);
			
		} catch (ModelCompilingException exception) {
			throw new SimulationRuntimeException(exception);
		}
	}

	public String getName() {
		return NAME + " (" + _functionObjectInstanceClassDeclaration.getName() + ")";
	}
}