package de.uniwue.ki.mas.basics.action.simulation;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.basics.type.ObjectInstanceDeclarationType;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public final class FunctionActionCreateObjectAndRemember
	extends AbstractFunctionAction
	implements IFunction {

	static final String NAME = "CreateObjectAndRemember";
	private final SingletonObjectFunctionList _singletonFunctionList = new SingletonObjectFunctionList();
	private final IFunction _functionObjectInstanceClassDeclaration;
	private final IFunction _functionFunctionObjectDeclaration;

	public FunctionActionCreateObjectAndRemember(
		IFunction functionObjectInstanceClassDeclaration,
		IFunction functionFunctionObjectDeclaration) {
		super();

		_functionObjectInstanceClassDeclaration = functionObjectInstanceClassDeclaration;
		_functionFunctionObjectDeclaration = functionFunctionObjectDeclaration;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs =
			(ISimulationExecuteFunctionArgs) executeFunctionArgs;

		try {
			IObject newObject =
				ObjectInstanceDeclarationType.createObject(
					_functionObjectInstanceClassDeclaration,
					(IObjectExecuteFunctionArgs) executeFunctionArgs);

			if (Assert.ASSERTION_ON)
				Assert.assertNotNull(newObject);

			simulationExecuteFunctionArgs.getSimulation().getSituation().getWorld().addObjectInSimulation(
				newObject,
				simulationExecuteFunctionArgs);

			rememberObject(newObject, simulationExecuteFunctionArgs);

		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeException(e);
		}
	}

	public String getName() {
		return NAME
			+ " ("
			+ _functionObjectInstanceClassDeclaration.getName()
			+ ", "
			+ _functionFunctionObjectDeclaration.getName()
			+ ")";
	}

	private void rememberObject(IObject object, IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		Object rememberFunction = _functionFunctionObjectDeclaration.execute(executeFunctionArgs);

		_singletonFunctionList.setFunction(object);

		try {
			AbstractSpecificFunctionType.getParametricFunction(rememberFunction).act(
				executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
				_singletonFunctionList);

		} catch (ModelCompilingException exception) {
			throw new SimulationRuntimeException(exception);
		}
	}
}