package de.uniwue.ki.mas.basics.action.simulation;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public final class FunctionActionKill extends AbstractFunctionAction implements IFunction {
	private final IFunction _objectFunction;

	public FunctionActionKill(IFunction objectFunction) {
		super();

		_objectFunction = objectFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs =
			(ISimulationExecuteFunctionArgs) executeFunctionArgs;

		IObject objectToKill = (IObject) _objectFunction.execute(simulationExecuteFunctionArgs);

		if (objectToKill == null) {
			throw new SimulationRuntimeException("Object is null in " + getName());
		}

		if (objectToKill == simulationExecuteFunctionArgs.getObject()) {
			throw new SimulationRuntimeException(
				"Not allowed to kill yourself (" + objectToKill.getName() + ") use exit activity instead!");
		}

		if (objectToKill.isDead()) {
			throw new SimulationRuntimeException(
				"Object '"
					+ objectToKill.getName()
					+ "' is already dead, try to call IsDead(SimObject) to test if object already died!");

		} else {
			simulationExecuteFunctionArgs.getSimulation().getSituation().getWorld().removeObjectInSimulation(
				objectToKill,
				simulationExecuteFunctionArgs);
		}
	}

	public String getName() {
		return "Kill(" + _objectFunction.getName() + ")";
	}
}