package de.uniwue.ki.mas.basics.feature;

import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.entity.variableclass.VariableClassDeclarationList;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (22.10.2001 12:58:36)
 * @author Christoph Oechslein
 */

public class BuildinFeatureClassDeclaration extends AbstractFeaturePlusObjectFeatureClassDeclaration {
	private IFunctionDeclarationList _functionDeclarationList;
	private IFunctionDeclarationList _defaultFunctionDeclarationList;
	private VariableClassDeclarationList _variableClassDeclarationList;

	protected BuildinFeatureClassDeclaration(String shortDescription) {
		this(shortDescription, new FunctionDeclarationList());
	}

	public BuildinFeatureClassDeclaration(
		String shortDescription,
		IFunctionDeclarationList defaultFunctionDeclarationList) {
		this(shortDescription, defaultFunctionDeclarationList, null);
	}

	public BuildinFeatureClassDeclaration(
		String shortDescription,
		IFunctionDeclarationList defaultFunctionDeclarationList,
		IFunctionDeclarationList functionDeclarationList) {

		this(shortDescription, defaultFunctionDeclarationList, functionDeclarationList, null);
	}

	public BuildinFeatureClassDeclaration(
		String shortDescription,
		IFunctionDeclarationList defaultFunctionDeclarationList,
		IFunctionDeclarationList functionDeclarationList,
		VariableClassDeclarationList variableClassDeclarationList) {

		super();

		setShortDescription(shortDescription);
		setUsesShortDescription(true);

		_functionDeclarationList = functionDeclarationList;
		_variableClassDeclarationList = variableClassDeclarationList;
		_defaultFunctionDeclarationList = defaultFunctionDeclarationList;
	}

	public ICopyable copy(Map objectMapping) {
		return this;
	}

	protected ICopyable createInstance(Map objectMapping) {
		return this;
	}

	protected final IFunctionDeclarationList getDefaultFunctionDeclarationList() {
		if (_defaultFunctionDeclarationList == null)
			_defaultFunctionDeclarationList = new FunctionDeclarationList();

		return _defaultFunctionDeclarationList;
	}

	public final Iterator getDefaultFunctionDeclarations() {
		return getDefaultFunctionDeclarationList().iterator();
	}

	protected final IFunctionDeclarationList getFunctionDeclarationList() {
		if (_functionDeclarationList == null)
			_functionDeclarationList = new FunctionDeclarationList();

		return _functionDeclarationList;
	}

	public final Iterator getFunctionDeclarations() {
		return getFunctionDeclarationList().iterator();
	}

	public String getName() {
		return getShortDescription();
	}

	protected final VariableClassDeclarationList getVariableClassDeclarationList() {
		if (_variableClassDeclarationList == null)
			_variableClassDeclarationList = new VariableClassDeclarationList();

		return _variableClassDeclarationList;
	}

	public final Iterator getVariableClassDeclarations() {
		return getVariableClassDeclarationList().iterator();
	}

	public void setUsesShortDescription(boolean usesShortDescription) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(usesShortDescription);
		super.setUsesShortDescription(true);
	}
}