package de.uniwue.ki.mas.basics.feature;
import java.util.Iterator;

import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

/**
 * A feature class declaration is a special feature which can be added to object class declarations
 * (for example a spatial map is a feature class declaration for worlds).
 * Creation date: (14.11.2000 10:13:49)
 *
 * @author Christoph Oechslein
 */

public interface IFeatureClassDeclaration extends IDocumentableDeclaration {

	/**
	 * Returns <code>true</code> if this feature can be used as a feature
	 * of the given object class declaration.
	 */
	boolean canAddFeatureClassDeclarationTo(IObjectClassDeclaration objectClassDeclaration);

	/**
	 * Generates an instance for adding to an object instance declaration,
	 * can produce null if no instance specific information is required.
	 */
	IObjectFeatureClassDeclaration createObjectFeatureClassDeclaration(IObjectClassDeclaration objectClassDeclaration);

	/**
	 * Returns <code>true</code> if this feature is a default feature for the given object class declaration
	 * ({@link #canAddFeatureClassDeclarationTo(IObjectClassDeclaration)} must return <code>true</code> if this
	 * method returns <code>true</code>).
	 */
	boolean isDefaultFeatureClassDeclarationFor(IObjectClassDeclaration objectClassDeclaration);

	/**
	 * Returns the standard {@link de.uniwue.ki.mas.model.basics.function.IFunctionDeclaration function declarations}
	 * of this feature declaration, these are avalible even without this feature and in e.g. user functions.
	 */

	Iterator getDefaultFunctionDeclarations();
	
	/**
	 * Returns all Function Declarations declared by this 
	 * Feature class.
	 */
	Iterator getFunctionDeclarations();
	
}