package de.uniwue.ki.mas.basics.feature;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * An feature instance declaration is generated by a {@link IFeatureClassDeclaration feature class declaration}
 * and is used in {@link IObjectInstancedeclaration object instance declarations}.
 * Creation date: (07.12.2000 11:04:43)
 *
 * @author Christoph Oechslein
 */

public interface IObjectFeatureInstanceDeclaration extends IDeclaration {
	/**
	 * Returns a null feature instance declaration (NULL pattern).
	 */
	IObjectFeatureInstanceDeclaration NULL_OBJECT_FEATURE_INSTANCE_DECLARATION =
		new NullObjectFeatureInstanceDeclarationCall();

	/**
	* A null feature instance declaration class (NULL pattern)
	* which returns <code>null</code> for alle methods.
	*/

	class NullObjectFeatureInstanceDeclarationCall
		extends IDeclaration.NullDeclarationClass
		implements IObjectFeatureInstanceDeclaration {

		/**
		 * @param createObjectFunctionArgs IObjectCreateFunctionArgs
		 * @return IObjectFeature
		 * @see de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration#createObjectFeature(IObjectCreateFunctionArgs)
		 */
		public IObjectFeature createObjectFeature(IObjectCreateFunctionArgs createObjectFunctionArgs) {
			return null;
		}

		/**
		 * @return IObjectFeatureClassDeclaration
		 * @see de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration#getObjectFeatureClassDeclaration()
		 */
		public IObjectFeatureClassDeclaration getObjectFeatureClassDeclaration() {
			return null;
		}
	};

	/**
	 * Compiles for an runtime instance of an IFeatureDeclaration,
	 * can produce null if no instance specific runtime information is required
	 *
	 */

	IObjectFeature createObjectFeature(IObjectCreateFunctionArgs createObjectFunctionArgs);

	/**
	 * Returns the class delcration which generated this instanc edeclaration.
	 */
	IObjectFeatureClassDeclaration getObjectFeatureClassDeclaration();
}