package de.uniwue.ki.mas.basics.feature.evolution;

import java.io.Serializable;
import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.util.math.RandomHelper;
/**
 * Creation date: 05.12.2002, 11:54:40
 * @author oechslein
 */
final class BitString implements Serializable {
	private final int[] _geneValueArray;

	public BitString(BitString bitString) {
		this(bitString.getLength());

		setBitString(bitString);
	}

	public BitString(int length) {
		_geneValueArray = new int[length];
	}

	public void setGeneValue(int index, int value) {
		_geneValueArray[index] = value;
	}

	public int getGeneValue(int index) {
		return _geneValueArray[index];
	}

	public void mutateGeneValue(int index, int minValue, int maxValue, double mutProb) {
		if (RandomHelper.getRandomBoolean(mutProb))
			setGeneValue(index, RandomHelper.getRandomInt(minValue, maxValue + 1));
	}

	public int getLength() {
		return _geneValueArray.length;
	}

	public void setBitString(BitString bitString) {
		Assert.assertEquals(getLength(), bitString.getLength());

		System.arraycopy(bitString._geneValueArray, 0, _geneValueArray, 0, getLength());
	}

	public String toString() {
		StringBuffer buff = new StringBuffer();

		buff.append(super.toString());
		buff.append(" [");
		appendName(buff, 0, 5);
		buff.append("]");

		return buff.toString();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			for (int i = 0; i < getLength(); i++) {
				buff.append(getGeneValue(i));

				if (i < getLength() - 1)
					buff.append(", ");
			}
		}
	}

	public static BitString crossover(BitString bitStringA, BitString bitStringB) {
		int length = bitStringA.getLength();

		Assert.assertEquals(length, bitStringB.getLength());

		int crossoverPoint = RandomHelper.getRandomInt(0, length + 1);

		if (crossoverPoint == 0) {
			return new BitString(bitStringA);

		} else if (crossoverPoint == length) {
			return new BitString(bitStringB);

		} else if (bitStringA.equals(bitStringB)) {
			return new BitString(bitStringA);

		} else {
			if (RandomHelper.getRandomBoolean()) {
				BitString dummy = bitStringA;
				bitStringA = bitStringB;
				bitStringB = dummy;
			}

			BitString child = new BitString(length);

			for (int i = 0; i < crossoverPoint; i++) {
				child.setGeneValue(i, bitStringA.getGeneValue(i));
			}

			for (int i = crossoverPoint; i < length; i++) {
				child.setGeneValue(i, bitStringB.getGeneValue(i));
			}

			return child;
		}
	}

	
	private static boolean inArray (int index, int[] array, int contentNumbers){
		boolean test = false;
		for (int i = 0; i<array.length; i++ ){
			if (array[i] == index)
				test = true;			
					}
		return test;
	}


	private static int[] generateAndFillCrossoverPointArray(int lengthBitString, int numberPoints){
		int[] crossoverPoints = new int[numberPoints];
		for (int i=0; i<numberPoints; i++){
			crossoverPoints[i] = lengthBitString + 1;
		}
		int currentNumber=0;
		while(currentNumber<numberPoints) {
			boolean successfull=false;
			while (!successfull) {
				int i = RandomHelper.getRandomInt(0, lengthBitString);
				successfull = !inArray(i,crossoverPoints, currentNumber);    
				if (successfull) {
					crossoverPoints[currentNumber] = i;
		            currentNumber++;  
				}
			}
		}
		Arrays.sort(crossoverPoints);
		return crossoverPoints;		
	}


	public static BitString multipleCrossover(
		BitString bitStringA,
		BitString bitStringB,
		int numberCrossoverPoints) {
		int length = bitStringA.getLength();

		Assert.assertEquals(length, bitStringB.getLength());
		Assert.assertTrue(
			"The number of crossover points is larger than the length of the gene!",
			length > numberCrossoverPoints);

		int[] crossoverPoints = generateAndFillCrossoverPointArray(length, numberCrossoverPoints);
		
		int currentPointIndex = 0;
		int currentPoint = crossoverPoints[currentPointIndex];
		boolean nowTakeBitstring1 = true;

		if ((numberCrossoverPoints == 1) && (currentPoint == 0)) {
			return new BitString(bitStringA);

		} else if ((numberCrossoverPoints == 1) && (currentPoint == length)) {
			return new BitString(bitStringB);

		} else if (bitStringA.equals(bitStringB)) {
			return new BitString(bitStringA);

		} else {

			BitString child = new BitString(length);
			
			for (int i = 0; i < length; i++) {
				if (currentPoint == i){
					currentPointIndex++;
					if (currentPointIndex < crossoverPoints.length){
					    currentPoint = crossoverPoints[currentPointIndex];
					}					
					nowTakeBitstring1 = !nowTakeBitstring1;				
				}
	
				child.setGeneValue(i, nowTakeBitstring1? bitStringA.getGeneValue(i) : bitStringB.getGeneValue(i));
			}

			return child;
		}
	}


	public boolean equals(BitString bitString) {
		if (getLength() != bitString.getLength()) {
			return false;

		} else {
			for (int i = 0; i < getLength(); i++) {
				if (getGeneValue(i) != bitString.getGeneValue(i))
					return false;
			}
			return true;
		}
	}

	public boolean equals(Object anObject) {
		if (anObject instanceof BitString)
			return equals((BitString) anObject);
		else
			return false;
	}

	public int hashCode() {
		int hashCode = getLength();
		for (int i = 0; i < getLength(); i++) {
			hashCode = 3 * hashCode + getGeneValue(i);
		}
		return hashCode;
	}
}
