package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.math.RandomHelper;
/**
 * Insert the type's description here.
 * Creation date: (22.01.2002 10:12:50)
 * @author Christoph Oechslein
 */
final class BitStringCrossoverHelper {

	public static void crossover(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		IObject parent1,
		IObject parent2,
		IObject child,
		double mutateProb)
		throws SimulationRuntimeException {

		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature1 =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, parent1);
		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature2 =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, parent2);
		BitStringEvolutionObjectFeature childBitStringEvolutionObjectFeature =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, child);

		if (parentBitStringEvolutionObjectFeature1.getBitString().getLength()
			!= childBitStringEvolutionObjectFeature.getBitString().getLength()) {
			throw new SimulationRuntimeException(
				"Child '"
					+ child.getName()
					+ "' can not get recombined genes from '"
					+ parent1
					+ "' (parent1) since amount of genes differs.");
		}

		if (parentBitStringEvolutionObjectFeature2.getBitString().getLength()
			!= childBitStringEvolutionObjectFeature.getBitString().getLength()) {
			throw new SimulationRuntimeException(
				"Child '"
					+ child.getName()
					+ "' can not get recombined genes from '"
					+ parent2
					+ "' (parent2) since amount of genes differs.");
		}

		if ((parentBitStringEvolutionObjectFeature1 != parentBitStringEvolutionObjectFeature2)
			&& RandomHelper.getRandomBoolean()) {
			BitStringEvolutionObjectFeature dummy = parentBitStringEvolutionObjectFeature1;
			parentBitStringEvolutionObjectFeature1 = parentBitStringEvolutionObjectFeature2;
			parentBitStringEvolutionObjectFeature2 = dummy;
		}

		if (childBitStringEvolutionObjectFeature instanceof DiploidBitStringEvolutionObjectFeature) {
			((DiploidBitStringEvolutionObjectFeature)childBitStringEvolutionObjectFeature).setBitStrings(
				createRecombinatedAndMutatedBitString(parentBitStringEvolutionObjectFeature1, mutateProb),
				createRecombinatedAndMutatedBitString(parentBitStringEvolutionObjectFeature2, mutateProb));

		} else {
			childBitStringEvolutionObjectFeature.setBitString(
				createRecombinatedAndMutatedBitString(parentBitStringEvolutionObjectFeature1, mutateProb));
		}

		childBitStringEvolutionObjectFeature.writeBitStringToVariables(executeFunctionArgs, child);
	}

	private static BitString createRecombinatedAndMutatedBitString(
		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature,
		double mutateProb) {

		BitStringEvolutionObjectFeatureHelper parentBitStringEvolutionObjectFeatureHelper =
			parentBitStringEvolutionObjectFeature.getBitStringEvolutionObjectFeatureHelper();

		if (parentBitStringEvolutionObjectFeature instanceof DiploidBitStringEvolutionObjectFeature) {
			BitString newBitString =
				BitString.crossover(
					parentBitStringEvolutionObjectFeature.getBitString(),
					((DiploidBitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature)
						.getBitString2());

			parentBitStringEvolutionObjectFeatureHelper.mutateAllGenes(newBitString, mutateProb);

			return newBitString;

		} else {
			return parentBitStringEvolutionObjectFeatureHelper.createMutatedBitString(
				parentBitStringEvolutionObjectFeature.getBitString(),
				mutateProb);
		}
	}

	private static BitStringEvolutionObjectFeature getBitStringEvolutionObjectFeature(
		IExecuteFunctionArgs executeFunctionArgs,
		IObject parent)
		throws SimulationRuntimeException {

		IBitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature =
			CrossoverFunction.getBitStringEvolutionObjectFeature(executeFunctionArgs, parent);

		if (parentBitStringEvolutionObjectFeature instanceof ChangeableBitStringEvolutionObjectFeature) {
			return ((ChangeableBitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature)
				.getCurrentBitStringEvolutionObjectFeature();

		} else {
			return (BitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature;
		}
	}
}