package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.feature.AbstractObjectFeature;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Insert the type's description here.
 * Creation date: (19.11.2001 19:05:08)
 * @author Christoph Oechslein
 */
class BitStringEvolutionObjectFeature
	extends AbstractObjectFeature
	implements IBitStringEvolutionObjectFeature {

	private final BitString _bitString;
	private final BitStringEvolutionObjectFeatureHelper _bitStringEvolutionObjectFeatureHelper;

	public BitStringEvolutionObjectFeature(BitStringEvolutionObjectFeatureHelper bitStringEvolutionObjectFeatureHelper) {
		this(
			bitStringEvolutionObjectFeatureHelper,
			new BitString(bitStringEvolutionObjectFeatureHelper.getLength()));
	}

	protected BitStringEvolutionObjectFeature(
		BitStringEvolutionObjectFeatureHelper evolutionObjectFeatureHelper,
		BitString bitString) {
		super();

		_bitStringEvolutionObjectFeatureHelper = evolutionObjectFeatureHelper;
		_bitString = bitString;
	}

	public final BitString getBitString() {
		return _bitString;
	}

	public final BitStringEvolutionObjectFeatureHelper getBitStringEvolutionObjectFeatureHelper() {
		return _bitStringEvolutionObjectFeatureHelper;
	}

	public void mutateAll(IObjectExecuteFunctionArgs executeFunctionArgs, IObject simObject, double prob)
		throws SimulationRuntimeException {
		mutateAllGenes(prob);
		writeBitStringToVariables(executeFunctionArgs, simObject);
	}

	public void mutateAllGenes(double mutateProb) {
		getBitStringEvolutionObjectFeatureHelper().mutateAllGenes(getBitString(), mutateProb);
	}

	public void reset(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		writeVariablesToBitString(((IObjectExecuteFunctionArgs) executeFunctionArgs).getObject());
	}

	public void setBitString(BitString newBitString) {
		getBitString().setBitString(newBitString);
	}

	public void writeBitStringToVariables(
		IObjectExecuteFunctionArgs objectExecuteFunctionArgs,
		IObject object)
		throws SimulationRuntimeException {
		_bitStringEvolutionObjectFeatureHelper.writeBitStringToVariables(
			objectExecuteFunctionArgs,
			object,
			getBitString());
	}

	public void writeVariablesToBitString(IObject object) throws SimulationRuntimeException {
		BitString bitString = getBitString();

		BitStringEvolutionObjectFeatureHelper helper = getBitStringEvolutionObjectFeatureHelper();

		for (int i = 0; i < bitString.getLength(); i++) {
			int newValue =
				Math.max(
					helper.getMinValueForIndex(i),
					Math.min(helper.getMaxValueForIndex(i), helper.getVariableValue(i, object)));

			bitString.setGeneValue(i, newValue);
		}
	}

	public void remove() {
		// do not remove helper or bitstring it is shared!!

		super.remove();
	}

	public String toString() {
		return super.getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		getBitString().appendName(buff, currDepth, maxDepth);
	}

	public final String getName() {
		return getNameIntern();
	}
}