package de.uniwue.ki.mas.basics.feature.evolution;
import java.io.Serializable;

import junit.framework.Assert;

import de.uniwue.ki.mas.entity.body.BodyVariableNotInitialized;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.AbstractNamed;
/**
 * Insert the type's description here.
 * Creation date: (19.11.2001 19:05:08)
 * @author Christoph Oechslein
 */
final class BitStringEvolutionObjectFeatureHelper implements Serializable {
	public static class EvoEntry extends AbstractNamed implements Serializable {
		final int _variableIndex;
		final int _minValue;
		final int _maxValue;

		public EvoEntry(int variableIndex, int minValue, int maxValue) {
			super();

			_variableIndex = variableIndex;
			_minValue = minValue;
			_maxValue = maxValue;
		}

		public String getName() {
			return _variableIndex + ", " + _minValue + ", " + _maxValue;
		}

	}

	private final EvoEntry[] _evoEntryArray;

	public BitStringEvolutionObjectFeatureHelper(EvoEntry[] evoEntryArray) {
		super();

		_evoEntryArray = evoEntryArray;
	}

	public int getLength() {
		return _evoEntryArray.length;
	}

	public int getVariableValue(int index, IObject simObject) throws SimulationRuntimeException {
		IBody fromBody = simObject.getBody();

		int variabeIndex = getVariableForIndex(index);

		if (Assert.ASSERTION_ON && !fromBody.isBodyVariableInitialized(variabeIndex))
			throw new BodyVariableNotInitialized(fromBody, variabeIndex);

		return Math.round(fromBody.getBodyVariableAt(variabeIndex).getIntValue());
	}

	public void setVariableValue(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		int index,
		IObject simObject,
		int newValue)
		throws SimulationRuntimeException {

		IBody fromBody = simObject.getBody();

		int variabeIndex = getVariableForIndex(index);

		if (Assert.ASSERTION_ON && !fromBody.isBodyVariableInitialized(variabeIndex))
			throw new BodyVariableNotInitialized(fromBody, variabeIndex);

		IBodyVariable bodyVariable = fromBody.getBodyVariableAt(variabeIndex);

		int oldValue = bodyVariable.getIntValue();

		bodyVariable.setIntValue(newValue);

		if (executeFunctionArgs.hasObjectsVariableListener()) {
			executeFunctionArgs.fireVariableValueChanged(
				simObject,
				executeFunctionArgs.getObject(),
				bodyVariable,
				new Integer(oldValue),
				new Integer(newValue));
		}
	}

	public void writeBitStringToVariables(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		IObject object,
		BitString bitString)
		throws SimulationRuntimeException {

		for (int i = 0; i < bitString.getLength(); i++) {
			Assert.assertFalse("Gene Values should never be lower than zero", bitString.getGeneValue(i) < 0);
			setVariableValue(executeFunctionArgs, i, object, bitString.getGeneValue(i));
		}
	}

	private int getVariableForIndex(int index) {
		return _evoEntryArray[index]._variableIndex;
	}

	public int getMinValueForIndex(int index) {
		return _evoEntryArray[index]._minValue;
	}

	public int getMaxValueForIndex(int index) {
		return _evoEntryArray[index]._maxValue;
	}

	public void mutateAllGenes(BitString bitString, double mutateProb) {
		int length = bitString.getLength();

		Assert.assertEquals(length, getLength());

		for (int i = 0; i < length; i++) {
			bitString.mutateGeneValue(i, getMinValueForIndex(i), getMaxValueForIndex(i), mutateProb);
		}
	}

	public BitString createMutatedBitString(BitString bitString, double mutateProb) {
		BitString newBitString = new BitString(bitString);

		mutateAllGenes(newBitString, mutateProb);

		return newBitString;
	}

	public BitString cloneBitString(BitString bitString) {

		return new BitString(bitString);

	}

}