package de.uniwue.ki.mas.basics.feature.evolution;
import java.io.Serializable;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: 13.11.2003, 01:46:35
 * @author Franziska Klgl
 */
class BitStringMeltingHelper implements Serializable {
	public static void melt(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		IObject parent1,
		IObject parent2,
		IObject child)
		throws SimulationRuntimeException {

		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature1 =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, parent1);
		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature2 =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, parent2);
		BitStringEvolutionObjectFeature childBitStringEvolutionObjectFeature =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, child);

		BitString parentBitString1 = parentBitStringEvolutionObjectFeature1.getBitString();
		BitString parentBitString2 = parentBitStringEvolutionObjectFeature2.getBitString();

		if ((parentBitString1.getLength() != parentBitString2.getLength())
			&& (parentBitString1.getLength()
				!= childBitStringEvolutionObjectFeature.getBitString().getLength())) {
			throw new SimulationRuntimeException(
				"Child '"
					+ child.getName()
					+ "' and parents '"
					+ parent1
					+ "' and '"
					+ parent2
					+ "' cannot melt since amount of genes differs.");
		}

		if (childBitStringEvolutionObjectFeature instanceof DiploidBitStringEvolutionObjectFeature) {

			DiploidBitStringEvolutionObjectFeature childDiploidBitStringEvolutionObjectFeature =
				(DiploidBitStringEvolutionObjectFeature) childBitStringEvolutionObjectFeature;

			childDiploidBitStringEvolutionObjectFeature.setBitStrings(parentBitString1, parentBitString2);

		} else {
			throw new SimulationRuntimeException(
				"Child '" + child.getName() + "' is not diploid -> it cannot be melted!");
		}

		childBitStringEvolutionObjectFeature.writeBitStringToVariables(executeFunctionArgs, child);
	}


	private static BitStringEvolutionObjectFeature getBitStringEvolutionObjectFeature(
		IExecuteFunctionArgs executeFunctionArgs,
		IObject parent)
		throws SimulationRuntimeException {

		IBitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature =
			CrossoverFunction.getBitStringEvolutionObjectFeature(executeFunctionArgs, parent);

		if (parentBitStringEvolutionObjectFeature instanceof ChangeableBitStringEvolutionObjectFeature) {
			return ((ChangeableBitStringEvolutionObjectFeature) parentBitStringEvolutionObjectFeature)
				.getCurrentBitStringEvolutionObjectFeature();

		} else {
			return (BitStringEvolutionObjectFeature) parentBitStringEvolutionObjectFeature;
		}
	}
}