package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Insert the type's description here.
 * Creation date: (22.01.2002 10:12:50)
 * @author Christoph Oechslein
 */
final class BitStringMultipleCrossoverHelper {

	public static void crossover(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		IObject parent,
		IObject child,
		int numberOfCrossoverPoints)
		throws SimulationRuntimeException {

		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, parent);
		BitStringEvolutionObjectFeature childBitStringEvolutionObjectFeature =
			getBitStringEvolutionObjectFeature(executeFunctionArgs, child);

		if (parentBitStringEvolutionObjectFeature.getBitString().getLength()
			!= childBitStringEvolutionObjectFeature.getBitString().getLength()) {
			throw new SimulationRuntimeException(
				"Child '"
					+ child.getName()
					+ "' can not get recombined genes from '"
					+ parent
					+ "' (parent) since amount of genes differs.");
		}


		if (childBitStringEvolutionObjectFeature instanceof DiploidBitStringEvolutionObjectFeature) {
			throw new SimulationRuntimeException(
				"Sorry, multiple recombination is just implemented for haploid childs");
		} else {
			childBitStringEvolutionObjectFeature.setBitString(
				createRecombinatedBitString(parentBitStringEvolutionObjectFeature, numberOfCrossoverPoints));
		}

		childBitStringEvolutionObjectFeature.writeBitStringToVariables(executeFunctionArgs, child);
	}

	private static BitString createRecombinatedBitString(
		BitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature,
		int numberOfCrossoverPoints) {

		BitStringEvolutionObjectFeatureHelper parentBitStringEvolutionObjectFeatureHelper =
			parentBitStringEvolutionObjectFeature.getBitStringEvolutionObjectFeatureHelper();

		if (parentBitStringEvolutionObjectFeature instanceof DiploidBitStringEvolutionObjectFeature) {
			BitString newBitString =
				BitString.multipleCrossover(
					parentBitStringEvolutionObjectFeature.getBitString(),
					((DiploidBitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature)
						.getBitString2(),
					  numberOfCrossoverPoints);

			return newBitString;

		} else {
			return parentBitStringEvolutionObjectFeatureHelper.cloneBitString(
				parentBitStringEvolutionObjectFeature.getBitString());
		}
	}

	private static BitStringEvolutionObjectFeature getBitStringEvolutionObjectFeature(
		IExecuteFunctionArgs executeFunctionArgs,
		IObject parent)
		throws SimulationRuntimeException {

		IBitStringEvolutionObjectFeature parentBitStringEvolutionObjectFeature =
			MultipleCrossoverFunction.getBitStringEvolutionObjectFeature(executeFunctionArgs, parent);

		if (parentBitStringEvolutionObjectFeature instanceof ChangeableBitStringEvolutionObjectFeature) {
			return ((ChangeableBitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature)
				.getCurrentBitStringEvolutionObjectFeature();

		} else {
			return (BitStringEvolutionObjectFeature)parentBitStringEvolutionObjectFeature;
		}
	}
}