package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.feature.AbstractObjectFeature;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Insert the type's description here.
 * Creation date: (19.02.2002 16:48:36)
 * @author Christoph Oechslein
 */
public class ChangeableBitStringEvolutionObjectFeature
	extends AbstractObjectFeature
	implements IBitStringEvolutionObjectFeature {

	private BitStringEvolutionObjectFeature _currentBitStringEvolutionObjectFeature;
	private final EvolutionObjectFeatureClassDeclaration.DominantType _dominantType;

	private ChangeableBitStringEvolutionObjectFeature(
		BitStringEvolutionObjectFeature currentBitStringEvolutionObjectFeature,
		EvolutionObjectFeatureClassDeclaration.DominantType dominantType) {
		super();

		_currentBitStringEvolutionObjectFeature = currentBitStringEvolutionObjectFeature;
		_dominantType = dominantType;
	}

	public ChangeableBitStringEvolutionObjectFeature(
		boolean diploid,
		EvolutionObjectFeatureClassDeclaration.DominantType dominantType,
		BitStringEvolutionObjectFeatureHelper bitStringEvolutionObjectFeatureHelper) {

		this(
			diploid
				? new DiploidBitStringEvolutionObjectFeature(
					bitStringEvolutionObjectFeatureHelper,
					dominantType)
				: new BitStringEvolutionObjectFeature(bitStringEvolutionObjectFeatureHelper),
			dominantType);
	}

	public BitStringEvolutionObjectFeature getCurrentBitStringEvolutionObjectFeature() {
		return _currentBitStringEvolutionObjectFeature;
	}

	public void mutateAll(IObjectExecuteFunctionArgs executeFunctionArgs, IObject simObject, double prob)
		throws SimulationRuntimeException {
		getCurrentBitStringEvolutionObjectFeature().mutateAll(executeFunctionArgs, simObject, prob);
	}

	public void remove() {
		getCurrentBitStringEvolutionObjectFeature().remove();
	}

	public void reset(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		getCurrentBitStringEvolutionObjectFeature().reset(executeFunctionArgs);
	}

	private void setCurrentBitStringEvolutionObjectFeature(BitStringEvolutionObjectFeature newCurrentBitStringEvolutionObjectFeature) {
		if (_currentBitStringEvolutionObjectFeature != null) {
			_currentBitStringEvolutionObjectFeature.remove();
		}

		_currentBitStringEvolutionObjectFeature = newCurrentBitStringEvolutionObjectFeature;
	}

	public void update(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		getCurrentBitStringEvolutionObjectFeature().update(executeFunctionArgs);
	}

	public void writeBitStringToVariables(IObjectExecuteFunctionArgs executeFunctionArgs, IObject object)
		throws SimulationRuntimeException {
		getCurrentBitStringEvolutionObjectFeature().writeBitStringToVariables(executeFunctionArgs, object);
	}

	public void writeVariablesToBitString(IObject object) throws SimulationRuntimeException {
		getCurrentBitStringEvolutionObjectFeature().writeVariablesToBitString(object);
	}

	public BitStringEvolutionObjectFeatureHelper getBitStringEvolutionObjectFeatureHelper() {
		return getCurrentBitStringEvolutionObjectFeature().getBitStringEvolutionObjectFeatureHelper();
	}

	public void setDiploid(
		IObjectExecuteFunctionArgs executeFunctionArgs,
		IObject simObject,
		boolean toDiploid)
		throws SimulationRuntimeException {
		boolean oldDiploidSetting = isDiploid();

		if (toDiploid == oldDiploidSetting) {
			return;

		} else if (oldDiploidSetting == true) {
			// toDiploid == false

			DiploidBitStringEvolutionObjectFeature oldDiploidBitStringEvolutionObjectFeature =
				(DiploidBitStringEvolutionObjectFeature) getCurrentBitStringEvolutionObjectFeature();

			BitString bitString = oldDiploidBitStringEvolutionObjectFeature.getBitString();

			oldDiploidBitStringEvolutionObjectFeature.setToDominantBitString(bitString);

			setCurrentBitStringEvolutionObjectFeature(
				new BitStringEvolutionObjectFeature(
					oldDiploidBitStringEvolutionObjectFeature.getBitStringEvolutionObjectFeatureHelper(),
					bitString));

			writeBitStringToVariables(executeFunctionArgs, simObject);

		} else {
			// toDiploid == true && oldDiploidSetting == false

			BitStringEvolutionObjectFeature oldBitStringEvolutionObjectFeature =
				getCurrentBitStringEvolutionObjectFeature();

			setCurrentBitStringEvolutionObjectFeature(
				new DiploidBitStringEvolutionObjectFeature(
					oldBitStringEvolutionObjectFeature.getBitStringEvolutionObjectFeatureHelper(),
					oldBitStringEvolutionObjectFeature.getBitString(),
					new BitString(oldBitStringEvolutionObjectFeature.getBitString()),
					_dominantType));

			writeBitStringToVariables(executeFunctionArgs, simObject);
		}
	}

	public boolean isDiploid() {
		return getCurrentBitStringEvolutionObjectFeature() instanceof DiploidBitStringEvolutionObjectFeature;
	}

	public String getName() {
		return getCurrentBitStringEvolutionObjectFeature().getName();
	}
}