package de.uniwue.ki.mas.basics.feature.evolution;
import java.util.WeakHashMap;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.SerializableObject;

/**
 * Insert the class' description here.
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class CrossoverFunction extends AbstractFunctionAction {
	private final IFunction _parent1Function;
	private final IFunction _parent2Function;
	private final IFunction _childFunction;
	private final IFunction _mutateProbFunction;

	public CrossoverFunction(
		IFunction parent1Function,
		IFunction parent2Function,
		IFunction childFunction,
		IFunction mutateProbFunction) {

		super();

		_parent1Function = parent1Function;
		_parent2Function = parent2Function;
		_childFunction = childFunction;
		_mutateProbFunction = mutateProbFunction;
	}

	public CrossoverFunction(
		IFunction parent2Function,
		IFunction childFunction,
		IFunction mutateProbFunction) {

		super();

		_parent1Function = null;
		_parent2Function = parent2Function;
		_childFunction = childFunction;
		_mutateProbFunction = mutateProbFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IObjectExecuteFunctionArgs objectExecuteFunctionArgs =
			(IObjectExecuteFunctionArgs) executeFunctionArgs;

		IObject parent1;

		if (_parent1Function == null) {
			parent1 = objectExecuteFunctionArgs.getObject();
		} else {
			parent1 = (IObject) _parent1Function.execute(objectExecuteFunctionArgs);
			if (parent1 == null)
				throw new SimulationRuntimeException("Parent1 in " + getName() + " is null.");
		}

		IObject parent2 = (IObject) _parent2Function.execute(objectExecuteFunctionArgs);
		if (parent2 == null)
			throw new SimulationRuntimeException("Parent2 in " + getName() + " is null.");

		IObject child = (IObject) _childFunction.execute(objectExecuteFunctionArgs);
		if (child == null)
			throw new SimulationRuntimeException("Child in " + getName() + " is null.");

		double mutateProb = _mutateProbFunction.executeForDouble(objectExecuteFunctionArgs);

		BitStringCrossoverHelper.crossover(objectExecuteFunctionArgs, parent1, parent2, child, mutateProb);
	}

	private static final SerializableObject MAP_MARKER = new SerializableObject();

	private static WeakHashMap getBitStringEvolutionObjectFeatureMap(IExecuteFunctionArgs executeFunctionArgs) {
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		if (createAndExecuteFunctionArgs.existsCompilationObject(MAP_MARKER, false)) {
			return (WeakHashMap) createAndExecuteFunctionArgs.getCompilationObject(MAP_MARKER, false);
		} else {
			WeakHashMap map = new WeakHashMap();
			createAndExecuteFunctionArgs.addCompilationObject(MAP_MARKER, map, false);
			return map;
		}
	}

	static IBitStringEvolutionObjectFeature getBitStringEvolutionObjectFeature(
		IExecuteFunctionArgs executeFunctionArgs,
		IObject simObject)
		throws SimulationRuntimeException {

		WeakHashMap map = getBitStringEvolutionObjectFeatureMap(executeFunctionArgs);
		if (map.containsKey(simObject)) {
			return (IBitStringEvolutionObjectFeature) map.get(simObject);
		} else {
			IBitStringEvolutionObjectFeature feature = getBitStringEvolutionObjectFeatureIntern(simObject);
			map.put(simObject, feature);
			return feature;
		}
	}

	private static IBitStringEvolutionObjectFeature getBitStringEvolutionObjectFeatureIntern(IObject simObject)
		throws SimulationRuntimeException {
		for (int i = 0; i < simObject.getObjectFeaturesCount(); i++) {
			IObjectFeature currIObjectFeature = simObject.getObjectFeatureAt(i);

			if (currIObjectFeature instanceof IBitStringEvolutionObjectFeature) {
				return (IBitStringEvolutionObjectFeature) currIObjectFeature;
			}
		}

		throw new SimulationRuntimeException("Evolution Feature is missing in: " + simObject.getName());
	}

	public String getName() {
		if (_parent1Function == null)
			return "Crossover(" + _parent2Function.getName() + ", " + _childFunction.getName() + ")";
		else
			return "Crossover("
				+ _parent1Function.getName()
				+ ", "
				+ _parent2Function.getName()
				+ ", "
				+ _childFunction.getName()
				+ ")";
	}
}