package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.action.AbstractFunctionActionDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.GenericSimObjectType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert the class' description here.
 * Creation date: (04.09.2001 13:15:18)
 * @author Christoph Oechslein
 */

final class CrossoverSpecificFunctionDeclaration extends AbstractFunctionActionDeclaration {

	public CrossoverSpecificFunctionDeclaration(boolean hasEvolutionFeature) {
		super(
			"Crossover",
			hasEvolutionFeature
				? CollectionsHelper.createList(
					GenericSimObjectType.getGeneralSimObjectType(),
					GenericSimObjectType.getGeneralSimObjectType(),
					DoubleType.getInstance())
				: CollectionsHelper.createList(
					GenericSimObjectType.getGeneralSimObjectType(),
					GenericSimObjectType.getGeneralSimObjectType(),
					GenericSimObjectType.getGeneralSimObjectType(),
					DoubleType.getInstance()),
			!hasEvolutionFeature);

		if (hasEvolutionFeature)
			setDocumentation(
				"Crossovers my genes and the parent2 (Arg1) genes to the child (Arg2)."
					+ " Arg3 is mutation probability.");
		else
			setDocumentation(
				"Crossovers parent1 (Arg1) genes and the parent2 (Arg2) genes to the child (Arg3)."
					+ " Arg4 is mutation probability.");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		if (getSpecificInputDefaultArgsLength() == 3)
			return new CrossoverFunction(
				functions.getFunction(0),
				functions.getFunction(1),
				functions.getFunction(2));
		else
			return new CrossoverFunction(
				functions.getFunction(0),
				functions.getFunction(1),
				functions.getFunction(2),
				functions.getFunction(3));
	}

	public String getSpecificInputTypeNameAt(int index) {
		if (getSpecificInputDefaultArgsLength() == 3) {
			if (index == 0)
				return "parent2";
			else if (index == 1)
				return "child";
			else
				return "prob";
		} else {
			if (index == 0)
				return "parent1";
			else if (index == 1)
				return "parent2";
			else if (index == 2)
				return "child";
			else
				return "prob";
		}
	}
}