package de.uniwue.ki.mas.basics.feature.evolution;
import junit.framework.Assert;

import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Insert the type's description here.
 * Creation date: (22.01.2002 10:12:50)
 * @author Christoph Oechslein
 */
public class DiploidBitStringEvolutionObjectFeature extends BitStringEvolutionObjectFeature {
	private final BitString _bitString2;
	private final EvolutionObjectFeatureClassDeclaration.DominantType _dominantType;

	public DiploidBitStringEvolutionObjectFeature(
		BitStringEvolutionObjectFeatureHelper evolutionObjectFeatureHelper,
		EvolutionObjectFeatureClassDeclaration.DominantType dominantType) {
		super(evolutionObjectFeatureHelper);

		_bitString2 = new BitString(evolutionObjectFeatureHelper.getLength());
		_dominantType = dominantType;
	}

	public DiploidBitStringEvolutionObjectFeature(
		BitStringEvolutionObjectFeatureHelper evolutionObjectFeatureHelper,
		BitString bitString1,
		BitString bitString2,
		EvolutionObjectFeatureClassDeclaration.DominantType dominantType) {

		super(evolutionObjectFeatureHelper, bitString1);

		_bitString2 = bitString2;
		_dominantType = dominantType;
	}

	public BitString getBitString2() {
		return _bitString2;
	}

	public void mutateAllGenes(double mutateProb) {
		super.mutateAllGenes(mutateProb);

		getBitStringEvolutionObjectFeatureHelper().mutateAllGenes(getBitString2(), mutateProb);
	}

	public void setBitString2(BitString newBitString2) {
		_bitString2.setBitString(newBitString2);
	}

	public void setBitStrings(BitString newBitString1, BitString newBitString2) {
		setBitString(newBitString1);
		setBitString2(newBitString2);
	}

	public void writeVariablesToBitString(IObject object) throws SimulationRuntimeException {
		super.writeVariablesToBitString(object);

		// just copy bitString	
		getBitString2().setBitString(getBitString());
	}

	public void writeBitStringToVariables(
		IObjectExecuteFunctionArgs objectExecuteFunctionArgs,
		IObject object)
		throws SimulationRuntimeException {

		BitStringEvolutionObjectFeatureHelper helper = getBitStringEvolutionObjectFeatureHelper();

		for (int i = 0; i < getBitString().getLength(); i++) {
			helper.setVariableValue(objectExecuteFunctionArgs, i, object, calcValue(i));
		}
	}

	private int calcValue(int index) {
		if (_dominantType.equals(EvolutionObjectFeatureClassDeclaration.CO_DOMINANT)) {
			return (int) Math.round(
				0.5 * (getBitString().getGeneValue(index) + getBitString2().getGeneValue(index)));

		} else if (_dominantType.equals(EvolutionObjectFeatureClassDeclaration.BITSTRING_DOMINANT)) {
			// bitstring one and two are already in a randomly chossen order
			//			if (RandomHelper.getRandomBoolean())
			//			return getBitString2().getGeneValue(index);
			//			else
			return getBitString().getGeneValue(index);

		} else {
			Assert.fail();
			return 0;
		}
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			super.appendName(buff, currDepth, maxDepth);
			buff.append("; ");
			getBitString2().appendName(buff, currDepth, maxDepth);
		}
	}

	public void setToDominantBitString(BitString dstBitString) {
		Assert.assertEquals(getBitString().getLength(), dstBitString.getLength());

		for (int i = 0; i < getBitString().getLength(); i++) {
			dstBitString.setGeneValue(i, calcValue(i));
		}
	}
}