package de.uniwue.ki.mas.basics.feature.evolution;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.AbstractFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert the type's description here.
 * Creation date: (19.11.2001 18:58:30)
 * @author Christoph Oechslein
 */
public final class EvolutionFeatureClassDeclaration extends AbstractFeatureClassDeclaration {

	private static final EvolutionFeatureClassDeclaration _instance = new EvolutionFeatureClassDeclaration();

	private IFunctionDeclarationList _functionDeclarationList;
	private IFunctionDeclarationList _defaultFunctionDeclarationList;

	private EvolutionFeatureClassDeclaration() {
		super("Evolution");
	}

	/**
	 * Generates an instance for adding to an object instance declaration,
	 * can produce null if no instance specific information is required.
	 * @param objectClassDeclaration IObjectClassDeclaration
	 * @return IObjectFeatureClassDeclaration
	 * @see de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration#createObjectFeatureClassDeclaration(IObjectClassDeclaration)
	 */
	public IObjectFeatureClassDeclaration createObjectFeatureClassDeclaration(IObjectClassDeclaration objectClassDeclaration) {
		return new EvolutionObjectFeatureClassDeclaration(objectClassDeclaration);
	}

	private synchronized IFunctionDeclarationList getFunctionDeclarationList() {
		if (_functionDeclarationList == null) {
			_functionDeclarationList = new FunctionDeclarationList();
			insertFunctions(_functionDeclarationList);
		}

		return _functionDeclarationList;
	}

	private synchronized IFunctionDeclarationList getDefaultFunctionDeclarationList() {
				if (_defaultFunctionDeclarationList == null) {
					_defaultFunctionDeclarationList = new FunctionDeclarationList();
					insertDefaultFunctions(_defaultFunctionDeclarationList);
		}

		return _defaultFunctionDeclarationList;
	}
	public Iterator getFunctionDeclarations() {
		return getFunctionDeclarationList().iterator();
	}

	public Iterator getDefaultFunctionDeclarations() {
		return getDefaultFunctionDeclarationList().iterator();
	}

	public static final EvolutionFeatureClassDeclaration getInstance() {
		return _instance;
	}

	public Iterator getVariableClassDeclarations() {
		return CollectionsHelper.EMPTY_ITERATOR;
	}

	private void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
		// inherit(SimObject)
		functionDeclarationList.add(new InheritSpecificFunctionDeclaration(true));

		// crossover(SimObject,SimObject, Double)
		functionDeclarationList.add(new CrossoverSpecificFunctionDeclaration(true));

		// melting(SimObject, Simobject)
		functionDeclarationList.add(new MeltingSpecificFunctionDeclaration(true));
		
		// resetGenes(SimObject)
		functionDeclarationList.add(new SetGeneValueSpecificFunctionDeclaration(false));
	}
	

	private void insertDefaultFunctions(IFunctionDeclarationList functionDeclarationList) {
		// isDiploid(SimObject)
		functionDeclarationList.add(new IsDiploidSpecificFunctionDeclaration(true));

		// inherit(SimObject,SimObject)
		functionDeclarationList.add(new InheritSpecificFunctionDeclaration(false));

		// setDiploid(SimObject,Boolean)
		functionDeclarationList.add(new SetDiploidSpecificFunctionDeclaration());

		// MutateAll(SimObject, Double)
		functionDeclarationList.add(new MutateAllSpecificFunctionDeclaration());

		// crossover(SimObject,SimObject,SimObject, Double)
		functionDeclarationList.add(new CrossoverSpecificFunctionDeclaration(false));

		// multipleCrossover(SimObject,SimObject, int)
		functionDeclarationList.add(new MultipleCrossoverSpecificFunctionDeclaration(true));

		// melting(SimObject,SimObject,SimObject)
		functionDeclarationList.add(new MeltingSpecificFunctionDeclaration(false));

		// RouletteWheel(Fun<T> Number, Iterator<T>, Number, Boolean)
		functionDeclarationList.add(new RouletteWheelGenericFunctionDeclaration());

	}
	
	public String getBuildNumber() {
		return "1301";
	}
}