package de.uniwue.ki.mas.basics.feature.evolution;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import javax.swing.table.TableModel;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.*;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.list.DefaultMutableListModel;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: (17.01.2002 15:45:13)
 * @author Christoph Oechslein
 */
public final class EvolutionObjectFeatureClassDeclaration
	extends AbstractDeclaration
	implements IObjectFeatureClassDeclaration {

	public static final class EvoEntry extends AbstractNamed implements Serializable {
		private final IBodyVariableClassDeclaration _bodyVariableClassDeclaration;
		private int _minValue = 0;
		private int _maxValue = 1000;

		public EvoEntry(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
			super();

			_bodyVariableClassDeclaration = bodyVariableClassDeclaration;
		}

		public EvoEntry(
			IBodyVariableClassDeclaration bodyVariableClassDeclaration,
			int minValue,
			int maxValue) {
			super();

			_bodyVariableClassDeclaration = bodyVariableClassDeclaration;
			setMinValue(minValue);
			setMaxValue(maxValue);
		}

		public String getName() {
			return getBodyVariableClassDeclaration().getName()
				+ " ("
				+ getMinValue()
				+ ", "
				+ getMaxValue()
				+ ")";
		}

		public IBodyVariableClassDeclaration getBodyVariableClassDeclaration() {
			return _bodyVariableClassDeclaration;
		}

		public void setMaxValue(int maxValue) {
			_maxValue = maxValue;
		}

		public int getMaxValue() {
			return _maxValue;
		}

		public void setMinValue(int minValue) {
			_minValue = minValue;
		}

		public int getMinValue() {
			return _minValue;
		}
	}

	public static final class DominantType extends AbstractNamed implements Serializable {
		private final int _id;
		private final String _name;

		protected DominantType(String name, int id) {
			_id = id;
			_name = name;
		}

		public String getName() {
			return _name;
		}

		public boolean equals(Object obj) {
			if (obj == this)
				return true;
			else if (obj instanceof DominantType)
				return ((DominantType) obj)._id == _id;
			else
				return false;
		}

		public int hashCode() {
			return _id * 31;
		}
	}

	private final class BodyVarClassDeclarationListListener
		implements DeclarationListListener, Serializable {

		public void declarationAdded(DeclarationListEvent event) {
		}

		public void declarationRemoved(DeclarationListEvent event) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration) event.getDeclaration();
			EvoEntry evoEntry = getEvoEntry(bodyVariableClassDeclaration);
			if (evoEntry != null) {
				getEvoEntryListModel().remove(evoEntry);
				fireStateChanged();
			}
		}
	}

	public static final DominantType CO_DOMINANT = new DominantType("co dominant", 1);
	public static final DominantType BITSTRING_DOMINANT = new DominantType("one bitstring dominant", 2);

	private final IObjectClassDeclaration _objectClassDeclaration;
	private final EvolutionObjectFeatureClassDeclarationTableModelAdapter _evolutionObjectFeatureClassDeclarationTableModelAdapter;

	private final IMutableListModel _evoEntryListModel = new DefaultMutableListModel();

	private boolean _diploid;

	private DominantType _dominantType = CO_DOMINANT;

	private DeclarationListListener _bodyVarClassDeclarationListListener =
		new BodyVarClassDeclarationListListener();

	public EvolutionObjectFeatureClassDeclaration(IObjectClassDeclaration objectClassDeclaration) {
		super();

		_objectClassDeclaration = objectClassDeclaration;
		_evolutionObjectFeatureClassDeclarationTableModelAdapter =
			new EvolutionObjectFeatureClassDeclarationTableModelAdapter(this);
		getObjectBodyVariableClassDeclarationList().addDeclarationListListener(
			_bodyVarClassDeclarationListListener);
	}

	public void checkAllEvolutionVariables() {
		for (int i = 0; i < getEvoEntryListModel().size(); i++) {
			EvoEntry evoEntry = (EvoEntry) getEvoEntryListModel().get(i);
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				evoEntry.getBodyVariableClassDeclaration();

			if (!getCanBeEvolved(bodyVariableClassDeclaration)) {
				getEvoEntryListModel().remove(i);
				fireStateChanged();

				i--;
			}
		}
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		if (intoObject == this)
			return;

		super.copyInto(intoObject, objectMapping);

		EvolutionObjectFeatureClassDeclaration intoEvolutionObjectFeatureClassDeclaration =
			(EvolutionObjectFeatureClassDeclaration) intoObject;

		intoEvolutionObjectFeatureClassDeclaration.setDiploid(isDiploid());

		intoEvolutionObjectFeatureClassDeclaration.getEvoEntryListModel().clear();

		for (int i = 0; i < getEvoEntryListModel().size(); i++) {
			EvoEntry evoEntry = (EvoEntry) getEvoEntryListModel().get(i);

			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				evoEntry.getBodyVariableClassDeclaration();

			if ((objectMapping == null) || (!objectMapping.containsKey(bodyVariableClassDeclaration)))
				intoEvolutionObjectFeatureClassDeclaration.getEvoEntryListModel().add(
					new EvoEntry(
						bodyVariableClassDeclaration,
						evoEntry.getMinValue(),
						evoEntry.getMaxValue()));

			else {
				intoEvolutionObjectFeatureClassDeclaration.getEvoEntryListModel().add(
					new EvoEntry(
						(IBodyVariableClassDeclaration) objectMapping.get(bodyVariableClassDeclaration),
						evoEntry.getMinValue(),
						evoEntry.getMaxValue()));
			}

			if (Assert.ASSERTION_ON) {
				IBodyVariableClassDeclaration intoBodyVariableClassDeclaration =
					((EvoEntry) intoEvolutionObjectFeatureClassDeclaration.getEvoEntryListModel().get(i))
						.getBodyVariableClassDeclaration();
				Assert.assertTrue(
					intoEvolutionObjectFeatureClassDeclaration
						.getObjectBodyVariableClassDeclarationList()
						.contains(
						intoBodyVariableClassDeclaration));
				IBodyVariableClassDeclaration bodyVariableClassDeclaration2 =
					intoBodyVariableClassDeclaration;
				Assert.assertTrue(
					(intoEvolutionObjectFeatureClassDeclaration.getEvoEntry(bodyVariableClassDeclaration2)
						!= null));
			}
		}

		_evolutionObjectFeatureClassDeclarationTableModelAdapter.copyInto(
			intoEvolutionObjectFeatureClassDeclaration
				._evolutionObjectFeatureClassDeclarationTableModelAdapter,
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		if ((objectMapping == null) || (!objectMapping.containsKey(getObjectClassDeclaration())))
			return new EvolutionObjectFeatureClassDeclaration(getObjectClassDeclaration());
		else
			return new EvolutionObjectFeatureClassDeclaration(
				(IObjectClassDeclaration) objectMapping.get(getObjectClassDeclaration()));
	}

	public IObjectFeatureInstanceDeclaration createObjectFeatureInstanceDeclaration(IObjectInstanceDeclaration objectInstanceDeclaration) {
		return new EvolutionObjectFeatureInstanceDeclaration(this);
	}

	BitStringEvolutionObjectFeatureHelper.EvoEntry[] createEvoEntryArray() {
		BitStringEvolutionObjectFeatureHelper.EvoEntry[] evoEntryArray =
			new BitStringEvolutionObjectFeatureHelper.EvoEntry[getEvoEntryListModel().size()];

		for (int i = 0; i < getEvoEntryListModel().size(); i++) {
			EvoEntry evoEntry = (EvoEntry) getEvoEntryListModel().get(i);

			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				evoEntry.getBodyVariableClassDeclaration();

			evoEntryArray[i] =
				new BitStringEvolutionObjectFeatureHelper.EvoEntry(
					getObjectBodyVariableClassDeclarationList().indexOf(bodyVariableClassDeclaration),
					evoEntry.getMinValue(),
					evoEntry.getMaxValue());
		}

		return evoEntryArray;
	}

	public EvoEntry getEvoEntry(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		for (int i = 0; i < getEvoEntryListModel().size(); i++) {
			EvoEntry evoEntry = (EvoEntry) getEvoEntryListModel().get(i);

			if (evoEntry.getBodyVariableClassDeclaration() == bodyVariableClassDeclaration)
				return evoEntry;
		}

		return null;
	}

	public void setBodyVariableClassDeclarationEvolveStatus(
		IBodyVariableClassDeclaration bodyVariableClassDeclaration,
		boolean status) {

		boolean oldStatus = getEvoEntry(bodyVariableClassDeclaration) != null;

		if (oldStatus == status) {

		} else if (status) {
			getEvoEntryListModel().add(new EvoEntry(bodyVariableClassDeclaration));

			fireStateChanged();

		} else {
			getEvoEntryListModel().remove(getEvoEntry(bodyVariableClassDeclaration));

			fireStateChanged();
		}
	}

	public boolean getCanBeEvolved(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		IVariableClassDeclaration variableClassDeclaration =
			bodyVariableClassDeclaration.getVariableClassDeclaration();

		return DoubleType.getInstance().isMoreSpecific(variableClassDeclaration.getSpecificType())
			&& variableClassDeclaration.isWriteable();
	}

	public IBodyVariableClassDeclaration getPossibleEvoBodyVariableClassDeclarationAt(int index) {
		Iterator it = getObjectBodyVariableClassDeclarationList().iterator();

		while (it.hasNext()) {
			IBodyVariableClassDeclaration currBodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration) it.next();

			if (getCanBeEvolved(currBodyVariableClassDeclaration)) {
				if (index == 0) {
					return currBodyVariableClassDeclaration;
				}

				index--;
			}
		}

		return null;
	}

	public int getPossibleEvoBodyVariableClassDeclarationCount() {
		Iterator it = getObjectBodyVariableClassDeclarationList().iterator();

		int count = 0;

		while (it.hasNext()) {
			IBodyVariableClassDeclaration currBodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration) it.next();

			if (getCanBeEvolved(currBodyVariableClassDeclaration)) {
				count++;
			}
		}

		return count;
	}

	public int getPossibleEvoBodyVariableClassDeclarationIndexOf(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		Iterator it = getObjectBodyVariableClassDeclarationList().iterator();

		int index = 0;

		while (it.hasNext()) {
			IBodyVariableClassDeclaration currBodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration) it.next();

			if (getCanBeEvolved(currBodyVariableClassDeclaration)) {
				if (currBodyVariableClassDeclaration == bodyVariableClassDeclaration) {
					return index;
				}
			}

			index++;
		}

		return -1;
	}

	public final TableModel getEvolutionObjectFeatureClassDeclarationTableModel() {
		return _evolutionObjectFeatureClassDeclarationTableModelAdapter;
	}

	public final IFeatureClassDeclaration getFeatureClassDeclaration() {
		return EvolutionFeatureClassDeclaration.getInstance();
	}

	public final Iterator getFunctionDeclarations() {
		return ((EvolutionFeatureClassDeclaration) getFeatureClassDeclaration()).getFunctionDeclarations();
	}

	public final String getName() {
		return getFeatureClassDeclaration().getName();
	}

	public final BodyVariableClassDeclarationList getObjectBodyVariableClassDeclarationList() {
		return getObjectClassDeclaration()
			.getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList();
	}

	private final IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	public final Iterator getVariableClassDeclarations() {
		return CollectionsHelper.EMPTY_ITERATOR;
	}

	public boolean isDiploid() {
		return _diploid;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void remove() {
		if (canBeRemoved()) {
			_evolutionObjectFeatureClassDeclarationTableModelAdapter.remove();
			getObjectBodyVariableClassDeclarationList().removeDeclarationListListener(
				_bodyVarClassDeclarationListListener);

			getEvoEntryListModel().clear();

			super.remove();
		}
	}

	public void setDiploid(boolean newDiploid) {
		if (newDiploid != isDiploid()) {
			_diploid = newDiploid;

			fireStateChanged();
		}
	}

	public IMutableListModel getEvoEntryListModel() {
		return _evoEntryListModel;
	}

	public void setDominantType(EvolutionObjectFeatureClassDeclaration.DominantType dominantType) {
		if (!Utils.equals(dominantType, getDominantType())) {
			_dominantType = dominantType;

			fireStateChanged();
		}
	}

	public EvolutionObjectFeatureClassDeclaration.DominantType getDominantType() {
		return _dominantType;
	}
	
	protected void fireStateChanged() {
		super.fireStateChanged();
	}
}