package de.uniwue.ki.mas.basics.feature.evolution;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.Map;

import javax.swing.table.AbstractTableModel;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.evolution.EvolutionObjectFeatureClassDeclaration.EvoEntry;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.util.declaration.DeclarationListEvent;
import de.uniwue.ki.mas.util.declaration.DeclarationListListener;
/**
 * Insert the type's description here.
 * Creation date: (18.01.2002 11:48:00)
 * @author Christoph Oechslein
 */
final class EvolutionObjectFeatureClassDeclarationTableModelAdapter extends AbstractTableModel {
	private final EvolutionObjectFeatureClassDeclaration _evolutionObjectFeatureClassDeclaration;

	private final BodyVarListListener _bodyVarListListener = new BodyVarListListener();

	private final class BodyVarListListener implements DeclarationListListener, PropertyChangeListener {

		public void propertyChange(PropertyChangeEvent evt) {
			getEvolutionObjectFeatureClassDeclaration().checkAllEvolutionVariables();
			fireTableDataChanged();
		}

		public void declarationAdded(DeclarationListEvent event) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration)event.getDeclaration();
			bodyVariableClassDeclaration.getVariableClassDeclaration().addPropertyChangeListener(
				"type",
				BodyVarListListener.this);
			bodyVariableClassDeclaration.getVariableClassDeclaration().addPropertyChangeListener(
				"writeable",
				BodyVarListListener.this);

			if (getEvolutionObjectFeatureClassDeclaration().getCanBeEvolved(bodyVariableClassDeclaration)) {

				fireTableRowInserted(
					getEvolutionObjectFeatureClassDeclaration()
						.getPossibleEvoBodyVariableClassDeclarationIndexOf(
						bodyVariableClassDeclaration));
			}
		}

		public void declarationRemoved(DeclarationListEvent event) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration)event.getDeclaration();
			bodyVariableClassDeclaration.getVariableClassDeclaration().removePropertyChangeListener(
				"type",
				BodyVarListListener.this);

			if (getEvolutionObjectFeatureClassDeclaration().getCanBeEvolved(bodyVariableClassDeclaration)) {
				fireTableRowDeleted(
					getEvolutionObjectFeatureClassDeclaration()
						.getPossibleEvoBodyVariableClassDeclarationIndexOf(
						bodyVariableClassDeclaration));
			}
		}
	}

	public EvolutionObjectFeatureClassDeclarationTableModelAdapter(EvolutionObjectFeatureClassDeclaration evolutionObjectFeatureClassDeclaration) {
		super();

		_evolutionObjectFeatureClassDeclaration = evolutionObjectFeatureClassDeclaration;

		_evolutionObjectFeatureClassDeclaration
			.getObjectBodyVariableClassDeclarationList()
			.addDeclarationListListener(_bodyVarListListener);

		Iterator it =
			_evolutionObjectFeatureClassDeclaration.getObjectBodyVariableClassDeclarationList().iterator();
		while (it.hasNext()) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration)it.next();

			bodyVariableClassDeclaration.getVariableClassDeclaration().addPropertyChangeListener(
				"type",
				_bodyVarListListener);
		}
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		EvolutionObjectFeatureClassDeclarationTableModelAdapter intoTableModelAdapter =
			(EvolutionObjectFeatureClassDeclarationTableModelAdapter)intoObject;

		for (int row = 0; row < getRowCount(); row++) {
			for (int col = 0; col < getColumnCount(); col++) {
				// should already be the same at this point
				// intoTableModelAdapter.setValue(getValue(row,col),row,col);
				Assert.assertEquals(getValueAt(row, col), intoTableModelAdapter.getValueAt(row, col));
			}
		}
	}

	public void fireTableDataChanged() {
		super.fireTableDataChanged();
	}

	public void fireTableRowDeleted(int index) {
		super.fireTableRowsDeleted(index, index);
	}

	public void fireTableRowInserted(int index) {
		super.fireTableRowsInserted(index, index);
	}

	private static final String[] COLUMN_NAMES = { "Evolve", "Name", "Min value", "Max value" };
	public String getColumnName(int columnIndex) {
		return COLUMN_NAMES[columnIndex];
	}

	private static final Class[] COLUMN_CLASSES =
		{ Boolean.class, String.class, Integer.class, Integer.class };
	public Class getColumnClass(int columnIndex) {
		return COLUMN_CLASSES[columnIndex];
	}

	public int getColumnCount() {
		Assert.assertEquals(COLUMN_NAMES.length, COLUMN_CLASSES.length);
		return COLUMN_NAMES.length;
	}

	protected EvolutionObjectFeatureClassDeclaration getEvolutionObjectFeatureClassDeclaration() {
		return _evolutionObjectFeatureClassDeclaration;
	}

	public int getRowCount() {
		if (getEvolutionObjectFeatureClassDeclaration() == null)
			return 0;
		else
			return getEvolutionObjectFeatureClassDeclaration()
				.getPossibleEvoBodyVariableClassDeclarationCount();
	}

	public Object getValueAt(int row, int col) {
		if (getEvolutionObjectFeatureClassDeclaration() == null)
			return null;

		else {
			IBodyVariableClassDeclaration currVar =
				getEvolutionObjectFeatureClassDeclaration().getPossibleEvoBodyVariableClassDeclarationAt(row);

			if (col == 1)
				return currVar.getName();

			else {
				EvoEntry evoEntry = getEvolutionObjectFeatureClassDeclaration().getEvoEntry(currVar);

				if (col == 0)
					return Boolean.valueOf(evoEntry != null);

				else if (evoEntry != null) {
					if (col == 2) {
						return new Integer(evoEntry.getMinValue() + "");
					} else {
						return new Integer(evoEntry.getMaxValue() + "");
					}

				} else
					return null;
			}
		}
	}

	public boolean isCellEditable(int row, int col) {
		if ((col == 2) || (col == 3)) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				getEvolutionObjectFeatureClassDeclaration().getPossibleEvoBodyVariableClassDeclarationAt(row);
			return (
				getEvolutionObjectFeatureClassDeclaration().getEvoEntry(bodyVariableClassDeclaration)
					!= null);

		} else {
			return true;
		}
	}

	public void remove() {
		for (int i = 0;
			i < getEvolutionObjectFeatureClassDeclaration().getPossibleEvoBodyVariableClassDeclarationCount();
			i++) {
			IBodyVariableClassDeclaration bodyVar =
				getEvolutionObjectFeatureClassDeclaration().getPossibleEvoBodyVariableClassDeclarationAt(i);
			IBodyVariableClassDeclaration bodyVariableClassDeclaration = bodyVar;

			if (getEvolutionObjectFeatureClassDeclaration().getEvoEntry(bodyVariableClassDeclaration)
				!= null) {
				bodyVar.removePropertyChangeListener("type", _bodyVarListListener);
			}
		}

		_evolutionObjectFeatureClassDeclaration
			.getObjectBodyVariableClassDeclarationList()
			.removeDeclarationListListener(_bodyVarListListener);
	}

	public void setValueAt(Object aValue, int row, int col) {
		if (getEvolutionObjectFeatureClassDeclaration() != null) {
			IBodyVariableClassDeclaration currVar =
				getEvolutionObjectFeatureClassDeclaration().getPossibleEvoBodyVariableClassDeclarationAt(row);

			if (col == 0) {
				getEvolutionObjectFeatureClassDeclaration().setBodyVariableClassDeclarationEvolveStatus(
					currVar,
					((Boolean)aValue).booleanValue());
				fireTableCellUpdated(row, col);

			} else if (col == 1) {
				currVar.getVariableClassDeclaration().setShortDescription((String)aValue);
				fireTableCellUpdated(row, col);

			} else if (col == 2) {
				getEvolutionObjectFeatureClassDeclaration().getEvoEntry(currVar).setMinValue(
					((Integer)aValue).intValue());
				fireTableCellUpdated(row, col);

			} else {
				getEvolutionObjectFeatureClassDeclaration().getEvoEntry(currVar).setMaxValue(
					((Integer)aValue).intValue());
				fireTableCellUpdated(row, col);
			}
		}
	}
}