package de.uniwue.ki.mas.basics.feature.evolution;

import java.util.*;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.SpecificVariableClassDeclarationType;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.*;

/**
 * Insert Documentation string.
 * Creation date: 04.09.2002, 19:18:58
 * @author oechslein
 */
public class GetGenCombinationsFunction extends AbstractFunctionDynamic {

	private final IFunction _setOfSetOfNumbersFunction;
	private final IFunction _setOfVariablesFunction;
	private final IFunction _setOfObjectsFunction;
	private final IFunction _thresholdFunction;

	public GetGenCombinationsFunction(
		IFunction setOfSetOfNumbersFunction,
		IFunction setOfVariablesFunction,
		IFunction setOfObjectsFunction,
		IFunction thresholdFunction) {
		super();

		_setOfSetOfNumbersFunction = setOfSetOfNumbersFunction;
		_setOfVariablesFunction = setOfVariablesFunction;
		_setOfObjectsFunction = setOfObjectsFunction;
		_thresholdFunction = thresholdFunction;
	}

	private List createSetOfSetOfNumbers(IExecuteFunctionArgs executeFunctionArgs) throws Exception {
		List outerList = new LinkedList();

		IExecuteIterator setOfSetOfNumbers =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _setOfSetOfNumbersFunction);

		while (setOfSetOfNumbers.hasNext()) {
			List innerList = new LinkedList();
			IExecuteIterator element =
				ExecuteIteratorFactory.createExecuteIterator(setOfSetOfNumbers.nextWithException());

			while (element.hasNext()) {
				innerList.add(element.nextWithException());
			}
			outerList.add(innerList);
		}

		return outerList;
	}

	private static class MapKey {
		final int[] _indexInSetOfSet;

		public MapKey(int[] indexInSetOfSet) {
			_indexInSetOfSet = indexInSetOfSet;
		}

		public boolean equals(Object anObject) {
			if (anObject == this)
				return true;

			if (anObject instanceof MapKey) {
				MapKey mapKey = (MapKey)anObject;
				return Arrays.equals(_indexInSetOfSet, mapKey._indexInSetOfSet);
			} else
				return false;
		}

		public int hashCode() {
			return _indexInSetOfSet.hashCode();
		}
	}

	private int[] createVariableValuesArray(IObject queringAgent, IBody fromBody, List setOfVariables)
		throws SimulationRuntimeException {

		int[] result = new int[setOfVariables.size()];
		Iterator it = setOfVariables.iterator();
		int i = 0;
		while (it.hasNext()) {
			int index = SpecificVariableClassDeclarationType.getVariableIndexOf(fromBody, it.next());

			result[i] = fromBody.getBodyVariableAt(index).getIntValue();
			i++;
		}

		return result;
	}

	private MapKey createMapKey(
		IObject queringAgent,
		IBody body,
		List setOfSetOfNumbers,
		List setOfVariables)
		throws SimulationRuntimeException {

		int[] variableValues = createVariableValuesArray(queringAgent, body, setOfVariables);

		int[] indexInSetOfSet = new int[variableValues.length];

		Iterator setOfNumbers = setOfSetOfNumbers.iterator();

		int i = 0;

		while (setOfNumbers.hasNext()) {
			List setOfNumber = (List)setOfNumbers.next();
			Iterator numbers = setOfNumber.iterator();

			boolean found = false;

			while (numbers.hasNext()) {
				double element = ((Double)numbers.next()).doubleValue();

				if (variableValues[i] < element) {
					indexInSetOfSet[i] = i;
					found = true;
					break;
				}
			}

			if (!found)
				indexInSetOfSet[i] = setOfNumber.size() - 1;

			i++;
		}
		return new MapKey(indexInSetOfSet);
	}

	private void addToMap(
		Map map,
		IObject queringAgent,
		IBody body,
		List setOfSetOfNumbers,
		List setOfVariables)
		throws SimulationRuntimeException {

		MapKey mapKey = createMapKey(queringAgent, body, setOfSetOfNumbers, setOfVariables);
		Integer oldValue = (Integer)map.get(mapKey);
		map.put(mapKey, new Integer((oldValue == null) ? 1 : oldValue.intValue() + 1));
	}

	private Iterator createListOfNumbersOverThreshold(
		Map map,
		List setOfSetOfNumbers,
		int amount,
		double threshold) {
		List result = new LinkedList();
		Iterator it = map.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry element = (Map.Entry)it.next();

			if (((Integer)element.getValue()).intValue() >= (amount * threshold))
				result.add(createResultEntry((MapKey)element.getKey(), setOfSetOfNumbers));

		}
		return result.iterator();
	}

	private Iterator createResultEntry(MapKey mapKey, List setOfSetOfNumbers) {
		List result = new LinkedList();
		Iterator it = setOfSetOfNumbers.iterator();
		int[] indexInSetOfSet = mapKey._indexInSetOfSet;
		for (int i = 0; i < indexInSetOfSet.length; i++) {
			result.add(((List)it.next()).get(indexInSetOfSet[i]));
		}
		return result.iterator();
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		try {
			List setOfSetOfNumbers = createSetOfSetOfNumbers(executeFunctionArgs);
			List setOfVariables =
				ExecuteIteratorFactory.createList(executeFunctionArgs, _setOfVariablesFunction);
			IExecuteIterator setOfObjects =
				ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _setOfObjectsFunction);

			double threshold = _thresholdFunction.executeForDouble(executeFunctionArgs);

			IObject queringAgent = ((IObjectExecuteFunctionArgs)executeFunctionArgs).getObject();

			Map map = new HashMap();

			int amount = 0;

			while (setOfObjects.hasNext()) {
				IObject element = (IObject)setOfObjects.nextWithException();

				addToMap(map, queringAgent, element.getBody(), setOfSetOfNumbers, setOfVariables);

				amount++;
			}

			return createListOfNumbersOverThreshold(map, setOfSetOfNumbers, amount, threshold);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}
}