package de.uniwue.ki.mas.basics.feature.evolution;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Insert the class' description here.
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class InheritFunction extends AbstractFunctionAction {
	private final IFunction _parentFunction;
	private final IFunction _childFunction;

	public InheritFunction(IFunction childFunction) {
		super();

		_parentFunction = null;
		_childFunction = childFunction;
	}

	public InheritFunction(IFunction parentFunction, IFunction childFunction) {
		super();

		_parentFunction = parentFunction;
		_childFunction = childFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IObjectExecuteFunctionArgs objectExecuteFunctionArgs =
			(IObjectExecuteFunctionArgs) executeFunctionArgs;

		IObject parent;

		if (_parentFunction == null) {
			parent = objectExecuteFunctionArgs.getObject();

		} else {
			parent = (IObject) _parentFunction.execute(executeFunctionArgs);

			if (parent == null)
				throw new SimulationRuntimeException("Parent in " + getName() + " is null.");
		}

		IObject child = (IObject) _childFunction.execute(executeFunctionArgs);

		if (child == null)
			throw new SimulationRuntimeException("Child in " + getName() + " is null.");

		BitStringInheritHelper.inherit(objectExecuteFunctionArgs, parent, child);
	}

	public String getName() {
		if (_parentFunction == null)
			return "Inherit(" + _childFunction.getName() + ")";
		else
			return "Inherit(" + _parentFunction.getName() + ", " + _childFunction.getName() + ")";
	}

}