package de.uniwue.ki.mas.basics.feature.evolution;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Insert the class' description here.
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class MeltingFunction extends AbstractFunctionAction {
	private final IFunction _parent1Function;
	private final IFunction _parent2Function;
	private final IFunction _childFunction;

	public MeltingFunction(IFunction parent1Function, IFunction parent2Function, IFunction childFunction) {

		super();

		_parent1Function = parent1Function;
		_parent2Function = parent2Function;
		_childFunction = childFunction;
	}

	public MeltingFunction(IFunction parent2Function, IFunction childFunction) {

		super();

		_parent1Function = null;
		_parent2Function = parent2Function;
		_childFunction = childFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IObjectExecuteFunctionArgs objectExecuteFunctionArgs =
			(IObjectExecuteFunctionArgs) executeFunctionArgs;

		IObject parent1;

		if (_parent1Function == null) {
			parent1 = objectExecuteFunctionArgs.getObject();
		} else {
			parent1 = (IObject) _parent1Function.execute(objectExecuteFunctionArgs);
			if (parent1 == null)
				throw new SimulationRuntimeException("Parent1 in " + getName() + " is null.");
		}

		IObject parent2 = (IObject) _parent2Function.execute(objectExecuteFunctionArgs);
		if (parent2 == null)
			throw new SimulationRuntimeException("Parent2 in " + getName() + " is null.");

		IObject child = (IObject) _childFunction.execute(objectExecuteFunctionArgs);
		if (child == null)
			throw new SimulationRuntimeException("Child in " + getName() + " is null.");


       BitStringMeltingHelper.melt(objectExecuteFunctionArgs, parent1, parent2, child);
		}
	

		public String getName() {
			if (_parent1Function == null)
				return "Melt with (" + _parent2Function.getName() + " to " + _childFunction.getName() + ")";
			else
				return "Melt ("
					+ _parent1Function.getName()
					+ "+ "
					+ _parent2Function.getName()
					+ "-> "
					+ _childFunction.getName()
					+ ")";
		}
	}