package de.uniwue.ki.mas.basics.feature.evolution;
import java.util.WeakHashMap;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.SerializableObject;

/**
 * Insert the class' description here.
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class MultipleCrossoverFunction extends AbstractFunctionAction {
	private final IFunction _parentFunction;
	private final IFunction _childFunction;
	private final IFunction _pointsFunction;

	public MultipleCrossoverFunction(
		IFunction parentFunction,
		IFunction childFunction,
		IFunction pointsFunction) {

		super();

		_parentFunction = parentFunction;
		_childFunction = childFunction;
		_pointsFunction = pointsFunction;
	}

	public MultipleCrossoverFunction(IFunction childFunction, IFunction pointsFunction) {

		super();

		_parentFunction = null;
		_childFunction = childFunction;
		_pointsFunction = pointsFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IObjectExecuteFunctionArgs objectExecuteFunctionArgs =
			(IObjectExecuteFunctionArgs) executeFunctionArgs;

		IObject parent;

		if (_parentFunction == null) {
			parent = objectExecuteFunctionArgs.getObject();
		} else {
			parent = (IObject) _parentFunction.execute(objectExecuteFunctionArgs);
			if (parent == null)
				throw new SimulationRuntimeException("Parent in " + getName() + " is null.");
		}

		IObject child = (IObject) _childFunction.execute(objectExecuteFunctionArgs);
		if (child == null)
			throw new SimulationRuntimeException("Child in " + getName() + " is null.");

		int crossPoints = _pointsFunction.executeForInt(objectExecuteFunctionArgs);

		BitStringMultipleCrossoverHelper.crossover(objectExecuteFunctionArgs, parent, child, crossPoints);
	}

	private static final SerializableObject MAP_MARKER = new SerializableObject();

	private static WeakHashMap getBitStringEvolutionObjectFeatureMap(IExecuteFunctionArgs executeFunctionArgs) {
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		if (createAndExecuteFunctionArgs.existsCompilationObject(MAP_MARKER, false)) {
			return (WeakHashMap) createAndExecuteFunctionArgs.getCompilationObject(MAP_MARKER, false);
		} else {
			WeakHashMap map = new WeakHashMap();
			createAndExecuteFunctionArgs.addCompilationObject(MAP_MARKER, map, false);
			return map;
		}
	}

	static IBitStringEvolutionObjectFeature getBitStringEvolutionObjectFeature(
		IExecuteFunctionArgs executeFunctionArgs,
		IObject simObject)
		throws SimulationRuntimeException {

		WeakHashMap map = getBitStringEvolutionObjectFeatureMap(executeFunctionArgs);
		if (map.containsKey(simObject)) {
			return (IBitStringEvolutionObjectFeature) map.get(simObject);
		} else {
			IBitStringEvolutionObjectFeature feature = getBitStringEvolutionObjectFeatureIntern(simObject);
			map.put(simObject, feature);
			return feature;
		}
	}

	private static IBitStringEvolutionObjectFeature getBitStringEvolutionObjectFeatureIntern(IObject simObject)
		throws SimulationRuntimeException {
		for (int i = 0; i < simObject.getObjectFeaturesCount(); i++) {
			IObjectFeature currIObjectFeature = simObject.getObjectFeatureAt(i);

			if (currIObjectFeature instanceof IBitStringEvolutionObjectFeature) {
				return (IBitStringEvolutionObjectFeature) currIObjectFeature;
			}
		}

		throw new SimulationRuntimeException("Evolution Feature is missing in: " + simObject.getName());
	}

	public String getName() {
		if (_parentFunction == null)
			return "MultipleCrossover(" + _childFunction.getName() + " at" + _pointsFunction.getName() + ")";
		else
			return "MulipleCrossover("
				+ _parentFunction.getName()
				+ ", "
				+ _childFunction.getName()
				+ " at"
				+ _pointsFunction.getName()
				+ ")";
	}
}