package de.uniwue.ki.mas.basics.feature.evolution;
import de.uniwue.ki.mas.basics.action.AbstractFunctionActionDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.GenericSimObjectType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert the class' description here.
 * Creation date: (04.02.2004)
 * @author Franziska Kluegl
 */

final class MultipleCrossoverSpecificFunctionDeclaration extends AbstractFunctionActionDeclaration {

	public MultipleCrossoverSpecificFunctionDeclaration(boolean hasEvolutionFeature) {
		super(
			"(Multiple)Rekombination",
			hasEvolutionFeature
				? CollectionsHelper.createList(
					GenericSimObjectType.getGeneralSimObjectType(),
					IntegerType.getInstance())
				: CollectionsHelper.createList(
					GenericSimObjectType.getGeneralSimObjectType(),
					GenericSimObjectType.getGeneralSimObjectType(),
					IntegerType.getInstance()),
			!hasEvolutionFeature);

		if (hasEvolutionFeature)
			setDocumentation(
				"Recombines my genes and transfers them to the child (Arg1)."
					+ " Arg2 is the number of Crossover Points.");
		else
			setDocumentation(
				"Recombines parent1 (Arg1) genes and transfers them to the child (Arg2)."
					+ " Arg3 is the number of Crossover Points.");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		if (getSpecificInputDefaultArgsLength() == 2)
			return new MultipleCrossoverFunction(
				functions.getFunction(0),
				functions.getFunction(1)
				);
		else
			return new MultipleCrossoverFunction(
				functions.getFunction(0),
				functions.getFunction(1),
				functions.getFunction(2)
				);
	}

	public String getSpecificInputTypeNameAt(int index) {
		if (getSpecificInputDefaultArgsLength() == 2) {
			if (index == 0)
				return "child";
			else
				return "points";
		} else {
			if (index == 0)
				return "parent";
			else if (index == 1)
				return "child";
			else
				return "points";
		}
	}
}