package de.uniwue.ki.mas.basics.feature.evolution;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * Insert Documentation string.
 * Creation date: 29.01.2003, 14:21:20
 * @author oechslein
 */
final class RouletteWheelGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	public RouletteWheelGenericFunctionDeclaration() {
		super("RouletteWheel", GenericListType.getInstance(), true);

		setDocumentation("Implements the Roulette Wheel operation. " +
			"First argument defines the value function of the objects in second argument. " +
			"Operation returns a list of length Arg3 where each entry is choosen randomly " +
			"due to the value function (higher value give higher probability). " +
			"Last argument decides if resulting list is duplicate free.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType))
			return createFunctionDeclaration(
				GenericListType.getInstance().createSpecificListType(ObjectType.getInstance()));
		else
			return createFunctionDeclaration((SpecificListType)outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(SpecificListType specificListType) {
		return new RouletteWheelSpecificFunctionDeclaration(this, specificListType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);
		ISpecificType specificTypeC = getNextSpecificType(inputTypes);
		ISpecificType specificTypeD = getNextSpecificType(inputTypes);

		if (!(specificTypeA instanceof ISpecificFunctionType)
			|| !(specificTypeB instanceof SpecificListType)
			|| !(IntegerType.getInstance().equals(specificTypeC))
			|| !(BooleanType.getInstance().equals(specificTypeD))
			|| (inputTypes.hasNext()))
			return null;

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)specificTypeA;
		SpecificListType specificListType = (SpecificListType)specificTypeB;

		if (!DoubleType.getInstance().isMoreSpecific(specificFunctionType.getSpecificOutputType())
			|| (specificFunctionType.getSpecificInputTypesLength() != 1))
			return null;

		if (specificListType
			.getIteratorSpecificType()
			.isMoreSpecific(specificFunctionType.getSpecificInputTypeAt(0)))
			return createFunctionDeclaration(specificListType);
		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		ISpecificType iteratorType;
		if (ObjectType.getInstance().equals(outputType)) {
			iteratorType = ObjectType.getInstance();
		} else {
			iteratorType = ((SpecificListType)outputType).getIteratorSpecificType();
		}

		return "(Fun<"
			+ iteratorType.getName()
			+ "> Number, List<"
			+ iteratorType.getName()
			+ ">, Integer, Boolean)";
	}
}
