package de.uniwue.ki.mas.basics.feature.evolution;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert Documentation string.
 * Creation date: 29.01.2003, 14:32:04
 * @author oechslein
 */
final class RouletteWheelSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final SpecificListType _specificListType;

	public RouletteWheelSpecificFunctionDeclaration(
		RouletteWheelGenericFunctionDeclaration rouletteWheelGenericFunctionDeclaration,
		SpecificListType specificListType) {

		super(rouletteWheelGenericFunctionDeclaration, 4);

		_specificListType = specificListType;
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		Assert.assertEquals(functions.size(),4);

		return new RouletteWheelFunction(
			functions.getFunction(0),
			functions.getFunction(1),
			functions.getFunction(2),
			functions.getFunction(3));
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return GenericFunctionType.getInstance().createSpecificFunctionType(
				DoubleType.getInstance(),
				CollectionsHelper.createList(_specificListType.getIteratorSpecificType()));

		} else if (index == 1) {
			return _specificListType;

		} else if (index == 2) {
			return IntegerType.getInstance();

		} else if (index == 3) {
			return BooleanType.getInstance();

		} else {
			Assert.fail(); 
			return null;
		}
	}
	
	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0) {
			return "valueFunction";

		} else if (index == 1) {
			return "objects";

		} else if (index == 2) {
			return "amount";

		} else if (index == 3) {
			return "allowDuplicates";

		} else {
			return super.getSpecificInputTypeNameAt(index);
		}
	}


	public ISpecificType getSpecificOutputType() {
		return _specificListType;
	}
}
