package de.uniwue.ki.mas.basics.feature.spatial;

import java.awt.Image;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.io.File;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (14.09.2001 10:44:37)
 * @author Christoph Oechslein
 */

public class CircularSpatialInfo 
	extends AbstractCopyable 
	implements ISpatialInfo, INamed, IIsomorphicHashable {
		
	private Object _anObject;
	private final Position2D _position = new Position2D();

	/** direction in rad */
	private double _direction = 0.0;
	private double _speed = 10.0;
	private double _radius = 5.0;
	private double _drawPriority = 3.0;

	private transient Image _image = ImageType.getDefaultAgentImage();
	private File _imagePath = null;

	public CircularSpatialInfo(ISpatialInfo spatialInfo, Object anObject) {
		super();

		_direction = spatialInfo.getDirection();
		_position.x = spatialInfo.getPosition().x;
		_position.y = spatialInfo.getPosition().y;
		_radius = spatialInfo.getRadius();
		_speed = spatialInfo.getSpeed();
		_direction = spatialInfo.getDirection();
		_image = spatialInfo.getImage();
		_imagePath = spatialInfo.getImagePath();
		_drawPriority = spatialInfo.getDrawPriority();

		_anObject = anObject;
	}

	public boolean isomorphicEquals(IIsomorphicHashable arg0) {
		if (arg0 == null)
			return false;

		if (arg0 == this)
			return true;

		if (!(arg0 instanceof CircularSpatialInfo))
			return false;

		CircularSpatialInfo arg = (CircularSpatialInfo)arg0;
		return (
			(arg.getDirection() == getDirection())
				&& (arg.getSpeed() == getSpeed())
				&& (arg.getRadius() == getRadius())
				&& ((arg.getPosition() == null && getPosition() == null)
					|| (arg.getPosition().equals(getPosition())))
				&& (arg.getDrawPriority() == (getDrawPriority()))
				&& (arg.getDirection() == getDirection())
				&& ((arg.getImagePath() == null && getImagePath() == null)
					|| (arg.getImagePath().equals(getImagePath()))));
	}

	public int isomorphicHashCode() {
		return (int)getDirection()
			+ (int)getSpeed() * 3
			+ 7 * ((getPosition() == null) ? 0 : getPosition().hashCode())
			+ ((getImagePath() == null) ? 0 : 11 * getImagePath().hashCode());
	}

	public CircularSpatialInfo(Object anObject) {
		super();

		_anObject = anObject;
	}

	public void copyInto(ISpatialInfo spatialInfo) {
		spatialInfo.setDirection(getDirection());
		spatialInfo.setDrawPriority(getDrawPriority());
		spatialInfo.setRadius(getRadius());
		spatialInfo.setSpeed(getSpeed());
		spatialInfo.getPosition().x = getPosition().x;
		spatialInfo.getPosition().y = getPosition().y;

		if (getImagePath() != null)
			spatialInfo.setImagePath(getImagePath());
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		copyInto((ISpatialInfo)dstObject);

		super.copyInto(dstObject, objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		throw new UnsupportedOperationException();
	}

	public double getDirection() {
		return _direction;
	}

	public Image getImage() {
		return _image;
	}

	public File getImagePath() {
		return _imagePath;
	}

	public String getName() {
		return "["
			+ getPosition().x
			+ ", "
			+ getPosition().y
			+ ", "
			+ getDirection()
			+ ", "
			+ getSpeed()
			+ ", "
			+ getRadius()
			+ "]";
	}

	public Object getObject() {
		return _anObject;
	}

	public Position2D getPosition() {
		return _position;
	}

	public double getRadius() {
		return _radius;
	}

	public double getSpeed() {
		return _speed;
	}

	public void setDirection(double newDirection) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(!Double.isNaN(newDirection));

		_direction = newDirection % (2 * Math.PI);
	}

	public void setImage(Image image) {
		if (image == null)
			_image = ImageType.getDefaultAgentImage();
		else
			_image = image;
	}

	public void setImagePath(File newFile) {
		_imagePath = newFile;

		if (newFile != null) {
			setImage(ImageType.getInstance().createImage(newFile));
		} else {
			setImage(null);
		}
	}

	public void setRadius(double newRadius) {
		Assert.assertTrue(!Double.isNaN(newRadius));

		_radius = newRadius;
	}

	public void setSpeed(double newSpeed) {
		Assert.assertTrue(!Double.isNaN(newSpeed));

		_speed = newSpeed;
	}

	public void setPosition(Point2D newPos) {
		_position.setLocation(newPos);
	}

	public Shape getShape() {
		return new Ellipse2D.Double(getPosition().x, getPosition().y, getRadius() * 2, getRadius() * 2);
	}

	public double getDrawPriority() {
		return _drawPriority;
	}

	public void setDrawPriority(double drawPriority) {
		if (drawPriority < 0)
			drawPriority = 0;
		
		_drawPriority = drawPriority;

	}
	
	public void setObject(Object anObject) {
		_anObject = anObject;
	}


}