/*
 * Created on 10.06.2003
 *
 */
package de.uniwue.ki.mas.basics.feature.spatial;

import java.io.File;
import java.util.*;

import de.uniwue.ki.util.copy.ICopyable;


/**
 * @author muehlberger
 *
 */
public class DefaultSpatialMapModel extends AbstractSpatialMapModel {

	private double _height = 100.0;
	private double _width = 100.0;
	private boolean _torus = false;

	private boolean _drawGrid = true;
	private boolean _discrete = true;
	private boolean _centeredPos = true;
	private File _backgroundImageFile = null;
	private boolean _rotateImages = false;

	private int _gridCols = 10;
	private int _gridRows = 10;

	private LinkedList _defaultSpatialList;

	public DefaultSpatialMapModel() {
		super();

		_defaultSpatialList = new LinkedList();

	}

	public synchronized void addSpatialInfo(ISpatialInfo spatialInfo) {
		
		if (spatialInfo != null){
		_defaultSpatialList.add(spatialInfo);
		
		fireSpatialInfoAdded(spatialInfo);
		}
	}

	public double getHeight() {
		return _height;
	}

	public double getWidth() {
		return _width;
	}

	public void setHeight(double height) {
		if (_height != height) {
			_height = height;
			rearrangeSpatialInfosPosY(height);
			fireSpatialMapModelChanged(this);
		}
	}

	public void setWidth(double width) {
		if (_width != width) {
			_width = width;
			rearrangeSpatialInfosPosX(width);
			fireSpatialMapModelChanged(this);
		}
	}

	public boolean isTorus() {
		return _torus;
	}

	public void setTorus(boolean torus) {
		if (_torus != torus) {
			_torus = torus;
			fireSpatialMapModelChanged(this);
		}
	}

	public boolean isDrawGrid() {
		return _drawGrid;
	}

	public void setDrawGrid(boolean drawGrid) {
		if (_drawGrid != drawGrid) {
			_drawGrid = drawGrid;
			fireSpatialMapModelChanged(this);
		}
	}

	public boolean isDiscrete() {
		return _discrete;
	}

	public void setDiscrete(boolean discrete) {
		if (_discrete != discrete) {
			_discrete = discrete;
			fireSpatialMapModelChanged(this);
		}
	}
	
	public boolean isCenteredPos() {
		return _centeredPos;
	}
	//TODO hier fireSpatialMapModelChanged ntig?!?
	public void setCenteredPos(boolean centeredPos){
		if (_centeredPos != centeredPos) {
		_centeredPos = centeredPos;
//		fireSpatialMapModelChanged(this);
		}
	}

	public File getBackgroundImageFile() {
		return _backgroundImageFile;
	}

	public void setBackgroundImageFile(File image) {
		_backgroundImageFile = image;
		fireSpatialMapModelChanged(this);
	}

	public int getGridCols() {
		return _gridCols;
	}

	public int getGridRows() {
		return _gridRows;
	}

	public void setGridCols(int cols) {
		if (cols < 2)
			_gridCols = 1;
		else
			_gridCols = cols;

		fireSpatialMapModelChanged(this);
	}

	public void setGridRows(int rows) {
		if (rows < 2)
			_gridRows = 1;
		else
			_gridRows = rows;

		fireSpatialMapModelChanged(this);
	}

	public Iterator getAllSpatialInfos() {
		return _defaultSpatialList.iterator();
	}

	public synchronized List getAllSpatialInfoList() {
		return new ArrayList(_defaultSpatialList);
	}

	public synchronized void removeSpatialInfo(ISpatialInfo spatialInfo) {
		_defaultSpatialList.remove(spatialInfo);

		fireSpatialInfoRemoved(spatialInfo);
		
	}

	public boolean isRotateImages() {
		return _rotateImages;
	}

	public void setRotateImages(boolean rotateImages) {
		if (_rotateImages != rotateImages) {
			_rotateImages = rotateImages;
			fireSpatialMapModelChanged(this);
		}
	}

	private void rearrangeSpatialInfosPosY(double height) {
		Iterator iter = getAllSpatialInfos();
		while (iter.hasNext()) {
			CircularSpatialInfo obj = (CircularSpatialInfo) iter.next();
			changePosition(obj, obj.getPosition().x, Math.min(obj.getPosition().y, height));
		}
	}

	private void rearrangeSpatialInfosPosX(double width) {
		Iterator iter = getAllSpatialInfos();
		while (iter.hasNext()) {
			CircularSpatialInfo obj = (CircularSpatialInfo) iter.next();
			changePosition(obj, Math.min(obj.getPosition().x, width), obj.getPosition().y);
		}
	}
	
	public ICopyable copy(Map objectMapping) {
	
		DefaultSpatialMapModel newDefaultSpatialMapModelInstance =
			new DefaultSpatialMapModel();

		copyInto(newDefaultSpatialMapModelInstance, objectMapping);

		return newDefaultSpatialMapModelInstance;
	}


	public void copyInto(Object inToObject, java.util.Map objectMapping) {

		DefaultSpatialMapModel dstDefaultSpatialMapModelInstance =
			(DefaultSpatialMapModel) inToObject;

		dstDefaultSpatialMapModelInstance.setBackgroundImageFile(this.getBackgroundImageFile());
		dstDefaultSpatialMapModelInstance.setCenteredPos(this.isCenteredPos());
		dstDefaultSpatialMapModelInstance.setDiscrete(_discrete);
		dstDefaultSpatialMapModelInstance.setDrawGrid(_drawGrid);
		dstDefaultSpatialMapModelInstance.setGridCols(_gridCols);
		dstDefaultSpatialMapModelInstance.setGridRows(_gridRows);
		dstDefaultSpatialMapModelInstance.setHeight(_height);
		dstDefaultSpatialMapModelInstance.setRotateImages(_rotateImages);
		dstDefaultSpatialMapModelInstance.setTorus(_torus);
		dstDefaultSpatialMapModelInstance.setWidth(_width);

	}
}