package de.uniwue.ki.mas.basics.feature.spatial;
import java.io.File;
import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (04.12.2000 20:07:09)
 * @author Christoph Oechslein
 */

public interface ISpatialMapModel extends Serializable, IRemoveable,ICopyable {

	void addSpatialMapModelListener(ISpatialMapModelListener spatialMapModelListener);

	void removeSpatialMapModelListener(ISpatialMapModelListener spatialMapModelListener);

	void addSpatialInfo(ISpatialInfo spatialInfo);

	void changeDirection(ISpatialInfo spatialInfo, double newDirectionInRad);

	void changeImage(ISpatialInfo spatialInfo, java.awt.Image newImage);

	void changeImagePath(ISpatialInfo spatialInfo, java.io.File newImagePath);

	/**
	  * Move spatial info to position and sets new position in spatial info.
	  */
	void changePosition(ISpatialInfo spatialInfo, double toPosX, double toPosY);

	/**
	  * Move spatial info to position and sets new position in spatial info.
	  */
	void changePosition(ISpatialInfo spatialInfo, Position2D toPosition);

	void changeRadius(ISpatialInfo spatialInfo, double newRadius);

	void changeSpeed(ISpatialInfo spatialInfo, double newSpeed);
	
	void changeDrawPriority(ISpatialInfo spatialInfo, double newPriority);

	double distance(ISpatialInfo spatialInfo1, ISpatialInfo spatialInfo2);

	double getHeight();

	Iterator getAllSpatialInfos();

	List getAllSpatialInfoList();

	Iterator getSpatialInfos(Position2D pos, double radius);

	Iterator getSpatialInfos(Position2D pos, double radius, double angleStartInRad, double angleEndInRad);

	double getWidth();

	boolean overlap(ISpatialInfo spatialInfo1, ISpatialInfo spatialInfo2);

	void removeSpatialInfo(ISpatialInfo spatialInfo);

	void setHeight(double newValue);

	void setWidth(double newValue);

	boolean isTorus();
	void setTorus(boolean newSetting);

	boolean isDrawGrid();
	void setDrawGrid(boolean drawGrid);

	boolean isDiscrete();
	void setDiscrete(boolean discrete);
	
	boolean isCenteredPos();
	void setCenteredPos(boolean centeredPos);

	boolean isRotateImages();
	void setRotateImages(boolean rotateImages);

	File getBackgroundImageFile();
	void setBackgroundImageFile(File image);

	int getGridCols();
	void setGridCols(int cols);

	int getGridRows();
	void setGridRows(int rows);

}