package de.uniwue.ki.mas.basics.feature.spatial;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.net.URL;
import java.util.WeakHashMap;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.AbstractSpecificType;
import de.uniwue.ki.mas.basics.type.argument.Argument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * Creation date: (05.10.2001 15:23:06)
 * @author Christoph Oechslein
 */

public final class ImageType extends AbstractSpecificType {
	private static final ImageType instance = new ImageType();
	private transient static final WeakHashMap _imageMap = new WeakHashMap();

	public static final URL DEFAULT_AGENT_IMAGE_URL = ImageType.class.getResource("defaultAgent.gif");
	private transient static Image _defaultAgentImage = null;

	protected ImageType() {
		super();
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return new Argument(createImage(anObject));
	}

	public Image createImage(Object value) {
		if (value instanceof Image)
			return (Image)value;

		else if (value instanceof String)
			return createImageFile(new File((String)value));

		else if (value instanceof File)
			return createImageFile((File)value);

		else if (value instanceof ISpatialInfo) {
			ISpatialInfo spatialInfo = (ISpatialInfo)value;

			if (spatialInfo.getImage() != null) {
				return spatialInfo.getImage();

			} else if (spatialInfo.getImagePath() != null) {
				return createImageFile(spatialInfo.getImagePath());

			} else {
				return getDefaultAgentImage();
			}

		} else {
			return getDefaultAgentImage();
		}
	}

	private final Image createImageFile(File file) {
		if (file.canRead()) {
			return createImageString(file.toString());
		} else {
			return null;
		}
	}

	private final Image createImageString(String string) {
		if (_imageMap.containsKey(string)) {
			return (Image)_imageMap.get(string);

		} else {
			// Image newImage = java.awt.Toolkit.getDefaultToolkit().getImage(string);
			Image newImage = Toolkit.getDefaultToolkit().createImage(string);
			_imageMap.put(string, newImage);
			return newImage;
		}
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}

	public static final ImageType getInstance() {
		return instance;
	}

	public String getName() {
		return "Image";
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof String) || (value instanceof File);
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (anObject instanceof BufferedImage) {
			BufferedImage bufferedImage = (BufferedImage)anObject;
			buff.append("[" + bufferedImage.getWidth() + ", " + bufferedImage.getHeight() + "]");

		} else if ((anObject instanceof String) || (anObject instanceof File)) {
			buff.append(anObject);

		} else if (anObject instanceof ISpatialInfo) {
			ISpatialInfo spatialInfo = (ISpatialInfo)anObject;

			if (spatialInfo.getImagePath() != null) {
				buff.append(spatialInfo.getImagePath());
			} else if (spatialInfo.getImage() != null) {
				appendNameForArgumentObject(buff, spatialInfo.getImage(), currDepth, maxDepth);

			} else {
				buff.append("defaultImage");
			}

		} else {
			super.appendNameForArgumentObject(buff, anObject, currDepth, maxDepth);
		}
	}

	public static Image getDefaultAgentImage() {
		if (_defaultAgentImage == null) {
			_defaultAgentImage = Toolkit.getDefaultToolkit().getImage(DEFAULT_AGENT_IMAGE_URL);
		}
		return _defaultAgentImage;
	}

}