package de.uniwue.ki.mas.basics.feature.spatial;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.AbstractObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclarationList;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.util.declaration.DefaultDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/**
 * @author oechslein
 * 
 * Creation date: 13.07.2003
 */
public class SpatialInfoObjectFeatureHelper {

	public static ISpatialInfo getSpatialInfo(IObjectInstanceDeclaration objectInstanceDeclaration) {
		SpatialInfoObjectFeatureInstDeclaration featureInstDeclaration =
			getSpatialInfoFeature(objectInstanceDeclaration);
		if (featureInstDeclaration == null)
			return null;
		else
			return featureInstDeclaration.getSpatialInfo();
	}
	public static SpatialInfoObjectFeatureInstDeclaration getSpatialInfoFeature(IObjectInstanceDeclaration objectInstanceDeclaration) {
		Iterator it = objectInstanceDeclaration.getObjectFeatureInstanceDeclarationList().iterator();

		while (it.hasNext()) {
			IObjectFeatureInstanceDeclaration currentFeatureInstanceDeclaration =
				(IObjectFeatureInstanceDeclaration)it.next();

			if (currentFeatureInstanceDeclaration instanceof SpatialInfoObjectFeatureInstDeclaration)
				return (SpatialInfoObjectFeatureInstDeclaration)currentFeatureInstanceDeclaration;
		}

		return null;
	}
	public static boolean hasSpatialInfoFeature(IObjectInstanceDeclaration objectInstanceDeclaration) {
		return getSpatialInfoFeature(objectInstanceDeclaration) != null;
	}

	public static boolean hasSpatialInfoFeature(IObjectClassDeclaration objectClassDeclaration) {
		return getSpatialInfoFeature(objectClassDeclaration) != null;
	}

	public static SpatialInfoObjectFeatureClassDeclaration getSpatialInfoFeature(IObjectClassDeclaration objectClassDeclaration) {
		Iterator it = objectClassDeclaration.getObjectFeatureClassDeclarationList().iterator();
		while (it.hasNext()) {
			IObjectFeatureClassDeclaration objectFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration)it.next();
			if (objectFeatureClassDeclaration instanceof SpatialInfoObjectFeatureClassDeclaration)
				return (SpatialInfoObjectFeatureClassDeclaration)objectFeatureClassDeclaration;
		}
		return null;
	}

	public static ISpatialInfo getSpatialInfo(IObjectClassDeclaration objectClassDeclaration) {
		SpatialInfoObjectFeatureClassDeclaration featureInstDeclaration =
			getSpatialInfoFeature(objectClassDeclaration);
		if (featureInstDeclaration == null)
			return null;
		else
			return featureInstDeclaration.getSpatialInfo();
	}

	public static ISpatialInfo getSpatialInfo(IObject simObject) {
		return SpatialInfoObjectFeature.getSpatialInfo(simObject);
	}
	
	public static IDeclarationList getAllObjectClassesWithSpatialInfoFeature(ISeSAmModelDeclaration seSAmModelDeclaration){
		
		if (seSAmModelDeclaration == null)
			return IDeclarationList.EMPTY_DECLARATION_LIST;			
		
		IDeclarationList objectsWithSpatialInfo = new DefaultDeclarationList();		
		
		IObjectClassDeclarationList objectClassDeclarationList =
			seSAmModelDeclaration.getAvailibleObjectClassDeclarationList();

		for (Iterator it = objectClassDeclarationList.iterator(); it.hasNext();) {
			AbstractObjectClassDeclaration element = (AbstractObjectClassDeclaration) it.next();
			 
			if (hasSpatialInfoFeature(element))
				objectsWithSpatialInfo.add(element);				
				
		}
		
		if (objectsWithSpatialInfo.isEmpty())
			return IDeclarationList.EMPTY_DECLARATION_LIST;
		else
			return objectsWithSpatialInfo;
		
	}
	
	
}
