package de.uniwue.ki.mas.basics.feature.spatial;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.AbstractObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: 04.09.2002, 18:40:18
 * @author oechslein
 */
final class SpatialInfoObjectFeatureInstDeclaration
	extends AbstractObjectFeatureInstanceDeclaration
	implements IHasSpatialInfoDeclaration {

	private final ISpatialInfo _spatialInfo;
	private ISpatialMapModel _spatialMapModel;
	
	public SpatialInfoObjectFeatureInstDeclaration(
		SpatialInfoObjectFeatureClassDeclaration infoObjectFeatureClassDeclaration,
		IObjectInstanceDeclaration objectInstanceDeclaration,
		ISpatialInfo spatialInfo) {

		super(infoObjectFeatureClassDeclaration);

		_spatialInfo = new CircularSpatialInfo(spatialInfo, objectInstanceDeclaration);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		SpatialInfoObjectFeatureInstDeclaration spatialInfoBuildinObjectFeatureInstanceDeclaration =
			(SpatialInfoObjectFeatureInstDeclaration)dstObject;

		getSpatialInfo().copyInto(
			spatialInfoBuildinObjectFeatureInstanceDeclaration.getSpatialInfo(),
			objectMapping);

		super.copyInto(spatialInfoBuildinObjectFeatureInstanceDeclaration, objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		Assert.assertNotNull(objectMapping);

		IObjectInstanceDeclaration newObjectInstDecl =
			(IObjectInstanceDeclaration)objectMapping.get(getSpatialInfo().getObject());

		Assert.assertNotNull(newObjectInstDecl);

		return new SpatialInfoObjectFeatureInstDeclaration(
			(SpatialInfoObjectFeatureClassDeclaration)getObjectFeatureClassDeclaration(),
			newObjectInstDecl,
			getSpatialInfo());
	}

	public IObjectFeature createObjectFeature(IObjectCreateFunctionArgs createFunctionArgs) {
		return new SpatialInfoObjectFeature(this, createFunctionArgs.getObject(), getSpatialInfo());
	}

	public final ISpatialInfo getSpatialInfo() {
		return _spatialInfo;
	}

	public ISpatialMapModel getSpatialMapModel() {
		return _spatialMapModel;
	}

	void setSpatialMapModel(ISpatialMapModel spatialMapModel) {
		_spatialMapModel = spatialMapModel;
	}
}