package de.uniwue.ki.mas.basics.feature.spatial;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.AbstractSpecificType;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;

/**
 * Creation date: (05.10.2001 15:24:18)
 * @author Christoph Oechslein
 */

public final class SpatialInfoType extends AbstractSpecificType {
	private static final SpatialInfoType instance = new SpatialInfoType();

	private static final class SpatialInfoArgument extends AbstractArgument implements IArgument {
		private final ISpatialInfo _spatialInfo;

		public SpatialInfoArgument(ISpatialInfo spatialInfo) {
			super();

			_spatialInfo = spatialInfo;
		}

		public final Object getConstantValue() {
			return _spatialInfo;
		}

		public String getName() {
			return _spatialInfo.getName();
		}
	}

	protected SpatialInfoType() {
		super();
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return new SpatialInfoArgument((ISpatialInfo) anObject);
	}

	public Object getArgumentDeclarationDefaultValue() {
		return new CircularSpatialInfo(null);
	}

	public static final SpatialInfoType getInstance() {
		return instance;
	}

	public String getName() {
		return "SpatialInfo";
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {
			ISpatialInfo spatialInfo = (ISpatialInfo) anObject;
			buff.append("[(");
			buff.append(spatialInfo.getPosition().x);
			buff.append(", ");
			buff.append(spatialInfo.getPosition().y);
			buff.append("), ");
			buff.append(spatialInfo.getDirection());
			buff.append(", ");
			buff.append(spatialInfo.getSpeed());
			buff.append(", ");
			buff.append(spatialInfo.getRadius());
			buff.append("]");
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value instanceof ISpatialInfo;
	}
}