package de.uniwue.ki.mas.basics.feature.spatial;

import java.util.*;

import de.uniwue.ki.mas.basics.feature.*;
import de.uniwue.ki.mas.basics.feature.spatial.function.FunctionDeclarationSpatialFactory;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (22.10.2001 13:41:52)
 * @author Christoph Oechslein
 */

public final class SpatialMapFeatureClassDeclaration extends BuildinFeatureClassDeclaration {
	private static final SpatialMapFeatureClassDeclaration _instance =
		new SpatialMapFeatureClassDeclaration();
	private static transient final Map _spatialMapHash = new WeakHashMap();

	public static final SpatialMapFeatureClassDeclaration getInstance() {
		return _instance;
	}

	public static SpatialMapObjectFeatureInstDeclaration getSpatialMapBuildinObjectFeatureInstanceDeclaration(IWorldInstanceDeclaration worldInstanceDeclaration) {
		Iterator it = worldInstanceDeclaration.getObjectFeatureInstanceDeclarationList().iterator();

		while (it.hasNext()) {
			IObjectFeatureInstanceDeclaration currentFeature = (IObjectFeatureInstanceDeclaration)it.next();

			if (currentFeature instanceof SpatialMapObjectFeatureInstDeclaration)
				return (SpatialMapObjectFeatureInstDeclaration)currentFeature;
		}

		return null;
	}

	public static ISpatialMapModel getSpatialMapModel(IWorld world) throws SimulationRuntimeException {
		if (_spatialMapHash.containsKey(world)) {
			return (ISpatialMapModel)_spatialMapHash.get(world);

		} else {
			ISpatialMapModel spatialMapModel = getSpatialMapModelIntern(world);

			if (spatialMapModel == null)
				throw new SimulationRuntimeException("World has no map.");

			_spatialMapHash.put(world, spatialMapModel);

			return spatialMapModel;
		}
	}

	private static ISpatialMapModel getSpatialMapModelIntern(IWorld world) {
		for (int i = 0; i < world.getObjectFeaturesCount(); i++) {
			IObjectFeature currIObjectFeature = world.getObjectFeatureAt(i);

			if (currIObjectFeature instanceof ISpatialMapObjectFeature) {
				return ((ISpatialMapObjectFeature)currIObjectFeature).getSpatialMapModel();
			}
		}

		return null;
	}

	public static boolean hasSpatialMapBuildinFeatureInstanceDeclaration(IWorldInstanceDeclaration worldInstanceDeclaration) {
		Iterator it = worldInstanceDeclaration.getObjectFeatureInstanceDeclarationList().iterator();

		while (it.hasNext()) {
			IObjectFeatureInstanceDeclaration currentFeature = (IObjectFeatureInstanceDeclaration)it.next();

			if (currentFeature instanceof SpatialMapObjectFeatureInstDeclaration)
				return true;
		}

		return false;
	}

	public static boolean hasSpatialMapModel(IWorld world) {
		return _spatialMapHash.containsKey(world) || (getSpatialMapModelIntern(world) != null);
	}

	private SpatialMapFeatureClassDeclaration() {
		this("SpatialMap");
	}

	private SpatialMapFeatureClassDeclaration(String name) {
		super(name);

		addFunctions();
	}

	private void addFunctions() {
		FunctionDeclarationSpatialFactory.insertMapFeatureFunctions(getFunctionDeclarationList());
	}

	public boolean canAddFeatureClassDeclarationTo(IObjectClassDeclaration objectClassDeclaration) {
		return objectClassDeclaration instanceof IWorldClassDeclaration;
	}

	public IObjectFeatureInstanceDeclaration createObjectFeatureInstanceDeclaration(IObjectInstanceDeclaration objectInstanceDeclaration) {
		if (objectInstanceDeclaration instanceof IWorldInstanceDeclaration)
			//			return new DiscretRectangularSpatialMapBuildinObjectFeatureInstanceDeclaration(
			//				this,
			//				(IWorldInstanceDeclaration)objectInstanceDeclaration);
			return new SpatialMapObjectFeatureInstDeclaration(
				this,
				(IWorldInstanceDeclaration)objectInstanceDeclaration);
		else
			return null;
	}

	public boolean isDefaultFeatureClassDeclarationFor(IObjectClassDeclaration objectClassDeclaration) {
		return objectClassDeclaration instanceof IWorldClassDeclaration;
	}
}