package de.uniwue.ki.mas.basics.feature.spatial;

import java.awt.Image;
import java.io.File;

import junit.framework.Assert;

/**
 * Creation date: 09.09.2002, 16:54:11
 * @author oechslein
 */
public class SpatialMapModelChangeEvent extends SpatialMapModelEvent {

	public static final int POS_CHANGED_TYPE = 0;
	public static final int RADIUS_CHANGED_TYPE = 1;
	public static final int DIRECTION_CHANGED_TYPE = 2;
	public static final int SPEED_CHANGED_TYPE = 3;
	public static final int IMAGE_CHANGED_TYPE = 4;
	public static final int MAP_MODEL_CHANGED = 5;
	public static final int DRAWPRIORITY_CHANGED_TYPE = 6;
	
	private final int _changedType;
	private final Image _oldImage;
	private final File _oldImagePath;
	private final double _oldPosX;
	private final double _oldPosY;
	private final double _oldValue;

	public SpatialMapModelChangeEvent(
		ISpatialMapModel spatialMapModel) {
		super(spatialMapModel);
		_oldPosX = -1;
		_oldPosY = -1;
		_oldValue = -1;
		_oldImage = null;
		_oldImagePath = null;

		_changedType = MAP_MODEL_CHANGED;
	}

	public SpatialMapModelChangeEvent(
		ISpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		Image oldImage, File oldImagePath) {
		super(spatialMapModel, spatialInfo);

		_oldPosX = -1;
		_oldPosY = -1;
		_oldValue = -1;
		_oldImage = oldImage;
		_oldImagePath = oldImagePath;

		_changedType = IMAGE_CHANGED_TYPE;
	}
	public SpatialMapModelChangeEvent(
		ISpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		double oldPosX,
		double oldPosY) {
		super(spatialMapModel, spatialInfo);

		_oldPosX = oldPosX;
		_oldPosY = oldPosY;
		_oldValue = -1;
		_oldImage = null;
		_oldImagePath = null;

		_changedType = POS_CHANGED_TYPE;
	}

	public SpatialMapModelChangeEvent(
		ISpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		double oldValue,
		int changedType) {
		super(spatialMapModel, spatialInfo);

		_oldPosX = -1;
		_oldPosY = -1;
		_oldValue = oldValue;
		_oldImage = null;
		_oldImagePath = null;

		Assert.assertTrue(changedType != POS_CHANGED_TYPE);
		Assert.assertTrue(changedType != IMAGE_CHANGED_TYPE);

		_changedType = changedType;
	}

	public int getChangedType() {
		return _changedType;
	}

	public final Image getOldImage() {
		return _oldImage;
	}

	public final double getOldPosX() {
		return _oldPosX;
	}

	public final double getOldPosY() {
		return _oldPosY;
	}

	public final double getOldValue() {
		return _oldValue;
	}
	
	public final File getOldImagePath(){
		return _oldImagePath;
	}
}
