package de.uniwue.ki.mas.basics.feature.spatial;

import java.awt.Image;
import java.io.File;
import java.util.*;

import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.Position2D;

/**
 * Creation date: 09.09.2002, 16:17:22
 * @author oechslein
 */
public final class SpatialMapModelListenerHelper implements IRemoveable {
	private final transient List _spatialMapModelListenerList = new LinkedList();

	public synchronized void addSpatialMapModelListener(ISpatialMapModelListener spatialMapModelListener) {
		_spatialMapModelListenerList.add(0, spatialMapModelListener);
	}

	public synchronized void fireSpatialInfoAdded(ISpatialMapModel spatialMapModel, ISpatialInfo spatialInfo) {
		if (hasListeners()) {
			SpatialMapModelEvent spatialMapModelEvent = new SpatialMapModelEvent(spatialMapModel, spatialInfo);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfoAdded(spatialMapModelEvent);
			}
		}
	}

	public synchronized void fireSpatialInfoChanged(
		ISpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		double oldValue,
		int changedType) {
		if (hasListeners()) {
			SpatialMapModelChangeEvent spatialMapModelEvent =
				new SpatialMapModelChangeEvent(spatialMapModel, spatialInfo, oldValue, changedType);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfoChanged(spatialMapModelEvent);
			}
		}
	}

	public synchronized void fireSpatialInfoChanged(
		AbstractSpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		Image oldImage,
		File oldImagePath) {
		if (hasListeners()) {
			SpatialMapModelChangeEvent spatialMapModelEvent =
				new SpatialMapModelChangeEvent(spatialMapModel, spatialInfo, oldImage, oldImagePath);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfoChanged(spatialMapModelEvent);
			}
		}
	}

	public synchronized void fireSpatialInfoChanged(
		ISpatialMapModel spatialMapModel,
		ISpatialInfo spatialInfo,
		double oldPosX,
		double oldPosY) {

		if (hasListeners()) {
			SpatialMapModelChangeEvent spatialMapModelEvent =
				new SpatialMapModelChangeEvent(spatialMapModel, spatialInfo, oldPosX, oldPosY);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfoChanged(spatialMapModelEvent);
			}
		}
	}

	public synchronized void fireSpatialInfoRemoved(ISpatialMapModel spatialMapModel, ISpatialInfo spatialInfo) {
		if (hasListeners()) {
			SpatialMapModelEvent spatialMapModelEvent = new SpatialMapModelEvent(spatialMapModel, spatialInfo);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfoRemoved(spatialMapModelEvent);
			}
		}
	}
	
	public synchronized void fireSpatialInfosMoved(ISpatialInfo[] movedSpatialInfos, Position2D[] oldPositions){
		if (hasListeners()) {
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener)it.next();
				spatialMapModelListener.spatialInfosMoved(movedSpatialInfos, oldPositions);
			}
		}		
		 
	}

	private boolean hasListeners() {
		return (_spatialMapModelListenerList != null) && !_spatialMapModelListenerList.isEmpty();
	}

	public synchronized void remove() {
		_spatialMapModelListenerList.clear();
	}

	public synchronized void removeSpatialMapModelListener(ISpatialMapModelListener spatialMapModelListener) {
		_spatialMapModelListenerList.remove(spatialMapModelListener);
	}

	public synchronized void fireSpatialMapModelChanged(ISpatialMapModel spatialMapModel) {
		if (hasListeners()) {
			SpatialMapModelChangeEvent spatialMapModelEvent = new SpatialMapModelChangeEvent(spatialMapModel);
			Iterator it = _spatialMapModelListenerList.iterator();
			while (it.hasNext()) {
				ISpatialMapModelListener spatialMapModelListener = (ISpatialMapModelListener) it.next();
				spatialMapModelListener.spatialMapModelChanged(spatialMapModelEvent);
            }
		}
	}
}
