package de.uniwue.ki.mas.basics.feature.spatial;

import java.io.Serializable;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.AbstractObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeature;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.util.declaration.DeclarationListEvent;
import de.uniwue.ki.mas.util.declaration.DeclarationListListener;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.copy.ICopyable;

public class SpatialMapObjectFeatureInstDeclaration extends AbstractObjectFeatureInstanceDeclaration {

	private final class SpatialMapModelListener implements ISpatialMapModelListener, Serializable {
		private final IWorldInstanceDeclaration _worldInstanceDeclaration;

		private SpatialMapModelListener(IWorldInstanceDeclaration worldInstanceDeclaration) {
			super();

			_worldInstanceDeclaration = worldInstanceDeclaration;
		}

		public void spatialInfoAdded(SpatialMapModelEvent spatialMapModelEvent) {
			if (!_inAddEvent) {
				try {
					_inAddEvent = true;
					IObjectInstanceDeclaration objectInstanceDeclaration =
						(IObjectInstanceDeclaration)spatialMapModelEvent.getSpatialInfo().getObject();

					_worldInstanceDeclaration.getObjectInstanceDeclarationList().add(objectInstanceDeclaration);

					SpatialInfoObjectFeatureInstDeclaration spatialInfoFeature =
						SpatialInfoObjectFeatureHelper.getSpatialInfoFeature(objectInstanceDeclaration);
					if (spatialInfoFeature != null) 
						spatialInfoFeature.setSpatialMapModel(getSpatialMapModel());
				} finally {
					_inAddEvent = false;
				}
			}
		}

		public void spatialInfoRemoved(SpatialMapModelEvent spatialMapModelEvent) {
			if (!_inRemoveEvent) {
				try {
					_inRemoveEvent = true;

					IObjectInstanceDeclaration simObject =
						(IObjectInstanceDeclaration)spatialMapModelEvent.getSpatialInfo().getObject();

					//					SpatialInfoObjectFeatureInstDeclaration spatialInfoFeature = null;
					//					spatialInfoFeature = getSpatialInfoObjectFeatureInstanceDeclaration(simObject);
					//					Assert.assertNotNull(spatialInfoFeature);

					_worldInstanceDeclaration.getObjectInstanceDeclarationList().remove(simObject);
					//					spatialMapModelEvent.getSpatialMapModel().removeSpatialInfo(spatialMapModelEvent.getSpatialInfo());

					//spatialInfoFeature.setSpatialMapModel(null);

				} finally {
					_inRemoveEvent = false;
				}
			}

		}

		public void spatialInfoChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
			fireStateChanged();
		}

		public void spatialMapModelChanged(SpatialMapModelChangeEvent spatialMapModelEvent) {
			fireStateChanged();
		}
		public void spatialInfosMoved(ISpatialInfo[] movedSpatialInfos, Position2D[] oldPosition) {
			fireStateChanged();

		}

	}

	private final class ObjectClassDeclListener implements DeclarationListListener, Serializable {
		private final ISpatialMapModel _spatialMapModel;

		private ObjectClassDeclListener(ISpatialMapModel spatialMapModel) {
			super();
			_spatialMapModel = spatialMapModel;
		}

		public void declarationAdded(DeclarationListEvent event) {
			if (!_inAddEvent) {
				try {
					_inAddEvent = true;

					IObjectInstanceDeclaration objectInstanceDeclaration =
						(IObjectInstanceDeclaration)event.getDeclaration();
					SpatialInfoObjectFeatureInstDeclaration spatialInfoFeature =
						SpatialInfoObjectFeatureHelper.getSpatialInfoFeature(objectInstanceDeclaration);
					if (spatialInfoFeature != null) {
						_spatialMapModel.addSpatialInfo(spatialInfoFeature.getSpatialInfo());
						spatialInfoFeature.setSpatialMapModel(getSpatialMapModel());
					}

				} finally {
					_inAddEvent = false;
				}
			}
		}

		public void declarationRemoved(DeclarationListEvent event) {
			if (!_inRemoveEvent) {
				try {
					_inRemoveEvent = true;

					IObjectInstanceDeclaration objectInstanceDeclaration =
						(IObjectInstanceDeclaration)event.getDeclaration();
					SpatialInfoObjectFeatureInstDeclaration spatialInfoFeature =
						SpatialInfoObjectFeatureHelper.getSpatialInfoFeature(objectInstanceDeclaration);
					if (spatialInfoFeature != null) {
						_spatialMapModel.removeSpatialInfo(spatialInfoFeature.getSpatialInfo());
						spatialInfoFeature.setSpatialMapModel(null);
					}

				} finally {
					_inRemoveEvent = false;
				}
			}
		}
	}

	private final ISpatialMapModel _spatialMapModel;
	private final IWorldInstanceDeclaration _worldInstanceDeclaration;

	private boolean _inAddEvent = false;
	private boolean _inRemoveEvent = false;

	private final ObjectClassDeclListener _objectDeclListListener;

	private final ISpatialMapModelListener _spatialMapModelListener;

	public SpatialMapObjectFeatureInstDeclaration(
		SpatialMapFeatureClassDeclaration spatialMapBuildinFeatureClassDeclaration,
		IWorldInstanceDeclaration worldInstanceDeclaration) {

		super(spatialMapBuildinFeatureClassDeclaration);

		_spatialMapModel = new DefaultSpatialMapModel();

		_worldInstanceDeclaration = worldInstanceDeclaration;

		_spatialMapModelListener = new SpatialMapModelListener(_worldInstanceDeclaration);
		_spatialMapModel.addSpatialMapModelListener(_spatialMapModelListener);

		_objectDeclListListener = new ObjectClassDeclListener(_spatialMapModel);
		_worldInstanceDeclaration.getObjectInstanceDeclarationList().addDeclarationListListener(
			_objectDeclListListener);
	}
	
	
	public SpatialMapObjectFeatureInstDeclaration(
			SpatialMapFeatureClassDeclaration spatialMapBuildinFeatureClassDeclaration,
			IWorldInstanceDeclaration worldInstanceDeclaration,
			ISpatialMapModel newSpatialMapModel) {
			
				super(spatialMapBuildinFeatureClassDeclaration);

				_spatialMapModel = newSpatialMapModel;

				_worldInstanceDeclaration = worldInstanceDeclaration;

				_spatialMapModelListener = new SpatialMapModelListener(_worldInstanceDeclaration);
				_spatialMapModel.addSpatialMapModelListener(_spatialMapModelListener);

				_objectDeclListListener = new ObjectClassDeclListener(_spatialMapModel);
				_worldInstanceDeclaration.getObjectInstanceDeclarationList().addDeclarationListListener(
					_objectDeclListListener);
			
			}
 

	public IObjectFeature createObjectFeature(IObjectCreateFunctionArgs objectCreateFunctionArgs) {
		if (objectCreateFunctionArgs.getObject() instanceof IWorld) {
			ISpatialMapModel spatialMapModel = new DefaultSpatialMapModel();
			spatialMapModel.setWidth(getSpatialMapModel().getWidth());
			spatialMapModel.setHeight(getSpatialMapModel().getHeight());
			spatialMapModel.setTorus(getSpatialMapModel().isTorus());
			spatialMapModel.setDiscrete(getSpatialMapModel().isDiscrete());
			spatialMapModel.setDrawGrid(getSpatialMapModel().isDrawGrid());
			spatialMapModel.setRotateImages(getSpatialMapModel().isRotateImages());
			spatialMapModel.setBackgroundImageFile(getSpatialMapModel().getBackgroundImageFile());
			spatialMapModel.setGridCols(getSpatialMapModel().getGridCols());
			spatialMapModel.setGridRows(getSpatialMapModel().getGridRows());

			return new SpatialMapObjectFeature(spatialMapModel, (IWorld)objectCreateFunctionArgs.getObject());
		} else {
			return null;
		}
	}

	public ICopyable copy(Map objectMapping) {
		Assert.assertNotNull(objectMapping);

		IWorldInstanceDeclaration newWorldInstanceDeclaration =
			(IWorldInstanceDeclaration)objectMapping.get(_worldInstanceDeclaration);

		Assert.assertNotNull(newWorldInstanceDeclaration);

		Assert.assertTrue(newWorldInstanceDeclaration != _worldInstanceDeclaration);

		SpatialMapObjectFeatureInstDeclaration spatialMapBuildinObjectFeatureInstanceDeclaration =
			new SpatialMapObjectFeatureInstDeclaration(
				(SpatialMapFeatureClassDeclaration)getObjectFeatureClassDeclaration(),
				newWorldInstanceDeclaration,
				(ISpatialMapModel)getSpatialMapModel().copy(objectMapping));

		copyInto(spatialMapBuildinObjectFeatureInstanceDeclaration, objectMapping);

		return spatialMapBuildinObjectFeatureInstanceDeclaration;
	}



	public void remove() {
		_spatialMapModel.removeSpatialMapModelListener(_spatialMapModelListener);

		_worldInstanceDeclaration.getObjectInstanceDeclarationList().removeDeclarationListListener(
			_objectDeclListListener);

		_spatialMapModel.remove();

		super.remove();
	}

	public final ISpatialMapModel getSpatialMapModel() {
		return _spatialMapModel;
	}


	protected void fireStateChanged() {
		super.fireStateChanged();
	}
}
