package de.uniwue.ki.mas.basics.feature.spatial.function;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

abstract class AbstractChangeFunction extends AbstractFunctionAction {
	private final IFunction _spatialFunction;
	private final IFunction _changeFunction;

	public AbstractChangeFunction(IFunction spatialFunction, IFunction changeFunction) {
		super();

		_spatialFunction = spatialFunction;
		_changeFunction = changeFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs) executeFunctionArgs;
		ISpatialMapModel mapModel =
			SpatialMapFeatureClassDeclaration.getSpatialMapModel(worldExecuteFunctionArgs.getWorld());
		ISpatialInfo spatialInfo = getSpatialInfo(worldExecuteFunctionArgs, _spatialFunction, this);

		changeSpatialInfo(mapModel, spatialInfo, _changeFunction.executeForDouble(executeFunctionArgs));
	}

	static ISpatialInfo getSpatialInfo(
		IWorldExecuteFunctionArgs worldExecuteFunctionArgs,
		IFunction spatialInfoFunction,
		IFunction function)
		throws SimulationRuntimeException {

		if (spatialInfoFunction == null) {
			return SpatialInfoObjectFeatureHelper.getSpatialInfo(
				worldExecuteFunctionArgs.getObject());

		} else {
			ISpatialInfo spatialInfo = (ISpatialInfo) spatialInfoFunction.execute(worldExecuteFunctionArgs);

			if (spatialInfo == null) {
				throw new SimulationRuntimeException("Spatial Info is null in " + function.getName());
			}

			return spatialInfo;
		}
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		appendFunctionName(buff, currDepth, maxDepth, getFunctionName(), _spatialFunction, _changeFunction);
	}

	static void appendFunctionName(
		StringBuffer buff,
		int currDepth,
		int maxDepth,
		String functionName,
		IFunction spatialFunction,
		IFunction changeFunction) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {

			buff.append(functionName);
			if (spatialFunction != null)
				buff.append("Of");

			buff.append("(");
			if (spatialFunction != null) {
				spatialFunction.appendName(buff, currDepth + 1, maxDepth);
				buff.append(", ");
			}
			changeFunction.appendName(buff, currDepth + 1, maxDepth);
			buff.append(")");
		}
	}

	protected abstract void changeSpatialInfo(
		ISpatialMapModel mapModel,
		ISpatialInfo spatialInfo,
		double change);

	protected abstract String getFunctionName();
}