package de.uniwue.ki.mas.basics.feature.spatial.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.spatial.SpatialInfoType;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (08.01.2002 15:27:57)
 * @author Christoph Oechslein
 */

abstract class AbstractHasSpatialFeatureSpecificFunctionDeclaration
	extends AbstractSpecificFunctionArgsDeclaration {
	private final boolean _hasSpatialFeature;

	public AbstractHasSpatialFeatureSpecificFunctionDeclaration(
		boolean hasSpatialFeature,
		String name,
		ISpecificType outputType,
		Iterator inputTypes) {

		super(
			name,
			outputType,
			CollectionsHelper.asLinkedList(
				CollectionsHelper.createConsIterator(SpatialInfoType.getInstance(), inputTypes)));

		_hasSpatialFeature = hasSpatialFeature;
		
		setExpertFunctionDeclaration(!hasSpatialFeature);
	}

	public AbstractHasSpatialFeatureSpecificFunctionDeclaration(
		boolean hasSpatialFeature,
		String name,
		ISpecificType outputType,
		Iterator inputTypes,
		boolean constant) {

		super(
			name,
			outputType,
			CollectionsHelper.asLinkedList(
				CollectionsHelper.createConsIterator(SpatialInfoType.getInstance(), inputTypes)),
			constant);

		_hasSpatialFeature = hasSpatialFeature;
		
		setExpertFunctionDeclaration(!hasSpatialFeature);
	}

	public boolean equals(Object anObject) {
		if ((anObject == null) || (anObject.getClass() != getClass()))
			return false;
		else
			return true;
	}

	public int hashCode() {
		return getClass().hashCode();
	}

	protected final boolean isHasSpatialFeature() {
		return _hasSpatialFeature;
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		if (index != 0) {
			return super.createDefaultFunctionCallInternDeclarationAt(index);

		} else if (!isHasSpatialFeature()) {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				FunctionDeclarationSpatialFactory.GET_SPATIAL_INFO_FUN_DECL);

		} else {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				FunctionDeclarationSpatialFactory.GET_MY_SPATIAL_FUNCTION_DECL);
		}
	}
}