package de.uniwue.ki.mas.basics.feature.spatial.function;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class ChangeImageFunction extends AbstractFunctionAction {
	private final IFunction _spatialFunction;
	private final IFunction _imageFunction;

	public ChangeImageFunction(IFunction spatialFunction, IFunction imageFunction) {
		super();

		_spatialFunction = spatialFunction;
		_imageFunction = imageFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs) executeFunctionArgs;
		ISpatialMapModel mapModel =
			SpatialMapFeatureClassDeclaration.getSpatialMapModel(worldExecuteFunctionArgs.getWorld());
		ISpatialInfo spatialInfo =
			AbstractChangeFunction.getSpatialInfo(worldExecuteFunctionArgs, _spatialFunction, this);

		Object obj = _imageFunction.execute(executeFunctionArgs);

		Image newImage = null;

		if (obj instanceof String)
			newImage = Toolkit.getDefaultToolkit().getImage((String) obj);
		else if (obj instanceof File)
			newImage = Toolkit.getDefaultToolkit().getImage(((File) obj).toString());
		else
			newImage = (Image) obj;

		mapModel.changeImage(spatialInfo, newImage);
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		AbstractChangeFunction.appendFunctionName(
			buff,
			currDepth,
			maxDepth,
			"ChangeImage",
			_spatialFunction,
			_imageFunction);
	}
}