package de.uniwue.ki.mas.basics.feature.spatial.function;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.*;
import java.util.List;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class CreateOverlayImageFunction extends AbstractFunction {

	private final IFunctionList _imageFunctions;

	public CreateOverlayImageFunction(IFunctionList imageFunctions) {
		super();

		_imageFunctions = imageFunctions;
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth > maxDepth) {
			buff.append("...");

		} else {
			buff.append("CreateOverlayImage");
			_imageFunctions.appendName(buff, currDepth, maxDepth);
		}
	}

	public String getName() {
		return getNameIntern();
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		List imageList = new ArrayList(_imageFunctions.size());

		Iterator functions = _imageFunctions.iterator();
		while (functions.hasNext()) {
/* added (changed) by Jrg Freudenstein, Aachen, 13.11.03 */
			Object obj = ((IFunction)functions.next()).execute(executeFunctionArgs);
			Image newImage = null;

			if (obj instanceof String)
				newImage = Toolkit.getDefaultToolkit().getImage((String) obj);
			else if (obj instanceof File)
				newImage = Toolkit.getDefaultToolkit().getImage(((File) obj).toString());
			else
				newImage = (Image) obj;
    
			imageList.add(newImage);
/* end added */
		}

		return createBufferedImage(imageList);
	}

	private BufferedImage createBufferedImage(List imageList) {
		BufferedImage image = createBufferedImage1(imageList);

		if (image == null) {
			synchronized (this) {
				do {
					try {
						wait(10);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}

					image = createBufferedImage1(imageList);
				} while (image == null);
			}
		}

		return image;
	}

	private BufferedImage createBufferedImage1(List images) {
		int width = -1;
		Iterator it = images.iterator();
		while (it.hasNext()) {
			Image image = (Image)it.next();
			int currWidth = image.getWidth(null);
			if (currWidth == -1)
				return null;
			else
				width = Math.max(width, currWidth);
		}

		int height = -1;
		it = images.iterator();
		while (it.hasNext()) {
			Image image = (Image)it.next();
			int currHeight = image.getHeight(null);
			if (currHeight == -1)
				return null;
			else
				height = Math.max(height, currHeight);
		}

		/* changed _RGB to _ARGB in next line to support transparency. Jrg Freudenstein 13.11.03 */
		BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);

		Graphics2D graphics2D = bufferedImage.createGraphics();

		it = images.iterator();
		while (it.hasNext()) {
			Image image = (Image)it.next();
			if (!graphics2D.drawImage(image, 0, 0, width, height, null)) {
				bufferedImage.flush();
				return null;
			}
		}

		return bufferedImage;
	}
}
