package de.uniwue.ki.mas.basics.feature.spatial.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;
import de.uniwue.ki.mas.basics.feature.spatial.SpatialMapFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.collection.IteratorWithException;

/**
 * Creation date: (30.07.2001 14:25:06)
 * @author Christoph Oechslein
 */

final class GetObjectsInDirectionFunction extends AbstractFunction {
	private final IFunction _posFunction;
	private final IFunction _directionFunction;
	private final IFunction _radiusFunction;
	private final IFunction _angleFunction;
	private final IFunction _includingMeFunction;

	public GetObjectsInDirectionFunction(
		IFunction posFunction,
		IFunction directionFunction,
		IFunction angleFunction,
		IFunction radiusFunction,
		IFunction includingMeFunction) {
		super();

		_posFunction = posFunction;
		_directionFunction = directionFunction;
		_angleFunction = angleFunction;
		_radiusFunction = radiusFunction;
		_includingMeFunction = includingMeFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs) executeFunctionArgs;

		IWorld world = worldExecuteFunctionArgs.getWorld();
		ISpatialMapModel mapModel = getMapModel(world);

		final IAgent observingAgent = (IAgent) worldExecuteFunctionArgs.getObject();

		Position2D pos = (Position2D) _posFunction.execute(executeFunctionArgs);
		double direction = Math.toRadians(_directionFunction.executeForDouble(executeFunctionArgs));
		double angle = Math.toRadians(_angleFunction.executeForDouble(executeFunctionArgs));
		double radius = _radiusFunction.executeForDouble(executeFunctionArgs);
		boolean includingMe = _includingMeFunction.executeForBoolean(executeFunctionArgs);

		Iterator spatialInfos = mapModel.getSpatialInfos(pos, radius, direction - angle, direction + angle);

		try {
			IteratorWithException allObjects =
				CollectionsHelper.mapWithException(CollectionsHelper.asIteratorWithException(spatialInfos), new SpatialInfoToObjectMappingFunction(observingAgent));

			if (includingMe) {
				return allObjects;

			} else {
				return CollectionsHelper.removeWithException(allObjects, observingAgent);
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	private ISpatialMapModel getMapModel(IWorld world) throws SimulationRuntimeException {
		return SpatialMapFeatureClassDeclaration.getSpatialMapModel(world);
	}

	public String getName() {
		return "ObserveObjectsInDirection("
			+ _posFunction.getName()
			+ ", "
			+ _directionFunction.getName()
			+ ", "
			+ _radiusFunction.getName()
			+ ", "
			+ _angleFunction.getName()
			+ ", "
			+ _includingMeFunction.getName()
			+ ")";
	}
}