package de.uniwue.ki.mas.basics.feature.spatial.function;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.spatial.position.Position2DType;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (17.09.2001 14:16:48)
 * @author Christoph Oechslein
 */

final class GetObjectsInDirectionSpecificFunctionDeclaration
	extends AbstractSpecificFunctionArgsDeclaration {

	private final boolean _hasSpatialFeature;

	public GetObjectsInDirectionSpecificFunctionDeclaration(boolean hasSpatialFeature) {
		super(
			"ObserveObjectsInDirection",
			GenericIteratorType.getInstance().createSpecificIteratorType(GenericSimObjectType.getGeneralSimObjectType()),
			CollectionsHelper.createList(
				Position2DType.getInstance(),
				DoubleType.getInstance(),
				DoubleType.getInstance(),
				DoubleType.getInstance(),
				BooleanType.getInstance()),
			false);

		_hasSpatialFeature = hasSpatialFeature;
		setDocumentation("Returns all objects within radius (Arg4) around position (Arg1) in direction (Arg2) with an angle (Arg3). According to Arg5 the current acting agent is included or not.");
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		switch (index) {
			case 0 :
				if (_hasSpatialFeature)
					return FunctionCallDeclarationFactory
						.getInstance()
						.createComposedFunctionCallInternDeclaration(
						null,
						FunctionDeclarationSpatialFactory.GET_MY_POSITION_FUNCTION_DECL);
				else
					return super.createDefaultFunctionCallInternDeclarationAt(0);

			case 1 :
				if (_hasSpatialFeature)
					return FunctionCallDeclarationFactory
						.getInstance()
						.createComposedFunctionCallInternDeclaration(
						null,
						FunctionDeclarationSpatialFactory
							.GET_DIRECTION_IN_DEGREE_FUNCTION_DECL_HAS_SPATIAL_INFO);
				else
					return FunctionCallDeclarationFactory
						.getInstance()
						.createComposedFunctionCallInternDeclaration(
						null,
						FunctionDeclarationSpatialFactory
							.GET_DIRECTION_IN_DEGREE_FUNCTION_DECL_HAS_NO_SPATIAL_INFO);
			case 2 :
				return FunctionCallDeclarationFactory
					.getInstance()
					.createAtomicFunctionCallInternDeclaration(
					null,
					DoubleType.getInstance(),
					new Double(90));

			case 3 :
				return FunctionCallDeclarationFactory
					.getInstance()
					.createAtomicFunctionCallInternDeclaration(
					null,
					DoubleType.getInstance(),
					new Double(100));

			default :
				Assert.assertEquals(4, index);

				return FunctionCallDeclarationFactory
					.getInstance()
					.createAtomicFunctionCallInternDeclaration(
					null,
					BooleanType.getInstance(),
					Boolean.FALSE);
		}
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		Assert.assertEquals(5, functions.size());

		return new GetObjectsInDirectionFunction(
			functions.getFunction(0),
			functions.getFunction(1),
			functions.getFunction(2),
			functions.getFunction(3),
			functions.getFunction(4));
	}

	public String getSpecificInputTypeNameAt(int index) {
		switch (index) {
			case 0 :
				return "Pos";
			case 1 :
				return "Direction";
			case 2 :
				return "Angle";
			case 3 :
				return "Radius";
			case 4 :
				return "Including me";
			default :
				return super.getSpecificInputTypeNameAt(index);
		}
	}

	public boolean equals(Object anObject) {
		if ((anObject == null) || (anObject.getClass() != getClass()))
			return false;
		else
			return true;
	}

	public int hashCode() {
		return getClass().hashCode();
	}
}