package de.uniwue.ki.mas.basics.feature.spatial.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.spatial.ISpatialMapModel;
import de.uniwue.ki.mas.basics.feature.spatial.SpatialMapFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.collection.IteratorWithException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class GetObjectsOnPositionFunction extends AbstractFunction {
	private final IFunction _posFunction;
	private final IFunction _radiusFunction;
	private final IFunction _includingMeFunction;

	public GetObjectsOnPositionFunction(
		IFunction posFunction,
		IFunction radiusFunction,
		IFunction includingMeFunction) {
		super();

		_posFunction = posFunction;
		_radiusFunction = radiusFunction;
		_includingMeFunction = includingMeFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs) executeFunctionArgs;
		IWorld world = worldExecuteFunctionArgs.getWorld();

		ISpatialMapModel mapModel = getMapModel(world);

		final IAgent observingAgent = (IAgent) worldExecuteFunctionArgs.getObject();

		Position2D pos = (Position2D) _posFunction.execute(worldExecuteFunctionArgs);
		double radius = _radiusFunction.executeForDouble(worldExecuteFunctionArgs);
		boolean includingMe = _includingMeFunction.executeForBoolean(worldExecuteFunctionArgs);

		Iterator spatialInfos = mapModel.getSpatialInfos(pos, radius);

		try {
			IteratorWithException allSimObjects =
				CollectionsHelper.mapWithException(CollectionsHelper.asIteratorWithException(spatialInfos), new SpatialInfoToObjectMappingFunction(observingAgent));

			if (includingMe) {
				return allSimObjects;
			} else {
				return CollectionsHelper.removeWithException(allSimObjects, observingAgent);
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	private ISpatialMapModel getMapModel(IWorld world) throws SimulationRuntimeException {
		return SpatialMapFeatureClassDeclaration.getSpatialMapModel(world);
	}

	public String getName() {
		return "ObserveObjectsOnPosition("
			+ _posFunction.getName()
			+ ", "
			+ _radiusFunction.getName()
			+ ", "
			+ _includingMeFunction.getName()
			+ ")";
	}
}