package de.uniwue.ki.mas.basics.feature.spatial.function;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.spatial.position.Position2DType;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (08.01.2002 15:28:13)
 * @author Christoph Oechslein
 */

class GetObjectsOnPositionSpecificFunctionDeclaration extends AbstractSpecificFunctionArgsDeclaration {

	private final boolean _hasSpatialFeature;

	public GetObjectsOnPositionSpecificFunctionDeclaration(boolean hasSpatialFeature) {
		super(
			"ObserveObjectsOnPosition",
			GenericIteratorType.getInstance().createSpecificIteratorType(GenericSimObjectType.getGeneralSimObjectType()),
			CollectionsHelper.createList(
				Position2DType.getInstance(),
				DoubleType.getInstance(),
				BooleanType.getInstance()),
			false);

		_hasSpatialFeature = hasSpatialFeature;
		setDocumentation("Returns all objects within radius (Arg2) around position (Arg1). According to Arg3 the current acting agent is included or not.");
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		if (index == 0) {
			if (_hasSpatialFeature)
				return FunctionCallDeclarationFactory
					.getInstance()
					.createComposedFunctionCallInternDeclaration(
					null,
					FunctionDeclarationSpatialFactory.GET_MY_POSITION_FUNCTION_DECL);
			else
				return super.createDefaultFunctionCallInternDeclarationAt(0);

		} else if (index == 1) {
			return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(100));

		} else {
			if (Assert.ASSERTION_ON)
				Assert.assertEquals(2, index);

			return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				BooleanType.getInstance(),
				Boolean.FALSE);
		}
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		Assert.assertEquals(3, functions.size());

		IFunction positionFunction = functions.getFunction(0);
		IFunction radiusFunction = functions.getFunction(1);
		IFunction includingMeFunction = functions.getFunction(2);

		return new GetObjectsOnPositionFunction(positionFunction, radiusFunction, includingMeFunction);
	}

	public String getSpecificInputTypeNameAt(int index) {
		switch (index) {
			case 0 :
				return "Pos";
			case 1 :
				return "Radius";
			case 2 :
				return "Including me";
			default :
				return super.getSpecificInputTypeNameAt(index);
		}
	}

	public boolean equals(Object anObject) {
		if ((anObject == null) || (anObject.getClass() != getClass()))
			return false;
		else
			return true;
	}

	public int hashCode() {
		return getClass().hashCode();
	}
}