package de.uniwue.ki.mas.basics.feature.spatial.function;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.math.MathHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class MoveTowardsPosSpatialInfoFunction extends AbstractFunctionAction {
	private final IFunction _spatialInfoFunction;
	private final IFunction _posFunction;
	private final IFunction _radiusFunction;

	public MoveTowardsPosSpatialInfoFunction(
		IFunction spatialInfoFunction,
		IFunction posFunction,
		IFunction radiusFunction) {
		super();

		_spatialInfoFunction = spatialInfoFunction;
		_posFunction = posFunction;
		_radiusFunction = radiusFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs)executeFunctionArgs;

		ISpatialInfo spatialInfo =
			AbstractChangeFunction.getSpatialInfo(worldExecuteFunctionArgs, _spatialInfoFunction, this);

		ISpatialMapModel mapModel =
			SpatialMapFeatureClassDeclaration.getSpatialMapModel(
				worldExecuteFunctionArgs.getWorld());

		Position2D goalPos = (Position2D)_posFunction.execute(worldExecuteFunctionArgs);
		double radius = _radiusFunction.executeForDouble(worldExecuteFunctionArgs);

		Position2D myPos = spatialInfo.getPosition();

		double distanceToGoalSq = AbstractSpatialMapModel.distanceSqWithTorus(myPos, goalPos, mapModel);

		double mySpeed = spatialInfo.getSpeed();

		if ((distanceToGoalSq <= (radius * radius)) || (distanceToGoalSq < (mySpeed * mySpeed))) {
			// we are in radius and/or can reach position in one step
			mapModel.changePosition(spatialInfo, goalPos);

		} else {
			double myNewDirection = goalPos.angle(myPos);

			if (!MathHelper.doubleEqual(myPos.distanceSq(goalPos), distanceToGoalSq)) {
				myNewDirection = myNewDirection + Math.PI;
			}

			mapModel.changeDirection(spatialInfo, myNewDirection);

			Position2D inbetweenPosition =
				Position2D.addWithDirectionAndLength(myPos, myNewDirection, mySpeed);
				
			mapModel.changePosition(spatialInfo, inbetweenPosition);
		}
	}

	public String getName() {
		return "MoveToPos(" + _posFunction.getName() + ", " + _radiusFunction.getName() + ")";
	}
}