package de.uniwue.ki.mas.basics.feature.spatial.function;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.spatial.position.Position2DType;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.VoidType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (04.09.2001 13:16:02)
 * @author Christoph Oechslein
 */

final class MoveTowardsPosSpatialInfoSpecificFunctionDeclaration
	extends AbstractHasSpatialFeatureSpecificFunctionDeclaration {

	public MoveTowardsPosSpatialInfoSpecificFunctionDeclaration(boolean hasSpatialInfo) {
		super(
			hasSpatialInfo,
			"MoveTowardsPos",
			VoidType.getInstance(),
			CollectionsHelper.createIterator(Position2DType.getInstance(), DoubleType.getInstance()));

		setDocumentation(
			"Moves the given spatial info (Arg1) towards position (Arg2), "
				+ "if the distance between the spatial info and the goal position is lower than the given value (arg3) "
				+ "then BeamTo position.");
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		if (index != getSpecificInputDefaultArgsLength()-1) {
			return super.createDefaultFunctionCallInternDeclarationAt(index);
		} else {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				isHasSpatialFeature()
					? FunctionDeclarationSpatialFactory.GET_SPEED_FUNCTION_DECL_HAS_SPATIAL_INFO
					: FunctionDeclarationSpatialFactory.GET_SPEED_FUNCTION_DECL_HAS_NO_SPATIAL_INFO);
		}
	}
	
	public String getSpecificInputTypeNameAt(int index) {
		if (index==0) return "object";
		if (index==1) return "targetposition";
		if (index==2) return "radius";
		
		return super.getSpecificInputTypeNameAt(index);
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(3, functions.size());

		return new MoveTowardsPosSpatialInfoFunction(
			functions.getFunction(0),
			functions.getFunction(1),
			functions.getFunction(2));
	}
}