package de.uniwue.ki.mas.basics.feature.spatial.position;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (05.09.2001 10:46:37)
 * @author Christoph Oechslein
 */

final class PosCreateSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	private static final String NAME = "CreatePos";

	public PosCreateSpecificFunctionArgsDeclaration() {
		super(
			NAME,
			Position2DType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()),
			true,
			false);

		setDocumentation("Creates a new position, the x-value given by (Arg1), the y-value given by (Arg2)");
	}

	/**
	 * @param createFunctionArgs ICreateFunctionArgs
	 * @param functions IFunctionList
	 * @return IFunction
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration#createFunction(ICreateFunctionArgs, IFunctionList)
	 */
	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, final IFunctionList functions) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());
			
		return new AbstractFunction() {
			private final IFunction _numberFunction1 = functions.getFunction(0);
			private final IFunction _numberFunction2 = functions.getFunction(1);

			public String getName() {
				return NAME + "(" + _numberFunction1.getName() + ", " + _numberFunction2.getName() + ")";
			}

			public Object execute(IExecuteFunctionArgs executeFunctionArgs)
				throws SimulationRuntimeException {
				double x = _numberFunction1.executeForDouble(executeFunctionArgs);
				double y = _numberFunction2.executeForDouble(executeFunctionArgs);

				return new Position2D(x, y);
			}
		};
	}
}