package de.uniwue.ki.mas.basics.feature.spatial.position;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (05.09.2001 10:46:37)
 * @author Christoph Oechslein
 */

final class PosDivideNumberSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	private static final String NAME = "Pos/Number";

	public PosDivideNumberSpecificFunctionArgsDeclaration() {
		super(
			NAME,
			Position2DType.getInstance(),
			CollectionsHelper.createList(Position2DType.getInstance(), DoubleType.getInstance()),
			true,
			false);

		setDocumentation("Divides every coordinate of position (Arg1) by number (Arg2)");
	}

	/**
	 * @param createFunctionArgs ICreateFunctionArgs
	 * @param functions IFunctionList
	 * @return IFunction
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration#createFunction(ICreateFunctionArgs, IFunctionList)
	 */
	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, final IFunctionList functions) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());
		return new AbstractFunction() {
			private final IFunction _posFunction = functions.getFunction(0);
			private final IFunction _doubleFunction = functions.getFunction(1);
			public String getName() {
				return NAME + "(" + _posFunction.getName() + ", " + _doubleFunction.getName() + ")";
			}

			public Object execute(IExecuteFunctionArgs executeFunctionArgs)
				throws SimulationRuntimeException {
				Position2D point = (Position2D) _posFunction.execute(executeFunctionArgs);
				double a = _doubleFunction.executeForDouble(executeFunctionArgs);
				return new Position2D(point.getX() / a, point.getY() / a);
			}
		};
	}
}