package de.uniwue.ki.mas.basics.feature.spatial.position;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (05.09.2001 10:46:37)
 * @author Christoph Oechslein
 */

final class PosGetLengthSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	private static final String NAME = "PosGetLength";
	public PosGetLengthSpecificFunctionArgsDeclaration() {
		super(
			NAME,
			DoubleType.getInstance(),
			CollectionsHelper.createList(Position2DType.getInstance()),
			true,
			false);

		setDocumentation("Returns the length of its position (Arg1) from position [0.0, 0.0]");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, final IFunctionList functions) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(1, functions.size());
			
		return new AbstractFunction() {
			private final IFunction _posFunction = ((IFunction) functions.get(0));
			public String getName() {
				return NAME + "(" + _posFunction.getName() + ")";
			}

			public Object execute(IExecuteFunctionArgs executeFunctionArgs)
				throws SimulationRuntimeException {
				return new Double(executeForDouble(executeFunctionArgs));
			}

			public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
				throws SimulationRuntimeException {
				Position2D point = (Position2D) _posFunction.execute(executeFunctionArgs);
				return point.distance();
			}
		};
	}
}
