package de.uniwue.ki.mas.basics.feature.spatial.position;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (05.09.2001 10:46:37)
 * @author Christoph Oechslein
 */

final class PosMinusSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
   private static final String NAME = "Pos-";
	public PosMinusSpecificFunctionArgsDeclaration() {
	      super(
	         NAME,
	         Position2DType.getInstance(),
	         CollectionsHelper.createList(Position2DType.getInstance(), Position2DType.getInstance()),
	         true,
	         false);
	         
	      setDocumentation("Subtracts position (Arg2) of position (Arg1)");
	}

/**
 * @param createFunctionArgs ICreateFunctionArgs
 * @param functions IFunctionList
 * @return IFunction
 * @see de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration#createFunction(ICreateFunctionArgs, IFunctionList)
 */
public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, final IFunctionList functions) {
    if (Assert.ASSERTION_ON)
        Assert.assertEquals(2, functions.size());
    return new AbstractFunction() {
        private final IFunction _posFunction1 = functions.getFunction(0);
        private final IFunction _posFunction2 = functions.getFunction(1);
        public String getName() {
            return NAME + "(" + _posFunction1.getName() + ", " + _posFunction2.getName() + ")";
        }

        public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
            Position2D point1 = (Position2D) _posFunction1.execute(executeFunctionArgs);
            Position2D point2 = (Position2D) _posFunction2.execute(executeFunctionArgs);
            return new Position2D(point1.getX() - point2.getX(), point1.getY() - point2.getY());
        }
    };
}
}
