package de.uniwue.ki.mas.basics.feature.spatial.position;
import java.awt.geom.Point2D;
import java.util.Map;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.AbstractSpecificType;
import de.uniwue.ki.mas.basics.type.ObjectType;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.util.Position2D;

public final class Position2DType extends AbstractSpecificType {
	private static final Position2DType instance = new Position2DType();

	private static final class PositionArgument extends AbstractArgument implements IArgument {
		private final Position2D _position2D;

		public PositionArgument(Position2D position2D) {
			super();

			_position2D = position2D;
		}

		public final Object getConstantValue() {
			return _position2D;
		}

		public String getName() {
			return "[" + _position2D.x + ", " + _position2D.y + "]";
		}
	}

	protected Position2DType() {
		super();

		ObjectType.getInstance().addSubType(this);
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		return new Position2D(((Position2D) anObject));
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return new PositionArgument((Position2D) anObject);
	}

	public Object getArgumentDeclarationDefaultValue() {
		return new Position2D(0, 0);
	}

	public static final Position2DType getInstance() {
		return instance;
	}

	public String getName() {
		return "Position";
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			Point2D point = (Point2D) anObject;
			buff.append("[");
			buff.append(point.getX());
			buff.append(" ");
			buff.append(point.getY());
			buff.append("]");
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value instanceof Position2D;
	}
}