package de.uniwue.ki.mas.basics.feature.spatial.position;
import junit.framework.Assert;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.featureList.FeatureClassDeclarationListManager;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.util.Position2D;
/**
 * Creation date: (05.09.2001 10:46:37)
 * @author Christoph Oechslein
 */public final class TestPosition2DFunction extends TestCase {
	private final ICreateAndExecuteFunctionArgs _executeFunArgs = new CreateAndExecuteFunctionArgs();

	private final IFunctionDeclarationList _functionDeclarationList;
	private final IArgumentDeclaration _number0ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			DoubleType.getInstance(),
			new Double(0.0));
	private final IArgumentDeclaration _number1ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			DoubleType.getInstance(),
			new Double(1.0));
	private final IArgumentDeclaration _position2D0_0ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			Position2DType.getInstance(),
			new Position2D(0.0, 0.0));
	private final IArgumentDeclaration _position2D0_1ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			Position2DType.getInstance(),
			new Position2D(0.0, 1.0));
	private final IArgumentDeclaration _position2D1_0ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			Position2DType.getInstance(),
			new Position2D(1.0, 0.0));
	private final IArgumentDeclaration _position2D1_1_minusArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			Position2DType.getInstance(),
			new Position2D(-1.0, -1.0));
	private final IArgumentDeclaration _position2D1_1ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			Position2DType.getInstance(),
			new Position2D(1.0, 1.0));
			
			public TestPosition2DFunction(String name) {
	super(name);
	
	_functionDeclarationList = FeatureClassDeclarationListManager.getInstance().getDefaultFunctionDeclarationList();
}

public void checkFunction(
	ISpecificFunctionDeclaration specificFunctionDeclaration,
	FunctionList functionList,
	String message,
	double expectedValue)
	throws Exception {

	IFunction resultFunction =
		specificFunctionDeclaration.createParametricFunction(_executeFunArgs).createFunction(
			_executeFunArgs,
			functionList);
	if (Assert.ASSERTION_ON)
		Assert.assertEquals(message, expectedValue, resultFunction.executeForDouble(_executeFunArgs), 0.0001);
}

public void checkFunction(
	ISpecificFunctionDeclaration specificFunctionDeclaration,
	FunctionList functionList,
	String message,
	Position2D expectedValue)
	throws Exception {

	Assert.assertNotNull("Didn't found functionDecl ", specificFunctionDeclaration);

IFunction resultFunction =
	specificFunctionDeclaration.createParametricFunction(_executeFunArgs).createFunction(
		_executeFunArgs,
		functionList);

Position2D result = (Position2D)resultFunction.execute(_executeFunArgs);

Assert.assertEquals(message, expectedValue, result);
}/**
	 * @param args String[]
	 */public static void main(String[] args) {
	junit.swingui.TestRunner.run(TestPosition2DFunction.class);
}

public void testPosAbs() throws Exception {
	ISpecificFunctionDeclaration posAbsDecl = _functionDeclarationList.getSpecificFunctionDeclaration("PosAbs", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_0 = _position2D1_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1_minus = _position2D1_1_minusArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posAbsDecl, new FunctionList(position0_0),"abs 0, 0", new Position2D(0, 0));
checkFunction(posAbsDecl, new FunctionList(position0_1),"abs 0, 1", new Position2D(0, 1));
checkFunction(posAbsDecl, new FunctionList(position1_0),"abs 1, 0", new Position2D(1, 0));
checkFunction(posAbsDecl, new FunctionList(position1_1),"abs 1, 1", new Position2D(1, 1));
checkFunction(posAbsDecl, new FunctionList(position1_1_minus),"abs -1, -1", new Position2D(1, 1));
}

public void testPosDistance() throws Exception {
	ISpecificFunctionDeclaration posDistanceDecl = _functionDeclarationList.getSpecificFunctionDeclaration("PosDistance", DoubleType.getInstance());
assertNotNull(posDistanceDecl);
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_0 = _position2D1_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posDistanceDecl, new FunctionList(position0_0, position0_0),"0, 0 <dist> 0, 0", 0);
checkFunction(posDistanceDecl, new FunctionList(position0_0, position0_1),"0, 0 <dist> 0, 1", 1);
checkFunction(posDistanceDecl, new FunctionList(position0_1, position0_0),"0, 1 <dist> 0, 0", 1);
checkFunction(posDistanceDecl, new FunctionList(position1_0, position0_1),"1, 0 <dist> 0, 1", Math.sqrt(1 + 1));
checkFunction(posDistanceDecl, new FunctionList(position0_1, position1_0),"0, 1 <dist> 1, 0", Math.sqrt(1 + 1));
checkFunction(posDistanceDecl, new FunctionList(position1_1, position1_1),"1, 1 <dist> 1, 1", 0);
}

public void testPosDurchNumber() throws Exception {
	ISpecificFunctionDeclaration posDurchDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos/Number", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number1 = _number1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posDurchDecl, new FunctionList(position0_0, number1),"0, 0 / 1", new Position2D(0.0 / 1.0, 0.0 / 1.0));
checkFunction(posDurchDecl, new FunctionList(position0_1, number1),"0, 1 / 1", new Position2D(0.0 / 1.0, 1.0 / 1.0));
checkFunction(posDurchDecl, new FunctionList(position1_1, number1),"1, 1 / 1", new Position2D(1.0 / 1.0, 1.0 / 1.0));
}/**
	 * Insert the method's description here.
	 * <BR>Creation date: (30.03.00 18:14:43)
	 * @throws Exception
	 */public void testPosMalNumber() throws Exception {
	ISpecificFunctionDeclaration posMalDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos*Number", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number0 = _number0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number1 = _number1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posMalDecl, new FunctionList(position0_0, number0),"0, 0 * 0", new Position2D(0.0 * 0.0, 0.0 * 0.0));
checkFunction(posMalDecl, new FunctionList(position0_1, number0),"0, 1 * 0", new Position2D(0.0 * 0.0, 1.0 * 0.0));
checkFunction(posMalDecl, new FunctionList(position1_1, number0),"1, 1 * 0", new Position2D(1.0 * 0.0, 1.0 * 0.0));
checkFunction(posMalDecl, new FunctionList(position0_0, number1),"0, 0 * 1", new Position2D(0.0 * 1.0, 0.0 * 1.0));
checkFunction(posMalDecl, new FunctionList(position0_1, number1),"0, 1 * 1", new Position2D(0.0 * 1.0, 1.0 * 1.0));
checkFunction(posMalDecl, new FunctionList(position1_1, number1),"1, 1 * 1", new Position2D(1.0 * 1.0, 1.0 * 1.0));
}/**
	 * Insert the method's description here.
	 * <BR>Creation date: (30.03.00 18:14:43)
	 * @throws Exception
	 */public void testPosMinus() throws Exception {
	ISpecificFunctionDeclaration posMinusDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos-", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_0 = _position2D1_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posMinusDecl, new FunctionList(position0_0, position0_0),"0, 0 - 0, 0", new Position2D(0.0 - 0.0, 0.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position0_0, position0_1),"0, 0 - 0, 1", new Position2D(0.0 - 0.0, 0.0 - 1.0));
checkFunction(posMinusDecl, new FunctionList(position0_1, position0_0),"0, 1 - 0, 0", new Position2D(0.0 - 0.0, 1.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position1_0, position0_1),"1, 0 - 0, 1", new Position2D(1.0 - 0.0, 0.0 - 1.0));
checkFunction(posMinusDecl, new FunctionList(position0_1, position1_0),"0, 1 - 1, 0", new Position2D(0.0 - 1.0, 1.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position1_1, position1_1),"1, 1 - 1, 1", new Position2D(1.0 - 1.0, 1.0 - 1.0));
}/**
	 * @throws Exception
	 */public void testPosMinusNumber() throws Exception {
	ISpecificFunctionDeclaration posMinusDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos-Number", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number0 = _number0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number1 = _number1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posMinusDecl, new FunctionList(position0_0, number0),"0, 0 - 0", new Position2D(0.0 - 0.0, 0.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position0_1, number0),"0, 1 - 0", new Position2D(0.0 - 0.0, 1.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position1_1, number0),"1, 1 - 0", new Position2D(1.0 - 0.0, 1.0 - 0.0));
checkFunction(posMinusDecl, new FunctionList(position0_0, number1),"0, 0 - 1", new Position2D(0.0 - 1.0, 0.0 - 1.0));
checkFunction(posMinusDecl, new FunctionList(position0_1, number1),"0, 1 - 1", new Position2D(0.0 - 1.0, 1.0 - 1.0));
checkFunction(posMinusDecl, new FunctionList(position1_1, number1),"1, 1 - 1", new Position2D(1.0 - 1.0, 1.0 - 1.0));
}/**
	 * @throws Exception
	 */public void testPosPlus() throws Exception {
	ISpecificFunctionDeclaration posPlusDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos+", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_0 = _position2D1_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posPlusDecl, new FunctionList(position0_0, position0_0),"0, 0 + 0, 0", new Position2D(0 + 0, 0 + 0));
checkFunction(posPlusDecl, new FunctionList(position0_0, position0_1),"0, 0 + 0, 1", new Position2D(0 + 0, 0 + 1));
checkFunction(posPlusDecl, new FunctionList(position0_1, position0_0),"0, 1 + 0, 0", new Position2D(0 + 0, 1 + 0));
checkFunction(posPlusDecl, new FunctionList(position1_0, position0_1),"1, 0 + 0, 1", new Position2D(1 + 0, 0 + 1));
checkFunction(posPlusDecl, new FunctionList(position0_1, position1_0),"0, 1 + 1, 0", new Position2D(0 + 1, 1 + 0));
checkFunction(posPlusDecl, new FunctionList(position1_1, position1_1),"1, 1 + 1, 1", new Position2D(1 + 1, 1 + 1));
}/**
	 * @throws Exception
	 */public void testPosPlusNumber() throws Exception {
	ISpecificFunctionDeclaration posPlusDecl = _functionDeclarationList.getSpecificFunctionDeclaration("Pos+Number", Position2DType.getInstance());
IArgument position0_0 = _position2D0_0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position0_1 = _position2D0_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument position1_1 = _position2D1_1ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number0 = _number0ArgumentDeclaration.createArgument(_executeFunArgs);
IArgument number1 = _number1ArgumentDeclaration.createArgument(_executeFunArgs);
checkFunction(posPlusDecl, new FunctionList(position0_0, number0),"0, 0 + 0", new Position2D(0.0 + 0.0, 0.0 + 0.0));
checkFunction(posPlusDecl, new FunctionList(position0_1, number0),"0, 1 + 0", new Position2D(0.0 + 0.0, 1.0 + 0.0));
checkFunction(posPlusDecl, new FunctionList(position1_1, number0),"1, 1 + 0", new Position2D(1.0 + 0.0, 1.0 + 0.0));
checkFunction(posPlusDecl, new FunctionList(position0_0, number1),"0, 0 + 1", new Position2D(0.0 + 1.0, 0.0 + 1.0));
checkFunction(posPlusDecl, new FunctionList(position0_1, number1),"0, 1 + 1", new Position2D(0.0 + 1.0, 1.0 + 1.0));
checkFunction(posPlusDecl, new FunctionList(position1_1, number1),"1, 1 + 1", new Position2D(1.0 + 1.0, 1.0 + 1.0));
}
}