package de.uniwue.ki.mas.basics.feature.user;
import java.util.Iterator;
import java.util.Map;

import de.uniwue.ki.mas.basics.feature.AbstractFeaturePlusObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionUser.*;
import de.uniwue.ki.mas.entity.variableclass.FeatureVariableClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.variableclass.VariableClassDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: (15.12.2000 13:12:09)
 * @author Christoph Oechslein
 */
class UserFeatureClassDeclaration
	extends AbstractFeaturePlusObjectFeatureClassDeclaration
	implements IUserFeatureClassDeclaration {

	private final UserFunctionDeclarationListAndFactory _userFunctionDeclarationListAndFactory =
		new UserFunctionDeclarationListAndFactory(false);

	private final FeatureVariableClassDeclarationListAndFactory _featureVariableClassDeclarationListAndFactory =
		new FeatureVariableClassDeclarationListAndFactory();

	public UserFeatureClassDeclaration() {
		this("Untitled User Feature");
	}

	/**
	 * @param name String
	 */
	public UserFeatureClassDeclaration(String name) {
		super(name);
	}

	/**
	 * Returns <code>true</code> if <code>recursive</code> is <code>true</code>
	 * or asks removeVetoListeners.
	 * @return boolean
	 * @see de.uniwue.ki.mas.util.declaration.IDeclaration#canBeRemoved()
	 */
	public boolean canBeRemoved() {
		return removingOkForListeners()
			&& getUserFunctionDeclarationList().canBeRemoved()
			&& getVariableClassDeclarationList().canBeRemoved();
	}

	/**
	 * @param inToObject Object
	 * @param objectMapping java.util.Map
	 * @see de.uniwue.ki.util.ICopyable#copyInto(Object, Map)
	 */
	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		UserFeatureClassDeclaration dstUserFeatureClassDeclaration = (UserFeatureClassDeclaration) inToObject;

		getUserFunctionDeclarationList().copyInto(
			dstUserFeatureClassDeclaration.getUserFunctionDeclarationList(),
			objectMapping);
		getVariableClassDeclarationList().copyInto(
			dstUserFeatureClassDeclaration.getVariableClassDeclarationList(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return UserFeatureClassDeclarationFactory.createUserFeatureClassDeclaration();
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration#getDefaultFunctionDeclarations()
	 */
	public Iterator getDefaultFunctionDeclarations() {
		return CollectionsHelper.select(getFunctionDeclarations(), new IPredicateFunction() {
			public boolean isElementOk(Object obj) {
				IUserFunctionDeclaration userFunctionDeclaration = (IUserFunctionDeclaration) obj;
				return userFunctionDeclaration.isExternal();
			}
		});
	}

	/**
	 * @return FeatureVariableClassDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration#getFeatureVariableClassDeclarationListAndFactory()
	 */
	public final FeatureVariableClassDeclarationListAndFactory getFeatureVariableClassDeclarationListAndFactory() {
		return _featureVariableClassDeclarationListAndFactory;
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration#getFunctionDeclarations()
	 */
	public Iterator getFunctionDeclarations() {
		return getUserFunctionDeclarationList().iterator();
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return super.getName();
	}

	private final UserFunctionDeclarationList getUserFunctionDeclarationList() {
		return getUserFunctionDeclarationListAndFactory().getUserFunctionDeclarationList();
	}

	/**
	 * @return UserFunctionDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration#getUserFunctionDeclarationListAndFactory()
	 */
	public final UserFunctionDeclarationListAndFactory getUserFunctionDeclarationListAndFactory() {
		return _userFunctionDeclarationListAndFactory;
	}

	private final VariableClassDeclarationList getVariableClassDeclarationList() {
		return getFeatureVariableClassDeclarationListAndFactory().getVariableClassDeclarationList();
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration#getVariableClassDeclarations()
	 */
	public Iterator getVariableClassDeclarations() {
		return getVariableClassDeclarationList().iterator();
	}

	/**
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		super.remove();

		getUserFunctionDeclarationList().remove();
		getVariableClassDeclarationList().remove();
	}
}