package de.uniwue.ki.mas.basics.featureList;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.DefaultDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * Creation date: (30.07.2001 14:24:00)
 * @author Christoph Oechslein
 */

public class FeatureClassDeclarationList extends DefaultDeclarationList implements IFeatureClassDeclarationList {

	public FeatureClassDeclarationList() {
		super();
	}

	/**
	 * @param declaration Object
	 * @return boolean
	 * @see de.uniwue.ki.util.IMutableListModel#accept(Object)
	 */
	public boolean accept(Object declaration) {
		return declaration instanceof IFeatureClassDeclaration;
	}

	/**
	 * @param objectClassDeclaration IObjectClassDeclaration
	 * @return boolean
	 */
	public boolean canAddFeatureClassDeclarationTo(IObjectClassDeclaration objectClassDeclaration) {
		Iterator it = getFeatureClassDeclarations();
		while (it.hasNext()) {
			IFeatureClassDeclaration featureClassDeclaration = (IFeatureClassDeclaration) it.next();
			if (!featureClassDeclaration.canAddFeatureClassDeclarationTo(objectClassDeclaration)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @param declaration IDeclaration
	 * @return boolean
	 * @see de.uniwue.ki.mas.util.declaration.IDeclarationList#canBeRemoved(IDeclaration)
	 */
	public boolean canBeRemoved(IDeclaration declaration) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(contains(declaration));

		return declaration.canBeRemoved();
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList#getDefaultFunctionDeclarations()
	 */
	public Iterator getDefaultFunctionDeclarations() {
		Iterator iteratorOfIterator =
			CollectionsHelper.map(getFeatureClassDeclarations(), new IMapFunction() {
			public Object map(Object anObject) {
				return ((IFeatureClassDeclaration) anObject).getDefaultFunctionDeclarations();
			}
		});

		return CollectionsHelper.removeDuplicates(CollectionsHelper.flatten(iteratorOfIterator));
	}

	protected final Iterator getFeatureClassDeclarations() {
		return iterator();
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Features";
	}

	public boolean isDefaultFeatureClassDeclarationFor(IObjectClassDeclaration objectClassDeclaration) {
		Iterator it = getFeatureClassDeclarations();
		while (it.hasNext()) {
			IFeatureClassDeclaration featureClassDeclaration = (IFeatureClassDeclaration) it.next();
			if (!featureClassDeclaration.isDefaultFeatureClassDeclarationFor(objectClassDeclaration)) {
				return false;
			}
		}
		return true;
	}
}