package de.uniwue.ki.mas.basics.featureList;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.*;
import de.uniwue.ki.mas.basics.function.expertlist.ExpertListFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;

/**
 * Creation date: (19.11.2001 18:44:27)
 * @author Christoph Oechslein
 */

public final class FeatureClassDeclarationListManager {
	private static final FeatureClassDeclarationListManager _instance =
		new FeatureClassDeclarationListManager();

	private BuildinFeatureClassDeclarationList _buildinFeatureClassDeclarationList;
	private IFunctionDeclarationList _defaultFunctionDeclarationList = null;

	private static class BuildinFeatureClassDeclarationList extends FeatureClassDeclarationList {

		public void add(int index, Object o) {
			if (Assert.ASSERTION_ON) {
				Assert.assertTrue(o instanceof IFeatureClassDeclaration);
				Assert.assertNull(
					"Feature with name " + ((IFeatureClassDeclaration)o).getName() + " already exists!",
					getDeclarationByName(((IFeatureClassDeclaration)o).getName()));
			}

			super.add(index, o);
		}
	}

	protected FeatureClassDeclarationListManager() {
		super();
	}

	public void addBuildinFeatureClassDeclaration(IFeatureClassDeclaration featureClassDeclaration) {
		getBuildinFeatureClassDeclarationList().add(featureClassDeclaration);
	}

	public static final IFeatureClassDeclarationList createFeatureClassDeclarationList() {
		return new FeatureClassDeclarationList();
	}

	public static final IFeatureClassDeclarationList createFeatureClassDeclarationList(
		IFeatureClassDeclarationList featureClassDeclarationListA,
		IFeatureClassDeclarationList featureClassDeclarationListB) {

		return new TwoUnmodifiableFeatureDeclarationList(
			featureClassDeclarationListA,
			featureClassDeclarationListB,
			false);
	}

	public IObjectFeatureClassDeclarationList createFeatureClassDeclarationListWithDefaultFeaturesFor(IObjectClassDeclaration objectClassDeclaration) {
		IObjectFeatureClassDeclarationList newObjectFeatureClassDeclarationList =
			new ObjectFeatureClassDeclarationList();

		insertStandardFeatures(objectClassDeclaration, newObjectFeatureClassDeclarationList);

		newObjectFeatureClassDeclarationList.setComposition(false);

		return newObjectFeatureClassDeclarationList;
	}

	public synchronized IFeatureClassDeclarationList getBuildinFeatureClassDeclarationList() {
		if (_buildinFeatureClassDeclarationList == null) {
			_buildinFeatureClassDeclarationList = new BuildinFeatureClassDeclarationList();
			_buildinFeatureClassDeclarationList.add(
				StandardFunctionsBuildinFeatureClassDeclaration.getInstance());
			_buildinFeatureClassDeclarationList.add(
				StandardObjectFunctionsBuildinFeatureClassDeclaration.getInstance());
			_buildinFeatureClassDeclarationList.add(ExpertListFeatureClassDeclaration.getInstance());
			// SEE SeSAMCmd#addTypesFeatures
		}
		return _buildinFeatureClassDeclarationList;
	}

	public IFeatureClassDeclarationList getBuildinFeatureClassDeclarationListFor(IObjectClassDeclaration objectClassDeclaration) {
		if (objectClassDeclaration == null) {
			return getBuildinFeatureClassDeclarationList();

		} else {
			FeatureClassDeclarationList featureClassDeclarationList = new FeatureClassDeclarationList();

			Iterator it = getBuildinFeatureClassDeclarationList().iterator();
			while (it.hasNext()) {
				IFeatureClassDeclaration featureClassDeclaration = (IFeatureClassDeclaration)it.next();
				if (featureClassDeclaration.canAddFeatureClassDeclarationTo(objectClassDeclaration)) {
					featureClassDeclarationList.add(featureClassDeclaration);
				}
			}

			return featureClassDeclarationList;
		}
	}

	public IFunctionDeclarationList getDefaultFunctionDeclarationList() {
		if (_defaultFunctionDeclarationList == null) {
			_defaultFunctionDeclarationList =
				new MapDefaultFunctionDeclarationList(getBuildinFeatureClassDeclarationList());
		}

		return _defaultFunctionDeclarationList;
	}

	public static final FeatureClassDeclarationListManager getInstance() {
		return _instance;
	}

	private void insertStandardFeatures(
		IObjectClassDeclaration objectClassDeclaration,
		IObjectFeatureClassDeclarationList objectFeatureClassDeclarationList) {

		Iterator it = getBuildinFeatureClassDeclarationList().iterator();
		while (it.hasNext()) {
			IFeatureClassDeclaration featureClassDeclaration = (IFeatureClassDeclaration)it.next();

			if (featureClassDeclaration.isDefaultFeatureClassDeclarationFor(objectClassDeclaration)
				&& featureClassDeclaration.canAddFeatureClassDeclarationTo(objectClassDeclaration)) {

				objectFeatureClassDeclarationList.add(
					featureClassDeclaration.createObjectFeatureClassDeclaration(objectClassDeclaration));
			}
		}
	}
}