package de.uniwue.ki.mas.basics.featureList;

import java.util.Iterator;
import java.util.List;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListHelper;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.util.declaration.*;

/**
 * Creation date: 01.08.2002, 11:16:57
 * @author oechslein
 */
public class MapDefaultFunctionDeclarationList
	extends DefaultDeclarationList
	implements IFunctionDeclarationList {

	private final IFeatureClassDeclarationList _delegateFeatureClassDeclarationList;

		private final transient DeclarationListListener _declarationListListener =
			new DeclarationListListener() {
			public void declarationAdded(DeclarationListEvent event) {
				addDefaultFunctionDeclarations((IFeatureClassDeclaration) event.getDeclaration());
			}
	
			public void declarationRemoved(DeclarationListEvent event) {
				removeDefaultFunctionDeclarations((IFeatureClassDeclaration) event.getDeclaration());
			}
		};

	public MapDefaultFunctionDeclarationList(IFeatureClassDeclarationList delegateFeatureClassDeclarationList) {
		super();

		_delegateFeatureClassDeclarationList = delegateFeatureClassDeclarationList;

		Iterator it = delegateFeatureClassDeclarationList.iterator();
		while (it.hasNext()) {
			IFeatureClassDeclaration element = (IFeatureClassDeclaration) it.next();
			addDefaultFunctionDeclarations(element);
		}

		_delegateFeatureClassDeclarationList.addDeclarationListListener(_declarationListListener);
	}
	
	public void remove() {
		_delegateFeatureClassDeclarationList.removeDeclarationListListener(_declarationListListener);
		
		super.remove();
	}

	public Iterator getFunctionDeclarations(ISpecificType outputType) {
		return FunctionDeclarationListHelper.getFunctionDeclarations(this, outputType);
	}

	public ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		String str,
		ISpecificType outputType,
		List inputTypes) {

		return FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
			this,
			str,
			outputType,
			inputTypes);
	}

	public ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		String name,
		ISpecificType outputType) {

		return FunctionDeclarationListHelper.getSpecificFunctionDeclaration(iterator(), name, outputType);
	}

	public Iterator getFunctionDeclarations() {
		return iterator();
	}

	public void stateChanged() {
		fireStateChanged();
	}

	private void addDefaultFunctionDeclarations(IFeatureClassDeclaration featureClassDeclaration) {
		Iterator it = featureClassDeclaration.getDefaultFunctionDeclarations();
		while (it.hasNext()) {
			add(it.next());
		}
	}


	private void removeDefaultFunctionDeclarations(IFeatureClassDeclaration featureClassDeclaration) {
		Iterator it = featureClassDeclaration.getDefaultFunctionDeclarations();
		while (it.hasNext()) {
			remove(it.next());
		}
	}
}
