package de.uniwue.ki.mas.basics.featureList;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * Creation date: (22.10.2001 13:06:49)
 * @author Christoph Oechslein
 */

class ObjectFeatureClassDeclarationList
	extends DefaultDeclarationList
	implements IObjectFeatureClassDeclarationList {

	private final RemoveVetoListener _neverRemoveVetoListener = new NeverRemoveVetoListener(this);

	public ObjectFeatureClassDeclarationList() {
		super();
	}

	public boolean accept(Object declaration) {
		return declaration instanceof IObjectFeatureClassDeclaration;
	}

	public void add(int index, Object o) {
		IObjectFeatureClassDeclaration objectFeatureClassDeclaration = (IObjectFeatureClassDeclaration) o;

		if (contains(objectFeatureClassDeclaration)) {

			if (indexOf(objectFeatureClassDeclaration) != index) {
				changeIndex(objectFeatureClassDeclaration, index);
			}

		} else if (
			containsFeatureClassDeclaration(objectFeatureClassDeclaration.getFeatureClassDeclaration())) {

			IObjectFeatureClassDeclaration oldObjectFeatureClassDeclaration =
				getObjectFeatureClassDeclaration(objectFeatureClassDeclaration.getFeatureClassDeclaration());

			if (indexOf(oldObjectFeatureClassDeclaration) != index) {
				changeIndex(oldObjectFeatureClassDeclaration, index);
			}

			objectFeatureClassDeclaration.remove();

		} else {
			super.add(index, objectFeatureClassDeclaration);

			objectFeatureClassDeclaration.addRemoveVetoListener(_neverRemoveVetoListener);
		}
	}

	/**
	 * @param declaration IDeclaration
	 * @return boolean
	 * @see de.uniwue.ki.mas.util.declaration.IDeclarationList#canBeRemoved(IDeclaration)
	 */
	public boolean canBeRemoved(IDeclaration declaration) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue(contains(declaration));

		if (declaration instanceof IUserFeatureClassDeclaration)
			return true;
		else
			return super.canBeRemoved(declaration);
	}

	private boolean containsFeatureClassDeclaration(IFeatureClassDeclaration featureClassDeclaration) {
		return getObjectFeatureClassDeclaration(featureClassDeclaration) != null;
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.featureList.IObjectFeatureClassDeclarationList#getFunctionDeclarations()
	 */
	public Iterator getFunctionDeclarations() {
		Iterator iteratorOfIterator = CollectionsHelper.map(iterator(), new IMapFunction() {
			public Object map(Object anObject) {
				return ((IObjectFeatureClassDeclaration) anObject).getFunctionDeclarations();
			}
		});
		return CollectionsHelper.removeDuplicates(CollectionsHelper.flatten(iteratorOfIterator));
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Features";
	}

	private IObjectFeatureClassDeclaration getObjectFeatureClassDeclaration(IFeatureClassDeclaration featureClassDeclaration) {
		Iterator it = iterator();

		while (it.hasNext()) {
			IObjectFeatureClassDeclaration currIObjectFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration) it.next();

			if (currIObjectFeatureClassDeclaration.getFeatureClassDeclaration() == featureClassDeclaration) {
				return currIObjectFeatureClassDeclaration;
			}
		}

		return null;
	}

	/**
	 * @return Iterator
	 * @see de.uniwue.ki.mas.basics.featureList.IObjectFeatureClassDeclarationList#getVariableClassDeclarations()
	 */
	public Iterator getVariableClassDeclarations() {
		Iterator iteratorOfIterator = CollectionsHelper.map(iterator(), new IMapFunction() {
			public Object map(Object anObject) {
				return ((IObjectFeatureClassDeclaration) anObject).getVariableClassDeclarations();
			}
		});

		/*
		return CollectionsHelper.removeDuplicates(CollectionsHelper.concatenate(iteratorOfIterator));
		*/

		return CollectionsHelper.flatten(iteratorOfIterator);
	}

	/**
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		Iterator it = iterator();
		while (it.hasNext()) {
			IObjectFeatureClassDeclaration featureClassDeclaration =
				(IObjectFeatureClassDeclaration) it.next();
			if (featureClassDeclaration != null) {
				featureClassDeclaration.removeRemoveVetoListener(_neverRemoveVetoListener);
			}
		}
		super.remove();
	}

	public Object remove(int index) {
		IObjectFeatureClassDeclaration featureClassDeclaration =
			(IObjectFeatureClassDeclaration) super.remove(index);
		if (featureClassDeclaration != null) {
			featureClassDeclaration.removeRemoveVetoListener(_neverRemoveVetoListener);
		}
		return featureClassDeclaration;
	}


	public Object set(int index, Object o) {
		IObjectFeatureClassDeclaration newFeatureClassDeclaration = (IObjectFeatureClassDeclaration) o;

		IObjectFeatureClassDeclaration oldFeatureClassDeclaration = null;

		if (!contains(newFeatureClassDeclaration)) {
			oldFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration) super.set(index, newFeatureClassDeclaration);
		}

		if (oldFeatureClassDeclaration != null) {
			oldFeatureClassDeclaration.removeRemoveVetoListener(_neverRemoveVetoListener);
		}
		if (newFeatureClassDeclaration != null) {
			newFeatureClassDeclaration.addRemoveVetoListener(_neverRemoveVetoListener);
		}
		return oldFeatureClassDeclaration;
	}
}