package de.uniwue.ki.mas.basics.featureList;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.util.declaration.*;

/**
 * Creation date: (30.07.2001 14:24:23)
 * @author Christoph Oechslein
 */
public class ObjectFeatureInstanceDeclarationList extends DefaultDeclarationList implements DeclarationListListener {
   private final IObjectFeatureClassDeclarationList objectFeatureClassDeclarationList;
   private final IObjectInstanceDeclaration objectInstanceDeclaration;

/**
 * ComposedFeatureInstanceDeclaration constructor comment.
 * @param objectInstanceDeclaration IObjectInstanceDeclaration
 * @param objectFeatureClassDeclarationList IObjectFeatureClassDeclarationList
 */
public ObjectFeatureInstanceDeclarationList(
   IObjectInstanceDeclaration objectInstanceDeclaration,
   IObjectFeatureClassDeclarationList objectFeatureClassDeclarationList) {

   super();

   this.objectFeatureClassDeclarationList = objectFeatureClassDeclarationList;
   this.objectInstanceDeclaration = objectInstanceDeclaration;

   objectFeatureClassDeclarationList.addDeclarationListListener(this);
   synchronize();
}


/**
 * @param declaration Object
 * @return boolean
 * @see de.uniwue.ki.util.IMutableListModel#accept(Object)
 */
public boolean accept(Object declaration) {
    return declaration instanceof IObjectFeatureInstanceDeclaration;
}


/**
 * @see de.uniwue.ki.mas.model.util.declaration.IDeclarationList
 */
public boolean canBeRemoved(boolean recursive) {
   return true;
}


/**
 * @param event DeclarationListEvent
 * @see de.uniwue.ki.mas.model.util.declaration.DeclarationListListener
 */
public void declarationAdded(DeclarationListEvent event) {
   Object newObject = mapElement(event.getDeclaration());
   if (newObject != null) {
      add(newObject);
   }
}


/**
 * @param event DeclarationListEvent
 * @see de.uniwue.ki.mas.model.util.declaration.DeclarationListListener
 */
public void declarationRemoved(DeclarationListEvent event) {
   IObjectFeatureInstanceDeclaration featureInstanceDeclaration =
      mapClassToInstanceDecl((IObjectFeatureClassDeclaration) event.getDeclaration());
   if (featureInstanceDeclaration != null) {
      remove(featureInstanceDeclaration);
   }
}


private static Object delegateMap(Object anObject, IObjectInstanceDeclaration objectInstanceDeclaration) {
   if (anObject == null) {
      return null;
   } // IFeatureInstanceDeclaration.NULL_FEATURE_INSTANCE_DECLARATION;
   else if (anObject instanceof IObjectFeatureInstanceDeclaration) {
      return anObject;
   } else {
      return delegateMap(
         ((IObjectFeatureClassDeclaration) anObject).createObjectFeatureInstanceDeclaration(objectInstanceDeclaration),
         objectInstanceDeclaration);
   }
}


/**
 * @return String
 * @see de.uniwue.ki.util.INamed#getName()
 */
public String getName() {
   return "Features";
}


private IObjectFeatureInstanceDeclaration mapClassToInstanceDecl(IObjectFeatureClassDeclaration objectFeatureClassDeclaration) {
   Iterator it = iterator();

   while (it.hasNext()) {
      IObjectFeatureInstanceDeclaration currentObjectFeatureInstanceDeclaration =
         (IObjectFeatureInstanceDeclaration) it.next();
      if (objectFeatureClassDeclaration.equals(currentObjectFeatureInstanceDeclaration.getObjectFeatureClassDeclaration())) {
         return currentObjectFeatureInstanceDeclaration;
      }
   }

   return null;
}


protected Object mapElement(Object anObject) {
    return delegateMap(anObject, objectInstanceDeclaration);
}


/**
 * @see de.uniwue.ki.util.IRemoveable#remove()
 */
public void remove() {
   objectFeatureClassDeclarationList.removeDeclarationListListener(this);

   super.remove();
}


/**
 * AbstractMapCopyDeclarationList constructor comment.
 * @param delegateListModel javax.swing.ListModel
 */
protected void synchronize() {
   Iterator it = objectFeatureClassDeclarationList.iterator();
   while (it.hasNext()) {
      Object newObject = mapElement(it.next());
      if (newObject != null) {
         add(newObject);
      }
   }
}
}