package de.uniwue.ki.mas.basics.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.collection.IteratorWithException;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.name.AbstractNamed;
/**
 * Creation date: (30.07.2001 14:15:07)
 * @author Christoph Oechslein
 */

public abstract class AbstractFunction extends AbstractNamed implements IFunction {

	public AbstractFunction() {
		super();
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		throw new WrongFunctionTypeException("act " + getName());
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		throw new WrongFunctionTypeException("executeForBoolean " + getName());
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		throw new WrongFunctionTypeException("executeForDouble " + getName());
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		throw new WrongFunctionTypeException("executeForInt " + getName());
	}

	public String getName() {
		String className = getClass().getName();
		int index = className.lastIndexOf('.');
		if (index == -1)
			return "<" + className + ">";
		else
			return "<" + className.substring(index + 1) + ">";
	}

	/**
	 * Converts Iterator of IFunctions to Iterator of Objects (by calling execute)
	 * @param:  executeFunctionArgs Arguments to all functions
	 *          functions Iterator of functions
	 * @return: IteratorWithException of Objects
	 * @author Christoph Oechslein
	 */
	protected static IteratorWithException mapFunctionsToObjects(
		final IExecuteFunctionArgs executeFunctionArgs,
		Iterator functions,
		boolean withSavingArguments)
		throws SimulationRuntimeException {

		IMapFunction mappingFunction;

		if (withSavingArguments) {
			final IArgumentBindings currArgumentBindings =
				executeFunctionArgs.saveCurrentArgumentBindings();

			mappingFunction = new IMapFunction() {
				public final Object map(Object obj) throws SimulationRuntimeException {
					executeFunctionArgs.pushArgumentBindings(currArgumentBindings);

					Object result = ((IFunction)obj).execute(executeFunctionArgs);

					executeFunctionArgs.popArgumentBindings();

					return result;
				}
			};
		} else {
			mappingFunction = new IMapFunction() {
				public final Object map(Object obj) throws SimulationRuntimeException {
					return ((IFunction)obj).execute(executeFunctionArgs);
				}
			};
		}

		try {
			return CollectionsHelper.mapWithException(
				CollectionsHelper.asIteratorWithException(functions),
				mappingFunction);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}

	protected static void appendFunctionName(
		StringBuffer buff,
		String name,
		Iterator functions,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append(name);
			buff.append('(');
			while (functions.hasNext()) {
				IFunction function = (IFunction)functions.next();
				function.appendName(buff, currDepth + 1, maxDepth);
				if (functions.hasNext())
					buff.append(", ");
			}
			buff.append(')');
		}
	}

	protected static void appendFunctionName(
		StringBuffer buff,
		String name,
		IFunctionList functions,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append(name);
			buff.append('(');
			int size = functions.size();
			for (int i = 0; i < size; i++) {
				functions.getFunction(i).appendName(buff, currDepth + 1, maxDepth);
				if (i < size - 1)
					buff.append(", ");
			}

			buff.append(')');
		}
	}

	protected static void appendFunctionName(
		StringBuffer buff,
		String name,
		IFunction function,
		int currDepth,
		int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append(name);
			buff.append('(');
			function.appendName(buff, currDepth + 1, maxDepth);
			buff.append(')');
		}
	}

	protected static void appendFunctionName(
		StringBuffer buff,
		String name,
		IFunction functionA,
		IFunction functionB,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append(name);
			buff.append('(');
			functionA.appendName(buff, currDepth + 1, maxDepth);
			buff.append(", ");
			functionB.appendName(buff, currDepth + 1, maxDepth);
			buff.append(')');
		}
	}
}