package de.uniwue.ki.mas.basics.function;

import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionArgument.*;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.hash.CachedMap;
import de.uniwue.ki.util.hash.IsomorphicHashTableWrapper;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: 03.07.2002, 08:46:05
 * @author oechslein
 */
public class CreateAndExecuteFunctionArgs implements ICreateAndExecuteFunctionArgs {
	private static final Logger _logger = Logger.getLogger(CreateAndExecuteFunctionArgs.class.getName());

	private static final boolean containsSameKey(Map map, Object key) {
		Iterator it = map.keySet().iterator();
		while (it.hasNext()) {
			if (it.next() == key)
				return true;
		}

		return false;
	}
	private LinkedList _argumentBindings = new LinkedList();
	private final Map _equalMap = new HashMap();

	private final FunctionDeclarationCompiler _functionDeclarationCompiler =
		new FunctionDeclarationCompiler();
	private final CachedMap _isomorphicMap = new CachedMap(new IsomorphicHashTableWrapper());

	public CreateAndExecuteFunctionArgs() {
		super();
	}

	public final void addCompilationObject(
		Object anObject,
		Object compilationObject,
		boolean isomorphicTest) {
		if (isomorphicTest) {
			_isomorphicMap.put(anObject, compilationObject);

		} else {
			_equalMap.put(anObject, compilationObject);
		}
	}

	public final ICreateAndExecuteFunctionArgs createCreateAndExecuteFunctionArgs() {
		return this;
	}

	public Object getArgumentValueFor(IFunctionArgument functionArgument) throws SimulationRuntimeException {

		Iterator it = _argumentBindings.iterator();

		while (it.hasNext()) {
			IArgumentBindings argumentBindings = (IArgumentBindings)it.next();
			if (argumentBindings.containsBindingFor(functionArgument)) {
				return argumentBindings.getArgumentValueFor(functionArgument);
			}
		}

		throw new SimulationRuntimeException(
			"Function Argument '" + functionArgument.getName() + "' not found!");
	}

	public final boolean existsCompilationObject(Object anObject, boolean isomorphicTest) {
		if (isomorphicTest) {
			return _isomorphicMap.containsKey(anObject);

		} else {
			return _equalMap.containsKey(anObject);
		}
	}

	public final Object getCompilationObject(Object anObject, boolean isomorphicTest) {
		Object cachedObject =
			isomorphicTest ? _isomorphicMap.get(anObject) : _equalMap.get(anObject);

		if (Assert.ASSERTION_ON && _logger.isLoggable(Level.FINE)) {
			boolean cachedObjectIsSame =
				containsSameKey(isomorphicTest ? (Map)_isomorphicMap : (Map)_equalMap, anObject);

			if (!cachedObjectIsSame) {
				String anObjectName =
					anObject instanceof INamed ? ((INamed)anObject).getName() : anObject + "";
				String cachedObjectName =
					cachedObject instanceof INamed ? ((INamed)cachedObject).getName() : cachedObject + "";

				_logger.fine("Retrieve cached object " + cachedObjectName + " of " + anObjectName);
			}
		}

		return cachedObject;
	}

	public final FunctionDeclarationCompiler getFunctionDeclarationCompiler() {
		return _functionDeclarationCompiler;
	}

	public void popArgumentBindings() throws SimulationRuntimeException {
		_argumentBindings.removeFirst();
	}
	
	// be carefull using this, rainer 11.05.2004
	public void clearArgumentBindings() {
		_argumentBindings=new LinkedList();
	}
	
	
	public boolean hasArgumentBindings()  {
		return (!(_argumentBindings==null) && !_argumentBindings.isEmpty());
	}
	
	

	public void pushArgumentBindings(IArgumentBindings argumentBindings) throws SimulationRuntimeException {
		Assert.assertNotSame(this, argumentBindings);

		_argumentBindings.addFirst(argumentBindings);
	}

	public IArgumentBindings saveCurrentArgumentBindings() {
		if (_argumentBindings.isEmpty()) {
			return IArgumentBindings.NULL_ARGUMENT_BINDINGS;

		} else if (_argumentBindings.size() == 1) {
			return ((IArgumentBindings)_argumentBindings.get(0)).saveCurrentArgumentBindings();

		} else {
			IArgumentBindings[] argumentBindingsArray = new IArgumentBindings[_argumentBindings.size()];
			Iterator it = _argumentBindings.iterator();
			int i = 0;
			while (it.hasNext()) {
				IArgumentBindings argumentBindings = (IArgumentBindings)it.next();
				argumentBindingsArray[i++] = argumentBindings.saveCurrentArgumentBindings();
			}
			return new ArgumentBindingsArray(argumentBindingsArray);
		}
	}

	public void remove() {
		_isomorphicMap.clear();
		_equalMap.clear();
		_argumentBindings = null;
		getFunctionDeclarationCompiler().remove();
	}
}
