package de.uniwue.ki.mas.basics.function;
import java.util.Iterator;

import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.IteratorWithException;
/**
 * Creation date: (28.03.2002 19:41:30)
 * @author Christoph Oechslein
 */
public final class ExecuteIteratorHelper {

	// object
	public static final Object nextExecute(Iterator it) throws SimulationRuntimeException {
		if (it instanceof IExecuteIterator) {
			return nextExecute((IExecuteIterator) it);

		} else if (it instanceof IteratorWithException) {
			return nextExecute((IteratorWithException) it);

		} else {
			return it.next();
		}
	}

	public static final Object nextExecute(IExecuteIterator it) throws SimulationRuntimeException {
		return it.nextExecute();
	}

	public static final Object nextExecute(IteratorWithException it) throws SimulationRuntimeException {
		try {
			return it.nextWithException();

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
				
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
				
			else
				throw new RuntimeException(e);
		}
	}

	// without value
	public static final void nextExecuteWithoutValue(Iterator it) throws SimulationRuntimeException {
		nextExecute(it);
	}

	// double
	public static final double nextExecuteForDouble(Iterator it) throws SimulationRuntimeException {
		if (it instanceof IExecuteIterator) {
			return nextExecuteForDouble((IExecuteIterator) it);

		} else if (it instanceof IteratorWithException) {
			return nextExecuteForDouble((IteratorWithException) it);

		} else {
			return ((Number) it.next()).doubleValue();
		}
	}

	public static final double nextExecuteForDouble(IExecuteIterator it) throws SimulationRuntimeException {
		return it.nextExecuteForDouble();
	}

	public static final double nextExecuteForDouble(IteratorWithException it)
		throws SimulationRuntimeException {
		try {
			return ((Number) it.nextWithException()).doubleValue();

		} catch (Exception exception) {
			if (exception instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) exception;
				
			else if (exception instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) exception);

			else
				throw new RuntimeException(exception);
		}
	}
	
	// int
	public static final int nextExecuteForInt(Iterator it) throws SimulationRuntimeException {
		if (it instanceof IExecuteIterator) {
			return nextExecuteForInt((IExecuteIterator) it);

		} else if (it instanceof IteratorWithException) {
			return nextExecuteForInt((IteratorWithException) it);

		} else {
			return ((Number) it.next()).intValue();
		}
	}

	public static final int nextExecuteForInt(IExecuteIterator it) throws SimulationRuntimeException {
		return it.nextExecuteForInt();
	}

	public static final int nextExecuteForInt(IteratorWithException it)
		throws SimulationRuntimeException {
		try {
			return ((Number) it.nextWithException()).intValue();

		} catch (Exception exception) {
			if (exception instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) exception;
				
			else if (exception instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) exception);

			else
				throw new RuntimeException(exception);
		}
	}


	// boolean
	public static final boolean nextExecuteForBoolean(Iterator it) throws SimulationRuntimeException {
		if (it instanceof IExecuteIterator) {
			return nextExecuteForBoolean((IExecuteIterator) it);

		} else if (it instanceof IteratorWithException) {
			return nextExecuteForBoolean((IteratorWithException) it);

		} else {
			return ((Boolean) it.next()).booleanValue();
		}
	}

	public static final boolean nextExecuteForBoolean(IExecuteIterator it)
		throws SimulationRuntimeException {
			
		return it.nextExecuteForBoolean();
	}

	public static final boolean nextExecuteForBoolean(IteratorWithException it)
		throws SimulationRuntimeException {
		try {
			return ((Boolean) it.nextWithException()).booleanValue();

		} catch (Exception exception) {
			if (exception instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) exception;
				
			else if (exception instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) exception);
				
			else
				throw new RuntimeException(exception);
		}
	}

}