package de.uniwue.ki.mas.basics.function;
import java.io.Serializable;

import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;

/**
 * Interface for using Functions as first order objects.
 * <BR>Creation date: (31.03.00 10:52:18)
 * @author Christoph Oechslein
 */

public interface IFunction extends INamed, Serializable {

	/**
	 * Function which returns nothing, just do side effects on <code>executeFunctionArg</code>.
	 *
	 * @input executeFunctionArgs are the arguments for this 'function'.
	 * @exception if an error occurs during the function call.
	 */
	void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Function on arguments which returns an Object, this is the standard method.
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */
	Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Function on a arguments which returns boolean (predicate function), should be the same as
	 * <code>execute(executeFunctionArgs).booleanValue()</code>.
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */
	boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Function on arguments which returns double, should be the same as
	 * <code>execute(executeFunctionArgs).doubleValue()</code>
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */
	double executeForDouble(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Function on arguments which returns int, should be the same as 
	 * <code>execute(executeFunctionArgs).intValue()</code>
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */
	int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;
}