package de.uniwue.ki.mas.basics.function;
import java.io.Serializable;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: (18.12.2001 09:33:42)
 * @author Christoph Oechslein
 */

public interface IParametricFunction extends INamed, Serializable {

	/**
	 * Must produce the same result as calling execute on the IFunction produced by createFunction but may be a lot faster.
	 * <BR>Kind of interpreting the declaration.
	 */
	void act(ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException;

	/**
	 * Creates an IFunction, the IFunctions in the IFunctionList must be called for producing the real input objects.
	 * In createFunctionArgs is the current compiling binding, e.g. the simulation for this function.
	 *
	 * @input functions is list of IFunctions, the size of functionsmust be ok for the IFunctionDeclaration
	 * (between min and max and the input types of the declaration must the same as the output types of the
	 * IFunctions in functions).
	 * @output returns a new IFunction, which implements the FunctionDeclaration and the ParameterConstellation.
	 * it must be of type outputType
	 * @throw ModelCompilingException if during creation of a new IFunction an error occurs.
	 * @label compiles
	 * @directed
	 */

	IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException;

	IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException;

	/**
	 * Must produce the same result as calling execute on the IFunction produced by createFunction but may be a lot faster.
	 * <BR>Kind of interpreting the declaration.
	 */
	Object execute(ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException;

	/**
	 * Must produce the same result as calling executeForDouble on the IFunction produced by createFunction but may be a lot
	 * faster. The outputType must be {@link de.uniwue.ki.mas.model.basics.type.BooleanType BooleanType}
	 * <BR>Kind of interpreting the declaration.
	 */
	boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException;

	/**
	 * Must produce the same result as calling executeForDouble on the IFunction produced by createFunction but may be a lot
	 * faster. The outputType must be {@link de.uniwue.ki.mas.model.basics.type.DoubleType DoubleType}.
	 * <BR>Kind of interpreting the declaration.
	 */
	double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException;

	/**
	 * Must produce the same result as calling executeForInt on the IFunction produced by createFunction but
	 * may be a lot faster. The outputType must be {@link de.uniwue.ki.mas.model.basics.type.IntegerrType
	 * IntegerrType}.
	 * <BR>Kind of interpreting the declaration.
	 */
	int executeForInt(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException;
}