package de.uniwue.ki.mas.basics.function;
import java.util.AbstractList;
import java.util.RandomAccess;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: (20.12.2001 11:08:42)
 * @author Christoph Oechslein
 */

public class SingletonObjectFunctionList extends AbstractList implements IFunctionList, RandomAccess {
	private SingletonFunctionListArgument _singletonFunctionListArgument;

	final private static class SingletonFunctionListArgument extends AbstractArgument implements IArgument {
		private Object _anObject;

		public void setObject(Object anObject) {
			Assert.assertFalse(anObject instanceof IBody);

			_anObject = anObject;
		}

		public final Object getConstantValue() {
			return _anObject;
		}

		public final String getName() {
			if (_anObject instanceof INamed) {
				return ((INamed) _anObject).getName();

			} else {
				return _anObject + "";
			}
		}
	}

	public SingletonObjectFunctionList() {
		super();
	}

	/**
	 * Returns the element at the specified position in this list.
	 *
	 * @throws IndexOutOfBoundsException if the index is out of range (index
	 * 		  &lt; 0 || index &gt;= size()).
	 * @param index index of element to return.
	 * @return the element at the specified position in this list.
	 * 
	 * @see java.util.List#get(int)
	 */
	public Object get(int index) {
		if (index != 0)
			throw new IndexOutOfBoundsException();

		return _singletonFunctionListArgument;
	}

	public IFunction getFunction(int index) {
		return _singletonFunctionListArgument;
	}

	public String getName() {
		return "(" + _singletonFunctionListArgument.getName() + ")";
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append("(");
			_singletonFunctionListArgument.appendName(buff, currDepth, maxDepth);
			buff.append(")");
		}
	}

	public void setFunction(Object anObject) {
		Assert.assertTrue(!(anObject instanceof IFunction));
		setFunctionObject(anObject);
	}

	public void setFunctionObject(Object anObject) {
		if (_singletonFunctionListArgument == null) {
			_singletonFunctionListArgument = new SingletonFunctionListArgument();
		}

		_singletonFunctionListArgument.setObject(anObject);
	}

	public int size() {
		return 1;
	}
}