package de.uniwue.ki.mas.basics.function;
import java.util.*;

import junit.framework.Test;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.feature.spatial.position.TestPosition2DFunction;
import de.uniwue.ki.mas.basics.function.bool.TestBooleanFunction;
import de.uniwue.ki.mas.basics.function.function.TestFunctionFunction;
import de.uniwue.ki.mas.basics.function.iterator.TestIteratorFunction;
import de.uniwue.ki.mas.basics.function.number.TestNumberFunction;
import de.uniwue.ki.mas.basics.function.variable.TestVariableFunction;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;

/**
 * Creation date: (15.11.2001 13:57:26)
 * @author Christoph Oechslein
 */

public final class TestFunction extends TestCase {
	private final IFunctionDeclarationList _allFunctionDeclarationList =
		FunctionDeclarationListManager.getAllFunctionDeclarationList();

	public TestFunction(String name) {
		super(name);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestFunction.class);
	}

	/**
	 * @return Test
	 */
	public static Test suite() {
		WithAllCompilerSettingsTestSuite anSuite = new WithAllCompilerSettingsTestSuite("Function");

		anSuite.addTestSuite(TestNumberFunction.class);
		anSuite.addTestSuite(TestBooleanFunction.class);
		anSuite.addTestSuite(TestPosition2DFunction.class);
		anSuite.addTestSuite(TestIteratorFunction.class);
		anSuite.addTestSuite(TestFunctionFunction.class);
		anSuite.addTestSuite(TestVariableFunction.class);
		anSuite.addTest(new TestFunction("testAllSpecificFunctionDeclarations"));
		anSuite.addTest(new TestFunction("testAllGenericFunctionDeclarations"));

		return anSuite;
	}

	/**
	 * @throws Exception
	 */
	public void testAllGenericFunctionDeclarations() throws Exception {

		Iterator it =
			FunctionDeclarationListHelper.getGenericFunctionDeclarations(
				_allFunctionDeclarationList.iterator());

		while (it.hasNext()) {
			IGenericFunctionDeclaration currIGenericFunctionDeclaration =
				(IGenericFunctionDeclaration) it.next();

			testGenericFunctionDeclaration(currIGenericFunctionDeclaration);
		}

	}

	/**
	 * @throws Exception
	 */
	public void testAllSpecificFunctionDeclarations() throws Exception {

		Iterator it =
			FunctionDeclarationListHelper.getSpecificFunctionDeclarations(
				_allFunctionDeclarationList.iterator());

		while (it.hasNext()) {
			ISpecificFunctionDeclaration currISpecificFunctionDeclaration =
				(ISpecificFunctionDeclaration) it.next();

			testSpecificFunctionDeclaration(currISpecificFunctionDeclaration);
		}

	}

	private void testGenericFunctionDeclaration(IGenericFunctionDeclaration genericFunctionDeclaration)
		throws Exception {
	}

	private void testSpecificFunctionDeclaration(ISpecificFunctionDeclaration specificFunctionDeclaration)
		throws Exception {

		int minArgsLength = specificFunctionDeclaration.getSpecificInputMinArgsLength();
		int defaultArgsLength = specificFunctionDeclaration.getSpecificInputDefaultArgsLength();
		int maxArgsLength = specificFunctionDeclaration.getSpecificInputMaxArgsLength();

		assertTrue(
			(minArgsLength <= defaultArgsLength)
				&& ((defaultArgsLength <= maxArgsLength)
					|| (maxArgsLength == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT)));

		testSpecificFunctionDeclaration(specificFunctionDeclaration, defaultArgsLength);
		if (defaultArgsLength != minArgsLength) {
			testSpecificFunctionDeclaration(specificFunctionDeclaration, minArgsLength);
		}

		if (maxArgsLength == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			testSpecificFunctionDeclaration(
				specificFunctionDeclaration,
				de.uniwue.ki.util.math.RandomHelper.getRandomInt(100) + minArgsLength);

		} else if (maxArgsLength != defaultArgsLength) {
			testSpecificFunctionDeclaration(specificFunctionDeclaration, maxArgsLength);
		}
	}

	private void testSpecificFunctionDeclaration(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		int size)
		throws Exception {
		List types = new ArrayList(size);

		for (int i = 0; i < specificFunctionDeclaration.getSpecificInputDefaultArgsLength(); i++) {
			types.add(specificFunctionDeclaration.getSpecificInputTypeAt(i));
		}

		testSpecificFunctionDeclaration(specificFunctionDeclaration, types);
	}

	private void testSpecificFunctionDeclaration(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List types)
		throws Exception {

		ISpecificFunctionDeclaration foundSpecificFunctionDeclaration =
			FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
				_allFunctionDeclarationList,
				specificFunctionDeclaration.getName(),
				specificFunctionDeclaration.getSpecificOutputType(),
				types);

		assertNotNull(foundSpecificFunctionDeclaration);
		assertEquals(specificFunctionDeclaration, foundSpecificFunctionDeclaration);
	}
}