package de.uniwue.ki.mas.basics.function;
import java.util.AbstractList;

import de.uniwue.ki.mas.basics.type.argument.Argument;
/**
 * Creation date: (16.01.2002 19:56:45)
 * @author Christoph Oechslein
 */

public class TwoObjectFunctionList extends AbstractList implements IFunctionList {
	private IFunction _functionA;
	private IFunction _functionB;

	public TwoObjectFunctionList() {
		super();
	}

	public TwoObjectFunctionList(IFunction functionA, IFunction functionB) {
		super();

		_functionA = functionA;
		_functionB = functionB;
	}

	public TwoObjectFunctionList(Object objectA, Object objectB) {
		this(
			(objectA instanceof IFunction) ? (IFunction) objectA : new Argument(objectA),
			(objectB instanceof IFunction) ? (IFunction) objectB : new Argument(objectB));
	}

	public Object get(int index) {
		if (index == 0) {
			return getFunctionA();
		} else if (index == 1) {
			return getFunctionB();
		} else {
			throw new IndexOutOfBoundsException();
		}
	}

	public IFunction getFunction(int index) {
		return (IFunction) get(index);
	}

	public IFunction getFunctionA() {
		return _functionA;
	}

	public IFunction getFunctionB() {
		return _functionB;
	}

	public String getName() {
		StringBuffer buff = new StringBuffer();
		appendName(buff, 0, 10);
		return buff.toString();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append("(");
			getFunctionA().appendName(buff, currDepth + 1, maxDepth);
			buff.append(", ");
			getFunctionB().appendName(buff, currDepth + 1, maxDepth);
			buff.append(")");
		}
	}

	public void setFunctionA(IFunction newFunctionA) {
		_functionA = newFunctionA;
	}

	public void setFunctionB(IFunction newFunctionB) {
		_functionB = newFunctionB;
	}

	public void setFunctionObjectA(Object newObjectA) {
		if (newObjectA instanceof IFunction) {
			setFunctionA((IFunction) newObjectA);
		} else {
			setFunctionA(new Argument(newObjectA));
		}
	}

	public void setFunctionObjectB(Object newObjectB) {
		if (newObjectB instanceof IFunction) {
			setFunctionB((IFunction) newObjectB);
		} else {
			setFunctionB(new Argument(newObjectB));
		}
	}

	public int size() {
		return 2;
	}
}