package de.uniwue.ki.mas.basics.function.bool;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: (07.03.2002 10:24:21)
 * @author Christoph Oechslein
 */
public class EqualGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final Map _map = new HashMap();
	private static final IDependentType _typeT = new UnconstraintDependentType("typ");

	public EqualGenericFunctionDeclaration() {
		super("Equal", BooleanType.getInstance(), CollectionsHelper.createList(_typeT));

		setDocumentation("Returns the verity content if its operands are equal");
	}

	private ISpecificFunctionDeclaration createEqualSpecificFunctionDeclaration(ISpecificType type) {
		if (!_map.containsKey(type)) {
			_map.put(type, new EqualSpecificFunctionArgsDeclaration(type));
		}

		return (EqualSpecificFunctionArgsDeclaration)_map.get(type);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {
		Assert.assertEquals(BooleanType.getInstance(), outputType);

		return createEqualSpecificFunctionDeclaration(bindings.getBindingFor(_typeT));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType type1 = getNextSpecificType(inputTypes);
		ISpecificType type2 = getNextSpecificType(inputTypes);

		if ((type1 == null) || (type2 == null) || (inputTypes.hasNext())) {
			return null;
		}

		if (type1.equals(type2)) {
			return createEqualSpecificFunctionDeclaration(type1);
		} else {
			return createEqualSpecificFunctionDeclaration(ObjectType.getInstance());
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(<T>, <T>)";
		} else {
			String name = outputType.getName();
			return "(<" + name + ">, <" + name + ">)";
		}
	}
}