package de.uniwue.ki.mas.basics.function.bool;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: (22.02.2002 13:50:06)
 * @author Christoph Oechslein
 */
final class EqualSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {

	public EqualSpecificFunctionArgsDeclaration(ISpecificType type) {
		super("Equal", BooleanType.getInstance(), CollectionsHelper.createList(type, type), true);

		setDocumentation("Returns the verity content if its operands are equal");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		if (getSpecificInputTypeAt(0).equals(DoubleType.getInstance())) {
			Assert.assertEquals(DoubleType.getInstance(), getSpecificInputTypeAt(1));

			return new FunctionEqualDouble(functions.getFunction(0), functions.getFunction(1));

		} else {
			return new FunctionEqual(functions.getFunction(0), functions.getFunction(1));
		}
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration child0 = composedFunctionCallInternDeclaration.getChild(0);
		IFunctionCallInternDeclaration child1 = composedFunctionCallInternDeclaration.getChild(1);

		ISpecificType child0Type = child0.getSpecificOutputType();
		ISpecificType child1Type = child1.getSpecificOutputType();

		if (IntegerType.getInstance().equals(child0Type) && IntegerType.getInstance().equals(child1Type)) {
			return new FunctionEqualInteger(
				child0.createFunction(createFunctionArgs),
				child1.createFunction(createFunctionArgs));

		} else if (
			DoubleType.getInstance().isMoreSpecific(child0Type)
				|| DoubleType.getInstance().isMoreSpecific(child1Type)) {
					
			return new FunctionEqualDouble(
				child0.createFunction(createFunctionArgs),
				child1.createFunction(createFunctionArgs));

		} else {
			return super.createFunction(createFunctionArgs, composedFunctionCallInternDeclaration);
		}
	}
}