package de.uniwue.ki.mas.basics.function.bool;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (30.10.2001 19:47:38)
 * 
 * @author Christoph Oechslein
 */

public class FunctionDeclarationBoolFactory {
    
    public static final ISpecificFunctionDeclaration LOWER_EQUAL_FUNCTION_DECLARATION = new LowerEqualSpecificFunctionDeclaration();

    public static final ISpecificFunctionDeclaration AND_SPECIFIC_FUNCTION_DECLARATION = new AndSpecificFunctionDeclaration();

    public static final ISpecificFunctionDeclaration OR_SPECIFIC_FUNCTION_DECLARATION = new OrSpecificFunctionDeclaration();

    public static final ISpecificFunctionDeclaration NOT_SPECIFIC_FUNCTION_DECLARATION = new NotSpecificFunctionDeclaration();

    private static IFunctionDeclarationList _cachedFunctionDeclarationList;

    public synchronized static void insertFunctions(
            IFunctionDeclarationList functionDeclarationList) {
        if (_cachedFunctionDeclarationList == null) {
            _cachedFunctionDeclarationList = new FunctionDeclarationList();
            insertFunctionsIntern(_cachedFunctionDeclarationList);
        }

        functionDeclarationList.addAll(_cachedFunctionDeclarationList);
    }

    private static void insertBoolOperatorFunctions(
            IFunctionDeclarationList functionDeclarationList) {
        // AND
        functionDeclarationList.add(AND_SPECIFIC_FUNCTION_DECLARATION);

        // OR
        functionDeclarationList.add(OR_SPECIFIC_FUNCTION_DECLARATION);

        // NOT
        functionDeclarationList.add(NOT_SPECIFIC_FUNCTION_DECLARATION);
    }

    private static void insertCompareFunctions(
            IFunctionDeclarationList functionDeclarationList) {
        // Equal
        functionDeclarationList.add(new EqualGenericFunctionDeclaration());

        // <
        ISpecificFunctionDeclaration functionDecl = new AbstractSpecificFunctionArgsDeclaration(
                "<", BooleanType.getInstance(), CollectionsHelper.createList(
                        DoubleType.getInstance(), DoubleType.getInstance())) {

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
                    throws ModelCompilingException {

                IFunctionCallInternDeclaration childA = composedFunctionCallInternDeclaration
                        .getChild(0);
                IFunctionCallInternDeclaration childB = composedFunctionCallInternDeclaration
                        .getChild(1);

                if (IntegerType.getInstance().equals(
                        childA.getSpecificOutputType())
                        && IntegerType.getInstance().equals(
                                childB.getSpecificOutputType())) {
                    return new FunctionLessInt(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                } else {
                    return new FunctionLess(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                }
            }

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IFunctionList functions) {
                Assert.assertEquals(2, functions.size());
                return new FunctionLess(functions.getFunction(0), functions
                        .getFunction(1));
            }
        };
        functionDecl
                .setDocumentation("Returns the verity content if (Arg1) is smaller (Arg2) e.g. <(5, 5) = FALSE");
        functionDeclarationList.add(functionDecl);

        // >
        functionDecl = new AbstractSpecificFunctionArgsDeclaration(">",
                BooleanType.getInstance(), CollectionsHelper.createList(
                        DoubleType.getInstance(), DoubleType.getInstance())) {

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
                    throws ModelCompilingException {

                IFunctionCallInternDeclaration childA = composedFunctionCallInternDeclaration
                        .getChild(0);
                IFunctionCallInternDeclaration childB = composedFunctionCallInternDeclaration
                        .getChild(1);

                if (IntegerType.getInstance().equals(
                        childA.getSpecificOutputType())
                        && IntegerType.getInstance().equals(
                                childB.getSpecificOutputType())) {
                    return new FunctionGreaterInt(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                } else {
                    return new FunctionGreater(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                }
            }

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IFunctionList functions) {
                Assert.assertEquals(2, functions.size());
                return new FunctionGreater(functions.getFunction(0), functions
                        .getFunction(1));
            }
        };
        functionDecl
                .setDocumentation("Returns the verity content if (Arg1) is greater (Arg2) e.g. >(5, 5) = FALSE");
        functionDeclarationList.add(functionDecl);

        // <=
        functionDeclarationList.add(LOWER_EQUAL_FUNCTION_DECLARATION);

        // >=
        functionDecl = new AbstractSpecificFunctionArgsDeclaration(">=",
                BooleanType.getInstance(), CollectionsHelper.createList(
                        DoubleType.getInstance(), DoubleType.getInstance())) {

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
                    throws ModelCompilingException {

                IFunctionCallInternDeclaration childA = composedFunctionCallInternDeclaration
                        .getChild(0);
                IFunctionCallInternDeclaration childB = composedFunctionCallInternDeclaration
                        .getChild(1);

                if (IntegerType.getInstance().equals(
                        childA.getSpecificOutputType())
                        && IntegerType.getInstance().equals(
                                childB.getSpecificOutputType())) {
                    return new FunctionGreaterOrEqualInt(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                } else {
                    return new FunctionGreaterOrEqual(childA
                            .createFunction(createFunctionArgs), childB
                            .createFunction(createFunctionArgs));
                }
            }

            public IFunction createFunction(
                    ICreateFunctionArgs createFunctionArgs,
                    IFunctionList functions) {
                Assert.assertEquals(2, functions.size());
                return new FunctionGreaterOrEqual(functions.getFunction(0),
                        functions.getFunction(1));
            }
        };
        functionDecl
                .setDocumentation("Returns the verity content if (Arg1) is greater or equal (Arg2) e.g. >=(5, 5) = TRUE");
        functionDeclarationList.add(functionDecl);
    }

    private static void insertFunctionsIntern(
            IFunctionDeclarationList functionDeclarationList) {
        insertBoolOperatorFunctions(functionDeclarationList);
        insertCompareFunctions(functionDeclarationList);
    }
}