package de.uniwue.ki.mas.basics.function.bool;
import java.util.Iterator;
import java.util.List;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.math.MathHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public final class FunctionEqual extends AbstractBooleanFunction {
	private final IFunction _function1;
	private final IFunction _function2;

	FunctionEqual(IFunction function1, IFunction function2) {
		super();

		_function1 = function1;
		_function2 = function2;
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return isEqual(executeFunctionArgs, _function1, _function2);
	}

	public static final boolean isEqual(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunction function1,
		IFunction function2)
		throws SimulationRuntimeException {

		return isEqual(
			executeFunctionArgs,
			function1.execute(executeFunctionArgs),
			function2.execute(executeFunctionArgs));
	}

	public static boolean isEqual(IExecuteFunctionArgs executeFunctionArgs, Object obj1, Object obj2)
		throws SimulationRuntimeException {

		if (obj1 instanceof Iterator) {
			if (obj2 instanceof Iterator) {
				return CollectionsHelper.isEqual((Iterator) obj1, (Iterator) obj2, true);
			} else if (obj2 instanceof List) {
				return CollectionsHelper.isEqual((Iterator) obj1, (List) obj2, true);
			} else {
				return false;
			}

		} else if (obj1 instanceof List) {
			if (obj2 instanceof Iterator) {
				return CollectionsHelper.isEqual((List) obj1, (Iterator) obj2, true);
			} else if (obj2 instanceof List) {
				return CollectionsHelper.isEqual((List) obj1, (List) obj2, true);
			} else {
				return false;
			}

		} else if (obj1 instanceof Number) {
			return MathHelper.doubleEqual(((Number) obj1).doubleValue(), ((Number) obj2).doubleValue());

		} else {
			return Utils.equals(obj1, obj2);
		}
	}

	public String getName() {
		return "(" + _function1.getName() + " == " + _function2.getName() + ")";
	}
}