package de.uniwue.ki.mas.basics.function.bool;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionOr2Args extends AbstractBooleanFunction {
	private final IFunction _functionA;
	private final IFunction _functionB;

	public FunctionOr2Args(IFunction functionA, IFunction functionB) {
		super();

		_functionA = functionA;
		_functionB = functionB;
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForBoolean(_functionA, _functionB, executeFunctionArgs);
	}

	public static final boolean executeForBoolean(
		IFunction functionA,
		IFunction functionB,
		IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		if (functionB instanceof IArgument) {
			return ((IArgument) functionB).executeForBoolean(executeFunctionArgs)
				|| functionA.executeForBoolean(executeFunctionArgs);

		} else {
			return functionA.executeForBoolean(executeFunctionArgs)
				|| functionB.executeForBoolean(executeFunctionArgs);
		}
	}

	public String getName() {
		return "Or(" + _functionA.getName() + ", " + _functionB.getName() + ")";
	}

	public IFunction getFunctionA() {
		return _functionA;
	}

	public IFunction getFunctionB() {
		return _functionB;
	}
}