package de.uniwue.ki.mas.basics.function.bool;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;


/**
 * Created on 12.03.2004
 * @author herrler
 */
public class LowerEqualSpecificFunctionDeclaration extends
        AbstractSpecificFunctionArgsDeclaration {

    public LowerEqualSpecificFunctionDeclaration() {
        super("<=", BooleanType.getInstance(), CollectionsHelper.createList(
                DoubleType.getInstance(), DoubleType.getInstance()));
        setDocumentation("Returns the verity content if (Arg1) is smaller or equal (Arg2) e.g. <=(5, 5) = TRUE");
    }

    public IFunction createFunction(
            ICreateFunctionArgs createFunctionArgs,
            IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
            throws ModelCompilingException {

        IFunctionCallInternDeclaration childA = composedFunctionCallInternDeclaration
                .getChild(0);
        IFunctionCallInternDeclaration childB = composedFunctionCallInternDeclaration
                .getChild(1);

        if (IntegerType.getInstance().equals(childA.getSpecificOutputType())
                && IntegerType.getInstance().equals(
                        childB.getSpecificOutputType())) {
            return new FunctionLessOrEqualInt(childA
                    .createFunction(createFunctionArgs), childB
                    .createFunction(createFunctionArgs));
        } else {
            return new FunctionLessOrEqual(childA
                    .createFunction(createFunctionArgs), childB
                    .createFunction(createFunctionArgs));
        }
    }

    public IFunction createFunction(ICreateFunctionArgs createFunctionArgs,
            IFunctionList functions) {
        Assert.assertEquals(2, functions.size());
        return new FunctionLessOrEqual(functions.getFunction(0), functions
                .getFunction(1));
    }
}
