package de.uniwue.ki.mas.basics.function.bool;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 24.10.2002, 10:25:38
 * @author oechslein
 */
final class NotSpecificFunctionDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	public NotSpecificFunctionDeclaration() {
		super("Not", BooleanType.getInstance(), CollectionsHelper.createList(BooleanType.getInstance()));

		setDocumentation("Returns the negation of its operand e.g. Not(TRUE) = FALSE");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		IFunction booleanFunction = functions.getFunction(0);

		if (!createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()) {
			return new FunctionNot(booleanFunction);
		} else if (booleanFunction instanceof FunctionNot) {
			// Not(Not(x)) == x
			return ((FunctionNot) booleanFunction).getBooleanFunction();
		} else if (booleanFunction == BooleanType.BOOLEAN_TYPE_TRUE_ARGUMENT) {
			return BooleanType.BOOLEAN_TYPE_FALSE_ARGUMENT;
		} else if (booleanFunction == BooleanType.BOOLEAN_TYPE_FALSE_ARGUMENT) {
			return BooleanType.BOOLEAN_TYPE_TRUE_ARGUMENT;
		} else {
			return new FunctionNot(booleanFunction);
		}
	}

	// just for faster dynamic calling
	public boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {
		return !functions.getFunction(0).executeForBoolean(createAndExecuteFunctionArgs);
	}
}
