package de.uniwue.ki.mas.basics.function.bool;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (30.10.2001 19:48:16)
 * @author Christoph Oechslein
 */

public final class TestBooleanFunction extends TestCase {
	private final ICreateAndExecuteFunctionArgs _executeFunArgs = new CreateAndExecuteFunctionArgs();

	private final IFunctionDeclarationList functionDeclarationList =
		FunctionDeclarationListManager.getAllFunctionDeclarationList();

	public TestBooleanFunction(String name) {
		super(name);
	}

	public void checkFunction(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List functionList,
		String message,
		boolean expectedValue)
		throws Exception {

		assertNotNull("FunctionDeclaration not found", specificFunctionDeclaration);

		IFunction resultFunction =
			FunctionCallDeclarationFactory
				.getInstance()
				.createComposedFunctionCallInternDeclaration(null, specificFunctionDeclaration, functionList)
				.createFunction(_executeFunArgs);

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(message, expectedValue, resultFunction.executeForBoolean(_executeFunArgs));
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestBooleanFunction.class);
	}

	public void testAND() throws Exception {
		ISpecificFunctionDeclaration aNdDecl =
			functionDeclarationList.getSpecificFunctionDeclaration("AND", BooleanType.getInstance());

		checkFunction(aNdDecl, createAtomicFunCallList(true, true), "TRUE, TRUE", true && true);
		checkFunction(aNdDecl, createAtomicFunCallList(false, true), "FALSE, TRUE", false && true);
		checkFunction(aNdDecl, createAtomicFunCallList(true, false), "TRUE, FALSE", true && false);
		checkFunction(aNdDecl, createAtomicFunCallList(false, false), "FALSE, FALSE", false && false);
	}

	public void testEqual() throws Exception {
		Iterator it = TypeManager.getInstance().getAllTypes();
		while (it.hasNext()) {
			IType type = (IType)it.next();
			if (type instanceof ISpecificType) {
				if (type.equals(ActivityType.getInstance())
					|| type.equals(VoidType.getInstance())
					|| type.equals(SimulationDeclarationType.getInstance()))
					continue;

				ISpecificType specificType = (ISpecificType)type;
				IFunctionCallInternDeclaration defaultValueFunction =
					specificType.createDefaultFunctionCallInternDeclaration();

				ISpecificFunctionDeclaration equalDecl =
					functionDeclarationList.getSpecificFunctionDeclaration(
						"Equal",
						BooleanType.getInstance(),
						CollectionsHelper.createList(specificType, specificType));

				checkFunction(
					equalDecl,
					CollectionsHelper.createList(defaultValueFunction, defaultValueFunction),
					"defaultValue equal defaultValue of Type: " + specificType.getName(),
					true);
			}
		}

		ISpecificFunctionDeclaration boolEqualDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"Equal",
				BooleanType.getInstance(),
				CollectionsHelper.createList(BooleanType.getInstance(), BooleanType.getInstance()));

		checkFunction(boolEqualDecl, createAtomicFunCallList(false, true), "!TRUE equal FALSE", false);
		checkFunction(boolEqualDecl, createAtomicFunCallList(true, true), "TRUE equal TRUE", true);

		ISpecificFunctionDeclaration numberEqualDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"Equal",
				BooleanType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));

		checkFunction(numberEqualDecl, createAtomicFunCallList(0, 0), "0 == 0", true);
		checkFunction(numberEqualDecl, createAtomicFunCallList(0, 1), "0 !0 1", false);
	}

	public void testGreater() throws Exception {
		ISpecificFunctionDeclaration greaterDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(">", BooleanType.getInstance());
		checkFunction(greaterDecl, createAtomicFunCallList(0, 0), "0 > 0", false);
		checkFunction(greaterDecl, createAtomicFunCallList(1, 0), "1 > 0", true);
		checkFunction(greaterDecl, createAtomicFunCallList(0, 1), "0 > 1", false);
	}

	public void testGreaterOrEqual() throws Exception {
		ISpecificFunctionDeclaration greaterOrEqualDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(">=", BooleanType.getInstance());

		checkFunction(greaterOrEqualDecl, createAtomicFunCallList(0, 0), "0 >= 0", true);
		checkFunction(greaterOrEqualDecl, createAtomicFunCallList(0, 1), "0 >= 1", false);
		checkFunction(greaterOrEqualDecl, createAtomicFunCallList(1, 0), "1 >= 0", true);
	}

	public void testLess() throws Exception {
		ISpecificFunctionDeclaration lessDecl =
			functionDeclarationList.getSpecificFunctionDeclaration("<", BooleanType.getInstance());

		checkFunction(lessDecl, createAtomicFunCallList(0, 0), "0 < 0", false);
		checkFunction(lessDecl, createAtomicFunCallList(0, 1), "0 < 1", true);
		checkFunction(lessDecl, createAtomicFunCallList(1, 0), "1 < 0", false);
	}

	public void testLessOrEqual() throws Exception {
		ISpecificFunctionDeclaration lessOrEqualDecl =
			functionDeclarationList.getSpecificFunctionDeclaration("<=", BooleanType.getInstance());

		checkFunction(lessOrEqualDecl, createAtomicFunCallList(0, 0), "0 <= 0", true);
		checkFunction(lessOrEqualDecl, createAtomicFunCallList(0, 1), "0 <= 1", true);
		checkFunction(lessOrEqualDecl, createAtomicFunCallList(1, 0), "1 <= 0", false);
	}

	public List createAtomicFunCallList(double value1, double value2) {
		return CollectionsHelper.createList(createAtomicFunCall(value1), createAtomicFunCall(value2));
	}

	public List createAtomicFunCallList(boolean value1, boolean value2) {
		return CollectionsHelper.createList(createAtomicFunCall(value1), createAtomicFunCall(value2));
	}

	private IAtomicFunctionCallInternDeclaration createAtomicFunCall(double value) {
		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
			null,
			DoubleType.getInstance(),
			new Double(value));
	}

	private IAtomicFunctionCallInternDeclaration createAtomicFunCall(boolean value) {
		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
			null,
			BooleanType.getInstance(),
			Boolean.valueOf(value));
	}

	public void testNOT() throws Exception {
		ISpecificFunctionDeclaration nOTDecl =
			functionDeclarationList.getSpecificFunctionDeclaration("NOT", BooleanType.getInstance());
		checkFunction(nOTDecl, CollectionsHelper.createList(createAtomicFunCall(true)), "FALSE", false);
		checkFunction(nOTDecl, CollectionsHelper.createList(createAtomicFunCall(false)), "TRUE", true);
	}

	public void testOR() throws Exception {
		ISpecificFunctionDeclaration oRDecl =
			functionDeclarationList.getSpecificFunctionDeclaration("OR", BooleanType.getInstance());
		checkFunction(oRDecl, createAtomicFunCallList(true, true), "TRUE, TRUE", true || true);
		checkFunction(oRDecl, createAtomicFunCallList(false, true), "FALSE, TRUE", false || true);
		checkFunction(oRDecl, createAtomicFunCallList(true, false), "TRUE, FALSE", true || false);
		checkFunction(oRDecl, createAtomicFunCallList(false, false), "FALSE, FALSE", false || false);
	}
}