package de.uniwue.ki.mas.basics.function.expertlist;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
  * Rainer Herrler
  */

final class PopAndReturnFirstGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {
	public PopAndReturnFirstGenericFunctionDeclaration() {
		super("Pop&ReturnFirst", GenericType.getInstance(), false);

		setExpertFunctionDeclaration(true);
		setDocumentation("Returns the first element of the list and deletes it.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
	
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return typeT;
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(0, index);

				return GenericListType.getInstance().createSpecificListType(typeT);
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionPopAndReturnFirst(functions.getFunction(0));
			}

			public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
				return false;
			}

			public boolean isReadOnly(
				ICreateFunctionArgs createFunctionArgs,
				IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
				throws ModelCompilingException {
				return false;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificListType))
			return null;

		SpecificListType specificListType = (SpecificListType) specificTypeA;

		return createFunctionDeclaration(specificListType.getListSpecificType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(List<" + outputType.getName() + ">)";
	}
}