package de.uniwue.ki.mas.basics.function.expertlist;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * @author Rainer Herrler
 */

final class RemoveDuplicatesGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {
	private static final RemoveDuplicatesGenericFunctionDeclaration _instance = new RemoveDuplicatesGenericFunctionDeclaration();

	public RemoveDuplicatesGenericFunctionDeclaration() {
		super("RemoveDuplicates", GenericIteratorType.getInstance());

		setDocumentation("Allows to remove all duplicates from a Set given by an Iterator");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		SpecificIteratorType outputSpecificSetType;

		if (outputType == ObjectType.getInstance()) {
			outputSpecificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			outputSpecificSetType = (SpecificIteratorType) outputType;
		}

		return new RemoveDuplicatesSpecificFunctionDeclaration(outputSpecificSetType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		
		ISpecificType specificTypeB = getNextSpecificType(inputArgs);
		if (inputArgs.hasNext()) return null;

		if (!(specificTypeB instanceof SpecificIteratorType))
			return null;

		SpecificIteratorType specificSetType = (SpecificIteratorType) specificTypeB;

		return createFunctionDeclaration(specificSetType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		
		SpecificIteratorType specificSetType;

		if (outputType == ObjectType.getInstance()) {
			specificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			specificSetType = (SpecificIteratorType) outputType;
		}

		return "("
			+ specificSetType.getName()
			+ ")";
	}

	public static final RemoveDuplicatesGenericFunctionDeclaration getInstance() {
		return _instance;
	}
}