package de.uniwue.ki.mas.basics.function.function;

import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))
  * T3 is unspecific
  */

final class BindFirstGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT3 = new UnconstraintDependentType("first type");

	public BindFirstGenericFunctionDeclaration() {
		super(
			"BindFirst",
			GenericFunctionType.getInstance(),
			CollectionsHelper.createList(typeDependentT3),
			false);

		setDocumentation("Returns a new one-argument function " +
			"which calls the given function two-argument function (Arg1). " +
			"The second argument is passed through and " +
			"the first argument is always the given value (Arg2)");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType typeSpecific) {
		return super.canCreateFunctionDeclarationWith(typeSpecific)
			&& (typeSpecific instanceof ISpecificFunctionType)
			&& (((ISpecificFunctionType)typeSpecific).getSpecificInputTypesLength() == 1);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))

		ISpecificFunctionType outputTypeFunction = (ISpecificFunctionType)outputType;

		ISpecificType typeT1 = outputTypeFunction.getSpecificOutputType();
		ISpecificType typeT2 = outputTypeFunction.getSpecificInputTypeAt(0);
		ISpecificType typeT3 = bindings.getBindingFor(typeDependentT3);
		ISpecificFunctionType inputFunction =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				typeT1,
				CollectionsHelper.createList(typeT3, typeT2));

		return createFunctionDeclaration(outputTypeFunction, typeT3, inputFunction);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))
		// T3 is unspecific

		ISpecificType typeT3 = getNextSpecificType(inputTypes);
		ISpecificType typeFunctionT1T3T2 = getNextSpecificType(inputTypes);

		if ((typeT3 == null) || (typeFunctionT1T3T2 == null) || inputTypes.hasNext())
			return null;

		if (!(typeFunctionT1T3T2 instanceof ISpecificFunctionType))
			return null;

		ISpecificFunctionType realTypeFunctionT1T3T2 = (ISpecificFunctionType)typeFunctionT1T3T2;

		if ((realTypeFunctionT1T3T2.getSpecificInputTypesLength() != 2)
			|| (!realTypeFunctionT1T3T2.getSpecificInputTypeAt(0).equals(typeT3)))
			return null;

		return createFunctionDeclaration(
			GenericFunctionType.getInstance().createSpecificFunctionType(
				realTypeFunctionT1T3T2.getSpecificOutputType(),
				CollectionsHelper.createList(realTypeFunctionT1T3T2.getSpecificInputTypeAt(1))),
			typeT3,
			realTypeFunctionT1T3T2);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final ISpecificFunctionType outputTypeFunction,
		final ISpecificType typeT3,
		final ISpecificFunctionType typeFunctionT1T3T2) {

		// bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))

		final ISpecificType typeT1 = outputTypeFunction.getSpecificOutputType();
		ISpecificType typeT2 = outputTypeFunction.getSpecificInputTypeAt(0);
		final List outputTypeArgumentList = CollectionsHelper.createList(typeT2);

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return outputTypeFunction;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return typeT3;
				} else if (index == 1) {
					return typeFunctionT1T3T2;
				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(2, functions.size());

				IFunction functionBindingObject = (IFunction)functions.get(0);
				IFunction functionFunction = (IFunction)functions.get(1);

				return new BindFirstFunction(
					functionBindingObject,
					functionFunction,
					typeT1,
					outputTypeArgumentList);
			}
		};
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(T3, <Fun(T3, T2) T1>)";

		} else {
			// bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))
			// T3 is unspecific

			ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)outputType;

			return "(T, "
				+ "<Fun(T, "
				+ specificFunctionType.getSpecificInputTypeAt(0).getName()
				+ ") "
				+ specificFunctionType.getSpecificOutputType().getName()
				+ ">)";
		}
	}

	protected String getFullSignatureOutputName() {
		return "<Fun(T2) T1>";
	}

}
