package de.uniwue.ki.mas.basics.function.function;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class BindSecondFunction extends AbstractFunctionDynamic {
	protected final IFunction _functionBindingObject;
	protected final IFunction _functionFunction;
	private final ISpecificType _typeT1;
	private final List _outputFunctionInputArgsList;

	public BindSecondFunction(
		IFunction functionBindingObject,
		IFunction functionFunction,
		ISpecificType typeT1,
		List outputFunctionInputArgsList) {

		_functionBindingObject = functionBindingObject;
		_functionFunction = functionFunction;
		_typeT1 = typeT1;
		_outputFunctionInputArgsList = outputFunctionInputArgsList;
	}

	public Object execute(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		return new AbstractSpecificFunctionArgsDeclaration(
			null,
			_typeT1,
			_outputFunctionInputArgsList,
			true) {

			private final TwoObjectFunctionList _twoObjectFunctionList =
				new TwoObjectFunctionList(null, _functionBindingObject);

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {

				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				try {
					_twoObjectFunctionList.setFunctionA(functions.getFunction(0));

					Object function = _functionFunction.execute(executeFunctionArgs);

					return AbstractSpecificFunctionType.getParametricFunction(function).createFunction(
						createFunctionArgs,
						_twoObjectFunctionList);

				} catch (SimulationRuntimeException exception) {
					throw new ModelCompilingException(exception);
				}
			}
		};
	}

	public String getName() {
		return "BindSecond(" + _functionBindingObject.getName() + ", '" + _functionFunction.getName() + "')";
	}
}