package de.uniwue.ki.mas.basics.function.function;
import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * bindSecond: <T1> function(<T3>) bindSecond (<T2>, <T1> function(<T3>,<T2>))
  * T2 is unspecific
  */

final class BindSecondGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("second type");

	public BindSecondGenericFunctionDeclaration() {
		super(
			"BindSecond",
			GenericFunctionType.getInstance(),
			CollectionsHelper.createList(typeDependentT2),
			true);
			
			setDocumentation("Returns a new one-argument function " +
				"which calls the given function two-argument function (Arg1). " +
				"The first argument is passed through and " +
				"the second argument is always the given value (Arg2)");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType typeSpecific) {
		return super.canCreateFunctionDeclarationWith(typeSpecific)
			&& (typeSpecific instanceof ISpecificFunctionType)
			&& (((ISpecificFunctionType) typeSpecific).getSpecificInputTypesLength() == 1);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final ISpecificFunctionType outputTypeFunction,
		final ISpecificType typeT2,
		final ISpecificFunctionType typeFunctionT1T3T2) {

		// bindSecond: <T1> function(<T3>) bindSecond (<T2>, <T1> function(<T3>,<T2>))
		// T2 is unspecific

		final ISpecificType typeT1 = outputTypeFunction.getSpecificOutputType();
		ISpecificType typeT3 = outputTypeFunction.getSpecificInputTypeAt(0);
		final List outputTypeArgumentList = CollectionsHelper.createList(typeT3);

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return outputTypeFunction;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return typeT2;
				} else if (index == 1) {
					return typeFunctionT1T3T2;
				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(2, functions.size());

				IFunction functionBindingObject = (IFunction) functions.get(0);
				IFunction functionFunction = (IFunction) functions.get(1);

				return new BindSecondFunction(
					functionBindingObject,
					functionFunction,
					typeT1,
					outputTypeArgumentList);
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// bindSecond: <T1> function(<T3>) bindSecond (<T2>, <T1> function(<T3>,<T2>))
		// T2 is unspecific

		ISpecificFunctionType outputTypeFunction = (ISpecificFunctionType) outputType;
		ISpecificType typeT1 = outputTypeFunction.getSpecificOutputType();
		ISpecificType typeT3 = outputTypeFunction.getSpecificInputTypeAt(0);
		ISpecificType typeT2 = bindings.getBindingFor(typeDependentT2);
		ISpecificFunctionType inputFunction =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				typeT1,
				CollectionsHelper.createList(typeT3, typeT2));

		return createFunctionDeclaration(outputTypeFunction, typeT2, inputFunction);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// bindSecond: <T1> function(<T3>) bindSecond (<T2>, <T1> function(<T3>,<T2>))
		// T2 is unspecific

		ISpecificType typeT2 = getNextSpecificType(inputTypes);
		ISpecificType typeFunctionT1T3T2 = getNextSpecificType(inputTypes);

		if ((typeT2 == null) || (typeFunctionT1T3T2 == null) || inputTypes.hasNext())
			return null;

		if (!(typeFunctionT1T3T2 instanceof ISpecificFunctionType))
			return null;

		ISpecificFunctionType realTypeFunctionT1T3T2 = (ISpecificFunctionType) typeFunctionT1T3T2;

		if ((realTypeFunctionT1T3T2.getSpecificInputTypesLength() != 2)
			|| (!realTypeFunctionT1T3T2.getSpecificInputTypeAt(1).equals(typeT2)))
			return null;

		return createFunctionDeclaration(
			GenericFunctionType.getInstance().createSpecificFunctionType(
				realTypeFunctionT1T3T2.getSpecificOutputType(),
				CollectionsHelper.createList(realTypeFunctionT1T3T2.getSpecificInputTypeAt(0))),
			typeT2,
			realTypeFunctionT1T3T2);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(T2, <Fun(T3, T2) T1>)";
			
		} else {

		// bindSecond: <T1> function(<T3>) bindSecond (<T2>, <T1> function(<T3>,<T2>))
		// T2 is unspecific

			ISpecificFunctionType specificFunctionType = (ISpecificFunctionType) outputType;

			return "(T, "
				+ "<Fun("
				+ specificFunctionType.getSpecificInputTypeAt(0).getName()
				+ ",T) "
				+ specificFunctionType.getSpecificOutputType().getName()
				+ ">)";
		}
	}
	
	protected String getFullSignatureOutputName() {
		return "<Fun(T3) T1>";
	}
}