package de.uniwue.ki.mas.basics.function.function;

import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author oechslein
 * 
 * Creation date: 03.06.2003
 */
final class RotateArgsFunction extends AbstractFunctionDynamic {
	private final IFunction _functionFunction;
	private final List _outputFunctionInputArgsList;
	private final ISpecificType _typeT1;

	public RotateArgsFunction(IFunction functionFunction, ISpecificFunctionType outputFunctionTypeT1T3T2) {
		super();

		_functionFunction = functionFunction;
		_outputFunctionInputArgsList =
			CollectionsHelper.createList(
				outputFunctionTypeT1T3T2.getSpecificInputTypeAt(0),
				outputFunctionTypeT1T3T2.getSpecificInputTypeAt(1));
		_typeT1 = outputFunctionTypeT1T3T2.getSpecificOutputType();
	}

	public String getName() {
		return "RotateArgs('" + _functionFunction.getName() + "')";
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		Object functionDeclaration = _functionFunction.execute(executeFunctionArgs);

		final IParametricFunction function =
			AbstractSpecificFunctionType.getParametricFunction(functionDeclaration);

		final TwoObjectFunctionList twoObjectFunctionList = new TwoObjectFunctionList();

		return new AbstractSpecificFunctionArgsDeclaration(
			null,
			_typeT1,
			_outputFunctionInputArgsList,
			true) {

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs2,
				IFunctionList functions2)
				throws ModelCompilingException {

				Assert.assertEquals(2, functions2.size());

				final IFunction t2ObjectFunction = (IFunction) functions2.get(0);
				// is obj of T2
				final IFunction t3ObjectFunction = (IFunction) functions2.get(1);
				// is obj of T3

				twoObjectFunctionList.setFunctionA(t3ObjectFunction);
				twoObjectFunctionList.setFunctionB(t2ObjectFunction);

				return function.createFunction(createFunctionArgs2, twoObjectFunctionList);
			}
		};
	}
}