package de.uniwue.ki.mas.basics.function.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))
 * T2, T3 are unspecific
 * Creation date: (27.10.2000 17:15:57)
 * @author Christoph Oechslein
 */

final class RotateArgsGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	public RotateArgsGenericFunctionDeclaration() {
		super("RotateArgs", GenericFunctionType.getInstance(), true);
		
		setDocumentation("Returns a new two-argument function" +
			" which calls the given two-argeumtn function (Arg1)" +
			" with swap arguments.");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType typeSpecific) {
		return super.canCreateFunctionDeclarationWith(typeSpecific)
			&& (typeSpecific instanceof ISpecificFunctionType)
			&& (((ISpecificFunctionType)typeSpecific).getSpecificInputTypesLength() == 2);

	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificFunctionType outputFunctionTypeT1T3T2,
		ISpecificFunctionType inputFunctionTypeT1T2T3) {

		// rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))

		return new RotateArgsSpecificFunctionDeclaration(
			this,
			outputFunctionTypeT1T3T2,
			inputFunctionTypeT1T2T3);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {

		// rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))

		SpecificFunctionType outputTypeFunction = (SpecificFunctionType)outputType;
		ISpecificType typeT1 = outputTypeFunction.getSpecificOutputType();
		ISpecificType typeT2 = outputTypeFunction.getSpecificInputTypeAt(1);
		ISpecificType typeT3 = outputTypeFunction.getSpecificInputTypeAt(0);

		ISpecificFunctionType inputFunction =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				typeT1,
				CollectionsHelper.createList(typeT2, typeT3));

		return createFunctionDeclaration(outputTypeFunction, inputFunction);
	}

	/**
	 * @param inputTypes Iterator
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))
		// T2, T3 are unspecific

		ISpecificType specificType = getNextSpecificType(inputTypes);

		if ((specificType == null) || inputTypes.hasNext())
			return null;

		if (!(specificType instanceof ISpecificFunctionType))
			return null;

		ISpecificFunctionType functionT1T2T3 = (ISpecificFunctionType)specificType;

		if ((functionT1T2T3.getSpecificInputTypesLength() != 2))
			return null;

		ISpecificFunctionType outputSpecificFunctionType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				functionT1T2T3.getSpecificOutputType(),
				CollectionsHelper.createList(
					functionT1T2T3.getSpecificInputTypeAt(1),
					functionT1T2T3.getSpecificInputTypeAt(0)));

		return createFunctionDeclaration(outputSpecificFunctionType, functionT1T2T3);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(<Fun(T1,T2) T3)";

		} else {
			// rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))

			SpecificFunctionType specificFunctionType = (SpecificFunctionType)outputType;

			return "(<Fun("
				+ specificFunctionType.getSpecificInputTypeAt(1).getName()
				+ ", "
				+ specificFunctionType.getSpecificInputTypeAt(0).getName()
				+ ") "
				+ specificFunctionType.getSpecificOutputType()
				+ ">)";
		}
	}

	protected String getFullSignatureOutputName() {
		return "<Fun(T2,T1) T3";
	}
}