package de.uniwue.ki.mas.basics.function.function;
import junit.framework.Assert;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (17.09.2001 18:12:28)
 * @author Christoph Oechslein
 */

public final class TestFunctionFunction extends TestCase {
	private final ICreateAndExecuteFunctionArgs _executeFunArgs = new CreateAndExecuteFunctionArgs();
	private final IFunctionDeclarationList functionDeclarationList =
		FunctionDeclarationListManager.getAllFunctionDeclarationList();

	/**
	 * @param name String
	 */
	public TestFunctionFunction(String name) {
		super(name);
	}

	private void checkFunction(IFunction resultFunction, String message, double expectedValue)
		throws Exception {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(
				message,
				expectedValue,
				resultFunction.executeForDouble(_executeFunArgs),
				0.0001);
	}

	private void checkFunction(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		IFunctionList functionList,
		String message,
		double expectedValue)
		throws Exception {
		checkFunction(createFunction(specificFunctionDeclaration, functionList), message, expectedValue);
	}

	private IFunction createFunction(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		IFunctionList functionList)
		throws Exception {

		return specificFunctionDeclaration.createParametricFunction(_executeFunArgs).createFunction(
			_executeFunArgs,
			functionList);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestFunctionFunction.class);
	}

	public void testBindFirst() throws Exception {
		// bindFirst: <T1> function(<T2>) bindFirst (<T3>, <T1> function(<T3>,<T2>))
		ISpecificType numberType = DoubleType.getInstance();

		ISpecificType numberFunctionTwoArgsType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				numberType,
				CollectionsHelper.createList(numberType, numberType));
		ISpecificFunctionDeclaration bindFirstDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"bindFirst",
				GenericFunctionType.getInstance().createSpecificFunctionType(
					numberType,
					CollectionsHelper.createList(numberType)),
				CollectionsHelper.createList(numberType, numberFunctionTwoArgsType));

		assertNotNull(bindFirstDecl);

		ISpecificFunctionDeclaration minusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"-",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));

		assertNotNull(minusDecl);

		IArgumentDeclaration number5ArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				DoubleType.getInstance(),
				new Double(5.0));
		IArgumentDeclaration minusFunctionArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				numberFunctionTwoArgsType,
				minusDecl);

		IArgument number5Argument = number5ArgumentDeclaration.createArgument(_executeFunArgs);

		IArgument minusFunctionArgument = minusFunctionArgumentDeclaration.createArgument(_executeFunArgs);

		createFunction(bindFirstDecl, new FunctionList(number5Argument, minusFunctionArgument));
	}

	/**
	 * @throws Exception
	 */
	public void testBindSecond() throws Exception {
		// bindSecond: <T1> function(<T2>) bindSecond (<T3>, <T1> function(<T2>,<T3>))
		ISpecificType numberType = DoubleType.getInstance();
		ISpecificType numberFunctionTwoArgsType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				numberType,
				CollectionsHelper.createList(numberType, numberType));

		ISpecificFunctionDeclaration bindSecondDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"bindSecond",
				GenericFunctionType.getInstance().createSpecificFunctionType(
					numberType,
					CollectionsHelper.createList(numberType)),
				CollectionsHelper.createList(numberType, numberFunctionTwoArgsType));
		assertNotNull(bindSecondDecl);

		ISpecificFunctionDeclaration minusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"-",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		assertNotNull(minusDecl);

		IArgumentDeclaration number5ArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				DoubleType.getInstance(),
				new Double(5.0));
		IArgumentDeclaration minusFunctionArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				numberFunctionTwoArgsType,
				minusDecl);
		IArgument number5Argument = number5ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument minusFunctionArgument = minusFunctionArgumentDeclaration.createArgument(_executeFunArgs);

		createFunction(bindSecondDecl, new FunctionList(number5Argument, minusFunctionArgument));

	}

	/**
	 * @throws Exception
	 */
	public void testRotateArgs() throws Exception {
		// rotateArgs: <T1> function(<T3>, <T2>) rotateArgs (<T1> function(<T2>, <T3>))
		ISpecificType numberType = DoubleType.getInstance();
		ISpecificType numberFunctionTwoArgsType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				numberType,
				CollectionsHelper.createList(numberType, numberType));
		ISpecificFunctionDeclaration rotateArgsDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"rotateArgs",
				GenericFunctionType.getInstance().createSpecificFunctionType(
					numberType,
					CollectionsHelper.createList(numberType, numberType)),
				CollectionsHelper.createList(numberFunctionTwoArgsType));
		assertNotNull(rotateArgsDecl);
		ISpecificFunctionDeclaration minusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"-",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		assertNotNull(minusDecl);
		IArgumentDeclaration number5ArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				DoubleType.getInstance(),
				new Double(5.0));
		IArgumentDeclaration number5minusArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				DoubleType.getInstance(),
				new Double(-5.0));
		IArgumentDeclaration minusFunctionArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				numberFunctionTwoArgsType,
				minusDecl);
		IArgument number5Argument = number5ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number5minusArgument = number5minusArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument minusFunctionArgument = minusFunctionArgumentDeclaration.createArgument(_executeFunArgs);
		IFunction bindFunctionDeclarationFunction =
			createFunction(rotateArgsDecl, new FunctionList(minusFunctionArgument));
		ISpecificFunctionDeclaration rotatedminusDecl =
			(ISpecificFunctionDeclaration)bindFunctionDeclarationFunction.execute(_executeFunArgs);
		checkFunction(rotatedminusDecl, new FunctionList(number5Argument, number5Argument), "5 - 5", 5 - 5);
		checkFunction(
			rotatedminusDecl,
			new FunctionList(number5minusArgument, number5Argument),
			"-5 - 5",
			5 - -5);
		checkFunction(
			rotatedminusDecl,
			new FunctionList(number5Argument, number5minusArgument),
			"5 - -5",
			-5 - 5);
		checkFunction(
			rotatedminusDecl,
			new FunctionList(number5minusArgument, number5minusArgument),
			"-5 - -5",
			-5 - -5);
	}

	/**
	 * @throws Exception
	 */
	public void testTwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration() throws Exception {
		// TwoArgsToUnlimitedArgs: <T1> function(&rest <T1>) twoArgsToUnlimitedArgs (<T1> function(<T1>, <T1>)  , <T1>)

		final ISpecificType numberType = DoubleType.getInstance();
		final ISpecificType numberFunctionTwoArgsType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				numberType,
				CollectionsHelper.createList(numberType, numberType));

		ISpecificFunctionDeclaration twoArgFunctionToUnlimitedArgsGenericFunctionDeclaration =
			functionDeclarationList.getSpecificFunctionDeclaration(
				TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration.getInstance().getName(),
				GenericFunctionType.getInstance().createSpecificFunctionType(numberType, numberType),
				CollectionsHelper.createList(numberType, numberFunctionTwoArgsType));

		assertNotNull(twoArgFunctionToUnlimitedArgsGenericFunctionDeclaration);

		ISpecificFunctionDeclaration plusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"+",
				numberType,
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		assertNotNull(plusDecl);

		IArgument plusFunctionArgument = new Argument(plusDecl);

		IArgument number1Argument = new Argument(new Double(1.0));
		IArgument number3Argument = new Argument(new Double(3.0));
		IArgument number5Argument = new Argument(new Double(5.0));

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			(ISpecificFunctionDeclaration)twoArgFunctionToUnlimitedArgsGenericFunctionDeclaration
				.createParametricFunction(_executeFunArgs)
				.execute(_executeFunArgs, new FunctionList(number5Argument, plusFunctionArgument));

		checkFunction(specificFunctionDeclaration, IFunctionList.EMPTY_FUNCTION_LIST, "5", 5.0);
		checkFunction(specificFunctionDeclaration, new FunctionList(number1Argument), "1 + 5", 1.0 + 5.0);
		checkFunction(
			specificFunctionDeclaration,
			new FunctionList(number1Argument, number3Argument),
			"1 + 3 + 5",
			1.0 + 3.0 + 5.0);
	}
}