package de.uniwue.ki.mas.basics.function.function;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * // TwoArgsToUnlimitedArgs: <T1> function(&rest <T1>) twoArgsToUnlimitedArgs (<T1> function(<T1>, <T1>)  , <T1>)
 * Creation date: (17.07.2001 10:30:42)
 * @author Christoph Oechslein
 */

final class TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	public static final TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration _instance =
		new TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration();

	public TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration() {
		super("TwoArgsToUnlimArgsFunction", GenericFunctionType.getInstance(), true);

		setDocumentation(
			"Returns an unlimited argument function. "
				+ "If this function gets called with no argument, the default value (Arg1) is returned. "
				+ "If this function gets called with one argument, "
				+ "the given function is called with default value and this argument. "
				+ "If this function gets called with n arguments, "
				+ "the given function is called with default value and the first argument, "
				+ "the function is called with the result of this call and the next argument ...");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType typeSpecific) {
		if (!super.canCreateFunctionDeclarationWith(typeSpecific)) {
			return false;
		}

		if (!(typeSpecific instanceof ISpecificFunctionType)) {
			return false;
		}

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)typeSpecific;

		if (specificFunctionType.getSpecificInputTypesLength()
			!= ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return false;
		}

		return true;

		/*
		} else {
		   return specificFunctionType.getSpecificOutputType().isSimpleType()
		      && specificFunctionType.getSpecificInputTypeAt(0).isSimpleType();
		}
		*/
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclarationSpecificFunctionType((ISpecificFunctionType)outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificType1 = getNextSpecificType(inputTypes);
		ISpecificType specificType2 = getNextSpecificType(inputTypes);

		if ((specificType2 == null) || (specificType1 == null) || (inputTypes.hasNext())) {
			return null;

		} else if (!(specificType2 instanceof ISpecificFunctionType)) {
			return null;

		} else {
			ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)specificType2;
			if ((!specificFunctionType.getSpecificOutputType().equals(specificType1))
				|| (specificFunctionType.getSpecificInputTypesLength() != 2)) {
				return null;

			} else if (
				(!specificFunctionType.getSpecificInputTypeAt(0).equals(specificType1))
					|| (!specificFunctionType.getSpecificInputTypeAt(1).equals(specificType1))) {
				return null;

			} else {
				return createFunctionDeclarationSpecificFunctionType(
					GenericFunctionType.getInstance().createSpecificFunctionType(
						specificType1,
						specificType1));
			}
		}
	}

	private ISpecificFunctionDeclaration createFunctionDeclarationSpecificFunctionType(ISpecificFunctionType outputType) {
		return new TwoArgFunctionToUnlimitedArgsSpecificFunctionDeclaration(this, outputType);
	}

	public static final TwoArgFunctionToUnlimitedArgsGenericFunctionDeclaration getInstance() {
		return _instance;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(<Fun(T, T) T>, T)";
		} else {
			String name = outputType.getName();
			return "(<Fun(" + name + ", " + name + ") " + name + ">)";
		}
	}

	protected String getFullSignatureOutputName() {
		return "T";
	}
}