package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * void ClearHashtable(<T1>, <T2>, HashTable<T1,T2>)
 */

final class ClearHashtableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _keyType =
		new UnconstraintDependentType("Type of key", StringType.getInstance());
	private static final IDependentType _valueType =
		new UnconstraintDependentType("Type of value", ObjectType.getInstance());

	public ClearHashtableGenericFunctionDeclaration() {
		super("ClearHashtable", VoidType.getInstance(), CollectionsHelper.createList(_keyType, _valueType));

		setDocumentation("Clears hashtable (Arg1).");
		setExpertFunctionDeclaration(true);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		Assert.assertEquals(VoidType.getInstance(), outputType);

		return createFunctionDeclaration(
			GenericHashtableType.getInstance().createSpecificHashtableType(
				bindings.getBindingFor(_keyType),
				bindings.getBindingFor(_valueType)));
	}

	private final ISpecificFunctionDeclaration createFunctionDeclaration(SpecificHashtableType specificHashtableType) {
		return new ClearHashtableSpecificFunctionDeclaration(this, specificHashtableType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType hashtableType = getNextSpecificType(inputTypes);

		if ((hashtableType == null) || (inputTypes.hasNext()))
			return null;

		if (!(hashtableType instanceof SpecificHashtableType))
			return null;

		return createFunctionDeclaration((SpecificHashtableType) hashtableType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Hashtable< keyT, valueT >)";
	}
}