package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Map;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

final class ContainsKeyFunction extends AbstractBooleanFunction {

	private final IFunction _keyValueFunction;
	private final IFunction _hashTableFunction;

	public ContainsKeyFunction(IFunction keyValueFunction, IFunction hashTableFunction) {
		super();

		_keyValueFunction = keyValueFunction;
		_hashTableFunction = hashTableFunction;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object key = _keyValueFunction.execute(executeFunctionArgs);
		Map map = (Map) _hashTableFunction.execute(executeFunctionArgs);

		if (map == null)
			throw new SimulationRuntimeException("Hashtable is null in " + getName());

		return map.containsKey(key);
	}

	public String getName() {
		return "ContainsKey(" + _keyValueFunction.getName() + ", " + _hashTableFunction.getName() + ")";
	}
}