package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * <T2> GetValue(<T1>, HashTable<T1,T2>)
 */

final class ContainsKeyOrValueGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _keyType =
		new UnconstraintDependentType("Type of key", StringType.getInstance());
	private static final IDependentType _valueType =
		new UnconstraintDependentType("Type of value", ObjectType.getInstance());

	static final int CONTAINS_KEY = 0;
	static final int CONTAINS_VALUE = 1;

	private final int _type;

	public ContainsKeyOrValueGenericFunctionDeclaration(int type) {
		super(
			ContainsKeyOrValueGenericFunctionDeclaration.getNameForType(type),
			BooleanType.getInstance(),
			CollectionsHelper.createList(_keyType, _valueType));

		setExpertFunctionDeclaration(true);
		setDocumentation(getDocumentationForType(type));
		_type = type;
	}

	static final String getNameForType(int type) {
		switch (type) {
			case CONTAINS_KEY :
				return "ContainsKey";

			case CONTAINS_VALUE :
				return "ContainsValue";

			default :
				Assert.fail();
				return "";
		}
	}

	private static final String getDocumentationForType(int type) {
		switch (type) {
			case CONTAINS_KEY :
				return "Returns the verity content if Hashtable (Arg2) contains key (Arg1).";

			case CONTAINS_VALUE :
				return "Returns the verity content if Hashtable (Arg2) contains value (Arg1).";

			default :
				Assert.fail();
				return "";
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		return new ContainsKeyOrValueSpecificFunctionDeclaration(
			this,
			GenericHashtableType.getInstance().createSpecificHashtableType(
				bindings.getBindingFor(_keyType),
				bindings.getBindingFor(_valueType)));
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(SpecificHashtableType specificHashtableType) {
		return new ContainsKeyOrValueSpecificFunctionDeclaration(this, specificHashtableType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType keyOrValueType = getNextSpecificType(inputTypes);
		ISpecificType hashtableType = getNextSpecificType(inputTypes);

		if ((keyOrValueType == null) || (hashtableType == null) || (inputTypes.hasNext()))
			return null;

		if (!(hashtableType instanceof SpecificHashtableType))
			return null;

		SpecificHashtableType specificHashtableType = (SpecificHashtableType) hashtableType;

		if ((getType() == CONTAINS_KEY)
			&& TypeManager.getInstance().isMoreGeneral(specificHashtableType.getKeyType(), keyOrValueType)) {
			return createFunctionDeclaration(specificHashtableType);

		} else if (
			TypeManager.getInstance().isMoreGeneral(specificHashtableType.getValueType(), keyOrValueType)) {
			return createFunctionDeclaration(specificHashtableType);
		}

		return null;

	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (getType() == CONTAINS_KEY) {
			return "(keyT, Hashtable< keyT, valueT)";
		} else {
			return "(valueT, Hashtable< keyT, valueT)";
		}
	}

	public int getType() {
		return _type;
	}

}