package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Map;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

final class ContainsValueFunction extends AbstractBooleanFunction {

	private final IFunction _valueValueFunction;
	private final IFunction _hashTableFunction;

	public ContainsValueFunction(IFunction valueValueFunction, IFunction hashTableFunction) {
		super();

		_valueValueFunction = valueValueFunction;
		_hashTableFunction = hashTableFunction;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object value = _valueValueFunction.execute(executeFunctionArgs);
		Map map = (Map) _hashTableFunction.execute(executeFunctionArgs);

		if (map == null)
			throw new SimulationRuntimeException("Hashtable is null in " + getName());

		return map.containsValue(value);
	}

	public String getName() {
		return "ContainsValue(" + _valueValueFunction.getName() + ", " + _hashTableFunction.getName() + ")";
	}
}