package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.HashMap;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * HashTable<T1,T2> createHashTable()
 */

final class CreateHashtableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	public CreateHashtableGenericFunctionDeclaration() {
		super("CreateHashtable", GenericHashtableType.getInstance());

		setDocumentation("Creates an empty hashtable.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclaration((SpecificHashtableType)outputType);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final SpecificHashtableType outputType) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 0) {

			public ISpecificType getSpecificOutputType() {
				return outputType;
			}
			
			public boolean equals(Object arg0) {
				if (arg0==null) return false;
				
				if (!(arg0 instanceof AbstractSpecificFunctionAdapterGenericFunctionDeclaration))
				return false;
				
				return ((AbstractSpecificFunctionAdapterGenericFunctionDeclaration)arg0).getSpecificOutputType().equals(getSpecificOutputType());
			}
			
			public int hashCode() {
				return getSpecificOutputType().hashCode();
			}
			
			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {

				Assert.assertTrue(functions.isEmpty());

				return new AbstractFunctionDynamic() {
					public Object execute(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {

						return new HashMap();
					}
				};
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				Assert.fail();
				return null;
			}

			public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
				return false;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (inputTypes.hasNext() || !(outputType instanceof SpecificHashtableType))
			return null;
		else
			return createFunctionDeclaration((SpecificHashtableType)outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException();
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "()";
	}

	protected String getFullSignatureOutputName() {
		return "Hashtable<T1,T2>";
	}
}