package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * List < <T2> > GetKeysFromHashtable(HashTable<T1,T2>)
 */

final class GetKeysFromHashtableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _valueType = new UnconstraintDependentType("Type of value");

	public GetKeysFromHashtableGenericFunctionDeclaration() {
		super(
			"GetKeysFromHashtable",
			GenericIteratorType.getInstance(),
			CollectionsHelper.createList(_valueType));

		setExpertFunctionDeclaration(true);
		setDocumentation("Returns all keys in hashtable (Arg1).");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		SpecificIteratorType outputSpecificSetType;

		if (outputType == ObjectType.getInstance()) {
			outputSpecificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			outputSpecificSetType = (SpecificIteratorType) outputType;
		}

		return createFunctionDeclaration(
			GenericHashtableType.getInstance().createSpecificHashtableType(
				outputSpecificSetType.getIteratorSpecificType(),
				bindings.getBindingFor(_valueType)));
	}

	private final ISpecificFunctionDeclaration createFunctionDeclaration(SpecificHashtableType specificHashtableType) {
		return new GetKeysFromHashtableSpecificFunctionDeclaration(this, specificHashtableType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType hashtableType = getNextSpecificType(inputTypes);

		if ((hashtableType == null) || (inputTypes.hasNext()))
			return null;

		if (!(hashtableType instanceof SpecificHashtableType))
			return null;

		SpecificHashtableType specificHashtableType = (SpecificHashtableType) hashtableType;

		return createFunctionDeclaration(specificHashtableType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		ISpecificType innerType;

		if (outputType == ObjectType.getInstance()) {
			innerType =
				ObjectType.getInstance();
		} else {
			innerType = ((SpecificIteratorType) outputType).getIteratorSpecificType();
		}

		return "(Hashtable<" + innerType.getName() + ", valueT >)";
	}

}