package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * <T2> GetValue(<T1>, HashTable<T1,T2>)
 */

final class GetValueForKeyGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _keyType = new UnconstraintDependentType("Type of key");

	public GetValueForKeyGenericFunctionDeclaration() {
		super("GetValueForKey", GenericType.getInstance(), CollectionsHelper.createList(_keyType));

		setExpertFunctionDeclaration(true);
		setDocumentation("Returns the value for key for Hashtable.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		return createFunctionDeclaration(bindings.getBindingFor(_keyType), outputType);
	}

	private final ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType keyType,
		ISpecificType valueType) {

		return new GetValueForKeySpecificFunctionDeclaration(this, keyType, valueType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType keyType = getNextSpecificType(inputTypes);
		ISpecificType hashtableType = getNextSpecificType(inputTypes);

		if ((keyType == null) || (hashtableType == null) || (inputTypes.hasNext()))
			return null;

		if (!(hashtableType instanceof SpecificHashtableType))
			return null;

		SpecificHashtableType specificHashtableType = (SpecificHashtableType) hashtableType;

		if (!TypeManager.getInstance().isMoreGeneral(specificHashtableType.getKeyType(), keyType))
			return null;

		return createFunctionDeclaration(keyType, specificHashtableType.getValueType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(keyT, Hashtable< keyT, " + outputType.getName() + ">)";
	}

}