package de.uniwue.ki.mas.basics.function.hashtable;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * <T2> GetValue(<T1>, HashTable<T1,T2>)
 */

final class GetValueForKeySpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final SpecificHashtableType _specificHashtableType;

	public GetValueForKeySpecificFunctionDeclaration(
		GetValueForKeyGenericFunctionDeclaration getValueForKeyGenericFunctionDeclaration,
		ISpecificType keyType,
		ISpecificType valueType) {
		super(getValueForKeyGenericFunctionDeclaration, 2);

		_specificHashtableType =
			GenericHashtableType.getInstance().createSpecificHashtableType(keyType, valueType);
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		Assert.assertEquals(2, functions.size());

		return new GetValueForKeyFunction(functions.getFunction(0), functions.getFunction(1));
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return _specificHashtableType.getKeyType();

		} else if (index == 1) {
			return _specificHashtableType;

		} else {
			Assert.fail();
			return null;
		}
	}

	public ISpecificType getSpecificOutputType() {
		return _specificHashtableType.getValueType();
	}

	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0) {
			return "key";

		} else if (index == 1) {
			return "hashtable";

		} else {
			Assert.fail();
			return null;
		}
	}
}