package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * List < <T2> > GetValuesFromHashtable(HashTable<T1,T2>)
 */

final class GetValuesFromHashtableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _keyType = new UnconstraintDependentType("Type of key");

	public GetValuesFromHashtableGenericFunctionDeclaration() {
		super(
			"GetValuesFromHashtable",
			GenericIteratorType.getInstance(),
			CollectionsHelper.createList(_keyType));

		setExpertFunctionDeclaration(true);
		setDocumentation("Returns all values for hashtable (Arg1).");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

			SpecificIteratorType outputSpecificSetType;

			if (outputType == ObjectType.getInstance()) {
				outputSpecificSetType =
					GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
			} else {
				outputSpecificSetType = (SpecificIteratorType) outputType;
			}

		return createFunctionDeclaration(
			GenericHashtableType.getInstance().createSpecificHashtableType(
				bindings.getBindingFor(_keyType),
			outputSpecificSetType.getIteratorSpecificType()));
	}

	private final ISpecificFunctionDeclaration createFunctionDeclaration(SpecificHashtableType specificHashtableType) {
		return new GetValuesFromHashtableSpecificFunctionDeclaration(this, specificHashtableType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType hashtableType = getNextSpecificType(inputTypes);

		if ((hashtableType == null) || (inputTypes.hasNext()))
			return null;

		if (!(hashtableType instanceof SpecificHashtableType))
			return null;

		SpecificHashtableType specificHashtableType = (SpecificHashtableType) hashtableType;

		return createFunctionDeclaration(specificHashtableType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		ISpecificType innerType;

		if (outputType == ObjectType.getInstance()) {
			innerType =
				ObjectType.getInstance();
		} else {
			innerType = ((SpecificIteratorType) outputType).getIteratorSpecificType();
		}
		
		return "(Hashtable< keyT, " + innerType.getName() + ">)";
	}

}