package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Map;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

final class PutValueForKeyFunction extends AbstractFunctionAction {

	private final IFunction _keyValueFunction;
	private final IFunction _valueValueFunction;
	private final IFunction _hashTableFunction;

	public PutValueForKeyFunction(
		IFunction keyValueFunction,
		IFunction valueValueFunction,
		IFunction hashTableFunction) {
		super();

		_keyValueFunction = keyValueFunction;
		_valueValueFunction = valueValueFunction;
		_hashTableFunction = hashTableFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object key = _keyValueFunction.execute(executeFunctionArgs);
		Object value = _valueValueFunction.execute(executeFunctionArgs);
		Map map = (Map) _hashTableFunction.execute(executeFunctionArgs);

		if (map == null)
			throw new SimulationRuntimeException("Hashtable is null in " + getName());

		map.put(key, value);
	}

	public String getName() {
		return "PutValueForKey("
			+ _keyValueFunction.getName()
			+ ", "
			+ _valueValueFunction.getName()
			+ ", "
			+ _hashTableFunction.getName()
			+ ")";
	}
}
