package de.uniwue.ki.mas.basics.function.hashtable;

import java.util.Map;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

final class RemoveKeyFunction extends AbstractFunctionAction {

	private final IFunction _keyValueFunction;
	private final IFunction _hashTableFunction;

	public RemoveKeyFunction(IFunction keyValueFunction, IFunction hashTableFunction) {
		super();

		_keyValueFunction = keyValueFunction;
		_hashTableFunction = hashTableFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object key = _keyValueFunction.execute(executeFunctionArgs);
		Map map = (Map) _hashTableFunction.execute(executeFunctionArgs);

		if (map == null)
			throw new SimulationRuntimeException("Hashtable is null in " + getName());

		map.remove(key);
	}

	public String getName() {
		return "RemoveKey(" + _keyValueFunction.getName() + ", " + _hashTableFunction.getName() + ")";
	}
}
