package de.uniwue.ki.mas.basics.function.iterator;
import java.util.*;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.collection.IteratorWithException;
import de.uniwue.ki.util.functions.IMapFunction;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

abstract class AbstractIteratorFunctionContainer extends AbstractFunctionContainer {

	public AbstractIteratorFunctionContainer(IFunctionList functions) {
		super(functions);
	}

	protected abstract Object createFunction(IteratorWithException iterator) throws Exception;

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		try {
			IteratorWithException iterator =
				mapFunctionsToExecuteIterator(executeFunctionArgs, getFunctions());
			return createFunction(iterator);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}

	protected static IteratorWithException mapFunctionsToExecuteIterator(
		final IExecuteFunctionArgs executeFunctionArgs,
		Iterator functions)
		throws SimulationRuntimeException {

		if (!CollectionsHelper.DEFERRED_PROCESSING) {
			List result = new LinkedList();

			while (functions.hasNext()) {
				result.add(
					ExecuteIteratorFactory.createExecuteIterator(
						executeFunctionArgs,
						(IFunction)functions.next()));
			}

			return CollectionsHelper.asIteratorWithException(result);

		} else {
			final IArgumentBindings currArgumentBindings =
				executeFunctionArgs.saveCurrentArgumentBindings();

			IMapFunction mappingFunction = new IMapFunction() {
				public final Object map(Object obj) throws SimulationRuntimeException {

					executeFunctionArgs.pushArgumentBindings(currArgumentBindings);

					IteratorWithException result =
						ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, ((IFunction)obj));

					executeFunctionArgs.popArgumentBindings();

					return result;
				}
			};

			try {
				return CollectionsHelper.mapWithException(
					CollectionsHelper.asIteratorWithException(functions),
					mappingFunction);

			} catch (Exception e) {
				if (e instanceof SimulationRuntimeException)
					throw (SimulationRuntimeException)e;
				else if (e instanceof ModelCompilingException)
					throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
				else
					throw new RuntimeException(e);
			}
		}
	}
}