package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * Creation date: (16.01.2002 21:29:21)
 * @author Christoph Oechslein
 */

abstract class AbstractIteratorUnlimArgsGenericFunctionDeclaration
	extends AbstractIteratorGenericFunctionDeclaration {

	/**
	 * @param name String
	 */
	public AbstractIteratorUnlimArgsGenericFunctionDeclaration(String name) {
		super(name);
	}

	/**
	 * @param outputType ISpecificType
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclarationType(outputType);
	}

	/**
	 * @param inputArgs Iterator
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {

		SpecificIteratorType specificSetType = getSpecificIteratorType(inputArgs);

		if (specificSetType == null) {
			return null;
		}

		while (inputArgs.hasNext()) {

			SpecificIteratorType nextSpecificSetType = getSpecificIteratorType(inputArgs);

			if (!specificSetType.isMoreSpecific(nextSpecificSetType)
				&& !nextSpecificSetType.isMoreSpecific(specificSetType)) {
				return null;
			}
		}

		return createFunctionDeclarationType(specificSetType);
	}

	private ISpecificFunctionDeclaration createFunctionDeclarationType(final ISpecificType specificType) {
		final SpecificIteratorType outputType;
		if (specificType instanceof SpecificListType)
			outputType =
				GenericIteratorType.getInstance().createSpecificIteratorType(
					((SpecificListType) specificType).getListSpecificType());
		else
			outputType = (SpecificIteratorType) specificType;

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return outputType;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				return outputType;
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				return AbstractIteratorUnlimArgsGenericFunctionDeclaration.this.createFunction(
					createFunctionArgs,
					functions);
			}

			public int getSpecificInputMaxArgsLength() {
				return ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT;
			}

			public int getSpecificInputMinArgsLength() {
				return 1;
			}

		};
	}

	protected abstract IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionList functions);

	/**
	 * @param outputType ISpecificType
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
	 */
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(" + outputType.getName() + " ...)";
	}
}