package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * Creation date: (16.01.2002 21:29:38)
 * @author Christoph Oechslein
 */

abstract class AbstractTwoIteratorGenericFunctionDeclaration
	extends AbstractIteratorGenericFunctionDeclaration {

	public AbstractTwoIteratorGenericFunctionDeclaration(String name) {
		super(name);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		SpecificIteratorType specificSetTypeA = getSpecificIteratorType(inputArgs);
		SpecificIteratorType specificSetTypeB = getSpecificIteratorType(inputArgs);

		if ((specificSetTypeA == null) || (specificSetTypeB == null))
			return null;

		if (inputArgs.hasNext())
			return null;

		// a = iterator, b = iterator is ok -> out is iterator
		// a = list, b = iterator is ok -> out is iterator
		// a = iterator, b = list is ok -> out is iterator
		// a = list, b = list is ok -> out is iterator
		if (!specificSetTypeA.isMoreSpecific(specificSetTypeB)
			&& !specificSetTypeB.isMoreSpecific(specificSetTypeA))
			return null;

		if (specificSetTypeA instanceof SpecificListType)
			return createFunctionDeclaration(
				GenericIteratorType.getInstance().createSpecificIteratorType(
					specificSetTypeA.getParameterSpecificType()));
		else
			return createFunctionDeclaration(specificSetTypeA);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType outputType) {

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return outputType;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				return outputType;
			}
			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				final IFunctionList functions) {
				return AbstractTwoIteratorGenericFunctionDeclaration.this.createFunction(
					functions.getFunction(0),
					functions.getFunction(1));
			}
		};
	}

	protected abstract IFunction createFunction(IFunction functionA, IFunction functionB);

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(" + outputType.getName() + ", " + outputType.getName() + ")";
	}
}