package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * Creation date: (26.02.2002 20:08:58)
 * @author Christoph Oechslein
 */
public class Aggregate2ArgsFunGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	protected Aggregate2ArgsFunGenericFunctionDeclaration() {
		super("Aggregate2ArgsFun", GenericType.getInstance(), true);

		setDocumentation(
			"Applies a 2-argument-function (Arg1) to an iterator (Arg3) with a start value (Arg2), "
				+ "e.g. Aggregate2ArgsFun(Max, GetFirst(Iterator<SimObject>), GetRest) = Maximum");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return new Aggregate2ArgsFunSpecificFunctionDeclaration(this, outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType functionType = getNextSpecificType(inputTypes);
		ISpecificType startValueType = getNextSpecificType(inputTypes);
		ISpecificType iteratorType = getNextSpecificType(inputTypes);

		if ((functionType == null)
			|| (startValueType == null)
			|| (iteratorType == null)
			|| inputTypes.hasNext()) {
			return null;
		}

		if (!(functionType instanceof ISpecificFunctionType)
			|| !(iteratorType instanceof SpecificIteratorType)) {
			return null;
		}

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)functionType;

		if ((specificFunctionType.getSpecificOutputType() != startValueType)
			|| (specificFunctionType.getSpecificInputTypesLength() != 2)
			|| (!specificFunctionType
				.getSpecificInputTypeAt(0)
				.equals(specificFunctionType.getSpecificInputTypeAt(1)))) {
			return null;
		}

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(specificFunctionType.getSpecificInputTypeAt(0), startValueType)) {
			return null;
		}

		SpecificIteratorType specificIteratorType = ((SpecificIteratorType)iteratorType);

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(
				specificFunctionType.getSpecificInputTypeAt(0),
				specificIteratorType.getIteratorSpecificType())) {
			return null;
		}

		if (TypeManager
			.getInstance()
			.isMoreGeneral(specificIteratorType.getIteratorSpecificType(), startValueType))
			return new Aggregate2ArgsFunSpecificFunctionDeclaration(
				this,
				specificIteratorType.getIteratorSpecificType());
		else
			return new Aggregate2ArgsFunSpecificFunctionDeclaration(this, startValueType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(<Fun(T, T) T>, Iterator<T>)";
		} else {
			String name = outputType.getName();
			return "(<Fun(" + name + ", " + name + ") " + name + ">, Iterator<" + name + ">)";
		}
	}

	protected String getFullSignatureOutputName() {
		return "T";
	}
}