package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * aggregate: <T1> aggregate (<T1>functionUnlim(<T2>), set<T2>)
 * T2 is unspecific
 * Creation date: (04.09.2001 16:53:35)
 * @author Christoph Oechslein
 */

final class AggregateGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("Iterator");

	public AggregateGenericFunctionDeclaration() {
		super("Aggregate", GenericType.getInstance(), CollectionsHelper.createList(typeDependentT2), true);

		setDocumentation("Applies a function with unlimited arguments (Arg1) to a Set (Arg2) e.g. Aggregate(Max, Set<number>) returns the maximum of the given iterator.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// aggregate: <T1> aggregate (<T1>functionUnlim(<T2>), set<T2>)

		final ISpecificType typeT1 = outputType;
		final ISpecificType typeT2 = bindings.getBindingFor(typeDependentT2);

		return createFunctionDeclaration(
			typeT1,
			GenericFunctionType.getInstance().createSpecificFunctionType(typeT1, typeT2),
			GenericIteratorType.getInstance().createSpecificIteratorType(typeT2));
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final ISpecificType outputTypeT1,
		final SpecificFunctionUnlimitedType functionTypeT1T2,
		final SpecificIteratorType setTypeT2) {
		// aggregate: <T1> aggregate (<T1>functionUnlim(<T2>), set<T2>)

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return outputTypeT1;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return functionTypeT1T2;
				} else if (index == 1) {
					return setTypeT2;
				} else {
					Assert.fail();
					return null;
				}

			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(2, functions.size());
				return new FunctionIteratorAggregate(
					(IFunction) functions.get(0),
					(IFunction) functions.get(1));
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		// aggregate: <T1> aggregate (<T1>functionUnlim(<T2>), set<T2>)
		// T2 is unspecific

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeB instanceof SpecificIteratorType)
			|| !(specificTypeA instanceof SpecificFunctionUnlimitedType))
			return null;

		SpecificIteratorType setTypeT2 = (SpecificIteratorType) specificTypeB;
		SpecificFunctionUnlimitedType functionTypeT1T2 = (SpecificFunctionUnlimitedType) specificTypeA;

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(functionTypeT1T2.getSpecificInputType(), setTypeT2.getIteratorSpecificType()))
			return null;

		return createFunctionDeclaration(
			functionTypeT1T2.getSpecificOutputType(),
			functionTypeT1T2,
			setTypeT2);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		// aggregate: <T1> aggregate (<T1>functionUnlim(<T2>), set<T2>)
		// T2 is unspecific

		return "(Fun<(T ...) " + outputType.getName() + "> , Iterator<T>)";
	}
}