package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * AsList: list<T> asList(iterator<T>)

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class AsIteratorGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	private static final AsIteratorGenericFunctionDeclaration _instance =
		new AsIteratorGenericFunctionDeclaration();

	public static final AsIteratorGenericFunctionDeclaration getInstance() {
		return _instance;
	}

	private AsIteratorGenericFunctionDeclaration() {
		super("AsIterator", GenericIteratorType.getInstance());

		setDocumentation("Creates an iterator from a list");
	}

	public final ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType))
			return createAsIteratorSpecificFunctionDeclaration(
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance()));
		else
			return createAsIteratorSpecificFunctionDeclaration((SpecificIteratorType) outputType);
	}

	private final ISpecificFunctionDeclaration createAsIteratorSpecificFunctionDeclaration(SpecificIteratorType specificListType) {
		return new AsIteratorSpecificFunctionDeclaration(specificListType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		ISpecificType specificType = getNextSpecificType(inputArgs);

		if ((specificType == null) || inputArgs.hasNext())
			return null;

		if (!(specificType instanceof SpecificListType))
			return null;

		SpecificListType specificListType = (SpecificListType) specificType;

		return createAsIteratorSpecificFunctionDeclaration(
			GenericIteratorType.getInstance().createSpecificIteratorType(
				specificListType.getListSpecificType()));
	}

	public final String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (outputType instanceof SpecificListType)
			return "("
				+ GenericListType
					.getInstance()
					.createSpecificListType(((SpecificIteratorType) outputType).getIteratorSpecificType())
					.getName()
				+ ")";
		else
			return "("
				+ GenericListType.getInstance().createSpecificListType(ObjectType.getInstance()).getName()
				+ ")";
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}
}