package de.uniwue.ki.mas.basics.function.iterator;
import java.util.List;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.ITwoArgResultFunction;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class CartesianProductFunction extends AbstractFunction {
	private final IFunction _functionFunction;
	private final IFunction _listFunction1;
	private final IFunction _listFunction2;

	public CartesianProductFunction(
		IFunction functionFunction,
		IFunction listFunction1,
		IFunction listFunction2) {
		super();

		_functionFunction = functionFunction;
		_listFunction1 = listFunction1;
		_listFunction2 = listFunction2;
	}

	public Object execute(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object twoArgFunction = _functionFunction.execute(executeFunctionArgs);

		final IArgumentBindings currArgumentBindings =
			executeFunctionArgs.saveCurrentArgumentBindings();

		IExecuteIterator iterator1 =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _listFunction1);

		List list2 = ExecuteIteratorFactory.createList(executeFunctionArgs, _listFunction2);

		try {
			return CollectionsHelper.cartesianProductWithException(new ITwoArgResultFunction() {

				private final TwoObjectFunctionList _twoObjectFunctionList = new TwoObjectFunctionList();

				public Object execute(Object obj1, Object obj2)
					throws SimulationRuntimeException, ModelCompilingException {
					_twoObjectFunctionList.setFunctionObjectA(obj1);
					_twoObjectFunctionList.setFunctionObjectB(obj2);

					executeFunctionArgs.pushArgumentBindings(currArgumentBindings);

					Object result =
						AbstractSpecificFunctionType.getParametricFunction(twoArgFunction).execute(
							executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
							_twoObjectFunctionList);

					executeFunctionArgs.popArgumentBindings();

					return result;
				}
			}, iterator1, list2);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}

	public String getName() {
		return "CartesianProduct('"
			+ _functionFunction.getName()
			+ "', "
			+ _listFunction1.getName()
			+ ", "
			+ _listFunction2.getName()
			+ ")";
	}
}