package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * concatenate: set<T> concatenate (set<T> set<T>)

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class CartesianProductGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType typeDependentT1 = new UnconstraintDependentType("Iterator type 1");
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("Iterator type 2");

	public CartesianProductGenericFunctionDeclaration() {
		super(
			"CartesianProduct",
			GenericIteratorType.getInstance(),
			CollectionsHelper.createList(typeDependentT1, typeDependentT2),
			true);

		setDocumentation(
			"Produces the cartesian product iterator of the last two arguments. "
				+ "Elements are results of calling the function.");
		setExpertFunctionDeclaration(true);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		return createFunctionDeclaration(
			bindings.getBindingFor(typeDependentT1),
			bindings.getBindingFor(typeDependentT2),
			(SpecificIteratorType) outputType);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType specificInnerIteratorType1,
		ISpecificType specificInnerIteratorType2,
		SpecificIteratorType outputType) {

		return new CartesianProductSpecificFunctionDeclaration(
			specificInnerIteratorType1,
			specificInnerIteratorType2,
			outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificType = getNextSpecificType(inputTypes);
		ISpecificType specificInnerType1 = getInnerTypeIfPossible(getNextSpecificType(inputTypes));
		ISpecificType specificInnerType2 = getInnerTypeIfPossible(getNextSpecificType(inputTypes));

		if ((specificType == null) || (specificInnerType2 == null) || (specificInnerType1 == null)) {
			return null;
		}

		if (!(specificType instanceof ISpecificFunctionType)) {
			return null;
		}

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType) specificType;

		if (specificFunctionType.getSpecificInputTypesLength() != 2) {
			return null;
		}

		ISpecificType functionType1 = specificFunctionType.getSpecificInputTypeAt(0);
		ISpecificType functionType2 = specificFunctionType.getSpecificInputTypeAt(1);

		ISpecificType mostGeneralType1 = TypeManager.getInstance().getMostGeneralType(specificInnerType1, functionType1);

		ISpecificType mostGeneralType2 = TypeManager.getInstance().getMostGeneralType(specificInnerType2, functionType2);

		if ((mostGeneralType1 == null) || (mostGeneralType2 == null)) {
			return null;
		}

		return createFunctionDeclaration(
			mostGeneralType1,
			mostGeneralType2,
			GenericIteratorType.getInstance().createSpecificIteratorType(
				specificFunctionType.getSpecificOutputType()));
	}

	private static ISpecificType getInnerTypeIfPossible(ISpecificType specificType) {
		if (specificType instanceof SpecificIteratorType) {
			return ((SpecificIteratorType) specificType).getIteratorSpecificType();
		} else {
			return null;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		ISpecificType innerType;
		if (ObjectType.getInstance().equals(outputType)) {
			innerType = ObjectType.getInstance();
		} else {
			innerType = ((SpecificIteratorType) outputType).getIteratorSpecificType();
		}
		return "(Fun<(T1, T2) " + innerType.getName() + ">, Iterator<T1>, Iterator<T2>)";
	}
}