package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * count: Number count (set<T>)
 * T is unspecific
 * Creation date: (27.10.2000 17:49:45)
 *
 * @author Christoph Oechslein
 */

final class CountGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("Iterator");

	public CountGenericFunctionDeclaration() {
		super("Count", IntegerType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Returns the quantity of the iterator");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// count: Number count (set<T>)

		GenericIteratorType genericSetPrototype = GenericIteratorType.getInstance();
		ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(genericSetPrototype.createSpecificIteratorType(typeT));
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final SpecificIteratorType specificSetType) {

		// count: Number count (set<T>)

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return specificSetType;
				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());
				return new FunctionIteratorCount((IFunction)functions.get(0));
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// count: Number count (set<T>)

		ISpecificType specificType = getNextSpecificType(inputTypes);
		if ((specificType == null) || !(specificType instanceof SpecificIteratorType))
			return null;

		return createFunctionDeclaration((SpecificIteratorType)specificType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(Iterator<T>)";
			
		} else {
			String name = outputType.getName();
			return "(Iterator<" + name + ">)";
		}
	}
}