package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.AbstractFunction;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.IFunctionList;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.GenericListType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.SpecificListType;
import de.uniwue.ki.mas.basics.type.enum.GenericEnumType;
import de.uniwue.ki.mas.basics.type.enum.ISpecificEnumType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: (27.10.2000 17:49:45)
 * @author Rainer Herrler
 */

public class CreateEnumIteratorGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {
	
	private static final String NAME = "CreateEnumFromIterator";
	
	public class CreateEnumIteratorFunction extends AbstractFunction {
		private final ISpecificEnumType _type;
		public CreateEnumIteratorFunction(ISpecificEnumType type) {
			_type = type;
		}
		
		public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			return _type.getItemList();
		}};
	
	public class CreateEnumIteratorSpecificFunctionDeclaration extends AbstractSpecificFunctionArgsDeclaration {
		
		public CreateEnumIteratorSpecificFunctionDeclaration(ISpecificType enumListType) {
			super(NAME,enumListType, CollectionsHelper.EMPTY_LIST);
			Assert.assertNotNull(enumListType);
		}
		
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) throws ModelCompilingException {
			ISpecificEnumType type = (ISpecificEnumType)((SpecificListType)getOutputType()).getListSpecificType();
			return new CreateEnumIteratorFunction(type);
		}
		
		public IGenericFunctionDeclaration getGenericFunctionDeclaration() {
			return CreateEnumIteratorGenericFunctionDeclaration.getInstance();
		}
	};	

	private final static CreateEnumIteratorGenericFunctionDeclaration _instance= new CreateEnumIteratorGenericFunctionDeclaration();
	
	public static CreateEnumIteratorGenericFunctionDeclaration getInstance() {
		return _instance;
	}
	
	public CreateEnumIteratorGenericFunctionDeclaration() {
		super(NAME,GenericListType.getInstance());

		setDocumentation("Creates a new iterator with all elements of an enumeration");
	}
	
	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		if (outputType==null) return false;
		if (outputType.getGenericType()==null) return false;
		if (!outputType.getGenericType().equals(GenericListType.getInstance()))
			return false;
		if (!isEnumListType(outputType))
			return false;
		return true;
		}

	private boolean isEnumListType(ISpecificType outputType) {
		ISpecificType type = ((SpecificListType)outputType).getListSpecificType();
		return isEnumType(type);
	}

	private boolean isEnumType(ISpecificType type) {
		return (type.getGenericType().equals(GenericEnumType.getInstance()));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return new CreateEnumIteratorSpecificFunctionDeclaration(outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		Assert.fail("Cannot create "+getName()+" from input types");
		return null;
	}
	
	public ISpecificFunctionDeclaration createFunctionDeclaration(
			ISpecificType outputType, Iterator inputTypes) {
		return createFunctionDeclaration(outputType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "()";
	}
	
	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}
	
	public boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		return true;
	}

	
}
