package de.uniwue.ki.mas.basics.function.iterator;
import java.util.*;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class CreateIteratorGenericFunctionDeclaration extends AbstractIteratorGenericFunctionDeclaration {

	private transient final Map _cache = new WeakHashMap();

	public CreateIteratorGenericFunctionDeclaration() {
		super("CreateIterator");

		setDocumentation("Creates a new iterator");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		SpecificIteratorType outputSpecificSetType;

		if (outputType == ObjectType.getInstance()) {
			outputSpecificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			outputSpecificSetType = (SpecificIteratorType) outputType;
		}

		return createFunctionDeclarationIntern(outputSpecificSetType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (outputType == null) {
			return createFunctionDeclaration(inputTypes);

		} else if (!(outputType instanceof SpecificIteratorType)) {
			return null;

		} else {

			SpecificIteratorType specificIteratorType = (SpecificIteratorType) outputType;

			ISpecificType specificType = specificIteratorType.getIteratorSpecificType();

			while (inputTypes.hasNext()) {
				if (!(specificType.isMoreSpecific(getNextSpecificType(inputTypes)))) {
					return null;
				}
			}

			return createFunctionDeclaration(specificIteratorType);
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		if (!inputArgs.hasNext()) {
			return createFunctionDeclaration(
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance()));
		}

		ISpecificType specificType = TypeManager.getInstance().getMostGeneralType(inputArgs);

		if (specificType == null) {
			return null;
		} else {
			return createFunctionDeclaration(
				GenericIteratorType.getInstance().createSpecificIteratorType(specificType));
		}
	}

	private ISpecificFunctionDeclaration createFunctionDeclarationIntern(SpecificIteratorType specificIteratorType) {
		if (_cache.containsKey(specificIteratorType)) {
			return (CreateIteratorSpecificFunctionDeclaration) _cache.get(specificIteratorType);
		} else {
			CreateIteratorSpecificFunctionDeclaration createListSpecificFunctionDeclaration =
				new CreateIteratorSpecificFunctionDeclaration(this, 1, specificIteratorType);
			_cache.put(specificIteratorType, createListSpecificFunctionDeclaration);
			return createListSpecificFunctionDeclaration;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		ISpecificType innerType;

		if (outputType == ObjectType.getInstance()) {
			innerType = ObjectType.getInstance();
		} else {
			innerType = ((SpecificIteratorType) outputType).getIteratorSpecificType();
		}

		return "(" + innerType.getName() + " ...)";
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}

	// since length args can be zero	
	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		return composedFunctionCallInternDeclaration.getChildCount() == 0;
	}
}