package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionAggregate2ArgsFun extends AbstractFunctionDynamic {
	private final IFunction _functionFunction;
	private final IFunction _functionStart;
	private final IFunction _functionSet;

	private final TwoObjectFunctionList _twoObjectFunctionList = new TwoObjectFunctionList();

	public FunctionAggregate2ArgsFun(
		IFunction functionFunction,
		IFunction functionStart,
		IFunction functionSet) {
		super();

		_functionFunction = functionFunction;
		_functionStart = functionStart;
		_functionSet = functionSet;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object aggregateFunction = _functionFunction.execute(executeFunctionArgs);
		Object startValue = _functionStart.execute(executeFunctionArgs);

		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		try {

			Object value = startValue;

			while (setIterator.hasNext()) {
				_twoObjectFunctionList.setFunctionObjectA(value);
				_twoObjectFunctionList.setFunctionObjectB(setIterator.nextWithException());

				value =
					AbstractSpecificFunctionType.getParametricFunction(aggregateFunction).execute(
						createAndExecuteFunctionArgs,
						_twoObjectFunctionList);
			}

			return value;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public String getName() {
		return "Aggregate2ArgsFun('"
			+ _functionFunction.getName()
			+ "',"
			+ _functionStart.getName()
			+ ", "
			+ _functionSet.getName()
			+ ")";
	}
}