package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorAggregate extends AbstractFunctionDynamic {
	private final IFunction _functionFunction;
	private final IFunction _functionIterator;

	public String getName() {
		return "Aggregate('" + _functionFunction.getName() + "', " + _functionIterator.getName() + ")";
	}

	public FunctionIteratorAggregate(IFunction functionFunction, IFunction functionIterator) {
		super();
		this._functionFunction = functionFunction;
		this._functionIterator = functionIterator;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object aggregateFunction = _functionFunction.execute(executeFunctionArgs);

		IExecuteIterator executeIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionIterator);

		try {

			return AbstractSpecificFunctionType.getParametricFunction(aggregateFunction).execute(
				executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
				new ObjectMapFunctionList(executeIterator));

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}
}