package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorGetNth extends AbstractFunctionDynamic {
	private final IFunction _setFunction;
	private final IFunction _indexFunction;

	/**
	 * @param indexFunction IFunction
	 * @param setFunction IFunction
	 */
	public FunctionIteratorGetNth(IFunction indexFunction, IFunction setFunction) {
		super();

		_setFunction = setFunction;
		_indexFunction = indexFunction;
	}

	public String getName() {
		return "GetNth(" + _indexFunction.getName() + ", " + _setFunction.getName() + ")";
	}

	/**
	 * @param executeFunctionArgs IExecuteFunctionArgs
	 * @return Object
	 * @throws SimulationRuntimeException
	 * @see de.uniwue.ki.mas.basics.function.IFunction#execute(IExecuteFunctionArgs)
	 */
	public final Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IExecuteIterator it =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _setFunction);
		int index = (int) Math.floor(_indexFunction.executeForDouble(executeFunctionArgs));
		
		if (index < 0) {
			throw new SimulationRuntimeException("Index " + index + " must no be lower than zero " + getName());
		}

		Object result = null;

		while (index >= 0) {
			if (!it.hasNext()) {
				throw new EmptyIteratorSimulationRuntimeException("Index " + index + "to large for list in " + getName());
			}

			result = ExecuteIteratorHelper.nextExecute(it);

			index--;
		}

		return result;
	}
}