package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * Creation date: (30.07.2001 14:16:30)
 * @author Christoph Oechslein
 */

final class FunctionIteratorMapper extends AbstractFunction {
	private final IFunction _functionFunction;
	private final IFunction _functionSet;

	public String getName() {
		return "Map('" + _functionFunction.getName() + "', " + _functionSet.getName() + ")";
	}

	public FunctionIteratorMapper(IFunction functionFunction, IFunction functionSet) {
		super();

		_functionFunction = functionFunction;
		_functionSet = functionSet;
	}

	public Object execute(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object mapFunction = _functionFunction.execute(executeFunctionArgs);

		final IArgumentBindings currArgumentBindings =
			executeFunctionArgs.saveCurrentArgumentBindings();

		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		try {
			return CollectionsHelper.mapWithException(setIterator, new IMapFunction() {

				private final SingletonObjectFunctionList _singletonFunctionList =
					new SingletonObjectFunctionList();

				public Object map(Object obj) throws SimulationRuntimeException, ModelCompilingException {
					_singletonFunctionList.setFunction(obj);

					return staticMap(
						executeFunctionArgs,
						currArgumentBindings,
						mapFunction,
						_singletonFunctionList);
				}
			});

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}

	protected static Object staticMap(
		IExecuteFunctionArgs executeFunctionArgs,
		IArgumentBindings currArgumentBindings,
		Object mapFunction,
		SingletonObjectFunctionList singletonFunctionList)
		throws SimulationRuntimeException, ModelCompilingException {

		AbstractSpecificFunctionType.pushArgumentBindingsTo(executeFunctionArgs, mapFunction);
		executeFunctionArgs.pushArgumentBindings(currArgumentBindings);

		Object result =
			AbstractSpecificFunctionType.getParametricFunction(mapFunction).execute(
				executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
				singletonFunctionList);

		executeFunctionArgs.popArgumentBindings();
		AbstractSpecificFunctionType.popArgumentBindingsFrom(executeFunctionArgs, mapFunction);

		return result;
	}
}