package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (30.07.2001 14:16:30)
 * @author Christoph Oechslein
 */

final class FunctionIteratorMaximize extends AbstractFunctionDynamic {
	private final IFunction _functionFunction;
	private final IFunction _functionSet;

	private final TwoObjectFunctionList _twoObjectFunctionList = new TwoObjectFunctionList();

	public FunctionIteratorMaximize(IFunction functionFunction, IFunction functionSet) {
		super();

		_functionFunction = functionFunction;
		_functionSet = functionSet;
	}

	public Object execute(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object compareFunction = _functionFunction.execute(executeFunctionArgs);

		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs =
			executeFunctionArgs.createCreateAndExecuteFunctionArgs();

		IExecuteIterator executeIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		if (!executeIterator.hasNext())
			throw new EmptyIteratorSimulationRuntimeException("Iterator is empty in maximize function.");

		Object maxObj = executeIterator.nextExecute();
		_twoObjectFunctionList.setFunctionObjectB(maxObj);

		while (executeIterator.hasNext()) {
			Object currObject = executeIterator.nextExecute();

			_twoObjectFunctionList.setFunctionObjectA(currObject);

			try {
				maxObj =
					AbstractSpecificFunctionType.getParametricFunction(compareFunction).execute(
						createAndExecuteFunctionArgs,
						_twoObjectFunctionList);
				_twoObjectFunctionList.setFunctionObjectB(maxObj);
			} catch (ModelCompilingException e) {
				throw new SimulationRuntimeException(e);
			}
		}

		return maxObj;
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append("Maximize");
			buff.append("('");
			_functionFunction.appendName(buff, currDepth + 1, maxDepth);
			buff.append("', ");
			_functionSet.appendName(buff, currDepth + 1, maxDepth);
			buff.append(')');
		}
	}

	public String getName() {
		return getNameIntern();
	}
}