package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.IteratorWithException;

/**
 * Creation date: (30.07.2001 14:16:30)
 * @author Christoph Oechslein
 */

final class FunctionIteratorNumberMapper extends AbstractFunction {
	private static class MapNumberExecuteIterator extends AbstractExecuteIterator {
		private final IteratorWithException _it;
		private final Object _mapFunction;
		private final IArgumentBindings _currArgumentBindings;
		private final IExecuteFunctionArgs _executeFunctionArgs;
		private final SingletonObjectFunctionList _singletonFunctionList = new SingletonObjectFunctionList();

		public MapNumberExecuteIterator(
			IExecuteIterator it,
			Object mapFunction,
			IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {
				
			_it = it;
			_mapFunction = mapFunction;
			_currArgumentBindings = executeFunctionArgs.saveCurrentArgumentBindings();
			_executeFunctionArgs = executeFunctionArgs;
		}

		public Object nextExecute() throws SimulationRuntimeException {
			return new Double(nextExecuteForDoubleIntern());
		}

		public double nextExecuteForDouble() throws SimulationRuntimeException {
			return nextExecuteForDoubleIntern();
		}

		public int nextExecuteForInt() throws SimulationRuntimeException {
			return nextExecuteForIntIntern();
		}

		private double nextExecuteForDoubleIntern() throws SimulationRuntimeException {
			try {

				setBindingsBefore();

				double result =
					AbstractSpecificFunctionType.getParametricFunction(_mapFunction).executeForDouble(
						_executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
						_singletonFunctionList);

				setBindingsAfter();

				return result;

			} catch (ModelCompilingException e) {
				throw new SimulationRuntimeCausedByModelCompileException(e);
			}
		}

		private int nextExecuteForIntIntern() throws SimulationRuntimeException {
			try {

				setBindingsBefore();

				int result =
					AbstractSpecificFunctionType.getParametricFunction(_mapFunction).executeForInt(
						_executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
						_singletonFunctionList);

				setBindingsAfter();

				return result;

			} catch (ModelCompilingException e) {
				throw new SimulationRuntimeCausedByModelCompileException(e);
			}
		}

		private void setBindingsAfter() throws SimulationRuntimeException {
			_executeFunctionArgs.popArgumentBindings();
			AbstractSpecificFunctionType.popArgumentBindingsFrom(_executeFunctionArgs, _mapFunction);
		}

		private void setBindingsBefore() throws SimulationRuntimeException {
			AbstractSpecificFunctionType.pushArgumentBindingsTo(_executeFunctionArgs, _mapFunction);
			_executeFunctionArgs.pushArgumentBindings(_currArgumentBindings);
			_singletonFunctionList.setFunction(ExecuteIteratorHelper.nextExecute(_it));
		}

		public boolean hasNext() {
			return _it.hasNext();
		}

	}

	private final IFunction _functionNumberFunction;
	private final IFunction _functionSet;

	public String getName() {
		return "Map('" + _functionNumberFunction.getName() + "', " + _functionSet.getName() + ")";
	}

	public FunctionIteratorNumberMapper(IFunction functionNumberFunction, IFunction functionSet) {
		super();

		_functionNumberFunction = functionNumberFunction;
		_functionSet = functionSet;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object mapFunction = _functionNumberFunction.execute(executeFunctionArgs);

		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		return new MapNumberExecuteIterator(setIterator, mapFunction, executeFunctionArgs);
	}
}