package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorSelect extends AbstractFunction {
	private final IFunction _functionFunction;
	private final IFunction _functionSet;

	public String getName() {
		return "Select('" + _functionFunction.getName() + "', " + _functionSet.getName() + ")";
	}

	public FunctionIteratorSelect(IFunction functionFunction, IFunction functionSet) {
		super();

		_functionFunction = functionFunction;
		_functionSet = functionSet;
	}

	public Object execute(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object selectFunction = _functionFunction.execute(executeFunctionArgs);

		final IArgumentBindings currArgumentBindings = executeFunctionArgs.saveCurrentArgumentBindings();

		IExecuteIterator setIterator =
			ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionSet);

		try {
			return CollectionsHelper.selectWithException(setIterator, new IPredicateFunction() {

				private final SingletonObjectFunctionList _singletonFunctionList =
					new SingletonObjectFunctionList();

				public boolean isElementOk(Object obj)
					throws SimulationRuntimeException, ModelCompilingException {
					_singletonFunctionList.setFunction(obj);

					return staticIsElementOk(
						executeFunctionArgs,
						currArgumentBindings,
						selectFunction,
						_singletonFunctionList);
				}
			});

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException)e);
			else
				throw new RuntimeException(e);
		}
	}

	protected static boolean staticIsElementOk(
		IExecuteFunctionArgs executeFunctionArgs,
		IArgumentBindings currArgumentBindings,
		Object selectFunction,
		SingletonObjectFunctionList singletonFunctionList)
		throws SimulationRuntimeException, ModelCompilingException {

		executeFunctionArgs.pushArgumentBindings(currArgumentBindings);

		boolean result =
			AbstractSpecificFunctionType.getParametricFunction(selectFunction).executeForBoolean(
				executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
				singletonFunctionList);

		executeFunctionArgs.popArgumentBindings();

		return result;
	}
}