package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;

/**
 * Creation date: (18.12.2001 09:27:51)
 * @author Christoph Oechslein
 */
//TODO change to integer type
final class GetNthGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {

	public GetNthGenericFunctionDeclaration() {
		super("GetNth", GenericType.getInstance());
		
		setDocumentation("Returns the nth (Arg1) element of its Set");
		setExpertFunctionDeclaration(true);
	}


	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType outputType) {
		return createFunctionDeclarationSetType(GenericIteratorType.getInstance().createSpecificIteratorType(outputType));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {

		ISpecificType numberType = getNextSpecificType(inputArgs);

		ISpecificType specificSetType = getNextSpecificType(inputArgs);

		if (inputArgs.hasNext() || (numberType == null) || (specificSetType == null)) {
			return null;
		}

		if ((!DoubleType.getInstance().isMoreSpecific(numberType)) || !(specificSetType instanceof SpecificIteratorType)) {
			return null;
		}

		return createFunctionDeclarationSetType((SpecificIteratorType) specificSetType);
	}


	public ISpecificFunctionDeclaration createFunctionDeclarationSetType(final SpecificIteratorType inputType) {

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return inputType.getIteratorSpecificType();
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0)
					return DoubleType.getInstance();

				else if (index == 1)
					return inputType;

				else {
					Assert.fail();
					return null;
				}

			}
			
			public String getSpecificInputTypeNameAt(int index) {
					if (index==0) return "Position";
		
					if (index==1) return "Iterator";
		
					Assert.fail("GetNth has just two arguments");
					return null;
				}

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				Assert.assertEquals(2, functions.size());

				return new FunctionIteratorGetNth(functions.getFunction(0), functions.getFunction(1));
			}
		};
	}

	/**
	 * @param outputType ISpecificType
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
	 */
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Number, Iterator<" + outputType.getName() + ">)";
	}
}