package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * isEmpty: boolean isEmpty (set<T>)
 * T is unspecific

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class IsEmptyGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("set");

	public IsEmptyGenericFunctionDeclaration() {
		super("IsEmpty", BooleanType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Returns the verity content if its Set is empty");
	}


	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// isEmpty: boolean isEmpty (set<T>)
		// T is unspecific

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(
			GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final SpecificIteratorType inputSpecificSetType) {

		// isEmpty: boolean isEmpty (set<T>)
		// T is unspecific

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return BooleanType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return inputSpecificSetType;
				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());
				final IFunction functionSet = (IFunction) functions.get(0);
				return new AbstractBooleanFunction() {
					public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
						IExecuteIterator it =
							ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, functionSet);

						return CollectionsHelper.isEmpty(it);
					}

					public String getName() {
						return "IsEmpty(" + functionSet.getName() + ")";
					}
				};
			}
		};
	}


	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// isEmpty: boolean isEmpty (set<T>)

		ISpecificType typeT = getNextSpecificType(inputTypes);

		if ((typeT == null) || !(typeT instanceof SpecificIteratorType))
			return null;

		SpecificIteratorType specificSetType = (SpecificIteratorType) typeT;

		return createFunctionDeclaration(specificSetType);
	}

	/**
	 * @param outputType ISpecificType
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
	 */
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Iterator<T>)";
	}
}