package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * map: set<T1> map (<T1>function(<T2>), set<T2>)
 * T2 is unspecific
 * Creation date: (27.10.2000 17:49:45)
 *
 * @author Christoph Oechslein
 */

final class MapGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("set");

	public MapGenericFunctionDeclaration() {
		super("Map", GenericIteratorType.getInstance(), CollectionsHelper.createList(typeDependentT2));

		setDocumentation("Maps its iterator (Arg2) using function (Arg1) into a Set (of another output type) e.g. Map(Iterator<body>, Max) = Iterator<number>");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// map: Iterator<T1> map (<T1>function(<T2>), Iterator<T2>)

		SpecificIteratorType outputSpecificSetType;

		if (outputType == ObjectType.getInstance()) {
			outputSpecificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			outputSpecificSetType = (SpecificIteratorType) outputType;
		}
						
		ISpecificType typeT1 = outputSpecificSetType.getIteratorSpecificType();
		ISpecificType typeT2 = bindings.getBindingFor(typeDependentT2);
		ISpecificFunctionType inputFunctionTypeT1T2 =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				typeT1,
				CollectionsHelper.createList(typeT2));
		SpecificIteratorType SpecificSetTypeT2 =
			GenericIteratorType.getInstance().createSpecificIteratorType(typeT2);

		return createFunctionDeclaration(outputSpecificSetType, inputFunctionTypeT1T2, SpecificSetTypeT2);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final SpecificIteratorType outputSetType,
		final ISpecificFunctionType inputFunctionType,
		final SpecificIteratorType inputSetType) {

		// map: set<T1> map (<T1>function(<T2>), set<T2>)

		return new MapSpecificFunctionDeclaration(this, outputSetType, inputFunctionType, inputSetType);
	}


	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// map: set<T1> map (<T1>function(<T2>), set<T2>)

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof ISpecificFunctionType)
			|| !(specificTypeB instanceof SpecificIteratorType))
			return null;

		ISpecificFunctionType functionTypeT1T2 = (ISpecificFunctionType) specificTypeA;
		SpecificIteratorType setTypeT2 = (SpecificIteratorType) specificTypeB;

		if ((functionTypeT1T2.getSpecificInputTypesLength() != 1)
			|| (!setTypeT2.getIteratorSpecificType().equals(functionTypeT1T2.getSpecificInputTypeAt(0))))
			return null;

		SpecificIteratorType outputSpecificSetType =
			GenericIteratorType.getInstance().createSpecificIteratorType(
					functionTypeT1T2.getSpecificOutputType());

		return createFunctionDeclaration(outputSpecificSetType, functionTypeT1T2, setTypeT2);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		// map: set<T1> map (<T1>function(<T2>), set<T2>)

		SpecificIteratorType specificSetType;

		if (outputType == ObjectType.getInstance()) {
			specificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			specificSetType = (SpecificIteratorType) outputType;
		}
		return "(Fun<(T) " + specificSetType.getIteratorSpecificType().getName() + ">, Iterator<T>)";
	}
}