package de.uniwue.ki.mas.basics.function.iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

public class MapSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final SpecificIteratorType _outputSetType;
	private final ISpecificFunctionType _inputFunctionType;
	private final SpecificIteratorType _inputSetType;

	/**
	 * @param mapGenericFunctionDeclaration MapGenericFunctionDeclaration
	 * @param outputSetType SpecificIteratorType
	 * @param inputFunctionType ISpecificFunctionType
	 * @param inputSetType SpecificIteratorType
	 */
	public MapSpecificFunctionDeclaration(
		MapGenericFunctionDeclaration mapGenericFunctionDeclaration,
		SpecificIteratorType outputSetType,
		ISpecificFunctionType inputFunctionType,
		SpecificIteratorType inputSetType) {

		super(mapGenericFunctionDeclaration, 2);

		_outputSetType = outputSetType;
		_inputFunctionType = inputFunctionType;
		_inputSetType = inputSetType;
	}

	public ISpecificType getSpecificOutputType() {
		return _outputSetType;
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return _inputFunctionType;

		} else if (index == 1) {
			return _inputSetType;

		} else {
			Assert.fail();
			return null;
		}
	}

	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0)
			return "Map function";

		if (index == 1)
			return "Iterator";

		Assert.fail("Map has just two arguments");
		return null;
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());

		if (DoubleType.getInstance().isMoreSpecific(_outputSetType.getIteratorSpecificType())) {
			return new FunctionIteratorNumberMapper(functions.getFunction(0), functions.getFunction(1));

		} else {
			return new FunctionIteratorMapper(functions.getFunction(0), functions.getFunction(1));
		}
	}
}