package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * T maximize(<bool fun(T,T), Iterator<T>)
 * Creation date: (26.02.2002 20:08:58)
 * @author Christoph Oechslein
 */
public final class MaximizeGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {
	protected MaximizeGenericFunctionDeclaration() {
		super("Maximize", GenericType.getInstance());

		setDocumentation(
			"Maximizes the given iterator via the given function, that should compare " +
			"two values and return the better value of these two."
				+ " Same as GetFirst (Sort ...) but more efficient");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return new MaximizeSpecificFunctionDeclaration(
			this,
			GenericIteratorType.getInstance().createSpecificIteratorType(outputType),
			GenericFunctionType.getInstance().createSpecificFunctionType(
				outputType,
				CollectionsHelper.createList(outputType, outputType)));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType type1 = getNextSpecificType(inputTypes);
		ISpecificType type2 = getNextSpecificType(inputTypes);

		if (inputTypes.hasNext()) {
			return null;
		}

		if ((type1 == null) || !(type1 instanceof ISpecificFunctionType))
			return null;

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)type1;

		if ((type2 == null) || !(type2 instanceof SpecificIteratorType))
			return null;

		if (specificFunctionType.getSpecificInputTypesLength() != 2)
			return null;

		ISpecificType mostGeneralTypeInputTypes =
			TypeManager.getInstance().getMostGeneralType(specificFunctionType.getSpecificInputTypeAt(0), specificFunctionType.getSpecificInputTypeAt(1));

		if (mostGeneralTypeInputTypes == null)
			return null;

		ISpecificType mostGeneralType =
			TypeManager.getInstance().getMostGeneralType(specificFunctionType.getSpecificOutputType(), mostGeneralTypeInputTypes);

		if (mostGeneralType == null)
			return null;

		SpecificIteratorType specificIteratorType = (SpecificIteratorType)type2;

		mostGeneralType =
			TypeManager.getInstance().getMostGeneralType(mostGeneralType, specificIteratorType.getIteratorSpecificType());

		if (mostGeneralType == null)
			return null;

		return new MaximizeSpecificFunctionDeclaration(
			this,
			GenericIteratorType.getInstance().createSpecificIteratorType(mostGeneralType),
			GenericFunctionType.getInstance().createSpecificFunctionType(
				mostGeneralType,
				CollectionsHelper.createList(mostGeneralType, mostGeneralType)));
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		StringBuffer buff = new StringBuffer();

		buff.append("(Fun<");
		outputType.appendName(buff, 0, 10);
		buff.append(", ");
		outputType.appendName(buff, 0, 10);
		buff.append("> ");
		outputType.appendName(buff, 0, 10);
		buff.append("), Iterator<");
		outputType.appendName(buff, 0, 10);
		buff.append(">)");

		return buff.toString();
	}
}