package de.uniwue.ki.mas.basics.function.iterator;
import java.util.*;

import de.uniwue.ki.mas.basics.function.ICreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.TwoObjectFunctionList;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: 29.01.2003, 14:45:32
 * @author oechslein
 */
public class ParametricFunctionComparator implements Comparator {
	private final Object _compareFunction;
	private final ICreateAndExecuteFunctionArgs _executeFunctionArgs;
	private final TwoObjectFunctionList _twoObjectFunctionList;

	public ParametricFunctionComparator(
		Object compareFunction,
		ICreateAndExecuteFunctionArgs executeFunctionArgs,
		TwoObjectFunctionList twoObjectFunctionList) {
		super();

		_compareFunction = compareFunction;
		_executeFunctionArgs = executeFunctionArgs;
		_twoObjectFunctionList = twoObjectFunctionList;
	}

	public int compare(Object o1, Object o2) {
		try {
			boolean result = executeForBoolean(o1, o2);

			if (!result)
				// means !(o1 >= o2) == o1 < o2
				return 1;
			else
				// normaly should test for equality
				return -1;

		} catch (SimulationRuntimeException e) {
			throw new SimulationRuntimeUncheckException(e);

		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeUncheckException(e);
		}
	}

	private boolean executeForBoolean(Object o1, Object o2)
		throws SimulationRuntimeException, ModelCompilingException {

		_twoObjectFunctionList.setFunctionObjectA(o1);
		_twoObjectFunctionList.setFunctionObjectB(o2);

		return AbstractSpecificFunctionType.getParametricFunction(_compareFunction).executeForBoolean(
			_executeFunctionArgs,
			_twoObjectFunctionList);
	}

	public static void sortList(
		final ICreateAndExecuteFunctionArgs executeFunctionArgs,
		final Object compareFunction,
		final TwoObjectFunctionList twoObjectFunctionList,
		List list)
		throws SimulationRuntimeException {

		try {
			Collections.sort(
				list,
				new ParametricFunctionComparator(
					compareFunction,
					executeFunctionArgs,
					twoObjectFunctionList));

		} catch (SimulationRuntimeUncheckException e) {
			Throwable cause = e.getCause();
			if (cause instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException)cause;
			else if (cause instanceof ModelCompilingException)
				throw new SimulationRuntimeException(cause);
			else
				throw e;
		}
	}
}