package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Comparator;

import de.uniwue.ki.mas.basics.function.ICreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.SingletonObjectFunctionList;
import de.uniwue.ki.mas.basics.type.AbstractSpecificFunctionType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.math.MathHelper;

/**
 * Creation date: 29.01.2003, 14:45:32
 * @author oechslein
 */
public class ParametricFunctionValueComparator implements Comparator {
	private final Object _valueFunction;
	private final ICreateAndExecuteFunctionArgs _executeFunctionArgs;
	private final SingletonObjectFunctionList _singletonObjectFunctionList;

	public ParametricFunctionValueComparator(
		Object valueFunction,
		ICreateAndExecuteFunctionArgs executeFunctionArgs,
		SingletonObjectFunctionList singletonObjectFunctionList) {
		super();

		_valueFunction = valueFunction;
		_executeFunctionArgs = executeFunctionArgs;
		_singletonObjectFunctionList = singletonObjectFunctionList;
	}

	public int compare(Object o1, Object o2) {
		try {

			double value1 = executeForDouble(o1);
			double value2 = executeForDouble(o2);

			if (MathHelper.doubleGreater(value2, value1))
				return 1;
			else if (MathHelper.doubleEqual(value1, value2))
				return 0;
			else
				return -1;

		} catch (SimulationRuntimeException e) {
			throw new SimulationRuntimeUncheckException(e);

		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeUncheckException(e);
		}
	}

	protected double executeForDouble(Object anObject)
		throws SimulationRuntimeException, ModelCompilingException {

		_singletonObjectFunctionList.setFunctionObject(anObject);
		return AbstractSpecificFunctionType.getParametricFunction(_valueFunction).executeForDouble(
			_executeFunctionArgs,
			_singletonObjectFunctionList);
	}
}