package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * select: set<T> select(boolean function(<T>), set<T>)
 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class SelectGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {
	private static final SelectGenericFunctionDeclaration _instance = new SelectGenericFunctionDeclaration();

	private SelectGenericFunctionDeclaration() {
		super("Select", GenericIteratorType.getInstance());

		setDocumentation("Allows to select a Set (Arg2) with a select function (Arg1)");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		SpecificIteratorType outputSpecificSetType;

		if (outputType == ObjectType.getInstance()) {
			outputSpecificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			outputSpecificSetType = (SpecificIteratorType) outputType;
		}

		return new SelectSpecificFunctionDeclaration(outputSpecificSetType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		/* select: set<T> select(boolean function(<T>), set<T>) */

		ISpecificType specificTypeA = getNextSpecificType(inputArgs);
		ISpecificType specificTypeB = getNextSpecificType(inputArgs);

		if ((specificTypeA == null) || (specificTypeB == null) || inputArgs.hasNext())
			return null;

		if (!(specificTypeA instanceof ISpecificFunctionType)
			|| !(specificTypeB instanceof SpecificIteratorType))
			return null;

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType) specificTypeA;
		SpecificIteratorType specificSetType = (SpecificIteratorType) specificTypeB;

		if ((specificFunctionType.getSpecificInputTypesLength() != 1)
			|| (!specificFunctionType.getSpecificOutputType().equals(BooleanType.getInstance())))
			return null;

		if (!specificSetType
			.getIteratorSpecificType()
			.equals(specificFunctionType.getSpecificInputTypeAt(0)))
			return null;

		return createFunctionDeclaration(specificSetType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		/* select: set<T> select(boolean function(<T>), set<T>) */

		SpecificIteratorType specificSetType;

		if (outputType == ObjectType.getInstance()) {
			specificSetType =
				GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
		} else {
			specificSetType = (SpecificIteratorType) outputType;
		}

		return "("
			+ specificSetType.getName()
			+ ", Fun<("
			+ specificSetType.getIteratorSpecificType().getName()
			+ ") "
			+ BooleanType.getInstance().getName()
			+ ")";
	}

	public static final SelectGenericFunctionDeclaration getInstance() {
		return _instance;
	}
}