package de.uniwue.ki.mas.basics.function.iterator;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * set-difference: set<T> set-difference(set<T>, set<T>)

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class SetDifferenceGenericFunctionDeclaration extends AbstractTwoIteratorGenericFunctionDeclaration {

	public static final class FunctionIteratorSetDifference extends AbstractFunction {
		private final IFunction _functionA;
		private final IFunction _functionB;

		protected FunctionIteratorSetDifference(IFunction functionA, IFunction functionB) {
			super();
			_functionA = functionA;
			_functionB = functionB;
		}

		public String getName() {
			return "SetDifference(" + _functionA.getName() + ", " + _functionB.getName() + ")";
		}

		public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			try {
				return CollectionsHelper.setDifferenceWithException(
					ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionA),
					ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _functionB));
			} catch (SimulationRuntimeException e) {
				throw e;
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
	}

	public SetDifferenceGenericFunctionDeclaration() {
		super("SetDifference");

		setDocumentation("Returns the difference of its Sets");
	}

	protected IFunction createFunction(IFunction functionA, IFunction functionB) {
		return new FunctionIteratorSetDifference(functionA, functionB);
	}
}