package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Iterator<T> Shuffle(<bool fun(T,T), Iterator<T>)
 * void Shuffle(<bool fun(T,T), List<T>)
 * Creation date: (26.02.2002 20:08:58)
 * @author Christoph Oechslein
 */
public final class ShuffleGenericFunctionDeclaration extends AbstractGenericFunctionDeclaration {
	private final IDependentType _iteratorType = new UnconstraintDependentType("Iterator type");

	protected ShuffleGenericFunctionDeclaration() {
		super("Shuffle", GenericType.getInstance(), true);

		setDocumentation("Shuffles the given iterator, i.e. it randomize the order.");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return GenericIteratorType.getInstance().canCreate(outputType)
			|| VoidType.getInstance().equals(outputType);
	}

	public Iterator getDependentTypes(ISpecificType outputType) {
		if (VoidType.getInstance().equals(outputType))
			return CollectionsHelper.createIterator(_iteratorType);
		else
			return CollectionsHelper.EMPTY_ITERATOR;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		if (VoidType.getInstance().equals(outputType))
			return new ShuffleVoidSpecificFunctionDeclaration(this, bindings.getBindingFor(_iteratorType));
		else {
			SpecificIteratorType outputSpecificSetType;

			if (outputType == ObjectType.getInstance()) {
				outputSpecificSetType =
					GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
			} else {
				outputSpecificSetType = (SpecificIteratorType)outputType;
			}

			return new ShuffleIteratorSpecificFunctionDeclaration(
				this,
				outputSpecificSetType.getIteratorSpecificType());
		}
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException();
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (VoidType.getInstance().equals(outputType))
			return createVoidSpecificFunctionDeclaration(inputTypes);
		else
			return createIteratorSpecificFunctionDeclaration(inputTypes);
	}

	public ISpecificFunctionDeclaration createVoidSpecificFunctionDeclaration(Iterator inputTypes) {

		// void Shuffle(List<T>)
		SpecificIteratorType specificIteratorType = testTypes(inputTypes);

		if ((specificIteratorType == null) || !(specificIteratorType instanceof SpecificListType)) {
			return null;
		}

		return new ShuffleVoidSpecificFunctionDeclaration(
			this,
			((SpecificListType)specificIteratorType).getListSpecificType());
	}

	public ISpecificFunctionDeclaration createIteratorSpecificFunctionDeclaration(Iterator inputTypes) {

		// Iterator<T> Shuffle(Iterator<T>)
		SpecificIteratorType specificIteratorType = testTypes(inputTypes);

		if (specificIteratorType == null) {
			return null;
		}

		return new ShuffleIteratorSpecificFunctionDeclaration(
			this,
			specificIteratorType.getIteratorSpecificType());
	}

	private SpecificIteratorType testTypes(Iterator inputTypes) {
		ISpecificType type2 = getNextSpecificType(inputTypes);

		if (inputTypes.hasNext()) {
			return null;
		}

		if ((type2 == null) || !(type2 instanceof SpecificIteratorType))
			return null;
		else
			return (SpecificIteratorType)type2;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (VoidType.getInstance().equals(outputType)) {
			return "(List<T>)";

		} else if (ObjectType.getInstance().equals(outputType)) {
			return "(Iterator<T>)";

		} else {
			return "(" + outputType.getName() + ")";
		}
	}

	protected String getFullSignatureOutputName() {
		return "Iterator<T>";
	}
}