package de.uniwue.ki.mas.basics.function.iterator;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Iterator<T> sort(<bool fun(T,T), Iterator<T>)
 * void sort(<bool fun(T,T), List<T>)
 * Creation date: (26.02.2002 20:08:58)
 * @author Christoph Oechslein
 */
public final class SortGenericFunctionDeclaration extends AbstractGenericFunctionDeclaration {
	private final IDependentType _iteratorType = new UnconstraintDependentType("Iterator type");

	protected SortGenericFunctionDeclaration() {
		super("Sort", GenericType.getInstance(), true);

		setDocumentation("Sorts the given iterator via the given function (e.g.: >=).");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return GenericIteratorType.getInstance().canCreate(outputType)
			|| VoidType.getInstance().equals(outputType);
	}

	public Iterator getDependentTypes(ISpecificType outputType) {
		if (VoidType.getInstance().equals(outputType))
			return CollectionsHelper.createIterator(_iteratorType);
		else
			return CollectionsHelper.EMPTY_ITERATOR;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		if (VoidType.getInstance().equals(outputType))
			return new SortVoidSpecificFunctionDeclaration(this, bindings.getBindingFor(_iteratorType));
		else {
			SpecificIteratorType outputSpecificSetType;

			if (outputType == ObjectType.getInstance()) {
				outputSpecificSetType =
					GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
			} else {
				outputSpecificSetType = (SpecificIteratorType)outputType;
			}

			return new SortIteratorSpecificFunctionDeclaration(
				this,
				outputSpecificSetType.getIteratorSpecificType());
		}
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException();
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (VoidType.getInstance().equals(outputType))
			return createVoidSpecificFunctionDeclaration(inputTypes);
		else
			return createIteratorSpecificFunctionDeclaration(inputTypes);
	}

	public ISpecificFunctionDeclaration createVoidSpecificFunctionDeclaration(Iterator inputTypes) {

		// void sort(<bool fun(T,T), List<T>)
		SpecificIteratorType specificIteratorType = testTypes(inputTypes);

		if ((specificIteratorType == null) || !(specificIteratorType instanceof SpecificListType)) {
			return null;
		}

		return new SortVoidSpecificFunctionDeclaration(
			this,
			((SpecificListType)specificIteratorType).getListSpecificType());
	}

	public ISpecificFunctionDeclaration createIteratorSpecificFunctionDeclaration(Iterator inputTypes) {

		// Iterator<T> sort(<bool fun(T,T), Iterator<T>)
		SpecificIteratorType specificIteratorType = testTypes(inputTypes);

		if (specificIteratorType == null) {
			return null;
		}

		return new SortIteratorSpecificFunctionDeclaration(
			this,
			specificIteratorType.getIteratorSpecificType());
	}

	private SpecificIteratorType testTypes(Iterator inputTypes) {

		ISpecificType type1 = getNextSpecificType(inputTypes);
		ISpecificType type2 = getNextSpecificType(inputTypes);

		if (inputTypes.hasNext()) {
			return null;
		}

		if ((type1 == null) || !(type1 instanceof ISpecificFunctionType))
			return null;

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType)type1;

		if ((type2 == null) || !(type2 instanceof SpecificIteratorType))
			return null;

		SpecificIteratorType specificIteratorType = (SpecificIteratorType)type2;
		ISpecificType specificType = specificIteratorType.getIteratorSpecificType();

		if ((BooleanType.getInstance().equals(specificFunctionType.getSpecificOutputType()))
			|| (specificFunctionType.getSpecificInputTypesLength() != 2)
			|| (!specificFunctionType.getSpecificInputTypeAt(0).equals(specificType))
			|| (!specificFunctionType.getSpecificInputTypeAt(1).equals(specificType)))
			return specificIteratorType;
		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (VoidType.getInstance().equals(outputType)) {
			return "(Fun<T, T> Boolean), List<T>)";

		} else if (ObjectType.getInstance().equals(outputType)) {
			return "(Fun<T, T> Boolean), Iterator<T>)";

		} else {
			SpecificIteratorType iteratorType;

			if (outputType == ObjectType.getInstance()) {
				iteratorType =
					GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());
			} else {
				iteratorType = (SpecificIteratorType)outputType;
			}

			StringBuffer buff = new StringBuffer();

			buff.append("(Fun<");
			iteratorType.getIteratorSpecificType().appendName(buff, 0, 10);
			buff.append(", ");
			iteratorType.getIteratorSpecificType().appendName(buff, 0, 10);
			buff.append("> Boolean), ");
			outputType.appendName(buff, 0, 10);
			buff.append(")");

			return buff.toString();
		}
	}

	protected String getFullSignatureOutputName() {
		return "Iterator<T>";
	}

}