/*
 * Created on 14.06.2004
 */
package de.uniwue.ki.mas.basics.function.iterator;

import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.IFunctionList;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.GenericIteratorType;
import de.uniwue.ki.mas.basics.type.GenericType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.SpecificIteratorType;
import de.uniwue.ki.util.collection.CollectionsHelper;


class SumGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {
	
	final class SumSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
		public SumSpecificFunctionArgsDeclaration(ISpecificType outputType) {
			super(
				FunctionIteratorSum.NAME,
				outputType,
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(outputType)));
		}

		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			if (Assert.ASSERTION_ON)
				Assert.assertEquals(1, functions.size());

			return new FunctionIteratorSum(functions.getFunction(0));
		}
	}
	
	SumGenericFunctionDeclaration() {
		super("Sum", GenericType.getInstance());
	}
	{
		setDocumentation("Returns the summation over its Sets");
	}
	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return new SumSpecificFunctionArgsDeclaration(outputType);
	}
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType type = getNextSpecificType(inputTypes);

		if ((type == null) || (inputTypes.hasNext()))
			return null;

		if (!(type instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificIteratorType = (SpecificIteratorType)type;
		ISpecificType iteratorSpecificType = specificIteratorType.getIteratorSpecificType();

		if (DoubleType.getInstance().isMoreSpecific(iteratorSpecificType)) {
			return createFunctionDeclaration(iteratorSpecificType);
		} else {
			return null;
		}
	}
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Iterator<" + outputType.getName() + ">)";
	}
}