package de.uniwue.ki.mas.basics.function.iterator;
import java.util.*;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.swingui.TestRunner;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

public final class TestIteratorFunction extends TestCase {
	private final ICreateAndExecuteFunctionArgs _executeFunArgs =
		new CreateAndExecuteFunctionArgs();

	private static IFunctionDeclarationList _functionDeclarationList;
	public TestIteratorFunction(String name) {
		super(name);
	}

	private Iterator calculateResult(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List numberListA,
		List numberListB,
		List numberListC)
		throws Exception {

		List list = new LinkedList();

		addArgument(numberListA, list);
		addArgument(numberListB, list);
		addArgument(numberListC, list);

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(specificFunctionDeclaration, list);

		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		return (Iterator) function.execute(_executeFunArgs);
	}

	private void addArgument(List numberList, List result) {
		if (numberList != null) {
			result.add(
				ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
					GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
					numberList));
		}
	}

	private IFunctionCallDeclaration createFunctionCallDeclaration(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List argumentDeclarations) {

		assertNotNull(specificFunctionDeclaration);

		List atomicFunctionCallInternDeclList = new ArrayList(argumentDeclarations.size());
		Iterator it = argumentDeclarations.iterator();
		while (it.hasNext()) {
			Object curr = it.next();
			if (curr instanceof IAtomicFunctionCallInternDeclaration)
				atomicFunctionCallInternDeclList.add(curr);
			else
				atomicFunctionCallInternDeclList.add(
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						(IArgumentDeclaration) curr));
		}
		return FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				specificFunctionDeclaration,
				atomicFunctionCallInternDeclList));
	}

	private ISpecificFunctionDeclaration getAlwaysTrueFunctionDeclaration() {
		return new AbstractSpecificFunctionArgsDeclaration(
			"alwaysTrue",
			BooleanType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance()),
			false,
			false) {
			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {
				return new AbstractBooleanFunction() {
					public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs) {
						return true;
					}
				};
			}
		};
	}

	private ISpecificFunctionDeclaration getDoubleNumberFunctionDeclaration() {
		return new AbstractSpecificFunctionArgsDeclaration(
			"doubleNumber",
			DoubleType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance()),
			false,
			false) {
			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {
				final IFunction numberFunction = (IFunction) functions.get(0);
				return new AbstractDoubleFunction() {
					public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
						return 2 * numberFunction.executeForDouble(executeFunctionArgs);
					}
				};
			}
		};
	}

	private ISpecificFunctionDeclaration getEvenTrueFunctionDeclaration() {
		return new AbstractSpecificFunctionArgsDeclaration(
			"evenTrue",
			BooleanType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance()),
			false,
			false) {
			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {
				final IFunction numberFunction = (IFunction) functions.get(0);
				return new AbstractBooleanFunction() {
					public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
						return numberFunction.executeForDouble(executeFunctionArgs) % 2 == 0;
					}
				};
			}
		};
	}

	public IFunctionDeclarationList getFunctionDeclarationList() {
		if (_functionDeclarationList == null)
			_functionDeclarationList = FunctionDeclarationListManager.getAllFunctionDeclarationList();
		return _functionDeclarationList;
	}

	private ISpecificFunctionDeclaration getIdentityFunctionDeclaration() {
		return new AbstractSpecificFunctionArgsDeclaration(
			"identity",
			DoubleType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance()),
			false,
			false) {
			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
				throws ModelCompilingException {
				final IFunction numberFunction = (IFunction) functions.get(0);

				Assert.assertNotNull(numberFunction);

				return new AbstractDoubleFunction() {
					public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
						return numberFunction.executeForDouble(executeFunctionArgs);
					}
				};
			}
		};
	}

	public static void main(String[] args) {
		TestRunner.run(TestIteratorFunction.class);
	}

	public void testAggregate() throws Exception {
		testAggregate(Collections.EMPTY_LIST, 0);
		testAggregate(CollectionsHelper.createList(new Double(1.0), new Double(2.0)), 1.0 + 2.0);
	}

	private void testAggregate(List numberList, double expectedResult) throws Exception {
		ISpecificType numberInAndOutType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				DoubleType.getInstance(),
				DoubleType.getInstance());
		ISpecificFunctionDeclaration aggregateFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"aggregate",
				DoubleType.getInstance(),
				CollectionsHelper.createList(
					numberInAndOutType,
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		ISpecificFunctionDeclaration plusFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"+",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IArgumentDeclaration argumentDeclarationFunction =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				numberInAndOutType,
				plusFunctionDeclaration);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				aggregateFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationFunction, argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);
		final double result = function.executeForDouble(_executeFunArgs);
		assertEquals(expectedResult, result, 0.001);
	}

	public void testConcatenate() throws Exception {
		testConcatenate(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testConcatenate(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0), new Double(3.0), new Double(4.0)));
		testConcatenate(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(5.0), new Double(6.0)),
			CollectionsHelper.createList(
				new Double(1.0),
				new Double(2.0),
				new Double(3.0),
				new Double(4.0),
				new Double(5.0),
				new Double(6.0)));
		testConcatenateNot(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(4.0), new Double(3.0), new Double(3.0), new Double(1.0)));
	}

	private void testConcatenate(List numberListA, List numberListB, List numberListC, List resultList)
		throws Exception {
		testLists(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"Concatenate",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			numberListC,
			resultList,
			true);
	}

	private void testConcatenateNot(List numberListA, List numberListB, List resultList) throws Exception {
		testListsNot(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"Concatenate",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			null,
			resultList,
			true);
	}

	public void testCount() throws Exception {
		testCount(Collections.EMPTY_LIST);
		testCount(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
	}

	private void testCount(List numberList) throws Exception {
		ISpecificFunctionDeclaration countFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"count",
				IntegerType.getInstance(),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				countFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		final double result = function.executeForDouble(_executeFunArgs);

		assertEquals(numberList.size(), (int) result);
	}

	public void testFilter() throws Exception {
		testFilter(Collections.EMPTY_LIST, Collections.EMPTY_LIST, false);
		testFilter(Collections.EMPTY_LIST, Collections.EMPTY_LIST, true);
		testFilter(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			false);
		testFilter(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0)),
			true);
	}

	private void testFilter(List numberList, List expectedResultList, boolean filterEvenNumbers)
		throws Exception {
		ISpecificType filterNumbersType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				BooleanType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance()));
		ISpecificFunctionDeclaration filterFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"select",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					filterNumbersType,
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		ISpecificFunctionDeclaration filterNumberFunctionDeclaration =
			filterEvenNumbers ? getEvenTrueFunctionDeclaration() : getAlwaysTrueFunctionDeclaration();
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IArgumentDeclaration argumentDeclarationFunction =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				filterNumbersType,
				filterNumberFunctionDeclaration);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				filterFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationFunction, argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);
		List resultList = ExecuteIteratorFactory.createList(_executeFunArgs, function);
		assertEquals(expectedResultList, resultList);
	}

	public void testGetFirst() throws Exception {
		testGetFirst(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testGetFirst(CollectionsHelper.createList(new Double(1.0)));
	}

	private void testGetFirst(List numberList) throws Exception {
		ISpecificFunctionDeclaration getFirstFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"GetFirst",
				DoubleType.getInstance(),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				getFirstFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);
		double result = function.executeForDouble(_executeFunArgs);
		assertEquals(((Number) numberList.get(0)).doubleValue(), result, Double.MIN_VALUE);
	}

	public void testGetNth() throws Exception {
		testGetNth(CollectionsHelper.createList(new Double(1.0), new Double(2.0)), 1);
		testGetNth(CollectionsHelper.createList(new Double(1.0)), 0);
	}

	private void testGetNth(List numberList, int index) throws Exception {
		ISpecificFunctionDeclaration getFirstFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"GetNth",
				DoubleType.getInstance(),
				CollectionsHelper.createList(
					DoubleType.getInstance(),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));

		IArgumentDeclaration numberArgumentDeclaration =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				DoubleType.getInstance(),
				new Double(index));

		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				getFirstFunctionDeclaration,
				CollectionsHelper.createList(numberArgumentDeclaration, argumentDeclarationList));

		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		double result = function.executeForDouble(_executeFunArgs);

		assertEquals(((Number) numberList.get(index)).doubleValue(), result, Double.MIN_VALUE);
	}

	public void testGetRandom() throws Exception {
		testGetRandom(CollectionsHelper.createList(new Double(1.0)));
		testGetRandom(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testGetRandom(
			CollectionsHelper.createList(
				new Double(1.0),
				new Double(2.0),
				new Double(3.0),
				new Double(4.0),
				new Double(5.0)));
	}

	private void testGetRandom(List numberList) throws Exception {
		ISpecificFunctionDeclaration getRandomFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"GetRandom",
				DoubleType.getInstance(),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				getRandomFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		double result = function.executeForDouble(_executeFunArgs);

		assertTrue(numberList.contains(new Double(result)));
	}

	public void testIntersection() throws Exception {
		testIntersection(Collections.EMPTY_LIST, Collections.EMPTY_LIST, null, Collections.EMPTY_LIST);
		testIntersection(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			Collections.EMPTY_LIST);
		testIntersection(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testIntersection(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(3.0)),
			null,
			CollectionsHelper.createList(new Double(2.0)));
		testIntersection(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(0.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0)));
		testIntersectionNot(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(4.0), new Double(3.0), new Double(3.0), new Double(1.0)));
	}

	private void testIntersection(List numberListA, List numberListB, List numberListC, List resultList)
		throws Exception {
		testLists(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"intersection",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			numberListC,
			resultList,
			false);
	}

	private void testIntersectionNot(List numberListA, List numberListB, List resultList) throws Exception {
		testListsNot(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"intersection",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			null,
			resultList,
			false);
	}

	public void testIsEmpty() throws Exception {
		testIsEmpty(Collections.EMPTY_LIST);
		testIsEmpty(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
	}

	private void testIsEmpty(List numberList) throws Exception {
		ISpecificFunctionDeclaration isEmptyFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"isEmpty",
				BooleanType.getInstance(),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				isEmptyFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);
		final boolean result = function.executeForBoolean(_executeFunArgs);
		assertEquals(numberList.isEmpty(), result);
	}

	private void testLists(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List numberListA,
		List numberListB,
		List numberListC,
		List exceptedResultList,
		boolean oderIsImportant)
		throws Exception {

		LinkedList result =
			CollectionsHelper.asLinkedList(
				calculateResult(specificFunctionDeclaration, numberListA, numberListB, numberListC));

		String errorMsg = "Result should be " + exceptedResultList + " but was " + result;

		if (oderIsImportant)
			assertEquals(errorMsg, exceptedResultList, result);
		else
			assertTrue(errorMsg, CollectionsHelper.isSetEqual(exceptedResultList, result));
	}

	private void testListsNot(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		List numberListA,
		List numberListB,
		List numberListC,
		List exceptedResultList,
		boolean oderIsImportant)
		throws Exception {
		Iterator result = calculateResult(specificFunctionDeclaration, numberListA, numberListB, numberListC);

		if (oderIsImportant)
			assertTrue(!CollectionsHelper.isEqual(exceptedResultList.iterator(), result, false));
		else
			assertTrue(!CollectionsHelper.isSetEqual(exceptedResultList.iterator(), result));
	}

	public void testMap() throws Exception {
		testMap(Collections.EMPTY_LIST, Collections.EMPTY_LIST, false);
		testMap(Collections.EMPTY_LIST, Collections.EMPTY_LIST, true);
		testMap(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			false);
		testMap(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(4.0)),
			true);
	}

	private void testMap(List numberList, List expectedResultList, boolean doubleNumbers) throws Exception {
		ISpecificType mapNumbersType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance()));
		ISpecificFunctionDeclaration mapFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"map",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					mapNumbersType,
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));
		ISpecificFunctionDeclaration mapNumberFunctionDeclaration =
			doubleNumbers ? getDoubleNumberFunctionDeclaration() : getIdentityFunctionDeclaration();
		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);
		IArgumentDeclaration argumentDeclarationFunction =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				mapNumbersType,
				mapNumberFunctionDeclaration);
		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				mapFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationFunction, argumentDeclarationList));
		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);
		List resultList = ExecuteIteratorFactory.createList(_executeFunArgs, function);
		assertEquals(expectedResultList, resultList);
	}

	public void testSetDifference() throws Exception {
		testSetDifference(Collections.EMPTY_LIST, Collections.EMPTY_LIST, null, Collections.EMPTY_LIST);
		testSetDifference(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			Collections.EMPTY_LIST);
		testSetDifference(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			Collections.EMPTY_LIST,
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testSetDifference(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			Collections.EMPTY_LIST);
		testSetDifference(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(3.0)),
			null,
			CollectionsHelper.createList(new Double(1.0)));
		testSetDifferenceNot(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(4.0), new Double(3.0), new Double(3.0), new Double(1.0)));
	}

	private void testSetDifference(List numberListA, List numberListB, List numberListC, List resultList)
		throws Exception {
		testLists(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"SetDifference",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			numberListC,
			resultList,
			false);
	}

	private void testSetDifferenceNot(List numberListA, List numberListB, List resultList) throws Exception {
		testListsNot(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"SetDifference",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			null,
			resultList,
			false);
	}

	public void testSum() throws Exception {
		testSum(Collections.EMPTY_LIST);
		testSum(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testSum(CollectionsHelper.createList(new Double(1.0), new Double(-2.0)));
	}

	private void testSum(List numberList) throws Exception {
		ISpecificFunctionDeclaration countFunctionDeclaration =
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"Sum",
				DoubleType.getInstance(),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())));

		IArgumentDeclaration argumentDeclarationList =
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance()),
				numberList);

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				countFunctionDeclaration,
				CollectionsHelper.createList(argumentDeclarationList));

		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		final double result = function.executeForDouble(_executeFunArgs);

		Iterator it = numberList.iterator();
		double realResult = 0.0;
		while (it.hasNext()) {
			Double currDouble = (Double) it.next();

			realResult += currDouble.doubleValue();
		}

		assertEquals(realResult, result, 0.00001);
	}

	public void testUnion() throws Exception {
		testUnion(Collections.EMPTY_LIST, Collections.EMPTY_LIST, null, Collections.EMPTY_LIST);
		testUnion(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testUnion(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testUnion(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(3.0)),
			null,
			CollectionsHelper.createList(new Double(1.0), new Double(2.0), new Double(3.0)));
		testUnion(
			CollectionsHelper.createList(new Double(1.0), new Double(2.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(3.0)),
			CollectionsHelper.createList(new Double(2.0), new Double(1.0)),
			CollectionsHelper.createList(new Double(1.0), new Double(2.0), new Double(3.0)));
		testUnionNot(
			Collections.EMPTY_LIST,
			CollectionsHelper.createList(new Double(3.0), new Double(4.0)),
			CollectionsHelper.createList(new Double(4.0), new Double(3.0), new Double(3.0), new Double(1.0)));
	}

	private void testUnion(List numberListA, List numberListB, List numberListC, List resultList)
		throws Exception {
		testLists(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"union",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			numberListC,
			resultList,
			false);
	}

	private void testUnionNot(List numberListA, List numberListB, List resultList) throws Exception {
		testListsNot(
			getFunctionDeclarationList().getSpecificFunctionDeclaration(
				"union",
				GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
				CollectionsHelper.createList(
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()))),
			numberListA,
			numberListB,
			null,
			resultList,
			false);
	}

	public void testCreate() throws Exception {
		testCreate(CollectionsHelper.createList(new Double(1.0)));
		testCreate(CollectionsHelper.createList(new Double(1.0), new Double(2.0)));
		testCreate(
			CollectionsHelper.createList(new Double(4.0), new Double(3.0), new Double(3.0), new Double(1.0)));
	}

	private void testCreate(List numberList) throws Exception {
		List numberTypeList = new ArrayList(numberList.size());

		CollectionsHelper.setSize(numberTypeList, numberList.size(), DoubleType.getInstance());

		List argumentList = new ArrayList(numberList.size());

		for (int i = 0; i < numberList.size(); i++) {
			argumentList.add(
				ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
					DoubleType.getInstance(),
					numberList.get(i)));
		}

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				getFunctionDeclarationList().getSpecificFunctionDeclaration(
					"CreateIterator",
					GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance()),
					numberTypeList),
				argumentList);

		IFunction function = functionCallDeclaration.createFunction(_executeFunArgs);

		Iterator result = (Iterator) function.execute(_executeFunArgs);

		assertEquals(numberList, CollectionsHelper.asLinkedList(result));
	}
}