package de.uniwue.ki.mas.basics.function.list;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * AsList: list<T> asList(iterator<T>)

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class AsListGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithoutDependents {

	private static final AsListGenericFunctionDeclaration _instance = new AsListGenericFunctionDeclaration();

	public static final AsListGenericFunctionDeclaration getInstance() {
		return _instance;
	}

	private AsListGenericFunctionDeclaration() {
		super("AsList", GenericListType.getInstance());

		setDocumentation("Creates a list from an iterator");
	}

	public final ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType))
			return createAsListSpecificFunctionDeclaration(
				GenericListType.getInstance().createSpecificListType(ObjectType.getInstance()));
		else
			return createAsListSpecificFunctionDeclaration((SpecificListType) outputType);
	}

	private final ISpecificFunctionDeclaration createAsListSpecificFunctionDeclaration(SpecificListType specificListType) {
		return new AsListSpecificFunctionDeclaration(specificListType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		ISpecificType specificType = getNextSpecificType(inputArgs);

		if ((specificType == null) || inputArgs.hasNext())
			return null;

		if (!(specificType instanceof SpecificIteratorType))
			return null;

		SpecificIteratorType specificIteratorType = (SpecificIteratorType) specificType;

		return createAsListSpecificFunctionDeclaration(
			GenericListType.getInstance().createSpecificListType(
				specificIteratorType.getIteratorSpecificType()));
	}

	public final String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (outputType instanceof SpecificListType)
			return "("
				+ GenericIteratorType
					.getInstance()
					.createSpecificIteratorType(((SpecificListType) outputType).getListSpecificType())
					.getName()
				+ ")";
		else
			return "("
				+ GenericIteratorType
					.getInstance()
					.createSpecificIteratorType(ObjectType.getInstance())
					.getName()
				+ ")";
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}
}