package de.uniwue.ki.mas.basics.function.list;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * ClearList(list<T>) 
  */

final class ClearListGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("List Type");

	public ClearListGenericFunctionDeclaration() {
		super("ClearList", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Clears the given list");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final SpecificListType specificListType) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(0, index);

				return specificListType;
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionActionClearList(functions.getFunction(0));
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(GenericListType.getInstance().createSpecificListType(typeT));
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificListType))
			return null;

		return createFunctionDeclaration((SpecificListType) specificTypeA);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(List<T>)";
	}
}