package de.uniwue.ki.mas.basics.function.list;
import java.util.*;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
/**
 * concatenate: set<T> concatenate (set<T> set<T>)

 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class CreateListGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	private transient final Map _cache = new WeakHashMap();

	public CreateListGenericFunctionDeclaration() {
		super("CreateList", GenericListType.getInstance());

		setDocumentation("Creates a new List");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclarationIntern((SpecificListType) outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (outputType == null) {
			return createFunctionDeclaration(inputTypes);

		} else if (!(outputType instanceof SpecificListType)) {
			return null;

		} else {
			SpecificListType specificListType = (SpecificListType) outputType;

			ISpecificType specificType = specificListType.getListSpecificType();

			while (inputTypes.hasNext()) {
				if (!(specificType.isMoreSpecific(getNextSpecificType(inputTypes)))) {
					return null;
				}
			}

			return createFunctionDeclaration(specificListType);
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {

		if (!inputArgs.hasNext()) {
			return createFunctionDeclaration(
				GenericListType.getInstance().createSpecificListType(ObjectType.getInstance()));
		}

		ISpecificType specificType = TypeManager.getInstance().getMostGeneralType(inputArgs);

		if (specificType == null) {
			return null;

		} else {
			return createFunctionDeclaration(
				GenericListType.getInstance().createSpecificListType(specificType));
		}
	}

	private ISpecificFunctionDeclaration createFunctionDeclarationIntern(SpecificListType specificListType) {
		if (_cache.containsKey(specificListType)) {
			return (CreateListSpecificFunctionDeclaration) _cache.get(specificListType);
		} else {
			CreateListSpecificFunctionDeclaration createListSpecificFunctionDeclaration =
				new CreateListSpecificFunctionDeclaration(this, 1, specificListType);
			_cache.put(specificListType, createListSpecificFunctionDeclaration);
			return createListSpecificFunctionDeclaration;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (outputType instanceof SpecificListType)
			return "(" + ((SpecificListType) outputType).getListSpecificType().getName() + " ...)";
		else
			return "(" + ObjectType.getInstance().getName() + " ...)";
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}

	// since length args can be zero	
	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		return composedFunctionCallInternDeclaration.getChildCount() == 0;
	}

}