package de.uniwue.ki.mas.basics.function.list;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Creation date: (19.03.2002 14:26:32)
 * @author Christoph Oechslein
 */
class FunctionActionSetNth extends AbstractFunctionAction {
	private final IFunction _functionIndex;
	private final IFunction _functionValue;
	private final IFunction _functionList;

	public FunctionActionSetNth(IFunction functionIndex, IFunction functionValue, IFunction functionList) {
		super();

		_functionIndex = functionIndex;
		_functionValue = functionValue;
		_functionList = functionList;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		List list = (List) _functionList.execute(executeFunctionArgs);

		if (list == null) {
			throw new SimulationRuntimeException("List mustn't be null.");
		}

		int index = (int) Math.floor(_functionIndex.executeForDouble(executeFunctionArgs));

		if (index < 0) {
			throw new SimulationRuntimeException("Index must be equal or greater than zero.");
		}

		if (index >= list.size()) {
			throw new SimulationRuntimeException(
				"Index must be lower than size (=" + list.size() + ") of list.");
		}

		Object value = _functionValue.execute(executeFunctionArgs);

		Assert.assertFalse(value instanceof IBody);

		list.set(index, value);
	}

	public String getName() {
		return "SetNth("
			+ _functionIndex.getName()
			+ ","
			+ _functionValue.getName()
			+ ","
			+ _functionList.getName()
			+ ")";
	}
}