package de.uniwue.ki.mas.basics.function.list;

import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 10.12.2002, 16:42:12
 * @author oechslein
 */
//TODO change to iterator function
public class IndexOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	/**
	 *  double indefOf(list<T>, T)
	  */

	private static final IDependentType typeDependentT = new UnconstraintDependentType("List Type");

	public IndexOfGenericFunctionDeclaration() {
		super("IndexOf", DoubleType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Returns the index of the given element in the list. Returns -1 if element isn't in the list.");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return DoubleType.getInstance();
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return GenericListType.getInstance().createSpecificListType(typeT);
				} else {
					if (Assert.ASSERTION_ON)
						Assert.assertEquals(1, index);
					return typeT;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(2, functions.size());

				return new FunctionIndexOf(functions.getFunction(0), functions.getFunction(1));
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificListType))
			return null;

		SpecificListType specificListType = (SpecificListType) specificTypeA;

		if (TypeManager.getInstance().isMoreGeneral(specificListType.getListSpecificType(), specificTypeB))
			return createFunctionDeclaration(specificListType.getListSpecificType());
		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(list<T>, <T>)";
	}
}
