package de.uniwue.ki.mas.basics.function.list;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * Pop(list<T>) throws SetIsEmpty
  */

final class PopGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
    private static final IDependentType typeDependentT = new UnconstraintDependentType("List Type");

    public PopGenericFunctionDeclaration() {
        super("Pop", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));
        
        setDocumentation("Pops the last element the list (without returning)");        		
		setExpertFunctionDeclaration(true);
    }


private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
    // Pop(variable<list<T>>) throws SetIsEmpty

    return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
        public ISpecificType getSpecificOutputType() {
            return VoidType.getInstance();
        }

        public ISpecificType getSpecificInputTypeAt(int index) {
            if (Assert.ASSERTION_ON)
                Assert.assertEquals(0, index);

            return GenericListType.getInstance().createSpecificListType(typeT);
        }

        public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
            if (Assert.ASSERTION_ON)
                Assert.assertEquals(1, functions.size());

            return new FunctionActionPop(functions.getFunction(0));
        }
    };
}


    /**
     * @param outputType ISpecificType
     * @param bindings DependentTypeBindings
     * @return ISpecificFunctionDeclaration
     * @see de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration#createFunctionDeclaration(ISpecificType, DependentTypeBindings)
     */
    public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType, DependentTypeBindings bindings) {

        final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

        return createFunctionDeclaration(typeT);
    }


    /**
     * @param inputTypes Iterator
     * @return ISpecificFunctionDeclaration
     */
    public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

        ISpecificType specificTypeA = getNextSpecificType(inputTypes);

        if ((specificTypeA == null) || inputTypes.hasNext())
            return null;

        if (!(specificTypeA instanceof SpecificListType))
            return null;

        SpecificListType specificListType = (SpecificListType) specificTypeA;

        return createFunctionDeclaration(specificListType.getListSpecificType());
    }


    /**
     * @param outputType ISpecificType
     * @return String
     * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
     */
    public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
        return "(List<T>)";
    }
}