package de.uniwue.ki.mas.basics.function.list;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * Push(<T>, list<T>)
  */

final class PushGenericFunctionDeclaration extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("List Type");

	public PushGenericFunctionDeclaration() {
		super("Push", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Pushs value (Arg1) at the end of list (Arg2).");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {

		// Push(<T>, variable<list<T>>)

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return typeT;

				} else if (index == 1) {
					return GenericListType.getInstance().createSpecificListType(typeT);

				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(2, functions.size());

				return new FunctionActionPush(functions.getFunction(0), functions.getFunction(1));
			}

			public String getSpecificInputTypeNameAt(int index) {
				if (index == 0)
					return "Element";
				else if (index == 1)
					return "List";
				return super.getSpecificInputTypeNameAt(index);
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		ISpecificType typeT = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeB == null) || (typeT == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeB instanceof SpecificListType))
			return null;

		ISpecificType listSpecificType = ((SpecificListType) specificTypeB).getListSpecificType();

		if (listSpecificType.isMoreSpecific(typeT))
			return createFunctionDeclaration(listSpecificType);
		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(T, List<T>)";
	}
}