package de.uniwue.ki.mas.basics.function.list;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * SetNth(number, <T>, variable<set<T>>)
  */
// TODO change to integer type
final class SetNthGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("List Type");

	public SetNthGenericFunctionDeclaration() {
		super("SetNth", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Changes the nth (Arg1) element into element (Arg2) of List (Arg3)");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 3) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return DoubleType.getInstance();

				} else if (index == 1) {
					return typeT;

				} else if (index == 2) {
					return GenericListType.getInstance().createSpecificListType(typeT);

				} else {
					Assert.fail();
					return null;
				}
			}

			public String getSpecificInputTypeNameAt(int index) {
				if (index == 0) {
					return "index";

				} else if (index == 1) {
					return "value";

				} else if (index == 2) {
					return "list";

				} else
					return null;
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(3, functions.size());

				return new FunctionActionSetNth(
					functions.getFunction(0),
					functions.getFunction(1),
					functions.getFunction(2));
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		ISpecificType numberType = getNextSpecificType(inputTypes);
		ISpecificType valueType = getNextSpecificType(inputTypes);
		ISpecificType listSpecificType = getNextSpecificType(inputTypes);

		if ((numberType == null)
			|| (valueType == null)
			|| (listSpecificType == null)
			|| inputTypes.hasNext())
			return null;

		if (!DoubleType.getInstance().isMoreSpecific(numberType)) {
			return null;
		}

		if (!(listSpecificType instanceof SpecificListType))
			return null;

		SpecificListType specificListType = (SpecificListType) listSpecificType;

		if (!specificListType.getListSpecificType().isMoreSpecific(valueType))
			return null;

		return createFunctionDeclaration(specificListType.getListSpecificType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Number, T, List<T>)";
	}
}