package de.uniwue.ki.mas.basics.function.number;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * Creation date: 31.01.2003, 17:20:01
 * @author oechslein
 */
public abstract class AbstractNumberGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	private final int _argLength;

	public AbstractNumberGenericFunctionDeclaration(String name) {
		this(name, ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT);
	}

	public AbstractNumberGenericFunctionDeclaration(String name, int argLength) {
		super(name, GenericNumberType.getInstance());

		_argLength = argLength;
	}

	public AbstractNumberGenericFunctionDeclaration(
		String name,
		int argLength,
		boolean expertFunctionDeclaration) {
		super(name, GenericNumberType.getInstance());

		_argLength = argLength;
		setExpertFunctionDeclaration(expertFunctionDeclaration);
	}

	public final ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclarationIntern(outputType);
	}

	protected abstract ISpecificFunctionDeclaration createFunctionDeclarationIntern(ISpecificType outputType);

/*	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return DoubleType.getInstance().isMoreSpecific(outputType);
	}*/

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		if (!inputTypes.hasNext())
			return null;
		else
			return createFunctionDeclaration(inputTypes, IntegerType.getInstance(), 0);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		Iterator inputTypes,
		ISpecificType specificType,
		int argLength) {

		ISpecificType nextSpecificType = getNextSpecificType(inputTypes);

		if (nextSpecificType == null) {
			if ((_argLength == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT)
				|| (argLength == _argLength))
				return createFunctionDeclaration(specificType);
			else
				return null;

		} else if (nextSpecificType.isMoreSpecific(specificType)) {
			return createFunctionDeclaration(inputTypes, nextSpecificType, 1 + argLength);

		} else if (specificType.isMoreSpecific(nextSpecificType)) {
			return createFunctionDeclaration(inputTypes, specificType, 1 + argLength);

		} else {
			return null;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		String argumentTypeName = outputType.getName();

		if (_argLength == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return "(" + argumentTypeName + ", ...)";
			
		} else {
			StringBuffer buff = new StringBuffer("(");

			buff.append(argumentTypeName);

			for (int i = 01; i < _argLength; i++) {
				buff.append(", ");
				buff.append(argumentTypeName);
			}

			buff.append(")");

			return buff.toString();
		}
	}
}
