package de.uniwue.ki.mas.basics.function.number;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: 31.01.2003, 17:41:23
 * @author oechslein
 */
final class DivideSpecificFunctionDeclaration extends AbstractSpecificFunctionArgsDeclaration {

	public DivideSpecificFunctionDeclaration(boolean isDoubleFunction) {
		super(
			isDoubleFunction ? "/" : "Div",
			isDoubleFunction
				? (ISpecificType) DoubleType.getInstance()
				: (ISpecificType) IntegerType.getInstance(),
			isDoubleFunction
				? CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance())
				: CollectionsHelper.createList(IntegerType.getInstance(), IntegerType.getInstance()));

		if (isDoubleFunction) {
			setExpertFunctionDeclaration(false);
			setDocumentation("Returns the division of its operands where (Arg1) is the dividend and (Arg2) is the divisor e.g. /(20, 5) = 4.");
		} else {
			setExpertFunctionDeclaration(true);
			setDocumentation("Returns the integer division of its operands where (Arg1) is the dividend and (Arg2) is the divisor e.g. Div(20, 5) = 4 but Div(5, 20) = 0. It is same as Truncate(/(Arg1, Arg2)).");
		}
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		Assert.assertEquals(2, functions.size());

		if (DoubleType.getInstance().equals(getSpecificOutputType()))
			return new FunctionDivide(functions.getFunction(0), functions.getFunction(1));
		else
			return new FunctionDivideInt(functions.getFunction(0), functions.getFunction(1));
	}
}