package de.uniwue.ki.mas.basics.function.number;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionMin extends AbstractDoubleFunction {
	private final IFunctionList _functions;

	public FunctionMin(IFunctionList functions) {
		super();

		_functions = functions;
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForDouble(_functions.iterator(), executeFunctionArgs);
	}

	public static final double executeForDouble(Iterator functions, IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		if (!functions.hasNext()) {
			throw new EmptyIteratorSimulationRuntimeException();
		}

		double minValue = ((IFunction) functions.next()).executeForDouble(executeFunctionArgs);

		while (functions.hasNext()) {
			IFunction function = (IFunction) functions.next();

			double value = function.executeForDouble(executeFunctionArgs);

			if (value < minValue)
				minValue = value;
		}

		return minValue;
	}

	public String getName() {
		return "Min" + _functions.getName();
	}
}