package de.uniwue.ki.mas.basics.function.number;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionMinInt extends AbstractIntegerFunction {
	private final IFunctionList _functions;

	public FunctionMinInt(IFunctionList functions) {
		super();

		_functions = functions;
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return executeForInt(_functions.iterator(), executeFunctionArgs);
	}

	public static final int executeForInt(Iterator functions, IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		if (!functions.hasNext()) {
			throw new EmptyIteratorSimulationRuntimeException();
		}

		int minValue = ((IFunction) functions.next()).executeForInt(executeFunctionArgs);

		while (functions.hasNext()) {
			IFunction function = (IFunction) functions.next();

			int value = function.executeForInt(executeFunctionArgs);

			if (value < minValue)
				minValue = value;
		}

		return minValue;
	}

	public String getName() {
		return "Min" + _functions.getName();
	}
}