package de.uniwue.ki.mas.basics.function.number;
import junit.framework.Assert;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (17.09.2001 18:12:39)
 * @author Christoph Oechslein
 */

public final class TestNumberFunction extends TestCase {
	private final ICreateAndExecuteFunctionArgs _executeFunArgs = new CreateAndExecuteFunctionArgs();

	private final IFunctionDeclarationList functionDeclarationList =
		FunctionDeclarationListManager.getAllFunctionDeclarationList();
	private final IArgumentDeclaration number0ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			DoubleType.getInstance(),
			new Double(0.0));
	private final IArgumentDeclaration number1ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			DoubleType.getInstance(),
			new Double(1.0));
	private final IArgumentDeclaration number2ArgumentDeclaration =
		ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
			DoubleType.getInstance(),
			new Double(2.0));

	public TestNumberFunction(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestNumberFunction.class);
	}

	public void checkFunction(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		FunctionList functionList,
		String message,
		double expectedValue)
		throws Exception {

		assertNotNull(specificFunctionDeclaration);

		IFunction resultFunction =
			specificFunctionDeclaration.createParametricFunction(_executeFunArgs).createFunction(
				_executeFunArgs,
				functionList);
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(
				message,
				expectedValue,
				resultFunction.executeForDouble(_executeFunArgs),
				0.0001);
	}

	public void testAbs() throws Exception {
		ISpecificFunctionDeclaration absDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"abs",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(absDecl, new FunctionList(number0), "abs (0)", Math.abs(0.0));
		checkFunction(absDecl, new FunctionList(number1), "abs (1)", Math.abs(1.0));
		checkFunction(absDecl, new FunctionList(number2), "abs (2)", Math.abs(2.0));
	}

	public void testDurch() throws Exception {
		ISpecificFunctionDeclaration durchDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"/",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(durchDecl, new FunctionList(number1, number2), "1 / 2", 1.0 / 2.0);
		checkFunction(durchDecl, new FunctionList(number0, number1), "0 / 1", 0.0 / 1.0);
		checkFunction(durchDecl, new FunctionList(number2, number1), "2 / 1", 2.0 / 1.0);
	}

	public void testExp() throws Exception {
		ISpecificFunctionDeclaration expDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"exp",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(expDecl, new FunctionList(number0), "exp (0)", Math.exp(0.0));
		checkFunction(expDecl, new FunctionList(number1), "exp (1)", Math.exp(1.0));
		checkFunction(expDecl, new FunctionList(number2), "exp (2)", Math.exp(2.0));
	}

	public void testHoch() throws Exception {
		ISpecificFunctionDeclaration hochDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"Power",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(hochDecl, new FunctionList(number0, number0), "0 ^ 0", Math.pow(0.0, 0.0));
		checkFunction(hochDecl, new FunctionList(number0, number1), "0 ^ 1", Math.pow(0.0, 1.0));
		checkFunction(hochDecl, new FunctionList(number1, number0), "1 ^ 0", Math.pow(1.0, 0.0));
		checkFunction(hochDecl, new FunctionList(number1, number2), "1 ^ 2", Math.pow(1.0, 2.0));
	}

	/**
	 * @throws Exception
	 */
	public void testMal() throws Exception {
		ISpecificFunctionDeclaration malDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"*",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(malDecl, new FunctionList(number0, number0), "0 * 0", 0.0 * 0.0);
		checkFunction(malDecl, new FunctionList(number0, number1), "0 * 1", 0.0 * 1.0);
		checkFunction(malDecl, new FunctionList(number1, number0), "1 * 0", 1.0 * 0.0);
		checkFunction(malDecl, new FunctionList(number0, number1, number2), "0 * 1 * 2", 0 * 1 * 2);
	}

	public void testMinus() throws Exception {
		ISpecificFunctionDeclaration minusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"-",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(minusDecl, new FunctionList(number0, number0), "0 - 0", 0.0 - 0.0);
		checkFunction(minusDecl, new FunctionList(number0, number1), "0 - 1", 0.0 - 1.0);
		checkFunction(minusDecl, new FunctionList(number1, number0), "1 - 0", 1.0 - 0.0);
		checkFunction(minusDecl, new FunctionList(number1, number2), "1 - 2", 1.0 - 2.0);
	}

	public void testModulo() throws Exception {
		ISpecificFunctionDeclaration moduloDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"%",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(moduloDecl, new FunctionList(number0, number1), "0 % 1", 0 % 1);
		checkFunction(moduloDecl, new FunctionList(number2, number1), "2 % 1", 2 % 1);
		checkFunction(moduloDecl, new FunctionList(number1, number2), "1 % 2", 1 % 2);
	}

	public void testPlus() throws Exception {
		ISpecificFunctionDeclaration plusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"+",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));
		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);
		checkFunction(plusDecl, new FunctionList(number0, number0), "0 + 0", 0 + 0);
		checkFunction(plusDecl, new FunctionList(number0, number1), "0 + 1", 0 + 1);
		checkFunction(plusDecl, new FunctionList(number1, number0), "1 + 0", 1 + 0);
		checkFunction(plusDecl, new FunctionList(number0, number1, number2), "0 + 1 + 2", 0 + 1 + 2);
	}

	public void testMax() throws Exception {
		ISpecificFunctionDeclaration plusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"Max",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));

		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);

		checkFunction(plusDecl, new FunctionList(number0, number0), "Max(0, 0)", Math.max(0, 0));
		checkFunction(plusDecl, new FunctionList(number0, number1), "Max(0, 1)", Math.max(0, 1));
		checkFunction(plusDecl, new FunctionList(number1, number0), "Max(1, 0)", Math.max(1, 0));

		checkFunction(
			plusDecl,
			new FunctionList(number0, number1, number2),
			"Max(0, 1, 2)",
			Math.max(0, Math.max(1, 2)));
	}

	/**
	 * @throws Exception
	 */
	public void testMin() throws Exception {
		ISpecificFunctionDeclaration plusDecl =
			functionDeclarationList.getSpecificFunctionDeclaration(
				"Min",
				DoubleType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()));

		IArgument number0 = number0ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number1 = number1ArgumentDeclaration.createArgument(_executeFunArgs);
		IArgument number2 = number2ArgumentDeclaration.createArgument(_executeFunArgs);

		checkFunction(plusDecl, new FunctionList(number0, number0), "Min(0, 0)", Math.min(0, 0));
		checkFunction(plusDecl, new FunctionList(number0, number1), "Min(0, 1)", Math.min(0, 1));
		checkFunction(plusDecl, new FunctionList(number1, number0), "Min(1, 0)", Math.min(1, 0));

		checkFunction(
			plusDecl,
			new FunctionList(number0, number1, number2),
			"Min(0, 1, 2)",
			Math.min(0, Math.min(1, 2)));
	}
}
