package de.uniwue.ki.mas.basics.function.object;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.object.world.IWorldExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: 03.12.2002, 14:48:45
 * @author oechslein
 */
abstract class AbstractGetAllObjectsFunction extends AbstractFunction {
	private final IFunction _includingAddedFunction;
	private final IFunction _excludingRemovedFunction;

	public AbstractGetAllObjectsFunction(
		IFunction includingAddedFunction,
		IFunction excludingRemovedFunction) {
		super();

		_includingAddedFunction = includingAddedFunction;
		_excludingRemovedFunction = excludingRemovedFunction;
	}

	protected String getAddedAndRemovedName() {
		return _includingAddedFunction.getName() + ", " + _excludingRemovedFunction.getName();
	}

	protected Iterator getAllObjects(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		IWorldExecuteFunctionArgs worldExecuteFunctionArgs = (IWorldExecuteFunctionArgs) executeFunctionArgs;
		IWorld world = worldExecuteFunctionArgs.getWorld();

		return world.getAllObjects(
			_includingAddedFunction.executeForBoolean(worldExecuteFunctionArgs),
			_excludingRemovedFunction.executeForBoolean(executeFunctionArgs));
	}

	protected Object selectObjects(
		IExecuteFunctionArgs executeFunctionArgs,
		IPredicateFunction predicateFunction)
		throws SimulationRuntimeException, SimulationRuntimeCausedByModelCompileException {

		try {
			return CollectionsHelper.select(getAllObjects(executeFunctionArgs), predicateFunction);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;

			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);

			else
				throw new RuntimeException(e);
		}
	}
	
	protected Object selectByType(
			IExecuteFunctionArgs executeFunctionArgs,
			final IObjectClassDeclaration objectClassDeclaration)
			throws SimulationRuntimeException, SimulationRuntimeCausedByModelCompileException {
			return selectObjects(executeFunctionArgs, new IPredicateFunction() {
				public boolean isElementOk(Object anObject) {
					IObject simObject = (IObject) anObject;
					return simObject.getObjectInstanceDeclaration().getObjectClassDeclaration().equals(
						objectClassDeclaration);
				}
			});
		}
}
