package de.uniwue.ki.mas.basics.function.object;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.argument.ArgumentWithBindings;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * @author Rainer Herrler
 */

final class FindSimObjectFunction extends AbstractGetAllObjectsFunction {
	private final IFunction _objectInstanceDeclarationFunction;

	public FindSimObjectFunction(IFunction featureClassDeclarationFunction) {
		super(BooleanType.BOOLEAN_TYPE_FALSE_ARGUMENT, BooleanType.BOOLEAN_TYPE_TRUE_ARGUMENT);

		_objectInstanceDeclarationFunction = featureClassDeclarationFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		ArgumentWithBindings.ResultWithBindings resultWithBindings =
			(ArgumentWithBindings.ResultWithBindings) _objectInstanceDeclarationFunction.execute(
				executeFunctionArgs);

		final IObjectInstanceDeclaration objectInstanceDeclaration =
			(IObjectInstanceDeclaration) resultWithBindings.getValue();

		Iterator list = (Iterator) selectObjects(executeFunctionArgs, new IPredicateFunction() {
			public boolean isElementOk(Object anObject) {
				IObject simObject = (IObject) anObject;
				return objectInstanceDeclaration.equals(simObject.getObjectInstanceDeclaration());
			}
		});

		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(list.hasNext());
			Object obj = list.next();
			Assert.assertFalse(list.hasNext());
			return obj;

		} else {
			Object obj = list.next();
			return obj;
		}
	}

	public String getName() {
		return "FindSimObject(" + _objectInstanceDeclarationFunction.getName() + ")";
	}
}