package de.uniwue.ki.mas.basics.function.object;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

public final class FunctionDeclarationObjectFactory {
	private static IFunctionDeclarationList _cachedFunctionDeclarationList;

	public static final ISpecificFunctionDeclaration GET_WORLD_FUNCTION_DECLARATION =
		new GetWorldSpecificFunctionDeclaration();

	public static final ISpecificFunctionDeclaration GET_CURRENT_SIMOBJECT_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"GetCurrentSimObject",
			GenericSimObjectType.getGeneralSimObjectType(),
			CollectionsHelper.EMPTY_LIST,
			false) {
		{
			setDocumentation("Returns the objects body");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(0, functions.size());
			return GetCurrentSimObjectFunction.getInstance();
		}
	};

	public static final ISpecificFunctionDeclaration GET_MY_CURR_ACTIVITY_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"GetMyCurrentActivity",
			ActivityType.getInstance(),
			CollectionsHelper.EMPTY_LIST,
			false) {
		{
			setDocumentation("Returns the objects current activity");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(0, functions.size());
			return GetMyCurrentActivityFunction.getInstance();
		}
	};

	public static final ISpecificFunctionDeclaration GET_CURR_ACTIVITY_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"GetCurrentActivity",
			ActivityType.getInstance(),
			CollectionsHelper.createList(GenericSimObjectType.getGeneralSimObjectType()),
			false) {
		{
			setDocumentation("Returns the given object current activity");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(1, functions.size());
			return new GetCurrentActivityFunction(functions.getFunction(0));
		}
	};

	public static final ISpecificFunctionDeclaration IS_IN_ACTIVITY_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"IsInActivity",
			BooleanType.getInstance(),
			CollectionsHelper.createList(
				ActivityType.getInstance(),
				GenericSimObjectType.getGeneralSimObjectType()),
			false,
			false) {
		{
			setDocumentation("Returns true if given object is in given activity.");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(2, functions.size());
			return new IsInActivityFunction(functions.getFunction(0), functions.getFunction(1));
		}
	};

	public static final ISpecificFunctionDeclaration GET_TIME_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"GetTime",
			DoubleType.getInstance(),
			CollectionsHelper.EMPTY_LIST,
			false) {
		{
			setDocumentation("Returns the simulation time (counted in 'ticks')");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(0, functions.size());
			return GetTimeFunction.getInstance();
		}
	};

	public static final ISpecificFunctionDeclaration OBJECT_ID_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"ObjectId",
			IntegerType.getInstance(),
			CollectionsHelper.createList(GenericSimObjectType.getGeneralSimObjectType()),
			false,
			true) {
		{
			setDocumentation("Returns the object id (unique integer) of the given simulation object.");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(1, functions.size());
			return new GetObjectIdFunction(functions.getFunction(0));
		}
	};

	public static final IGenericFunctionDeclaration GET_ALL_OBJECTS_GENERIC_FUNCTION_DECLARATION =
		new GetAllObjectsGenericFunctionDeclaration();

	public static final ISpecificFunctionDeclaration GET_ALL_OBJECTS_OF_TYPE_SPECIFIC_FUNCTION_DECLARATION =
		new GetAllObjectsOfTypeSpecificFunctionDeclaration();

	public static final ISpecificFunctionDeclaration GET_ALL_OBJECTS_WITH_FEATURE_SPECIFIC_FUNCTION_DECLARATION =
		new GetAllObjectsWithFeatureSpecificFunctionDeclaration();

	public static final ISpecificFunctionDeclaration FIND_SIMOBJECT_FROM_INSTANCE_DECLARATION =
		new FindSimObjectFunctionDeclaration();

	public static final ISpecificFunctionDeclaration INSTANCE_OF_SPECIFIC_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"InstanceOf",
			BooleanType.getInstance(),
			CollectionsHelper.createList(
				GenericSimObjectType.getGeneralSimObjectType(),
				ObjectClassDeclarationType.getInstance()),
			true) {
		{
			setDocumentation("Returns the verity content if object (Arg1) is an instance of object class (Arg2)");
		}
		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(2, functions.size());
			return new InstanceOfObjectClassFunction(functions.getFunction(0), functions.getFunction(1));
		}
	};

	public static final ISpecificFunctionDeclaration HAS_FEATURE_SPECIFIC_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"HasFeature",
			BooleanType.getInstance(),
			CollectionsHelper.createList(
				FeatureClassDeclarationType.getInstance(),
				GenericSimObjectType.getGeneralSimObjectType()),
			true) {
		{
			setDocumentation("Returns the verity content if (Arg1) is one of the features of object (Arg2)");
		}

		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(2, functions.size());

			return new HasFeatureFunction(functions.getFunction(0), functions.getFunction(1));
		}
	};

	public static final ISpecificFunctionDeclaration IS_DEAD_SPECIFIC_FUNCTION_DECLARATION =
		new AbstractSpecificFunctionArgsDeclaration(
			"IsDead",
			BooleanType.getInstance(),
			CollectionsHelper.createList(GenericSimObjectType.getGeneralSimObjectType()),
			false) {

		{
			setDocumentation("Returns the verity content if its operand is dead");
		}

		public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
			Assert.assertEquals(1, functions.size());

			return new IsDeadFunction(functions.getFunction(0));
		}
	};

	public synchronized static void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
		if (_cachedFunctionDeclarationList == null) {
			_cachedFunctionDeclarationList = new FunctionDeclarationList();
			insertFunctionsIntern(_cachedFunctionDeclarationList);
		}

		functionDeclarationList.addAll(_cachedFunctionDeclarationList);
	}

	private static void insertFunctionsIntern(IFunctionDeclarationList functionDeclarationList) {
		// GetWorldBody
		functionDeclarationList.add(GET_WORLD_FUNCTION_DECLARATION);

		// GetMyBody
		functionDeclarationList.add(GET_CURRENT_SIMOBJECT_FUNCTION_DECLARATION);

		// GetMyCurrentActivity
		functionDeclarationList.add(GET_MY_CURR_ACTIVITY_FUNCTION_DECLARATION);

		// GetCurrentActivity
		functionDeclarationList.add(GET_CURR_ACTIVITY_FUNCTION_DECLARATION);

		// IsInActivity
		functionDeclarationList.add(IS_IN_ACTIVITY_FUNCTION_DECLARATION);

		// GetTime
		functionDeclarationList.add(GET_TIME_FUNCTION_DECLARATION);

		// getAllObjects
		functionDeclarationList.add(GET_ALL_OBJECTS_GENERIC_FUNCTION_DECLARATION);

		// getAllObjectsOfType
		functionDeclarationList.add(GET_ALL_OBJECTS_OF_TYPE_SPECIFIC_FUNCTION_DECLARATION);

		// getAllObjectsWithFeature
		functionDeclarationList.add(GET_ALL_OBJECTS_WITH_FEATURE_SPECIFIC_FUNCTION_DECLARATION);

		//findSimObjectFromInstanceDeclaration
		functionDeclarationList.add(FIND_SIMOBJECT_FROM_INSTANCE_DECLARATION);

		// instanceOf
		functionDeclarationList.add(INSTANCE_OF_SPECIFIC_FUNCTION_DECLARATION);

		// hasFeature
		functionDeclarationList.add(HAS_FEATURE_SPECIFIC_FUNCTION_DECLARATION);

		// isDead
		functionDeclarationList.add(IS_DEAD_SPECIFIC_FUNCTION_DECLARATION);

		functionDeclarationList.add(OBJECT_ID_DECLARATION);
	}
}