package de.uniwue.ki.mas.basics.function.object;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author herrler
 * Created on 12.01.2004
 *
 */
public class GetAllObjectsGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {
		
    public GetAllObjectsGenericFunctionDeclaration() {
		super("GetAllObjects", GenericSimObjectType.getInstance(), false);
		setDocumentation(
			"Returns all current objects fitting to the return value."
				+ "The order is not specified and also not random.");
	}
	
	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createFunctionDeclaration(outputType,CollectionsHelper.createIterator(BooleanType.getInstance(),BooleanType.getInstance()));
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(boolean, boolean)";
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException("Must call #createFunctionDeclaration(ISpecificType, Iterator) instead!");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (outputType == null)
			outputType =
				GenericIteratorType.getInstance().createSpecificIteratorType(
					GenericSimObjectType.getGeneralSimObjectType());

		if (!(outputType instanceof SpecificIteratorType)) return null;
		 
		ISpecificType type = ((SpecificIteratorType)outputType).getIteratorSpecificType();

		ISpecificFunctionDeclaration specificFunctionDeclaration = null;

		if (GenericSimObjectType.getGeneralSimObjectType() == type)
			specificFunctionDeclaration = new GetAllObjectsSpecificFunctionDeclaration();
		if (type instanceof ISimObjectType)
			specificFunctionDeclaration = new GetAllObjectsSpecificFunctionDeclaration(outputType);

		if ((specificFunctionDeclaration != null)
			&& (outputType != null)
			&& !TypeManager.getInstance().isMoreGeneral(
				outputType,
				specificFunctionDeclaration.getSpecificOutputType())) {
			return null;

		} else {
			return specificFunctionDeclaration;
		}
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}
	
	public boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		return true;
	}


	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		if (outputType instanceof SpecificIteratorType) {
			ISpecificType type = ((SpecificIteratorType)outputType).getIteratorSpecificType();
			if (type == GenericSimObjectType.getGeneralSimObjectType())
				return true;
			if (type instanceof ISimObjectType)
				return true;
		}
		return false;
	}

}
