package de.uniwue.ki.mas.basics.function.object;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (04.09.2001 17:26:30)
 * @author Christoph Oechslein
 */

final class GetAllObjectsSpecificFunctionDeclaration
	extends AbstractGetAllObjectsSpecificFunctionDeclaration {
	
	private ISimObjectType simObjectType = null;

	public GetAllObjectsSpecificFunctionDeclaration() {
		super(
			"GetAllObjects",
			CollectionsHelper.createList(BooleanType.getInstance(), BooleanType.getInstance()));

		setDocumentation(
			"Returns all current objects (including the in this tick added objects and excluding the killed objects)."
				+ "The order is not specified and also not random.");
	}
	
	public GetAllObjectsSpecificFunctionDeclaration(ISpecificType outputType) {
			super(
				"GetAllObjects",
				outputType,
				CollectionsHelper.createList(BooleanType.getInstance(), BooleanType.getInstance()));
			
			simObjectType = (ISimObjectType)(((SpecificIteratorType)outputType).getIteratorSpecificType());

			setDocumentation(
				"Returns all current objects (including the in this tick added objects and excluding the killed objects)."
					+ "The order is not specified and also not random.");
		}
		
	public IGenericFunctionDeclaration getGenericFunctionDeclaration() {
				return FunctionDeclarationObjectFactory.GET_ALL_OBJECTS_GENERIC_FUNCTION_DECLARATION;
			}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(2, functions.size());
			
	    if (simObjectType.getObjectClassDeclaration()==null)
		return new GetAllObjectsFunction(functions.getFunction(0), functions.getFunction(1));
		else
		return new GetAllObjectsFunction(simObjectType.getObjectClassDeclaration(), functions.getFunction(0), functions.getFunction(1));
	}

}