package de.uniwue.ki.mas.basics.function.object;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class GetAllObjectsWithFeatureFunction extends AbstractGetAllObjectsFunction {
	private final IFunction _featureClassDeclarationFunction;

	public GetAllObjectsWithFeatureFunction(
		IFunction featureClassDeclarationFunction,
		IFunction includingAddedFunction,
		IFunction excludingRemovedFunction) {
		super(includingAddedFunction, excludingRemovedFunction);

		_featureClassDeclarationFunction = featureClassDeclarationFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		final IFeatureClassDeclaration featureClassDeclaration =
			(IFeatureClassDeclaration) _featureClassDeclarationFunction.execute(executeFunctionArgs);

		return selectObjects(executeFunctionArgs, new IPredicateFunction() {
			public boolean isElementOk(Object anObject) {
				IObject simObject = (IObject) anObject;
				return HasFeatureFunction.hasFeature(simObject, featureClassDeclaration);
			}
		});
	}

	public String getName() {
		return "GetAllObjectsWithFeature("
			+ _featureClassDeclarationFunction.getName()
			+ ", "
			+ getAddedAndRemovedName()
			+ ")";
	}
}