package de.uniwue.ki.mas.basics.function.object;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (06.09.2001 20:09:38)
 * @author Christoph Oechslein
 */

public final class HasFeatureFunction extends AbstractBooleanFunction {
	private final IFunction _featureClassDeclarationFunction;
	private final IFunction _objectFunction;

	public HasFeatureFunction(IFunction featureClassDeclarationFunction, IFunction objectFunction) {
		super();

		_featureClassDeclarationFunction = featureClassDeclarationFunction;
		_objectFunction = objectFunction;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		IFeatureClassDeclaration featureClassDeclaration =
			(IFeatureClassDeclaration) _featureClassDeclarationFunction.execute(executeFunctionArgs);

		IObject object = (IObject) _objectFunction.execute(executeFunctionArgs);

		if (object == null)
			throw new SimulationRuntimeException("Body is null!");

		return hasFeature(object, featureClassDeclaration);
	}

	public String getName() {
		return "HasFeature("
			+ _featureClassDeclarationFunction.getName()
			+ ", "
			+ _objectFunction.getName()
			+ ")";
	}

	public static final boolean hasFeature(
		IObject simObject,
		IFeatureClassDeclaration featureClassDeclaration) {

		return hasFeature(
			simObject.getObjectInstanceDeclaration().getObjectClassDeclaration(),
			featureClassDeclaration);
	}

	/**
	 * @param objectClassDeclaration IObjectClassDeclaration
	 * @param featureClassDeclaration IFeatureClassDeclaration
	 * @return boolean
	 */
	public static boolean hasFeature(
		IObjectClassDeclaration objectClassDeclaration,
		IFeatureClassDeclaration featureClassDeclaration) {

		Iterator it = objectClassDeclaration.getObjectFeatureClassDeclarationList().iterator();

		while (it.hasNext()) {
			IObjectFeatureClassDeclaration currIObjectFeatureClassDeclaration =
				(IObjectFeatureClassDeclaration) it.next();

			if (currIObjectFeatureClassDeclaration
				.getFeatureClassDeclaration()
				.equals(featureClassDeclaration))
				return true;
		}

		return false;
	}
}