package de.uniwue.ki.mas.basics.function.object;

import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (06.09.2001 20:09:38)
 * @author Christoph Oechslein
 */
public final class InstanceOfObjectClassFunction extends AbstractBooleanFunction {

	private final IFunction _objectFunction;
	private final IFunction _objectClassDeclarationFunction;

	public InstanceOfObjectClassFunction(
		IFunction objectFunction,
		IFunction objectClassDeclarationFunction) {
		super();

		_objectFunction = objectFunction;
		_objectClassDeclarationFunction = objectClassDeclarationFunction;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		IObject object = (IObject) _objectFunction.execute(executeFunctionArgs);

		if (object == null) {
		return false;
		
		} else {
		IObjectClassDeclaration objectClassDeclaration =
			(IObjectClassDeclaration) _objectClassDeclarationFunction.execute(executeFunctionArgs);

		return object.getObjectInstanceDeclaration().getObjectClassDeclaration() == objectClassDeclaration;
		}
	}

	public String getName() {
		return "InstanceOf("
			+ _objectFunction.getName()
			+ ", "
			+ _objectClassDeclarationFunction.getName()
			+ ")";
	}

}