package de.uniwue.ki.mas.basics.function.object;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.bool.AbstractBooleanFunction;
import de.uniwue.ki.mas.behaviour.activityGraph.IActivityGraphLeaf;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class IsInActivityFunction extends AbstractBooleanFunction {
	private final IFunction _activityFunction;
	private final IFunction _bodyFunction;

	public IsInActivityFunction(IFunction activityFunction, IFunction bodyFunction) {
		super();

		_activityFunction = activityFunction;
		_bodyFunction = bodyFunction;
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		IActivityGraphNodeDeclaration activityDeclaration =
			(IActivityGraphNodeDeclaration)_activityFunction.execute(executeFunctionArgs);

		IObject object = (IObject)_bodyFunction.execute(executeFunctionArgs);

		if (object == null)
			throw new SimulationRuntimeException("Object is null!");

		if (!(object instanceof IActivityAgent))
			throw new SimulationRuntimeException("Object is no activity agent: " + object.getName());

		IActivityAgent agent = (IActivityAgent)object;

		for (int i = 0; i < agent.getCurrentActivityGraphLeafs().length; i++) {
			IActivityGraphLeaf activityGraphLeaf = agent.getCurrentActivityGraphLeafs()[i];

			IActivityGraphLeafDeclaration currAtomicActivityDeclaration =
				(IActivityGraphLeafDeclaration) (activityGraphLeaf.getActivityGraphNodeDeclaration());

			if (activityDeclaration instanceof IActivityGraphComposedNodeDeclaration) {
				if (isInActivityGraphDeclaration(currAtomicActivityDeclaration,
					((IActivityGraphComposedNodeDeclaration)activityDeclaration)
						.getActivityGraphDeclaration())) {
					return true;
				}

			} else if (activityDeclaration.equals(currAtomicActivityDeclaration)) {
				return true;
			}
		}

		return false;
	}

	private boolean isInActivityGraphDeclaration(
		IActivityGraphNodeDeclaration currActivityDeclaration,
		IActivityGraphDeclaration activityGraphDeclaration) {
		if (currActivityDeclaration == null)
			return false;
		if (activityGraphDeclaration.equals(currActivityDeclaration.getActivityGraphDeclaration()))
			return true;
		else
			return isInActivityGraphDeclaration(
				currActivityDeclaration
					.getActivityGraphDeclaration()
					.getParentActivityGraphComposedNodeDeclaration(),
				activityGraphDeclaration);
	}

	public String getName() {
		return "IsInActivity(" + _activityFunction.getName() + _bodyFunction.getName() + ")";
	}
}