package de.uniwue.ki.mas.basics.function.random;
import java.util.List;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.math.RandomHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionListGetRandom extends AbstractFunctionDynamic {
	private final IFunction _setFunction;

	public FunctionListGetRandom(IFunction setFunction) {
		super();
		_setFunction = setFunction;
	}

	public final Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		List list = ExecuteIteratorFactory.createList(executeFunctionArgs, _setFunction);

		if (list.isEmpty()) {
			throw new EmptyIteratorSimulationRuntimeException("Empty List error in GetRandom");
		}

		try {
			return RandomHelper.chooseObject(list);

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public String getName() {
		return "GetRandom(" + _setFunction.getName() + ")";
	}
}