package de.uniwue.ki.mas.basics.function.random;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.math.RandomHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionRandomDouble extends AbstractDoubleFunction {
	private final IFunction fromNumberFunction;
	private final IFunction toNumberFunction;


	public FunctionRandomDouble(IFunction fromNumberFunction, IFunction toNumberFunction) {
		super();
		this.fromNumberFunction = fromNumberFunction;
		this.toNumberFunction = toNumberFunction;
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForDoubleStatic(executeFunctionArgs, fromNumberFunction, toNumberFunction);
	}


	public static final double executeForDoubleStatic(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunction fromNumberFunction,
		IFunction toNumberFunction)
		throws SimulationRuntimeException {

		double from = fromNumberFunction.executeForDouble(executeFunctionArgs);
		double to = toNumberFunction.executeForDouble(executeFunctionArgs);

		if (from > to)
			throw new SimulationRuntimeException(
				"Intervall [from; to] musn't be empty: from (" + from + ") > to (" + to + ").");

		return RandomHelper.getRandomDouble(from, to);
	}

	public String getName() {
		return "RandomDouble(" + fromNumberFunction.getName() + ", " + toNumberFunction.getName() + ")";
	}
}