package de.uniwue.ki.mas.basics.function.random;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.number.AbstractIntegerFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.math.RandomHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionRandomInt extends AbstractIntegerFunction {
	private final IFunction fromNumberFunction;
	private final IFunction toNumberFunction;

	public FunctionRandomInt(IFunction fromNumberFunction, IFunction toNumberFunction) {
		super();

		this.fromNumberFunction = fromNumberFunction;
		this.toNumberFunction = toNumberFunction;
	}

	public final int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		int from = fromNumberFunction.executeForInt(executeFunctionArgs);
		int to = toNumberFunction.executeForInt(executeFunctionArgs);

		if (from >= to)
			throw new SimulationRuntimeException(
				"Intervall [from; to) musn't be empty: from (" + from + ") >= to (" + to + ").");

		return RandomHelper.getRandomInt(from, to);
	}

	public String getName() {
		return "RandomInt(" + fromNumberFunction.getName() + ", " + toNumberFunction.getName() + ")";
	}
}