package de.uniwue.ki.mas.basics.function.random;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.math.RandomHelper;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionRandomNormalDouble extends AbstractDoubleFunction {
	private final IFunction _expectedNumberFunction;
	private final IFunction _varianceNumberFunction;

	public FunctionRandomNormalDouble(IFunction expectedNumberFunction, IFunction varianceNumberFunction) {
		super();

		_expectedNumberFunction = expectedNumberFunction;
		_varianceNumberFunction = varianceNumberFunction;
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForDoubleStatic(executeFunctionArgs, _expectedNumberFunction, _varianceNumberFunction);
	}

	public static final double executeForDoubleStatic(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunction mNumberFunction,
		IFunction s2NumberFunction)
		throws SimulationRuntimeException {

		return RandomHelper.getRandomNormalDouble(
			mNumberFunction.executeForDouble(executeFunctionArgs),
			s2NumberFunction.executeForDouble(executeFunctionArgs));
	}

	public String getName() {
		return "RandomNormal("
			+ _expectedNumberFunction.getName()
			+ ", "
			+ _varianceNumberFunction.getName()
			+ ")";
	}
}