package de.uniwue.ki.mas.basics.function.random;

import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (16.01.2002 13:39:36)
 * @author Christoph Oechslein
 */

final class RandomBooleanSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	private static final Logger _logger =
		Logger.getLogger(RandomBooleanSpecificFunctionArgsDeclaration.class.getName());

	public RandomBooleanSpecificFunctionArgsDeclaration() {
		super(
			"RandomBoolean",
			BooleanType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance()),
			false);

		setDocumentation("Returns a random boolean value with the probability given by (Arg1)");
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		Assert.assertEquals(1, functions.size());
		return new FunctionRandomBoolean(functions.getFunction(0));
	}

	// just for faster dynamic calling
	public boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {
		return FunctionRandomBoolean.executeForBooleanStatic(
			createAndExecuteFunctionArgs,
			functions.getFunction(0));
	}

	public String getSpecificInputTypeNameAt(int index) {
		return "prob";
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration probFunCall = composedFunctionCallInternDeclaration.getChild(0);

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& probFunCall.isConstant(createFunctionArgs)) {
			try {
				double value =
					probFunCall.createFunction(createFunctionArgs).executeForDouble(
						createFunctionArgs.createCreateAndExecuteFunctionArgs());

				if (value <= 0.0) {
					if (_logger.isLoggable(Level.INFO)) {
						_logger.info(
							"Transformed redundand random boolean generation'"
								+ probFunCall.getName()
								+ "'"
								+ " ( = 0.0 -> FALSE)");
					}

					return BooleanType.BOOLEAN_TYPE_FALSE_ARGUMENT;

				} else if (value >= 1.0) {
					if (_logger.isLoggable(Level.INFO)) {
						_logger.info(
							"Removed redundand random boolean generation'"
								+ probFunCall.getName()
								+ "'"
								+ " ( = 0.0 -> TRUE)");
					}

					return BooleanType.BOOLEAN_TYPE_TRUE_ARGUMENT;
				}

			} catch (SimulationRuntimeException e) {
				throw new ModelCompilingException(e);
			}
		}

		return super.createFunction(createFunctionArgs, composedFunctionCallInternDeclaration);
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration probFunCall = composedFunctionCallInternDeclaration.getChild(0);

		if (probFunCall.isConstant(createFunctionArgs)) {
			try {
				double value =
					probFunCall.createFunction(createFunctionArgs).executeForDouble(
						createFunctionArgs.createCreateAndExecuteFunctionArgs());

				if ((value <= 0.0) || value >= 1.0) {
					return true;
				}

			} catch (SimulationRuntimeException e) {
				throw new ModelCompilingException(e);
			}
		}

		return false;
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
			null,
			DoubleType.getInstance(),
			new Double(0.5));
	}
}