package de.uniwue.ki.mas.basics.function.random;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (16.01.2002 13:39:36)
 * @author Christoph Oechslein
 */

final class RandomNormalSpecificFunctionArgsDeclaration extends AbstractSpecificFunctionArgsDeclaration {
	public RandomNormalSpecificFunctionArgsDeclaration() {
		super(
			"RandomNormal",
			DoubleType.getInstance(),
			CollectionsHelper.createList(DoubleType.getInstance(), DoubleType.getInstance()),
			false);

		setDocumentation("Returns a random value from a distribution with a mean (Arg1) and a variance (Arg2)");
	}

	/**
	 * @param createFunctionArgs ICreateFunctionArgs
	 * @param functions IFunctionList
	 * @return IFunction
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration#createFunction(ICreateFunctionArgs, IFunctionList)
	 */
	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		Assert.assertEquals(2, functions.size());
		return new FunctionRandomNormalDouble(functions.getFunction(0), functions.getFunction(1));
	}

	/**
	 * @param createAndExecuteFunctionArgs ICreateAndExecuteFunctionArgs
	 * @param functions IFunctionList
	 * @return double
	 * @throws SimulationRuntimeException
	 * @see de.uniwue.ki.mas.basics.function.IParametricFunction#executeForDouble(ICreateAndExecuteFunctionArgs, IFunctionList)
	 */
	// just for faster dynamic calling
	public double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {
		return FunctionRandomNormalDouble.executeForDoubleStatic(
			createAndExecuteFunctionArgs,
			functions.getFunction(0),
			functions.getFunction(1));
	}

	/**
	 * @param index int
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration#getSpecificInputTypeNameAt(int)
	 */
	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0)
			return "mean";
		else
			return "variance";
	}

}