package de.uniwue.ki.mas.basics.function.rest;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.GenericType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * cast: <T1> cast(<T2>)
 * T2 is unspecific
 * Creation date: (27.10.2000 17:15:57)
 *
 * @author Christoph Oechslein
 */
final class CastGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("type");

	public CastGenericFunctionDeclaration() {
		super("Cast", GenericType.getInstance(), CollectionsHelper.createList(typeDependentT2), true);
		
		setDocumentation("Cast one type into another type, with can lead to errors!");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		return createFunctionDeclaration(outputType, bindings.getBindingFor(typeDependentT2));
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		final ISpecificType outputType,
		final ISpecificType typeT2) {

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return outputType;
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return typeT2;

				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return (IFunction) functions.get(0);
			}
		};

	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		ISpecificType typeT2 = getNextSpecificType(inputTypes);

		if (inputTypes.hasNext())
			return null;

		return createFunctionDeclaration(outputType, typeT2);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException("Must call #createFunctionDeclaration(ISpecificType, Iterator) instead!");
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Object)";
	}
}