package de.uniwue.ki.mas.basics.function.rest;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * DoWith: <T1> DoWith (fun<(T2) T1>, T2)
 * T2 is unspecific
 * Creation date: (04.09.2001 16:53:35)
 * @author Christoph Oechslein
 */

final class DoWithGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT = new UnconstraintDependentType("type");

	public DoWithGenericFunctionDeclaration() {
		super("DoWith", GenericType.getInstance(), CollectionsHelper.createList(typeDependentT), true);
		
		setDocumentation("Uses function (Arg1) on element (Arg2)");
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificFunctionType specificFunctionType,
		ISpecificType specificType) {

		return new DoWithSpecificFunctionDeclaration(this, specificFunctionType, specificType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT2 = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(
			GenericFunctionType.getInstance().createSpecificFunctionType(
				outputType,
				CollectionsHelper.createList(typeT2)),
			typeT2);
	}


	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificFunctionType))
			return null;

		SpecificFunctionType functiontypeT = (SpecificFunctionType) specificTypeA;

		if ((functiontypeT.getSpecificInputTypesLength() != 1)
			|| (!functiontypeT.getSpecificInputTypeAt(0).isMoreSpecific(specificTypeB)))
			return null;

		return createFunctionDeclaration(functiontypeT, specificTypeB);
	}


	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		// DoWith: void DoWith (fun<(T) void>, T)
		// T is unspecific

		return "(Fun<(T) " + outputType.getName() + "> , T)";
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return VoidType.getInstance().equals(outputType) || super.canCreateFunctionDeclarationWith(outputType);
	}
}