package de.uniwue.ki.mas.basics.function.rest;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificFunctionType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * Creation date: (29.01.2002 11:02:02)
 * @author Christoph Oechslein
 */
public class DoWithSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {
	private final ISpecificFunctionType _argumentSpecificFunctionType;
	private final ISpecificType _argumentSpecificType;

	public DoWithSpecificFunctionDeclaration(
		DoWithGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificFunctionType specificFunctionType,
		ISpecificType specificType) {

		super(genericFunctionDeclaration, 2);

		_argumentSpecificFunctionType = specificFunctionType;
		_argumentSpecificType = specificType;
	}
	
	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return getArgumentSpecificFunctionType();

		} else if (index == 1) {
			return getArgumentSpecificType();

		} else {
			Assert.fail();
			return null;
		}

	}

	public ISpecificType getSpecificOutputType() {
		return getArgumentSpecificFunctionType().getSpecificOutputType();
	}

	public ISpecificFunctionType getArgumentSpecificFunctionType() {
		return _argumentSpecificFunctionType;
	}

	public ISpecificType getArgumentSpecificType() {
		return _argumentSpecificType;
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		throw new UnsupportedOperationException();
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(composedFunctionCallInternDeclaration);

		IComposedFunctionCallInternDeclaration newComposedFunctionCallInternDeclaration =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				specificFunctionDeclaration,
				getArgumentsComposedFunctionCallDeclList(composedFunctionCallInternDeclaration));
		
		return newComposedFunctionCallInternDeclaration.createFunction(createFunctionArgs);
	}

	private List getArgumentsComposedFunctionCallDeclList(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		List list = new LinkedList();

		HashMap objectMapping = new HashMap();

		for (int i = 1; i < composedFunctionCallInternDeclaration.getChildCount(); i++) {
			list.add(composedFunctionCallInternDeclaration.getChild(i).copy(objectMapping));
		}

		return list;
	}

	private ISpecificFunctionDeclaration getSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		if (composedFunctionCallInternDeclaration.getChild(0)
			instanceof IAtomicFunctionCallInternDeclaration)
			return (ISpecificFunctionDeclaration)
				((IAtomicFunctionCallInternDeclaration)composedFunctionCallInternDeclaration.getChild(0))
				.getArgumentDeclaration()
				.getObject();
		else
			return (ISpecificFunctionDeclaration)
				((IComposedFunctionCallInternDeclaration)composedFunctionCallInternDeclaration.getChild(0))
				.getSpecificFunctionDeclaration();
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(composedFunctionCallInternDeclaration);

		return specificFunctionDeclaration.isConstant(
			createFunctionArgs,
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				specificFunctionDeclaration,
				getArgumentsComposedFunctionCallDeclList(composedFunctionCallInternDeclaration)));
	}
}