package de.uniwue.ki.mas.basics.function.rest;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIfThenElse extends AbstractFunction {
	private final IFunction _functionIf;
	private final IFunction _functionThen;
	private final IFunction _functionElse;

	public FunctionIfThenElse(IFunction functionIf, IFunction functionThen, IFunction functionElse) {
		super();

		_functionIf = functionIf;
		_functionThen = functionThen;
		_functionElse = functionElse;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		if (_functionIf.executeForBoolean(executeFunctionArgs)) {
			_functionThen.act(executeFunctionArgs);
		} else {
			_functionElse.act(executeFunctionArgs);
		}
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		if (_functionIf.executeForBoolean(executeFunctionArgs)) {
			return _functionThen.execute(executeFunctionArgs);
		} else {
			return _functionElse.execute(executeFunctionArgs);
		}
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		if (_functionIf.executeForBoolean(executeFunctionArgs)) {
			return _functionThen.executeForBoolean(executeFunctionArgs);
		} else {
			return _functionElse.executeForBoolean(executeFunctionArgs);
		}
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		if (_functionIf.executeForBoolean(executeFunctionArgs)) {
			return _functionThen.executeForDouble(executeFunctionArgs);
		} else {
			return _functionElse.executeForDouble(executeFunctionArgs);
		}
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		if (_functionIf.executeForBoolean(executeFunctionArgs)) {
			return _functionThen.executeForInt(executeFunctionArgs);
		} else {
			return _functionElse.executeForInt(executeFunctionArgs);
		}
	}

	public String getName() {
		return "IfThenElse("
			+ _functionIf.getName()
			+ ", "
			+ _functionThen.getName()
			+ ", "
			+ _functionElse.getName()
			+ ")";
	}
}