package de.uniwue.ki.mas.basics.function.rest;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;

/**
 * Creation date: (27.10.2000 17:49:45)
 * @author Christoph Oechslein
 */

final class IfThenElseGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	private transient final Map _cache = new WeakHashMap();

	public IfThenElseGenericFunctionDeclaration() {
		super("If Then Else", GenericType.getInstance());

		setDocumentation(
			"Returns (or executes) the then part (Arg2) or the else part (Arg3) "
				+ "depending on the condition (Arg1).");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return TypeManager.getInstance().isInAllTypes(outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		if (_cache.containsKey(outputType)) {
			return (IfThenElseSpecificFunctionDeclaration)_cache.get(outputType);
		} else {
			IfThenElseSpecificFunctionDeclaration setComposedTypeValueSpecificFunctionDeclaration =
				new IfThenElseSpecificFunctionDeclaration(this, outputType);
			_cache.put(outputType, setComposedTypeValueSpecificFunctionDeclaration);
			return setComposedTypeValueSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		ISpecificType ifType = getNextSpecificType(inputArgs);
		ISpecificType thenType = getNextSpecificType(inputArgs);
		ISpecificType elseType = getNextSpecificType(inputArgs);

		if ((ifType == null) || (thenType == null) || (elseType == null) || (inputArgs.hasNext())) {
			return null;

		} else if (!BooleanType.getInstance().equals(ifType)) {
			return null;

		} else if (thenType.isMoreSpecific(elseType)) {
			return createFunctionDeclaration(thenType);

		} else if (elseType.isMoreSpecific(thenType)) {
			return createFunctionDeclaration(elseType);

		} else {
			return null;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		String outputName = outputType.getName();
		return "(Fun<() "
			+ BooleanType.getInstance().getName()
			+ ">, "
			+ " Fun<() "
			+ outputName
			+ ">, "
			+ " Fun<() "
			+ outputName
			+ ">)";
	}
}