package de.uniwue.ki.mas.basics.function.rest;

import java.util.logging.Level;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: 12.06.2002, 13:23:58
 * @author oechslein
 */
final class IfThenElseSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final ISpecificType _outputType;

	public IfThenElseSpecificFunctionDeclaration(
		IfThenElseGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType outputType) {
		super(genericFunctionDeclaration, 3);

		_outputType = outputType;
	}

	public ISpecificType getSpecificOutputType() {
		return _outputType;
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return BooleanType.getInstance();

		} else if (index == 1) {
			return _outputType;

		} else if (index == 2) {
			return _outputType;

		} else {
			Assert.fail();
			return null;
		}
	}
	
	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0) {
			return "predicate";

		} else if (index == 1) {
			return "then";

		} else if (index == 2) {
			return "else";
		} else {
			Assert.fail();
			return null;
		}
	}


	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
		return createFunctionIntern(createFunctionArgs, functions);
	}

	private final IFunction createFunctionIntern(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionList functions) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(3, functions.size());

		IFunction ifFunction = functions.getFunction(0);

		if (!createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()) {
			return new FunctionIfThenElse(ifFunction, functions.getFunction(1), functions.getFunction(2));
		} else if (ifFunction == BooleanType.BOOLEAN_TYPE_TRUE_ARGUMENT) {
			return functions.getFunction(1);
		} else if (ifFunction == BooleanType.BOOLEAN_TYPE_FALSE_ARGUMENT) {
			return functions.getFunction(2);
		} else {
			return new FunctionIfThenElse(ifFunction, functions.getFunction(1), functions.getFunction(2));
		}
	}

	private boolean getConstantValue(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration testFunCall = composedFunctionCallInternDeclaration.getChild(0);
		try {
			return testFunCall.createFunction(createFunctionArgs).executeForBoolean(
				createFunctionArgs.createCreateAndExecuteFunctionArgs());
		} catch (SimulationRuntimeException e) {
			throw new ModelCompilingException(e);
		}
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration testFunCall = composedFunctionCallInternDeclaration.getChild(0);

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& testFunCall.isConstant(createFunctionArgs)) {
			if (getConstantValue(createFunctionArgs, composedFunctionCallInternDeclaration)) {
				if (Assert.ASSERTION_ON && FunctionDeclarationCompiler.LOGGER.isLoggable(Level.INFO)) {
					FunctionDeclarationCompiler.LOGGER.info(
						"Removed redundand IF test ( = TRUE) inserted '"
							+ composedFunctionCallInternDeclaration.getChild(1).getName()
							+ "' in '"
							+ composedFunctionCallInternDeclaration.getName()
							+ "'");
				}

				return composedFunctionCallInternDeclaration.getChild(1).createFunction(createFunctionArgs);

			} else {
				if (Assert.ASSERTION_ON && FunctionDeclarationCompiler.LOGGER.isLoggable(Level.INFO)) {
					FunctionDeclarationCompiler.LOGGER.info(
						"Removed redundand IF test ( = FALSE) inserted '"
							+ composedFunctionCallInternDeclaration.getChild(2).getName()
							+ "' in '"
							+ composedFunctionCallInternDeclaration.getName()
							+ "'");
				}
				return composedFunctionCallInternDeclaration.getChild(2).createFunction(createFunctionArgs);
			}

		} else {
			try {
				return createFunctionIntern(
					createFunctionArgs,
					FunctionCallDeclarationCompiler.createFunctionList(
						createFunctionArgs,
						composedFunctionCallInternDeclaration));

			} catch (ModelCompilingException exception) {
				throw new FunctionCallInternDeclCompileException(
					composedFunctionCallInternDeclaration,
					exception);
			}
		}
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		IFunctionCallInternDeclaration testFunCall = composedFunctionCallInternDeclaration.getChild(0);

		if (testFunCall.isConstant(createFunctionArgs)) {
			if (getConstantValue(createFunctionArgs, composedFunctionCallInternDeclaration)) {
				return composedFunctionCallInternDeclaration.getChild(1).isConstant(createFunctionArgs);

			} else {
				return composedFunctionCallInternDeclaration.getChild(2).isConstant(createFunctionArgs);
			}

		} else
			return super.isConstant(createFunctionArgs, composedFunctionCallInternDeclaration);
	}
}
