package de.uniwue.ki.mas.basics.function.rest;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author Rainer Herrler
 */
final class InstanceOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType typeDependentT2 = new UnconstraintDependentType("type");

	public InstanceOfGenericFunctionDeclaration() {
		super("InstanceOfType", BooleanType.getInstance(), CollectionsHelper.createList(typeDependentT2), true);
		
		setDocumentation("Returns true if the object (Arg1) has the same type as the output type.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {
		final ISpecificType typeT2 = bindings.getBindingFor(typeDependentT2);
		return createFunctionDeclaration(typeT2);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT2) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			
//			public String getName() {
//				//super.getName();
//				return "InstanceOfType<"+typeT2+">";
//			}


			public ISpecificType getSpecificOutputType() {
				return BooleanType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) 
					return ObjectType.getInstance();
				
				Assert.fail();
				return null;
			}

			public String getSpecificInputTypeNameAt(int index) {
				if (index == 0) return "Object";
				return super.getSpecificInputTypeNameAt(index);
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				final IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new IFunction() {
					public String getName() {
						return "InstanceOfType<"+typeT2+">("+((IFunction)functions.get(0)).getName()+")";
					}
					
					public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
						buff.append(getName());
					}
					
					public void act(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
							throw new UnsupportedOperationException();
					}

					
					public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
							return typeT2.isArgumentDeclarationValueOk(((IFunction)functions.getFunction(0)).execute(executeFunctionArgs));		
					}
					
					public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
							throw new UnsupportedOperationException();
					}

					public int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
							throw new UnsupportedOperationException();
					}

					public Object execute(IExecuteFunctionArgs executeFunctionArgs)
						throws SimulationRuntimeException {
					 	return Boolean.valueOf(executeForBoolean(executeFunctionArgs));
					}
				};
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		ISpecificType typeT2 = getNextSpecificType(inputTypes);

		if (inputTypes.hasNext())
			return null;

		return createFunctionDeclaration(typeT2);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException("Must call #createFunctionDeclaration(ISpecificType, Iterator) instead!");
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Object)";
	}
}