package de.uniwue.ki.mas.basics.function.statistic;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionArgsDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.GenericIteratorType;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (04.09.2001 16:51:57)
 * @author Christoph Oechslein
 */

public final class FunctionDeclarationStatisticFactory {
	private static IFunctionDeclarationList _cachedFunctionDeclarationList;

	public synchronized static void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
				if (_cachedFunctionDeclarationList == null) {
					_cachedFunctionDeclarationList = new FunctionDeclarationList();
					insertFunctionsIntern(_cachedFunctionDeclarationList);
		}

		functionDeclarationList.addAll(_cachedFunctionDeclarationList);
	}

	private static void insertFunctionsIntern(IFunctionDeclarationList functionDeclarationList) {
		// Number mean(set<Number>)
		AbstractSpecificFunctionArgsDeclaration meanSpecificFunctionDeclaration =
			new AbstractSpecificFunctionArgsDeclaration(
				FunctionIteratorMean.NAME,
				DoubleType.getInstance(),
				CollectionsHelper.createList(GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())),
				true) {

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionIteratorMean(functions.getFunction(0));
			}
		};

		meanSpecificFunctionDeclaration.setDocumentation("Returns the mean of its Set");
		functionDeclarationList.add(meanSpecificFunctionDeclaration);

		// Number variance(set<Number>)
		AbstractSpecificFunctionArgsDeclaration varianceSpecificFunctionDaclaration =
			new AbstractSpecificFunctionArgsDeclaration(
				FunctionIteratorVariance.NAME,
				DoubleType.getInstance(),
				CollectionsHelper.createList(GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())),
				true) {

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionIteratorVariance(functions.getFunction(0));
			}
		};

		varianceSpecificFunctionDaclaration.setDocumentation("Returns the variance of its Set");
		functionDeclarationList.add(varianceSpecificFunctionDaclaration);

		// Number stdDeviation(set<Number>)
		AbstractSpecificFunctionArgsDeclaration stdDeviationSpecificFunctionDeclaration =
			new AbstractSpecificFunctionArgsDeclaration(
				FunctionIteratorStandardDeviation.NAME,
				DoubleType.getInstance(),
				CollectionsHelper.createList(GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())),
				true) {

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionIteratorStandardDeviation(functions.getFunction(0));
			}
		};

		stdDeviationSpecificFunctionDeclaration.setDocumentation("Returns the standard deviation of its Set");
		functionDeclarationList.add(stdDeviationSpecificFunctionDeclaration);

		// Number stdError(set<Number>)
		AbstractSpecificFunctionArgsDeclaration stdErrorSpecificFunctionDeclaration =
			new AbstractSpecificFunctionArgsDeclaration(
				FunctionIteratorStandardError.NAME,
				DoubleType.getInstance(),
				CollectionsHelper.createList(GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance())),
				true) {

			public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(1, functions.size());

				return new FunctionIteratorStandardError(functions.getFunction(0));
			}
		};

		stdErrorSpecificFunctionDeclaration.setDocumentation("Returns the standard error of its Set");
		functionDeclarationList.add(stdErrorSpecificFunctionDeclaration);
	}
}