package de.uniwue.ki.mas.basics.function.statistic;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorMean extends AbstractDoubleFunction {
	static String NAME = "Mean";
	private final IFunction _setFunction;

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		try {

			double sum = 0;
			double count = 0;
			IExecuteIterator it =
				ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _setFunction);

			while (it.hasNext()) {
				sum += ExecuteIteratorHelper.nextExecuteForDouble(it);
				count++;
			}

			return sum / count;

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	public String getName() {
		return NAME + " (" + _setFunction.getName() + ")";
	}

	public FunctionIteratorMean(IFunction setFunction) {
		super();

		_setFunction = setFunction;
	}
}