package de.uniwue.ki.mas.basics.function.statistic;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorStandardDeviation extends AbstractDoubleFunction {
	static String NAME = "StdDeviation";
	private final IFunction _setFunction;

	public FunctionIteratorStandardDeviation(IFunction setFunction) {
		super();

		_setFunction = setFunction;
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForDoubleStatic(executeFunctionArgs, _setFunction);
	}

	public static final double executeForDoubleStatic(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunction setFunction)
		throws SimulationRuntimeException {
			
		double variance = FunctionIteratorVariance.executeForDoubleStatic(executeFunctionArgs, setFunction);
		
		if (Double.isNaN(variance) || Double.isInfinite(variance))
			return variance;
		else
			return Math.sqrt(variance);
	}

	public String getName() {
		return NAME + " (" + _setFunction.getName() + ")";
	}
}