package de.uniwue.ki.mas.basics.function.statistic;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorStandardError extends AbstractDoubleFunction {
	static String NAME = "StdError";
	private final IFunction _setFunction;

	public FunctionIteratorStandardError(IFunction setFunction) {
		super();

		_setFunction = setFunction;
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		// compute the (n-1) stdDeviation and divide it by sqrt(n)
		try {

			double sum = 0;
			double sumSq = 0;
			double count = 0;
			IExecuteIterator it =
				ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, _setFunction);

			while (it.hasNext()) {
				double value = ((Number) it.nextWithException()).doubleValue();

				sum += value;
				sumSq += value * value;

				count++;
			}

			if (count == 0) {
				return Double.NaN;

			} else if (count == 1) {
				return 0;

			} else {
				return Math.sqrt((sumSq - (sum * sum) / count) / (count * (count - 1)));
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return NAME + " (" + _setFunction.getName() + ")";
	}
}