package de.uniwue.ki.mas.basics.function.statistic;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.number.AbstractDoubleFunction;
import de.uniwue.ki.mas.model.*;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionIteratorVariance extends AbstractDoubleFunction {
	static String NAME = "Variance";
	private final IFunction _setFunction;

	public FunctionIteratorVariance(IFunction setFunction) {
		super();

		_setFunction = setFunction;
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return executeForDoubleStatic(executeFunctionArgs, _setFunction);
	}

	public static final double executeForDoubleStatic(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunction setFunction)
		throws SimulationRuntimeException {
		// compute the (n-1) variance
		try {

			double sum = 0;
			double sumSq = 0;
			double count = 0;
			IExecuteIterator it =
				ExecuteIteratorFactory.createExecuteIterator(executeFunctionArgs, setFunction);

			while (it.hasNext()) {
				double value = ExecuteIteratorHelper.nextExecuteForDouble(it);

				sum += value;
				sumSq += value * value;

				count++;
			}

			if (count == 0) {
				return Double.NaN;

			} else if (count == 1) {
				return 0;

			} else {
				return ((sumSq - (sum * sum) / count)) / (count - 1);
			}

		} catch (Exception e) {
			if (e instanceof SimulationRuntimeException)
				throw (SimulationRuntimeException) e;
			else if (e instanceof ModelCompilingException)
				throw new SimulationRuntimeCausedByModelCompileException((ModelCompilingException) e);
			else
				throw new RuntimeException(e);
		}
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return NAME + " (" + _setFunction.getName() + ")";
	}
}