package de.uniwue.ki.mas.basics.function.string;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionSubstring extends AbstractFunctionDynamic {
	private final IFunction _stringIndexFunction;
	private final IFunction _startIndexFunction;
	private final IFunction _endIndexFunction;

	public FunctionSubstring(
		IFunction stringIndexFunction,
		IFunction startIndexFunction,
		IFunction endIndexFunction) {
		super();

		_stringIndexFunction = stringIndexFunction;
		_startIndexFunction = startIndexFunction;
		_endIndexFunction = endIndexFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		String string = (String) _stringIndexFunction.execute(executeFunctionArgs);
		int start = _startIndexFunction.executeForInt(executeFunctionArgs);
		int end = _endIndexFunction.executeForInt(executeFunctionArgs);

		if (end > string.length()) {
			throw new SimulationRuntimeException(
				"end index (" + end + ") is greater than string length in " + getName() + ".");
		}

		if (start < 0) {
			throw new SimulationRuntimeException(
				"start index (" + start + ") is lower than 0 in " + getName() + ".");
		}

		if (start > end) {
			throw new SimulationRuntimeException(
				"start index (" + start + ") is greater than end index (" + end + ") in " + getName() + ".");
		}

		return string.substring(start, end);
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			buff.append("Substring(");
			_stringIndexFunction.appendName(buff, currDepth + 1, maxDepth);
			buff.append(", ");
			_startIndexFunction.appendName(buff, currDepth + 1, maxDepth);
			buff.append(", ");
			_endIndexFunction.appendName(buff, currDepth + 1, maxDepth);
			buff.append(')');
		}
	}

	public String getName() {
		return getNameIntern();
	}
}