package de.uniwue.ki.mas.basics.function.variable;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.body.BodyVariableNotInitialized;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionActionIncrementVariable extends AbstractFunctionAction implements IFunction {
	private ObjectClassVariableIndexCache _objectClassVariableIndexCache;

	private final IFunction _variableFunction;
	private final IFunction _valueFunction;
	private final ISpecificType _numberType;

	public FunctionActionIncrementVariable(
		IFunction variableFunction,
		IFunction valueFunction,
		ISpecificType numberType) {
		super();

		_variableFunction = variableFunction;
		_valueFunction = valueFunction;
		_numberType = numberType;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IObjectExecuteFunctionArgs objectExecuteFunctionArgs =
			((IObjectExecuteFunctionArgs) executeFunctionArgs);
		IObject simObject = objectExecuteFunctionArgs.getObject();

		int index = getIndex(simObject, executeFunctionArgs);

		incrementValue(simObject, simObject, index, _valueFunction, _numberType, objectExecuteFunctionArgs);
	}

	private int getIndex(IObject fromObject, IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		Object obj = _variableFunction.execute(executeFunctionArgs);

		if (_objectClassVariableIndexCache != null) {
			return _objectClassVariableIndexCache.getIndexFor(fromObject, obj);

		} else if (obj instanceof Number) {
			return ((Number) obj).intValue();

		} else {
			_objectClassVariableIndexCache = new ObjectClassVariableIndexCache();
			return _objectClassVariableIndexCache.getIndexFor(fromObject, obj);
		}
	}

	public String getName() {
		return "IncrementVar(" + _variableFunction.getName() + ", " + _valueFunction.getName() + ")";
	}

	static void incrementValue(
		IObject fromObject,
		IObject settingAgent,
		int index,
		IFunction valueFunction,
		ISpecificType numberType,
		IObjectExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		IBody fromBody = fromObject.getBody();

		if (Assert.ASSERTION_ON && !fromBody.isBodyVariableInitialized(index))
			throw new BodyVariableNotInitialized(fromBody, index);

		IBodyVariable bodyVariable = fromBody.getBodyVariableAt(index);

		if (DoubleType.getInstance().equals(numberType)) {
			double oldValue = bodyVariable.getDoubleValue();
			double newValue = oldValue + valueFunction.executeForDouble(executeFunctionArgs);

			bodyVariable.setDoubleValue(newValue);

			if (executeFunctionArgs.hasObjectsVariableListener()) {
				executeFunctionArgs.fireVariableValueChanged(
					fromObject,
					settingAgent,
					bodyVariable,
					new Double(oldValue),
					new Double(newValue));
			}
		} else {
			int oldValue = bodyVariable.getIntValue();
			int newValue = oldValue + valueFunction.executeForInt(executeFunctionArgs);

			bodyVariable.setIntValue(newValue);

			if (executeFunctionArgs.hasObjectsVariableListener()) {
				executeFunctionArgs.fireVariableValueChanged(
					fromObject,
					settingAgent,
					bodyVariable,
					new Integer(oldValue),
					new Integer(newValue));
			}
		}
	}
}