package de.uniwue.ki.mas.basics.function.variable;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.SpecificVariableClassDeclarationType;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.agent.IAgentExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionActionIncrementVariableOf extends AbstractFunctionAction implements IFunction {
	private final IFunction _functionVariable;
	private final IFunction _functionObject;
	private final IFunction _functionValue;

	private final ISpecificType _numberType;

	public FunctionActionIncrementVariableOf(
		IFunction functionVariable,
		IFunction functionValue,
		IFunction functionObject,
		ISpecificType numberType) {
		super();

		_functionVariable = functionVariable;
		_functionValue = functionValue;
		_functionObject = functionObject;
		_numberType = numberType;
	}

	public final void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object variableOrIndexToVariable = _functionVariable.execute(executeFunctionArgs);

		IObject fromObject = (IObject) _functionObject.execute(executeFunctionArgs);

		if (fromObject == null)
			throw new SimulationRuntimeException("Object is null in " + getName());

		int index =
			SpecificVariableClassDeclarationType.getVariableIndexOf(
				fromObject.getBody(),
				variableOrIndexToVariable);

		IObject settingAgent = ((IAgentExecuteFunctionArgs) executeFunctionArgs).getObject();

		FunctionActionIncrementVariable.incrementValue(
			fromObject,
			settingAgent,
			index,
			_functionValue,
			_numberType,
			(IObjectExecuteFunctionArgs) executeFunctionArgs);
	}

	public String getName() {
		return "IncrementVarOf("
			+ _functionVariable.getName()
			+ ", "
			+ _functionValue.getName()
			+ ", "
			+ _functionObject.getName()
			+ ")";
	}
}