package de.uniwue.ki.mas.basics.function.variable;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionActionSetVariableOf extends AbstractFunctionAction implements IFunction {
	private final int _valueType;
	private ObjectClassVariableIndexCache _objectClassVariableIndexCache =
		new ObjectClassVariableIndexCache();

	private final IFunction _functionVariable;
	private final IFunction _functionBody;
	private final IFunction _functionValue;

	public FunctionActionSetVariableOf(
		ISpecificType type,
		IFunction functionVariable,
		IFunction functionValue,
		IFunction functionBody) {
		super();

		_valueType = FunctionActionSetVariable.convertToTypeValue(type);

		_functionVariable = functionVariable;
		_functionValue = functionValue;
		_functionBody = functionBody;
	}

	public final void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		Object variableOrIndexToVariable = _functionVariable.execute(executeFunctionArgs);

		IObject fromObject = (IObject) _functionBody.execute(executeFunctionArgs);

		if (fromObject == null)
			throw new SimulationRuntimeException("Error in SetVariableOf: Object == null");

		//int index = FunctionGetVariableOf.getVariableIndexOf(fromBody, variableOrIndexToVariable);
		int index = _objectClassVariableIndexCache.getIndexFor(fromObject, variableOrIndexToVariable);

		FunctionActionSetVariable.setValue(
			_valueType,
			(IObjectExecuteFunctionArgs) executeFunctionArgs,
			fromObject,
			index,
			_functionValue);
	}

	public String getName() {
		return "SetVariableOf("
			+ _functionVariable.getName()
			+ ", "
			+ _functionValue.getName()
			+ ", "
			+ _functionBody.getName()
			+ ")";
	}

}