package de.uniwue.ki.mas.basics.function.variable;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.body.BodyVariableNotInitialized;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

class FunctionGetVariable extends AbstractFunctionDynamic {
	private ObjectClassVariableIndexCache _objectClassVariableIndexCache;

	private final IFunction _variableFunction;

	public FunctionGetVariable(IFunction variableFunction) {
		super();

		_variableFunction = variableFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return getBodyVariable(executeFunctionArgs).getValue();
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return getBodyVariable(executeFunctionArgs).getBooleanValue();
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getDoubleValue();
	}

	public final int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getIntValue();
	}

	private IBodyVariable getBodyVariable(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		IObject fromObject = ((IObjectExecuteFunctionArgs) executeFunctionArgs).getObject();

		int index = getIndex(fromObject, executeFunctionArgs);

		IBody fromBody = fromObject.getBody();

		if (Assert.ASSERTION_ON) {
			if (!fromBody.isBodyVariableInitialized(index)) {
				throw new BodyVariableNotInitialized(fromBody, index);
			}
			Assert.assertTrue((index >= 0) && (index < fromBody.getBodyVariablesLength()));
		}

		return fromBody.getBodyVariableAt(index);
	}

	private int getIndex(IObject fromObject, IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		Object obj = _variableFunction.execute(executeFunctionArgs);

		if (_objectClassVariableIndexCache != null) {
			return _objectClassVariableIndexCache.getIndexFor(fromObject, obj);

		} else if (obj instanceof Number) {
			return ((Number) obj).intValue();

		} else {
			_objectClassVariableIndexCache = new ObjectClassVariableIndexCache();
			return _objectClassVariableIndexCache.getIndexFor(fromObject, obj);
		}
	}

	public String getName() {
		return "GetVariable(" + _variableFunction.getName() + ")";
	}
}