package de.uniwue.ki.mas.basics.function.variable;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.body.BodyVariableNotInitialized;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public class FunctionGetVariableOf extends AbstractFunctionDynamic {
	private final ObjectClassVariableIndexCache _objectClassVariableIndexCache =
		new ObjectClassVariableIndexCache();

	private final IFunction _variableFunction;
	private final IFunction _fromObjectFunction;

	public FunctionGetVariableOf(IFunction variableFunction, IFunction fromObjectFunction) {
		super();

		_variableFunction = variableFunction;
		_fromObjectFunction = fromObjectFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getValue();
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getBooleanValue();
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getDoubleValue();
	}

	public final int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		return getBodyVariable(executeFunctionArgs).getIntValue();
	}

	private IBodyVariable getBodyVariable(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException, BodyVariableNotInitialized {

		Object variableID = _variableFunction.execute(executeFunctionArgs);
		IObject fromObject = (IObject) _fromObjectFunction.execute(executeFunctionArgs);

		if (fromObject == null) {
			throw new SimulationRuntimeException("Error Object == null in " + getName());
		}

		IBody fromBody = fromObject.getBody();

		int index = _objectClassVariableIndexCache.getIndexFor(fromObject, variableID);

		if (Assert.ASSERTION_ON && !fromBody.isBodyVariableInitialized(index))
			throw new BodyVariableNotInitialized(fromBody, index);

		return fromBody.getBodyVariableAt(index);
	}

	public String getName() {
		return "GetVariableOf(" + _variableFunction.getName() + ", " + _fromObjectFunction.getName() + ")";
	}

}