package de.uniwue.ki.mas.basics.function.variable;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;

/**
 * <T> get_var(variable<T>, bodyImage)
 * Creation date: (27.10.2000 17:49:45)
 *
 * @author Christoph Oechslein
 */

final class GetVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents {

	private transient final Map _cache = new WeakHashMap();

	public GetVariableOfGenericFunctionDeclaration() {
		super("GetVariableFrom", GenericType.getInstance());

		setDocumentation("Returns a variable (Arg1) of an object (Arg2)");

	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return super.canCreateFunctionDeclarationWith(outputType)
			&& (TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(outputType)
				|| (GenericIteratorType.getInstance().canCreate(outputType)));

		// return VoidType.getInstance().equals(outputType);
	}

	private ISpecificType convertIteratorType(ISpecificType outputType) {
		if (outputType instanceof SpecificIteratorType) {
			SpecificIteratorType specificIteratorType = (SpecificIteratorType) outputType;
			return GenericListType.getInstance().createSpecificListType(
				specificIteratorType.getIteratorSpecificType());

		} else {
			return outputType;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		outputType = convertIteratorType(outputType);

		if (_cache.containsKey(outputType)) {
			return (GetVariableOfSpecificFunctionDeclaration) _cache.get(outputType);
		} else {
			GetVariableOfSpecificFunctionDeclaration getVariableOfSpecificFunctionDeclaration =
				new GetVariableOfSpecificFunctionDeclaration(this, outputType);
			_cache.put(outputType, getVariableOfSpecificFunctionDeclaration);
			return getVariableOfSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputArgs) {
		/* <T> get_var(variable<T>, bodyImage) */

		ISpecificType specificTypeA = getNextSpecificType(inputArgs);
		ISpecificType specificTypeB = getNextSpecificType(inputArgs);

		if ((specificTypeA == null) || (specificTypeB == null) || inputArgs.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType)
			|| (!specificTypeB.equals(GenericSimObjectType.getGeneralSimObjectType())))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		ISpecificType inVariableType = specificVariableClassDeclarationType.getVariableClassDeclarationType();

		if (TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(inVariableType))
			return createFunctionDeclaration(inVariableType);

		if (inVariableType instanceof ISpecificComposedType)
			return createFunctionDeclaration(inVariableType);

		if (GenericIteratorType.getInstance().canCreate(inVariableType)) {
			return createFunctionDeclaration(inVariableType);

		} else {
			return null;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		outputType = convertIteratorType(outputType);

		return "(Var<" + outputType.getName() + ">, " + GenericSimObjectType.getGeneralSimObjectType().getName() + ")";
	}
}