package de.uniwue.ki.mas.basics.function.variable;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (01.02.2002 10:52:31)
 * @author Christoph Oechslein
 */

public class GetVariableSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration {

	private final ISpecificType _outputType;

	public GetVariableSpecificFunctionDeclaration(
		GetVariableGenericFunctionDeclaration getVariableGenericFunctionDeclaration,
		ISpecificType outputType) {
		super(getVariableGenericFunctionDeclaration, 1);

		_outputType = outputType;
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(1, functions.size());

		IFunction variableFunction = functions.getFunction(0);

		return new FunctionGetVariable(variableFunction);
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(1, composedFunctionCallInternDeclaration.getChildCount());

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& isConstant(createFunctionArgs, composedFunctionCallInternDeclaration)) {
			try {
				return createConstantFunction(
					createFunctionArgs.createCreateAndExecuteFunctionArgs(),
					composedFunctionCallInternDeclaration.getChild(0));

			} catch (SimulationRuntimeException exception) {
				throw new ModelCompilingException(exception);
			}

		} else {
			return super.createFunction(createFunctionArgs, composedFunctionCallInternDeclaration);
		}
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index == 0) {
			return GenericVariableClassDeclarationType.getInstance(false, false).createSpecificVariableType(
				_outputType);

		} else {
			Assert.fail();
			return null;
		}

	}
	
	public String getSpecificInputTypeNameAt(int index) {
		if (index == 0) 
			return "variable";
			
		return super.getSpecificInputTypeNameAt(index);
	}


	public ISpecificType getSpecificOutputType() {
		return _outputType;
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		Iterator functionCallInternDeclarations = composedFunctionCallInternDeclaration.getChildren();

		Assert.assertTrue(functionCallInternDeclarations.hasNext());

		IFunctionCallInternDeclaration variableFunctionCallInternDeclaration =
			(IFunctionCallInternDeclaration) functionCallInternDeclarations.next();

		Assert.assertTrue(!functionCallInternDeclarations.hasNext());

		if (variableFunctionCallInternDeclaration instanceof IAtomicFunctionCallInternDeclaration) {
			return SpecificVariableClassDeclarationType.isConstantStatic(
				createFunctionArgs,
				getObjectFromVariableFunctionCallInternDecl(variableFunctionCallInternDeclaration));

		} else {
			return false;
		}
	}

	private static IFunction createConstantFunction(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionCallInternDeclaration variableFunctionCallInternDeclaration)
		throws ModelCompilingException, SimulationRuntimeException {

		IObject simObject = ((IObjectCreateFunctionArgs) createFunctionArgs).getObject();

		IBodyVariableClassDeclaration bodyVariableClassDeclaration =
			SpecificVariableClassDeclarationType.getBodyVariableClassDeclaration(
				simObject,
				getObjectFromVariableFunctionCallInternDecl(variableFunctionCallInternDeclaration));

		//		Object constantValue =
		//			bodyVariableInstanceDeclaration
		//				.getSpecificFunctionCallDeclaration()
		//				.createFunction(createAndExecuteFunctionArgs)
		//				.execute(createAndExecuteFunctionArgs);

		Object constantValue =
			simObject
				.getBody()
				.getBodyVariableAt(
					simObject
						.getObjectInstanceDeclaration()
						.getBodyInstanceDeclaration()
						.getBodyClassDeclaration()
						.getBodyVariableIndexOf(bodyVariableClassDeclaration))
				.getValue();

		return bodyVariableClassDeclaration.getVariableClassDeclaration().getSpecificType().createArgument(
			createFunctionArgs,
			constantValue);
	}

	private static Object getObjectFromVariableFunctionCallInternDecl(IFunctionCallInternDeclaration variableFunctionCallInternDeclaration) {
		return ((IAtomicFunctionCallInternDeclaration) variableFunctionCallInternDeclaration)
			.getArgumentDeclaration()
			.getObject();
	}
}