package de.uniwue.ki.mas.basics.function.variable;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.ConstraintGenericDependentType;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 31.01.2003, 18:30:12
 * @author oechslein
 */
public final class IncrementVarGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final ConstraintGenericDependentType _numberType =
		new ConstraintGenericDependentType("number Type", GenericNumberType.getInstance());

	private transient final Map _cache = new WeakHashMap();

	public IncrementVarGenericFunctionDeclaration() {
		super("IncrementVar", VoidType.getInstance(), CollectionsHelper.createList(_numberType));
		
		setDocumentation("Increments a variable (Arg1) by the value (Arg2)");
	}
	
	private IncrementVariableSpecificFunctionDeclaration createFunctionDeclaration(ISpecificType specificType) {
		if (_cache.containsKey(specificType)) {
			return (IncrementVariableSpecificFunctionDeclaration) _cache.get(specificType);
		} else {
			IncrementVariableSpecificFunctionDeclaration incrementVariableSpecificFunctionDeclaration =
				new IncrementVariableSpecificFunctionDeclaration(specificType);
			_cache.put(specificType, incrementVariableSpecificFunctionDeclaration);
			return incrementVariableSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		ISpecificType specificType = bindings.getBindingFor(_numberType);

		return createFunctionDeclaration(specificType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType typeA = getNextSpecificType(inputTypes);
		ISpecificType typeB = getNextSpecificType(inputTypes);

		if ((typeA == null) || (typeB == null) || (inputTypes.hasNext()))
			return null;

		if (!(typeA instanceof SpecificVariableClassDeclarationType)) {
			return null;
		}

		ISpecificType variableClassDeclarationType =
			((SpecificVariableClassDeclarationType) typeA).getVariableClassDeclarationType();

		if (variableClassDeclarationType.isMoreSpecific(typeB)
			&& DoubleType.getInstance().isMoreSpecific(typeB)) {
			return createFunctionDeclaration(variableClassDeclarationType);
		} else {
			return null;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Variable<T>, <T>)";
	}

}