package de.uniwue.ki.mas.basics.function.variable;
import java.io.Serializable;

import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variable.VariableCompileException;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Creation date: (21.02.2002 14:12:49)
 * @author Christoph Oechslein
 */

public final class ObjectClassVariableIndexCache implements Serializable {
	private IObjectClassDeclaration _oldCachedObjectClassDeclaration = null;
	private IVariableClassDeclaration _oldCachedVariableClassDeclaration = null;
	private int _oldCachedIndex = -1;

	public ObjectClassVariableIndexCache() {
		super();
	}

	public int getIndexForVariableClassDeclaration(
		IObject fromObject,
		IVariableClassDeclaration variableClassDeclaration)
		throws SimulationRuntimeException {

		IObjectClassDeclaration objectClassDeclaration =
			fromObject.getObjectInstanceDeclaration().getObjectClassDeclaration();

		IBody fromBody = fromObject.getBody();

		if ((variableClassDeclaration == _oldCachedVariableClassDeclaration)
			&& (_oldCachedObjectClassDeclaration == objectClassDeclaration)) {

			return _oldCachedIndex;
		}

		_oldCachedObjectClassDeclaration = objectClassDeclaration;
		_oldCachedVariableClassDeclaration = variableClassDeclaration;

		try {
			int index =
				fromBody.getBodyInstanceDeclaration().getBodyClassDeclaration().getBodyVariableIndexOf(
					variableClassDeclaration);

			if (index == -1)
				throw new VariableCompileException(
					"Variable " + variableClassDeclaration.getName() + " not found in " + fromBody.getName());

			_oldCachedIndex = index;

			return index;

		} catch (ModelCompilingException exception) {
			throw new SimulationRuntimeException(exception);
		}
	}

	public int getIndexFor(IObject fromObject, Object indexOrVariable) throws SimulationRuntimeException {
		if (indexOrVariable instanceof Number) {
			return ((Number) indexOrVariable).intValue();

		} else {
			return getIndexForVariableClassDeclaration(
				fromObject,
				(IVariableClassDeclaration) indexOrVariable);
		}
	}
}