package de.uniwue.ki.mas.basics.function.variable;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * Set_var(variable<T>, <T>)
  */

final class SetVariableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("Type");

	private transient final Map _cache = new WeakHashMap();

	public SetVariableGenericFunctionDeclaration() {
		super("SetVariable", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Changes the value of private variable (Arg1) to value (Arg2)");
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return super.canCreateFunctionDeclarationWith(outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType typeT) {
		if (_cache.containsKey(typeT)) {
			return (SetVariableSpecificFunctionDeclaration) _cache.get(typeT);
		} else {
			SetVariableSpecificFunctionDeclaration setVariableSpecificFunctionDeclaration =
				new SetVariableSpecificFunctionDeclaration(this, 2, typeT);
			_cache.put(typeT, setVariableSpecificFunctionDeclaration);
			return setVariableSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// Set_var(variable<T>, <T>)

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// Set_var(variable<T>, <T>)

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType typeT = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (typeT == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		ISpecificType variableClassDeclarationType =
			specificVariableClassDeclarationType.getVariableClassDeclarationType();

		if (!variableClassDeclarationType.isMoreSpecific(typeT))
			return null;

		else if (
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(variableClassDeclarationType))
			return createFunctionDeclaration(variableClassDeclarationType);

		else if (typeT instanceof ISpecificComposedType)
			return createFunctionDeclaration(variableClassDeclarationType);

		/*
		else if (GenericIteratorType.getInstance().canCreate(typeT))
			return createFunctionDeclaration(typeT);
		*/

		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<T>, T)";
	}
}