package de.uniwue.ki.mas.basics.function.variable;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * Set_var_of(variable<T>, <T>, BodyImage)
  */

final class SetVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("type");

	private transient final Map _cache = new WeakHashMap();

	public SetVariableOfGenericFunctionDeclaration() {
		super("SetVariableOf", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Changes the value of variable (Arg1) of an object (Arg3) to value (Arg2)");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType specificTypeT) {
		if (_cache.containsKey(specificTypeT)) {
			return (SetVariableOfSpecificFunctionDeclaration) _cache.get(specificTypeT);
		} else {
			SetVariableOfSpecificFunctionDeclaration setVariableOfSpecificFunctionDeclaration =
				new SetVariableOfSpecificFunctionDeclaration(this, 3, specificTypeT);
			_cache.put(specificTypeT, setVariableOfSpecificFunctionDeclaration);
			return setVariableOfSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// Set_var_of(variable<T>, <T>, BodyImage)

		ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// Set_var_of(variable<T>, <T>, BodyImage)

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType typeT = getNextSpecificType(inputTypes);
		ISpecificType bodyImageType = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (typeT == null) || (bodyImageType == null) || inputTypes.hasNext())
			return null;

//		Changed this to accept DelayedSimObjectType 27.Mai 2004
//		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType)
//			|| (!GenericSimObjectType.getGeneralSimObjectType().equals(bodyImageType)))
//			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType)
				|| (!(bodyImageType instanceof ISimObjectType)))
				return null;
				
		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;
		ISpecificType variableClassDeclarationType =
			specificVariableClassDeclarationType.getVariableClassDeclarationType();

		if (!variableClassDeclarationType.isMoreSpecific(typeT))
			return null;

		if (TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(variableClassDeclarationType))
			return createFunctionDeclaration(variableClassDeclarationType);

		if (variableClassDeclarationType instanceof ISpecificComposedType)
			return createFunctionDeclaration(variableClassDeclarationType);

		/*
		else if (GenericIteratorType.getInstance().canCreate(typeT))
			return createFunctionDeclaration(typeT);
		*/

		else
			return null;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<T>, T, Body)";
	}
}