package de.uniwue.ki.mas.basics.function.variable;
import java.util.LinkedList;
import java.util.List;

import junit.framework.TestCase;
import junit.swingui.TestRunner;
import de.uniwue.ki.mas.basics.function.object.FunctionDeclarationObjectFactory;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.ActivityAgentClassDeclarationFactory;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.sim.simulation.SimulationFunctionArgs;

/**
 * Creation date: (24.07.2001 15:18:13)
 * @author Christoph Oechslein
 */

public final class TestVariableFunction extends TestCase {
	private static final SpecificListType _numberListType =
		GenericListType.getInstance().createSpecificListType(DoubleType.getInstance());
	private static final SpecificListType _objectListType =
		GenericListType.getInstance().createSpecificListType(ObjectType.getInstance());

	private IObjectClassDeclaration _objectClassDeclaration;

	/**
	 * @param name String
	 */
	public TestVariableFunction(String name) {
		super(name);
	}

	private IFunctionCallDeclaration createFunctionCallDeclaration(
		ISpecificFunctionDeclaration specificGetVariableFunctionDecl,
		ISpecificType specificType1,
		Object value1) {

		return createFunctionCallDeclaration(
			specificGetVariableFunctionDecl,
			specificType1,
			value1,
			null,
			null);
	}

	private IFunctionCallDeclaration createFunctionCallDeclaration(
		ISpecificFunctionDeclaration specificGetVariableFunctionDecl,
		ISpecificType specificType1,
		Object value1,
		ISpecificType specificType2,
		Object value2) {

		return createFunctionCallDeclaration(
			specificGetVariableFunctionDecl,
			specificType1,
			value1,
			specificType2,
			value2,
			null,
			null);
	}

	private IFunctionCallDeclaration createFunctionCallDeclaration(
		ISpecificFunctionDeclaration specificGetVariableFunctionDecl,
		ISpecificType specificType1,
		Object value1,
		ISpecificType specificType2,
		Object value2,
		ISpecificType specificType3,
		Object value3) {

		assertNotNull(specificGetVariableFunctionDecl);

		List list = new LinkedList();

		if (specificType3 != null) {
			list.add(createFunctionCallInternDeclaration(specificType3, value3));
		}

		if (specificType2 != null) {
			list.add(0, createFunctionCallInternDeclaration(specificType2, value2));
		}

		if (specificType1 != null) {
			list.add(0, createFunctionCallInternDeclaration(specificType1, value1));
		}

		IFunctionCallDeclaration functionCallDeclaration =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
				FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
					null,
					specificGetVariableFunctionDecl,
					list));

		return functionCallDeclaration;
	}

	private IFunctionCallInternDeclaration createFunctionCallInternDeclaration(
		ISpecificType specificType,
		Object value) {

		if (value instanceof IFunctionCallInternDeclaration) {
			return (IFunctionCallInternDeclaration) value;

		} else if (value instanceof ISpecificFunctionDeclaration) {
			return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				(ISpecificFunctionDeclaration) value);

		} else if (value instanceof IGenericFunctionDeclaration) {
			return createFunctionCallInternDeclaration(
				specificType,
				((IGenericFunctionDeclaration) value).createFunctionDeclaration(
					specificType,
					DependentTypeBindings.EMPTY_BINDINGS));

		} else {
			return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				specificType,
				value);
		}
	}

	private ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType outputType,
		ISpecificType specificType1) {

		return getSpecificFunctionDeclaration(genericFunctionDeclaration, outputType, specificType1, null);
	}

	private ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType outputType,
		ISpecificType specificType1,
		ISpecificType specificType2) {

		return getSpecificFunctionDeclaration(
			genericFunctionDeclaration,
			outputType,
			specificType1,
			specificType2,
			null);
	}

	private ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType outputType,
		ISpecificType specificType1,
		ISpecificType specificType2,
		ISpecificType specificType3) {

		List list = new LinkedList();

		if (specificType1 != null) {
			list.add(specificType1);
		}

		if (specificType2 != null) {
			list.add(specificType2);
		}

		if (specificType3 != null) {
			list.add(specificType3);
		}

		ISpecificFunctionDeclaration specificGetVariableFunctionDecl =
			genericFunctionDeclaration.createFunctionDeclaration(outputType, list.iterator());

		return specificGetVariableFunctionDecl;
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		TestRunner.run(TestVariableFunction.class);
	}

	protected void setUp() {
		_objectClassDeclaration = ActivityAgentClassDeclarationFactory.createActivityAgentClassDeclaration();
	}

	protected void tearDown() {
		_objectClassDeclaration.remove();
		_objectClassDeclaration = null;
	}

	private void testCanCreateFunction(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		ISpecificType specificType,
		boolean shouldExist)
		throws Exception {

		assertEquals(shouldExist, genericFunctionDeclaration.canCreateFunctionDeclarationWith(specificType));
	}

	private void testCanCreateGetVariable(ISpecificType specificType, boolean shouldExist) throws Exception {

		testCanCreateFunction(
			FunctionDeclarationVariableFactory.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION,
			specificType,
			shouldExist);

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION,
				specificType,
				GenericVariableClassDeclarationType.getInstance(false, false).createSpecificVariableType(
					specificType));

		assertTrue(
			shouldExist ? (specificFunctionDeclaration != null) : (specificFunctionDeclaration == null));
	}

	private void testCanCreateGetVariableOf(ISpecificType specificType, boolean shouldExist)
		throws Exception {

		testCanCreateFunction(
			FunctionDeclarationVariableFactory.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION,
			specificType,
			shouldExist);

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION,
				specificType,
				GenericVariableClassDeclarationType.getInstance(true, false).createSpecificVariableType(
					specificType),
				GenericSimObjectType.getGeneralSimObjectType());

		assertTrue(
			shouldExist ? (specificFunctionDeclaration != null) : (specificFunctionDeclaration == null));
	}

	private void testCanCreateSetVariable(ISpecificType specificType, boolean shouldExist) throws Exception {

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.SET_VARIABLE_GENERIC_FUNCTION_DECLARATION,
				VoidType.getInstance(),
				GenericVariableClassDeclarationType.getInstance(false, true).createSpecificVariableType(
					specificType),
				specificType);

		assertTrue(
			shouldExist ? (specificFunctionDeclaration != null) : (specificFunctionDeclaration == null));
	}

	private void testCanCreateSetVariableOf(ISpecificType specificType, boolean shouldExist)
		throws Exception {

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.SET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION,
				VoidType.getInstance(),
				GenericVariableClassDeclarationType.getInstance(true, true).createSpecificVariableType(
					specificType),
				specificType,
				GenericSimObjectType.getGeneralSimObjectType());

		assertTrue(
			shouldExist ? (specificFunctionDeclaration != null) : (specificFunctionDeclaration == null));
	}

	/**
	 * @throws Exception
	 */
	public void testGetVariable() throws Exception {

		// test with NumberType, ObjectType, set<Number> and set<Object>
		testCanCreateGetVariable(DoubleType.getInstance(), true);
		testCanCreateGetVariable(ObjectType.getInstance(), true);
		testCanCreateGetVariable(_numberListType, true);
		testCanCreateGetVariable(_objectListType, true);

		testCanCreateGetVariable(VoidType.getInstance(), false);
		testCanCreateGetVariable(
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		testGetVariable(0.0);
		testGetVariable(5.0);
		testGetVariable(-5.0);

		testGetVariable((Math.random() - 0.5) * 1000);
	}

	private void testGetVariable(double value) throws Exception {
		IVariableClassDeclaration variableClassDeclaration =
			VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration(
				DoubleType.getInstance());

		variableClassDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(value)));

		IBodyVariableClassDeclaration bodyVariableClassDeclaration =
			variableClassDeclaration.createBodyVariableClassDeclaration();

		_objectClassDeclaration
			.getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.add(bodyVariableClassDeclaration);

		ISpecificType variableType =
			GenericVariableClassDeclarationType.getInstance(false, false).createSpecificVariableType(
				DoubleType.getInstance());

		ISpecificFunctionDeclaration specificGetVariableFunctionDecl =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION,
				DoubleType.getInstance(),
				variableType);

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				specificGetVariableFunctionDecl,
				variableType,
				bodyVariableClassDeclaration);

		testGetVariable(functionCallDeclaration, value);

	}

	private void testGetVariable(IFunctionCallDeclaration functionCallDeclaration, double value)
		throws Exception {
		IVariableClassDeclaration variableClassDeclaration =
			VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration(
				DoubleType.getInstance());

		variableClassDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(value)));

		IBodyVariableClassDeclaration bodyVariableClassDeclaration =
			variableClassDeclaration.createBodyVariableClassDeclaration();

		_objectClassDeclaration
			.getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.add(bodyVariableClassDeclaration);

		SimulationFunctionArgs simulationFunctionArgs = new SimulationFunctionArgs(null);

		IObjectInstanceDeclaration objectInstanceDeclaration =
			_objectClassDeclaration.createObjectInstanceDeclaration();

		IObject object = objectInstanceDeclaration.createObject(simulationFunctionArgs);
		object.initialize(simulationFunctionArgs);

		simulationFunctionArgs.setObject(object);

		object.reset(simulationFunctionArgs);

		assertEquals(
			value,
			functionCallDeclaration.createFunction(simulationFunctionArgs).executeForDouble(
				simulationFunctionArgs),
			0.0);

		object.remove();
		objectInstanceDeclaration.remove();
	}

	public void testGetVariableOf() throws Exception {

		// test with NumberType, ObjectType, set<Number> and set<Object>
		testCanCreateGetVariableOf(DoubleType.getInstance(), true);
		testCanCreateGetVariableOf(ObjectType.getInstance(), true);
		testCanCreateGetVariableOf(_numberListType, true);
		testCanCreateGetVariableOf(_objectListType, true);

		testCanCreateGetVariableOf(VoidType.getInstance(), false);
		testCanCreateGetVariableOf(
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		testGetVariableOf(0.0);
		testGetVariableOf(5.0);
		testGetVariableOf(-5.0);

		testGetVariableOf((Math.random() - 0.5) * 1000);
	}

	private void testGetVariableOf(double value) throws Exception {
		IVariableClassDeclaration variableClassDeclaration =
			VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration(
				DoubleType.getInstance());

		variableClassDeclaration.setExternal(true);

		variableClassDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(value)));

		IBodyVariableClassDeclaration bodyVariableClassDeclaration =
			variableClassDeclaration.createBodyVariableClassDeclaration();

		_objectClassDeclaration
			.getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.add(bodyVariableClassDeclaration);

		ISpecificType variableType =
			GenericVariableClassDeclarationType.getInstance(true, false).createSpecificVariableType(
				DoubleType.getInstance());

		ISpecificFunctionDeclaration specificGetVariableFunctionDecl =
			getSpecificFunctionDeclaration(
				FunctionDeclarationVariableFactory.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION,
				DoubleType.getInstance(),
				variableType,
				GenericSimObjectType.getGeneralSimObjectType());

		IFunctionCallDeclaration functionCallDeclaration =
			createFunctionCallDeclaration(
				specificGetVariableFunctionDecl,
				variableType,
				bodyVariableClassDeclaration,
				GenericSimObjectType.getGeneralSimObjectType(),
				FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
					null,
					FunctionDeclarationObjectFactory.GET_CURRENT_SIMOBJECT_FUNCTION_DECLARATION));

		testGetVariable(functionCallDeclaration, value);

	}

	/**
	 * @throws Exception
	 */
	public void testSetVariable() throws Exception {

		IGenericFunctionDeclaration setVariableGenericFunction =
			FunctionDeclarationVariableFactory.SET_VARIABLE_GENERIC_FUNCTION_DECLARATION;

		testCanCreateFunction(setVariableGenericFunction, VoidType.getInstance(), true);

		testCanCreateFunction(setVariableGenericFunction, DoubleType.getInstance(), false);
		testCanCreateFunction(setVariableGenericFunction, ObjectType.getInstance(), false);
		testCanCreateFunction(setVariableGenericFunction, _numberListType, false);
		testCanCreateFunction(setVariableGenericFunction, _objectListType, false);
		testCanCreateFunction(
			setVariableGenericFunction,
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		// test with NumberType, ObjectType, set<Number> and set<Object>
		testCanCreateSetVariable(DoubleType.getInstance(), true);
		testCanCreateSetVariable(ObjectType.getInstance(), true);
		testCanCreateSetVariable(_numberListType, true);
		testCanCreateSetVariable(_objectListType, true);

		testCanCreateSetVariable(
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		testCanCreateSetVariable(VoidType.getInstance(), false);
	}

	/**
	 * @throws Exception
	 */
	public void testSetVariableOf() throws Exception {

		IGenericFunctionDeclaration setVariableOfGenericFunction =
			FunctionDeclarationVariableFactory.SET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION;

		testCanCreateFunction(setVariableOfGenericFunction, VoidType.getInstance(), true);

		testCanCreateFunction(setVariableOfGenericFunction, DoubleType.getInstance(), false);
		testCanCreateFunction(setVariableOfGenericFunction, ObjectType.getInstance(), false);
		testCanCreateFunction(setVariableOfGenericFunction, _numberListType, false);
		testCanCreateFunction(setVariableOfGenericFunction, _objectListType, false);
		testCanCreateFunction(
			setVariableOfGenericFunction,
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		// test with NumberType, ObjectType, set<Number> and set<Object>
		testCanCreateSetVariableOf(DoubleType.getInstance(), true);
		testCanCreateSetVariableOf(ObjectType.getInstance(), true);
		testCanCreateSetVariableOf(_numberListType, true);
		testCanCreateSetVariableOf(_objectListType, true);

		testCanCreateSetVariableOf(
			GenericVariableClassDeclarationType.getInstance().createSpecificVariableType(
				DoubleType.getInstance()),
			false);

		testCanCreateSetVariableOf(VoidType.getInstance(), false);
	}
}