package de.uniwue.ki.mas.basics.functionArgument;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: (12.06.2001 14:20:58)
 * @author Christoph Oechslein
 */

public abstract class AbstractParametricFunction extends AbstractNamed implements IParametricFunction {

	public abstract IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException;

	public void act(ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs, IFunctionList functions)
		throws ModelCompilingException, SimulationRuntimeException {

		createFunction(createAndExecuteFunctionArgs, functions).act(createAndExecuteFunctionArgs);
	}

	public Object execute(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws ModelCompilingException, SimulationRuntimeException {

		return createFunction(createAndExecuteFunctionArgs, functions).execute(createAndExecuteFunctionArgs);
	}

	public boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws ModelCompilingException, SimulationRuntimeException {

		return createFunction(createAndExecuteFunctionArgs, functions).executeForBoolean(
			createAndExecuteFunctionArgs);
	}

	public double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws ModelCompilingException, SimulationRuntimeException {

		return createFunction(createAndExecuteFunctionArgs, functions).executeForDouble(
			createAndExecuteFunctionArgs);
	}

	public int executeForInt(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws ModelCompilingException, SimulationRuntimeException {

		return createFunction(createAndExecuteFunctionArgs, functions).executeForInt(
			createAndExecuteFunctionArgs);
	}

	public final IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return AbstractParametricFunction.createFunctionStatic(
			createFunctionArgs,
			this,
			composedFunctionCallInternDeclaration);
	}

	public static IFunction createFunctionStatic(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunction parametricFunction,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		try {
			return parametricFunction.createFunction(
				createFunctionArgs,
				FunctionCallDeclarationCompiler.createFunctionList(
					createFunctionArgs,
					composedFunctionCallInternDeclaration));

		} catch (ModelCompilingException exception) {
			throw new FunctionCallInternDeclCompileException(
				composedFunctionCallInternDeclaration,
				exception);
		}
	}
}