package de.uniwue.ki.mas.basics.functionArgument;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class ArgumentBindings extends AbstractMutableArgumentBindings {

	private final Object[] _currentValues;
	private final IFunctionArgument[] _functionArguments;

	private IFunctionList _functionList;

	public ArgumentBindings(IFunctionArgumentDeclarationList functionArgumentDeclarationList) {

		super();

		_currentValues = new Object[functionArgumentDeclarationList.size()];
		_functionArguments = createFunctionArguments(functionArgumentDeclarationList);
	}

	private static IFunctionArgument[] createFunctionArguments(IFunctionArgumentDeclarationList functionArgumentDeclarationList) {
		IFunctionArgument[] functionArguments = new IFunctionArgument[functionArgumentDeclarationList.size()];
		int i = 0;
		Iterator it = functionArgumentDeclarationList.iterator();
		while (it.hasNext()) {
			IFunctionArgumentDeclaration functionArgumentDeclaration =
				(IFunctionArgumentDeclaration)it.next();
			functionArguments[i++] = functionArgumentDeclaration.getFunctionArgument();
		}
		return functionArguments;
	}

	public boolean containsBindingFor(IFunctionArgument functionArgument) {
		for (int i = 0; i < _functionArguments.length; i++) {
			if (_functionArguments[i] == functionArgument) {
				return true;
			}
		}

		return super.containsBindingFor(functionArgument);
	}

	public Object getArgumentValueFor(IFunctionArgument functionArgument) throws SimulationRuntimeException {
		for (int i = 0; i < _functionArguments.length; i++) {
			if (_functionArguments[i] == functionArgument) {
				Assert.assertNotNull(_currentValues[i]);
				return _currentValues[i];
			}
		}

		return super.getArgumentValueFor(functionArgument);
	}

	protected void appendNameIntern(StringBuffer buff) {
		for (int i = 0; i < _functionArguments.length; i++) {
			appendName(
				buff,
				_functionArguments[i],
				(_functionList == null) ? null : _functionList.getFunction(i));

			if (i != _functionArguments.length - 1) {
				buff.append(", ");
			}
		}
	}

	public void calculateArguments(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		for (int i = 0; i < _currentValues.length; i++) {
			IFunction function = _functionList.getFunction(i);
			_currentValues[i] = function.execute(executeFunctionArgs);
		}
	}

	public void setFunctionList(IFunctionList newFunctionList) {
		Assert.assertTrue((_functionList == null) || (newFunctionList == _functionList));
		Assert.assertEquals(newFunctionList.size(), _currentValues.length);

		_functionList = newFunctionList;
	}

	public IArgumentBindings saveCurrentArgumentBindings() {
		return new SavedArgumentBindings(_functionArguments, _currentValues);
	}

}