package de.uniwue.ki.mas.basics.functionArgument;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class FunctionWithArgumentBinds extends AbstractFunction {
	private String _name;
	private IFunction _function;
	private IMutableArgumentBindings _argumentBindings;
	private final boolean _shouldCallCalculateArguments;

	public FunctionWithArgumentBinds() {
		_shouldCallCalculateArguments = false;
	}

	public FunctionWithArgumentBinds(
		String name,
		IFunction function,
		IMutableArgumentBindings argumentBindings) {

		super();

		_shouldCallCalculateArguments = true;
		setName(name);
		setFunction(function);
		setMutableArgumentBindings(argumentBindings);
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		executeBefore(executeFunctionArgs);
		getFunction().act(executeFunctionArgs);
		executeAfter(executeFunctionArgs);
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		executeBefore(executeFunctionArgs);
		Object value = getFunction().execute(executeFunctionArgs);
		executeAfter(executeFunctionArgs);

		return value;
	}

	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		executeBefore(executeFunctionArgs);
		boolean value = getFunction().executeForBoolean(executeFunctionArgs);
		executeAfter(executeFunctionArgs);

		return value;
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {

		executeBefore(executeFunctionArgs);
		double value = getFunction().executeForDouble(executeFunctionArgs);
		executeAfter(executeFunctionArgs);

		return value;
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		executeBefore(executeFunctionArgs);
		int value = getFunction().executeForInt(executeFunctionArgs);
		executeAfter(executeFunctionArgs);

		return value;
	}

	private void executeAfter(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		executeFunctionArgs.popArgumentBindings();
	}

	private void executeBefore(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		if (_shouldCallCalculateArguments)
			getMutableArgumentBindings().calculateArguments(executeFunctionArgs);
		executeFunctionArgs.pushArgumentBindings(getMutableArgumentBindings());
	}

	public final IMutableArgumentBindings getMutableArgumentBindings() {
		return _argumentBindings;
	}

	public String getName() {
		return getNameWithoutRealAndBindings()
			+ " {real: "
			+ getFunction().getName()
			+ "; binds: ("
			+ getMutableArgumentBindings().getName()
			+ ")}";
	}

	public String getNameWithoutRealAndBindings() {
		return _name;
		//return getArgumentBindings().getName() + " " + getFunction().getName();
	}

	public void setName(String name) {
		_name = name;
	}

	public IFunction getFunction() {
		return _function;
	}

	public void setFunction(IFunction function) {
		_function = function;
	}

	public void setMutableArgumentBindings(IMutableArgumentBindings argumentBindings) {
		_argumentBindings = argumentBindings;
	}
}