package de.uniwue.ki.mas.basics.functionArgument;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class OneArgumentArgumentBindings extends AbstractMutableArgumentBindings {

	private final IFunctionArgument _functionArgument;
	private IFunction _function;
	private Object _currValue;

	public OneArgumentArgumentBindings(IFunctionArgumentDeclaration functionArgumentDeclaration) {

		super();

		_functionArgument = functionArgumentDeclaration.getFunctionArgument();
	}

	public boolean containsBindingFor(IFunctionArgument functionArgument) {
		if (_functionArgument == functionArgument) {
			return true;
		}

		return super.containsBindingFor(functionArgument);
	}

	public Object getArgumentValueFor(IFunctionArgument functionArgument) throws SimulationRuntimeException {

		if (_functionArgument == functionArgument) {
			return _currValue;
		}

		return super.getArgumentValueFor(functionArgument);
	}

	protected void appendNameIntern(StringBuffer buff) {
		appendName(buff, _functionArgument, _function);
	}

	public void calculateArguments(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		_currValue = _function.execute(executeFunctionArgs);
	}

	public void setFunctionList(IFunctionList newFunctionList) {
		Assert.assertEquals(1, newFunctionList.size());

		_function = newFunctionList.getFunction(0);
	}

	public IArgumentBindings saveCurrentArgumentBindings() {
		return new SavedOneArgumentBindings(_functionArgument, _currValue);
	}
}