package de.uniwue.ki.mas.basics.functionArgument;
import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (12.06.2001 10:56:18)
 * @author Christoph Oechslein
 */

class ParametricFunctionCallDeclaration
	extends AbstractDeclaration
	implements IParametricFunctionCallDeclaration {

	private transient final ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private final ParametricFunctionCallDeclarationHelper _parametricFunctionCallDeclarationHelper =
		new ParametricFunctionCallDeclarationHelper(this);

	private final FunctionArgumentDeclarationList _functionArgumentDeclarationList =
		new FunctionArgumentDeclarationList();

	private boolean _inlinePossible = true;

	public ParametricFunctionCallDeclaration() {
		super();

		_parametricFunctionCallDeclarationHelper.addChangeListener(_changeListener);
		_functionArgumentDeclarationList.addChangeListener(_changeListener);
	}

	public ParametricFunctionCallDeclaration(ISpecificType mostGeneralOutputArgumentDeclaration) {
		super();

		_parametricFunctionCallDeclarationHelper.setMostGeneralOutputType(
			mostGeneralOutputArgumentDeclaration);
	}

	public ParametricFunctionCallDeclaration(
		ISpecificType mostGeneralOutputArgumentDeclaration,
		IFunctionCallInternDeclaration functionCallInternDeclaration) {

		super();

		_parametricFunctionCallDeclarationHelper.setMostGeneralOutputType(
			mostGeneralOutputArgumentDeclaration);
		_parametricFunctionCallDeclarationHelper.setFunctionCallInternDeclaration(
			functionCallInternDeclaration);
	}

	//	public boolean equals(Object arg0) {
	//		if (arg0==null) return false;
	//		
	//		if (!(arg0 instanceof ParametricFunctionCallDeclaration))
	//			return false;
	//
	//		IParametricFunctionCallDeclaration arg = (IParametricFunctionCallDeclaration)arg0;
	//
	//		return (
	//			CollectionsHelper.isEqual(
	//				arg.getFunctionArgumentDeclarationList(),
	//				getFunctionArgumentDeclarationList(),
	//				true)
	//				&& getFunctionCallDeclaration().equals(arg.getFunctionCallDeclaration())
	//				&& (arg.isInlinePossible() == isInlinePossible()));
	//	}

	//	public int hashCode() {
	//		int hashCode=0;
	//		
	//		hashCode += getFunctionCallDeclaration().hashCode();
	//		Iterator it = getFunctionArgumentDeclarationList().iterator();
	//		while (it.hasNext()) {
	//			IFunctionArgumentDeclaration element = (IFunctionArgumentDeclaration)it.next();
	//			hashCode += element.hashCode();
	//		}
	//		
	//		return hashCode;
	//	}

	public void copyInto(Object inToObject, Map objectMapping) {
		if (objectMapping == null) {
			objectMapping = new HashMap();
		}

		IParametricFunctionCallDeclaration dstFunctionCallDeclarationWithArgDeclBinds =
			(IParametricFunctionCallDeclaration)inToObject;

		getFunctionArgumentDeclarationList().copyInto(
			dstFunctionCallDeclarationWithArgDeclBinds.getFunctionArgumentDeclarationList(),
			objectMapping);

		getFunctionCallDeclaration().copyInto(
			dstFunctionCallDeclarationWithArgDeclBinds.getFunctionCallDeclaration(),
			objectMapping);

		dstFunctionCallDeclarationWithArgDeclBinds.setInlinePossible(isInlinePossible());

		super.copyInto(dstFunctionCallDeclarationWithArgDeclBinds, objectMapping);
	}

	//	private void copyIntoSetLinksToFunctionArguments(
	//		IFunctionCallInternDeclaration srcFunctionCallInternDeclaration,
	//		IComposedFunctionCallInternDeclaration dstParent,
	//		IFunctionCallInternDeclaration dstFunctionCallInternDeclaration,
	//		IParametricFunctionCallDeclaration dstParametricFunctionCallDeclaration) {
	//
	//		if (srcFunctionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration) {
	//			Iterator srcIt =
	//				((IComposedFunctionCallInternDeclaration) srcFunctionCallInternDeclaration).getChildren();
	//			Iterator dstIt =
	//				((IComposedFunctionCallInternDeclaration) dstFunctionCallInternDeclaration).getChildren();
	//
	//			while (srcIt.hasNext()) {
	//				copyIntoSetLinksToFunctionArguments(
	//					(IFunctionCallInternDeclaration) srcIt.next(),
	//					(IComposedFunctionCallInternDeclaration) dstFunctionCallInternDeclaration,
	//					(IFunctionCallInternDeclaration) dstIt.next(),
	//					dstParametricFunctionCallDeclaration);
	//			}
	//		} else {
	//			IArgumentDeclaration srcArgumentDeclaration =
	//				((IAtomicFunctionCallInternDeclaration) srcFunctionCallInternDeclaration)
	//					.getArgumentDeclaration();
	//
	//			if (srcArgumentDeclaration instanceof IFunctionArgumentDeclaration) {
	//				int index = getFunctionArgumentDeclarationList().indexOf(srcArgumentDeclaration);
	//
	//				dstParametricFunctionCallDeclaration
	//					.getFunctionCallDeclaration()
	//					.replaceFunctionCallInternDeclarationTo(
	//					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
	//						dstParent,
	//						(IFunctionArgumentDeclaration) dstParametricFunctionCallDeclaration
	//							.getFunctionArgumentDeclarationList()
	//							.get(
	//							index)),
	//					dstFunctionCallInternDeclaration);
	//			}
	//		}
	//	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {
		return createFunctionArgs.getFunctionDeclarationCompiler().createFunction(
			createFunctionArgs,
			this,
			functions);
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return createFunctionArgs.getFunctionDeclarationCompiler().createFunction(
			createFunctionArgs,
			this,
			composedFunctionCallInternDeclaration);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new ParametricFunctionCallDeclaration();
	}

	public IParametricFunction createParametricFunction(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		return createFunctionArgs.getFunctionDeclarationCompiler().createParametricFunction(
			createFunctionArgs,
			this);
	}

	public IFunctionArgumentDeclarationList getFunctionArgumentDeclarationList() {
		return _functionArgumentDeclarationList;
	}

	public IFunctionCallDeclaration getFunctionCallDeclaration() {
		return _parametricFunctionCallDeclarationHelper;
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {
			IDeclarationList declarationList = getFunctionArgumentDeclarationList();
			AbstractDeclarationList.appendAllNames(buff, declarationList, currDepth, maxDepth, -1);
			buff.append(" ");

			_parametricFunctionCallDeclarationHelper.appendName(buff, currDepth, maxDepth);
		}
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getFunctionCallDeclaration().isConstant(createFunctionArgs);
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ParametricFunctionCallDeclaration)) {
			return false;
		}

		ParametricFunctionCallDeclaration parametricFunctionCallDeclaration =
			(ParametricFunctionCallDeclaration)anObject;

		return _parametricFunctionCallDeclarationHelper.isomorphicEquals(
			parametricFunctionCallDeclaration._parametricFunctionCallDeclarationHelper);
	}

	public int isomorphicHashCode() {
		return _parametricFunctionCallDeclarationHelper.isomorphicHashCode();
	}

	public void remove() {
		_parametricFunctionCallDeclarationHelper.removeChangeListener(_changeListener);
		_functionArgumentDeclarationList.removeChangeListener(_changeListener);
		_parametricFunctionCallDeclarationHelper.remove();
		_functionArgumentDeclarationList.remove();

		super.remove();
	}

	public void setFunctionCallInternDeclaration(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		getFunctionCallDeclaration().setFunctionCallInternDeclaration(functionCallInternDeclaration);
		fireStateChanged();
	}

	public void setInlinePossible(boolean inlinePossible) {
		_inlinePossible = inlinePossible;
		fireStateChanged();
	}

	public boolean isInlinePossible() {
		return _inlinePossible;
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	public Iterator getReferencedSpecificTypes() {
		return CollectionsHelper
			.concatenate(
				CollectionsHelper
				.map(getFunctionArgumentDeclarationList().iterator(), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				IArgumentDeclaration decl = (IArgumentDeclaration)obj;
				return decl.getSpecificType();
			}
		}), CollectionsHelper.createIterator(getFunctionCallDeclaration().getMostGeneralOutputType()));
	}
}