package de.uniwue.ki.mas.basics.functionArgument;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (12.06.2001 14:20:58)
 * @author Christoph Oechslein
 */

final class ParametricFunctionDeferred
	extends AbstractParametricFunction
	implements IParametricFunctionDeferred {

	protected final class NormalInnerFunction extends AbstractFunctionDynamic {
		protected final FunctionWithArgumentBinds _functionWithArgumentBinds;
		protected final IFunctionList _functions;
		protected int _counter = 0;

		protected NormalInnerFunction(
			FunctionWithArgumentBinds functionWithArgumentBinds,
			IFunctionList functions) {
			super();

			_functionWithArgumentBinds = functionWithArgumentBinds;
			_functions = functions;
		}

		public Object execute(final IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {

			executeBefore();

			Object result;

			if (_functionArgument) {
				result = _functions.getFunction(0).execute(executeFunctionArgs);

			} else {
				result = ParametricFunctionDeferred.this.executeIntern(executeFunctionArgs, _functions);
			}

			executeAfter();

			return result;
		}

		public void act(final IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

			executeBefore();

			if (_functionArgument) {
				_functions.getFunction(0).act(executeFunctionArgs);

			} else {
				ParametricFunctionDeferred.this.actIntern(executeFunctionArgs, _functions);
			}

			executeAfter();
		}

		public boolean executeForBoolean(final IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {

			executeBefore();

			boolean result;

			if (_functionArgument) {
				result = _functions.getFunction(0).executeForBoolean(executeFunctionArgs);

			} else {
				result =
					ParametricFunctionDeferred.this.executeForBooleanIntern(executeFunctionArgs, _functions);
			}

			executeAfter();

			return result;
		}

		public int executeForInt(final IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {

			executeBefore();

			int result;

			if (_functionArgument) {
				result = _functions.getFunction(0).executeForInt(executeFunctionArgs);

			} else {
				result = ParametricFunctionDeferred.this.executeForIntIntern(executeFunctionArgs, _functions);
			}

			executeAfter();

			return result;
		}

		public double executeForDouble(final IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {

			executeBefore();

			double result;

			if (_functionArgument) {
				result = _functions.getFunction(0).executeForDouble(executeFunctionArgs);

			} else {
				result =
					ParametricFunctionDeferred.this.executeForDoubleIntern(executeFunctionArgs, _functions);
			}

			executeAfter();

			return result;
		}

		protected void executeAfter() {
			_counter--;
		}

		protected void executeBefore() throws SimulationRuntimeException {
			Assert.assertTrue(_initialized);

			if (_counter > 10000) {
				throw new SimulationRuntimeException("Maybe endless recursion in : " + getName());
			}

			_counter++;
		}

		public String getName() {
			return _functionWithArgumentBinds.getNameWithoutRealAndBindings();
		}
	}

	protected boolean _initialized = false;
	protected boolean _functionArgument = false;

	protected final FunctionWithArgumentBinds _functionWithArgumentBinds = new FunctionWithArgumentBinds();
	protected final IFunctionArgumentDeclarationList _functionArgumentDeclarationList;

	public ParametricFunctionDeferred(IParametricFunctionCallDeclaration parametricFunctionCallDeclaration) {
		super();

		_functionWithArgumentBinds.setName(parametricFunctionCallDeclaration.getName());
		_functionArgumentDeclarationList =
			parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList();
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, final IFunctionList functions)
		throws ModelCompilingException {

		return new NormalInnerFunction(_functionWithArgumentBinds, functions);
	}

	public final String getName() {
		return _functionWithArgumentBinds.getName();
	}

	public void setRealFunction(IFunction function) {
		_initialized = true;

		if (function instanceof IFunctionArgument) {
			Assert.assertSame(
				function,
				((IFunctionArgumentDeclaration)_functionArgumentDeclarationList.get(0))
					.getFunctionArgument());
			Assert.assertEquals(1, _functionArgumentDeclarationList.size());

			_functionArgument = true;

		} else {
			_functionArgument = false;
			_functionWithArgumentBinds.setFunction(function);
		}
	}

	public final Object execute(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		return executeIntern(createAndExecuteFunctionArgs, functions);
	}

	public final void act(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		actIntern(createAndExecuteFunctionArgs, functions);
	}

	public final boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		return executeForBooleanIntern(createAndExecuteFunctionArgs, functions);
	}

	public final int executeForInt(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		return executeForIntIntern(createAndExecuteFunctionArgs, functions);
	}

	public final double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		return executeForDoubleIntern(createAndExecuteFunctionArgs, functions);
	}

	protected Object executeIntern(IExecuteFunctionArgs executeFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException {

		IMutableArgumentBindings oldArgumentBindings = executeInternBefore(executeFunctionArgs, functions);

		Object result = _functionWithArgumentBinds.execute(executeFunctionArgs);

		executeInternAfter(oldArgumentBindings);

		return result;
	}

	protected void actIntern(IExecuteFunctionArgs executeFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException {

		IMutableArgumentBindings oldArgumentBindings = executeInternBefore(executeFunctionArgs, functions);

		_functionWithArgumentBinds.act(executeFunctionArgs);

		executeInternAfter(oldArgumentBindings);
	}

	protected boolean executeForBooleanIntern(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {

		IMutableArgumentBindings oldArgumentBindings = executeInternBefore(executeFunctionArgs, functions);

		boolean result = _functionWithArgumentBinds.executeForBoolean(executeFunctionArgs);

		executeInternAfter(oldArgumentBindings);

		return result;
	}

	protected int executeForIntIntern(IExecuteFunctionArgs executeFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException {

		IMutableArgumentBindings oldArgumentBindings = executeInternBefore(executeFunctionArgs, functions);

		int result = _functionWithArgumentBinds.executeForInt(executeFunctionArgs);

		executeInternAfter(oldArgumentBindings);

		return result;
	}

	protected double executeForDoubleIntern(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {

		IMutableArgumentBindings oldArgumentBindings = executeInternBefore(executeFunctionArgs, functions);

		double result = _functionWithArgumentBinds.executeForDouble(executeFunctionArgs);

		executeInternAfter(oldArgumentBindings);

		return result;
	}

	private void executeInternAfter(IMutableArgumentBindings oldArgumentBindings) {
		_functionWithArgumentBinds.setMutableArgumentBindings(oldArgumentBindings);
	}

	private IMutableArgumentBindings executeInternBefore(
		IExecuteFunctionArgs executeFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException {

		//save old bindings 
		IMutableArgumentBindings oldArgumentBindings =
			_functionWithArgumentBinds.getMutableArgumentBindings();

		// create new Binding
		IMutableArgumentBindings argumentBindings =
			createNewArgumentBindings(executeFunctionArgs.createCreateAndExecuteFunctionArgs(), functions);

		// calc Arguments under the current binding
		argumentBindings.calculateArguments(executeFunctionArgs);

		// now add new bindings to function
		_functionWithArgumentBinds.setMutableArgumentBindings(argumentBindings);

		Assert.assertNotSame(oldArgumentBindings, _functionWithArgumentBinds.getMutableArgumentBindings());

		return oldArgumentBindings;
	}

	protected IMutableArgumentBindings createNewArgumentBindings(
		ICreateFunctionArgs createFunctionArgs,
		IFunctionList functions) {
		IMutableArgumentBindings argumentBindings =
			ParametricFunctionArgumentBindingsFactory.createMutableArgumentBindings(
				createFunctionArgs,
				_functionArgumentDeclarationList);

		argumentBindings.setFunctionList(functions);
		return argumentBindings;
	}
}