package de.uniwue.ki.mas.basics.functionArgument;

import java.util.Arrays;

import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * @author oechslein
 * 
 * Creation date: 20.05.2003
 */
final class SavedArgumentBindings extends AbstractNamed implements IArgumentBindings {
	private final IFunctionArgument[] _functionArguments;
	private final Object[] _values;

	public SavedArgumentBindings(IFunctionArgument[] functionArguments, Object[] values) {
		_functionArguments = functionArguments;
		_values = values;
	}

	public String getName() {
		return getNameIntern();
	}

	public Object getArgumentValueFor(IFunctionArgument functionArgument) throws SimulationRuntimeException {
		for (int i = 0; i < _functionArguments.length; i++) {
			if (_functionArguments[i] == functionArgument) {
				return _values[i];
			}
		}

		throw new SimulationRuntimeException("Argument '" + functionArgument.getName() + "' not found!");
	}

	public boolean containsBindingFor(IFunctionArgument functionArgument) {
		for (int i = 0; i < _functionArguments.length; i++) {
			if (_functionArguments[i] == functionArgument) {
				return true;
			}
		}

		return false;
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		for (int i = 0; i < _functionArguments.length; i++) {
			_functionArguments[i].appendName(buff, currDepth, maxDepth);
			buff.append(" = ");
			buff.append(_values[i]);

			if (i < _functionArguments.length - 1)
				buff.append(", ");
		}
	}

	public IArgumentBindings saveCurrentArgumentBindings() {
		return this;
	}

	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		} else if (!(obj instanceof SavedArgumentBindings)) {
			return false;
		} else {
			SavedArgumentBindings savedArgumentBindings = (SavedArgumentBindings)obj;
			return Arrays.equals(_functionArguments, savedArgumentBindings._functionArguments)
				&& Arrays.equals(_values, savedArgumentBindings._values);
		}
	}
	
	public int hashCode() {
		int hashCode = 42;
		
		for (int i = 0; i < _functionArguments.length; i++) {
			hashCode += 3 * _functionArguments[i].hashCode();
		}
		
		for (int i = 0; i < _values.length; i++) {
			hashCode += 3 * _values[i].hashCode();
		}
		
		return hashCode;
	}	
}
