package de.uniwue.ki.mas.basics.functionArgument;
import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (12.06.2001 10:40:07)
 * @author Christoph Oechslein
 */
public class SpecificFunctionDeclarationWithArgDeclBinds
	extends AbstractSpecificFunctionDeclaration
	implements ISpecificFunctionDeclarationWithArgDeclBinds {

	private final IParametricFunctionCallDeclaration _parametricFunctionCallDeclaration;
	private transient final ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};
	
//	public boolean equals(Object arg0) {
//		if (arg0==null) return false;
//		
//		if (arg0==this) return true;
//		
//		if (!(arg0 instanceof SpecificFunctionDeclarationWithArgDeclBinds)) return false;
//		
//		SpecificFunctionDeclarationWithArgDeclBinds arg = (SpecificFunctionDeclarationWithArgDeclBinds)arg0;
//		return (Utils.equals(getName(), arg.getName()) &&
//		getParametricFunctionCallDeclaration().equals(arg.getParametricFunctionCallDeclaration()));
//	}
//	
//	public int hashCode() {
//		return getParametricFunctionCallDeclaration().hashCode();
//	}
	
//	protected int getObjectHashCode() {
//		return super.hashCode();
//	}
	
	public SpecificFunctionDeclarationWithArgDeclBinds() {
		this(new ParametricFunctionCallDeclaration());
	}

	public SpecificFunctionDeclarationWithArgDeclBinds(ISpecificType outputType) {
		this(new ParametricFunctionCallDeclaration(outputType));
	}

	public SpecificFunctionDeclarationWithArgDeclBinds(IParametricFunctionCallDeclaration parametricFunctionCallDeclaration) {
		super();

		_parametricFunctionCallDeclaration = parametricFunctionCallDeclaration;

		_parametricFunctionCallDeclaration.addChangeListener(_changeListener);

	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		SpecificFunctionDeclarationWithArgDeclBinds specificFunctionDeclarationWithArgDeclBinds =
			(SpecificFunctionDeclarationWithArgDeclBinds) inToObject;

		getParametricFunctionCallDeclaration().copyInto(
			specificFunctionDeclarationWithArgDeclBinds.getParametricFunctionCallDeclaration(),
			objectMapping);

	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {

		return getSpecificInputTypeAt(index).createDefaultFunctionCallInternDeclaration();

		//		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
		//			null,
		//			getSpecificInputTypeAt(index));
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functionList)
		throws ModelCompilingException {
		return _parametricFunctionCallDeclaration.createFunction(createFunctionArgs, functionList);
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclaration.createFunction(
			createFunctionArgs,
			composedFunctionCallInternDeclaration);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new SpecificFunctionDeclarationWithArgDeclBinds();
	}

	public IParametricFunction createParametricFunction(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclaration.createParametricFunction(createFunctionArgs);
	}

	public IArgumentDeclaration getDefaultArgumentDeclarationAt(int index) {
		return getFunctionArgumentDeclarationAt(index);
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		getParametricFunctionCallDeclaration().appendName(buff, currDepth, maxDepth);
	}

	public String getName() {
		return getNameIntern();
	}

	protected IFunctionArgumentDeclaration getFunctionArgumentDeclarationAt(int index) {
		return (IFunctionArgumentDeclaration) getFunctionArgumentDeclarationList().get(index);
	}

	public IFunctionArgumentDeclarationList getFunctionArgumentDeclarationList() {
		return _parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList();
	}

	public IFunctionCallDeclaration getFunctionCallDeclaration() {
		return getParametricFunctionCallDeclaration().getFunctionCallDeclaration();
	}

	public IGenericFunctionDeclaration getGenericFunctionDeclaration() {
		return null;
	}

	public int getSpecificInputDefaultArgsLength() {
		return getFunctionArgumentDeclarationList().size();
	}

	public int getSpecificInputMaxArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	public int getSpecificInputMinArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	public IType getOutputType() {
		return getSpecificOutputType();
	}

	public final IParametricFunctionCallDeclaration getParametricFunctionCallDeclaration() {
		return _parametricFunctionCallDeclaration;
	}

	protected String getSignatureWithoutNameAndOutputType() {
		StringBuffer buff = new StringBuffer();

		buff.append("(");

		Iterator it = getFunctionArgumentDeclarationList().iterator();
		while (it.hasNext()) {
			IFunctionArgumentDeclaration functionArgumentDeclaration =
				(IFunctionArgumentDeclaration) it.next();

			buff.append(functionArgumentDeclaration.getSpecificType().getName());

			if (it.hasNext()) {
				buff.append(", ");
			}
		}

		buff.append(")");

		return buff.toString();
	}

	public ISpecificFunctionType getSpecificFunctionType() {
		List types = new ArrayList(getFunctionArgumentDeclarationList().size());

		Iterator it = getFunctionArgumentDeclarationList().iterator();
		while (it.hasNext()) {
			IFunctionArgumentDeclaration currFunctionArgumentDeclaration =
				(IFunctionArgumentDeclaration) it.next();

			types.add(currFunctionArgumentDeclaration.getSpecificType());
		}

		return GenericFunctionType.getInstance().createSpecificFunctionType(getSpecificOutputType(), types);
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		return getFunctionArgumentDeclarationAt(index).getSpecificType();
	}

	public String getSpecificInputTypeNameAt(int index) {
		return getFunctionArgumentDeclarationAt(index).getName();
	}

	public ISpecificType getSpecificOutputType() {
		return getFunctionCallDeclaration().getMostGeneralOutputType();
	}

	public final boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return createFunctionArgs.getFunctionDeclarationCompiler().isConstant(
			createFunctionArgs,
			getParametricFunctionCallDeclaration(),
			composedFunctionCallInternDeclaration);
	}

	public boolean isReadOnly(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return createFunctionArgs.getFunctionDeclarationCompiler().isReadOnly(
			createFunctionArgs,
			getParametricFunctionCallDeclaration(),
			composedFunctionCallInternDeclaration);
	}

	public boolean isExpertFunctionDeclaration() {
		return false;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (this == anObject) {
			return true;
		}

		if (!(anObject instanceof ISpecificFunctionDeclarationWithArgDeclBinds)) {
			return false;
		}

		ISpecificFunctionDeclarationWithArgDeclBinds specificFunctionDeclarationWithArgDeclBinds =
			(ISpecificFunctionDeclarationWithArgDeclBinds) anObject;

		return getParametricFunctionCallDeclaration().isomorphicEquals(
			specificFunctionDeclarationWithArgDeclBinds.getParametricFunctionCallDeclaration());
	}

	public int isomorphicHashCode() {
		return getParametricFunctionCallDeclaration().isomorphicHashCode();
	}

	public void remove() {
		_parametricFunctionCallDeclaration.removeChangeListener(_changeListener);
		_parametricFunctionCallDeclaration.remove();

		super.remove();
	}

	public void setSpecificOutputType(ISpecificType specificType) {
		getFunctionCallDeclaration().setMostGeneralOutputType(specificType);
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}
}