package de.uniwue.ki.mas.basics.functionCall;
import javax.swing.event.*;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;

/**
 * Creation date: (26.07.2001 12:35:09)
 * @author Christoph Oechslein
 */

public abstract class AbstractFunctionCallDeclaration
	extends AbstractDeclaration
	implements IFunctionCallDeclaration, TreeModel {
	protected final EventListenerList listenerList = new EventListenerList();

	public AbstractFunctionCallDeclaration() {
		super();
	}

	public void addTreeModelListener(TreeModelListener l) {
		listenerList.add(TreeModelListener.class, l);
	}

	/**
	 * Invoked this to insert newChild at location index in parents children.
	 * This will then message nodesWereInserted to create the appropriate
	 * event. This is the preferred way to add children as it will create
	 * the appropriate event.
	 */
	protected void changeNodeTo(
		IFunctionCallInternDeclaration newChild,
		IComposedFunctionCallInternDeclaration parent,
		int index) {
		parent.setChild(index, newChild);
		int[] changeIndexs = new int[1];
		changeIndexs[0] = index;
		nodesChanged(parent, changeIndexs);
	}

	/**
	* Notify all listeners that have registered interest for
	* notification on this event type.  The event instance
	* is lazily created using the parameters passed into
	* the fire method.
	*
	@see EventListenerList
	*/

	protected void fireTreeNodesChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		TreeModelEvent e = null;
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == TreeModelListener.class) {
				// Lazily create the event:
				if (e == null) {
					e = new TreeModelEvent(source, path, childIndices, children);
				}
				((TreeModelListener) listeners[i + 1]).treeNodesChanged(e);
			}
		}
	}

	/**
	* Notify all listeners that have registered interest for
	* notification on this event type.  The event instance
	* is lazily created using the parameters passed into
	* the fire method.
	*
	@see EventListenerList
	*/

	protected void fireTreeNodesInserted(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		TreeModelEvent e = null;
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == TreeModelListener.class) {
				// Lazily create the event:
				if (e == null) {
					e = new TreeModelEvent(source, path, childIndices, children);
				}
				((TreeModelListener) listeners[i + 1]).treeNodesInserted(e);
			}
		}
	}

	/**
	* Notify all listeners that have registered interest for
	* notification on this event type.  The event instance
	* is lazily created using the parameters passed into
	* the fire method.
	*
	@see EventListenerList
	*/

	protected void fireTreeNodesRemoved(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		TreeModelEvent e = null;
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == TreeModelListener.class) {
				// Lazily create the event:
				if (e == null) {
					e = new TreeModelEvent(source, path, childIndices, children);
				}
				((TreeModelListener) listeners[i + 1]).treeNodesRemoved(e);
			}
		}
	}

	/**
	* Notify all listeners that have registered interest for
	* notification on this event type.  The event instance
	* is lazily created using the parameters passed into
	* the fire method.
	*
	@see EventListenerList
	*/

	protected void fireTreeStructureChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		TreeModelEvent e = null;
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == TreeModelListener.class) {
				// Lazily create the event:
				if (e == null) {
					e = new TreeModelEvent(source, path, childIndices, children);
				}
				((TreeModelListener) listeners[i + 1]).treeStructureChanged(e);
			}
		}
	}

	public Object getChild(Object parent, int index) {
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration =
			(IComposedFunctionCallInternDeclaration) parent;
		return composedFunctionCallInternDeclaration.getChild(index);
	}

	public int getChildCount(Object parent) {
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration =
			(IComposedFunctionCallInternDeclaration) parent;
		return composedFunctionCallInternDeclaration.getChildCount();
	}

	public abstract IFunctionCallInternDeclaration getFunctionCallInternDeclaration();

	public int getIndexOfChild(Object parent, Object child) {
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration =
			(IComposedFunctionCallInternDeclaration) parent;
		return composedFunctionCallInternDeclaration.getIndexOfChild((IFunctionCallInternDeclaration) child);
	}

	/**
	 * Builds the parents of node up to and including the root node,
	 * where the original node is the last element in the returned array.
	 * The length of the returned array gives the node's depth in the
	 * tree.
	 *
	 * @param aNode the TreeNode to get the path for
	 * @return IFunctionCallInternDeclaration[]
	 * @see de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration#getPathToRoot(IFunctionCallInternDeclaration)
	 */
	public IFunctionCallInternDeclaration[] getPathToRoot(IFunctionCallInternDeclaration aNode) {
		return getPathToRoot(aNode, 0);
	}

	/**
	 * Builds the parents of node up to and including the root node,
	 * where the original node is the last element in the returned array.
	 * The length of the returned array gives the node's depth in the
	 * tree.
	 *
	 * @param aNode  the TreeNode to get the path for
	 * @param depth  an int giving the number of steps already taken towards
	 *        the root (on recursive calls), used to size the returned array
	 * @return an array of TreeNodes giving the path from the root to the
	 *         specified node
	 */
	protected IFunctionCallInternDeclaration[] getPathToRoot(
		IFunctionCallInternDeclaration aNode,
		int depth) {
		IFunctionCallInternDeclaration[] retNodes;
		// This method recurses, traversing towards the root in order
		// size the array. On the way back, it fills in the nodes,
		// starting from the root and working back to the original node.

		/* Check for null, in case someone passed in a null node, or
		they passed in an element that isn't rooted at root. */

		if (aNode == null) {
			if (depth == 0) {
				return null;
			} else {
				retNodes = new IFunctionCallInternDeclaration[depth];
			}
		} else {
			final int newDepth = depth + 1;
			if (aNode == getRoot()) {
				retNodes = new IFunctionCallInternDeclaration[newDepth];
			} else {
				retNodes = getPathToRoot(aNode.getParent(), newDepth);
			}
			retNodes[retNodes.length - newDepth] = aNode;
		}
		return retNodes;
	}

	public Object getRoot() {
		return getFunctionCallInternDeclaration();
	}

	public TreeModel getTreeModel() {
		return this;
	}

	/**
	 * Invoked this to insert newChild at location index in parents children.
	 * This will then message nodesWereInserted to create the appropriate
	 * event. This is the preferred way to add children as it will create
	 * the appropriate event.
	 */
	protected void insertNodeInto(
		IFunctionCallInternDeclaration newChild,
		IComposedFunctionCallInternDeclaration parent,
		int index) {
		parent.setChild(index, newChild);
		int[] newIndexs = new int[1];
		newIndexs[0] = index;
		nodesWereInserted(parent, newIndexs);
	}

	public boolean isLeaf(Object node) {
		return (node instanceof IAtomicFunctionCallInternDeclaration);
	}

	/**
	 * Invoke this method after you've changed how the children identified by
	 * childIndicies are to be represented in the tree.
	 */
	protected void nodesChanged(IFunctionCallInternDeclaration node, int[] childIndices) {
		if (node != null && childIndices != null) {
			final int cCount = childIndices.length;
			if (cCount > 0) {
				Object[] cChildren = new Object[cCount];
				for (int counter = 0; counter < cCount; counter++) {
					cChildren[counter] = getChild(node, childIndices[counter]);
				}
				fireTreeNodesChanged(this, getPathToRoot(node), childIndices, cChildren);
			}
		}
	}

	/**
	 * Invoke this method if you've totally changed the children of
	 * node and its childrens children...  This will post a
	 * treeStructureChanged event.
	 */
	protected void nodeStructureChanged(IFunctionCallInternDeclaration node) {
		if (node != null) {
			fireTreeStructureChanged(this, getPathToRoot(node), null, null);
		}
	}

	/**
	 * Invoke this method after you've inserted some TreeNodes into
	 * node.  childIndices should be the index of the new elements and
	 * must be sorted in ascending order.
	 */
	protected void nodesWereInserted(IComposedFunctionCallInternDeclaration node, int[] childIndices) {
		if (listenerList != null && node != null && childIndices != null && childIndices.length > 0) {
			final int cCount = childIndices.length;
			Object[] newChildren = new Object[cCount];
			for (int counter = 0; counter < cCount; counter++) {
				newChildren[counter] = getChild(node, childIndices[counter]);
			}
			fireTreeNodesInserted(this, getPathToRoot(node), childIndices, newChildren);
		}
	}

	/**
	 * Invoke this method after you've removed some TreeNodes from
	 * node.  childIndices should be the index of the removed elements and
	 * must be sorted in ascending order. And removedChildren should be
	 * the array of the children objects that were removed.
	 */
	protected void nodesWereRemoved(
		IComposedFunctionCallInternDeclaration node,
		int[] childIndices,
		Object[] removedChildren) {
		if (node != null && childIndices != null) {
			fireTreeNodesRemoved(this, getPathToRoot(node), childIndices, removedChildren);
		}
	}

	/**
	 * Invoke this method if you've modified the TreeNodes upon which this
	 * model depends.  The model will notify all of its listeners that the
	 * model has changed.
	 */
	protected void reload() {
		nodeStructureChanged(getFunctionCallInternDeclaration());
	}

	/**
	 * Remove Object (this).
	 * Gets called if Object will be deleted from modell
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		super.remove();
	}

	public void removeTreeModelListener(TreeModelListener l) {
		listenerList.remove(TreeModelListener.class, l);
	}

	protected void replaceFunctionCallInternDeclarationTo(
		IComposedFunctionCallInternDeclaration parent,
		IFunctionCallInternDeclaration newCall,
		IFunctionCallInternDeclaration oldCall) {

		if (Assert.ASSERTION_ON) {
			Assert.assertEquals("parent of newCall differs", parent, newCall.getParent());
			Assert.assertEquals("parent of oldCall differs", parent, oldCall.getParent());
		}

		if (newCall == oldCall) {
			nodeStructureChanged(newCall);

		} else if (parent == null) {
			// set root object
			setFunctionCallInternDeclaration(newCall);

		} else {
			if (Assert.ASSERTION_ON) {
				Assert.assertTrue("old call wrong parent!!", parent.getIndexOfChild(oldCall) != -1);
			}

			structureChangeNodeTo(newCall, parent, parent.getIndexOfChild(oldCall));
		}
	}

	public void replaceFunctionCallInternDeclarationTo(
		IFunctionCallInternDeclaration newCall,
		IFunctionCallInternDeclaration oldCall) {

		replaceFunctionCallInternDeclarationTo(oldCall.getParent(), newCall, oldCall);
	}

	/**
	 * Invoked this to insert newChild at location index in parents children.
	 * This will then message nodesWereInserted to create the appropriate
	 * event. This is the preferred way to add children as it will create
	 * the appropriate event.
	 */
	protected void structureChangeNodeTo(
		IFunctionCallInternDeclaration newChild,
		IComposedFunctionCallInternDeclaration parent,
		int index) {

		Assert.assertTrue("index out of bounds: " + index, (0 <= index) && (index < parent.getChildCount()));

		Object oldChild = parent.getChild(index);
		int[] changeIndexs = new int[1];
		Object[] removedObjects = new Object[1];
		changeIndexs[0] = index;
		removedObjects[0] = oldChild;
		//
		parent.setChild(index, newChild);
		nodesWereRemoved(parent, changeIndexs, removedObjects);
		nodesWereInserted(parent, changeIndexs);
		// nodesStructureChanged(parent);
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
		IFunctionCallInternDeclaration oldValue =
			(IFunctionCallInternDeclaration) path.getLastPathComponent();
		IComposedFunctionCallInternDeclaration parent = oldValue.getParent();

		replaceFunctionCallInternDeclarationTo(parent, (IFunctionCallInternDeclaration) newValue, oldValue);
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getFunctionCallInternDeclaration().isConstant(createFunctionArgs);
	}

	public boolean isReadOnly(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getFunctionCallInternDeclaration().isReadOnly(createFunctionArgs);
	}
}