package de.uniwue.ki.mas.basics.functionCall;
import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.util.IRemoveListener;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.event.StateChangeMulticaster;

/**
 * Creation date: (30.07.2001 14:14:30)
 * @author Christoph Oechslein
 */

public abstract class AbstractFunctionCallInternDeclaration
	extends AbstractCopyable
	implements IFunctionCallInternDeclaration {

	private IComposedFunctionCallInternDeclaration _parent;
	private IFunctionCallDeclaration _creator;
	private transient List _removeListener;

	public void addRemoveListener(IRemoveListener removeListener) {
		if (_removeListener == null) {
			_removeListener = new LinkedList();
		}
		_removeListener.add(removeListener);
	}

	public void removeRemoveListener(IRemoveListener removeListener) {
		if (_removeListener != null) {
			_removeListener.remove(removeListener);
		}
	}

	public AbstractFunctionCallInternDeclaration(IComposedFunctionCallInternDeclaration parent) {
		super();
		setParent(parent);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IFunctionCallInternDeclaration dstFunctionCallInternDeclaration =
			(IFunctionCallInternDeclaration)intoObject;

		dstFunctionCallInternDeclaration.setParent(getParent());
	}

	public IComposedFunctionCallInternDeclaration getParent() {
		return _parent;
	}

	public void setParent(IComposedFunctionCallInternDeclaration newParent) {
		//		if (_parent != null && _parent != newParent) {
		//			System.out.println("Changed parent\nfrom: " + _parent + "\nto: " + newParent+ "\n");
		//		}
		//		
		_parent = newParent;
	}

	public void remove() {
		if (_removeListener != null) {
			Iterator it = _removeListener.iterator();

			while (it.hasNext()) {
				((IRemoveListener)it.next()).removed(this);
				it.remove();
			}
			_removeListener = null;
		}
	}

	public ISpecificType getMostGeneralSpecificOutputType() {
		if (getParent() != null) {
			int index = getParent().getIndexOfChild(this);
			if (index != -1) {
				return getParent().getSpecificFunctionDeclaration().getSpecificInputTypeAt(index);
			}
		}

		return getSpecificOutputType();
	}

	private transient ChangeListener _changeListener = null;

	public void addChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.add(_changeListener, newListener);
	}
	
	public void removeChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.remove(_changeListener,newListener);
	}

	protected void fireStateChanged() {
		if (_changeListener == null) {
			return;
		} else {
			try {
			_changeListener.stateChanged(new ChangeEvent(this));
			} catch (ConcurrentModificationException e) {
				e.printStackTrace();
			}
		}
	}

	public IFunctionCallDeclaration getCreator() {
		return _creator;
	}

	public void setCreator(IFunctionCallDeclaration creator) {
		this._creator = creator;
	}

}