package de.uniwue.ki.mas.basics.functionCall;
import java.util.Map;

import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (25.07.2001 16:09:50)
 * @author Christoph Oechslein
 */

final class AtomicFunctionCallInternDeclaration
	extends AbstractFunctionCallInternDeclaration
	implements IAtomicFunctionCallInternDeclaration {

	private final IArgumentDeclaration _argumentDeclaration;

	public AtomicFunctionCallInternDeclaration(IArgumentDeclaration argumentDeclaration) {
		this(null, argumentDeclaration);
	}

	public AtomicFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		IArgumentDeclaration argumentDeclaration) {

		super(parent);

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(argumentDeclaration);

		_argumentDeclaration = argumentDeclaration;
	}
	
	public ICopyable copy(Map objectMapping) {
		return ((IArgumentDeclaration) getArgumentDeclaration().copy(objectMapping))
			.createAtomicFunctionCallInternDeclaration();
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		throw new UnsupportedOperationException();
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		try {
			return getArgumentDeclaration().createArgument(createFunctionArgs);

		} catch (ModelCompilingException exception) {
			throw new FunctionCallInternDeclCompileException(this, exception);
		}
	}

	public IArgumentDeclaration getArgumentDeclaration() {
		return _argumentDeclaration;
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int depth, int maxDepth) {
		if (depth < maxDepth) {
			getArgumentDeclaration().appendName(buff, depth, maxDepth);
		}
	}

	public ISpecificType getSpecificOutputType() {
		return getArgumentDeclaration().getSpecificType();
	}

	public final boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getArgumentDeclaration().isConstant(createFunctionArgs);
	}

	public final boolean isReadOnly(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return true;
	}

//	public boolean equals(Object obj) {
//		if (obj==null) return false;
//		
//		if (obj==this) return true;
//		
//		if (obj instanceof IAtomicFunctionCallInternDeclaration)
//		return ((IAtomicFunctionCallInternDeclaration)obj).getArgumentDeclaration().equals(getArgumentDeclaration());
//		else return false;
//	}
//	
//	public int hashCode() {
//		return 33+ getArgumentDeclaration().hashCode();
//	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (this == obj)
			return true;

		if (!(obj instanceof IAtomicFunctionCallInternDeclaration)) {
			return false;
		}

		IAtomicFunctionCallInternDeclaration atomicFunctionCallInternDeclaration =
			(IAtomicFunctionCallInternDeclaration) obj;

		return getArgumentDeclaration().isomorphicEquals(
			atomicFunctionCallInternDeclaration.getArgumentDeclaration());
	}

	public int isomorphicHashCode() {

		if (getArgumentDeclaration() == null)
			return hashCode();

		else
			return getArgumentDeclaration().isomorphicHashCode();
	}

	public void remove() {
		if (getParent() != null) {
			setParent(null);
		}
		_argumentDeclaration.remove();

		super.remove();
	}


	public void checkInvariants() {
	}

	
}