package de.uniwue.ki.mas.basics.functionCall;
import java.util.Map;

import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (30.10.2001 19:53:30)
 * @author Christoph Oechslein
 */

public class FunctionCallDeclaration
	extends AbstractFunctionCallDeclaration
	implements IFunctionCallDeclaration, ChangeListener {

	private ISpecificType _mostGeneralOutputType;
	private IFunctionCallInternDeclaration _functionCallInternDeclaration = null;

	public FunctionCallDeclaration() {
		this(TypeManager.getInstance().getDefaultType());
	}

	private boolean functionCallInternEquals(FunctionCallDeclaration arg) {
		return (
			(((arg.getFunctionCallInternDeclaration() == null))
				&& (getFunctionCallInternDeclaration() == null))
				|| (((arg.getFunctionCallInternDeclaration() != null))
					&& arg.getFunctionCallInternDeclaration().equals(getFunctionCallInternDeclaration())));
	}

	private boolean mostGeneralOutputTypeEquals(FunctionCallDeclaration arg) {
		return (
			(((arg.getMostGeneralOutputType() == null)) && (getMostGeneralOutputType() == null))
				|| (((arg.getMostGeneralOutputType() != null))
					&& arg.getMostGeneralOutputType().equals(getMostGeneralOutputType())));
	}

	public FunctionCallDeclaration(ISpecificType mostGeneralOutputType) {
		this(mostGeneralOutputType, mostGeneralOutputType.createDefaultFunctionCallInternDeclaration());
	}

	public FunctionCallDeclaration(
		ISpecificType mostGeneralOutputType,
		IFunctionCallInternDeclaration functionCallInternDeclaration) {
		super();

		if (Assert.ASSERTION_ON)
			Assert.assertTrue(
				TypeManager.getInstance().isMoreGeneral(
					mostGeneralOutputType,
					functionCallInternDeclaration.getSpecificOutputType()));

		setMostGeneralOutputTypeAndFunctionCallInternDeclaration(
			mostGeneralOutputType,
			functionCallInternDeclaration);
	}

	public FunctionCallDeclaration(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		this(functionCallInternDeclaration.getSpecificOutputType(), functionCallInternDeclaration);
	}

	public void checkInvariants() {
		Assert.assertTrue(
			TypeManager.getInstance().isMoreGeneral(getMostGeneralOutputType(), getSpecificOutputType()));
		Assert.assertTrue(
			TypeManager.getInstance().isMoreGeneral(
				getMostGeneralOutputType(),
				getFunctionCallInternDeclaration().getSpecificOutputType()));
		Assert.assertNull(getFunctionCallInternDeclaration().getParent());

		getFunctionCallInternDeclaration().checkInvariants();
	}

	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		FunctionCallDeclaration dstFunctionCallDeclaration = (FunctionCallDeclaration)intoObject;

		Assert.assertNotNull("dstFunctionCallDeclaration may not be null", dstFunctionCallDeclaration);
		Assert.assertNotNull("functionCallIntern Decl may not be null", getFunctionCallInternDeclaration());

		dstFunctionCallDeclaration.setMostGeneralOutputTypeAndFunctionCallInternDeclaration(
			getMostGeneralOutputType(),
			(IFunctionCallInternDeclaration)getFunctionCallInternDeclaration().copy(objectMapping));

	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return createFunctionArgs.getFunctionDeclarationCompiler().createFunction(this, createFunctionArgs);
	}

	protected void fireTreeNodesChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {

		super.fireTreeNodesChanged(source, path, childIndices, children);

		firePropertyChange("name", null, getName());
		fireStateChanged();
	}

	protected void fireTreeNodesInserted(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {

		super.fireTreeNodesInserted(source, path, childIndices, children);

		firePropertyChange("name", null, getName());
		fireStateChanged();
	}

	protected void fireTreeNodesRemoved(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {

		super.fireTreeNodesRemoved(source, path, childIndices, children);

		firePropertyChange("name", null, getName());
		fireStateChanged();
	}

	protected void fireTreeStructureChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {

		super.fireTreeStructureChanged(source, path, childIndices, children);

		//firePropertyChange("name", null, getName());
		fireStateChanged();
	}

	public IFunctionCallInternDeclaration getFunctionCallInternDeclaration() {
		return _functionCallInternDeclaration;
	}

	public ISpecificType getMostGeneralOutputType() {
		return _mostGeneralOutputType;
	}

	public String getName() {
		return super.getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (getFunctionCallInternDeclaration() == null)
			buff.append("null");
		else
			getFunctionCallInternDeclaration().appendName(buff, currDepth, maxDepth);
	}

	public ISpecificType getSpecificOutputType() {
		Assert.assertNotNull("FunctionCallInternDeclaration was null",getFunctionCallInternDeclaration());

		return getFunctionCallInternDeclaration().getSpecificOutputType();
	}

//	public boolean equals(Object arg0) {
//		if (arg0 == null)
//			return false;
//		if (arg0 == this)
//			return true;
//
//		if (arg0 instanceof FunctionCallDeclaration) {
//			FunctionCallDeclaration arg = (FunctionCallDeclaration)arg0;
//			return (mostGeneralOutputTypeEquals(arg) && functionCallInternEquals(arg));
//		} else
//			return false;
//	}
//
//	public int hashCode() {
//		int hashCode = 23;
//
//		if (_mostGeneralOutputType != null)
//			hashCode += 3 * _mostGeneralOutputType.hashCode();
//		if (_functionCallInternDeclaration != null)
//			hashCode += 7 * _functionCallInternDeclaration.hashCode();
//		return super.hashCode();
//	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof IFunctionCallDeclaration)) {
			return false;
		}

		IFunctionCallDeclaration functionCallDeclaration = (IFunctionCallDeclaration)obj;

		if (!Utils.equals(getMostGeneralOutputType(), functionCallDeclaration.getMostGeneralOutputType())) {
			return false;
		}

		if (!Utils.equals(getSpecificOutputType(), functionCallDeclaration.getSpecificOutputType())) {
			return false;
		}

		if (!isomorphicEquals(getFunctionCallInternDeclaration(),
			functionCallDeclaration.getFunctionCallInternDeclaration())) {
			return false;
		}

		return true;
	}

	public int isomorphicHashCode() {
		return 3 * getMostGeneralOutputType().hashCode() + getFunctionCallInternDeclaration().isomorphicHashCode();
	}

	public void remove() {
		// remove 1:1 link (no listener!)
		if (getFunctionCallInternDeclaration() != null) {
			getFunctionCallInternDeclaration().remove();
			setFunctionCallInternDeclaration(null);
		}

		super.remove();
	}

	public void setFunctionCallInternDeclaration(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		if (Assert.ASSERTION_ON && (functionCallInternDeclaration != null)) {
			Assert.assertNotNull(functionCallInternDeclaration.getSpecificOutputType());
			Assert.assertTrue(
				"Wrong type : "
					+ functionCallInternDeclaration.getSpecificOutputType().getName()
					+ " should be "
					+ getMostGeneralOutputType().getName(),
				TypeManager.getInstance().isMoreGeneral(
					getMostGeneralOutputType(),
					functionCallInternDeclaration.getSpecificOutputType()));
		}

		IFunctionCallInternDeclaration oldValue = _functionCallInternDeclaration;

		if (!Utils.equals(oldValue, functionCallInternDeclaration)) {
			_functionCallInternDeclaration = functionCallInternDeclaration;
			if (functionCallInternDeclaration!=null) functionCallInternDeclaration.setCreator(this);

			if (functionCallInternDeclaration != null) {
				reload();
			}

			firePropertyChange("functionCallInternDeclaration", oldValue, functionCallInternDeclaration);
			// das auch nicht
			//nodeStructureChanged(this.getFunctionCallInternDeclaration());
			
			fireStateChanged();

			if (oldValue != null)
				oldValue.remove();
		}

		if (Assert.ASSERTION_ON && (functionCallInternDeclaration != null)) {
			checkInvariants();
		}
	}

	public void setMostGeneralOutputType(ISpecificType newOutputType) {
		ISpecificType oldOutputType = _mostGeneralOutputType;

		if (!Utils.equals(oldOutputType, newOutputType)) {
			_mostGeneralOutputType = newOutputType;

			if ((newOutputType != null)
				&& ((getFunctionCallInternDeclaration() == null)
					|| !TypeManager.getInstance().isMoreGeneral(
						newOutputType,
						getFunctionCallInternDeclaration().getSpecificOutputType())
					|| (getFunctionCallInternDeclaration() instanceof IAtomicFunctionCallInternDeclaration))) {

				setFunctionCallInternDeclaration(newOutputType.createDefaultFunctionCallInternDeclaration());

				firePropertyChange("mostGeneralOutputType", oldOutputType, newOutputType);
			} else {
				fireStateChanged();
			}
		}
	}

	public void setMostGeneralOutputTypeAndFunctionCallInternDeclaration(
		ISpecificType newOutputType,
		IFunctionCallInternDeclaration functionCallInternDeclaration) {

		Assert.assertTrue(
			(functionCallInternDeclaration == null)
				|| TypeManager.getInstance().isMoreGeneral(
					newOutputType,
					functionCallInternDeclaration.getSpecificOutputType()));

		ISpecificType oldOutputType = _mostGeneralOutputType;
		if (!Utils.equals(oldOutputType, newOutputType)) {

			_mostGeneralOutputType = newOutputType;
			firePropertyChange("mostGeneralOutputType", oldOutputType, newOutputType);

		}

		IFunctionCallInternDeclaration oldValue = _functionCallInternDeclaration;

		if (!Utils.equals(oldValue, functionCallInternDeclaration)) {
			if (functionCallInternDeclaration == null) {
				_functionCallInternDeclaration = null;

			} else {
				_functionCallInternDeclaration = functionCallInternDeclaration;
				if (functionCallInternDeclaration!=null) functionCallInternDeclaration.setCreator(this);

				reload();
			}

			firePropertyChange("functionCallInternDeclaration", oldValue, functionCallInternDeclaration);
			// das hat nicht geholfen: 
			// nodeStructureChanged(getFunctionCallInternDeclaration());
		}

		if (oldValue != null)
			oldValue.remove();

		fireStateChanged();

		if (Assert.ASSERTION_ON && (functionCallInternDeclaration != null)) {
			checkInvariants();
		}
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new FunctionCallDeclaration();
	}
}