package de.uniwue.ki.mas.basics.functionCall;

import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (04.09.2001 16:05:27)
 * @author Christoph Oechslein
 */

public final class FunctionCallDeclarationFactory
	extends AbstractDeclarationFactory
	implements IFunctionCallDeclarationFactory {

	private static final FunctionCallDeclarationFactory _instance = new FunctionCallDeclarationFactory();
	//private final ISpecificType _type;

	public FunctionCallDeclarationFactory(ISpecificType type) {
		super();

		//_type = type;
	}

	public static final FunctionCallDeclarationFactory getInstance() {
		return _instance;
	}

	public final IAtomicFunctionCallInternDeclaration createAtomicFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		IArgumentDeclaration argumentDeclaration) {

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(argumentDeclaration);

		IAtomicFunctionCallInternDeclaration newAtomicFunctionCallInternDeclaration =
			argumentDeclaration.createAtomicFunctionCallInternDeclaration();

		newAtomicFunctionCallInternDeclaration.setParent(parent);

		return newAtomicFunctionCallInternDeclaration;
	}

	public final IAtomicFunctionCallInternDeclaration createAtomicFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificType specificType) {

		if (Assert.ASSERTION_ON)
			Assert.assertTrue("VoidType not allowed", !VoidType.getInstance().equals(specificType));

		return createAtomicFunctionCallInternDeclaration(
			parent,
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(specificType));
	}

	public final IAtomicFunctionCallInternDeclaration createAtomicFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificType specificType,
		Object value) {

		if (Assert.ASSERTION_ON)
			Assert.assertTrue("VoidType not allowed", !VoidType.getInstance().equals(specificType));

		return createAtomicFunctionCallInternDeclaration(
			parent,
			ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(specificType, value));
	}

	public final IComposedFunctionCallInternDeclaration createComposedFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificFunctionDeclaration functionDeclaration) {
		return new ComposedFunctionCallInternDeclaration(parent, functionDeclaration);
	}

	public final IComposedFunctionCallInternDeclaration createComposedFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificFunctionDeclaration functionDeclaration,
		IComposedFunctionCallInternDeclaration oldComposedFunctionCallInternDeclaration) {

		return new ComposedFunctionCallInternDeclaration(
			parent,
			functionDeclaration,
			oldComposedFunctionCallInternDeclaration);
	}

	public final IComposedFunctionCallInternDeclaration createComposedFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificFunctionDeclaration functionDeclaration,
		Iterator inputArgumenDeclarations) {

		return new ComposedFunctionCallInternDeclaration(
			parent,
			functionDeclaration,
			inputArgumenDeclarations);
	}

	public final IComposedFunctionCallInternDeclaration createComposedFunctionCallInternDeclaration(
		IComposedFunctionCallInternDeclaration parent,
		ISpecificFunctionDeclaration functionDeclaration,
		List inputArgumenDeclarations) {

		return new ComposedFunctionCallInternDeclaration(
			parent,
			functionDeclaration,
			inputArgumenDeclarations);
	}

	public final IDeclaration createDeclaration() {
		return createFunctionCallDeclaration();
	}

	public final IFunctionCallDeclaration createFunctionCallDeclaration() {
		return new FunctionCallDeclaration();
	}

	public final IFunctionCallDeclaration createFunctionCallDeclaration(ISpecificType type) {
		return new FunctionCallDeclaration(type);
	}

	public final IFunctionCallDeclaration createFunctionCallDeclaration(
		ISpecificType mostGeneralType,
		IFunctionCallInternDeclaration functionCallInternDeclaration) {
		return createFunctionCallDeclaration(mostGeneralType, functionCallInternDeclaration);
	}

	public final IFunctionCallDeclaration createFunctionCallDeclaration(ISpecificFunctionDeclaration functionDeclaration) {
		return createFunctionCallDeclaration(
			createComposedFunctionCallInternDeclaration(null, functionDeclaration));
	}

	public final IFunctionCallDeclaration createFunctionCallDeclaration(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		return new FunctionCallDeclaration(
			functionCallInternDeclaration.getSpecificOutputType(),
			functionCallInternDeclaration);
	}

	protected FunctionCallDeclarationFactory() {
		this(TypeManager.getInstance().getDefaultType());
	}

	public final boolean isDefaultAtomicFunctionCallDeclaration(IFunctionCallDeclaration functionCallDeclaration) {
		return functionCallDeclaration
			.getSpecificOutputType()
			.createDefaultFunctionCallInternDeclaration()
			.isomorphicEquals(functionCallDeclaration);
	}
}