package de.uniwue.ki.mas.basics.functionCall;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;

/**
 * A <code>IComposedFunctionCallInternDeclaration</code> is inner node of a {@link IFunctionCallDeclaration},
 * it consits of a {@link ISpecificFunctionDeclaration} and
 * {@link IFunctionCallInternDeclaration IFunctionCallInternDeclarations} as arguments.
 */

public interface IComposedFunctionCallInternDeclaration extends IFunctionCallInternDeclaration {

	void addChild();

	void addChild(int index);

	void addChild(int index, IFunctionCallInternDeclaration child);

	void addChild(IFunctionCallInternDeclaration child);

	Iterator getAllAtomicFunctionCallInternDeclarations();

	/**
	 * Returns the <code>index</code>-th argument of this function call
	 * (the <code>index</code>-th child of this inner node).
	 *
	 * @see setChild(int, IFunctionCallInternDeclaration)
	 */
	IFunctionCallInternDeclaration getChild(int index);

	/**
	 * Returns the argument length of this function call
	 *     (childrens count of this inner node).
	 */
	int getChildCount();

	/**
	 * Returns an the arguments of this function call
	 * (Iterator of {@link IFunctionCallInternDeclaration IFunctionCallInternDeclarations}).
	 */
	Iterator getChildren();

	/**
	 * Returns the index of the given argument.
	 */
	int getIndexOfChild(IFunctionCallInternDeclaration child);

	void removeChild(int index);

	void removeChild(IFunctionCallInternDeclaration child);

	/**
	 * Replaces the <code>index</code>-th argument of this function call
	 * with a new function call.
	 *
	 * @see getChild(int)
	 */
	void setChild(int index, IFunctionCallInternDeclaration child);

	/**
	 * Returns the function declaration of this inner node.
	 */

	ISpecificFunctionDeclaration getSpecificFunctionDeclaration();
	
}
