package de.uniwue.ki.mas.basics.functionCall;
import java.io.Serializable;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.IRemoveListener;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.name.INamed;

/**
 * A <code>IFunctionCallInternDeclaration</code> is a node of a {@link IFunctionCallDeclaration}.
 * <BR>Creation date: (31.03.00 13:40:55)
 *
 * @author Christoph Oechslein
 */

public interface IFunctionCallInternDeclaration
	extends IIsomorphicHashable, INamed, ICopyable, IRemoveable, Serializable {

	void addRemoveListener(IRemoveListener removeListener);
	void removeRemoveListener(IRemoveListener removeListener);

	/**
	 * Checks invariants of this function call declaration:
	 * </ul>
	 *   <li> Most general type is more general than type of root function call intern declaration.
	 *   <li> Parent in function call intern declaration is correct.
	 *   <li> Type of function call intern declaration are correct argument types of parent.
	 * </ul>
	 */
	void checkInvariants();

	/**
	 * Compiles a new {@link IFunction}, calling execute of the results executs the declared function call tree.
	 *
	 * @see ISpecificFunctionDeclaration#createFunction(ICreateFunctionArgs, IFunctionList)
	 * @see IArgumentDeclaration#createFunction(ICreateFunctionArgs)
	 */

	IFunction createFunction(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException;

	/**
	 * Returns the parent of the node in the tree, returns <code>null</code> if it's
	 * a root node.
	 *
	 * @see #setParent(IComposedFunctionCallInternDeclaration)
	 */

	IComposedFunctionCallInternDeclaration getParent();
	
	/**
	 * If parent is null means that this is the rootnude of a function call. Here
	 * we can get the creatinf FunctionCall where this composed function call
	 * intern declaration belongs to.
	 */
	IFunctionCallDeclaration getCreator();
	
	void setCreator(IFunctionCallDeclaration decl);

	/**
	 * Returns the output type of this node, the return type of a function
	 * or the type of a value (leaf).
	 */
	ISpecificType getSpecificOutputType();

	/**
	 * Returns the most general output type of this node, i.e. the typ of the index of the parent function.
	 */
	ISpecificType getMostGeneralSpecificOutputType();

	/**
	 * @see ISpecificFunctionDeclaration#isConstant(ICreateFunctionArgs, IComposedFunctionCallInternDeclaration)
	 */
	boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException;

	/**
	 * @see ISpecificFunctionDeclaration#isReadOnly(ICreateFunctionArgs, IComposedFunctionCallInternDeclaration)
	 */
	boolean isReadOnly(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException;

	/**
	 * Sets a new parent node.
	 *
	 * @see #getParent
	 */
	void setParent(IComposedFunctionCallInternDeclaration newParent);

	void appendName(StringBuffer buff, int depth, int maxDepth);
	
	/**
	 * Add a change listener, if something changes in this declaration
	 * (or in dependend declarations) a {@link ChangeEvent change event} is fired.
	 *
	 * @see# removeChangeListener(ChangeListener)
	 */
	void addChangeListener(ChangeListener newListener);
	
	void removeChangeListener(ChangeListener newListener);
	
	//void fireStateChanged();
}