package de.uniwue.ki.mas.basics.functionCall;
import java.util.Map;

import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (27.11.2001 16:20:19)
 * @author Christoph Oechslein
 */

public class NamedFunctionCallDeclaration
	extends FunctionCallDeclaration
	implements IDocumentableDeclaration {
		
	private boolean _usesShortDescription = false;

	private String _shortDescription = "";

	private String _documentation = "";

	public NamedFunctionCallDeclaration() {
		super();
	}


	public NamedFunctionCallDeclaration(ISpecificType mostGeneralOutputArgumentDeclarationDeclaration) {
		super(mostGeneralOutputArgumentDeclarationDeclaration);
	}

	public NamedFunctionCallDeclaration(
		ISpecificType mostGeneralOutputArgumentDeclaration,
		IFunctionCallInternDeclaration functionCallInternDeclaration) {

		super(mostGeneralOutputArgumentDeclaration, functionCallInternDeclaration);
	}

	public NamedFunctionCallDeclaration(IFunctionCallInternDeclaration functionCallInternDeclaration) {

		super(functionCallInternDeclaration);
	}

	protected String getDefaultName() {
		return super.getName();
	}

	public String getName() {
		return getUsesShortDescription() ? getShortDescription() : getDefaultName();
	}

	/**
	 * @return java.lang.String
	 * @see de.uniwue.ki.util.IDocumentable#getShortDescription()
	 */
	public java.lang.String getShortDescription() {
		return _shortDescription;
	}

	/**
	 * @return boolean
	 * @see de.uniwue.ki.util.IDocumentable#getUsesShortDescription()
	 */
	public boolean getUsesShortDescription() {
		return _usesShortDescription;
	}

	/**
	 * @return boolean
	 */
	public boolean isUsesShortDescription() {
		return _usesShortDescription;
	}

	/**
	 * @param shortDescription String
	 * @see de.uniwue.ki.util.IDocumentable#setShortDescription(String)
	 */
	public void setShortDescription(String shortDescription) {
		String oldValue = _shortDescription;

		if (!Utils.equals(oldValue, shortDescription)) {
			_shortDescription = shortDescription;
			firePropertyChange("shortDescription", oldValue, shortDescription);
			fireStateChanged();
		}
	}

	/**
	 * @param usesShortDescription boolean
	 * @see de.uniwue.ki.util.IDocumentable#setUsesShortDescription(boolean)
	 */
	public void setUsesShortDescription(boolean usesShortDescription) {
		boolean oldValue = getUsesShortDescription();

		if (oldValue != usesShortDescription) {
			_usesShortDescription = usesShortDescription;
			firePropertyChange("usesShortDescription", oldValue, usesShortDescription);
			fireStateChanged();
		}
	}

	/**
	 * Gets the documentation.
	 * @return Returns a String
	 * @see de.uniwue.ki.util.IDocumentable#getDocumentation()
	 */
	public String getDocumentation() {
		return _documentation;
	}

	/**
	 * Sets the documentation.
	 * @param documentation The documentation to set
	 * @see de.uniwue.ki.util.IDocumentable#setDocumentation(String)
	 */
	public void setDocumentation(String documentation) {
		String oldValue = getDocumentation();

		if (!Utils.equals(oldValue, documentation)) {
			_documentation = documentation;
			firePropertyChange("documentation", oldValue, documentation);
			fireStateChanged();
		}
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new NamedFunctionCallDeclaration();
	}

}