package de.uniwue.ki.mas.basics.functionCall;

import java.util.HashMap;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * @author oechslein
 * 
 * Creation date: 27.08.2003
 */
public class NamedFunctionCallDeclarationFactory extends AbstractDeclarationFactory {
	private static final NamedFunctionCallDeclarationFactory _instance =
		new NamedFunctionCallDeclarationFactory();

	private final IFunctionCallInternDeclaration _prototypeFunctionCallInternDecl;

	public static final NamedFunctionCallDeclarationFactory getInstance() {
		return _instance;
	}

	private NamedFunctionCallDeclarationFactory() {
		this(TypeManager.getInstance().getDefaultType());
	}

	public NamedFunctionCallDeclarationFactory(ISpecificType specificType) {
		this(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				specificType));
	}

	public NamedFunctionCallDeclarationFactory(ISpecificType specificType, Object startValue) {
		this(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				specificType,
				startValue));
	}

	public NamedFunctionCallDeclarationFactory(IFunctionCallInternDeclaration prototypeFunctionCallInternDecl) {
		_prototypeFunctionCallInternDecl = prototypeFunctionCallInternDecl;
	}

	public final IDeclaration createDeclaration() {
		NamedFunctionCallDeclaration namedFunctionCallDeclaration = new NamedFunctionCallDeclaration();

		namedFunctionCallDeclaration.setMostGeneralOutputType(
			_prototypeFunctionCallInternDecl.getSpecificOutputType());

		namedFunctionCallDeclaration.setFunctionCallInternDeclaration(
			(IFunctionCallInternDeclaration)_prototypeFunctionCallInternDecl.copy(new HashMap()));

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(
				_prototypeFunctionCallInternDecl.getSpecificOutputType(),
				namedFunctionCallDeclaration.getMostGeneralOutputType());

		return namedFunctionCallDeclaration;
	}
}