package de.uniwue.ki.mas.basics.functionDeclaration;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.AbstractParametricFunction;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
/**
 * Creation date: (08.01.2002 15:25:19)
 * @author Christoph Oechslein
 */

public abstract class AbstractDefaultSpecificFunctionDeclaration
	extends AbstractSpecificFunctionDeclaration
	implements ISpecificFunctionDeclaration, IParametricFunction {

	private final ISpecificType _specificOutputType;
	private boolean _expert = false;
	private boolean _constant;

	public AbstractDefaultSpecificFunctionDeclaration(String name, ISpecificType specificOutputType) {
		this(name, specificOutputType, !VoidType.getInstance().equals(specificOutputType));
	}

	public AbstractDefaultSpecificFunctionDeclaration(
		String name,
		ISpecificType specificOutputType,
		boolean constant) {

		super(name);

		_specificOutputType = specificOutputType;
		_constant = constant;

		if (Assert.ASSERTION_ON && VoidType.getInstance().equals(specificOutputType)) {
			Assert.assertTrue(!constant);
		}
	}

	public abstract IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException;

	public void act(ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {

		createFunction(createAndExecuteFunctionArgs, functions).act(createAndExecuteFunctionArgs);
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		return getSpecificInputTypeAt(index).createDefaultFunctionCallInternDeclaration();
	}

	public IParametricFunction createParametricFunction(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {
		return this;
	}

	public Object execute(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).execute(createAndExecuteFunctionArgs);
	}

	public boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForBoolean(
			createAndExecuteFunctionArgs);
	}

	public double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForDouble(
			createAndExecuteFunctionArgs);
	}

	public int executeForInt(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForInt(
			createAndExecuteFunctionArgs);
	}

	public IGenericFunctionDeclaration getGenericFunctionDeclaration() {
		return null;
	}

	public final IType getOutputType() {
		return getSpecificOutputType();
	}

	public final ISpecificType getSpecificOutputType() {
		return _specificOutputType;
	}

	public final boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return _constant;
	}

	public boolean isExpertFunctionDeclaration() {
		return _expert;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ISpecificFunctionDeclaration)) {
			return false;
		}

		ISpecificFunctionDeclaration specificFunctionDeclaration = (ISpecificFunctionDeclaration) anObject;

		if ((getSpecificInputDefaultArgsLength()
			!= specificFunctionDeclaration.getSpecificInputDefaultArgsLength())
			|| (getSpecificInputMinArgsLength() != specificFunctionDeclaration.getSpecificInputMinArgsLength())
			|| (getSpecificInputMaxArgsLength() != specificFunctionDeclaration.getSpecificInputMaxArgsLength()))
			return false;

		if ((isExpertFunctionDeclaration() != specificFunctionDeclaration.isExpertFunctionDeclaration())
			|| !isomorphicEquals(getGenericFunctionDeclaration(),
				specificFunctionDeclaration.getGenericFunctionDeclaration())) {
			// already done in super-Method via getOutputType().equals(... !
			// && getSpecificOutputType().equals(specificFunctionDeclaration.getSpecificOutputType())
			return false;
		}

		if (getSpecificInputMaxArgsLength() == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			if (!getSpecificInputTypeAt(getSpecificInputMinArgsLength())
				.equals(
					specificFunctionDeclaration.getSpecificInputTypeAt(getSpecificInputMinArgsLength()))) {
				return false;
			}

		} else {
			for (int i = getSpecificInputMinArgsLength(); i < getSpecificInputMaxArgsLength(); i++) {
				if (!getSpecificInputTypeAt(i)
					.equals(specificFunctionDeclaration.getSpecificInputTypeAt(i))) {
					return false;

				} else if (
					!getSpecificInputTypeNameAt(i).equals(
						specificFunctionDeclaration.getSpecificInputTypeNameAt(i))) {
					return false;

				}
			}
		}

		return true;
	}

	public int isomorphicHashCode() {
		return 5 * super.isomorphicHashCode()
			+ 3 * isomorphicHashCode(getGenericFunctionDeclaration())
			+ isomorphicHashCode(isExpertFunctionDeclaration())
			+ getSpecificInputDefaultArgsLength()
			+ getSpecificInputMinArgsLength()
			+ getSpecificInputMaxArgsLength();

		// already done in super-Method via getOutputType().hasCode(... !
		// + getSpecificOutputType().hashCode()
	}

	public void remove() {
	}

	public final void setConstant(boolean constant) {
		if (Assert.ASSERTION_ON && VoidType.getInstance().equals(getSpecificOutputType())) {
			Assert.assertTrue(!constant);
		}

		_constant = constant;
	}

	public void setExpertFunctionDeclaration(boolean newExpertFunctionDeclaration) {
		_expert = newExpertFunctionDeclaration;
	}

	public int getSpecificInputMaxArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	public int getSpecificInputMinArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	protected String getSignatureWithoutNameAndOutputType() {
		StringBuffer buff = new StringBuffer();

		buff.append(" (");

		int size = getSpecificInputDefaultArgsLength();

		for (int j = 0; j < size; j++) {
			buff.append("<T" + j + ">");

			if (j != size - 1) {
				buff.append(", ");
			}
		}

		buff.append(")");

		return buff.toString();
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return AbstractParametricFunction.createFunctionStatic(
			createFunctionArgs,
			this,
			composedFunctionCallInternDeclaration);
	}

}