package de.uniwue.ki.mas.basics.functionDeclaration;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
/**
 * Creation date: (19.03.2002 14:40:11)
 * @author Christoph Oechslein
 */

public abstract class AbstractGenericFunctionDeclaration
	extends AbstractFunctionDeclaration
	implements IGenericFunctionDeclaration {
	private boolean _expertFunctionDeclaration = false;
	private final IType _outputType;

	protected AbstractGenericFunctionDeclaration(String name, IType outputType) {
		this(name, outputType, false);
	}

	protected AbstractGenericFunctionDeclaration(
		String name,
		IType outputType,
		boolean isExpertFunctionDeclaration) {

		super(name);

		_outputType = outputType;

		setExpertFunctionDeclaration(isExpertFunctionDeclaration);
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		if (!TypeManager.getInstance().isInAllTypes(outputType)) {
			return false;

		} else if (getOutputType() instanceof IGenericType) {
			return ((IGenericType)getOutputType()).canCreate(outputType);

			//} else if (getOutputType().equals(outputType)) {
		} else if (outputType.isMoreSpecific((ISpecificType)getOutputType())) {
			return true;

		} else {
			return false;
		}
	}
	
	public boolean canCreateFunctionDeclarationWith(
		ISeSAmDeclaration sesamDeclaration,
		ISpecificType outputType) {
		return canCreateFunctionDeclarationWith(outputType);
	}	

	protected ISpecificType getNextSpecificType(Iterator inputTypes) {
		if (!inputTypes.hasNext())
			return null;

		return (ISpecificType)inputTypes.next();
	}

	public IType getOutputType() {
		return _outputType;
	}

	public boolean isExpertFunctionDeclaration() {
		return _expertFunctionDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void setExpertFunctionDeclaration(boolean newEexpertFunctionDeclaration) {
		_expertFunctionDeclaration = newEexpertFunctionDeclaration;
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return false;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (Assert.ASSERTION_ON) {
			Assert.assertFalse(needSpecificOutputTypeToCreateSpecificFunctionDeclaration());
		}

		ISpecificFunctionDeclaration specificFunctionDeclaration = createFunctionDeclaration(inputTypes);

		if ((specificFunctionDeclaration != null)
			&& (outputType != null)
			&& !TypeManager.getInstance().isMoreGeneral(
				outputType,
				specificFunctionDeclaration.getSpecificOutputType())) {
			return null;

		} else {
			return specificFunctionDeclaration;
		}
	}
	
	public boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration) {
		return needSpecificOutputTypeToCreateSpecificFunctionDeclaration();
	}

	public abstract ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes);

	public String getFullSignature() {
		return getFullSignatureOutputName() + ' ' + getName() + getSignatureWithoutNameAndOutputType();
	}

	protected final String getSignatureWithoutNameAndOutputType() {
		if (getOutputType() instanceof ISpecificType) {
			return getSignatureWithoutNameAndOutputType((ISpecificType)getOutputType());
		} else {
			ISpecificType defaultSpecificType = ((IGenericType)getOutputType()).createDefaultSpecificType();
			if (canCreateFunctionDeclarationWith(defaultSpecificType)) {
				return getSignatureWithoutNameAndOutputType(defaultSpecificType);
			} else {
				return getSignatureWithoutNameAndOutputType(ObjectType.getInstance());
			}
		}
	}

	protected String getFullSignatureOutputName() {
		return getOutputType().getName();
	}
}