package de.uniwue.ki.mas.basics.functionDeclaration;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.AbstractParametricFunction;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
/**
 * Creation date: (30.07.2001 14:45:05)
 * @author Christoph Oechslein
 */

public abstract class AbstractSpecificFunctionAdapterGenericFunctionDeclaration
	extends AbstractSpecificFunctionDeclaration
	implements IParametricFunction {

	private final IGenericFunctionDeclaration _genericFunctionDeclaration;
	private final int _defaultArgLength;

	public AbstractSpecificFunctionAdapterGenericFunctionDeclaration(
		IGenericFunctionDeclaration genericFunctionDeclaration,
		int defaultArgLength) {

		super(genericFunctionDeclaration.getName());

		_genericFunctionDeclaration = genericFunctionDeclaration;
		_defaultArgLength = defaultArgLength;
	}

	public void act(ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs, IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		createFunction(createAndExecuteFunctionArgs, functions).act(createAndExecuteFunctionArgs);
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclarationAt(int index) {
		return getSpecificInputTypeAt(index).createDefaultFunctionCallInternDeclaration();
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return AbstractParametricFunction.createFunctionStatic(
			createFunctionArgs,
			this,
			composedFunctionCallInternDeclaration);
	}

	public IParametricFunction createParametricFunction(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {
		return this;
	}

	public Object execute(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).execute(createAndExecuteFunctionArgs);
	}

	public boolean executeForBoolean(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForBoolean(
			createAndExecuteFunctionArgs);
	}

	public double executeForDouble(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForDouble(
			createAndExecuteFunctionArgs);
	}

	public int executeForInt(
		ICreateAndExecuteFunctionArgs createAndExecuteFunctionArgs,
		IFunctionList functions)
		throws SimulationRuntimeException, ModelCompilingException {
		return createFunction(createAndExecuteFunctionArgs, functions).executeForInt(
			createAndExecuteFunctionArgs);
	}

	public final IGenericFunctionDeclaration getGenericFunctionDeclaration() {
		return _genericFunctionDeclaration;
	}

	public int getSpecificInputDefaultArgsLength() {
		return _defaultArgLength;
	}

	public int getSpecificInputMaxArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	public int getSpecificInputMinArgsLength() {
		return getSpecificInputDefaultArgsLength();
	}

	public IType getOutputType() {
		return getSpecificOutputType();
	}

	protected String getSignatureWithoutNameAndOutputType() {
		if (getSpecificInputMaxArgsLength() == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return "( " + getSpecificInputTypeAt(0).getName() + " ... )";

		} else {
			StringBuffer buff = new StringBuffer();

			buff.append("(");

			int size = getSpecificInputMaxArgsLength();

			for (int i = 0; i < size; i++) {
				ISpecificType currISpecificType = getSpecificInputTypeAt(i);

				buff.append(currISpecificType.getName());

				if (i != size - 1) {
					buff.append(", ");
				}
			}

			buff.append(")");

			return buff.toString();
		}
	}
	
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return getSignatureWithoutNameAndOutputType();
	}

	public ISpecificFunctionType getSpecificFunctionType() {
		if (getSpecificInputMaxArgsLength() == ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return GenericFunctionType.getInstance().createSpecificFunctionType(
				getSpecificOutputType(),
				getSpecificInputTypeAt(0));

		} else {
			java.util.List list = new java.util.ArrayList(getSpecificInputMaxArgsLength() + 1);

			for (int i = 0; i <= getSpecificInputMaxArgsLength(); i++) {
				list.add(getSpecificInputTypeAt(i));
			}

			return GenericFunctionType.getInstance().createSpecificFunctionType(
				getSpecificOutputType(),
				list);
		}
	}

	public String getSpecificInputTypeNameAt(int index) {
		// return getSpecificInputTypeAt(index).getName();
		return null;
	}

	public ISpecificType getSpecificOutputType() {
		return (ISpecificType) _genericFunctionDeclaration.getOutputType();
	}

	public boolean isExpertFunctionDeclaration() {
		return _genericFunctionDeclaration.isExpertFunctionDeclaration();
	}

	public void remove() {
	}
}