package de.uniwue.ki.mas.basics.functionDeclaration;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.*;
/**
 * Creation date: (08.01.2002 15:25:09)
 * @author Christoph Oechslein
 */

public abstract class AbstractSpecificFunctionArgsDeclaration extends AbstractDefaultSpecificFunctionDeclaration {
    private final List _inputTypes;

	public AbstractSpecificFunctionArgsDeclaration(String name, ISpecificType outputType, List inputTypes) {
	    super(name, outputType);
	
	    if (Assert.ASSERTION_ON) {
	        _inputTypes = Collections.unmodifiableList(inputTypes);
	
	    } else {
	        _inputTypes = inputTypes;
	    }
	}

	public AbstractSpecificFunctionArgsDeclaration(
	    String name,
	    ISpecificType outputType,
	    List inputTypes,
	    boolean constant) {
	
	    this(name, outputType, inputTypes);
	
	    setConstant(constant);
	}

	public AbstractSpecificFunctionArgsDeclaration(
	    String name,
	    ISpecificType outputType,
	    List inputTypes,
	    boolean constant,
	    boolean expertFunctionDecl) {
	
	    this(name, outputType, inputTypes, constant);
	
	    setExpertFunctionDeclaration(expertFunctionDecl);
	}

	public final int getSpecificInputDefaultArgsLength() {
	    return _inputTypes.size();
	}

	protected String getSignatureWithoutNameAndOutputType() {
	    StringBuffer buff = new StringBuffer();
	
	    buff.append("(");
	
	    Iterator it = getSpecificInputTypes();
	    while (it.hasNext()) {
	        ISpecificType currISpecificType = (ISpecificType) it.next();
	
	        buff.append(currISpecificType.getName());
	
	        if (it.hasNext()) {
	            buff.append(", ");
	        }
	    }
	
	    buff.append(")");
	
	    return buff.toString();
	}

	public ISpecificFunctionType getSpecificFunctionType() {
	    return GenericFunctionType.getInstance().createSpecificFunctionType(getSpecificOutputType(), _inputTypes);
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
	    return (ISpecificType) _inputTypes.get(index);
	}

	public String getSpecificInputTypeNameAt(int index) {
	    return null;
	}

	public Iterator getSpecificInputTypes() {
	    return _inputTypes.iterator();
	}
}