package de.uniwue.ki.mas.basics.functionDeclaration;

import java.util.Iterator;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * Creation date: 03.12.2002, 16:27:50
 * @author oechslein
 */
public abstract class AbstractSpecificFunctionDeclaration
	extends AbstractFunctionDeclaration
	implements ISpecificFunctionDeclaration {

	public AbstractSpecificFunctionDeclaration(String name) {
		super(name);
	}

	public AbstractSpecificFunctionDeclaration() {
		super();
	}

	public boolean isSpecificFunctionDeclOk(ISpecificType outputType, List inputTypes) {

		if ((outputType != null)
			&& !TypeManager.getInstance().isMoreGeneral(getSpecificOutputType(), outputType))
			return false;

		if ((getSpecificInputMaxArgsLength() != ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT)
			&& (inputTypes.size() > getSpecificInputMaxArgsLength()))
			return false;

		if ((getSpecificInputMaxArgsLength() != ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT)
			&& (inputTypes.size() < getSpecificInputMinArgsLength()))
			return false;

		Iterator it = inputTypes.iterator();
		int i = 0;
		while (it.hasNext()) {
			ISpecificType currType = (ISpecificType)it.next();
			if (!getSpecificInputTypeAt(i).isMoreSpecific(currType))
				return false;

			i++;
		}

		return true;
	}

	public boolean isReadOnly(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return isReadOnlyStatic(createFunctionArgs, composedFunctionCallInternDeclaration);
	}

	public boolean isReadOnly(ICreateFunctionArgs createFunctionArgs) {
		return VoidType.getInstance().equals(getSpecificOutputType());
	}

	public static boolean isReadOnlyStatic(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		if (VoidType.getInstance().equals(composedFunctionCallInternDeclaration.getSpecificOutputType())) {
			return false;

		} else {
			Iterator functionCallInternDeclarations = composedFunctionCallInternDeclaration.getChildren();
			while (functionCallInternDeclarations.hasNext()) {
				IFunctionCallInternDeclaration functionCall =
					(IFunctionCallInternDeclaration)functionCallInternDeclarations.next();

				if (!functionCall.isReadOnly(createFunctionArgs)) {
					Assert.assertTrue(!functionCall.isConstant(createFunctionArgs));
					return false;
				}
			}

			return true;
		}
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return !VoidType.getInstance().equals(getOutputType());
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		if (!isConstant(createFunctionArgs)) {
			return false;

		} else {
			Iterator functionCallInternDeclarations = composedFunctionCallInternDeclaration.getChildren();
			while (functionCallInternDeclarations.hasNext()) {
				if (!((IFunctionCallInternDeclaration)functionCallInternDeclarations.next())
					.isConstant(createFunctionArgs)) {
					return false;
				}
			}

			return true;
		}
	}

	public String getFullSignature() {
		return getSpecificOutputType().getName() + ' ' + getName() + getSignatureWithoutNameAndOutputType();
	}

	protected abstract String getSignatureWithoutNameAndOutputType();

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return getSignatureWithoutNameAndOutputType();
	}

	public String getDocumentation() {
		String documentation = super.getDocumentation();
		if ((documentation != null) && (documentation.length() > 0))
			return documentation;
		else if (getGenericFunctionDeclaration() != null)
			return getGenericFunctionDeclaration().getDocumentation();
		else
			return "";
	}
}
