package de.uniwue.ki.mas.basics.functionDeclaration;

import java.io.Serializable;
import java.util.logging.Logger;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.ParametricFunctionCallDeclarationCompiler;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.IRemoveable;

/**
 * Creation date: 03.07.2002, 08:35:27
 * @author oechslein
 */
public final class FunctionDeclarationCompiler implements IRemoveable, Serializable {
	public static final Logger LOGGER = Logger.getLogger(FunctionDeclarationCompiler.class.getName());

	private static boolean _defaultCompileOn = true;
	private static boolean _defaultInlineOn = true;

	public static boolean isDefaultCompileOn() {
		return _defaultCompileOn;
	}

	public static boolean isDefaultInlineOn() {
		return _defaultInlineOn;
	}

	public static void setDefaultCompileOn(boolean defaultCompileOn) {
		_defaultCompileOn = defaultCompileOn;
	}

	public static void setDefaultInlineOn(boolean defaultInlineOn) {
		_defaultInlineOn = defaultInlineOn;
	}

	//	static {
	//		if (Assert.ASSERTION_ON) {
	//			Handler[] handlers = LOGGER.getHandlers();
	//			for (int i = 0; i < handlers.length; i++) {
	//				Logger.getLogger("").removeHandler(handlers[i]);
	//			}
	//			try {
	//				FileHandler handler = new FileHandler("log/CompileOut.log", 1024 * 1024, 1, true);
	//				handler.setFormatter(new VerySimpleFormatter());
	//				LOGGER.addHandler(handler);
	//			} catch (SecurityException e) {
	//				e.printStackTrace();
	//				LOGGER.addHandler(Logger.getLogger("").getHandlers()[0]);
	//			} catch (IOException e) {
	//				e.printStackTrace();
	//				LOGGER.addHandler(Logger.getLogger("").getHandlers()[0]);
	//			}
	//		}
	//	}

	private final FunctionCallDeclarationCompiler _functionCallDeclarationCompiler;

	private final ParametricFunctionCallDeclarationCompiler _parametricFunctionCallDeclarationCompiler;

	public FunctionDeclarationCompiler() {
		super();

		_functionCallDeclarationCompiler = new FunctionCallDeclarationCompiler(isDefaultCompileOn());
		_parametricFunctionCallDeclarationCompiler =
			new ParametricFunctionCallDeclarationCompiler(isDefaultInlineOn());
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws FunctionCallInternDeclCompileException {

		//		long start = System.currentTimeMillis();

		IFunction result =
			_functionCallDeclarationCompiler.createFunction(
				createFunctionArgs,
				composedFunctionCallInternDeclaration);

		//		long usedTime = System.currentTimeMillis() - start;
		//
		//		if (usedTime > 10)
		//			System.out.println("Needed " + (usedTime) + "msecs to compile " + result.getName() + ".");

		return result;
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclarationCompiler.createFunction(
			createFunctionArgs,
			parametricFunctionCallDeclaration,
			composedFunctionCallInternDeclaration);
	}

	public IFunction createFunction(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
		IFunctionList functions)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclarationCompiler.createFunction(
			createFunctionArgs,
			parametricFunctionCallDeclaration,
			functions);
	}

	public IFunction createFunction(
		IFunctionCallDeclaration functionCallDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws FunctionCallDeclCompileException {

		try {
			return functionCallDeclaration.getFunctionCallInternDeclaration().createFunction(
				createFunctionArgs);

		} catch (ModelCompilingException exception) {
			if (exception instanceof FunctionCallDeclCompileException)
				throw (FunctionCallDeclCompileException) exception;
			else
				throw new FunctionCallDeclCompileException(functionCallDeclaration, exception);
		}
	}

	public IParametricFunction createParametricFunction(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclarationCompiler.createParametricFunction(
			createFunctionArgs,
			parametricFunctionCallDeclaration);
	}

	public final boolean isCompileOn() {
		return _functionCallDeclarationCompiler.isCompileOn();
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws FunctionCallInternDeclCompileException {

		return _functionCallDeclarationCompiler.isConstant(
			createFunctionArgs,
			composedFunctionCallInternDeclaration);
	}

	public boolean isConstant(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclarationCompiler.isConstant(
			createFunctionArgs,
			parametricFunctionCallDeclaration,
			composedFunctionCallInternDeclaration);
	}

	public boolean isReadOnly(
		ICreateFunctionArgs createFunctionArgs,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws FunctionCallInternDeclCompileException {

		return _functionCallDeclarationCompiler.isReadOnly(
			createFunctionArgs,
			composedFunctionCallInternDeclaration);
	}

	public boolean isReadOnly(
		ICreateFunctionArgs createFunctionArgs,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws ModelCompilingException {

		return _parametricFunctionCallDeclarationCompiler.isReadOnly(
			createFunctionArgs,
			parametricFunctionCallDeclaration,
			composedFunctionCallInternDeclaration);
	}

	public void remove() {
		_functionCallDeclarationCompiler.remove();
		_parametricFunctionCallDeclarationCompiler.remove();
	}
}
