package de.uniwue.ki.mas.basics.functionDeclaration;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.IType;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
/**
 * The declaration of a function gives the name, return type and the arguments (length and types), a declaration can generate
 * an {@link IFunction}.
 * <BR>The length of the parameter list can be having a min, max value and it must have a default value.
 * Also unlimited length is possible through {@link #UNLIMITED_ARGUMENT_COUNT}.
 * <BR> The input arguments can also have dynamic type {@link IGenericFunctionDeclaration IGenericFunctionDeclaration},
 * for 'normal' function declaration {@link ISpecificFunctionDeclaration}. And since the output type is {@link IType}.
 * <BR>Creation date: (10.05.00 17:28:46)
 *
 * @author Christoph Oechslein
 * @see de.uniwue.ki.mas.model.basics.type.IType
 */

public interface IFunctionDeclaration extends IDocumentableDeclaration {
	/**
	 * Gets the output type of theis function declaration. <BR>Creation date: (10.05.00 17:29:23)
	 * @author Christoph Oechslein
	 */
	IType getOutputType();

	String getSignatureWithoutNameAndOutputType(ISpecificType outputType);

	/**
	 * Returns true if this function declaration is only for experts.
	 */
	boolean isExpertFunctionDeclaration();
	
	String getFullSignature();
}