package de.uniwue.ki.mas.basics.functionDeclaration;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;

/**
 * A generic function declaration can produce specific function declarations:
 * {@link ISpecificFunctionDeclaration ISpecificFunctionDeclarations}.
 * It is like a C++ generic function, for example <code>set<T> concatenate (set<T>, set<T>)</code> is a
 * generic function dedclaration with can produce specif function declarations like
 * <code>set_BooleanType concatenate (set_BooleanType, set_BooleanType)</code>
 * <BR>The unspecific types (like <code>T</code>) are represented by dependent Types.
 *
 * @author Christoph Oechslein
 */

public interface IGenericFunctionDeclaration extends IFunctionDeclaration {

	/**
	 * Returns <code>true</code> if this generic function declaration can generate a 
	 * specific function declaration with specific output type <code>outputType</code>.
	 * <BR> For example the <code>concatenate</code> generic function can produce 
	 * a set of boolean but cannot return a boolean.
	 */
	boolean canCreateFunctionDeclarationWith(ISpecificType outputType);
	
	boolean canCreateFunctionDeclarationWith(ISeSAmDeclaration sesamDeclaration, ISpecificType outputType);

	/**
	 * Generate a SpecificFunctionDeclaration for the given output and dependent types.
	 * Can produce an error if outputType is wrong, must successed if canCreateFunctionDeclaration is true
	 * @preconditions {@link #canCreateFunctionDeclaration(ISpecificType outputType)} must return true.
	 */

	ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings);

	boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration(IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration);

	/**
	 * This method generates a SpecificFunctionDeclaration for the given input types,
	 * it returns <code>null</code> if input types are incorrect.
	 */
	ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType, Iterator inputTypes);

	/**
	 * Get the dependent types for this generic function declaration.
	 *
	 * @see DependentType
	 */
	Iterator getDependentTypes(ISpecificType outputSpecificType);

}