package de.uniwue.ki.mas.basics.functionDeclaration.list;
import java.util.*;

import javax.swing.event.ChangeEvent;

import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.util.declaration.DefaultDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (24.07.2001 15:18:22)
 * @author Christoph Oechslein
 */

public class FunctionDeclarationList extends DefaultDeclarationList implements IFunctionDeclarationList {
	public FunctionDeclarationList() {
		super();
	}

	public FunctionDeclarationList(Iterator it) {
		super();

		CollectionsHelper.addAll(this, it);
	}

	public FunctionDeclarationList(IFunctionDeclarationList functionDeclarationList) {
		super(functionDeclarationList);
	}

	public boolean accept(Object declaration) {
		if (!(declaration instanceof IFunctionDeclaration))
			return false;

		if (declaration instanceof IUserFunctionDeclaration)
			return true;

		IFunctionDeclaration functionDecl = (IFunctionDeclaration)declaration;
		if (functionDecl.getName() == null)
			return false;

		String documentation = functionDecl.getDocumentation();

		return (documentation != null) && (documentation.length() > 0);
	}

	public void add(int index, Object element) {
		super.add(index, element);

		stateChanged();
	}

	public boolean add(Object element) {
		if (super.add(element)) {
			stateChanged();
			return true;
		} else
			return false;
	}

	public boolean canBeRemoved(boolean recursive) {
		return false;
	}

	public ICopyable copy(Map objectMapping) {
		return new FunctionDeclarationList(this);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		IFunctionDeclarationList dstFunctionDeclarationList = (IFunctionDeclarationList)dstObject;

		dstFunctionDeclarationList.clear();
		dstFunctionDeclarationList.addAll(this);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new FunctionDeclarationList();
	}

	public final ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		String str,
		ISpecificType outputType) {

		return FunctionDeclarationListHelper.getSpecificFunctionDeclaration(iterator(), str, outputType);
	}

	public final ISpecificFunctionDeclaration getSpecificFunctionDeclaration(
		String str,
		ISpecificType outputType,
		List inputTypes) {

		return FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
			this,
			str,
			outputType,
			inputTypes);
	}

	public final Iterator getFunctionDeclarations(ISpecificType outputType) {
		return FunctionDeclarationListHelper.getFunctionDeclarations(this, outputType);
	}

	public Iterator getGenericFunctionDeclarations() {
		return FunctionDeclarationListHelper.getGenericFunctionDeclarations(iterator());
	}

	public Iterator getSpecificFunctionDeclarations() {
		return FunctionDeclarationListHelper.getSpecificFunctionDeclarations(iterator());
	}

	public Object set(int index, Object element) {
		Object oldValue = super.set(index, element);

		stateChanged();

		return oldValue;
	}

	public Iterator getFunctionDeclarations() {
		return iterator();
	}

	public void stateChanged() {
		stateChanged(new ChangeEvent(this));
	}
}