package de.uniwue.ki.mas.basics.functionDeclaration.list;
import de.uniwue.ki.mas.basics.action.FunctionActionDeclarationFactory;
import de.uniwue.ki.mas.basics.function.bool.FunctionDeclarationBoolFactory;
import de.uniwue.ki.mas.basics.function.function.FunctionDeclarationFunctionFactory;
import de.uniwue.ki.mas.basics.function.hashtable.FunctionDeclarationHashTableFactory;
import de.uniwue.ki.mas.basics.function.iterator.FunctionDeclarationIteratorFactory;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.number.FunctionDeclarationNumberFactory;
import de.uniwue.ki.mas.basics.function.object.FunctionDeclarationObjectFactory;
import de.uniwue.ki.mas.basics.function.random.FunctionDeclarationRandomFactory;
import de.uniwue.ki.mas.basics.function.rest.FunctionDeclarationRestFactory;
import de.uniwue.ki.mas.basics.function.statistic.FunctionDeclarationStatisticFactory;
import de.uniwue.ki.mas.basics.function.string.FunctionDeclarationStringFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.composed.function.FunctionDeclarationComposedTypeFactory;
import de.uniwue.ki.mas.basics.type.enum.FunctionDeclarationEnumTypeFactory;
import de.uniwue.ki.mas.sim.experiment.function.FunctionDeclarationExperimentFactory;

/**
 * Creation date: (16.01.2002 13:43:23)
 * @author Christoph Oechslein
 */

public final class FunctionDeclarationListManager {
	private static IFunctionDeclarationList _allFunctionDeclarationList;
	private static IFunctionDeclarationList _defaultFunctionDeclarationList;
	private static IFunctionDeclarationList _defaultObjectFunctionDeclarationList;

	public FunctionDeclarationListManager() {
		super();
	}

	public static void addFunctionToDefaultFunctions(IFunctionDeclaration functionDeclaration) {
		getDefaultFunctionDeclarationList().add(functionDeclaration);

		_allFunctionDeclarationList = null;
	}

	// not longer necessary ?
	//	public static void checkFunctions(IFunctionDeclarationList functionDeclarationList) {
	//		for (int i = 0; i < functionDeclarationList.size(); i++) {
	//			for (int j = 0; j < functionDeclarationList.size(); j++) {
	//				if (j != i) {
	//					IFunctionDeclaration currIFunctionDeclaration =
	//						(IFunctionDeclaration) functionDeclarationList.get(i);
	//					IFunctionDeclaration currIFunctionDeclaration2 =
	//						(IFunctionDeclaration) functionDeclarationList.get(j);
	//
	//					Assert.assertTrue(
	//						"FunctionDeclarations have same name '"
	//							+ currIFunctionDeclaration.getName()
	//							+ "' "
	//							+ currIFunctionDeclaration
	//							+ " and "
	//							+ currIFunctionDeclaration2,
	//						!currIFunctionDeclaration.getName().equals(currIFunctionDeclaration2.getName()));
	//				}
	//			}
	//		}
	//	}

	public synchronized static IFunctionDeclarationList getAllFunctionDeclarationList() {
		if (_allFunctionDeclarationList == null) {
			_allFunctionDeclarationList = new FunctionDeclarationList();
			_allFunctionDeclarationList.addAll(getDefaultFunctionDeclarationList());
			_allFunctionDeclarationList.addAll(getDefaultObjectFunctionDeclarationList());
		}
		return _allFunctionDeclarationList;
	}

	public synchronized static IFunctionDeclarationList getDefaultFunctionDeclarationList() {
		if (_defaultFunctionDeclarationList == null) {
			_defaultFunctionDeclarationList = new FunctionDeclarationList();
			insertFunctions(_defaultFunctionDeclarationList);
			// checkFunctions(_defaultFunctionDeclarationList);
		}
		return _defaultFunctionDeclarationList;
	}

	public synchronized static IFunctionDeclarationList getDefaultObjectFunctionDeclarationList() {
		if (_defaultObjectFunctionDeclarationList == null) {
				_defaultObjectFunctionDeclarationList = new FunctionDeclarationList();
				insertObjectFunctions(_defaultObjectFunctionDeclarationList);

				//checkFunctions(_defaultObjectFunctionDeclarationList);
		}
		return _defaultObjectFunctionDeclarationList;
	}

	private static void insertFunctions(IFunctionDeclarationList functionDeclarationList) {
		FunctionDeclarationBoolFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationNumberFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationStringFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationStatisticFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationRandomFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationIteratorFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationListFunctionsFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationHashTableFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationFunctionFactory.insertFunctions(functionDeclarationList);

		FunctionActionDeclarationFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationRestFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationExperimentFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationComposedTypeFactory.insertFunctions(functionDeclarationList);

		FunctionDeclarationEnumTypeFactory.insertFunctions(functionDeclarationList);
	}

	private static void insertObjectFunctions(IFunctionDeclarationList functionDeclarationList) {
		FunctionDeclarationVariableFactory.insertFunctions(functionDeclarationList);
		FunctionDeclarationObjectFactory.insertFunctions(functionDeclarationList);
	}
}