package de.uniwue.ki.mas.basics.functionDeclaration.list;
import java.text.Collator;
import java.util.*;

import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * Creation date: (17.09.2001 18:10:27)
 * @author Christoph Oechslein
 */

public final class FunctionDeclarationListModel extends DefaultListModel implements ListModel {
	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			updateAllFunctionDeclarations();
		}
	};

	private List _allFunctionDeclarations = null;

	private ISpecificType _specificType = TypeManager.getInstance().getDefaultType();

	private IFunctionDeclarationListAble _functionDeclarationListAble;

	private final FunctionDeclarationComparator _functionDeclarationComparator =
		new FunctionDeclarationComparator();

	private class FunctionDeclarationComparator implements Comparator {
		public boolean equals(Object obj) {
			return (obj.getClass() == getClass());
		}

		public int hashCode() {
			return getClass().hashCode();
		}

		public int compare(Object o1, Object o2) {
			IFunctionDeclaration functionDeclaration1 = (IFunctionDeclaration) o1;
			IFunctionDeclaration functionDeclaration2 = (IFunctionDeclaration) o2;
			return Collator.getInstance().compare(
				functionDeclaration1.getName()
					+ getSignatureWithoutNameAndOutputType(functionDeclaration1, getSpecificType()),
				functionDeclaration2.getName()
					+ getSignatureWithoutNameAndOutputType(functionDeclaration2, getSpecificType()));
		}
	}

	public FunctionDeclarationListModel() {
		super();

		setFunctionDeclarationListAble(FunctionDeclarationListManager.getAllFunctionDeclarationList());
	}

	public FunctionDeclarationListModel(ISpecificType specificType) {
		this();

		setSpecificType(specificType);
	}

	private Iterator calculateFunctions() {
		if (getFunctionDeclarationListAble() != null)
			return calculateFunctions(getSpecificType());
		else
			return CollectionsHelper.EMPTY_ITERATOR;
	}

	private Iterator calculateFunctions(ISpecificType specificType) {
		return FunctionDeclarationListHelper.getFunctionDeclarations(
			getAllFunctionDeclarations(),
			specificType);
	}

	public ISpecificType getSpecificType() {
		return _specificType;
	}

	private void initializeList() {
		if (getSpecificType() == null) {
			clear();

		} else {

			List newList =
				CollectionsHelper.asSortedArrayList(calculateFunctions(), _functionDeclarationComparator);

			//			if (Assert.ASSERTION_ON)
			//				Assert.assertFalse(
			//					"Duplicates: " + CollectionsHelper.getDuplicateEntries(newList),
			//					CollectionsHelper.hasDuplicateEntries(newList));

			clear();
			addAll(newList);
		}
	}

	private void addAll(Collection col) {
		Iterator it = col.iterator();
		while (it.hasNext()) {
			addElement(it.next());
		}
	}

	public IFunctionDeclarationListAble getFunctionDeclarationListAble() {
		return _functionDeclarationListAble;
	}

	public void setFunctionDeclarationListAble(IFunctionDeclarationListAble newFunctionDeclarationListAble) {
		IFunctionDeclarationListAble oldFunctionDeclarationListAble = getFunctionDeclarationListAble();

		if (Utils.equals(newFunctionDeclarationListAble, oldFunctionDeclarationListAble))
			return;

		if (oldFunctionDeclarationListAble != null) {
			oldFunctionDeclarationListAble.removeChangeListener(_changeListener);
		}

		_functionDeclarationListAble = newFunctionDeclarationListAble;

		if (newFunctionDeclarationListAble != null) {
			newFunctionDeclarationListAble.addChangeListener(_changeListener);
		}

		updateAllFunctionDeclarations();
	}

	protected void updateAllFunctionDeclarations() {
		if (getFunctionDeclarationListAble() == null)
			setAllFunctionDeclarations(CollectionsHelper.EMPTY_LIST);
		else
			setAllFunctionDeclarations(getFunctionDeclarationListAble().getFunctionDeclarations());
	}

	public void setSpecificType(ISpecificType newType) {
		ISpecificType oldType = getSpecificType();
		if (!Utils.equals(oldType, newType)) {
			_specificType = newType;
			initializeList();
		}
	}

	public void remove() {
		setFunctionDeclarationListAble(null);
		setSpecificType(null);
	}

	private void setAllFunctionDeclarations(List allFunctionDeclarations) {
		if (!Utils.equals(getAllFunctionDeclarations(), allFunctionDeclarations)) {
			_allFunctionDeclarations = allFunctionDeclarations;
			initializeList();
		}
	}

	private void setAllFunctionDeclarations(Iterator allFunctionDeclarations) {
		setAllFunctionDeclarations(CollectionsHelper.asLinkedList(allFunctionDeclarations));
	}

	private List getAllFunctionDeclarations() {
		return _allFunctionDeclarations;
	}

	public static String getSignatureWithoutNameAndOutputType(
		IFunctionDeclaration functionDeclaration,
		ISpecificType outputSpecificType) {

		if (functionDeclaration instanceof IGenericFunctionDeclaration) {
			IGenericFunctionDeclaration genericFunctionDeclaration =
				(IGenericFunctionDeclaration) functionDeclaration;
			ISpecificType newOutputType =
				FunctionDeclarationListHelper.getOutputTypeWithCanCreateFunctionDeclaration(
					genericFunctionDeclaration,
					outputSpecificType);

			if (newOutputType==null) return "Error";
			Assert.assertNotNull(newOutputType);
			return genericFunctionDeclaration.getSignatureWithoutNameAndOutputType(newOutputType);

		} else {
			return functionDeclaration.getSignatureWithoutNameAndOutputType(outputSpecificType);
		}
	}
}