package de.uniwue.ki.mas.basics.functionUser;
import java.util.*;

import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.SpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (27.11.2001 14:38:10)
 * @author Christoph Oechslein
 */

final class UserFunctionDeclaration
	extends SpecificFunctionDeclarationWithArgDeclBinds
	implements IUserFunctionDeclaration {

	private boolean _expertFunctionDeclaration = false;
	private boolean _external = false;

	public UserFunctionDeclaration() {
		super();

		initialize();
	}
	
	public UserFunctionDeclaration(ISpecificType specificType) {
		super(specificType);

		initialize();
	}

	private void initialize() {
		setUsesShortDescription(true);
		setShortDescription("Untitled");
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new UserFunctionDeclaration();
	}

	public boolean isExpertFunctionDeclaration() {
		return _expertFunctionDeclaration;
	}

	public void setExpertFunctionDeclaration(boolean expertFunctionDeclaration) {
		boolean oldValue = _expertFunctionDeclaration;
		_expertFunctionDeclaration = expertFunctionDeclaration;

		if (oldValue != expertFunctionDeclaration)
			fireStateChanged();
	}

	public void setExternal(boolean external) {
		boolean oldValue = _external;
		_external = external;

		if (oldValue != external)
			fireStateChanged();
	}

	public boolean isExternal() {
		return _external;
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		UserFunctionDeclaration dstUserFunctionDeclaration = (UserFunctionDeclaration)dstObject;

		dstUserFunctionDeclaration.setExternal(isExternal());
		dstUserFunctionDeclaration.setExpertFunctionDeclaration(isExpertFunctionDeclaration());
		if (getRemovingNotOkListener() != null) {
			//dstUserFunctionDeclaration.removeAllRemoveVetoListeners();
			dstUserFunctionDeclaration.removeRemoveVetoListener(getRemovingNotOkListener());
			dstUserFunctionDeclaration.addRemoveVetoListener(getRemovingNotOkListener());
		}
	}

	public void setInlinePossible(boolean inlinePossible) {
		getParametricFunctionCallDeclaration().setInlinePossible(inlinePossible);
		fireStateChanged();
	}

	public boolean isInlinePossible() {
		return getParametricFunctionCallDeclaration().isInlinePossible();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		buff.append(getShortDescription());
	}

	private LinkedList usedTypes = new LinkedList();

	protected void fireStateChanged() {
		super.fireStateChanged();

		//alte VetoListeners auf Argumenttypen entfernen
		Iterator it = usedTypes.iterator();
		while (it.hasNext()) {
			ISpecificType element = (ISpecificType)it.next();

			if (element instanceof ISpecificComposedType) {
				removeRemoveVetoListenerFrom(((ISpecificComposedType)element).getDeclaration());
			}
		}
		usedTypes = new LinkedList();

		IParametricFunctionCallDeclaration para = getParametricFunctionCallDeclaration();
		it = para.getReferencedSpecificTypes();

		while (it.hasNext()) {
			ISpecificType specificType = (ISpecificType)it.next();
			if (specificType instanceof ISpecificComposedType) {
				usedTypes.add(specificType);
				addRemoveVetoListernerTo(((ISpecificComposedType)specificType).getDeclaration());
			}
		}
	}
}
