package de.uniwue.ki.mas.basics.search;

import java.util.*;

import javax.swing.tree.DefaultMutableTreeNode;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeAttribute;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.basics.type.user.UserTypeDeclarationList;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * @author herrler
 */
public class SearchChildrenHelper {

	public static Iterator findChildrenUserTypeDeclarations(
		final IDeclaration searchedDeclaration,
		ISeSAmDeclaration sesamDeclaration) {

		Assert.assertTrue(
			"Tried to children dor search for " + searchedDeclaration + " which is not possible",
			isSearchAbleDeclaration(searchedDeclaration));

		return findChildrenUserTypedeclarations(
			((IUserTypeDeclaration)searchedDeclaration).getSpecificComposedType(),
			sesamDeclaration);
	}

	public static Iterator findChildrenUserTypedeclarations(
		final ISpecificComposedType comType,
		ISeSAmDeclaration sesamDeclaration) {

		UserTypeDeclarationList typeDeclarationList =
			sesamDeclaration
				.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserTypeDeclarationListAndFactory()
				.getUserTypeDeclarationList();

		LinkedList childrenList = new LinkedList();
		Iterator it = typeDeclarationList.iterator();
		while (it.hasNext()) {
			IUserTypeDeclaration userTypeDeckl = (IUserTypeDeclaration)it.next();
			if ((userTypeDeckl.isComposedType())
				&& (userTypeDeckl.getSpecificComposedType().getFather() == comType)) {
				childrenList.add(userTypeDeckl);
			}
		}

		return childrenList.iterator();
	}

	public static Iterator findChildrenComposedTypes(
		final ISpecificComposedType fatherComposedType,
		ISeSAmDeclaration sesamDeclaration) {

		LinkedList childrenList = new LinkedList();
		Iterator it = TypeManager.getInstance().getSystemComposedTypes();
		while (it.hasNext()) {
			ISpecificComposedType specificComposedType = (ISpecificComposedType)it.next();
			if ((specificComposedType.getFather() == fatherComposedType)) {
				childrenList.add(specificComposedType);
			}
		}

		return CollectionsHelper.concatenate(
			findChildrenUserTypedeclarations(fatherComposedType, sesamDeclaration),
			childrenList.iterator());
	}

	public static boolean isSearchAbleDeclaration(final Object searchedDeclaration) {
		return (
			(searchedDeclaration instanceof IUserTypeDeclaration)
				&& ((IUserTypeDeclaration)searchedDeclaration).isComposedType());
	}

	public static DefaultMutableTreeNode buildDescendantTree(
		ISpecificComposedType comType,
		ISeSAmDeclaration sesamDeclaration) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(comType);

		Iterator it = findChildrenComposedTypes(comType, sesamDeclaration);

		while (it.hasNext()) {
			Object composedTypeOrUserType = it.next();
			if (composedTypeOrUserType instanceof IUserTypeDeclaration)
				composedTypeOrUserType =
					((IUserTypeDeclaration)composedTypeOrUserType).getSpecificComposedType();

			node.add(buildDescendantTree((ISpecificComposedType)composedTypeOrUserType, sesamDeclaration));
		}

		return node;
	}

	public static DefaultMutableTreeNode buildAncestorTree(
		ISpecificComposedType comType,
		ISeSAmDeclaration sesamDeclaration) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(comType);

		ISpecificComposedType father = comType.getFather();

		if (father != null)
			buildAncestorTree(father, sesamDeclaration).add(node);

		return node;
	}

	public static DefaultMutableTreeNode buildTree(
		ISpecificComposedType comType,
		ISeSAmDeclaration sesamDecl) {
		ISpecificComposedType father = comType.getFather();

		if (father != null) {
			DefaultMutableTreeNode ancestor = buildAncestorTree(father, sesamDecl);
			ancestor.add(buildDescendantTree(comType, sesamDecl));
			return (DefaultMutableTreeNode)ancestor.getFirstChild();
		}

		return buildDescendantTree(comType, sesamDecl);
	}

	public static boolean isInheritedAttribut(ISpecificComposedType comType, IComposedTypeAttribute attr) {
		Iterator it = comType.getOwnAttributes();
		boolean inherited = false;

		while (it.hasNext())
			inherited |= it.next() == attr;

		return !inherited;
	}

	public static HashMap getAttributeDefiners(ISpecificComposedType comType) {
		HashMap map = new HashMap();
		getAttributeDefiners(map, comType);
		return map;
	}

	private static void getAttributeDefiners(HashMap map, ISpecificComposedType comType) {
		ISpecificComposedType father = comType.getFather();

		if (father != null)
			getAttributeDefiners(map, father);

		Iterator it = comType.getOwnAttributes();

		while (it.hasNext())
			map.put(it.next(), comType.getName());
	}

}
