package de.uniwue.ki.mas.basics.type;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.object.HasFeatureFunction;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (30.08.2000 09:55:35)
 * @author Christoph Oechslein
 */

public abstract class AbstractSimObjectType extends AbstractSpecificType implements ISimObjectType {
	public AbstractSimObjectType() {
		super();
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			appendNameForNamedObject(buff, (IObject)anObject, currDepth, maxDepth);
		}
	}

	public IGenericType getGenericType() {
		return GenericSimObjectType.getInstance();
	}

	public String getName() {
		if (getObjectClassDeclaration() == null) {
			if (getFeatureClassDeclaration() == null) {
				return "SimObject";

			} else {
				return getFeatureClassDeclaration().getName() + " <SimObject>";
			}

		} else {
			return getObjectClassDeclaration().getName() + " <SimObject>";
		}

	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		if (value == null) {
			return true;

		} else if (!(value instanceof IObject)) {
			return false;

		} else if (getObjectClassDeclaration() == null) {
			IObject object = (IObject)value;

			return (
				object.getObjectInstanceDeclaration().getObjectClassDeclaration()
					== getObjectClassDeclaration());

		} else if (getFeatureClassDeclaration() == null) {
			IObject object = (IObject)value;

			return HasFeatureFunction.hasFeature(object, getFeatureClassDeclaration());

		} else {
			if (Assert.ASSERTION_ON) {
				Assert.assertTrue(
					(getObjectClassDeclaration() == null) && (getFeatureClassDeclaration() == null));
			}
			return true;
		}
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (!(type instanceof ISimObjectType))
			return false;

		AbstractSimObjectType bodyImageType = (AbstractSimObjectType)type;

		if (getObjectClassDeclaration() != null) {
			return getObjectClassDeclaration().equals(bodyImageType.getObjectClassDeclaration());

		} else if (getFeatureClassDeclaration() != null) {
			if (bodyImageType.getFeatureClassDeclaration() != null) {
				return getFeatureClassDeclaration().equals(bodyImageType.getFeatureClassDeclaration());

			} else if (bodyImageType.getObjectClassDeclaration() != null) {
				return HasFeatureFunction.hasFeature(
					bodyImageType.getObjectClassDeclaration(),
					getFeatureClassDeclaration());

			} else {
				if (Assert.ASSERTION_ON)
					Assert.assertTrue(
						(bodyImageType.getObjectClassDeclaration() == null)
							&& (bodyImageType.getFeatureClassDeclaration() == null));
				return false;
			}

		} else {
			if (Assert.ASSERTION_ON)
				Assert.assertTrue(
					(getObjectClassDeclaration() == null) && (getFeatureClassDeclaration() == null));
			return true;
		}
	}

	public boolean equals(Object obj) {
		if (obj == null) {
			return false;

		} else if (this == obj) {
			return true;

		} else if (obj instanceof ISimObjectType) {
			ISimObjectType simObjectType = (ISimObjectType)obj;
			return Utils.equals(getFeatureClassDeclaration(), simObjectType.getFeatureClassDeclaration())
				&& Utils.equals(getObjectClassDeclaration(), simObjectType.getObjectClassDeclaration());
		} else {
			return false;
		}
	}
}