package de.uniwue.ki.mas.basics.type;

import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionArgument.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.name.INamed;

/**
 * @author oechslein
 * 
 * Creation date: 15.05.2003
 */
public abstract class AbstractSpecificFunctionType
	extends AbstractSpecificType
	implements ISpecificFunctionType {

	/** 
	 * Supporting closures means that you can create a function which returns a function which 
	 * capture outer parameters. 
	 **/
	private static boolean _supportClosures = false;

	public static IParametricFunction getParametricFunction(Object value) {
		if (isSupportClosures() && (value instanceof ArgumentWithBindings.ResultWithBindings)) {
			return (IParametricFunction) ((ArgumentWithBindings.ResultWithBindings)value).getValue();
		} else {
			return (IParametricFunction)value;
		}
	}

	private static IArgumentBindings getSavedArgumentBindings(Object value) {
		if (isSupportClosures() && (value instanceof ArgumentWithBindings.ResultWithBindings)) {
			return ((ArgumentWithBindings.ResultWithBindings)value).getSavedArgumentBindings();
		} else {
			return null;
		}
	}

	public static void popArgumentBindingsFrom(IExecuteFunctionArgs executeFunctionArgs, Object value)
		throws SimulationRuntimeException {
		if (isSupportClosures()) {
			IArgumentBindings savedArgumentBindings = getSavedArgumentBindings(value);
			if (savedArgumentBindings != null)
				executeFunctionArgs.popArgumentBindings();
		}
	}

	public static void pushArgumentBindingsTo(IExecuteFunctionArgs executeFunctionArgs, Object value)
		throws SimulationRuntimeException {
		if (isSupportClosures()) {
			IArgumentBindings savedArgumentBindings = getSavedArgumentBindings(value);
			if (savedArgumentBindings != null)
				executeFunctionArgs.pushArgumentBindings(savedArgumentBindings);
		}
	}

	private final ISpecificType _outputType;

	public AbstractSpecificFunctionType(ISpecificType outputType) {
		_outputType = outputType;
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {
			buff.append("'");
			appendNameForNamedObject(buff, (ISpecificFunctionDeclaration)anObject, currDepth, maxDepth);
			buff.append("'");
		}
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {
			buff.append("'");
			appendNameForNamedObject(buff, (INamed)anObject, currDepth, maxDepth);
			buff.append("'");
		}
	}

	public IArgumentDeclaration copyArgumentDeclaration(
		IArgumentDeclaration argumentDeclaration,
		Map objectMapping) {

		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);
		Assert.assertEquals(argumentDeclaration.getSpecificType(), this);

		if (argumentDeclaration.getObject() instanceof IUserFunctionDeclaration)
			return new ArgumentDeclarationWithNeverRemoveVetoListener(
				this,
				(IUserFunctionDeclaration)argumentDeclaration.getObject());
		else
			return super.copyArgumentDeclaration(argumentDeclaration, objectMapping);
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		if (anObject == null) {
			return null;

		} else {
			ISpecificFunctionDeclaration specificFunctionDeclaration = (ISpecificFunctionDeclaration)anObject;

			if (specificFunctionDeclaration instanceof SpecificFunctionDeclarationWithArgDeclBinds) {
				return specificFunctionDeclaration.copy(
					objectMapping == null ? new HashMap() : objectMapping);

			} else {
				return anObject;
			}
		}
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return createArgumentStatic(anObject, createFunctionArgs);
	}

	public IArgumentDeclaration createArgumentDeclaration(Object anObject) {
		if (anObject instanceof IUserFunctionDeclaration)
			return new ArgumentDeclarationWithNeverRemoveVetoListener(
				this,
				(IUserFunctionDeclaration)anObject);
		else
			return super.createArgumentDeclaration(anObject);
	}

	private IArgument createArgumentStatic(Object anObject, ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {
		IParametricFunction paramFunction =
			((ISpecificFunctionDeclaration)anObject).createParametricFunction(
				createFunctionArgs.createCreateAndExecuteFunctionArgs());

		if (isSupportClosures())
			return new ArgumentWithBindings(paramFunction);
		else
			return new Argument(paramFunction);
	}

	public IGenericType getGenericType() {
		return GenericFunctionType.getInstance();
	}

	public final ISpecificType getSpecificOutputType() {
		return _outputType;
	}

	public Iterator getSubTypes() {
		return CollectionsHelper.EMPTY_ITERATOR;
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value == null || value instanceof ISpecificFunctionDeclaration;
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		if (anObject == null) {
			return true;

		} else if (anObject instanceof ISpecificFunctionDeclarationWithArgDeclBinds) {
			return ((ISpecificFunctionDeclarationWithArgDeclBinds)anObject)
				.getParametricFunctionCallDeclaration()
				.getFunctionCallDeclaration()
				.isConstant(createFunctionArgs);

		} else {
			return ((ISpecificFunctionDeclaration)anObject).isConstant(createFunctionArgs);
		}
	}

	public boolean isReadOnly(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		if (anObject == null) {
			return true;

		} else if (anObject instanceof ISpecificFunctionDeclarationWithArgDeclBinds) {
			return ((ISpecificFunctionDeclarationWithArgDeclBinds)anObject)
				.getParametricFunctionCallDeclaration()
				.getFunctionCallDeclaration()
				.isReadOnly(createFunctionArgs);

		} else {
			return ((ISpecificFunctionDeclaration)anObject).isReadOnly(createFunctionArgs);
		}
	}

	public void removeArgumentObject(Object anObject) {
		if ((anObject instanceof ISpecificFunctionDeclarationWithArgDeclBinds)
			&& !(anObject instanceof IUserFunctionDeclaration)) {
			ISpecificFunctionDeclarationWithArgDeclBinds specificFunctionDeclarationWithArgDeclBinds =
				(ISpecificFunctionDeclarationWithArgDeclBinds)anObject;

			if (specificFunctionDeclarationWithArgDeclBinds.canBeRemoved())
				specificFunctionDeclarationWithArgDeclBinds.remove();
		}

		super.removeArgumentObject(anObject);
	}

	public static void setSupportClosures(boolean _supportClosures) {
		AbstractSpecificFunctionType._supportClosures = _supportClosures;
	}

	public static boolean isSupportClosures() {
		return _supportClosures;
	}
}
