package de.uniwue.ki.mas.basics.type;

import de.uniwue.ki.util.Utils;

/**
 * Creation date: 04.09.2002, 18:40:19
 * @author oechslein
 */
public abstract class AbstractSpecificParametricType extends AbstractSpecificType {
	private final String _name;
	private final ISpecificType _type;

	public AbstractSpecificParametricType(String name, ISpecificType type) {
		super();
		_name = name;
		_type = type;
	}

	public String getName() {
		return _name + "<" + _type.getName() + ">";
	}

	/**
	 * Compares two objects for equality. Returns a boolean that indicates
	 * whether this object is equivalent to the specified object. This method
	 * is used when an object is stored in a hashtable.
	 * @param obj the Object to compare with
	 * @return true if these Objects are equal; false otherwise.
	 * @see java.util.Hashtable
	 */
	public boolean equals(Object obj) {
		// NOTE: obj might be an instance of any class
		if (!super.equals(obj)) {
			return false;
		}

		AbstractSpecificParametricType otherSpecificParametricType = (AbstractSpecificParametricType) obj;
		return Utils.equals(
			getParameterSpecificType(),
			otherSpecificParametricType.getParameterSpecificType());
	}

	/**
	 * Generates a hash code for the receiver.
	 * This method is supported primarily for
	 * hash tables, such as those provided in java.util.
	 * @return an integer hash code for the receiver
	 * @see java.util.Hashtable
	 */
	public int hashCode() {
		// NOTE: if two objects are equal (equals(Object) returns true) they must have the same hash code
		return 3 * super.hashCode() + getParameterSpecificType().hashCode();
	}

	public ISpecificType getParameterSpecificType() {
		return _type;
	}
}