package de.uniwue.ki.mas.basics.type;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (05.12.2001 09:31:49)
 * @author Christoph Oechslein
 */

public abstract class AbstractSpecificType extends AbstractType implements ISpecificType {
	private List _subtypes;

	protected AbstractSpecificType() {
		super();
	}

	public void addSubType(ISpecificType newSubType) {
		if (_subtypes == null) {
			_subtypes = new LinkedList();
		}

		if (!_subtypes.contains(newSubType)) {
			_subtypes.add(newSubType);
		}
	}

	public void removeSubType(ISpecificType subType) {
		Assert.assertNotNull(_subtypes);
		Assert.assertTrue(_subtypes.contains(subType));

		_subtypes.remove(subType);
	}

	public Iterator getSubTypes() {
		if (_subtypes == null)
			return CollectionsHelper.EMPTY_ITERATOR;
		else
			return _subtypes.iterator();
	}

	public boolean canCreate(ISpecificType type) {
		return equals(type);
	}

	public IArgumentDeclaration copyArgumentDeclaration(
		IArgumentDeclaration argumentDeclaration,
		Map objectMapping) {

		Assert.assertEquals(argumentDeclaration.getSpecificType(), this);
		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);

		return new ArgumentDeclaration(
			this,
			copyArgumentObject(argumentDeclaration.getObject(), objectMapping));
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		return anObject;
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return new Argument(anObject);
	}

	public IArgumentDeclaration createArgumentDeclaration(Object anObject) {
		if (anObject instanceof Collection)
		try {
			return new ArgumentDeclaration(this, createArgument(null,((Collection)anObject).iterator()));
		} catch (ModelCompilingException e) {
			Assert.fail(e.getMessage());
			return null;
		}
		else
		return new ArgumentDeclaration(this, anObject);
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclaration() {
		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
			null,
			this,
			getArgumentDeclarationDefaultValue());
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}

	public IGenericType getGenericType() {
		return null;
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (anObject == null) {
			// return "<" + getName() + ">";
			// return "";
			buff.append("null");

		} else {
			buff.append(anObject);
		}
	}

	protected final void appendNameForNamedObject(
		StringBuffer buff,
		INamed named,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (named == null) {
			// return "<" + getName() + ">";
			// return "";
			buff.append("null");

		} else {
			named.appendName(buff, currDepth + 1, maxDepth);
		}
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		appendNameForArgumentObject(buff, anObject, currDepth, maxDepth);
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return true;
	}

	public boolean isReadOnly(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return true;
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (equals(type)) {
			return true;

		} else if (_subtypes == null) {
			return false;

		} else {
			Iterator it = getSubTypes();
			while (it.hasNext()) {
				if (((ISpecificType) it.next()).isMoreSpecific(type))
					return true;
			}
			return false;
		}
	}

	public void removeArgumentDeclaration(IArgumentDeclaration argumentDeclaration) {
		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);

		removeArgumentObject(argumentDeclaration.getObject());
	}

	public void removeArgumentObject(Object anObject) {
	}

	public boolean isAtomicDefaultValue() {
		return true;
	}
}