package de.uniwue.ki.mas.basics.type;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.argument.Argument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;

public final class ActivityType extends AbstractSpecificType {
	private static final ActivityType instance = new ActivityType();

	public static final ActivityType getInstance() {
		return instance;
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		return new Argument(anObject);
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}

	public String getName() {
		return "Activity";
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (anObject == null) {
			super.appendNameForArgumentObject(buff, null, currDepth, maxDepth);

		} else {
			IActivityGraphNodeDeclaration activityDeclaration = (IActivityGraphNodeDeclaration)anObject;
			IActivityGraphDeclaration parentActivityGraphDeclaration =
				activityDeclaration.getActivityGraphDeclaration();

			activityDeclaration.appendName(buff, currDepth, maxDepth);

			if (parentActivityGraphDeclaration != null)
				appendParentName(buff, parentActivityGraphDeclaration, currDepth, maxDepth);
		}
	}

	private void appendParentName(
		StringBuffer buff,
		IActivityGraphDeclaration parentActivityGraphDeclaration,
		int currDepth,
		int maxDepth) {

		if (parentActivityGraphDeclaration.getParentActivityGraphComposedNodeDeclaration() != null) {
			buff.append(" [");
			parentActivityGraphDeclaration.appendName(buff, currDepth, maxDepth);
			appendParentName(
				buff,
				parentActivityGraphDeclaration
					.getParentActivityGraphComposedNodeDeclaration()
					.getInnerActivityGraphDeclaration(),
				currDepth + 1,
				maxDepth);
			buff.append("]");
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof IActivityGraphNodeDeclaration);
	}

	protected ActivityType() {
		super();

		ObjectType.getInstance().addSubType(this);
	}
}
