package de.uniwue.ki.mas.basics.type;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

public final class BooleanType extends AbstractSpecificType {
	public static final IArgument BOOLEAN_TYPE_TRUE_ARGUMENT = new AbstractArgument() {
		public String getName() {
			return "true";
		}

		public Object getConstantValue() throws SimulationRuntimeException {
			return Boolean.TRUE;
		}

		public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {
			return true;
		}
	};

	public static final IArgument BOOLEAN_TYPE_FALSE_ARGUMENT = new AbstractArgument() {
		public String getName() {
			return "false";
		}

		public Object getConstantValue() throws SimulationRuntimeException {
			return Boolean.FALSE;
		}

		public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
			throws SimulationRuntimeException {
			return false;
		}
	};

	private static final BooleanType instance = new BooleanType();

	/**
	 * @return BooleanType
	 */
	public static final BooleanType getInstance() {
		return instance;
	}

	/**
	 * @return Object
	 * @see de.uniwue.ki.mas.basics.type.ISpecificType#getDefaultValue()
	 */
	public Object getArgumentDeclarationDefaultValue() {
		return Boolean.TRUE;
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Boolean";
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value instanceof Boolean;
	}

	/**
	 * @param createFunctionArgs ICreateFunctionArgs
	 * @param value Object
	 * @return IArgument
	 * @see de.uniwue.ki.mas.basics.type.ISpecificType#createArgument(ICreateFunctionArgs, Object)
	 */
	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object value) {
		if (((Boolean) value).booleanValue())
			return BOOLEAN_TYPE_TRUE_ARGUMENT;
		else
			return BOOLEAN_TYPE_FALSE_ARGUMENT;
	}

	private BooleanType() {
		super();

		ObjectType.getInstance().addSubType(this);
	}
}
