package de.uniwue.ki.mas.basics.type;
import java.text.FieldPosition;
import java.text.NumberFormat;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;

/**
 * Creation date: (30.08.2000 09:55:35)
 * @author Christoph Oechslein
 */

public final class DoubleType extends AbstractSpecificType {
	private static final DoubleType instance = new DoubleType();
	private static final class NumberArgument extends AbstractArgument implements IArgument, IFunction {
		private Double value;

		public NumberArgument(double value) {
			this(new Double(value));
		}

		public NumberArgument(Double value) {
			super();
			this.value = value;
		}

		public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs) {
			return value.doubleValue();
		}

		public Object getConstantValue() {
			return value;
		}

		public String getName() {
			return value + "";
		}
	}

	protected DoubleType() {
		super();

		ObjectType.getInstance().addSubType(this);
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return new NumberArgument(((Number) anObject).doubleValue());
	}

	public Object getArgumentDeclarationDefaultValue() {
		return new Double(0.0);
	}

	public static final DoubleType getInstance() {
		return instance;
	}

	public IGenericType getGenericType() {
		return GenericNumberType.getInstance();
	}

	public String getName() {
		return "Number<Double>";
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value instanceof Number;
	}

	private static final NumberFormat _numberFormat;

	static {
		_numberFormat = (NumberFormat) NumberFormat.getNumberInstance().clone();
		_numberFormat.setGroupingUsed(false);
		_numberFormat.setMaximumFractionDigits(Integer.MAX_VALUE);
		_numberFormat.setMaximumIntegerDigits(Integer.MAX_VALUE);
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			if (anObject instanceof Number)
				_numberFormat.format(anObject, buff, new FieldPosition(0));
			else
				buff.append(anObject);
		}
	}
}