package de.uniwue.ki.mas.basics.type;
import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.util.Pair;
import de.uniwue.ki.util.collection.CollectionsHelper;

public class GenericFunctionType extends AbstractGenericType {
	private static final GenericFunctionType _instance = new GenericFunctionType();

	public static final GenericFunctionType getInstance() {
		return _instance;
	}

	private transient final Map _cache = new WeakHashMap();

	public GenericFunctionType() {
		super();
	}

	public boolean canCreate(ISpecificType outputType) {
		if (!(outputType instanceof ISpecificFunctionType)) {
			return false;
		}

		ISpecificFunctionType specificFunctionType = (ISpecificFunctionType) outputType;

		if (specificFunctionType.getSpecificOutputType() instanceof ISpecificFunctionType) {
			return false;
		}

		if (specificFunctionType.getSpecificInputTypesLength()
			== ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return !(specificFunctionType.getSpecificInputTypeAt(0) instanceof ISpecificFunctionType);

		} else {
			for (int i = 0; i < specificFunctionType.getSpecificInputTypesLength(); i++) {
				if (specificFunctionType.getSpecificInputTypeAt(i) instanceof ISpecificFunctionType)
					return false;
			}
			return true;
		}
	}

	public ISpecificType createDefaultSpecificType() {
		return createSpecificFunctionType(DoubleType.getInstance(), CollectionsHelper.EMPTY_LIST);
	}

	public SpecificFunctionUnlimitedType createSpecificFunctionType(
		ISpecificType outputType,
		ISpecificType inputType) {
		Pair signature = new Pair(outputType, inputType);

		if (_cache.containsKey(signature)) {
			return (SpecificFunctionUnlimitedType) _cache.get(signature);
		} else {
			SpecificFunctionUnlimitedType specificFunctionType =
				new SpecificFunctionUnlimitedType(outputType, inputType);
			_cache.put(signature, specificFunctionType);
			return specificFunctionType;
		}
	}
	
	public ISpecificFunctionType createSpecificFunctionType(ISpecificType outputType, List inputTypes) {
		Pair signature = new Pair(outputType, inputTypes);

		if (_cache.containsKey(signature)) {
			return (SpecificFunctionType) _cache.get(signature);
		} else {
			SpecificFunctionType specificFunctionType = new SpecificFunctionType(outputType, inputTypes);
			_cache.put(signature, specificFunctionType);
			return specificFunctionType;
		}
	}

	public String getName() {
		return "Function<(T1, ...) T2>";
	}

}