package de.uniwue.ki.mas.basics.type;

import java.util.WeakHashMap;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;

/**
 * @author oechslein
 * 
 * Creation date: 02.07.2003
 */
public final class GenericSimObjectType extends AbstractGenericType {
	private static final GenericSimObjectType _instance = new GenericSimObjectType();

	private static final SimObjectType _generalSimObjectType = new SimObjectType();
	private static transient WeakHashMap _hashMap;

	public static final ISimObjectType getGeneralSimObjectType() {
		return _generalSimObjectType;
	}

	private static synchronized WeakHashMap getHashMap() {
		if (_hashMap == null) {
			_hashMap = new WeakHashMap();
		}
		return _hashMap;
	}

	public static GenericSimObjectType getInstance() {
		return _instance;
	}

	public static final ISimObjectType getSimObjectType(IFeatureClassDeclaration featureClassDeclaration) {
		if (getHashMap().containsKey(featureClassDeclaration)) {
			return (ISimObjectType)getHashMap().get(featureClassDeclaration);
		} else {
			SimObjectType newSimObjectType = new SimObjectType(featureClassDeclaration);
			getHashMap().put(featureClassDeclaration, newSimObjectType);
			return newSimObjectType;
		}
	}

	public static final ISimObjectType getSimObjectType(IObjectClassDeclaration objectClassDeclaration) {
		if (getHashMap().containsKey(objectClassDeclaration)) {
			return (ISimObjectType)getHashMap().get(objectClassDeclaration);
		} else {
			SimObjectType newSimObjectType = new SimObjectType(objectClassDeclaration);
			getHashMap().put(objectClassDeclaration, newSimObjectType);
			return newSimObjectType;
		}
	}

	private GenericSimObjectType() {
	}

	public boolean canCreate(ISpecificType type) {
		return type instanceof ISimObjectType;
	}

	public ISpecificType createDefaultSpecificType() {
		return getGeneralSimObjectType();
	}

	public ISpecificType createSpecificType(IFeatureClassDeclaration featureClassDeclaration) {
		return getSimObjectType(featureClassDeclaration);
	}

	public ISimObjectType createSpecificType(IObjectClassDeclaration objectClassDeclaration) {
		return getSimObjectType(objectClassDeclaration);
	}

	public String getName() {
		return "SimObject";
	}
}
