package de.uniwue.ki.mas.basics.type;

import java.util.HashMap;
import java.util.Map;

import junit.framework.Assert;
/**
 * Creation date: 04.09.2002, 18:40:19
 * @author oechslein
 */public class GenericVariableClassDeclarationType extends AbstractGenericType {

	private static final GenericVariableClassDeclarationType[] _instances =
		{
			new GenericVariableClassDeclarationType(),
			new GenericVariableClassDeclarationType(false, false),
			new GenericVariableClassDeclarationType(false, true),
			new GenericVariableClassDeclarationType(true, false),
			new GenericVariableClassDeclarationType(true, true),
			};

	public static final GenericVariableClassDeclarationType getInstance() {
		return _instances[0];
	}

	public static final GenericVariableClassDeclarationType getInstance(
		boolean external,
		boolean writeable) {

		GenericVariableClassDeclarationType genericVariableClassDeclarationType =
			_instances[(external ? 2 : 0) + (writeable ? 1 : 0) + 1];

		Assert.assertEquals(genericVariableClassDeclarationType.isExternal(), external);
		Assert.assertEquals(genericVariableClassDeclarationType.isWriteable(), writeable);

		return genericVariableClassDeclarationType;
	}
	private final boolean _external;/* Are external and writeable important in this instance (true if not). */private final boolean _realGenericVariable;
	private final boolean _writeable;
	private final Map _cache = new HashMap();
	
	public GenericVariableClassDeclarationType() {
		super();
	
		_realGenericVariable = true;
		_external = true;
		_writeable = true;
	}
	
	public GenericVariableClassDeclarationType(boolean external, boolean writeable) {
		super();
	
		_external = external;
		_writeable = writeable;
		_realGenericVariable = false;
	}
	
	public boolean canCreate(ISpecificType type) {// old definition isn't good since getInstance(true,true).createSpecificType(type) != getInstance(false,true).createSpecificType(type)
	// return type instanceof SpecificVariableClassDeclarationType;
	if (!(type instanceof SpecificVariableClassDeclarationType))
		return false;
	
	SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
		(SpecificVariableClassDeclarationType)type;
	
	if (_realGenericVariable == true) {
		return true;
	} else {
		return (specificVariableClassDeclarationType.isExternal() == isExternal())
			&& (specificVariableClassDeclarationType.isWriteable() == isWriteable());
	}
	}
	
	public ISpecificType createDefaultSpecificType() {
		return createSpecificVariableType(DoubleType.getInstance());
	}
	
	public ISpecificType createSpecificVariableType(ISpecificType type) {
		SpecificVariableClassDeclarationType variableType;
		if (_cache.containsKey(type)) {
			variableType = (SpecificVariableClassDeclarationType)_cache.get(type);
		} else {
			variableType = new SpecificVariableClassDeclarationType(type, isExternal(), isWriteable());
			_cache.put(type, variableType);
		}
	
		Assert.assertEquals(type, variableType.getVariableClassDeclarationType());
	
		return variableType;
	
	}
	
	public boolean equals(Object obj) {
		if (!super.equals(obj))
			return false;
	
		if (!(obj instanceof GenericVariableClassDeclarationType))
			return false;
	
		GenericVariableClassDeclarationType genericVariableClassDeclarationType =
			(GenericVariableClassDeclarationType)obj;
	
		if (_realGenericVariable)
			return genericVariableClassDeclarationType._realGenericVariable == true;
		else
			return genericVariableClassDeclarationType.isExternal() == isExternal()
				&& genericVariableClassDeclarationType.isWriteable() == isWriteable();
	}
	
	public int hashCode() {
		if (_realGenericVariable)
			return super.hashCode();
		else
			return 3 * super.hashCode() + (isExternal() ? 0 : 1) + (isWriteable() ? 3 : 5);
	}
	
	public String getName() {
		return"Variable<T,...>";
	}
	
	public final boolean isExternal() {
		return _external;
	}
	
	public final boolean isWriteable() {
		return _writeable;
	}
}