package de.uniwue.ki.mas.basics.type;
import java.text.FieldPosition;
import java.text.NumberFormat;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;

/**
 * Creation date: (30.08.2000 09:55:35)
 * @author Christoph Oechslein
 */

public final class IntegerType extends AbstractSpecificType {
	private static final IntegerType instance = new IntegerType();
	private static final class NumberArgument extends AbstractArgument implements IArgument, IFunction {
		private Integer value;

		public NumberArgument(int value) {
			this(new Integer(value));
		}

		public NumberArgument(Integer value) {
			super();

			this.value = value;
		}

		public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs) {
			return value.doubleValue();
		}

		public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) {
			return value.intValue();
		}

		public Object getConstantValue() {
			return value;
		}

		public String getName() {
			return value + "";
		}
	}

	protected IntegerType() {
		super();

		DoubleType.getInstance().addSubType(this);
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return new NumberArgument(((Number) anObject).intValue());
	}

	public Object getArgumentDeclarationDefaultValue() {
		return new Integer(0);
	}

	public static final IntegerType getInstance() {
		return instance;
	}

	public IGenericType getGenericType() {
		return GenericNumberType.getInstance();
	}

	public String getName() {
		return "Number<Integer>";
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return value instanceof Number;
	}

	private static final NumberFormat _numberFormat;

	static {
		_numberFormat = (NumberFormat) NumberFormat.getIntegerInstance().clone();
		_numberFormat.setGroupingUsed(false);
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			if (anObject instanceof Number)
				_numberFormat.format(anObject, buff, new FieldPosition(0));
			else
				buff.append(anObject);
		}
	}
}