package de.uniwue.ki.mas.basics.type;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.mas.entity.object.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (05.11.2001 16:39:33)
 * @author Christoph Oechslein
 */

public final class ObjectInstanceDeclarationType extends AbstractSpecificType {
	private static final ObjectInstanceDeclarationType _instance = new ObjectInstanceDeclarationType();

	private static final class ObjectInstanceArgumentDeclaration
		extends ArgumentDeclarationWithNeverRemoveVetoListener {
		public ObjectInstanceArgumentDeclaration(IObjectInstanceDeclaration objectInstanceDeclaration) {
			super(ObjectInstanceDeclarationType.getInstance(), objectInstanceDeclaration);
		}

		public final void setDeclaration(IDeclaration newDeclaration) {
			IDeclaration oldDeclaration = getDeclaration();

			super.setDeclaration(newDeclaration);

			if (oldDeclaration != null) {
				if (oldDeclaration.canBeRemoved())
					oldDeclaration.remove();
			}
		}
	}

	private ObjectInstanceDeclarationType() {
		super();

		ObjectType.getInstance().addSubType(this);
	}

	public IArgumentDeclaration copyArgumentDeclaration(
		IArgumentDeclaration argumentDeclaration,
		Map objectMapping) {

		Assert.assertEquals(argumentDeclaration.getSpecificType(), this);
		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);

		return new ObjectInstanceArgumentDeclaration(
			copyObjectInstanceDeclaration(
				(IObjectInstanceDeclaration)argumentDeclaration.getObject(),
				objectMapping));
	}

	private IObjectInstanceDeclaration copyObjectInstanceDeclaration(
		IObjectInstanceDeclaration objectInstanceDeclaration,
		Map objectMapping) {

		if (objectInstanceDeclaration == null)
			return null;
		else
			return (IObjectInstanceDeclaration)objectInstanceDeclaration.copy(objectMapping);
	}

	public IArgumentDeclaration createArgumentDeclaration(Object anObject) {
		return new ObjectInstanceArgumentDeclaration((IObjectInstanceDeclaration)anObject);
	}

	public static final ObjectInstanceDeclarationType getInstance() {
		return _instance;
	}

	public String getName() {
		return "Object Instance";
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			appendNameForNamedObject(buff, (IObjectInstanceDeclaration)anObject, currDepth, maxDepth);
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return ((value == null) || (value instanceof IObjectInstanceDeclaration));
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		return new ArgumentWithBindings(anObject);
	}

	public static IObject createObject(
		IFunction objectInstanceDeclarationFunction,
		IObjectExecuteFunctionArgs objectExecuteFunctionArgs)
		throws ModelCompilingException, SimulationRuntimeException {

		ArgumentWithBindings.ResultWithBindings resultWithBindings =
			(ArgumentWithBindings.ResultWithBindings)objectInstanceDeclarationFunction.execute(
				objectExecuteFunctionArgs);

		IArgumentBindings bindings = resultWithBindings.getSavedArgumentBindings();

		objectExecuteFunctionArgs.pushArgumentBindings(bindings);

		IObjectInstanceDeclaration objectInstanceDeclaration =
			(IObjectInstanceDeclaration)resultWithBindings.getValue();

		IObjectCreateFunctionArgs objectCreateFunctionArgs =
			(IObjectCreateFunctionArgs)objectExecuteFunctionArgs.createCreateAndExecuteFunctionArgs();

		IObject newObject = objectInstanceDeclaration.createObject(objectCreateFunctionArgs);
		newObject.initialize(objectCreateFunctionArgs);

		objectExecuteFunctionArgs.popArgumentBindings();

		return newObject;
	}

}