package de.uniwue.ki.mas.basics.type;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;

/**
 * Typ reprsentiert eine Instanz einer ObjektInstanzklasse, 
 * entweder IObjectInstanceDeclaration oder IObject (zur Laufzeit).
 * Zustzlich kann der genaue Typ noch angegeben werden.
 */

final class SimObjectType extends AbstractSimObjectType {

	private final IFeatureClassDeclaration _featureClassDeclaration;
	private final IObjectClassDeclaration _objectClassDeclaration;

	public SimObjectType() {
		super();

		ObjectType.getInstance().addSubType(this);
		_objectClassDeclaration = null;
		_featureClassDeclaration = null;
	}

	public SimObjectType(IFeatureClassDeclaration featureClassDeclaration) {
		super();

		_featureClassDeclaration = featureClassDeclaration;
		_objectClassDeclaration = null;
	}

	public SimObjectType(IObjectClassDeclaration objectClassDeclaration) {
		super();

		_objectClassDeclaration = objectClassDeclaration;
		_featureClassDeclaration = null;
	}

	public IFeatureClassDeclaration getFeatureClassDeclaration() {
		return _featureClassDeclaration;
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	/**
	 * @deprecated use GenericSimObjectType#getGeneralSimObjectType 
	 * and the interface ISimObjectType instead
	 */
	//TODO remove deprecated method
	public static final SimObjectType getInstance() {
		return (SimObjectType)GenericSimObjectType.getGeneralSimObjectType();
	}
}