package de.uniwue.ki.mas.basics.type;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.experiment.IExperimentExecuteFunctionArgs;
import de.uniwue.ki.mas.sim.simulation.*;

/**
 * Creation date: (28.01.2002 19:42:38)
 * @author Christoph Oechslein
 */
public final class SimulationDeclarationType extends AbstractSpecificType {
	private static final SimulationDeclarationType _instance = new SimulationDeclarationType();

	private SimulationDeclarationType() {
		super();
	}

	public static final SimulationDeclarationType getInstance() {
		return _instance;
	}

	public String getName() {
		return "Simulation Declaration";
	}

	public IArgumentDeclaration copyArgumentDeclaration(
		IArgumentDeclaration argumentDeclaration,
		Map objectMapping) {

		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);
		Assert.assertEquals(argumentDeclaration.getSpecificType(), this);

		ISimulationDeclaration simulationDecl = (ISimulationDeclaration)argumentDeclaration.getObject();

		if (simulationDecl instanceof ISimulationWithNameFunctionCallDeclDeclaration)
			return new ArgumentDeclarationWithNeverRemoveVetoListener(
				this,
				((ISimulationDeclaration)simulationDecl.copy(objectMapping)));
		else
			return new ArgumentDeclarationWithNeverRemoveVetoListener(this, simulationDecl);
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			appendNameForNamedObject(buff, (ISimulationDeclaration)anObject, currDepth + 1, maxDepth);
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof ISimulationDeclaration);
	}

	public static ISimulation createSimulation(
		IFunction simulationDeclarationFunction,
		IExperimentExecuteFunctionArgs experimentExecuteFunctionArgs)
		throws ModelCompilingException, SimulationRuntimeException {

		ArgumentWithBindings.ResultWithBindings resultWithBindings =
			(ArgumentWithBindings.ResultWithBindings)simulationDeclarationFunction.execute(
				experimentExecuteFunctionArgs);

		IArgumentBindings bindings = resultWithBindings.getSavedArgumentBindings();

		experimentExecuteFunctionArgs.pushArgumentBindings(bindings);

		ISimulation newSimulation =
			experimentExecuteFunctionArgs.getExperiment().createSimulation(
				(ISimulationDeclaration)resultWithBindings.getValue());

		experimentExecuteFunctionArgs.popArgumentBindings();

		return newSimulation;
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		return new ArgumentWithBindings(anObject);
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {
		return false;
	}
}