package de.uniwue.ki.mas.basics.type;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;

/**
 * Creation date: (17.09.2001 17:25:39)
 * @author Christoph Oechslein
 */

public final class SpecificFunctionUnlimitedType extends AbstractSpecificFunctionType {

	private final ISpecificType _inputType;

	SpecificFunctionUnlimitedType(ISpecificType outputType, ISpecificType inputType) {
		super(outputType);

		_inputType = inputType;
	}

	public boolean canCreate(ISpecificType type) {
		return equals(type);
	}

	/**
	 * Compares two objects for equality. Returns a boolean that indicates
	 * whether this object is equivalent to the specified object. This method
	 * is used when an object is stored in a hashtable.
	 * @param obj the Object to compare with
	 * @return true if these Objects are equal; false otherwise.
	 * @see java.util.Hashtable
	 */
	public boolean equals(Object obj) {
		// NOTE: obj might be an instance of any class
		if (!(obj instanceof SpecificFunctionUnlimitedType)) {
			return false;
		}

		SpecificFunctionUnlimitedType otherSpecificFunctionType = (SpecificFunctionUnlimitedType)obj;
		return getSpecificOutputType().equals(otherSpecificFunctionType.getSpecificOutputType())
			& getSpecificInputType().equals(otherSpecificFunctionType.getSpecificInputType());
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (super.isMoreSpecific(type)) {
			return true;

		} else if (!(type instanceof SpecificFunctionUnlimitedType)) {
			return false;

		} else {
			SpecificFunctionUnlimitedType otherSpecificFunctionType = (SpecificFunctionUnlimitedType)type;
			return getSpecificOutputType().isMoreSpecific(otherSpecificFunctionType.getSpecificOutputType())
				& getSpecificInputType().isMoreSpecific(otherSpecificFunctionType.getSpecificInputType());
		}
	}

	public boolean isAtomicDefaultValue() {
		return false;
	}

	private ISpecificFunctionDeclaration _argumentDeclarationDefaultValue = null;

	public Object getArgumentDeclarationDefaultValue() {
		if (_argumentDeclarationDefaultValue == null) {
			_argumentDeclarationDefaultValue = getArgumentDeclarationDefaultValueIntern();
		}

		return _argumentDeclarationDefaultValue;
	}

	// use above since we cannot edit unlimited arg functions

	//	public Object getArgumentDeclarationDefaultValue() {
	//		SpecificFunctionDeclarationWithArgDeclBinds specificFunctionDeclarationWithArgDeclBinds =
	//			SpecificFunctionDeclarationWithArgDeclBindsFactory
	//				.createSpecificFunctionDeclarationWithArgDeclBinds();
	//
	//		specificFunctionDeclarationWithArgDeclBinds.setSpecificOutputType(getSpecificOutputType());
	//
	//		for (int i = 0; i < 0; i++) {
	//			specificFunctionDeclarationWithArgDeclBinds.getFunctionArgumentDeclarationList().add(
	//				FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
	//					getSpecificInputType()));
	//		}
	//
	//		return specificFunctionDeclarationWithArgDeclBinds;
	//	}

	private ISpecificFunctionDeclaration getArgumentDeclarationDefaultValueIntern() {
		Iterator it =
			FunctionDeclarationListManager.getAllFunctionDeclarationList().getFunctionDeclarations(
				getSpecificOutputType());
		while (it.hasNext()) {
			IFunctionDeclaration currFunctionDeclaration = (IFunctionDeclaration)it.next();

			if (currFunctionDeclaration instanceof ISpecificFunctionDeclaration) {
				if (hasSameSignature((ISpecificFunctionDeclaration)currFunctionDeclaration)) {
					return (ISpecificFunctionDeclaration)currFunctionDeclaration;
				}
			}
		}

		return null;
	}

	public String getName() {
		return "Fun<("
			+ getSpecificInputType().getName()
			+ " ...) "
			+ getSpecificOutputType().getName()
			+ ">";
	}

	public final ISpecificType getSpecificInputType() {
		return _inputType;
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		return getSpecificInputType();
	}

	public int getSpecificInputTypesLength() {
		return ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT;
	}

	/**
	 * Generates a hash code for the receiver.
	 * This method is supported primarily for
	 * hash tables, such as those provided in java.util.
	 * @return an integer hash code for the receiver
	 * @see java.util.Hashtable
	 */
	public int hashCode() {
		// NOTE: if two objects are equal (equals(Object) returns true) they must have the same hash code
		return 7 * getSpecificOutputType().hashCode() + getSpecificInputType().hashCode();
	}

	public boolean hasSameSignature(ISpecificFunctionDeclaration specificFunctionDeclaration) {
		// should be the same as equals(specificFunctionDeclaration.getSpecificFunctionType());

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(getSpecificOutputType(), specificFunctionDeclaration.getSpecificOutputType())) {
			return false;
		}

		if (specificFunctionDeclaration.getSpecificInputMaxArgsLength()
			!= ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
			return false;
		}

		ISpecificType specificFunctionDeclarationSpecificInputType =
			specificFunctionDeclaration.getSpecificInputTypeAt(
				specificFunctionDeclaration.getSpecificInputDefaultArgsLength());

		return TypeManager.getInstance().isMoreGeneral(
			getSpecificInputType(),
			specificFunctionDeclarationSpecificInputType);
	}

	public boolean isSimpleType() {
		return false;
	}
}