package de.uniwue.ki.mas.basics.type;
import java.util.HashMap;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.hashtable.FunctionDeclarationHashTableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

public final class SpecificHashtableType extends AbstractSpecificParametricType {
	private final ISpecificType _valueType;

	public SpecificHashtableType(ISpecificType keyType, ISpecificType valueType) {
		super("HashTable", keyType);

		_valueType = valueType;
		
		Assert.assertNotNull(keyType);
		Assert.assertNotNull(valueType);
	}

	public final ISpecificType getValueType() {
		return _valueType;
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		if (anObject instanceof Map) {
			return super.createArgument(createFunctionArgs, new HashMap((Map) anObject));

		} else {
			Assert.assertNull(anObject);
			return super.createArgument(createFunctionArgs, null);
		}
	}

	public boolean equals(Object obj) {
		return super.equals(obj)
			&& (obj instanceof SpecificHashtableType)
			&& ((SpecificHashtableType) obj).getValueType().equals(getValueType());
	}

	public int hashCode() {
		return 3 * super.hashCode() + getValueType().hashCode();
	}
	
	public IGenericType getGenericType() {
		return GenericHashtableType.getInstance();
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
			
		} else if (anObject == null) {
			buff.append("null");
			
		} else {
			super.appendNameForArgumentObject(buff, anObject, currDepth, maxDepth);
		}
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (type instanceof SpecificHashtableType) {
			SpecificHashtableType specificHashtableType = (SpecificHashtableType) type;

			if (!getKeyType().isMoreSpecific(specificHashtableType.getKeyType())) {
				return false;
			}

			if (!getValueType().isMoreSpecific(specificHashtableType.getValueType())) {
				return false;
			}

			return true;

		} else {
			return false;
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof Map);
	}

	public final ISpecificType getKeyType() {
		return getParameterSpecificType();
	}

	public String getName() {
		return "HashTable<" + getKeyType().getName() + ", " + getValueType().getName() + ">";
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return anObject == null;
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclaration() {
		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			null,
			FunctionDeclarationHashTableFactory.CREATE_HASHTABLE_GENERIC_FUN_DECL.createFunctionDeclaration(
				this,
				CollectionsHelper.EMPTY_ITERATOR),
			CollectionsHelper.EMPTY_ITERATOR);
	}

	public boolean isAtomicDefaultValue() {
		return true;
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}
	
	public static void main(String[] arguments) {
		ISpecificType type = GenericHashtableType.getInstance().createDefaultSpecificType();
		
		IFunctionCallInternDeclaration f1 = type.createDefaultFunctionCallInternDeclaration();
		IFunctionCallInternDeclaration f2 = type.createDefaultFunctionCallInternDeclaration();
		
		Assert.assertEquals(f1,f2);
	}
}