package de.uniwue.ki.mas.basics.type;
import java.util.*;

import de.uniwue.ki.mas.basics.function.ExecuteIteratorFactory;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.argument.AbstractArgument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (29.08.2000 18:07:27)
 * @author Christoph Oechslein
 */

public class SpecificIteratorType extends AbstractSpecificParametricType {
	static final class EmptyIteratorArgument extends AbstractArgument {
		public Object getConstantValue() throws SimulationRuntimeException {
			return ExecuteIteratorFactory.EMPTY_EXECUTE_ITERATOR;
		}

		public final String getName() {
			return "[]";
		}
	}

	public static final EmptyIteratorArgument EMPTY_ITERATOR_ARGUMENT = new EmptyIteratorArgument();

	// changed to return same iterator 
	//	private static final class IteratorArgument extends AbstractArgument {
	//		private Object _iteratorOrList;
	//
	//		public IteratorArgument(Iterator it) {
	//			_iteratorOrList = it;
	//		}
	//
	//		public Object getConstantValue() throws SimulationRuntimeException {
	//			List list = getConstantList();
	//
	//			return ExecuteIteratorFactory.createExecuteIterator(list);
	//		}
	//
	//		private List getConstantList() throws SimulationRuntimeException {
	//			if (_iteratorOrList instanceof List)
	//				return (List) _iteratorOrList;
	//
	//			else {
	//				List list = ExecuteIteratorFactory.createList(_iteratorOrList);
	//				_iteratorOrList = list;
	//				return list;
	//			}
	//		}
	//
	//		protected String getNameWithException() throws SimulationRuntimeException {
	//			return getConstantList().toString();
	//		}
	//	}
	private static final class IteratorArgument extends AbstractArgument {
		private Iterator _iterator;

		public IteratorArgument(Iterator it) {
			_iterator = it;
		}

		public Object getConstantValue() throws SimulationRuntimeException {
			return _iterator;
		}

		protected String getNameWithException() throws SimulationRuntimeException {
			return "[...]";
		}
	}
	SpecificIteratorType(ISpecificType type) {
		this("Iterator", type);

		addSubType(GenericListType.getInstance().createSpecificListType(type));
		ObjectType.getInstance().addSubType(this);
	}

	protected SpecificIteratorType(String name, ISpecificType type) {
		super(name, type);
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		Iterator it;
		if (anObject instanceof List) {
			it = ((List) anObject).iterator();
		} else {
			it = (Iterator) anObject;
		}

		if (CollectionsHelper.isEmpty(it))
			return EMPTY_ITERATOR_ARGUMENT;
		else
			return new IteratorArgument(it);
	}

	public Object getArgumentDeclarationDefaultValue() {
		return ExecuteIteratorFactory.EMPTY_EXECUTE_ITERATOR;
	}

	public IGenericType getGenericType() {
		return GenericIteratorType.getInstance();
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (anObject == null) {
			buff.append("null");

		} else if (anObject instanceof List) {
			buff.append(anObject);

		} else if (CollectionsHelper.isEmpty((Iterator) anObject)) {
			buff.append("[]");

		} else {
			buff.append("[...]");
		}
	}

	public final ISpecificType getIteratorSpecificType() {
		return getParameterSpecificType();
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (type instanceof SpecificIteratorType) {
			SpecificIteratorType specificSetType = (SpecificIteratorType) type;
			return (getIteratorSpecificType().isMoreSpecific(specificSetType.getIteratorSpecificType()));

		} else {
			return false;
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return ((value instanceof IteratorArgument) || (value == getArgumentDeclarationDefaultValue()));
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		if (anObject == ExecuteIteratorFactory.EMPTY_EXECUTE_ITERATOR)
			return ExecuteIteratorFactory.EMPTY_EXECUTE_ITERATOR;
		else
			throw new UnsupportedOperationException();
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		return anObject == null;
	}

}