package de.uniwue.ki.mas.basics.type;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.argument.Argument;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (29.08.2000 18:07:27)
 * @author Christoph Oechslein
 */

public final class SpecificListType extends SpecificIteratorType {
	SpecificListType(ISpecificType type) {
		super("List", type);

		ObjectType.getInstance().addSubType(this);
	}

	public static final List createNewList() {
		return new ArrayList();
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}

	public IGenericType getGenericType() {
		return GenericListType.getInstance();
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if ((anObject == null) || (anObject instanceof Iterator)) {
			super.appendNameForArgumentObject(buff, anObject, currDepth, maxDepth);

		} else {
			Assert.assertTrue(anObject instanceof List);

			buff.append("[");

			Iterator it = ((List) anObject).iterator();
			while (it.hasNext()) {
				getListSpecificType().appendNameForArgumentObject(buff, it.next(), currDepth + 1, maxDepth);

				if (it.hasNext())
					buff.append(", ");
			}

			buff.append("]");
		}
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
			
		if (anObject instanceof Argument) anObject=((Argument)anObject).getConstantValue();

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if ((anObject == null) || (anObject instanceof Iterator)) {
			super.appendNameForArgumentObject(buff, anObject, currDepth, maxDepth);

		} else {
			Assert.assertTrue(anObject instanceof List);

			buff.append("[");

			Iterator it = ((List) anObject).iterator();
			while (it.hasNext()) {
				getListSpecificType().appendNameForArgumentDeclarationObject(
					buff,
					it.next(),
					currDepth,
					maxDepth);

				if (it.hasNext())
					buff.append(", ");
			}

			buff.append("]");
		}
	}

	public final ISpecificType getListSpecificType() {
		return getParameterSpecificType();
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (type instanceof SpecificListType) {
			SpecificListType specificSetType = (SpecificListType) type;
			return (getListSpecificType().isMoreSpecific(specificSetType.getListSpecificType()));

		} else if (type instanceof SpecificIteratorType) {
			//			SpecificIteratorType specificSetType = (SpecificIteratorType) type;
			//			return (getListSpecificType().isMoreSpecific(specificSetType.getIteratorSpecificType()));
			return false;

		} else {
			return false;
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		if (value instanceof Argument) value = ((Argument)value).getConstantValue();
		return (value instanceof List) || (value instanceof Iterator) || (value == null);
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		if (anObject == null) {
			return null;

		} else {
			List newList = new LinkedList();
			if (anObject instanceof List)
				newList.addAll(((List) anObject));
			else
				CollectionsHelper.addAll(newList, (Iterator) anObject);

			return newList;
		}
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object value)
		throws ModelCompilingException {

		if (value == null) {
			return new Argument(null);
			
		} else {
			List newList = createNewList();
			
			if (value instanceof Argument) 
			    value=((Argument)value).getConstantValue();

			if (value instanceof Iterator) {
				CollectionsHelper.addAll(newList, (Iterator) value);
			} else {
				newList.addAll((List) value);
			}

			return new Argument(newList);
		}
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclaration() {
		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			null,
			FunctionDeclarationListFunctionsFactory
				.CREATE_LIST_GENERIC_FUNCTION_DECL
				.createFunctionDeclaration(
				this,
				CollectionsHelper.EMPTY_ITERATOR),
			CollectionsHelper.EMPTY_ITERATOR);
	}

	public boolean isAtomicDefaultValue() {
		return true;
	}
}