package de.uniwue.ki.mas.basics.type;
import java.util.Iterator;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.argument.*;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.entity.variable.VariableCompileException;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.name.INamed;

/**
 * The type stands for IVariableClassDeclaration or IBodyVariableClassDeclaration (if isExternal() is false)
 * Creation date: (29.08.2000 18:07:27)
 * @author Christoph Oechslein
 */

public final class SpecificVariableClassDeclarationType extends AbstractSpecificParametricType {
	private static final String NAME = "Variable";
	private final boolean _external;
	private final boolean _writeable;

	public SpecificVariableClassDeclarationType(ISpecificType type, boolean external, boolean writeable) {
		super(NAME, type);

		_external = external;
		_writeable = writeable;
	}

	private void checkType(IVariableClassDeclaration variableClassDeclaration)
		throws ModelCompilingException {

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(getVariableClassDeclarationType(), variableClassDeclaration.getSpecificType())) {
			throw new VariableCompileException(
				"Wrong type for Variable: "
					+ variableClassDeclaration.getName()
					+ ". Expected '"
					+ getVariableClassDeclarationType().getName()
					+ "' got '"
					+ variableClassDeclaration.getSpecificType().getName()
					+ ".");
		}
	}

	public IArgumentDeclaration copyArgumentDeclaration(
		IArgumentDeclaration argumentDeclaration,
		Map objectMapping) {

		Assert.assertFalse(argumentDeclaration instanceof IFunctionArgumentDeclaration);
		Assert.assertEquals(argumentDeclaration.getSpecificType(), this);

		return new ArgumentDeclarationWithNeverRemoveVetoListener(
			this,
			(IDeclaration) argumentDeclaration.getObject());
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		IObject simObject = ((IObjectCreateFunctionArgs) createFunctionArgs).getObject();

		if (anObject == null) {
			throw new VariableCompileException("Variable is null");

		} else if (!isExternal()) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration = null;

			if (anObject instanceof IFeatureVariableClassDeclaration) {
				bodyVariableClassDeclaration =
					searchBodyVariableClassDeclaration(
						simObject,
						(IFeatureVariableClassDeclaration) anObject);

			} else if (anObject instanceof IBodyVariableClassDeclaration) {
				bodyVariableClassDeclaration = (IBodyVariableClassDeclaration) anObject;

			} else {
				Assert.fail("Variable Error: " + anObject);
			}

			if (bodyVariableClassDeclaration == null) {
				throw new VariableCompileException("Could not find variable :" + anObject);
			}

			if (Assert.ASSERTION_ON)
				checkType(bodyVariableClassDeclaration.getVariableClassDeclaration());

			if (bodyVariableClassDeclaration instanceof IBodyFeatureVariableClassDeclaration) {
				return super.createArgument(
					createFunctionArgs,
					bodyVariableClassDeclaration.getVariableClassDeclaration());

			} else {
				Assert.assertTrue(
					simObject
						.getObjectInstanceDeclaration()
						.getObjectClassDeclaration()
						.getBodyClassDeclaration()
						.getBodyVariableClassDeclarationListAndFactory()
						.getBodyVariableClassDeclarationList()
						.contains(bodyVariableClassDeclaration));

				return super.createArgument(
					createFunctionArgs,
					new Integer(getIndexOf(simObject, bodyVariableClassDeclaration)));
			}

		} else {
			if (anObject instanceof IVariableClassDeclaration) {
				IVariableClassDeclaration variableClassDeclaration = (IVariableClassDeclaration) anObject;

				if (Assert.ASSERTION_ON)
					checkType(variableClassDeclaration);

				return super.createArgument(createFunctionArgs, variableClassDeclaration);

			} else {
				// We know the bodyVariableClassDeclaration, but how to find the correct index ?!
				// so just use the varibaleClassDeclaration and find the index at runtime
				IBodyVariableClassDeclaration bodyVariableClassDeclaration =
					(IBodyVariableClassDeclaration) anObject;

				if (Assert.ASSERTION_ON)
					checkType(bodyVariableClassDeclaration.getVariableClassDeclaration());

				// that was wrong X-) 
				// return super.createArgument(simObject, new Integer(getIndexOf(simObject,)));
				return super.createArgument(
					createFunctionArgs,
					bodyVariableClassDeclaration.getVariableClassDeclaration());
			}
		}
	}

	public IArgumentDeclaration createArgumentDeclaration(Object anObject) {
		return new ArgumentDeclarationWithNeverRemoveVetoListener(this, (IDeclaration) anObject);
	}

	public boolean equals(Object obj) {
		return super.equals(obj)
			&& (obj instanceof SpecificVariableClassDeclarationType)
			&& ((SpecificVariableClassDeclarationType) obj).isExternal() == isExternal()
			&& ((SpecificVariableClassDeclarationType) obj).isWriteable() == isWriteable();
	}

	public int hashCode() {
		return 3 * super.hashCode() + (isExternal() ? 0 : 1) + (isWriteable() ? 3 : 5);
	}
	
	public static IBodyVariableClassDeclaration getBodyVariableClassDeclaration(
		IObject simObject,
		Object anObject)
		throws VariableCompileException {

		if (anObject instanceof IVariableClassDeclaration)
			return getBodyVariableClassDeclaration(simObject, (IVariableClassDeclaration) anObject);

		else if (anObject instanceof IBodyVariableClassDeclaration)
			return (IBodyVariableClassDeclaration) anObject;

		else
			return null;
	}

	public static IBodyVariableClassDeclaration getBodyVariableClassDeclaration(
		IObject simObject,
		IVariableClassDeclaration variableClassDeclaration) {

		return simObject
			.getObjectInstanceDeclaration()
			.getBodyInstanceDeclaration()
			.getBodyVariableInstanceDeclarationList()
			.getBodyVariableInstanceDeclaration(variableClassDeclaration)
			.getBodyVariableClassDeclaration();
	}

	public IGenericType getGenericType() {
		return GenericVariableClassDeclarationType.getInstance(isExternal(), isWriteable());
	}

	private int getIndexOf(IObject simObject, IBodyVariableClassDeclaration bodyVariableClassDeclaration)
		throws ModelCompilingException {

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(simObject);
		Assert.assertNotNull(simObject.getBody());

		return simObject
			.getObjectInstanceDeclaration()
			.getObjectClassDeclaration()
			.getBodyClassDeclaration()
			.getBodyVariableIndexOf(bodyVariableClassDeclaration);
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (anObject == null) {
			super.appendNameForArgumentObject(buff, null, currDepth, maxDepth);

		} else if (anObject instanceof INamed) {
			appendNameForNamedObject(buff, (INamed) anObject, currDepth + 1, maxDepth);

		} else {
			buff.append(((Integer) anObject).toString());
		}
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
		appendNameForNamedObject(buff, (INamed) anObject, currDepth, maxDepth);
	}

	public static IVariableClassDeclaration getVariableClassDeclaration(Object anObject) {
		if (anObject instanceof IVariableClassDeclaration)
			return (IVariableClassDeclaration) anObject;

		else if (anObject instanceof IBodyVariableClassDeclaration)
			return ((IBodyVariableClassDeclaration) anObject).getVariableClassDeclaration();

		else
			return null;
	}

	public ISpecificType getVariableClassDeclarationType() {
		return getParameterSpecificType();
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		return isConstantStatic(createFunctionArgs, anObject);
	}

	public static boolean isConstantStatic(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		if (anObject instanceof IVariableClassDeclaration) {
			return isAllConstant(createFunctionArgs, (IVariableClassDeclaration) anObject);

		} else if (anObject instanceof IBodyVariableClassDeclaration) {
			return isAllConstant(createFunctionArgs, (IBodyVariableClassDeclaration) anObject);

		} else {
			Assert.assertNull(anObject);

			return true;
		}
	}

	private static boolean isAllConstant(
		ICreateFunctionArgs createFunctionArgs,
		IVariableClassDeclaration variableClassDeclaration)
		throws ModelCompilingException {

		Iterator it = variableClassDeclaration.getBodyVariableClassDeclarations();
		while (it.hasNext()) {
			IBodyVariableClassDeclaration bodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration) it.next();

			if (!isAllConstant(createFunctionArgs, bodyVariableClassDeclaration))
				return false;
		}

		return true;
	}

	private static boolean isAllConstant(
		ICreateFunctionArgs createFunctionArgs,
		IBodyVariableClassDeclaration bodyVariableClassDeclaration)
		throws ModelCompilingException {

		if (!bodyVariableClassDeclaration.isConstant(createFunctionArgs)) {
			return false;

		} else if (bodyVariableClassDeclaration.isInWorldClassDeclaration()) {
			// since there is only one world, just test if all bodyVarInstance Decls are constant
			Iterator bodyVarInstanceDecls =
				bodyVariableClassDeclaration.getBodyVariableInstanceDeclarations();

			while (bodyVarInstanceDecls.hasNext()) {
				IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
					(IBodyVariableInstanceDeclaration) bodyVarInstanceDecls.next();

				// don't call isConstsnt
				if (bodyVariableInstanceDeclaration.getUsesOwnDefaulValueFunctionCallDeclaration()
					&& !bodyVariableInstanceDeclaration.getDefaultValueFunctionCallDeclaration().isConstant(
						createFunctionArgs)) {
					return false;
				}
			}

			return true;

		} else {
			Iterator bodyVarInstanceDecls =
				bodyVariableClassDeclaration.getBodyVariableInstanceDeclarations();

			boolean onlyOneBodyVarInstanceDecl = true;

			while (bodyVarInstanceDecls.hasNext()) {
				IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
					(IBodyVariableInstanceDeclaration) bodyVarInstanceDecls.next();

				if (bodyVariableInstanceDeclaration.getUsesOwnDefaulValueFunctionCallDeclaration()) {

					// if there are more than one IBodyVariableInstDecl and at most one has it's own default value,
					// then the value of all instDecls for bodyVarClassDecl is not constant
					// there are three exception: 
					// 1. only one IBodyVariableInstDecl
					// 2. all own def. values are constant _and_ the same!
					// 3. we look at to many IBodyVariableInstDecl since we use all defined ones, but
					//    they needn't to be used in this experiment run (-> world!!)

					if (onlyOneBodyVarInstanceDecl && !bodyVarInstanceDecls.hasNext()) {
						// so we are the only bodyVariableInstanceDeclaration
						// and we can only test if the bodyVarClassDecl is const,
						// since the own default value of the bodyVarInstDecl is the only 'real' value
						return true;

					} else {
						return false;
					}
				}

				onlyOneBodyVarInstanceDecl = false;
			}

			return true;
		}
	}

	public final boolean isExternal() {
		return _external;
	}

	public boolean isMoreSpecific(ISpecificType type) {
		if (type instanceof SpecificVariableClassDeclarationType) {
			SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
				(SpecificVariableClassDeclarationType) type;

			if (!getParameterSpecificType()
				.isMoreSpecific(specificVariableClassDeclarationType.getParameterSpecificType())) {
				return false;
			}

			if (isExternal() && !specificVariableClassDeclarationType.isExternal()) {
				return false;
			}

			if (isWriteable() && !specificVariableClassDeclarationType.isWriteable()) {
				return false;
			}

			return true;

		} else {
			return false;
		}
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		if (value == null) {
			return true;
		}

		IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclaration(value);

		if (variableClassDeclaration == null) {
			return false;
		}

		// be more tolerant, due to change var to private 
		// -> all functions using this variable aren't editable any more
		//
		//		if (isWriteable() && !variableClassDeclaration.isWriteable()) {
		//			return false;
		//		}
		//
		//		if (isExternal() && !variableClassDeclaration.isExternal()) {
		//			return false;
		//		}

		return true;
	}

	/**
	 * @return boolean
	 */
	public final boolean isWriteable() {
		return _writeable;
	}

	private IBodyVariableClassDeclaration searchBodyVariableClassDeclaration(
		IObject simObject,
		IVariableClassDeclaration variableClassDeclaration) {

		Iterator it =
			simObject
				.getObjectInstanceDeclaration()
				.getBodyInstanceDeclaration()
				.getBodyVariableInstanceDeclarationList()
				.iterator();
		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration currBodyInstVar = (IBodyVariableInstanceDeclaration) it.next();
			IBodyVariableClassDeclaration currBodyVar = currBodyInstVar.getBodyVariableClassDeclaration();

			if (currBodyVar.getVariableClassDeclaration() == variableClassDeclaration) {
				return currBodyVar;
			}
		}

		return null;
	}

	public String getName() {
		return NAME
			+ "<"
			+ getVariableClassDeclarationType().getName()
			+ (isExternal() ? ", ext" : "")
			+ (isWriteable() ? ", writeable" : "")
			+ ">";
	}

	public static int getVariableIndexOf(IBody fromBody, Object variableOrIndexToVariable)
		throws SimulationRuntimeException {

		if (Assert.ASSERTION_ON) {
			Assert.assertNotNull("fromBodyImage is null", fromBody);
			Assert.assertNotNull("variableOrIndexToVariable is null", variableOrIndexToVariable);
		}

		int index;

		if (variableOrIndexToVariable instanceof Number) {
			index = ((Number) variableOrIndexToVariable).intValue();

		} else {
			IVariableClassDeclaration variableClassDeclaration =
				(IVariableClassDeclaration) variableOrIndexToVariable;

			try {
				index =
					fromBody.getBodyInstanceDeclaration().getBodyClassDeclaration().getBodyVariableIndexOf(
						variableClassDeclaration);

			} catch (ModelCompilingException exception) {
				throw new SimulationRuntimeException(exception);
			}
		}

		return index;
	}
}