package de.uniwue.ki.mas.basics.type;
import java.util.Iterator;

import junit.framework.*;

/**
 * Creation date: (05.12.2001 09:54:13)
 * @author Christoph Oechslein
 */

public final class TestTypes extends TestCase {

	/**
	 * TestTypes constructor comment.
	 * @param name java.lang.String
	 */
	public TestTypes(String name) {
		super(name);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestTypes.class);
	}

	/**
	 * Insert the method's description here.
	 * <BR>Creation date: (29.03.00 13:25:53)
	 */
	public static Test suite() {
		TestSuite anSuite = new TestSuite("Types");

		anSuite.addTest(new TestTypes("testDefaultValues"));
		anSuite.addTest(new TestTypes("testMoreGeneral"));
		anSuite.addTest(new TestTypes("testIsInVariableTypes"));
		anSuite.addTest(new TestTypes("testOther"));

		return anSuite;
	}

	/**
	 * @throws Exception
	 */
	public void testDefaultValues() throws Exception {

		Iterator it = TypeManager.getInstance().getAllSpecificTypes();

		while (it.hasNext()) {
			ISpecificType currSpecificType = (ISpecificType) it.next();
			if (currSpecificType.isAtomicDefaultValue()) {
				currSpecificType.isArgumentDeclarationValueOk(
					currSpecificType.getArgumentDeclarationDefaultValue());
			}
		}
	}

	public void testMoreGeneral() throws Exception {

		assertTrue(
			TypeManager.getInstance().isMoreGeneral(DoubleType.getInstance(), DoubleType.getInstance()));
		assertTrue(
			TypeManager.getInstance().isMoreGeneral(ObjectType.getInstance(), DoubleType.getInstance()));
		assertTrue(
			!TypeManager.getInstance().isMoreGeneral(DoubleType.getInstance(), ObjectType.getInstance()));

		Iterator it = TypeManager.getInstance().getAllTypes();

		while (it.hasNext()) {
			IType currType = (IType) it.next();

			assertEquals(currType, currType);

			if (currType instanceof ISpecificType)
				assertTrue(
					TypeManager.getInstance().isMoreGeneral(
						(ISpecificType) currType,
						(ISpecificType) currType));

		}

		testStrictMoreGeneral(ObjectType.getInstance(), DoubleType.getInstance());

		SpecificIteratorType numberSetType =
			GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance());
		SpecificIteratorType objectSetType =
			GenericIteratorType.getInstance().createSpecificIteratorType(ObjectType.getInstance());

		assertEquals(true, GenericIteratorType.getInstance().canCreate(numberSetType));
		assertEquals(true, GenericIteratorType.getInstance().canCreate(objectSetType));

		testStrictMoreGeneral(objectSetType, numberSetType);
	}

	/**
	 * @throws Exception
	 */
	public void testIsInVariableTypes() throws Exception {

		assertEquals(
			true,
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(BooleanType.getInstance()));

		assertEquals(
			true,
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(ObjectType.getInstance()));

		assertEquals(
			false,
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(VoidType.getInstance()));

		assertEquals(
			true,
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(
				GenericListType.getInstance().createSpecificListType(DoubleType.getInstance())));

		assertEquals(
			false,
			TypeManager.getInstance().isInAllTypesForVariableClassDeclaration(
				GenericFunctionType.getInstance().createSpecificFunctionType(
					DoubleType.getInstance(),
					DoubleType.getInstance())));

	}

	/**
	 * @throws Exception
	 */
	public void testOther() throws Exception {

		assertEquals(
			false,
			TypeManager.getInstance().isMoreGeneral(DoubleType.getInstance(), ObjectType.getInstance()));
		// -> false

		assertEquals(
			true,
			TypeManager.getInstance().isMoreGeneral(ObjectType.getInstance(), DoubleType.getInstance()));
		// -> true

		assertEquals(true, TypeManager.getInstance().isInAllTypes(BooleanType.getInstance()));

		assertEquals(true, TypeManager.getInstance().isInAllTypes(VoidType.getInstance()));
	}

	private void testStrictMoreGeneral(ISpecificType generalType, ISpecificType specificType)
		throws Exception {
		assertTrue(TypeManager.getInstance().isMoreGeneral(generalType, specificType));
		assertTrue(!TypeManager.getInstance().isMoreGeneral(specificType, generalType));
	}
}