package de.uniwue.ki.mas.basics.type;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.action.rest.FunctionActionRestDeclarationFactory;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (13.09.2001 15:26:49)
 * @author Christoph Oechslein
 */

public final class VoidType extends AbstractSpecificType {
	private static final VoidType _instance = new VoidType();

	public static final class ActionArgument extends AbstractFunctionAction implements IArgument {
		private final IFunction _functionAction;

		public ActionArgument(IFunction functionAction) {
			super();

			_functionAction = functionAction;
		}

		public final void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			_functionAction.act(executeFunctionArgs);
		}

		public final IFunction getFunctionAction() {
			return _functionAction;
		}

		public String getName() {
			return "void";
		}
	}

	protected VoidType() {
		super();
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object anObject) {
		// shouldn't happen
		Assert.fail("Argument with VoidType not good -> ERROR");

		return null;
		/*
		if (anObject instanceof ISpecificFunctionDeclaration) {
		 ISpecificFunctionDeclaration specificFunctionDeclaration = (ISpecificFunctionDeclaration)anObject;
		 return new ActionArgument(specificFunctionDeclaration.createFunction(createFunctionArgs,
		     IFunctionList.EMPTY_FUNCTIONLIST));
		} else if (anObject instanceof IArgument) {
		 return (IArgument)anObject;
		} else {
		 return new ActionArgument((IFunction)anObject);
		}
		*/
	}

	public IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclaration() {
		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			null,
			FunctionActionRestDeclarationFactory.NOOP_SPECIFIC_FUNCTION_DECL);
	}

	public Object getArgumentDeclarationDefaultValue() {
		throw new UnsupportedOperationException();
	}

	public static final VoidType getInstance() {
		return _instance;
	}

	public String getName() {
		return "Void";
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			appendNameForNamedObject(buff, (INamed) anObject, currDepth + 1, maxDepth);
		}
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
		appendNameForNamedObject(buff, (ISpecificFunctionDeclaration) anObject, currDepth, maxDepth);
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof ISpecificFunctionDeclaration);
	}

	public boolean isAtomicDefaultValue() {
		return false;
	}
}