package de.uniwue.ki.mas.basics.type.argument;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

public abstract class AbstractArgument extends AbstractFunctionDynamic implements IArgument {
	public abstract Object getConstantValue() throws SimulationRuntimeException;

	/**
	* Function which returns nothing, just do side effects on <code>executeFunctionArg</code>.
	*
	* @input executeFunctionArgs are the arguments for this 'function'.
	* @exception if an error occurs during the function call.
	*/
	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {

		((IFunction) getConstantValue()).act(executeFunctionArgs);
	}

	/**
	 * Function on arguments which returns an Object, this is the standard method.
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */
	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return getConstantValue();
	}

	/**
	 * Function on a arguments which returns boolean (predicate function), should be the same as
	 * <code>execute(executeFunctionArgs).booleanValue()</code>.
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 * @param executeFunctionArgs IExecuteFunctionArgs
	 * @return boolean
	 * @throws SimulationRuntimeException
	 * @see de.uniwue.ki.mas.basics.function.IFunction#executeForBoolean(IExecuteFunctionArgs)
	 */
	public boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		final Object value = getConstantValue();

		Assert.assertTrue(value instanceof Boolean);

		return ((Boolean) value).booleanValue();
	}

	public double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		final Object value = getConstantValue();

		Assert.assertTrue(
			"value is no Number: " + (value == null ? "null" : value.getClass().toString()),
			value instanceof Number);

		return ((Number) value).doubleValue();
	}

	public int executeForInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		final Object value = getConstantValue();

		Assert.assertTrue(
			"value is no Number: " + (value == null ? "null" : value.getClass().toString()),
			value instanceof Number);

		return ((Number) value).intValue();
	}

	public String getName() {
		try {
			return getNameWithException();
		} catch (SimulationRuntimeException e) {
			e.printStackTrace();
			return "Error during printing: " + e.getMessage();
		}
	}

	protected String getNameWithException() throws SimulationRuntimeException {
		Object value = getConstantValue();

		if (value instanceof INamed) {
			return ((INamed) value).getName();

		} else {
			return value + "";
		}
	}
}