package de.uniwue.ki.mas.basics.type.argument;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionCall.AtomicFunctionCallInternDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IAtomicFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.VoidType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (01.02.2002 13:10:06)
 * @author Christoph Oechslein
 */

public class ArgumentDeclaration extends AbstractDeclaration implements IArgumentDeclaration {
	private final ISpecificType _specificType;
	private Object _object = null;

	public ArgumentDeclaration(ISpecificType specificType, Object anObject) {
		super();

		Assert.assertTrue("VoidType not allowed", !VoidType.getInstance().equals(specificType));

		_specificType = specificType;

		setObject(anObject);
	}

//	public boolean equals(Object arg0) {
//		if (arg0==null) return false;
//		
//		if (arg0==this) return true;
//		
//		if (!(arg0 instanceof ArgumentDeclaration)) return false; 
//		
//		ArgumentDeclaration arg = (ArgumentDeclaration)arg0;
//	
//		return (Utils.equals(arg.getSpecificType(),getSpecificType()) && Utils.equals(arg.getObject(),getObject()));
//	}
//	
//	public int hashCode() {
//		int value = 16;
//		
//		if (getObject() != null) {
//			value = getObject().hashCode() * 3;
//		}
//		if (getSpecificType()!=null) {
//			value = getSpecificType().hashCode() * 7;
//		}
//		
//		return value;
//	}


	public final ICopyable copy(Map objectMapping) {
		return getSpecificType().copyArgumentDeclaration(this, objectMapping);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		throw new UnsupportedOperationException();
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getSpecificType().createArgument(createFunctionArgs, getObject());
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (getSpecificType() != null) {
			getSpecificType().appendNameForArgumentDeclarationObject(buff, getObject(), currDepth, maxDepth);

		} else {
			buff.append(getObject());
		}
	}

	public Object getObject() {
		return _object;
	}

	public ISpecificType getSpecificType() {
		return _specificType;
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return _specificType.isConstant(createFunctionArgs, _object);
	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (this == obj)
			return true;

		if (!(obj instanceof IArgumentDeclaration) || (obj instanceof IFunctionArgumentDeclaration)) {
			return false;
		}

		IArgumentDeclaration argumentDeclaration = (IArgumentDeclaration)obj;

		if (!Utils.equals(getSpecificType(), argumentDeclaration.getSpecificType())) {
			return false;
		}

		if (getObject() instanceof IIsomorphicHashable) {
			if (!(((IIsomorphicHashable)getObject())
				.isomorphicEquals((IIsomorphicHashable)argumentDeclaration.getObject()))) {

				return false;
			}

		} else {
			if (!Utils.equals(getObject(), argumentDeclaration.getObject())) {
				return false;
			}
		}

		return true;
	}

	public int isomorphicHashCode() {

		if (getObject() == null)
			return getSpecificType().hashCode();

		else if (getObject() instanceof IIsomorphicHashable)
			return 3 * getSpecificType().hashCode() + ((IIsomorphicHashable)getObject()).isomorphicHashCode();

		else
			return 3 * getSpecificType().hashCode() + getObject().hashCode();
	}

	public void remove() {

		getSpecificType().removeArgumentDeclaration(this);

		super.remove();
	}

	public void setObject(Object object) {
		if (Assert.ASSERTION_ON && !_specificType.isArgumentDeclarationValueOk(object)) {
			Assert.fail(
				"Object: "
					+ object
					+ " is not valid for type: "
					+ _specificType.getName()
					+ " (calling ISpecificType#isArgumentDeclarationValueOk(Object)).");
		}

		Object oldValue = getObject();
		_object = object;
		firePropertyChange("object", oldValue, object);
	}

	public IAtomicFunctionCallInternDeclaration createAtomicFunctionCallInternDeclaration() {
		return AtomicFunctionCallInternDeclarationFactory.createAtomicFunctionCallInternDeclaration(this);
	}

}