package de.uniwue.ki.mas.basics.type.argument;
import java.util.*;

import javax.swing.DefaultListModel;
import javax.swing.ListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.IGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.comparator.NamedComparator;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
* Creation date: (26.05.00 10:47:41)
* @author Christoph Oechslein
*/

public class ArgumentDeclarationListModel extends DefaultListModel implements ListModel, IRemoveable {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			initializeList();
		}
	};

	private static final Comparator _argumentComparator = new Comparator() {
		public int compare(Object o1, Object o2) {
			return NamedComparator.getInstance().compare(
				((IArgumentDeclaration)o1).getSpecificType(),
				((IArgumentDeclaration)o2).getSpecificType());
		}
	};

	private ISpecificType _type = TypeManager.getInstance().getDefaultType();
	private IFunctionDeclarationListAble _functionDeclarationListAble =
		FunctionDeclarationListManager.getAllFunctionDeclarationList();

	private IPredicateFunction filterFunction = null;

	public ArgumentDeclarationListModel() {
		super();
		initializeList();
	}

	/**
	 * With filter given in this constructor unwanted declarations can be filtered out.
	 * E.g. filtering all argument declarations wich have no editorPane
	 */
	public ArgumentDeclarationListModel(IPredicateFunction filter) {
		super();
		filterFunction = filter;
		initializeList();
	}

	public void setFunctionDeclarationListAble(IFunctionDeclarationListAble functionDeclarationListAble) {
		if (!Utils.equals(functionDeclarationListAble, _functionDeclarationListAble)) {
			IFunctionDeclarationListAble oldFunctionDeclarationListAble = getFunctionDeclarationListAble();

			if (oldFunctionDeclarationListAble != null) {
				oldFunctionDeclarationListAble.removeChangeListener(_changeListener);
			}

			_functionDeclarationListAble = functionDeclarationListAble;

			if (functionDeclarationListAble != null) {
				functionDeclarationListAble.addChangeListener(_changeListener);
				initializeList();
			}
		}
	}

	public ISpecificType getType() {
		return _type;
	}

	public void setSpecificType(ISpecificType newType) {
		if (!Utils.equals(newType, getType())) {
			_type = newType;
			initializeList();
		}
	}

	public int indexOf(ISpecificType specificType) {
		for (int index = 0; index < getSize(); index++) {
			IArgumentDeclaration currArgumentDeclaration = (IArgumentDeclaration)getElementAt(index);
			if (!(currArgumentDeclaration instanceof IFunctionArgumentDeclaration)
				&& Utils.equals(specificType, currArgumentDeclaration.getSpecificType()))
				return index;
		}
		for (int index = 0; index < getSize(); index++) {
			IArgumentDeclaration currArgumentDeclaration = (IArgumentDeclaration)getElementAt(index);
			if (!(currArgumentDeclaration instanceof IFunctionArgumentDeclaration)
				&& TypeManager.getInstance().isMoreGeneral(
					currArgumentDeclaration.getSpecificType(),
					specificType))
				return index;
		}
		return -1;
	}

	protected IFunctionDeclarationListAble getFunctionDeclarationListAble() {
		return _functionDeclarationListAble;
	}

	private static List convertToArgumentDeclaraion(List input) {
		return CollectionsHelper
			.asArrayList(
				CollectionsHelper
				.map(CollectionsHelper.select(input.iterator(), new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				return ((ISpecificType)obj).isAtomicDefaultValue();
			}
		}), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				return ArgumentDeclarationFactory.getInstance().createArgumentDeclaration((ISpecificType)obj);
			}
		}));
	}

	protected void initializeList() {

		clear();
		if (getType() != null) {

			List newList = convertToArgumentDeclaraion(TypeManager.getSubTypes(getType()));

			// why did I do that?!
			// to show functions as arguments for function manipulation functions, like BindFirst

			if (getType() instanceof ISpecificFunctionType) {
				initializeListSpecificFunctionType((ISpecificFunctionType)getType(), newList);
			}

			Collections.sort(newList, _argumentComparator);

			Iterator it = newList.iterator();
			while (it.hasNext()) {
				Object argumentDeclaration = it.next();
				try {
					if (filterFunction.isElementOk(argumentDeclaration))
						addElement(argumentDeclaration);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			if (getFunctionArgumentDeclarationList() != null) {

				int oldSize = getSize();

				int index = 0;
				it = getFunctionArgumentDeclarationList().iterator();
				while (it.hasNext()) {
					IFunctionArgumentDeclaration currFunctionArgumentDeclaration =
						(IFunctionArgumentDeclaration)it.next();
					if (TypeManager
						.getInstance()
						.isMoreGeneral(getType(), currFunctionArgumentDeclaration.getSpecificType())) {
						add(index++, currFunctionArgumentDeclaration);
					}

					if (oldSize < getSize())
						fireIntervalAdded(this, oldSize, getSize());
				}
			}

		}
	}

	private void initializeListSpecificFunctionType(
		ISpecificFunctionType specificFunctionType,
		List newList) {

		if (getFunctionDeclarationListAble() != null) {
			Iterator allSpecificFunctions =
				FunctionDeclarationListHelper.getSpecificFunctionDeclarations(
					getFunctionDeclarationListAble().getFunctionDeclarations());

			while (allSpecificFunctions.hasNext()) {
				ISpecificFunctionDeclaration currSpecificFunctionDeclaration =
					(ISpecificFunctionDeclaration)allSpecificFunctions.next();
				if (specificFunctionType.hasSameSignature(currSpecificFunctionDeclaration)) {
					newList.add(
						ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
							specificFunctionType,
							currSpecificFunctionDeclaration));
				}
			}

			Iterator allGenericFunctions =
				FunctionDeclarationListHelper.getGenericFunctionDeclarations(
					getFunctionDeclarationListAble().getFunctionDeclarations());

			int length = specificFunctionType.getSpecificInputTypesLength();

			if (specificFunctionType.getSpecificInputTypesLength()
				== ISpecificFunctionDeclaration.UNLIMITED_ARGUMENT_COUNT) {
				length = 2;
			}

			List inputList = new ArrayList(length);
			for (int i = 0; i < length; i++) {
				inputList.add(specificFunctionType.getSpecificInputTypeAt(i));
			}

			while (allGenericFunctions.hasNext()) {
				IGenericFunctionDeclaration currGenericFunctionDeclaration =
					(IGenericFunctionDeclaration)allGenericFunctions.next();
				if (currGenericFunctionDeclaration
					.canCreateFunctionDeclarationWith(specificFunctionType.getSpecificOutputType())) {
					ISpecificFunctionDeclaration specificFunctionDecl =
						currGenericFunctionDeclaration.createFunctionDeclaration(
							specificFunctionType.getSpecificOutputType(),
							inputList.iterator());
					if (specificFunctionDecl != null) {
						newList.add(
							ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
								specificFunctionType,
								specificFunctionDecl));
					}
				}
			}

		}
	}

	public void remove() {
		setFunctionArgumentDeclarationList(null);
		setFunctionDeclarationListAble(null);
	}

	protected IFunctionArgumentDeclarationList _functionArgumentDeclarationList;

	protected final transient ChangeListener _functionArgumentDeclarationListStateChangeListener =
		new ChangeListener() {
		public void stateChanged(ChangeEvent evt) {
			if (evt.getSource() == getFunctionArgumentDeclarationList()) {
				initializeList();
			}
		}
	};

	public IFunctionArgumentDeclarationList getFunctionArgumentDeclarationList() {
		return _functionArgumentDeclarationList;
	}

	public void setFunctionArgumentDeclarationList(IFunctionArgumentDeclarationList newFunctionArgumentDeclarationList) {
		if (newFunctionArgumentDeclarationList == getFunctionArgumentDeclarationList())
			return;

		if (getFunctionArgumentDeclarationList() != null) {
			getFunctionArgumentDeclarationList().removeChangeListener(
				_functionArgumentDeclarationListStateChangeListener);
		}

		_functionArgumentDeclarationList = newFunctionArgumentDeclarationList;

		if (newFunctionArgumentDeclarationList != null) {
			newFunctionArgumentDeclarationList.addChangeListener(
				_functionArgumentDeclarationListStateChangeListener);

			initializeList();
		}
	}
}