package de.uniwue.ki.mas.basics.type.argument;
import java.io.Serializable;

import de.uniwue.ki.mas.basics.function.AbstractFunctionDynamic;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.functionArgument.IArgumentBindings;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.name.AbstractNamed;
import de.uniwue.ki.util.name.INamed;
/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */public class ArgumentWithBindings extends AbstractFunctionDynamic implements IArgument {

	public class ResultWithBindings extends AbstractNamed implements Serializable {
		private final IArgumentBindings _savedArgumentBindings;

		public ResultWithBindings(IArgumentBindings savedArgumentBindings) {
			_savedArgumentBindings = savedArgumentBindings;
		}

		public IArgumentBindings getSavedArgumentBindings() {
			return _savedArgumentBindings;
		}

		public Object getValue() {
			return ArgumentWithBindings.this.getValue();
		}

		public String getName() {
			return getNameIntern();
		}

		public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
			if (currDepth > maxDepth) {
				buff.append("...");
} else {
	ArgumentWithBindings.this.appendName(buff, currDepth, maxDepth);
	buff.append(" [");
getSavedArgumentBindings().appendName(buff, currDepth + 1, maxDepth);
buff.append("]");
}
}

public boolean equals(Object obj) {
	if (obj == this) {
		return true;
	} else if (!(obj instanceof ResultWithBindings)) {
		return false;
	} else {
		ResultWithBindings resultWithBindings = (ResultWithBindings)obj;
		return Utils.equals(getSavedArgumentBindings(), resultWithBindings.getSavedArgumentBindings())
			&& Utils.equals(getValue(), resultWithBindings.getValue());
	}
}

public int hashCode() {
	return 3 * getSavedArgumentBindings().hashCode() + (getValue() == null ? 31 : getValue().hashCode());
}
}

private final Object _value;

public ArgumentWithBindings(Object value) {
	_value = value;
}/**
	 * Function on arguments which returns an Object, this is the standard method.
	 *
	 * @input executeFunctionArgs are the arguments for this function
	 * @exception if an error occurs during the function call
	 */public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
	return new ResultWithBindings(executeFunctionArgs.saveCurrentArgumentBindings());
}

protected Object getValue() {
	return _value;
}

public String getName() {
	return getNameIntern();
}

public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
	if (currDepth > maxDepth) {
		buff.append("...");
} else {
	if (getValue() instanceof INamed) {
		((INamed)getValue()).appendName(buff, currDepth + 1, maxDepth);
	} else {
		buff.append(getValue());
	}
}
}
}