package de.uniwue.ki.mas.basics.type.argument;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IAtomicFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.IType;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * This class encapsulate a type and an instance of this type and
 * can produce an {@link IArgument}.
 * Creation date: (29.08.2000 14:59:44)
 *
 * @author Christoph Oechslein
 */

public interface IArgumentDeclaration extends IDeclaration {

    /**
     * Creates a new {@link IArgument} with a compiling binding.
     * Compiling takes place for example for an {@link IActivityGraphNodeDeclaration} which is compiled
     * to an {@link IActivity}.
     */

    IArgument createArgument(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException;

    /**
     * Get the instance of this Declaration
     */
    Object getObject();

    /**
     * Gets the type.
     */

    ISpecificType getSpecificType();

    /**
     * Set the instance of this Declaration,
     * <code>value</code> must be an correct instance of #getType()
     *
     * @see IType#isValueOk(Object)
     */
    void setObject(Object anObject);

	/**
	 * @see IFunctionCallInternDeclaration#isConstant(ICreateFunctionArgs)
	 */
	boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException;
    
    IAtomicFunctionCallInternDeclaration createAtomicFunctionCallInternDeclaration();
}
