/*
 * Created on 20.05.2003
 *
 */
package de.uniwue.ki.mas.basics.type.composed;

import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.RemoveVetoListener;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.collection.IteratorWithException;

/**
 * @author herrler
 *
 */
public abstract class AbstractSpecificComposedType
	extends AbstractCopyAbleSpecificType
	implements ISpecificType, ISpecificComposedType {

	private ArrayList _attributeList = new ArrayList();
	private ISpecificComposedType _father = null;
	private IUserTypeDeclaration declaration=null;
	
	public void setDeclaration(IUserTypeDeclaration decl){
		declaration=decl;
	}
	
	/*
	 * returns the declaration if the composed type is created 
	 * by one (rainer)
	 */
	public IDeclaration getDeclaration() {
		return declaration;
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
		Assert.assertTrue((anObject==null) || (anObject instanceof IComposedTypeValue));

		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			appendNameForNamedObject(buff, (IComposedTypeValue)anObject, currDepth, maxDepth);
		}
	}

	public Object copyArgumentObject(Object anObject, Map objectMapping) {
		if (anObject == null)
			return null;
		else
			return ((IComposedTypeValue)anObject).copy(objectMapping);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		Assert.assertNotNull(dstObject);
		super.copyInto(dstObject, objectMapping);

		AbstractSpecificComposedType dstSpecificComposedType = (AbstractSpecificComposedType)dstObject;
		dstSpecificComposedType._attributeList.clear();

		Iterator it = getOwnAttributes();
		while (it.hasNext()) {
			IComposedTypeAttribute typeAttribute = (IComposedTypeAttribute)it.next();
			dstSpecificComposedType._attributeList.add(typeAttribute.copy(objectMapping));
		}

		if ((objectMapping != null) && (objectMapping.containsKey(_father))) {
			dstSpecificComposedType._father = (ISpecificComposedType)objectMapping.get(_father);
		} else {
			dstSpecificComposedType._father = _father;
		}
	}

	public final IComposedTypeValue createComposedTypeValue(IteratorWithException values)
		throws SimulationRuntimeException {
		return new SpecificComposedTypeValue(this, values);
	}

	public IComposedTypeValue createEmptyComposedTypeValue() {
		return SpecificComposedTypeValue.createEmptyArgumentValue(this);
	}

	public boolean equals(Object obj) {
		return obj == this;
	}

	public Object getArgumentDeclarationDefaultValue() {
		return null;
	}

	public IComposedTypeAttribute getAttributeAt(int index) {
		return (IComposedTypeAttribute)CollectionsHelper.asArrayList(getAttributes()).get(index);
	}

	protected List getAttributeList() {
		return _attributeList;
	}

	public int getAttributeListSize() {
		return CollectionsHelper.size(getAttributes());
	}

	public Iterator getAttributes() {
		if (_father != null)
			return CollectionsHelper.concatenate(_father.getAttributes(), getAttributeList().iterator());
		else
			return getAttributeList().iterator();
	}

	public ISpecificComposedType getFather() {
		return _father;
	}

	public IGenericType getGenericType() {
		return GenericComposedType.getInstance();
	}

	public Iterator getOwnAttributes() {
		return getAttributeList().iterator();
	}

	public int hashCode() {
		return System.identityHashCode(this);
	}

	public boolean hasWriteableAttributes() {
		Iterator it = getAttributes();
		while (it.hasNext()) {
			IComposedTypeAttribute element = (IComposedTypeAttribute)it.next();

			if (element.isWriteable()) {
				return true;
			}
		}

		return false;
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		if (value == null)
			return true;

		else if (!(value instanceof IComposedTypeValue))
			return false;

		else
			return TypeManager.getSubTypes(this).contains(((IComposedTypeValue)value).getComposedType());
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject) {
	    
		if (anObject == null)
			return true;
		
		/**
		 * Msste eigentlich nie aufgerufen werden, da ein 
		 * Composed Type, nie direct in eine ArgumentDeclaration eingeben 
		 * werden kann.
		 * Abgesehen davon msste logischerweise unten auch die 
		 * Bedingung negiert werden??
		 */
		return hasWriteableAttributes();
	}

	public boolean isMoreSpecific(ISpecificType type) {
		return super.isMoreSpecific(type);
	}
	
	private final RemoveVetoListener removeVetoListener = new RemoveVetoListener() {
		public boolean canBeRemoved() {
			return false;
		}
	};
	
	protected void addAttribute(String name, ISpecificType type) {
			ComposedTypeAttribute attribute = new ComposedTypeAttribute();
			attribute.setAttributeName(name);
			attribute.setAttributeType(type);
			attribute.setUsesShortDescription(false);
			getAttributeList().add(attribute);
		}

	public void setFather(ISpecificComposedType type) {
		if (_father == type)
			return;

		if (_father != null) {
			_father.removeSubType(this);
			if (_father.getDeclaration()!=null) _father.getDeclaration().removeRemoveVetoListener(removeVetoListener);
		} 
		_father = type;
		if (_father != null) {
			_father.addSubType(this);
			if (_father.getDeclaration()!=null) _father.getDeclaration().addRemoveVetoListener(removeVetoListener);
		}
	}
}
