package de.uniwue.ki.mas.basics.type.composed;
import java.util.Map;

import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.name.IDocumentable;

/**
 * Creation date: (03.12.2001 17:38:01)
 * @author Rainer Herrler
 */
public final class ComposedTypeAttribute
	extends AbstractDocumentableDeclaration
	implements IComposedTypeAttribute, IDocumentable {

	private ISpecificType _attributeType;
	private boolean _writeable = true;

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		ComposedTypeAttribute dstComposedTypeAttribute = (ComposedTypeAttribute)dstObject;

		if ((_attributeType instanceof ICopyable)
			&& (objectMapping != null)
			&& objectMapping.containsKey(_attributeType)) {
				
			dstComposedTypeAttribute._attributeType = (ISpecificType)objectMapping.get(_attributeType);
			
		} else {
			dstComposedTypeAttribute._attributeType = _attributeType;
		}
		
		dstComposedTypeAttribute._writeable = _writeable;

		fireStateChanged();
	}

	public ComposedTypeAttribute() {
		super();

		setAttributeName("Untitled");
		setAttributeType(TypeManager.getInstance().getDefaultTypeForVariableClassDeclaration());
		setUsesShortDescription(true);
	}

	public String getAttributeName() {
		return getShortDescription();
	}

	public ISpecificType getAttributeType() {
		return _attributeType;
	}

	public String getName() {
		return getShortDescription() + " : " + _attributeType.getName();
	}

	public void setAttributeName(String newAttributeName) {
		setShortDescription(newAttributeName);
	}

	public void setAttributeType(ISpecificType newAttributeType) {
		_attributeType = newAttributeType;
	}

	public boolean isWriteable() {
		return _writeable;
	}

	public void setWriteable(boolean writeable) {
		_writeable = writeable;
	}
}