package de.uniwue.ki.mas.basics.type.composed;

import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;

/**
 * Creation date: 13.06.2002, 18:28:39
 * @author oechslein
 */
public final class GenericComposedTypeAttributeType extends AbstractGenericType {
	private static final GenericComposedTypeAttributeType _instance =
		new GenericComposedTypeAttributeType(false);
	private static final GenericComposedTypeAttributeType _instanceWriteable =
		new GenericComposedTypeAttributeType(true);

	public static final GenericComposedTypeAttributeType getInstance() {
		return _instance;
	}

	public static final GenericComposedTypeAttributeType getInstanceWriteable() {
		return _instanceWriteable;
	}

	private transient final Map _cache = new WeakHashMap();

	private final boolean _writeable;

	private GenericComposedTypeAttributeType(boolean writeable) {
		_writeable = writeable;
	}

	public boolean canCreate(ISpecificType type) {
		return type instanceof SpecificComposedTypeAttributeType;
	}
	
	
	/*
	 * returns the first attribute of any composed type that is known in the 
	 * SeSAm-declaration, 
	 * Returns null if no such attribute exists!
	 */
	public ISpecificType createDefaultSpecificType(ISeSAmDeclaration declaration) {
			 Iterator it = TypeManager.getInstance().getAllComposedTypes(declaration);
			while (it.hasNext()) {
				ISpecificComposedType type = (ISpecificComposedType)it.next();
				if (type.getAttributes().hasNext()) {
					return new SpecificComposedTypeAttributeType(type.getAttributeAt(0),type,type.getAttributeAt(0).isWriteable()); 
				}
			}
			return null; 
		}
	

	/*
	 * returns the first attribute of any composed type that is known in the 
	 * SeSAm-declaration and that is compatible with mostGeneralOutputType
	 */	
	public static ISpecificType createDefaultSpecificType(ISeSAmDeclaration declaration, ISpecificType mostGeneralOutputType) {
				Iterator it = TypeManager.getInstance().getAllComposedTypes(declaration);
				while (it.hasNext()) {
					ISpecificComposedType type = (ISpecificComposedType)it.next();
					Iterator it2 = type.getAttributes(); 
					while (it2.hasNext()) {
						IComposedTypeAttribute attr = (IComposedTypeAttribute)it2.next();
						if (attr.getAttributeType().isMoreSpecific(mostGeneralOutputType))
						return new SpecificComposedTypeAttributeType(attr,type,attr.isWriteable()); 
					}
				}
		
				return null; //createDefaultSpecificType();
			}
			
	public ISpecificType createDefaultSpecificType() {
		//new SpecificComposedTypeAttributeType(null, null, _writeable); 
		return null;
	}

	public SpecificComposedTypeAttributeType createSpecificComposedTypeAttributeType(
		IComposedTypeAttribute composedTypeAttribute,
		ISpecificComposedType composedType) {

		SpecificComposedTypeAttributeType specificComposedTypeAttributeType;

		if (_cache.containsKey(composedTypeAttribute)) {
			specificComposedTypeAttributeType =
				(SpecificComposedTypeAttributeType) _cache.get(composedTypeAttribute);
			Assert.assertEquals(composedType, specificComposedTypeAttributeType.getComposedType());
			Assert.assertEquals(
				composedTypeAttribute,
				specificComposedTypeAttributeType.getComposedTypeAttribute());
			Assert.assertEquals(_writeable, specificComposedTypeAttributeType.isWriteable());
		} else {
			specificComposedTypeAttributeType =
				new SpecificComposedTypeAttributeType(composedTypeAttribute, composedType, _writeable);
			_cache.put(composedTypeAttribute, specificComposedTypeAttributeType);
		}

		return specificComposedTypeAttributeType;
	}

	public String getName() {
		return "ComposedAttribute<name>";
	}
}
