package de.uniwue.ki.mas.basics.type.composed;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.ObjectType;
import de.uniwue.ki.mas.refactoring.ChangeUserTypeAttributeRepair;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.event.StateChangeMulticaster;

/**
 * Creation date: (03.12.2001 17:20:08)
 * @author Rainer Herrler
 */
public final class SpecificComposedType
	extends AbstractSpecificComposedType
	implements ISpecificComposedType {
		
	private transient ChangeListener _changeListener;
	private String _name;

	public SpecificComposedType() {
		this("Untitled Composed Type");
	}

	public SpecificComposedType(String name) {
		this(name, true);
	}

	public SpecificComposedType(String name, boolean addToObjectType) {
		super();

		_name = name;

		if (addToObjectType)
			ObjectType.getInstance().addSubType(this);
			
		setFather(RootComposedType.getInstance());
		fireStateChanged();
	}
	
	public void addAttribute(IComposedTypeAttribute attribute) {
		getAttributeList().add(attribute);

		fireStateChanged();
	}

	public void addChangeListener(ChangeListener changeListenerToAdd) {
		_changeListener = StateChangeMulticaster.add(_changeListener, changeListenerToAdd);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		SpecificComposedType dstSpecificComposedType = (SpecificComposedType)dstObject;
		dstSpecificComposedType._name = _name;

		fireStateChanged();
	}

	public IComposedTypeAttribute createComposedTypeAttribute() {
		return new ComposedTypeAttribute();
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new SpecificComposedType();
	}

	protected void fireStateChanged() {
		if (_changeListener != null)
			_changeListener.stateChanged(new ChangeEvent(this));
	}

	protected void fireStateChanged(int changeType) {
		if (_changeListener != null)
			_changeListener.stateChanged(new ComposedTypeChangeEvent(this, changeType));
	}

	public String getName() {
		return _name;
	}

	public void remove() {
		fireStateChanged(ComposedTypeChangeEvent.COMPOSEDTYPE_REMOVED);		
		_changeListener = null;
		
		if (NULL_COMPOSED_TYPE != this) {
			setFather(null);
			ObjectType.getInstance().removeSubType(this);
		}
	}

	public void removeAttribute(IComposedTypeAttribute attribute) {
		getAttributeList().remove(attribute);

		fireStateChanged();
	}

	public void removeChangeListener(ChangeListener changeListenerToAdd) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, changeListenerToAdd);
	}

	public void setName(String newName) {
		_name = newName;

		fireStateChanged(ComposedTypeChangeEvent.NAME_CHANGED);
	}
	
	public void setFather(ISpecificComposedType type) {
		super.setFather(type);
		fireStateChanged(ComposedTypeChangeEvent.FATHER_CHANGED);
	}
	
	public void attributeAltered(){
		fireStateChanged();
	}
	
	public void addSubType(ISpecificType newSubType) {
		super.addSubType(newSubType);
		fireStateChanged(ComposedTypeChangeEvent.CHILD_ADDED);
	}

	public void removeSubType(ISpecificType subType) {
		super.removeSubType(subType);
		fireStateChanged(ComposedTypeChangeEvent.CHILD_REMOVED);

	}

	public void attributeAltered(IComposedTypeAttribute ivjMySelectedAttribute) {
		attributeAltered();
	}

}