package de.uniwue.ki.mas.basics.type.composed;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.AbstractSpecificType;
import de.uniwue.ki.mas.basics.type.IGenericType;
import de.uniwue.ki.mas.basics.type.argument.IArgument;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 13.06.2002, 17:41:59
 * @author oechslein
 */
public final class SpecificComposedTypeAttributeType extends AbstractSpecificType {
	private final IComposedTypeAttribute _composedTypeAttribute;
	private final ISpecificComposedType _composedType;
	private final boolean _writeable;

	SpecificComposedTypeAttributeType(
		IComposedTypeAttribute composedTypeAttribute,
		ISpecificComposedType composedType,
		boolean writeable) {

		super();

		_composedTypeAttribute = composedTypeAttribute;
		_composedType = composedType;
		_writeable = writeable;
	}

	public boolean isArgumentDeclarationValueOk(Object value) {
		return (value == null) || (value instanceof IComposedTypeAttribute);
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if ((_composedTypeAttribute == null) && (_composedType == null)) {
			getGenericType().appendName(buff, currDepth, maxDepth);

		} else {
			appendNameForComposedTypeAttribute(buff, _composedTypeAttribute, currDepth, maxDepth);
		}
	}

	public final IComposedTypeAttribute getComposedTypeAttribute() {
		return _composedTypeAttribute;
	}

	public final ISpecificComposedType getComposedType() {
		return _composedType;
	}

	public IGenericType getGenericType() {
		if (isWriteable())
			return GenericComposedTypeAttributeType.getInstanceWriteable();
		else
			return GenericComposedTypeAttributeType.getInstance();
	}

	public IArgument createArgument(ICreateFunctionArgs createFunctionArgs, Object value)
		throws ModelCompilingException {

		return super.createArgument(
			createFunctionArgs,
			new Integer(getAttributValueIndex(getComposedType(), getComposedTypeAttribute(value))));
	}

	public void appendNameForArgumentObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
			
		if (currDepth >= maxDepth) {
			buff.append("...");
			
		} else if (anObject == null) {
			super.appendNameForArgumentObject(buff, null, currDepth, maxDepth);

		} else {
			IComposedTypeAttribute composedTypeAttribute =
				_composedType.getAttributeAt(((Integer) anObject).intValue());

			Assert.assertSame(composedTypeAttribute, _composedTypeAttribute);

			appendNameForComposedTypeAttribute(buff, composedTypeAttribute, currDepth, maxDepth);
		}
	}

	public void appendNameForArgumentDeclarationObject(
		StringBuffer buff,
		Object anObject,
		int currDepth,
		int maxDepth) {
		appendNameForComposedTypeAttribute(buff, getComposedTypeAttribute(anObject), currDepth, maxDepth);
	}

	private void appendNameForComposedTypeAttribute(
		StringBuffer buff,
		IComposedTypeAttribute composedTypeAttribute,
		int currDepth,
		int maxDepth) {

		composedTypeAttribute.appendName(buff, currDepth, maxDepth);
		buff.append(" [");
		_composedType.appendName(buff, currDepth + 1, maxDepth);
		buff.append("]");
	}

	private IComposedTypeAttribute getComposedTypeAttribute(Object value) {
		if (value == null)
			return _composedTypeAttribute;
		else
			return (IComposedTypeAttribute) value;
	}

	public static int getAttributValueIndex(
		ISpecificComposedType composedType,
		IComposedTypeAttribute composedTypeAttribute)
		throws AttributeCompilingException {

		int index = CollectionsHelper.indexOf(composedType.getAttributes(), composedTypeAttribute);

		if (index == -1)
			throw new AttributeCompilingException(composedTypeAttribute);

		return index;
	}

	public static void setAttributValue(
		IComposedTypeValue composedTypeValue,
		Object runTimeAttributeValue,
		Object newValue)
		throws SimulationRuntimeException {

		composedTypeValue.setAttributeValue(((Integer) runTimeAttributeValue).intValue(), newValue);
	}

	public static Object getAttributValue(IComposedTypeValue composedTypeValue, Object runTimeAttributeValue)
		throws SimulationRuntimeException {
			
		return composedTypeValue.getAttributeValue(((Integer) runTimeAttributeValue).intValue());
	}

	public boolean equals(Object obj) {
		return super.equals(obj)
			&& (obj instanceof SpecificComposedTypeAttributeType)
			&& Utils.equals(
				((SpecificComposedTypeAttributeType) obj).getComposedTypeAttribute(),
				getComposedTypeAttribute())
			&& Utils.equals(((SpecificComposedTypeAttributeType) obj).getComposedType(), getComposedType());
	}

	public int hashCode() {
		Assert.assertNotNull(getComposedTypeAttribute());
		return getComposedTypeAttribute().hashCode();
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs, Object anObject)
		throws ModelCompilingException {

		return !getComposedTypeAttribute(anObject).isWriteable();
	}

	public boolean isWriteable() {
		return _writeable;
	}
}
