package de.uniwue.ki.mas.basics.type.composed;
import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.IteratorWithException;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.copy.ICopyable;
/**
 * The evaluated Value of a Composed Type
 *
 * Creation date: (03.12.2001 17:53:48)
 * @author Rainer Herrler
 */
class SpecificComposedTypeValue extends AbstractCopyable implements IComposedTypeValue {
	private ISpecificComposedType _type;
	private ArrayList _attributeValues;

	public SpecificComposedTypeValue(ISpecificComposedType type) {
		super();

		_type = type;
		_attributeValues = new ArrayList(type.getAttributeListSize());

		for (int i = 0; i < type.getAttributeListSize(); i++) {
			_attributeValues.add(null);
		}
	}

	public SpecificComposedTypeValue(ISpecificComposedType type, IteratorWithException values)
		throws SimulationRuntimeException {
		super();

		_type = type;
		_attributeValues = new ArrayList(type.getAttributeListSize());

		try {
			for (int i = 0; i < type.getAttributeListSize(); i++) {
				Assert.assertTrue(values.hasNext());
				_attributeValues.add(values.nextWithException());
			}
		} catch (SimulationRuntimeException e) {
			throw e;
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

		Assert.assertFalse(values.hasNext());
	}

	public Object getAttributeValue(int i) throws SimulationRuntimeException {
		return _attributeValues.get(i);
	}

	public String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");

		} else {
			for (int i = 0; i < _attributeValues.size(); i++) {
				buff.append(getComposedType().getAttributeAt(i).getAttributeName());
				buff.append(": ");
				getComposedType().getAttributeAt(i).getAttributeType().appendNameForArgumentObject(
					buff,
					_attributeValues.get(i),
					currDepth + 1,
					maxDepth);

				if (i < _attributeValues.size() - 1)
					buff.append("; ");
			}
		}
	}

	public ISpecificComposedType getComposedType() {
		return _type;
	}

	public void setAttributeValue(int i, Object value) throws SimulationRuntimeException {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(getComposedType().getAttributeAt(i).isWriteable());
		}

		_attributeValues.set(i, value);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		SpecificComposedTypeValue dstComposedTypeValue = (SpecificComposedTypeValue) dstObject;

		Assert.assertEquals(getComposedType(), dstComposedTypeValue.getComposedType());

		Iterator types = getComposedType().getOwnAttributes();

		for (int i = 0; i < getComposedType().getAttributeListSize(); i++) {
			ISpecificType currType = (ISpecificType) types.next();
			dstComposedTypeValue._attributeValues.set(
				i,
				currType.copyArgumentObject(_attributeValues.get(i), objectMapping));
		}
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new SpecificComposedTypeValue(getComposedType());
	}

	/**
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		Iterator types = getComposedType().getOwnAttributes();

		for (int i = 0; i < getComposedType().getAttributeListSize(); i++) {
			ISpecificType currType = (ISpecificType) types.next();

			currType.removeArgumentObject(_attributeValues.get(i));
		}
	}

	/**
	 * @param obj Object
	 * @return boolean
	 */
	public boolean equals(Object obj) {
		if (obj == this)
			return true;

		if (!(obj instanceof SpecificComposedTypeValue))
			return false;

		SpecificComposedTypeValue composedTypeValue = (SpecificComposedTypeValue) obj;

		return Utils.equals(getComposedType(), composedTypeValue.getComposedType())
			&& Utils.equals(_attributeValues, composedTypeValue._attributeValues);
	}

	/**
	 * @return int
	 */
	public int hashCode() {
		return getComposedType().hashCode();
	}

	/**
	 * @param composedType SpecificComposedType
	 * @return SpecificComposedTypeValue
	 */
	public static SpecificComposedTypeValue createEmptyArgumentValue(ISpecificComposedType composedType) {
		SpecificComposedTypeValue retObj = new SpecificComposedTypeValue(composedType);

		for (int i = 0; i < retObj._attributeValues.size(); i++) {
			retObj._attributeValues.set(
				i,
				composedType.getAttributeAt(i).getAttributeType().getArgumentDeclarationDefaultValue());
		}

		return retObj;
	}
}