package de.uniwue.ki.mas.basics.type.composed.function;

import java.util.*;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * ComposedType CreateComposedTypeValue(T1,...,Tn)
 * or ComposedType CreateEmptyComposedTypeValue()
 * Creation date: 13.06.2002, 18:44:26
 * @author oechslein
 */
final class CreateComposedTypeValueGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	//	private static final IDependentType _dependentComposedType =
	//			new ConstraintGenericDependentType("Composed Type", GenericComposedType.getInstance());

	private static IDependentType _dependentComposedType =
		new ConstrainedToSubtypesDependentType("Composed Type");

	private transient final Map _cache = new WeakHashMap();

	private final boolean _isEmptyVersion;

	public CreateComposedTypeValueGenericFunctionDeclaration(boolean isEmptyVersion) {

		super(
			isEmptyVersion ? "CreateEmptyComposedTypeValue" : "CreateComposedTypeValue",
			GenericComposedType.getInstance(),
			CollectionsHelper.createList(_dependentComposedType));

		_isEmptyVersion = isEmptyVersion;

		if (isEmptyVersion)
			setDocumentation("Creates a composed type with all attributes set to their default value.");
		else
			setDocumentation("Creates a composed type with the given attribute values.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {
		return createFunctionDeclaration(bindings.getBindingFor(_dependentComposedType));
	}

	private CreateComposedTypeValueSpecificFunctionDeclaration createCreateComposedTypeValueSpecificFunctionDeclaration(ISpecificComposedType specificComposedType) {
		if (_cache.containsKey(specificComposedType)) {
			return (CreateComposedTypeValueSpecificFunctionDeclaration)_cache.get(specificComposedType);
		} else {
			CreateComposedTypeValueSpecificFunctionDeclaration createComposedTypeValueSpecificFunctionDeclaration =
				new CreateComposedTypeValueSpecificFunctionDeclaration(this, specificComposedType);
			_cache.put(specificComposedType, createComposedTypeValueSpecificFunctionDeclaration);
			return createComposedTypeValueSpecificFunctionDeclaration;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return createCreateComposedTypeValueSpecificFunctionDeclaration((ISpecificComposedType)outputType);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		if (!(outputType instanceof ISpecificComposedType)) {
			return null;
		}

		ISpecificComposedType composedType = (ISpecificComposedType)outputType;

		if (isEmptyVersion()) {
			if (inputTypes.hasNext())
				return null;
			else
				return createCreateComposedTypeValueSpecificFunctionDeclaration(composedType);

		} else {
			Iterator it = composedType.getAttributes();

			while (it.hasNext()) {
				IComposedTypeAttribute element = (IComposedTypeAttribute)it.next();

				if (!testAttribute(element, inputTypes))
					return null;
			}

			return createCreateComposedTypeValueSpecificFunctionDeclaration(composedType);
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException("Must call #createFunctionDeclaration(ISpecificType, Iterator) instead!");
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (isEmptyVersion()) {
			return "()";

		} else if (ObjectType.getInstance().equals(outputType)) {
			return "(...)";

		} else {
			ISpecificComposedType composedType = (ISpecificComposedType)outputType;
			StringBuffer buff = new StringBuffer();

			buff.append("(");

			Iterator it = composedType.getAttributes();

			while (it.hasNext()) {
				IComposedTypeAttribute element = (IComposedTypeAttribute)it.next();

				buff.append(element.getAttributeType().getName());
				if (it.hasNext())
					buff.append(", ");

			}

			buff.append(")");

			return buff.toString();
		}
	}

	protected boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}

	final boolean isEmptyVersion() {
		return _isEmptyVersion;
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	private boolean testAttribute(IComposedTypeAttribute composedTypeAttribute, Iterator inputTypes) {
		ISpecificType nextType = getNextSpecificType(inputTypes);
		if (nextType == null)
			// return false;
			// return true since the ComposedType maybe was changed before saving
			return true;
		else
			return TypeManager.getInstance().isMoreGeneral(
				composedTypeAttribute.getAttributeType(),
				nextType);
	}
}
