package de.uniwue.ki.mas.basics.type.composed.function;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractSpecificFunctionAdapterGenericFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.model.ModelCompilingException;

/**
 * Function Declaration to create a certain composed type value.
 * 
 * Creation date: 13.06.2002, 17:39:15
 * @author oechslein
 */
final class CreateComposedTypeValueSpecificFunctionDeclaration
	extends AbstractSpecificFunctionAdapterGenericFunctionDeclaration implements ICreateComposedTypeValueSpecificFunctionDeclaration{

	private final ISpecificComposedType _composedType;

	public CreateComposedTypeValueSpecificFunctionDeclaration(
		CreateComposedTypeValueGenericFunctionDeclaration createComposedTypeValueGenericFunctionDeclaration,
		ISpecificComposedType composedType) {

		super(
			createComposedTypeValueGenericFunctionDeclaration,
			getInputArgLength(createComposedTypeValueGenericFunctionDeclaration, composedType));

		_composedType = composedType;

		addRemoveVetoListernerTo(_composedType.getDeclaration());
	}

	public IFunction createFunction(ICreateFunctionArgs createFunctionArgs, IFunctionList functions)
		throws ModelCompilingException {

		if (getCreateComposedTypeValueGenericFunctionDeclaration().isEmptyVersion())
			return new CreateEmptyComposedTypeValueFunction(_composedType);
		else
			return new CreateComposedTypeValueFunction(_composedType, functions);
	}

	public ISpecificType getSpecificInputTypeAt(int index) {
		if (index < getInputArgLength())
			return _composedType.getAttributeAt(index).getAttributeType();
		else
			return DoubleType.getInstance();
	}

	public String getSpecificInputTypeNameAt(int index) {
		if (index < getInputArgLength())
			return _composedType.getAttributeAt(index).getAttributeName();
		else
			return "Removed Attribute";
	}

	public ISpecificType getSpecificOutputType() {
		return _composedType;
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return !_composedType.hasWriteableAttributes();
	}

	private final CreateComposedTypeValueGenericFunctionDeclaration getCreateComposedTypeValueGenericFunctionDeclaration() {
		return ((CreateComposedTypeValueGenericFunctionDeclaration)getGenericFunctionDeclaration());
	}

	public final int getSpecificInputDefaultArgsLength() {
		return getInputArgLength();
	}

	private final int getInputArgLength() {
		return getInputArgLength(getCreateComposedTypeValueGenericFunctionDeclaration(), _composedType);
	}

	private static int getInputArgLength(
		CreateComposedTypeValueGenericFunctionDeclaration createComposedTypeValueGenericFunctionDeclaration,
		ISpecificComposedType composedType) {

		if (createComposedTypeValueGenericFunctionDeclaration.isEmptyVersion())
			return 0;
		else
			return composedType.getAttributeListSize();
	}

	public ISpecificComposedType getSpecificComposedType() {
		return _composedType;
	}
}
