package de.uniwue.ki.mas.basics.type.composed.function;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeValue;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedTypeAttributeType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: 13.06.2002, 18:32:46
 * @author oechslein
 */
final class GetComposedTypeValueFunction extends AbstractFunctionDynamic {

	private final IFunction _composedTypeValueFunction;
	private final IFunction _composedTypeAttributeFunction;

	public GetComposedTypeValueFunction(
		IFunction composedTypeValueFunction,
		IFunction composedTypeAttributeFunction) {
		super();

		_composedTypeValueFunction = composedTypeValueFunction;
		_composedTypeAttributeFunction = composedTypeAttributeFunction;
	}

	public Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IComposedTypeValue composedTypeValue =
			(IComposedTypeValue) _composedTypeValueFunction.execute(executeFunctionArgs);
		Object runTimeAttributeValue = _composedTypeAttributeFunction.execute(executeFunctionArgs);
		
		if (composedTypeValue == null) {
			throw new SimulationRuntimeException("Composed Type Instance is null in " + getName());
		}

		if (runTimeAttributeValue == null) {
			throw new SimulationRuntimeException("Composed Type Attribute is null in " + getName());
		}

		return SpecificComposedTypeAttributeType.getAttributValue(composedTypeValue, runTimeAttributeValue);
	}

	public String getName() {
		return "GetComposedTypeValue ("
			+ _composedTypeValueFunction.getName()
			+ ", "
			+ _composedTypeAttributeFunction.getName()
			+ ")";
	}
}
