package de.uniwue.ki.mas.basics.type.composed.function;

import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.GenericType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Tj GetValueOfComposedType(comp<A1,T1;...;An,Tn>, <Aj,Tj>)
 * Creation date: 13.06.2002, 18:44:26
 * @author oechslein
 */
final class GetComposedTypeValueGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {
	private static final IDependentType _dependentComposedTypeAttributeType =
		new ConstraintGenericDependentType("Composed Type", GenericComposedTypeAttributeType.getInstance());

	private transient final Map _cache = new WeakHashMap();

	public GetComposedTypeValueGenericFunctionDeclaration() {

		super(
			"GetComposedTypeValue",
			GenericType.getInstance(),
			CollectionsHelper.createList(_dependentComposedTypeAttributeType));

		setDocumentation("Returns the given attribute value (Arg2) of the given composed type value (Arg1).");
	}
	
	public boolean canCreateFunctionDeclarationWith(ISeSAmDeclaration sesamDeclaration, ISpecificType outputType) {
		return (GenericComposedTypeAttributeType.createDefaultSpecificType(sesamDeclaration,outputType)!=null);
	}



	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		ISpecificType specificType = bindings.getBindingFor(_dependentComposedTypeAttributeType);

		if (specificType instanceof SpecificComposedTypeAttributeType) {
			return createFunctionDeclaration((SpecificComposedTypeAttributeType)specificType);
		} else {
			return null;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || (inputTypes.hasNext()))
			return null;

		if (!(specificTypeA instanceof ISpecificComposedType))
			return null;

		ISpecificComposedType composedType = (ISpecificComposedType)specificTypeA;

		if (!(specificTypeB instanceof SpecificComposedTypeAttributeType))
			return null;

		SpecificComposedTypeAttributeType specificComposedTypeAttributeType =
			(SpecificComposedTypeAttributeType)specificTypeB;

		if (!Utils.equals(specificComposedTypeAttributeType.getComposedType(), composedType))
			return null;

		return createFunctionDeclaration(specificComposedTypeAttributeType);
	}

	private final GetComposedTypeValueSpecificFunctionDeclaration createFunctionDeclaration(SpecificComposedTypeAttributeType specificComposedTypeAttributeType) {
		if (_cache.containsKey(specificComposedTypeAttributeType)) {
			return (GetComposedTypeValueSpecificFunctionDeclaration)_cache.get(
				specificComposedTypeAttributeType);
		} else {
			GetComposedTypeValueSpecificFunctionDeclaration getComposedTypeValueSpecificFunctionDeclaration =
				new GetComposedTypeValueSpecificFunctionDeclaration(this, specificComposedTypeAttributeType);
			_cache.put(specificComposedTypeAttributeType, getComposedTypeValueSpecificFunctionDeclaration);
			return getComposedTypeValueSpecificFunctionDeclaration;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(ComposedType, ComposedType Attribute<" + outputType.getName() + ">)";
	}
}
