package de.uniwue.ki.mas.basics.type.composed.function;

import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeValue;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedTypeAttributeType;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: 13.06.2002, 18:32:46
 * @author oechslein
 */
final class SetComposedTypeValueFunction extends AbstractFunctionAction {

	private final IFunction _composedTypeValueFunction;
	private final IFunction _composedTypeAttributeFunction;
	private final IFunction _newValueFunction;

	public SetComposedTypeValueFunction(
		IFunction composedTypeValueFunction,
		IFunction composedTypeAttributeFunction,
		IFunction newValueFunction) {
		super();

		_composedTypeValueFunction = composedTypeValueFunction;
		_composedTypeAttributeFunction = composedTypeAttributeFunction;
		_newValueFunction = newValueFunction;
	}

	/**
	 * @param executeFunctionArgs IExecuteFunctionArgs
	 * @throws SimulationRuntimeException
	 * @see de.uniwue.ki.mas.basics.function.IFunction#act(IExecuteFunctionArgs)
	 */
	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		IComposedTypeValue composedTypeValue =
			(IComposedTypeValue) _composedTypeValueFunction.execute(executeFunctionArgs);
		Object runTimeAttributeValue = _composedTypeAttributeFunction.execute(executeFunctionArgs);

		Object newValue = _newValueFunction.execute(executeFunctionArgs);

		SpecificComposedTypeAttributeType.setAttributValue(
			composedTypeValue,
			runTimeAttributeValue,
			newValue);
	}

	public String getName() {
		return "SetComposedTypeValue ("
			+ _composedTypeValueFunction.getName()
			+ ", "
			+ _composedTypeAttributeFunction.getName()
			+ ", "
			+ _newValueFunction.getName()
			+ ")";
	}

}
