package de.uniwue.ki.mas.basics.type.composed.function;

import java.util.*;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * void SetValueOfComposedType(comp<A1,T1;...;An,Tn>, <Aj,Tj>, Tj)
 * Creation date: 13.06.2002, 18:44:26
 * @author oechslein
 */
final class SetComposedTypeValueGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final IDependentType _dependentComposedTypeAttributeType =
		new ConstraintGenericDependentType(
			"Composed Type",
			GenericComposedTypeAttributeType.getInstanceWriteable());

	private transient final Map _cache = new WeakHashMap();

	public SetComposedTypeValueGenericFunctionDeclaration() {
		super(
			"SetComposedTypeValue",
			VoidType.getInstance(),
			CollectionsHelper.createList(_dependentComposedTypeAttributeType));

		setDocumentation(
			"Sets the attribute value (Arg2)"
				+ " of the composed type value (Arg1)"
				+ " to the given value (Arg3).");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		ISpecificType specificType = bindings.getBindingFor(_dependentComposedTypeAttributeType);

		if (specificType instanceof SpecificComposedTypeAttributeType) {
			return createFunctionDeclaration((SpecificComposedTypeAttributeType)specificType);
		} else {
			return null;
		}
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);
		ISpecificType specificTypeC = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || (inputTypes.hasNext()))
			return null;

		if (!(specificTypeA instanceof ISpecificComposedType))
			return null;

		ISpecificComposedType composedType = (ISpecificComposedType)specificTypeA;

		if (!(specificTypeB instanceof SpecificComposedTypeAttributeType))
			return null;

		SpecificComposedTypeAttributeType specificComposedTypeAttributeType =
			(SpecificComposedTypeAttributeType)specificTypeB;

		if (!Utils.equals(specificComposedTypeAttributeType.getComposedType(), composedType))
			return null;

		if (!TypeManager
			.getInstance()
			.isMoreGeneral(
				specificComposedTypeAttributeType.getComposedTypeAttribute().getAttributeType(),
				specificTypeC))
			return null;

		return createFunctionDeclaration(specificComposedTypeAttributeType);
	}

	private final SetComposedTypeValueSpecificFunctionDeclaration createFunctionDeclaration(SpecificComposedTypeAttributeType specificComposedTypeAttributeType) {
		if (_cache.containsKey(specificComposedTypeAttributeType)) {
			return (SetComposedTypeValueSpecificFunctionDeclaration)_cache.get(
				specificComposedTypeAttributeType);
		} else {
			SetComposedTypeValueSpecificFunctionDeclaration setComposedTypeValueSpecificFunctionDeclaration =
				new SetComposedTypeValueSpecificFunctionDeclaration(this, specificComposedTypeAttributeType);
			_cache.put(specificComposedTypeAttributeType, setComposedTypeValueSpecificFunctionDeclaration);
			return setComposedTypeValueSpecificFunctionDeclaration;
		}
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "("
			+ GenericComposedType.getInstance().getName()
			+ ", "
			+ GenericComposedTypeAttributeType.getInstance().getName()
			+ ", T)";
	}
}
