package de.uniwue.ki.mas.basics.type.enum;
import java.util.List;
import java.util.Map;

import javax.swing.event.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.ObjectType;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.event.StateChangeMulticaster;
import de.uniwue.ki.util.factory.AbstractObjectFactory;
import de.uniwue.ki.util.list.DefaultMutableListModel;
import de.uniwue.ki.util.list.DefaultMutableListModelAndFactory;

/**
 * A specific Enumeration Type that can be modified 
 * (Items added, Name changed)
 * 
 * Creation date: (03.12.2001 17:20:08)
 * @author Rainer Herrler
 */
public final class SpecificEnumType
	extends AbstractSpecificEnumType
	implements ISpecificType, ISpecificEnumType {

	private final class EnumItemObjectFactory extends AbstractObjectFactory {
		public final Object createObject() {
			return createItem();
		}
		public String getName() {
			return "Item Factory";
		}
	}

	private transient ChangeListener _changeListener;
	private final DefaultMutableListModelAndFactory _itemListAndFactory =
		new DefaultMutableListModelAndFactory(new DefaultMutableListModel(), new EnumItemObjectFactory());

	private String _name;

	public SpecificEnumType() {
		this("Untitled Enum Type");
	}

	public SpecificEnumType(String name) {
		this(name, true);
	}

	SpecificEnumType(String name, boolean addToObjectType) {
		super();

		_name = name;

		if (addToObjectType)
			ObjectType.getInstance().addSubType(this);

		_itemListAndFactory.getMutableListModel().addListDataListener(new ListDataListener() {

			public void contentsChanged(ListDataEvent e) {
				fireStateChanged();
			}
			public void intervalAdded(ListDataEvent e) {
				fireStateChanged();
			}

			public void intervalRemoved(ListDataEvent e) {
				fireStateChanged();
			}
		});

		fireStateChanged();
	}

	public void addChangeListener(ChangeListener changeListenerToAdd) {
		_changeListener = StateChangeMulticaster.add(_changeListener, changeListenerToAdd);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		Assert.assertNotNull(dstObject);
		
		super.copyInto(dstObject, objectMapping);

		SpecificEnumType dstSpecificEnumType = (SpecificEnumType)dstObject;
		dstSpecificEnumType._name = _name;

		_itemListAndFactory.copyInto(dstSpecificEnumType._itemListAndFactory, objectMapping);

		fireStateChanged();
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new SpecificEnumType();
	}

	public EnumItem createItem() {
		return new EnumItem(this);
	}

	protected void fireStateChanged() {
		if (_changeListener != null)
			_changeListener.stateChanged(new ChangeEvent(this));
	}

	public Object getArgumentDeclarationDefaultValue() {
		try {
			return (EnumItem)getItemListAndFactory().getFirstObjectOrCreateNewIfEmpty();
		} catch (Exception e) {
			// shouldn't occur since function calls (indirectly) createItem
			throw new RuntimeException(e);
		}
	}

	public List getItemList() {
		return CollectionsHelper.asArrayList(getItemListAndFactory().getMutableListModel().iterator());
	}

	public DefaultMutableListModelAndFactory getItemListAndFactory() {
		return _itemListAndFactory;
	}

	public String getName() {
		return _name;
	}

	public void remove() {
		_changeListener = null;

		if (this != NULL_ENUM_TYPE)
			ObjectType.getInstance().removeSubType(this);
	}

	public void removeChangeListener(ChangeListener changeListenerToAdd) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, changeListenerToAdd);
	}

	public void setName(String newName) {
		_name = newName;

		fireStateChanged();
	}
}