package de.uniwue.ki.mas.basics.type.enum;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: 05.07.2002, 12:59:36
 * @author oechslein
 */
final class SwitchOnEnumerationFunction extends AbstractFunction {

	private final ISpecificEnumType _specificEnumType;
	private final IFunctionList _functionList;

	public SwitchOnEnumerationFunction(ISpecificEnumType specificEnumType, IFunctionList functionList) {
		super();

		_specificEnumType = specificEnumType;
		_functionList = functionList;
	}

	public final Object execute(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return getCaseFunction(executeFunctionArgs).execute(executeFunctionArgs);
	}

	public final void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		getCaseFunction(executeFunctionArgs).act(executeFunctionArgs);
	}

	public final boolean executeForBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return getCaseFunction(executeFunctionArgs).executeForBoolean(executeFunctionArgs);
	}

	public final double executeForDouble(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return getCaseFunction(executeFunctionArgs).executeForDouble(executeFunctionArgs);
	}

	public final int executeForInt(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return getCaseFunction(executeFunctionArgs).executeForInt(executeFunctionArgs);
	}

	private IFunction getCaseFunction(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		IFunction _specificEnumTypeFunction = _functionList.getFunction(0);

		EnumItem currEnumItem = (EnumItem) _specificEnumTypeFunction.execute(executeFunctionArgs);

		if (currEnumItem == null)
			throw new SimulationRuntimeException("Item for " + _specificEnumType.getName() + " is null.");

		int index = CollectionsHelper.indexOf(_specificEnumType.getItemList().iterator(),currEnumItem);

		if (index == -1)
			throw new SimulationRuntimeException(
				"Item '" + currEnumItem.getName() + "' does not exist in." + _specificEnumType.getName());

		return _functionList.getFunction(index + 1);
	}

	public String getName() {
		return "SwitchOnEnum " + _functionList.getName();
	}

}
