package de.uniwue.ki.mas.basics.type.enum;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithUnspecificDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.ConstraintGenericDependentType;
import de.uniwue.ki.mas.basics.type.dependent.DependentTypeBindings;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * T SwitchOnEnum(enum<T>, T,T,T)
 * 
 * Creation date: 05.07.2002, 11:37:11
 * @author oechslein
 */
final class SwitchOnEnumerationGenericFunctionDeclarationWithUnspecificDependents
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents {

	private static final ConstraintGenericDependentType _enumType =
		new ConstraintGenericDependentType("enum Type", GenericEnumType.getInstance());

	public SwitchOnEnumerationGenericFunctionDeclarationWithUnspecificDependents() {
		super("SwitchOnEnum", GenericType.getInstance(), CollectionsHelper.createList(_enumType));

		setDocumentation(
			"Depending of the value of the given enum typ (Arg1),"
				+ " it returns (or executes) the corresponding entry.");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeEntry = getNextSpecificType(inputTypes);

		if (!(specificTypeA instanceof ISpecificEnumType))
			return null;

		if (specificTypeEntry == null)
			return null;

		while (inputTypes.hasNext()) {
			ISpecificType currSpecificTypeEntry = getNextSpecificType(inputTypes);
			if (TypeManager.getInstance().isMoreGeneral(specificTypeEntry, currSpecificTypeEntry)) {
				// everything fine
			} else if (TypeManager.getInstance().isMoreGeneral(currSpecificTypeEntry, specificTypeEntry)) {
				specificTypeEntry = currSpecificTypeEntry;
			} else {
				return null;
			}
		}

		return createFunctionDeclaration((SpecificEnumType)specificTypeA, specificTypeEntry);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		SpecificEnumType specificEnumType = (SpecificEnumType)bindings.getBindingFor(_enumType);

		return createFunctionDeclaration(specificEnumType, outputType);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(
		SpecificEnumType specificEnumType,
		ISpecificType outputType) {

		return new SwitchOnEnumerationSpecificFunctionDeclarationWithUnspecificDependents(
			this,
			specificEnumType,
			outputType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		if (ObjectType.getInstance().equals(outputType)) {
			return "(" + GenericEnumType.getInstance().getName() + ", T, T, ...)";
		} else {
			String name = outputType.getName();
			return "(" + GenericEnumType.getInstance().getName() + ", " + name + ", " + name + ", ...)";
		}
	}

	protected String getFullSignatureOutputName() {
		return "T";
	}

	public boolean canCreateFunctionDeclarationWith(ISpecificType outputType) {
		return VoidType.getInstance().equals(outputType)
			|| super.canCreateFunctionDeclarationWith(outputType);
	}
}
