package de.uniwue.ki.mas.basics.type.user;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedType;
import de.uniwue.ki.mas.basics.type.enum.SpecificEnumType;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (06.12.2001 17:33:39)
 * @author Rainer Herrler
 */

class UserTypeDeclaration extends AbstractDeclaration implements IUserTypeDeclaration {
	private boolean _composedType = true;
	private String _documentation = "";
	private SpecificComposedType _specificComposedType;
	private SpecificEnumType _specificEnumType;

	private transient final ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	protected ICopyable createInstance(Map objectMapping) {
		return new UserTypeDeclaration();
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		UserTypeDeclaration dstUserTypeDeclaration = (UserTypeDeclaration)dstObject;

		dstUserTypeDeclaration._composedType = _composedType;
		dstUserTypeDeclaration._documentation = _documentation;

		if (_specificComposedType != null)
			dstUserTypeDeclaration._specificComposedType =
				(SpecificComposedType)_specificComposedType.copy(objectMapping);

		if (_specificEnumType != null)
			dstUserTypeDeclaration._specificEnumType =
				(SpecificEnumType)_specificEnumType.copy(objectMapping);

		dstUserTypeDeclaration.fireStateChanged();
	}

	public UserTypeDeclaration() {
		super();
	}

	public UserTypeDeclaration(boolean composedType) {
		super();

		_composedType = composedType;
	}

	public String getName() {
		return getSpecificType().getName();
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public String getShortDescription() {
		return getName();
	}

	public boolean getUsesShortDescription() {
		return true;
	}

	public void setShortDescription(String newDescription) {
		String oldDescription = getShortDescription();

		if (isComposedType())
			getSpecificComposedType().setName(newDescription);
		else
			getSpecificEnumType().setName(newDescription);

		firePropertyChange("shortDescription", oldDescription, newDescription);
		fireStateChanged();
	}

	public void setUsesShortDescription(boolean shortDescription) {
	}

	public String getDocumentation() {
		return _documentation;
	}

	public void setDocumentation(String documentation) {
		String oldDocumentation = getDocumentation();

		_documentation = documentation;

		firePropertyChange("documentation", oldDocumentation, documentation);
		fireStateChanged();
	}

	public SpecificComposedType getSpecificComposedType() {
		Assert.assertTrue(isComposedType());

		if (_specificComposedType == null) {
			_specificComposedType = new SpecificComposedType();
			_specificComposedType.setDeclaration(this);
			_specificComposedType.addChangeListener(_changeListener);
		}

		return _specificComposedType;
	}

	public SpecificEnumType getSpecificEnumType() {
		Assert.assertFalse(isComposedType());

		if (_specificEnumType == null) {
			_specificEnumType = new SpecificEnumType();
			_specificEnumType.addChangeListener(_changeListener);
		}

		return _specificEnumType;
	}

	public boolean isComposedType() {
		return _composedType;
	}

	public void setComposedType(boolean composedType) {
		_composedType = composedType;

		fireStateChanged();
	}

	public ISpecificType getSpecificType() {
		if (isComposedType())
			return getSpecificComposedType();
		else
			return getSpecificEnumType();
	}

	public void remove() {
		if (isComposedType()) {
			getSpecificComposedType().remove();
		} else {
			getSpecificEnumType().remove();
		}

		super.remove();
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}
}