package de.uniwue.ki.mas.behaviour.activityGraph;
import java.util.LinkedList;
import java.util.List;

import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRule;
import de.uniwue.ki.mas.entity.object.agent.IAgentExecuteFunctionArgs;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.math.RandomHelper;

/**
 * Creation date: (23.08.2001 19:41:37)
 * @author Christoph Oechslein
 */

abstract class AbstractCalculateNextActivityGraphLeaf implements ICalculateNextActivityGraphLeafAble {

	protected static final IActivityGraphLeaf calculateActivityGraphLeafInRuleList(
		IActivityGraphRule[] rules,
		IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {

		if (rules == null)
			return null;

		switch (rules.length) {
			case 0 :
				return null;

			case 1 :
				if (rules[0].isExecuteable(objectExecuteFunctionArgs))
					return rules[0].getActivityGraphLeaf();
				else
					return null;

			case 2 :
				boolean isExecuteableRuleOne = rules[0].isExecuteable(objectExecuteFunctionArgs);
				boolean isExecuteableRuleTwo = rules[1].isExecuteable(objectExecuteFunctionArgs);

				if (isExecuteableRuleOne) {
					if (!isExecuteableRuleTwo) {
						return rules[0].getActivityGraphLeaf();
					} else if (RandomHelper.getRandomBoolean()) {
						return rules[0].getActivityGraphLeaf();
					} else {
						return rules[1].getActivityGraphLeaf();
					}
				} else {
					//!isExecuteableRuleOne
					if (isExecuteableRuleTwo) {
						return rules[1].getActivityGraphLeaf();
					} else {
						return null;
					}

				}

			default :
				return calculateActivityGraphLeafInRuleListIntern(rules, objectExecuteFunctionArgs);
		}
	}

	protected static final IActivityGraphLeaf calculateActivityGraphLeafInRuleListIntern(
		IActivityGraphRule[] rules,
		IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {

		IActivityGraphRule firstExecutableRule = null;
		List restExecutableRules = null;

		for (int i = 0; i < rules.length; i++) {
			IActivityGraphRule currRuleActivity = rules[i];

			if (currRuleActivity.isExecuteable(objectExecuteFunctionArgs)) {
				if (firstExecutableRule == null) {
					firstExecutableRule = currRuleActivity;

				} else {
					if (restExecutableRules == null) {
						restExecutableRules = new LinkedList();
					}

					restExecutableRules.add(currRuleActivity);
				}
			}
		}

		if (firstExecutableRule == null) {
			return null;

		} else if (restExecutableRules == null) {
			return firstExecutableRule.getActivityGraphLeaf();

		} else {
			return ((IActivityGraphRule) RandomHelper.chooseObject(firstExecutableRule, restExecutableRules))
				.getActivityGraphLeaf();
		}
	}
}