package de.uniwue.ki.mas.behaviour.activityGraph;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineCalculationArgs;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineResult;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgentExecuteFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: (13.09.2001 15:14:04)
 * @author Christoph Oechslein
 */

final class ActivityGraph extends AbstractNamed implements IActivityGraph {
	private boolean _noopActivityGraph = false;

	private boolean _alreadyInitialized = false;
	private IActivityGraphLeaf _entryActivity;
	private Map _activityGraphLeafDeclarationToActivityGraphLeafMap;
	private final IActivityGraphDeclaration _activityGraphDeclaration;

	private final ActivityGraphLeafReasonResult _atomicActivityReasonResult =
		new ActivityGraphLeafReasonResult();

	public ActivityGraph(IActivityGraphDeclaration activityGraphDeclaration) {
		super();

		_activityGraphDeclaration = activityGraphDeclaration;
	}

	public ActivityGraph(
		IActivityGraphDeclaration activityGraphDeclaration,
		IActivityGraphLeaf entryActivityGraphLeaf,
		Map activityDeclarationToActivityMap) {

		super();

		_entryActivity = entryActivityGraphLeaf;
		_activityGraphLeafDeclarationToActivityGraphLeafMap = activityDeclarationToActivityMap;
		_activityGraphDeclaration = activityGraphDeclaration;
	}

	public IActivityReasoningEngineResult calculateReasoningEngineResult(IActivityReasoningEngineCalculationArgs activityReasoningEngineCalculationArgs)
		throws SimulationRuntimeException {

		IActivityGraphLeaf currentAtomicActivity =
			activityReasoningEngineCalculationArgs.getCurrentActivityGraphLeaf();
		IActivityAgentExecuteFunctionArgs activityAgentExecuteFunctionArgs =
			(IActivityAgentExecuteFunctionArgs)activityReasoningEngineCalculationArgs
				.getAgentExecuteFunctionArgs();

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(currentAtomicActivity);

		currentAtomicActivity.calculateResult(activityAgentExecuteFunctionArgs, _atomicActivityReasonResult);

		return _atomicActivityReasonResult;
	}

	public IActivityGraphLeaf getActivityGraphLeafForDeclaration(IActivityGraphNodeDeclaration activityDeclaration) {
		return (IActivityGraphLeaf)_activityGraphLeafDeclarationToActivityGraphLeafMap.get(
			activityDeclaration);
	}

	protected IActivityGraphDeclaration getActivityGraphDeclaration() {
		return _activityGraphDeclaration;
	}

	public IActivityGraphLeaf getEntryActivityGraphLeaf() {
		return _entryActivity;
	}

	public void init(IObjectCreateFunctionArgs objectCreateFunctionArgs) throws ModelCompilingException {
		if (_alreadyInitialized) {
			return;

		} else {
			new ActivityGraphCompiler(this, objectCreateFunctionArgs).initActivityGraph();

			_alreadyInitialized = true;
		}
	}

	void setActivityGraphLeafDeclarationToActivityGraphLeafMap(Map newActivityDeclarationToActivityMap) {
		_activityGraphLeafDeclarationToActivityGraphLeafMap = newActivityDeclarationToActivityMap;
	}

	void setEntryActivityGraphLeaf(IActivityGraphLeaf newEntryActivity) {
		_entryActivity = newEntryActivity;
	}

	public void setNoopActivityGraph(boolean noopActivityGraph) {
		_noopActivityGraph = noopActivityGraph;
	}

	public boolean isNoopActivityGraph() {
		return _noopActivityGraph;
	}

	public final String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");

		} else if (getActivityGraphDeclaration() == null) {
			buff.append("NULL");

		} else {
			getActivityGraphDeclaration().appendName(buff, currDepth, maxDepth);
			AbstractActivityGraphNode.appendName(
				buff,
				getActivityGraphDeclaration(),
				currDepth + 1,
				maxDepth);
		}
	}
}