package de.uniwue.ki.mas.behaviour.activityGraph;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.action.FunctionActionFactory;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (14.03.2002 10:59:10)
 * @author Christoph Oechslein
 */

final class ActivityGraphLeafReasonResult implements IActivityGraphLeafReasonResult {
	private IActivityGraphLeaf _nextAtomicActivity;
	private FunctionActionTwo _functionActionTwo = new FunctionActionTwo();
	private boolean _hasEnded;

	private static final class FunctionActionTwo extends AbstractFunctionAction implements IFunction {
		private IFunction _functionAction1;
		private IFunction _functionAction2;

		public FunctionActionTwo() {
			super();
		}

		public final void setFunctions(IFunction function1, IFunction function2) {
			_functionAction1 = function1;
			_functionAction2 = function2;
		}

		public final void setFunction(IFunction function) {
			setFunctions(function, null);
		}

		public final IFunction getFunction() {
			if ((_functionAction2 == null) || (FunctionActionFactory.isNoopFunction(_functionAction2))) {
				return _functionAction1;

			} else if (FunctionActionFactory.isNoopFunction(_functionAction1)) {
				return _functionAction2;

			} else
				return this;
		}

		public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
			try {
				_functionAction1.act(executeFunctionArgs);

			} catch (EmptyIteratorSimulationRuntimeException exception) {
			}

			if (_functionAction2 != null) {
				try {
					_functionAction2.act(executeFunctionArgs);

				} catch (EmptyIteratorSimulationRuntimeException exception) {
				}
			}
		}

		public String getName() {
			if ((_functionAction2 == null) || (FunctionActionFactory.isNoopFunction(_functionAction2))) {
				return _functionAction1.getName();

			} else if (FunctionActionFactory.isNoopFunction(_functionAction1)) {
				return _functionAction2.getName();

			} else
				return "[" + _functionAction1.getName() + ", " + _functionAction2.getName() + "]";
		}
	}

	public ActivityGraphLeafReasonResult() {
		super();
	}

	public IFunction getAction() {
		return _functionActionTwo.getFunction();
	}

	public IActivityGraphLeaf getNextActivityGraphLeaf() {
		return _nextAtomicActivity;
	}

	public boolean hasCurrentActivityGraphLeafEnded() {
		return _hasEnded;
	}

	public void setAction(IFunction newFunctionAction) {
		_functionActionTwo.setFunction(newFunctionAction);
	}

	public void setAction(IFunction functionAction1, IFunction functionAction2) {
		_functionActionTwo.setFunctions(functionAction1, functionAction2);
	}

	public void setCurrentActivityGraphLeafHasEnded(boolean hasEnded) {
		_hasEnded = hasEnded;
	}

	public void setNextActivityGraphLeaf(IActivityGraphLeaf nextAtomicActivity) {
		_nextAtomicActivity = nextAtomicActivity;
	}
}