package de.uniwue.ki.mas.behaviour.activityGraph;

import java.io.Serializable;

import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineCalculationArgs;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineResult;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;

/**
 * A composed activity knows what to do in this activity
 * and can translate {@link IActivityGraphNodeDeclaration activity declarations} to runtime activity objects.
 * Creation date: (11.09.2000 09:07:42)
 *
 * @see IActivityGraphLeafDeclaration
 * @author Christoph Oechslein
 */

public interface IActivityGraph extends INamed, Serializable {

	public IActivityReasoningEngineResult calculateReasoningEngineResult(IActivityReasoningEngineCalculationArgs activityReasoningEngineCalculationArgs)
		throws SimulationRuntimeException;

	/**
	 * Translate {@link IActivityGraphNodeDeclaration activity declarations} to runtime activity objects.
	 */
	IActivityGraphLeaf getActivityGraphLeafForDeclaration(IActivityGraphNodeDeclaration activityDeclaration);

	/**
	 * Must be called before other method works, compiles the graph.
	 */
	void init(IObjectCreateFunctionArgs objectCreateFunctionArgs) throws ModelCompilingException;

	IActivityGraphLeaf getEntryActivityGraphLeaf();

	boolean isNoopActivityGraph();
}
