package de.uniwue.ki.mas.behaviour.activityGraph;
import java.util.Iterator;

import junit.framework.*;
import de.uniwue.ki.mas.basics.action.rest.FunctionActionRestDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngine;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRule;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRuleDeclaration;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgentInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.ActivityAgentClassDeclarationFactory;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.sim.simulation.SimulationFunctionArgs;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (27.07.2001 09:49:16)
 * @author Christoph Oechslein
 */

public final class TestActivityBuilder extends TestCase {
	private IActivityAgentClassDeclaration _activityAgentClassDeclaration;
	private IActivityGraphDeclaration _composedActivityDeclaration;
	private IExitActivityGraphLeafDeclaration _exitActivityDecl;
	private IEntryActivityGraphLeafDeclaration _entryActivityDecl;
	private IActivityAgentInstanceDeclaration _activityAgentInstanceDeclaration;
	private IActionActivityGraphLeafDeclaration _actionActivityDecl;

	public TestActivityBuilder(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestActivityBuilder.class);
	}

	/**
	 * @throws Exception
	 */
	public void setUp() throws Exception {
		_activityAgentClassDeclaration =
			ActivityAgentClassDeclarationFactory.createActivityAgentClassDeclaration();

		_composedActivityDeclaration =
			((IActivityReasoningEngineDeclaration) _activityAgentClassDeclaration
				.getReasoningEngineDeclarationListAndFactory()
				.getDeclarationList()
				.get(0))
				.getRootActivityGraphDeclaration();

		_exitActivityDecl = _composedActivityDeclaration.getExitActivityGraphLeafDeclaration();
		_entryActivityDecl = _composedActivityDeclaration.getEntryActivityGraphLeafDeclaration();

		_actionActivityDecl =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
				_composedActivityDeclaration);

		_composedActivityDeclaration.addActivityGraphNodeDeclaration(_actionActivityDecl);

		// insert NOOP Action
		_actionActivityDecl
			.getFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.add(
				FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
					FunctionActionRestDeclarationFactory.NOOP_SPECIFIC_FUNCTION_DECL));

		// Remove all chain rules
		Iterator it =
			CollectionsHelper.asLinkedList(_entryActivityDecl.getActivityGraphEdgeDeclarations()).iterator();
		while (it.hasNext()) {
			_composedActivityDeclaration.removeDeclarationNetEdge(
				(IActivityGraphRuleDeclaration) it.next(),
				true);

		}

		// add chain rule from entry to new activity
		_composedActivityDeclaration.addActivityGraphEdgeDeclaration(_entryActivityDecl, _actionActivityDecl);

		_activityAgentInstanceDeclaration =
			_activityAgentClassDeclaration.createActivityAgentInstanceDeclaration();
	}

	/**
	 * Insert the method's description here.
	 * <BR>Creation date: (29.03.00 13:25:53)
	 * @return Test
	 */
	public static Test suite() {
		TestSuite anSuite = new TestSuite("ActivityBuilder");
		anSuite.addTest(new TestActivityBuilder("testNormalActivities"));
		return anSuite;
	}

	/**
	 * @throws Exception
	 */
	public void tearDown() throws Exception {
		_activityAgentInstanceDeclaration.remove();
		_activityAgentClassDeclaration.remove();
	}

	/**
	 * @throws Throwable
	 */
	public void testNormalActivities() throws Throwable {
		SimulationFunctionArgs simulationFunctionArgs = new SimulationFunctionArgs(null);

		IActivityAgent agent =
			(IActivityAgent) _activityAgentInstanceDeclaration.createObject(simulationFunctionArgs);
		agent.initialize(simulationFunctionArgs);

		simulationFunctionArgs.setObject(agent);

		IActivityGraph composedActivity =
			((IActivityReasoningEngine) agent.getReasoningEngineAt(0)).getActivityGraph();

		IActivityGraphLeaf entryActivity =
			composedActivity.getActivityGraphLeafForDeclaration(_entryActivityDecl);
		IActivityGraphLeaf exitActivity =
			composedActivity.getActivityGraphLeafForDeclaration(_exitActivityDecl);
		IActivityGraphLeaf actionActivity =
			composedActivity.getActivityGraphLeafForDeclaration(_actionActivityDecl);

		assertNotNull(entryActivity);
		assertNotNull(exitActivity);

		IActivityGraphLeafDeclaration dummyActivityDecl =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(null);

		assertTrue(composedActivity.getActivityGraphLeafForDeclaration(dummyActivityDecl) == null);
		assertTrue((exitActivity.getChainRules() == null) || (exitActivity.getChainRules().length == 0));
		assertEquals(
			exitActivity.getActivityGraphNodeDeclaration(),
			_composedActivityDeclaration.getExitActivityGraphLeafDeclaration());
		assertTrue(
			(exitActivity.getEmergencyRules() == null) || (exitActivity.getEmergencyRules().length == 0));
		assertEquals(1, entryActivity.getChainRules().length);
		assertEquals(
			entryActivity.getActivityGraphNodeDeclaration(),
			_composedActivityDeclaration.getEntryActivityGraphLeafDeclaration());
		assertTrue(
			(entryActivity.getEmergencyRules() == null) || (entryActivity.getEmergencyRules().length == 0));

		IActivityGraphRule activityRule = entryActivity.getChainRules()[0];
		assertTrue(activityRule.isExecuteable(simulationFunctionArgs));
		assertEquals(activityRule.getActivityGraphLeaf(), actionActivity);

		assertSame(agent, simulationFunctionArgs.getObject());

	}
}