package de.uniwue.ki.mas.behaviour.activityGraphDeclaration;
import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.net.INetEdge;
import mygale.mcge.net.INetGraphModel;
import mygale.mcge.net.INetNode;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (30.07.2001 14:18:00)
 * @author Christoph Oechslein
 */

public abstract class AbstractActivityGraphNodeDeclaration
	extends AbstractDocumentableDeclaration
	implements IActivityGraphNodeDeclaration {

	private IActivityGraphDeclaration _activityGraphDeclaration;

	public Iterator getNodeEdges() {
		if (getActivityGraphDeclaration() == null)
			return CollectionsHelper.EMPTY_ITERATOR;
		else
			return CollectionsHelper
				.select(getActivityGraphDeclaration().getEdges(), new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				INetEdge netEdge = (INetEdge)obj;
				return (netEdge.getBeginNode() == AbstractActivityGraphNodeDeclaration.this)
					|| (netEdge.getEndNode() == AbstractActivityGraphNodeDeclaration.this);
			}
		});
	}

	public AbstractActivityGraphNodeDeclaration(IActivityGraphDeclaration parentComposedActivityDeclaration) {
		this("Untitled Activity", parentComposedActivityDeclaration);
	}

	public AbstractActivityGraphNodeDeclaration(
		String name,
		IActivityGraphDeclaration parentComposedActivityDeclaration) {
		super(name);

		_activityGraphDeclaration = parentComposedActivityDeclaration;
	}
	
	public INetGraphModel getNetGraphModel() {
		return _activityGraphDeclaration;
	}

	public boolean canBeRemoved() {
		return true;
	}

	public boolean canConnect(INetEdge edge) {
		if (edge instanceof IGraphDocumentEdgeDeclaration) {
			return true;

		} else if (edge instanceof IOtherwiseActivityGraphEdgeDeclaration) {
			return !hasOtherwiseActivityGraphEdgeDeclaration();

		} else {
			return edge instanceof ActivityGraphRuleDeclaration;
		}

	}

	public boolean canConnect(INetEdge edge, INetNode node) {
		return true; //!(edge instanceof IGraphDocumentEdgeDeclaration);
	}

	public IActivityGraphDeclaration getActivityGraphDeclaration() {
		return _activityGraphDeclaration;
	}

	public Iterator getActivityGraphEdgeDeclarations() {
		if (getActivityGraphDeclaration() == null) {
			Assert.assertTrue(CollectionsHelper.isEmpty(getOutgoingNodeEdges()));

			return CollectionsHelper.EMPTY_ITERATOR;

		} else {
			return getOutgoingNodeEdges();
		}
	}

	public Iterator getIncomingNodeEdges() {
		Iterator incomingAndOutgoingEdges = getNodeEdges();
		return CollectionsHelper.select(incomingAndOutgoingEdges, new IPredicateFunction() {
			public boolean isElementOk(Object anObject) {
				return ((INetEdge)anObject).getEndNode() == AbstractActivityGraphNodeDeclaration.this;
			}
		});
	}

	public Iterator getOutgoingNodeEdges() {
		Iterator incomingAndOutgoingEdges = getNodeEdges();
		return CollectionsHelper.select(incomingAndOutgoingEdges, new IPredicateFunction() {
			public boolean isElementOk(Object anObject) {
				return ((INetEdge)anObject).getBeginNode() == AbstractActivityGraphNodeDeclaration.this;
			}
		});
	}

	private boolean hasOtherwiseActivityGraphEdgeDeclaration() {
		Iterator it = getActivityGraphEdgeDeclarations();
		while (it.hasNext()) {
			IActivityGraphEdgeDeclaration activityEdgeDeclaration = (IActivityGraphEdgeDeclaration)it.next();

			if (activityEdgeDeclaration instanceof IOtherwiseActivityGraphEdgeDeclaration)
				return true;
		}
		return false;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IActivityGraphNodeDeclaration)) {
			return false;
		}

		//IActivityDeclaration activityDeclaration = (IActivityDeclaration) anObject;

		// dont'test chainrules (test only in composedactivity)
		// return isomorphicEquals(getChainRuleDeclarations(), activityDeclaration.getChainRuleDeclarations());
		return true;
	}

	public void remove() {
		// COPY since we are removing the list
		Iterator incomingAndOutgoingEdges = CollectionsHelper.asLinkedList(getNodeEdges()).iterator();

		while (incomingAndOutgoingEdges.hasNext()) {
			IActivityGraphEdgeDeclaration activityArgumentRuleDeclaration =
				(IActivityGraphEdgeDeclaration)incomingAndOutgoingEdges.next();
			activityArgumentRuleDeclaration.remove();
		}

		// remove 1:n link
		setActivityGraphDeclaration(null);

		super.remove();
	}

	public void removed() {
	}

	public void setActivityGraphDeclaration(IActivityGraphDeclaration composedActivityDeclaration) {
		IActivityGraphDeclaration oldParentComposedActivityDeclaration =
			getActivityGraphDeclaration();
		_activityGraphDeclaration = composedActivityDeclaration;
		firePropertyChange(
			"parentComposedActivityDeclaration",
			oldParentComposedActivityDeclaration,
			composedActivityDeclaration);
	}
}