package de.uniwue.ki.mas.behaviour.activityGraphDeclaration;
import java.util.Map;

import junit.framework.Assert;
import mygale.mcge.net.INetNode;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * @author herrler
 */
public abstract class AbstractGraphEdgeDeclaration
	extends AbstractDeclaration
	implements IDeclarationNetEdge {

	private IDeclarationNetNode _fromDeclaration = null;
	private IDeclarationNetNode _toDeclaration = null;

	public INetNode getBeginNode() {
		return getFromDeclarationNetNode();
	}

	public INetNode getEndNode() {

		return getToDeclarationNetNode();
	}

	public void setBeginNode(INetNode newNode) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newNode);

		setFromDeclarationNetNode((IDeclarationNetNode) newNode);
	}

	public void setEndNode(INetNode newNode) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newNode);

		setToDeclarationNetNode((IDeclarationNetNode) newNode);
	}

	public void removed() {
	}

	public boolean canBeRemoved() {
		return true;
	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IDeclarationNetEdge inToEdgeDeclaration =
			(IDeclarationNetEdge) inToObject;

		if ((objectMapping != null) && (objectMapping.containsKey(getFromDeclarationNetNode()))) {
			inToEdgeDeclaration.setFromDeclarationNetNode(
				(IDeclarationNetNode) objectMapping.get(getFromDeclarationNetNode()));

			inToEdgeDeclaration.setToDeclarationNetNode(
				(IDeclarationNetNode) objectMapping.get(getToDeclarationNetNode()));

		} else {
			inToEdgeDeclaration.setFromDeclarationNetNode(
				getFromDeclarationNetNode());
			inToEdgeDeclaration.setToDeclarationNetNode(
				getToDeclarationNetNode());
		}
	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (!super.isomorphicEquals(obj)) {
			return false;
		}

		if (!(obj instanceof IDeclarationNetEdge))
			return false;

		IDeclarationNetEdge edgeDeclaration = (IDeclarationNetEdge) obj;

		return getFromDeclarationNetNode().isomorphicEquals(
			edgeDeclaration.getFromDeclarationNetNode())
			&& getToDeclarationNetNode().isomorphicEquals(
				edgeDeclaration.getToDeclarationNetNode());
	}

	public int isomorphicHashCode() {
		return super.isomorphicHashCode()
			+ 3 * getFromDeclarationNetNode().isomorphicHashCode()
			+ 5 * getToDeclarationNetNode().isomorphicHashCode();
	}

	public void remove() {
		super.remove();
		removed();
	}

	public void setFromDeclarationNetNode(IDeclarationNetNode fromDeclaration) {
		
		String name = fromDeclaration != null ? fromDeclaration.getName() : "null";
		
		Assert.assertTrue("Edge from Declaration "+ name + " to itself",
			(fromDeclaration == null)
				|| (fromDeclaration != getToDeclarationNetNode()));
			
		IDeclarationNetNode oldValue = _fromDeclaration;

		_fromDeclaration = fromDeclaration;

		if (!Utils.equals(oldValue, fromDeclaration)) {
			firePropertyChange("fromDeclaration", oldValue, fromDeclaration);

			fireStateChanged();
		}
	}

	public void setToDeclarationNetNode(IDeclarationNetNode toDeclaration) {
		
		String name = toDeclaration != null ? toDeclaration.getName() : "null";
		
		Assert.assertTrue("Edge from Declaration "+ name + " to itself",
			(toDeclaration == null)
				|| (toDeclaration != getFromDeclarationNetNode()));
				
		IDeclarationNetNode oldValue = _toDeclaration;

		_toDeclaration = toDeclaration;

		if (!Utils.equals(oldValue, toDeclaration)) {
			firePropertyChange("toDeclaration", oldValue, toDeclaration);

			fireStateChanged();
		}
	}
	
	public IDeclarationNetNode getToDeclarationNetNode() {
		return _toDeclaration;
	}
	
	public IDeclarationNetNode getFromDeclarationNetNode() {
		return _fromDeclaration;
	}
}