package de.uniwue.ki.mas.behaviour.activityGraphDeclaration;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationListAndFactory;
import de.uniwue.ki.mas.basics.action.FunctionActionFactory;
import de.uniwue.ki.mas.behaviour.activityGraph.ActivityFactory;
import de.uniwue.ki.mas.behaviour.activityGraph.IActivityGraphLeaf;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (11.03.2002 14:36:52)
 * @author Christoph Oechslein
 */

class ActionActivityGraphLeafDeclaration
	extends AbstractActivityGraphLeafDeclaration
	implements IActionActivityGraphLeafDeclaration {

	private IActivityGraphNodeDeclarationTypeable.Type _type = ACTIVITY_LIKE;

	private boolean _instantlyActivityDeclaration = false;

	private final FunctionActionCallDeclarationListAndFactory _exitFunctionActionCallDeclarationListAndFactory =
		new FunctionActionCallDeclarationListAndFactory("Exit Actions");
	private final FunctionActionCallDeclarationListAndFactory _functionActionCallDeclarationListAndFactory =
		new FunctionActionCallDeclarationListAndFactory("Actions");
	private final FunctionActionCallDeclarationListAndFactory _entryFunctionActionCallDeclarationListAndFactory =
		new FunctionActionCallDeclarationListAndFactory("Entry Actions");

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	public ActionActivityGraphLeafDeclaration(IActivityGraphDeclaration parentComposedActivityDeclaration) {
		super(parentComposedActivityDeclaration);

		getFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.addChangeListener(
			_changeListener);
		getEntryFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.addChangeListener(_changeListener);
		getExitFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.addChangeListener(
			_changeListener);
	}

	public void copyInto(Object inToObject, java.util.Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IActionActivityGraphLeafDeclaration inActionAtomicActivityDeclaration =
			(IActionActivityGraphLeafDeclaration) inToObject;

		inActionAtomicActivityDeclaration.setType(getType());
		inActionAtomicActivityDeclaration.setInstantly(isInstantly());

		getEntryFunctionActionCallDeclarationListAndFactory().copyInto(
			inActionAtomicActivityDeclaration.getEntryFunctionActionCallDeclarationListAndFactory(),
			objectMapping);
		getFunctionActionCallDeclarationListAndFactory().copyInto(
			inActionAtomicActivityDeclaration.getFunctionActionCallDeclarationListAndFactory(),
			objectMapping);
		getExitFunctionActionCallDeclarationListAndFactory().copyInto(
			inActionAtomicActivityDeclaration.getExitFunctionActionCallDeclarationListAndFactory(),
			objectMapping);
	}

	public IActivityGraphLeaf createActivityGraphLeaf(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		return ActivityFactory.createActionAtomicActivity(
			this,
			isInstantly(),
			FunctionActionFactory.createFunctionAction(
				getEntryFunctionActionCallDeclarationListAndFactory().getFunctionActionCallDeclarationList(),
				objectCreateFunctionArgs),
			FunctionActionFactory.createFunctionAction(
				getFunctionActionCallDeclarationListAndFactory().getFunctionActionCallDeclarationList(),
				objectCreateFunctionArgs),
			FunctionActionFactory.createFunctionAction(
				getExitFunctionActionCallDeclarationListAndFactory().getFunctionActionCallDeclarationList(),
				objectCreateFunctionArgs));
	}

	public ICopyable createInstance(Map objectMapping) {
		return ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
			getActivityGraphDeclaration());
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	public final FunctionActionCallDeclarationListAndFactory getEntryFunctionActionCallDeclarationListAndFactory() {
		return _entryFunctionActionCallDeclarationListAndFactory;
	}

	public final FunctionActionCallDeclarationListAndFactory getExitFunctionActionCallDeclarationListAndFactory() {
		return _exitFunctionActionCallDeclarationListAndFactory;
	}

	public final FunctionActionCallDeclarationListAndFactory getFunctionActionCallDeclarationListAndFactory() {
		return _functionActionCallDeclarationListAndFactory;
	}

	public IActivityGraphNodeDeclarationTypeable.Type getType() {
		return _type;
	}

	public final boolean isInstantly() {
		return _instantlyActivityDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IActionActivityGraphLeafDeclaration)) {
			return false;
		}

		IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration =
			(IActionActivityGraphLeafDeclaration) anObject;

		return (isInstantly() == actionAtomicActivityDeclaration.isInstantly())
			&& getType().equals(actionAtomicActivityDeclaration.getType())
			&& getEntryFunctionActionCallDeclarationListAndFactory().isomorphicEquals(
				actionAtomicActivityDeclaration.getEntryFunctionActionCallDeclarationListAndFactory())
			&& getFunctionActionCallDeclarationListAndFactory().isomorphicEquals(
				actionAtomicActivityDeclaration.getFunctionActionCallDeclarationListAndFactory())
			&& getExitFunctionActionCallDeclarationListAndFactory().isomorphicEquals(
				actionAtomicActivityDeclaration.getExitFunctionActionCallDeclarationListAndFactory());
	}
	
	public int isomorphicHashCode() {
		return 11 * super.isomorphicHashCode()
			+ 7 * getEntryFunctionActionCallDeclarationListAndFactory().isomorphicHashCode()
			+ 5 * getFunctionActionCallDeclarationListAndFactory().isomorphicHashCode()
			+ 3 * getExitFunctionActionCallDeclarationListAndFactory().isomorphicHashCode()
			+ isomorphicHashCode(isInstantly())
			+ getType().hashCode();
	}

	public void remove() {
		getFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.removeChangeListener(
			_changeListener);
		getEntryFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.removeChangeListener(_changeListener);
		getExitFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.removeChangeListener(_changeListener);

		getEntryFunctionActionCallDeclarationListAndFactory().remove();
		getFunctionActionCallDeclarationListAndFactory().remove();
		getExitFunctionActionCallDeclarationListAndFactory().remove();

		super.remove();
	}

	public void setInstantly(boolean newIsInstantlyActivityDeclaration) {
		final boolean oldIsInstantlyActivityDeclaration = isInstantly();
		_instantlyActivityDeclaration = newIsInstantlyActivityDeclaration;
		firePropertyChange(
			"instantlyActivityDeclaration",
			oldIsInstantlyActivityDeclaration,
			newIsInstantlyActivityDeclaration);
		fireStateChanged();
	}

	public void setType(IActivityGraphNodeDeclarationTypeable.Type newType) {
		if ((newType != ACTIVITY_LIKE)
			&& (newType != STATE_LIKE)
			&& (newType != SIGNAL_IN_LIKE)
			&& (newType != SIGNAL_OUT_LIKE)) {
			throw new IllegalArgumentException("Must be any of ACTIVITY_LIKE, STATE_LIKE, SIGNAL_IN_LIKE or SIGNAL_OUT_LIKE");
		}

		IActivityGraphNodeDeclarationTypeable.Type oldType = getType();

		_type = newType;

		firePropertyChange("type", oldType, newType);
		fireStateChanged();
	}
}