package de.uniwue.ki.mas.behaviour.activityGraphDeclaration;

import java.util.Iterator;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * @author oechslein
 * 
 * Creation date: 11.07.2003
 */
final class ActivityGraphComposedNodeDeclaration
	extends AbstractActivityGraphNodeDeclaration
	implements IActivityGraphComposedNodeDeclaration {

	private IActivityGraphNodeDeclarationTypeable.Type _type = ACTIVITY_LIKE;
	private final IActivityGraphDeclaration _innerActivityGraphDeclaration =
		new ActivityGraphDeclaration(this);
		
	private transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent arg0) {
			fireStateChanged();
		}
	};

	public ActivityGraphComposedNodeDeclaration() {
		super(null);

		_innerActivityGraphDeclaration.setUsesShortDescription(getUsesShortDescription());
		_innerActivityGraphDeclaration.setShortDescription(getShortDescription());
		_innerActivityGraphDeclaration.setDocumentation(getDocumentation());
		
		_innerActivityGraphDeclaration.addChangeListener(_changeListener);
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		IActivityGraphComposedNodeDeclaration dstActivityGraphComposedNodeDeclaration =
			(IActivityGraphComposedNodeDeclaration)dstObject;

		dstActivityGraphComposedNodeDeclaration.setType(getType());

		_innerActivityGraphDeclaration.copyInto(
			dstActivityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration(),
			objectMapping);
	}

	public String getDocumentation() {
		return _innerActivityGraphDeclaration.getDocumentation();
	}

	public final IActivityGraphDeclaration getInnerActivityGraphDeclaration() {
		return _innerActivityGraphDeclaration;
	}

	public String getName() {
		return _innerActivityGraphDeclaration.getName();
	}

	public String getShortDescription() {
		return _innerActivityGraphDeclaration.getShortDescription();
	}

	public boolean getUsesShortDescription() {
		return _innerActivityGraphDeclaration.getUsesShortDescription();
	}

	public boolean isInstantly() {
		Iterator allActivities = _innerActivityGraphDeclaration.getNodes();

		while (allActivities.hasNext()) {
			IActivityGraphNodeDeclaration currIActivityDeclaration =
				(IActivityGraphNodeDeclaration)allActivities.next();

			if (currIActivityDeclaration
				!= _innerActivityGraphDeclaration.getEmergencyActivityGraphLeafDeclaration()) {
				if (!currIActivityDeclaration.isInstantly()) {
					return false;
				}
			}
		}

		return true;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return _innerActivityGraphDeclaration.isomorphicEquals(anObject);
	}

	public int isomorphicHashCode() {
		return _innerActivityGraphDeclaration.isomorphicHashCode();
	}

	public void remove() {
		_innerActivityGraphDeclaration.remove();
	}

	public void setDocumentation(String newDocumentation) {
		_innerActivityGraphDeclaration.setDocumentation(newDocumentation);
	}

	public void setShortDescription(String newDescription) {
		if (_innerActivityGraphDeclaration == null) {
			//during init
			super.setShortDescription(newDescription);
		} else {
			_innerActivityGraphDeclaration.setShortDescription(newDescription);

			fireStateChanged();
		}
	}

	public void setUsesShortDescription(boolean shortDescription) {
		if (_innerActivityGraphDeclaration == null) {
			//during init
			super.setUsesShortDescription(shortDescription);
		} else {
			_innerActivityGraphDeclaration.setUsesShortDescription(shortDescription);

			fireStateChanged();
		}
	}

	public final void setType(IActivityGraphNodeDeclarationTypeable.Type newType) {
		IActivityGraphNodeDeclarationTypeable.Type oldType = getType();
		_type = newType;

		firePropertyChange("type", oldType, newType);
		fireStateChanged();
	}

	public final IActivityGraphNodeDeclarationTypeable.Type getType() {
		return _type;
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new ActivityGraphComposedNodeDeclaration();
	}
}
