package de.uniwue.ki.mas.behaviour.activityGraphDeclaration;
import mygale.mcge.net.INetNode;
import mygale.mcge.net.INetNodeFactory;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (30.07.2001 17:59:57)
 * @author Christoph Oechslein
 */

public final class ActivityGraphComposedNodeDeclarationFactory
	extends AbstractDeclarationFactory
	implements IActivityGraphNodeDeclarationFactory, INetNodeFactory {

	private static final ActivityGraphComposedNodeDeclarationFactory _instance =
		new ActivityGraphComposedNodeDeclarationFactory();

	private final IActivityGraphNodeDeclarationTypeable.Type _type;
	private final boolean _addIdleActivity;

	private ActivityGraphComposedNodeDeclarationFactory() {
		this(IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE, false);
	}

	public ActivityGraphComposedNodeDeclarationFactory(
		IActivityGraphNodeDeclarationTypeable.Type type,
		boolean addIdleActivity) {
		super();

		_type = type;
		_addIdleActivity = addIdleActivity;
	}

	public ActivityGraphComposedNodeDeclarationFactory(boolean addIdleActivity) {
		this(IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE, addIdleActivity);
	}

	public final IActivityGraphNodeDeclaration createActivityGraphNodeDeclaration() {
		return createActivityGraphComposedNodeDeclaration(_type, _addIdleActivity);
	}

	public static final IActivityGraphComposedNodeDeclaration createActivityGraphComposedNodeDeclaration() {
		return new ActivityGraphComposedNodeDeclaration();
	}

	public static final IActivityGraphComposedNodeDeclaration createActivityGraphComposedNodeDeclaration(
		IActivityGraphNodeDeclarationTypeable.Type type,
		boolean addIdleActivity) {

		IActivityGraphComposedNodeDeclaration newActivityGraphComposedNodeDeclaration =
			createActivityGraphComposedNodeDeclaration();

		if (addIdleActivity)
			ActivityGraphDeclarationFactory.addIdleActivity(
				newActivityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration());

		newActivityGraphComposedNodeDeclaration.setType(type);

		return newActivityGraphComposedNodeDeclaration;
	}

	public final IDeclaration createDeclaration() {
		return createActivityGraphNodeDeclaration();
	}

	public final INetNode createNetNode() {
		return createActivityGraphNodeDeclaration();
	}

	public static final ActivityGraphComposedNodeDeclarationFactory getInstance() {
		return _instance;
	}
}