package de.uniwue.ki.mas.behaviour.reasoning;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (30.03.00 15:49:27)
 * @author Christoph Oechslein
 */

final public class ActivityReasoningEngineDeclaration
	extends AbstractReasoningEngineDeclaration
	implements IActivityReasoningEngineDeclaration, PropertyChangeListener, ChangeListener {

	private final IActivityGraphDeclaration _composedActivityDeclaration;

	public ActivityReasoningEngineDeclaration(IActivityGraphDeclaration rootComposedActivityDeclaration) {
		this(null, rootComposedActivityDeclaration);
	}

	public ActivityReasoningEngineDeclaration(
		String name,
		IActivityGraphDeclaration rootComposedActivityDeclaration) {
		super();

		_composedActivityDeclaration = rootComposedActivityDeclaration;
		addListenerToComposedActivityDeclaration();

		if (name != null) {
			getRootActivityGraphDeclaration().setUsesShortDescription(true);
			getRootActivityGraphDeclaration().setShortDescription(name);
		}
	}

	protected void addListenerToComposedActivityDeclaration() {
		getRootActivityGraphDeclaration().addPropertyChangeListener(this);
		getRootActivityGraphDeclaration().addChangeListener(this);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IActivityReasoningEngineDeclaration inActivityReasoningEngineDeclaration =
			(IActivityReasoningEngineDeclaration)intoObject;

		getRootActivityGraphDeclaration().copyInto(
			inActivityReasoningEngineDeclaration.getRootActivityGraphDeclaration(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return ActivityReasoningEngineDeclarationFactory.createActivityReasoningEngineDeclaration();
	}

	protected IReasoningEngine createNewReasoningEngine(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		return new ActivityReasoningEngine(this, objectCreateFunctionArgs);
	}

	public final IActivityGraphDeclaration getRootActivityGraphDeclaration() {
		return _composedActivityDeclaration;
	}

	public String getDocumentation() {
		return getRootActivityGraphDeclaration().getDocumentation();
	}

	public String getName() {
		return getRootActivityGraphDeclaration().getName();
	}

	public String getShortDescription() {
		return getRootActivityGraphDeclaration().getShortDescription();
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IActivityReasoningEngineDeclaration)) {
			return false;
		}

		IActivityReasoningEngineDeclaration activityReasoningEngineDeclaration =
			(IActivityReasoningEngineDeclaration)anObject;

		return isomorphicEquals(
			getRootActivityGraphDeclaration(),
			activityReasoningEngineDeclaration.getRootActivityGraphDeclaration());
	}

	public int isomorphicHashCode() {
		return 3 * super.isomorphicHashCode() + getRootActivityGraphDeclaration().isomorphicHashCode();
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getSource() == getRootActivityGraphDeclaration()) {
			String propertyName = evt.getPropertyName();
			if ("name".equals(propertyName)) {
				firePropertyChange("name", evt.getOldValue(), evt.getNewValue());

			} else {
				if ("documentation".equals(propertyName)) {
					firePropertyChange("documentation", evt.getOldValue(), evt.getNewValue());
				} else {
				}
			}
		}
	}

	public void remove() {
		// remove 1:1 link
		removeListenerToComposedActivityDeclaration();
		remove(getRootActivityGraphDeclaration());

		super.remove();
	}

	protected void removeListenerToComposedActivityDeclaration() {
		getRootActivityGraphDeclaration().removePropertyChangeListener(this);
		getRootActivityGraphDeclaration().removeChangeListener(this);
	}

	public void setDocumentation(String newDocumentation) {
		// final Document oldDocumentation = getComposedActivityDeclaration().getDocumentation();
		getRootActivityGraphDeclaration().setDocumentation(newDocumentation);
		// firePropertyChange("documentation", oldDocumentation, newDocumentation);
		// this listen to changes
	}

	public void setShortDescription(String newShortDescription) {
		// final String oldShortDescription = ((IShortDescriptiond)getComposedActivityDeclaration()).getShortDescription();
		getRootActivityGraphDeclaration().setShortDescription(newShortDescription);
		// firePropertyChange("shortDescription", oldShortDescription, newShortDescription);
		// this listen to changes
	}

	/**
	 * @param e ChangeEvent
	 * @see javax.swing.event.ChangeListener#stateChanged(ChangeEvent)
	 */
	public void stateChanged(ChangeEvent e) {
		fireStateChanged();
	}
}