package de.uniwue.ki.mas.behaviour.rule;
import java.util.Map;

import junit.framework.Assert;
import mygale.mcge.net.INetNode;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.AbstractGraphEdgeDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: 04.11.2002, 22:22:39
 * @author oechslein
 */
abstract class AbstractActivityGraphEdgeDeclaration
	extends AbstractGraphEdgeDeclaration
	implements IActivityGraphEdgeDeclaration {

	public INetNode getBeginNode() {

		return getFromActivityGraphNodeDeclaration();
	}

	public INetNode getEndNode() {

		return getToActivityGraphNodeDeclaration();
	}

	public void setBeginNode(INetNode newNode) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newNode);

		setFromActivityGraphNodeDeclaration((IActivityGraphNodeDeclaration) newNode);
	}

	public void setEndNode(INetNode newNode) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newNode);

		setToActivityGraphNodeDeclaration((IActivityGraphNodeDeclaration) newNode);
	}

	public void removed() {
	}

	public boolean canBeRemoved() {
		return true;
	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IActivityGraphEdgeDeclaration inToActivityEdgeDeclaration =
			(IActivityGraphEdgeDeclaration) inToObject;

		if ((objectMapping != null) && (objectMapping.containsKey(getFromActivityGraphNodeDeclaration()))) {
			inToActivityEdgeDeclaration.setFromActivityGraphNodeDeclaration(
				(IActivityGraphNodeDeclaration) objectMapping.get(getFromActivityGraphNodeDeclaration()));

			inToActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(
				(IActivityGraphNodeDeclaration) objectMapping.get(getToActivityGraphNodeDeclaration()));

		} else {
			inToActivityEdgeDeclaration.setFromActivityGraphNodeDeclaration(
				getFromActivityGraphNodeDeclaration());
			inToActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(
				getToActivityGraphNodeDeclaration());
		}
	}

	public IActivityGraphNodeDeclaration getFromActivityGraphNodeDeclaration() {
		return (IActivityGraphNodeDeclaration)getFromDeclarationNetNode();
	}

	public IActivityGraphNodeDeclaration getToActivityGraphNodeDeclaration() {
		return (IActivityGraphNodeDeclaration)getToDeclarationNetNode();
	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (!super.isomorphicEquals(obj)) {
			return false;
		}

		if (!(obj instanceof IActivityGraphEdgeDeclaration))
			return false;

		IActivityGraphEdgeDeclaration activityEdgeDeclaration = (IActivityGraphEdgeDeclaration) obj;

		return getFromActivityGraphNodeDeclaration().isomorphicEquals(
			activityEdgeDeclaration.getFromActivityGraphNodeDeclaration())
			&& getToActivityGraphNodeDeclaration().isomorphicEquals(
				activityEdgeDeclaration.getToActivityGraphNodeDeclaration());
	}

	public int isomorphicHashCode() {
		return super.isomorphicHashCode()
			+ 3 * getFromActivityGraphNodeDeclaration().isomorphicHashCode()
			+ 5 * getToActivityGraphNodeDeclaration().isomorphicHashCode();
	}

	public void remove() {
		super.remove();

		removed();
	}

	public void setFromActivityGraphNodeDeclaration(IActivityGraphNodeDeclaration fromActivityDeclaration) {
		
		IActivityGraphNodeDeclaration oldValue = getFromActivityGraphNodeDeclaration();

		setFromDeclarationNetNode(fromActivityDeclaration);

		if (!Utils.equals(oldValue, fromActivityDeclaration)) {
			firePropertyChange("fromActivityDeclaration", oldValue, fromActivityDeclaration);
			//fireStateChanged();
		}
	}

	public void setToActivityGraphNodeDeclaration(IActivityGraphNodeDeclaration toActivityDeclaration) {
		IActivityGraphNodeDeclaration oldValue = getToActivityGraphNodeDeclaration();

		setToDeclarationNetNode(toActivityDeclaration);

		if (!Utils.equals(oldValue, toActivityDeclaration)) {
			firePropertyChange("toActivityDeclaration", oldValue, toActivityDeclaration);
			//fireStateChanged();
		}
	}
}