package de.uniwue.ki.mas.behaviour.rule;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (17.09.2001 19:39:37)
 * @author Christoph Oechslein
 */
final public class ActivityGraphRuleDeclaration
	extends AbstractActivityGraphEdgeDeclaration
	implements IActivityGraphRuleDeclaration {

	private final IFunctionCallDeclaration _conditionFunctionCallDeclaration =
		FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(BooleanType.getInstance());

	private boolean _usesShortDescription = false;
	private String _shortDescription = "";
	private String _documentation = "";

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	public ActivityGraphRuleDeclaration() {
		super();

		getConditionFunctionCallDeclaration().addChangeListener(_changeListener);
	}

	private void copyDocumentablePartInto(
		IActivityGraphRuleDeclaration intoActivityRuleDeclaration,
		Map objectMapping) {
		intoActivityRuleDeclaration.setUsesShortDescription(getUsesShortDescription());
		intoActivityRuleDeclaration.setShortDescription(getShortDescription());
		intoActivityRuleDeclaration.setDocumentation(getDocumentation());
	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IActivityGraphRuleDeclaration inRuleDeclaration = (IActivityGraphRuleDeclaration) inToObject;

		copyDocumentablePartInto(inRuleDeclaration, objectMapping);

		getConditionFunctionCallDeclaration().copyInto(
			inRuleDeclaration.getConditionFunctionCallDeclaration(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new ActivityGraphRuleDeclaration();
	}

	public IRule createRule(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return new ActivityRuleDeclarationCompiler(this, createFunctionArgs).createRule();
	}

	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		super.firePropertyChange(propertyName, oldValue, newValue);
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	protected String getDefaultName() {
		// return getName(getPreconditionFunctionCallDeclaration()) + " [" + getName(getStochasticFunctionCallDeclaration()) + "]";

		// return super.getDefaultName() + " -> " + getToActivityDeclaration().getName();

		StringBuffer buff = new StringBuffer();

		buff.append(getConditionFunctionCallDeclaration().getName());
		buff.append(" ");

		return buff.toString();
	}

	public String getDocumentation() {
		return _documentation;
	}

	public String getName() {
		return getUsesShortDescription() ? getShortDescription() : getDefaultName();
	}

	public IFunctionCallDeclaration getConditionFunctionCallDeclaration() {
		return _conditionFunctionCallDeclaration;
	}

	public String getShortDescription() {
		return _shortDescription;
	}

	public boolean getUsesShortDescription() {
		return _usesShortDescription;
	}

	public boolean isomorphicEquals(IIsomorphicHashable obj) {
		if (this == obj)
			return true;

		if (!(obj instanceof IActivityGraphRuleDeclaration))
			return false;

		IActivityGraphRuleDeclaration activityRuleDeclaration = (IActivityGraphRuleDeclaration) obj;

		// Rainer 27. Nov 2003
		// 
		// return getConditionFunctionCallDeclaration().isomorphicEquals(
		//	activityRuleDeclaration.getConditionFunctionCallDeclaration());

		return (getConditionFunctionCallDeclaration().isomorphicEquals(
			activityRuleDeclaration.getConditionFunctionCallDeclaration()) && super.isomorphicEquals(obj));
	}

	public int isomorphicHashCode() {
		// return getConditionFunctionCallDeclaration().isomorphicHashCode();
		return getConditionFunctionCallDeclaration().isomorphicHashCode() + (31*super.isomorphicHashCode());
	}

	protected static boolean isTrueFunctionCallDeclaration(IFunctionCallDeclaration functionCallDeclaration) {

		IFunctionCallInternDeclaration functionCallInternDeclaration =
			functionCallDeclaration.getFunctionCallInternDeclaration();

		if (functionCallInternDeclaration instanceof IAtomicFunctionCallInternDeclaration) {
			Object value =
				((IAtomicFunctionCallInternDeclaration) functionCallInternDeclaration)
					.getArgumentDeclaration()
					.getObject();
			return ((Boolean) value).booleanValue();
		} else {
			return false;
		}
	}

	public boolean isUsesShortDescription() {
		return _usesShortDescription;
	}

	public void remove() {
		// remove 1:1 link
		remove(getConditionFunctionCallDeclaration());

		super.remove();
	}

	public void setShortDescription(String shortDescription) {
		String oldValue = _shortDescription;

		if (!Utils.equals(oldValue, shortDescription)) {
			_shortDescription = shortDescription;
			firePropertyChange("shortDescription", oldValue, shortDescription);
			fireStateChanged();
		}
	}

	public void setUsesShortDescription(boolean usesShortDescription) {
		boolean oldValue = getUsesShortDescription();

		if (oldValue != usesShortDescription) {
			_usesShortDescription = usesShortDescription;
			firePropertyChange("usesShortDescription", oldValue, usesShortDescription);
			fireStateChanged();
		}
	}

	public void setDocumentation(String documentation) {
		_documentation = documentation;
	}
}