package de.uniwue.ki.mas.behaviour.rule;
import mygale.mcge.net.INetEdge;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (04.09.2001 16:05:27)
 * @author Christoph Oechslein
 */
public final class ActivityGraphRuleDeclarationFactory
	extends AbstractDeclarationFactory
	implements IActivityGraphEdgeDeclarationFactory {

	private static final ActivityGraphRuleDeclarationFactory _instance =
		new ActivityGraphRuleDeclarationFactory();

	private ActivityGraphRuleDeclarationFactory() {
		super();
	}

	public final IActivityGraphEdgeDeclaration createActivityGraphEdgeDeclaration() {
		return new ActivityGraphRuleDeclaration();
	}

	public static IActivityGraphRuleDeclaration createActivityGraphRuleDeclaration() {
		return new ActivityGraphRuleDeclaration();
	}

	public static IActivityGraphRuleDeclaration createActivityGraphRuleDeclaration(
		IActivityGraphNodeDeclaration fromActivityDeclaration,
		IActivityGraphNodeDeclaration toActivityDeclaration) {

		IActivityGraphRuleDeclaration activityRuleDeclaration = new ActivityGraphRuleDeclaration();

		activityRuleDeclaration.setFromActivityGraphNodeDeclaration(fromActivityDeclaration);
		activityRuleDeclaration.setToActivityGraphNodeDeclaration(toActivityDeclaration);

		return activityRuleDeclaration;
	}

	static IActivityGraphRuleDeclaration createActivityGraphRuleDeclaration(
		IActivityGraphNodeDeclaration fromActivityDeclaration,
		IActivityGraphNodeDeclaration toActivityDeclaration,
		IFunctionCallDeclaration preconditionCall) {

		IActivityGraphRuleDeclaration activityRuleDeclaration =
			createActivityGraphRuleDeclaration(fromActivityDeclaration, toActivityDeclaration);

		activityRuleDeclaration.getConditionFunctionCallDeclaration().setFunctionCallInternDeclaration(
			preconditionCall.getFunctionCallInternDeclaration());

		return activityRuleDeclaration;
	}

	public final IDeclaration createDeclaration() {
		return createActivityGraphEdgeDeclaration();
	}

	public final INetEdge createNetEdge() {
		return createActivityGraphEdgeDeclaration();
	}

	public static final ActivityGraphRuleDeclarationFactory getInstance() {
		return _instance;
	}
}