package de.uniwue.ki.mas.behaviour.rule;
import java.io.Serializable;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraph.IActivityGraphLeaf;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngine;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * Creation date: (14.03.2002 09:30:06)
 * @author Christoph Oechslein
 */

final class ActivityRuleDeclarationCompiler implements Serializable {

	private IActivityGraphRuleDeclaration _activityRuleDeclaration;
	private Boolean _preconditionValue;
	private IFunction _preconditionFunction;

	private ICreateFunctionArgs _createFunctionArgs;

	public ActivityRuleDeclarationCompiler(
		IActivityGraphRuleDeclaration activityRuleDeclaration,
		ICreateFunctionArgs createFunctionArgs) {

		super();

		_createFunctionArgs = createFunctionArgs;
		_activityRuleDeclaration = activityRuleDeclaration;

	}

	private IFunction createFunction(IFunctionCallDeclaration functionCallDeclaration)
		throws ModelCompilingException {
		return functionCallDeclaration.createFunction(_createFunctionArgs);
	}

	private IRule createNewRule() throws ModelCompilingException {
		try {
			if (isAlwaysNotExecutableRule()) {
				return null;

			} else if (isPreconditionPartIsConstant()) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(true, getPreconditionValue());
				return new ActivityGraphTrueRule(getAtomicActivity());

			} else {
				return new ActivityGraphConditionRule(getPreconditionFunction(), getAtomicActivity());
			}

		} catch (SimulationRuntimeException exception) {
			throw new ModelCompilingException(exception);
		}
	}

	/**
	 * 24.Mai 2004 Rainer: changed Rule from isomorph-tests to Object.equal-Test
	 * because isomorph test of activities do not include rule conditions
	 */
	public IRule createRule() throws ModelCompilingException {
		if (_createFunctionArgs.existsCompilationObject(getActivityRuleDeclaration(), false))
			return (IRule) _createFunctionArgs.getCompilationObject(getActivityRuleDeclaration(), false);
		else {
			try {
				IRule newRule = createNewRule();
				_createFunctionArgs.addCompilationObject(getActivityRuleDeclaration(), newRule, false);
				return newRule;

			} catch (ModelCompilingException exception) {
				throw new ActivityRuleDeclarationCompilingException(getActivityRuleDeclaration(), exception);
			}
		}
	}

	private IActivityGraphLeaf getAtomicActivity() throws ModelCompilingException {

		IActivityAgent activityAgent =
			(IActivityAgent) ((IObjectCreateFunctionArgs) _createFunctionArgs).getObject();

		for (int i = 0; i < activityAgent.getReasoningEngineCount(); i++) {
			IActivityReasoningEngine activityReasoningEngine =
				(IActivityReasoningEngine) activityAgent.getReasoningEngineAt(i);

			IActivityGraphLeaf atomicActivity =
				activityReasoningEngine.getAtomicActivityForDeclaration(
					getActivityRuleDeclaration().getToActivityGraphNodeDeclaration());

			if (atomicActivity != null) {
				return atomicActivity;
			}
		}

		Assert.fail();

		return null;
	}

	private IFunction getPreconditionFunction() throws ModelCompilingException {
		if (_preconditionFunction == null) {
			_preconditionFunction =
				createFunction(_activityRuleDeclaration.getConditionFunctionCallDeclaration());
		}
		return _preconditionFunction;
	}

	private boolean getPreconditionValue() throws SimulationRuntimeException, ModelCompilingException {
		if (_preconditionValue == null) {
			_preconditionValue =
				Boolean.valueOf(
					getPreconditionFunction().executeForBoolean(
						_createFunctionArgs.createCreateAndExecuteFunctionArgs()));
		}
		return _preconditionValue.booleanValue();
	}

	private IActivityGraphRuleDeclaration getActivityRuleDeclaration() {
		return _activityRuleDeclaration;
	}

	private boolean isAlwaysNotExecutableRule() throws SimulationRuntimeException, ModelCompilingException {
		return (isPreconditionPartIsConstant() && getPreconditionValue() == false);
	}

	private boolean isPreconditionPartIsConstant() throws ModelCompilingException {
		return _createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& _activityRuleDeclaration.getConditionFunctionCallDeclaration().isConstant(_createFunctionArgs);
	}
}