package de.uniwue.ki.mas.behaviour.rule;
import junit.framework.Test;
import junit.framework.TestCase;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.BooleanType;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.object.agent.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.ActivityAgentClassDeclarationFactory;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.SimulationFunctionArgs;

/**
 * Creation date: (14.03.2002 09:10:14)
 * @author Christoph Oechslein
 */

public final class TestActivityRuleDeclaration extends TestCase {
	private IFunctionCallDeclaration _alwaysTrueFunctionCallDecl;
	private IFunctionCallDeclaration _alwaysFalseFunctionCallDecl;
	private IActivityGraphRuleDeclaration _trueActivityArgumentRuleDeclaration;
	private IActivityGraphRuleDeclaration _falseActivityArgumentRuleDeclaration;
	private IActivityGraphNodeDeclaration _fromActivityDeclaration;
	private IActivityGraphNodeDeclaration _toActivityDeclaration;
	private IActivityAgentClassDeclaration _activityAgentClassDeclaration;
	private IActivityAgentInstanceDeclaration _activityAgentInstanceDeclaration;
	private IActivityAgent _agent;

	private SimulationFunctionArgs _objectCreateFunctionArgs = new SimulationFunctionArgs(null);

	private SimulationFunctionArgs _objectExecuteFunctionArgs = _objectCreateFunctionArgs;

	/**
	 * @param name String
	 */
	public TestActivityRuleDeclaration(String name) {
		super(name);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestActivityRuleDeclaration.class);
	}

	protected void setUp() throws Exception {
		_alwaysTrueFunctionCallDecl =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
				FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
					null,
					BooleanType.getInstance(),
					Boolean.TRUE));
		_alwaysFalseFunctionCallDecl =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
				FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
					null,
					BooleanType.getInstance(),
					Boolean.FALSE));
		_activityAgentClassDeclaration =
			ActivityAgentClassDeclarationFactory.createActivityAgentClassDeclaration();
		IActivityGraphDeclaration composedActivityDeclaration =
			((IActivityReasoningEngineDeclaration) _activityAgentClassDeclaration
				.getReasoningEngineDeclarationListAndFactory()
				.getDeclarationList()
				.get(0))
				.getRootActivityGraphDeclaration();
		_fromActivityDeclaration = composedActivityDeclaration.getEntryActivityGraphLeafDeclaration();
		_toActivityDeclaration = composedActivityDeclaration.getExitActivityGraphLeafDeclaration();
		_trueActivityArgumentRuleDeclaration =
			ActivityGraphRuleDeclarationFactory.createActivityGraphRuleDeclaration(
				_fromActivityDeclaration,
				_toActivityDeclaration,
				_alwaysTrueFunctionCallDecl);
		_falseActivityArgumentRuleDeclaration =
			ActivityGraphRuleDeclarationFactory.createActivityGraphRuleDeclaration(
				_fromActivityDeclaration,
				_toActivityDeclaration,
				_alwaysFalseFunctionCallDecl);

		_activityAgentInstanceDeclaration =
			_activityAgentClassDeclaration.createActivityAgentInstanceDeclaration();
		_agent =
			(IActivityAgent) _activityAgentInstanceDeclaration.createObject(_objectCreateFunctionArgs);
		_agent.initialize(_objectCreateFunctionArgs);
		_objectCreateFunctionArgs.setObject(_agent);
	}

	public static Test suite() {
		/*
		TestSuite suite = new TestSuite();
		
		suite.addTest(new TestActivityRuleDeclaration("testActivityRuleDeclaration"));
		
		return suite;
		*/

		return new TestActivityRuleDeclaration("testActivityRuleDeclaration");
	}

	protected void tearDown() {
		_activityAgentInstanceDeclaration.remove();
		_activityAgentClassDeclaration.remove();
		_activityAgentClassDeclaration = null;

		_alwaysTrueFunctionCallDecl = null;
		_alwaysFalseFunctionCallDecl = null;

		_trueActivityArgumentRuleDeclaration.remove();
		_falseActivityArgumentRuleDeclaration.remove();
		_trueActivityArgumentRuleDeclaration = null;
		_falseActivityArgumentRuleDeclaration = null;

		_fromActivityDeclaration = null;
		_toActivityDeclaration = null;

		_agent = null;
	}

	protected void test(
		IObjectCreateFunctionArgs objectCreateFunctionArgs,
		IAgentExecuteFunctionArgs objectExecuteFunctionArgs,
		IActivityGraphRuleDeclaration activityRuleDeclaration,
		boolean isExecutable,
		boolean isExecuteableProbability)
		throws SimulationRuntimeException, ModelCompilingException {

		IActivityGraphRule rule =
			(IActivityGraphRule) activityRuleDeclaration.createRule(objectCreateFunctionArgs);

		if (rule == null) {
			assertEquals(false, isExecutable && isExecuteableProbability);

		} else {
			assertEquals(isExecuteableProbability, rule.isExecuteable(objectExecuteFunctionArgs));
			if (rule.isExecuteable(objectExecuteFunctionArgs)) {
				assertEquals(isExecutable, rule.isExecuteable(objectExecuteFunctionArgs));
				assertEquals(
					_toActivityDeclaration,
					rule.getActivityGraphLeaf().getActivityGraphNodeDeclaration());
			}
			assertEquals(
				_fromActivityDeclaration,
				_trueActivityArgumentRuleDeclaration.getFromActivityGraphNodeDeclaration());
		}
	}

	public void testActivityRuleDeclaration() throws Exception {
		test(
			_objectCreateFunctionArgs,
			_objectExecuteFunctionArgs,
			_trueActivityArgumentRuleDeclaration,
			true,
			true);
		test(
			_objectCreateFunctionArgs,
			_objectExecuteFunctionArgs,
			_falseActivityArgumentRuleDeclaration,
			false,
			false);
	}
}