package de.uniwue.ki.mas.entity.body;
import java.util.Iterator;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.entity.variable.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (06.02.2002 15:10:42)
 * @author Christoph Oechslein
 */

class BodyInstanceDeclaration extends AbstractDeclaration implements IBodyInstanceDeclaration {
	private final IBodyClassDeclaration _bodyClassDeclaration;
	private final BodyVariableInstanceDeclarationList _bodyVariableInstanceDeclarationList;
	
	private final transient ChangeListener _changeListener = new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fireStateChanged();
			}
		};

	public BodyInstanceDeclaration(IBodyClassDeclaration bodyClassDeclaration) {
		super();

		_bodyClassDeclaration = bodyClassDeclaration;
		_bodyVariableInstanceDeclarationList =
			new BodyVariableInstanceDeclarationList(
				bodyClassDeclaration
					.getBodyVariableClassDeclarationListAndFactory()
					.getBodyVariableClassDeclarationList());
		_bodyVariableInstanceDeclarationList.addChangeListener(_changeListener);
	}


	public IBody createBody(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {
		return new Body(
			this,
			objectCreateFunctionArgs.getObject(),
			createBodyVariables(getBodyVariableInstanceDeclarationList(), objectCreateFunctionArgs));
	}

	private static IBodyVariable[] createBodyVariables(
		BodyVariableInstanceDeclarationList bodyVariableInstanceDeclarationList,
		final IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		IBodyVariable[] variables = new IBodyVariable[bodyVariableInstanceDeclarationList.size()];

		Iterator it = bodyVariableInstanceDeclarationList.iterator();
		int index = 0;

		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration) it.next();
			variables[index] = bodyVariableInstanceDeclaration.createBodyVariable(objectCreateFunctionArgs);

			index++;
		}

		return variables;
	}

	protected ICopyable createInstance(Map objectMapping) {
		return BodyInstanceDeclarationFactory.getInstance().createBodyInstanceDeclaration(
			getBodyClassDeclaration());
	}


	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IBodyInstanceDeclaration dstBodyInstanceDeclaration = (IBodyInstanceDeclaration) intoObject;

		Assert.assertSame(getBodyClassDeclaration(), dstBodyInstanceDeclaration.getBodyClassDeclaration());

		getBodyVariableInstanceDeclarationList().copyInto(
			dstBodyInstanceDeclaration.getBodyVariableInstanceDeclarationList(), objectMapping);
	}


	public IBodyClassDeclaration getBodyClassDeclaration() {
		return _bodyClassDeclaration;
	}


	public BodyVariableInstanceDeclarationList getBodyVariableInstanceDeclarationList() {
		return _bodyVariableInstanceDeclarationList;
	}


	public String getName() {
		return getBodyClassDeclaration().getName();
	}


	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}


	public int isomorphicHashCode() {
		return hashCode();
	}


	public void remove() {
		remove(getBodyVariableInstanceDeclarationList());
		//TODO: changeListener removen
		super.remove();
	}
}