package de.uniwue.ki.mas.entity.body;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.IObjectExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.name.INamed;

/**
 * A body has instance variables, can translate variable declarations to runtime
 * variable objects/value and produces body events.
 * Creation date: (23.10.2000 16:16:25)
 *
 * @author Christoph Oechslein
 */

public interface IBody extends INamed, IRemoveable, java.io.Serializable {
	/**
	 * Returns the body variable at index, call <code>checkIfVariableIsInitialized</code>
	 * to see if variable is already initialized.
	 *
	 * @see #getBodyVariables()
	 * @see #getBodyVariablesLength()
	 */
	IBodyVariable getBodyVariableAt(int index);
	
	/**
	 * get the Name of this Body that is displayed at runtime,
	 * returns null if a generic name should be displayed.
	 */
	public String getRuntimeName();

	boolean isBodyVariableInitialized(int index);

	IBodyVariableInstanceDeclaration getBodyVariableInstanceDeclarationFor(IBodyVariable bodyVariable);

	/**
	 * Retuns the length of the body variables of this body.
	 *
	 * @see #getBodyVariableAt(int)
	 * @see #getBodyVariables()
	 */
	int getBodyVariablesLength();

	/**
	 * Resets the body to default values.
	 */
	void reset(IObjectExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Sets back link to object, is needed because of the declaration translations.
	 *
	 * @see #getObject()
	 */
	void setObject(IObject object);

	/**
	 * Updates the body under the given execution bindings, for example updates the variables.
	 */
	void update(IObjectExecuteFunctionArgs objectExecuteFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Returns back link to object, is needed because of the declaration translations.
	 *
	 * @see #setObject(IObject)
	 */
	IObject getObject();

	IBodyInstanceDeclaration getBodyInstanceDeclaration();
}