package de.uniwue.ki.mas.entity.bodyclass;

import java.util.Map;

import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.entity.variable.VariableCompileException;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (26.07.2001 11:30:00)
 * @author Christoph Oechslein
 */

class BodyClassDeclaration extends AbstractDeclaration implements IBodyClassDeclaration, ChangeListener {
	private final BodyVariableClassDeclarationListAndFactory _bodyVariableClassDeclarationListAndFactory =
		new BodyVariableClassDeclarationListAndFactory();

	public BodyClassDeclaration() {
		super();
		//Bugfix Rainer: StateChangeEvent when adding new Variables was lost
		getBodyVariableClassDeclarationListAndFactory().getBodyVariableClassDeclarationList().addChangeListener(this);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IBodyClassDeclaration dstBodyClassDeclaration = (IBodyClassDeclaration) intoObject;

		getBodyVariableClassDeclarationListAndFactory().copyInto(
			dstBodyClassDeclaration.getBodyVariableClassDeclarationListAndFactory(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new BodyClassDeclaration();
	}

	public final BodyVariableClassDeclarationListAndFactory getBodyVariableClassDeclarationListAndFactory() {
		return _bodyVariableClassDeclarationListAndFactory;
	}

	public String getName() {
		return "Body";
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IBodyClassDeclaration)) {
			return false;
		}

		IBodyClassDeclaration bodyClassDeclaration = (IBodyClassDeclaration) anObject;

		return isomorphicEquals(
			getBodyVariableClassDeclarationListAndFactory(),
			bodyClassDeclaration.getBodyVariableClassDeclarationListAndFactory());
	}

	public int isomorphicHashCode() {
		return 11 * super.isomorphicHashCode()
			+ 7 * isomorphicHashCode(getBodyVariableClassDeclarationListAndFactory());
	}

	/**
	 * Remove Object (this).
	 * Gets called if Object will be deleted from modell
	 */
	public void remove() {
		// remove 1:1 link
		getBodyVariableClassDeclarationListAndFactory().remove();

		super.remove();
	}

	public IBodyVariableClassDeclaration getBodyVariableClassDeclaration(IVariableClassDeclaration variableClassDeclaration)
		throws VariableCompileException {

		if (variableClassDeclaration == null) {
			throw new VariableCompileException("Variable is null");

		} else {
			BodyVariableClassDeclarationList bodyVarDeclList =
				getBodyVariableClassDeclarationListAndFactory().getBodyVariableClassDeclarationList();

			for (int i = 0; i < bodyVarDeclList.size(); i++) {
				IBodyVariableClassDeclaration currBodyVariableClassDeclaration =
					(IBodyVariableClassDeclaration) bodyVarDeclList.get(i);

				if (Utils
					.equals(
						variableClassDeclaration,
						currBodyVariableClassDeclaration.getVariableClassDeclaration()))
					return currBodyVariableClassDeclaration;
			}

			throw new VariableCompileException(
				"Could not find variable: "
					+ variableClassDeclaration.getName()
					+ " in body "
					+ getName()
					+ " with variables (in object class): "
					+ bodyVarDeclList.getName());

		}
	}


	public int getBodyVariableIndexOf(IBodyVariableClassDeclaration bodyVariableClassDeclaration)
		throws ModelCompilingException {

		if (bodyVariableClassDeclaration == null) {
			throw new VariableCompileException("Variable = <null>");

		} else {
			BodyVariableClassDeclarationList bodyVarDeclList =
				getBodyVariableClassDeclarationListAndFactory().getBodyVariableClassDeclarationList();

			for (int i = 0; i < bodyVarDeclList.size(); i++) {

				if (Utils.equals(bodyVarDeclList.get(i), bodyVariableClassDeclaration))
					return i;
			}

			throw new VariableCompileException(
				"Did not find variable: " + bodyVariableClassDeclaration.getName());
		}
	}

	public int getBodyVariableIndexOf(IVariableClassDeclaration variableClassDeclaration)
		throws ModelCompilingException {
		return getBodyVariableIndexOf(getBodyVariableClassDeclaration(variableClassDeclaration));
	}
}