package de.uniwue.ki.mas.entity.object;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.featureList.ObjectFeatureInstanceDeclarationList;
import de.uniwue.ki.mas.entity.body.BodyInstanceDeclarationFactory;
import de.uniwue.ki.mas.entity.body.IBodyInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (03.09.2001 15:00:02)
 * @author Christoph Oechslein
 */

public abstract class AbstractObjectInstanceDeclaration
	extends AbstractDocumentableDeclaration
	implements IObjectInstanceDeclaration {
	private final IObjectClassDeclaration _objectClassDeclaration;
	private final IBodyInstanceDeclaration _bodyInstanceDeclaration;

	private ObjectFeatureInstanceDeclarationList _objectFeatureInstanceDeclarationList;
	private final RemoveVetoListener _neverRemoveVetoListener = new NeverRemoveVetoListener(this);
	
	private final transient ChangeListener _changeListener = new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				fireStateChanged();
			}
		};

	public AbstractObjectInstanceDeclaration(IObjectClassDeclaration objectClassDeclaration) {
		super();

		_objectClassDeclaration = objectClassDeclaration;
		_bodyInstanceDeclaration =
			BodyInstanceDeclarationFactory.getInstance().createBodyInstanceDeclaration(
				objectClassDeclaration.getBodyClassDeclaration());
				
		_bodyInstanceDeclaration.addChangeListener(_changeListener);
		getObjectClassDeclaration().addRemoveVetoListener(_neverRemoveVetoListener);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		if (objectMapping != null)
			objectMapping.put(this, intoObject);

		super.copyInto(intoObject, objectMapping);

		IObjectInstanceDeclaration dstObjectInstanceDeclaration = (IObjectInstanceDeclaration) intoObject;

		Assert.assertSame(
			getObjectClassDeclaration(),
			dstObjectInstanceDeclaration.getObjectClassDeclaration());

		getBodyInstanceDeclaration().copyInto(
			dstObjectInstanceDeclaration.getBodyInstanceDeclaration(),
			objectMapping);

		getObjectFeatureInstanceDeclarationList().copyInto(
			dstObjectInstanceDeclaration.getObjectFeatureInstanceDeclarationList(),
			objectMapping);
	}

	public abstract IObject createObject(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException;

	public IBodyInstanceDeclaration getBodyInstanceDeclaration() {
		return _bodyInstanceDeclaration;
	}

	protected String getDefaultName() {
		return getObjectClassDeclaration().getName();
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		return _objectClassDeclaration;
	}

	public ObjectFeatureInstanceDeclarationList getObjectFeatureInstanceDeclarationList() {
		return _objectFeatureInstanceDeclarationList;
	}

	protected void initObjectFeatureInstanceDeclarationList(IObjectClassDeclaration objectClassDeclaration) {
		_objectFeatureInstanceDeclarationList =
			new ObjectFeatureInstanceDeclarationList(
				this,
				objectClassDeclaration.getObjectFeatureClassDeclarationList());
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void remove() {
		if (canBeRemoved()) {
			_bodyInstanceDeclaration.removeChangeListener(_changeListener);
			remove(getBodyInstanceDeclaration());
			remove(getObjectFeatureInstanceDeclarationList());
			getObjectClassDeclaration().removeRemoveVetoListener(_neverRemoveVetoListener);
			super.remove();
		}
	}
}