package de.uniwue.ki.mas.entity.object.agent;
import java.util.Iterator;
import java.util.logging.Level;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.FunctionActionFactory;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngine;
import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.object.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/** 
 * Creation date: (13.09.2001 16:53:55)
 * @author Christoph Oechslein
 */

abstract class AbstractAgent extends AbstractObject implements IAgent {
	private final IReasoningEngine[] _reasoningEngines;
	private IFunction _currentAction = FunctionActionFactory.getNoopFunction();

	public AbstractAgent(
		IAgentInstanceDeclaration declaration,
		IObjectCreateFunctionArgs objectCreateFunctionArgs) {

		super(declaration, objectCreateFunctionArgs);

		int reasoningEngineSize =
			declaration
				.getAgentClassDeclaration()
				.getReasoningEngineDeclarationListAndFactory()
				.getDeclarationList()
				.size();
		_reasoningEngines = new IReasoningEngine[reasoningEngineSize];
	}

	public void update(IObjectExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {
		super.update(objectExecuteFunctionArgs);

		if (!isDead()) {
			reasonAndAct((IAgentExecuteFunctionArgs) objectExecuteFunctionArgs);
		}

	}

	private final void reasonAndAct(IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {

		for (int i = 0; i < _reasoningEngines.length; i++) {
			reasonAndAct(i, objectExecuteFunctionArgs);
		}
	}

	protected void reasonAndAct(int index, IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {
		do {
			int counter = 0;

			reason(index, objectExecuteFunctionArgs);
			act(index, objectExecuteFunctionArgs);

			if (isDead()) {
				return;
			}

			if (Assert.ASSERTION_ON) {
				counter++;

				if (counter > 10000)
					throw new SimulationRuntimeException("Maybe endless loop in activity graph?");
			}

		} while (!isReasoningFinished(index));
	}

	protected abstract boolean isReasoningFinished(int index);

	protected abstract void reason(int index, IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException;
	protected abstract void act(int index, IAgentExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException;

	public IFunction getCurrentAction() {
		return _currentAction;
	}

	public void reset(IObjectExecuteFunctionArgs objectExecuteFunctionArgs)
		throws SimulationRuntimeException {

		setCurrentAction(FunctionActionFactory.getNoopFunction());

		super.reset(objectExecuteFunctionArgs);

		try {
			for (int i = 0; i < getReasoningEngineCount(); i++) {
				getReasoningEngineAt(i).init(
					(IObjectCreateFunctionArgs) objectExecuteFunctionArgs
						.createCreateAndExecuteFunctionArgs());
			}
		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeCausedByModelCompileException(e);
		}
	}

	public void setCurrentAction(IFunction newCurrentAction) {
		_currentAction = newCurrentAction;
	}

	public int getReasoningEngineCount() {
		return _reasoningEngines.length;
	}

	public IReasoningEngine getReasoningEngineAt(int index) {
		return _reasoningEngines[index];
	}

	protected void initializeIntern(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		setBody(createBody(objectCreateFunctionArgs));
		setObjectFeatures(createObjectFeatureList(objectCreateFunctionArgs));

		setReasoningEngines(objectCreateFunctionArgs);

		try {
			reset((IObjectExecuteFunctionArgs) objectCreateFunctionArgs.createCreateAndExecuteFunctionArgs());

		} catch (SimulationRuntimeException exception) {
			throw new ModelCompilingException(exception);
		}
	}

	private void setReasoningEngines(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {

		IDeclarationList reasoningEngineDeclarationList =
			((IAgentClassDeclaration) getObjectInstanceDeclaration().getObjectClassDeclaration())
				.getReasoningEngineDeclarationListAndFactory()
				.getDeclarationList();

		Iterator it = reasoningEngineDeclarationList.iterator();

		int i = -1;
		while (it.hasNext()) {
			i++;
			IReasoningEngineDeclaration reasoningEngineDeclaration = (IReasoningEngineDeclaration) it.next();

			IReasoningEngine reasoningEngine =
				reasoningEngineDeclaration.createReasoningEngine(objectCreateFunctionArgs);

			if (reasoningEngine == null) {
				if (Assert.ASSERTION_ON && FunctionDeclarationCompiler.LOGGER.isLoggable(Level.INFO)) {
					FunctionDeclarationCompiler.LOGGER.info(
						"Removed noop reasoning engine: " + reasoningEngineDeclaration.getName());
				}

			} else {
				Assert.assertNull(_reasoningEngines[i]);

				_reasoningEngines[i] = reasoningEngine;
			}
		}
	}
}