package de.uniwue.ki.mas.entity.object.agent;
import java.util.ArrayList;
import java.util.List;

import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.behaviour.activityGraph.IActivityGraphLeaf;
import de.uniwue.ki.mas.entity.object.IObjectCreateFunctionArgs;
import de.uniwue.ki.mas.sim.listener.*;

/**
 * Creation date: (06.12.2000 17:12:12)
 * @author Christoph Oechslein
 */

public class ActivityAgentFunctionArgs
	extends ObjectFunctionArgs
	implements IActivityAgentExecuteFunctionArgs, IObjectCreateFunctionArgs {

	private List _agentsListener = null;
	private List _agentsActionListener = null;
	private List _agentsActivityListener = null;

	public ActivityAgentFunctionArgs() {
		super();
	}

	public void fireCurrentActivityChanged(
		IActivityAgent agent,
		IActivityGraphLeaf oldActivity,
		IActivityGraphLeaf newActivity,
		int currActivityReasoningEngineIndex) {

		if (!hasAgentsActivityListener())
			return;

		CurrentActivityGraphLeafChangedEvent activityAgentEvent =
			new CurrentActivityGraphLeafChangedEvent(
				agent,
				oldActivity,
				newActivity,
				currActivityReasoningEngineIndex);

		for (int i = 0; i < _agentsActivityListener.size(); i++) {
			IAgentsActivityListener agentsActivityListener =
				(IAgentsActivityListener) _agentsActivityListener.get(i);
			agentsActivityListener.currentActivityGraphLeafChanged(activityAgentEvent);
		}
	}

	public void fireCurrentActionChanged(IAgent agent, IFunction oldAction, IFunction newAction) {
		if (!hasAgentsActionListener())
			return;

		AgentActionChangedEvent actionChangedEvent =
			new AgentActionChangedEvent(agent, oldAction, newAction);

		for (int i = 0; i < _agentsActionListener.size(); i++) {
			IAgentsActionListener agentsActionListener =
				(IAgentsActionListener) _agentsActionListener.get(i);
			agentsActionListener.currentActionChanged(actionChangedEvent);
		}
	}

	public void fireFinishedActing(IActivityAgent agent, int currActivityReasoningEngineIndex) {
		if (!hasAgentsActionListener())
			return;

		ActivityAgentEvent agentEvent =
			new ActivityAgentEvent(agent, currActivityReasoningEngineIndex);

		for (int i = 0; i < _agentsListener.size(); i++) {
			IActivityAgentsListener agentsListener =
				(IActivityAgentsListener) _agentsListener.get(i);
			agentsListener.actingFinished(agentEvent);
		}
	}

	public void fireFinishedReasoning(IActivityAgent agent, int currActivityReasoningEngineIndex) {
		if (!hasAgentsActionListener())
			return;

		ActivityAgentEvent agentEvent =
			new ActivityAgentEvent(agent, currActivityReasoningEngineIndex);

		for (int i = 0; i < _agentsListener.size(); i++) {
			IActivityAgentsListener agentsListener =
				(IActivityAgentsListener) _agentsListener.get(i);
			agentsListener.reasoningFinished(agentEvent);
		}
	}

	public void fireStartsActing(IActivityAgent agent, int currActivityReasoningEngineIndex) {
		if (!hasAgentsActionListener())
			return;

		ActivityAgentEvent agentEvent =
			new ActivityAgentEvent(agent, currActivityReasoningEngineIndex);

		for (int i = 0; i < _agentsListener.size(); i++) {
			IActivityAgentsListener agentsListener =
				(IActivityAgentsListener) _agentsListener.get(i);
			agentsListener.actingStarts(agentEvent);
		}
	}

	public void fireStartsReasoning(IActivityAgent agent, int currActivityReasoningEngineIndex) {
		if (!hasAgentsActionListener())
			return;

		ActivityAgentEvent agentEvent =
			new ActivityAgentEvent(agent, currActivityReasoningEngineIndex);

		for (int i = 0; i < _agentsListener.size(); i++) {
			IActivityAgentsListener agentsListener =
				(IActivityAgentsListener) _agentsListener.get(i);
			agentsListener.reasoningStarts(agentEvent);
		}
	}

	public void addAgentsListener(IActivityAgentsListener objectsListener) {
		if (!hasAgentsListener()) {
			_agentsListener = new ArrayList(1);
		}

		_agentsListener.add(objectsListener);
	}

	public void removeAgentsListener(IActivityAgentsListener objectsListener) {
		if (_agentsListener != null)
			_agentsListener.remove(objectsListener);
	}

	public boolean hasAgentsListener() {
		return (_agentsListener != null) && (!_agentsListener.isEmpty());
	}

	public void addAgentsActionListener(IAgentsActionListener objectsListener) {
		if (!hasAgentsActionListener()) {
			_agentsActionListener = new ArrayList(1);
		}

		_agentsActionListener.add(objectsListener);
	}

	public void removeAgentsActionListener(IAgentsActionListener objectsListener) {
		if (_agentsActionListener != null)
			_agentsActionListener.remove(objectsListener);
	}

	public boolean hasAgentsActionListener() {
		return (_agentsActionListener != null) && (!_agentsActionListener.isEmpty());
	}

	public void addAgentsActivityListener(IAgentsActivityListener objectsListener) {
		if (!hasAgentsActivityListener()) {
			_agentsActivityListener = new ArrayList(1);
		}

		_agentsActivityListener.add(objectsListener);
	}

	public void removeAgentsActivityListener(IAgentsActivityListener objectsListener) {
		if (_agentsActivityListener != null)
			_agentsActivityListener.remove(objectsListener);
	}

	public boolean hasAgentsActivityListener() {
		return (_agentsActivityListener != null) && (!_agentsActivityListener.isEmpty());
	}

	public void remove() {
		_agentsActionListener = null;
		_agentsActivityListener = null;
		_agentsListener = null;

		super.remove();
	}
}