package de.uniwue.ki.mas.entity.object.agent;
import java.util.ArrayList;
import java.util.List;

import de.uniwue.ki.mas.basics.function.CreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.object.*;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;
import de.uniwue.ki.mas.sim.listener.*;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (06.12.2000 17:12:12)
 * @author Christoph Oechslein
 */

public class ObjectFunctionArgs
	extends CreateAndExecuteFunctionArgs
	implements IObjectExecuteFunctionArgs, IObjectCreateFunctionArgs {

	private List _objectsVariableListener = null;

	private IObject _obj;

	private List _objectsListener = null;

	public ObjectFunctionArgs() {
		super();
	}

	public IObject getObject() {
		return _obj;
	}

	public void setObject(IObject newSimObject) {
		_obj = newSimObject;
	}

	public void fireFinishedUpdate(IObject object) {
		if (!hasObjectsListener())
			return;

		ObjectEvent objectEvent = new ObjectEvent(object);

		for (int i = 0; i < _objectsListener.size(); i++) {
			IObjectsListener objectsListener = (IObjectsListener) _objectsListener.get(i);
			objectsListener.updateFinished(objectEvent);
		}
	}

	public void fireStartsUpdate(IObject object) {
		if (!hasObjectsListener())
			return;

		ObjectEvent objectEvent = new ObjectEvent(object);

		for (int i = 0; i < _objectsListener.size(); i++) {
			IObjectsListener objectsListener = (IObjectsListener) _objectsListener.get(i);
			objectsListener.updateStarts(objectEvent);
		}
	}

	public void fireVariableValueChanged(
		IObject inObject,
		IObject settingAgent,
		IBodyVariable bodyVariable,
		Object oldValue,
		Object newValue) {

		if (!hasObjectsListener() || Utils.equals(oldValue, newValue))
			return;

		VariableChangedEvent bodyEvent =
			new VariableChangedEvent(inObject, settingAgent, bodyVariable, oldValue, newValue);

		for (int i = 0; i < _objectsVariableListener.size(); i++) {
			IObjectsVariableListener objectsVariableListener =
				(IObjectsVariableListener) _objectsVariableListener.get(i);
			objectsVariableListener.variableValueChanged(bodyEvent);
		}
	}

	public void addObjectsListener(IObjectsListener objectsListener) {
		if (!hasObjectsListener()) {
			_objectsListener = new ArrayList(1);
		}

		_objectsListener.add(objectsListener);
	}

	public void removeObjectsListener(IObjectsListener objectsListener) {
		if (_objectsListener != null)
			_objectsListener.remove(objectsListener);
	}

	public boolean hasObjectsListener() {
		return (_objectsListener != null) && (!_objectsListener.isEmpty());
	}

	public boolean hasObjectsVariableListener() {
		return (_objectsVariableListener != null) && (!_objectsVariableListener.isEmpty());
	}

	public void removeObjectsVariableListener(IObjectsVariableListener objectsListener) {
		if (_objectsVariableListener != null)
			_objectsVariableListener.remove(objectsListener);
	}

	public void addObjectsVariableListener(IObjectsVariableListener objectsListener) {
		if (!hasObjectsVariableListener()) {
			_objectsVariableListener = new ArrayList(1);
		}

		_objectsVariableListener.add(objectsListener);
	}

	public void remove() {
		_obj = null;
		_objectsListener = null;
		_objectsVariableListener = null;

		super.remove();
	}
}