package de.uniwue.ki.mas.entity.object.world;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.entity.object.*;
import de.uniwue.ki.mas.entity.object.agent.AbstractActivityAgentInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityWorldClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (11.09.2001 14:00:22)
 * @author Christoph Oechslein
 */

public class ActivityWorldInstanceDeclaration
	extends AbstractActivityAgentInstanceDeclaration
	implements IActivityWorldInstanceDeclaration {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private final ObjectInstanceDeclarationList _objectInstanceDeclarationList;

	public ActivityWorldInstanceDeclaration(IActivityWorldClassDeclaration worldClassDeclaration) {
		super(worldClassDeclaration);

		_objectInstanceDeclarationList = new ObjectInstanceDeclarationList();
		_objectInstanceDeclarationList.addChangeListener(_changeListener);

		initObjectFeatureInstanceDeclarationList(worldClassDeclaration);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return getActivityWorldClassDeclaration().createActivityWorldInstanceDeclaration();
	}

	public IObject createObject(IObjectCreateFunctionArgs objectCreateFunctionArgs)
		throws ModelCompilingException {
		return new ActivityWorld(this, objectCreateFunctionArgs);
	}

	public IActivityWorldClassDeclaration getActivityWorldClassDeclaration() {
		return (IActivityWorldClassDeclaration)getActivityAgentClassDeclaration();
	}

	public ObjectInstanceDeclarationList getObjectInstanceDeclarationList() {
		return _objectInstanceDeclarationList;
	}

	public IWorldClassDeclaration getWorldClassDeclaration() {
		return (IWorldClassDeclaration)getObjectClassDeclaration();
	}

	public void remove() {
		remove(getObjectInstanceDeclarationList());
		//TODO: ChangeListener entfernen
		super.remove();
	}

	public void copyInto(Object dstObject, Map objectMapping) {
		super.copyInto(dstObject, objectMapping);

		IActivityWorldInstanceDeclaration dstActivityWorldInstanceDeclaration =
			(IActivityWorldInstanceDeclaration)dstObject;

		getObjectInstanceDeclarationList().copyInto(
			dstActivityWorldInstanceDeclaration.getObjectInstanceDeclarationList(),
			objectMapping);
	}

}