package de.uniwue.ki.mas.entity.object.world;
import java.util.Iterator;

import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
 * A world enhance an {@link IAgent} by having a group of objects and
 * by generating {@link WorldEvent world events}.
 * Creation date: (13.11.2000 10:42:56)
 *
 * @author Christoph Oechslein
 */
public interface IWorld extends IAgent {

	/**
	 * Adds a new object during a simulation to this group and fires event; the object group is updated during updateAfter.
	 *
	 * @see #updateAfter
	 * @see #removeObjectInSimulation(IObject)
	 * @see IObjectGroup#addObject(IObject)
	 */
	void addObjectInSimulation(IObject newObject, IWorldExecuteFunctionArgs worldExecuteFunctionArgs)
		throws SimulationRuntimeException;

	/**
	 * Adds a world listener.
	 *
	 * @see WorldEvent
	 * @see #removeWorldListener(WorldListener)
	 */
	void addWorldListener(WorldListener newListener);
	
	Iterator getAllObjects(boolean includingAdded, boolean excludingRemoved);

	int getObjectCount(boolean includingAdded, boolean excludingRemoved);

	/**
	 * Shuffle the object group list.
	 *
	 * @see #getObjects()
	 */
	void shuffleObjects();

	/**
	* Removes an object during a simulation to this group and fires event; the object group is updated during updateAfter.
	*
	* @see #updateAfter
	* @see #addObjectInSimulation(IObject)
	* @see IObjectGroup#removeObject(IObject)
	*/
	void removeObjectInSimulation(IObject anObject, IWorldExecuteFunctionArgs worldExecuteFunctionArgs);

	/**
	 * Removes a world listener.
	 *
	 * @see WorldEvent
	 * @see #addWorldListener(WorldListener)
	 */
	void removeWorldListener(WorldListener newListener);
}