package de.uniwue.ki.mas.entity.objectclass;

import java.util.*;

import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.FeatureClassDeclarationListManager;
import de.uniwue.ki.mas.basics.featureList.IObjectFeatureClassDeclarationList;
import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (25.07.2001 15:05:06)
 * 
 * @author Christoph Oechslein
 */

public abstract class AbstractObjectClassDeclaration extends
        AbstractDocumentableDeclaration implements IObjectClassDeclaration,
        ChangeListener {

    private final IBodyClassDeclaration _bodyClassDeclaration;

    private final IObjectFeatureClassDeclarationList _objectFeatureClassDeclarationList;

    private final DeclarationListListener _featureClassDeclarationListListener = new FeatureClassDeclarationListListener();

    protected final RemoveVetoListener _neverRemoveVetoListener = new NeverRemoveVetoListener(
            this);

    private final UserFeatureVariableDeclarationListListener _userFeatureVariableDeclarationListListener = new UserFeatureVariableDeclarationListListener();

    private final class FeatureClassDeclarationListListener implements
            DeclarationListListener {

        public void declarationAdded(DeclarationListEvent e) {
            if (e.getSource() == getObjectFeatureClassDeclarationList()) {
                featureClassDeclarationAdded((IObjectFeatureClassDeclaration) e
                        .getDeclaration());
            }
        }

        public void declarationRemoved(DeclarationListEvent e) {
            if (e.getSource() == getObjectFeatureClassDeclarationList()) {
                featureClassDeclarationRemoved((IObjectFeatureClassDeclaration) e
                        .getDeclaration());
            }
        }
    }

    private final class UserFeatureVariableDeclarationListListener implements
            DeclarationListListener {

        public void declarationAdded(DeclarationListEvent e) {
            VariableClassDeclarationList variableClassDeclarationList = (VariableClassDeclarationList) e
                    .getSource();
            featureClassDeclarationVariableClassDeclarationAdded(
                    (IVariableClassDeclaration) e.getDeclaration(),
                    variableClassDeclarationList);
        }

        public void declarationRemoved(DeclarationListEvent e) {
            VariableClassDeclarationList variableClassDeclarationList = (VariableClassDeclarationList) e
                    .getSource();
            featureClassDeclarationVariableClassDeclarationRemoved(
                    (IVariableClassDeclaration) e.getDeclaration(),
                    variableClassDeclarationList);
        }
    }

    public AbstractObjectClassDeclaration(
            IBodyClassDeclaration bodyClassDeclaration) {
        super();

        setUsesShortDescription(true);
        setShortDescription("Untitled Object Class");

        bodyClassDeclaration.addChangeListener(this);
        _bodyClassDeclaration = bodyClassDeclaration;

        _objectFeatureClassDeclarationList = FeatureClassDeclarationListManager
                .getInstance()
                .createFeatureClassDeclarationListWithDefaultFeaturesFor(this);

        initialize();
    }

    public void copyInto(Object intoObject, Map objectMapping) {
        if (objectMapping == null) objectMapping = new HashMap();

        objectMapping.put(this, intoObject);

        super.copyInto(intoObject, objectMapping);

        AbstractObjectClassDeclaration intoObjectClassDeclaration = (AbstractObjectClassDeclaration) intoObject;

        getBodyClassDeclaration().copyInto(
                intoObjectClassDeclaration.getBodyClassDeclaration(),
                objectMapping);

        getObjectFeatureClassDeclarationList().copyInto(
                intoObjectClassDeclaration
                        .getObjectFeatureClassDeclarationList(), objectMapping);

        Assert.assertEquals(getBodyClassDeclaration()
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList().size(),
                intoObjectClassDeclaration.getBodyClassDeclaration()
                        .getBodyVariableClassDeclarationListAndFactory()
                        .getBodyVariableClassDeclarationList().size());

        Assert.assertEquals(getObjectFeatureClassDeclarationList().size(),
                intoObjectClassDeclaration
                        .getObjectFeatureClassDeclarationList().size());
    }

    protected void featureClassDeclarationAdded(
            IObjectFeatureClassDeclaration featureClassDeclaration) {
        BodyVariableClassDeclarationList bodyVariableClassDeclarationList = getBodyClassDeclaration()
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList();

        Iterator it = featureClassDeclaration.getVariableClassDeclarations();

        while (it.hasNext()) {
            IVariableClassDeclaration currVariableClassDeclaration = (IVariableClassDeclaration) it
                    .next();

            if (bodyVariableClassDeclarationList
                    .getBodyVariableClassDeclaration(currVariableClassDeclaration) == null) {
                bodyVariableClassDeclarationList
                        .add(currVariableClassDeclaration
                                .createBodyVariableClassDeclaration());
                bodyVariableClassDeclarationList
                        .getBodyVariableClassDeclaration(
                                currVariableClassDeclaration)
                        .addRemoveVetoListener(_neverRemoveVetoListener);
            }
        }

        if (featureClassDeclaration instanceof IUserFeatureClassDeclaration) {
            IUserFeatureClassDeclaration userFeatureClassDeclaration = (IUserFeatureClassDeclaration) featureClassDeclaration;
            userFeatureClassDeclaration
                    .getFeatureVariableClassDeclarationListAndFactory()
                    .getVariableClassDeclarationList()
                    .addDeclarationListListener(
                            _userFeatureVariableDeclarationListListener);
        }
    }

    protected void featureClassDeclarationRemoved(
            IObjectFeatureClassDeclaration featureClassDeclaration) {
        if (getBodyClassDeclaration() != null) {
            BodyVariableClassDeclarationList bodyVariableClassDeclarationList = getBodyClassDeclaration()
                    .getBodyVariableClassDeclarationListAndFactory()
                    .getBodyVariableClassDeclarationList();

            Iterator it = featureClassDeclaration
                    .getVariableClassDeclarations();

            while (it.hasNext()) {
                IVariableClassDeclaration currVariableClassDeclaration = (IVariableClassDeclaration) it
                        .next();

                IBodyVariableClassDeclaration bodyVariableClassDeclaration = bodyVariableClassDeclarationList
                        .getBodyVariableClassDeclaration(currVariableClassDeclaration);

                if (Assert.ASSERTION_ON)
                        Assert.assertNotNull(
                                "Body Variable for Feature is missing: "
                                        + currVariableClassDeclaration + "("
                                        + featureClassDeclaration + ")",
                                bodyVariableClassDeclaration);

                bodyVariableClassDeclaration
                        .removeRemoveVetoListener(_neverRemoveVetoListener);

                if (bodyVariableClassDeclaration.canBeRemoved()) {
                    bodyVariableClassDeclarationList
                            .remove(bodyVariableClassDeclaration);
                }
            }
        }

        if (featureClassDeclaration instanceof IUserFeatureClassDeclaration) {
            IUserFeatureClassDeclaration userFeatureClassDeclaration = (IUserFeatureClassDeclaration) featureClassDeclaration;
            userFeatureClassDeclaration
                    .getFeatureVariableClassDeclarationListAndFactory()
                    .getVariableClassDeclarationList()
                    .removeDeclarationListListener(
                            _userFeatureVariableDeclarationListListener);
        }

    }

    protected void featureClassDeclarationVariableClassDeclarationAdded(
            IVariableClassDeclaration variableClassDeclaration,
            VariableClassDeclarationList variableClassDeclarationList) {

        BodyVariableClassDeclarationList bodyVariableClassDeclarationList = getBodyClassDeclaration()
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList();

        if (bodyVariableClassDeclarationList
                .getBodyVariableClassDeclaration(variableClassDeclaration) == null) {
            bodyVariableClassDeclarationList.add(variableClassDeclaration
                    .createBodyVariableClassDeclaration());
            bodyVariableClassDeclarationList.getBodyVariableClassDeclaration(
                    variableClassDeclaration).addRemoveVetoListener(
                    _neverRemoveVetoListener);
        }
    }

    protected void featureClassDeclarationVariableClassDeclarationRemoved(
            IVariableClassDeclaration variableClassDeclaration,
            VariableClassDeclarationList variableClassDeclarationList) {

        BodyVariableClassDeclarationList bodyVariableClassDeclarationList = getBodyClassDeclaration()
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList();

        IBodyVariableClassDeclaration bodyVariableClassDeclaration = bodyVariableClassDeclarationList
                .getBodyVariableClassDeclaration(variableClassDeclaration);

        bodyVariableClassDeclaration
                .removeRemoveVetoListener(_neverRemoveVetoListener);

        boolean result = bodyVariableClassDeclarationList
                .remove(bodyVariableClassDeclaration);

        Assert.assertTrue(result);
    }

    public IBodyClassDeclaration getBodyClassDeclaration() {
        return _bodyClassDeclaration;
    }

    public IObjectFeatureClassDeclarationList getObjectFeatureClassDeclarationList() {
        return _objectFeatureClassDeclarationList;
    }

    public IObjectFeatureClassDeclaration getFeatureClassDeclarationByName(
            String name) {
        Iterator it = getObjectFeatureClassDeclarationList().iterator();
        while (it.hasNext()) {
            IObjectFeatureClassDeclaration objectFeatureClassDeclaration = (IObjectFeatureClassDeclaration) it
                    .next();
            if (objectFeatureClassDeclaration.getFeatureClassDeclaration()
                    .getName().equals(name))
                    return objectFeatureClassDeclaration;
        }
        return null;
    }

    private void initialize() {
        Iterator it = _objectFeatureClassDeclarationList.iterator();
        while (it.hasNext()) {
            featureClassDeclarationAdded((IObjectFeatureClassDeclaration) it
                    .next());
        }
        _objectFeatureClassDeclarationList
                .addDeclarationListListener(_featureClassDeclarationListListener);
    }

    public boolean isomorphicEquals(IIsomorphicHashable anObject) {
        if (anObject == this) return true;

        if (!super.isomorphicEquals(anObject)) { return false; }

        if (!(anObject instanceof IObjectClassDeclaration)) { return false; }

        IObjectClassDeclaration objectClassDeclaration = (IObjectClassDeclaration) anObject;

        return isomorphicEquals(getBodyClassDeclaration(),
                objectClassDeclaration.getBodyClassDeclaration())
                && isomorphicEquals(getObjectFeatureClassDeclarationList(),
                        objectClassDeclaration
                                .getObjectFeatureClassDeclarationList());
    }

    public int isomorphicHashCode() {
        return 11 * super.isomorphicHashCode() + 7
                * isomorphicHashCode(getBodyClassDeclaration()) + 3
                * isomorphicHashCode(getObjectFeatureClassDeclarationList());
    }

    public void remove() {
        // remove 1:n link (aggregation)
        Assert.assertTrue(!getObjectFeatureClassDeclarationList()
                .isComposition());
        getObjectFeatureClassDeclarationList().remove();
        _objectFeatureClassDeclarationList
                .removeDeclarationListListener(_featureClassDeclarationListListener);

        // remove 1:1 link
        remove(getBodyClassDeclaration());
        getBodyClassDeclaration().removeChangeListener(this);

        super.remove();
    }
}