package de.uniwue.ki.mas.entity.objectclass;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.object.*;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (04.09.2001 16:05:27)
 * @author Christoph Oechslein
 */

public final class RessourceClassDeclarationFactory extends AbstractDeclarationFactory implements IRessourceClassDeclarationFactory {
    private static final RessourceClassDeclarationFactory instance = new RessourceClassDeclarationFactory();

    private final RessourceClassDeclarationList agentClassDeclarationList;

    protected RessourceClassDeclarationFactory() {
        super();
        this.agentClassDeclarationList = null;
    }

    public RessourceClassDeclarationFactory(RessourceClassDeclarationList agentClassDeclarationList) {
        super();
        if (Assert.ASSERTION_ON)
            Assert.assertNotNull(agentClassDeclarationList);
        this.agentClassDeclarationList = agentClassDeclarationList;
    }

    /**
     * @return IRessourceClassDeclaration
     * @see de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclarationFactory#createRessourceClassDeclaration()
     */
    public final IRessourceClassDeclaration createRessourceClassDeclaration() {
        return new RessourceClassDeclaration();
    }

    /**
     * @return IRessourceInstanceDeclarationFactory
     * @see de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclarationFactory#createRessourceInstanceDeclarationFactory()
     */
    public final IRessourceInstanceDeclarationFactory createRessourceInstanceDeclarationFactory() {
        return createRessourceInstanceDeclarationFactory(getRessourceClassDeclaration());
    }
    /**
     * @param activityRessourceClassDeclaration IRessourceClassDeclaration
     * @return IRessourceInstanceDeclarationFactory
     */
    public static final IRessourceInstanceDeclarationFactory createRessourceInstanceDeclarationFactory(IRessourceClassDeclaration activityRessourceClassDeclaration) {
        return new RessourceInstanceDeclarationFactory(activityRessourceClassDeclaration);
    }

    /**
     * @return IDeclaration
     * @see de.uniwue.ki.mas.util.declaration.IDeclarationFactory#createDeclaration()
     */
    public final IDeclaration createDeclaration() {
        return createRessourceClassDeclaration();
    }
    public final IObjectClassDeclaration createObjectClassDeclaration() {
        return createRessourceClassDeclaration();
    }
    /**
     * @return IObjectInstanceDeclarationFactory
     * @see de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclarationFactory#createObjectInstanceDeclarationFactory()
     */
    public IObjectInstanceDeclarationFactory createObjectInstanceDeclarationFactory() {
        return createRessourceInstanceDeclarationFactory();
    }
    protected IRessourceClassDeclaration getRessourceClassDeclaration() {
        if (agentClassDeclarationList.size() > 0) {
            return (IRessourceClassDeclaration) agentClassDeclarationList.get(0);
        } else {
            IRessourceClassDeclaration activityRessourceClassDeclaration = createRessourceClassDeclaration();
            agentClassDeclarationList.add(activityRessourceClassDeclaration);
            return activityRessourceClassDeclaration;
        }
    }
    /**
     * @return RessourceClassDeclarationFactory
     */
    public static final RessourceClassDeclarationFactory getInstance() {
        return instance;
    }
}