package de.uniwue.ki.mas.entity.objectclass.agentclass;

import java.util.Map;

import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.ReasoningEngineDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.AbstractObjectClassDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (26.07.2001 10:56:37)
 * @author Christoph Oechslein
 */

abstract class AbstractAgentClassDeclaration
	extends AbstractObjectClassDeclaration
	implements IAgentClassDeclaration {

	private final ReasoningEngineDeclarationListAndFactory _reasoningEngineDeclarationListAndFactory;

	public AbstractAgentClassDeclaration(
		IBodyClassDeclaration bodyClassDeclaration,
		ReasoningEngineDeclarationListAndFactory reasoningEngineDeclarationListAndFactory) {

		super(bodyClassDeclaration);

		setUsesShortDescription(true);
		setShortDescription("Untitled Agent Class");

		_reasoningEngineDeclarationListAndFactory = reasoningEngineDeclarationListAndFactory;
		//Bugfix Rainer: Statechange when adding new reasonings was missing
		reasoningEngineDeclarationListAndFactory.getDeclarationList().addChangeListener(this);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IAgentClassDeclaration intoAgentClassDeclaration = (IAgentClassDeclaration) intoObject;

		getReasoningEngineDeclarationListAndFactory().copyInto(
			intoAgentClassDeclaration.getReasoningEngineDeclarationListAndFactory(),
			objectMapping);
	}

	public IReasoningEngineDeclaration getReasoningEngineDeclaration() {
		return (IReasoningEngineDeclaration) _reasoningEngineDeclarationListAndFactory
			.getDeclarationList()
			.get(
			0);
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IAgentClassDeclaration)) {
			return false;
		}

		IAgentClassDeclaration agentClassDeclaration = (IAgentClassDeclaration) anObject;

		return isomorphicEquals(
			getReasoningEngineDeclarationListAndFactory(),
			agentClassDeclaration.getReasoningEngineDeclarationListAndFactory());
	}

	public int isomorphicHashCode() {
		return 11 * super.isomorphicHashCode()
			+ 7 * isomorphicHashCode(getReasoningEngineDeclarationListAndFactory());
	}

	/**
	 * Remove Object (this).
	 * Gets called if Object will be deleted from modell
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		// remove 1:1 link

		getReasoningEngineDeclarationListAndFactory().remove();

		super.remove();
	}
	

	public ReasoningEngineDeclarationListAndFactory getReasoningEngineDeclarationListAndFactory() {
		return _reasoningEngineDeclarationListAndFactory;
	}

}
