package de.uniwue.ki.mas.entity.objectclass.worldclass;
import junit.framework.Assert;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgentInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.world.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.ActivityAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityWorldClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.util.declaration.DeclarationListEvent;
import de.uniwue.ki.mas.util.declaration.DeclarationListListener;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (08.01.2002 15:27:05)
 * @author Christoph Oechslein
 */

class ActivityWorldClassDeclaration
	extends ActivityAgentClassDeclaration
	implements IActivityWorldClassDeclaration {

	private final transient DeclarationListListener _bodyVariableDeclarationListListener =
		new DeclarationListListener() {
		public void declarationAdded(DeclarationListEvent event) {
			((IBodyVariableClassDeclaration) event.getDeclaration()).setInWorldClassDeclaration(
				true);
		}

		public void declarationRemoved(DeclarationListEvent event) {
		}
	};

	private boolean _updateBeforeAllObjects = false;

	public ActivityWorldClassDeclaration() {
		super();

		setUsesShortDescription(true);
		setShortDescription("Untitled World Class");

		getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.addDeclarationListListener(_bodyVariableDeclarationListListener);
	}

	public IActivityAgentInstanceDeclaration createActivityAgentInstanceDeclaration() {
		return createActivityWorldInstanceDeclaration();
	}

	public IActivityWorldInstanceDeclaration createActivityWorldInstanceDeclaration() {
		return ActivityWorldInstanceDeclarationFactory.createActivityWorldInstanceDeclaration(this);
	}

	protected ICopyable createInstance(java.util.Map objectMapping) {
		if (Assert.ASSERTION_ON)
			Assert.assertEquals(getClass(), ActivityWorldClassDeclaration.class);
		return new ActivityWorldClassDeclaration();
	}

	public IWorldInstanceDeclaration createWorldInstanceDeclaration() {
		return createActivityWorldInstanceDeclaration();
	}

	public void remove() {
		getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.removeDeclarationListListener(_bodyVariableDeclarationListListener);

		super.remove();
	}

	public boolean isUpdateBeforeAllObjects() {
		return _updateBeforeAllObjects;
	}

	public void setUpdateBeforeAllObjects(boolean updateBeforeAllObjects) {
		_updateBeforeAllObjects = updateBeforeAllObjects;
	}
}