package de.uniwue.ki.mas.entity.variable;
import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.AbstractNamed;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (17.01.2002 10:43:38)
 * @author Christoph Oechslein
 */

public abstract class AbstractBodyVariable extends AbstractNamed implements IBodyVariable {
	private final boolean _writeable;
	private final boolean _partOfName;

	public AbstractBodyVariable(boolean writeable) {
		super();

		_writeable = writeable;
		_partOfName = false;
	}

	public AbstractBodyVariable(boolean writeable, boolean partOfName) {
		super();

		_writeable = writeable;
		_partOfName = partOfName;		
    }

    public void initializeNextValueFunction(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {
	}

	public boolean isDynamicVariable() {
		return false;
	}

	protected IFunction createNextValueFunction(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		return bodyVariableInstanceDeclaration
			.getBodyVariableClassDeclaration()
			.getVariableClassDeclaration()
			.getNextValueFunctionCallDeclaration()
			.createFunction(createFunctionArgs);
	}

	public boolean getBooleanValue() throws SimulationRuntimeException {
		Object object = getValue();

		if (object instanceof Boolean)
			return ((Boolean) object).booleanValue();
		else
			throw new WrongFunctionTypeException();
	}

	public double getDoubleValue() throws SimulationRuntimeException {
		Object object = getValue();

		if (object instanceof Number)
			return ((Number) object).doubleValue();
		else
			throw new WrongFunctionTypeException();
	}

	public int getIntValue() throws SimulationRuntimeException {
		Object object = getValue();

		if (object instanceof Number)
			return ((Number) object).intValue();
		else
			throw new WrongFunctionTypeException();
	}

	public void setBooleanValue(boolean newValue) throws SimulationRuntimeException {
		setValue(newValue ? Boolean.TRUE : Boolean.FALSE);
	}

	public void setDoubleValue(double newValue) throws SimulationRuntimeException {
		setValue(new Double(newValue));
	}

	public void setIntValue(int newValue) throws SimulationRuntimeException {
		setValue(new Integer(newValue));
	}

	public Object update(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		throw new UnsupportedOperationException();
	}

	public boolean updateBoolean(IExecuteFunctionArgs executeFunctionArgs)
		throws SimulationRuntimeException {
		return ((Boolean) update(executeFunctionArgs)).booleanValue();
	}

	public double updateDouble(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return ((Double) update(executeFunctionArgs)).doubleValue();
	}

	public int updateInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		return ((Integer) update(executeFunctionArgs)).intValue();
	}

	public final String getName() {
		return getNameIntern();
	}

	public void appendName(StringBuffer buff, int currDepth, int maxDepth) {
		if (currDepth >= maxDepth) {
			buff.append("...");
		} else {
			try {
				Object value = getValue();
				if (value instanceof IBody) {
					// maybe recursive cycle in body variables!!
					 ((IBody) value).appendName(buff, currDepth + 1, maxDepth);

				} else if (value instanceof INamed) {
					((INamed) value).appendName(buff, currDepth, maxDepth);

				} else {
					buff.append("" + value);
				}

			} catch (SimulationRuntimeException e) {
				buff.append("Error getting value");
			}
		}
	}

	public boolean isWriteable() {
		return _writeable;
	}
    
	public boolean isPartOfName() {
        return _partOfName;
    }
}