package de.uniwue.ki.mas.entity.variable;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (06.02.2002 15:12:45)
 * @author Christoph Oechslein
 */

public abstract class AbstractBodyVariableInstanceDeclaration
	extends AbstractDeclaration
	implements IBodyVariableInstanceDeclaration, ChangeListener, PropertyChangeListener {

	private final IBodyVariableClassDeclaration _bodyVariableClassDeclaration;

	private final DefaultOwnDefaultValueFunctionCallDeclaration _defaultOwnDefaultValueFunctionCallDeclaration =
		new DefaultOwnDefaultValueFunctionCallDeclaration();

	public AbstractBodyVariableInstanceDeclaration(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		super();

		_bodyVariableClassDeclaration = bodyVariableClassDeclaration;

		initializeDefaultValueFunctionCallDeclaration();

		// so you cannot delete a bodyVariable in the objectclass
		// getBodyVariableClassDeclaration().addRemoveVetoListener(neverRemoveVetoListener);

		getBodyVariableClassDeclaration().addChangeListener(this);
		getBodyVariableClassDeclaration().addPropertyChangeListener("name", this);
		getBodyVariableClassDeclaration().getVariableClassDeclaration().addPropertyChangeListener(
			"type",
			this);
		_defaultOwnDefaultValueFunctionCallDeclaration.addChangeListener(this);
		getBodyVariableClassDeclaration().addBodyVariableInstanceDeclaration(this);
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IBodyVariableInstanceDeclaration dstBodyVariableInstanceDeclaration =
			(IBodyVariableInstanceDeclaration) intoObject;

		if (Assert.ASSERTION_ON)
			Assert.assertSame(
				"body var class must be the same: "
					+ getName(getBodyVariableClassDeclaration())
					+ " != "
					+ getName(dstBodyVariableInstanceDeclaration.getBodyVariableClassDeclaration()),
				getBodyVariableClassDeclaration(),
				dstBodyVariableInstanceDeclaration.getBodyVariableClassDeclaration());

		dstBodyVariableInstanceDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(
			getUsesOwnDefaulValueFunctionCallDeclaration());

		getDefaultValueFunctionCallDeclaration().copyInto(
			dstBodyVariableInstanceDeclaration.getDefaultValueFunctionCallDeclaration(),
			objectMapping);
	}

	public IBodyVariableClassDeclaration getBodyVariableClassDeclaration() {
		return _bodyVariableClassDeclaration;
	}

	public IFunctionCallDeclaration getDefaultValueFunctionCallDeclaration() {
		return _defaultOwnDefaultValueFunctionCallDeclaration.getDefaultValueFunctionCallDeclaration();
	}

	public String getName() {
		return getBodyVariableClassDeclaration().getName();
	}

	public IFunctionCallDeclaration getSpecificFunctionCallDeclaration()
		throws SimulationRuntimeException, ModelCompilingException {

		if (getBodyVariableClassDeclaration() instanceof IBodyFeatureVariableClassDeclaration) {
			return _defaultOwnDefaultValueFunctionCallDeclaration.getRightDefaulValueFunctionCallDeclaration(
				(IBodyFeatureVariableClassDeclaration) getBodyVariableClassDeclaration(),
				getVariableClassDeclaration().getDefaultValueFunctionCallDeclaration());

		} else {
			return _defaultOwnDefaultValueFunctionCallDeclaration.getRightDefaulValueFunctionCallDeclaration(
				getVariableClassDeclaration().getDefaultValueFunctionCallDeclaration());
		}
	}

	public boolean getUsesOwnDefaulValueFunctionCallDeclaration() {
		return _defaultOwnDefaultValueFunctionCallDeclaration.getUsesOwnDefaulValueFunctionCallDeclaration();
	}

	public IVariableClassDeclaration getVariableClassDeclaration() {
		return getBodyVariableClassDeclaration().getVariableClassDeclaration();
	}

	private void initializeDefaultValueFunctionCallDeclaration() {
		initializeDefaultValueFunctionCallDeclaration(
			getBodyVariableClassDeclaration().getVariableClassDeclaration().getSpecificType());
	}

	private void initializeDefaultValueFunctionCallDeclaration(ISpecificType type) {
		if (type != null) {
			getDefaultValueFunctionCallDeclaration().setMostGeneralOutputType(type);
		}
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void propertyChange(PropertyChangeEvent evt) {

		if (evt.getSource() == getBodyVariableClassDeclaration()) {
			if (evt.getPropertyName().equals("name")) {
				firePropertyChange("name", evt.getOldValue(), evt.getNewValue());
			}

		} else if (evt.getSource() == getBodyVariableClassDeclaration().getVariableClassDeclaration()) {
			if (evt.getPropertyName().equals("type")) {
				initializeDefaultValueFunctionCallDeclaration((ISpecificType) evt.getNewValue());
			}
		}
	}

	public void remove() {
		getBodyVariableClassDeclaration().removeChangeListener(this);
		_defaultOwnDefaultValueFunctionCallDeclaration.removeChangeListener(this);
		getBodyVariableClassDeclaration().removePropertyChangeListener("name", this);
		getBodyVariableClassDeclaration().getVariableClassDeclaration().removePropertyChangeListener(
			"type",
			this);
		getBodyVariableClassDeclaration().removeBodyVariableInstanceDeclaration(this);

		remove(getDefaultValueFunctionCallDeclaration());

		super.remove();
	}

	public void setUsesOwnDefaulValueFunctionCallDeclaration(boolean usesOwnDefaulValueFunctionCallDeclaration) {
		final boolean oldValue = getUsesOwnDefaulValueFunctionCallDeclaration();

		_defaultOwnDefaultValueFunctionCallDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(
			usesOwnDefaulValueFunctionCallDeclaration);

		firePropertyChange(
			"usesOwnDefaulValueFunctionCallDeclaration",
			oldValue,
			usesOwnDefaulValueFunctionCallDeclaration);
		fireStateChanged();
	}

	public void stateChanged(ChangeEvent e) {
		fireStateChanged();
	}
}