package de.uniwue.ki.mas.entity.variable;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.copy.ICopyable;
/**
 * Creation date: (24.11.2000 15:59:04)
 * @author Christoph Oechslein
 */

class BodyVariableInstanceDeclaration extends AbstractBodyVariableInstanceDeclaration {
	private static final Logger _logger = Logger.getLogger(BodyVariableInstanceDeclaration.class.getName());

	public BodyVariableInstanceDeclaration(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		super(bodyVariableClassDeclaration);
	}

	public IBodyVariable createBodyVariable(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		if (createFunctionArgs.getFunctionDeclarationCompiler().isCompileOn()
			&& isConstant(createFunctionArgs)
			&& !getBodyVariableClassDeclaration().isInWorldClassDeclaration()) {
			return getConstantBodyVariable(createFunctionArgs);

		} else {
			return createBodyVariableIntern(createFunctionArgs);
		}
	}

	private IBodyVariable getConstantBodyVariable(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		if (!createFunctionArgs.existsCompilationObject(getVariableClassDeclaration(), false)) {
			if (Assert.ASSERTION_ON && _logger.isLoggable(Level.INFO)) {
				_logger.info(
					"Adding constant bodyVariable '"
						+ getVariableClassDeclaration().getName()
						+ "' and caching it");
			}

			IBodyVariable newBodyVariable = createBodyVariableIntern(createFunctionArgs);
			createFunctionArgs.addCompilationObject(getVariableClassDeclaration(), newBodyVariable, false);

			return newBodyVariable;

		} else {
			if (Assert.ASSERTION_ON && _logger.isLoggable(Level.FINE)) {
				_logger.fine(
					"Retrieving constant bodyVariable '" + getVariableClassDeclaration().getName() + "'");
			}

			return (IBodyVariable) createFunctionArgs.getCompilationObject(
				getVariableClassDeclaration(),
				false);
		}
	}

	private IBodyVariable createBodyVariableIntern(ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException {

		boolean writeable = getBodyVariableClassDeclaration().getVariableClassDeclaration().isWriteable();
		boolean partOfName = getBodyVariableClassDeclaration().getVariableClassDeclaration().isPartOfName();
		ISpecificType variableType = getVariableClassDeclaration().getSpecificType();

		if (DoubleType.getInstance().equals(variableType)) {

			if (getVariableClassDeclaration().getHasNextValueFunctionCallDeclaration()) {
				if (Assert.ASSERTION_ON)
					Assert.assertFalse(isConstant(createFunctionArgs));
				return new DoubleBodyVariableWithUpdateFunction(writeable);

			} else {
				return new DoubleBodyVariable(writeable);
			}

		} else if (IntegerType.getInstance().equals(variableType)) {

			if (getVariableClassDeclaration().getHasNextValueFunctionCallDeclaration()) {
				if (Assert.ASSERTION_ON)
					Assert.assertFalse(isConstant(createFunctionArgs));
				return new IntegerBodyVariableWithUpdateFunction(writeable);

			} else {
				return new IntegerBodyVariable(writeable);
			}

		} else if (getVariableClassDeclaration().getHasNextValueFunctionCallDeclaration()) {
			if (Assert.ASSERTION_ON)
				Assert.assertFalse(isConstant(createFunctionArgs));
			return new BodyVariableWithUpdateFunction(writeable, partOfName);

		} else {
			return new BodyVariable(writeable, partOfName);
		}
	}

	protected ICopyable createInstance(Map objectMapping) {
		return BodyVariableInstanceDeclarationFactory.createBodyVariableInstanceDeclaration(
			getBodyVariableClassDeclaration());
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		return getBodyVariableClassDeclaration().isConstant(createFunctionArgs)
			&& !getUsesOwnDefaulValueFunctionCallDeclaration();
	}
}