package de.uniwue.ki.mas.entity.variable;
import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.util.declaration.AbstractMapCopyDeclarationList;

/**
 * Creation date: (06.02.2002 15:18:54)
 * @author Christoph Oechslein
 */

public class BodyVariableInstanceDeclarationList extends AbstractMapCopyDeclarationList {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	public BodyVariableInstanceDeclarationList(BodyVariableClassDeclarationList delegateDeclarationList) {
		super(delegateDeclarationList);
		Iterator it = iterator();
		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration element = (IBodyVariableInstanceDeclaration)it.next();
			element.addChangeListener(_changeListener);
		}
	}

	public boolean accept(Object declaration) {
		return false;
	}

	public void add(int index, Object element) {
		throw new UnsupportedOperationException();
	}

	public boolean canBeRemoved() {
		return true;
	}

	protected void contentsChanged(int index0, int index1) {
		// maybe only the order has changed
		ArrayList list = new ArrayList(index1 - index0 + 1);
		for (int i = index0; i <= index1; i++) {
			IBodyVariableClassDeclaration currBodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration)getDelegateDeclarationList().get(i);

			IBodyVariableInstanceDeclaration oldBodyVariableInstanceDeclaration =
				getBodyVariableInstanceDeclaration(currBodyVariableClassDeclaration);

			list.add(
				(oldBodyVariableInstanceDeclaration != null)
					? oldBodyVariableInstanceDeclaration
					: mapBodyVariableClassDeclaration(currBodyVariableClassDeclaration));
		}

		boolean oldComposition = getCopyDeclarationList().isComposition();
		getCopyDeclarationList().setComposition(false);
		for (int i = 0; i < list.size(); i++) {
			IBodyVariableInstanceDeclaration currBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)list.get(i);
			IBodyVariableInstanceDeclaration oldBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)getCopyDeclarationList().get(i + index0);

			getCopyDeclarationList().set(i + index0, currBodyVariableInstanceDeclaration);

			if (!list.contains(oldBodyVariableInstanceDeclaration)) {
				oldBodyVariableInstanceDeclaration.remove();
			}
		}
		getCopyDeclarationList().setComposition(oldComposition);

		fireContentsChanged(this, index0, index1);
	}

	public void copyInto(Object intoObject, Map objectMapping) {

		BodyVariableInstanceDeclarationList dstBodyVariableInstanceDeclarationList =
			(BodyVariableInstanceDeclarationList)intoObject;

		Assert.assertEquals(size(), dstBodyVariableInstanceDeclarationList.size());

		Iterator it = iterator();
		Iterator it2 = dstBodyVariableInstanceDeclarationList.iterator();

		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration currBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)it.next();
			IBodyVariableInstanceDeclaration currDstBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)it2.next();

			currBodyVariableInstanceDeclaration.copyInto(
				currDstBodyVariableInstanceDeclaration,
				objectMapping);
		}
	}

	public IBodyVariableInstanceDeclaration getBodyVariableInstanceDeclaration(IVariableClassDeclaration variableClassDeclaration) {
		Iterator it = iterator();
		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration currIBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)it.next();

			if (currIBodyVariableInstanceDeclaration
				.getBodyVariableClassDeclaration()
				.getVariableClassDeclaration()
				== variableClassDeclaration) {
				return currIBodyVariableInstanceDeclaration;
			}
		}

		return null;
	}

	public IBodyVariableInstanceDeclaration getBodyVariableInstanceDeclaration(IBodyVariableClassDeclaration bodyVariableClassDeclaration) {
		Iterator it = iterator();
		while (it.hasNext()) {
			IBodyVariableInstanceDeclaration currIBodyVariableInstanceDeclaration =
				(IBodyVariableInstanceDeclaration)it.next();

			if (currIBodyVariableInstanceDeclaration.getBodyVariableClassDeclaration()
				== bodyVariableClassDeclaration) {
				return currIBodyVariableInstanceDeclaration;
			}
		}

		return null;
	}

	public String getName() {
		return "Body Instance Variables";
	}

	public Object mapElement(Object newAddedObject) {
		return mapBodyVariableClassDeclaration((IBodyVariableClassDeclaration)newAddedObject);
	}

	private IBodyVariableInstanceDeclaration mapBodyVariableClassDeclaration(IBodyVariableClassDeclaration newAddedBodyVariableClassDeclaration) {
		return newAddedBodyVariableClassDeclaration.createBodyVariableInstanceDeclaration();
	}

	public void remove() {
		getCopyDeclarationList().remove();
		Iterator it = iterator();
		while (it.hasNext()) {
			BodyVariableInstanceDeclaration element = (BodyVariableInstanceDeclaration)it.next();
			element.removeChangeListener(_changeListener);
		}

		super.remove();
	}

	public Object remove(int index) {
		throw new UnsupportedOperationException();
	}

	public boolean remove(Object o) {
		throw new UnsupportedOperationException();
	}

	public Object set(int index, Object element) {
		throw new UnsupportedOperationException();
	}

	public void sort(Comparator comparator) {
		throw new UnsupportedOperationException();
	}

	public int changeIndex(Object anObject, int newIndex) {
		throw new UnsupportedOperationException();
	}
}