package de.uniwue.ki.mas.entity.variable;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.WrongFunctionTypeException;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
* Creation date: (12.05.00 18:09:34)
* @author Christoph Oechslein
*/

class DoubleBodyVariable extends AbstractBodyVariable implements IBodyVariable {
	private double _value;

	public DoubleBodyVariable(boolean writeable) {
		super(writeable);
	}

	public boolean getBooleanValue() throws SimulationRuntimeException {
		throw new WrongFunctionTypeException();
	}

	public double getDoubleValue() {
		return _value;
	}

	public int getIntValue() {
		throw new UnsupportedOperationException();
	}

	public Object getValue() {
		return new Double(_value);
	}

	public void initializeDefaultValue(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException, SimulationRuntimeException {

		setDoubleValueIntern(
			bodyVariableInstanceDeclaration
				.getSpecificFunctionCallDeclaration()
				.createFunction(createFunctionArgs)
				.executeForDouble(createFunctionArgs.createCreateAndExecuteFunctionArgs()));
	}

	public void setBooleanValue(boolean newValue) throws SimulationRuntimeException {
		throw new WrongFunctionTypeException();
	}

	public final void setDoubleValue(double newValue) {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(isWriteable());
		}

		setDoubleValueIntern(newValue);
	}

	public final void setIntValue(int newValue) {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(isWriteable());
		}

		setDoubleValueIntern(newValue);
	}

	public final void setValue(Object newValue) {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(isWriteable());
			Assert.assertTrue(newValue instanceof Number);
		}

		setDoubleValueIntern(((Number) newValue).doubleValue());
	}

	protected final void setDoubleValueIntern(double newValue) {
		_value = newValue;
	}

	public void appendName(StringBuffer buff, int depth) {
		buff.append(getDoubleValue() + "");
	}
}