package de.uniwue.ki.mas.entity.variable;

import java.io.Serializable;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.agent.IAgent;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.util.name.INamed;

/**
 * A body variable is a runtime simulation object
 * which is an instance variable of an agent.
 * <BR>Creation date: (12.05.00 18:47:47)
 *
 * @author Christoph Oechslein
 */

public interface IBodyVariable extends Serializable, INamed {

	/**
	 * Returns the value of this Variable.
	 * Look at IBody the 'getValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#getValueOfVariable(IBodyVariable)
	 * @see #setValue(Object)
	 * @see #getValueForSensingBy(IAgent)
	 */
	Object getValue() throws SimulationRuntimeException;

	/**
	  * Initialize the default value of this.
	  */
	void initializeDefaultValue(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException, SimulationRuntimeException;

	/**
	  * Compiles the next value function.
	  */
	void initializeNextValueFunction(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException;

	/**
	 * Sets the value of this Variable.
	 * Look at IBody the 'setValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#setValueOfVariable(IBodyVariable, Object)
	 * @see #getValue()
	 */
	void setValue(Object obj) throws SimulationRuntimeException;

	/**
	 * Returns the value of this Variable.
	 * Look at IBody the 'getValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#getValueOfVariable(IBodyVariable)
	 * @see #setValue(Object)
	 * @see #getValueForSensingBy(IAgent)
	 */
	boolean getBooleanValue() throws SimulationRuntimeException;

	/**
	 * Returns the value of this Variable.
	 * Look at IBody the 'getValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#getValueOfVariable(IBodyVariable)
	 * @see #setValue(Object)
	 * @see #getValueForSensingBy(IAgent)
	 */
	double getDoubleValue() throws SimulationRuntimeException;

	/**
	 * Returns the value of this Variable.
	 * Look at IBody the 'getValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#getValueOfVariable(IBodyVariable)
	 * @see #setValue(Object)
	 * @see #getValueForSensingBy(IAgent)
	 */
	int getIntValue() throws SimulationRuntimeException;

	/* Returns true if this variable must be updated */
	boolean isDynamicVariable();
	
	/* returns true if this Variable should be uses as SimObjectName */
	public boolean isPartOfName();
	    

	/**
	 * Sets the value of this Variable.
	 * Look at IBody the 'setValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#setValueOfVariable(IBodyVariable, Object)
	 * @see #getValue()
	 */
	void setBooleanValue(boolean value) throws SimulationRuntimeException;

	/**
	 * Sets the value of this Variable.
	 * Look at IBody the 'setValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#setValueOfVariable(IBodyVariable, Object)
	 * @see #getValue()
	 */
	void setDoubleValue(double value) throws SimulationRuntimeException;

	/**
	 * Sets the value of this Variable.
	 * Look at IBody the 'setValueOfVariable' may ba a better choice.
	 *
	 * @see IBody#setValueOfVariable(IBodyVariable, Object)
	 * @see #getValue()
	 */
	void setIntValue(int value) throws SimulationRuntimeException;

	/**
	 * Update variable for next tick (dynamic variables), returns oldValue (can only be called for dynamic variables). 
	 */
	Object update(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Update (boolean) variable for next tick (dynamic variables), returns oldValue (can only be called for dynamic variables). 
	 */
	boolean updateBoolean(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Update (double) variable for next tick (dynamic variables), returns oldValue (can only be called for dynamic variables). 
	 */
	double updateDouble(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;

	/**
	 * Update (double) variable for next tick (dynamic variables), returns oldValue (can only be called for dynamic variables). 
	 */
	int updateInt(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException;
}
