package de.uniwue.ki.mas.entity.variable;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.WrongFunctionTypeException;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

/**
* Creation date: (12.05.00 18:09:34)
* @author Christoph Oechslein
*/

class IntegerBodyVariable extends AbstractBodyVariable implements IBodyVariable {
	private int _value;

	public IntegerBodyVariable(boolean writeable) {
		super(writeable);
	}

	public boolean getBooleanValue() throws SimulationRuntimeException {
		throw new WrongFunctionTypeException();
	}

	public double getDoubleValue() {
		return _value;
	}

	public int getIntValue() {
		return _value;
	}

	public Object getValue() {
		return new Integer(_value);
	}

	public void initializeDefaultValue(
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration,
		ICreateFunctionArgs createFunctionArgs)
		throws ModelCompilingException, SimulationRuntimeException {

		setIntValueIntern(
			bodyVariableInstanceDeclaration
				.getSpecificFunctionCallDeclaration()
				.createFunction(createFunctionArgs)
				.executeForInt(createFunctionArgs.createCreateAndExecuteFunctionArgs()));
	}

	public void setBooleanValue(boolean newValue) throws SimulationRuntimeException {
		throw new WrongFunctionTypeException();
	}

	public final void setDoubleValue(double newValue) {
		//for backward compatibility
		if (newValue == (int) newValue)
			setIntValue((int) newValue);
		else
			throw new UnsupportedOperationException();
	}

	public final void setIntValue(int newValue) {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(isWriteable());
		}

		setIntValueIntern(newValue);
	}

	public final void setValue(Object newValue) {
		if (Assert.ASSERTION_ON) {
			Assert.assertTrue(isWriteable());
			Assert.assertTrue(newValue instanceof Number);
		}

		setIntValueIntern(((Number) newValue).intValue());
	}

	protected final void setIntValueIntern(int newValue) {
		_value = newValue;
	}

	public void appendName(StringBuffer buff, int depth) {
		buff.append(getDoubleValue() + "");
	}
}