package de.uniwue.ki.mas.entity.variable;
import junit.framework.*;
import de.uniwue.ki.mas.basics.function.CreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.ICreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;

public final class TestBodyVariable extends TestCase {

	private final ICreateAndExecuteFunctionArgs _executeFunArgs =
		new CreateAndExecuteFunctionArgs();
	/**
	 * @param methodName String
	 */
	public TestBodyVariable(String methodName) {
		super(methodName);
	}

	private IBodyVariable createBodyVariable(IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration)
		throws ModelCompilingException, SimulationRuntimeException {

		IBodyVariable bodyVariable = bodyVariableInstanceDeclaration.createBodyVariable(_executeFunArgs);

		bodyVariable.initializeDefaultValue(bodyVariableInstanceDeclaration, _executeFunArgs);

		bodyVariable.initializeNextValueFunction(bodyVariableInstanceDeclaration, _executeFunArgs);

		return bodyVariable;
	}

	private IBodyVariable createBodyVariable(IBodyVariableClassDeclaration bodyVariableClassDeclaration)
		throws ModelCompilingException, SimulationRuntimeException {
		return createBodyVariable(bodyVariableClassDeclaration.createBodyVariableInstanceDeclaration());
	}

	private IBodyVariable createBodyVariable(IVariableClassDeclaration variableClassDeclaration)
		throws Exception {
		return createBodyVariable(variableClassDeclaration.createBodyVariableClassDeclaration());
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestBodyVariable.class);
	}

	/**
	 * @return Test
	 */
	public static Test suite() {
		TestSuite anSuite = new TestSuite("BodyVariable");

		anSuite.addTest(new TestBodyVariable("testBodyVariable"));
		anSuite.addTest(new TestBodyVariable("testBodyDynamicVariable"));

		return anSuite;
	}

	public void testBodyDynamicVariable() throws Exception {
		IVariableClassDeclaration variableClassDeclaration =
			VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration(
				DoubleType.getInstance());

		variableClassDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(1.0)));

		variableClassDeclaration.setHasNextValueFunctionCallDeclaration(true);
		variableClassDeclaration.setWriteable(true);

		variableClassDeclaration.getNextValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(999.0)));

		IBodyVariable bodyVariable = createBodyVariable(variableClassDeclaration);

		testBodyVariable(bodyVariable, 1.0);

		bodyVariable.updateDouble(_executeFunArgs);

		testBodyVariable(bodyVariable, 999.0);

		bodyVariable.updateDouble(_executeFunArgs);

		testBodyVariable(bodyVariable, 999.0);
	}

	/**
	 * @throws Exception
	 */
	public void testBodyVariable() throws Exception {
		IFeatureVariableClassDeclaration variableClassDeclaration =
			FeatureVariableClassDeclarationFactory.createFeatureVariableClassDeclaration(
				DoubleType.getInstance());

		variableClassDeclaration.setWriteable(true);

		variableClassDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(9.0)));

		testBodyVariable(createBodyVariable(variableClassDeclaration), 9.0);

		//

		IBodyFeatureVariableClassDeclaration bodyVariableClassDeclaration =
			(IBodyFeatureVariableClassDeclaration) variableClassDeclaration
				.createBodyVariableClassDeclaration();

		bodyVariableClassDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(true);
		bodyVariableClassDeclaration
			.getDefaultValueFunctionCallDeclaration()
			.setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(99.0)));

		testBodyVariable(createBodyVariable(bodyVariableClassDeclaration), 99.0);

		//

		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
			variableClassDeclaration
				.createBodyVariableClassDeclaration()
				.createBodyVariableInstanceDeclaration();

		bodyVariableInstanceDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(true);
		bodyVariableInstanceDeclaration
			.getDefaultValueFunctionCallDeclaration()
			.setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				DoubleType.getInstance(),
				new Double(999.0)));

		testBodyVariable(createBodyVariable(bodyVariableInstanceDeclaration), 999.0);
	}

	private void testBodyVariable(IBodyVariable bodyVariable, double startValue)
		throws SimulationRuntimeException {

		assertNotNull(bodyVariable);
		assertNotNull(bodyVariable.getValue());
		assertEquals(startValue, bodyVariable.getDoubleValue(), 0.0001);

		assertTrue(bodyVariable.getValue() instanceof Double);

		double newValue = Math.random();

		bodyVariable.setDoubleValue(newValue);

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(newValue, bodyVariable.getDoubleValue(), 0.0001);
	}
}