package de.uniwue.ki.mas.entity.variableclass;
import java.beans.PropertyChangeEvent;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (12.05.00 15:27:17)
 * @author
 */
public abstract class AbstractBodyFeatureVariableClassDeclaration
	extends AbstractBodyVariableClassDeclaration
	implements IBodyFeatureVariableClassDeclaration {

	private final DefaultOwnDefaultValueFunctionCallDeclaration _defaultOwnDefaultValueFunctionCallDeclaration =
		new DefaultOwnDefaultValueFunctionCallDeclaration();

	public AbstractBodyFeatureVariableClassDeclaration(IFeatureVariableClassDeclaration variableClassDeclaration) {
		super(variableClassDeclaration);

		initializeDefaultValueFunctionCallDeclaration();
	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IBodyFeatureVariableClassDeclaration dstBodyFeatureVariableClassDeclaration =
			(IBodyFeatureVariableClassDeclaration) inToObject;

		dstBodyFeatureVariableClassDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(
			getUsesOwnDefaulValueFunctionCallDeclaration());

		getDefaultValueFunctionCallDeclaration().copyInto(
			dstBodyFeatureVariableClassDeclaration.getDefaultValueFunctionCallDeclaration(),
			objectMapping);

		if (Assert.ASSERTION_ON)
			Assert.assertSame(
				getVariableClassDeclaration(),
				dstBodyFeatureVariableClassDeclaration.getVariableClassDeclaration());
	}

	protected void eventVariableClassPropertyChange(PropertyChangeEvent evt) {

		if (evt.getPropertyName().equals("type")) {
			initializeDefaultValueFunctionCallDeclaration((ISpecificType) evt.getNewValue());

		} else {
			super.eventVariableClassPropertyChange(evt);
		}
	}

	public IFunctionCallDeclaration getDefaultValueFunctionCallDeclaration() {
		return _defaultOwnDefaultValueFunctionCallDeclaration.getDefaultValueFunctionCallDeclaration();
	}

	public String getName() {
		if (getVariableClassDeclaration() == null) {
			return "";
		} else {
			return getVariableClassDeclaration().getName();
		}
	}

	public boolean getUsesOwnDefaulValueFunctionCallDeclaration() {
		return _defaultOwnDefaultValueFunctionCallDeclaration.getUsesOwnDefaulValueFunctionCallDeclaration();
	}

	private void initializeDefaultValueFunctionCallDeclaration() {
		initializeDefaultValueFunctionCallDeclaration(getVariableClassDeclaration().getSpecificType());
	}

	private void initializeDefaultValueFunctionCallDeclaration(
		de.uniwue.ki.mas.basics.type.ISpecificType type) {
		if (type != null) {
			getDefaultValueFunctionCallDeclaration().setMostGeneralOutputType(type);
		}
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IBodyFeatureVariableClassDeclaration)) {
			return false;
		}

		IBodyFeatureVariableClassDeclaration bodyFeatureVariableClassDeclaration =
			(IBodyFeatureVariableClassDeclaration) anObject;

		return isomorphicEquals(
			getDefaultValueFunctionCallDeclaration(),
			bodyFeatureVariableClassDeclaration.getDefaultValueFunctionCallDeclaration())
			&& getUsesOwnDefaulValueFunctionCallDeclaration()
				== bodyFeatureVariableClassDeclaration.getUsesOwnDefaulValueFunctionCallDeclaration();
	}

	public int isomorphicHashCode() {
		return 11 * super.isomorphicHashCode()
			+ 7 * isomorphicHashCode(getVariableClassDeclaration())
			+ 3 * isomorphicHashCode(getDefaultValueFunctionCallDeclaration())
			+ isomorphicHashCode(getUsesOwnDefaulValueFunctionCallDeclaration());
	}

	public void remove() {
		super.remove();

		getDefaultValueFunctionCallDeclaration().remove();
	}

	public void setUsesOwnDefaulValueFunctionCallDeclaration(boolean usesOwnDefaulValue) {
		boolean oldValue = getUsesOwnDefaulValueFunctionCallDeclaration();

		_defaultOwnDefaultValueFunctionCallDeclaration.setUsesOwnDefaulValueFunctionCallDeclaration(
			usesOwnDefaulValue);

		firePropertyChange("usesOwnDefaulValueFunctionCallDeclaration", oldValue, usesOwnDefaulValue);
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclaration();
		if (variableClassDeclaration.isWriteable())
			return false;

		if (variableClassDeclaration.getHasNextValueFunctionCallDeclaration())
			return false;

		if (getUsesOwnDefaulValueFunctionCallDeclaration())
			return getDefaultValueFunctionCallDeclaration().isConstant(createFunctionArgs);
		else
			return variableClassDeclaration.getDefaultValueFunctionCallDeclaration().isConstant(
				createFunctionArgs);
	}
}