package de.uniwue.ki.mas.entity.variableclass;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.variable.BodyVariableInstanceDeclarationFactory;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (12.05.00 15:27:17)
 * @author
 */
public abstract class AbstractBodyVariableClassDeclaration
	extends AbstractDeclaration
	implements IBodyVariableClassDeclaration {

	private final IVariableClassDeclaration _variableClassDeclaration;
	private final List _bodyVariableInstanceDeclarationList = new LinkedList();
	
	private boolean _inWorldClassDeclaration;

	private final transient EventListener _eventListener = new EventListener();
	private class EventListener implements PropertyChangeListener, ChangeListener {
		public void stateChanged(ChangeEvent evt) {
			if (evt.getSource() == getVariableClassDeclaration()) {
				fireStateChanged();
			}
		}

		public void propertyChange(PropertyChangeEvent evt) {
			if (evt.getSource() == getVariableClassDeclaration()) {
				eventVariableClassPropertyChange(evt);
			}
		}
	}
	

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);
		
		// don't copy the _inWorldClassDeclaration since it will be set correctly for
		// WorldClassDecls
	}	

	public AbstractBodyVariableClassDeclaration(IVariableClassDeclaration variableClassDeclaration) {
		super();

		_variableClassDeclaration = variableClassDeclaration;

		addListeners(variableClassDeclaration);
	}

	private void addListeners(IVariableClassDeclaration variableClassDeclaration) {
		if (variableClassDeclaration != null) {
			variableClassDeclaration.addBodyVariableClassDeclaration(this);
			variableClassDeclaration.addChangeListener(_eventListener);
			variableClassDeclaration.addPropertyChangeListener("name", _eventListener);

			// problem with variables in userFeature: once added no variable in the feature can be removed since bodyVar has a lock
			// variableClassDeclaration.addRemoveVetoListener(_neverRemoveVetoListener);
		}
	}

	/**
	 * @return IBodyVariableInstanceDeclaration
	 * @see de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration#createBodyVariableInstanceDeclaration()
	 */
	public IBodyVariableInstanceDeclaration createBodyVariableInstanceDeclaration() {
		return BodyVariableInstanceDeclarationFactory.createBodyVariableInstanceDeclaration(this);
	}

	protected void eventVariableClassPropertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals("name")) {
			firePropertyChange("name", evt.getOldValue(), evt.getNewValue());
		}
	}

	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		super.firePropertyChange(propertyName, oldValue, newValue);
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	public String getName() {
		if (getVariableClassDeclaration() == null) {
			return "";
		} else {
			return getVariableClassDeclaration().getName();
		}
	}

	/**
	 * @return IVariableClassDeclaration
	 * @see de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration#getVariableClassDeclaration()
	 */
	public IVariableClassDeclaration getVariableClassDeclaration() {
		return _variableClassDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);

		// leads to error in the compiling
		/*
		if (anObject == this)
		    return true;
		
		if (!super.isomorphicEquals(anObject)) {
		    return false;
		}
		
		if (!(anObject instanceof IBodyVariableClassDeclaration)) {
		    return false;
		}
		
		IBodyVariableClassDeclaration bodyVariableClassDeclaration = (IBodyVariableClassDeclaration) anObject;
		
		return isomorphicEquals(getVariableClassDeclaration(), bodyVariableClassDeclaration.getVariableClassDeclaration());
		*/
	}

	/**
	 * @return int
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicHashCode()
	 */
	public int isomorphicHashCode() {
		return hashCode();

		// leads to error in the compiling
		/*
		return 3 * super.isomorphicHashCode() + 1 * isomorphicHashCode(getVariableClassDeclaration());
		*/
	}

	/**
	 * Remove Object (this).
	 * Gets called if Object will be deleted from modell
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		// remove link (1:n) to variableClassDeclaration
		removeListeners(getVariableClassDeclaration());

		super.remove();
	}

	private void removeListeners(IVariableClassDeclaration variableClassDeclaration) {
		if (variableClassDeclaration != null) {
			variableClassDeclaration.removeChangeListener(_eventListener);
			variableClassDeclaration.removePropertyChangeListener("name", _eventListener);
			variableClassDeclaration.removeBodyVariableClassDeclaration(this);

			// see addListeners
			// variableClassDeclaration.removeRemoveVetoListener(_neverRemoveVetoListener);
		}
	}

	public void addBodyVariableInstanceDeclaration(IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration) {
		Assert.assertFalse(_bodyVariableInstanceDeclarationList.contains(bodyVariableInstanceDeclaration));

		_bodyVariableInstanceDeclarationList.add(bodyVariableInstanceDeclaration);
	}

	public void removeBodyVariableInstanceDeclaration(IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration) {
		Assert.assertTrue(_bodyVariableInstanceDeclarationList.contains(bodyVariableInstanceDeclaration));

		_bodyVariableInstanceDeclarationList.remove(bodyVariableInstanceDeclaration);
	}

	public Iterator getBodyVariableInstanceDeclarations() {
		return _bodyVariableInstanceDeclarationList.iterator();
	}

	public void setInWorldClassDeclaration(boolean inWorldClassDeclaration) {
		_inWorldClassDeclaration = inWorldClassDeclaration;
	}

	public boolean isInWorldClassDeclaration() {
		return _inWorldClassDeclaration;
	}
}