package de.uniwue.ki.mas.entity.variableclass;
import java.util.Map;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.util.declaration.NeverRemoveVetoListener;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (25.07.2001 14:03:25)
 * @author Christoph Oechslein
 */

final class BodyVariableClassDeclaration extends AbstractBodyVariableClassDeclaration {
	private final NeverRemoveVetoListener _neverRemoveVetoListener =
		new NeverRemoveVetoListener(this);

	public BodyVariableClassDeclaration(IVariableClassDeclaration variableClassDeclaration) {
		super(variableClassDeclaration);

		addListeners(variableClassDeclaration);
	}

	private void addListeners(IVariableClassDeclaration variableClassDeclaration) {
		if (variableClassDeclaration != null) {
			variableClassDeclaration.addRemoveVetoListener(getNeverRemoveVetoListener());
		}
	}

	public ICopyable copy(Map objectMapping) {
		BodyVariableClassDeclaration bodyVariableClassDeclaration =
			new BodyVariableClassDeclaration(
				(IVariableClassDeclaration) getVariableClassDeclaration().copy(objectMapping));

		super.copyInto(bodyVariableClassDeclaration, objectMapping);

		return bodyVariableClassDeclaration;
	}

	public void copyInto(Object inToObject, java.util.Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IBodyVariableClassDeclaration dstBodyVariableClassDeclaration =
			(IBodyVariableClassDeclaration) inToObject;

		getVariableClassDeclaration().copyInto(
			dstBodyVariableClassDeclaration.getVariableClassDeclaration(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return getVariableClassDeclaration().createBodyVariableClassDeclaration();
	}

	protected final NeverRemoveVetoListener getNeverRemoveVetoListener() {
		return _neverRemoveVetoListener;
	}

	public void remove() {
		removeListeners(getVariableClassDeclaration());

		if (getVariableClassDeclaration() != null) {
			if (getVariableClassDeclaration().canBeRemoved()) {
				getVariableClassDeclaration().remove();
			}
		}

		super.remove();
	}

	private void removeListeners(IVariableClassDeclaration variableClassDeclaration) {
		if (variableClassDeclaration != null) {
			variableClassDeclaration.removeRemoveVetoListener(getNeverRemoveVetoListener());
		}
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) throws ModelCompilingException {
		IVariableClassDeclaration variableClassDeclaration = getVariableClassDeclaration();

		if (variableClassDeclaration.isWriteable())
			return false;

		if (variableClassDeclaration.getHasNextValueFunctionCallDeclaration())
			return false;

		return variableClassDeclaration.getDefaultValueFunctionCallDeclaration().isConstant(
			createFunctionArgs);
	}
}