package de.uniwue.ki.mas.entity.variableclass;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

/**
 * This class is the declaration of a instance variable of an agent.
 * <BR>Creation date: (31.03.00 13:42:52)
 *
 * @author Christoph Oechslein
 */

public interface IVariableClassDeclaration extends IDocumentableDeclaration {

	/**
	 * Creates new instance declaration form this class declaration.
	 */
	IBodyVariableClassDeclaration createBodyVariableClassDeclaration();

	/**
	 * Returns the default value of this variable class declaration.
	 */
	IFunctionCallDeclaration getDefaultValueFunctionCallDeclaration();

	/**
	 * Returns <code>true</code> if this variable class declaration is
	 * dynamic and has a next value function call declaration.
	 *
	 * @see #setHasNextValueFunctionCallDeclaration(boolean)
	 * @see #getNextValueFunctionCallDeclaration()
	 */
	boolean getHasNextValueFunctionCallDeclaration();

	/**
	 * Returns the next value function call, variable class declaration uses this
	 * if has next value function is <code>true></code>.
	 *
	 * @see #getHasNextValueFunctionCallDeclaration()
	 * @see #setHasNextValueFunctionCallDeclaration(boolean)
	 */
	IFunctionCallDeclaration getNextValueFunctionCallDeclaration();

	/**
	 * Returns <code>true</code> if this variable class declaration is not constant.
	 */
	boolean isWriteable();

	/**
	 * Set to <code>true</code> if this variable class declaration is
	 * dynamic and has a next value function call declaration.
	 *
	 * @see #getHasNextValueFunctionCallDeclaration()
	 * @see #getNextValueFunctionCallDeclaration()
	 */
	void setHasNextValueFunctionCallDeclaration(boolean hasNextValueFunctionCallDeclaration);

	/**
	 * Set to <code>false</code> for constant variables.
	 *
	 * @see #isWriteable()
	 */
	void setWriteable(boolean writeable);

	/**
	 * Returns <code>true</code> if this variable class declaration can
	 * be modified by other agents.
	 *
	 * @see #setExternal(boolean)
	 */
	boolean isExternal();

	/**
	 *  Set to <code>true</code> if this variable class declaration can
	 * be modified by other agents.
	 *
	 * @see #isExternal()
	 */
	void setExternal(boolean newSetting);
	
	/**
	 * Returns <code>true</code> if this variable class declaration should
	 * be used to generate the runtimename of the agent
	 */
	boolean isPartOfName();
    
	void setPartOfName(boolean partOfName);

	/**
	 * Returns the tpye of this variable class declaration.
	 *
	 * @see #setType(ISpecificType)
	 */
	ISpecificType getSpecificType();

	/**
	 * Sets the tpye of this variable class declaration.
	 *
	 * @see #getType()
	 */
	void setSpecificType(ISpecificType type);

	void addBodyVariableClassDeclaration(IBodyVariableClassDeclaration BodyVariableClassDeclaration);
	void removeBodyVariableClassDeclaration(IBodyVariableClassDeclaration BodyVariableClassDeclaration);
	Iterator getBodyVariableClassDeclarations();
}
