package de.uniwue.ki.mas.entity.variableclass;

import java.util.Map;

import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.util.copy.ICopyable;
/**
* Creation date: (24.11.2000 15:34:58)
* @author Christoph Oechslein
*/
class VariableClassDeclaration extends AbstractVariableClassDeclaration {
	
	private boolean _hasNextValueFunctionCallDeclaration = false;
	private final IFunctionCallDeclaration _nextValueFunctionCallDeclaration;

	public VariableClassDeclaration() {
		super();

		_nextValueFunctionCallDeclaration =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(getSpecificType());
	}

	public VariableClassDeclaration(ISpecificType type) {
		super(type);

		_nextValueFunctionCallDeclaration =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(getSpecificType());
	}

	public VariableClassDeclaration(ISpecificType type, IFunctionCallDeclaration defaultValue) {
		super(type, defaultValue);

		_nextValueFunctionCallDeclaration =
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(getSpecificType());
	}

	public IFunctionCallDeclaration getNextValueFunctionCallDeclaration() {
		return _nextValueFunctionCallDeclaration;
	}

	public void copyInto(Object inToObject, Map objectMapping) {
		super.copyInto(inToObject, objectMapping);

		IVariableClassDeclaration dstVariableClassDeclaration = (IVariableClassDeclaration) inToObject;

		dstVariableClassDeclaration.setHasNextValueFunctionCallDeclaration(
			getHasNextValueFunctionCallDeclaration());
		getNextValueFunctionCallDeclaration().copyInto(
			dstVariableClassDeclaration.getNextValueFunctionCallDeclaration(),
			objectMapping);
	}

	/**
	 * @return IBodyVariableClassDeclaration
	 * @see de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration#createBodyVariableClassDeclaration()
	 */
	public IBodyVariableClassDeclaration createBodyVariableClassDeclaration() {
		return new BodyVariableClassDeclaration(this);
	}

	/**
	 * @return boolean
	 * @see de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration#getHasNextValueFunctionCallDeclaration()
	 */
	public boolean getHasNextValueFunctionCallDeclaration() {
		return _hasNextValueFunctionCallDeclaration;
	}

	/**
	 * Remove Object (this).
	 * Gets called if Object will be deleted from modell
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		super.remove();

		getNextValueFunctionCallDeclaration().remove();
	}

	/**
	 * @param hasNextValueFunctionCallDeclaration boolean
	 * @see de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration#setHasNextValueFunctionCallDeclaration(boolean)
	 */
	public void setHasNextValueFunctionCallDeclaration(boolean hasNextValueFunctionCallDeclaration) {
		boolean oldValue = _hasNextValueFunctionCallDeclaration;
		_hasNextValueFunctionCallDeclaration = hasNextValueFunctionCallDeclaration;
		firePropertyChange(
			"hasNextValueFunctionCallDeclaration",
			Boolean.valueOf(oldValue),
			Boolean.valueOf(hasNextValueFunctionCallDeclaration));
		fireStateChanged();
	}

	/**
	 * @param type ISpecificType
	 * @see de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration#setSpecificType(ISpecificType)
	 */
	public void setSpecificType(ISpecificType type) {
		super.setSpecificType(type);

		if ((getSpecificType() != null)
			&& getNextValueFunctionCallDeclaration() != null
			&& !getSpecificType().equals(getNextValueFunctionCallDeclaration().getMostGeneralOutputType())) {
				
			initializeNextValueFunctionCallDeclaration();
			fireStateChanged();
		}
	}

	protected ICopyable createInstance(Map objectMapping) {
		return VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration();
	}

	private void initializeNextValueFunctionCallDeclaration() {
		initializeFunctionCallDeclaration(getNextValueFunctionCallDeclaration());
	}
}