package de.uniwue.ki.mas.model;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.objectclass.RessourceClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.objectclass.agentclass.*;
import de.uniwue.ki.mas.entity.objectclass.worldclass.*;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (26.07.2001 10:42:26)
 * @author Christoph Oechslein
 */

abstract class AbstractSeSAmModelDeclaration
	extends AbstractDocumentableDeclaration
	implements ISeSAmModelDeclaration {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private final IWorldClassDeclarationListAndFactory _worldClassDeclarationListAndFactory;
	private final IAgentClassDeclarationListAndFactory _agentClassDeclarationListAndFactory;
	private final IRessourceClassDeclarationListAndFactory _ressourceClassDeclarationListAndFactory;
	private final ISeSAmModelBasicsDeclaration _seSAmModelBasicsDeclaration = new SeSAmModelBasicsDeclaration();
	private final ISimulationElementsDeclaration _simulationElementsDeclaration;

	public AbstractSeSAmModelDeclaration() {
		super();

		_ressourceClassDeclarationListAndFactory = new RessourceClassDeclarationListAndFactory();

		ActivityAgentClassDeclarationFactory activityAgentClassDeclarationFactory =
			new ActivityAgentClassDeclarationFactory();
		_agentClassDeclarationListAndFactory =
			new AgentClassDeclarationListAndFactory(
				new AgentClassDeclarationList(),
				activityAgentClassDeclarationFactory);
		activityAgentClassDeclarationFactory.setAgentClassDeclarationListAndFactory(
			_agentClassDeclarationListAndFactory);

		ActivityWorldClassDeclarationFactory activityWorldClassDeclarationFactory =
			new ActivityWorldClassDeclarationFactory();
		_worldClassDeclarationListAndFactory =
			new WorldClassDeclarationListAndFactory(
				new WorldClassDeclarationList(),
				activityWorldClassDeclarationFactory);
		activityWorldClassDeclarationFactory.setWorldClassDeclarationListAndFactory(
			_worldClassDeclarationListAndFactory);

		_simulationElementsDeclaration = new SimulationElementsDeclaration(this);

		getSeSAmModelBasicsDeclaration().addChangeListener(_changeListener);
		getWorldClassDeclarationListAndFactory().getWorldClassDeclarationList().addChangeListener(_changeListener);
		getAgentClassDeclarationListAndFactory().getAgentClassDeclarationList().addChangeListener(_changeListener);
		getRessourceClassDeclarationListAndFactory().getRessourceClassDeclarationList().addChangeListener(
			_changeListener);
		getSimulationElementsDeclaration().addChangeListener(_changeListener);
	}


	public void clear() {
		getSimulationElementsDeclaration().clear();

		getRessourceClassDeclarationListAndFactory().getRessourceClassDeclarationList().clear();

		getAgentClassDeclarationListAndFactory().getAgentClassDeclarationList().clear();

		getWorldClassDeclarationListAndFactory().getDeclarationList().clear();

		getSeSAmModelBasicsDeclaration().clear();
	}

	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		ISeSAmModelDeclaration intoSeSAmModelDeclaration = (ISeSAmModelDeclaration) intoObject;

		getSimulationElementsDeclaration().copyInto(
			intoSeSAmModelDeclaration.getSimulationElementsDeclaration(),
			objectMapping);

		getWorldClassDeclarationListAndFactory().copyInto(
			intoSeSAmModelDeclaration.getWorldClassDeclarationListAndFactory(),
			objectMapping);
		getAgentClassDeclarationListAndFactory().copyInto(
			intoSeSAmModelDeclaration.getAgentClassDeclarationListAndFactory(),
			objectMapping);
		getRessourceClassDeclarationListAndFactory().copyInto(
			intoSeSAmModelDeclaration.getRessourceClassDeclarationListAndFactory(),
			objectMapping);

		getSeSAmModelBasicsDeclaration().copyInto(
			intoSeSAmModelDeclaration.getSeSAmModelBasicsDeclaration(),
			objectMapping);
	}

	/**
	 * @return IAgentClassDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISeSAmModelDeclaration#getAgentClassDeclarationListAndFactory()
	 */
	public final IAgentClassDeclarationListAndFactory getAgentClassDeclarationListAndFactory() {
		return _agentClassDeclarationListAndFactory;
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Model";
	}

	/**
	 * @return IRessourceClassDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISeSAmModelDeclaration#getRessourceClassDeclarationListAndFactory()
	 */
	public final IRessourceClassDeclarationListAndFactory getRessourceClassDeclarationListAndFactory() {
		return _ressourceClassDeclarationListAndFactory;
	}

	/**
	 * @return ISeSAmModelBasicsDeclaration
	 * @see de.uniwue.ki.mas.model.ISeSAmModelDeclaration#getSeSAmModelBasicsDeclaration()
	 */
	public final ISeSAmModelBasicsDeclaration getSeSAmModelBasicsDeclaration() {
		return _seSAmModelBasicsDeclaration;
	}

	/**
	 * @return ISimulationElementsDeclaration
	 * @see de.uniwue.ki.mas.model.ISeSAmModelDeclaration#getSimulationElementsDeclaration()
	 */
	public final ISimulationElementsDeclaration getSimulationElementsDeclaration() {
		return _simulationElementsDeclaration;
	}

	/**
	 * @return IWorldClassDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISeSAmModelDeclaration#getWorldClassDeclarationListAndFactory()
	 */
	public final IWorldClassDeclarationListAndFactory getWorldClassDeclarationListAndFactory() {
		return _worldClassDeclarationListAndFactory;
	}

	/**
	 * @param anObject IIsomorphicHashable
	 * @return boolean
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicEquals(IIsomorphicHashable)
	 */
	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ISeSAmModelDeclaration)) {
			return false;
		}

		ISeSAmModelDeclaration seSAmModelDeclaration = (ISeSAmModelDeclaration) anObject;

		return isomorphicEquals(
			getRessourceClassDeclarationListAndFactory(),
			seSAmModelDeclaration.getRessourceClassDeclarationListAndFactory())
			&& isomorphicEquals(
				getAgentClassDeclarationListAndFactory(),
				seSAmModelDeclaration.getAgentClassDeclarationListAndFactory())
			&& isomorphicEquals(
				getWorldClassDeclarationListAndFactory(),
				seSAmModelDeclaration.getWorldClassDeclarationListAndFactory())
			&& isomorphicEquals(getSeSAmModelBasicsDeclaration(), seSAmModelDeclaration.getSeSAmModelBasicsDeclaration())
			&& isomorphicEquals(
				getSimulationElementsDeclaration(),
				seSAmModelDeclaration.getSimulationElementsDeclaration());
	}


	public int isomorphicHashCode() {
		return 13 * super.isomorphicHashCode()
			+ 11 * isomorphicHashCode(getRessourceClassDeclarationListAndFactory())
			+ 7 * isomorphicHashCode(getAgentClassDeclarationListAndFactory())
			+ 5 * isomorphicHashCode(getSimulationElementsDeclaration())
			+ 3 * isomorphicHashCode(getSeSAmModelBasicsDeclaration())
			+ isomorphicHashCode(getWorldClassDeclarationListAndFactory());
	}

	public void remove() {
		getSeSAmModelBasicsDeclaration().removeChangeListener(_changeListener);
		getWorldClassDeclarationListAndFactory().getWorldClassDeclarationList().removeChangeListener(_changeListener);
		getAgentClassDeclarationListAndFactory().getAgentClassDeclarationList().removeChangeListener(_changeListener);
		getRessourceClassDeclarationListAndFactory().getRessourceClassDeclarationList().removeChangeListener(
			_changeListener);
		getSimulationElementsDeclaration().removeChangeListener(_changeListener);

		getSimulationElementsDeclaration().remove();
		getWorldClassDeclarationListAndFactory().remove();
		getAgentClassDeclarationListAndFactory().remove();
		getRessourceClassDeclarationListAndFactory().remove();
		getSeSAmModelBasicsDeclaration().remove();

		super.remove();
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}
}