package de.uniwue.ki.mas.model;
import java.io.Serializable;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.UserFunctionsOfAllUserFeatureDeclarationList;
import de.uniwue.ki.mas.basics.featureList.FeatureClassDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * Creation date: (12.10.2001 10:48:30)
 * @author Christoph Oechslein
 */

final class AvailibleFunctionHelper implements Serializable {
	private final ISeSAmModelDeclaration _seSAmModelDeclaration;

	private UserFunctionsOfAllUserFeatureDeclarationList _userFunctionsOfAllUserFeatureDeclarationList;
	private IFunctionDeclarationList _availibleFunctionDeclarationList;

	public AvailibleFunctionHelper(ISeSAmModelDeclaration seSAmModelDeclaration) {
		_seSAmModelDeclaration = seSAmModelDeclaration;
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationList() {
		return new FilterFunctionDeclarationListAble(
			getAvailibleFunctionDeclarationListIntern(),
			filterUserFunctions(true),
			false);
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IObjectClassDeclaration objectClassDeclaration) {
		IFunctionDeclarationList objectfeatureFunctionDeclarationList = new FunctionDeclarationList();
		objectfeatureFunctionDeclarationList.setComposition(false);

		CollectionsHelper.addAll(
			objectfeatureFunctionDeclarationList,
			FeatureClassDeclarationListManager.getInstance().getDefaultFunctionDeclarationList().iterator());

		CollectionsHelper.addAll(
			objectfeatureFunctionDeclarationList,
			objectClassDeclaration.getObjectFeatureClassDeclarationList().getFunctionDeclarations());

		CollectionsHelper.removeDuplicates(objectfeatureFunctionDeclarationList);

		IFunctionDeclarationListAble allInternalDecls =
			new FilterFunctionDeclarationListAble(
				objectfeatureFunctionDeclarationList,
				filterUserFunctions(false),
				true);

		IFunctionDeclarationListAble allExternalUserFunctionFunctionDeclarationListAble =
			new FilterFunctionDeclarationListAble(
				new ConcatenateFunctionDeclarationListAble(
					_seSAmModelDeclaration
						.getSeSAmModelBasicsDeclaration()
						.getUserFunctionDeclarationListAndFactory()
						.getUserFunctionDeclarationList(),
					getUserFunctionsOfAllUserFeatureDeclarationList(),
					false),
				filterUserFunctions(true),
				true);

		return new ConcatenateFunctionDeclarationListAble(
			allExternalUserFunctionFunctionDeclarationListAble,
			allInternalDecls,
			true);
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IUserFeatureClassDeclaration userFeatureClassDeclaration) {
		return new ConcatenateFunctionDeclarationListAble(
			new FilterFunctionDeclarationListAble(
				getAvailibleFunctionDeclarationListIntern(),
				filterUserFunctions(true),
				false),
			new FilterFunctionDeclarationListAble(
				userFeatureClassDeclaration
					.getUserFunctionDeclarationListAndFactory()
					.getUserFunctionDeclarationList(),
				filterUserFunctions(false),
				false),
			true);
	}

	public Iterator getAllAvailibleFunctionDeclarations() {
		return CollectionsHelper
			.concatenate(
				_seSAmModelDeclaration
					.getSeSAmModelBasicsDeclaration()
					.getUserFunctionDeclarationListAndFactory()
					.getUserFunctionDeclarationList()
					.iterator(),
				CollectionsHelper
					.flatten(
						FeatureClassDeclarationListManager
							.getInstance()
							.getBuildinFeatureClassDeclarationList()
							.iterator(),
						new IMapFunction() {
			public Object map(Object anObj) {
				IFeatureClassDeclaration featureClassDeclaration = (IFeatureClassDeclaration) anObj;
				Iterator defaultFunctions = featureClassDeclaration.getDefaultFunctionDeclarations();

				//9. Februar 2004 Rainer
				return CollectionsHelper.concatenate(
						((IFeatureClassDeclaration) anObj).getFunctionDeclarations(),
						defaultFunctions);
				
				//if (featureClassDeclaration instanceof IObjectFeatureClassDeclaration)
				//	return CollectionsHelper.concatenate(
				//		((IObjectFeatureClassDeclaration) anObj).getFunctionDeclarations(),
				//		defaultFunctions);
				//else
				//	return defaultFunctions;
			}
		}));
	}

	private IPredicateFunction filterUserFunctions(final boolean external) {
		return new IPredicateFunction() {
			public boolean isElementOk(Object obj) throws Exception {
				if (obj instanceof IUserFunctionDeclaration) {
					IUserFunctionDeclaration userFunctionDeclaration = (IUserFunctionDeclaration) obj;
					return userFunctionDeclaration.isExternal() == external;

				} else
					return true;
			}
		};
	}

	private IFunctionDeclarationList getAvailibleFunctionDeclarationListIntern() {
		if (_availibleFunctionDeclarationList == null) {
			_availibleFunctionDeclarationList =
				new TwoUnmodifiableFunctionDeclarationList(
					getUserFunctionsOfAllUserFeatureDeclarationList(),
					new TwoUnmodifiableFunctionDeclarationList(
						_seSAmModelDeclaration
							.getSeSAmModelBasicsDeclaration()
							.getUserFunctionDeclarationListAndFactory()
							.getUserFunctionDeclarationList(),
						FeatureClassDeclarationListManager.getInstance().getDefaultFunctionDeclarationList(),
						false),
					true);
		}

		return _availibleFunctionDeclarationList;
	}

	private UserFunctionsOfAllUserFeatureDeclarationList getUserFunctionsOfAllUserFeatureDeclarationList() {
		if (_userFunctionsOfAllUserFeatureDeclarationList == null) {
			_userFunctionsOfAllUserFeatureDeclarationList =
				new UserFunctionsOfAllUserFeatureDeclarationList(
					_seSAmModelDeclaration
						.getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory()
						.getUserFeatureClassDeclarationList());
		}

		return _userFunctionsOfAllUserFeatureDeclarationList;
	}

	public void remove() {
		if (_userFunctionsOfAllUserFeatureDeclarationList != null) {
			_userFunctionsOfAllUserFeatureDeclarationList.remove();
			_userFunctionsOfAllUserFeatureDeclarationList = null;
		}

		if (_availibleFunctionDeclarationList != null) {
			_availibleFunctionDeclarationList.remove();
			_availibleFunctionDeclarationList = null;
		}
	}
}