package de.uniwue.ki.mas.model;
import java.io.File;

import de.uniwue.ki.mas.model.graph.UMLLayoutGraphDeclarationList;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.mas.sim.experiment.ExperimentListAndFactory;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

/**
 * Main SeSAm interface, consists of a sesam model, situations and simulations.
 * Creation date: (13.10.2000 18:00:54)
 * @author Christoph Oechslein
 */

public interface ISeSAmDeclaration extends IDocumentableDeclaration {

	/** 
	 * Returns the file where the model is stored
	 * 
	 * @see #setFile(File)
	 */
    File getFile();

	/** 
	 * Sets the file where the model is stored
	 * 
	 * @see #getFile()
	 */
    void setFile(File newFile);

	/** 
	 * Returns <code>true</code> if model was modified.
	 * 
	 * @see #setModelModified(boolean)
	 */
    boolean isModelModified();
    
	/** 
	 * Sets the modified state.
	 * 
	 * @see #isModelModified()
	 */
    void setModelModified(boolean newModelModified);

    ExperimentListAndFactory getExperimentListAndFactory();

    ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel();

    ISeSAmModelDeclaration getSeSAmModelDeclaration();
    
    UMLLayoutGraphDeclarationList getUMLGraphDeclerationList();

    void clear();
}