package de.uniwue.ki.mas.model;
import java.util.Iterator;
import java.util.List;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationListAble;
import de.uniwue.ki.mas.entity.objectclass.*;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclarationListAndFactory;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

/**
 * Encapsulate one simulation model (lists of objectClassDeclarations, ...) 
 * and factories for new declarations.
 * @author Christoph Oechslein
 */

public interface ISeSAmModelDeclaration extends IDocumentableDeclaration {

	void clear();

	IAgentClassDeclarationListAndFactory getAgentClassDeclarationListAndFactory();

	/**
	 * Returns a list of all availible features in this SeSAm model.
	 */
	IFeatureClassDeclarationList getAvailibleFeatureClassDeclarationList();

	/**
	 * Returns a list of all availible features in this SeSAm model which are applicable for
	 * the given {@link IObjectClassDeclaration}.
	 * 
	 * @see IFeatureClassDeclaration#isDefaultFeatureClassDeclarationFor(IObjectClassDeclaration)
	 */
	IFeatureClassDeclarationList getAvailibleFeatureClassDeclarationListFor(IObjectClassDeclaration objectClassDeclaration);

	/**
	 * Returns a newly created IFunctionDeclarationListAble of all default availible functions (without the
	 * functions of the features) in this SeSAm model. You should call IFunctionDeclarationListAble#remove().
	 * 
	 * @see #getAllAvailibleFunctionDeclarations()
	 */
	IFunctionDeclarationListAble getAvailibleFunctionDeclarationList();

	/**
	 * Returns a newly created IFunctionDeclarationListAble of all availible functions in this SeSAm model for
	 * the given {@link IObjectClassDeclaration}. You should call IFunctionDeclarationListAble#remove().
	 */
	IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IObjectClassDeclaration objectClassDeclaration);

	/**
	 * Returns a newly created IFunctionDeclarationListAble of all availible functions in this SeSAm model for
	 * the given {@link IUserFeatureClassDeclaration}. You should call IFunctionDeclarationListAble#remove().
	 */
	IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IUserFeatureClassDeclaration userFeatureClassDeclaration);

	/**
	 * Returns aList of all availible types in this SeSAm model.
	 */
	List getAvailibleTypeList();

	/**
	 * Returns a list of all availible types in this SeSAm model 
	 * which can be used in variables.
	 */
	List getAvailibleTypeListForVariableClassDeclaration();

	IWorldClassDeclarationListAndFactory getWorldClassDeclarationListAndFactory();

	IRessourceClassDeclarationListAndFactory getRessourceClassDeclarationListAndFactory();

	/**
	 * Returns a list of all availible object classes in this SeSAm model,
	 * its a unmodifiable concatenation of agents and ressources.
	 */
	IObjectClassDeclarationList getAvailibleObjectClassDeclarationList();

	/**
	 * Returns an iterator of all availible functions (plus the functions of the features)
	 * in this SeSAm model.
	 * 
	 * @see #getAvailibleFunctionDeclarations()
	 */
	Iterator getAllAvailibleFunctionDeclarations();

	ISeSAmModelBasicsDeclaration getSeSAmModelBasicsDeclaration();

	ISimulationElementsDeclaration getSimulationElementsDeclaration();
}