package de.uniwue.ki.mas.model;
import java.io.File;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.model.graph.UMLLayoutGraphDeclarationList;
import de.uniwue.ki.mas.model.tree.ISeSAmDeclarationTreeModel;
import de.uniwue.ki.mas.model.tree.SeSAmDeclarationTreeModelAdapterFactory;
import de.uniwue.ki.mas.sim.experiment.ExperimentListAndFactory;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.file.FileUtilities;

/**
 * Creation date: (14.09.2001 10:12:36)
 * @author Christoph Oechslein
 */

class SeSAmDeclaration extends AbstractDocumentableDeclaration implements ISeSAmDeclaration {
	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private boolean _modelModified = false;
	private File _file;
	private final ISeSAmModelDeclaration _seSAmModelDeclaration;
	private final ExperimentListAndFactory _experimentListAndFactory;
	private final UMLLayoutGraphDeclarationList _umlList;

	private final transient ISeSAmDeclarationTreeModel _seSAmDeclarationTreeModel;

	public SeSAmDeclaration() {
		this(new SeSAmModelDeclaration());
	}

	public SeSAmDeclaration(boolean withTree) {
		this(new SeSAmModelDeclaration(), withTree);
	}

	public SeSAmDeclaration(ISeSAmModelDeclaration seSAmModelDeclaration) {
		this(seSAmModelDeclaration, true);
	}

	public SeSAmDeclaration(ISeSAmModelDeclaration seSAmModelDeclaration, boolean withTree) {
		super();

		_seSAmModelDeclaration = seSAmModelDeclaration;

		_experimentListAndFactory =
			new ExperimentListAndFactory(
				this,
				_seSAmModelDeclaration
					.getSimulationElementsDeclaration()
					.getExperimentDeclarationListAndFactory()
					.getExperimentDeclarationList(),
				_seSAmModelDeclaration
					.getSimulationElementsDeclaration()
					.getSimulationDeclarationListAndFactory());
		
		_umlList = new UMLLayoutGraphDeclarationList();

		_seSAmDeclarationTreeModel =
			withTree ? SeSAmDeclarationTreeModelAdapterFactory.createSeSAmDeclarationTreeModel(this) : null;

		getSeSAmModelDeclaration().addChangeListener(_changeListener);
	}

	public void clear() {
		if (getSeSAmDeclarationTreeModel() == null) {
			clearIntern();

		} else {
			boolean oldValue = getSeSAmDeclarationTreeModel().isEventCreation();
			try {
				getSeSAmDeclarationTreeModel().setEventCreation(false);

				clearIntern();

			} finally {
				getSeSAmDeclarationTreeModel().setEventCreation(oldValue);
			}
		}
	}

	private void clearIntern() {
		getExperimentListAndFactory().getExperimentList().remove();
		getSeSAmModelDeclaration().clear();
	}

	protected ICopyable createInstance(Map objectMapping) {
		return SeSAmDeclarationFactory.createSeSAmDeclaration();
	}

	protected void fireStateChanged() {
		super.fireStateChanged();

		setModelModified(true);
	}

	public final ExperimentListAndFactory getExperimentListAndFactory() {
		return _experimentListAndFactory;
	}

	public File getFile() {
		return _file;
	}

	public final ISeSAmDeclarationTreeModel getSeSAmDeclarationTreeModel() {
		return _seSAmDeclarationTreeModel;
	}

	public final ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return _seSAmModelDeclaration;
	}

	public boolean isModelModified() {
		return _modelModified;
	}

	public void remove() {
		if (getSeSAmDeclarationTreeModel() != null)
			getSeSAmDeclarationTreeModel().setEventCreation(false);
		getSeSAmModelDeclaration().removeChangeListener(_changeListener);

		if (getSeSAmDeclarationTreeModel() != null)
			getSeSAmDeclarationTreeModel().remove();

		getExperimentListAndFactory().getExperimentList().remove();

		getSeSAmModelDeclaration().remove();

		super.remove();
	}

	public void setFile(File newFile) {
		File oldValue = _file;

		if (newFile == null) {
			_file = null;

		} else {
			_file = FileUtilities.getFileWithSuffix(newFile, "xml");
		}

		firePropertyChange("file", oldValue, newFile);

	}

	
	public UMLLayoutGraphDeclarationList getUMLGraphDeclerationList() {
		return _umlList;
	}
	
	public void setModelModified(boolean newModelModified) {
		boolean oldValue = isModelModified();

		_modelModified = newModelModified;

		firePropertyChange("modelModified", oldValue, newModelModified);
		// dont !!
		// fireStateChanged();
	}
	
}