package de.uniwue.ki.mas.model;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.feature.user.UserFeatureClassDeclarationListAndFactory;
import de.uniwue.ki.mas.basics.functionUser.UserFunctionDeclarationListAndFactory;
import de.uniwue.ki.mas.basics.type.user.UserTypeDeclarationListAndFactory;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (28.01.2002 17:02:52)
 * @author Christoph Oechslein
 */
public class SeSAmModelBasicsDeclaration
	extends AbstractDeclaration
	implements ISeSAmModelBasicsDeclaration {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private final UserTypeDeclarationListAndFactory _userTypeDeclarationListAndFactory =
		new UserTypeDeclarationListAndFactory();

	private final UserFeatureClassDeclarationListAndFactory _userFeatureClassDeclarationListAndFactory =
		new UserFeatureClassDeclarationListAndFactory();

	private final UserFunctionDeclarationListAndFactory _userDefinedFunctionDeclarationListAndFactory =
		new UserFunctionDeclarationListAndFactory(true);

	public SeSAmModelBasicsDeclaration() {
		super();

		getUserTypeDeclarationListAndFactory().getUserTypeDeclarationList().addChangeListener(
			_changeListener);

		getUserFunctionDeclarationListAndFactory().getUserFunctionDeclarationList().addChangeListener(
			_changeListener);

		getUserFeatureClassDeclarationListAndFactory()
			.getUserFeatureClassDeclarationList()
			.addChangeListener(
			_changeListener);
	}

	public void clear() {
		getUserFeatureClassDeclarationListAndFactory().getUserFeatureClassDeclarationList().clear();

		getUserFunctionDeclarationListAndFactory().getUserFunctionDeclarationList().clear();

		getUserTypeDeclarationListAndFactory().getDeclarationList().clear();
	}

	/**
	 * @param intoObject Object
	 * @param objectMapping java.util.Map
	 * @see de.uniwue.ki.util.ICopyable#copyInto(Object, Map)
	 */
	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		ISeSAmModelBasicsDeclaration intoSeSAmModelBasicsDeclaration =
			(ISeSAmModelBasicsDeclaration) intoObject;

		getUserFeatureClassDeclarationListAndFactory().copyInto(
			intoSeSAmModelBasicsDeclaration.getUserFeatureClassDeclarationListAndFactory(),
			objectMapping);

		getUserFunctionDeclarationListAndFactory().copyInto(
			intoSeSAmModelBasicsDeclaration.getUserFunctionDeclarationListAndFactory(),
			objectMapping);

		getUserTypeDeclarationListAndFactory().getDeclarationList().copyInto(
			intoSeSAmModelBasicsDeclaration.getUserTypeDeclarationListAndFactory().getDeclarationList(),
			objectMapping);

	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	/**
	 * Returns a short printable representation of this object.
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Model Basics";
	}

	/**
	 * @return UserFeatureClassDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISeSAmModelBasicsDeclaration#getUserFeatureClassDeclarationListAndFactory()
	 */
	public final UserFeatureClassDeclarationListAndFactory getUserFeatureClassDeclarationListAndFactory() {
		return _userFeatureClassDeclarationListAndFactory;
	}

	public final UserFunctionDeclarationListAndFactory getUserFunctionDeclarationListAndFactory() {
		return _userDefinedFunctionDeclarationListAndFactory;
	}

	/**
	 * @return UserTypeDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISeSAmModelBasicsDeclaration#getUserTypeDeclarationListAndFactory()
	 */
	public UserTypeDeclarationListAndFactory getUserTypeDeclarationListAndFactory() {
		return _userTypeDeclarationListAndFactory;
	}

	/**
	 * @param anObject de.uniwue.ki.util.hash.IIsomorphicHashable
	 * @return boolean
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicEquals(IIsomorphicHashable)
	 */
	public boolean isomorphicEquals(de.uniwue.ki.util.hash.IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ISeSAmModelBasicsDeclaration)) {
			return false;
		}

		ISeSAmModelBasicsDeclaration seSAmModelBasicsDeclaration = (ISeSAmModelBasicsDeclaration) anObject;

		return isomorphicEquals(
			getUserFeatureClassDeclarationListAndFactory(),
			seSAmModelBasicsDeclaration.getUserFeatureClassDeclarationListAndFactory())
			&& isomorphicEquals(
				getUserFunctionDeclarationListAndFactory(),
				seSAmModelBasicsDeclaration.getUserFunctionDeclarationListAndFactory())
			&& isomorphicEquals(
				getUserTypeDeclarationListAndFactory(),
				seSAmModelBasicsDeclaration.getUserTypeDeclarationListAndFactory());
	}

	/**
	 * @return int
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicHashCode()
	 */
	public int isomorphicHashCode() {
		return 23 * super.isomorphicHashCode()
			+ 5 * isomorphicHashCode(getUserFunctionDeclarationListAndFactory())
			+ 3 * isomorphicHashCode(getUserFeatureClassDeclarationListAndFactory())
			+ isomorphicHashCode(getUserTypeDeclarationListAndFactory());
	}

	/**
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {
		getUserFeatureClassDeclarationListAndFactory()
			.getUserFeatureClassDeclarationList()
			.removeChangeListener(
			_changeListener);
		getUserFunctionDeclarationListAndFactory().getUserFunctionDeclarationList().removeChangeListener(
			_changeListener);
		getUserTypeDeclarationListAndFactory().getUserTypeDeclarationList().removeChangeListener(
			_changeListener);

		getUserFeatureClassDeclarationListAndFactory().remove();
		getUserFunctionDeclarationListAndFactory().remove();
		getUserTypeDeclarationListAndFactory().remove();

		super.remove();
	}
}