package de.uniwue.ki.mas.model;
import java.util.*;

import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.FeatureClassDeclarationListManager;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationListAble;
import de.uniwue.ki.mas.basics.type.TypeManager;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclarationList;
import de.uniwue.ki.mas.util.declaration.TwoUnmodifiableDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;

/**
 * Creation date: (12.10.2001 10:48:30)
 * @author Christoph Oechslein
 */

class SeSAmModelDeclaration extends AbstractSeSAmModelDeclaration {
	private final AvailibleFunctionHelper _availibleFunctionHelper = new AvailibleFunctionHelper(this);

	private final Map _objectClassDeclFeatureClassDeclarationList = new HashMap();

	private IFeatureClassDeclarationList _availibleFeatureClassDeclarationList;
	private IObjectClassDeclarationList _availibleObjectClassDeclarationList;

	private static class TwoUnmodifiableObjectClassDeclarationList
		extends TwoUnmodifiableDeclarationList
		implements IObjectClassDeclarationList {

		TwoUnmodifiableObjectClassDeclarationList(
			IObjectClassDeclarationList ressources,
			IObjectClassDeclarationList agents) {
			super(ressources, agents, false);
		}
	}

	public SeSAmModelDeclaration() {
		super();
	}

	protected ICopyable createInstance(Map objectMapping) {
		return SeSAmDeclarationFactory.createSeSAmModelDeclaration();
	}

	public IFeatureClassDeclarationList getAvailibleFeatureClassDeclarationList() {
		if (_availibleFeatureClassDeclarationList == null) {
			_availibleFeatureClassDeclarationList =
				FeatureClassDeclarationListManager.createFeatureClassDeclarationList(
					FeatureClassDeclarationListManager.getInstance().getBuildinFeatureClassDeclarationList(),
					getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory()
						.getUserFeatureClassDeclarationList());
		}

		return _availibleFeatureClassDeclarationList;
	}

	public IFeatureClassDeclarationList getAvailibleFeatureClassDeclarationListFor(IObjectClassDeclaration objectClassDeclaration) {
		if (_objectClassDeclFeatureClassDeclarationList.containsKey(objectClassDeclaration)) {
			return (IFeatureClassDeclarationList) _objectClassDeclFeatureClassDeclarationList.get(
				objectClassDeclaration);

		} else {
			IFeatureClassDeclarationList featureClassDeclarationList =
				FeatureClassDeclarationListManager.createFeatureClassDeclarationList(
					FeatureClassDeclarationListManager
						.getInstance()
						.getBuildinFeatureClassDeclarationListFor(
						objectClassDeclaration),
					getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory()
						.getUserFeatureClassDeclarationList());

			_objectClassDeclFeatureClassDeclarationList.put(
				objectClassDeclaration,
				featureClassDeclarationList);

			return featureClassDeclarationList;
		}
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationList() {
		return _availibleFunctionHelper.getAvailibleFunctionDeclarationList();
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IObjectClassDeclaration objectClassDeclaration) {
		return _availibleFunctionHelper.getAvailibleFunctionDeclarationListFor(objectClassDeclaration);
	}

	public IFunctionDeclarationListAble getAvailibleFunctionDeclarationListFor(IUserFeatureClassDeclaration userFeatureClassDeclaration) {
		return _availibleFunctionHelper.getAvailibleFunctionDeclarationListFor(userFeatureClassDeclaration);
	}

	public IObjectClassDeclarationList getAvailibleObjectClassDeclarationList() {
		if (_availibleObjectClassDeclarationList == null) {
			_availibleObjectClassDeclarationList =
				new TwoUnmodifiableObjectClassDeclarationList(
					getRessourceClassDeclarationListAndFactory().getRessourceClassDeclarationList(),
					getAgentClassDeclarationListAndFactory().getAgentClassDeclarationList());
		}

		return _availibleObjectClassDeclarationList;

	}

	public List getAvailibleTypeList() {
		return CollectionsHelper.asLinkedList(TypeManager.getInstance().getAllTypes());
	}

	public List getAvailibleTypeListForVariableClassDeclaration() {
		return CollectionsHelper.asLinkedList(
			TypeManager.getInstance().getAllTypesForVariableClassDeclaration());
	}

	public Iterator getAllAvailibleFunctionDeclarations() {
		return _availibleFunctionHelper.getAllAvailibleFunctionDeclarations();
	}

	public void remove() {
		removeObjectClassDeclFeatureClassDeclarationList();

		if (_availibleFeatureClassDeclarationList != null) {
			_availibleFeatureClassDeclarationList.remove();
			_availibleFeatureClassDeclarationList = null;
		}

		if (_availibleObjectClassDeclarationList != null) {
			_availibleObjectClassDeclarationList.remove();
			_availibleObjectClassDeclarationList = null;
		}

		_availibleFunctionHelper.remove();

		super.remove();
	}

	private void removeObjectClassDeclFeatureClassDeclarationList() {
		Iterator it = _objectClassDeclFeatureClassDeclarationList.values().iterator();
		while (it.hasNext()) {
			IFeatureClassDeclarationList element = (IFeatureClassDeclarationList) it.next();
			element.remove();
			it.remove();
		}
	}

	public void clear() {
		removeObjectClassDeclFeatureClassDeclarationList();

		super.clear();
	}

}