package de.uniwue.ki.mas.model;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclarationListAndFactory;
import de.uniwue.ki.mas.model.userInterface.UserInterfaceDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.analysis.AnalysisDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.experiment.ExperimentDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.simulation.SimulationDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.situation.SituationDeclarationListAndFactory;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (28.01.2002 17:24:01)
 * @author Christoph Oechslein
 */
final class SimulationElementsDeclaration
	extends AbstractDeclaration
	implements ISimulationElementsDeclaration {
	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private final UserInterfaceDeclarationListAndFactory _userInterfaceDeclarationListAndFactory;
	private final SimulationDeclarationListAndFactory _simulationDeclarationListAndFactory;
	private final SituationDeclarationListAndFactory _situationDeclarationListAndFactory;
	private final AnalysisDeclarationListAndFactory _analysisDeclarationListAndFactory =
		new AnalysisDeclarationListAndFactory();
	private final ExperimentDeclarationListAndFactory _experimentDeclarationListAndFactory;

	public SimulationElementsDeclaration(ISeSAmModelDeclaration seSAmModelDeclaration) {
		super();

		IWorldClassDeclarationListAndFactory worldClassDeclarationListAndFactory =
			seSAmModelDeclaration.getWorldClassDeclarationListAndFactory();

		Assert.assertNotNull(worldClassDeclarationListAndFactory);

		_situationDeclarationListAndFactory =
			new SituationDeclarationListAndFactory(worldClassDeclarationListAndFactory);
		_simulationDeclarationListAndFactory =
			new SimulationDeclarationListAndFactory(_situationDeclarationListAndFactory,
			getAnalysisDeclarationListAndFactory().getAnalysisDeclarationList());
		_userInterfaceDeclarationListAndFactory =
			new UserInterfaceDeclarationListAndFactory(_simulationDeclarationListAndFactory);
		_experimentDeclarationListAndFactory =
			new ExperimentDeclarationListAndFactory(_simulationDeclarationListAndFactory);

		getSimulationDeclarationListAndFactory().getSimulationDeclarationList().addChangeListener(
			_changeListener);
		getUserInterfaceDeclarationListAndFactory().getUserInterfaceDeclarationList().addChangeListener(
			_changeListener);
		getSituationDeclarationListAndFactory().getSituationDeclarationList().addChangeListener(
			_changeListener);
		getExperimentDeclarationListAndFactory().getExperimentDeclarationList().addChangeListener(
			_changeListener);
	}

	public void clear() {
		getExperimentDeclarationListAndFactory().getExperimentDeclarationList().clear();
		getUserInterfaceDeclarationListAndFactory().getUserInterfaceDeclarationList().clear();
		getSimulationDeclarationListAndFactory().getSimulationDeclarationList().clear();
		getSituationDeclarationListAndFactory().getSituationDeclarationList().clear();
		getAnalysisDeclarationListAndFactory().getDeclarationList().clear();
	}

	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		ISimulationElementsDeclaration intoSimulationElementsDeclaration =
			(ISimulationElementsDeclaration) intoObject;

		getExperimentDeclarationListAndFactory().copyInto(
			intoSimulationElementsDeclaration.getExperimentDeclarationListAndFactory(),
			objectMapping);

		getUserInterfaceDeclarationListAndFactory().copyInto(
			intoSimulationElementsDeclaration.getUserInterfaceDeclarationListAndFactory(),
			objectMapping);
			
		getSimulationDeclarationListAndFactory().copyInto(
			intoSimulationElementsDeclaration.getSimulationDeclarationListAndFactory(),
			objectMapping);

		getAnalysisDeclarationListAndFactory().getDeclarationList().copyInto(
			intoSimulationElementsDeclaration.getAnalysisDeclarationListAndFactory().getDeclarationList(),
			objectMapping);

		getSituationDeclarationListAndFactory().copyInto(
			intoSimulationElementsDeclaration.getSituationDeclarationListAndFactory(),
			objectMapping);
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	/**
	 * @return AnalysisDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISimulationElementsDeclaration#getAnalysisDeclarationListAndFactory()
	 */
	public final AnalysisDeclarationListAndFactory getAnalysisDeclarationListAndFactory() {
		return _analysisDeclarationListAndFactory;
	}

	/**
	 * @return ExperimentDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISimulationElementsDeclaration#getExperimentDeclarationListAndFactory()
	 */
	public final ExperimentDeclarationListAndFactory getExperimentDeclarationListAndFactory() {
		return _experimentDeclarationListAndFactory;
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Simulation Elements";
	}

	/**
	 * @return SimulationDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISimulationElementsDeclaration#getSimulationDeclarationListAndFactory()
	 */
	public final SimulationDeclarationListAndFactory getSimulationDeclarationListAndFactory() {
		return _simulationDeclarationListAndFactory;
	}

	/**
	 * @return UserInterfaceDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISimulationElementsDeclaration#getUserInterfaceDeclarationListAndFactory()
	 */
	public final UserInterfaceDeclarationListAndFactory getUserInterfaceDeclarationListAndFactory() {
		return _userInterfaceDeclarationListAndFactory;
	}

	/**
	 * @return SituationDeclarationListAndFactory
	 * @see de.uniwue.ki.mas.model.ISimulationElementsDeclaration#getSituationDeclarationListAndFactory()
	 */
	public final SituationDeclarationListAndFactory getSituationDeclarationListAndFactory() {
		return _situationDeclarationListAndFactory;
	}

	/**
	 * @param anObject IIsomorphicHashable
	 * @return boolean
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicEquals(IIsomorphicHashable)
	 */
	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ISimulationElementsDeclaration)) {
			return false;
		}

		ISimulationElementsDeclaration simulationElementsDeclaration =
			(ISimulationElementsDeclaration) anObject;

		return isomorphicEquals(
			getExperimentDeclarationListAndFactory(),
			simulationElementsDeclaration.getExperimentDeclarationListAndFactory())
			&& isomorphicEquals(
				getSimulationDeclarationListAndFactory(),
				simulationElementsDeclaration.getSimulationDeclarationListAndFactory())
			&& isomorphicEquals(
				getUserInterfaceDeclarationListAndFactory(),
				simulationElementsDeclaration.getUserInterfaceDeclarationListAndFactory())
			&& isomorphicEquals(
				getAnalysisDeclarationListAndFactory(),
				simulationElementsDeclaration.getAnalysisDeclarationListAndFactory())
			&& isomorphicEquals(
				getSituationDeclarationListAndFactory(),
				simulationElementsDeclaration.getSituationDeclarationListAndFactory());
	}

	/**
	 * @return int
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicHashCode()
	 */
	public int isomorphicHashCode() {
		return 13 * super.isomorphicHashCode()
			+ 9 * isomorphicHashCode(getExperimentDeclarationListAndFactory())
			+ 7 * isomorphicHashCode(getSimulationDeclarationListAndFactory())
			+ 5 * isomorphicHashCode(getUserInterfaceDeclarationListAndFactory())
			+ 3 * isomorphicHashCode(getAnalysisDeclarationListAndFactory())
			+ isomorphicHashCode(getSituationDeclarationListAndFactory());
	}

	/**
	 * @see de.uniwue.ki.util.IRemoveable#remove()
	 */
	public void remove() {

		getSimulationDeclarationListAndFactory().getSimulationDeclarationList().removeChangeListener(
			_changeListener);
		getUserInterfaceDeclarationListAndFactory().getUserInterfaceDeclarationList().removeChangeListener(
			_changeListener);
		getSituationDeclarationListAndFactory().getSituationDeclarationList().removeChangeListener(
			_changeListener);
		getExperimentDeclarationListAndFactory().getExperimentDeclarationList().removeChangeListener(
			_changeListener);

		getExperimentDeclarationListAndFactory().remove();
		getSimulationDeclarationListAndFactory().remove();
		getUserInterfaceDeclarationListAndFactory().remove();
		getAnalysisDeclarationListAndFactory().remove();
		getSituationDeclarationListAndFactory().remove();

		super.remove();
	}
}