package de.uniwue.ki.mas.model;
import junit.framework.*;
import de.uniwue.ki.mas.util.declaration.*;

/**
 * Creation date: (05.09.2001 10:23:24)
 * @author Christoph Oechslein
 */

public final class TestSeSAmDeclaration extends TestCase {
	private ISeSAmDeclaration _seSAmDeclaration;

	/**
	 * @param name String
	 */
	public TestSeSAmDeclaration(String name) {
		super(name);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestSeSAmDeclaration.class);
	}

	protected void setUp() throws Exception {
		_seSAmDeclaration = SeSAmDeclarationFactory.createSeSAmDeclaration();
	}

	public static Test suite() {
		TestSuite anSuite = new TestSuite("SeSAmDeclaration");
		anSuite.addTest(new TestSeSAmDeclaration("testSeSAmModelDeclaration"));
		return anSuite;
	}

	protected void tearDown() throws Exception {
		_seSAmDeclaration.remove();
	}

	private void testDeclList(IDeclarationList declarationList, IDeclarationFactory declarationFactory)
		throws Exception {

		assertTrue(declarationList.isComposition());

		IDeclaration newDeclaration = declarationFactory.createDeclaration();

		declarationList.add(newDeclaration);
		assertEquals(1, declarationList.size());
		assertTrue(declarationList.contains(newDeclaration));

		declarationList.remove(newDeclaration);
		assertTrue(!declarationList.contains(newDeclaration));
		assertEquals(0, declarationList.size());
	}

	private void testDeclList(IDeclarationListAndFactory declarationListAndFactory) throws Exception {
		testDeclList(
			declarationListAndFactory.getDeclarationList(),
			declarationListAndFactory.getDeclarationFactory());
	}

	/**
	 * @throws Exception
	 */
	public void testSeSAmModelDeclaration() throws Exception {
		ISeSAmModelDeclaration seSAmModelDeclaration = _seSAmDeclaration.getSeSAmModelDeclaration();

		testDeclList(seSAmModelDeclaration.getAgentClassDeclarationListAndFactory());
		testDeclList(seSAmModelDeclaration.getRessourceClassDeclarationListAndFactory());
		testDeclList(seSAmModelDeclaration.getWorldClassDeclarationListAndFactory());
		testDeclList(
			seSAmModelDeclaration
				.getSeSAmModelBasicsDeclaration()
				.getUserFeatureClassDeclarationListAndFactory());
		testDeclList(
			seSAmModelDeclaration
				.getSeSAmModelBasicsDeclaration()
				.getUserFunctionDeclarationListAndFactory());
		testDeclList(
			seSAmModelDeclaration.getSeSAmModelBasicsDeclaration().getUserTypeDeclarationListAndFactory());
		testDeclList(
			seSAmModelDeclaration.getSimulationElementsDeclaration().getSituationDeclarationListAndFactory());
		testDeclList(
			seSAmModelDeclaration
				.getSimulationElementsDeclaration()
				.getSimulationDeclarationListAndFactory());
	}
}