/**
 * Created on 11.02.2004
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.mas.model.graph;

import java.util.Hashtable;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

public class UMLLayoutGraphEdgeFactory {

    private static UMLLayoutGraphEdgeFactory _instance = new UMLLayoutGraphEdgeFactory();
    private Hashtable _nodeMapper;

    protected UMLLayoutGraphEdgeFactory() {

    }

    public static UMLLayoutGraphEdgeFactory getInstance() {

        return _instance;
    }

    public void setNodeMapper(Hashtable mapper) {
        _nodeMapper = mapper;
    }

    private IUMLLayoutGraphNode getMappedNode(IDocumentableDeclaration decl) { // IObjectClassDeclaration

        return (IUMLLayoutGraphNode) _nodeMapper.get(decl);
    }

    public UMLLayoutGraphEdge createEdge(
        IUMLLayoutGraphNode node,
        Variable var,
        IObjectClassDeclaration toDecl) {

        IUMLLayoutGraphNode toNode = getMappedNode(toDecl);

        return createEdge(node, toNode, var.getName());
    }

    public UMLLayoutGraphEdge createEdge(IUMLLayoutGraphNode node, IFeatureClassDeclaration featDecl) {
        IUMLLayoutGraphNode toNode = getMappedNode(featDecl);

        return createEdge(node, toNode, toNode.getName());
    }

    public UMLLayoutGraphEdge createEdge(IUMLLayoutGraphNode node, IUMLLayoutGraphNode toNode, String name) {

        Assert.assertNotNull("Node not contained", node);
        Assert.assertNotNull("Node not contained", toNode);

        if (node.getDeclaration() instanceof IFeatureClassDeclaration)
            return new UMLLayoutGraphFeatureEdge(node, toNode, toNode.getName());
        if (node == toNode)
            return new UMLLayoutGraphReflectiveEdge(node, name);
        else
            return new UMLLayoutGraphEdge(node, toNode, name);

    }

}
