/**
 * Created on 01.07.2003
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.mas.model.graph;

import java.util.Hashtable;
import java.util.Iterator;

import mygale.mcge.net.INetGraphModel;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.type.ISimObjectType;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;

//TODO wr's nicht schner wenn die GraphNodes das selber machen wrden?

public class UMLLayoutGraphParser {

    private IObjectClassDeclarationList _declList = null;
    private IFeatureClassDeclarationList _featList = null;

    private INetGraphModel _graphModel = null;

    public UMLLayoutGraphParser(
        IObjectClassDeclarationList declList,
        IFeatureClassDeclarationList featList) {

        _declList = declList;
        _featList = featList;

        generateModel();
    }

    private void extractEdges() {

        Iterator nodeIterator = getGraphModel().getNodes();
        while (nodeIterator.hasNext()) {
            IUMLLayoutGraphNode node = (IUMLLayoutGraphNode) nodeIterator.next();

            if (node instanceof UMLLayoutGraphObjectNode) {
                UMLLayoutGraphObjectNode fromNode = (UMLLayoutGraphObjectNode) node;
                Iterator varIter = fromNode.variableIterator();

                while (varIter.hasNext()) {
                    Variable var = (Variable) varIter.next();
                    IObjectClassDeclaration toDecl = null;

                    // TODO maybe pull this into an Object.... --steffen
                    if (var instanceof SimObjectTypeVariable) {
                        SimObjectTypeVariable simVar = (SimObjectTypeVariable) var;

                        toDecl = simVar.getObjectDeclaration();

                        if (toDecl != null) {
                            createEdge(fromNode, var, toDecl);
                        } else {
//TODO do something?
                        }
                    } else if (var instanceof ListVariable) {
                        ListVariable listVar = (ListVariable) var;

                        if (listVar.getType() instanceof ISimObjectType) {
                            Iterator subIter = listVar.getSubtypeIterator();
                            while (subIter.hasNext()) {
                                ISimObjectType element = (ISimObjectType) subIter.next();
                                toDecl = element.getObjectClassDeclaration();

                                if (toDecl != null)
                                    createEdge(fromNode, var, toDecl);
                            }
                        }
                    }
                }

                Iterator featIt =
                    ((IObjectClassDeclaration) fromNode.getDeclaration())
                        .getObjectFeatureClassDeclarationList()
                        .iterator();
                while (featIt.hasNext()) {
                    IObjectFeatureClassDeclaration oDecl = (IObjectFeatureClassDeclaration) featIt.next();
                    IFeatureClassDeclaration fDecl = oDecl.getFeatureClassDeclaration();
                    
                    if (fDecl instanceof IUserFeatureClassDeclaration) {
                        IUserFeatureClassDeclaration featDecl = (IUserFeatureClassDeclaration) fDecl;
                        createEdge(node, featDecl);
                    }

                }
            }
            //            else if (node instanceof UMLLayoutGraphFeatureNode) {
            //                UMLLayoutGraphFeatureNode uN = (UMLLayoutGraphFeatureNode) node;
            //                IFeatureClassDeclaration featDecl = (IFeatureClassDeclaration) uN.getDeclaration();
            //
            //                // TODO stelle Kanten zwischen Features und ObjectClasses her.
            //            }
        }
    }

    private void createEdge(IUMLLayoutGraphNode node, IUserFeatureClassDeclaration featDecl) {

        UMLLayoutGraphEdge edge = getEdgeFactory().createEdge(node, featDecl);

        getGraphModel().addEdge(edge);
    }

    private void createEdge(IUMLLayoutGraphNode node, Variable var, IObjectClassDeclaration toDecl) {

        UMLLayoutGraphEdge edge = getEdgeFactory().createEdge(node, var, toDecl);

        getGraphModel().addEdge(edge);
    }

    private UMLLayoutGraphEdgeFactory _edgeFact;

    private UMLLayoutGraphEdgeFactory getEdgeFactory() {
        if (_edgeFact == null) {

            _edgeFact = UMLLayoutGraphEdgeFactory.getInstance();
            _edgeFact.setNodeMapper(getDeclToNodeMapper());
        }

        return _edgeFact;
    }

    private IUMLLayoutGraphNode getMappedNode(IDocumentableDeclaration decl) { // IObjectClassDeclaration

        return (IUMLLayoutGraphNode) getDeclToNodeMapper().get(decl);
    }

    private void extractNodes() {

        UMLLayoutGraphNodeFactory nodeFac = UMLLayoutGraphNodeFactory.getInstance();
        nodeFac.setGraphModel(getGraphModel());

        Iterator declListIterator = getObjectDeclList().iterator();
        while (declListIterator.hasNext()) {

            IObjectClassDeclaration objDecl = (IObjectClassDeclaration) declListIterator.next();
            try {
                IUMLLayoutGraphNode node = nodeFac.createNode(objDecl);

                map(objDecl, node);
                getGraphModel().addNode(node);
            } catch (CannotCreateNodeException e) {
            }
        }

        Iterator featListIterator = getFeatureList().iterator();
        while (featListIterator.hasNext()) {

            IFeatureClassDeclaration featDecl = (IFeatureClassDeclaration) featListIterator.next();

            try {
                IUMLLayoutGraphNode node = nodeFac.createNode(featDecl);

                map(featDecl, node);
                getGraphModel().addNode(node);
            } catch (CannotCreateNodeException e) {
                System.out.println(e.getMessage());
            }

        }
    }

    private IFeatureClassDeclarationList getFeatureList() {
        return _featList;
    }

    private void generateModel() {

        extractNodes();
        extractEdges();
    }

    private Hashtable _declToNodeMapper = null;

    private Hashtable getDeclToNodeMapper() {

        if (_declToNodeMapper == null)
            _declToNodeMapper = new Hashtable();

        return _declToNodeMapper;
    }

    public UMLLayoutGraphDeclaration getGraphModel() {

        if (_graphModel == null)
            _graphModel = new UMLLayoutGraphDeclaration();

        return (UMLLayoutGraphDeclaration) _graphModel;
    }

    private IObjectClassDeclarationList getObjectDeclList() {

        return _declList;
    }

    //IObjectClassDeclaration -> Object
    private void map(Object decl, IUMLLayoutGraphNode node) {

        if (getDeclToNodeMapper().get(decl) == null)
            getDeclToNodeMapper().put(decl, node);
    }

}
