package de.uniwue.ki.mas.model.tree;

import java.util.Iterator;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRuleDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.tree.DefaultObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNode;

/**
 * @author herrler
 *
 */
public class ActivityRuleDeclarationTreeNode
	extends DefaultObjectTreeNode
	implements IUpdateChildrenAbleNode {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			ActivityRuleDeclarationTreeNode.this.stateChanged(e);
		}
	};

	public ActivityRuleDeclarationTreeNode(IObjectTreeNode parent, IActivityGraphRuleDeclaration declaration) {
		super(parent, declaration);

		initialize(declaration);
	}

	public ActivityRuleDeclarationTreeNode(
		IObjectTreeNode parent,
		IActivityGraphRuleDeclaration declaration,
		Iterator childrens) {
		super(parent, declaration, childrens);

		initialize(declaration);
	}

	public ActivityRuleDeclarationTreeNode(
		IObjectTreeNode parent,
		IActivityGraphRuleDeclaration declaration,
		java.util.List childrenList) {
		super(parent, declaration, childrenList);

		initialize(declaration);
	}

	private void initialize(IActivityGraphRuleDeclaration declaration) {
		updateChildrenIntern();

		declaration.addChangeListener(_changeListener);

		//updateChildrenDeffered();
	}

	private void updateChildrenDeffered() {
		UpdateChildrenAbleNodeThread.getInstance().addNode(this);
	}

	public void updateChildren() {
		updateChildrenIntern();

		SeSAmDeclarationTreeModelAdapter seSAmDeclarationTreeModelAdapter =
			SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(this);

		if (seSAmDeclarationTreeModelAdapter != null)
			seSAmDeclarationTreeModelAdapter.reloadNode(this, true);
	}

	private void updateChildrenIntern() {
		IActivityGraphRuleDeclaration declaration = (IActivityGraphRuleDeclaration) getDeclaration();
		removeChildren();

		addChild(
			new FunctionCallInternalTreeNode(
				ActivityRuleDeclarationTreeNode.this,
				declaration.getConditionFunctionCallDeclaration().getFunctionCallInternDeclaration()));

	}

	public IDeclaration getDeclaration() {
		return (IDeclaration) getUserObject();
	}

	public void remove() {
		getDeclaration().removeChangeListener(_changeListener);
		((IActivityGraphRuleDeclaration) getDeclaration()).removeChangeListener(_changeListener);

		super.remove();
	}

	protected void stateChanged(ChangeEvent e) {
		updateChildrenDeffered();
	}
}
