package de.uniwue.ki.mas.model.tree;
import javax.swing.event.*;

import de.uniwue.ki.mas.sim.analysis.IAnalysisDeclaration;
import de.uniwue.ki.mas.sim.analysis.IAnalysisItemDeclaration;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.tree.*;

final class AnalysisDeclarationTreeNode extends AbstractMutableListTreeNode {

	private static final AnalysisItemDeclarationTreeNodeFactory _analysisItemDeclarationTreeNodeFactory =
		new AnalysisItemDeclarationTreeNodeFactory();

	private static final class AnalysisItemDeclarationTreeNodeFactory implements IObjectTreeNodeFactory {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent, Object userObject) {
			return new AnalysisItemDeclarationTreeNode(parent, (IAnalysisItemDeclaration) userObject);
		}
	}

	private static final class AnalysisItemDeclarationTreeNode extends DefaultDeclarationTreeNode {
		public AnalysisItemDeclarationTreeNode(
			IObjectTreeNode parent,
			IAnalysisItemDeclaration analysisItemDeclaration) {

			super(parent, analysisItemDeclaration);

			addChild(
				FunctionCallDeclarationTreeNodeFactory.getInstance().createObjectTreeNode(
					this,
					analysisItemDeclaration.getValueFunctionCallDeclaration()));
		}
	}

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			SeSAmDeclarationTreeModelCreater
				.getSeSAmDeclarationTreeModelAdapter(AnalysisDeclarationTreeNode.this)
				.reloadNode(AnalysisDeclarationTreeNode.this, false);
		}
	};

	private final IAnalysisDeclaration _analysisDeclaration;

	public AnalysisDeclarationTreeNode(IObjectTreeNode parent, IAnalysisDeclaration analysisDeclaration) {
		super(parent, analysisDeclaration, _analysisItemDeclarationTreeNodeFactory);

		_analysisDeclaration = analysisDeclaration;

		_analysisDeclaration.addChangeListener(_changeListener);
		addListener(getMutableListModel());
	}

	public void remove() {
		_analysisDeclaration.removeChangeListener(_changeListener);

		super.remove();
	}

	protected void contentsChanged(ListDataEvent e) {
		if (isChildrenNodesCreated()) {
			super.contentsChanged(e);

			SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(this).reloadNode(this);
		}
	}

	protected void intervalAdded(ListDataEvent e) {
		if (isChildrenNodesCreated()) {
			super.intervalAdded(e);

			int index0 = e.getIndex0();
			int index1 = e.getIndex1();

			SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(this).nodesWereInserted(
				this,
				index0,
				index1);
		}
	}

	protected void intervalRemoved(ListDataEvent e) {
		if (isChildrenNodesCreated()) {
			int index0 = e.getIndex0();
			int index1 = e.getIndex1();

			IObjectTreeNode[] childrenArray = new IObjectTreeNode[index1 - index0 + 1];

			for (int i = 0; i <= index1 - index0; i++) {
				childrenArray[i] = (IObjectTreeNode) getChildrenNodes().remove(index0);
			}

			SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(this).nodesWereRemoved(
				this,
				index0,
				index1,
				childrenArray);

			for (int i = 0; i < childrenArray.length; i++) {
				childrenArray[i].remove();
			}
		}
	}

	public IMutableListModel getMutableListModel() {
		return _analysisDeclaration
			.getAnalysisItemDeclarationListAndFactory()
			.getAnalysisItemDeclarationList();
	}
}