/**
 * Rainer Herrler
 * 
 * Created on 05.06.2004
 */
package de.uniwue.ki.mas.model.tree;

import java.util.Iterator;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.util.tree.DefaultObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNodeFactory;


final class FunctionCallDeclarationTreeNodeFactory implements IObjectTreeNodeFactory {
	public class FunctionCallDeclarationTreeNode
	extends DefaultObjectTreeNode
	implements IUpdateChildrenAbleNode {

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			FunctionCallDeclarationTreeNode.this.stateChanged(e);
		}
	};

	public FunctionCallDeclarationTreeNode(IObjectTreeNode parent, IFunctionCallDeclaration declaration) {
		super(parent, declaration);

		initialize(declaration);
	}

	public FunctionCallDeclarationTreeNode(
		IObjectTreeNode parent,
		IFunctionCallDeclaration declaration,
		Iterator childrens) {
		super(parent, declaration, childrens);

		initialize(declaration);
	}

	public FunctionCallDeclarationTreeNode(
		IObjectTreeNode parent,
		IFunctionCallDeclaration declaration,
		java.util.List childrenList) {
		super(parent, declaration, childrenList);

		initialize(declaration);
	}

	private void initialize(IFunctionCallDeclaration declaration) {
		updateChildrenIntern();

		declaration.addChangeListener(_changeListener);

		//updateChildrenDeffered();
	}

	private void updateChildrenDeffered() {
		UpdateChildrenAbleNodeThread.getInstance().addNode(this);
	}

	public void updateChildren() {
		updateChildrenIntern();

		SeSAmDeclarationTreeModelAdapter seSAmDeclarationTreeModelAdapter =
			SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(this);

		if (seSAmDeclarationTreeModelAdapter != null)
			seSAmDeclarationTreeModelAdapter.reloadNode(this, true);
	}

	private void updateChildrenIntern() {

		IFunctionCallDeclaration declaration = (IFunctionCallDeclaration) getDeclaration();
		removeChildren();

		IFunctionCallInternDeclaration functionCallInternDeclaration =
			declaration.getFunctionCallInternDeclaration();
		if (functionCallInternDeclaration != null)
			addChild(
				new FunctionCallInternalTreeNode(
					FunctionCallDeclarationTreeNode.this,
					functionCallInternDeclaration));
	}

	public IDeclaration getDeclaration() {
		return (IDeclaration) getUserObject();
	}

	public void remove() {
		getDeclaration().removeChangeListener(_changeListener);
		((IFunctionCallDeclaration) getDeclaration()).removeChangeListener(_changeListener);

		super.remove();
	}

	protected void stateChanged(ChangeEvent e) {
		updateChildrenDeffered();
	}
}

	
	public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
			Object userObject) {
		return new FunctionCallDeclarationTreeNode(parent,
				(IFunctionCallDeclaration) userObject);
	}
	
	
	private static final FunctionCallDeclarationTreeNodeFactory instance = new FunctionCallDeclarationTreeNodeFactory();
	
	public static FunctionCallDeclarationTreeNodeFactory getInstance() {
		return instance;
	}
}