package de.uniwue.ki.mas.model.tree;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.SpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.type.GenericFunctionType;
import de.uniwue.ki.util.tree.DefaultObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNode;

/**
 * @author herrler
 */
public class FunctionCallInternalTreeNode extends DefaultObjectTreeNode {
	private boolean _isLeaf = true;
	private final IFunctionCallInternDeclaration _functionCall;

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			SeSAmDeclarationTreeModelAdapter seSAmDeclarationTreeModelAdapter =
				SeSAmDeclarationTreeModelCreater.getSeSAmDeclarationTreeModelAdapter(FunctionCallInternalTreeNode.this);
				
			createChildNodes(FunctionCallInternalTreeNode.this._functionCall);

			if (seSAmDeclarationTreeModelAdapter != null)
				seSAmDeclarationTreeModelAdapter.reloadNode(FunctionCallInternalTreeNode.this, false);
		}
	};

	public FunctionCallInternalTreeNode(
		IObjectTreeNode parent,
		IFunctionCallInternDeclaration functionCall) {
		super(parent, functionCall);

		_functionCall = functionCall;
		createChildNodes(functionCall);
	}

	void createChildNodes(IFunctionCallInternDeclaration functionCall) {
		removeChildren();
		if (functionCall instanceof IComposedFunctionCallInternDeclaration) {
			IComposedFunctionCallInternDeclaration composedCall =
				(IComposedFunctionCallInternDeclaration)functionCall;
			for (int i = 0; i < composedCall.getChildCount(); i++) {
				addChild(new FunctionCallInternalTreeNode(this, composedCall.getChild(i)));
				_isLeaf = false;
			}
			functionCall.addChangeListener(_changeListener);
		
		} else if (functionCall instanceof IAtomicFunctionCallInternDeclaration) {
			IAtomicFunctionCallInternDeclaration atomicCall =
				(IAtomicFunctionCallInternDeclaration)functionCall;
			if ((atomicCall.getArgumentDeclaration() != null)
				&& (GenericFunctionType.getInstance().canCreate(atomicCall.getSpecificOutputType()))
				&& !(atomicCall.getArgumentDeclaration() instanceof IFunctionArgumentDeclaration)
				&& (atomicCall.getArgumentDeclaration().getObject()
					instanceof SpecificFunctionDeclarationWithArgDeclBinds)) {
				addChild(
					new FunctionCallInternalTreeNode(
						this,
						((SpecificFunctionDeclarationWithArgDeclBinds)atomicCall
							.getArgumentDeclaration()
							.getObject())
							.getParametricFunctionCallDeclaration()
							.getFunctionCallDeclaration()
							.getFunctionCallInternDeclaration()));
				_isLeaf = false;
			}
		
		} else {
			Assert.assertNull(functionCall);
		}
	}

	public boolean isLeaf() {
		return _isLeaf;
	}

}
