package de.uniwue.ki.mas.model.tree;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import junit.framework.Assert;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.tree.*;

/**
 * Creation date: (05.10.2001 09:32:57)
 * @author Christoph Oechslein
 */
final class SeSAmDeclarationTreeModelAdapter
	extends AbstractTreeModel
	implements ISeSAmDeclarationTreeModel {

	private IObjectTreeNode _rootTreeNode;
	private ISeSAmDeclaration _seSAmDeclaration;
	private boolean _eventCreation = true;

	public SeSAmDeclarationTreeModelAdapter() {
		super();
	}

	public void remove() {
		setEventCreation(false);
		_rootTreeNode = new DefaultObjectTreeNodeLeaf(null, "Closing");
	}

	/**
	 * Returns the child of <I>parent</I> at index <I>index</I> in the parent's
	 * child array.  <I>parent</I> must be a node previously obtained from
	 * this data source. This should not return null if <i>index</i>
	 * is a valid index for <i>parent</i> (that is <i>index</i> >= 0 &&
	 * <i>index</i> < getChildCount(<i>parent</i>)).
	 *
	 * @param   parent  a node in the tree, obtained from this data source
	 * @return  the child of <I>parent</I> at index <I>index</I>
	 */
	public Object getChild(Object parent, int index) {
		return ((TreeNode) parent).getChildAt(index);
	}

	/**
	 * Returns the number of children of <I>parent</I>.  Returns 0 if the node
	 * is a leaf or if it has no children.  <I>parent</I> must be a node
	 * previously obtained from this data source.
	 *
	 * @param   parent  a node in the tree, obtained from this data source
	 * @return  the number of children of the node <I>parent</I>
	 */
	public int getChildCount(Object parent) {
		return ((TreeNode) parent).getChildCount();
	}

	public int getIndexOfChild(Object parent, Object child) {
		return ((TreeNode) parent).getIndex((TreeNode) child);
	}

	public Object getParentOf(Object aNode) {
		if (aNode == null)
			return null;
		else
			return ((TreeNode) aNode).getParent();
	}

	public Object getRoot() {

		if (_rootTreeNode == null) {
			_rootTreeNode = new SeSAmDeclarationTreeModelCreater(this).createSeSAmDeclarationTreeNode();
		}

		return _rootTreeNode;
	}

	public ISeSAmDeclaration getSeSAmDeclaration() {
		return _seSAmDeclaration;
	}

	public Object getUserObjectParentOf(Object aNode) {
		IObjectTreeNode parent;

		if (aNode instanceof IObjectTreeNode) {
			parent = (IObjectTreeNode) getParentOf(aNode);
		} else {
			parent = (IObjectTreeNode) getParentOf(searchObjectTreeNode(aNode));
		}

		//		Assert.assertNotNull("Could not find parent of " + aNode, parent);

		if (parent == null)
			return null;
		else
			return parent.getUserObject();
	}

	/**
	 * Returns true if <I>node</I> is a leaf.  It is possible for this method
	 * to return false even if <I>node</I> has no children.  A directory in a
	 * filesystem, for example, may contain no files; the node representing
	 * the directory is not a leaf, but it also has no children.
	 *
	 * @param   node    a node in the tree, obtained from this data source
	 * @return  true if <I>node</I> is a leaf
	 */
	public boolean isLeaf(Object node) {
		return ((TreeNode) node).isLeaf();
	}

	void nodesWereInserted(Object parent, int index0, int index1) {
		int[] indexArray = new int[index1 - index0 + 1];

		for (int i = index0; i <= index1; i++) {
			indexArray[i - index0] = i;
		}

		nodesWereInserted(parent, indexArray);
	}

	void nodesWereRemoved(Object parent, int index0, int index1, Object[] childrenArray) {
		int[] indexArray = new int[index1 - index0 + 1];

		for (int i = 0; i <= index1 - index0; i++) {
			indexArray[i] = index0 + i;
		}

		nodesWereRemoved(parent, indexArray, childrenArray);
	}

	public void reloadNode(Object node) {
		reloadNode(node, true);
	}

	public void reloadNode(Object node, boolean withChildren) {
		if (isEventCreation()) {
			IObjectTreeNode objectTreeNode =
				(node instanceof IObjectTreeNode) ? ((IObjectTreeNode) node) : searchObjectTreeNode(node);

			if (withChildren) {
				nodeStructureChanged(objectTreeNode);

			} else {
				TreeNode parent = objectTreeNode.getParent();

				if (parent != null) {
					int index = parent.getIndex(objectTreeNode);
					if (index != -1) {
						nodesChanged(parent, new int[] { index });
					}
				}
			}
		}
	}

	private IObjectTreeNode searchObjectTreeNode(IObjectTreeNode currNode, Object node) {
		if (currNode.getUserObject() == node)
			return currNode;

		else if (!currNode.isLeaf() && currNode.getAllowsChildren()) {
			for (int j = 0; j < currNode.getChildCount(); j++) {
				IObjectTreeNode childNode = (IObjectTreeNode) currNode.getChildAt(j);

				IObjectTreeNode foundNode = searchObjectTreeNode(childNode, node);

				if (foundNode != null)
					return foundNode;
			}

		}

		return null;
	}

	public IObjectTreeNode searchObjectTreeNode(Object node) {
		return searchObjectTreeNode((IObjectTreeNode) getRoot(), node);
	}

	public void setSeSAmDeclaration(ISeSAmDeclaration newSeSAmDeclaration) {
		// ISeSAmDeclaration oldSeSAmDeclaration = getSeSAmDeclaration();

		_seSAmDeclaration = newSeSAmDeclaration;

		reload();
	}

	/**
	  * Messaged when the user has altered the value for the item identified
	  * by <I>path</I> to <I>newValue</I>.  If <I>newValue</I> signifies
	  * a truly new value the model should post a treeNodesChanged
	  * event.
	  *
	  * @param path path to the node that the user has altered.
	  * @param newValue the new value from the TreeCellEditor.
	  */
	public void valueForPathChanged(TreePath path, Object newValue) {
		Assert.fail();
	}

	protected void fireTreeNodesChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {

		if (isEventCreation())
			super.fireTreeNodesChanged(source, path, childIndices, children);
	}

	protected void fireTreeNodesInserted(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		if (isEventCreation())
			super.fireTreeNodesInserted(source, path, childIndices, children);
	}

	protected void fireTreeNodesRemoved(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		if (isEventCreation())
			super.fireTreeNodesRemoved(source, path, childIndices, children);
	}

	protected void fireTreeStructureChanged(
		Object source,
		Object[] path,
		int[] childIndices,
		Object[] children) {
		if (isEventCreation())
			super.fireTreeStructureChanged(source, path, childIndices, children);
	}

	public void setEventCreation(boolean eventCreation) {
		boolean oldValue = isEventCreation();

		_eventCreation = eventCreation;

		UpdateChildrenAbleNodeThread.getInstance().setUpdate(eventCreation);

		if (eventCreation && !oldValue) {
			reload();
		}
	}

	public boolean isEventCreation() {
		return _eventCreation;
	}

}