package de.uniwue.ki.mas.model.tree;
import java.util.LinkedList;

import javax.swing.tree.TreeNode;

import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationListAndFactory;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.IObjectFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.functionUser.UserFunctionDeclarationListAndFactory;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.ActivityGraphNodeDeclarationList;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActionActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphComposedNodeDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.ReasoningEngineDeclarationListAndFactory;
import de.uniwue.ki.mas.behaviour.rule.ActivityGraphEdgeDeclarationList;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphEdgeDeclaration;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphRuleDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.ObjectInstanceDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.BodyVariableClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.variableclass.FeatureVariableClassDeclarationListAndFactory;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelBasicsDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.model.ISimulationElementsDeclaration;
import de.uniwue.ki.mas.model.userInterface.IUserInterfaceDeclaration;
import de.uniwue.ki.mas.sim.analysis.AnalysisDeclarationList;
import de.uniwue.ki.mas.sim.analysis.AnalysisDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.analysis.IAnalysisDeclaration;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationListAndFactory;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.list.IMutableListModel;
import de.uniwue.ki.util.list.IMutableListModelAndFactory;
import de.uniwue.ki.util.tree.DefaultObjectTreeNodeLeaf;
import de.uniwue.ki.util.tree.IObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNodeFactory;
import de.uniwue.ki.util.tree.MutableListModelAndFactoryTreeNodeAdapter;

/**
 * Creation date: (05.10.2001 09:32:57)
 * 
 * @author Christoph Oechslein
 */
final class SeSAmDeclarationTreeModelCreater {
	
	class RootObjectTreeNode extends DefaultDeclarationTreeNode {
		public RootObjectTreeNode() {
			super(null, getSeSAmDeclaration());
		}

		SeSAmDeclarationTreeModelAdapter getSeSAmDeclarationTreeModelAdapter() {
			return _seSAmDeclarationTreeModelAdapter;
		}
	}

	static SeSAmDeclarationTreeModelAdapter getSeSAmDeclarationTreeModelAdapter(
			TreeNode treeNode) {
		if (treeNode == null) {
			return null;

		} else if (treeNode instanceof RootObjectTreeNode) {
			return ((RootObjectTreeNode) treeNode)
					.getSeSAmDeclarationTreeModelAdapter();

		} else {
			return getSeSAmDeclarationTreeModelAdapter(treeNode.getParent());
		}
	}

	private final IObjectTreeNodeFactory _activityTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IActivityGraphNodeDeclaration activityDeclaration = (IActivityGraphNodeDeclaration) userObject;
			if (activityDeclaration instanceof IActivityGraphComposedNodeDeclaration) {
				return createActivityGraphDeclarationTreeNode(
						parent,
						((IActivityGraphComposedNodeDeclaration) activityDeclaration)
								.getInnerActivityGraphDeclaration());

			} else if (activityDeclaration instanceof IActionActivityGraphLeafDeclaration) {
				return createActionAtomicActivityDeclaration(
						parent,
						(IActionActivityGraphLeafDeclaration) activityDeclaration);

			} else {
				return new DefaultDeclarationTreeNodeLeaf(parent,
						activityDeclaration);
			}
		}
	};

	private final IObjectTreeNodeFactory _agentTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IAgentClassDeclaration userObject1 = (IAgentClassDeclaration) userObject;
			DefaultDeclarationTreeNode agentTreeNode = new DefaultDeclarationTreeNode(
					parent, userObject1);

			agentTreeNode.addChild(createObjectFeatureListTreeNode(
					agentTreeNode, userObject1
							.getObjectFeatureClassDeclarationList()));

			agentTreeNode.addChild(createBodyVariableListTreeNode(
					agentTreeNode, userObject1.getBodyClassDeclaration()
							.getBodyVariableClassDeclarationListAndFactory()));

			agentTreeNode.addChild(createReasoningEngineDeclListTreeNode(
					agentTreeNode, userObject1
							.getReasoningEngineDeclarationListAndFactory()));

			return agentTreeNode;
		}
	};

	private final static IObjectTreeNodeFactory _analysisTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			return new AnalysisDeclarationTreeNode(parent,
					((IAnalysisDeclaration) userObject));
		}
	};

	/**
	 * Here bodyVariableClassDeclarations and bodyVariableInstancedeclarations
	 * are handled
	 */
	private final IObjectTreeNodeFactory _bodyVariableDeclarationTreeNodeFactory = new VariableDeclarationTreeNodeFactory();

	private final IObjectTreeNodeFactory _experimentTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IExperimentDeclaration experimentDeclaration = (IExperimentDeclaration) userObject;
			DefaultDeclarationTreeNode modelBasicsTreeNode = new DefaultDeclarationTreeNode(
					parent, experimentDeclaration);

			modelBasicsTreeNode.addChild(FunctionCallDeclarationTreeNodeFactory.getInstance().createObjectTreeNode(
					parent, experimentDeclaration
							.getSimulationActionFunctionCallDeclaration()));

			return modelBasicsTreeNode;
		}
	};

	private final IObjectTreeNodeFactory _objectFeatureTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IObjectFeatureClassDeclaration userObject1 = (IObjectFeatureClassDeclaration) userObject;
			if (userObject1 instanceof IUserFeatureClassDeclaration) {
				return _userFeatureTreeNodeFactory.createObjectTreeNode(
						parent, (IUserFeatureClassDeclaration) userObject1);

			} else {
				return new DefaultDeclarationTreeNodeLeaf(parent, userObject1);
			}
		}
	};

	private final IObjectTreeNodeFactory _reasoningEngineDeclTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			return createActivityGraphDeclarationTreeNode(
					parent,
					((IActivityReasoningEngineDeclaration) ((IReasoningEngineDeclaration) userObject))
							.getRootActivityGraphDeclaration());
		}
	};

	private final IObjectTreeNodeFactory _ressourceTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IRessourceClassDeclaration userObject1 = (IRessourceClassDeclaration) userObject;
			DefaultDeclarationTreeNode ressourceTreeNode = new DefaultDeclarationTreeNode(
					parent, userObject1);

			ressourceTreeNode
					.addChildrens(CollectionsHelper
							.createIterator(
									createObjectFeatureListTreeNode(
											ressourceTreeNode,
											userObject1
													.getObjectFeatureClassDeclarationList()),
									createBodyVariableListTreeNode(
											ressourceTreeNode,
											userObject1
													.getBodyClassDeclaration()
													.getBodyVariableClassDeclarationListAndFactory())));

			return ressourceTreeNode;
		}
	};

	private final IObjectTreeNodeFactory _ruleTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IActivityGraphEdgeDeclaration activityEdgeDeclaration = (IActivityGraphEdgeDeclaration) userObject;
			if (activityEdgeDeclaration instanceof IActivityGraphRuleDeclaration)
				return new ActivityRuleDeclarationTreeNode(parent,
						(IActivityGraphRuleDeclaration) activityEdgeDeclaration);
			else
				return new DefaultObjectTreeNodeLeaf(parent,
						activityEdgeDeclaration);
		}
	};

	protected final SeSAmDeclarationTreeModelAdapter _seSAmDeclarationTreeModelAdapter;

	private final IObjectTreeNodeFactory _userFeatureTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			IUserFeatureClassDeclaration userFeatureClassDeclaration = (IUserFeatureClassDeclaration) userObject;
			DefaultDeclarationTreeNode userFeatureTreeNode = new DefaultDeclarationTreeNode(
					parent, userFeatureClassDeclaration);

			userFeatureTreeNode
					.addChildrens(CollectionsHelper
							.createIterator(
									createFeatureVariableListTreeNode(
											userFeatureTreeNode,
											userFeatureClassDeclaration
													.getFeatureVariableClassDeclarationListAndFactory()),
									createUserFunctionListTreeNode(
											userFeatureTreeNode,
											userFeatureClassDeclaration
													.getUserFunctionDeclarationListAndFactory())));

			return userFeatureTreeNode;
		}
	};

	private final IObjectTreeNodeFactory _userInterfaceTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object userObject) {
			return createUserInterfaceTreeNode(parent,
					(IUserInterfaceDeclaration) userObject);
		}
	};

	private final IObjectTreeNodeFactory _userTypeTreeNodeFactory = new UserTypeDeclarationTreeNodeFactory();

	private final IObjectTreeNodeFactory _variableClassDeclarationTreeNodeFactory = new IObjectTreeNodeFactory() {
		public IObjectTreeNode createObjectTreeNode(IObjectTreeNode parent,
				Object obj) {
			IVariableClassDeclaration userObject = (IVariableClassDeclaration) obj;
			DefaultDeclarationTreeNode variableClassdeclarationTreeNode = new DefaultDeclarationTreeNode(
					parent, userObject);

			variableClassdeclarationTreeNode
					.addChildrens(CollectionsHelper
							.createIterator(
									FunctionCallDeclarationTreeNodeFactory.getInstance()
											.createObjectTreeNode(
													variableClassdeclarationTreeNode,
													userObject
															.getDefaultValueFunctionCallDeclaration()),
									FunctionCallDeclarationTreeNodeFactory.getInstance()
											.createObjectTreeNode(
													variableClassdeclarationTreeNode,
													userObject
															.getNextValueFunctionCallDeclaration())));

			return variableClassdeclarationTreeNode;
		}
	};

	public SeSAmDeclarationTreeModelCreater(
			SeSAmDeclarationTreeModelAdapter seSAmDeclarationTreeModelAdapter) {
		super();

		_seSAmDeclarationTreeModelAdapter = seSAmDeclarationTreeModelAdapter;
	}

	private IObjectTreeNode createActionAtomicActivityDeclaration(
			IObjectTreeNode parent,
			IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration) {

		DefaultDeclarationTreeNode actionActivityTreeNode = new DefaultDeclarationTreeNode(
				parent, actionAtomicActivityDeclaration);

		actionActivityTreeNode.addChild(createFunctionActionCallListTreeNode(
				actionActivityTreeNode, actionAtomicActivityDeclaration
						.getFunctionActionCallDeclarationListAndFactory()));

		if (!actionAtomicActivityDeclaration
				.getEntryFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList().isEmpty()) {
			actionActivityTreeNode
					.addChild(createFunctionActionCallListTreeNode(
							actionActivityTreeNode,
							actionAtomicActivityDeclaration
									.getEntryFunctionActionCallDeclarationListAndFactory()));
		}

		if (!actionAtomicActivityDeclaration
				.getExitFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList().isEmpty()) {
			actionActivityTreeNode
					.addChild(createFunctionActionCallListTreeNode(
							actionActivityTreeNode,
							actionAtomicActivityDeclaration
									.getExitFunctionActionCallDeclarationListAndFactory()));
		}

		return actionActivityTreeNode;
	}

	protected IObjectTreeNode createActivityGraphDeclarationTreeNode(
			IObjectTreeNode parent,
			IActivityGraphDeclaration activityGraphDeclaration) {

		DefaultDeclarationTreeNode composedActivityTreeNode = new DefaultDeclarationTreeNode(
				parent, activityGraphDeclaration);

		composedActivityTreeNode.addChildrens(CollectionsHelper.createIterator(
				createActivityListTreeNode(composedActivityTreeNode,
						activityGraphDeclaration
								.getActivityGraphNodeDeclarationList()),
				createRuleListTreeNode(composedActivityTreeNode,
						activityGraphDeclaration
								.getActivityGraphEdgeDeclarationList())));

		return composedActivityTreeNode;
	}

	protected IObjectTreeNode createActivityListTreeNode(
			IObjectTreeNode parent,
			ActivityGraphNodeDeclarationList activityDeclarationList) {
		return createMutableListModelTreeNodeAdapter(parent,
				activityDeclarationList, _activityTreeNodeFactory);
	}

	protected IObjectTreeNode createAgentListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmModelDeclaration()
						.getAgentClassDeclarationListAndFactory(),
				_agentTreeNodeFactory);
	}

	protected static IObjectTreeNode createAnalysisListTreeNode(
			IObjectTreeNode parent,
			AnalysisDeclarationList analysisDeclarationList) {
		return createMutableListModelTreeNodeAdapter(parent,
				analysisDeclarationList, _analysisTreeNodeFactory);
	}

	protected IObjectTreeNode createAnalysisListTreeNode(
			IObjectTreeNode parent,
			AnalysisDeclarationListAndFactory analysisDeclarationListAndFactory) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				analysisDeclarationListAndFactory, _analysisTreeNodeFactory);
	}

	protected IObjectTreeNode createBodyVariableListTreeNode(
			IObjectTreeNode parent,
			BodyVariableClassDeclarationListAndFactory bodyVariableClassDeclarationListAndFactory) {

		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				bodyVariableClassDeclarationListAndFactory,
				_bodyVariableDeclarationTreeNodeFactory);
	}

	protected static IObjectTreeNode createEndFunctionListTreeNode(
			IObjectTreeNode parent,
			IDeclarationListAndFactory functionDeclarationListAndFactory) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				functionDeclarationListAndFactory,
				FunctionCallDeclarationTreeNodeFactory.getInstance());
	}

	protected IObjectTreeNode createExperimentListTreeNode(
			IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSimulationElementsDeclaration()
						.getExperimentDeclarationListAndFactory(),
				_experimentTreeNodeFactory);
	}

	protected IObjectTreeNode createFeatureVariableListTreeNode(
			IObjectTreeNode parent,
			FeatureVariableClassDeclarationListAndFactory featureVariableClassDeclarationListAndFactory) {

		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				featureVariableClassDeclarationListAndFactory,
				_variableClassDeclarationTreeNodeFactory);
	}

	protected IObjectTreeNode createFunctionActionCallListTreeNode(
			IObjectTreeNode parent,
			FunctionActionCallDeclarationListAndFactory functionActionCallDeclarationListAndFactory) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				functionActionCallDeclarationListAndFactory,
				FunctionCallDeclarationTreeNodeFactory.getInstance());
	}

	protected IObjectTreeNode createModelBasicsTreeNode(IObjectTreeNode parent) {
		DefaultDeclarationTreeNode modelBasicsTreeNode = new DefaultDeclarationTreeNode(
				parent, getSeSAmModelBasicsDeclaration());

		modelBasicsTreeNode.addChildrens(CollectionsHelper.createIterator(
				createUserFunctionListTreeNode(modelBasicsTreeNode,
						getSeSAmModelBasicsDeclaration()
								.getUserFunctionDeclarationListAndFactory()),
				createUserFeatureListTreeNode(modelBasicsTreeNode),
				createUserTypeListTreeNode(modelBasicsTreeNode)));

		return modelBasicsTreeNode;
	}

	private static MutableListModelAndFactoryTreeNodeAdapter createMutableListModelAndFactoryTreeNodeAdapter(
			IObjectTreeNode parent,
			IMutableListModelAndFactory mutableListModelAndFactory,
			IObjectTreeNodeFactory objectTreeNodeFactory) {

		return new MutableListModelAndFactoryTreeNodeAdapterWithUpdate(parent,
				mutableListModelAndFactory, objectTreeNodeFactory);
	}

	static MutableListModelTreeNodeAdapterWithUpdate createMutableListModelTreeNodeAdapter(
			IObjectTreeNode parent, IMutableListModel mutableListModel,
			IObjectTreeNodeFactory objectTreeNodeFactory) {

		return new MutableListModelTreeNodeAdapterWithUpdate(parent,
				mutableListModel, objectTreeNodeFactory);
	}

	protected IObjectTreeNode createObjectFeatureListTreeNode(
			IObjectTreeNode parent,
			IObjectFeatureClassDeclarationList objectFeatureClassDeclarationList) {

		return createMutableListModelTreeNodeAdapter(parent,
				objectFeatureClassDeclarationList,
				_objectFeatureTreeNodeFactory);
	}

	protected static IObjectTreeNode createObjectInstanceListTreeNode(
			IObjectTreeNode parent,
			ObjectInstanceDeclarationList objectInstanceDeclarationList) {
		return createMutableListModelTreeNodeAdapter(parent,
				objectInstanceDeclarationList,
				ObjectInstanceDeclarationTreeNodeFactory.getInstance());
	}

	protected IObjectTreeNode createReasoningEngineDeclListTreeNode(
			IObjectTreeNode parent,
			ReasoningEngineDeclarationListAndFactory reasoningEngineDeclarationListAndFactory) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				reasoningEngineDeclarationListAndFactory,
				_reasoningEngineDeclTreeNodeFactory);
	}

	protected IObjectTreeNode createRessourceListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmModelDeclaration()
						.getRessourceClassDeclarationListAndFactory(),
				_ressourceTreeNodeFactory);
	}

	private IObjectTreeNode createRuleListTreeNode(IObjectTreeNode parent,
			ActivityGraphEdgeDeclarationList ruleDeclarationList) {
		return createMutableListModelTreeNodeAdapter(parent,
				ruleDeclarationList, _ruleTreeNodeFactory);
	}

	public IObjectTreeNode createSeSAmDeclarationTreeNode() {
		RootObjectTreeNode rootTreeNode = new RootObjectTreeNode();

		rootTreeNode
				.addChildrens(CollectionsHelper.createConsIterator(
						createModelBasicsTreeNode(rootTreeNode),
						createRessourceListTreeNode(rootTreeNode),
						createAgentListTreeNode(rootTreeNode),
						CollectionsHelper.createIterator(
								createWorldListTreeNode(rootTreeNode),
								createSimulationElementsTreeNode(rootTreeNode),
								createSimulationRunListTreeNode(rootTreeNode))));

		return rootTreeNode;
	}

	private IObjectTreeNode createSimulationElementsTreeNode(
			IObjectTreeNode parent) {
		DefaultDeclarationTreeNode simulationElementsTreeNode = new DefaultDeclarationTreeNode(
				parent, getSimulationElementsDeclaration());

		LinkedList simulationElems = new LinkedList();
		simulationElems
				.add(createSituationListTreeNode(simulationElementsTreeNode));
		simulationElems.add(createAnalysisListTreeNode(
				simulationElementsTreeNode, getSimulationElementsDeclaration()
						.getAnalysisDeclarationListAndFactory()));
		simulationElems
				.add(createSimulationListTreeNode(simulationElementsTreeNode));
		if (false) //Einstiegspunkt fr Diana:
			simulationElems
					.add(createUserInterfaceListTreeNode(simulationElementsTreeNode));
		simulationElems
				.add(createExperimentListTreeNode(simulationElementsTreeNode));

		simulationElementsTreeNode.addChildrens(simulationElems.iterator());

		return simulationElementsTreeNode;
	}

	private IObjectTreeNode createSimulationListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSimulationElementsDeclaration()
						.getSimulationDeclarationListAndFactory(),
				SimulationDeclarationTreeNodeFactory.getInstance());
	}

	protected IObjectTreeNode createSimulationRunListTreeNode(
			IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmDeclaration().getExperimentListAndFactory(),
				DefaultDeclarationTreeNodeLeafFactory.getInstance());
	}

	protected IObjectTreeNode createSituationListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSimulationElementsDeclaration()
						.getSituationDeclarationListAndFactory(),
				SituationDeclarationTreeNodeFactory.getInstance());
	}

	protected IObjectTreeNode createUserFeatureListTreeNode(
			IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory(),
				_userFeatureTreeNodeFactory);
	}

	protected IObjectTreeNode createUserFunctionListTreeNode(
			IObjectTreeNode parent,
			UserFunctionDeclarationListAndFactory userDefinedFunctionDeclarationListAndFactory) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				userDefinedFunctionDeclarationListAndFactory,
				UserFunctionDeclarationTreeNodeFactory.getInstance());
	}

	private IObjectTreeNode createUserInterfaceListTreeNode(
			IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSimulationElementsDeclaration()
						.getUserInterfaceDeclarationListAndFactory(),
				_userInterfaceTreeNodeFactory);
	}

	protected IObjectTreeNode createUserInterfaceTreeNode(
			final IObjectTreeNode parent,
			final IUserInterfaceDeclaration userObject) {
		return new UserInterfaceTreeNode(this, parent, userObject);
	}

	protected IObjectTreeNode createUserTypeListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmModelBasicsDeclaration()
						.getUserTypeDeclarationListAndFactory(),
				_userTypeTreeNodeFactory);
	}

	protected IObjectTreeNode createWorldListTreeNode(IObjectTreeNode parent) {
		return createMutableListModelAndFactoryTreeNodeAdapter(parent,
				getSeSAmModelDeclaration()
						.getWorldClassDeclarationListAndFactory(),
				_agentTreeNodeFactory);
	}

	public ISeSAmDeclaration getSeSAmDeclaration() {
		return _seSAmDeclarationTreeModelAdapter.getSeSAmDeclaration();
	}

	private ISeSAmModelBasicsDeclaration getSeSAmModelBasicsDeclaration() {
		return getSeSAmModelDeclaration().getSeSAmModelBasicsDeclaration();
	}

	private ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return getSeSAmDeclaration().getSeSAmModelDeclaration();
	}

	private ISimulationElementsDeclaration getSimulationElementsDeclaration() {
		return getSeSAmModelDeclaration().getSimulationElementsDeclaration();
	}
}