package de.uniwue.ki.mas.model.tree;
import java.util.*;
/**
 * Creation date: 07.11.2002, 09:38:21
 * @author oechslein
 */
public final class UpdateChildrenAbleNodeThread extends Thread implements Runnable {

	private static final UpdateChildrenAbleNodeThread _instance = new UpdateChildrenAbleNodeThread();

	private boolean _started = false;
	private boolean _update = true;
	private final Set _nodesToUpdate = Collections.synchronizedSet(new LinkedHashSet());

	private UpdateChildrenAbleNodeThread() {
		super("Updating Children");

		setPriority(Thread.MIN_PRIORITY);
		setDaemon(true);
	}

	public void run() {
		while (true) {
			while (isUpdate() && !_nodesToUpdate.isEmpty()) {
				ArrayList copyList;
				synchronized (_nodesToUpdate) {
					if (_nodesToUpdate.isEmpty())
						break;

					copyList = new ArrayList(_nodesToUpdate);
					_nodesToUpdate.clear();
				}

				updateChildrens(copyList.iterator());

				Thread.yield();
			}

			try {
				synchronized (this) {
					wait();
				}
			} catch (InterruptedException e) {
				// there is a new node in _nodesToUpdate (called #notify() in #addNode())
			}
		}
	}

	public void updateChildrens(Iterator it) {
		while (isUpdate() && it.hasNext()) {
			((IUpdateChildrenAbleNode) it.next()).updateChildren();
		}
	}

	public void addNode(IUpdateChildrenAbleNode updateChildrenAbleNode) {
		boolean result = _nodesToUpdate.add(updateChildrenAbleNode);

		if (result && isUpdate()) {
			newNodeAdded();
		}
	}

	private synchronized void newNodeAdded() {
		if (!_started) {
			_started = true;
			start();
		}

		notifyAll();
	}

	public void forceUpdateChildren() {
		if (isUpdate()) {
			synchronized (_nodesToUpdate) {
				updateChildrens(_nodesToUpdate.iterator());

				_nodesToUpdate.clear();
			}
		}
	}

	public static UpdateChildrenAbleNodeThread getInstance() {
		return _instance;
	}

	void setUpdate(boolean update) {
		boolean oldValue = isUpdate();

		_update = update;

		if (!oldValue && update) {
			forceUpdateChildren();

			newNodeAdded();
		}
	}

	boolean isUpdate() {
		return _update;
	}
}