/**
 * Created on 13.06.2002
 *
 * BufferedHTMLWriter extends BufferedWriter and provides an write-method that replaces newlines with System.getProperty("line.separator").
 * It replaces tabs by spaces, too.
 * And provides methods to write html-document headers (@see writeDocumentHeader()) and footers (@see writeDocumentFooter()) and a 
 * convenient way of closing the Stream.
 */
package de.uniwue.ki.mas.modelprint;

import java.io.*;

/**
 * @author glueckselig
 *
 */
public class BufferedHTMLWriter extends BufferedWriter {
	protected static String NL = System.getProperty("line.separator"); // NewLine of the current system
	protected static String SP = "   "; //spaces used for tabbing output
	protected String Filename;

	public BufferedHTMLWriter(File file) throws IOException {
		super(new FileWriter(file));
		Filename = file.getName();
	}

	/** Method write(String) replaces newlines with System.getProperty("line.separator") and tabs with spaces.
	 * The @see IOException thrown by @see java.io.Writer#write(String) gets directly caught.	
	 */
	public void write(String str) {
		try {
			super.write(str.replaceAll("\n", NL).replaceAll("\t", SP) + NL);
		} catch (IOException e) {
			System.out.println("Error writing to file!");
		}
	}

	public void writeDocumentHeader() {
		write("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
		write("\n<html>\n\t<head>");
		write("\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">");
		write("\n\t<title>XMLSeSAmModel</title>");
		//			write("\n\t\t<link type=\"text/css\" rel=\"stylesheet\" href=\"style.css\"/>");
		write(generateStylesheet());
		write("\n\t</head>\n\t<body style=\"font-family:monospace, Georgia,'Times New Roman',Times,serif;background-color: #FFFFee;color:#444444;margin-left: 12.5%;margin-right: 12.5%;margin-top: 5%;\">");
	}

	private String generateStylesheet() {
		StringBuffer result = new StringBuffer();

		result.append("\n\t<style type=\"text/css\">");
		result.append("\n\t\tem { font-weight: 600;}");
		result.append("\n\t\tem.composedActivity { font-size: 133%;color:green;}");
		result.append("\n\t\th1,h2,h3,h4,h5 { color:#CC9933;}");
		result.append("\n\t</style>");

		return result.toString();
	}

	public void writeDocumentFooter() {
		write("<div style=\"margin-top:100%\"><br></div>");
		write("\n\t</body>\n</html>");
	}

	public void writeHead(String version) {
		write(
			"<h1 style=\"word-spacing:1.5em;\">Version "
				+ version
				+ " of "
				+ "<u>"
				+ Filename
				+ "</u>"
				+ "</h1>\n<hr size=\"1\"/>");
	}

	public void closeWriter() {
		try {
			flush();
			close();
		} catch (IOException e) {
			System.out.println("Unable to close HTML-file.");
		}
	}

}
