/**
 * Created on 03.07.2002
 *
 * This class is responsible for writing the functiondeclarations.
 * @author glueckselig
*/

package de.uniwue.ki.mas.modelprint;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionArgument.*;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.UserFunctionDeclarationList;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;

public final class HTMLFunctionsWriter extends HTMLModelWriterBase {

	public HTMLFunctionsWriter() {
	}

	public void writeUserFunctionsMain(UserFunctionDeclarationList userFuncList) {
		write("<br><br>" + _tags.openTag(SUB1_HEADING) + "UserFunctions" + _tags.closeTag());

		write(createUserFunctionsFromIterator(userFuncList.iterator()));
	}

	//called by HTMLModelWriter.writeUserFeatures(IFeatureClassDeclarationList userFeatList)
	public void writeUserFunctionsSubtitle(UserFunctionDeclarationList userFuncList) {
		write("<br><br>" + _tags.openTag(SUB3_HEADING) + "UserFunctions" + _tags.closeTag());

		write(createUserFunctionsFromIterator(userFuncList.iterator()));
	}

	private String createUserFunctionsFromIterator(Iterator iterator) {
		StringBuffer buffer = new StringBuffer();

		while (iterator.hasNext()) {
			IUserFunctionDeclaration userFun = ((IUserFunctionDeclaration) iterator.next());

			buffer.append(_tags.openTag(USER_FUNCTION_MARGIN_TOP));
			buffer.append(
				"<em>"
					+ escape(userFun.getName())
					+ "</em>"
					+ createSpecificFunctionDeclarationWithArgDeclBinds(userFun));
			buffer.append(_tags.closeTag());
		}

		return buffer.toString();
	}

	private static String createSpecificFunctionDeclarationWithArgDeclBinds(ISpecificFunctionDeclarationWithArgDeclBinds userFun) {
		StringBuffer result = new StringBuffer();

		result.append(createAnchor(userFun));

		IParametricFunctionCallDeclaration currentDecl = userFun.getParametricFunctionCallDeclaration();
		result.append(_tags.openTag(USER_FUNCTION_PARAMETER_BOX));
		result.append(createParameter(currentDecl));
		result.append(_tags.closeTag());

		result.append(createFunctionCall(currentDecl.getFunctionCallDeclaration()));

		return result.toString();
	}

	private static String createParameter(IParametricFunctionCallDeclaration currentDecl) {
		StringBuffer buffer = new StringBuffer();
		buffer.append("( ");

		Iterator it = currentDecl.getFunctionArgumentDeclarationList().iterator();
		while (it.hasNext()) {
			IFunctionArgumentDeclaration element = (IFunctionArgumentDeclaration) it.next();
			buffer.append("<u>" + element.getName() + "</u>");
			buffer.append(":");
			buffer.append(escape(element.getSpecificType().getName()));

			if (it.hasNext())
				buffer.append(", ");
		}

		buffer.append(" )");

		return buffer.toString();
	}

	public static String createFunctionCallIterator(Iterator functionsIterator) {
		StringBuffer buffer = new StringBuffer();

		while (functionsIterator.hasNext()) {
			buffer.append(createFunctionCall((IFunctionCallDeclaration) functionsIterator.next()));
		}

		return buffer.toString();
	}

	public static String createFunctionCall(IFunctionCallDeclaration call) {
		return createFunctionCallIntern(((IFunctionCallDeclaration) call).getFunctionCallInternDeclaration());
	}

	public static String createFunctionCallIntern(IFunctionCallInternDeclaration functionCallInternDecl) {
		StringBuffer buffer = new StringBuffer();

		if (functionCallInternDecl instanceof IComposedFunctionCallInternDeclaration) {
			buffer.append(_tags.openTag(BOXED_COMPOSED));
			buffer.append(
				createReasoningComposedFunctionCall(
					(IComposedFunctionCallInternDeclaration) functionCallInternDecl));
			buffer.append(_tags.closeTag());
		} else {
			buffer.append(_tags.openTag(BOXED_ATOMIC));
			buffer.append(
				createReasoningAtomicFunctionCall(
					(IAtomicFunctionCallInternDeclaration) functionCallInternDecl));
			buffer.append(_tags.closeTag());
		}

		return buffer.toString();
	}

	public static String interpolateTypeResult(IFunctionCallInternDeclaration functionCallInternDeclaration) {
		String result;
		if (functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration) {
			result =
				createReasoningComposedFunctionCall(
					(IComposedFunctionCallInternDeclaration) functionCallInternDeclaration);
		} else if (functionCallInternDeclaration instanceof IAtomicFunctionCallInternDeclaration) {
			result =
				//				"<span style=\"background: yellow\">"+ 
	createReasoningAtomicFunctionCall((IAtomicFunctionCallInternDeclaration) functionCallInternDeclaration)
				//					+ "</span>"
	;
		} else {
			result = "Error in HTMLFunctions.interpolateTypeResult()";
		}

		return result;
	}

	private static String createReasoningComposedFunctionCall(IComposedFunctionCallInternDeclaration icDec) {
		StringBuffer buffer = new StringBuffer();

		ISpecificFunctionDeclaration functionDeclaration = icDec.getSpecificFunctionDeclaration();

		String functionsName = functionDeclaration.getName();
		if (functionsName.equals("Filter")
			|| functionsName.equals("DoWith")
			|| functionsName.equals("ForElements")
			|| functionsName.equals("Map")) {
			if (icDec.getChildCount() == 2) {

				buffer.append(escape(functionsName));

				IFunctionCallInternDeclaration param2 = icDec.getChild(0);
				IFunctionCallInternDeclaration param1 = icDec.getChild(1);

				String param1Result = interpolateTypeResult(param1);
				String param2Result = interpolateTypeResult(param2);

				buffer.append(" (" + param1Result + ") (" + param2Result + ")");

			} else {
				System.out.println(
					"Function argumentlength not matched in HTMLFunctionsWriter.createReasoningComposedFunctionCall(Composed)");
			}

		} else if (functionsName.equals("ForTimes")) {

			buffer.append(escape(functionsName));

			IFunctionCallInternDeclaration param1 = icDec.getChild(1);
			IFunctionCallInternDeclaration param2 = icDec.getChild(2);
			IFunctionCallInternDeclaration param3 = icDec.getChild(0);

			String param1Result = interpolateTypeResult(param1);
			String param2Result = interpolateTypeResult(param2);
			String param3Result = interpolateTypeResult(param3);

			buffer.append(": " + param1Result + " <i>to</i> " + param2Result + param3Result);

		} else if (functionsName.equals("If Then Else")) {
			IFunctionCallInternDeclaration param1 = icDec.getChild(0);
			IFunctionCallInternDeclaration param2 = icDec.getChild(1);
			IFunctionCallInternDeclaration param3 = icDec.getChild(2);

			String param1Result = interpolateTypeResult(param1);
			String param2Result = interpolateTypeResult(param2);
			String param3Result = interpolateTypeResult(param3);

			buffer.append(
				"<u>If</u> "
					+ param1Result
					+ " <u>then</u> "
					+ param2Result
					+ " <u>else</u> "
					+ param3Result
					+ "<br><br>");

		} else if (functionsName.equals("GetVariableFrom")) {
			buffer.append(
				"Get "
					+ interpolateTypeResult(icDec.getChild(0))
					+ " from "
					+ interpolateTypeResult(icDec.getChild(1)));

		} else if (functionsName.equals("SetVariableOf")) {
			buffer.append(
				"Set "
					+ interpolateTypeResult(icDec.getChild(0))
					+ " of "
					+ interpolateTypeResult(icDec.getChild(2))
					+ " to "
					+ interpolateTypeResult(icDec.getChild(1)));

		} else if (functionsName.equals("IncrementVarOf")) {
			buffer.append(
				"Increment "
					+ interpolateTypeResult(icDec.getChild(0))
					+ " of "
					+ interpolateTypeResult(icDec.getChild(2))
					+ " for "
					+ interpolateTypeResult(icDec.getChild(1)));

		} else if (functionsName.equals("RandomInt")) {
			buffer.append(
				"RandomInt between "
					+ "<span style=\"background:khaki;\">"
					+ interpolateTypeResult(icDec.getChild(0))
					+ "</span>"
					+ " and "
					+ "<span style=\"background:khaki;\">"
					+ interpolateTypeResult(icDec.getChild(1))
					+ "</span>");
		} else {

			if (functionDeclaration instanceof IUserFunctionDeclaration) {
				buffer.append(createLink(functionDeclaration, functionsName));
			} else {
				buffer.append(escape(functionsName));
			}

			//			boolean shortName = functionsName.length() <= 5;		

			buffer.append(" ");

			int index = -1;
			Iterator it = icDec.getChildren();
			while (it.hasNext()) {
				index++;
				IFunctionCallInternDeclaration functionCallInternDeclaration =
					(IFunctionCallInternDeclaration) it.next();
				boolean shortName = false; //isShortLine(buffer);
				if (!shortName) {
					//				if (!shortName) {
					//					if (functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration) {
					//						buffer.append(_tags.openTag(INVISIBLE_BOX_AFTER_FUNCTION_NAME));
					//					} else if (
					//						functionCallInternDeclaration instanceof IAtomicFunctionCallInternDeclaration) {
					//						buffer.append(_tags.openTag("<span>"));//_tags.openTag(SPACE_TO_FUNCTION_NAME));//
					//					} else {
					//						//					throw new Exception("Error in HTMLFunctionsWriter.createReasoningComposedFunctionCall()");
					//					}

					buffer.append(_tags.openTag(INVISIBLE_BOX_AFTER_FUNCTION_NAME));

				}

				String argName = functionDeclaration.getSpecificInputTypeNameAt(index);

				if (argName != null) {
					buffer.append("<em>");
					buffer.append(argName);
					buffer.append("</em>: <br>");
					buffer.append(_tags.openTag(INVISIBLE_BOX_AFTER_FUNCTION_NAME));
				}
				buffer.append(interpolateTypeResult(functionCallInternDeclaration) + " ");
				if (!shortName)
					buffer.append(_tags.closeTag());

				if (argName != null) {
					buffer.append(_tags.closeTag());
				}
			}
		}

		return buffer.toString();
	}

	private static String createReasoningAtomicFunctionCall(IAtomicFunctionCallInternDeclaration atomic) {
		return getArgument(atomic.getArgumentDeclaration());
	}

	// wird auch von UserFunctions aufgerufen
	private static String getArgument(IArgumentDeclaration argument) {
		if (argument instanceof IFunctionArgumentDeclaration) {
			return "(" + escape(((IFunctionArgumentDeclaration) argument).getName()) + ":Parameter" + ")";
		} else {
			ISpecificType specificType = argument.getSpecificType();

			if (GenericNumberType.getInstance().canCreate(specificType)
				|| specificType instanceof BooleanType) {
				return escape(getArgDeclObjectName(argument));

			} else if (argument.getObject() instanceof ISpecificFunctionDeclarationWithArgDeclBinds) {
				return createSpecificFunctionDeclarationWithArgDeclBinds(
					(ISpecificFunctionDeclarationWithArgDeclBinds) argument.getObject());

			} else if (specificType instanceof ObjectInstanceDeclarationType) {
				// kann noch nicht richtig verlinkt werden.
				// ::::::::::::::::::::::::::::::::::::::::::::::
				return "<em>" + escape(getArgDeclObjectName(argument)) + "</em>";

			} else {
				return createLink(argument.getObject(), getArgDeclObjectName(argument));
			}
		}
	}

	private static void appendArgDeclObjectName(
		StringBuffer buff,
		IArgumentDeclaration argumentDeclaration) {
		argumentDeclaration.getSpecificType().appendNameForArgumentDeclarationObject(
			buff,
			argumentDeclaration.getObject(),
			0,
			100);
	}

	private static String getArgDeclObjectName(IArgumentDeclaration argumentDeclaration) {
		StringBuffer buff = new StringBuffer();
		appendArgDeclObjectName(buff, argumentDeclaration);
		return buff.toString();
	}
}
