/**
 * Created on 16.07.2002
 *
 * This class is responsible for creating the overview over the content of the complete html-file.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.mas.modelprint;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.sim.analysis.IAnalysisDeclaration;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;

class HTMLHeaderWriter extends HTMLModelWriterBase {
	private ISeSAmDeclaration _seSAmDeclaration;

	private static final String MARGIN_LEFT = "<div style=\"margin-left:3em;\">";

	public HTMLHeaderWriter(ISeSAmDeclaration seSAmDeclaration) {
		super();

		_seSAmDeclaration = seSAmDeclaration;
	}

	public void writeOutContent() {
		write("<h2>Content</h2>");
		writeUserFunctions();
		writeUserFeatures();
		writeClasses();
		writeSituation();
		writeAnalysis();
		writeSimulation();
		writeExperiments();
		write("\n<hr size=\"1\"/>");
	}

	private void writeExperiments() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "Experiments" + close());

		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getExperimentDeclarationListAndFactory()
				.getExperimentDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IExperimentDeclaration element = (IExperimentDeclaration) it.next();
			result.append(createLink(element, element.getName()));
			
			breakLine(result, it.hasNext());
		}

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private void writeSimulation() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "Simulations" + close());
		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationList()
				.iterator();

		while (it.hasNext()) {
			ISimulationDeclaration element = (ISimulationDeclaration) it.next();
			result.append(createLink(element, element.getName()));

			breakLine(result, it.hasNext());
		}

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private void breakLine(StringBuffer result, boolean hasNext) {
		if (hasNext)
			result.append("<br>\n");
	}

	private void writeAnalysis() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "Analysis'" + close());

		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getAnalysisDeclarationListAndFactory()
				.getDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IAnalysisDeclaration element = (IAnalysisDeclaration) it.next();
			result.append(createLink(element, element.getName()));

			breakLine(result, it.hasNext());
		}

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private void writeSituation() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "Situations" + close());

		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSituationDeclarationListAndFactory()
				.getSituationDeclarationList()
				.iterator();

		while (it.hasNext()) {
			ISituationDeclaration element = (ISituationDeclaration) it.next();
			result.append(createLink(element, element.getName()));

			breakLine(result, it.hasNext());
		}

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private void writeClasses() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "Classes" + close());

		result.append(
			createClasses(
				_seSAmDeclaration
					.getSeSAmModelDeclaration()
					.getAgentClassDeclarationListAndFactory()
					.getAgentClassDeclarationList()
					.iterator(),
				_seSAmDeclaration
					.getSeSAmModelDeclaration()
					.getWorldClassDeclarationListAndFactory()
					.getWorldClassDeclarationList()
					.iterator()));

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private String createClasses(Iterator AgentIt, Iterator WorldIt) {
		StringBuffer result = new StringBuffer();

		boolean wasNotEmpty = AgentIt.hasNext();

		result.append(createAgentOrWorldClassLink(AgentIt));
		breakLine(result, wasNotEmpty && WorldIt.hasNext());
		result.append(createAgentOrWorldClassLink(WorldIt));

		return result.toString();
	}

	private String createAgentOrWorldClassLink(Iterator ClassIt) {
		StringBuffer result = new StringBuffer();

		while (ClassIt.hasNext()) {
			IAgentClassDeclaration element = (IAgentClassDeclaration) ClassIt.next();
			result.append(createLink(element, element.getName()));

			breakLine(result, ClassIt.hasNext());
		}

		return result.toString();
	}

	private void writeUserFeatures() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "UserFeatures" + close());

		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserFeatureClassDeclarationListAndFactory()
				.getUserFeatureClassDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IUserFeatureClassDeclaration element = (IUserFeatureClassDeclaration) it.next();

			result.append("<em>" + createLink(element, element.getName()) + "</em>");
			result.append(open(MARGIN_LEFT));
			result.append(open(SUB2_HEADING) + "UserFunctions" + close());
			result.append(
				createUserFunction(
					element
						.getUserFunctionDeclarationListAndFactory()
						.getUserFunctionDeclarationList()
						.iterator()));

			result.append(_tags.closeTag());
			breakLine(result, it.hasNext());
		}

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private void writeUserFunctions() {
		StringBuffer result = new StringBuffer();
		result.append(open(MARGIN_LEFT));
		result.append(open(SUB1_HEADING) + "UserFunctions" + close());

		Iterator it =
			_seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserFunctionDeclarationListAndFactory()
				.getUserFunctionDeclarationList()
				.iterator();

		result.append(createUserFunction(it));

		result.append(_tags.closeTag());

		write(result.toString());
	}

	private String createUserFunction(Iterator it) {
		StringBuffer result = new StringBuffer();
		while (it.hasNext()) {
			IUserFunctionDeclaration element = (IUserFunctionDeclaration) it.next();
			result.append(createLink(element, element.getName()));

			breakLine(result, it.hasNext());
		}

		return result.toString();
	}

	private String open(String element) {
		return _tags.openTag(element);
	}

	private String close() {
		return _tags.closeTag();
	}
}
