/**
 * Created on 04.07.2002
 *
 * This class is responsible for writing the interlinkage of activities.
 * 
 * @author glueckselig
 */
package de.uniwue.ki.mas.modelprint;

import java.util.Iterator;

import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.behaviour.rule.*;

class HTMLInterlinkageWriter extends HTMLModelWriterBase {
	public HTMLInterlinkageWriter() {
		_functionsWriter = new HTMLFunctionsWriter();
	}

	public void writeReasoningInterlinkage(IActivityGraphDeclaration composition) {
		write(_tags.openTag(SUB2_HEADING) + "Interlinkage of activities" + _tags.closeTag());

		Iterator chainRuleIterator = composition.getEdges();
		write("<table>");
		while (chainRuleIterator.hasNext()) {
			IActivityGraphEdgeDeclaration currentEdge =
				(IActivityGraphEdgeDeclaration) chainRuleIterator.next();

			IActivityGraphNodeDeclaration fromAct = currentEdge.getFromActivityGraphNodeDeclaration();
			IActivityGraphNodeDeclaration toAct = currentEdge.getToActivityGraphNodeDeclaration();

			if (currentEdge instanceof IActivityGraphTransitionEdgeDeclaration)
				writeInterlinkage(
					fromAct,
					toAct,
					getFunctionChainRuleCondition((IActivityGraphTransitionEdgeDeclaration) currentEdge));
		}
		write("</table>");
	}

	private void writeInterlinkage(
		IActivityGraphNodeDeclaration from,
		IActivityGraphNodeDeclaration to,
		String Function) {

		StringBuffer result = new StringBuffer();

		// :::::::::::::::::::::::::::::::: Zeile 1
		result.append(
			"\t<tr>\n\t\t"
				+ _tags.openTag("<td width=\"75\" valign=\"top\">")
				+ "\n\t\t\t<em>From:</em>\n\t\t"
				+ _tags.closeTag()
				+ "\t\t"
				+ _tags.openTag("<td valign=\"top\">")
				+ "\n\t\t\t<a href=\"#"
				+ _idMap.getReferenceCaughtExcpetion(from)
				+ "\">"
				+ escape(from.getName())
				+ "</a>\n\t\t"
				+ _tags.closeTag()
				+ "\n\t"
				+ "\t\t"
				+ _tags.openTag("<td width=15>")
				+ _tags.closeTag()
				+ "\n\t\t"
				+ _tags.openTag("<td rowspan =\"2\" valign =\"top\" >")
				+ "\n\t\t\t"
				+ Function
				+ "\t\t"
				+ _tags.closeTag()
				+ "\n\t</tr>");
		// :::::::::::::::::::::::::::::::: Zeile 2
		result.append(
			"\n\t<tr>\n\t\t"
				+ _tags.openTag("<td valign=\"bottom\">")
				+ "\n\t\t\t<em>To:</em>\n\t\t"
				+ _tags.closeTag()
				+ "\t\t"
				+ _tags.openTag("<td valign=\"bottom\">")
				+ "\n\t\t\t<a href=\"#"
				+ _idMap.getReferenceCaughtExcpetion(to)
				+ "\">"
				+ escape(to.getName())
				+ "</a>"
				+ "\t\t"
				+ _tags.closeTag()
				+ "\n\t\t"
				+ _tags.openTag("<td width=15>")
				+ _tags.closeTag()
				+ "\n\t</tr>");
		// :::::::::::::::::::::::::::::::: Zeile 3
		result.append(
			"\n\t<tr>\n\t\t"
				+ _tags.openTag("<td height=\"35\" colspan=\"4\" valign=\"middle\">")
				+ "<hr>"
				+ _tags.closeTag()
				+ "\n\t</tr>");

		write(result.toString());
	}

	private String getFunctionChainRuleCondition(IActivityGraphTransitionEdgeDeclaration decl) {

		if (decl instanceof IOtherwiseActivityGraphEdgeDeclaration)
			return "";
		else if (decl instanceof ActivityGraphRuleDeclaration)
			// TODO evtl. Fehler?
			return "";
		else {
			IActivityGraphRuleDeclaration rule = (IActivityGraphRuleDeclaration) decl;

			StringBuffer buffer =
				new StringBuffer(
					"<u>precondition:</u>"
						+ HTMLFunctionsWriter.createFunctionCall(rule.getConditionFunctionCallDeclaration()));

			return buffer.toString();
		}
	}
}
