/**
 * Created on 04.07.2002
 *
 * This class is responsible for writing bodyVariables.
 * 
 * @author glueckselig
 */

package de.uniwue.ki.mas.modelprint;

import java.util.Iterator;

import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;

class HTMLVariableWriter extends HTMLModelWriterBase {

	public HTMLVariableWriter() {

	}

	public void writeBodyVariables(IBodyClassDeclaration bodyClass) {
		StringBuffer result = new StringBuffer();
		result.append(
			"<br><br>" + _tags.openTag(SUB1_HEADING) + escape(bodyClass.getName()) + _tags.closeTag() + "<br><br>");

		Iterator variableIt =
			bodyClass
				.getBodyVariableClassDeclarationListAndFactory()
				.getBodyVariableClassDeclarationList()
				.iterator();

		result.append("<table cellpadding=\"10\" >");

		while (variableIt.hasNext()) {

			IBodyVariableClassDeclaration varDec = (IBodyVariableClassDeclaration) variableIt.next();
			IVariableClassDeclaration iVar = varDec.getVariableClassDeclaration();

			String title;

			if (varDec instanceof IBodyFeatureVariableClassDeclaration) {

				IBodyFeatureVariableClassDeclaration currentBodyFeature =
					(IBodyFeatureVariableClassDeclaration) varDec;
				title = createLink(currentBodyFeature.getVariableClassDeclaration(), varDec.getName());

				result.append(createBodyFeatureVariableClassDeclaration(currentBodyFeature, title, ""));
			} else {
				title = escape(iVar.getName());

				result.append(createVariableClassDeclaration(iVar, title, createAnchor(varDec)));
			}

		}

		result.append("</table>");

		write(result.toString());
	}

	private String createBodyFeatureVariableClassDeclaration(
		IBodyFeatureVariableClassDeclaration feature,
		String title,
		String anchor) {

		IVariableClassDeclaration declaration = feature.getVariableClassDeclaration();

		StringBuffer result = new StringBuffer();

		result.append(createVariableDeclaration(title, createAnchor(feature), declaration));

		return result.toString();
	}

	// wird von HTMLModelWriter.writeUserFeatureVariables(IUserFeatureClassDeclaration userFeat) verwendet
	public String createVariableClassDeclaration(IVariableClassDeclaration iVar) {
		return createVariableClassDeclaration(iVar, iVar.getName(), createAnchor(iVar));
	}

	// zum Rausschreiben von IFeatureVariableDecl und IBodyVariableDecl
	private String createVariableClassDeclaration(
		IVariableClassDeclaration iVar,
		String title,
		String anchor) {
		StringBuffer result = new StringBuffer();

		result.append(createVariableDeclaration(title, anchor, iVar));
		result.append(createNextValueFunctionDeclaration(iVar));
		result.append(createDefaultValueDeclaration(iVar));

		return result.toString();
	}

	private String createDefaultValueDeclaration(IVariableClassDeclaration declaration) {
		return "<tr><td style=\"padding-left:3em\">default value:</td><td>"
			+ HTMLFunctionsWriter.createFunctionCall(declaration.getDefaultValueFunctionCallDeclaration())
			+ "</td></tr>";
	}

	private String createNextValueFunctionDeclaration(IVariableClassDeclaration declaration) {
		if (declaration.getHasNextValueFunctionCallDeclaration()) {
			StringBuffer result = new StringBuffer();

			result.append("<tr><td style=\"padding-left:3em\">next value:</td><td>");
			result.append(
				HTMLFunctionsWriter.createFunctionCall(declaration.getNextValueFunctionCallDeclaration()));
			result.append("</td></tr>");

			return result.toString();
		} else {
			return "";
		}
	}

	private String createVariableDeclaration(
		String title,
		String anchor,
		IVariableClassDeclaration declaration) {

		return _emptyTableRow
			+ "<tr><td valign=\"top\">"
			+ anchor
			+ "<em>"
			+ title
			+ ":"
			+ "</em> "
			+ "<u>"
			+ escape(declaration.getSpecificType().getName())
			+ "</u>"
			+ "</td>"
			+ "<td valign=\"top\">"
			+ " (writeable: "
			+ declaration.isWriteable()
			+ ", external: "
			+ declaration.isExternal()
			+ ")"
			+ "</td></tr>";
	}
}
