/**
 * Created on 18.06.2002
 *
 * This class generates unique ids for anchoring and linking in the html-document.
 * It uses a hashmap to map objects to a value consisting of the object's class concatenated
 * with an integer resulting in the unique id for that object. 
 * The same object will get the same id.
 */
package de.uniwue.ki.mas.modelprint.auxiliary;

import java.util.HashMap;

import de.uniwue.ki.mas.modelprint.Base;
import de.uniwue.ki.mas.modelprint.exceptions.CreateNoLinkException;

/**
 * @author glueckselig
 */
public class UniqueAnchorGenerator extends HashMap {
	protected final static int initialCapacity = 100;
	protected HashMap idMapper;

	public UniqueAnchorGenerator() {
		super(initialCapacity);

		idMapper = new HashMap();
	}
	//#####################################
	//  zur Erzeugung von eindeutigen IDs
	//#####################################

	/** 
	 * Erzeugt fr das bergebene Object eine eindeutige ID.
	 * Gibt einen leeren String zurck, falls das bergebene Object null ist.
	 */
	public String getReference(Object anObject) throws CreateNoLinkException {

		try {
			if (!containsKey(anObject)) {
				put(anObject, createString(anObject));
			}

			return Base.escape((String)get(anObject));
		} catch (NullPointerException e) {
			throw new CreateNoLinkException();
		}
	}

	/**
	 * Method getReferenceCaughtExcpetion. Gets called by @see HTMLInterlinkageWriter#writeInterlinkage(IActivityGraphNodeDeclaration,IActivityGraphNodeDeclaration,String).
	 * This method just calls getReference() but catches occuring exceptions.
	 * @param anObject to get mapped to an id
	 * @return String the id corresponding with anObject
	 */
	public String getReferenceCaughtExcpetion(Object anObject) {
		try {
			return getReference(anObject);
		} catch (CreateNoLinkException e) {
			System.out.println("Caught Exception in UniqueAnchorGen.getAnchorCaughtExcpetion().");
			return "";
		}
	}

	/**
	 * Method createString. Gets invoked by @see getReference(Object) to generate the value mapped to the key anObject.
	 * @param anObject the key in the hashmap to which a value should be generated
	 * @return String the value is a unique String corresponding to anObject.
	 * @throws NullPointerException if anObject or the class of anObject is null.
	 */
	/**
	 *  throws NullPointerException if  anObject == null
	 */
	private String createString(Object anObject) throws NullPointerException {
		String className = anObject.getClass().getName();
		String classNameWithoutPackage = className.substring(className.lastIndexOf(".") + 1);

		int id = getUniqueId(anObject);

		if (classNameWithoutPackage.endsWith("Declaration"))
			return classNameWithoutPackage.substring(
				0,
				classNameWithoutPackage.length() - "Declaration".length())
				+ id;
		else
			return classNameWithoutPackage + id;
	}

	/**
	 * Method getUniqueId. Returns the integer that gets concatenated with the className of the object
	 * to create the value matched to the key object in the hashmap.
	 * @param anObject the key in the hashmap
	 * @return int the counter for objects of the same type.
	 */
	private final int getUniqueId(Object anObject) {
		Class aClass;
		try {
			aClass = anObject.getClass();
		} catch (NullPointerException e) {
			aClass = String.class;
		}

		if (idMapper.containsKey(aClass)) {
			Integer oldValue = (Integer)idMapper.get(aClass);
			int currentID = oldValue.intValue() + 1;

			idMapper.put(aClass, new Integer(currentID));
			return currentID;
		} else {
			idMapper.put(aClass, new Integer(0));
			return 0;
		}
	}
}
