package de.uniwue.ki.mas.refactoring;

import java.util.AbstractList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;

import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelBasicsDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;
import de.uniwue.ki.util.tree.IObjectTreeNode;

public abstract class AbstractRefactoring {

	public abstract boolean isApplicable();
	public abstract boolean execute(IRefactoringUI ui);

	protected String name;
	private Object refactoringSubject;
	private ISeSAmDeclaration seSAmDeclaration;

	protected AbstractRefactoring(ISeSAmDeclaration seSAmDeclaration, Object _refactoringSubject) {
		setRefactoringSubject(_refactoringSubject);
		setSeSAmDeclaration(seSAmDeclaration);
	}

	public String toString() { return this.name; }
	
	protected void setRefactoringSubject(Object refactoringSubject) {
		this.refactoringSubject = refactoringSubject;
	}

	protected Object getRefactoringSubject() {
		return refactoringSubject;
	}

	protected void setSeSAmDeclaration(ISeSAmDeclaration seSAmDeclaration) {
		this.seSAmDeclaration = seSAmDeclaration;
	}

	protected ISeSAmDeclaration getSeSAmDeclaration() {
		return seSAmDeclaration;
	}
	
	protected Collection collectFunctionCallDeclarations(AbstractObjectTreeNode node) {
		LinkedList calls = new LinkedList();
		
		// Recursion
		Enumeration e = node.children();
		while (e.hasMoreElements())
			calls.addAll(collectFunctionCallDeclarations((AbstractObjectTreeNode) e.nextElement()));
		
		// Check only IComposedFunctionCallInternDeclaration
		if (! (node.getUserObject() instanceof IComposedFunctionCallInternDeclaration))
			return calls;
		
		// Function calls to RefactoringSubject
		if (((IComposedFunctionCallInternDeclaration) node.getUserObject())
			.getSpecificFunctionDeclaration().equals(getRefactoringSubject()))
				calls.add(node.getUserObject());
	
		return calls;
	}

	protected Collection getUserFeatureFunctionDeclarationLists() {
		LinkedList declarationLists = new LinkedList();

		ISeSAmModelBasicsDeclaration basicDeclaration = getSeSAmDeclaration()
		.getSeSAmModelDeclaration()
		.getSeSAmModelBasicsDeclaration();
		
		Iterator featureClasses = basicDeclaration.getUserFeatureClassDeclarationListAndFactory()
			.getUserFeatureClassDeclarationList().iterator();
		while (featureClasses.hasNext())
			declarationLists.add(((IUserFeatureClassDeclaration) featureClasses.next()).
					getUserFunctionDeclarationListAndFactory().getUserFunctionDeclarationList());
		
		return declarationLists;
	}
	
	protected IDeclarationList getUserFunctionDeclarationListFor(IUserFunctionDeclaration function) {
		LinkedList declarationLists = new LinkedList();
		
		ISeSAmModelBasicsDeclaration basicDeclaration = getSeSAmDeclaration()
		.getSeSAmModelDeclaration()
		.getSeSAmModelBasicsDeclaration();
		
		declarationLists.addAll(getUserFeatureFunctionDeclarationLists());
		
		declarationLists.add(basicDeclaration.getUserFunctionDeclarationListAndFactory()
				.getUserFunctionDeclarationList());
	
		
		Iterator iter = declarationLists.iterator();
		while (iter.hasNext()) {
			Object object = iter.next();
			AbstractList list = (AbstractList) object;
			if (list.contains(function))
				return (IDeclarationList) list;
		}
		
		return null;
	}

	protected void updateUserFunctionCalls(Object treeNode, 
			IUserFunctionDeclaration oldDeclaration, IUserFunctionDeclaration newDeclaration) 
	{
		for (int i = 0; i < getSeSAmDeclaration().getSeSAmDeclarationTreeModel()
			.getChildCount(treeNode); i++)
				updateUserFunctionCalls(
						getSeSAmDeclaration().getSeSAmDeclarationTreeModel().getChild(treeNode, i),
						oldDeclaration, 
						newDeclaration);
	
		Object object = ((IObjectTreeNode) treeNode).getUserObject();
		
		if (!(object instanceof IComposedFunctionCallInternDeclaration))
			return;
		
		if (((IComposedFunctionCallInternDeclaration) object).getSpecificFunctionDeclaration() == oldDeclaration)
			Morpher.replaceUserFunctionCall((IComposedFunctionCallInternDeclaration) object, newDeclaration);
		
	}
}
