package de.uniwue.ki.mas.refactoring;

import java.util.Enumeration;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.composed.IComposedTypeAttribute;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.function.ICreateComposedTypeValueSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;

/**
 * @author Rainer Herrler
 */
public class AddUserTypeAttribute extends AbstractRefactoring {

	protected AddUserTypeAttribute(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Add User Type Attribute";
	}

	public boolean isApplicable() {
		return ((getRefactoringSubject() instanceof IUserTypeDeclaration) && ((IUserTypeDeclaration) getRefactoringSubject())
				.isComposedType());
	}

	public boolean execute(IRefactoringUI ui) {
		
		SpecificComposedType specificComposedType = null;
		
		if (getRefactoringSubject() instanceof IUserTypeDeclaration) {
			specificComposedType = ((IUserTypeDeclaration) getRefactoringSubject()).getSpecificComposedType();
		}
		
		if (getRefactoringSubject() instanceof SpecificComposedType) {
			specificComposedType = (SpecificComposedType)getRefactoringSubject();
		}			

		if (specificComposedType == null)
			return false;

		IComposedTypeAttribute newAttribute = specificComposedType
				.createComposedTypeAttribute();
		specificComposedType.addAttribute(newAttribute);

		changeFunctionCallDeclarations(
				(AbstractObjectTreeNode) getSeSAmDeclaration()
						.getSeSAmDeclarationTreeModel().getRoot(),
				newAttribute, specificComposedType);

		return true;
	}

	public static boolean execute(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		return (new AddUserTypeAttribute(seSAmDeclaration, _refactoringSubject))
				.execute(null);
	}

	/*
	 * Returns the number of calls that have been changed
	 */
	private int changeFunctionCallDeclarations(AbstractObjectTreeNode node,
			IComposedTypeAttribute addedAttribute,
			ISpecificComposedType composedType) {
		int changedCalls = 0;

		// Recursion
		Enumeration e = node.children();
		while (e.hasMoreElements())
			changedCalls += changeFunctionCallDeclarations(
					(AbstractObjectTreeNode) e.nextElement(), addedAttribute,
					composedType);

		// Check only IComposedFunctionCallInternDeclaration -> end of recursion
		if (!(node.getUserObject() instanceof IComposedFunctionCallInternDeclaration))
			return changedCalls;

		//remove child of createComposedTypeValue
		IComposedFunctionCallInternDeclaration funCallInternDecl = (IComposedFunctionCallInternDeclaration) node
				.getUserObject();
		ISpecificFunctionDeclaration specFunDecls = funCallInternDecl
				.getSpecificFunctionDeclaration();
		if ((specFunDecls instanceof ICreateComposedTypeValueSpecificFunctionDeclaration)
				&& ((ICreateComposedTypeValueSpecificFunctionDeclaration) specFunDecls)
						.getSpecificComposedType().equals(composedType)) {
			IFunctionCallInternDeclaration newChild = addedAttribute.getAttributeType()
			.createDefaultFunctionCallInternDeclaration();
			newChild.setParent(funCallInternDecl);
			funCallInternDecl.addChild(newChild);
			changedCalls++;
		}

		return changedCalls;
	}
}