package de.uniwue.ki.mas.refactoring;

import java.util.Enumeration;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.ComposedTypeAttribute;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.function.ICreateComposedTypeValueSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.composed.function.IGetComposedTypeValueSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;

/**
 * @author Rainer Herrler - This is no real refactoring since the original 
 * semantics does not persist.
 */
public class ChangeUserTypeAttributeRepair extends AbstractRefactoring {

	protected ChangeUserTypeAttributeRepair(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Change User Type Attribute";
	}

	public boolean isApplicable() {
		if (!(this.getRefactoringSubject() instanceof ComposedTypeAttribute))
			return false;

		return true;
	}

	/**
	 * prerequisite - The attribute given by getRefactoringSubject has been changed. Now
	 * all references and calls have been changed to meet the new syntax. The semantics
	 * will ne differnt from before.
	 */
	public boolean execute(IRefactoringUI ui) {
		ComposedTypeAttribute attribute = (ComposedTypeAttribute) getRefactoringSubject();
		ISpecificComposedType specificComposedType = getSpecificComposedType(
				getSeSAmDeclaration(), attribute);

		if (specificComposedType == null)
			return false; // this attribute cannot be removed
		if (!(specificComposedType instanceof SpecificComposedType))
			return false;

		int pos = CollectionsHelper.asArrayList(
				specificComposedType.getAttributes()).indexOf(attribute);

		changeFunctionCallDeclarations(
				(AbstractObjectTreeNode) getSeSAmDeclaration()
						.getSeSAmDeclarationTreeModel().getRoot(), pos,
				attribute, specificComposedType);

		return true;
	}

	public static boolean execute(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		return (new ChangeUserTypeAttributeRepair(seSAmDeclaration,
				_refactoringSubject)).execute(null);
	}

	public static ISpecificComposedType getSpecificComposedType(
			ISeSAmDeclaration sesamDecl, ComposedTypeAttribute attribute) {
		Iterator it = sesamDecl.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserTypeDeclarationListAndFactory().getDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IUserTypeDeclaration userType = (IUserTypeDeclaration) it.next();
			ISpecificComposedType element = userType.getSpecificComposedType();
			if (element != null)
				if (CollectionsHelper.asArrayList(element.getAttributes())
						.contains(attribute))
					return element;
		}
		return null;
	}

	/*
	 * Returns the number of calls that have been changed
	 */
	private int changeFunctionCallDeclarations(AbstractObjectTreeNode node,
			int index, ComposedTypeAttribute changedAttribute,
			ISpecificComposedType composedType) {
		int changedCalls = 0;

		// Recursion
		Enumeration e = node.children();
		while (e.hasMoreElements())
			changedCalls += changeFunctionCallDeclarations(
					(AbstractObjectTreeNode) e.nextElement(), index,
					changedAttribute, composedType);

		// Check only IComposedFunctionCallInternDeclaration -> end of recursion
		if (!(node.getUserObject() instanceof IComposedFunctionCallInternDeclaration))
			return changedCalls;

		//chnage child of createComposedTypeValue
		IComposedFunctionCallInternDeclaration funCallInternDecl = (IComposedFunctionCallInternDeclaration) node
				.getUserObject();
		ISpecificFunctionDeclaration specFunDecls = funCallInternDecl
				.getSpecificFunctionDeclaration();
		if ((specFunDecls instanceof ICreateComposedTypeValueSpecificFunctionDeclaration)
				&& ((ICreateComposedTypeValueSpecificFunctionDeclaration) specFunDecls)
						.getSpecificComposedType().equals(composedType)) {
			IFunctionCallInternDeclaration calllToChange = funCallInternDecl
					.getChild(index);
			//	look if return value of intern call is matching
			if (!calllToChange.getSpecificOutputType().isMoreSpecific(
					changedAttribute.getAttributeType())) {
				IFunctionCallInternDeclaration newCall = changedAttribute
						.getAttributeType()
						.createDefaultFunctionCallInternDeclaration();
				Morpher.replaceInternCall(calllToChange, newCall);
				Morpher.findCreator(newCall).setTODOFlag(true);
				changedCalls++;
			}
		}

		// set getComposedTypeValue to default value,except if new attribute
		// value is
		// more specific than the desired return value
		if ((specFunDecls instanceof IGetComposedTypeValueSpecificFunctionDeclaration)
				&& ((IGetComposedTypeValueSpecificFunctionDeclaration) specFunDecls)
						.getComposedTypeAttribute().equals(changedAttribute)) {
			
			//funCallInternDecl.getMostGeneralSpecificOutputType()
			ISpecificType oldAttributeType = getExpectedOutputType(funCallInternDecl);
			

			if (!changedAttribute.getAttributeType().isMoreSpecific(
						oldAttributeType)) {			
			IFunctionCallInternDeclaration newCall = oldAttributeType
					.createDefaultFunctionCallInternDeclaration();
			Morpher.replaceInternCall(funCallInternDecl, newCall);
			Morpher.findCreator(newCall).setTODOFlag(true);
			changedCalls++;}
		}

		return changedCalls;
	}

	/**
	 * output-Type of this call and expected input type by the parent might be inconsistent. This procedure
	 * looks at the parent and returns the expected input types that the call _should_ return.
	 */
	private ISpecificType getExpectedOutputType(IComposedFunctionCallInternDeclaration funCallInternDecl) {
		ISpecificType oldAttributeType;
		if (funCallInternDecl.getParent()!=null) {
		int i = funCallInternDecl.getParent().getIndexOfChild(funCallInternDecl);
		oldAttributeType = funCallInternDecl.getSpecificFunctionDeclaration().getSpecificInputTypeAt(i);
		}
		else {
			oldAttributeType = funCallInternDecl.getCreator().getMostGeneralOutputType();
			}
		return oldAttributeType;
	}
}