package de.uniwue.ki.mas.refactoring;
import java.util.Iterator;
import java.util.LinkedList;

import de.uniwue.ki.mas.basics.functionArgument.FunctionArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.ISpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.functionCall.AbstractFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.AtomicFunctionCallInternDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IAtomicFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
 * @author Michael Scherer & Rainer Herrler
 * 
 */
public class ExtractMethod extends AbstractRefactoring {

	protected ExtractMethod(ISeSAmDeclaration seSAmDeclaration, Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Extract Method";
	}
	
	static void collectFunctionArgumentDeclarations(
		IFunctionCallInternDeclaration internDecl,
		LinkedList found) 
	{
		/*
		Iterator atoms = new AtomicFunctionCallInternDeclarationFilter().execute(internDecl).iterator();
		while (atoms.hasNext()) { 
			IFunctionArgumentDeclaration argDecl =
				(IFunctionArgumentDeclaration) 
					((IAtomicFunctionCallInternDeclaration)atoms.next()).getArgumentDeclaration();
			if (!found.contains(argDecl))
				found.add(argDecl);
		}
*/		
		
		if (internDecl instanceof IComposedFunctionCallInternDeclaration) {
			IComposedFunctionCallInternDeclaration comFun = 
				(IComposedFunctionCallInternDeclaration)internDecl;
			Iterator it = comFun.getChildren();
			while (it.hasNext())
				collectFunctionArgumentDeclarations((IFunctionCallInternDeclaration)it.next(), found);
		} else if (internDecl instanceof IAtomicFunctionCallInternDeclaration) {
			IAtomicFunctionCallInternDeclaration atom = (IAtomicFunctionCallInternDeclaration)internDecl;
			if (atom.getArgumentDeclaration() instanceof IFunctionArgumentDeclaration) {
				IFunctionArgumentDeclaration argDecl =
					(IFunctionArgumentDeclaration)atom.getArgumentDeclaration();
				if (!found.contains(argDecl))
					found.add(argDecl);
			} else if (
				atom.getArgumentDeclaration().getObject()
					instanceof ISpecificFunctionDeclarationWithArgDeclBinds) 
			{
				// Funktion Hherer Ordnung als Argument
				ISpecificFunctionDeclarationWithArgDeclBinds funHighLevel =
					(ISpecificFunctionDeclarationWithArgDeclBinds) (atom
						.getArgumentDeclaration()
						.getObject());
				collectFunctionArgumentDeclarations(
					funHighLevel
						.getParametricFunctionCallDeclaration()
						.getFunctionCallDeclaration()
						.getFunctionCallInternDeclaration(),
					found);
			}
		}
	}

	/**
	 * Prft, ob das RefactoringSubject eine AtomicFunctionCallInternDeclaration
	 * ist, deren Argument auf eine Funktion verweist. Dies ist der Fall, wenn eine
	 * Funktion als Argument einer Funktion hherer Ordnung angegeben wird.
	 * 
	 * @return true, wenn auf eine Funktion verweisen wird, sonst false
	 */
	protected boolean isHigherOrderFunctionArgument() {
		try {
			return (
				((IAtomicFunctionCallInternDeclaration) getRefactoringSubject())
				.getArgumentDeclaration()
				.getObject() 
				instanceof ISpecificFunctionDeclarationWithArgDeclBinds
			);
		} catch (Exception ex) {
			return false;
		}
	}

	/**
	 * Prft, ob der bergebene Knoten eine AtomicFunctionCallDeclaration
	 * ist, deren Argument auf eine UserFunction verweist. Solche Knoten sollen
	 * nicht extrahiert werde, da dies eine neue Userfunction erstellen wrde, 
	 * die eine lediglich eine andere Userfunction aufruft.
	 * 
	 * @param  node  Knotenelement, das berprft werden soll
	 * @return true, wenn auf eine UserFunction verwiesen wird, false sonst
	 */
	protected boolean isUserFunctionDeclaration(Object node) {
		if (!(node instanceof IAtomicFunctionCallInternDeclaration))
			return false;
	
		try {
			return (
				((IAtomicFunctionCallInternDeclaration) node)
				.getArgumentDeclaration()
				.getObject()
				instanceof IUserFunctionDeclaration
			);
		} catch (Exception ex) {
			return false;
		}
	}

	public boolean isApplicable() {
		return (
				(getRefactoringSubject() instanceof AbstractFunctionCallInternDeclaration) && 
				!isUserFunctionDeclaration(getRefactoringSubject()) &&
				!isHigherOrderFunctionArgument()
		);
	}

	public boolean execute(IRefactoringUI ui) {
		return execute(ui, "undefined");
	}
		
	public boolean execute(IRefactoringUI ui, String defaultName) {
		return execute(ui, defaultName, (IFunctionCallInternDeclaration) getRefactoringSubject());
	}
	
	public boolean execute(IRefactoringUI ui, String defaultName, IFunctionCallInternDeclaration subject) {
		/*
		 * Name der neuen UserFunction abfragen Rainer: Noch nicht schoen: User
		 * Interface sollte hier Parameter direkt bergeben
		 */
		String newFunctionName = ui.getString("Please specify new function name", defaultName);
		if (newFunctionName == "")
			return false;
		
		/* Indentify important arguments of the extracted tree  */
		LinkedList oldArgs = new LinkedList();
		collectFunctionArgumentDeclarations(subject, oldArgs);

		IUserFunctionDeclaration extractedFunctionDeclaration = 
			createExtractedFunctionCallDeclaration(subject, oldArgs, newFunctionName);
		
		getSeSAmDeclaration()
			.getSeSAmModelDeclaration()
			.getSeSAmModelBasicsDeclaration()
			.getUserFunctionDeclarationListAndFactory()
			.getUserFunctionDeclarationList()
			.add(extractedFunctionDeclaration);

		replaceOldCallWithExtracedFunction(subject, oldArgs, extractedFunctionDeclaration);
		return true;
	}

	/**
	 * Den extrahierten Codezweig durch einen Aufruf der neuen UserFunction
	 * ersetzen
	 */
	private void replaceOldCallWithExtracedFunction(
		IFunctionCallInternDeclaration subject, 
		LinkedList oldArgs, 
		IUserFunctionDeclaration extractedFunctionDeclaration) 
	{
		
		IComposedFunctionCallInternDeclaration newCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				extractedFunctionDeclaration);

		Morpher.replaceInternCall(subject, newCall);

		Iterator iter = oldArgs.iterator();
		int index = 0;
		while (iter.hasNext()) {
			IFunctionArgumentDeclaration argDec = (IFunctionArgumentDeclaration)iter.next();
			Morpher.replaceInternCall(
				newCall.getChild(index),
				AtomicFunctionCallInternDeclarationFactory.createAtomicFunctionCallInternDeclaration(argDec));
			index++;
		}
	}

	/**
	 * This creates a User Function Declaration from the RefactoringSubject  
	 */
	private IUserFunctionDeclaration createExtractedFunctionCallDeclaration(
			IFunctionCallInternDeclaration subject, LinkedList oldArgs, String newName) 
	{
		/* creating a new UserFunction */
		IUserFunctionDeclaration extractedFunctionDeclaration =
			getSeSAmDeclaration()
				.getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserFunctionDeclarationListAndFactory()
				.getUserFunctionDeclarationFactory()
				.createUserFunctionDeclaration();

		//creating new FunktionsArgumentDeclarations
		LinkedList newArgs =
			CollectionsHelper.asLinkedList(CollectionsHelper.map(oldArgs.iterator(), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				IFunctionArgumentDeclaration oldArgDec = (IFunctionArgumentDeclaration)obj;
				IFunctionArgumentDeclaration newArgDec =
					FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
						oldArgDec.getName() + " copy",
						oldArgDec.getSpecificType());
				newArgDec.setDocumentation(oldArgDec.getDocumentation());

				return newArgDec;
			}
		}));
		
		//creating new substitute calls 
		LinkedList newCalls =
			CollectionsHelper.asLinkedList(CollectionsHelper.map(newArgs.iterator(), new IMapFunction() {
			public Object map(Object obj) throws Exception {
				IFunctionArgumentDeclaration argDec = (IFunctionArgumentDeclaration)obj;
				return AtomicFunctionCallInternDeclarationFactory.createAtomicFunctionCallInternDeclaration(
					argDec);
			}
		}));

		/* Defining function sigunature */
		extractedFunctionDeclaration.setSpecificOutputType(
			subject.getSpecificOutputType());
		extractedFunctionDeclaration
			.getParametricFunctionCallDeclaration()
			.getFunctionArgumentDeclarationList()
			.addAll(newArgs);

		/* defining the function code */
		AbstractFunctionCallInternDeclaration functionBody =
			(AbstractFunctionCallInternDeclaration) subject.copy(null);
		functionBody.setParent(null);
		extractedFunctionDeclaration
			.getParametricFunctionCallDeclaration()
			.getFunctionCallDeclaration()
			.setFunctionCallInternDeclaration(functionBody);

		Morpher.morph(oldArgs, newCalls, functionBody);
		
		/*
		 * create dokumentation
		 */
		extractedFunctionDeclaration.setDocumentation(
				"Extracted from "
				+ (Morpher.findCreator(subject)));
		extractedFunctionDeclaration.setTODOFlag(true);
		extractedFunctionDeclaration.setShortDescription(newName);
		
		return extractedFunctionDeclaration;
	}
}
