package de.uniwue.ki.mas.refactoring;

import java.util.Iterator;
import java.util.LinkedList;

import de.uniwue.ki.mas.basics.functionArgument.FunctionArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionCall.AtomicFunctionCallInternDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.functions.IMapFunction;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;

/**
 * @author Michael Scherer
 */
public class IntroduceArgument extends AbstractRefactoring {

	protected IntroduceArgument(ISeSAmDeclaration seSAmDeclaration, Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Introduce New Argument";
	}

	public boolean isApplicable() {
		return (getRefactoringSubject() instanceof IUserFunctionDeclaration);
	}

	public boolean execute(IRefactoringUI ui) {
		ArgumentInfo argInfo = ui.getNewArgument(ArgumentInfo.getDefault());
		return execute(argInfo);
	}
	
	public boolean execute(ArgumentInfo argInfo) {
		if (argInfo == null)
			return false;
		
		IUserFunctionDeclaration oldDeclaration = (IUserFunctionDeclaration) getRefactoringSubject();
		IUserFunctionDeclaration newDeclaration = (IUserFunctionDeclaration) oldDeclaration.copy(null);

		addArgument(newDeclaration, argInfo);
		
		updateFunctionCalls(newDeclaration, argInfo);
				
		Morpher.replaceFunctionDeclaration(
				getUserFunctionDeclarationListFor(oldDeclaration),
				oldDeclaration,
				newDeclaration);
		
		return true;
	}
	
	protected void addArgument(IUserFunctionDeclaration declaration, ArgumentInfo argInfo) {
		IFunctionArgumentDeclaration newArgument = FunctionArgumentDeclarationFactory.getInstance().
		createArgumentDeclaration(argInfo.getName(), argInfo.getType());
	
		declaration.getFunctionArgumentDeclarationList().add(newArgument);
	}

	protected void updateFunctionCalls(IUserFunctionDeclaration newDeclaration, 
			ArgumentInfo argInfo)
	{
		Iterator decls = collectFunctionCallDeclarations(
				(AbstractObjectTreeNode) getSeSAmDeclaration()
					.getSeSAmDeclarationTreeModel().getRoot()).iterator();
		
		while(decls.hasNext()) {
			IComposedFunctionCallInternDeclaration oldFunctionCall = 
				(IComposedFunctionCallInternDeclaration) decls.next();

			// Copy all old arguments			
			LinkedList newArgumentList = CollectionsHelper.asLinkedList(
					CollectionsHelper.map(
						oldFunctionCall.getChildren(), 
						new IMapFunction () {
							public Object map(Object obj) throws Exception {
								return ((ICopyable) obj).copy(null);
							}
						}
					));
			
			// Add new argument
			newArgumentList.add(AtomicFunctionCallInternDeclarationFactory
				.createAtomicFunctionCallInternDeclaration(
					ArgumentDeclarationFactory.getInstance()
					.createArgumentDeclaration(
						argInfo.getType(), 
						argInfo.getType().getArgumentDeclarationDefaultValue()
					)));

			// Replace old with new function call
			IComposedFunctionCallInternDeclaration newFunctionCall =
				FunctionCallDeclarationFactory.getInstance().
				createComposedFunctionCallInternDeclaration(
					null,
					newDeclaration,
					newArgumentList
				);			
			Morpher.replaceInternCall(oldFunctionCall, newFunctionCall);
		}
	}
}
