/*
 * Rainer Herrler
 * 
 * this morphes a function call using a translation table from
 * argumentDeclarations to functionCallInternCallDeclarations to replace.
 */
package de.uniwue.ki.mas.refactoring;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.ISpecificFunctionDeclarationWithArgDeclBinds;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IAtomicFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;

public class Morpher {
	private final List _argumentDeclarations;
	private final List _oldArgumentValueTrees;

	
	public static void morph(List argumentDeclarations, List oldArgumentValueTrees, IFunctionCallInternDeclaration funDecl) {
		Morpher morpher = new Morpher(argumentDeclarations, oldArgumentValueTrees);
		morpher.morph(funDecl);
	}
		
	public Morpher(List argumentDeclarations, List oldArgumentValueTrees) {
		_argumentDeclarations = argumentDeclarations;
		_oldArgumentValueTrees = oldArgumentValueTrees;
	}

	public void morph(IFunctionCallInternDeclaration funDecl) {
		if (funDecl instanceof IComposedFunctionCallInternDeclaration) {
			IComposedFunctionCallInternDeclaration comFun = (IComposedFunctionCallInternDeclaration)funDecl;
			Iterator it = comFun.getChildren();
			while (it.hasNext())
				morph((IFunctionCallInternDeclaration)it.next());
		} else if (funDecl instanceof IAtomicFunctionCallInternDeclaration) {
			IAtomicFunctionCallInternDeclaration atom = (IAtomicFunctionCallInternDeclaration)funDecl;
			if (atom.getArgumentDeclaration() instanceof IFunctionArgumentDeclaration) {
				// Beliebiges Argument von Oben
				int pos = _argumentDeclarations.indexOf(atom.getArgumentDeclaration());
				if (pos != -1) {
					Morpher.replaceInternCall(
						atom,
						(IFunctionCallInternDeclaration) ((ICopyable)_oldArgumentValueTrees.get(pos)).copy(
							new HashMap()));
				}
			} else if (
				atom.getArgumentDeclaration().getObject()
					instanceof ISpecificFunctionDeclarationWithArgDeclBinds) {
				// Funktion Hherer Ordnung als Argument
				ISpecificFunctionDeclarationWithArgDeclBinds funHighLevel =
					(ISpecificFunctionDeclarationWithArgDeclBinds) (atom
						.getArgumentDeclaration()
						.getObject());
				morph(
					funHighLevel
						.getParametricFunctionCallDeclaration()
						.getFunctionCallDeclaration()
						.getFunctionCallInternDeclaration());
			}
		}
	}

	static void replaceUserFunctionCall(IComposedFunctionCallInternDeclaration oldCall, 
			IUserFunctionDeclaration newTarget) 
	{
		IFunctionCallInternDeclaration newCall = FunctionCallDeclarationFactory.getInstance()
			.createComposedFunctionCallInternDeclaration(oldCall.getParent(), newTarget, 
				(IComposedFunctionCallInternDeclaration) oldCall.copy(null));
		replaceInternCall(oldCall, newCall);
	}

	static void replaceInternCall(
			IFunctionCallInternDeclaration declToModify,
		IFunctionCallInternDeclaration funCopy) 
	{
		IFunctionCallDeclaration functionCallDeclaration = Morpher.findCreator(declToModify);
		if (functionCallDeclaration != null) {
			funCopy.setParent(declToModify.getParent());
			functionCallDeclaration.replaceFunctionCallInternDeclarationTo(funCopy, declToModify);
		} else
			Assert.fail("Replacing Function Call Failed!");
	}
	
	public static Collection getParentNode(AbstractObjectTreeNode node, IDeclaration child) {
		LinkedList foundItems = new LinkedList();
		
		Iterator iter = CollectionsHelper.asIterator(node.children());
		while (iter.hasNext())
				foundItems.add(getParentNode((AbstractObjectTreeNode) iter.next(), child));
		
		if (node.getUserObject() == child)
			foundItems.add(node.getParent());
		
		return foundItems;
	}
	
	public static void replaceFunctionDeclaration(IDeclarationList list,
			IFunctionDeclaration oldDeclaration, IFunctionDeclaration newDeclaration) 
	{
		replaceFunctionDeclaration(list, list, oldDeclaration, newDeclaration);
	}

	public static void replaceFunctionDeclaration(
			IDeclarationList oldDeclarationList, IDeclarationList newDeclarationList,
			IFunctionDeclaration oldDeclaration, IFunctionDeclaration newDeclaration)
	{
		Assert.assertTrue(oldDeclarationList.contains(oldDeclaration));
		Assert.assertFalse(newDeclarationList.contains(newDeclaration));
		
		if (oldDeclarationList == newDeclarationList)
			// Preserve index if replacing within the same DeclarationList 
			newDeclarationList.add(oldDeclarationList.indexOf(oldDeclaration), newDeclaration);
		else
			newDeclarationList.add(newDeclaration);
		
		oldDeclarationList.remove(oldDeclaration);
	}

	static IFunctionCallDeclaration findCreator(IFunctionCallInternDeclaration finIntern) {
		if (finIntern.getCreator() != null)
			return finIntern.getCreator();
		if (finIntern.getParent() != null)
			return findCreator(finIntern.getParent());

		return null;
	}
}