package de.uniwue.ki.mas.refactoring;

import java.util.ArrayList;

import de.uniwue.ki.mas.model.ISeSAmDeclaration;

public class RefactoringFactory {

	protected static RefactoringFactory instance;
	
	protected RefactoringFactory() {
		super();
	}

	public static RefactoringFactory getInstance() {
		if (instance == null) instance = new RefactoringFactory();
		return instance;
	}
	
	private ArrayList getAllRefactorings(ISeSAmDeclaration declaration, Object _refactoringSubject) {
		ArrayList refactorings = new ArrayList();
		
		// Add Refactorings here
		refactorings.add(new ExtractMethod(declaration, _refactoringSubject));
		refactorings.add(new ExtractHigherOrderFunctionArgument(declaration, _refactoringSubject));
		refactorings.add(new InlineMethod(declaration, _refactoringSubject));
		refactorings.add(new IntroduceArgument(declaration, _refactoringSubject));
		refactorings.add(new RemoveUnusedArguments(declaration, _refactoringSubject));
		refactorings.add(new RemoveUserTypeAttribute(declaration, _refactoringSubject));
		refactorings.add(new AddUserTypeAttribute(declaration, _refactoringSubject));
		refactorings.add(new MoveToUserFunctions(declaration, _refactoringSubject));
		refactorings.add(new MoveToUserFeature(declaration, _refactoringSubject));
		refactorings.add(new SplitActivityNode(declaration, _refactoringSubject));
		
		return refactorings;
	}

	private ArrayList filterApplicableRefactorings(ArrayList refactorings) {
		int i = refactorings.size();
		while (i-- > 0)
			if (!((AbstractRefactoring) refactorings.get(i)).isApplicable())
				refactorings.remove(i);
		return refactorings;
	}
		
	public ArrayList getApplicableRefactorings(ISeSAmDeclaration declaration, Object _refactoringSubject) {
		return filterApplicableRefactorings(getAllRefactorings(declaration, _refactoringSubject));
	}
}
