package de.uniwue.ki.mas.refactoring;

import java.util.Enumeration;
import java.util.Iterator;

import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.composed.ComposedTypeAttribute;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.SpecificComposedType;
import de.uniwue.ki.mas.basics.type.composed.function.ICreateComposedTypeValueSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.composed.function.IGetComposedTypeValueSpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.tree.AbstractObjectTreeNode;

/**
 * @author Rainer Herrler
 */
public class RemoveUserTypeAttribute extends AbstractRefactoring {

	protected RemoveUserTypeAttribute(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Remove User Type Attribute";
	}

	public boolean isApplicable() {
		if (!(this.getRefactoringSubject() instanceof ComposedTypeAttribute))
			return false;

		return true;
	}

	public boolean execute(IRefactoringUI ui) {
		ComposedTypeAttribute attribute = (ComposedTypeAttribute) getRefactoringSubject();
		ISpecificComposedType specificComposedType = getSpecificComposedType(attribute);

		if (specificComposedType == null)
			return false; // this attribute cannot be removed
		if (!(specificComposedType instanceof SpecificComposedType))
			return false;

		int pos = CollectionsHelper.asArrayList(
				specificComposedType.getAttributes()).indexOf(attribute);
		((SpecificComposedType) specificComposedType)
				.removeAttribute(attribute);

		changeFunctionCallDeclarations(
				(AbstractObjectTreeNode) getSeSAmDeclaration()
						.getSeSAmDeclarationTreeModel().getRoot(), pos,
				attribute, specificComposedType);

		return true;
	}

	public static boolean execute(ISeSAmDeclaration seSAmDeclaration,
			Object _refactoringSubject) {
		return (new RemoveUserTypeAttribute(seSAmDeclaration,
				_refactoringSubject)).execute(null);
	}

	private ISpecificComposedType getSpecificComposedType(
			ComposedTypeAttribute attribute) {
		Iterator it = getSeSAmDeclaration().getSeSAmModelDeclaration()
				.getSeSAmModelBasicsDeclaration()
				.getUserTypeDeclarationListAndFactory().getDeclarationList()
				.iterator();

		while (it.hasNext()) {
			IUserTypeDeclaration userType = (IUserTypeDeclaration) it.next();
			ISpecificComposedType element = userType.getSpecificComposedType();
			if (element != null)
				if (CollectionsHelper.asArrayList(element.getAttributes())
						.contains(attribute))
					return element;
		}
		return null;
	}

	/*
	 * Returns the number of calls that have been changed
	 */
	private int changeFunctionCallDeclarations(AbstractObjectTreeNode node,
			int removeindex, ComposedTypeAttribute attributeToRemove,
			ISpecificComposedType composedType) {
		int changedCalls = 0;

		// Recursion
		Enumeration e = node.children();
		while (e.hasMoreElements())
			changedCalls += changeFunctionCallDeclarations(
					(AbstractObjectTreeNode) e.nextElement(), removeindex,
					attributeToRemove, composedType);

		// Check only IComposedFunctionCallInternDeclaration -> end of recursion
		if (!(node.getUserObject() instanceof IComposedFunctionCallInternDeclaration))
			return changedCalls;

		//remove child of createComposedTypeValue
		IComposedFunctionCallInternDeclaration funCallInternDecl = (IComposedFunctionCallInternDeclaration) node
				.getUserObject();
		ISpecificFunctionDeclaration specFunDecls = funCallInternDecl
				.getSpecificFunctionDeclaration();
		if ((specFunDecls instanceof ICreateComposedTypeValueSpecificFunctionDeclaration)
				&& ((ICreateComposedTypeValueSpecificFunctionDeclaration) specFunDecls)
						.getSpecificComposedType().equals(composedType)) {
			funCallInternDecl.removeChild(removeindex);
			changedCalls++;
		}

		//set getComposedTypeValue to default value
		if ((specFunDecls instanceof IGetComposedTypeValueSpecificFunctionDeclaration)
				&& ((IGetComposedTypeValueSpecificFunctionDeclaration) specFunDecls)
						.getComposedTypeAttribute().equals(attributeToRemove)) {
			IFunctionCallInternDeclaration newCall = funCallInternDecl.getSpecificOutputType().createDefaultFunctionCallInternDeclaration();
			Morpher.replaceInternCall(funCallInternDecl,newCall);
			Morpher.findCreator(newCall).setTODOFlag(true);
			changedCalls++;
		}

		return changedCalls;
	}
}