package de.uniwue.ki.mas.refactoring;

import java.util.Iterator;

import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.ActionActivityGraphLeafDeclarationFactory;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActionActivityGraphLeafDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphDeclaration;
import de.uniwue.ki.mas.behaviour.rule.ActivityGraphRuleDeclarationFactory;
import de.uniwue.ki.mas.behaviour.rule.IActivityGraphEdgeDeclaration;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;

/**
 * @author Michael Scherer
 */
public class SplitActivityNode extends AbstractRefactoring {

	public SplitActivityNode(ISeSAmDeclaration seSAmDeclaration, Object _refactoringSubject) {
		super(seSAmDeclaration, _refactoringSubject);
		this.name = "Split Entry Actions";
	}

	public boolean isApplicable() {
		if (!(getRefactoringSubject() instanceof IActionActivityGraphLeafDeclaration))
			return false;
		
		IActionActivityGraphLeafDeclaration node = 
			(IActionActivityGraphLeafDeclaration) getRefactoringSubject();
		
		return (doContainEntryActions(node) && 
					(doContainActions(node) || doContainExitActions(node)));
	}

	public boolean execute(IRefactoringUI ui) {
		return execute((IActionActivityGraphLeafDeclaration) getRefactoringSubject());
	}

	public boolean execute(IActionActivityGraphLeafDeclaration subject) {
		IActivityGraphDeclaration activityGraphDeclaration = subject.getActivityGraphDeclaration();
		IActionActivityGraphLeafDeclaration newNode = createNewActivityNode(activityGraphDeclaration,
				"Entry Actions of '" + subject.getShortDescription() + "'");
		
		moveEntryActionsToNewNode(subject, newNode);
		remapIncomingEdgesToNewNode(subject, newNode);
		connect(newNode, subject);

		return true;
	}
	
	private IActionActivityGraphLeafDeclaration createNewActivityNode(
			IActivityGraphDeclaration activityGraphDeclaration,
			String title) 
	{
		IActionActivityGraphLeafDeclaration newActionAtomicActivityDeclaration = 
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
					activityGraphDeclaration);

		newActionAtomicActivityDeclaration.setShortDescription(title);
		newActionAtomicActivityDeclaration.setInstantly(true);
		activityGraphDeclaration.addActivityGraphNodeDeclaration(newActionAtomicActivityDeclaration);
		
		return newActionAtomicActivityDeclaration;
	}

	private void moveEntryActionsToNewNode(IActionActivityGraphLeafDeclaration subject, 
			IActionActivityGraphLeafDeclaration newNode) 
	{
		IDeclarationList entryFunctionDeclarationListOfSubject = 
			subject.getEntryFunctionActionCallDeclarationListAndFactory().getDeclarationList();
		
		entryFunctionDeclarationListOfSubject.copyInto(
				newNode.getFunctionActionCallDeclarationListAndFactory().getDeclarationList(), null);
		entryFunctionDeclarationListOfSubject.clear();
	}

	private void remapIncomingEdgesToNewNode(IActionActivityGraphLeafDeclaration subject, 
			IActionActivityGraphLeafDeclaration newNode) 
	{
		Iterator edges = subject.getActivityGraphDeclaration().getEdges();
		
		while(edges.hasNext()) {
			IActivityGraphEdgeDeclaration edge = (IActivityGraphEdgeDeclaration) edges.next();
			if (edge.getToActivityGraphNodeDeclaration() == subject)
				remap(edge, subject, newNode);
		}
	}

	private void connect(IActionActivityGraphLeafDeclaration fromNode,
			IActionActivityGraphLeafDeclaration toNode) 
	{
		IActivityGraphEdgeDeclaration connection = 
			ActivityGraphRuleDeclarationFactory.createActivityGraphRuleDeclaration(fromNode, toNode);
		fromNode.getActivityGraphDeclaration().addEdge(connection);
	}
	private void remap(IActivityGraphEdgeDeclaration oldEdge, IActionActivityGraphLeafDeclaration subject,
			IActionActivityGraphLeafDeclaration newNode) 
	{		
		IActivityGraphEdgeDeclaration newEdge = (IActivityGraphEdgeDeclaration) oldEdge.copy(null);
		newEdge.setToActivityGraphNodeDeclaration(newNode);
		subject.getActivityGraphDeclaration().addEdge(newEdge);
		subject.getActivityGraphDeclaration().removeEdge(oldEdge);
	}
	
	private boolean doContainEntryActions(IActionActivityGraphLeafDeclaration node) {
		return (! node.getEntryFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList().isEmpty());
	}

	private boolean doContainActions(IActionActivityGraphLeafDeclaration node) {
		return (! node.getFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList().isEmpty());
	}

	private boolean doContainExitActions(IActionActivityGraphLeafDeclaration node) {
		return (! node.getExitFunctionActionCallDeclarationListAndFactory()
				.getFunctionActionCallDeclarationList().isEmpty());
	}
}
