package de.uniwue.ki.mas.sim.analysis;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: (15.10.2001 17:08:42)
 * @author Christoph Oechslein
 */
public abstract class AbstractAnalysis extends AbstractNamed implements IAnalysis {

	private final int _deltaTicks;
	private final int _startTick;
	private final IFunction _decisionFunction;

	private final IFunction[] _valueFunctionList;
	private final String[] _nameList;

	private final IAnalysisDeclaration _analysisDeclaration;

	public AbstractAnalysis(
		IAnalysisDeclaration analysisDeclaration,
		ICreateFunctionArgs createFunctionArgs,
		String[] nameFunctions,
		IFunction[] valueFunctions)
		throws ModelCompilingException {

		super();

		if (Assert.ASSERTION_ON) {
			Assert.assertEquals(nameFunctions.length, valueFunctions.length);
		}

		_analysisDeclaration = analysisDeclaration;

		IAnalysisDeltaSettings analysisDeltaSettings =
			analysisDeclaration.getAnalysisDeltaSettings();

		if (analysisDeltaSettings.isUsingDecisionFunctionCallDeclaration()) {
			_decisionFunction =
				analysisDeltaSettings.getDecisionFunctionCallDeclaration().createFunction(
					createFunctionArgs);
			_deltaTicks = -1;
			_startTick = -1;

		} else {
			_decisionFunction = null;
			_deltaTicks = analysisDeltaSettings.getDeltaTicks();
			_startTick = analysisDeltaSettings.getStartTick();
		}

		_nameList = nameFunctions;
		_valueFunctionList = valueFunctions;
	}

	protected final IFunction getDecisionFunction() {
		return _decisionFunction;
	}

	protected final int getStartTick() {
		return _startTick;
	}

	public final IAnalysisDeclaration getAnalysisDeclaration() {
		return _analysisDeclaration;
	}

	public void remove() {
	}

	protected final int getDeltaTicks() {
		return _deltaTicks;
	}

	public String getName() {
		return getAnalysisDeclaration().getName();
	}

	final String[] getNameArray() {
		return _nameList;
	}

	protected boolean timeToAnalyse(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		if (isUsingDecisionFunction()) {
			return getDecisionFunction().executeForBoolean(simulationExecuteFunctionArgs);

		} else {
			final int currentTicks =
				simulationExecuteFunctionArgs
					.getSimulation()
					.getSituation()
					.getTimeDurationInTicks();

			return (currentTicks >= getStartTick())
				&& (((currentTicks - getStartTick()) % getDeltaTicks()) == 0) ;
		}
	}

	protected boolean isUsingDecisionFunction() {
		return getDecisionFunction() != null;
	}

	protected abstract void analyseIntern(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException;

	public void analyse(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		if (timeToAnalyse(simulationExecuteFunctionArgs)) {
			analyseIntern(simulationExecuteFunctionArgs);
		}
	}

	final IFunction[] getValueFunctionArray() {
		return _valueFunctionList;
	}
}