package de.uniwue.ki.mas.sim.analysis;
import java.util.Iterator;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.function.iterator.EmptyIteratorSimulationRuntimeException;
import de.uniwue.ki.mas.basics.type.DoubleType;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;
import de.uniwue.ki.util.name.AbstractNamed;

/**
 * Creation date: (15.10.2001 17:08:42)
 * @author Christoph Oechslein
 */
public abstract class AbstractValueAnalysis extends AbstractAnalysis {
	private final Object[] _currentValues;
	protected final ISpecificType[] _analyseValueTypeArray;

	public AbstractValueAnalysis(
		IAnalysisDeclaration analysisDeclaration,
		ICreateFunctionArgs createFunctionArgs,
		String[] nameList,
		IFunction[] valueFunctionList,
		ISpecificType[] analyseValueTypeArray)
		throws ModelCompilingException {

		super(analysisDeclaration, createFunctionArgs, nameList, valueFunctionList);

		Assert.assertEquals(analyseValueTypeArray.length, nameList.length);

		_currentValues = new Object[valueFunctionList.length];
		_analyseValueTypeArray = analyseValueTypeArray;
	}

	protected void analyseIntern(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {
		updateValueArray(simulationExecuteFunctionArgs);
	}

	protected final int getCurrentValuesCount() {
		return _currentValues.length;
	}

	public void init(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {
		if (timeToAnalyse(simulationExecuteFunctionArgs))
			updateValueArray(simulationExecuteFunctionArgs);
	}

	private void updateValueArray(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		for (int i = 0; i < _currentValues.length; i++) {
			IFunction function = getValueFunctionArray()[i];
			try {
				_currentValues[i] = function.execute(simulationExecuteFunctionArgs);

			} catch (EmptyIteratorSimulationRuntimeException exception) {
				_currentValues[i] = null;
			}
		}
	}

	protected String getNameForEntry(int index) {
		return getNameForEntry(_currentValues[index], _analyseValueTypeArray[index]);
	}

	private String getNameForEntry(Object result, ISpecificType type) {
		StringBuffer buff = new StringBuffer();

		if (DoubleType.getInstance().equals(type)) {
			Number value = (Number) result;

			if ((value instanceof Double) && ((Double) value).isNaN()) {
				buff.append("");
			} else {
				DoubleType.getInstance().appendNameForArgumentObject(buff, result, 0, 30);
			}

		} else if (result instanceof Iterator) {
			AbstractNamed.append((Iterator) result, buff, 0, 30);

		} else {
			type.appendNameForArgumentObject(buff, result, 0, 30);
		}

		return buff.toString();
	}}