package de.uniwue.ki.mas.sim.analysis;
import java.util.*;

import org.jfree.data.*;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (15.10.2001 17:08:42)
 * @author Christoph Oechslein
 */
public class BlockChartAnalysis extends AbstractAnalysis implements CategoryDataset {
	protected List _listeners;

	private final DefaultCategoryDataset _defaultCategoryDataset = new DefaultCategoryDataset();

	private static final String KEY = "Value";

	public BlockChartAnalysis(
		IAnalysisDeclaration analysisDeclaration,
		ICreateFunctionArgs createFunctionArgs,
		String[] nameList,
		IFunction[] valueFunctionList)
		throws ModelCompilingException {

		super(analysisDeclaration, createFunctionArgs, nameList, valueFunctionList);

		for (int i = 0; i < getNameArray().length; i++) {
			_defaultCategoryDataset.setValue(0, getNameArray()[i], KEY);
		}
	}

	/**
	 * Registers an object for notification of changes to the dataset.
	 * @param listener The object to register.
	 */
	public void addChangeListener(DatasetChangeListener listener) {
		if (_listeners == null) {
			_listeners = new LinkedList();
		}

		_listeners.add(listener);
	}

	protected void analyseIntern(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		for (int i = 0; i < getValueFunctionArray().length; i++) {
			IFunction function = getValueFunctionArray()[i];

			_defaultCategoryDataset.setValue(
				function.executeForDouble(simulationExecuteFunctionArgs),
				getNameArray()[i],
				KEY);
		}

		fireDatasetChanged();
	}

	/**
	 * Notifies all registered listeners that the dataset has changed.
	 */
	protected void fireDatasetChanged() {
		notifyListeners(new DatasetChangeEvent(this, this));
	}

	/**
	 * Returns the legend item labels.
	 * @return The legend item labels.
	 */
	//	public String[] getLegendItemLabels() {
	//
	//		int seriesCount = this.getSeriesCount();
	//		String[] labels = new String[seriesCount];
	//		for (int i = 0; i < seriesCount; i++) {
	//			labels[i] = this.getSeriesName(i);
	//		}
	//		return labels;
	//
	//	}

	public void init(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {
		for (int i = 0; i < getNameArray().length; i++) {
			_defaultCategoryDataset.setValue(0, getNameArray()[i], KEY);
		}
	}

	/**
	 * Notifies all registered listeners that the dataset has changed.
	 * @param event Contains information about the event that triggered the notification.
	 */
	protected void notifyListeners(DatasetChangeEvent event) {
		if (_listeners != null) {
			Iterator iterator = _listeners.iterator();
			while (iterator.hasNext()) {
				DatasetChangeListener listener = (DatasetChangeListener) iterator.next();
				listener.datasetChanged(event);
			}
		}
	}

	/**
	 * Deregisters an object for notification of changes to the dataset.
	 * @param listener The object to deregister.
	 */
	public void removeChangeListener(DatasetChangeListener listener) {
		_listeners.remove(listener);
	}

	public Comparable getRowKey(int row) {
		return _defaultCategoryDataset.getRowKey(row);
	}

	public int getRowIndex(Comparable key) {
		return _defaultCategoryDataset.getRowIndex(key);
	}

	public List getRowKeys() {
		return _defaultCategoryDataset.getRowKeys();
	}

	public Comparable getColumnKey(int column) {
		return _defaultCategoryDataset.getColumnKey(column);
	}

	public int getColumnIndex(Comparable key) {
		return _defaultCategoryDataset.getColumnIndex(key);
	}

	public List getColumnKeys() {
		return _defaultCategoryDataset.getColumnKeys();
	}

	public Number getValue(Comparable rowKey, Comparable columnKey) {
		return _defaultCategoryDataset.getValue(rowKey, columnKey);
	}

	public int getRowCount() {
		return _defaultCategoryDataset.getRowCount();
	}

	public int getColumnCount() {
		return _defaultCategoryDataset.getColumnCount();
	}

	public Number getValue(int row, int column) {
		return _defaultCategoryDataset.getValue(row, column);
	}

	public DatasetGroup getGroup() {
		return _defaultCategoryDataset.getGroup();
	}

	public void setGroup(DatasetGroup group) {
		_defaultCategoryDataset.setGroup(group);
	}
}