package de.uniwue.ki.mas.sim.analysis;
import java.io.*;
import java.util.*;
import java.util.prefs.Preferences;

import junit.framework.Assert;
import edu.oswego.cs.dl.util.concurrent.SyncSet;
import edu.oswego.cs.dl.util.concurrent.WriterPreferenceReadWriteLock;

/**
 * Creation date: 04.02.2003, 13:30:42
 * @author oechslein
 */
public final class CSVAnalysisSaveAllFinished {

	private static final class SaveAllFinishedThread extends Thread implements Runnable {
		public SaveAllFinishedThread() {
			super("Save Finished Analysis Lists");
		}

		public void run() {
			cleanCreatedDirectories();
			
			saveSet(All_FINISHED, PREFS_ALL_FINISHED_NAME);
			saveSet(All_STARTED, PREFS_ALL_STARTED_NAME);
			saveSet(All_DIRECTORIES, PREFS_ALL_DIRECTORIES_NAME);
		}

		private void saveSet(SyncSet set, String name) {
			try {
				if (!set.isEmpty()) {
					File tempFile = File.createTempFile(name, "analysis.tmp");
					getPrefs().put(name, tempFile.toString());
					ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(tempFile));
					oos.writeObject(new ArrayList(set));
					oos.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	protected static final SyncSet All_FINISHED;
	protected static final SyncSet All_STARTED;
	protected static final SyncSet All_DIRECTORIES;
	protected static final String PREFS_ALL_FINISHED_NAME = "AllFinished";
	protected static final String PREFS_ALL_STARTED_NAME = "AllStarted";
	protected static final String PREFS_ALL_DIRECTORIES_NAME = "AllDirectories";

	static {
		Runtime.getRuntime().addShutdownHook(new SaveAllFinishedThread());

		All_FINISHED = loadSet(PREFS_ALL_FINISHED_NAME);
		All_STARTED = loadSet(PREFS_ALL_STARTED_NAME);
		All_DIRECTORIES = loadSet(PREFS_ALL_DIRECTORIES_NAME);
	}

	private static SyncSet loadSet(String name) {
		SyncSet syncSet = new SyncSet(new HashSet(), new WriterPreferenceReadWriteLock());
		String fileString = getPrefs().get(name, null);
		if ((fileString != null) && (new File(fileString).canRead())) {
			try {
				ObjectInputStream ois = new ObjectInputStream(new FileInputStream(fileString));
				Collection newCol = (Collection) ois.readObject();
				syncSet.addAll(newCol);
				ois.close();
			} catch (IOException e) {
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			}
			getPrefs().remove(name);
		}

		return syncSet;
	}

	protected static final Preferences getPrefs() {
		return Preferences.userNodeForPackage(CSVAnalysis.class);
	}

	public synchronized static void addToAllFinishedList(File file) {
		All_FINISHED.add(file);
		All_STARTED.remove(file);
	}

	public static Iterator getAllFinishedFiles() {
		return All_FINISHED.iterator();
	}

	public static void removeFinishedFile(File file) {
		All_FINISHED.remove(file);
	}

	public static void addToAllStartedList(File file) {
		All_STARTED.add(file);
	}

	public static Iterator getAllStartedFiles() {
		return All_STARTED.iterator();
	}

	public static void addToCreatedDirectories(File dir) {
		Assert.assertTrue(dir.isDirectory());
		All_DIRECTORIES.add(dir);
	}

	public static Iterator getAllCreatedDirectories() {
		return All_DIRECTORIES.iterator();
	}

	public static boolean mkdirs(File file) {
		if (file.exists()) {
			return false;
		} else if (file.mkdir()) {
			addToCreatedDirectories(file);
			return true;
		} else {
			File parent = file.getParentFile();
			if ((parent != null) && mkdirs(parent) && file.mkdir()) {
				addToCreatedDirectories(file);
				return true;
			} else {
				return false;
			}
		}
	}

	public static void cleanCreatedDirectories() {
		Iterator it = getAllCreatedDirectories();
		while (it.hasNext()) {
			File dir = (File) it.next();
			if (!dir.exists() || (dir.isDirectory() && dir.delete())) {
				it.remove();
			}
		}
	}
}