package de.uniwue.ki.mas.sim.analysis;
import java.io.File;

import junit.framework.Assert;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.simulation.ISimulationCreateFunctionArgs;
import de.uniwue.ki.util.file.FileUtilities;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
/**
 * Creation date: (28.11.2001 17:20:13)
 * @author Christoph Oechslein
 */
public class CSVAnalysisTypeDeclaration
	extends AbstractAnalysisTypeDeclaration
	implements IAnalysisTypeDeclaration {

	public static final String DEFAULT_TIME_COLUMN_NAME = "Time";
	public static final String DEFAULT_SEPARATOR_STRING = ";";
	
	private File _file = new File("analysis.csv");
	private String _separatorString = DEFAULT_SEPARATOR_STRING;
	private String _timeColumnName = DEFAULT_TIME_COLUMN_NAME;

	public CSVAnalysisTypeDeclaration() {
		super();
	}

	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		CSVAnalysisTypeDeclaration intoCSVAnalysisTypeDeclaration = (CSVAnalysisTypeDeclaration) intoObject;

		intoCSVAnalysisTypeDeclaration.setFileString(getFileString());
		intoCSVAnalysisTypeDeclaration.setSeparatorString(getSeparatorString());
		intoCSVAnalysisTypeDeclaration.setTimeColumnName(getTimeColumnName());
	}

	public IAnalysis createAnalysis(
		ISimulationCreateFunctionArgs createFunctionArgs,
		IAnalysisDeclaration analysisDeclaration)
		throws ModelCompilingException {

		return new CSVAnalysis(
			analysisDeclaration,
			createFunctionArgs,
			createStringArray(analysisDeclaration),
			createValueFunctionArray(createFunctionArgs, analysisDeclaration),
			createValueTypeArray(analysisDeclaration));
	}

	public String getFileString() {
		return _file + "";
	}

	public IAnalysisItemDeclarationFactory getAnalysisItemDeclarationFactory() {
		return CSVAnalysisItemDeclarationFactory.getInstance();
	}

	public String getName() {
		return "To File";
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void setFileString(String newFileString) {
		if (newFileString != null) {
			File newFile = FileUtilities.getFileWithSuffix(newFileString, "csv");

			if (!newFile.equals(_file)) {
				String oldFileString = getFileString();

				_file = newFile;

				firePropertyChange("fileString", oldFileString, newFileString);
				fireStateChanged();
			}
		}
	}

	public void setSeparatorString(String separatorString) {
		Assert.assertTrue((separatorString != null) && (separatorString.length() > 0));
		
		_separatorString = separatorString;
		fireStateChanged();
	}

	public String getSeparatorString() {
		return _separatorString;
	}

	public void setTimeColumnName(String timeColumnName) {
		Assert.assertTrue(timeColumnName != null);
		
		_timeColumnName = timeColumnName;
		fireStateChanged();
	}

	public String getTimeColumnName() {
		return _timeColumnName;
	}
}