package de.uniwue.ki.mas.sim.analysis;
import java.util.Map;

import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.ObjectType;
import de.uniwue.ki.mas.util.declaration.AbstractDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (15.10.2001 16:55:56)
 * @author: Christoph Oechslein
 */
class DefaultAnalysisItemDeclaration extends AbstractDeclaration implements IAnalysisItemDeclaration {
	private String _name = "Untitled Analysis Item";
	private final IFunctionCallDeclaration _valueFunctionCallDeclaration;

	public DefaultAnalysisItemDeclaration() {
		this(ObjectType.getInstance());
	}

	public DefaultAnalysisItemDeclaration(ISpecificType specificType) {
		this(FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(specificType));
	}

	public DefaultAnalysisItemDeclaration(IFunctionCallDeclaration valueFunctionCallDeclaration) {
		super();

		_valueFunctionCallDeclaration = valueFunctionCallDeclaration;
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IAnalysisItemDeclaration intoAnalysisItemDeclaration = (IAnalysisItemDeclaration) intoObject;

		intoAnalysisItemDeclaration.setName(getName());

		getValueFunctionCallDeclaration().copyInto(
			intoAnalysisItemDeclaration.getValueFunctionCallDeclaration(), objectMapping);
	}

	protected ICopyable createInstance(java.util.Map objectMapping) {
		return new DefaultAnalysisItemDeclaration();
	}

	public String getName() {
		return _name;
	}

	public final IFunctionCallDeclaration getValueFunctionCallDeclaration() {
		return _valueFunctionCallDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public void remove() {
		_valueFunctionCallDeclaration.remove();

		super.remove();

	}

	public void setName(String newName) {
		String oldValue = getName();

		_name = newName;

		firePropertyChange("name", oldValue, newName);
		fireStateChanged();
	}
}