package de.uniwue.ki.mas.sim.analysis;
import java.util.*;

import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.simulation.ISimulationCreateFunctionArgs;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
/**
 * Creation date: (28.11.2001 17:20:13)
 * @author Christoph Oechslein
 */
public class SeriesChartAnalysisTypeDeclaration
	extends AbstractAnalysisTypeDeclaration
	implements IAnalysisTypeDeclaration {

	private int _windowSize = 100;

	public SeriesChartAnalysisTypeDeclaration() {
		super();
	}

	public void copyInto(Object intoObject, java.util.Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		SeriesChartAnalysisTypeDeclaration intoSeriesChartAnalysisTypeDeclaration =
			(SeriesChartAnalysisTypeDeclaration) intoObject;

		intoSeriesChartAnalysisTypeDeclaration.setWindowSize(getWindowSize());
	}

	public IAnalysis createAnalysis(
		ISimulationCreateFunctionArgs createFunctionArgs,
		IAnalysisDeclaration analysisDeclaration)
		throws ModelCompilingException {

		AnalysisItemDeclarationList analysisItemDeclarationList =
			getAnalysisItemDeclarationList(analysisDeclaration);

		List nameFunctions = new ArrayList(analysisItemDeclarationList.size());
		List valueFunctions = new ArrayList(analysisItemDeclarationList.size());

		Iterator it = analysisItemDeclarationList.iterator();
		while (it.hasNext()) {
			IAnalysisItemDeclaration currIAnalysisItemDeclaration = (IAnalysisItemDeclaration) it.next();

			nameFunctions.add(currIAnalysisItemDeclaration.getName());
			valueFunctions.add(
				currIAnalysisItemDeclaration.getValueFunctionCallDeclaration().createFunction(
					createFunctionArgs));
		}

		return new SeriesChartAnalysis(
			analysisDeclaration,
			createFunctionArgs,
			getWindowSize(),
			createStringArray(analysisDeclaration),
			createValueFunctionArray(createFunctionArgs, analysisDeclaration));
	}

	public IAnalysisItemDeclarationFactory getAnalysisItemDeclarationFactory() {
		return ChartAnalysisItemDeclarationFactory.getInstance();
	}

	/**
	 * @return String
	 * @see de.uniwue.ki.util.INamed#getName()
	 */
	public String getName() {
		return "Series Chart";
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (01.12.2001 13:25:42)
	 * @return int
	 */
	public int getWindowSize() {
		return _windowSize;
	}

	/**
	 * @param anObject IIsomorphicHashable
	 * @return boolean
	 * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicEquals(IIsomorphicHashable)
	 */
	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	/**
	 * @param newWindowSize int
	 */
	public void setWindowSize(int newWindowSize) {
		int oldValue = getWindowSize();

		_windowSize = Math.max(1, newWindowSize);

		firePropertyChange("windowSize", oldValue, _windowSize);
	}
}