package de.uniwue.ki.mas.sim.analysis;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (15.10.2001 17:08:42)
 * @author Christoph Oechslein
 */
public final class TableModelAnalysis extends AbstractValueAnalysis {
	private final TableAnalysisTableModelAdapter _tableAnalysisTableModelAdapter =
		new TableAnalysisTableModelAdapter();

	private final class TableAnalysisTableModelAdapter extends AbstractTableModel {
		public TableAnalysisTableModelAdapter() {
			super();
		}

		public void fireTableRowsUpdated(int firstRow, int lastRow, int cols) {
			fireTableChanged(new TableModelEvent(this, firstRow, lastRow, cols));
		}

		public Class getColumnClass(int col) {
				return String.class;
		}

		public int getColumnCount() {
			return 2;
		}

		public int getRowCount() {
			return getValueFunctionArray().length;
		}

		public Object getValueAt(int row, int col) {
			if (col == 0) {
				return getNameArray()[row];
			} else {
				return getNameForEntry(row);
			}
		}
	}

	public TableModelAnalysis(
		IAnalysisDeclaration analysisDeclaration,
		ICreateFunctionArgs createFunctionArgs,
		String[] nameList,
		IFunction[] valueFunctionList,
		ISpecificType[] analyseValueTypeArray)
		throws ModelCompilingException {

		super(analysisDeclaration, createFunctionArgs, nameList, valueFunctionList, analyseValueTypeArray);
	}

	protected void analyseIntern(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		super.analyseIntern(simulationExecuteFunctionArgs);

		_tableAnalysisTableModelAdapter.fireTableRowsUpdated(0, getCurrentValuesCount() - 1, 1);
	}

	public final TableModel getTableModel() {
		return _tableAnalysisTableModelAdapter;
	}

	public void init(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {
		super.init(simulationExecuteFunctionArgs);
	}
}