package de.uniwue.ki.mas.sim.experiment;
import java.io.File;
import java.io.IOException;
import java.util.List;

import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.file.FileUtilities;
import de.uniwue.ki.util.file.TokenReplacer;

/**
 * Creation date: (29.01.2002 15:45:22)
 * @author Christoph Oechslein
 */

public abstract class AbstractExperiment extends AbstractStartPauseFinishAble implements IExperiment {
	public static final TokenReplacer TOKEN_REPLACER =
		TokenReplacer.createTokenReplacer(createTokenReplaceConfigList(null));

	private static List createTokenReplaceConfigList(final IExperiment experiment) {
		return CollectionsHelper.createList(new TokenReplacer.TokenReplaceConfig('E', "Experiment") {
			public String getReplaceString() {
				if (experiment == null)
					return "Untitled Experiment";
				else
					return experiment.getName();
			}
		});
	}
	private final File _configurationFile;
	private transient ExperimentListener _experimentListener = null;
	private final String _name;

	public AbstractExperiment(String name) throws ModelCompilingException {
		this(name, null);
	}

	public AbstractExperiment(String name, File configurationFile) throws ModelCompilingException {
		super();

		_name = name;

		if (configurationFile == null) {
			_configurationFile = null;

		} else {
			try {

				_configurationFile =
					FileUtilities.getWriteableNonExistingFile(
						getAbsoluteFileWithReplacedTokens(configurationFile),
						"experiment",
						".csv",
						true);

			} catch (IOException exception) {
				throw new ModelCompilingException(exception);
			}
		}
	}

	public void addExperimentListener(ExperimentListener newListener) {
		_experimentListener = ExperimentEventMulticaster.add(_experimentListener, newListener);
	}

	public void fireCurrentSimulationChanged(ISimulation oldSimulation, ISimulation newSimulation) {
		if (_experimentListener != null)
			_experimentListener.currentSimulationChanged(
				new ExperimentCurrentSimulationChangedEvent(this, oldSimulation, newSimulation));
	}

	protected void fireFinished() {
		super.fireFinished();

		if (_experimentListener != null)
			_experimentListener.finished(new StartPauseFinishEvent(this));
	}

	protected void fireFinished(Throwable exception) {
		super.fireFinished(exception);

		if (_experimentListener != null)
			_experimentListener.finished(new StartPauseFinishEvent(this, exception));

		exception.printStackTrace(System.err);
	}

	protected void firePaused() {
		super.firePaused();

		if (_experimentListener != null)
			_experimentListener.paused(new StartPauseFinishEvent(this));
	}

	protected void fireStarted() {
		super.fireStarted();

		if (_experimentListener != null)
			_experimentListener.started(new StartPauseFinishEvent(this));
	}

	private File getAbsoluteFileWithReplacedTokens(File file) {
		return new File(
			TokenReplacer.createTokenReplacer(createTokenReplaceConfigList(this)).replaceTokens(
				file.toString()));
	}

	public final File getConfigurationFile() {
		return _configurationFile;
	}

	public final String getName() {
		return _name;
	}

	public void remove() {
	}
	
	public void removeSimulation(ISimulation simulation) {
	}

	public void removeExperimentListener(ExperimentListener newListener) {
		_experimentListener = ExperimentEventMulticaster.remove(_experimentListener, newListener);
	}
}