package de.uniwue.ki.mas.sim.experiment;
import java.io.File;
import java.util.Map;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.rest.FunctionDeclarationRestFactory;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.file.FileUtilities;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (28.01.2002 17:53:56)
 * @author Christoph Oechslein
 */
final class ExperimentDeclaration extends AbstractDocumentableDeclaration implements IExperimentDeclaration {
	private static IFunctionCallInternDeclaration createDefaultFunctionCallInternDeclaration(ISimulationDeclaration simulationDeclaration) {
		ISpecificFunctionDeclaration doWhileDecl =
			FunctionDeclarationRestFactory.DO_WITH_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(
					GenericFunctionType.getInstance().createSpecificFunctionType(
						VoidType.getInstance(),
						CollectionsHelper.createList(SimulationDeclarationType.getInstance())),
					SimulationDeclarationType.getInstance()));

		IComposedFunctionCallInternDeclaration doWhileFunCallDecl =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				doWhileDecl);

		doWhileFunCallDecl.setChild(
			1,
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				doWhileFunCallDecl,
				SimulationDeclarationType.getInstance(),
				simulationDeclaration));

		return doWhileFunCallDecl;
	}

	private String _configurationFileName = "experimentConfig.csv";
	private final IFunctionCallDeclaration _simulationActionFunctionCallDeclaration;
	private boolean _testSimulationDeclaration;

	private ExperimentDeclaration() {
		this(
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
				VoidType.getInstance()));
	}

	public ExperimentDeclaration(IFunctionCallDeclaration simulationActionFunctionCallDeclaration) {
		super("Untitled Experiment");

		Assert.assertEquals(
			simulationActionFunctionCallDeclaration.getSpecificOutputType(),
			VoidType.getInstance());

		_simulationActionFunctionCallDeclaration = simulationActionFunctionCallDeclaration;
	}

	public ExperimentDeclaration(ISimulationDeclaration simulationDeclaration) {
		this(
			FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
				createDefaultFunctionCallInternDeclaration(simulationDeclaration)));
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		IExperimentDeclaration intoExperimentDeclaration = (IExperimentDeclaration)intoObject;

		intoExperimentDeclaration.setTestSimulationDeclaration(isTestSimulationDeclaration());

		getSimulationActionFunctionCallDeclaration().copyInto(
			intoExperimentDeclaration.getSimulationActionFunctionCallDeclaration(),
			objectMapping);

		intoExperimentDeclaration.setConfigurationFileName(getConfigurationFileName());
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new ExperimentDeclaration();
	}

	public String getConfigurationFileName() {
		return _configurationFileName;
	}

	public final IFunctionCallDeclaration getSimulationActionFunctionCallDeclaration() {
		return _simulationActionFunctionCallDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		return equals(anObject);
	}

	public int isomorphicHashCode() {
		return hashCode();
	}

	public boolean isTestSimulationDeclaration() {
		return _testSimulationDeclaration;
	}

	public void remove() {
		getSimulationActionFunctionCallDeclaration().remove();

		super.remove();
	}

	public void setConfigurationFileName(String newConfigurationFileName) {
		_configurationFileName =
			FileUtilities.getFileStringWithSuffix(new File(newConfigurationFileName), "csv");

		fireStateChanged();
	}

	public void setTestSimulationDeclaration(boolean testSimulationDeclaration) {
		_testSimulationDeclaration = testSimulationDeclaration;

		fireStateChanged();
	}
}