package de.uniwue.ki.mas.sim.experiment;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.simulation.SimulationDeclarationListAndFactory;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.util.factory.AbstractObjectFactory;

/**
 * Creation date: (28.11.2001 13:47:22)
 * @author Christoph Oechslein
 */

public class ExperimentFactory extends AbstractObjectFactory {
	private final ExperimentDeclarationList _experimentDeclarationList;
	private final SimulationDeclarationListAndFactory _simulationDeclarationListAndFactory;
	private final ISeSAmDeclaration _seSAmDeclaration;

	public ExperimentFactory(
		ISeSAmDeclaration seSAmDeclaration,
		ExperimentDeclarationList experimentDeclarationList,
		SimulationDeclarationListAndFactory simulationDeclarationListAndFactory) {

		super();

		_seSAmDeclaration = seSAmDeclaration;
		_experimentDeclarationList = experimentDeclarationList;
		_simulationDeclarationListAndFactory = simulationDeclarationListAndFactory;
	}

	public final IExperiment createExperiment() throws ModelCompilingException {
		if (!_experimentDeclarationList.isEmpty()) {
			return createExperiment(
				_seSAmDeclaration,
				(IExperimentDeclaration) _experimentDeclarationList.get(0));

		} else if (!_simulationDeclarationListAndFactory.getSimulationDeclarationList().isEmpty()) {
			return createExperiment(
				_seSAmDeclaration,
				(ISimulationDeclaration) _simulationDeclarationListAndFactory
					.getSimulationDeclarationList()
					.get(
					0));

		} else {
			ISimulationDeclaration newSimulationDeclaration =
				_simulationDeclarationListAndFactory
					.getSimulationDeclarationFactory()
					.createSimulationDeclaration();

			//_simulationDeclarationListAndFactory.getSimulationDeclarationList().add(newSimulationDeclaration);

			return createExperiment(_seSAmDeclaration, newSimulationDeclaration);
		}
	}

	public static final IExperiment createExperiment(
		ISeSAmDeclaration seSAmDeclaration,
		IExperimentDeclaration experimentDeclaration)
		throws ModelCompilingException {

		return new ThreadedExperiment(seSAmDeclaration, experimentDeclaration);
	}

	public static final IExperiment createExperiment(
		ISeSAmDeclaration seSAmDeclaration,
		ISimulationDeclaration simulationDeclaration)
		throws ModelCompilingException {
			

//		long time = System.currentTimeMillis();
//
//		OneSimulationExperiment exp = null;
//
//		int AMOUNT = 1;
//
//		for (int i = 0; i < AMOUNT; i++) {
//			exp = new OneSimulationExperiment(seSAmDeclaration, simulationDeclaration);
//		}
//
//		System.out.println((System.currentTimeMillis() - time) / AMOUNT + "ms");
//		System.out.flush();
//
//		return exp;

		return new OneSimulationExperiment(seSAmDeclaration, simulationDeclaration);
	}

	public static final IExperiment createExperiment(
		ISeSAmDeclaration seSAmDeclaration,
		ISituationDeclaration situationDeclaration)
		throws ModelCompilingException {

		ISimulationDeclaration simulationDeclaration =
			seSAmDeclaration
				.getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationFactory()
				.createSimulationDeclaration(situationDeclaration);

		return createExperiment(seSAmDeclaration, simulationDeclaration);
	}

	public final Object createObject() throws ModelCompilingException {
		return createExperiment();
	}

	public String getName() {
		return "Experiment Factory";
	}
}