package de.uniwue.ki.mas.sim.experiment;

import java.util.Iterator;

import de.uniwue.ki.mas.sim.simulation.StartPauseFinishEvent;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.list.DefaultMutableListModel;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (15.10.2001 18:16:31)
 * @author Christoph Oechslein
 */

public class ExperimentList extends DefaultMutableListModel implements INamed, IRemoveable {

	private transient final ExperimentListener _experimentListener = new ExperimentListener() {
		public void currentSimulationChanged(ExperimentCurrentSimulationChangedEvent event) {
			fireContentsChanged(event.getExperiment());
		}

		public void finished(StartPauseFinishEvent event) {
			fireContentsChanged((IExperiment) event.getSource());
		}

		public void paused(StartPauseFinishEvent event) {
			fireContentsChanged((IExperiment) event.getSource());
		}

		public void started(StartPauseFinishEvent event) {
			fireContentsChanged((IExperiment) event.getSource());
		}

		private void fireContentsChanged(IExperiment experiment) {
			int index = indexOf(experiment);
			if (index == -1) {
				// shouldn't happen
				experiment.removeExperimentListener(this);
			} else {
				ExperimentList.this.fireContentsChanged(ExperimentList.this, index, index);
			}
		}

	};

	public ExperimentList() {
		super();
	}

	protected void fireContentsChanged(Object source, int index0, int index1) {
		super.fireContentsChanged(source, index0, index1);
	}

	public String getName() {
		return "Simulation Runs";
	}

	public void add(int index, Object element) {
		super.add(index, element);

		((IExperiment) element).addExperimentListener(_experimentListener);
	}

	public Object remove(int index) {
		IExperiment removedExperiment = (IExperiment) super.remove(index);

		removedExperiment.removeExperimentListener(_experimentListener);
		removedExperiment.remove();

		return removedExperiment;
	}

	public Object set(int index, Object element) {
		IExperiment removedExperiment = (IExperiment) super.set(index, element);

		if (removedExperiment != element) {
			removedExperiment.removeExperimentListener(_experimentListener);
			removedExperiment.remove();
			((IExperiment) element).addExperimentListener(_experimentListener);
		}

		return removedExperiment;
	}

	public void remove() {
		Iterator it = iterator();

		while (it.hasNext()) {
			IExperiment element = (IExperiment) it.next();
			element.remove();
			it.remove();
		}
	}
}