package de.uniwue.ki.mas.sim.experiment;
import junit.framework.Assert;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.mas.sim.simulation.*;

/**
 * Creation date: (29.01.2002 15:45:22)
 * @author Christoph Oechslein
 */
public final class OneSimulationExperiment extends AbstractExperiment implements IExperiment {
	private final IThreadedSimulation _threadedSimulation;

	private final StartPauseFinishListener _simulationStartPauseFinishListener =
		new StartPauseFinishListener() {
		public void paused(StartPauseFinishEvent event) {
			firePaused();
		}

		public void finished(StartPauseFinishEvent event) {
			if (event.getException() == null)
				fireFinished();
			else
				fireFinished(event.getException());
		}

		public void started(StartPauseFinishEvent event) {
			fireStarted();
		}
	};

	public OneSimulationExperiment(
		ISeSAmDeclaration seSAmDeclaration,
		ISimulationDeclaration simulationDeclaration)
		throws ModelCompilingException {

		super(simulationDeclaration.getName());

		//long startTime = System.currentTimeMillis();

		_threadedSimulation =
			SimulationFactory.createThreadedSimulation(seSAmDeclaration, simulationDeclaration);

		//System.out.println("Last " + (System.currentTimeMillis() - startTime) + " msecs");

		getThreadedSimulation().addStartPauseFinishListener(_simulationStartPauseFinishListener);
	}

	public ISimulation createSimulation(ISimulationDeclaration simulationDeclaration)
		throws ModelCompilingException {

		Assert.fail("Not possible");

		return getCurrentSimulation();
	}

	public ISimulation getCurrentSimulation() {
		return getThreadedSimulation();
	}

	public final IThreadedSimulation getThreadedSimulation() {
		return _threadedSimulation;
	}

	public boolean isFinished() {
		return getThreadedSimulation().isFinished();
	}

	public boolean isFinishedWithErrors() {
		return getThreadedSimulation().isFinishedWithErrors();
	}

	public boolean isPaused() {
		return getThreadedSimulation().isPaused();
	}

	public boolean isRunning() {
		return getThreadedSimulation().isRunning();
	}

	public void pauseInThread() {
		getThreadedSimulation().pauseInThread();
	}

	public void terminateInThread() {
		getThreadedSimulation().terminateInThread();
	}

	public void remove() {
		terminateInThread();
		getThreadedSimulation().removeStartPauseFinishListener(_simulationStartPauseFinishListener);
		getThreadedSimulation().remove();

		super.remove();
	}

	public void startExperiment() throws SimulationRuntimeException {
		if (isRunning()) {
			throw new IllegalStateException("Already in running state");
		}

		if (isFinished() || isFinishedWithErrors()) {
			throw new IllegalStateException("Already finished");
		}

		getThreadedSimulation().startSimulation();
	}

	public void startInThread() {
		getThreadedSimulation().startInThread();
	}

	public void startSimulation(ISimulation simulation) {
		throw new IllegalStateException("Cannot occur in this class");
	}

	public Throwable getException() {
		return getThreadedSimulation().getException();
	}

	public IExperimentDeclaration getExperimentDeclaration() {
		return null;
	}
}