package de.uniwue.ki.mas.sim.experiment.function;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.action.AbstractFunctionAction;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.analysis.CSVAnalysisSaveAllFinished;
import de.uniwue.ki.mas.sim.experiment.IExperiment;
import de.uniwue.ki.mas.sim.experiment.IExperimentExecuteFunctionArgs;
import de.uniwue.ki.mas.sim.simulation.ISimulation;
import de.uniwue.ki.mas.sim.simulation.IThreadedSimulation;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Chripauseh Oechslein
 */

final class FunctionActionSimulateSimulation extends AbstractFunctionAction implements IFunction {
	static final String NAME = "Simulate";

	private final IFunction _simulationFunction;
	private final IFunction _writeConfigFunction;

	private final SimulationRunConfigurationAndResults _simulationRunConfigurationAndResults;

	public FunctionActionSimulateSimulation(
		SimulationRunConfigurationAndResults simulationRunConfigurationAndResults,
		IFunction simulationFunction,
		IFunction writeConfigFunction) {
		super();

		_simulationRunConfigurationAndResults = simulationRunConfigurationAndResults;

		_simulationFunction = simulationFunction;
		_writeConfigFunction = writeConfigFunction;
	}

	public void act(IExecuteFunctionArgs executeFunctionArgs) throws SimulationRuntimeException {
		ISimulation simulation = (ISimulation)_simulationFunction.execute(executeFunctionArgs);
		boolean writeConfig = _writeConfigFunction.executeForBoolean(executeFunctionArgs);

		IExperimentExecuteFunctionArgs experimentExecuteFunctionArgs =
			(IExperimentExecuteFunctionArgs)executeFunctionArgs;

		IExperiment experiment = experimentExecuteFunctionArgs.getExperiment();

		experimentExecuteFunctionArgs.setSimulation(simulation);

		if (writeConfig) {
			_simulationRunConfigurationAndResults.readStartValues(executeFunctionArgs, simulation);
		}

		experiment.startSimulation(simulation);
		

		if (simulation instanceof IThreadedSimulation)
			Assert.assertTrue(
				((IThreadedSimulation)simulation).isFinished()
					|| ((IThreadedSimulation)simulation).isFinishedWithErrors());

		if (writeConfig) {
			_simulationRunConfigurationAndResults.readEndValues(executeFunctionArgs, simulation);
			_simulationRunConfigurationAndResults.writeConfiguration(executeFunctionArgs, simulation);
			CSVAnalysisSaveAllFinished.addToAllFinishedList(experiment.getConfigurationFile());
		}

		// Now it is gooing to be removed, that no memory is wasted.
		// This implies that you cannot start a simulation twice.
		experiment.removeSimulation(simulation);	
	}

	public String getName() {
		return NAME + "(" + _simulationFunction.getName() + ")";
	}
}