package de.uniwue.ki.mas.sim.experiment.function;
import java.io.*;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.List;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.function.IExecuteFunctionArgs;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.entity.body.IBody;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.mas.sim.analysis.CSVAnalysisSaveAllFinished;
import de.uniwue.ki.mas.sim.experiment.IExperiment;
import de.uniwue.ki.mas.sim.experiment.IExperimentExecuteFunctionArgs;
import de.uniwue.ki.mas.sim.simulation.ISimulation;

/**
 * Creation date: (04.09.2001 15:46:50)
 * @author Christoph Oechslein
 */

final class SimulationRunConfigurationAndResults implements Serializable {
	private final List _variableStartValues;
	private final List _variableEndValues;

	public SimulationRunConfigurationAndResults(IExperiment experiment) {
		super();

		_variableStartValues = new ArrayList();
		_variableEndValues = new ArrayList();
	}

	private boolean isRightType(ISpecificType type) {
		return true;
		// (type == NumberType.getInstance()) || (type == BooleanType.getInstance()) || (type == StringType.getInstance());
	}

	private void printCategories(PrintWriter printWriter, ISimulation simulation) {

		// config
		printWriter.print("Configuration ; ");
		printNTimes(printWriter, " ; ", 5);

		printWriter.print("StartValues ; ");
		printNTimes(printWriter, " ; ", _variableStartValues.size());

		printWriter.print("EndValues ; ");
		printNTimes(printWriter, " ; ", _variableEndValues.size());

		printWriter.println();
	}

	private void printHeaders(
		ICreateFunctionArgs createFunctionArgs,
		PrintWriter printWriter,
		ISimulation simulation)
		throws ModelCompilingException {

		printWriter.print(" ; ");
		printWriter.print("Date ; ");
		printWriter.print("Duration ; ");
		printWriter.print("Simulation ; ");
		printWriter.print("Situation ; ");
		printWriter.print("World ; ");

		printVariableNames(
			createFunctionArgs,
			printWriter,
			simulation.getSituation().getWorld().getBody(),
			false);

		printWriter.print(" ; ");

		printVariableNames(
			createFunctionArgs,
			printWriter,
			simulation.getSituation().getWorld().getBody(),
			true);

		printWriter.println();
	}

	private void printNTimes(PrintWriter printWriter, String str, int times) {
		for (int i = 0; i < times; i++) {
			printWriter.print(str);
		}
	}

	private void printValues(
		ICreateFunctionArgs createFunctionArgs,
		PrintWriter printWriter,
		ISimulation simulation)
		throws ModelCompilingException {

		printWriter.print(" ; ");
		printWriter.print(DateFormat.getDateTimeInstance().format(simulation.getStartDate()));
		printWriter.print(" ; ");
		printWriter.print(simulation.getSituation().getTimeDurationInTicks());
		printWriter.print(" ; ");
		printWriter.print(simulation.getSimulationDeclaration().getName());
		printWriter.print(" ; ");
		printWriter.print(simulation.getSimulationDeclaration().getSituationDeclaration().getName());
		printWriter.print(" ; ");
		printWriter.print(
			simulation
				.getSimulationDeclaration()
				.getSituationDeclaration()
				.getWorldInstanceDeclaration()
				.getWorldClassDeclaration()
				.getName());

		printWriter.print(" ; ");

		printVariableValues(
			createFunctionArgs,
			printWriter,
			simulation.getSituation().getWorld().getBody(),
			false);

		printWriter.print(" ; ");

		printVariableValues(
			createFunctionArgs,
			printWriter,
			simulation.getSituation().getWorld().getBody(),
			true);
	}

	private void printVariableNames(
		ICreateFunctionArgs createFunctionArgs,
		PrintWriter printWriter,
		IBody body,
		boolean endValues)
		throws ModelCompilingException {
		int size = body.getBodyVariablesLength();

		for (int i = 0; i < size; i++) {
			IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
				body.getBodyVariableInstanceDeclarationFor(body.getBodyVariableAt(i));

			if (endValues
				? shouldWriteEndValueOf(createFunctionArgs, bodyVariableInstanceDeclaration)
				: shouldWriteStartValueOf(createFunctionArgs, bodyVariableInstanceDeclaration)) {
				printWriter.print(" ; ");

				printWriter.print(bodyVariableInstanceDeclaration.getName());
			}
		}
	}

	private void printVariableValues(
		ICreateFunctionArgs createFunctionArgs,
		PrintWriter printWriter,
		IBody body,
		boolean endValues)
		throws ModelCompilingException {
		int size = body.getBodyVariablesLength();

		int currentValueIndex = 0;

		StringBuffer buff = new StringBuffer();

		for (int i = 0; i < size; i++) {
			IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
				body.getBodyVariableInstanceDeclarationFor(body.getBodyVariableAt(i));

			if (endValues
				? shouldWriteEndValueOf(createFunctionArgs, bodyVariableInstanceDeclaration)
				: shouldWriteStartValueOf(createFunctionArgs, bodyVariableInstanceDeclaration)) {
				buff.append(" ; ");

				Object value =
					endValues
						? _variableEndValues.get(currentValueIndex)
						: _variableStartValues.get(currentValueIndex);

				currentValueIndex++;

				bodyVariableInstanceDeclaration
					.getBodyVariableClassDeclaration()
					.getVariableClassDeclaration()
					.getSpecificType()
					.appendNameForArgumentObject(buff, value, 0, 30);
			}
		}

		printWriter.write(buff.toString());
	}

	public void readEndValues(IExecuteFunctionArgs executeFunctionArgs, ISimulation simulation)
		throws SimulationRuntimeException {

		try {
			IWorld world = simulation.getSituation().getWorld();
			IBody body = world.getBody();

			int size = body.getBodyVariablesLength();

			for (int i = 0; i < size; i++) {
				IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
					body.getBodyVariableInstanceDeclarationFor(body.getBodyVariableAt(i));

				if (shouldWriteEndValueOf(executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
					bodyVariableInstanceDeclaration)) {

					try {
						int index =
							body
								.getBodyInstanceDeclaration()
								.getBodyClassDeclaration()
								.getBodyVariableIndexOf(
								bodyVariableInstanceDeclaration.getBodyVariableClassDeclaration());

						_variableEndValues.add(body.getBodyVariableAt(index).getValue());

					} catch (ModelCompilingException exception) {
						throw new SimulationRuntimeException(exception);

					}
				}
			}
		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeCausedByModelCompileException(e);
		}
	}

	public void readStartValues(IExecuteFunctionArgs executeFunctionArgs, ISimulation simulation)
		throws SimulationRuntimeException {

		try {
			IWorld world = simulation.getSituation().getWorld();
			IBody body = world.getBody();

			int size = body.getBodyVariablesLength();

			for (int i = 0; i < size; i++) {
				IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
					body.getBodyVariableInstanceDeclarationFor(body.getBodyVariableAt(i));

				if (shouldWriteStartValueOf(executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
					bodyVariableInstanceDeclaration)) {

					int index =
						body.getBodyInstanceDeclaration().getBodyClassDeclaration().getBodyVariableIndexOf(
							bodyVariableInstanceDeclaration.getBodyVariableClassDeclaration());

					_variableStartValues.add(body.getBodyVariableAt(index).getValue());
				}
			}
		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeCausedByModelCompileException(e);
		}
	}

	private boolean shouldWriteEndValueOf(
		ICreateFunctionArgs createFunctionArgs,
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration)
		throws ModelCompilingException {

		IVariableClassDeclaration variableClassDeclaration =
			bodyVariableInstanceDeclaration.getBodyVariableClassDeclaration().getVariableClassDeclaration();
		return variableClassDeclaration.isWriteable()
			&& isRightType(variableClassDeclaration.getSpecificType());
	}

	private boolean shouldWriteStartValueOf(
		ICreateFunctionArgs createFunctionArgs,
		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration)
		throws ModelCompilingException {

		return isRightType(
			bodyVariableInstanceDeclaration
				.getBodyVariableClassDeclaration()
				.getVariableClassDeclaration()
				.getSpecificType());
	}

	public void writeConfiguration(IExecuteFunctionArgs executeFunctionArgs, ISimulation simulation)
		throws SimulationRuntimeException {
		IExperimentExecuteFunctionArgs experimentExecuteFunctionArgs =
			(IExperimentExecuteFunctionArgs)executeFunctionArgs;

		try {
			writeConfiguration(
				executeFunctionArgs.createCreateAndExecuteFunctionArgs(),
				experimentExecuteFunctionArgs.getExperiment().getConfigurationFile(),
				simulation);

		} catch (IOException exception) {
			throw new SimulationRuntimeException(exception);

		} catch (ModelCompilingException e) {
			throw new SimulationRuntimeCausedByModelCompileException(e);

		} finally {
			_variableEndValues.clear();
			_variableStartValues.clear();
		}
	}

	private void writeConfiguration(
		ICreateFunctionArgs createFunctionArgs,
		File file,
		ISimulation simulation)
		throws IOException, ModelCompilingException {

		boolean fileExists = file.exists();

		if (!fileExists && (file.getParentFile() != null))
			CSVAnalysisSaveAllFinished.mkdirs(file.getParentFile());

		PrintWriter printWriter = new PrintWriter(new BufferedWriter(new FileWriter(file, true)));

		if (!fileExists) {
			printCategories(printWriter, simulation);

			printHeaders(createFunctionArgs, printWriter, simulation);
		}

		printValues(createFunctionArgs, printWriter, simulation);

		printWriter.println();

		if (printWriter.checkError()) {
			throw new IOException("Error during printing!");
		}

		printWriter.close();
	}
}