package de.uniwue.ki.mas.sim.listener;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.variable.IBodyVariable;

/**
 * Creation date: (20.02.2002 15:20:11)
 * @author Christoph Oechslein
 */

public final class VariableChangedEvent extends ObjectEvent {
	private final IObject _settingObject;
	private final Object _oldValue;
	private final Object _newValue;
	private final IBodyVariable _bodyVariable;
	private final boolean _dynamicUpdate;

	public VariableChangedEvent(
		IObject source,
		IObject settingObject,
		IBodyVariable bodyVariable,
		Object oldValue,
		Object newValue) {

		super(source);

		_settingObject = settingObject;
		_oldValue = oldValue;
		_newValue = newValue;
		_bodyVariable = bodyVariable;
		_dynamicUpdate = false;
	}

	/**
	 * @param source IBody
	 * @param bodyVariable IBodyVariable
	 * @param oldValue Object
	 * @param newValue Object
	 * @param dynamicUpdate boolean
	 */
	public VariableChangedEvent(
		IObject source,
		IBodyVariable bodyVariable,
		Object oldValue,
		Object newValue,
		boolean dynamicUpdate) {
		super(source);

		_settingObject = null;
		_oldValue = oldValue;
		_newValue = newValue;
		_bodyVariable = bodyVariable;
		_dynamicUpdate = dynamicUpdate;
	}

	/**
	 * Returns the associated body variable.
	 * @return IBodyVariable
	 */
	public final IBodyVariable getBodyVariable() {
		return _bodyVariable;
	}

	public final IObject getBodyVariableOwner() {
		return ((IObject) getSource());
	}

	/**
	 * Returns the new value of the body variable.
	 * @return Object
	 */
	public final Object getNewValue() {
		return _newValue;
	}

	/**
	 * Returns the old value of the body variable.
	 * @return Object
	 */
	public final Object getOldValue() {
		return _oldValue;
	}

	public final IObject getSettingObject() {
		return _settingObject;
	}

	/**
	 * @return boolean
	 */
	public boolean isDynamicUpdate() {
		return _dynamicUpdate;
	}

}