package de.uniwue.ki.mas.sim.logging;
import de.uniwue.ki.mas.entity.object.world.*;
import de.uniwue.ki.mas.sim.experiment.*;
import de.uniwue.ki.mas.sim.listener.*;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.mas.sim.situation.*;

/**
 * Creation date: (20.10.2000 20:05:33)
 * @author Christoph Oechslein
 */
public class AbstractLogger
	implements
		IActivityAgentsListener,
		IObjectsListener,
		IAgentsActionListener,
		IAgentsActivityListener,
		IObjectsVariableListener,
		WorldListener,
		SimulationListener,
		SituationListener,
		StartPauseFinishListener,
		ExperimentListener,
		IAnalysisListener {

	private ISituation _situation;
	private ISimulation _simulation;
	private IExperiment _experiment;

	public AbstractLogger() {
		super();
	}

	public void addedObject(WorldEvent event) {
	}

	public final synchronized void addSituationListeners(ISituation situation) {
		if (getSituation() == situation)
			return;

		setSituation(situation);

		situation.addSituationListener(this);

		IWorld world = situation.getWorld();
		world.addWorldListener(this);
	}

	public void currentActionChanged(AgentActionChangedEvent event) {
	}

	public void currentActivityGraphLeafChanged(CurrentActivityGraphLeafChangedEvent event) {
	}

	public void nextTimeStep(SituationEvent event) {
	}

	public void removedObject(WorldEvent event) {
	}

	public final synchronized void removeSituationListeners(ISituation situation) {
		situation.removeSituationListener(this);

		IWorld world = situation.getWorld();

		if (world != null)
			world.removeWorldListener(this);

		setSituation(null);
	}

	/**
	 * @param event StartPauseFinishEvent
	 * @see de.uniwue.ki.mas.sim.simulation.StartPauseFinishListener#finished(StartPauseFinishEvent)
	 */
	public final void finished(StartPauseFinishEvent event) {
		if (event.getSource() instanceof ISimulation)
			finishedSimulation((ISimulation) event.getSource());
		else
			finishedExperiment((IExperiment) event.getSource());
	}

	protected void finishedSimulation(ISimulation sim) {
		setSimulation(null);
	}

	protected void finishedExperiment(IExperiment exp) {
		setExperiment(null);
	}

	public final void paused(StartPauseFinishEvent event) {
		if (event.getSource() instanceof ISimulation)
			pausedSimulation((ISimulation) event.getSource());
		else
			pausedExperiment((IExperiment) event.getSource());
	}

	protected void pausedSimulation(ISimulation sim) {
		removeSituationListeners(sim.getSituation());
	}

	protected void pausedExperiment(IExperiment exp) {
	}

	public void reseted(SimulationEvent event) {
		addSituationListeners(((ISimulation) event.getSource()).getSituation());
	}

	public final void started(StartPauseFinishEvent event) {
		if (event.getSource() instanceof ISimulation)
			startedSimulation((ISimulation) event.getSource());
		else
			startedExperiment((IExperiment) event.getSource());
	}

	protected void startedSimulation(ISimulation sim) {
		setSimulation(sim);
	}

	protected void startedExperiment(IExperiment exp) {
	}

	public void setSituation(ISituation currSituation) {
		_situation = currSituation;
	}

	public ISituation getSituation() {
		return _situation;
	}

	public void setSimulation(ISimulation simulation) {
		if (getSimulation() != null)
			removeSituationListeners(getSimulation().getSituation());

		_simulation = simulation;

		if (simulation != null)
			addSituationListeners(simulation.getSituation());
	}

	public ISimulation getSimulation() {
		return _simulation;
	}

	public void currentSimulationChanged(ExperimentCurrentSimulationChangedEvent event) {
		if (event.getOldSimulation() != null) {

			event.getOldSimulation().removeSimulationListener(this);

			if (event.getNewSimulation() != null) {
				event.getNewSimulation().addSimulationListener(this);
			}
		}
	}

	public void setExperiment(IExperiment experiment) {
		_experiment = experiment;
	}

	public IExperiment getExperiment() {
		return _experiment;
	}

	public void actingFinished(ActivityAgentEvent event) {
	}

	public void actingStarts(ActivityAgentEvent event) {
	}

	public void reasoningFinished(ActivityAgentEvent event) {
	}

	public void reasoningStarts(ActivityAgentEvent event) {
	}

	public void updateFinished(ObjectEvent event) {
	}

	public void updateStarts(ObjectEvent event) {
	}

	public void variableValueChanged(VariableChangedEvent event) {
	}

	public void analysisFinished(AnalysisEvent analysisEvent) {
	}

	public void analysisStarted(AnalysisEvent analysisEvent) {
	}
}