package de.uniwue.ki.mas.sim.logging;
import java.io.PrintWriter;

import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.behaviour.activityGraph.IActivityGraphLeaf;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.agent.IActivityAgent;
import de.uniwue.ki.mas.entity.object.world.WorldEvent;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.sim.listener.*;
import de.uniwue.ki.mas.sim.simulation.ISimulation;
import de.uniwue.ki.mas.sim.simulation.SimulationEvent;
import de.uniwue.ki.mas.sim.situation.ISituation;
import de.uniwue.ki.mas.sim.situation.SituationEvent;

/**
 * Creation date: (20.10.2000 20:05:33)
 * @author Christoph Oechslein
 */
public final class StreamOutputLogger extends AbstractLogger {

	private static final int MAX_DEPTH = 10;

	private PrintWriter _printWriter;

	public StreamOutputLogger() {
		super();
	}

	public void addedObject(WorldEvent event) {
		super.addedObject(event);

		println(
			"Added '"
				+ event.getObject().getName()
				+ "' by '"
				+ event.getActiveObject().getName()
				+ "'",
			true);
	}

	public void currentActionChanged(AgentActionChangedEvent event) {
		super.currentActionChanged(event);

		IObject object = (IObject) event.getSource();
		println(
			"CurrentActionChanged from: "
				+ event.getOldCurrentAction().getName()
				+ " to: "
				+ event.getNewCurrentAction().getName()
				+ " by '"
				+ object.getName()
				+ "'",
			true);
	}

	public void currentActivityGraphLeafChanged(CurrentActivityGraphLeafChangedEvent event) {
		super.currentActivityGraphLeafChanged(event);

		IActivityAgent activityAgent = (IActivityAgent) event.getSource();
		IActivityGraphLeaf oldAtomicActivity = event.getOldActivityGraphLeaf();
		IActivityGraphLeaf newAtomicActivity = event.getNewActivityGraphLeaf();
		println(
			"CurrentActivityChanged from: '"
				+ ((oldAtomicActivity == null) ? "null" : oldAtomicActivity.getName())
				+ "' to: '"
				+ ((newAtomicActivity == null) ? "null" : newAtomicActivity.getName())
				+ "' by '"
				+ activityAgent.getName()
				+ "'",
			true);
	}

	public PrintWriter getPrintWriter() {
		return _printWriter;
	}

	public void nextTimeStep(SituationEvent event) {
		super.nextTimeStep(event);

		ISituation situation = (ISituation) event.getSource();
		println(
			"NextTimeStep in '"
				+ situation.getName()
				+ "' current Time: "
				+ event.getTimeDurationInTicks(),
			true);

		getPrintWriter().flush();
	}

	//	private void print(String string, boolean withTime) {
	//		// sometimes some objects won't be unregistered
	//		// Assert.assertNotNull(getSituation());
	//
	//		if (getSituation() != null)
	//			getPrintWriter().print(withTime ? ("[" + System.currentTimeMillis() + "]: " + string) : string);
	//	}

	private void println(String string, boolean withTime) {
		// sometimes some objects won't be unregistered
		// Assert.assertNotNull(getSituation());

		if (getSituation() != null)
			getPrintWriter().println(
				withTime ? ("[" + System.currentTimeMillis() + "]: " + string) : string);
	}

	public void removedObject(WorldEvent event) {
		super.removedObject(event);

		println(
			"Removed '"
				+ event.getObject().getName()
				+ "' by '"
				+ event.getActiveObject().getName()
				+ "'",
			true);
	}

	public void setPrintWriter(PrintWriter newPrintWriter) {
		_printWriter = newPrintWriter;
	}

	protected void finishedSimulation(ISimulation simulation) {
		super.finishedSimulation(simulation);

		println("Simulation '" + simulation.getName() + "' finished", true);
	}

	protected void pausedSimulation(ISimulation simulation) {
		super.pausedSimulation(simulation);

		println("Simulation '" + simulation.getName() + "' paused", true);
	}

	public void reseted(SimulationEvent event) {
		super.reseted(event);

		ISimulation simulation = (ISimulation) event.getSource();

		println("Simulation '" + simulation.getName() + "' reseted", true);
	}

	protected void startedSimulation(ISimulation simulation) {
		super.startedSimulation(simulation);

		println("Simulation '" + simulation.getName() + "' started", true);
	}

	public void variableValueChanged(VariableChangedEvent event) {
		super.variableValueChanged(event);

		IObject variableOwner = event.getBodyVariableOwner();
		IObject settingObject = event.getSettingObject();

		StringBuffer buff = new StringBuffer();

		IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration =
			variableOwner.getBody().getBodyVariableInstanceDeclarationFor(event.getBodyVariable());
		ISpecificType type =
			bodyVariableInstanceDeclaration
				.getBodyVariableClassDeclaration()
				.getVariableClassDeclaration()
				.getSpecificType();

		if (event.isDynamicUpdate())
			buff.append("Updated");
		else
			buff.append("Changed");

		buff.append(" value of variable '");

		bodyVariableInstanceDeclaration.appendName(buff, 0, MAX_DEPTH);

		buff.append("' from: '");
		type.appendNameForArgumentObject(buff, event.getOldValue(), 0, MAX_DEPTH);

		buff.append("' to: '");
		type.appendNameForArgumentObject(buff, event.getNewValue(), 0, MAX_DEPTH);

		buff.append("' of '");
		variableOwner.appendName(buff, 0, MAX_DEPTH);

		if ((variableOwner != settingObject) && (settingObject != null)) {
			buff.append("' by: '");
			settingObject.appendName(buff, 0, MAX_DEPTH);
		}

		buff.append("'");

		println(buff.toString(), true);
	}

	public void actingFinished(ActivityAgentEvent event) {
		super.actingFinished(event);

		println("Agent '" + event.getAgent().getName() + "' finished acting", true);
	}

	public void actingStarts(ActivityAgentEvent event) {
		super.actingStarts(event);

		println(
			"Agent '"
				+ event.getAgent().getName()
				+ "' starts acting of "
				+ event.getAgent().getCurrentAction().getName(),
			true);
	}

	public void reasoningFinished(ActivityAgentEvent event) {
		super.reasoningFinished(event);

		println("Agent '" + event.getAgent().getName() + "' finished reasoning", true);
	}

	public void reasoningStarts(ActivityAgentEvent event) {
		super.reasoningStarts(event);

		println("Agent '" + event.getAgent().getName() + "' starts reasoning", true);
	}

	public void updateFinished(ObjectEvent event) {
		super.updateFinished(event);

		println("Object '" + event.getObject().getName() + "' finished update", true);
	}

	public void updateStarts(ObjectEvent event) {
		super.updateStarts(event);

		println("Object '" + event.getObject().getName() + "' starts update", true);
	}

	//	public void analysisStarted(AnalysisEvent analysisEvent) {
	//		super.analysisStarted(analysisEvent);
	//
	//		println("Analysis '" + analysisEvent.getAnalysis().getName() + "' started.", true);
	//	}
	//
	//	public void analysisFinished(AnalysisEvent analysisEvent) {
	//		super.analysisFinished(analysisEvent);
	//
	//		println("Analysis '" + analysisEvent.getAnalysis().getName() + "' finished.", true);
	//	}
}