package de.uniwue.ki.mas.sim.simulation;
import java.util.Date;

import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.analysis.AnalysisList;
import de.uniwue.ki.mas.sim.situation.ISituation;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.name.INamed;

/**
 * An <code>ISimulation</code> is a simulation run with
 * a {@link ISituation specific situation}
 * and generates {@link SimulationEvent simulation events}.
 * Creation date: (05.12.2000 15:58:59)
 *
 * @author Christoph Oechslein
 */

public interface ISimulation extends INamed, IRemoveable, IStartPauseFinishAble, java.io.Serializable {

	/**
	 * Adds a new simulation listener.
	 *
	 * @see SimulationEvent
	 */
	void addSimulationListener(SimulationListener newListener);

	AnalysisList getAnalysisList();

	/**
	 * Returns the declaration of this runtime object.
	 */
	ISimulationDeclaration getSimulationDeclaration();

	/**
	 * Returns the strategie for this simulation
	 * (mostly update strategie for entailed {@link IObject objects}).
	 */

	ISimulationStrategie getSimulationStrategie();

	/**
	 * Get the current Situation for this simulation
	 */

	ISituation getSituation();

	/**
	 * Removes a new simulation listener.
	 *
	 * @see SimulationEvent
	 */
	void removeSimulationListener(SimulationListener newListener);

	/**
	 *
	 * @see #isSimulationRunning()
	 * @see #startSimulation()
	 * @see #pauseSimulation()
	 */
	void resetSimulation() throws SimulationRuntimeException;

	void setName(String newName);

	void startSimulation() throws SimulationRuntimeException;

	Throwable getException();

	Date getStartDate();

	boolean isUpdateWorldBeforeAllObjects();

	boolean isShouldTerminate();
}