package de.uniwue.ki.mas.sim.simulation;
import java.util.HashMap;
import java.util.Iterator;

import de.uniwue.ki.mas.sim.analysis.AnalysisDeclarationList;
import de.uniwue.ki.mas.sim.analysis.IAnalysisDeclaration;
import de.uniwue.ki.mas.sim.situation.*;
import de.uniwue.ki.mas.util.declaration.AbstractDeclarationFactory;
import de.uniwue.ki.mas.util.declaration.IDeclaration;

/**
 * Creation date: (04.09.2001 16:05:27)
 * @author Christoph Oechslein
 */

public class SimulationDeclarationFactory extends AbstractDeclarationFactory {
	private final SituationDeclarationFactory _situationDeclarationFactory;
	private final SituationDeclarationList _situationDeclarationList;
	private final AnalysisDeclarationList _analysisDeclarationList;

	public SimulationDeclarationFactory(
		SituationDeclarationList situationDeclarationList,
		SituationDeclarationFactory situationDeclarationFactory,
		AnalysisDeclarationList analysisDeclarationList) {
		super();

		_situationDeclarationList = situationDeclarationList;
		_situationDeclarationFactory = situationDeclarationFactory;
		_analysisDeclarationList = analysisDeclarationList;
	}

	public IDeclaration createDeclaration() {
		return createSimulationDeclaration();
	}

	public ISimulationDeclaration createSimulationDeclaration() {
		if (_situationDeclarationList.size() > 0)
			return createSimulationDeclaration(
				(ISituationDeclaration) _situationDeclarationList.getElementAt(0));
		else {
			ISituationDeclaration newSituationDeclaration =
				_situationDeclarationFactory.createSituationDeclaration();
			_situationDeclarationList.add(newSituationDeclaration);

			return createSimulationDeclaration(newSituationDeclaration);
		}
	}

	public ISimulationDeclaration createSimulationDeclaration(ISituationDeclaration situationDeclaration) {
		ISimulationDeclaration simulationDeclaration = new SimulationDeclaration(situationDeclaration);

		addDefaultAnalysisDeclarations(simulationDeclaration);

		return simulationDeclaration;
	}

	private void addDefaultAnalysisDeclarations(ISimulationDeclaration simulationDeclaration) {
		AnalysisDeclarationList analysisDeclarationList = simulationDeclaration.getAnalysisDeclarationList();

		Iterator it = _analysisDeclarationList.iterator();
		while (it.hasNext()) {
			IAnalysisDeclaration element = (IAnalysisDeclaration) it.next();
			if (element.isDefaultAnalysis() && (!analysisDeclarationList.contains(element))) {
				analysisDeclarationList.add(element);
			}
		}
	}

	public ISimulationWithNameFunctionCallDeclDeclaration createSimulationWithNameFunctionCallDeclDeclaration(ISituationDeclaration situationDeclaration) {
		ISimulationWithNameFunctionCallDeclDeclaration simulationDeclaration =
			new SimulationWithNameFunctionCallDeclDeclaration(situationDeclaration);

		addDefaultAnalysisDeclarations(simulationDeclaration);

		return simulationDeclaration;
	}

	public ISimulationWithNameFunctionCallDeclDeclaration createSimulationWithNameFunctionCallDeclDeclaration(ISimulationDeclaration simulationDeclaration) {
		HashMap objectMapping = new HashMap();

		if (simulationDeclaration instanceof ISimulationWithNameFunctionCallDeclDeclaration) {
			return (ISimulationWithNameFunctionCallDeclDeclaration) simulationDeclaration.copy(objectMapping);

		} else {
			SimulationWithNameFunctionCallDeclDeclaration declaration =
				new SimulationWithNameFunctionCallDeclDeclaration(
					(ISituationDeclaration) simulationDeclaration.getSituationDeclaration().copy(
						objectMapping));

			simulationDeclaration.copyInto(declaration, objectMapping);

			return declaration;
		}
	}

	public ISimulationWithNameFunctionCallDeclDeclaration createSimulationWithNameFunctionCallDeclDeclaration() {
		ISimulationWithNameFunctionCallDeclDeclaration simulationDeclaration =
			new SimulationWithNameFunctionCallDeclDeclaration(
				_situationDeclarationFactory.createSituationDeclaration());

		addDefaultAnalysisDeclarations(simulationDeclaration);

		return simulationDeclaration;
	}
}