package de.uniwue.ki.mas.sim.simulation;

/**
 * Creation date: 04.09.2002, 18:40:20
 * @author oechslein
 */
public class SimulationEventMulticaster implements SimulationListener {
	private final SimulationListener _a;
	private final SimulationListener _b;

	protected SimulationEventMulticaster(SimulationListener a, SimulationListener b) {
		super();

		_a = a;
		_b = b;
	}

	/**
	 * Add new listener to support multicast events.
	 */
	public static SimulationListener add(SimulationListener a, SimulationListener b) {
		return addInternal(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 */
	protected static SimulationListener addInternal(SimulationListener a, SimulationListener b) {
		if (a == null)
			return b;
		if (b == null)
			return a;
		return new SimulationEventMulticaster(a, b);
	}

	protected SimulationListener remove(SimulationListener oldl) {
		if (oldl == _a)
			return _b;
		if (oldl == _b)
			return _a;

		SimulationListener a2 = removeInternal(_a, oldl);
		SimulationListener b2 = removeInternal(_b, oldl);

		if (a2 == _a && b2 == _b)
			return this;

		return addInternal(a2, b2);
	}

	private static SimulationListener removeInternal(SimulationListener l, SimulationListener oldl) {
		if (l == oldl || l == null) {
			return null;
		} else if (l instanceof SimulationEventMulticaster) {
			return ((SimulationEventMulticaster) l).remove(oldl);
		} else {
			return l; // it's not here
		}
	}

	public static SimulationListener remove(SimulationListener l, SimulationListener oldl) {
		if (l == oldl || l == null)
			return null;
		if (l instanceof SimulationEventMulticaster)
			return ((SimulationEventMulticaster) l).remove(oldl);
		return l;
	}

	public void reseted(SimulationEvent event) {
		_a.reseted(event);
		_b.reseted(event);
	}
}