package de.uniwue.ki.mas.sim.simulation;
import de.uniwue.ki.mas.basics.function.IFunction;
import de.uniwue.ki.mas.basics.type.StringType;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.util.factory.AbstractObjectFactory;

/**
 * Creation date: (28.11.2001 13:47:22)
 * @author Christoph Oechslein
 */

public class SimulationFactory extends AbstractObjectFactory {
	private final SimulationDeclarationList _simulationDeclarationList;
	private final ISeSAmDeclaration _seSAmDeclaration;

	public SimulationFactory(
		SimulationDeclarationList simulationDeclarationList,
		ISeSAmDeclaration seSAmDeclaration) {
		super();

		_simulationDeclarationList = simulationDeclarationList;
		_seSAmDeclaration = seSAmDeclaration;
	}

	public final Object createObject() throws ModelCompilingException, SimulationRuntimeException {
		return createSimulation();
	}

	public final ISimulation createSimulation() throws ModelCompilingException {
		ISimulationDeclaration simulationDeclaration = getSimulationDeclaration();

		if (simulationDeclaration != null)
			return createThreadedSimulation(_seSAmDeclaration, getSimulationDeclaration());

		else
			return null;
	}

	public String getName() {
		return "Simulation Factory";
	}

	private final ISimulationDeclaration getSimulationDeclaration() {
		if (_simulationDeclarationList.isEmpty()) {
			return null;

		} else {
			return (ISimulationDeclaration) _simulationDeclarationList.get(0);
		}
	}

	public static final ISimulation createSimulation(
		ISimulationDeclaration simulationDeclaration,
		SimulationFunctionArgs simulationFunctionArgs)
		throws ModelCompilingException {

		Simulation simulation =
			new Simulation(
				getNameFunction(simulationDeclaration, simulationFunctionArgs),
				simulationDeclaration,
				simulationFunctionArgs);

		return simulation;
	}

	public static final IThreadedSimulation createThreadedSimulation(
		ISeSAmDeclaration seSAmDeclaration,
		ISimulationDeclaration simulationDeclaration)
		throws ModelCompilingException {

		return createThreadedSimulation(simulationDeclaration, new SimulationFunctionArgs(seSAmDeclaration));
	}

	public static final IThreadedSimulation createThreadedSimulation(
		ISimulationDeclaration simulationDeclaration,
		SimulationFunctionArgs simulationFunctionArgs)
		throws ModelCompilingException {

		ThreadedSimulation threadedSimulation =
			new ThreadedSimulation(
				getNameFunction(simulationDeclaration, simulationFunctionArgs),
				simulationDeclaration,
				simulationFunctionArgs);

		return threadedSimulation;
	}

	private static IFunction getNameFunction(
		ISimulationDeclaration simulationDeclaration,
		ISimulationCreateFunctionArgs simulationCreateFunctionArgs)
		throws ModelCompilingException {

		if (simulationDeclaration instanceof ISimulationWithNameFunctionCallDeclDeclaration) {
			ISimulationWithNameFunctionCallDeclDeclaration simulationWithNameFunctionCallDeclDeclaration =
				(ISimulationWithNameFunctionCallDeclDeclaration) simulationDeclaration;

			return simulationWithNameFunctionCallDeclDeclaration
				.getNameFunctionCallDeclaration()
				.createFunction(
				simulationCreateFunctionArgs);

		} else {
			return StringType.getInstance().createArgument(
				simulationCreateFunctionArgs,
				simulationDeclaration.getName());
		}
	}
}