package de.uniwue.ki.mas.sim.simulation;
import java.util.*;

import de.uniwue.ki.mas.entity.object.agent.ActivityAgentFunctionArgs;
import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.sim.analysis.IAnalysis;

/**
 * Creation date: (06.12.2000 17:12:12)
 * @author Christoph Oechslein
 */

public class SimulationFunctionArgs
	extends ActivityAgentFunctionArgs
	implements ISimulationExecuteFunctionArgs, ISimulationCreateFunctionArgs {

	private List _analysisListeners;

	private ISimulation _simulation;
	private IWorld _world;

	private ISeSAmDeclaration _seSAmDeclaration;

	public SimulationFunctionArgs(ISeSAmDeclaration seSAmDeclaration) {
		super();

		_seSAmDeclaration = seSAmDeclaration;
	}

	public ISimulation getSimulation() {
		return _simulation;
	}

	public IWorld getWorld() {
		return _world;
	}

	public void setSimulation(ISimulation newSimulation) {
		_simulation = newSimulation;
	}

	/**
	 * @param newSimWorld IWorld
	 * @see de.uniwue.ki.mas.entity.object.IWorldExecuteFunctionArgs#setWorld(IWorld)
	 */
	public void setWorld(IWorld newSimWorld) {
		// if (Assert.ASSERTION_ON) Assert.assertTrue(world == null || world == newSimWorld);
		_world = newSimWorld;
		// setObject(newSimWorld);
	}

	public ISeSAmDeclaration getSeSAmDeclaration() {
		return _seSAmDeclaration;
	}

	public void addAnalysisListener(IAnalysisListener analysisListener) {
		if (_analysisListeners == null)
			_analysisListeners = new LinkedList();

		_analysisListeners.add(analysisListener);
	}

	public boolean hasAnalysisListeners() {
		return (_analysisListeners != null) && (!_analysisListeners.isEmpty());
	}

	public void removeAnalysisListener(IAnalysisListener analysisListener) {
		if (_analysisListeners != null)
			_analysisListeners.remove(analysisListener);
	}

	public void fireStartedAnalysis(IAnalysis analysis, ISimulation simulation) {
		if (hasAnalysisListeners()) {
			AnalysisEvent analysisEvent = new AnalysisEvent(analysis, simulation);
			Iterator it = _analysisListeners.iterator();
			while (it.hasNext()) {
				IAnalysisListener element = (IAnalysisListener) it.next();
				element.analysisStarted(analysisEvent);
			}
		}
	}
	public void fireFinishedAnalysis(IAnalysis analysis, ISimulation simulation) {
		if (hasAnalysisListeners()) {
			AnalysisEvent analysisEvent = new AnalysisEvent(analysis, simulation);
			Iterator it = _analysisListeners.iterator();
			while (it.hasNext()) {
				IAnalysisListener element = (IAnalysisListener) it.next();
				element.analysisFinished(analysisEvent);
			}
		}

	}

	public void remove() {
		_analysisListeners = null;
		_seSAmDeclaration = null;
		_simulation = null;
		_world = null;

		super.remove();
	}
}