package de.uniwue.ki.mas.sim.simulation;


/**
 * Creation date: (31.01.2002 10:59:34)
 * @author Christoph Oechslein
 */

public class StartPauseFinishEventMulticaster implements StartPauseFinishListener {
	protected final StartPauseFinishListener _a;
	protected final StartPauseFinishListener _b;

	protected StartPauseFinishEventMulticaster(StartPauseFinishListener a, StartPauseFinishListener b) {
		super();

		_a = a;
		_b = b;
	}

	public static StartPauseFinishListener add(StartPauseFinishListener a, StartPauseFinishListener b) {
		if (a == null)
			return b;
		if (b == null)
			return a;
		return new StartPauseFinishEventMulticaster(a, b);
	}

	private StartPauseFinishListener remove(StartPauseFinishListener oldl) {
		if (oldl == _a)
			return _b;
		if (oldl == _b)
			return _a;
		StartPauseFinishListener a2 = removeInternal(_a, oldl);
		StartPauseFinishListener b2 = removeInternal(_b, oldl);
		if (a2 == _a && b2 == _b)
			return this;
		return add(a2, b2);
	}

	protected static StartPauseFinishListener removeInternal(
		StartPauseFinishListener l,
		StartPauseFinishListener oldl) {
		if (l == oldl || l == null) {
			return null;
		} else if (l instanceof StartPauseFinishEventMulticaster) {
			return ((StartPauseFinishEventMulticaster) l).remove(oldl);
		} else {
			return l; // it's not here
		}
	}

	/**
	 * @param l StartPauseFinishListener
	 * @param oldl StartPauseFinishListener
	 * @return StartPauseFinishListener
	 */
	public static StartPauseFinishListener remove(StartPauseFinishListener l, StartPauseFinishListener oldl) {
		if (l == oldl || l == null)
			return null;
		if (l instanceof StartPauseFinishEventMulticaster)
			return ((StartPauseFinishEventMulticaster) l).remove(oldl);
		return l;
	}

	/**
	 * @param event StartPauseFinishEvent
	 * @see de.uniwue.ki.mas.sim.simulation.StartPauseFinishListener#finished(StartPauseFinishEvent)
	 */
	public void finished(StartPauseFinishEvent event) {
		_a.finished(event);
		_b.finished(event);
	}

	public void paused(StartPauseFinishEvent event) {
		_a.paused(event);
		_b.paused(event);
	}

	/**
	 * @param event StartPauseFinishEvent
	 * @see de.uniwue.ki.mas.sim.simulation.StartPauseFinishListener#started(StartPauseFinishEvent)
	 */
	public void started(StartPauseFinishEvent event) {
		_a.started(event);
		_b.started(event);
	}
}