package de.uniwue.ki.mas.sim.situation;
import java.beans.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.event.StateChangeMulticaster;

/**
 * Creation date: (13.03.2002 17:54:21)
 * @author Christoph Oechslein
 */

abstract class AbstractSituation extends AbstractCopyable implements ISituation {
	private String _documentation = "";
	private String _shortDescription = "";
	private boolean _usesShortDescription = false;
	private transient ChangeListener _changeListener = null;
	private transient PropertyChangeSupport _propertyChange;

	public void addChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.add(_changeListener, newListener);
	}

	public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
		getPropertyChange().addPropertyChangeListener(listener);
	}

	public synchronized void addPropertyChangeListener(
		String propertyName,
		PropertyChangeListener listener) {
		getPropertyChange().addPropertyChangeListener(propertyName, listener);
	}

	protected void firePropertyChange(PropertyChangeEvent evt) {
		getPropertyChange().firePropertyChange(evt);
	}

	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		getPropertyChange().firePropertyChange(propertyName, oldValue, newValue);
	}

	protected void fireStateChanged() {
		if (_changeListener != null) {
			_changeListener.stateChanged(new ChangeEvent(this));
		}
	}

	public String getDocumentation() {
		return _documentation;
	}

	public String getShortDescription() {
		return _shortDescription;
	}

	public boolean getUsesShortDescription() {
		return _usesShortDescription;
	}

	public boolean isUsesShortDescription() {
		return _usesShortDescription;
	}

	public void removeChangeListener(ChangeListener listener) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, listener);
	}

	public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
		getPropertyChange().removePropertyChangeListener(listener);
	}

	public synchronized void removePropertyChangeListener(
		String propertyName,
		PropertyChangeListener listener) {
		getPropertyChange().removePropertyChangeListener(propertyName, listener);
	}

	public void setShortDescription(String newShortDescription) {
		_shortDescription = newShortDescription;
	}

	public void setUsesShortDescription(boolean newUsesShortDescription) {
		_usesShortDescription = newUsesShortDescription;
	}

	public void setDocumentation(String documentation) {
		_documentation = documentation;
	}

	public void remove() {
		_propertyChange = null;
		_changeListener = null;
	}

	private PropertyChangeSupport getPropertyChange() {
		if (_propertyChange == null) {
			_propertyChange = new PropertyChangeSupport(this);
		}

		return _propertyChange;
	}
}