package de.uniwue.ki.mas.sim.situation;
import java.io.Serializable;

import de.uniwue.ki.mas.entity.object.world.IWorld;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.name.IDocumentable;

/**
 * A Situation has a clock and one {@link IWorld world},
 * it also generates situation events.
 * Creation date: (30.10.2000 10:36:21)
 *
 * @author Christoph Oechslein
 */

public interface ISituation extends IDocumentable, IRemoveable, Serializable {

	/**
	 * Adds new situation listener to listen for situation events: clock advance.
	 *
	 * @see #removeSituationListener(SituationListener)
	 * @see SituationEvent
	 */
	void addSituationListener(SituationListener newListener);

	/**
	 * Returns the associated world.
	 */

	IWorld getWorld();

	/**
	 * Advance time to next step.
	 */
	void nextTimeStep();

	/**
	 * Removes new situation listener to listen for situation events.
	 *
	 * @see #addSituationListener(SituationListener)
	 * @see SituationEvent
	 */
	void removeSituationListener(SituationListener newListener);

	/**
	 * Resets the situation to default values.
	 */
	void reset(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException;

	/**
	 * Returns the current time in ticks.
	 */
	int getTimeDurationInTicks();
}