package de.uniwue.ki.mas.sim.situation;
import de.uniwue.ki.mas.entity.object.IObject;
import de.uniwue.ki.mas.entity.object.world.*;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.sim.simulation.ISimulationExecuteFunctionArgs;

/**
 * Creation date: (13.03.2002 17:57:44)
 * @author Christoph Oechslein
 */

final class Situation extends AbstractSituation {
	private IWorld _world;
	private int _timeInTicks = 0;

	private transient SituationListener _situationListener;

	public Situation(
		IWorldInstanceDeclaration worldInstanceDeclaration,
		IWorldCreateFunctionArgs worldCreateFunctionArgs)
		throws ModelCompilingException {
		super();

		_world = (IWorld) worldInstanceDeclaration.createObject(worldCreateFunctionArgs);
		_world.initialize(worldCreateFunctionArgs);
	}

	public void addSituationListener(SituationListener newListener) {
		_situationListener = SituationEventMulticaster.add(_situationListener, newListener);
	}

	protected void fireNextTimeStep() {
		if (_situationListener != null)
			_situationListener.nextTimeStep(new SituationEvent(this, getTimeDurationInTicks()));
	}

	protected void fireNextTimeStep(SituationEvent event) {
		if (_situationListener != null)
			_situationListener.nextTimeStep(event);
	}

	public String getName() {
		return "Sit: " + getWorld().getName();
	}

	public int getTimeDurationInTicks() {
		return _timeInTicks;
	}

	public IWorld getWorld() {
		return _world;
	}

	public void nextTimeStep() {
		_timeInTicks++;
		fireNextTimeStep();
	}

	public void remove() {
		if (_world != null) {
			_world.remove();
		}

		_timeInTicks = -1;
		_world = null;
		_situationListener = null;

		super.remove();
	}

	public void removeSituationListener(SituationListener newListener) {
		_situationListener = SituationEventMulticaster.remove(_situationListener, newListener);
	}

	public void reset(ISimulationExecuteFunctionArgs simulationExecuteFunctionArgs)
		throws SimulationRuntimeException {

		IObject currentActiveObject = simulationExecuteFunctionArgs.getObject();
		IWorld currentActiveWorld = simulationExecuteFunctionArgs.getWorld();

		try {
			simulationExecuteFunctionArgs.setWorld(getWorld());
			simulationExecuteFunctionArgs.setObject(getWorld());

			getWorld().reset(simulationExecuteFunctionArgs);

			_timeInTicks = 0;

		} finally {
			simulationExecuteFunctionArgs.setWorld(currentActiveWorld);
			simulationExecuteFunctionArgs.setObject(currentActiveObject);
		}
	}

	public void setWorld(IWorld world) {
		_world = world;
	}
}