package de.uniwue.ki.mas.sim.situation;
import java.util.HashMap;
import java.util.Map;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.sim.simulation.ISimulationCreateFunctionArgs;
import de.uniwue.ki.mas.util.declaration.AbstractDocumentableDeclaration;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;

/**
 * Creation date: (12.09.2001 10:14:54)
 * @author Christoph Oechslein
 */

class SituationDeclaration
	extends AbstractDocumentableDeclaration
	implements ISituationDeclaration {
	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	private IWorldInstanceDeclaration fieldWorldInstanceDeclaration = null;

	public SituationDeclaration(IWorldInstanceDeclaration worldInstanceDeclaration) {
		super();

		setUsesShortDescription(true);
		setShortDescription("Untitled Situation");

		setWorldInstanceDeclaration(worldInstanceDeclaration);
	}

	public SituationDeclaration(IWorldClassDeclaration worldClassDeclaration) {
		this(worldClassDeclaration.createWorldInstanceDeclaration());
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		if (objectMapping == null)
			objectMapping = new HashMap();

		super.copyInto(intoObject, objectMapping);

		ISituationDeclaration situationDeclaration = (ISituationDeclaration) intoObject;

		getWorldInstanceDeclaration().copyInto(
			situationDeclaration.getWorldInstanceDeclaration(),
			objectMapping);
	}

	protected ICopyable createInstance(Map objectMapping) {
		return new SituationDeclaration(getWorldInstanceDeclaration().getWorldClassDeclaration());
	}

	public ISituation createSituation(ISimulationCreateFunctionArgs simulationCreateFunctionArgs)
		throws ModelCompilingException {
		return new Situation(getWorldInstanceDeclaration(), simulationCreateFunctionArgs);
	}

	protected void fireStateChanged() {
		super.fireStateChanged();
	}

	public IWorldInstanceDeclaration getWorldInstanceDeclaration() {
		return fieldWorldInstanceDeclaration;
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof ISituationDeclaration)) {
			return false;
		}

		ISituationDeclaration situationDeclaration = (ISituationDeclaration) anObject;

		return isomorphicEquals(
			getWorldInstanceDeclaration(),
			situationDeclaration.getWorldInstanceDeclaration());
	}

	public int isomorphicHashCode() {
		return 3 * super.isomorphicHashCode() + getWorldInstanceDeclaration().isomorphicHashCode();
	}

	public void remove() {
		//TODO ChangeListener entfernen
		remove(getWorldInstanceDeclaration());
		super.remove();
	}

	/**
	 * @param worldInstanceDeclaration IWorldInstanceDeclaration
	 * @see de.uniwue.ki.mas.sim.situation.ISituationDeclaration#setWorldInstanceDeclaration(IWorldInstanceDeclaration)
	 */
	public void setWorldInstanceDeclaration(IWorldInstanceDeclaration worldInstanceDeclaration) {
		IWorldInstanceDeclaration oldValue = fieldWorldInstanceDeclaration;

		if (oldValue != null) {
			oldValue.removeChangeListener(_changeListener);
		}

		fieldWorldInstanceDeclaration = worldInstanceDeclaration;

		if (worldInstanceDeclaration != null) {
			worldInstanceDeclaration.addChangeListener(_changeListener);
			//worldInstanceDeclaration.removeChangeListener(_changeListener);
		}

		firePropertyChange("worldInstanceDeclaration", oldValue, worldInstanceDeclaration);
		fireStateChanged();
	}
}