package de.uniwue.ki.mas.sim.situation;


/**
 * Creation date: (20.02.2002 16:47:28)
 * @author Christoph Oechslein
 */

public class SituationEventMulticaster implements SituationListener {
	private SituationListener _situationListenerA;
	private SituationListener _situationListenerB;

	private SituationEventMulticaster(SituationListener a, SituationListener b) {
		super();

		_situationListenerA = a;
		_situationListenerB = b;
	}

	public static SituationListener add(SituationListener a, SituationListener b) {
		if (a == null)
			return b;
		if (b == null)
			return a;

		return new SituationEventMulticaster(a, b);
	}

	public void nextTimeStep(SituationEvent event) {
		_situationListenerA.nextTimeStep(event);
		_situationListenerB.nextTimeStep(event);
	}

	private SituationListener remove(SituationListener oldl) {
		if (oldl == _situationListenerA)
			return _situationListenerB;

		if (oldl == _situationListenerB)
			return _situationListenerA;

		SituationListener a2 = removeInternal(_situationListenerA, oldl);
		SituationListener b2 = removeInternal(_situationListenerB, oldl);

		if (a2 == _situationListenerA && b2 == _situationListenerB)
			return this;
		else
			return SituationEventMulticaster.add(a2, b2);
	}

	protected static SituationListener removeInternal(SituationListener l, SituationListener oldl) {
		if (l == oldl || l == null) {
			return null;
		} else if (l instanceof SituationEventMulticaster) {
			return ((SituationEventMulticaster) l).remove(oldl);
		} else {
			return l; // it's not here
		}
	}

	/**
	 * @param l SituationListener
	 * @param oldl SituationListener
	 * @return SituationListener
	 */
	public static SituationListener remove(SituationListener l, SituationListener oldl) {
		if (l == oldl || l == null)
			return null;

		if (l instanceof SituationEventMulticaster)
			return ((SituationEventMulticaster) l).remove(oldl);

		else
			return l;
	}
}