package de.uniwue.ki.mas.util.declaration;
import java.util.Map;

import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
/**
 * Creation date: (16.05.00 16:37:00)
 * @author Christoph Oechslein
 */
public abstract class AbstractDocumentableDeclaration
	extends AbstractDeclaration
	implements IDocumentableDeclaration {
		
	private String _shortDescription = "";
	private String _documentation = "";
	private boolean _usesShortDescription = false;

	public AbstractDocumentableDeclaration() {
		super();
	}

	public AbstractDocumentableDeclaration(String name) {
		super();

		setShortDescription(name);
		setUsesShortDescription(true);
	}

	public AbstractDocumentableDeclaration(boolean initializeDefaultNameAsShortDescription) {
		super();

		if (initializeDefaultNameAsShortDescription) {
			setShortDescription(getDefaultName());
			setUsesShortDescription(true);
		}
	}

	public void copyInto(Object intoObject, Map objectMapping) {
		super.copyInto(intoObject, objectMapping);

		AbstractDocumentableDeclaration documentableDeclaration =
			(AbstractDocumentableDeclaration) intoObject;

		documentableDeclaration.setUsesShortDescription(getUsesShortDescription());
		documentableDeclaration.setShortDescription(getShortDescription());

		documentableDeclaration.setDocumentation(getDocumentation());
	}

	protected String getDefaultName() {
		return "Untitled";
	}

	public String getName() {
		return getUsesShortDescription() ? getShortDescription() : getDefaultName();
	}

	public boolean isomorphicEquals(IIsomorphicHashable anObject) {
		if (anObject == this)
			return true;

		if (!super.isomorphicEquals(anObject)) {
			return false;
		}

		if (!(anObject instanceof IDocumentableDeclaration)) {
			return false;
		}

		IDocumentableDeclaration documentableDeclaration = (IDocumentableDeclaration) anObject;

		if (getUsesShortDescription() != documentableDeclaration.getUsesShortDescription()) {
			return false;
		}

		if (!Utils.equals(getShortDescription(), documentableDeclaration.getShortDescription())) {
			return false;
		}

		/*
		if (!Utils.equals(getDocumentation(), documentableDeclaration.getDocumentation())) {
		   return false;
		}
		*/

		return true;
	}

	public int isomorphicHashCode() {
		return 5 * super.isomorphicHashCode() + 3 * getShortDescription().hashCode()
		// + 5 * getDocumentation().hashCode()
		+ (getUsesShortDescription() ? 0 : 1);
	}

	public void setDocumentation(String documentation) {
		
//		System.out.println("AbstractDocumentable: "+documentation);
		
		String oldValue = _documentation;

		_documentation = documentation;

		if (!Utils.equals(oldValue, documentation))
			fireStateChanged();
	}

	public void setShortDescription(String shortDescription) {
		String oldValue = _shortDescription;

		_shortDescription = shortDescription;

		if (!Utils.equals(oldValue, shortDescription))
			fireStateChanged();
	}

	public void setUsesShortDescription(boolean usesShortDescription) {
		boolean oldValue = _usesShortDescription;

		_usesShortDescription = usesShortDescription;

		if (oldValue != usesShortDescription)
			fireStateChanged();
	}

	public String toString() {
		return getClass().getName()
			+ "@"
			+ hashCode()
			+ " ["
			+ (getUsesShortDescription() ? getShortDescription() : getName())
			+ "]";
	}

	public String getDocumentation() {
		return _documentation;
	}

	public String getShortDescription() {
		return _shortDescription;
	}

	public boolean getUsesShortDescription() {
		return _usesShortDescription;
	}
}