package de.uniwue.ki.mas.util.declaration;

import java.util.Iterator;

/**
 * Look at a declaration list and map all elements by {@link #mapElement(Object)}.
 * Creation date: (02.11.2000 11:13:05)
 *
 * @author Christoph Oechslein
 */
public abstract class AbstractMapCopyDeclarationList
	extends AbstractCopyDeclarationList
	implements IDeclarationList {

	public AbstractMapCopyDeclarationList(IDeclarationList delegateDeclarationList) {
		this(delegateDeclarationList, true);
	}

	public AbstractMapCopyDeclarationList(
		IDeclarationList delegateDeclarationList,
		boolean shouldSynchronize) {
		super(delegateDeclarationList);
		if (shouldSynchronize)
			synchronize();
	}

	public abstract Object mapElement(Object newAddedObject);

	protected void contentsChanged(int index0, int index1) {
		for (int i = index0; i <= index1; i++) {
			getCopyDeclarationList().set(i, mapElement(getDelegateDeclarationList().get(i)));
		}

		fireContentsChanged(this, index0, index1);
	}

	protected void intervalAdded(int index0, int index1) {
		for (int i = index0; i <= index1; i++) {
			if (i >= getCopyDeclarationList().size())
				getCopyDeclarationList().add(mapElement(getDelegateDeclarationList().get(i)));
			else
				getCopyDeclarationList().add(i, mapElement(getDelegateDeclarationList().get(i)));
		}

		fireIntervalAdded(this, index0, index1);
	}

	protected void intervalRemoved(int index0, int index1) {
		//for (int i = index0; i < Math.min(index1 + 1, size()); i++) {
		for (int i = index0; i <= index1; i++) {
			getCopyDeclarationList().remove(i);
		}

		fireIntervalRemoved(this, index0, index1);
	}

	protected void synchronize() {
		Iterator it = getDelegateDeclarationList().iterator();
		while (it.hasNext()) {
			getCopyDeclarationList().add(mapElement(it.next()));
		}
	}

	public boolean accept(Object anObject) {
		return true;
	}

	public boolean canBeRemoved() {
		return true;
	}
}
