package de.uniwue.ki.mas.util.declaration;
import java.util.EventObject;

/**
 * This is the event class to support the {@link DeclarationListListener} interface.
 * @author oechslein
 */
public class DeclarationListEvent extends EventObject {
	private static final int ADDED_DECLARATION = 0;
	private static final int REMOVED_DECLARATION = 1;
	
	private final int _type;
	private final IDeclaration _declaration;
	private final int _index;

	public DeclarationListEvent(IDeclarationList source, int type, IDeclaration declaration, int index) {
	   super(source);

	   _type = type;
	   _declaration = declaration;
	   _index = index;
	}

public static final DeclarationListEvent createAddedDeclarationListEvent(
   IDeclarationList source,
   IDeclaration declaration,
   int index) {
   return new DeclarationListEvent(source, ADDED_DECLARATION, declaration, index);
}

public static final DeclarationListEvent createRemovedDeclarationListEvent(
   IDeclarationList source,
   IDeclaration declaration,
   int index) {
   return new DeclarationListEvent(source, REMOVED_DECLARATION, declaration, index);
}

	/**
	 * Returns _type of this _declaration event: added or deleted event.
	 *
	 * @see #ADDED_DECLARATION
	 * @see #REMOVED_DECLARATION
	 */
	public final int getType() {
	    return _type;
	}

	/**
	 * @return IDeclaration
	 */
	public final IDeclaration getDeclaration() {
	    return _declaration;
	}

	/**
	 * @return IDeclarationList
	 */
	public final IDeclarationList getDeclarationList() {
	   return (IDeclarationList) getSource();
	}

	/**
	 * @return int
	 */
	public final int getIndex() {
	    return _index;
	}
}
