package de.uniwue.ki.mas.util.declaration;
import java.util.Iterator;

import javax.swing.event.*;

import junit.framework.Assert;

/**
 * Adapter for using a declaration list as a mutable combobox model.
 * Creation date: (19.06.2000 19:20:15)

 * @author Christoph Oechslein
 */

public class DeclarationListMutableComboBoxModel
	extends AbstractDeclarationList
	implements javax.swing.MutableComboBoxModel {

	private boolean _compositionLink;

	private IDeclarationList ivjDeclarationList = null;
	private Object selectedItem = null;

	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();

	private final class IvjEventHandler implements ChangeListener, ListDataListener {
		public void contentsChanged(ListDataEvent e) {
			if (e.getSource() == DeclarationListMutableComboBoxModel.this.getDeclarationList())
				fireContentsChanged(e);
		}

		public void intervalAdded(ListDataEvent e) {
			if (e.getSource() == DeclarationListMutableComboBoxModel.this.getDeclarationList())
				fireIntervalAdded(e);
		}

		public void intervalRemoved(ListDataEvent e) {
			if (e.getSource() == DeclarationListMutableComboBoxModel.this.getDeclarationList())
				fireIntervalRemoved(e);
		}

		public void stateChanged(ChangeEvent e) {
			if (e.getSource() == DeclarationListMutableComboBoxModel.this.getDeclarationList())
				fireStateChanged();
		}
	}

	public DeclarationListMutableComboBoxModel() {
		super();
		initialize();
	}

	public boolean accept(Object declaration) {
		return getDeclarationList().accept(declaration);
	}

	public void add(int index, Object element) {
		getDeclarationList().add(index, element);
		if (getSize() == 1 && getSelectedItem() == null && element != null) {
			setSelectedItem(element);
		}
	}

	public void addElement(Object anObject) {
		add(size(), anObject);
	}

	public boolean canBeRemoved() {
		return true;
	}

	protected void fireContentsChanged(ListDataEvent evt) {
		fireContentsChanged(this, evt.getIndex0(), evt.getIndex1());
	}

	protected void fireIntervalAdded(ListDataEvent evt) {
		fireIntervalAdded(this, evt.getIndex0(), evt.getIndex1());
	}

	protected void fireIntervalRemoved(ListDataEvent evt) {
		fireIntervalRemoved(this, evt.getIndex0(), evt.getIndex1());
	}

	public Object get(int index) {
		return getElementAt(index);
	}

	public IDeclarationList getDeclarationList() {

		return ivjDeclarationList;
	}

	public Object getElementAt(int index) {
		if (index >= 0 && index < getSize())
			return getDeclarationList().getElementAt(index);
		else
			return null;
	}

	public Object getSelectedItem() {
		return selectedItem;
	}

	public int getSize() {
		if (getDeclarationList() == null)
			return 0;
		else
			return getDeclarationList().getSize();
	}

	private void initConnections() {
	}

	private void initialize() {
		initConnections();
	}

	public void insertElementAt(Object obj, int index) {
		getDeclarationList().add(index, obj);
	}

	public boolean isComposition() {
		return _compositionLink;
	}

	public Iterator iterator() {
		return getDeclarationList().iterator();
	}

	public Object remove(int index) {
		if (getElementAt(index) == getSelectedItem()) {
			if (index == 0) {
				if (getSize() == 1)
					setSelectedItem(null);
				else
					setSelectedItem(getElementAt(1));
			} else
				setSelectedItem(getElementAt(index - 1));
		}
		return getDeclarationList().remove(index);
	}

	public void removeAllElements() {
		final int firstIndex = 0;
		final int lastIndex = getSize() - 1;
		getDeclarationList().clear();
		selectedItem = null;
		fireIntervalRemoved(this, firstIndex, lastIndex);
	}

	public void removeElement(Object anObject) {
		final int index = getDeclarationList().indexOf(anObject);
		if (index != -1) {
			removeElementAt(index);
		}
	}

	public void removeElementAt(int index) {
		remove(index);
	}

	public Object set(int index, Object element) {
		return getDeclarationList().set(index, element);
	}

	public void setComposition(boolean newIsComposition) {
		_compositionLink = newIsComposition;
	}

	public void setDeclarationList(IDeclarationList newValue) {
		if (ivjDeclarationList != newValue) {
			IDeclarationList oldValue = getDeclarationList();

			if (ivjDeclarationList != null) {
				ivjDeclarationList.removeListDataListener(ivjEventHandler);
				ivjDeclarationList.removeChangeListener(ivjEventHandler);
			}
			ivjDeclarationList = newValue;

			if (ivjDeclarationList != null) {
				ivjDeclarationList.addListDataListener(ivjEventHandler);
				ivjDeclarationList.addChangeListener(ivjEventHandler);
			}

			if (oldValue != null)
				fireIntervalRemoved(this, 0, oldValue.size());
			if (newValue != null) {
				if (getSize() > 0) {
					selectedItem = getElementAt(0);
					fireIntervalAdded(this, 0, newValue.size());
				}
			}
		}
	}

	public void setSelectedItem(Object anItem) {
		if (Assert.ASSERTION_ON)
			Assert.assertTrue((anItem == null) || (anItem instanceof IDeclaration));

		if (selectedItem != anItem) {
			selectedItem = anItem;

			// fire Change Event to fire an itemSelectionChangedEvent
			fireContentsChanged(this, -1, -1);
		}
	}

	public int size() {
		return getDeclarationList().size();
	}
}