package de.uniwue.ki.mas.util.declaration;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Map;

import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.copy.AbstractCopyable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.name.INamed;

/**
 * A declaration has an id and a name, can inform listeners if something changed and
 * can be informed if the will be removed.
 * <BR>Creation date: (31.03.00 13:19:48)
 *
 * @author Christoph Oechslein
 */

public interface IDeclaration extends ICopyable, INamed, IRemoveable, IIsomorphicHashable, Serializable {
	/**
	* Return <code>null</code> for all methods (NULL pattern).
	*/
	IDeclaration NULL_DECLARATION = new NullDeclarationClass();
	public class NullDeclarationClass extends AbstractCopyable implements IDeclaration {

		public boolean isomorphicEquals(IIsomorphicHashable anObject) {
			if (!(anObject instanceof IDeclarationList)) {
				return false;
			} else {

				return CollectionsHelper.isEmpty(((IDeclarationList)anObject).iterator());
			}
		}

		public int isomorphicHashCode() {
			return 42;
		}

		public ICopyable copy(Map objectMapping) {
			return this;
		}

		public void addChangeListener(ChangeListener newListener) {
		}

		public void addPropertyChangeListener(PropertyChangeListener listener) {
		}

		public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
		}

		public void addRemoveVetoListener(RemoveVetoListener newListener) {
		}

		public void removeAllRemoveVetoListeners() {
		}

		public boolean canBeRemoved() {
			return false;
		}

		public String getName() {
			return null;
		}

		public boolean hasListeners(String propertyName) {
			return false;
		}

		public void remove() {
		}

		public void removeChangeListener(ChangeListener newListener) {
		}

		public void removePropertyChangeListener(PropertyChangeListener listener) {
		}

		public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
		}

		public void removeRemoveVetoListener(RemoveVetoListener newListener) {
		}
		
		public boolean hasTODOFlag() {
			return false;
		}
		
		public void setTODOFlag(boolean flag) {
		}
	}

	void addChangeListener(ChangeListener newListener);

	void addPropertyChangeListener(PropertyChangeListener listener);

	void addPropertyChangeListener(String propertyName, PropertyChangeListener listener);

	void addRemoveVetoListener(RemoveVetoListener newListener);

	boolean canBeRemoved();

	void removeChangeListener(ChangeListener newListener);

	void removePropertyChangeListener(PropertyChangeListener listener);

	void removePropertyChangeListener(String propertyName, PropertyChangeListener listener);

	void removeRemoveVetoListener(RemoveVetoListener newListener);
	
	boolean hasTODOFlag();
	
	void setTODOFlag(boolean flag);
}