package de.uniwue.ki.mas.util.declaration;
import java.io.Serializable;

import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.copy.ICopyable;
import de.uniwue.ki.util.hash.IIsomorphicHashable;
import de.uniwue.ki.util.list.IMutableListModel;

/**
 * A list of declarations with helper methods,
 * for example methods for removing a declaration list.
 *
 * @see AbstractDeclarationList
 * @see DeclarationList
 */

public interface IDeclarationList
	extends IMutableListModel, ICopyable, IIsomorphicHashable, IRemoveable, Serializable {
	IDeclarationList EMPTY_DECLARATION_LIST = DeclarationListHelper.EMPTY_DECLARATION_LIST;

	/**
	 * Add a change listener, get informed if the list changes and
	 * if one declaration in the list changes (fires a change event).
	 */
	void addChangeListener(ChangeListener newListener);

	/**
	 *
	 * @param newListener de.uniwue.ki.mas.model.util.declaration.DeclarationListListener
	 */
	void addDeclarationListListener(DeclarationListListener newListener);

	boolean canBeRemoved(IDeclaration declaration);

	void clear(boolean withCallingRemove);

	/**
	 * Returns <code>true</code> if the given declaration exists in this list.
	 *
	 * @see #containsDeclaration(IDeclaration)
	 */
	boolean containsDeclaration(String name);

	/**
	 * Returns declaration with the given name.
	 *
	 * @see #getDeclarationByName(int)
	 * @see #getDeclarationByName(Integer)
	 */
	IDeclaration getDeclarationByName(String name);

	/**
	  * Gets the aggregation type of this list, it returns <code>true</code> if the list is
	  * a composition of objects, that is if the list is removed (by {@link #remove()}), remove
	  * all declarations by calling its {@link IDeclaration#remove()} method.
	  *
	  * @see #setComposition(boolean);
	  * @see #remove();
	  * @see IDeclaration#remove();
	  */
	boolean isComposition();

	Object remove(int index, boolean withCallingRemove);

	void remove(IDeclaration declaration, boolean withCallingRemove);

	/**
	 * Remove a change listener.
	 */
	void removeChangeListener(ChangeListener newListener);

	/**
	 *
	 * @param newListener de.uniwue.ki.mas.model.util.declaration.DeclarationListListener
	 */
	void removeDeclarationListListener(DeclarationListListener newListener);

	/**
	  * Sets the aggregation type of this list to composition, if <code>true</code> {@link #remove()} remove
	  * all declarations by calling its {@link IDeclaration#remove()} method.
	  *
	  * @see #IsComposition();
	  * @see #remove();
	  * @see IDeclaration#remove();
	  */
	void setComposition(boolean newIsComposition);
}