package de.uniwue.ki.mas.util.declaration;
import java.lang.ref.WeakReference;
import java.util.Iterator;

import de.uniwue.ki.util.event.AbstractListenerMulticaster;

/**
 * Creation date: (30.10.2001 18:21:16)
 * @author Christoph Oechslein
 */
public final class RemoveVetoMulticaster extends AbstractListenerMulticaster implements RemoveVetoListener {
	private static final ListenerMulticasterFactory _listenerMulticasterFactory =
		new ListenerMulticasterFactory() {
		public AbstractListenerMulticaster createAbstractListenerMulticaster() {
			return new RemoveVetoMulticaster();
		}

		public Object createObject() {
			return createAbstractListenerMulticaster();
		}

		public String getName() {
			return "RemoveVetoMulticasterFactory";
		}
	};

	protected RemoveVetoMulticaster() {
		super();
	}

	public boolean canBeRemoved() {
		return getRemovingNotOkListener() == null;
	}

	public static final RemoveVetoListener add(
		RemoveVetoListener removeVetoListener,
		RemoveVetoListener removeVetoListenerToAdd) {

		return (RemoveVetoListener)AbstractListenerMulticaster.add(
			removeVetoListener,
			removeVetoListenerToAdd,
			_listenerMulticasterFactory);
	}

	private RemoveVetoListener getRemovingNotOkListener() {
		Iterator it = getListeners();
		while (it.hasNext()) {
			WeakReference reference = (WeakReference)it.next();
			RemoveVetoListener element = (RemoveVetoListener)reference.get();
			if (element == null)
				it.remove();
			else if (!element.canBeRemoved())
				return element;
		}
		return null;
	}

	public static RemoveVetoListener getRemovingNotOkListener(RemoveVetoListener removeVetoListener) {
		if (removeVetoListener instanceof RemoveVetoMulticaster) {
			return ((RemoveVetoMulticaster)removeVetoListener).getRemovingNotOkListener();

		} else if (!removeVetoListener.canBeRemoved()) {
			return removeVetoListener;

		} else {
			return null;
		}
	}


	public static final RemoveVetoListener remove(
		RemoveVetoListener removeVetoListener,
		RemoveVetoListener removeVetoListenerToRemove) {

		return (RemoveVetoListener)AbstractListenerMulticaster.remove(
			removeVetoListener,
			removeVetoListenerToRemove);
	}
}