package de.uniwue.ki.mas.util.declaration;
import java.util.Map;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import de.uniwue.ki.util.Utils;
import de.uniwue.ki.util.copy.ICopyable;
/**
* Insert the type's description here.
* Creation date: (03.11.2000 13:45:58)
* @author Christoph Oechslein
*/
public final class TestDeclarationList extends TestCase {
	private static class TestDeclaration extends AbstractDeclaration implements IDeclaration {
		private final int number;

		public TestDeclaration(int number) {
			this.number = number;
		}

		public int getNumber() {
			return number;
		}

		public String getName() {
			return "Test Declaration";
		}

		public ICopyable copy(Map objectMapping) {
			return new TestDeclaration(getNumber());
		}
	}
	private static class ListDataListenerCounter implements ListDataListener {
		private int addCounter = 0;
		private int removeCounter = 0;
		private int contentsChangedCounter = 0;

		protected int getAddCounter() {
			return addCounter;
		}

		protected int getRemoveCounter() {
			return removeCounter;
		}

		public void intervalAdded(ListDataEvent listDataEvent) {
			addCounter += listDataEvent.getIndex1() - listDataEvent.getIndex0() + 1;
		}

		public void intervalRemoved(ListDataEvent listDataEvent) {
			removeCounter += listDataEvent.getIndex1() - listDataEvent.getIndex0() + 1;
		}

		public void contentsChanged(ListDataEvent listDataEvent) {
			contentsChangedCounter += listDataEvent.getIndex1() - listDataEvent.getIndex0() + 1;
		}
	}

	public TestDeclarationList(String name) {
		super(name);
	}

	protected void addToDeclarationLists(
		IDeclarationList declarationListA,
		IDeclarationList declarationListB)
		throws Throwable {
		declarationListA.add(new TestDeclaration(99));
		declarationListA.add(new TestDeclaration(2));
		declarationListA.add(1, new TestDeclaration(1));
		declarationListA.set(0, new TestDeclaration(66));
		declarationListB.add(new TestDeclaration(4));
		declarationListB.add(new TestDeclaration(3));
		declarationListB.set(0, new TestDeclaration(0));
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (01.11.2000 17:47:04)
	 * @author Christoph Oechslein
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestDeclarationList.class);
	}

	/**
	 * @return TestSuite
	 */
	public static TestSuite suite() {
		TestSuite testSuite = new TestSuite("DeclarationList");
		testSuite.addTest(new TestDeclarationList("testDeclarationList"));
		testSuite.addTest(new TestDeclarationList("testMapCopyDeclarationList"));
		return testSuite;
	}

	public void testDeclarationList() throws Throwable {
		DefaultDeclarationList testDeclarationList = new DefaultDeclarationList();
		ListDataListenerCounter listDataListenerCounter = new ListDataListenerCounter();
		testDeclarationList.addListDataListener(listDataListenerCounter);
		testDeclarationList.add(new TestDeclaration(-1));
		testDeclarationList.add(new TestDeclaration(0));
		testDeclarationList.add(new TestDeclaration(1));
		testDeclarationList.add(new TestDeclaration(2));
		testDeclarationList.add(new TestDeclaration(4));
		testDeclarationList.add(4, new TestDeclaration(3));
		testDeclarationList.remove(0);
		testDeclarationList.set(0, new TestDeclaration(0));
		assertEquals(testDeclarationList.size(), testDeclarationList.getSize());
		assertEquals(testDeclarationList.size(), 5);
		assertEquals(((TestDeclaration) testDeclarationList.get(2)).getNumber(), 2);
		assertEquals(((TestDeclaration) testDeclarationList.get(3)).getNumber(), 3);
		assertEquals(((TestDeclaration) testDeclarationList.get(4)).getNumber(), 4);
		assertEquals(listDataListenerCounter.getAddCounter(), 6);
		assertEquals(listDataListenerCounter.getRemoveCounter(), 1);
		testDeclarationList.clear();
		assertEquals(testDeclarationList.size(), 0);
		assertEquals(listDataListenerCounter.getRemoveCounter(), 1 + 5);
	}

	protected void testDeclarationListsForEquality(
		IDeclarationList declarationListA,
		IDeclarationList declarationListB,
		int declarationListLength)
		throws Throwable {

		assertEquals(declarationListA.size(), declarationListB.size());
		assertEquals(declarationListA.size(), declarationListLength);

		for (int i = 0; i < declarationListLength; i++) {
			assertEquals(declarationListA.get(i), declarationListB.get(i));
		}

		assertTrue(!Utils.equals(declarationListA, declarationListB));

		return;
	}

	/**
	 * @throws Throwable
	 */
	public void testMapCopyDeclarationList() throws Throwable {
		DefaultDeclarationList testDeclarationList = new DefaultDeclarationList();
		AbstractMapCopyDeclarationList testAbstractMapCopyDeclarationList =
			new AbstractMapCopyDeclarationList(testDeclarationList) {
			public Object mapElement(Object anObject) {
				return anObject;
			}
			public Object set(int index, Object anObject) {
				return null;
			}
			public void add(int index, Object anObject) {
			}
			public Object remove(int index) {
				return null;
			}
		};

		testAbstractMapCopyDeclarationList.setComposition(false);
		testDeclarationList.setComposition(false);

		ListDataListenerCounter listDataListenerCounter = new ListDataListenerCounter();
		testAbstractMapCopyDeclarationList.addListDataListener(listDataListenerCounter);
		addToDeclarationLists(testDeclarationList, testAbstractMapCopyDeclarationList);
		testDeclarationListsForEquality(testDeclarationList, testAbstractMapCopyDeclarationList, 3);
	}
}