package de.uniwue.ki.mas.xml;
import java.io.File;
import java.util.Iterator;

import junit.framework.*;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.UserFeatureClassDeclarationFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionArgument.*;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListManager;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.UserFunctionDeclarationFactory;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.ActivityAgentClassDeclarationFactory;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.ActivityWorldClassDeclarationFactory;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.simulation.SimulationDeclarationFactory;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.sim.situation.SituationDeclarationFactory;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert the type's description here.
 * Creation date: (31.10.2000 13:40:53)
 * @author Christoph Oechslein
 */

public final class TestXML extends TestCase {
	private static final String EMPTY_FILENAME_POSTFIX = "Empty";
	private static final String SOME_FILENAME_POSTFIX = "Some";
	private static final String FULL_FILENAME_POSTFIX = "Full";
	private static final String ONE_SIMULATION_FILENAME_POSTFIX = "One_Simulation";

	private ISeSAmModelDeclaration _seSAmModelDeclaration;
	private IWorldClassDeclaration _worldClassDeclaration;
	private IAgentClassDeclaration _agentClassDeclaration;
	private IUserFeatureClassDeclaration _testFeatureClassDeclaration;
	private IVariableClassDeclaration _testFeatureVariableClassDeclaration;
	private IBodyVariableClassDeclaration _testBodyVariableClassDeclaration;

	/**
	 * @param name String
	 */
	public TestXML(String name) {
		super(name);
	}

	private void addAgentsToModel() {
		_agentClassDeclaration = ActivityAgentClassDeclarationFactory.createActivityAgentClassDeclaration();

		_seSAmModelDeclaration.getAgentClassDeclarationListAndFactory().getDeclarationList().add(
			_agentClassDeclaration);
	}

	private void addFullObjectsToModel() {
		addSomeObjectsToModel();

		IWorldInstanceDeclaration worldInstanceDeclaration =
			_worldClassDeclaration.createWorldInstanceDeclaration();

		worldInstanceDeclaration.getObjectInstanceDeclarationList().add(
			_agentClassDeclaration.createAgentInstanceDeclaration());

		ISituationDeclaration situationDeclaration =
			SituationDeclarationFactory.createSituationDeclaration(worldInstanceDeclaration);

		_seSAmModelDeclaration
			.getSimulationElementsDeclaration()
			.getSituationDeclarationListAndFactory()
			.getDeclarationList()
			.add(situationDeclaration);

		SimulationDeclarationFactory simulationDeclarationFactory =
			_seSAmModelDeclaration
				.getSimulationElementsDeclaration()
				.getSimulationDeclarationListAndFactory()
				.getSimulationDeclarationFactory();

		ISimulationDeclaration simulationDeclaration =
			simulationDeclarationFactory.createSimulationDeclaration(situationDeclaration);

		_seSAmModelDeclaration
			.getSimulationElementsDeclaration()
			.getSimulationDeclarationListAndFactory()
			.getDeclarationList()
			.add(simulationDeclaration);
	}

	private void addSimulationToModel() {
		_seSAmModelDeclaration
			.getSimulationElementsDeclaration()
			.getSimulationDeclarationListAndFactory()
			.getDeclarationList()
			.add(
				_seSAmModelDeclaration
					.getSimulationElementsDeclaration()
					.getSimulationDeclarationListAndFactory()
					.getSimulationDeclarationFactory()
					.createSimulationDeclaration());
	}

	private void addSomeObjectsToModel() {

		addUserFeatureToModel();
		addUserFunctionToModel();
		addAgentsToModel();

		_worldClassDeclaration = ActivityWorldClassDeclarationFactory.createActivityWorldClassDeclaration();

		IVariableClassDeclaration variableClassDeclaration =
			VariableClassDeclarationFactory.getInstance().createVariableClassDeclaration();

		_seSAmModelDeclaration.getWorldClassDeclarationListAndFactory().getDeclarationList().add(
			_worldClassDeclaration);

		variableClassDeclaration.setWriteable(true);

		_testBodyVariableClassDeclaration = variableClassDeclaration.createBodyVariableClassDeclaration();

		_agentClassDeclaration
			.getBodyClassDeclaration()
			.getBodyVariableClassDeclarationListAndFactory()
			.getBodyVariableClassDeclarationList()
			.add(_testBodyVariableClassDeclaration);

		_agentClassDeclaration.getObjectFeatureClassDeclarationList().add(_testFeatureClassDeclaration);

		IActivityReasoningEngineDeclaration activityReasoningEngineDeclaration =
			(IActivityReasoningEngineDeclaration)_agentClassDeclaration
				.getReasoningEngineDeclarationListAndFactory()
				.getFirstDeclarationOrCreateNewIfEmpty();

		IActivityGraphDeclaration composedActivityDeclaration =
			activityReasoningEngineDeclaration.getRootActivityGraphDeclaration();

		composedActivityDeclaration.addActivityGraphNodeDeclaration(
			createActionAtomicActivityDeclaration(composedActivityDeclaration));
		composedActivityDeclaration.addActivityGraphNodeDeclaration(
			createComposedActivityDeclaration(composedActivityDeclaration));

		Iterator it = composedActivityDeclaration.getActivityGraphNodeDeclarationList().iterator();
		while (it.hasNext()) {
			assertEquals(
				composedActivityDeclaration,
				((IActivityGraphNodeDeclaration)it.next()).getActivityGraphDeclaration());
		}
	}

	private void addUserFeatureToModel() {
		IFeatureClassDeclaration emptyFeatureClassDeclaration =
			UserFeatureClassDeclarationFactory.createUserFeatureClassDeclaration();
		emptyFeatureClassDeclaration.setShortDescription("Empty");

		_testFeatureClassDeclaration = UserFeatureClassDeclarationFactory.createUserFeatureClassDeclaration();
		_testFeatureClassDeclaration.setShortDescription("Test");

		_testFeatureVariableClassDeclaration =
			FeatureVariableClassDeclarationFactory.createFeatureVariableClassDeclaration(
				BooleanType.getInstance());

		_testFeatureVariableClassDeclaration.setShortDescription("Test Variable From Feature");

		_testFeatureClassDeclaration
			.getFeatureVariableClassDeclarationListAndFactory()
			.getVariableClassDeclarationList()
			.add(_testFeatureVariableClassDeclaration);

		_seSAmModelDeclaration
			.getSeSAmModelBasicsDeclaration()
			.getUserFeatureClassDeclarationListAndFactory()
			.getDeclarationList()
			.add(emptyFeatureClassDeclaration);
		_seSAmModelDeclaration
			.getSeSAmModelBasicsDeclaration()
			.getUserFeatureClassDeclarationListAndFactory()
			.getDeclarationList()
			.add(_testFeatureClassDeclaration);
	}

	private void addUserFunctionToModel() {
		IUserFunctionDeclaration userDefinedFunctionDeclaration =
			UserFunctionDeclarationFactory.getInstance(true).createUserFunctionDeclaration();

		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration =
			userDefinedFunctionDeclaration.getParametricFunctionCallDeclaration();

		SpecificIteratorType specificSetType =
			GenericIteratorType.getInstance().createSpecificIteratorType(DoubleType.getInstance());

		IFunctionArgumentDeclaration functionArgumentDeclarationA =
			FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				"numberParameter",
				DoubleType.getInstance());
		IFunctionArgumentDeclaration functionArgumentDeclarationB =
			FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				"booleanParameterA",
				BooleanType.getInstance());
		IFunctionArgumentDeclaration functionArgumentDeclarationC =
			FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				"booleanParameterB",
				BooleanType.getInstance());

		parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList().add(
			functionArgumentDeclarationA);
		parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList().add(
			functionArgumentDeclarationB);
		parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList().add(
			functionArgumentDeclarationC);

		userDefinedFunctionDeclaration.setSpecificOutputType(specificSetType);

		SpecificFunctionDeclarationWithArgDeclBinds specificFunDeclWArgDeclBinds =
			createSpecificFunctionDeclarationWithArgDeclBindsForUserFunction();

		ISpecificFunctionType specificFunDeclWArgDeclBindsFunctionType =
			GenericFunctionType.getInstance().createSpecificFunctionType(
				BooleanType.getInstance(),
				CollectionsHelper.createList(DoubleType.getInstance()));

		ISpecificFunctionDeclaration filterFunctionDecl =
			FunctionDeclarationListManager.getAllFunctionDeclarationList().getSpecificFunctionDeclaration(
				"Select",
				specificSetType,
				CollectionsHelper.createList(specificFunDeclWArgDeclBindsFunctionType, specificSetType));

		assertNotNull(filterFunctionDecl);

		parametricFunctionCallDeclaration.getFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				filterFunctionDecl,
				CollectionsHelper.createList(
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						specificFunDeclWArgDeclBindsFunctionType,
						specificFunDeclWArgDeclBinds),
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						specificSetType))));

		_seSAmModelDeclaration
			.getSeSAmModelBasicsDeclaration()
			.getUserFunctionDeclarationListAndFactory()
			.getDeclarationList()
			.add(userDefinedFunctionDeclaration);
	}

	private static void checkEquality(
		ISeSAmModelDeclaration originalSeSAmModelDeclaration,
		ISeSAmModelDeclaration seSAmModelDeclaration)
		throws Throwable {

		// check is broken due to FunctionArgumentDeclaration#isomorphicEquals
		// which must return false !!
		//assertTrue(originalSeSAmModelDeclaration.isomorphicEquals(seSAmModelDeclaration));
	}

	private IActionActivityGraphLeafDeclaration createActionAtomicActivityDeclaration(IActivityGraphDeclaration composedActivityDeclaration) {

		IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
				composedActivityDeclaration);

		actionAtomicActivityDeclaration
			.getFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.add(createSetVariableAction());

		return actionAtomicActivityDeclaration;
	}

	private IActivityGraphComposedNodeDeclaration createComposedActivityDeclaration(IActivityGraphDeclaration composedActivityDeclaration) {
		return ActivityGraphComposedNodeDeclarationFactory.createActivityGraphComposedNodeDeclaration();
	}

	private SpecificFunctionDeclarationWithArgDeclBinds createSpecificFunctionDeclarationWithArgDeclBindsForUserFunction() {
		SpecificFunctionDeclarationWithArgDeclBinds specificFunDeclWArgDeclBinds =
			SpecificFunctionDeclarationWithArgDeclBindsFactory
				.createSpecificFunctionDeclarationWithArgDeclBinds();

		IFunctionArgumentDeclaration functionArgumentDeclarationDummy =
			FunctionArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
				"dummy",
				DoubleType.getInstance());

		specificFunDeclWArgDeclBinds.setSpecificOutputType(BooleanType.getInstance());
		specificFunDeclWArgDeclBinds.getFunctionArgumentDeclarationList().add(
			functionArgumentDeclarationDummy);

		ISpecificFunctionDeclaration lessFunctionDecl =
			FunctionDeclarationListManager.getAllFunctionDeclarationList().getSpecificFunctionDeclaration(
				">",
				BooleanType.getInstance());

		assertNotNull(lessFunctionDecl);

		specificFunDeclWArgDeclBinds.getFunctionCallDeclaration().setFunctionCallInternDeclaration(
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				lessFunctionDecl,
				CollectionsHelper.createList(
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						functionArgumentDeclarationDummy),
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						DoubleType.getInstance()))));

		return specificFunDeclWArgDeclBinds;
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestXML.class);
	}

	private void readSeSAmModel(File file) throws Throwable {
		if (_seSAmModelDeclaration != null) {
			_seSAmModelDeclaration.remove();
		}

		ISeSAmDeclaration seSAmDeclaration = SeSAmDeclarationFactory.createSeSAmDeclaration(false);

		XMLReader.readModelInto(file, seSAmDeclaration, null, true);

		_seSAmModelDeclaration = seSAmDeclaration.getSeSAmModelDeclaration();
	}

	/**
	 * Sets up the fixture, for example, open a network connection.
	 * This method is called before a test is executed.
	 */
	protected void setUp() throws Exception {
		_seSAmModelDeclaration = SeSAmDeclarationFactory.createSeSAmModelDeclaration();
	}

	/**
	 * @return TestSuite
	 */
	public static TestSuite suite() {
		TestSuite anSuite = new TestSuite("XML");

		anSuite.addTest(new TestXML("testXMLModelWriterEmpty"));
		anSuite.addTest(new TestXML("testXMLModelWriterSome"));
		anSuite.addTest(new TestXML("testXMLModelWriterFull"));
		anSuite.addTest(new TestXML("testXMLModelReaderEmpty"));
		anSuite.addTest(new TestXML("testXMLModelReaderSome"));
		anSuite.addTest(new TestXML("testXMLModelReaderFull"));
		anSuite.addTest(new TestXML("testXMLModelReaderFullWithTests"));
		anSuite.addTest(new TestXML("testXMLModelReaderOneSimulation"));

		return anSuite;
	}

	protected void tearDown() {
		if (_seSAmModelDeclaration != null) {
			_seSAmModelDeclaration.remove();
		}
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelReaderEmpty() throws Throwable {
		File file = writeSeSAmModel(EMPTY_FILENAME_POSTFIX);
		readSeSAmModel(file);
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelReaderFull() throws Throwable {
		addFullObjectsToModel();
		File file = writeSeSAmModel(FULL_FILENAME_POSTFIX);
		readSeSAmModel(file);
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelReaderFullWithTests() throws Throwable {
		addFullObjectsToModel();
		File file = writeSeSAmModel(FULL_FILENAME_POSTFIX);
		ISeSAmModelDeclaration originalSeSAmModelDeclaration = _seSAmModelDeclaration;
		_seSAmModelDeclaration = null;
		readSeSAmModel(file);
		checkEquality(originalSeSAmModelDeclaration, _seSAmModelDeclaration);
		originalSeSAmModelDeclaration.remove();
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelReaderOneSimulation() throws Throwable {
		addSimulationToModel();
		File file = writeSeSAmModel(ONE_SIMULATION_FILENAME_POSTFIX);
		ISeSAmModelDeclaration originalSeSAmModelDeclaration = _seSAmModelDeclaration;
		_seSAmModelDeclaration = null;
		readSeSAmModel(file);
		checkEquality(originalSeSAmModelDeclaration, _seSAmModelDeclaration);
		originalSeSAmModelDeclaration.remove();
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelReaderSome() throws Throwable {
		addSomeObjectsToModel();
		File file = writeSeSAmModel(SOME_FILENAME_POSTFIX);
		readSeSAmModel(file);
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelWriterEmpty() throws Throwable {
		writeSeSAmModel(EMPTY_FILENAME_POSTFIX);
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelWriterFull() throws Throwable {
		addFullObjectsToModel();
		writeSeSAmModel(FULL_FILENAME_POSTFIX);
	}

	/**
	 * @throws Throwable
	 */
	public void testXMLModelWriterSome() throws Throwable {
		addSomeObjectsToModel();
		writeSeSAmModel(SOME_FILENAME_POSTFIX);
	}

	private File writeSeSAmModel(String filenamePostfix) throws Throwable {
		File file = new File("c:\\temp\\sesam-model-test" + filenamePostfix + ".xml");

		ISeSAmDeclaration seSAmDeclaration = SeSAmDeclarationFactory.createSeSAmDeclaration();

		XMLWriter.writeModel(file, seSAmDeclaration, null);

		return file;
	}

	private IFunctionCallDeclaration createSetVariableAction() {

		ISpecificFunctionDeclaration setVariableFunction =
			FunctionDeclarationVariableFactory
				.SET_VARIABLE_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(
					GenericVariableClassDeclarationType.getInstance(false, true).createSpecificVariableType(
						DoubleType.getInstance()),
					DoubleType.getInstance()));

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(setVariableFunction);

		IBodyVariableClassDeclaration bodyVariableClassDeclaration = null;
		Iterator it =
			_agentClassDeclaration
				.getBodyClassDeclaration()
				.getBodyVariableClassDeclarationListAndFactory()
				.getBodyVariableClassDeclarationList()
				.iterator();
		while (it.hasNext()) {
			IBodyVariableClassDeclaration currIBodyVariableClassDeclaration =
				(IBodyVariableClassDeclaration)it.next();

			if (currIBodyVariableClassDeclaration.getVariableClassDeclaration()
				== _testFeatureVariableClassDeclaration) {
				bodyVariableClassDeclaration = currIBodyVariableClassDeclaration;
				break;
			}
		}

		Assert.assertNotNull(bodyVariableClassDeclaration);

		return FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				setVariableFunction,
				CollectionsHelper.createList(
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						GenericVariableClassDeclarationType.getInstance(
							false,
							true).createSpecificVariableType(
							DoubleType.getInstance()),
						_testBodyVariableClassDeclaration),
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						null,
						DoubleType.getInstance(),
						new Double(1.0)))));
	}
}