package de.uniwue.ki.mas.xml;
import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;

import junit.framework.Assert;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import de.uniwue.ki.mas.basics.feature.BuildinFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.FeatureClassDeclarationListManager;
import de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.composed.ISpecificComposedType;
import de.uniwue.ki.mas.basics.type.enum.ISpecificEnumType;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.VariableClassDeclarationList;
import de.uniwue.ki.mas.xml.castor.v01.ini.*;
import de.uniwue.ki.mas.xml.reader.*;
import de.uniwue.ki.mas.xml.writer.*;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (29.10.2001 13:56:19)
 * @author Christoph Oechslein
 */
public class XMLIniReader {
	protected XMLIniFileGroup _xmlIniFileGroup;
	protected final ClassLoader _classLoader;

	public XMLIniReader(ClassLoader classLoader) {
		super();

		_classLoader = classLoader;
	}

	public void parseIniFile(File file) throws XMLIniFileException {
		try {
			BufferedReader stream = new BufferedReader(new FileReader(file));
			try {
				parseIniStream(stream);

			} finally {
				stream.close();
			}
		} catch (FileNotFoundException e) {
			throw new XMLIniFileException(e);
		} catch (XMLIniFileException e) {
			throw new XMLIniFileException(e);
		} catch (IOException e) {
			throw new XMLIniFileException(e);
		}
	}

	public void parseIniReaders(Iterator files) throws XMLIniFileException {
		while (files.hasNext()) {

			Reader reader = (Reader) files.next();
			try {
				parseIniStream(reader);
			} finally {
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
					throw new XMLIniFileException(e);
				}
			}
		}
	}

	public void parseIniStream(Reader stream) throws XMLIniFileException {

		try {
			_xmlIniFileGroup = XMLIniFileGroup.unmarshalXMLIniFileGroup(stream);

		} catch (ValidationException e) {
			throw new XMLIniFileException(e);

		} catch (MarshalException e) {
			throw new XMLIniFileException(e);

		}

		try {
			parseIniFileIntern();

		} catch (Throwable e) {
			throw new XMLIniFileException(e);
		}
	}

	protected void parseIniFileIntern()
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			NoSuchMethodException,
			XMLIniFileException {

		parseFunctionSection();
		parseTypeSection();
		parseFeatureSection();
	}

	private void parseType(XMLIniFileType xmlIniFileType)
		throws
			IllegalArgumentException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {

		IType loadedType = null;

		Object newInstance = loadClassAndCreateInstanceIfNeeded(xmlIniFileType);

		if (newInstance != null) {
			if (!(newInstance instanceof IType)) {
				throw new XMLIniFileException(
					"Error in Type Section for Type '"
						+ xmlIniFileType.getClassName()
						+ "' getInstanceMethod must return object of 'IType'");
			} else {
				loadedType = (IType) newInstance;
			}
		}

		if ((xmlIniFileType.getShouldAdd()) || (!xmlIniFileType.hasShouldAdd())) {
			if (loadedType instanceof ISpecificEnumType) {
				TypeManager.getInstance().addSystemAbstractType((ISpecificEnumType) loadedType);
			} else if (loadedType instanceof ISpecificComposedType) {
				TypeManager.getInstance().addSystemAbstractType((ISpecificComposedType) loadedType);
			} else if (loadedType instanceof ISpecificType) {
				TypeManager.getInstance().addSpecificType((ISpecificType) loadedType);

				if ((xmlIniFileType.getVariableType()) && (xmlIniFileType.hasVariableType())) {
					TypeManager.getInstance().addVariableSpecificType((ISpecificType) loadedType);
				}
			} else if (loadedType instanceof IGenericType) {
				TypeManager.getInstance().addGenericType((IGenericType) loadedType);

				if ((xmlIniFileType.getVariableType()) && (xmlIniFileType.hasVariableType())) {
					TypeManager.getInstance().addVariableGenericType((IGenericType) loadedType);
				}
			} else
				Assert.fail("Type " + loadedType + " not supported");
		}

		if (loadedType instanceof ISpecificType)
			parseArgumentDeclarationEditorPane(xmlIniFileType, (ISpecificType) loadedType);

		// xmlReader und Writer zu Typen laden
		try {
			if (xmlIniFileType.getXmlReaderClassName() != null) {
				XMLModelTypeReader.addTypeReader(
				(IXMLTypeReader)Utils.newInstanceOf(
				_classLoader.loadClass(xmlIniFileType.getXmlReaderClassName())));
									}
			if (xmlIniFileType.getXmlWriterClassName() != null) {
				XMLModelTypeWriter.addTypeWriter(
					(IXMLTypeWriter) Utils.newInstanceOf(
						_classLoader.loadClass(xmlIniFileType.getXmlWriterClassName())));
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		
	}

	protected void parseArgumentDeclarationEditorPane(
		XMLIniFileType xmlIniFileType,
		ISpecificType specificType)
		throws XMLIniFileException, InstantiationException, IllegalAccessException, ClassNotFoundException {
	}

	private void parseTypeSection()
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {

		if (_xmlIniFileGroup.getTypeSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getTypeSection().getTypeCount(); i++) {
				parseType(_xmlIniFileGroup.getTypeSection().getType(i));
			}
		}
	}

	private IVariableClassDeclaration parseVariable(XMLIniFileVariable xmlIniFileVariable)
		throws
			IllegalArgumentException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {
		Object newInstance = loadClassAndCreateInstanceIfNeeded(xmlIniFileVariable);

		if (!(newInstance instanceof IVariableClassDeclaration)) {
			throw new XMLIniFileException(
				"Error in Variable Section: "
					+ xmlIniFileVariable.getClassName()
					+ " must be of type 'IVariableClassDeclaration'");
		}

		return (IVariableClassDeclaration) newInstance;
	}

	private Object loadClassAndCreateInstanceIfNeeded(AbstractXMLIniFileClassFactory abstractXMLIniFileClassFactory)
		throws
			ClassNotFoundException,
			IllegalArgumentException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {
		String className = abstractXMLIniFileClassFactory.getClassName();

		Class anClass = _classLoader.loadClass(className);

		if (!abstractXMLIniFileClassFactory.hasShouldInstantiate()
			|| abstractXMLIniFileClassFactory.getShouldInstantiate()) {
			String getInstanceMethodName = abstractXMLIniFileClassFactory.getSingletonMethodName();

			if (getInstanceMethodName != null) {
				try {
					Method getInstanceMethod = anClass.getMethod(getInstanceMethodName, null);
					Assert.assertNotNull(getInstanceMethod);
					return getInstanceMethod.invoke(null, new Object[]{});
				} catch (NoSuchMethodException e) {
					// ignore it 
				} 
			}
			return anClass.newInstance();

		} else if (
			(abstractXMLIniFileClassFactory instanceof AbstractXMLIniFileClassFactoryWithAdd)
				&& (!((AbstractXMLIniFileClassFactoryWithAdd) abstractXMLIniFileClassFactory).getShouldAdd())) {
			throw new XMLIniFileException(
				"Typef '"
					+ abstractXMLIniFileClassFactory.getClassName()
					+ "' shouldInstantiate must be true if shouldAdd is true.");

		} else {
			return null;
		}
	}

	private IFeatureClassDeclaration parseComposedFeature(XMLIniFileComposedFeature xmlIniFileComposedFeature)
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {

		IFunctionDeclarationList defaultFunctionDeclarationList = new FunctionDeclarationList();
		IFunctionDeclarationList functionDeclarationList = new FunctionDeclarationList();
		VariableClassDeclarationList variableClassDeclarationList = new VariableClassDeclarationList();

		for (int i = 0; i < xmlIniFileComposedFeature.getDefaultFunctionCount(); i++) {
			IFunctionDeclaration functionDeclaration =
				parseFunction(xmlIniFileComposedFeature.getDefaultFunction(i));

			if (functionDeclaration == null) {
				throw new XMLIniFileException(
					"Error in Composed Feature : "
						+ xmlIniFileComposedFeature.getName()
						+ "function '"
						+ xmlIniFileComposedFeature.getDefaultFunction(i).getClassName()
						+ "' shouldInstantiate must be true");
			}

			defaultFunctionDeclarationList.add(functionDeclaration);
		}

		for (int i = 0; i < xmlIniFileComposedFeature.getFunctionCount(); i++) {
			IFunctionDeclaration functionDeclaration =
				parseFunction(xmlIniFileComposedFeature.getFunction(i));

			if (functionDeclaration == null) {
				throw new XMLIniFileException(
					"Error in Composed Feature : "
						+ xmlIniFileComposedFeature.getName()
						+ "function '"
						+ xmlIniFileComposedFeature.getFunction(i).getClassName()
						+ "' shouldInstantiate must be true");
			}

			functionDeclarationList.add(functionDeclaration);
		}

		for (int i = 0; i < xmlIniFileComposedFeature.getVariableCount(); i++) {
			IVariableClassDeclaration variableClassDeclaration =
				parseVariable(xmlIniFileComposedFeature.getVariable(i));

			if (variableClassDeclaration == null) {
				throw new XMLIniFileException(
					"Error in Composed Feature : "
						+ xmlIniFileComposedFeature.getName()
						+ "variable '"
						+ xmlIniFileComposedFeature.getVariable(i).getClassName()
						+ "' shouldInstantiate must be true");
			}

			variableClassDeclarationList.add(variableClassDeclaration);
		}

		for (int i = 0; i < xmlIniFileComposedFeature.getTypeCount(); i++) {
			parseType(xmlIniFileComposedFeature.getType(i));
		}

		IFeatureClassDeclaration featureClassDeclaration =
			new BuildinFeatureClassDeclaration(
				xmlIniFileComposedFeature.getName(),
				defaultFunctionDeclarationList,
				functionDeclarationList,
				variableClassDeclarationList);

		FeatureClassDeclarationListManager.getInstance().addBuildinFeatureClassDeclaration(
			featureClassDeclaration);

		return featureClassDeclaration;
	}

	private IFeatureClassDeclaration parseFeature(XMLIniFileFeature xmlIniFileFeature)
		throws
			IllegalArgumentException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {
		Object newInstance = loadClassAndCreateInstanceIfNeeded(xmlIniFileFeature);

		if (!(newInstance instanceof IFeatureClassDeclaration)) {
			throw new XMLIniFileException(
				"Error in Feature : "
					+ xmlIniFileFeature.getClassName()
					+ " must be of type 'IFeatureClassDeclaration'");
		}

		if ((xmlIniFileFeature.getShouldAdd()) || (!xmlIniFileFeature.hasShouldAdd())) {

			FeatureClassDeclarationListManager.getInstance().addBuildinFeatureClassDeclaration(
				(IFeatureClassDeclaration) newInstance);
		}

		try {
			if (xmlIniFileFeature.getXmlReaderClassName() != null) {
				XMLModelObjectReader.addFeatureReader(
					(IXMLFeatureReader) Utils.newInstanceOf(
						_classLoader.loadClass(xmlIniFileFeature.getXmlReaderClassName())));
			}
			if (xmlIniFileFeature.getXmlWriterClassName() != null) {
				XMLModelObjectWriter.addFeatureWriter(
					(IXMLFeatureWriter) Utils.newInstanceOf(
						_classLoader.loadClass(xmlIniFileFeature.getXmlWriterClassName())));
			}
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		return (IFeatureClassDeclaration) newInstance;
	}

	private void parseFeatureSection()
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {

		if (_xmlIniFileGroup.getFeatureSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getFeatureSection().getFeatureCount(); i++) {
				parseFeature(_xmlIniFileGroup.getFeatureSection().getFeature(i));
			}

			for (int i = 0; i < _xmlIniFileGroup.getFeatureSection().getComposedFeatureCount(); i++) {
				parseComposedFeature(_xmlIniFileGroup.getFeatureSection().getComposedFeature(i));
			}
		}

	}

	private IFunctionDeclaration parseFunction(XMLIniFileFunction xmlIniFileFunction)
		throws
			IllegalArgumentException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {
		Object newInstance = loadClassAndCreateInstanceIfNeeded(xmlIniFileFunction);

		if (!(newInstance instanceof IFunctionDeclaration)) {
			throw new XMLIniFileException(
				"Error in Function Section: "
					+ xmlIniFileFunction.getClassName()
					+ " must be of type 'IFunctionDeclaration'");
		}

		if ((xmlIniFileFunction.getShouldAdd()) || (!xmlIniFileFunction.hasShouldAdd())) {

			FunctionDeclarationListManager.addFunctionToDefaultFunctions((IFunctionDeclaration) newInstance);
		}

		return (IFunctionDeclaration) newInstance;
	}

	private void parseFunctionSection()
		throws
			IllegalArgumentException,
			IndexOutOfBoundsException,
			ClassNotFoundException,
			InvocationTargetException,
			InstantiationException,
			IllegalAccessException,
			XMLIniFileException {

		if (_xmlIniFileGroup.getFunctionSection() != null) {

			for (int i = 0; i < _xmlIniFileGroup.getFunctionSection().getFunctionCount(); i++) {
				parseFunction(_xmlIniFileGroup.getFunctionSection().getFunction(i));
			}
		}
	}
}