package de.uniwue.ki.mas.xml.reader;

import de.uniwue.ki.mas.basics.function.CreateAndExecuteFunctionArgs;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.model.ModelCompilingException;
import de.uniwue.ki.mas.model.SimulationRuntimeException;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
 * Created on 12.02.2004
 * 
 * @author herrler
 */
public abstract class AbstractXMLFeatureReader implements IXMLFeatureReader {

    IObjectClassDeclaration objectInstanceDeclaration = null;

    protected Object evaluateValue(XMLModelReader xmlModelReader,
            IObjectClassDeclaration objectClassDeclaration,
            XMLFeature xmlFeature, String attributeName) throws XMLReadException {
        XMLGenericFeatureValue genericFeatureValue = getGenericFeatureValue(
                (XMLGenericFeature) xmlFeature, attributeName);
        
        return evaluateFeatureValue(xmlModelReader, objectClassDeclaration,
                genericFeatureValue);
    }

    protected Object evaluateValue(XMLModelReader xmlModelReader,
            IObjectClassDeclaration objectClassDeclaration,
            XMLFeatureInstance xmlFeatureInstance, String attributeName) throws XMLReadException{
        XMLGenericFeatureValue genericFeatuerValue = getGenericFeatureValue(
                (XMLGenericFeatureInstance) xmlFeatureInstance, attributeName);

        return evaluateFeatureValue(xmlModelReader, objectClassDeclaration,
                genericFeatuerValue);
    }

    private Object evaluateFeatureValue(XMLModelReader xmlModelReader,
            IObjectClassDeclaration objectClassDeclaration,
            XMLGenericFeatureValue genericFeatureValue) {
        Object evaluatedObject = null;
        try {
            evaluatedObject = xmlModelReader
                    .createRealFunctionCallInternDeclaration(
                            objectClassDeclaration,
                            genericFeatureValue.getXMLFunctionCallInternGroup())
                    .createFunction(new CreateAndExecuteFunctionArgs())
                    .execute(null);
        } catch (SimulationRuntimeException e) {
            e.printStackTrace();
        } catch (ModelCompilingException e) {
            e.printStackTrace();
        } catch (XMLReadException e) {
            e.printStackTrace();
        }
        return evaluatedObject;
    }

    private XMLGenericFeatureValue getGenericFeatureValue(
            XMLGenericFeature xmlFeature, String attributeName) throws XMLReadException {
        for (int i = 0; i < xmlFeature.getAttributeCount(); i++) {
            XMLGenericFeatureValue attribute = xmlFeature.getAttribute(i);
            if (attribute.getAttributeName().equals(attributeName))
                    return attribute;
        }
        throw new XMLReadException("Attribute "+attributeName+" not found in feature "+xmlFeature.getFeatureName());
    }

    private XMLGenericFeatureValue getGenericFeatureValue(
            XMLGenericFeatureInstance xmlFeatureInstance, String attributeName) throws XMLReadException{
        for (int i = 0; i < xmlFeatureInstance.getAttributeCount(); i++) {
            XMLGenericFeatureValue attribute = xmlFeatureInstance
                    .getAttribute(i);
            if (attribute.getAttributeName().equals(attributeName))
                    return attribute;
        }
        throw new XMLReadException("Attribute "+attributeName+" not found in feature "+xmlFeatureInstance.getFeatureName());
    }

    public boolean isResponsibleFor(Object obj) {
        if (obj instanceof XMLFeature)
                return isResponsibleFor((XMLFeature) obj);
        if (obj instanceof XMLFeatureInstance)
                return isResponsibleFor((XMLFeatureInstance) obj);
        return false;
    }

    public abstract boolean isResponsibleFor(XMLFeature xmlFeature);

    public abstract boolean isResponsibleFor(
            XMLFeatureInstance xmlFeatureinstance);
}
