package de.uniwue.ki.mas.xml.reader;

import java.util.Iterator;
import java.util.LinkedList;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.xml.IDelaySettings;
import de.uniwue.ki.mas.xml.castor.v01.model.XMLFunctionCall;

/**
 * Insert the type's description here.
 * Creation date: (22.08.2001 08:49:46)
 * @author Christoph Oechslein
 */
final class DelaySetterHelper {
	private static class FunctionCallDelaySettings implements IDelaySettings {
		private final IFunctionCallDeclaration _functionCallDeclaration;
		private final ISpecificType _mostGeneralOutputType;
		private final XMLFunctionCall _xmlFunctionCall;

		private final XMLModelFunctionReader _xmlModelFunctionReader;

		public FunctionCallDelaySettings(
			IFunctionCallDeclaration functionCallDeclaration,
			XMLFunctionCall xmlFunctionCall,
			ISpecificType mostGeneralOutputType,
			XMLModelFunctionReader xmlModelFunctionReader) {

			Assert.assertNotNull(functionCallDeclaration);
			Assert.assertNotNull(xmlFunctionCall);

			_xmlFunctionCall = xmlFunctionCall;
			_xmlModelFunctionReader = xmlModelFunctionReader;
			_functionCallDeclaration = functionCallDeclaration;
			_mostGeneralOutputType = mostGeneralOutputType;
		}

		public void doSetting() throws XMLReadException {

			if (_mostGeneralOutputType == null)
				_xmlModelFunctionReader.setRealFunctionCall(
					_functionCallDeclaration,
					_xmlFunctionCall.getXMLFunctionCallInternGroup(),
					_xmlFunctionCall.getXMLTypeGroup());
			else
				_xmlModelFunctionReader.setRealFunctionCall(
					_functionCallDeclaration,
					_xmlFunctionCall.getXMLFunctionCallInternGroup(),
					_mostGeneralOutputType);

			if (Assert.ASSERTION_ON) {
				_functionCallDeclaration.checkInvariants();
			}
		}

		public boolean isSettingFunctionCallDeclaration(IFunctionCallDeclaration functionCallDeclaration) {
			return (functionCallDeclaration.equals(_functionCallDeclaration));
		}
	}

	private final LinkedList _delaySettingsList = new LinkedList();

	public DelaySetterHelper() {
		super();
	}

	public void addDelaySettings(IDelaySettings delaySettings) {
		_delaySettingsList.addFirst(delaySettings);
	}

	public void addDelaySettings(
		IFunctionCallDeclaration functionCallDeclaration,
		XMLFunctionCall xmlFunctionCall,
		ISpecificType mostGeneralFunCallTyp,
		XMLModelFunctionReader xmlModelFunctionReader) {

		addDelaySettings(
			new FunctionCallDelaySettings(
				functionCallDeclaration,
				xmlFunctionCall,
				mostGeneralFunCallTyp,
				xmlModelFunctionReader));
	}

	public void addDelaySettings(
		IFunctionCallDeclaration functionCallDeclaration,
		XMLFunctionCall xmlFunctionCall,
		XMLModelFunctionReader xmlModelFunctionReader) {

		addDelaySettings(
			new FunctionCallDelaySettings(
				functionCallDeclaration,
				xmlFunctionCall,
				null,
				xmlModelFunctionReader));
	}

	public void doAllDelaySettings() throws XMLReadException {

		// the _delaySettingsList can become longer during doSetting evaluation, e.g. an atomic parametric function declaration
		while (_delaySettingsList.size() > 0) {

			IDelaySettings delaySettings = (IDelaySettings)_delaySettingsList.get(0);
			_delaySettingsList.remove(0);
			try {
				delaySettings.doSetting();
			} catch (XMLReadException e) {
				throw e;
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
	}

	public void doDelaySettingsForAndRemove(IFunctionCallDeclaration functionCallDeclaration)
		throws XMLReadException {

		FunctionCallDelaySettings delaySettings = getFunctionCallDelaySettingsFor(functionCallDeclaration);
		_delaySettingsList.remove(delaySettings);
		delaySettings.doSetting();
	}

	private FunctionCallDelaySettings getFunctionCallDelaySettingsFor(IFunctionCallDeclaration functionCallDeclaration) {

		Iterator it = _delaySettingsList.iterator();

		while (it.hasNext()) {

			IDelaySettings currDelaySettings = (IDelaySettings)it.next();

			if (currDelaySettings instanceof FunctionCallDelaySettings) {
				FunctionCallDelaySettings currFunctionCallDeclaration =
					(FunctionCallDelaySettings)currDelaySettings;

				if (currFunctionCallDeclaration.isSettingFunctionCallDeclaration(functionCallDeclaration)) {
					return currFunctionCallDeclaration;
				}
			}
		}

		return null;
	}

	public boolean hasDelaySettingsFor(IFunctionCallDeclaration functionCallDeclaration) {
		return getFunctionCallDelaySettingsFor(functionCallDeclaration) != null;
	}
}