package de.uniwue.ki.mas.xml.reader;
import junit.framework.Assert;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.xml.IDelaySettings;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;

/**
 * Creation date: (30.08.2000 09:55:35)
 * @author Christoph Oechslein
 */

final class DelayedSimObjectType extends AbstractSimObjectType implements IDelaySettings {
	private IFeatureClassDeclaration _featureClassDeclaration;
	private IObjectClassDeclaration _objectClassDeclaration;

	private final XMLRootObject _rootObject;
	private final transient XMLModelReader _modelReader;
	private boolean _evaluated;

	public DelayedSimObjectType(XMLRootObject rootObject, XMLModelReader modelReader) {
		super();

		_rootObject = rootObject;
		_modelReader = modelReader;
		_evaluated = false;

		modelReader.addDelaySettings(this);
	}

	public IFeatureClassDeclaration getFeatureClassDeclaration() {
		testEvaluation();
		return _featureClassDeclaration;
	}

	public IObjectClassDeclaration getObjectClassDeclaration() {
		testEvaluation();
		return _objectClassDeclaration;
	}

	private void testEvaluation() {
		if (!_evaluated)
			if (!(_modelReader.containsXMLObject(_rootObject)))
				Assert.fail("DelayedSimObjectType(" + _rootObject + ") is accessed before it was evaluated");
			else
				try {
					doSetting();
				} catch (Exception e) {
					e.printStackTrace();
					Assert.fail(e.getMessage());
				}
	}

	public void doSetting() throws Exception {
		if (_evaluated)
			return;

		IDeclaration declaration = _modelReader.getDeclaration(_rootObject);
		if (declaration instanceof IObjectClassDeclaration) {
			_objectClassDeclaration = (IObjectClassDeclaration)declaration;
		} else {
			_featureClassDeclaration = (IFeatureClassDeclaration)declaration;
		}

		_evaluated = true;
		ObjectType.getInstance().addSubType(this);
	}

	public boolean equals(Object obj) {
		if (_evaluated)
			return super.equals(obj);

		if (obj == null) {
			return false;
		} else if (this == obj) {
			return true;
		} else if (!(obj instanceof ISimObjectType)) {
			return false;
		} else if (obj instanceof DelayedSimObjectType) {
			return ((DelayedSimObjectType)obj)._rootObject.equals(_rootObject);
		} else {
			return false;
		}
	}

	public String getName() {
		if (_evaluated)
			return super.getName();
		else if (_rootObject == null)
			return "SimObject";
		else
			return "DelayedSimObject";
	}

}