package de.uniwue.ki.mas.xml.reader;

import java.util.*;

import junit.framework.Assert;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.ArgumentDeclarationFactory;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.mas.xml.reader.oldfunctions.RemovedFunctionDeclarationList;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IMapFunction;

/**
  * Creation date: (07.02.2001 13:20:43)
  * @author Christoph Oechslein
  */
class XMLModelFunctionReader {

	private final RemovedFunctionDeclarationList _removedFunctionDeclarationList =
		new RemovedFunctionDeclarationList();
	private final transient XMLModelReader _xmlModelReader;
	private final IFunctionDeclarationListAble _functionDeclarationListAble;

	public XMLModelFunctionReader(
		XMLModelReader xmlModelReader,
		IFunctionDeclarationListAble functionDeclarationListAble)
		throws XMLReadException {

		super();

		Assert.assertNotNull(functionDeclarationListAble);
		_xmlModelReader = xmlModelReader;
		_functionDeclarationListAble = functionDeclarationListAble;
	}

	private IAtomicFunctionCallInternDeclaration createRealAtomicFunctionCallIntern(XMLTypeValueGroup xmlTypeValue)
		throws XMLReadException {

		return FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
			null,
			_xmlModelReader.createRealArgumentDeclaration(xmlTypeValue, _functionDeclarationListAble));
	}

	private IComposedFunctionCallInternDeclaration createRealComposedFunctionCallIntern(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		XMLComposedFunctionCallIntern xmlComposedFunctionCallIntern)
		throws XMLReadException {

		IComposedFunctionCallInternDeclaration functionCallInternDeclaration =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				specificFunctionDeclaration);

		for (int i = 0; i < xmlComposedFunctionCallIntern.getXMLFunctionCallInternGroupCount(); i++) {

			IFunctionCallInternDeclaration childFunctionCallInternDeclaration =
				createRealFunctionCallIntern(xmlComposedFunctionCallIntern.getXMLFunctionCallInternGroup(i));
			childFunctionCallInternDeclaration.setParent(functionCallInternDeclaration);
			functionCallInternDeclaration.setChild(i, childFunctionCallInternDeclaration);
		}

		testVaribleGetterAndSetterFunctions(specificFunctionDeclaration, functionCallInternDeclaration);

		return functionCallInternDeclaration;
	}

	private IComposedFunctionCallInternDeclaration createRealComposedFunctionCallIntern(
		XMLUserFunction xmlUserFunction,
		XMLComposedFunctionCallIntern xmlComposedFunctionCallIntern)
		throws XMLReadException {

		IUserFunctionDeclaration userDefinedFunctionDeclaration =
			_xmlModelReader.getRealUserFunctionDeclaration(xmlUserFunction);
		IFunctionCallDeclaration userDefinedFunctionCallDeclaration =
			userDefinedFunctionDeclaration
				.getParametricFunctionCallDeclaration()
				.getFunctionCallDeclaration();
		_xmlModelReader.doDelaySettingsIfAvailible(userDefinedFunctionCallDeclaration);

		return createRealComposedFunctionCallIntern(
			userDefinedFunctionDeclaration,
			xmlComposedFunctionCallIntern);
	}

	private ISpecificType createSpecificOutputType(XMLComposedFunctionCallIntern xmlComposedFunctionCallIntern)
		throws XMLReadException {
		if (xmlComposedFunctionCallIntern.getXMLTypeGroup() == null)
			return null;
		else
			return _xmlModelReader.createRealType(
				xmlComposedFunctionCallIntern.getXMLTypeGroup(),
				_functionDeclarationListAble);
	}

	private IComposedFunctionCallInternDeclaration createRealComposedFunctionCallIntern(
		String function,
		XMLComposedFunctionCallIntern xmlComposedFunctionCallIntern)
		throws XMLReadException {

		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration;

		if (xmlComposedFunctionCallIntern == null) {
			composedFunctionCallInternDeclaration =
				FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
					null,
					getRealSpecificFunctionDeclaration(
						function,
						createSpecificOutputType(xmlComposedFunctionCallIntern),
						CollectionsHelper.EMPTY_ITERATOR));
		} else {

			int childrenSize = xmlComposedFunctionCallIntern.getXMLFunctionCallInternGroupCount();
			List realFunctionCallInterns = new ArrayList(childrenSize);

			for (int i = 0; i < childrenSize; i++) {

				IFunctionCallInternDeclaration currentFunctionCallInternDeclaration =
					createRealFunctionCallIntern(
						xmlComposedFunctionCallIntern.getXMLFunctionCallInternGroup(i));
				realFunctionCallInterns.add(currentFunctionCallInternDeclaration);
			}

			ISpecificFunctionDeclaration specificFunctionDeclaration =
				getRealSpecificFunctionDeclaration(xmlComposedFunctionCallIntern, realFunctionCallInterns);

			composedFunctionCallInternDeclaration =
				FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
					null,
					specificFunctionDeclaration,
					realFunctionCallInterns);

			testVaribleGetterAndSetterFunctions(
				specificFunctionDeclaration,
				composedFunctionCallInternDeclaration);
		}

		return _removedFunctionDeclarationList.transform(
			composedFunctionCallInternDeclaration,
			_xmlModelReader);
	}

	IFunctionCallDeclaration createRealFunctionCall(
		XMLFunctionCall xmlFunctionCall,
		IFunctionCallDeclarationFactory functionCallDeclarationFactory)
		throws XMLReadException {

		if (xmlFunctionCall == null) {
			return null;

		} else {

			IFunctionCallDeclaration functionCallDeclaration =
				functionCallDeclarationFactory.createFunctionCallDeclaration();

			_xmlModelReader.addDelaySettings(functionCallDeclaration, xmlFunctionCall, this);

			return functionCallDeclaration;
		}
	}

	IFunctionCallInternDeclaration createRealFunctionCallIntern(XMLFunctionCallInternGroup xmlFunctionCallIntern)
		throws XMLReadException {

		if (Assert.ASSERTION_ON) {
			Assert.assertNotNull(xmlFunctionCallIntern);
		}

		IFunctionCallInternDeclaration functionCallInternDeclaration;
		XMLComposedFunctionCallIntern composedSequence = xmlFunctionCallIntern.getCall();
		XMLTypeValueGroup atomicSequence = xmlFunctionCallIntern.getXMLTypeValueGroup();

		// Assert.assertTrue(composedSequence != null || atomicSequence != null);
		if ((atomicSequence != null)) {
			functionCallInternDeclaration = createRealAtomicFunctionCallIntern(atomicSequence);
		} else if ((composedSequence != null) && (composedSequence.getUserFunctionID() != null)) {
			functionCallInternDeclaration =
				createRealComposedFunctionCallIntern(
					(XMLUserFunction) composedSequence.getUserFunctionID(),
					composedSequence);
		} else {
			functionCallInternDeclaration =
				createRealComposedFunctionCallIntern(composedSequence.getFunctionName(), composedSequence);
		}

		return functionCallInternDeclaration;
	}

	/**
	 * @param xmlFunctionCall XMLFunctionCall
	 * @return ISpecificType
	 * @throws XMLReadException
	 */
	public ISpecificType createRealType(XMLFunctionCall xmlFunctionCall) throws XMLReadException {

		if (xmlFunctionCall.getXMLTypeGroup() != null) {

			return _xmlModelReader.createRealType(
				xmlFunctionCall.getXMLTypeGroup(),
				_functionDeclarationListAble);
		} else {

			XMLFunctionCallInternGroup xmlFunctionCallInternGroup =
				xmlFunctionCall.getXMLFunctionCallInternGroup();

			return createRealType(xmlFunctionCallInternGroup);
		}
	}

	private ISpecificType createRealType(XMLFunctionCallInternGroup xmlFunctionCallIntern)
		throws XMLReadException {

		XMLComposedFunctionCallIntern composedSequence = xmlFunctionCallIntern.getCall();
		XMLTypeValueGroup atomicSequence = xmlFunctionCallIntern.getXMLTypeValueGroup();

		if ((atomicSequence != null)) {
			return _xmlModelReader.createRealType(atomicSequence, _functionDeclarationListAble);

		} else if ((composedSequence != null) && (composedSequence.getUserFunctionID() != null)) {

			XMLUserFunction xmlUserFunction = (XMLUserFunction) composedSequence.getUserFunctionID();
			if (xmlUserFunction.getXMLTypeGroup() == null) {
				return createRealType(xmlUserFunction.getFunctionCall());
			} else {
				return _xmlModelReader
					.getRealUserFunctionDeclaration(xmlUserFunction)
					.getSpecificOutputType();
			}

		} else {
			return getRealSpecificFunctionDeclaration(composedSequence).getSpecificOutputType();
		}
	}

	private IFunctionDeclarationListAble getFunctionDeclarationListAble() throws XMLReadException {
		return _functionDeclarationListAble;
	}

	private ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(XMLComposedFunctionCallIntern composedFunctionCallIntern)
		throws XMLReadException {

		String function = composedFunctionCallIntern.getFunctionName();
		int childrenSize = composedFunctionCallIntern.getXMLFunctionCallInternGroupCount();
		List types = new ArrayList(childrenSize);

		for (int i = 0; i < childrenSize; i++) {
			types.add(createRealType(composedFunctionCallIntern.getXMLFunctionCallInternGroup(i)));
		}

		return getRealSpecificFunctionDeclaration(
			function,
			createSpecificOutputType(composedFunctionCallIntern),
			types);
	}

	private ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(
		XMLComposedFunctionCallIntern composedFunctionCallIntern,
		List realFunctionCallInterns)
		throws XMLReadException {

		return getRealSpecificFunctionDeclaration(
			composedFunctionCallIntern.getFunctionName(),
			createSpecificOutputType(composedFunctionCallIntern),
			CollectionsHelper.map(realFunctionCallInterns.iterator(), new IMapFunction() {
			public Object map(Object anObject) {

				return ((IFunctionCallInternDeclaration) anObject).getSpecificOutputType();
			}
		}));
	}

	public ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(
		String function,
		ISpecificFunctionType specificFunctionType)
		throws XMLReadException {

		if (specificFunctionType instanceof SpecificFunctionType) {

			return getRealSpecificFunctionDeclaration(
				function,
				((SpecificFunctionType) specificFunctionType).getSpecificOutputType(),
				((SpecificFunctionType) specificFunctionType).getSpecificInputTypes());

		} else {
			return getRealSpecificFunctionDeclaration(
				function,
				((SpecificFunctionUnlimitedType) specificFunctionType).getSpecificOutputType(),
				CollectionsHelper.createList(
					((SpecificFunctionUnlimitedType) specificFunctionType).getSpecificInputType()));
		}
	}

	private ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(
		String function,
		ISpecificType outputType,
		Iterator types)
		throws XMLReadException {

		return getRealSpecificFunctionDeclaration(
			function,
			outputType,
			CollectionsHelper.asLinkedList(types));
	}

	private ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(
		String function,
		ISpecificType outputType,
		List typeList)
		throws XMLReadException {

		ISpecificFunctionDeclaration functionDeclaration =
			getRealSpecificFunctionDeclarationIntern(function, outputType, typeList);

		if (functionDeclaration != null) {
			return functionDeclaration;

		} else {
			StringBuffer buffer = new StringBuffer("Function with name '");
			buffer.append(function);
			buffer.append("' and types [");

			Iterator it = typeList.iterator();

			while (it.hasNext()) {

				IType element = (IType) it.next();
				buffer.append(element.getName());

				if (it.hasNext()) {
					buffer.append(", ");
				}
			}

			buffer.append("] not found.");
			throw new XMLReadException(buffer.toString());
		}
	}

	private ISpecificFunctionDeclaration getRealSpecificFunctionDeclarationIntern(
		String function,
		ISpecificType outputType,
		List typeList)
		throws XMLReadException {

		ISpecificFunctionDeclaration functionDeclaration =
			FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
				_removedFunctionDeclarationList,
				function,
				outputType,
				typeList);
		if (functionDeclaration != null) {
			return functionDeclaration;
		}

		functionDeclaration =
			FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
				getFunctionDeclarationListAbleAsCollection(),
				function,
				outputType,
				typeList);
		if (functionDeclaration != null) {
			return functionDeclaration;
		}

		if ((getFunctionDeclarationListAble() instanceof CacheObjectClassDeclFunctionDeclarationListAble)) {
			((CacheObjectClassDeclFunctionDeclarationListAble) getFunctionDeclarationListAble()).update();
			functionDeclaration =
				FunctionDeclarationListHelper.getSpecificFunctionDeclaration(
					getFunctionDeclarationListAbleAsCollection(),
					function,
					outputType,
					typeList);
		}

		return functionDeclaration;
	}

	private Collection getFunctionDeclarationListAbleAsCollection() throws XMLReadException {
		if (getFunctionDeclarationListAble() instanceof IFunctionDeclarationList) {
			return (IFunctionDeclarationList) getFunctionDeclarationListAble();
		} else if (
			getFunctionDeclarationListAble() instanceof CacheObjectClassDeclFunctionDeclarationListAble) {
			return ((CacheObjectClassDeclFunctionDeclarationListAble) getFunctionDeclarationListAble())
				.getFunctionDeclarationList();
		} else {
			return CollectionsHelper.asLinkedList(getFunctionDeclarationListAble().getFunctionDeclarations());
		}
	}

	public void setParametricFunctionCallDeclaration(
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
		XMLUnnamedParametricFunction xmlUnnamedParametricFunction)
		throws XMLReadException {

		for (int i = 0; i < xmlUnnamedParametricFunction.getParameterCount(); i++) {
			parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList().add(
				_xmlModelReader.getFunctionArgumentDeclaration(xmlUnnamedParametricFunction.getParameter(i)));
		}

		_xmlModelReader.setRealFunctionCallDelay(
			parametricFunctionCallDeclaration.getFunctionCallDeclaration(),
			_functionDeclarationListAble,
			xmlUnnamedParametricFunction.getFunctionCall());
	}

	public void setRealFunctionCall(
		IFunctionCallDeclaration functionCallDeclaration,
		XMLFunctionCallInternGroup xmlFunctionCallIntern,
		XMLTypeGroup xmlMostGeneralOutputType)
		throws XMLReadException {

		if (xmlMostGeneralOutputType == null)
			setRealFunctionCall(functionCallDeclaration, xmlFunctionCallIntern, (ISpecificType) null);
		else
			setRealFunctionCall(
				functionCallDeclaration,
				xmlFunctionCallIntern,
				_xmlModelReader.createRealType(xmlMostGeneralOutputType, _functionDeclarationListAble));

	}

	public void setRealFunctionCall(
		IFunctionCallDeclaration functionCallDeclaration,
		XMLFunctionCallInternGroup xmlFunctionCallIntern,
		ISpecificType mostGeneralOutputType)
		throws XMLReadException {

		IFunctionCallInternDeclaration functionCallInternDeclaration =
			createRealFunctionCallIntern(xmlFunctionCallIntern);

		if (mostGeneralOutputType != null) {
			functionCallDeclaration.setMostGeneralOutputType(mostGeneralOutputType);
		} else {
			functionCallDeclaration.setMostGeneralOutputType(
				functionCallInternDeclaration.getSpecificOutputType());
		}
		functionCallDeclaration.setFunctionCallInternDeclaration(functionCallInternDeclaration);
	}

	public void setRealFunctionCallDelay(
		IFunctionCallDeclaration functionCallDeclaration,
		XMLFunctionCall xmlFunctionCall)
		throws XMLReadException {

		_xmlModelReader.addDelaySettings(functionCallDeclaration, xmlFunctionCall, this);
	}

	private void testVaribleGetterAndSetterFunctions(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration)
		throws XMLReadException {

		if ((specificFunctionDeclaration.getGenericFunctionDeclaration()
			== FunctionDeclarationVariableFactory.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION)
			|| (specificFunctionDeclaration.getGenericFunctionDeclaration()
				== FunctionDeclarationVariableFactory.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION)
			|| (specificFunctionDeclaration.getGenericFunctionDeclaration()
				== FunctionDeclarationVariableFactory.SET_VARIABLE_GENERIC_FUNCTION_DECLARATION)
			|| (specificFunctionDeclaration.getGenericFunctionDeclaration()
				== FunctionDeclarationVariableFactory.SET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION)) {
			testVaribleGetterAndSetterFunctions(
				specificFunctionDeclaration,
				composedFunctionCallInternDeclaration,
				0);
		}
	}

	private void testVaribleGetterAndSetterFunctions(
		ISpecificFunctionDeclaration specificFunctionDeclaration,
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		int index)
		throws XMLReadException {

		IAtomicFunctionCallInternDeclaration child =
			(IAtomicFunctionCallInternDeclaration) composedFunctionCallInternDeclaration.getChild(index);
		ISpecificType correctType = specificFunctionDeclaration.getSpecificInputTypeAt(index);
		ISpecificType childType = child.getArgumentDeclaration().getSpecificType();
		if (!correctType.equals(childType)) {

			IArgumentDeclaration oldArgumentDeclaration = child.getArgumentDeclaration();
			if (oldArgumentDeclaration instanceof IFunctionArgumentDeclaration) {
				/* attention we are 'modifying the type of this instance which could be used in an other functioncall!! */
				if (!TypeManager.getInstance().isMoreGeneral(correctType, childType)) {
					((IFunctionArgumentDeclaration) oldArgumentDeclaration).setSpecificType(correctType);
				}
			} else {

				IArgumentDeclaration newArgumentDeclaration =
					ArgumentDeclarationFactory.getInstance().createArgumentDeclaration(
						specificFunctionDeclaration.getSpecificInputTypeAt(index),
						oldArgumentDeclaration.getObject());
				composedFunctionCallInternDeclaration.setChild(
					index,
					FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
						composedFunctionCallInternDeclaration,
						newArgumentDeclaration));
			}
		}
	}
}