package de.uniwue.ki.mas.xml.reader;

import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationList;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.featureList.ObjectFeatureInstanceDeclarationList;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.IFunctionDeclarationListAble;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.ActivityReasoningEngineDeclarationFactory;
import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.body.IBodyInstanceDeclaration;
import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IActivityAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.ActivityWorldClassDeclarationFactory;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
 * Creation date: (28.11.2001 15:04:34)
 * 
 * @author Christoph Oechslein
 */
public final class XMLModelObjectReader {

    private static final List _xmlFeatureReaders = new LinkedList();

    private final Map _objectFeatureClassMap = new HashMap();

    public static void addFeatureReader(IXMLFeatureReader featureReader) {
        _xmlFeatureReaders.add(featureReader);
    }

    private final transient XMLModelReader _xmlModelReader;

    public XMLModelObjectReader(XMLModelReader xmlModelReader) {
        super();
        _xmlModelReader = xmlModelReader;
    }

    public boolean containsXMLObject(XMLRootObject object) {
        return _xmlModelReader.containsXMLObject(object);
    }

    private IAgentClassDeclaration createRealAgentClassDeclaration(
            XMLAgentClass xmlAgent) throws XMLReadException {

        IActivityAgentClassDeclaration realAgent = (IActivityAgentClassDeclaration) getSeSAmModelDeclaration()
                .getAgentClassDeclarationListAndFactory()
                .getAgentClassDeclarationFactory()
                .createAgentClassDeclaration();
        //setRealAgentClassDeclaration(realAgent, xmlAgent);

        return realAgent;
    }

    private IBodyVariableClassDeclaration createRealBodyVariableClassDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            XMLBodyVariableClass xmlBodyVariableClass) throws XMLReadException {

        IVariableClassDeclaration realVariableClassDeclaration;

        if (xmlBodyVariableClass.getVariableClassID() != null) {
            realVariableClassDeclaration = getRealVariableClassDeclaration(
                    objectClassDeclaration,
                    (XMLVariableClass) xmlBodyVariableClass
                            .getVariableClassID());

            IBodyFeatureVariableClassDeclaration realBodyFeatureVariableClassDeclaration = (IBodyFeatureVariableClassDeclaration) realVariableClassDeclaration
                    .createBodyVariableClassDeclaration();
            setRealBodyFeatureVariableClassDeclaration(
                    realBodyFeatureVariableClassDeclaration,
                    objectClassDeclaration, xmlBodyVariableClass);

            return realBodyFeatureVariableClassDeclaration;

        } else {
            realVariableClassDeclaration = getRealVariableClassDeclaration(
                    objectClassDeclaration, xmlBodyVariableClass
                            .getVariableClass());

            IBodyVariableClassDeclaration realBodyVariable = realVariableClassDeclaration
                    .createBodyVariableClassDeclaration();
            setRealBodyVariableClassDeclaration(realBodyVariable,
                    objectClassDeclaration, xmlBodyVariableClass);

            return realBodyVariable;
        }
    }

    private IObjectInstanceDeclaration createRealObjectInstanceDeclaration(
            XMLObjectInstance xmlObjectInstance) throws XMLReadException {

        IObjectClassDeclaration objectClassDeclaration = (IObjectClassDeclaration) getDeclaration((XMLRootObject) xmlObjectInstance
                .getObjectClassID());
        IObjectInstanceDeclaration objectInstanceDeclaration = objectClassDeclaration
                .createObjectInstanceDeclaration();
        setRealObjectInstanceDeclaration(objectInstanceDeclaration,
                xmlObjectInstance);

        return objectInstanceDeclaration;
    }

    private IReasoningEngineDeclaration createRealReasoningDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            XMLReasoning xmlReasoning) throws XMLReadException {

        return new XMLModelReasoningActivityReader(objectClassDeclaration,
                _xmlModelReader)
                .createRealActivityReasoningDeclaration(xmlReasoning);
    }

    private IRessourceClassDeclaration createRealRessourceClassDeclaration(
            XMLRessourceClass xmlRessource) throws XMLReadException {

        IRessourceClassDeclaration realRessource = getSeSAmModelDeclaration()
                .getRessourceClassDeclarationListAndFactory()
                .getRessourceClassDeclarationFactory()
                .createRessourceClassDeclaration();
        //setRealRessourceClassDeclaration(realRessource, xmlRessource);

        return realRessource;
    }

    private IWorldClassDeclaration createRealWorldClassDeclaration(
            XMLWorldClass xmlWorld) throws XMLReadException {

        IWorldClassDeclaration realWorldClass = ActivityWorldClassDeclarationFactory
                .createActivityWorldClassDeclaration();

        return realWorldClass;
    }

    private IWorldInstanceDeclaration createRealWorldInstanceDeclaration(
            XMLWorldInstance xmlWorldInstance) throws XMLReadException {

        IWorldClassDeclaration worldClassDeclaration = (IWorldClassDeclaration) getDeclaration((XMLRootObject) xmlWorldInstance
                .getObjectClassID());
        IWorldInstanceDeclaration worldInstanceDeclaration = worldClassDeclaration
                .createWorldInstanceDeclaration();
        setRealWorldInstanceDeclaration(worldInstanceDeclaration,
                xmlWorldInstance);

        return worldInstanceDeclaration;
    }

    public IDeclaration getDeclaration(XMLRootObject object) {

        return _xmlModelReader.getDeclaration(object);
    }

    IAgentClassDeclaration getRealAgentClassDeclaration(XMLAgentClass xmlObject)
            throws XMLReadException {

        if (!containsXMLObject(xmlObject)) {
            setDeclaration(xmlObject,
                    createRealAgentClassDeclaration(xmlObject));
        }

        IAgentClassDeclaration objectClassDeclaration = (IAgentClassDeclaration) getDeclaration(xmlObject);

        if (Assert.ASSERTION_ON) {
            Assert.assertNotNull(objectClassDeclaration);
        }

        return objectClassDeclaration;
    }

    void fillRealAgentClassDeclaration(XMLAgentClass xmlObject)
            throws XMLReadException {

        if (!containsXMLObject(xmlObject)) {
            Assert.fail("AgentClassDeclaration " + xmlObject + " not found!");
        }

        setRealAgentClassDeclaration(
                (IAgentClassDeclaration) getDeclaration(xmlObject), xmlObject);
    }

    public IBodyVariableClassDeclaration getRealBodyVariableClassDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            XMLBodyVariableClass xmlBodyVariableClass) throws XMLReadException {
        Assert.assertNotNull(xmlBodyVariableClass);

        if (!containsXMLObject(xmlBodyVariableClass)) {
            setDeclaration(xmlBodyVariableClass,
                    createRealBodyVariableClassDeclaration(
                            objectClassDeclaration, xmlBodyVariableClass));
        }

        IBodyVariableClassDeclaration realBodyVariable = (IBodyVariableClassDeclaration) getDeclaration(xmlBodyVariableClass);
        Assert.assertNotNull(realBodyVariable);

        return realBodyVariable;
    }

    IRessourceClassDeclaration getRealRessourceClassDeclaration(
            XMLRessourceClass xmlObject) throws XMLReadException {

        if (!containsXMLObject(xmlObject)) {
            setDeclaration(xmlObject,
                    createRealRessourceClassDeclaration(xmlObject));
        }

        IRessourceClassDeclaration objectClassDeclaration = (IRessourceClassDeclaration) getDeclaration(xmlObject);

        if (Assert.ASSERTION_ON) {
            Assert.assertNotNull(objectClassDeclaration);
        }

        return objectClassDeclaration;
    }

    void fillRealRessourceClassDeclaration(XMLRessourceClass xmlObject)
            throws XMLReadException {

        if (!containsXMLObject(xmlObject)) {
            Assert.fail("RessourceClassDeclaration " + xmlObject
                    + " not found!");
        }

        IRessourceClassDeclaration objectClassDeclaration = (IRessourceClassDeclaration) getDeclaration(xmlObject);

        setRealRessourceClassDeclaration(objectClassDeclaration, xmlObject);
    }

    private IVariableClassDeclaration getRealVariableClassDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            XMLVariableClass xmlVariableClass) throws XMLReadException {

        return _xmlModelReader.getRealVariableClassDeclaration(_xmlModelReader
                .getFunctionDeclarationListFor(objectClassDeclaration),
                xmlVariableClass);
    }

    IWorldClassDeclaration getRealWorldClassDeclaration(XMLWorldClass xmlObject)
            throws XMLReadException {

        if (!containsXMLObject(xmlObject)) {
            setDeclaration(xmlObject,
                    createRealWorldClassDeclaration(xmlObject));
        }

        IWorldClassDeclaration objectClassDeclaration = (IWorldClassDeclaration) getDeclaration(xmlObject);
        Assert.assertNotNull(objectClassDeclaration);

        return objectClassDeclaration;
    }

    void fillRealWorldClassDeclaration(XMLWorldClass xmlObject)
            throws XMLReadException {

        if (!containsXMLObject(xmlObject))
                Assert.fail("XMLWorldClass " + xmlObject + " not found!");

        IWorldClassDeclaration realWorldClass = (IWorldClassDeclaration) getDeclaration(xmlObject);

        setRealAgentClassDeclaration(realWorldClass, xmlObject);
        if (xmlObject.hasUpdateBeforeAllObjects())
                realWorldClass.setUpdateBeforeAllObjects(xmlObject
                        .getUpdateBeforeAllObjects());
    }

    private ISeSAmModelDeclaration getSeSAmModelDeclaration() {

        return _xmlModelReader.getSeSAmModelDeclaration();
    }

    private void initRealBodyInstanceDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            IBodyInstanceDeclaration bodyInstanceDeclaration,
            XMLBodyVariableInstance[] bodyVariableInstances)
            throws XMLReadException {

        Iterator it = bodyInstanceDeclaration
                .getBodyVariableInstanceDeclarationList().iterator();

        for (int i = 0; i < bodyVariableInstances.length; i++) {

            IBodyVariableInstanceDeclaration bodyVariableInstanceDeclaration = (IBodyVariableInstanceDeclaration) it
                    .next();
            XMLBodyVariableInstance bodyVariableInstance = bodyVariableInstances[i];
            _xmlModelReader.setRealFunctionCallDelay(
                    bodyVariableInstanceDeclaration
                            .getDefaultValueFunctionCallDeclaration(),
                    objectClassDeclaration, bodyVariableInstance
                            .getDefaultValue());

            /*
             * IFunctionCallDeclaration functionCallDeclaration =
             * createRealFunctionCall(objectClassDeclaration,
             * bodyVariableInstance.getDefaultValue(), this);
             * 
             * if (functionCallDeclaration != null)
             * bodyVariableInstanceDeclaration.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
             * functionCallDeclaration.getFunctionCallInternDeclaration());
             */
            bodyVariableInstanceDeclaration
                    .setUsesOwnDefaulValueFunctionCallDeclaration(bodyVariableInstance
                            .getUsesOwnDefaulValue());
        }
    }

    private void replaceBodyVariableClassDeclaration(
            IBodyVariableClassDeclaration currBodyVariableClassDeclaration,
            IBodyVariableClassDeclaration entailedBodyVariableClassDeclaration,
            XMLBodyVariableClass xmlBodyVariableClass,
            IObjectClassDeclaration objectClassDeclaration)
            throws XMLReadException {

        currBodyVariableClassDeclaration.copyInto(
                entailedBodyVariableClassDeclaration, new HashMap());

        XMLVariableClass xmlVariableClass;

        if (xmlBodyVariableClass.getVariableClassID() != null) {
            xmlVariableClass = (XMLVariableClass) xmlBodyVariableClass
                    .getVariableClassID();
        } else {
            xmlVariableClass = xmlBodyVariableClass.getVariableClass();
        }

        Assert.assertNotNull(entailedBodyVariableClassDeclaration
                .getVariableClassDeclaration());
        Assert.assertNotNull(xmlVariableClass);

        if (entailedBodyVariableClassDeclaration instanceof IBodyFeatureVariableClassDeclaration) {
            Assert
                    .assertTrue(currBodyVariableClassDeclaration instanceof IBodyFeatureVariableClassDeclaration);

            IBodyFeatureVariableClassDeclaration entailedBodyFeatureVariableClassDeclaration = (IBodyFeatureVariableClassDeclaration) entailedBodyVariableClassDeclaration;

            // pay attention to the deferred functionCall evaluation
            _xmlModelReader.setRealFunctionCallDelay(
                    entailedBodyFeatureVariableClassDeclaration
                            .getDefaultValueFunctionCallDeclaration(),
                    objectClassDeclaration, xmlBodyVariableClass
                            .getDefaultValue());
            _xmlModelReader.setRealFunctionCallDelay(
                    entailedBodyFeatureVariableClassDeclaration
                            .getVariableClassDeclaration()
                            .getDefaultValueFunctionCallDeclaration(),
                    objectClassDeclaration, xmlVariableClass.getDefaultValue());
            _xmlModelReader.setRealFunctionCallDelay(
                    entailedBodyFeatureVariableClassDeclaration
                            .getVariableClassDeclaration()
                            .getNextValueFunctionCallDeclaration(),
                    objectClassDeclaration, xmlVariableClass.getNextValue());
        }

        // don't remove the VariableClassDeclaration since it is also
        // referenced in entailedBodyVariableClassDeclaration
        NeverRemoveVetoListener neverRemoveVetoListener = new NeverRemoveVetoListener(
                this);
        currBodyVariableClassDeclaration.getVariableClassDeclaration()
                .addRemoveVetoListener(neverRemoveVetoListener);
        currBodyVariableClassDeclaration.remove();
        currBodyVariableClassDeclaration.getVariableClassDeclaration()
                .removeRemoveVetoListener(neverRemoveVetoListener);
    }

    private IBodyVariableClassDeclaration searchBodyVariableClassDeclaration(
            IVariableClassDeclaration variableClassDeclaration,
            IObjectClassDeclaration objectClassDeclaration) {

        return objectClassDeclaration.getBodyClassDeclaration()
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList()
                .getBodyVariableClassDeclaration(variableClassDeclaration);
    }

    public void setDeclaration(XMLRootObject object, IDeclaration declaration) {
        _xmlModelReader.setDeclaration(object, declaration);
    }

    private void setRealAgentClassDeclaration(
            IAgentClassDeclaration agentClassDeclaration, XMLAgentClass xmlAgent)
            throws XMLReadException {

        setRealObjectClassDeclaration(agentClassDeclaration, xmlAgent);

        IDeclarationList reasoningEngineDeclarationList = agentClassDeclaration
                .getReasoningEngineDeclarationListAndFactory()
                .getDeclarationList();

        reasoningEngineDeclarationList.clear();

        setRealUpdateActionDeclList(agentClassDeclaration, xmlAgent);

        for (int i = 0; i < xmlAgent.getReasoningCount(); i++) {
            reasoningEngineDeclarationList.add(createRealReasoningDeclaration(
                    agentClassDeclaration, xmlAgent.getReasoning(i)));
        }

    }

    private void setRealUpdateActionDeclList(
            IAgentClassDeclaration agentClassDeclaration, XMLAgentClass xmlAgent)
            throws XMLReadException {

        IDeclarationList reasoningEngineDeclarationList = agentClassDeclaration
                .getReasoningEngineDeclarationListAndFactory()
                .getDeclarationList();

        IActivityGraphDeclaration realActivityGraphDeclaration = ActivityGraphDeclarationFactory
                .createEmptyActivityGraphDeclaration();
        realActivityGraphDeclaration.setShortDescription("Update Behaviour");
        realActivityGraphDeclaration
                .setDocumentation("Automatically transformed");

        IActionActivityGraphLeafDeclaration actionActivityGraphLeafDeclaration = ActionActivityGraphLeafDeclarationFactory
                .createActionActivityGraphLeafDeclaration(realActivityGraphDeclaration);
        actionActivityGraphLeafDeclaration.setShortDescription("Update");
        actionActivityGraphLeafDeclaration
                .setDocumentation("Automatically transformed");

        realActivityGraphDeclaration
                .addActivityGraphNodeDeclaration(actionActivityGraphLeafDeclaration);

        FunctionActionCallDeclarationList updateActionCallDeclarationList = actionActivityGraphLeafDeclaration
                .getFunctionActionCallDeclarationListAndFactory()
                .getFunctionActionCallDeclarationList();

        IFunctionDeclarationListAble functionDeclarationList = _xmlModelReader
                .getFunctionDeclarationListFor(agentClassDeclaration);

        FunctionActionCallDeclarationFactory functionActionCallDeclarationFactory = actionActivityGraphLeafDeclaration
                .getFunctionActionCallDeclarationListAndFactory()
                .getFunctionActionCallDeclarationFactory();

        for (int i = 0; i < xmlAgent.getReasoningCount(); i++) {
            XMLReasoning currReasoning = xmlAgent.getReasoning(i);
            for (int j = 0; j < currReasoning.getUpdateActionCount(); j++) {
                IFunctionCallDeclaration updateActionFunction = _xmlModelReader
                        .createRealFunctionCall(currReasoning
                                .getUpdateAction(j), functionDeclarationList,
                                functionActionCallDeclarationFactory);
                updateActionCallDeclarationList.add(updateActionFunction);
            }
        }

        realActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
                realActivityGraphDeclaration
                        .getEntryActivityGraphLeafDeclaration(),
                actionActivityGraphLeafDeclaration);

        if (!updateActionCallDeclarationList.isEmpty()) {
            reasoningEngineDeclarationList
                    .add(ActivityReasoningEngineDeclarationFactory
                            .createActivityReasoningEngineDeclaration(realActivityGraphDeclaration));
        }
    }

    private void setRealBodyClassDeclaration(
            IObjectClassDeclaration objectClassDeclaration, XMLBodyClass xmlBody)
            throws XMLReadException {

        IBodyClassDeclaration realBody = objectClassDeclaration
                .getBodyClassDeclaration();
        BodyVariableClassDeclarationList bodyVariableClassDeclarationList = realBody
                .getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList();

        if (xmlBody == null) {
            Assert.assertTrue(bodyVariableClassDeclarationList.isEmpty());
        } else {

            XMLBodyVariableClass[] bodyVariables = xmlBody
                    .getBodyVariableClass();
            for (int i = 0; i < bodyVariables.length; i++) {

                XMLBodyVariableClass xmlBodyVariableClass = bodyVariables[i];
                IBodyVariableClassDeclaration currBodyVariableClassDeclaration = getRealBodyVariableClassDeclaration(
                        objectClassDeclaration, xmlBodyVariableClass);
                IBodyVariableClassDeclaration entailedBodyVariableClassDeclaration = searchBodyVariableClassDeclaration(
                        currBodyVariableClassDeclaration
                                .getVariableClassDeclaration(),
                        objectClassDeclaration);

                if (entailedBodyVariableClassDeclaration == null) {
                    bodyVariableClassDeclarationList
                            .add(currBodyVariableClassDeclaration);
                } else {
                    replaceBodyVariableClassDeclaration(
                            currBodyVariableClassDeclaration,
                            entailedBodyVariableClassDeclaration,
                            xmlBodyVariableClass, objectClassDeclaration);
                    bodyVariableClassDeclarationList.changeIndex(
                            entailedBodyVariableClassDeclaration, i);

                    // now change mapping for deferred function call evaluation
                    _xmlModelReader.changeDeclaration(xmlBodyVariableClass,
                            currBodyVariableClassDeclaration,
                            entailedBodyVariableClassDeclaration);
                }
            }
        }
    }

    private void setRealBodyFeatureVariableClassDeclaration(
            IBodyFeatureVariableClassDeclaration realBodyFeatureVariable,
            IObjectClassDeclaration objectClassDeclaration,
            XMLBodyVariableClass xmlBodyVariableClass) throws XMLReadException {
        setRealBodyVariableClassDeclaration(realBodyFeatureVariable,
                objectClassDeclaration, xmlBodyVariableClass);
        realBodyFeatureVariable
                .setUsesOwnDefaulValueFunctionCallDeclaration(xmlBodyVariableClass
                        .getUsesOwnDefaulValue());
        _xmlModelReader.setRealFunctionCallDelay(realBodyFeatureVariable
                .getDefaultValueFunctionCallDeclaration(),
                objectClassDeclaration, xmlBodyVariableClass.getDefaultValue());

        /*
         * IFunctionCallDeclaration functionCallDeclaration =
         * createRealFunctionCall(objectClassDeclaration,
         * xmlBodyVariableClass.getDefaultValue(), this); if
         * (functionCallDeclaration != null)
         * realBodyVariable.getDefaultValueFunctionCallDeclaration().setFunctionCallInternDeclaration(
         * functionCallDeclaration.getFunctionCallInternDeclaration());
         */
    }

    private void setRealBodyVariableClassDeclaration(
            IBodyVariableClassDeclaration realBodyVariable,
            IObjectClassDeclaration objectClassDeclaration,
            XMLBodyVariableClass xmlBodyVariableClass) throws XMLReadException {
    }

    private void setRealObjectClassDeclaration(
            IObjectClassDeclaration objectClassDeclaration,
            XMLAbstractObjectClass xmlObject) throws XMLReadException {
        _xmlModelReader.setName(objectClassDeclaration, xmlObject);

        // first body variables then features since error in evolution feature
        // (due to _xmlModelReader.changeDeclaration)
        setRealBodyClassDeclaration(objectClassDeclaration, xmlObject
                .getBodyClass());
        setRealObjectFeatureClassDeclarationList(objectClassDeclaration,
                xmlObject);
    }

    private void setRealObjectFeatureClassDeclarationList(
            IObjectClassDeclaration objectClassDeclaration,
            XMLAbstractObjectClass xmlObject) throws XMLReadException {

        XMLFeatureClassList xmlFeatureClassList = xmlObject.getFeatures();
        List features = new ArrayList(xmlFeatureClassList.getBuildinCount()
                + xmlFeatureClassList.getUserCount());

        for (int i = 0; i < xmlFeatureClassList.getBuildinCount(); i++) {
            features.add(_xmlModelReader.getRealBuildinFeatureClassDeclaration(
                    xmlFeatureClassList.getBuildin(i), objectClassDeclaration));
        }

        for (int i = 0; i < xmlFeatureClassList.getUserCount(); i++) {
            features.add(_xmlModelReader.getRealUserFeatureClassDeclaration(
                    (XMLUserFeatureClass) xmlFeatureClassList.getUser(i)
                            .getId()).createObjectFeatureClassDeclaration(
                    objectClassDeclaration));
        }
        
        //somewhat strange but for compatibility reasons:
        XMLEvolutionFeature evolution = xmlFeatureClassList.getEvolution();
        if (evolution != null) {
            xmlFeatureClassList.addFeatures(evolution);
            //addFeature(objectClassDeclaration, evolution, features);
        }

        for (int i = 0; i < xmlFeatureClassList.getFeaturesCount(); i++) {
            addFeature(objectClassDeclaration, xmlFeatureClassList
                    .getFeatures(i), features);
        }

        if (xmlFeatureClassList.getSpatialInfo() != null) {
            IHasSpatialInfoDeclaration declaration = (IHasSpatialInfoDeclaration) SpatialInfoFeatureClassDeclaration
                    .getInstance().createObjectFeatureClassDeclaration(
                            objectClassDeclaration);
            features.add(declaration);

            _xmlModelReader
                    .getXMLModelTypeReader(objectClassDeclaration)
                    .createRealSpatialInfo(xmlFeatureClassList.getSpatialInfo())
                    .copyInto(declaration.getSpatialInfo());
        }

        //      objectClassDeclaration.getObjectFeatureClassDeclarationList().addAll(features);
        // changed since, with the above all default features are in the
        // objectclass
        // especially if you removed the spatialinfo, after saving and loading
        // it will be there again
        // therefore:
        objectClassDeclaration.getObjectFeatureClassDeclarationList().clear();
        objectClassDeclaration.getObjectFeatureClassDeclarationList().addAll(
                features);
    }

    private void addFeature(IObjectClassDeclaration objectClassDeclaration,
            XMLFeature xmlFeature, List features) throws XMLReadException {

        Iterator it = _xmlFeatureReaders.iterator();
        while (it.hasNext()) {
            IXMLFeatureReader featureReader = (IXMLFeatureReader) it.next();
            if (featureReader.isResponsibleFor(xmlFeature)) {
                IObjectFeatureClassDeclaration featureClassDeclaration = featureReader
                        .createObjectFeatureClassDeclaration(_xmlModelReader,
                                objectClassDeclaration, xmlFeature);
                _objectFeatureClassMap.put(xmlFeature, featureClassDeclaration);
                features.add(featureClassDeclaration);
            }
        }
    }

    private IObjectFeatureInstanceDeclaration createRealObjectFeatureInstanceDeclaration(
            IObjectInstanceDeclaration objectInstanceDeclaration,
            XMLFeatureInstance featureInstance) throws XMLReadException {

        Iterator it = _xmlFeatureReaders.iterator();
        while (it.hasNext()) {
            IXMLFeatureReader featureReader = (IXMLFeatureReader) it.next();
            if (featureReader.isResponsibleFor(featureInstance)) {
                IObjectFeatureClassDeclaration featureClassParent = (IObjectFeatureClassDeclaration) _objectFeatureClassMap
                        .get((XMLFeature) featureInstance
                                .getFeatureObjectClassID());
                if (Assert.ASSERTION_ON)
                        Assert.assertNotNull("Object Feature Instance "
                                + featureInstance + " has no Object Feature",
                                featureClassParent);
                return featureReader.createObjectFeatureInstanceDeclaration(
                        _xmlModelReader, objectInstanceDeclaration
                                .getObjectClassDeclaration(), featureInstance,
                        featureClassParent);
            }
        }
        return null;
    }

    private void setRealObjectInstanceDeclaration(
            IObjectInstanceDeclaration objectInstanceDeclaration,
            XMLObjectInstance xmlObjectInstance) throws XMLReadException {

        if (xmlObjectInstance.getSpatialInfo() != null) {

            ISpatialInfo realSpatialInfo = SpatialInfoObjectFeatureHelper
                    .getSpatialInfo(objectInstanceDeclaration);
            ISpatialInfo savedSpatialInfo = _xmlModelReader
                    .getXMLModelTypeReader(
                            objectInstanceDeclaration
                                    .getObjectClassDeclaration())
                    .createRealSpatialInfo(xmlObjectInstance.getSpatialInfo());
            savedSpatialInfo.copyInto(realSpatialInfo);
        }

        //Features einlesen
        Iterator it = xmlObjectInstance.getFeatureInstanceAsReference()
                .iterator();

        while (it.hasNext()) {
            IObjectFeatureInstanceDeclaration featureInstance = createRealObjectFeatureInstanceDeclaration(
                    objectInstanceDeclaration, (XMLFeatureInstance) it.next());

            if (featureInstance != null) {
                // gleichtypische features entfernen
                removeFeature(objectInstanceDeclaration
                        .getObjectFeatureInstanceDeclarationList(),
                        featureInstance.getObjectFeatureClassDeclaration());
                // eingelesene hinzufgen
                objectInstanceDeclaration
                        .getObjectFeatureInstanceDeclarationList().add(
                                featureInstance);
            }
        }

        initRealBodyInstanceDeclaration(objectInstanceDeclaration
                .getObjectClassDeclaration(), objectInstanceDeclaration
                .getBodyInstanceDeclaration(), xmlObjectInstance
                .getBodyVariableInstance());
    }

    private void removeFeature(ObjectFeatureInstanceDeclarationList list,
            IObjectFeatureClassDeclaration featureClassToRemove) {
        Iterator it = list.iterator();
        List candidatesToRemove = new LinkedList();

        while (it.hasNext()) {
            IObjectFeatureInstanceDeclaration featureInstance = (IObjectFeatureInstanceDeclaration) it
                    .next();
            if (featureInstance.getObjectFeatureClassDeclaration().equals(
                    featureClassToRemove))
                    candidatesToRemove.add(featureInstance);
        }

        list.removeAll(candidatesToRemove);
    }

    private void setRealRessourceClassDeclaration(
            IRessourceClassDeclaration ressourceClassDeclaration,
            XMLRessourceClass xmlRessource) throws XMLReadException {
        setRealObjectClassDeclaration(ressourceClassDeclaration, xmlRessource);
    }

    private void setRealWorldInstanceDeclaration(
            IWorldInstanceDeclaration worldInstanceDeclaration,
            XMLWorldInstance xmlWorldInstance) throws XMLReadException {
        setRealObjectInstanceDeclaration(worldInstanceDeclaration,
                xmlWorldInstance);

        if (xmlWorldInstance.getMapSettings() != null) {
            Assert
                    .assertTrue(SpatialMapFeatureClassDeclaration
                            .hasSpatialMapBuildinFeatureInstanceDeclaration(worldInstanceDeclaration));

            SpatialMapObjectFeatureInstDeclaration spatialMapBuildinWorldFeatureInstanceDeclaration = SpatialMapFeatureClassDeclaration
                    .getSpatialMapBuildinObjectFeatureInstanceDeclaration(worldInstanceDeclaration);

            ISpatialMapModel spatialMapModel = spatialMapBuildinWorldFeatureInstanceDeclaration
                    .getSpatialMapModel();

            XMLMapSettings xmlMapSettings = xmlWorldInstance.getMapSettings();

            if (xmlMapSettings.hasTorus()) {
                spatialMapModel.setTorus(xmlMapSettings.getTorus());
            }

            if (xmlMapSettings.getMapSize() != null) {
                spatialMapModel
                        .setWidth(xmlMapSettings.getMapSize().getWidth());
                spatialMapModel.setHeight(xmlMapSettings.getMapSize()
                        .getHeight());
            }

            if (xmlMapSettings.getPatchSize() != null) {
                spatialMapModel.setGridCols((int) Math.round(xmlMapSettings
                        .getMapSize().getWidth()
                        / xmlMapSettings.getPatchSize().getWidth()));
                spatialMapModel.setGridRows((int) Math.round(xmlMapSettings
                        .getMapSize().getHeight()
                        / xmlMapSettings.getPatchSize().getHeight()));
            }

            if (xmlMapSettings.hasDiscrete()) {
                spatialMapModel.setDiscrete(xmlMapSettings.getDiscrete());
            }
            if (xmlMapSettings.hasDrawGrid()) {
                spatialMapModel.setDrawGrid(xmlMapSettings.getDrawGrid());
            }
            if (xmlMapSettings.hasRotateImages()) {
                spatialMapModel.setRotateImages(xmlMapSettings
                        .getRotateImages());
            }
            if (xmlMapSettings.getBackgroundImage() != null) {
                spatialMapModel.setBackgroundImageFile(_xmlModelReader
                        .createRealAbsolutePath(xmlMapSettings
                                .getBackgroundImage().getValue()));
            }

        }
    }

    IObjectInstanceDeclaration getRealObjectInstanceDeclaration(
            XMLObjectInstance xmlObjectInstance) throws XMLReadException {

        Assert.assertFalse(xmlObjectInstance instanceof XMLWorldInstance);

        if (!containsXMLObject(xmlObjectInstance)) {
            setDeclaration(xmlObjectInstance,
                    createRealObjectInstanceDeclaration(xmlObjectInstance));
        }

        IObjectInstanceDeclaration objectInstanceDeclaration = (IObjectInstanceDeclaration) getDeclaration(xmlObjectInstance);
        Assert.assertNotNull(objectInstanceDeclaration);

        return objectInstanceDeclaration;
    }

    IWorldInstanceDeclaration getRealWorldInstanceDeclaration(
            XMLWorldInstance xmlWorldInstance) throws XMLReadException {

        if (!containsXMLObject(xmlWorldInstance)) {
            setDeclaration(xmlWorldInstance,
                    createRealWorldInstanceDeclaration(xmlWorldInstance));
        }

        IWorldInstanceDeclaration worldInstanceDeclaration = (IWorldInstanceDeclaration) getDeclaration(xmlWorldInstance);
        Assert.assertNotNull(worldInstanceDeclaration);

        return worldInstanceDeclaration;
    }
}