package de.uniwue.ki.mas.xml.reader;

import java.awt.Color;
import java.io.*;
import java.util.*;
import java.util.logging.Logger;

import junit.framework.Assert;
import mygale.mcge.fig.*;

import org.exolab.castor.xml.CastorException;
import org.exolab.castor.xml.MarshalException;

import de.uniwue.ki.mas.basics.action.debugging.FunctionActionDebuggingDeclarationFactory;
import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.spatial.CircularSpatialInfo;
import de.uniwue.ki.mas.basics.feature.spatial.function.FunctionDeclarationSpatialFactory;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.user.UserFeatureClassDeclarationFactory;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.list.*;
import de.uniwue.ki.mas.basics.functionUser.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.enum.EnumItem;
import de.uniwue.ki.mas.basics.type.enum.SpecificEnumType;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.basics.type.user.UserTypeDeclarationList;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.ObjectInstanceDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.RessourceClassDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.agentclass.AgentClassDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.worldclass.WorldClassDeclarationList;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.model.ISeSAmDeclaration;
import de.uniwue.ki.mas.model.ISeSAmModelDeclaration;
import de.uniwue.ki.mas.model.graph.*;
import de.uniwue.ki.mas.sim.analysis.*;
import de.uniwue.ki.mas.sim.experiment.*;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.mas.sim.situation.*;
import de.uniwue.ki.mas.util.declaration.*;
import de.uniwue.ki.mas.xml.IDelaySettings;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.util.IRemoveable;
import de.uniwue.ki.util.Utils;

/**
 * Creation date: (14.09.2000 17:16:59)
 * 
 * @author Christoph Oechslein
 */
public class XMLModelReader implements IXMLObjectToDeclaration, IRemoveable {

    public static final Logger LOGGER = Logger.getLogger(XMLModelReader.class.getName());

    public static String deescapeDocumentationString(String stringToDeescape) {
        if (stringToDeescape != null)
            return stringToDeescape.replaceAll("&#010;", "\n").replaceAll("&#009;", "\t");
        else
            return null;
    }
    private final IFigMapper _figMapper;
    private final boolean _setCompileFlags;
    private final Map _functionDeclarationListAbleCache = new HashMap();

    private final IFunctionDeclarationList _nullDeclarationFunctionDeclarationList;
    private final Map _objectClassDeclToXMLModelFunctionReader = new HashMap();
    private final Map _objectClassDeclToXMLModelTypeReader = new HashMap();
    private final ISeSAmDeclaration _seSAmDeclaration;
    private final DelaySetterHelper _xmlModelFunctionDelaySetter = new DelaySetterHelper();
    private final XMLModelObjectReader _xmlModelObjectReader = new XMLModelObjectReader(this);
    private final XMLObjectToDeclaration _xmlObjectToDeclaration = new XMLObjectToDeclaration();

    private XMLSeSAmModel _xmlSeSAmModel;

    public XMLModelReader(
        ISeSAmDeclaration seSAmDeclaration,
        IFigMapper figMapper,
        boolean setCompileFlags) {
        super();

        _figMapper = figMapper;
        _setCompileFlags = setCompileFlags;
        _seSAmDeclaration = seSAmDeclaration;
        _nullDeclarationFunctionDeclarationList =
            new FunctionDeclarationList(
                _seSAmDeclaration.getSeSAmModelDeclaration().getAllAvailibleFunctionDeclarations());
        _nullDeclarationFunctionDeclarationList.setComposition(false);
        //for backward compatibility
        FunctionDeclarationSpatialFactory.insertSpatialFeatureFunctions(
            _nullDeclarationFunctionDeclarationList);
    }

    public void addDelaySettings(IDelaySettings delaySettings) {

        _xmlModelFunctionDelaySetter.addDelaySettings(delaySettings);
    }

    public void addDelaySettings(
        IFunctionCallDeclaration functionCallDeclaration,
        XMLFunctionCall xmlFunctionCall,
        ISpecificType mostGeneralFunCallTyp,
        XMLModelFunctionReader xmlModelFunctionReader) {

        _xmlModelFunctionDelaySetter.addDelaySettings(
            functionCallDeclaration,
            xmlFunctionCall,
            mostGeneralFunCallTyp,
            xmlModelFunctionReader);
    }

    public void addDelaySettings(
        IFunctionCallDeclaration functionCallDeclaration,
        XMLFunctionCall xmlFunctionCall,
        XMLModelFunctionReader xmlModelFunctionReader) {

        _xmlModelFunctionDelaySetter.addDelaySettings(
            functionCallDeclaration,
            xmlFunctionCall,
            xmlModelFunctionReader);
    }

    public void addRealSeSAmModel() throws XMLReadException {
        addToRealUserTypeDeclarationList();
        addToRealWorldClassDeclarationList();
        addToRealAgentClassDeclarationList();
        addToRealRessourceClassDeclarationList();
        addToRealUserFunctionDeclarationList();
        addToRealUserFeatureClassDeclarationList();
        evaluateWorldClassDeclarationList();
        evaluateAgentClassDeclarationList();
        evaluateRessourceClassDeclarationList();
        doAllDelaySettings();
        addToRealSituationDeclarationList();
        addToRealAnalysisDeclarationList();
        addToRealSimulationDeclarationList();
        addToRealExperimentDeclarationList();
        addToRealUmlLayoutList();
        doAllDelaySettings();
    }

    private void addToRealUmlLayoutList() {
        UMLLayoutGraphDeclarationList list = getSeSAmDeclaration().getUMLGraphDeclerationList();
        XMLUMLLayout[] layouts = _xmlSeSAmModel.getUmlLayout();

        final IUMLLayoutGraphDeclaration originalGraph =
            new UMLLayoutGraphParser(
                getSeSAmModelDeclaration().getAvailibleObjectClassDeclarationList(),
                getSeSAmModelDeclaration().getAvailibleFeatureClassDeclarationList())
                .getGraphModel();

        if (layouts != null) {

            for (int i = 0; i < layouts.length; i++) {
                IUMLLayoutGraphDeclaration realUmlLayout = getRealUmlLayout(layouts[i], originalGraph);

                //                realUmlLayout.overlayWithOtherGraphDeclaration(originalGraph);
                list.add(realUmlLayout);
            }
        }

    }

    private IUMLLayoutGraphDeclaration getRealUmlLayout(
        XMLUMLLayout xmlLayout,
        IUMLLayoutGraphDeclaration originalGraph) {

        if (!containsXMLObject(xmlLayout)) {
            IUMLLayoutGraphDeclaration declaration = createRealUmlLayoutDeclaration(xmlLayout);
            declaration.overlayWithOtherGraphDeclaration(originalGraph);
            setDeclaration(xmlLayout, declaration);
        }

        UMLLayoutGraphDeclaration layout = (UMLLayoutGraphDeclaration) getDeclaration(xmlLayout);

        if (Assert.ASSERTION_ON) {
            Assert.assertNotNull(layout);
        }

        return layout;
    }

    private UMLLayoutGraphNodeFactory _umlFac;
    private UMLLayoutGraphNodeFactory getUMLLayoutGraphNodeFactory() {
        if (_umlFac == null) {

            _umlFac = UMLLayoutGraphNodeFactory.getInstance();
        }

        return _umlFac;
    }
    private IUMLLayoutGraphDeclaration createRealUmlLayoutDeclaration(XMLUMLLayout xmlLayout) {
        UMLLayoutGraphDeclaration layout = new UMLLayoutGraphDeclaration();

        layout.setShortDescription(xmlLayout.getLayoutName());
        UMLLayoutGraphNodeFactory factory = getUMLLayoutGraphNodeFactory();
        factory.setGraphModel(layout);

        XMLUMLNode[] nodes = xmlLayout.getNode();
        for (int i = 0; i < nodes.length; i++) {
            XMLUMLNode xmlNode = nodes[i];

            try {
                IUMLLayoutGraphNode realNode =
                    factory.createNode(
                        (IDocumentableDeclaration) getDeclaration((XMLRootObject) xmlNode.getObjectID()));

                // es muss dann noch gleich die Fig erzeugt werden, um dieser die Bounds und den
                // Fixed-Status mitzugeben. Datenmodell und Fig mssen dann per FigMapper verknpft werden.
                // Problem: fr das Erzeugen von FigNodes bruchte man hier das SeSAm GUI-Projekt fr die
                // FigNodeFactory. Auf das Projekt hat man hier jedoch keinen Zugriff...
                // Wie wird das fr Activities gemacht - dort msste es ja hnlich Probleme geben.

                try {

                    IFigNodeFactory figUmlLayoutFactory =
                        (IFigNodeFactory) getClass()
                            .getClassLoader()
                            .loadClass("de.uniwue.ki.gui.mas.model.graph.FigUMLGraphNodeFactory")
                            .newInstance();

                    IFigNode figNode = FigHelper.createFigNode(getFigMapper(), realNode, figUmlLayoutFactory);

                    if (Assert.ASSERTION_ON) {
                        Assert.assertEquals(figNode.getNetObject(), realNode);
                    }

                    // TODO fig nicht in negativen Bereich positionieren
                    figNode.setBounds(
                        xmlNode.getXpos(),
                        xmlNode.getYpos(),
                        xmlNode.getWidth(),
                        xmlNode.getHeight());

                } catch (ClassNotFoundException exception) {
                    throw new RuntimeException(exception);
                } catch (IllegalAccessException exception) {
                    throw new RuntimeException(exception);
                } catch (InstantiationException exception) {
                    throw new RuntimeException(exception);
                }

                layout.addNode(realNode);

            } catch (CannotCreateNodeException e) {
                System.out.println(
                    "XMLModelReader - cannot load UMLGraph-node for "
                        + ((IDocumentableDeclaration) getDeclaration((XMLRootObject) xmlNode.getObjectID()))
                            .getName());
            }

        }

        return layout;
    }

    private void addToRealAgentClassDeclarationList() throws XMLReadException {

        AgentClassDeclarationList agentClassDeclarationList =
            getSeSAmModelDeclaration()
                .getAgentClassDeclarationListAndFactory()
                .getAgentClassDeclarationList();
        XMLAgentClass[] xmlAgents = _xmlSeSAmModel.getAgentClass();

        if (xmlAgents != null) {

            for (int i = 0; i < xmlAgents.length; i++) {
                agentClassDeclarationList.add(
                    _xmlModelObjectReader.getRealAgentClassDeclaration(xmlAgents[i]));
            }
        }
    }

    private void evaluateAgentClassDeclarationList() throws XMLReadException {
        XMLAgentClass[] xmlAgents = _xmlSeSAmModel.getAgentClass();

        if (xmlAgents != null) {
            for (int i = 0; i < xmlAgents.length; i++) {
                _xmlModelObjectReader.fillRealAgentClassDeclaration(xmlAgents[i]);
            }
        }
    }

    private void addToRealAnalysisDeclarationList() throws XMLReadException {

        AnalysisDeclarationList analysisDeclarationList =
            (AnalysisDeclarationList) getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getAnalysisDeclarationListAndFactory()
                .getDeclarationList();
        XMLAnalysis[] xmlAnalysiss = _xmlSeSAmModel.getAnalysis();

        if (xmlAnalysiss != null) {

            for (int i = 0; i < xmlAnalysiss.length; i++) {
                analysisDeclarationList.add(getRealAnalysisDeclaration(xmlAnalysiss[i]));
            }
        }
    }

    private void addToRealExperimentDeclarationList() throws XMLReadException {

        ExperimentDeclarationList simulationDeclarationList =
            getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getExperimentDeclarationListAndFactory()
                .getExperimentDeclarationList();
        XMLExperiment[] xmlExperiments = _xmlSeSAmModel.getExperiment();

        if (xmlExperiments != null) {

            for (int i = 0; i < xmlExperiments.length; i++) {
                simulationDeclarationList.add(createRealExperimentDeclaration(xmlExperiments[i]));
            }
        }
    }

    private void addToRealRessourceClassDeclarationList() throws XMLReadException {

        RessourceClassDeclarationList agentClassDeclarationList =
            getSeSAmModelDeclaration()
                .getRessourceClassDeclarationListAndFactory()
                .getRessourceClassDeclarationList();
        XMLRessourceClass[] xmlRessources = _xmlSeSAmModel.getRessourceClass();

        if (xmlRessources != null) {

            for (int i = 0; i < xmlRessources.length; i++) {
                agentClassDeclarationList.add(
                    _xmlModelObjectReader.getRealRessourceClassDeclaration(xmlRessources[i]));
            }
        }
    }

    private void evaluateRessourceClassDeclarationList() throws XMLReadException {

        XMLRessourceClass[] xmlRessources = _xmlSeSAmModel.getRessourceClass();

        if (xmlRessources != null) {
            for (int i = 0; i < xmlRessources.length; i++) {
                _xmlModelObjectReader.fillRealRessourceClassDeclaration(xmlRessources[i]);
            }
        }
    }

    private void addToRealSimulationDeclarationList() throws XMLReadException {

        SimulationDeclarationList simulationDeclarationList =
            getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getSimulationDeclarationListAndFactory()
                .getSimulationDeclarationList();
        XMLSimulation[] xmlSimulations = _xmlSeSAmModel.getSimulation();

        if (xmlSimulations != null) {

            for (int i = 0; i < xmlSimulations.length; i++) {
                simulationDeclarationList.add(getRealSimulationDeclaration(xmlSimulations[i]));
            }
        }
    }

    private void addToRealSituationDeclarationList() throws XMLReadException {

        SituationDeclarationList situationDeclarationList =
            getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getSituationDeclarationListAndFactory()
                .getSituationDeclarationList();
        XMLSituation[] xmlSituations = _xmlSeSAmModel.getSituation();

        if (xmlSituations != null) {

            for (int i = 0; i < xmlSituations.length; i++) {
                situationDeclarationList.add(getRealSituationDeclaration(xmlSituations[i]));
            }
        }
    }

    private void addToRealUserFeatureClassDeclarationList() throws XMLReadException {

        IFeatureClassDeclarationList featureClassDeclarationList =
            getSeSAmModelDeclaration()
                .getSeSAmModelBasicsDeclaration()
                .getUserFeatureClassDeclarationListAndFactory()
                .getUserFeatureClassDeclarationList();
        XMLUserFeatureClass[] userFeatureClasses = _xmlSeSAmModel.getUserFeatureClass();

        if (userFeatureClasses != null) {

            for (int i = 0; i < userFeatureClasses.length; i++) {
                featureClassDeclarationList.add(getRealUserFeatureClassDeclaration(userFeatureClasses[i]));
            }
        }
    }

    private void addToRealUserFunctionDeclarationList() throws XMLReadException {

        UserFunctionDeclarationList userFunctionDeclarationList =
            getSeSAmModelDeclaration()
                .getSeSAmModelBasicsDeclaration()
                .getUserFunctionDeclarationListAndFactory()
                .getUserFunctionDeclarationList();
        XMLUserFunction[] userFunctions = _xmlSeSAmModel.getUserFunction();

        if (userFunctions != null) {

            for (int i = 0; i < userFunctions.length; i++) {
                userFunctionDeclarationList.add(getRealUserFunctionDeclaration(userFunctions[i]));
            }
        }
    }

    private void addToRealUserTypeDeclarationList() throws XMLReadException {

        UserTypeDeclarationList userTypeDeclaration =
            getSeSAmModelDeclaration()
                .getSeSAmModelBasicsDeclaration()
                .getUserTypeDeclarationListAndFactory()
                .getUserTypeDeclarationList();
        XMLUserType[] userTypes = _xmlSeSAmModel.getUserType();

        if (userTypes != null) {
            for (int i = 0; i < userTypes.length; i++) {
                userTypeDeclaration.add(getRealUserTypeDeclaration(userTypes[i]));
            }
        }
    }

    private void addToRealWorldClassDeclarationList() throws XMLReadException {

        WorldClassDeclarationList worldClassDeclarationList =
            getSeSAmModelDeclaration()
                .getWorldClassDeclarationListAndFactory()
                .getWorldClassDeclarationList();
        XMLWorldClass[] xmlWorlds = _xmlSeSAmModel.getWorldClass();

        if (xmlWorlds != null) {

            for (int i = 0; i < xmlWorlds.length; i++) {
                worldClassDeclarationList.add(
                    _xmlModelObjectReader.getRealWorldClassDeclaration(xmlWorlds[i]));
            }
        }
    }

    private void evaluateWorldClassDeclarationList() throws XMLReadException {
        XMLWorldClass[] xmlWorlds = _xmlSeSAmModel.getWorldClass();

        if (xmlWorlds != null) {
            for (int i = 0; i < xmlWorlds.length; i++) {
                _xmlModelObjectReader.fillRealWorldClassDeclaration(xmlWorlds[i]);
            }
        }
    }

    public void changeDeclaration(
        XMLRootObject object,
        IDeclaration oldDeclaration,
        IDeclaration newDeclaration) {
        _xmlObjectToDeclaration.changeDeclaration(object, oldDeclaration, newDeclaration);
    }

    public boolean containsXMLObject(XMLRootObject object) {

        return _xmlObjectToDeclaration.containsXMLObject(object);
    }

    private IAnalysisDeclaration createRealAnalysisDeclaration(XMLAnalysis xmlAnalysis)
        throws XMLReadException {

        IAnalysisDeclaration analysisDeclaration =
            (IAnalysisDeclaration) getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getAnalysisDeclarationListAndFactory()
                .getDeclarationFactory()
                .createDeclaration();

        setName(analysisDeclaration, xmlAnalysis);

        if (xmlAnalysis.hasDefault()) {
            analysisDeclaration.setDefaultAnalysis(xmlAnalysis.getDefault());
        } else {
            analysisDeclaration.setDefaultAnalysis(false);
        }

        setRealAnalysisSettings(xmlAnalysis, analysisDeclaration.getAnalysisDeltaSettings());

        if ((xmlAnalysis.getType() == null) || ("CSV".equalsIgnoreCase(xmlAnalysis.getType()))) {

            CSVAnalysisTypeDeclaration csvAnalysisTypeDeclaration = new CSVAnalysisTypeDeclaration();
            analysisDeclaration.setAnalysisTypeDeclaration(csvAnalysisTypeDeclaration);

            if (xmlAnalysis.getFile() != null) {
                csvAnalysisTypeDeclaration.setFileString(
                    createRealAbsolutePath(xmlAnalysis.getFile()).toString());
            }

            if (xmlAnalysis.getSeparatorString() != null) {
                csvAnalysisTypeDeclaration.setSeparatorString(xmlAnalysis.getSeparatorString());
            }
            if (xmlAnalysis.getTimeColumnName() != null) {
                csvAnalysisTypeDeclaration.setTimeColumnName(xmlAnalysis.getTimeColumnName());
            }

        } else if ("Block".equalsIgnoreCase(xmlAnalysis.getType())) {
            analysisDeclaration.setAnalysisTypeDeclaration(new BlockChartAnalysisTypeDeclaration());

        } else if ("Series".equalsIgnoreCase(xmlAnalysis.getType())) {

            SeriesChartAnalysisTypeDeclaration seriesChartAnalysisTypeDeclaration =
                new SeriesChartAnalysisTypeDeclaration();
            analysisDeclaration.setAnalysisTypeDeclaration(seriesChartAnalysisTypeDeclaration);

            if (xmlAnalysis.hasWindowSize()) {
                seriesChartAnalysisTypeDeclaration.setWindowSize(xmlAnalysis.getWindowSize());
            }
        } else if ("Table".equalsIgnoreCase(xmlAnalysis.getType())) {

            TableAnalysisTypeDeclaration tableAnalysisTypeDeclaration = new TableAnalysisTypeDeclaration();
            analysisDeclaration.setAnalysisTypeDeclaration(tableAnalysisTypeDeclaration);

        } else {
            throw new XMLReadException("Unknown Analyse Type: " + xmlAnalysis.getType());
        }

        AnalysisItemDeclarationList analysisItemDeclarationList =
            analysisDeclaration.getAnalysisItemDeclarationListAndFactory().getAnalysisItemDeclarationList();

        for (int i = 0; i < xmlAnalysis.getItemCount(); i++) {
            analysisItemDeclarationList.add(
                createRealAnalysisItemDeclaration(
                    analysisDeclaration.getAnalysisTypeDeclaration(),
                    xmlAnalysis.getItem(i)));
        }

        return analysisDeclaration;
    }

    private IAnalysisItemDeclaration createRealAnalysisItemDeclaration(
        IAnalysisTypeDeclaration analysisTypeDeclaration,
        XMLAnalysisItem xmlAnalysisItem)
        throws XMLReadException {

        IAnalysisItemDeclaration analysisItemDeclaration =
            analysisTypeDeclaration.getAnalysisItemDeclarationFactory().createAnalysisItemDeclaration();
        analysisItemDeclaration.setName(xmlAnalysisItem.getName());
        setRealFunctionCallDelay(analysisItemDeclaration.getValueFunctionCallDeclaration(), xmlAnalysisItem);

        if (xmlAnalysisItem.hasColorRed()
            && (analysisItemDeclaration instanceof ColorAnalysisItemDeclaration)) {

            Color color =
                new Color(
                    xmlAnalysisItem.getColorRed(),
                    xmlAnalysisItem.getColorGreen(),
                    xmlAnalysisItem.getColorBlue());
            ((ColorAnalysisItemDeclaration) analysisItemDeclaration).setColor(color);
        }

        return analysisItemDeclaration;
    }

    IArgumentDeclaration createRealArgumentDeclaration(
        XMLTypeValueGroup xmlTypeValue,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelTypeReader(functionDeclarationList).createRealArgumentDeclaration(xmlTypeValue);
    }

    private IExperimentDeclaration createRealExperimentDeclaration(XMLExperiment xmlExperiment)
        throws XMLReadException {

        IExperimentDeclaration experimentDeclaration =
            ExperimentDeclarationFactory.createExperimentDeclaration(
                FunctionCallDeclarationFactory.getInstance().createFunctionCallDeclaration(
                    VoidType.getInstance()));
        setName(experimentDeclaration, xmlExperiment);

        setRealFunctionCallDelay(
            experimentDeclaration.getSimulationActionFunctionCallDeclaration(),
            xmlExperiment.getFunctionCall());

        experimentDeclaration.setConfigurationFileName(
            createRealAbsolutePath(xmlExperiment.getConfigurationFile()).toString());

        if (xmlExperiment.hasTest())
            experimentDeclaration.setTestSimulationDeclaration(xmlExperiment.getTest());

        return experimentDeclaration;
    }

    public File createRealAbsolutePath(String fileName) throws XMLReadException {
        return getXMLModelTypeReader(_nullDeclarationFunctionDeclarationList).createRealAbsolutePath(
            fileName);
    }

    private IVariableClassDeclaration createRealFeatureVariableClassDeclaration(XMLVariableClass xmlVariable)
        throws XMLReadException {

        if (Assert.ASSERTION_ON) {
            Assert.assertNotNull(xmlVariable);
        }

        IVariableClassDeclaration realVariableClass =
            FeatureVariableClassDeclarationFactory.createFeatureVariableClassDeclaration();
        setRealVariableClassDeclaration(
            realVariableClass,
            _nullDeclarationFunctionDeclarationList,
            xmlVariable);

        return realVariableClass;
    }

    IFunctionCallDeclaration createRealFunctionCall(
        XMLFunctionCall xmlFunctionCall,
        IFunctionDeclarationListAble functionDeclarationList,
        IFunctionCallDeclarationFactory functionCallDeclarationFactory)
        throws XMLReadException {

        return getXMLModelFunctionReader(functionDeclarationList).createRealFunctionCall(
            xmlFunctionCall,
            functionCallDeclarationFactory);
    }
    
    /*
     * Neu bei Rainer: TestGate fr GenericFeature (12.03.2004)
     */
    public IFunctionCallInternDeclaration createRealFunctionCallInternDeclaration(
            IObjectClassDeclaration objectClassdeclaration,
            XMLFunctionCallInternGroup xmlFunctionCall)
            throws XMLReadException {

            return getXMLModelFunctionReader(getFunctionDeclarationListFor(objectClassdeclaration)).createRealFunctionCallIntern(
                xmlFunctionCall);
        }

    private ISimulationDeclaration createRealSimulationDeclaration(XMLSimulation xmlSimulation)
        throws XMLReadException {

        Assert.assertNull(xmlSimulation.getNameCall());
        Assert.assertNull(xmlSimulation.getSituation());

        ISimulationDeclaration simulationDeclaration =
            getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getSimulationDeclarationListAndFactory()
                .getSimulationDeclarationFactory()
                .createSimulationDeclaration(
                    getRealSituationDeclaration((XMLSituation) xmlSimulation.getStartSituationID()));

        setRealSimulationDeclaration(xmlSimulation, simulationDeclaration);

        return simulationDeclaration;
    }

    ISimulationWithNameFunctionCallDeclDeclaration createRealSimulationWithNameFunctionCallDeclDeclaration(XMLSimulation xmlSimulation)
        throws XMLReadException {

        Assert.assertNull(xmlSimulation.getStartSituationID());

        ISimulationWithNameFunctionCallDeclDeclaration simulationDeclaration =
            getSeSAmModelDeclaration()
                .getSimulationElementsDeclaration()
                .getSimulationDeclarationListAndFactory()
                .getSimulationDeclarationFactory()
                .createSimulationWithNameFunctionCallDeclDeclaration(
                    createRealSituationDeclaration(xmlSimulation.getSituation()));

        setRealFunctionCallDelay(
            simulationDeclaration.getNameFunctionCallDeclaration(),
            xmlSimulation.getNameCall());

        setRealSimulationDeclaration(xmlSimulation, simulationDeclaration);

        return simulationDeclaration;
    }

    private ISituationDeclaration createRealSituationDeclaration(XMLSituation xmlSituation)
        throws XMLReadException {

        ISituationDeclaration situationDeclaration =
            SituationDeclarationFactory.createSituationDeclaration(
                _xmlModelObjectReader.getRealWorldInstanceDeclaration(xmlSituation.getWorldInstance()));
        setName(situationDeclaration, xmlSituation);

        ObjectInstanceDeclarationList objectInstanceDeclarationList =
            situationDeclaration.getWorldInstanceDeclaration().getObjectInstanceDeclarationList();
        XMLObjectInstance[] xmlObjectInstances = xmlSituation.getObjectInstance();

        for (int i = 0; i < xmlObjectInstances.length; i++) {
            objectInstanceDeclarationList.add(
                _xmlModelObjectReader.getRealObjectInstanceDeclaration(xmlObjectInstances[i]));
        }

        return situationDeclaration;
    }

    CircularSpatialInfo createRealSpatialInfo(
        XMLSpatialInfo xmlSpatialInfo,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelTypeReader(functionDeclarationList).createRealSpatialInfo(xmlSpatialInfo);
    }

    ISpecificType createRealType(
        XMLFunctionCall xmlFunctionCall,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelFunctionReader(functionDeclarationList).createRealType(xmlFunctionCall);
    }

    ISpecificType createRealType(XMLTypeGroup xmlType, IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelTypeReader(functionDeclarationList).createRealType(xmlType);
    }

    ISpecificType createRealType(
        XMLTypeValueGroup xmlTypeValue,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelTypeReader(functionDeclarationList).createRealType(xmlTypeValue);
    }

    private IFeatureClassDeclaration createRealUserFeatureClassDeclaration(XMLUserFeatureClass xmlUserFeatureClass)
        throws XMLReadException {

        IUserFeatureClassDeclaration userFeatureClassDeclaration =
            UserFeatureClassDeclarationFactory.createUserFeatureClassDeclaration();
        setName(userFeatureClassDeclaration, xmlUserFeatureClass);

        for (int i = 0; i < xmlUserFeatureClass.getVariableClassCount(); i++) {
            userFeatureClassDeclaration
                .getFeatureVariableClassDeclarationListAndFactory()
                .getVariableClassDeclarationList()
                .add(getRealFeatureVariableClassDeclaration(xmlUserFeatureClass.getVariableClass(i)));
        }

        if (xmlUserFeatureClass.getUserFunctionCount() > 0) {

            for (int i = 0; i < xmlUserFeatureClass.getUserFunctionCount(); i++) {
                userFeatureClassDeclaration
                    .getUserFunctionDeclarationListAndFactory()
                    .getUserFunctionDeclarationList()
                    .add(
                    getRealUserFunctionDeclaration(xmlUserFeatureClass.getUserFunction(i)));
            }
        }

        return userFeatureClassDeclaration;
    }

    private IUserFunctionDeclaration createRealUserFunctionDeclaration(XMLUserFunction xmlUserFunction)
        throws XMLReadException {

        IUserFunctionDeclaration userFunctionDeclaration =
            UserFunctionDeclarationFactory.getInstance(true).createUserFunctionDeclaration();

        setName(
            userFunctionDeclaration,
            xmlUserFunction.getName(),
            deescapeDocumentationString(xmlUserFunction.getDocumentation()));

        if (xmlUserFunction.getXMLTypeGroup() != null)
            userFunctionDeclaration.setSpecificOutputType(
                createRealType(xmlUserFunction.getXMLTypeGroup(), _nullDeclarationFunctionDeclarationList));

        setParametricFunctionCallDeclaration(
            userFunctionDeclaration.getParametricFunctionCallDeclaration(),
            xmlUserFunction,
            _nullDeclarationFunctionDeclarationList);

        if (!xmlUserFunction.hasExpert())
            userFunctionDeclaration.setExpertFunctionDeclaration(false);
        else
            userFunctionDeclaration.setExpertFunctionDeclaration(xmlUserFunction.getExpert());

        if (!xmlUserFunction.hasExternal())
            userFunctionDeclaration.setExternal(true);
        else
            userFunctionDeclaration.setExternal(xmlUserFunction.getExternal());

        if (!xmlUserFunction.hasInline())
            userFunctionDeclaration.setInlinePossible(true);
        else
            userFunctionDeclaration.setInlinePossible(xmlUserFunction.getInline());

        return userFunctionDeclaration;
    }

    private IVariableClassDeclaration createRealVariableClassDeclaration(
        IFunctionDeclarationListAble functionDeclarationList,
        XMLVariableClass xmlVariable)
        throws XMLReadException {
        Assert.assertNotNull(xmlVariable);

        IVariableClassDeclaration realVariableClass =
            (IVariableClassDeclaration) VariableClassDeclarationFactory.getInstance().createDeclaration();
        setRealVariableClassDeclaration(realVariableClass, functionDeclarationList, xmlVariable);

        return realVariableClass;
    }

    public void doAllDelaySettings() throws XMLReadException {
        _xmlModelFunctionDelaySetter.doAllDelaySettings();
    }

    public void doDelaySettingsIfAvailible(IFunctionCallDeclaration functionCallDeclaration)
        throws XMLReadException {

        if (_xmlModelFunctionDelaySetter.hasDelaySettingsFor(functionCallDeclaration)) {
            _xmlModelFunctionDelaySetter.doDelaySettingsForAndRemove(functionCallDeclaration);
        }
    }

    public IDeclaration getDeclaration(XMLRootObject object) {

        return _xmlObjectToDeclaration.getDeclaration(object);
    }

    public final IFigMapper getFigMapper() {
        return _figMapper;
    }

    IFunctionArgumentDeclaration getFunctionArgumentDeclaration(XMLFunctionParameter xmlFunctionParameter)
        throws XMLReadException {

        return getXMLModelTypeReader(_nullDeclarationFunctionDeclarationList).getFunctionArgumentDeclaration(
            xmlFunctionParameter);
    }

    IFunctionDeclarationListAble getFunctionDeclarationListFor(IObjectClassDeclaration objectClassDeclaration) {

        if (_functionDeclarationListAbleCache.containsKey(objectClassDeclaration)) {
            return (IFunctionDeclarationListAble) _functionDeclarationListAbleCache.get(
                objectClassDeclaration);

        } else {
            CacheObjectClassDeclFunctionDeclarationListAble listAble =
                new CacheObjectClassDeclFunctionDeclarationListAble(
                    getSeSAmModelDeclaration(),
                    objectClassDeclaration);

            _functionDeclarationListAbleCache.put(objectClassDeclaration, listAble);

            return listAble;
        }
    }

    public Integer getIdFromUniqueXMLId(String idString) throws NumberFormatException {

        int index = idString.lastIndexOf("_");

        if (index == -1) {
            throw new NumberFormatException();
        } else {

            return Integer.valueOf(idString.substring(index + 1));
        }
    }

    private IAnalysisDeclaration getRealAnalysisDeclaration(XMLAnalysis xmlAnalysis)
        throws XMLReadException {

        if (!containsXMLObject(xmlAnalysis)) {
            setDeclaration(xmlAnalysis, createRealAnalysisDeclaration(xmlAnalysis));
        }

        IAnalysisDeclaration analysisDeclaration = (IAnalysisDeclaration) getDeclaration(xmlAnalysis);
        Assert.assertNotNull(analysisDeclaration);

        return analysisDeclaration;
    }

    IObjectFeatureClassDeclaration getRealBuildinFeatureClassDeclaration(
        String name,
        IObjectClassDeclaration objectClassDeclaration)
        throws XMLReadException {

        IObjectFeatureClassDeclaration objectFeatureClassDeclaration =
            (IObjectFeatureClassDeclaration) objectClassDeclaration
                .getObjectFeatureClassDeclarationList()
                .getDeclarationByName(name);

        if (objectFeatureClassDeclaration == null) {

            IFeatureClassDeclaration nonDefaultFeatureClassDeclaration =
                (IFeatureClassDeclaration) getSeSAmModelDeclaration()
                    .getAvailibleFeatureClassDeclarationListFor(objectClassDeclaration)
                    .getDeclarationByName(name);

            //Assert.ASSERTION_ON = true;
            Assert.assertNotNull(
                "BuildinFeature " + name + " not found! Maybe there is a plugin missing?",
                nonDefaultFeatureClassDeclaration);

            return nonDefaultFeatureClassDeclaration.createObjectFeatureClassDeclaration(
                objectClassDeclaration);
        } else {

            return objectFeatureClassDeclaration;
        }
    }

    private IVariableClassDeclaration getRealFeatureVariableClassDeclaration(XMLVariableClass xmlVariable)
        throws XMLReadException {
        Assert.assertNotNull(xmlVariable);

        if (!containsXMLObject(xmlVariable)) {
            setDeclaration(xmlVariable, createRealFeatureVariableClassDeclaration(xmlVariable));
        }

        IVariableClassDeclaration variableClassDeclaration =
            (IVariableClassDeclaration) getDeclaration(xmlVariable);
        Assert.assertNotNull(variableClassDeclaration);

        return variableClassDeclaration;
    }

    IObjectInstanceDeclaration getRealObjectInstanceDeclaration(XMLObjectInstance xmlObjectInstance)
        throws XMLReadException {
        return _xmlModelObjectReader.getRealObjectInstanceDeclaration(xmlObjectInstance);
    }

    private ISimulationDeclaration getRealSimulationDeclaration(XMLSimulation xmlSimulation)
        throws XMLReadException {

        if (!containsXMLObject(xmlSimulation)) {
            setDeclaration(xmlSimulation, createRealSimulationDeclaration(xmlSimulation));
        }

        ISimulationDeclaration simulationDeclaration = (ISimulationDeclaration) getDeclaration(xmlSimulation);
        Assert.assertNotNull(simulationDeclaration);

        return simulationDeclaration;
    }

    private ISituationDeclaration getRealSituationDeclaration(XMLSituation xmlSituation)
        throws XMLReadException {

        if (!containsXMLObject(xmlSituation)) {
            setDeclaration(xmlSituation, createRealSituationDeclaration(xmlSituation));
        }

        ISituationDeclaration situationDeclaration = (ISituationDeclaration) getDeclaration(xmlSituation);
        Assert.assertNotNull(situationDeclaration);

        return situationDeclaration;
    }

    ISpecificFunctionDeclaration getRealSpecificFunctionDeclaration(
        String name,
        ISpecificFunctionType specificFunctionType,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        return getXMLModelFunctionReader(functionDeclarationList).getRealSpecificFunctionDeclaration(
            name,
            specificFunctionType);
    }

    IFeatureClassDeclaration getRealUserFeatureClassDeclaration(XMLUserFeatureClass xmlUserFeatureClass)
        throws XMLReadException {

        if (!containsXMLObject(xmlUserFeatureClass)) {
            setDeclaration(xmlUserFeatureClass, createRealUserFeatureClassDeclaration(xmlUserFeatureClass));
        }

        IFeatureClassDeclaration userFeatureClassDeclaration =
            (IFeatureClassDeclaration) getDeclaration(xmlUserFeatureClass);
        Assert.assertNotNull(userFeatureClassDeclaration);

        return userFeatureClassDeclaration;
    }

    IUserFunctionDeclaration getRealUserFunctionDeclaration(XMLUserFunction xmlUserFunction)
        throws XMLReadException {

        if (!containsXMLObject(xmlUserFunction)) {
            setDeclaration(xmlUserFunction, createRealUserFunctionDeclaration(xmlUserFunction));
        }

        return (IUserFunctionDeclaration) getDeclaration(xmlUserFunction);
    }

    IUserTypeDeclaration getRealUserTypeDeclaration(XMLUserType xmlUserType) throws XMLReadException {
        if (!containsXMLObject(xmlUserType)) {
            IUserTypeDeclaration userTypeDeclaration =
                getSeSAmModelDeclaration()
                    .getSeSAmModelBasicsDeclaration()
                    .getUserTypeDeclarationListAndFactory()
                    .getUserTypeDeclarationFactory()
                    .createUserTypeDeclaration(Utils.equals(xmlUserType.getType(), "composed"));
            // first put to map then set attributes due to recursive composed types
            setDeclaration(xmlUserType, userTypeDeclaration);
            setRealUserTypeDeclaration(xmlUserType, userTypeDeclaration);
        }

        return (IUserTypeDeclaration) getDeclaration(xmlUserType);
    }

    IVariableClassDeclaration getRealVariableClassDeclaration(
        IFunctionDeclarationListAble functionDeclarationList,
        XMLVariableClass xmlVariable)
        throws XMLReadException {

        Assert.assertNotNull(xmlVariable);

        if (!containsXMLObject(xmlVariable)) {
            setDeclaration(
                xmlVariable,
                createRealVariableClassDeclaration(functionDeclarationList, xmlVariable));
        }

        IVariableClassDeclaration variableClassDeclaration =
            (IVariableClassDeclaration) getDeclaration(xmlVariable);
        Assert.assertNotNull(variableClassDeclaration);

        return variableClassDeclaration;
    }

    public final ISeSAmDeclaration getSeSAmDeclaration() {

        return _seSAmDeclaration;
    }

    public final ISeSAmModelDeclaration getSeSAmModelDeclaration() {

        return getSeSAmDeclaration().getSeSAmModelDeclaration();
    }

    public ISpecificType getSpecificTypeByID(XMLUserTypeID userTypeID) throws XMLReadException {
        if (userTypeID.getSystemID() != null) {
            ISpecificType actualType =
                TypeManager.getSpecificTypeByName(
                    userTypeID.getSystemID(),
                    TypeManager.getInstance().getComplexTypes());
            if (actualType == null)
                throw new XMLReadException(
                    "System Type "
                        + userTypeID.getSystemID()
                        + " not found. Maybe there is a plugin missing ?");
            return actualType;
        }

        IUserTypeDeclaration userTypeDeclaration =
            getRealUserTypeDeclaration((XMLUserType) userTypeID.getId());

        return userTypeDeclaration.getSpecificType();
    }

    public double getVersion() {
        return _xmlSeSAmModel.getVersion();
    }

    private XMLModelFunctionReader getXMLModelFunctionReader(IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        if (!_objectClassDeclToXMLModelFunctionReader.containsKey(functionDeclarationList)) {
            _objectClassDeclToXMLModelFunctionReader.put(
                functionDeclarationList,
                new XMLModelFunctionReader(this, functionDeclarationList));
        }

        return (XMLModelFunctionReader) _objectClassDeclToXMLModelFunctionReader.get(functionDeclarationList);
    }

    XMLModelFunctionReader getXMLModelFunctionReader(IObjectClassDeclaration objectClassDeclaration)
        throws XMLReadException {

        return getXMLModelFunctionReader(getFunctionDeclarationListFor(objectClassDeclaration));
    }

    XMLModelTypeReader getXMLModelTypeReader(IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        if (!_objectClassDeclToXMLModelTypeReader.containsKey(functionDeclarationList)) {
            _objectClassDeclToXMLModelTypeReader.put(
                functionDeclarationList,
                new XMLModelTypeReader(this, functionDeclarationList));
        }

        return (XMLModelTypeReader) _objectClassDeclToXMLModelTypeReader.get(functionDeclarationList);
    }

    XMLModelTypeReader getXMLModelTypeReader(IObjectClassDeclaration objectClassDeclaration)
        throws XMLReadException {

        return getXMLModelTypeReader(getFunctionDeclarationListFor(objectClassDeclaration));
    }

    public void readXMLFile(Reader reader) throws XMLReadException, IOException {

        try {
            _xmlSeSAmModel = XMLSeSAmModel.unmarshalXMLSeSAmModel(reader);

            if (_setCompileFlags) {
                if (_xmlSeSAmModel.hasGlobalAssert())
                    Assert.setAssertionFlag(_xmlSeSAmModel.getGlobalAssert());
                if (_xmlSeSAmModel.hasCompileDebugFunction())
                    FunctionActionDebuggingDeclarationFactory.setCompileDebugFunctions(
                        _xmlSeSAmModel.getCompileDebugFunction());
                if (_xmlSeSAmModel.hasRunDebugFunction())
                    FunctionActionDebuggingDeclarationFactory.setRunDebugFunctions(
                        _xmlSeSAmModel.getRunDebugFunction());
                if (_xmlSeSAmModel.hasOptimize())
                    FunctionDeclarationCompiler.setDefaultCompileOn(_xmlSeSAmModel.getOptimize());
                if (_xmlSeSAmModel.hasInline())
                    FunctionDeclarationCompiler.setDefaultInlineOn(_xmlSeSAmModel.getInline());
                if (_xmlSeSAmModel.hasClosures())
                    AbstractSpecificFunctionType.setSupportClosures(_xmlSeSAmModel.getClosures());
            }

            addRealSeSAmModel();

        } catch (CastorException exception) {
            if (exception instanceof MarshalException) {
            	Throwable nestedException = ((MarshalException)exception).getException(); 
            	
            	if (nestedException instanceof InterruptedIOException)            
	                throw (InterruptedIOException) nestedException;
      	            
            } else{
				throw new XMLReadException(exception);	
            }
            
        } finally {
            reader.close();
            remove();
        }
    }

    public void remove() {
        removeFunctionDeclarationListAbleCache();
        _nullDeclarationFunctionDeclarationList.remove();
    }

    private void removeFunctionDeclarationListAbleCache() {
        Iterator it = _functionDeclarationListAbleCache.values().iterator();
        while (it.hasNext()) {
            CacheObjectClassDeclFunctionDeclarationListAble element =
                (CacheObjectClassDeclFunctionDeclarationListAble) it.next();
            element.remove();
        }
    }

    public void setDeclaration(XMLRootObject object, IDeclaration declaration) {
        _xmlObjectToDeclaration.setDeclaration(object, declaration);
    }

    void setName(IDocumentableDeclaration documentableDeclaration, String name, String documentation) {
        if (name != null) {
            documentableDeclaration.setShortDescription(name);
            documentableDeclaration.setUsesShortDescription(true);

        } else {
            documentableDeclaration.setUsesShortDescription(false);
        }

        if ((documentation != null) && (documentation.length() > 0)) {
            documentableDeclaration.setDocumentation(documentation);
        }
    }

    void setName(
        IDocumentableDeclaration documentableDeclaration,
        XMLAbstractIdAndNamed xmlAbstractIdAndNamed) {

        String documentation = null;

        if (xmlAbstractIdAndNamed instanceof XMLAbstractIdAndDocumentable) {
            XMLAbstractIdAndDocumentable xmlAbstractIdAndDocumentable =
                (XMLAbstractIdAndDocumentable) xmlAbstractIdAndNamed;

            documentation = deescapeDocumentationString(xmlAbstractIdAndDocumentable.getDocumentation());
        }
        setName(documentableDeclaration, xmlAbstractIdAndNamed.getName(), documentation);
        
        documentableDeclaration.setTODOFlag(xmlAbstractIdAndNamed.getTodoFlag());
    }

    void setParametricFunctionCallDeclaration(
        IParametricFunctionCallDeclaration parametricFunctionCallDeclaration,
        XMLUnnamedParametricFunction xmlUnnamedParametricFunction,
        IFunctionDeclarationListAble functionDeclarationList)
        throws XMLReadException {

        getXMLModelFunctionReader(functionDeclarationList).setParametricFunctionCallDeclaration(
            parametricFunctionCallDeclaration,
            xmlUnnamedParametricFunction);
    }

    private void setRealAnalysisSettings(
        XMLAnalysis xmlAnalysis,
        IAnalysisDeltaSettings analysisDeltaSettings)
        throws XMLReadException {

        analysisDeltaSettings.setDeltaTicks(xmlAnalysis.getDeltaTicks());
        analysisDeltaSettings.setStartTick(xmlAnalysis.hasStartTick() ? xmlAnalysis.getStartTick() : 0);
        analysisDeltaSettings.setUsingDecisionFunctionCallDeclaration(
            xmlAnalysis.hasUsingDecision() ? xmlAnalysis.getUsingDecision() : false);
        setRealFunctionCallDelay(
            analysisDeltaSettings.getDecisionFunctionCallDeclaration(),
            xmlAnalysis.getDecision());
    }

    void setRealFunctionCallDelay(
        IFunctionCallDeclaration functionCallDeclaration,
        IFunctionDeclarationListAble functionDeclarationList,
        XMLFunctionCall xmlFunctionCall)
        throws XMLReadException {

        if (xmlFunctionCall != null) {
            addDelaySettings(
                functionCallDeclaration,
                xmlFunctionCall,
                getXMLModelFunctionReader(functionDeclarationList));
        }
    }

    void setRealFunctionCallDelay(
        IFunctionCallDeclaration functionCallDeclaration,
        IFunctionDeclarationListAble functionDeclarationList,
        XMLFunctionCall xmlFunctionCall,
        ISpecificType mostGeneralType)
        throws XMLReadException {

        if (xmlFunctionCall != null) {
            addDelaySettings(
                functionCallDeclaration,
                xmlFunctionCall,
                mostGeneralType,
                getXMLModelFunctionReader(functionDeclarationList));
        }
    }

    void setRealFunctionCallDelay(
        IFunctionCallDeclaration functionCallDeclaration,
        IObjectClassDeclaration objectClassDeclaration,
        XMLFunctionCall xmlFunctionCall)
        throws XMLReadException {

        if (xmlFunctionCall != null) {
            getXMLModelFunctionReader(objectClassDeclaration).setRealFunctionCallDelay(
                functionCallDeclaration,
                xmlFunctionCall);
        }
    }

    void setRealFunctionCallDelay(
        IFunctionCallDeclaration functionCallDeclaration,
        XMLFunctionCall xmlFunctionCall)
        throws XMLReadException {

        if (xmlFunctionCall != null) {
            addDelaySettings(
                functionCallDeclaration,
                xmlFunctionCall,
                getXMLModelFunctionReader(_nullDeclarationFunctionDeclarationList));
        }
    }

    private void setRealSimulationDeclaration(
        XMLSimulation xmlSimulation,
        ISimulationDeclaration simulationDeclaration)
        throws XMLReadException {

        setName(simulationDeclaration, xmlSimulation);

        if (xmlSimulation.hasTest())
            simulationDeclaration.setTestSimulationDeclaration(xmlSimulation.getTest());

        AnalysisDeclarationList analysisDeclarationList = simulationDeclaration.getAnalysisDeclarationList();
        Assert.assertFalse(analysisDeclarationList.isComposition());
        analysisDeclarationList.clear();

        for (int i = 0; i < xmlSimulation.getAnalysisCount(); i++) {
            XMLAnalysisID xmlAnalysisID = xmlSimulation.getAnalysis(i);
            IAnalysisDeclaration analysisDeclaration =
                getRealAnalysisDeclaration((XMLAnalysis) xmlAnalysisID.getId());

            if (!analysisDeclarationList.contains(analysisDeclaration)) {
                analysisDeclarationList.add(analysisDeclaration);
            }
        }

        IDeclarationList endFunctionDeclList =
            simulationDeclaration.getEndFunctionDeclarationListAndFactory().getDeclarationList();
        IDeclarationFactory endDeclarationFactory =
            simulationDeclaration.getEndFunctionDeclarationListAndFactory().getDeclarationFactory();

        for (int i = 0; i < xmlSimulation.getEndfunctionCount(); i++) {
            XMLNamedFunctionCall xmlNamedFunctionCall = xmlSimulation.getEndfunction(i);
            endFunctionDeclList.add(
                createNamedFunctionCallDeclaration(endDeclarationFactory, xmlNamedFunctionCall));
        }

        for (int i = 0; i < xmlSimulation.getAfterRunTestCount(); i++) {
            XMLNamedFunctionCall xmlNamedFunctionCall = xmlSimulation.getAfterRunTest(i);
            simulationDeclaration.getAfterRunTestActionDefinitionList().add(
                createNamedFunctionCallDeclaration(
                    NamedFunctionCallDeclarationFactory.getInstance(),
                    xmlNamedFunctionCall));
        }

        for (int i = 0; i < xmlSimulation.getDuringRunTestCount(); i++) {
            XMLNamedFunctionCall xmlNamedFunctionCall = xmlSimulation.getDuringRunTest(i);
            simulationDeclaration.getDuringRunTestActionDefinitionList().add(
                createNamedFunctionCallDeclaration(
                    NamedFunctionCallDeclarationFactory.getInstance(),
                    xmlNamedFunctionCall));
        }
    }

    private NamedFunctionCallDeclaration createNamedFunctionCallDeclaration(
        IDeclarationFactory declarationFactory,
        XMLNamedFunctionCall xmlNamedFunctionCall)
        throws XMLReadException {

        NamedFunctionCallDeclaration namedFunctionCallDeclaration =
            (NamedFunctionCallDeclaration) declarationFactory.createDeclaration();
        setName(namedFunctionCallDeclaration, xmlNamedFunctionCall);
        setRealFunctionCallDelay(namedFunctionCallDeclaration, xmlNamedFunctionCall.getFunctionCall());

        return namedFunctionCallDeclaration;
    }

    private void setRealUserTypeDeclaration(
        XMLUserType xmlUserType,
        IUserTypeDeclaration userTypeDeclaration)
        throws XMLReadException {
        setName(userTypeDeclaration, xmlUserType);

        boolean composed = (Utils.equals(xmlUserType.getType(), "composed"));

        if (composed) {
            SpecificComposedType specificComposedType = userTypeDeclaration.getSpecificComposedType();

            if ((xmlUserType.getFather() != null)
                && ((xmlUserType.getFather().getId() != null)
                    || (xmlUserType.getFather().getSystemID() != null)))
                specificComposedType.setFather(
                    (ISpecificComposedType) getSpecificTypeByID(xmlUserType.getFather()));
            for (int i = 0; i < xmlUserType.getAttributeCount(); i++) {
                XMLComposedTypeAttribute xmlComposedTypeItem = xmlUserType.getAttribute(i);
                IComposedTypeAttribute composedTypeAttribute =
                    specificComposedType.createComposedTypeAttribute();

                composedTypeAttribute.setAttributeName(xmlComposedTypeItem.getName());
                composedTypeAttribute.setAttributeType(
                    getXMLModelTypeReader(_nullDeclarationFunctionDeclarationList).createRealType(
                        xmlComposedTypeItem.getXMLTypeGroup()));
                if (xmlComposedTypeItem.getDocumentation() != null)
                    composedTypeAttribute.setDocumentation(
                        deescapeDocumentationString(xmlComposedTypeItem.getDocumentation()));

                if (xmlComposedTypeItem.hasWriteable())
                    composedTypeAttribute.setWriteable(xmlComposedTypeItem.getWriteable());

                specificComposedType.addAttribute(composedTypeAttribute);
            }

        } else {
            SpecificEnumType specificEnumType = userTypeDeclaration.getSpecificEnumType();

            for (int i = 0; i < xmlUserType.getItemCount(); i++) {
                XMLEnumTypeItem xmlEnumTypeItem = xmlUserType.getItem(i);

                EnumItem enumItem = specificEnumType.createItem();

                enumItem.setName(xmlEnumTypeItem.getName());

                specificEnumType.getItemListAndFactory().getMutableListModel().add(enumItem);
            }
        }
    }

    private void setRealVariableClassDeclaration(
        IVariableClassDeclaration variableClassDeclaration,
        IFunctionDeclarationListAble functionDeclarationList,
        XMLVariableClass xmlVariable)
        throws XMLReadException {

        Assert.assertNotNull(xmlVariable);
        setName(variableClassDeclaration, xmlVariable);

        ISpecificType type = createRealType(xmlVariable.getXMLTypeGroup(), functionDeclarationList);

        if (Utils.equals(type.getGenericType(), GenericIteratorType.getInstance())) {

            // due to backward compatibility reasons
            variableClassDeclaration.setSpecificType(
                GenericListType.getInstance().createSpecificListType(
                    ((SpecificIteratorType) type).getIteratorSpecificType()));
        } else {
            variableClassDeclaration.setSpecificType(type);
        }

        if (xmlVariable.hasExternal()) {
            variableClassDeclaration.setExternal(xmlVariable.getExternal());
        }

        if (xmlVariable.hasWriteable()) {
            variableClassDeclaration.setWriteable(xmlVariable.getWriteable());
        }
        
        if (xmlVariable.hasPartOfName()) {
        	variableClassDeclaration.setPartOfName(xmlVariable.getPartOfName());
        }

        setRealFunctionCallDelay(
            variableClassDeclaration.getDefaultValueFunctionCallDeclaration(),
            functionDeclarationList,
            xmlVariable.getDefaultValue(),
            type);

        setRealFunctionCallDelay(
            variableClassDeclaration.getNextValueFunctionCallDeclaration(),
            functionDeclarationList,
            xmlVariable.getNextValue(),
            type);
        variableClassDeclaration.setHasNextValueFunctionCallDeclaration(xmlVariable.getHasNextValue());
    }
    
    public XMLModelObjectReader getXMLModelObjectReader() {
        return _xmlModelObjectReader;
    }
}