package de.uniwue.ki.mas.xml.reader;

import java.awt.Color;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import junit.framework.Assert;
import mygale.mcge.fig.*;
import de.uniwue.ki.mas.basics.action.FunctionActionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.ActivityReasoningEngineDeclarationFactory;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
  * Creation date: (07.02.2001 09:54:13)
  * @author Christoph Oechslein
  */
class XMLModelReasoningActivityReader {
	private final IObjectClassDeclaration _objectClassDeclaration;
	private final transient XMLModelReader _xmlModelReader;

	public XMLModelReasoningActivityReader(
		IObjectClassDeclaration objectClassDeclaration,
		XMLModelReader xmlModelReader) {
		super();
		_objectClassDeclaration = objectClassDeclaration;
		_xmlModelReader = xmlModelReader;
	}

	private void addFunctionsToActionDeclarationList(
		IDeclarationList realActionDeclarationList,
		XMLActionCall[] xmlActionCalls)
		throws XMLReadException {

		if (xmlActionCalls != null) {

			for (int i = 0; i < xmlActionCalls.length; i++) {
				realActionDeclarationList.add(
					createRealFunctionActionCall(_objectClassDeclaration, xmlActionCalls[i]));
			}
		}
	}

	private IActionActivityGraphLeafDeclaration createRealActionAtomicActivity(
		XMLActionActivityGraphLeaf xmlActionActivityGraphLeaf,
		IActivityGraphDeclaration parentActivityGraphDeclaration)
		throws XMLReadException {

		IActionActivityGraphLeafDeclaration realActionAtomicActivity =
			ActionActivityGraphLeafDeclarationFactory.createActionActivityGraphLeafDeclaration(
				parentActivityGraphDeclaration);
		setRealActionAtomicActivity(realActionAtomicActivity, xmlActionActivityGraphLeaf);

		return realActionAtomicActivity;
	}

	private IActivityGraphRuleDeclaration createRealActivityRuleDeclaration(XMLAbstractRule XMLAbstractRule)
		throws XMLReadException {

		IActivityGraphRuleDeclaration activityArgumentRuleDeclaration =
			ActivityGraphRuleDeclarationFactory.createActivityGraphRuleDeclaration();
		IActivityGraphNodeDeclaration toActivityDeclaration =
			getRealActivity((XMLAbstractActivityGraphNode)XMLAbstractRule.getToActivityID());
		activityArgumentRuleDeclaration.setToActivityGraphNodeDeclaration(toActivityDeclaration);
		setRealActivityRuleDeclaration(activityArgumentRuleDeclaration, XMLAbstractRule);

		return activityArgumentRuleDeclaration;
	}

	IActivityReasoningEngineDeclaration createRealActivityReasoningDeclaration(XMLReasoning xmlReasoning)
		throws XMLReadException {

		XMLActivityGraph xmlActivityGraph = xmlReasoning.getActivityGraph();
		if (xmlActivityGraph == null) {
			xmlActivityGraph = xmlReasoning.getComposedActivity();
		} else {
			Assert.assertNull(xmlReasoning.getComposedActivity());
		}

		IActivityReasoningEngineDeclaration activityReasoningEngineDeclaration =
			ActivityReasoningEngineDeclarationFactory.createActivityReasoningEngineDeclaration(
				createRealActivityGraphDeclaration(xmlActivityGraph));

		return activityReasoningEngineDeclaration;
	}

	private IActivityGraphDeclaration createRealActivityGraphDeclaration(XMLActivityGraph XMLActivityGraph)
		throws XMLReadException {

		IActivityGraphDeclaration realActivityGraphDeclaration =
			ActivityGraphDeclarationFactory.createEmptyActivityGraphDeclaration();

		setRealActivityGraphDeclaration(realActivityGraphDeclaration, XMLActivityGraph);

		return realActivityGraphDeclaration;
	}

	private IActivityGraphComposedNodeDeclaration createRealActivityGraphComposedNodeDeclaration(XMLActivityGraph xmlActivityGraph)
		throws XMLReadException {

		IActivityGraphComposedNodeDeclaration realActivityGraphComposedNodeDeclaration =
			ActivityGraphComposedNodeDeclarationFactory.createActivityGraphComposedNodeDeclaration();

		setRealActivityGraphDeclaration(
			realActivityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration(),
			xmlActivityGraph);
		setRealActivity(realActivityGraphComposedNodeDeclaration, xmlActivityGraph);
		setRealActivityDeclarationTypeable(
			realActivityGraphComposedNodeDeclaration,
			xmlActivityGraph.getType());

		return realActivityGraphComposedNodeDeclaration;
	}

	private void setRealActivityGraphDeclaration(
		IActivityGraphDeclaration realActivityGraphDeclaration,
		XMLActivityGraph xmlActivityGraph)
		throws XMLReadException {

		_xmlModelReader.setName(realActivityGraphDeclaration, xmlActivityGraph);
		setEntryAndExitActivity(xmlActivityGraph, realActivityGraphDeclaration);

		setEmergencyActivity(
			xmlActivityGraph.getEmergencyActivity(),
			realActivityGraphDeclaration.getEmergencyActivityGraphLeafDeclaration());

		for (int i = 0; i < xmlActivityGraph.getXMLActivityGraphNodeGroupCount(); i++) {
			XMLActivityGraphNodeGroup XMLActivityGraphNodeGroup =
				xmlActivityGraph.getXMLActivityGraphNodeGroup(i);
			IActivityGraphNodeDeclaration realActivity =
				getRealActivity(XMLActivityGraphNodeGroup, realActivityGraphDeclaration);

			realActivityGraphDeclaration.addActivityGraphNodeDeclaration(realActivity);
		}

		processChainRules(realActivityGraphDeclaration, xmlActivityGraph);
		processOtherwiseChainRules(realActivityGraphDeclaration, xmlActivityGraph);
		processEmergencyRules(realActivityGraphDeclaration, xmlActivityGraph);
		processDocumentEdges(realActivityGraphDeclaration, xmlActivityGraph);
	}

	private void setEmergencyActivity(
		XMLActivityGraphLeaf xmlEmergencyActivity,
		IEmergencyActivityGraphLeafDeclaration realEmergencyDeclaration) {

		if (xmlEmergencyActivity != null) {
			setMappingOfRealActivity(xmlEmergencyActivity, realEmergencyDeclaration);

			setRealActivity(realEmergencyDeclaration, xmlEmergencyActivity);
		}
	}

	private IActivityGraphComposedNodeDeclaration createRealActivityGraphComposedNodeDeclaration(
		XMLActivityGraph xmlActivityGraph,
		IActivityGraphDeclaration parent)
		throws XMLReadException {

		IActivityGraphComposedNodeDeclaration realActivityGraphDeclaration =
			createRealActivityGraphComposedNodeDeclaration(xmlActivityGraph);
		realActivityGraphDeclaration.setActivityGraphDeclaration(parent);

		return realActivityGraphDeclaration;
	}

	private IDecisionGraphLeafDeclaration createRealDecisionAtomicActivity(
		XMLDecisionGraphLeaf xmlDecisionAtomicActivity,
		IActivityGraphDeclaration parentActivityGraphDeclaration)
		throws XMLReadException {

		IDecisionGraphLeafDeclaration realDecisionAtomicActivity =
			DecisionGraphLeafDeclarationFactory.createDecisionActivityGraphLeafDeclaration(
				parentActivityGraphDeclaration);
		setRealDecisionAtomicActivity(realDecisionAtomicActivity, xmlDecisionAtomicActivity);

		return realDecisionAtomicActivity;
	}

	private GraphDocumentationLeafDeclaration createRealDocumentationNode(
		XMLDocumentationGraphLeaf xmlDocumentation,
		IActivityGraphDeclaration parentActivityGraphDeclaration)
		throws XMLReadException {

		GraphDocumentationLeafDeclaration realDocumentation =
			GraphDocumentationLeafDeclarationFactory.createGraphDocumentationLeafDeclaration(
				parentActivityGraphDeclaration);
		setRealDocumentationNode(realDocumentation, xmlDocumentation);

		return realDocumentation;
	}

	private IActivityGraphRuleDeclaration createRealEmergencyRule(
		XMLEmergencyActivityGraphRule XMLEmergencyActivityGraphRule,
		IEmergencyActivityGraphLeafDeclaration emergencyAtomicActivityDeclaration)
		throws XMLReadException {

		IActivityGraphRuleDeclaration emergencyRule =
			createRealActivityRuleDeclaration(XMLEmergencyActivityGraphRule);
		emergencyRule.setFromActivityGraphNodeDeclaration(emergencyAtomicActivityDeclaration);

		return emergencyRule;
	}

	IFunctionCallDeclaration createRealFunctionActionCall(
		IObjectClassDeclaration objectClassDeclaration,
		XMLActionCall xmlRule)
		throws XMLReadException {

		if (xmlRule.getAction() != null) {
			return _xmlModelReader.createRealFunctionCall(
				xmlRule.getAction(),
				_xmlModelReader.getFunctionDeclarationListFor(objectClassDeclaration),
				FunctionActionCallDeclarationFactory.getInstance());

		} else {

			return FunctionActionCallDeclarationFactory.getInstance().createFunctionActionCallDeclaration();
		}
	}

	IActivityGraphNodeDeclaration getRealActivity(XMLAbstractActivityGraphNode xmlActivity)
		throws XMLReadException {

		return (IActivityGraphNodeDeclaration)_xmlModelReader.getDeclaration(xmlActivity);
	}

	IActivityGraphNodeDeclaration getRealActivity(
		XMLAbstractActivityGraphNode xmlActivity,
		IActivityGraphDeclaration parentActivityGraphDeclaration)
		throws XMLReadException {

		if (!_xmlModelReader.containsXMLObject(xmlActivity)) {

			if (xmlActivity instanceof XMLActivityGraph) {
				setMappingOfRealActivity(
					xmlActivity,
					createRealActivityGraphComposedNodeDeclaration(
						(XMLActivityGraph)xmlActivity,
						parentActivityGraphDeclaration));
			} else if (xmlActivity instanceof XMLDecisionGraphLeaf) {
				setMappingOfRealActivity(
					xmlActivity,
					createRealDecisionAtomicActivity(
						(XMLDecisionGraphLeaf)xmlActivity,
						parentActivityGraphDeclaration));
			} else if (xmlActivity instanceof XMLDocumentationGraphLeaf) {
				setMappingOfRealActivity(
					xmlActivity,
					createRealDocumentationNode(
						(XMLDocumentationGraphLeaf)xmlActivity,
						parentActivityGraphDeclaration));
			} else if (xmlActivity instanceof XMLActionActivityGraphLeaf) {
				//
				setMappingOfRealActivity(
					xmlActivity,
					createRealActionAtomicActivity(
						(XMLActionActivityGraphLeaf)xmlActivity,
						parentActivityGraphDeclaration));
			} else {
				Assert.fail();
			}
		}

		return getRealActivity(xmlActivity);
	}

	IActivityGraphNodeDeclaration getRealActivity(
		XMLActivityGraphNodeGroup XMLActivityGraphNodeGroup,
		IActivityGraphDeclaration parentActivityGraphDeclaration)
		throws XMLReadException {

		if (XMLActivityGraphNodeGroup.getActionActivity() != null) {
			return getRealActivity(
				XMLActivityGraphNodeGroup.getActionActivity(),
				parentActivityGraphDeclaration);

		} else if (XMLActivityGraphNodeGroup.getComposedActivity() != null) {
			return getRealActivity(
				XMLActivityGraphNodeGroup.getComposedActivity(),
				parentActivityGraphDeclaration);

		} else if (XMLActivityGraphNodeGroup.getDecisionActivity() != null) {
			return getRealActivity(
				XMLActivityGraphNodeGroup.getDecisionActivity(),
				parentActivityGraphDeclaration);

		} else if (XMLActivityGraphNodeGroup.getDocumentationNode() != null) {
			return getRealActivity(
				XMLActivityGraphNodeGroup.getDocumentationNode(),
				parentActivityGraphDeclaration);

		} else {
			Assert.fail();

			return null;
		}
	}

	private void processChainRules(
		IActivityGraphDeclaration realActivityGraphDeclaration,
		XMLActivityGraph XMLActivityGraph)
		throws XMLReadException {

		XMLChainRule[] xmlChainRules = XMLActivityGraph.getChainRule();

		for (int i = 0; i < xmlChainRules.length; i++) {

			IActivityGraphRuleDeclaration realChainRule = createRealActivityRuleDeclaration(xmlChainRules[i]);
			realChainRule.setFromActivityGraphNodeDeclaration(
				getRealActivity((XMLAbstractActivityGraphNode)xmlChainRules[i].getFromActivityID()));

			realActivityGraphDeclaration.addActivityGraphEdgeDeclaration(realChainRule);
		}
	}

	private void processEmergencyRules(
		IActivityGraphDeclaration realActivityGraphDeclaration,
		XMLActivityGraph xmlActivityGraph)
		throws XMLReadException {

		IEmergencyActivityGraphLeafDeclaration emergencyActivityDeclaration =
			realActivityGraphDeclaration.getEmergencyActivityGraphLeafDeclaration();
		XMLEmergencyActivityGraphRule[] xmlEmergencyActivityGraphRules = xmlActivityGraph.getEmergencyRule();

		for (int j = 0; j < xmlEmergencyActivityGraphRules.length; j++) {
			realActivityGraphDeclaration.addEmergencyRuleDeclaration(
				createRealEmergencyRule(xmlEmergencyActivityGraphRules[j], emergencyActivityDeclaration));
		}
	}

	private void setEntryAndExitActivity(
		XMLActivityGraph XMLActivityGraph,
		IActivityGraphDeclaration realActivityGraphDeclaration)
		throws XMLReadException {

		XMLAbstractActivityGraphLeaf entryXMLActivityGraphLeaf = XMLActivityGraph.getEntryActivity();
		XMLAbstractActivityGraphLeaf exitXMLActivityGraphLeaf = XMLActivityGraph.getExitActivity();

		IEntryActivityGraphLeafDeclaration entryActivity =
			realActivityGraphDeclaration.getEntryActivityGraphLeafDeclaration();
		IExitActivityGraphLeafDeclaration exitActivity =
			realActivityGraphDeclaration.getExitActivityGraphLeafDeclaration();

		setMappingOfRealActivity(entryXMLActivityGraphLeaf, entryActivity);
		setMappingOfRealActivity(exitXMLActivityGraphLeaf, exitActivity);

		setRealAtomicActivity(entryActivity, entryXMLActivityGraphLeaf);
		setRealAtomicActivity(exitActivity, exitXMLActivityGraphLeaf);
	}

	private void setMappingOfRealActivity(
		XMLAbstractActivityGraphNode xmlActivity,
		IActivityGraphNodeDeclaration realActivityDeclaration) {

		_xmlModelReader.setDeclaration(xmlActivity, realActivityDeclaration);
	}

	private void setRealActionAtomicActivity(
		IActionActivityGraphLeafDeclaration realAtomicActivity,
		XMLActionActivityGraphLeaf xmlActionActivityGraphLeaf)
		throws XMLReadException {

		setRealAtomicActivity(realAtomicActivity, xmlActionActivityGraphLeaf);

		if (xmlActionActivityGraphLeaf.hasStateLike() && xmlActionActivityGraphLeaf.getStateLike()) {
			realAtomicActivity.setType(IActivityGraphNodeDeclarationTypeable.STATE_LIKE);
		} else {
			setRealActivityDeclarationTypeable(realAtomicActivity, xmlActionActivityGraphLeaf.getType());
		}

		realAtomicActivity.setInstantly(xmlActionActivityGraphLeaf.getInstantly());

		if (xmlActionActivityGraphLeaf.getEntryActionRuleCount() != 0) {
			addFunctionsToActionDeclarationList(
				realAtomicActivity
					.getEntryFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList(),
				xmlActionActivityGraphLeaf.getEntryActionRule());
		}

		if (xmlActionActivityGraphLeaf.getActionRuleCount() != 0) {
			addFunctionsToActionDeclarationList(
				realAtomicActivity
					.getFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList(),
				xmlActionActivityGraphLeaf.getActionRule());
		}

		if (xmlActionActivityGraphLeaf.getExitActionRuleCount() != 0) {
			addFunctionsToActionDeclarationList(
				realAtomicActivity
					.getExitFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList(),
				xmlActionActivityGraphLeaf.getExitActionRule());
		}
	}

	private void setRealActivity(
		IActivityGraphNodeDeclaration realActivity,
		XMLAbstractActivityGraphNode xmlActivity) {

		_xmlModelReader.setName((IDocumentableDeclaration)realActivity, xmlActivity);

		if (_xmlModelReader.getFigMapper() != null) {
			setRealActivityPosition(realActivity, xmlActivity);

			IFigObject figObject = _xmlModelReader.getFigMapper().getFigFor(realActivity);
			if (figObject instanceof AbstractFigNode) {

				AbstractFigNode absNode = (AbstractFigNode)figObject;

				XMLColor color = xmlActivity.getBackgroundColor();
				if (color != null) {
					absNode.setBackgroundColor(new Color(color.getRed(), color.getGreen(), color.getBlue()));
				}
			}
		}
	}

	private void setRealActivityPosition(
		IActivityGraphNodeDeclaration realActivity,
		XMLAbstractActivityGraphNode xmlActivity) {

		if (xmlActivity.hasX() || xmlActivity.hasY() || xmlActivity.hasWidth() || xmlActivity.hasHeight()) {

			try {

				Method figActivityDeclarationFactoryMethod =
					getClass()
						.getClassLoader()
						.loadClass("de.uniwue.ki.gui.mas.behaviour.activityGraph.fig.FigActivityGraphNodeDeclarationFactory")
						.getMethod("getInstance", null);
				//				Method figActivityPortDeclarationFactoryMethod =
				//					getClass()
				//						.getClassLoader()
				//						.loadClass("de.uniwue.ki.gui.mas.behaviour.activityGraph.fig.FigActivityGraphNodePortDeclarationFactory")
				//						.getMethod("getInstance", null);

				IFigNode figNode =
					FigHelper.createFigNode(
						_xmlModelReader.getFigMapper(),
						realActivity,
						(IFigNodeFactory)figActivityDeclarationFactoryMethod.invoke(null, null));

				if (Assert.ASSERTION_ON) {
					Assert.assertEquals(figNode.getNetObject(), realActivity);
				}

				//TODO set (min-x,min-y) to (0,0)

				figNode.setBounds(
					xmlActivity.getX(),
					xmlActivity.getY(),
					xmlActivity.getWidth(),
					xmlActivity.getHeight());

			} catch (ClassNotFoundException exception) {
				throw new RuntimeException(exception);
			} catch (NoSuchMethodException exception) {
				throw new RuntimeException(exception);
			} catch (IllegalAccessException exception) {
				throw new RuntimeException(exception);
			} catch (InvocationTargetException exception) {
				throw new RuntimeException(exception.getTargetException());
			}
		}
	}

	private void setRealActivityRuleDeclaration(
		IActivityGraphRuleDeclaration activityRuleDeclaration,
		XMLAbstractRule xmlAbstractRule)
		throws XMLReadException {

		_xmlModelReader.setName(
			activityRuleDeclaration,
			xmlAbstractRule.getName(),
			XMLModelReader.deescapeDocumentationString(xmlAbstractRule.getDocumentation()));

		if ((xmlAbstractRule.getIsStochastic()) && (xmlAbstractRule.getStochastic() != null)) {
			XMLFunctionCall newXMLFunctionCall = new XMLFunctionCall();

			XMLComposedFunctionCallIntern randomXMLCallIntern = new XMLComposedFunctionCallIntern();
			randomXMLCallIntern.setFunctionName("RandomBoolean");
			randomXMLCallIntern.addXMLFunctionCallInternGroup(
				xmlAbstractRule.getStochastic().getXMLFunctionCallInternGroup());
			XMLFunctionCallInternGroup xmlRandomFunctionCallInternGroup = new XMLFunctionCallInternGroup();
			xmlRandomFunctionCallInternGroup.setCall(randomXMLCallIntern);

			if (xmlAbstractRule.getPrecondition() == null) {
				// condition == true
				newXMLFunctionCall.setXMLFunctionCallInternGroup(xmlRandomFunctionCallInternGroup);

			} else {
				XMLFunctionCallInternGroup xmlPrecondFunCallInternGroup =
					xmlAbstractRule.getPrecondition().getXMLFunctionCallInternGroup();

				if ((xmlPrecondFunCallInternGroup.getXMLTypeValueGroup() == null)
					|| (xmlPrecondFunCallInternGroup.getXMLTypeValueGroup().getBoolean().getValue() == false)) {

					XMLFunctionCallInternGroup xmlFunctionCallInternGroup = new XMLFunctionCallInternGroup();
					newXMLFunctionCall.setXMLFunctionCallInternGroup(xmlFunctionCallInternGroup);
					XMLComposedFunctionCallIntern andXMLCallIntern = new XMLComposedFunctionCallIntern();
					xmlFunctionCallInternGroup.setCall(andXMLCallIntern);
					andXMLCallIntern.setFunctionName("AND");

					andXMLCallIntern.addXMLFunctionCallInternGroup(xmlRandomFunctionCallInternGroup);

					andXMLCallIntern.addXMLFunctionCallInternGroup(xmlPrecondFunCallInternGroup);

				} else {
					// condition == true
					newXMLFunctionCall.setXMLFunctionCallInternGroup(xmlRandomFunctionCallInternGroup);
				}
			}

			_xmlModelReader.setRealFunctionCallDelay(
				activityRuleDeclaration.getConditionFunctionCallDeclaration(),
				_objectClassDeclaration,
				newXMLFunctionCall);

		} else {
			if (xmlAbstractRule.getPrecondition() != null) {
				_xmlModelReader.setRealFunctionCallDelay(
					activityRuleDeclaration.getConditionFunctionCallDeclaration(),
					_objectClassDeclaration,
					xmlAbstractRule.getPrecondition());
			}
		}

		setRealActivityGraphEdgeDeclarationControlPoints(activityRuleDeclaration, xmlAbstractRule);
	}

	private void setRealActivityGraphEdgeDeclarationControlPoints(
		IActivityGraphEdgeDeclaration activityGraphEdgeDeclaration,
		XMLAbstractActivityGraphEdge xmlAbstractActivityGraphEdge) {

		if (_xmlModelReader.getFigMapper() != null) {
			if (xmlAbstractActivityGraphEdge.getControlPointCount() > 0) {
				try {
					Method FigActivityRuleDeclarationFactoryMethod =
						getClass()
							.getClassLoader()
							.loadClass("de.uniwue.ki.gui.mas.behaviour.activityGraph.fig.FigActivityGraphRuleDeclarationFactory")
							.getMethod("getInstance", null);
					IFigEdge figEdge =
						FigHelper.createFigEdge(
							_xmlModelReader.getFigMapper(),
							activityGraphEdgeDeclaration,
							(IFigEdgeFactory)FigActivityRuleDeclarationFactoryMethod.invoke(null, null));

					Assert.assertEquals(figEdge.getNetObject(), activityGraphEdgeDeclaration);
					for (int i = xmlAbstractActivityGraphEdge.getControlPointCount() - 1; i >= 0; i--) {

						XMLPosInteger xmlPos = xmlAbstractActivityGraphEdge.getControlPoint(i);
						figEdge.insertCtrlPointAt(1, new java.awt.Point(xmlPos.getX(), xmlPos.getY()));
					}

					figEdge.updateEndsLocation();

				} catch (ClassNotFoundException exception) {
					throw new RuntimeException(exception);
				} catch (NoSuchMethodException exception) {
					throw new RuntimeException(exception);
				} catch (IllegalAccessException exception) {
					throw new RuntimeException(exception);
				} catch (InvocationTargetException exception) {
					throw new RuntimeException(exception.getTargetException());
				}
			}
		}
	}

	private void setRealAtomicActivity(
		IActivityGraphLeafDeclaration realAtomicActivity,
		XMLAbstractActivityGraphLeaf XMLActivityGraphLeaf)
		throws XMLReadException {

		setRealActivity(realAtomicActivity, XMLActivityGraphLeaf);
	}

	private void setRealDecisionAtomicActivity(
		IDecisionGraphLeafDeclaration realAtomicActivity,
		XMLDecisionGraphLeaf xmlDecisionAtomicActivity)
		throws XMLReadException {
		setRealAtomicActivity(realAtomicActivity, xmlDecisionAtomicActivity);
	}

	private void setRealDocumentationNode(
		GraphDocumentationLeafDeclaration realAtomicActivity,
		XMLDocumentationGraphLeaf xmlDecisionAtomicActivity)
		throws XMLReadException {

		setRealAtomicActivity(realAtomicActivity, xmlDecisionAtomicActivity);
	}

	private IOtherwiseActivityGraphEdgeDeclaration createRealOtherwiseActivityEdgeDeclaration(XMLOtherwiseActivityGraphEdge xmlOtherwiseActivityGraphEdge)
		throws XMLReadException {

		IOtherwiseActivityGraphEdgeDeclaration otherwiseActivityEdgeDeclaration =
			(IOtherwiseActivityGraphEdgeDeclaration)OtherwiseActivityEdgeDeclarationFactory
				.getInstance()
				.createActivityGraphEdgeDeclaration();

		IActivityGraphNodeDeclaration toActivityDeclaration =
			getRealActivity((XMLAbstractActivityGraphNode)xmlOtherwiseActivityGraphEdge.getToActivityID());

		IActivityGraphNodeDeclaration fromActivityDeclaration =
			getRealActivity(
				(XMLAbstractActivityGraphNode)xmlOtherwiseActivityGraphEdge.getFromDecisionActivityID());

		otherwiseActivityEdgeDeclaration.setToActivityGraphNodeDeclaration(toActivityDeclaration);
		otherwiseActivityEdgeDeclaration.setFromActivityGraphNodeDeclaration(fromActivityDeclaration);

		setRealActivityGraphEdgeDeclarationControlPoints(
			otherwiseActivityEdgeDeclaration,
			xmlOtherwiseActivityGraphEdge);

		return otherwiseActivityEdgeDeclaration;
	}

	private void processOtherwiseChainRules(
		IActivityGraphDeclaration realActivityGraphDeclaration,
		XMLActivityGraph XMLActivityGraph)
		throws XMLReadException {

		XMLOtherwiseActivityGraphEdge[] xmlOtherwiseActivityGraphEdges =
			XMLActivityGraph.getOtherwiseChainRule();

		for (int i = 0; i < xmlOtherwiseActivityGraphEdges.length; i++) {
			realActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
				createRealOtherwiseActivityEdgeDeclaration(xmlOtherwiseActivityGraphEdges[i]));
		}
	}

	private void processDocumentEdges(
		IActivityGraphDeclaration realActivityGraphDeclaration,
		XMLActivityGraph XMLActivityGraph)
		throws XMLReadException {

		XMLDocumentActivityGraphEdge[] xmlDocumentActivityGraphEdge = XMLActivityGraph.getDocumentEdge();

		for (int i = 0; i < xmlDocumentActivityGraphEdge.length; i++) {
			realActivityGraphDeclaration.addActivityGraphEdgeDeclaration(
				createRealGraphDocumentEdgeDeclaration(xmlDocumentActivityGraphEdge[i]));
		}
	}

	private IGraphDocumentEdgeDeclaration createRealGraphDocumentEdgeDeclaration(XMLDocumentActivityGraphEdge xmlDocumentActivityGraphEdge)
		throws XMLReadException {

		IGraphDocumentEdgeDeclaration documentEdgeDeclaration =
			(IGraphDocumentEdgeDeclaration)DocumentEdgeDeclerationFactory
				.getInstance()
				.createActivityGraphEdgeDeclaration();

		IActivityGraphNodeDeclaration toActivityDeclaration =
			getRealActivity((XMLAbstractActivityGraphNode)xmlDocumentActivityGraphEdge.getToActivityID());

		IActivityGraphNodeDeclaration fromActivityDeclaration =
			getRealActivity((XMLAbstractActivityGraphNode)xmlDocumentActivityGraphEdge.getFromActivityID());

		documentEdgeDeclaration.setToActivityGraphNodeDeclaration(toActivityDeclaration);
		documentEdgeDeclaration.setFromActivityGraphNodeDeclaration(fromActivityDeclaration);

		documentEdgeDeclaration.setName(xmlDocumentActivityGraphEdge.getName());

		setRealActivityGraphEdgeDeclarationControlPoints(
			documentEdgeDeclaration,
			xmlDocumentActivityGraphEdge);

		return documentEdgeDeclaration;
	}

	private void setRealActivityDeclarationTypeable(
		IActivityGraphNodeDeclarationTypeable activityDeclarationTypeable,
		String type)
		throws XMLReadException {

		if (type == null) {
			activityDeclarationTypeable.setType(IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE);
		} else if (type.equalsIgnoreCase(IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE.getName())) {
			activityDeclarationTypeable.setType(IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE);
		} else if (type.equalsIgnoreCase(IActivityGraphNodeDeclarationTypeable.STATE_LIKE.getName())) {
			activityDeclarationTypeable.setType(IActivityGraphNodeDeclarationTypeable.STATE_LIKE);
		} else if (type.equalsIgnoreCase(IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE.getName())) {
			activityDeclarationTypeable.setType(IActivityGraphNodeDeclarationTypeable.SIGNAL_IN_LIKE);
		} else if (type.equalsIgnoreCase(IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE.getName())) {
			activityDeclarationTypeable.setType(IActivityGraphNodeDeclarationTypeable.SIGNAL_OUT_LIKE);
		} else {
			Assert.fail();
		}
	}
}