package de.uniwue.ki.mas.xml.reader.oldfunctions;

import java.util.Iterator;

import de.uniwue.ki.mas.basics.function.ICreateFunctionArgs;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.ISpecificType;
import de.uniwue.ki.mas.basics.type.composed.GenericComposedType;
import de.uniwue.ki.mas.basics.type.composed.function.FunctionDeclarationComposedTypeFactory;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * userType CreateUserTypeValue(T1,...,Tn)
 * or userType CreateEmptyUserTypeValue()
 * Creation date: 13.06.2002, 18:44:26
 * @author oechslein
 */
final class CreateComposedTypeValueGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	private final boolean _isEmptyVersion;

	public CreateComposedTypeValueGenericFunctionDeclaration(boolean isEmptyVersion) {

		super(
			isEmptyVersion ? "CreateEmptyUserTypeValue" : "CreateUserTypeValue",
			GenericComposedType.getInstance());

		_isEmptyVersion = isEmptyVersion;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		throw new UnsupportedOperationException("Must call #createFunctionDeclaration(ISpecificType, Iterator) instead!");
	}

	private AbstractGenericFunctionDeclarationWithUnspecificDependents getRightGenericFunction() {
		if (isEmptyVersion())
			return FunctionDeclarationComposedTypeFactory.CREATE_EMPTY_COMPOSED_TYPE_VALUE_GENERIC_FUN_DECL;
		else
			return FunctionDeclarationComposedTypeFactory.CREATE_COMPOSED_TYPE_VALUE_GENERIC_FUN_DECL;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		Iterator inputTypes) {

		return getRightGenericFunction().createFunctionDeclaration(outputType, inputTypes);
	}

	public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
		return false;
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType outputType) {
		return getRightGenericFunction().createFunctionDeclaration(outputType, CollectionsHelper.EMPTY_ITERATOR);
	}

	protected boolean needSpecificOutputTypeToCreateSpecificFunctionDeclaration() {
		return true;
	}

	final boolean isEmptyVersion() {
		return _isEmptyVersion;
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {

		return composedFunctionCallInternDeclaration;
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return null;
	}
}
