package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * <T> PopAndReturnVar(variable<set<T>>) throws SetIsEmpty
  */

final class PopAndReturnVariableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	public PopAndReturnVariableGenericFunctionDeclaration() {

		super("Pop&ReturnVar", GenericType.getInstance(), false);

		setExpertFunctionDeclaration(true);
		setDocumentation("Returns the last-in element of its Set");
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		// PopAndReturnVar(variable<set<T>>) throws SetIsEmpty

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return typeT;
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(0, index);

				return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
					.getInstance(false, true)
					.createSpecificVariableType(
						GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
					
				Assert.fail();
				return null;
			}

			public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
				return false;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// PopAndReturnVar(variable<set<T>>) throws SetIsEmpty

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();

		return createFunctionDeclaration(specificSetType.getIteratorSpecificType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<List<" + outputType.getName() + ">>)";
	}

		public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {
			
	ISpecificFunctionDeclaration popVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);
		ISpecificType valueType = popVarSpecificFunctionDeclaration.getSpecificOutputType();

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(
					composedFunctionCallInternDeclaration.getChild(variableIndex)));

		SpecificListType listType = GenericListType.getInstance().createSpecificListType(
						((SpecificIteratorType) variableType.getVariableClassDeclarationType())
							.getIteratorSpecificType());
							
		ISpecificFunctionDeclaration popSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.POP_RETURN_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				valueType,
				CollectionsHelper.createIterator(
					listType));

		Assert.assertNotNull(popSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			popSpecFunDecl,
			CollectionsHelper.createList(
				getVariableCall));
	}
}