package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * <T> PopAndReturnVar(variable<set<T>>, body) throws SetIsEmpty
  */

final class PopAndReturnVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	public PopAndReturnVariableOfGenericFunctionDeclaration() {
		super("Pop&ReturnVarOf", GenericType.getInstance(), false);

		setExpertFunctionDeclaration(true);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return typeT;
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0)
					return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
						.getInstance(true, true)
						.createSpecificVariableType(
							GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
				else
					return GenericSimObjectType.getGeneralSimObjectType();
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {
					
				Assert.fail();
				return null;
			}

			public boolean isConstant(ICreateFunctionArgs createFunctionArgs) {
				return false;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!specificTypeB.equals(GenericSimObjectType.getGeneralSimObjectType()))
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();

		return createFunctionDeclaration(specificSetType.getIteratorSpecificType());
	}

	/**
	 * @param outputType ISpecificType
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
	 */
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<Set<" + outputType.getName() + ">>, Body)";
	}

		public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {
			
	ISpecificFunctionDeclaration popVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;
		int bodyIndex = 1;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);
		ISpecificType valueType = popVarSpecificFunctionDeclaration.getSpecificOutputType();
		ISpecificType bodyType = popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(bodyIndex);

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType, bodyType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(
					composedFunctionCallInternDeclaration.getChild(variableIndex),
					composedFunctionCallInternDeclaration.getChild(bodyIndex)));

		SpecificListType listType = GenericListType.getInstance().createSpecificListType(
						((SpecificIteratorType) variableType.getVariableClassDeclarationType())
							.getIteratorSpecificType());
							
		ISpecificFunctionDeclaration popSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.POP_RETURN_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				valueType,
				CollectionsHelper.createIterator(
					listType));

		Assert.assertNotNull(popSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			popSpecFunDecl,
			CollectionsHelper.createList(
				getVariableCall));
	}
}