package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * PopVar(variable<set<T>>) throws SetIsEmpty
  */

final class PopVariableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("Set Type");

	public PopVariableGenericFunctionDeclaration() {
		super("PopVariable", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Pops the last-in element of its Set (without returning)");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		// PopVar(variable<set<T>>) throws SetIsEmpty

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 1) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (Assert.ASSERTION_ON)
					Assert.assertEquals(0, index);

				return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
					.getInstance(false, true)
					.createSpecificVariableType(
						GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {

				Assert.fail();
				return null;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// PopVar(variable<set<T>>) throws SetIsEmpty

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// PopVar(variable<set<T>>) throws SetIsEmpty

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();

		return createFunctionDeclaration(specificSetType.getIteratorSpecificType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<List<T>>)";
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {
			
		ISpecificFunctionDeclaration popVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(composedFunctionCallInternDeclaration.getChild(variableIndex)));

		SpecificListType listType =
			GenericListType.getInstance().createSpecificListType(
				((SpecificIteratorType) variableType.getVariableClassDeclarationType())
					.getIteratorSpecificType());

		ISpecificFunctionDeclaration popSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.POP_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(listType));

		Assert.assertNotNull(popSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			popSpecFunDecl,
			CollectionsHelper.createList(getVariableCall));
	}
}