package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * PopVarOf(variable<set<T>>, body) throws SetIsEmpty
  */

final class PopVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("Set Type");

	public PopVariableOfGenericFunctionDeclaration() {
		super("PopVariableOf", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Pops the last-in element of the Set of body (Arg2) (without returning)");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}

			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0)
					return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
						.getInstance(true, true)
						.createSpecificVariableType(
							GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
				else
					return GenericSimObjectType.getGeneralSimObjectType();
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {

				Assert.fail();
				return null;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	/**
	 * @param inputTypes Iterator
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType specificTypeB = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (specificTypeB == null) || inputTypes.hasNext())
			return null;

		if (!specificTypeB.equals(GenericSimObjectType.getGeneralSimObjectType()))
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();

		return createFunctionDeclaration(specificSetType.getIteratorSpecificType());
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<Set<T>>, Body)";
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {
			
		ISpecificFunctionDeclaration popVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;
		int bodyIndex = 1;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);
		ISpecificType bodyType = popVarSpecificFunctionDeclaration.getSpecificInputTypeAt(bodyIndex);

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType, bodyType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(
					composedFunctionCallInternDeclaration.getChild(variableIndex),
					composedFunctionCallInternDeclaration.getChild(bodyIndex)));

		SpecificListType listType =
			GenericListType.getInstance().createSpecificListType(
				((SpecificIteratorType) variableType.getVariableClassDeclarationType())
					.getIteratorSpecificType());

		ISpecificFunctionDeclaration popSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.POP_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(listType));

		Assert.assertNotNull(popSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			popSpecFunDecl,
			CollectionsHelper.createList(getVariableCall));
	}
}