package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * PushVar(variable<set<T>>, <T>)
  */

final class PushVariableGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("Set Type");

	public PushVariableGenericFunctionDeclaration() {
		super("PushVariable", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setDocumentation("Pushs operand (Arg2) onto Set (Arg1), so (Arg2) is the new last-in value of Set (Arg1)");
		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {

		// PushVar(variable<set<T>>, <T>)

		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 2) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
						.getInstance(false, true)
						.createSpecificVariableType(
							GenericIteratorType.getInstance().createSpecificIteratorType(typeT));
				} else if (index == 1) {
					return typeT;
				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {

				Assert.fail();
				return null;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {

		// PushVar(variable<set<T>>, <T>)

		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	/**
	 * @param inputTypes Iterator
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {

		// PushVar(variable<set<T>>, <T>)

		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType typeT = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (typeT == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) specificTypeA;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();

		ISpecificType iteratorSpecificType = specificSetType.getIteratorSpecificType();
		if (!iteratorSpecificType.isMoreSpecific(typeT))
			return null;
		else
			return createFunctionDeclaration(iteratorSpecificType);
	}

	/**
	 * @param outputType ISpecificType
	 * @return String
	 * @see de.uniwue.ki.mas.basics.functionDeclaration.IFunctionDeclaration#getSignatureWithoutOutputType(ISpecificType)
	 */
	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<Set<T>>, T)";
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {

		ISpecificFunctionDeclaration pushVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;
		int valueIndex = 1;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) pushVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);
		ISpecificType valueType = pushVarSpecificFunctionDeclaration.getSpecificInputTypeAt(valueIndex);

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(composedFunctionCallInternDeclaration.getChild(variableIndex)));

		SpecificListType listType =
			GenericListType.getInstance().createSpecificListType(
				((SpecificIteratorType) variableType.getVariableClassDeclarationType())
					.getIteratorSpecificType());

		ISpecificFunctionDeclaration pushSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.PUSH_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(valueType, listType));

		Assert.assertNotNull(pushSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			pushSpecFunDecl,
			CollectionsHelper.createList(
				composedFunctionCallInternDeclaration.getChild(valueIndex),
				getVariableCall));
	}
}