package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.*;
import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.dependent.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * PushVarOf(variable<set<T>>, <T>, Body)
  */

final class PushVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithUnspecificDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	private static final IDependentType typeDependentT = new UnconstraintDependentType("Set Type");

	public PushVariableOfGenericFunctionDeclaration() {
		super("PushVariableOf", VoidType.getInstance(), CollectionsHelper.createList(typeDependentT));

		setExpertFunctionDeclaration(true);
	}

	private ISpecificFunctionDeclaration createFunctionDeclaration(final ISpecificType typeT) {
		return new AbstractSpecificFunctionAdapterGenericFunctionDeclaration(this, 3) {
			public ISpecificType getSpecificOutputType() {
				return VoidType.getInstance();
			}
			public ISpecificType getSpecificInputTypeAt(int index) {
				if (index == 0) {
					return (SpecificVariableClassDeclarationType) GenericVariableClassDeclarationType
						.getInstance(true, true)
						.createSpecificVariableType(
							GenericIteratorType.getInstance().createSpecificIteratorType(typeT));

				} else if (index == 1) {
					return typeT;

				} else if (index == 2) {
					return GenericSimObjectType.getGeneralSimObjectType();

				} else {
					Assert.fail();
					return null;
				}
			}

			public IFunction createFunction(
				ICreateFunctionArgs createFunctionArgs,
				IFunctionList functions) {

				Assert.fail();
				return null;
			}
		};
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(
		ISpecificType outputType,
		DependentTypeBindings bindings) {
		final ISpecificType typeT = bindings.getBindingFor(typeDependentT);

		return createFunctionDeclaration(typeT);
	}

	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType variableType = getNextSpecificType(inputTypes);
		ISpecificType typeT = getNextSpecificType(inputTypes);
		ISpecificType bodyImageType = getNextSpecificType(inputTypes);

		if ((variableType == null) || (typeT == null) || (bodyImageType == null) || inputTypes.hasNext())
			return null;

		if (!bodyImageType.equals(GenericSimObjectType.getGeneralSimObjectType()))
			return null;

		if (!(variableType instanceof SpecificVariableClassDeclarationType))
			return null;

		SpecificVariableClassDeclarationType specificVariableClassDeclarationType =
			(SpecificVariableClassDeclarationType) variableType;

		if (!(specificVariableClassDeclarationType.getVariableClassDeclarationType()
			instanceof SpecificIteratorType)) {
			return null;
		}

		SpecificIteratorType specificSetType =
			(SpecificIteratorType) specificVariableClassDeclarationType.getVariableClassDeclarationType();
		ISpecificType iteratorSpecificType = specificSetType.getIteratorSpecificType();

		if (!iteratorSpecificType.isMoreSpecific(typeT))
			return null;
		else
			return createFunctionDeclaration(iteratorSpecificType);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<Set<T>>, T, Body)";
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {

		ISpecificFunctionDeclaration pushVarSpecificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		int variableIndex = 0;
		int valueIndex = 1;
		int bodyIndex = 2;

		SpecificVariableClassDeclarationType variableType =
			(SpecificVariableClassDeclarationType) pushVarSpecificFunctionDeclaration.getSpecificInputTypeAt(
				variableIndex);
		ISpecificType valueType = pushVarSpecificFunctionDeclaration.getSpecificInputTypeAt(valueIndex);
		ISpecificType bodyType = pushVarSpecificFunctionDeclaration.getSpecificInputTypeAt(bodyIndex);

		ISpecificFunctionDeclaration getVariableOfSpecFunDecl =
			FunctionDeclarationVariableFactory
				.GET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				variableType.getVariableClassDeclarationType(),
				CollectionsHelper.createIterator(variableType, bodyType));

		Assert.assertNotNull(getVariableOfSpecFunDecl);

		IComposedFunctionCallInternDeclaration getVariableCall =
			FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
				null,
				getVariableOfSpecFunDecl,
				CollectionsHelper.createList(
					composedFunctionCallInternDeclaration.getChild(variableIndex),
					composedFunctionCallInternDeclaration.getChild(bodyIndex)));

		SpecificListType listType =
			GenericListType.getInstance().createSpecificListType(
				((SpecificIteratorType) variableType.getVariableClassDeclarationType())
					.getIteratorSpecificType());

		ISpecificFunctionDeclaration pushSpecFunDecl =
			FunctionDeclarationListFunctionsFactory.PUSH_GENERIC_FUNCTION_DECL.createFunctionDeclaration(
				VoidType.getInstance(),
				CollectionsHelper.createIterator(valueType, listType));

		Assert.assertNotNull(pushSpecFunDecl);

		return FunctionCallDeclarationFactory.getInstance().createComposedFunctionCallInternDeclaration(
			composedFunctionCallInternDeclaration.getParent(),
			pushSpecFunDecl,
			CollectionsHelper.createList(
				composedFunctionCallInternDeclaration.getChild(valueIndex),
				getVariableCall));
	}
}