package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.*;
import java.util.logging.Level;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.spatial.ImageType;
import de.uniwue.ki.mas.basics.feature.spatial.function.FunctionDeclarationSpatialFactory;
import de.uniwue.ki.mas.basics.feature.spatial.position.Position2DType;
import de.uniwue.ki.mas.basics.function.iterator.FunctionDeclarationIteratorFactory;
import de.uniwue.ki.mas.basics.function.object.FunctionDeclarationObjectFactory;
import de.uniwue.ki.mas.basics.functionCall.IComposedFunctionCallInternDeclaration;
import de.uniwue.ki.mas.basics.functionDeclaration.*;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationList;
import de.uniwue.ki.mas.basics.functionDeclaration.list.FunctionDeclarationListHelper;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.sim.experiment.function.FunctionDeclarationExperimentFactory;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.mas.xml.reader.XMLReadException;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Insert the type's description here.
 * Creation date: (12.03.2002 10:07:23)
 * @author Christoph Oechslein
 */
public class RemovedFunctionDeclarationList extends FunctionDeclarationList {

	public interface ITransformer {
		IComposedFunctionCallInternDeclaration transform(
			IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
			XMLModelReader xmlModelReader)
			throws XMLReadException;
	}

	private final Map _transformFunctionCallMap = new HashMap();

	public RemovedFunctionDeclarationList() {
		super();

		addTransformFunctionDeclaration(new SimulateNTimesSimulationSpecificFunctionDeclarationOld());
		addTransformFunctionDeclaration(new TransformSetVariableGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new TransformSetVariableOfGenericFunctionDeclaration());

		addTransformFunctionDeclaration(new PopVariableGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new PopAndReturnVariableGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new PushVariableGenericFunctionDeclaration());

		addTransformFunctionDeclaration(new PopVariableOfGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new PopAndReturnVariableOfGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new PushVariableOfGenericFunctionDeclaration());

		addTransformFunctionDeclaration(new CreateComposedTypeValueGenericFunctionDeclaration(true));
		addTransformFunctionDeclaration(new CreateComposedTypeValueGenericFunctionDeclaration(false));
		addTransformFunctionDeclaration(new GetValueOfComposedTypeGenericFunctionDeclaration());
		addTransformFunctionDeclaration(new SetValueOfComposedTypeGenericFunctionDeclaration());

		addTransformFunctionDeclaration(new FunctionActionDeclarationCreateObjectOld());

		addTransformFunctionDeclaration(
			new ChangeNameGenericFunctionDeclaration(
				"Filter",
				FunctionDeclarationIteratorFactory.SELECT_GENERIC_FUN_DECL));

		addTransformFunctionDeclaration(new OldWhileSpecificFunctionDeclaration());

		addTransformFunctionDeclaration(new GetAllObjectsSpecificFunctionDeclaration());
		addTransformFunctionDeclaration(new GetAllObjectsOfTypeSpecificFunctionDeclaration());
		addTransformFunctionDeclaration(new GetAllObjectsWithFeatureSpecificFunctionDeclaration());

		addTransformFunctionDeclaration(
			new ChangeNameSpecificFunctionDeclaration(
				"GetWorldBody",
				FunctionDeclarationObjectFactory.GET_WORLD_FUNCTION_DECLARATION));
		addTransformFunctionDeclaration(
			new ChangeNameSpecificFunctionDeclaration(
				"GetMyBody",
				FunctionDeclarationObjectFactory.GET_CURRENT_SIMOBJECT_FUNCTION_DECLARATION));

		addTransformFunctionDeclaration(
			new ChangeNameSpecificFunctionDeclaration(
				"GetWorldBodyFromSim",
				FunctionDeclarationExperimentFactory.GET_WORLD_FROM_SIMULATION_SPECIFIC_FUNCTION_DECL));

		addTransformFunctionDeclaration(new BeamBodyToPositionSpecificFunctionDeclaration());

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				Position2DType.getInstance(),
				FunctionDeclarationSpatialFactory.BEAM_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				DoubleType.getInstance(),
				FunctionDeclarationSpatialFactory.CHANGE_DIRECTION_BY_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				DoubleType.getInstance(),
				FunctionDeclarationSpatialFactory.CHANGE_DIRECTION_TO_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				DoubleType.getInstance(),
				FunctionDeclarationSpatialFactory.CHANGE_SPEED_BY_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				DoubleType.getInstance(),
				FunctionDeclarationSpatialFactory.CHANGE_SPEED_TO_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				ImageType.getInstance(),
				FunctionDeclarationSpatialFactory.CHANGE_IMAGE_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new ChangeNameSpecificFunctionDeclaration(
				"GetMapPatch",
				FunctionDeclarationSpatialFactory.GET_PATCH_FUN_DECL));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				"MoveToPos",
				CollectionsHelper.createList(Position2DType.getInstance(), DoubleType.getInstance()),
				FunctionDeclarationSpatialFactory.MOVE_TOWARDS_POS_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(
			new InsertGetSpatialInfoSpecificFunctionDeclaration(
				CollectionsHelper.EMPTY_LIST,
				FunctionDeclarationSpatialFactory.MOVE_SPECIFIC_FUN_DECL_HAS_SPATIAL_INFO));

		addTransformFunctionDeclaration(new RandomIntGenericFunctionArgsDeclaration());

		addTransformFunctionDeclaration(
			new SimulationTransformSpecificFunctionDeclaration(
				FunctionDeclarationExperimentFactory.GET_SIMULATION_NAME_SPECIFIC_FUNCTION_DECL,
				StringType.getInstance()));

		addTransformFunctionDeclaration(
			new SimulationTransformSpecificFunctionDeclaration(
				FunctionDeclarationExperimentFactory.SET_SIMULATION_NAME_SPECIFIC_FUNCTION_DECL,
				VoidType.getInstance(),
				CollectionsHelper.createList(
					SimulationDeclarationType.getInstance(),
					StringType.getInstance())));

		addTransformFunctionDeclaration(
			new SimulationTransformSpecificFunctionDeclaration(
				FunctionDeclarationExperimentFactory.SIM_SIMULATION_SPECIFIC_FUNCTION_DECL,
				VoidType.getInstance(),
				CollectionsHelper.createList(
					SimulationDeclarationType.getInstance(),
					BooleanType.getInstance())));

		addTransformFunctionDeclaration(
			new SimulationTransformSpecificFunctionDeclaration(
				FunctionDeclarationExperimentFactory.GET_WORLD_FROM_SIMULATION_SPECIFIC_FUNCTION_DECL,
				GenericSimObjectType.getGeneralSimObjectType()));

		addTransformFunctionDeclaration(
			new SimulationTransformSpecificFunctionDeclaration(
				FunctionDeclarationExperimentFactory.GET_WORLD_FROM_SIMULATION_SPECIFIC_FUNCTION_DECL,
				"GetWorldBodyFromSim",
				GenericSimObjectType.getGeneralSimObjectType(),
				CollectionsHelper.createList(SimulationDeclarationType.getInstance())));
	}

	public boolean contains(ISpecificFunctionDeclaration specificFunctionDeclaration) {
		if (super.contains(specificFunctionDeclaration)) {
			return true;

		} else if (specificFunctionDeclaration.getGenericFunctionDeclaration() != null) {
			IGenericFunctionDeclaration genericFunctionDeclaration =
				specificFunctionDeclaration.getGenericFunctionDeclaration();
			Iterator it = FunctionDeclarationListHelper.getGenericFunctionDeclarations(iterator());

			while (it.hasNext()) {
				IGenericFunctionDeclaration currGenericFunctionDeclaration =
					(IGenericFunctionDeclaration) it.next();

				if (currGenericFunctionDeclaration.equals(genericFunctionDeclaration)) {
					return true;
				}
			}
		}

		return false;
	}

	private void addTransformFunctionDeclaration(IFunctionDeclaration functionDeclaration) {
		addTransformFunctionDeclaration(functionDeclaration, (ITransformer) functionDeclaration);
	}

	private void addTransformFunctionDeclaration(
		IFunctionDeclaration functionDeclaration,
		ITransformer transformer) {
		add(functionDeclaration);
		_transformFunctionCallMap.put(functionDeclaration, transformer);
	}

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader)
		throws XMLReadException {

		ISpecificFunctionDeclaration specificFunctionDeclaration =
			composedFunctionCallInternDeclaration.getSpecificFunctionDeclaration();

		if (_transformFunctionCallMap.containsKey(specificFunctionDeclaration)) {
			IComposedFunctionCallInternDeclaration result =
				((ITransformer) _transformFunctionCallMap.get(specificFunctionDeclaration)).transform(
					composedFunctionCallInternDeclaration,
					xmlModelReader);

			if (Assert.ASSERTION_ON
				&& (result != composedFunctionCallInternDeclaration)
				&& XMLModelReader.LOGGER.isLoggable(Level.INFO)) {
				XMLModelReader.LOGGER.info(
					"Transformed old call from "
						+ composedFunctionCallInternDeclaration.getName()
						+ " to "
						+ result.getName());
			}

			return result;

		} else if (
			_transformFunctionCallMap.containsKey(
				specificFunctionDeclaration.getGenericFunctionDeclaration())) {
			return (
				(ITransformer) _transformFunctionCallMap.get(
					specificFunctionDeclaration.getGenericFunctionDeclaration())).transform(
				composedFunctionCallInternDeclaration,
				xmlModelReader);

		} else {
			return composedFunctionCallInternDeclaration;
		}
	}
	
	public boolean accept(Object declaration) {
		return declaration instanceof IFunctionDeclaration;
	}
}