package de.uniwue.ki.mas.xml.reader.oldfunctions;
import java.util.Iterator;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.function.list.FunctionDeclarationListFunctionsFactory;
import de.uniwue.ki.mas.basics.function.variable.FunctionDeclarationVariableFactory;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.AbstractGenericFunctionDeclarationWithoutDependents;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.xml.reader.XMLModelReader;
import de.uniwue.ki.util.collection.CollectionsHelper;

/**
  * Set_var(variable<T>, <T>)
  */

final class TransformSetVariableOfGenericFunctionDeclaration
	extends AbstractGenericFunctionDeclarationWithoutDependents
	implements RemovedFunctionDeclarationList.ITransformer {

	public IComposedFunctionCallInternDeclaration transform(
		IComposedFunctionCallInternDeclaration composedFunctionCallInternDeclaration,
		XMLModelReader xmlModelReader) {

		SpecificIteratorType specificIteratorType =
			(SpecificIteratorType) composedFunctionCallInternDeclaration.getChild(1).getSpecificOutputType();

		SpecificListType specificListType =
			GenericListType.getInstance().createSpecificListType(
				specificIteratorType.getIteratorSpecificType());

		ISpecificType specificVariableListType =
			GenericVariableClassDeclarationType.getInstance(true, true).createSpecificVariableType(
				specificListType);

		ISpecificFunctionDeclaration setVarFunctionDeclaration =
			FunctionDeclarationVariableFactory
				.SET_VARIABLE_OF_GENERIC_FUNCTION_DECLARATION
				.createFunctionDeclaration(
				null,
				CollectionsHelper.createIterator(
					specificVariableListType,
					specificListType,
					GenericSimObjectType.getGeneralSimObjectType()));

		Assert.assertNotNull(setVarFunctionDeclaration);

		// don't use copy since we don't know if the composedFunctionCallInternDeclaration is fully expanded (see delaySettings)

		// Map objectMap = new HashMap();

		IComposedFunctionCallInternDeclaration asListCall =
			FunctionCallDeclarationFactory
				.getInstance()
				.createComposedFunctionCallInternDeclaration(
					null,
					FunctionDeclarationListFunctionsFactory
						.AS_LIST_GENERIC_FUNCTION_DECL
						.createFunctionDeclaration(
						null,
						CollectionsHelper.createIterator(specificIteratorType)),
					CollectionsHelper.createIterator(composedFunctionCallInternDeclaration.getChild(1)
		/*.copy(objectMap)*/
		));

		IAtomicFunctionCallInternDeclaration variableFunctionCall =
			FunctionCallDeclarationFactory.getInstance().createAtomicFunctionCallInternDeclaration(
				null,
				specificVariableListType,
				((IAtomicFunctionCallInternDeclaration) composedFunctionCallInternDeclaration.getChild(0))
					.getArgumentDeclaration()
					.getObject());

		IComposedFunctionCallInternDeclaration setVarCall =
			FunctionCallDeclarationFactory
				.getInstance()
				.createComposedFunctionCallInternDeclaration(
					composedFunctionCallInternDeclaration.getParent(),
					setVarFunctionDeclaration,
					CollectionsHelper
						.createIterator(
							variableFunctionCall,
							asListCall,
							composedFunctionCallInternDeclaration.getChild(2)
		/*.copy(objectMap)*/
		));

		// dont call remove since it will remove also the argument list and so on
		// composedFunctionCallInternDeclaration.remove();

		return setVarCall;
	}

	public TransformSetVariableOfGenericFunctionDeclaration() {
		super("SetVariableOf", VoidType.getInstance());
	}

	/**
	 * @param typeT ISpecificType
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(ISpecificType typeT) {
		Assert.assertTrue(GenericIteratorType.getInstance().canCreate(typeT));

		SpecificIteratorType specificIteratorType = (SpecificIteratorType) typeT;

		return new TransformSetVariableOfSpecificFunctionDeclaration(this, specificIteratorType);
	}

	/**
	 * @param inputTypes Iterator
	 * @return ISpecificFunctionDeclaration
	 */
	public ISpecificFunctionDeclaration createFunctionDeclaration(Iterator inputTypes) {
		ISpecificType specificTypeA = getNextSpecificType(inputTypes);
		ISpecificType typeT = getNextSpecificType(inputTypes);
		ISpecificType bodyImageType = getNextSpecificType(inputTypes);

		if ((specificTypeA == null) || (typeT == null) || (bodyImageType == null) || inputTypes.hasNext())
			return null;

		if (!(specificTypeA instanceof SpecificVariableClassDeclarationType))
			return null;

//		Replaced this because of delayed reading!	Rainer 27.Mai 2004
//		if (!GenericSimObjectType.getGeneralSimObjectType().equals(bodyImageType))
//			return null;
		if (!(bodyImageType instanceof ISimObjectType)) 
			return null;

//		Comment: I don't know if this made sence ever 
		if (!GenericIteratorType.getInstance().canCreate(typeT))
			return null;

		return createFunctionDeclaration(typeT);
	}

	public String getSignatureWithoutNameAndOutputType(ISpecificType outputType) {
		return "(Var<T>, T, body)";
	}
}