package de.uniwue.ki.mas.xml.writer;
import java.util.*;

import de.uniwue.ki.mas.basics.functionArgument.IFunctionArgumentDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.functionUser.UserFunctionDeclarationList;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.util.name.INamed;

/**
* Insert the type's description here.
* Creation date: (07.02.2001 14:21:01)
* @author Christoph Oechslein
*/

class XMLModelFunctionWriter {
	private final XMLModelWriter _xmlModelWriter;
	private final Map _userFunctionMap = new HashMap();

	public XMLModelFunctionWriter(XMLModelWriter xmlModelWriter) {
		super();

		_xmlModelWriter = xmlModelWriter;
	}

	XMLFunctionCallInternGroup createXMLAtomicFunctionCallIntern(IAtomicFunctionCallInternDeclaration realAtomicFunctionCallIntern)
		throws XMLWriteException {

		XMLFunctionCallInternGroup functionCallIntern = new XMLFunctionCallInternGroup();
		// ISpecificType type = realAtomicFunctionCallIntern.getSpecificOutputType();
		// functionCallIntern.setType(xmlModelWriter.createXMLType(type));

		functionCallIntern.setXMLTypeValueGroup(
			_xmlModelWriter.createXMLTypeValueGroup(realAtomicFunctionCallIntern.getArgumentDeclaration()));

		return functionCallIntern;
	}

	XMLFunctionCallInternGroup createXMLComposedFunctionCallIntern(IComposedFunctionCallInternDeclaration realComposedFunctionCallIntern)
		throws XMLWriteException {

		XMLFunctionCallInternGroup xmlFunctionCallInternGroup = new XMLFunctionCallInternGroup();
		XMLComposedFunctionCallIntern xmlComposedFunctionCallIntern = new XMLComposedFunctionCallIntern();

		xmlFunctionCallInternGroup.setCall(xmlComposedFunctionCallIntern);

		// functionCallInternGroup.setType(xmlModelWriter.createXMLType(realComposedFunctionCallIntern.getOutputType()));

		ISpecificFunctionDeclaration functionDeclaration =
			realComposedFunctionCallIntern.getSpecificFunctionDeclaration();

		if (functionDeclaration instanceof IUserFunctionDeclaration)
			xmlComposedFunctionCallIntern.setUserFunctionID(
				getXMLUserFunction((IUserFunctionDeclaration) functionDeclaration));
		else
			xmlComposedFunctionCallIntern.setFunctionName(functionDeclaration.getName());

		if ((functionDeclaration.getGenericFunctionDeclaration() != null)
			&& (functionDeclaration
				.getGenericFunctionDeclaration()
				.needSpecificOutputTypeToCreateSpecificFunctionDeclaration(realComposedFunctionCallIntern))) {

			xmlComposedFunctionCallIntern.setXMLTypeGroup(
				_xmlModelWriter.createXMLTypeGroup(functionDeclaration.getSpecificOutputType()));
		}

		Iterator it = realComposedFunctionCallIntern.getChildren();
		while (it.hasNext()) {
			IFunctionCallInternDeclaration functionCallInternDeclaration =
				(IFunctionCallInternDeclaration) it.next();

			xmlComposedFunctionCallIntern.addXMLFunctionCallInternGroup(
				createXMLFunctionCallIntern(functionCallInternDeclaration));
		}

		return xmlFunctionCallInternGroup;
	}

	XMLFunctionCall createXMLFunctionCall(
		Object outerObject,
		IFunctionCallDeclaration functionCallDeclaration) {

		XMLFunctionCall functionCall = new XMLFunctionCall();

		setXMLFunctionCallDelayed(outerObject, functionCall, functionCallDeclaration);

		return functionCall;
	}

	private XMLFunctionCallInternGroup createXMLFunctionCallIntern(IFunctionCallInternDeclaration functionCallInternDeclaration)
		throws XMLWriteException {
		if (functionCallInternDeclaration instanceof IComposedFunctionCallInternDeclaration)
			return createXMLComposedFunctionCallIntern(
				(IComposedFunctionCallInternDeclaration) functionCallInternDeclaration);
		else
			return createXMLAtomicFunctionCallIntern(
				(IAtomicFunctionCallInternDeclaration) functionCallInternDeclaration);
	}

	private XMLUserFunction createXMLUserFunction(IUserFunctionDeclaration realUserDefinedFunctionDeclaration)
		throws XMLWriteException {

		XMLUserFunction xmlUserFunction = new XMLUserFunction();

		xmlUserFunction.setId(_xmlModelWriter.getXMLUniqueId(realUserDefinedFunctionDeclaration));
		if (realUserDefinedFunctionDeclaration.getUsesShortDescription()) {
			xmlUserFunction.setName(realUserDefinedFunctionDeclaration.getShortDescription());
		}

		if (realUserDefinedFunctionDeclaration.getDocumentation().length() > 0) {

			xmlUserFunction.setDocumentation(
				XMLModelWriter.escapeDocumentationString(
					realUserDefinedFunctionDeclaration.getDocumentation()));
		}

		if (!realUserDefinedFunctionDeclaration.isExternal()) {
			xmlUserFunction.setExternal(false);
		}

		if (realUserDefinedFunctionDeclaration.isExpertFunctionDeclaration()) {
			xmlUserFunction.setExpert(true);
		}

		xmlUserFunction.setXMLTypeGroup(
			_xmlModelWriter.createXMLTypeGroup(realUserDefinedFunctionDeclaration.getSpecificOutputType()));

		if (!realUserDefinedFunctionDeclaration.isInlinePossible()) {
			xmlUserFunction.setInline(false);
		}

		setXMLUnnamedParametricFunction(
			realUserDefinedFunctionDeclaration,
			xmlUserFunction,
			realUserDefinedFunctionDeclaration
				.getParametricFunctionCallDeclaration());

		return xmlUserFunction;
	}

	XMLUserFunction[] createXMLUserFunctionList(UserFunctionDeclarationList realUserDefinedFunctionDeclarationList)
		throws XMLWriteException {

		XMLUserFunction[] userFunctionList =
			new XMLUserFunction[realUserDefinedFunctionDeclarationList.size()];
		for (int i = 0; i < realUserDefinedFunctionDeclarationList.size(); i++) {
			userFunctionList[i] =
				getXMLUserFunction((IUserFunctionDeclaration) realUserDefinedFunctionDeclarationList.get(i));
		}
		return userFunctionList;
	}

	XMLUserFunction getXMLUserFunction(IUserFunctionDeclaration userDefinedFunctionDeclaration)
		throws XMLWriteException {

		if (!_userFunctionMap.containsKey(userDefinedFunctionDeclaration))
			_userFunctionMap.put(
				userDefinedFunctionDeclaration,
				createXMLUserFunction(userDefinedFunctionDeclaration));

		return (XMLUserFunction) _userFunctionMap.get(userDefinedFunctionDeclaration);
	}

	void setXMLFunctionCall(
		Object outerObject,
		XMLFunctionCall functionCall,
		IFunctionCallDeclaration functionCallDeclaration)
		throws XMLWriteException {

		if (!functionCallDeclaration
			.getMostGeneralOutputType()
			.equals(functionCallDeclaration.getSpecificOutputType())) {
			functionCall.setXMLTypeGroup(
				_xmlModelWriter.createXMLTypeGroup(functionCallDeclaration.getMostGeneralOutputType()));
		}

		try {
			functionCall.setXMLFunctionCallInternGroup(
				createXMLFunctionCallIntern(functionCallDeclaration.getFunctionCallInternDeclaration()));

		} catch (XMLWriteException e) {
			throw new XMLWriteException(
				"Error in "
					+ ((outerObject instanceof INamed)
						? ((INamed) outerObject).getName()
						: outerObject.toString())
					+ "\nError while writing function: "
					+ functionCallDeclaration.getFunctionCallInternDeclaration().getName()
					+ "\nError: "
					+ e.getMessage());
		}
	}

	private void setXMLFunctionCallDelayed(
		Object outerObject,
		XMLFunctionCall functionCall,
		IFunctionCallDeclaration functionCallDeclaration) {

		_xmlModelWriter.addDelaySettings(outerObject, functionCall, functionCallDeclaration);
	}

	public void setXMLUnnamedParametricFunction(
		Object outerObject,
		XMLUnnamedParametricFunction xmlUnnamedParametricFunction,
		IParametricFunctionCallDeclaration parametricFunctionCallDeclaration)
		throws XMLWriteException {

		/*
		XMLFunctionCall xmlFunctionCall =
		   xmlModelWriter.createXMLFunctionCall(parametricFunctionCallDeclaration, declarationToXMLObject);
		
		xmlUnnamedParametricFunction.setXMLFunctionCallInternGroup(xmlFunctionCall.getXMLFunctionCallInternGroup());
		xmlUnnamedParametricFunction.setXMLTypeGroup(xmlFunctionCall.getXMLTypeGroup());
		*/

		xmlUnnamedParametricFunction.setFunctionCall(
			_xmlModelWriter.createXMLFunctionCall(
				outerObject,
				parametricFunctionCallDeclaration.getFunctionCallDeclaration()));

		Iterator it = parametricFunctionCallDeclaration.getFunctionArgumentDeclarationList().iterator();
		while (it.hasNext()) {
			IFunctionArgumentDeclaration currFunctionArgumentDeclaration =
				(IFunctionArgumentDeclaration) it.next();
			XMLFunctionParameter xmlFunctionParameter = new XMLFunctionParameter();

			_xmlModelWriter.setXMLIdAndNameDocumentable(
				xmlFunctionParameter,
				currFunctionArgumentDeclaration);
			xmlFunctionParameter.setXMLTypeGroup(
				_xmlModelWriter.createXMLTypeGroup(currFunctionArgumentDeclaration.getSpecificType()));

			_xmlModelWriter.setObject(currFunctionArgumentDeclaration, xmlFunctionParameter);

			xmlUnnamedParametricFunction.addParameter(xmlFunctionParameter);
		}
	}
}