package de.uniwue.ki.mas.xml.writer;

import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.IObjectFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.IObjectFeatureInstanceDeclaration;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.reasoning.IReasoningEngineDeclaration;
import de.uniwue.ki.mas.entity.bodyclass.IBodyClassDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.object.world.IWorldInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.entity.variable.IBodyVariableInstanceDeclaration;
import de.uniwue.ki.mas.entity.variableclass.*;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
 * @author oechslein
 * 
 * Creation date: 21.05.2003
 */
public class XMLModelObjectWriter {

    private final Map _objectFeatureClassMap = new HashMap();

    private static final List _xmlFeatureWriters = new LinkedList();

    public static void addFeatureWriter(IXMLFeatureWriter featureWriter) {
        _xmlFeatureWriters.add(featureWriter);
    }

    private final XMLModelWriter _xmlModelWriter;

    public XMLModelObjectWriter(XMLModelWriter xmlModelWriter) {
        _xmlModelWriter = xmlModelWriter;
    }

    public boolean containsDeclaration(IDeclaration declaration) {
        return _xmlModelWriter.containsDeclaration(declaration);
    }

    public XMLRootObject getObject(IDeclaration declaration)
            throws XMLWriteException {
        return _xmlModelWriter.getObject(declaration);
    }

    public void setObject(IDeclaration declaration, XMLRootObject object) {
        _xmlModelWriter.setObject(declaration, object);
    }

    public XMLBodyVariableClass getXMLBodyVariableClass(
            IBodyVariableClassDeclaration bodyVariableClassDeclaration)
            throws XMLWriteException {
        if (!containsDeclaration(bodyVariableClassDeclaration))
                setObject(
                        bodyVariableClassDeclaration,
                        createXMLBodyVariableClass(bodyVariableClassDeclaration));
        return (XMLBodyVariableClass) getObject(bodyVariableClassDeclaration);
    }

    private XMLBodyVariableClass createXMLBodyVariableClass(
            IBodyVariableClassDeclaration bodyVariableClass)
            throws XMLWriteException {

        XMLBodyVariableClass xmlBodyVariableClass = new XMLBodyVariableClass();
        setXMLBodyVariableClass(xmlBodyVariableClass, bodyVariableClass);

        XMLVariableClass xmlVariableClass = getXMLVariableClass(bodyVariableClass
                .getVariableClassDeclaration());

        if (isVariableClassFromFeature(bodyVariableClass
                .getVariableClassDeclaration())) {
            xmlBodyVariableClass.setVariableClassID(xmlVariableClass);
        } else {
            xmlBodyVariableClass.setVariableClass(xmlVariableClass);
        }

        return xmlBodyVariableClass;
    }

    private void setXMLBodyVariableClass(
            XMLBodyVariableClass xmlBodyVariableClass,
            IBodyVariableClassDeclaration bodyVariableClass) {

        _xmlModelWriter.setXMLId(xmlBodyVariableClass, bodyVariableClass);

        if (bodyVariableClass instanceof IBodyFeatureVariableClassDeclaration) {
            IBodyFeatureVariableClassDeclaration bodyFeatureVariableClass = (IBodyFeatureVariableClassDeclaration) bodyVariableClass;

            if (bodyFeatureVariableClass
                    .getUsesOwnDefaulValueFunctionCallDeclaration()) {
                xmlBodyVariableClass
                        .setDefaultValue(_xmlModelWriter
                                .createXMLFunctionCall(
                                        bodyVariableClass,
                                        bodyFeatureVariableClass
                                                .getDefaultValueFunctionCallDeclaration()));
                xmlBodyVariableClass
                        .setUsesOwnDefaulValue(bodyFeatureVariableClass
                                .getUsesOwnDefaulValueFunctionCallDeclaration());
            }
        }
    }

    XMLVariableClass getXMLVariableClass(
            IVariableClassDeclaration variableClassDeclaration)
            throws XMLWriteException {
        if (!containsDeclaration(variableClassDeclaration))
                setObject(variableClassDeclaration,
                        createXMLVariableClass(variableClassDeclaration));
        return (XMLVariableClass) getObject(variableClassDeclaration);
    }

    private XMLVariableClass createXMLVariableClass(
            IVariableClassDeclaration variableClassDeclaration)
            throws XMLWriteException {
        XMLVariableClass xmlVariableClass = new XMLVariableClass();

        _xmlModelWriter.setXMLIdAndNameDocumentable(xmlVariableClass,
                variableClassDeclaration);
        xmlVariableClass
                .setXMLTypeGroup(_xmlModelWriter
                        .createXMLTypeGroup(variableClassDeclaration
                                .getSpecificType()));

        if (variableClassDeclaration.isExternal()) {
            xmlVariableClass.setExternal(true);
        }

        if (variableClassDeclaration.isWriteable()) {
            xmlVariableClass.setWriteable(true);
        }
        
        if (variableClassDeclaration.isPartOfName()) {
        	xmlVariableClass.setPartOfName(true);
        }

        xmlVariableClass.setDefaultValue(_xmlModelWriter.createXMLFunctionCall(
                variableClassDeclaration, variableClassDeclaration
                        .getDefaultValueFunctionCallDeclaration()));

        if (variableClassDeclaration.getHasNextValueFunctionCallDeclaration()) {
            xmlVariableClass.setHasNextValue(variableClassDeclaration
                    .getHasNextValueFunctionCallDeclaration());
            xmlVariableClass.setNextValue(_xmlModelWriter
                    .createXMLFunctionCall(variableClassDeclaration,
                            variableClassDeclaration
                                    .getNextValueFunctionCallDeclaration()));
        }

        return xmlVariableClass;
    }

    private boolean isVariableClassFromFeature(
            IVariableClassDeclaration variableClassDeclaration) {
        Iterator it = _xmlModelWriter.getSeSAmModelDeclaration()
                .getSeSAmModelBasicsDeclaration()
                .getUserFeatureClassDeclarationListAndFactory()
                .getUserFeatureClassDeclarationList().iterator();
        while (it.hasNext()) {

            IUserFeatureClassDeclaration currUserFeatureClassDeclaration = (IUserFeatureClassDeclaration) it
                    .next();

            VariableClassDeclarationList featureVariableClassDeclarationList = currUserFeatureClassDeclaration
                    .getFeatureVariableClassDeclarationListAndFactory()
                    .getVariableClassDeclarationList();

            for (int i = 0; i < featureVariableClassDeclarationList.size(); i++) {
                IVariableClassDeclaration currVariableClassDeclaration = (IVariableClassDeclaration) featureVariableClassDeclarationList
                        .get(i);

                if (variableClassDeclaration
                        .equals(currVariableClassDeclaration)) { return true; }
            }
        }

        return false;
    }

    XMLAgentClass getXMLAgentClass(IAgentClassDeclaration agent)
            throws XMLWriteException {
        if (!containsDeclaration(agent))
                setObject(agent, createXMLAgentClass(agent));
        return (XMLAgentClass) getObject(agent);
    }

    private XMLAgentClass createXMLAgentClass(IAgentClassDeclaration agent)
            throws XMLWriteException {
        XMLAgentClass xmlAgentClass = new XMLAgentClass();
        setXMLAgentClass(xmlAgentClass, agent);
        return xmlAgentClass;
    }

    private XMLBodyClass createXMLBodyClass(IBodyClassDeclaration bodyClass)
            throws XMLWriteException {
        Iterator it = bodyClass.getBodyVariableClassDeclarationListAndFactory()
                .getBodyVariableClassDeclarationList().iterator();
        if (!it.hasNext()) {
            return null;
        } else {
            XMLBodyClass xmlBodyClass = new XMLBodyClass();
            while (it.hasNext()) {
                XMLBodyVariableClass xmlBodyVariableClass = getXMLBodyVariableClass((IBodyVariableClassDeclaration) it
                        .next());
                if (xmlBodyVariableClass != null)
                        xmlBodyClass.addBodyVariableClass(xmlBodyVariableClass);
            }
            return xmlBodyClass;
        }
    }

    private XMLRessourceClass createXMLRessourceClass(
            IRessourceClassDeclaration agent) throws XMLWriteException {
        XMLRessourceClass xmlRessourceClass = new XMLRessourceClass();
        setXMLRessourceClass(xmlRessourceClass, agent);
        return xmlRessourceClass;
    }

    private XMLWorldClass createXMLWorldClass(IWorldClassDeclaration world)
            throws XMLWriteException {
        XMLWorldClass xmlWorldClass = new XMLWorldClass();
        setXMLWorldClass(xmlWorldClass, world);
        return xmlWorldClass;
    }

    XMLWorldClass getXMLWorldClass(IWorldClassDeclaration world)
            throws XMLWriteException {
        if (!containsDeclaration(world))
                setObject(world, createXMLWorldClass(world));
        return (XMLWorldClass) getObject(world);
    }

    private XMLAbstractObjectClass getXMLObjectClass(
            IObjectClassDeclaration object) throws XMLWriteException {

        XMLAbstractObjectClass xmlAbstractObjectClass = (XMLAbstractObjectClass) getObject(object);
        return xmlAbstractObjectClass;
    }

    private void setXMLAgentClass(XMLAgentClass xmlAgentClass,
            IAgentClassDeclaration realAgent) throws XMLWriteException {
        setXMLObjectClass(xmlAgentClass, realAgent);

        Iterator it = realAgent.getReasoningEngineDeclarationListAndFactory()
                .getDeclarationList().iterator();
        while (it.hasNext()) {
            xmlAgentClass
                    .addReasoning(createXMLReasoning((IActivityReasoningEngineDeclaration) it
                            .next()));
        }
    }

    private void setXMLObjectClass(XMLAbstractObjectClass xmlObjectClass,
            IObjectClassDeclaration realObject) throws XMLWriteException {

        _xmlModelWriter.setXMLIdAndNameDocumentable(xmlObjectClass, realObject);

        xmlObjectClass.setBodyClass(createXMLBodyClass(realObject
                .getBodyClassDeclaration()));
        Iterator features = realObject.getObjectFeatureClassDeclarationList()
                .iterator();

        XMLFeatureClassList xmlFeatureClassList = new XMLFeatureClassList();
        xmlObjectClass.setFeatures(xmlFeatureClassList);

        while (features.hasNext()) {
            IObjectFeatureClassDeclaration currentFeatureClassDeclaration = (IObjectFeatureClassDeclaration) features
                    .next();

            if (currentFeatureClassDeclaration instanceof IUserFeatureClassDeclaration) {
                XMLUserFeatureClassID xmlUserFeatureClassID = new XMLUserFeatureClassID();
                xmlUserFeatureClassID
                        .setId(_xmlModelWriter
                                .getXMLUserFeatureClass((IUserFeatureClassDeclaration) currentFeatureClassDeclaration));

                xmlFeatureClassList.addUser(xmlUserFeatureClassID);

            } else if (currentFeatureClassDeclaration instanceof IHasSpatialInfoDeclaration) {
                xmlFeatureClassList
                        .setSpatialInfo(_xmlModelWriter
                                .createXMLSpatialInfo(((IHasSpatialInfoDeclaration) currentFeatureClassDeclaration)
                                        .getSpatialInfo()));

            } else {
                XMLFeature xmlFeature = getXMLObjectFeatureClassDeclaration(currentFeatureClassDeclaration);

                if (xmlFeature instanceof XMLEvolutionFeature) {
                    xmlFeatureClassList
                            .setEvolution((XMLEvolutionFeature) xmlFeature);

                } else if (xmlFeature == null) {
                    xmlFeatureClassList
                            .addBuildin(currentFeatureClassDeclaration
                                    .getName());

                } else {
                    xmlFeatureClassList.addFeatures(xmlFeature);
                }
            }
        }

    }

    private XMLFeature createXMLObjectFeatureClassDeclaration(
            IObjectFeatureClassDeclaration currentFeatureClassDeclaration)
            throws XMLWriteException {

        Iterator it = _xmlFeatureWriters.iterator();

        while (it.hasNext()) {
            IXMLFeatureWriter featureWriter = (IXMLFeatureWriter) it.next();
            if (featureWriter.isResponsibleFor(currentFeatureClassDeclaration)) {
                XMLFeature xmlFeature = featureWriter
                        .createXMLObjectFeatureClassDeclaration(
                                _xmlModelWriter, this,
                                currentFeatureClassDeclaration);
                xmlFeature.setId(_xmlModelWriter
                        .getXMLUniqueId(currentFeatureClassDeclaration));
                return xmlFeature;
            }
        }
        return null;
    }

    private XMLFeature getXMLObjectFeatureClassDeclaration(
            IObjectFeatureClassDeclaration currentFeatureClassDeclaration)
            throws XMLWriteException {
        if (!_objectFeatureClassMap.containsKey(currentFeatureClassDeclaration)) {
            XMLFeature newXMLFeature = createXMLObjectFeatureClassDeclaration(currentFeatureClassDeclaration);
            _objectFeatureClassMap.put(currentFeatureClassDeclaration,
                    newXMLFeature);
        }
        return (XMLFeature) _objectFeatureClassMap
                .get(currentFeatureClassDeclaration);
    }

    private XMLFeatureInstance createXMLObjectFeatureInstanceDeclaration(
            IObjectFeatureInstanceDeclaration currentFeatureInstanceDeclaration)
            throws XMLWriteException {

        Iterator it = _xmlFeatureWriters.iterator();

        while (it.hasNext()) {
            IXMLFeatureWriter featureWriter = (IXMLFeatureWriter) it.next();
            if (featureWriter
                    .isResponsibleFor(currentFeatureInstanceDeclaration
                            .getObjectFeatureClassDeclaration())) {
                XMLFeatureInstance featureInstance = featureWriter
                        .createXMLObjectFeatureInstanceDeclaration(
                                _xmlModelWriter, this,
                                currentFeatureInstanceDeclaration);
                if (featureInstance == null) return null;

                Assert.assertNotNull(_objectFeatureClassMap
                        .get(currentFeatureInstanceDeclaration
                                .getObjectFeatureClassDeclaration()));

                // Link top the objectFeatureClassDeclaration
                featureInstance.setFeatureObjectClassID(_objectFeatureClassMap
                        .get(currentFeatureInstanceDeclaration
                                .getObjectFeatureClassDeclaration()));
                return featureInstance;
            }
        }
        return null;
    }

    XMLRessourceClass getXMLRessourceClass(IRessourceClassDeclaration agent)
            throws XMLWriteException {
        if (!containsDeclaration(agent))
                setObject(agent, createXMLRessourceClass(agent));
        return (XMLRessourceClass) getObject(agent);
    }

    private void setXMLRessourceClass(XMLRessourceClass xmlRessourceClass,
            IRessourceClassDeclaration realRessource) throws XMLWriteException {

        setXMLObjectClass(xmlRessourceClass, realRessource);
    }

    private void setXMLWorldClass(XMLWorldClass xmlWorldClass,
            IWorldClassDeclaration realWorld) throws XMLWriteException {

        setXMLAgentClass(xmlWorldClass, realWorld);

        xmlWorldClass.setUpdateBeforeAllObjects(realWorld
                .isUpdateBeforeAllObjects());
    }

    private void setXMLWorldInstance(XMLWorldInstance xmlWorldInstance,
            IWorldInstanceDeclaration worldInstanceDeclaration)
            throws XMLWriteException {

        setXMLObjectInstance(xmlWorldInstance, worldInstanceDeclaration);

        if (SpatialMapFeatureClassDeclaration
                .hasSpatialMapBuildinFeatureInstanceDeclaration(worldInstanceDeclaration)) {

            SpatialMapObjectFeatureInstDeclaration spatialMapBuildinObjectFeatureInstanceDeclaration = SpatialMapFeatureClassDeclaration
                    .getSpatialMapBuildinObjectFeatureInstanceDeclaration(worldInstanceDeclaration);

            ISpatialMapModel spatialMapModel = spatialMapBuildinObjectFeatureInstanceDeclaration
                    .getSpatialMapModel();

            XMLMapSettings xmlMapSettings = new XMLMapSettings();

            xmlMapSettings.setTorus(spatialMapModel.isTorus());

            XMLDimension mapSize = new XMLDimension();
            mapSize.setWidth(spatialMapModel.getWidth());
            mapSize.setHeight(spatialMapModel.getHeight());

            XMLDimension patchSize = new XMLDimension();
            patchSize.setWidth(spatialMapModel.getWidth()
                    / spatialMapModel.getGridCols());
            patchSize.setHeight(spatialMapModel.getHeight()
                    / spatialMapModel.getGridRows());

            xmlMapSettings.setMapSize(mapSize);
            xmlMapSettings.setPatchSize(patchSize);

            xmlMapSettings.setDiscrete(spatialMapModel.isDiscrete());
            xmlMapSettings.setDrawGrid(spatialMapModel.isDrawGrid());
            xmlMapSettings.setRotateImages(spatialMapModel.isRotateImages());
            if (spatialMapModel.getBackgroundImageFile() != null) {
                XMLImageFile xmlImageFile = new XMLImageFile();
                _xmlModelWriter.setXMLFile(xmlImageFile, spatialMapModel
                        .getBackgroundImageFile());
                xmlMapSettings.setBackgroundImage(xmlImageFile);
            }

            xmlWorldInstance.setMapSettings(xmlMapSettings);
        }
    }

    void setXMLObjectInstance(XMLObjectInstance xmlObjectInstance,
            IObjectInstanceDeclaration objectInstanceDeclaration)
            throws XMLWriteException {

        _xmlModelWriter.setXMLIdAndNameDocumentable(xmlObjectInstance,
                objectInstanceDeclaration);

        xmlObjectInstance
                .setObjectClassID(getXMLObjectClass(objectInstanceDeclaration
                        .getObjectClassDeclaration()));

        ISpatialInfo spatialInfo = SpatialInfoObjectFeatureHelper
                .getSpatialInfo(objectInstanceDeclaration);
        if (spatialInfo != null)
                xmlObjectInstance.setSpatialInfo(_xmlModelWriter
                        .createXMLSpatialInfo(spatialInfo));

        //write Features 12. march 2004
        Iterator features = objectInstanceDeclaration
                .getObjectFeatureInstanceDeclarationList().iterator();
        while (features.hasNext()) {
            IObjectFeatureInstanceDeclaration currentObjectFeatureInstanceDeclaration = (IObjectFeatureInstanceDeclaration) features
                    .next();

            if (currentObjectFeatureInstanceDeclaration instanceof IHasSpatialInfoDeclaration) {
                //do nothing
            } else {
                XMLFeatureInstance xmlFeature = createXMLObjectFeatureInstanceDeclaration(currentObjectFeatureInstanceDeclaration);

                if (xmlFeature != null)
                        xmlObjectInstance.addFeatureInstance(xmlFeature);
            }
        }

        Iterator it = objectInstanceDeclaration.getBodyInstanceDeclaration()
                .getBodyVariableInstanceDeclarationList().iterator();

        while (it.hasNext()) {
            IBodyVariableInstanceDeclaration currentBodyVariableInstanceDeclaration = (IBodyVariableInstanceDeclaration) it
                    .next();
            XMLBodyVariableInstance newBodyVariableInstance = new XMLBodyVariableInstance();
            newBodyVariableInstance
                    .setBodyVariableClassID(getXMLBodyVariableClass(currentBodyVariableInstanceDeclaration
                            .getBodyVariableClassDeclaration()));
            if (currentBodyVariableInstanceDeclaration
                    .getUsesOwnDefaulValueFunctionCallDeclaration()) {
                newBodyVariableInstance
                        .setDefaultValue(_xmlModelWriter
                                .createXMLFunctionCall(
                                        currentBodyVariableInstanceDeclaration,
                                        currentBodyVariableInstanceDeclaration
                                                .getDefaultValueFunctionCallDeclaration()));
                newBodyVariableInstance
                        .setUsesOwnDefaulValue(currentBodyVariableInstanceDeclaration
                                .getUsesOwnDefaulValueFunctionCallDeclaration());
            }
            xmlObjectInstance.addBodyVariableInstance(newBodyVariableInstance);
        }
    }

    private XMLReasoning createXMLReasoning(
            IReasoningEngineDeclaration reasoning) throws XMLWriteException {
        return new XMLModelReasoningActivityWriter(_xmlModelWriter)
                .createXMLReasoning((IActivityReasoningEngineDeclaration) reasoning);
    }

    XMLWorldInstance createXMLWorldInstance(
            IWorldInstanceDeclaration worldInstanceDeclaration)
            throws XMLWriteException {
        XMLWorldInstance xmlWorldInstance = new XMLWorldInstance();
        setXMLWorldInstance(xmlWorldInstance, worldInstanceDeclaration);
        return xmlWorldInstance;
    }
}
