package de.uniwue.ki.mas.xml.writer;
import java.awt.Color;
import java.awt.Point;
import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.fig.*;

import de.uniwue.ki.mas.basics.functionCall.FunctionCallDeclarationFactory;
import de.uniwue.ki.mas.basics.functionCall.IFunctionCallDeclaration;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.*;
import de.uniwue.ki.mas.behaviour.reasoning.IActivityReasoningEngineDeclaration;
import de.uniwue.ki.mas.behaviour.rule.*;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
* Insert the type's description here.
* Creation date: (07.02.2001 14:21:21)
* @author Christoph Oechslein
*/
class XMLModelReasoningActivityWriter {
	private final XMLModelWriter _xmlModelWriter;

	public XMLModelReasoningActivityWriter(XMLModelWriter xmlModelWriter) {
		super();

		_xmlModelWriter = xmlModelWriter;
	}

	private XMLActionCall createXMLActionCall(
		IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration,
		IFunctionCallDeclaration actionFunctionCallDeclaration) {

		XMLActionCall xmlActionCall = new XMLActionCall();

		xmlActionCall.setAction(
			_xmlModelWriter.createXMLFunctionCall(
				actionAtomicActivityDeclaration,
				actionFunctionCallDeclaration));

		return xmlActionCall;
	}

	private XMLActivityGraphNodeGroup createXMLActivityGraphNodeGroup(XMLAbstractActivityGraphNode xmlActivity) {

		XMLActivityGraphNodeGroup xmlActivityGraphNodeGroup = new XMLActivityGraphNodeGroup();

		if (xmlActivity instanceof XMLActionActivityGraphLeaf) {
			xmlActivityGraphNodeGroup.setActionActivity((XMLActionActivityGraphLeaf)xmlActivity);

		} else if (xmlActivity instanceof XMLActivityGraph) {
			xmlActivityGraphNodeGroup.setComposedActivity((XMLActivityGraph)xmlActivity);

		} else if (xmlActivity instanceof XMLDecisionGraphLeaf) {
			xmlActivityGraphNodeGroup.setDecisionActivity((XMLDecisionGraphLeaf)xmlActivity);

		} else if (xmlActivity instanceof XMLDocumentationGraphLeaf) {
			xmlActivityGraphNodeGroup.setDocumentationNode((XMLDocumentationGraphLeaf)xmlActivity);

		} else {
			Assert.fail();
		}

		return xmlActivityGraphNodeGroup;
	}

	private XMLAbstractActivityGraphLeaf createXMLActivityGraphLeaf(IActivityGraphLeafDeclaration atomicActivityDeclaration) {

		if (atomicActivityDeclaration instanceof IDecisionGraphLeafDeclaration) {

			XMLDecisionGraphLeaf xmlDecisionAtomicActivity = new XMLDecisionGraphLeaf();
			setXMLDecisionAtomicActivity(
				xmlDecisionAtomicActivity,
				(IDecisionGraphLeafDeclaration)atomicActivityDeclaration);
			return xmlDecisionAtomicActivity;

		} else if (atomicActivityDeclaration instanceof GraphDocumentationLeafDeclaration) {
			XMLDocumentationGraphLeaf xmlDocumentationGraphLeaf = new XMLDocumentationGraphLeaf();
			setXMLDocumentationGraphLeaf(
				xmlDocumentationGraphLeaf,
				(GraphDocumentationLeafDeclaration)atomicActivityDeclaration);
			return xmlDocumentationGraphLeaf;

		} else if (atomicActivityDeclaration instanceof IActionActivityGraphLeafDeclaration) {
			XMLActionActivityGraphLeaf xmlActionActivityGraphLeaf = new XMLActionActivityGraphLeaf();
			setXMLActionActivityGraphLeaf(
				xmlActionActivityGraphLeaf,
				(IActionActivityGraphLeafDeclaration)atomicActivityDeclaration);
			return xmlActionActivityGraphLeaf;

		} else {
			XMLActivityGraphLeaf XMLActivityGraphLeaf = new XMLActivityGraphLeaf();
			setXMLAbstractActivityGraphNode(XMLActivityGraphLeaf, atomicActivityDeclaration);
			return XMLActivityGraphLeaf;
		}
	}

	private XMLChainRule createXMLChainRule(IActivityGraphRuleDeclaration realRule)
		throws XMLWriteException {

		XMLChainRule chainRule = new XMLChainRule();
		setXMLActivityRule(chainRule, realRule);

		IActivityGraphNodeDeclaration fromActivityDeclaration =
			realRule.getFromActivityGraphNodeDeclaration();
		IActivityGraphNodeDeclaration toActivityDeclaration = realRule.getToActivityGraphNodeDeclaration();

		chainRule.setFromActivityID(getXMLActivity(fromActivityDeclaration));
		chainRule.setToActivityID(getXMLActivity(toActivityDeclaration));

		return chainRule;
	}

	private XMLActivityGraph createXMLActivityGraph(IActivityGraphDeclaration activityGraphDeclaration)
		throws XMLWriteException {

		XMLActivityGraph xmlActivityGraph = new XMLActivityGraph();

		setXMLActivityGraph(xmlActivityGraph, activityGraphDeclaration);

		return xmlActivityGraph;
	}

	private XMLActivityGraph createXMLActivityGraph(IActivityGraphComposedNodeDeclaration activityGraphComposedNodeDeclaration)
		throws XMLWriteException {

		XMLActivityGraph xmlActivityGraph = new XMLActivityGraph();

		setXMLActivityGraph(
			xmlActivityGraph,
			activityGraphComposedNodeDeclaration.getInnerActivityGraphDeclaration());
		setXMLAbstractActivityGraphNode(xmlActivityGraph, activityGraphComposedNodeDeclaration);
		xmlActivityGraph.setType(createXMLActivityTypeableType(activityGraphComposedNodeDeclaration));

		return xmlActivityGraph;
	}

	private void setXMLActivityGraph(
		XMLActivityGraph xmlActivityGraph,
		IActivityGraphDeclaration activityGraphDeclaration)
		throws XMLWriteException {

		_xmlModelWriter.setXMLIdAndNameDocumentable(xmlActivityGraph, activityGraphDeclaration);

		xmlActivityGraph.setEntryActivity(
			(XMLActivityGraphLeaf)getXMLActivityGraphLeaf(activityGraphDeclaration
				.getEntryActivityGraphLeafDeclaration()));
		xmlActivityGraph.setExitActivity(
			(XMLActivityGraphLeaf)getXMLActivityGraphLeaf(activityGraphDeclaration
				.getExitActivityGraphLeafDeclaration()));

		xmlActivityGraph.setEmergencyActivity(
			(XMLActivityGraphLeaf)getXMLActivityGraphLeaf(activityGraphDeclaration
				.getEmergencyActivityGraphLeafDeclaration()));

		xmlActivityGraphProcessActivities(xmlActivityGraph, activityGraphDeclaration);

		xmlActivityGraphProcessAllEdges(xmlActivityGraph, activityGraphDeclaration);
	}

	private XMLEmergencyActivityGraphRule createXMLEmergencyActivityGraphRule(IActivityGraphRuleDeclaration emergencyRuleDeclaration)
		throws XMLWriteException {

		XMLEmergencyActivityGraphRule xmlEmergencyActivityGraphRule = new XMLEmergencyActivityGraphRule();

		setXMLActivityRule(xmlEmergencyActivityGraphRule, emergencyRuleDeclaration);

		xmlEmergencyActivityGraphRule.setToActivityID(
			getXMLActivity(emergencyRuleDeclaration.getToActivityGraphNodeDeclaration()));

		return xmlEmergencyActivityGraphRule;
	}

	XMLReasoning createXMLReasoning(IActivityReasoningEngineDeclaration reasoning) throws XMLWriteException {
		XMLReasoning xmlReasoning = new XMLReasoning();

		XMLActivityGraph rootComposedActivity =
			getXMLActivityGraph(reasoning.getRootActivityGraphDeclaration());
		xmlReasoning.setComposedActivity(rootComposedActivity);

		return xmlReasoning;
	}

	private XMLAbstractActivityGraphNode getXMLActivity(IActivityGraphNodeDeclaration activityDeclaration)
		throws XMLWriteException {

		if (activityDeclaration instanceof IActivityGraphComposedNodeDeclaration) {
			return getXMLActivityGraph((IActivityGraphComposedNodeDeclaration)activityDeclaration);
		} else {
			return getXMLActivityGraphLeaf((IActivityGraphLeafDeclaration)activityDeclaration);
		}
	}

	private XMLAbstractActivityGraphLeaf getXMLActivityGraphLeaf(IActivityGraphLeafDeclaration atomicActivityDeclaration)
		throws XMLWriteException {

		if (!_xmlModelWriter.containsDeclaration(atomicActivityDeclaration)) {
			_xmlModelWriter.setObject(
				atomicActivityDeclaration,
				createXMLActivityGraphLeaf(atomicActivityDeclaration));
		}
		return (XMLAbstractActivityGraphLeaf)_xmlModelWriter.getObject(atomicActivityDeclaration);
	}

	private XMLActivityGraph getXMLActivityGraph(IActivityGraphDeclaration activityGraphDeclaration)
		throws XMLWriteException {

		if (!_xmlModelWriter.containsDeclaration(activityGraphDeclaration)) {
			_xmlModelWriter.setObject(
				activityGraphDeclaration,
				createXMLActivityGraph(activityGraphDeclaration));
		}
		return (XMLActivityGraph)_xmlModelWriter.getObject(activityGraphDeclaration);
	}

	private XMLActivityGraph getXMLActivityGraph(IActivityGraphComposedNodeDeclaration activityGraphComposedNodeDeclaration)
		throws XMLWriteException {

		if (!_xmlModelWriter.containsDeclaration(activityGraphComposedNodeDeclaration)) {
			_xmlModelWriter.setObject(
				activityGraphComposedNodeDeclaration,
				createXMLActivityGraph(activityGraphComposedNodeDeclaration));
		}
		return (XMLActivityGraph)_xmlModelWriter.getObject(activityGraphComposedNodeDeclaration);
	}

	private void setXMLDocumentationGraphLeaf(
		XMLDocumentationGraphLeaf xmlDocumentationGraphLeaf,
		GraphDocumentationLeafDeclaration activityDeclaration) {

		setXMLAbstractActivityGraphNode(xmlDocumentationGraphLeaf, activityDeclaration);
	}

	private void setXMLAbstractActivityGraphNode(
		XMLAbstractActivityGraphNode XMLAbstractActivityGraphNode,
		IActivityGraphNodeDeclaration activityDeclaration) {

		if (activityDeclaration instanceof IDocumentableDeclaration)
			_xmlModelWriter.setXMLIdAndNameDocumentable(
				XMLAbstractActivityGraphNode,
				(IDocumentableDeclaration)activityDeclaration);

		if (_xmlModelWriter.getFigMapper() != null) {

			IFigMapper mapper = _xmlModelWriter.getFigMapper();

			IFigObject figObject = mapper.getFigFor(activityDeclaration);

			if (figObject != null) {
				XMLAbstractActivityGraphNode.setX(figObject.getX());
				XMLAbstractActivityGraphNode.setY(figObject.getY());
				XMLAbstractActivityGraphNode.setWidth(figObject.getWidth());
				XMLAbstractActivityGraphNode.setHeight(figObject.getHeight());

				if (figObject instanceof AbstractFigNode) {

					AbstractFigNode absNode = (AbstractFigNode)figObject;
					Color color = absNode.getBackgroundColor();

					if (color != null) {

						XMLColor xmlColor = new XMLColor();
						xmlColor.setRed(color.getRed());
						xmlColor.setGreen(color.getGreen());
						xmlColor.setBlue(color.getBlue());

						XMLAbstractActivityGraphNode.setBackgroundColor(xmlColor);
					}
				}
			}
		}
	}

	private void setXMLActionActivityGraphLeaf(
		XMLActionActivityGraphLeaf xmlActionActivityGraphLeaf,
		IActionActivityGraphLeafDeclaration actionAtomicActivityDeclaration) {

		setXMLAbstractActivityGraphNode(xmlActionActivityGraphLeaf, actionAtomicActivityDeclaration);

		if (actionAtomicActivityDeclaration.isInstantly())
			xmlActionActivityGraphLeaf.setInstantly(actionAtomicActivityDeclaration.isInstantly());

		xmlActionActivityGraphLeaf.setType(createXMLActivityTypeableType(actionAtomicActivityDeclaration));

		if (!actionAtomicActivityDeclaration
			.getEntryFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.isEmpty()) {
			Iterator it =
				actionAtomicActivityDeclaration
					.getEntryFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList()
					.iterator();
			while (it.hasNext()) {
				xmlActionActivityGraphLeaf.addEntryActionRule(
					createXMLActionCall(
						actionAtomicActivityDeclaration,
						(IFunctionCallDeclaration)it.next()));
			}
		}

		if (!actionAtomicActivityDeclaration
			.getFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.isEmpty()) {
			Iterator it =
				actionAtomicActivityDeclaration
					.getFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList()
					.iterator();
			while (it.hasNext()) {
				xmlActionActivityGraphLeaf.addActionRule(
					createXMLActionCall(
						actionAtomicActivityDeclaration,
						(IFunctionCallDeclaration)it.next()));
			}
		}

		if (!actionAtomicActivityDeclaration
			.getExitFunctionActionCallDeclarationListAndFactory()
			.getFunctionActionCallDeclarationList()
			.isEmpty()) {
			Iterator it =
				actionAtomicActivityDeclaration
					.getExitFunctionActionCallDeclarationListAndFactory()
					.getFunctionActionCallDeclarationList()
					.iterator();
			while (it.hasNext()) {
				xmlActionActivityGraphLeaf.addExitActionRule(
					createXMLActionCall(
						actionAtomicActivityDeclaration,
						(IFunctionCallDeclaration)it.next()));
			}
		}
	}

	private void setXMLDecisionAtomicActivity(
		XMLDecisionGraphLeaf xmlDecisionAtomicActivity,
		IDecisionGraphLeafDeclaration DecisionAtomicActivityDeclaration) {

		setXMLAbstractActivityGraphNode(xmlDecisionAtomicActivity, DecisionAtomicActivityDeclaration);

	}

	private void xmlActivityGraphProcessActivities(
		XMLActivityGraph xmlActivityGraph,
		IActivityGraphDeclaration activityGraphDeclaration)
		throws XMLWriteException {

		Iterator it = activityGraphDeclaration.getActivityGraphNodeDeclarationsWithoutEmergencyAtAll();
		while (it.hasNext()) {

			xmlActivityGraphProcessActivityWithAdding(
				xmlActivityGraph,
				(IActivityGraphNodeDeclaration)it.next());
		}
	}

	private void xmlActivityGraphProcessActivityWithAdding(
		XMLActivityGraph xmlActivityGraph,
		IActivityGraphNodeDeclaration activityDeclaration)
		throws XMLWriteException {

		XMLAbstractActivityGraphNode xmlActivity = getXMLActivity(activityDeclaration);

		xmlActivityGraph.addXMLActivityGraphNodeGroup(createXMLActivityGraphNodeGroup(xmlActivity));
	}

	private void xmlActivityGraphProcessAllEdges(
		XMLActivityGraph xmlActivityGraph,
		IActivityGraphDeclaration activityGraphDeclaration)
		throws XMLWriteException {

		Iterator rules = activityGraphDeclaration.getActivityGraphEdgeDeclarationList().iterator();
		while (rules.hasNext()) {
			IActivityGraphEdgeDeclaration activityEdgeDeclaration =
				(IActivityGraphEdgeDeclaration)rules.next();

			if (activityEdgeDeclaration instanceof IActivityGraphRuleDeclaration) {

				if (activityEdgeDeclaration.getFromActivityGraphNodeDeclaration()
					== activityGraphDeclaration.getEmergencyActivityGraphLeafDeclaration()) {
					xmlActivityGraph.addEmergencyRule(
						createXMLEmergencyActivityGraphRule(
							(IActivityGraphRuleDeclaration)activityEdgeDeclaration));

				} else {
					XMLChainRule xmlChainRule =
						createXMLChainRule((IActivityGraphRuleDeclaration)activityEdgeDeclaration);

					xmlActivityGraph.addChainRule(xmlChainRule);
				}

			} else if (activityEdgeDeclaration instanceof IGraphDocumentEdgeDeclaration) {
				XMLDocumentActivityGraphEdge xmlChainRule =
					createXMLDocumentActivityGraphEdge(
						(IGraphDocumentEdgeDeclaration)activityEdgeDeclaration);

				xmlActivityGraph.addDocumentEdge(xmlChainRule);

			} else {
				XMLOtherwiseActivityGraphEdge xmlOtherwiseActivityGraphEdge =
					createXMLOtherwiseActivityGraphEdge(
						(IOtherwiseActivityGraphEdgeDeclaration)activityEdgeDeclaration);

				xmlActivityGraph.addOtherwiseChainRule(xmlOtherwiseActivityGraphEdge);
			}
		}
	}

	private String createXMLActivityTypeableType(IActivityGraphNodeDeclarationTypeable activityDeclarationTypeable) {
		if (activityDeclarationTypeable.getType() == IActivityGraphNodeDeclarationTypeable.ACTIVITY_LIKE)
			return null;
		else
			return activityDeclarationTypeable.getType().getName().toLowerCase();

	}

	private XMLOtherwiseActivityGraphEdge createXMLOtherwiseActivityGraphEdge(IOtherwiseActivityGraphEdgeDeclaration realRule)
		throws XMLWriteException {

		XMLOtherwiseActivityGraphEdge otherwiseChainRule = new XMLOtherwiseActivityGraphEdge();
		setXMLRuleControlPoints(otherwiseChainRule, realRule);

		IActivityGraphNodeDeclaration fromActivityDeclaration =
			realRule.getFromActivityGraphNodeDeclaration();
		IActivityGraphNodeDeclaration toActivityDeclaration = realRule.getToActivityGraphNodeDeclaration();

		otherwiseChainRule.setFromDecisionActivityID(getXMLActivity(fromActivityDeclaration));
		otherwiseChainRule.setToActivityID(getXMLActivity(toActivityDeclaration));

		return otherwiseChainRule;
	}

	private XMLDocumentActivityGraphEdge createXMLDocumentActivityGraphEdge(IGraphDocumentEdgeDeclaration realRule)
		throws XMLWriteException {

		XMLDocumentActivityGraphEdge docEdge = new XMLDocumentActivityGraphEdge();
		setXMLRuleControlPoints(docEdge, realRule);

		IActivityGraphNodeDeclaration fromActivityDeclaration =
			realRule.getFromActivityGraphNodeDeclaration();
		IActivityGraphNodeDeclaration toActivityDeclaration = realRule.getToActivityGraphNodeDeclaration();

		docEdge.setFromActivityID(getXMLActivity(fromActivityDeclaration));
		docEdge.setToActivityID(getXMLActivity(toActivityDeclaration));

		docEdge.setName(realRule.getName());

		return docEdge;
	}

	private void setXMLActivityRule(
		XMLAbstractRule xmlRule,
		IActivityGraphRuleDeclaration activityRuleDeclaration) {

		setXMLRuleControlPoints(xmlRule, activityRuleDeclaration);

		if (activityRuleDeclaration.getUsesShortDescription()) {
			xmlRule.setName(activityRuleDeclaration.getShortDescription());
		}

		String documentationString = activityRuleDeclaration.getDocumentation();

		if ((documentationString != null) && (documentationString.length() > 0)) {
			xmlRule.setDocumentation(XMLModelWriter.escapeDocumentationString(documentationString));
		}

		if (!FunctionCallDeclarationFactory
			.getInstance()
			.isDefaultAtomicFunctionCallDeclaration(
				activityRuleDeclaration.getConditionFunctionCallDeclaration())) {

			xmlRule.setPrecondition(
				_xmlModelWriter.createXMLFunctionCall(
					activityRuleDeclaration,
					activityRuleDeclaration.getConditionFunctionCallDeclaration()));
		}
	}

	private void setXMLRuleControlPoints(
		XMLAbstractActivityGraphEdge xmlGraphEdge,
		IActivityGraphEdgeDeclaration activityGraphEdgeDeclaration) {
		if (_xmlModelWriter.getFigMapper() != null) {

			IFigEdge figEdge =
				(IFigEdge)_xmlModelWriter.getFigMapper().getFigFor(activityGraphEdgeDeclaration);

			if (figEdge != null) {
				for (int i = 1; i < figEdge.getCtrlPointSize() - 1; i++) {

					Point pt = figEdge.getCtrlPointAt(i);
					XMLPosInteger xmlPos = new XMLPosInteger();

					xmlPos.setX(pt.x);
					xmlPos.setY(pt.y);

					xmlGraphEdge.addControlPoint(xmlPos);
				}
			}
		}
	}
}