package de.uniwue.ki.mas.xml.writer;
import java.io.File;
import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.mas.basics.feature.IFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.feature.spatial.*;
import de.uniwue.ki.mas.basics.feature.spatial.position.Position2DType;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.functionArgument.*;
import de.uniwue.ki.mas.basics.functionDeclaration.ISpecificFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.Argument;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.basics.type.composed.*;
import de.uniwue.ki.mas.basics.type.enum.*;
import de.uniwue.ki.mas.basics.type.user.IUserTypeDeclaration;
import de.uniwue.ki.mas.basics.type.user.UserTypeDeclarationList;
import de.uniwue.ki.mas.behaviour.activityGraphDeclaration.IActivityGraphNodeDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IObjectClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IBodyVariableClassDeclaration;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationDeclaration;
import de.uniwue.ki.mas.sim.simulation.ISimulationWithNameFunctionCallDeclDeclaration;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.xml.IDelaySettings;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;
import de.uniwue.ki.mas.xml.castor.v01.model.*;
import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.file.FileUtilities;
import de.uniwue.ki.util.list.IMutableListModel;

/**
 * Creation date: (07.02.2001 14:21:47)
 * 
 * @author Christoph Oechslein
 */
public class XMLModelTypeWriter {
	private final XMLModelWriter _xmlModelWriter;
	private final Map _userTypeMap = new HashMap();
	private static final List _xmlTypeWriters = new LinkedList();

	public static void addTypeWriter(IXMLTypeWriter typeWriter) {
		_xmlTypeWriters.add(typeWriter);
	}

	public XMLModelTypeWriter(XMLModelWriter xmlModelWriter) {
		super();

		_xmlModelWriter = xmlModelWriter;
	}

	String createRelativeFileStringIfPossible(File file) {
		return FileUtilities.createRelativeFileStringIfPossible(file,
				_xmlModelWriter.getSeSAmDeclaration().getFile());
	}

	XMLSpatialInfo createXMLSpatialInfo(ISpatialInfo spatialInfo) {
		XMLSpatialInfo xmlSpatialInfo = new XMLSpatialInfo();
		xmlSpatialInfo.setX(spatialInfo.getPosition().x);
		xmlSpatialInfo.setY(spatialInfo.getPosition().y);
		xmlSpatialInfo.setDirection(spatialInfo.getDirection());
		xmlSpatialInfo.setSpeed(spatialInfo.getSpeed());
		xmlSpatialInfo.setRadius(spatialInfo.getRadius());
		xmlSpatialInfo.setDrawPriority(spatialInfo.getDrawPriority());
		if (spatialInfo.getImagePath() != null)
			xmlSpatialInfo
					.setImagePath(createRelativeFileStringIfPossible(spatialInfo
							.getImagePath()));

		return xmlSpatialInfo;
	}

	XMLType createXMLType(ISpecificType type) throws XMLWriteException {
		XMLType xmlType = new XMLType();
		xmlType.setXMLTypeGroup(createXMLTypeGroup(type));
		return xmlType;
	}

	XMLTypeGroup createXMLTypeGroup(ISpecificType type)
			throws XMLWriteException {

		XMLTypeGroup xmlType = new XMLTypeGroup();

		if (VoidType.getInstance().equals(type))
			xmlType.setVoidType(new XMLVoidType());

		else if (BooleanType.getInstance().equals(type))
			xmlType.setBooleanType(new XMLBooleanType());

		else if (ObjectType.getInstance().equals(type))
			xmlType.setObjectType(new XMLObjectType());

		else if (DoubleType.getInstance().equals(type))
			xmlType.setDoubleType(new XMLDoubleType());

		else if (IntegerType.getInstance().equals(type))
			xmlType.setIntegerType(new XMLIntegerType());

		else if (SpatialInfoType.getInstance().equals(type))
			xmlType.setSpatialInfoType(new XMLSpatialInfoType());

		else if (Position2DType.getInstance().equals(type))
			xmlType.setPosition2DType(new XMLPosition2DType());

		else if (StringType.getInstance().equals(type))
			xmlType.setStringType(new XMLStringType());

		else if (ImageType.getInstance().equals(type))
			xmlType.setImageType(new XMLImageFileType());

		else if (VoidType.getInstance().equals(type))
			xmlType.setVoidType(new XMLVoidType());

		else if (ActivityType.getInstance().equals(type))
			xmlType.setActivityType(new XMLActivityType());

		else if (ObjectClassDeclarationType.getInstance().equals(type))
			xmlType.setObjectClassType(new XMLObjectClassType());

		else if (FeatureClassDeclarationType.getInstance().equals(type))
			xmlType.setFeatureClassType(new XMLFeatureClassType());

		else if (ObjectInstanceDeclarationType.getInstance().equals(type))
			xmlType.setObjectInstanceType(new XMLObjectInstanceType());

		else if (GenericIteratorType.getInstance().canCreate(type)) {
			SpecificIteratorType specificIteratorType = (SpecificIteratorType) type;
			XMLIteratorType xmlIteratorType = new XMLIteratorType();
			xmlIteratorType
					.setXMLTypeGroup(createXMLTypeGroup(specificIteratorType
							.getIteratorSpecificType()));
			xmlType.setIteratorType(xmlIteratorType);

		} else if (GenericListType.getInstance().canCreate(type)) {
			SpecificListType specificListType = (SpecificListType) type;
			XMLListType xmlListType = new XMLListType();
			xmlListType.setXMLTypeGroup(createXMLTypeGroup(specificListType
					.getIteratorSpecificType()));
			xmlType.setListType(xmlListType);

		} else if (GenericHashtableType.getInstance().canCreate(type)) {
			SpecificHashtableType specificHashtableType = (SpecificHashtableType) type;
			XMLHashTableType xmlHashTableType = new XMLHashTableType();

			xmlHashTableType.setKeyType(createXMLType(specificHashtableType
					.getKeyType()));
			xmlHashTableType.setValueType(createXMLType(specificHashtableType
					.getValueType()));

			xmlType.setHashtableType(xmlHashTableType);

		} else if (type instanceof SpecificVariableClassDeclarationType) {
			SpecificVariableClassDeclarationType specificVariableClassDeclarationType = (SpecificVariableClassDeclarationType) type;
			XMLVariableType variableType = new XMLVariableType();
			variableType.setExternal(specificVariableClassDeclarationType
					.isExternal());
			variableType.setWriteable(specificVariableClassDeclarationType
					.isWriteable());
			variableType
					.setXMLTypeGroup(createXMLTypeGroup(specificVariableClassDeclarationType
							.getVariableClassDeclarationType()));
			xmlType.setVariableType(variableType);

		} else if (type instanceof SpecificFunctionType) {
			SpecificFunctionType specificFunctionType = (SpecificFunctionType) type;
			XMLFunctionArgListType functionArgListType = new XMLFunctionArgListType();

			functionArgListType
					.setXMLTypeGroup(createXMLTypeGroup(specificFunctionType
							.getSpecificOutputType()));

			XMLFunctionArgListInputTypes xmlFunctionArgListInputTypes = new XMLFunctionArgListInputTypes();
			Iterator it = specificFunctionType.getSpecificInputTypes();
			while (it.hasNext()) {
				xmlFunctionArgListInputTypes
						.addXMLTypeGroup(createXMLTypeGroup(((ISpecificType) it
								.next())));
			}

			functionArgListType.setInputTypes(xmlFunctionArgListInputTypes);

			xmlType.setFunctionArgListType(functionArgListType);

		} else if (type instanceof SpecificFunctionUnlimitedType) {
			SpecificFunctionUnlimitedType specificFunctionUnlimitedType = (SpecificFunctionUnlimitedType) type;
			XMLFunctionArgRestType functionArgRestType = new XMLFunctionArgRestType();

			functionArgRestType
					.setXMLTypeGroup(createXMLTypeGroup(specificFunctionUnlimitedType
							.getSpecificOutputType()));

			XMLType inputType = new XMLType();
			inputType
					.setXMLTypeGroup(createXMLTypeGroup(specificFunctionUnlimitedType
							.getSpecificInputType()));
			functionArgRestType.setInputType(inputType);
			xmlType.setFunctionArgRestType(functionArgRestType);

		} else if (GenericSimObjectType.getInstance().canCreate(type)) {
			final XMLSimObjectType xmlSimObjectType = new XMLSimObjectType();

			final ISimObjectType simObjectType = (ISimObjectType) type;

			_xmlModelWriter.addDelaySettings(new IDelaySettings() {
				public void doSetting() throws XMLWriteException {
					xmlSimObjectType.setId(getSimObjectTypeId(simObjectType));
				}
			});

			xmlType.setSimObjectType(xmlSimObjectType);

		} else if (SimulationDeclarationType.getInstance().equals(type)) {
			xmlType.setSimulationType(new XMLSimulationType());

		} else if (SimulationType.getInstance().equals(type)) {
			xmlType.setSimulationRunType(new XMLSimulationRunType());

		} else if (GenericComposedType.getInstance().canCreate(type)) {
			XMLUserTypeID xmlComposedTypeID = new XMLUserTypeID();

			if (type instanceof SpecificComposedType)
				xmlComposedTypeID
						.setId(getXMLComposedType((SpecificComposedType) type));
			else
				xmlComposedTypeID.setSystemID(((ISpecificComposedType) type)
						.getName());

			xmlType.setUserType(xmlComposedTypeID);

		} else if (GenericEnumType.getInstance().canCreate(type)) {
			XMLUserTypeID xmlComposedTypeID = new XMLUserTypeID();

			if (type instanceof SpecificEnumType)
				xmlComposedTypeID
						.setId(getXMLEnumType((SpecificEnumType) type));
			else
				xmlComposedTypeID.setSystemID(((ISpecificEnumType) type)
						.getName());

			xmlType.setUserType(xmlComposedTypeID);

		} else if (GenericComposedTypeAttributeType.getInstance().canCreate(
				type)) {
			SpecificComposedTypeAttributeType typeAttributeType = (SpecificComposedTypeAttributeType) type;
			XMLComposedTypeAttributeType xmlComposedTypeAttributeType = new XMLComposedTypeAttributeType();

			if (typeAttributeType.getComposedType() instanceof SpecificComposedType)
				xmlComposedTypeAttributeType
						.setComposedTypeID(getXMLComposedType(typeAttributeType
								.getComposedType()));
			else
				xmlComposedTypeAttributeType
						.setComposedTypeName(typeAttributeType
								.getComposedType().getName());

			try {
				xmlComposedTypeAttributeType
						.setIndex(SpecificComposedTypeAttributeType
								.getAttributValueIndex(typeAttributeType
										.getComposedType(), typeAttributeType
										.getComposedTypeAttribute()));

			} catch (AttributeCompilingException e) {
				throw new XMLWriteException(e);
			}

			if (typeAttributeType.isWriteable())
				xmlComposedTypeAttributeType.setWriteable(true);

			xmlType.setComposedTypeAttributeType(xmlComposedTypeAttributeType);

		} else {
			//try additional TypeWriters from Plugins
			Iterator it = _xmlTypeWriters.iterator();
			boolean typeWriterFound = false;
			while (it.hasNext()) {
				IXMLTypeWriter typeWriter = (IXMLTypeWriter) it.next();
				if (typeWriter.isResponsibleFor(type)) {
					xmlType = typeWriter.createXMLTypeGroup(this, type);
					typeWriterFound = true;
				}
			}

			if (!typeWriterFound) {
				XMLUnknownType xmlUnknownType = new XMLUnknownType();

				xmlUnknownType.setName(type.getName());
				//xmlUnknownType.setSettings();

				xmlType.setUnknownType(xmlUnknownType);
			}
		}

		return xmlType;
	}

	XMLTypeValueGroup createXMLTypeValueGroup(
			IArgumentDeclaration argumentDeclaration) throws XMLWriteException {

		if (argumentDeclaration instanceof IFunctionArgumentDeclaration) {
			XMLFunctionParameter xmlFunctionParameter = (XMLFunctionParameter) _xmlModelWriter
					.getObject(argumentDeclaration);

			XMLFunctionParameterID xmlFunctionParameterID = new XMLFunctionParameterID();
			xmlFunctionParameterID.setId(xmlFunctionParameter);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setParameterID(xmlFunctionParameterID);
			return xmlTypeValue;

		} else {

			return createXMLTypeValueGroup(argumentDeclaration
					.getSpecificType(), argumentDeclaration.getObject());
		}
	}

	XMLTypeValueGroup createXMLTypeValueGroup(ISpecificType type, Object value)
			throws XMLWriteException {

		if (type.equals(BooleanType.getInstance())) {
			XMLBoolean xmlBoolean = new XMLBoolean();
			xmlBoolean.setValue(((Boolean) value).booleanValue());

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setBoolean(xmlBoolean);
			return xmlTypeValue;
		}

		if (type.equals(DoubleType.getInstance())) {
			XMLDouble xmlDouble = new XMLDouble();
			xmlDouble.setValue(((Number) value).doubleValue());

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setDouble(xmlDouble);
			return xmlTypeValue;
		}

		if (type.equals(IntegerType.getInstance())) {
			XMLInteger xmlInt = new XMLInteger();
			xmlInt.setValue(((Number) value).intValue());

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setInteger(xmlInt);
			return xmlTypeValue;
		}

		if (type.equals(StringType.getInstance())) {
			XMLString xmlString = new XMLString();
			xmlString.setValue((String) value);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setString(xmlString);
			return xmlTypeValue;
		}

		if (type.equals(ImageType.getInstance())) {
			XMLImageFile xmlImageFile = new XMLImageFile();

			if (value instanceof File)
				setXMLFile(xmlImageFile, (File) value);
			else if (value != null)
				setXMLFile(xmlImageFile, new File((String) value));

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setImage(xmlImageFile);
			return xmlTypeValue;
		}

		if (type.equals(SpatialInfoType.getInstance())) {
			XMLSpatialInfo xmlSpatialInfo = createXMLSpatialInfo((ISpatialInfo) value);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setSpatialInfo(xmlSpatialInfo);
			return xmlTypeValue;
		}

		if (type.equals(Position2DType.getInstance())) {
			Position2D pos = (Position2D) value;
			XMLPosition2D xmlPosition2D = new XMLPosition2D();
			xmlPosition2D.setX(pos.x);
			xmlPosition2D.setY(pos.y);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setPosition2D(xmlPosition2D);
			return xmlTypeValue;
		}

		if (type.equals(ActivityType.getInstance())) {
			XMLActivityID xmlActivityID = new XMLActivityID();

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setActivityID(xmlActivityID);

			if (value != null) {
				XMLAbstractActivityGraphNode xmlActivity = (XMLAbstractActivityGraphNode) _xmlModelWriter
						.getObject((IActivityGraphNodeDeclaration) value);
				xmlActivityID.setId(xmlActivity);
			}

			return xmlTypeValue;
		}

		if (type.equals(SimulationDeclarationType.getInstance())) {
			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();

			ISimulationDeclaration simulationDeclaration = (ISimulationDeclaration) value;
			if (simulationDeclaration instanceof ISimulationWithNameFunctionCallDeclDeclaration) {
				Assert.assertFalse(_xmlModelWriter.getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getSimulationDeclarationListAndFactory()
						.getSimulationDeclarationList().contains(
								simulationDeclaration));

				xmlTypeValue
						.setSimulation(_xmlModelWriter
								.createXMLSimulation((ISimulationWithNameFunctionCallDeclDeclaration) simulationDeclaration));

			} else {
				XMLSimulationID xmlSimulationID = new XMLSimulationID();

				if (value != null) {
					XMLSimulation xmlSimulation = (XMLSimulation) _xmlModelWriter
							.getObject(simulationDeclaration);
					xmlSimulationID.setId(xmlSimulation);
				}

				xmlTypeValue.setSimulationID(xmlSimulationID);
			}

			return xmlTypeValue;
		}

		if (type.equals(SimulationType.getInstance())) {
			Assert.assertNull(value);
			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setSimulationRun(new XMLSimulationRun());
			return xmlTypeValue;
		}

		if (type.equals(ObjectClassDeclarationType.getInstance())) {
			XMLAbstractObjectClass xmlAgentClass = (XMLAbstractObjectClass) _xmlModelWriter
					.getObject((IObjectClassDeclaration) value);
			XMLObjectClassID xmlObjectClassID = new XMLObjectClassID();
			xmlObjectClassID.setId(xmlAgentClass);
			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setObjectClassID(xmlObjectClassID);
			return xmlTypeValue;
		}

		if (type.equals(FeatureClassDeclarationType.getInstance())) {

			XMLFeatureClassID xmlFeatureClassID = new XMLFeatureClassID();

			if (value instanceof IUserFeatureClassDeclaration) {
				xmlFeatureClassID.setId(_xmlModelWriter
						.getObject((IUserFeatureClassDeclaration) value));
			} else {
				xmlFeatureClassID.setBuildin(((IFeatureClassDeclaration) value)
						.getName());
			}

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setFeatureClassID(xmlFeatureClassID);
			return xmlTypeValue;
		}

		if (type.equals(ObjectInstanceDeclarationType.getInstance())) {
			final IObjectInstanceDeclaration objectInstanceDeclaration = (IObjectInstanceDeclaration) value;

			final XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			final XMLModelWriter xmlModelWriter = _xmlModelWriter;

			if (xmlModelWriter.containsDeclaration(objectInstanceDeclaration)) {
				XMLObjectInstanceID xmlInstanceID = new XMLObjectInstanceID();
				xmlInstanceID.setId(xmlModelWriter
						.getXMLObjectInstance(objectInstanceDeclaration));
				xmlTypeValue.setObjectInstanceID(xmlInstanceID);

			} else {
				IDelaySettings delaySettings = new IDelaySettings() {
					public void doSetting() throws XMLWriteException {
						if (xmlModelWriter
								.containsDeclaration(objectInstanceDeclaration)) {
							XMLObjectInstanceID xmlInstanceID = new XMLObjectInstanceID();
							xmlInstanceID
									.setId(xmlModelWriter
											.getXMLObjectInstance(objectInstanceDeclaration));
							xmlTypeValue.setObjectInstanceID(xmlInstanceID);

						} else {
							xmlTypeValue
									.setObjectInstance(xmlModelWriter
											.getXMLObjectInstance(objectInstanceDeclaration));
						}
					}
				};

				_xmlModelWriter.addDelaySettings(delaySettings);
			}

			return xmlTypeValue;
		}

		if (GenericListType.getInstance().canCreate(type)) {
			ISpecificType listType;
			if (type == ObjectType.getInstance())
				listType = ObjectType.getInstance();
			else
				listType = ((SpecificListType) type).getListSpecificType();

			XMLList xmlList = new XMLList();

			xmlList.setXMLTypeGroup(createXMLTypeGroup(listType));

			if (value != null) {
				Iterator it;
				if (value instanceof Argument)
					it = ((List) ((Argument) value).getConstantValue())
							.iterator();
				else
					it = ((Collection) value).iterator();
				while (it.hasNext()) {
					xmlList.addXMLTypeValueGroup(createXMLTypeValueGroup(
							listType, it.next()));
				}
			}

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setList(xmlList);
			return xmlTypeValue;
		}

		if (GenericIteratorType.getInstance().canCreate(type)) {
			ISpecificType iteratorType;
			if (type == ObjectType.getInstance())
				iteratorType = ObjectType.getInstance();
			else
				iteratorType = ((SpecificIteratorType) type)
						.getIteratorSpecificType();

			XMLIterator xmlIterator = new XMLIterator();

			xmlIterator.setXMLTypeGroup(createXMLTypeGroup(iteratorType));

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setIterator(xmlIterator);
			return xmlTypeValue;
		}

		if (type instanceof SpecificHashtableType) {
			XMLHashTable xmlHashTable = createXMLHashTable(
					(SpecificHashtableType) type, value);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setHashtable(xmlHashTable);

			return xmlTypeValue;
		}

		if (type.equals(VoidType.getInstance())) {
			XMLVoid xmlVoid = new XMLVoid();

			xmlVoid.setValue(((ISpecificFunctionDeclaration) value).getName());

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setVoid(xmlVoid);
			return xmlTypeValue;
		}

		if (GenericSimObjectType.getInstance().canCreate(type)) {
			final XMLSimObject xmlSimObject = new XMLSimObject();

			final ISimObjectType simObjectType = (ISimObjectType) type;

			_xmlModelWriter.addDelaySettings(new IDelaySettings() {
				public void doSetting() throws XMLWriteException {
					xmlSimObject.setId(getSimObjectTypeId(simObjectType));
				}
			});

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setSimObject(xmlSimObject);

			return xmlTypeValue;
		}

		if (type.equals(ObjectType.getInstance())) {
			XMLObject xmlObject = new XMLObject();

			Assert.assertNull("Cannot write Object value other than null",
					value);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setObject(xmlObject);
			return xmlTypeValue;
		}

		if (type instanceof SpecificVariableClassDeclarationType) {
			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setVariableID(createXMLVariableID(
					(SpecificVariableClassDeclarationType) type,
					(IDeclaration) value));

			return xmlTypeValue;
		}

		if (type instanceof ISpecificFunctionType) {
			if (value instanceof SpecificFunctionDeclarationWithArgDeclBinds) {
				SpecificFunctionDeclarationWithArgDeclBinds specificFunctionDeclarationWithArgDeclBinds = (SpecificFunctionDeclarationWithArgDeclBinds) value;

				XMLUnnamedParametricFunction xmlUnnamedParametricFunction = createXMLUnnamedParametricFunction(
						specificFunctionDeclarationWithArgDeclBinds,
						specificFunctionDeclarationWithArgDeclBinds
								.getParametricFunctionCallDeclaration());

				XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
				xmlTypeValue
						.setParametricFunction(xmlUnnamedParametricFunction);

				return xmlTypeValue;

			} else {
				XMLFunction xmlFunction = new XMLFunction();

				ISpecificFunctionDeclaration specificFunctionDeclaration = (ISpecificFunctionDeclaration) value;

				xmlFunction.setName(specificFunctionDeclaration.getName());
				xmlFunction.setXMLTypeGroup(createXMLTypeGroup(type));

				XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
				xmlTypeValue.setFunction(xmlFunction);

				return xmlTypeValue;
			}
		}

		if (GenericComposedType.getInstance().canCreate(type)) {
			ISpecificComposedType composedType = (ISpecificComposedType) type;

			XMLComposedTypeValue composedTypeValue = new XMLComposedTypeValue();
			if (composedType instanceof SpecificComposedType)
				composedTypeValue
						.setComposedTypeID(getXMLComposedType(composedType));
			else
				composedTypeValue.setComposedTypeName(composedType.getName());

			Assert.assertNull("Can only write null value", value);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setComposed(composedTypeValue);

			return xmlTypeValue;
		}

		if (GenericComposedTypeAttributeType.getInstance().canCreate(type)) {
			SpecificComposedTypeAttributeType typeAttributeType = (SpecificComposedTypeAttributeType) type;

			XMLComposedTypeAttributeValue xmlComposedTypeAttribute = new XMLComposedTypeAttributeValue();

			if (typeAttributeType.getComposedType() instanceof SpecificComposedType)
				xmlComposedTypeAttribute
						.setComposedTypeID(getXMLComposedType(typeAttributeType
								.getComposedType()));
			else
				xmlComposedTypeAttribute.setComposedTypeName(typeAttributeType
						.getComposedType().getName());

			try {
				xmlComposedTypeAttribute
						.setIndex(SpecificComposedTypeAttributeType
								.getAttributValueIndex(typeAttributeType
										.getComposedType(), typeAttributeType
										.getComposedTypeAttribute()));

			} catch (AttributeCompilingException e) {
				throw new XMLWriteException(e);
			}

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setComposedAttribute(xmlComposedTypeAttribute);

			return xmlTypeValue;
		}

		if (GenericEnumType.getInstance().canCreate(type)) {
			ISpecificEnumType enumType = (ISpecificEnumType) type;

			XMLEnumTypeValue xmlEnumTypeValue = new XMLEnumTypeValue();

			if (enumType instanceof SpecificEnumType)
				xmlEnumTypeValue
						.setEnumTypeID(getXMLEnumType((SpecificEnumType) enumType));
			else
				xmlEnumTypeValue.setEnumTypeName(enumType.getName());

			int index = enumType.getItemList().indexOf(value);

			Assert.assertTrue("Enum item '" + ((EnumItem) value).getName()
					+ "' in type '" + enumType.getName() + "' not availible!",
					index != -1);

			xmlEnumTypeValue.setIndex(index);

			XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
			xmlTypeValue.setEnum(xmlEnumTypeValue);

			return xmlTypeValue;
		}

		//check if any valid TypeWriters are there
		Iterator it = _xmlTypeWriters.iterator();
		while (it.hasNext()) {
			IXMLTypeWriter typeWriter = (IXMLTypeWriter) it.next();
			if (typeWriter.isResponsibleFor(type)) {
				return typeWriter.createXMLTypeValueGroup(type, value);
			}
		}

		XMLUnknownTypeValue xmlUnknownTypeValue = new XMLUnknownTypeValue();
		XMLUnknownType xmlUnknownType = new XMLUnknownType();
		xmlUnknownType.setName(type.getName());
		xmlUnknownTypeValue.setUnknownType(xmlUnknownType);
		XMLTypeValueGroup xmlTypeValue = new XMLTypeValueGroup();
		xmlTypeValue.setUnknown(xmlUnknownTypeValue);

		return xmlTypeValue;
	}

	private XMLRootObject getSimObjectTypeId(ISimObjectType simObjectType)
			throws XMLWriteException {
		if (simObjectType.getObjectClassDeclaration() != null)
			return _xmlModelWriter.getObject(simObjectType
					.getObjectClassDeclaration());
		else if (simObjectType.getFeatureClassDeclaration() != null)
			return _xmlModelWriter.getObject(simObjectType
					.getFeatureClassDeclaration());
		else
			return null;
	}

	private XMLHashTable createXMLHashTable(
			SpecificHashtableType specificHashtableType, Object value)
			throws XMLWriteException {
		XMLHashTable xmlHashTable = new XMLHashTable();

		xmlHashTable.setKeyType(createXMLType(specificHashtableType
				.getKeyType()));
		xmlHashTable.setValueType(createXMLType(specificHashtableType
				.getValueType()));

		if (value != null) {
			Iterator it = ((Map) value).entrySet().iterator();

			while (it.hasNext()) {
				Map.Entry entry = (Map.Entry) it.next();

				XMLTypeValue xmlKeyTypeValue = new XMLTypeValue();
				xmlKeyTypeValue.setXMLTypeValueGroup(createXMLTypeValueGroup(
						specificHashtableType.getKeyType(), entry.getKey()));

				XMLTypeValue xmlValueTypeValue = new XMLTypeValue();
				xmlValueTypeValue
						.setXMLTypeValueGroup(createXMLTypeValueGroup(
								specificHashtableType.getValueType(), entry
										.getValue()));

				XMLHashTableEntry xmlHashTableEntry = new XMLHashTableEntry();

				xmlHashTableEntry.setKey(xmlKeyTypeValue);
				xmlHashTableEntry.setValue(xmlValueTypeValue);

				xmlHashTable.addEntry(xmlHashTableEntry);

			}
		}
		return xmlHashTable;
	}

	private XMLUnnamedParametricFunction createXMLUnnamedParametricFunction(
			Object outerObject,
			IParametricFunctionCallDeclaration parametricFunctionCallDeclaration)
			throws XMLWriteException {

		XMLUnnamedParametricFunction xmlUnnamedParametricFunction = new XMLUnnamedParametricFunction();

		_xmlModelWriter
				.setXMLUnnamedParametricFunction(outerObject,
						xmlUnnamedParametricFunction,
						parametricFunctionCallDeclaration);

		return xmlUnnamedParametricFunction;
	}

	private XMLVariableID createXMLVariableID(
			SpecificVariableClassDeclarationType specificVariableClassDeclarationType,
			IDeclaration value) throws XMLWriteException {

		XMLVariableID xmlVariableID = new XMLVariableID();

		if (value != null) {

			if (!_xmlModelWriter.containsDeclaration(value))
				throw new XMLWriteException("Variable '"
						+ ((IDeclaration) value).getName()
						+ "' does not exists: ");

			XMLRootObject xmlRootObject = _xmlModelWriter.getObject(value);

			xmlVariableID.setId(xmlRootObject);

			IVariableClassDeclaration variableClassDeclaration;

			if (value instanceof IBodyVariableClassDeclaration) {
				variableClassDeclaration = ((IBodyVariableClassDeclaration) value)
						.getVariableClassDeclaration();

			} else {
				if (Assert.ASSERTION_ON)
					Assert
							.assertTrue(value instanceof IVariableClassDeclaration);

				variableClassDeclaration = (IVariableClassDeclaration) value;
			}

			if (!variableClassDeclaration.getSpecificType().equals(
					specificVariableClassDeclarationType
							.getParameterSpecificType())) {
				xmlVariableID
						.setXMLTypeGroup(createXMLTypeGroup(specificVariableClassDeclarationType
								.getParameterSpecificType()));
			}

			if (variableClassDeclaration.isWriteable() != specificVariableClassDeclarationType
					.isWriteable()) {
				xmlVariableID.setWriteable(specificVariableClassDeclarationType
						.isWriteable());
			}

			if (variableClassDeclaration.isExternal() != specificVariableClassDeclarationType
					.isExternal()) {
				xmlVariableID.setExternal(specificVariableClassDeclarationType
						.isExternal());
			}

		} else {
			xmlVariableID.setWriteable(specificVariableClassDeclarationType
					.isWriteable());
			xmlVariableID.setExternal(specificVariableClassDeclarationType
					.isExternal());
			xmlVariableID
					.setXMLTypeGroup(createXMLTypeGroup(specificVariableClassDeclarationType
							.getParameterSpecificType()));
		}

		return xmlVariableID;
	}

	void setXMLFile(XMLFile xmlFile, File file) {
		xmlFile.setValue(createRelativeFileStringIfPossible(file));
	}

	XMLUserType[] createXMLUserTypeList(
			UserTypeDeclarationList realUserTypeDeclarationList)
			throws XMLWriteException {

		XMLUserType[] userTypes = new XMLUserType[realUserTypeDeclarationList
				.size()];
		for (int i = 0; i < realUserTypeDeclarationList.size(); i++) {
			userTypes[i] = getXMLUserType((IUserTypeDeclaration) realUserTypeDeclarationList
					.get(i));
		}
		return userTypes;
	}

	XMLUserType getXMLUserType(IUserTypeDeclaration userTypeDeclaration)
			throws XMLWriteException {

		XMLUserType _userType;

		if (userTypeDeclaration.isComposedType()) {
			_userType = getXMLComposedType(userTypeDeclaration
					.getSpecificComposedType());

		} else {
			_userType = getXMLEnumType(userTypeDeclaration
					.getSpecificEnumType());
		}

		_xmlModelWriter.setXMLIdAndNameDocumentable(_userType,
				userTypeDeclaration);

		return _userType;
	}

	XMLUserType getXMLComposedType(ISpecificComposedType composedType)
			throws XMLWriteException {

		if (!_userTypeMap.containsKey(composedType)) {
			XMLUserType newXMLUserType = new XMLUserType();
			// first put the insert attributes due to recursive definitions
			_userTypeMap.put(composedType, newXMLUserType);
			setXMLUserType(composedType, newXMLUserType);
		}

		return (XMLUserType) _userTypeMap.get(composedType);
	}

	XMLUserType getXMLEnumType(SpecificEnumType enumType)
			throws XMLWriteException {

		if (!_userTypeMap.containsKey(enumType)) {
			XMLUserType newUserType = createXMLUserType(enumType);
			_userTypeMap.put(enumType, newUserType);
		}

		return (XMLUserType) _userTypeMap.get(enumType);
	}

	private XMLUserType createXMLUserType(SpecificEnumType specificEnumType)
			throws XMLWriteException {
		XMLUserType xmlUserType;
		xmlUserType = new XMLUserType();
		xmlUserType.setType("enum");

		IMutableListModel list = specificEnumType.getItemListAndFactory()
				.getMutableListModel();

		for (int i = 0; i < list.size(); i++) {
			EnumItem enumItem = (EnumItem) list.get(i);

			XMLEnumTypeItem xmlEnumTypeItem = new XMLEnumTypeItem();

			xmlEnumTypeItem.setName(enumItem.getName());

			xmlUserType.addItem(xmlEnumTypeItem);
		}

		return xmlUserType;
	}

	private void setXMLUserType(ISpecificComposedType composedType,
			XMLUserType xmlUserType) throws XMLWriteException {

		xmlUserType.setType("composed");
		XMLUserTypeID father = new XMLUserTypeID();
		xmlUserType.setFather(null);

		if (composedType.getFather() != null) {
			if (CollectionsHelper.contains(TypeManager.getInstance()
					.getSystemComposedTypes(), composedType.getFather()))
				father.setSystemID(composedType.getFather().getName());
			else
				father.setId(getXMLComposedType(composedType.getFather()));
			//vergleiche:
			// xmlComposedTypeID.setId(getXMLComposedType((SpecificComposedType)
			// type));
			xmlUserType.setFather(father);
		}

		Iterator it = composedType.getOwnAttributes();

		while (it.hasNext()) {
			IComposedTypeAttribute element = (IComposedTypeAttribute) it.next();

			XMLComposedTypeAttribute xmlComposedTypeAttribute = new XMLComposedTypeAttribute();

			xmlComposedTypeAttribute.setId(_xmlModelWriter
					.getXMLUniqueId(element));
			xmlComposedTypeAttribute.setName(element.getAttributeName());
			if ((element.getDocumentation() != null)
					&& (element.getDocumentation().length() > 0)) {

				xmlComposedTypeAttribute.setDocumentation(XMLModelWriter
						.escapeDocumentationString(element.getDocumentation()));
			}
			xmlComposedTypeAttribute.setXMLTypeGroup(createXMLTypeGroup(element
					.getAttributeType()));

			if (!element.isWriteable())
				xmlComposedTypeAttribute.setWriteable(false);

			if (Assert.ASSERTION_ON) {
				Assert.assertNotNull(xmlUserType);
			}

			xmlUserType.addAttribute(xmlComposedTypeAttribute);
		}
	}
}