package de.uniwue.ki.mas.xml.writer;
import java.io.*;
import java.util.Iterator;

import junit.framework.Assert;
import mygale.mcge.fig.IFigMapper;
import mygale.mcge.fig.IFigObject;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.ValidationException;

import de.uniwue.ki.mas.basics.action.debugging.FunctionActionDebuggingDeclarationFactory;
import de.uniwue.ki.mas.basics.feature.spatial.ISpatialInfo;
import de.uniwue.ki.mas.basics.feature.user.IUserFeatureClassDeclaration;
import de.uniwue.ki.mas.basics.featureList.IFeatureClassDeclarationList;
import de.uniwue.ki.mas.basics.functionArgument.IParametricFunctionCallDeclaration;
import de.uniwue.ki.mas.basics.functionCall.*;
import de.uniwue.ki.mas.basics.functionDeclaration.FunctionDeclarationCompiler;
import de.uniwue.ki.mas.basics.functionUser.IUserFunctionDeclaration;
import de.uniwue.ki.mas.basics.type.*;
import de.uniwue.ki.mas.basics.type.argument.IArgumentDeclaration;
import de.uniwue.ki.mas.entity.object.IObjectInstanceDeclaration;
import de.uniwue.ki.mas.entity.objectclass.IRessourceClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.RessourceClassDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.agentclass.AgentClassDeclarationList;
import de.uniwue.ki.mas.entity.objectclass.agentclass.IAgentClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.IWorldClassDeclaration;
import de.uniwue.ki.mas.entity.objectclass.worldclass.WorldClassDeclarationList;
import de.uniwue.ki.mas.entity.variableclass.IVariableClassDeclaration;
import de.uniwue.ki.mas.model.*;
import de.uniwue.ki.mas.model.graph.*;
import de.uniwue.ki.mas.sim.analysis.*;
import de.uniwue.ki.mas.sim.experiment.ExperimentDeclarationList;
import de.uniwue.ki.mas.sim.experiment.IExperimentDeclaration;
import de.uniwue.ki.mas.sim.simulation.*;
import de.uniwue.ki.mas.sim.situation.ISituationDeclaration;
import de.uniwue.ki.mas.sim.situation.SituationDeclarationList;
import de.uniwue.ki.mas.util.declaration.IDeclaration;
import de.uniwue.ki.mas.util.declaration.IDocumentableDeclaration;
import de.uniwue.ki.mas.xml.IDelaySettings;
import de.uniwue.ki.mas.xml.castor.XMLRootObject;
import de.uniwue.ki.mas.xml.castor.v01.model.*;

/**
 * Creation date: (14.09.2000 17:16:59)
 * 
 * @author Christoph Oechslein
 */

public class XMLModelWriter implements IDeclarationToXMLObject {

	public static String escapeDocumentationString(String stringToEscape) {
		if (stringToEscape != null)
			return stringToEscape.replaceAll("\n", "&#010;").replaceAll("\t",
					"&#009;");
		else
			return null;
	}

	private final DeclarationToXMLObject _declarationToXMLObject = new DeclarationToXMLObject();
	private final DelaySetterHelper _delaySetterHelper = new DelaySetterHelper(
			this);

	private final IFigMapper _figMapper;
	private final ISeSAmDeclaration _seSAmDeclaration;
	private final UniqueIdMapper _uniqueIdMapper = new UniqueIdMapper();
	private final XMLModelFunctionWriter _xmlModelFunctionWriter;
	private final XMLModelObjectWriter _xmlModelObjectWriter;

	private final XMLModelTypeWriter _xmlModelTypeWriter;

	public XMLModelWriter(ISeSAmDeclaration seSAmDeclaration,
			IFigMapper figMapper) {
		super();

		_figMapper = figMapper;
		_seSAmDeclaration = seSAmDeclaration;

		_xmlModelTypeWriter = new XMLModelTypeWriter(this);
		_xmlModelFunctionWriter = new XMLModelFunctionWriter(this);
		_xmlModelObjectWriter = new XMLModelObjectWriter(this);
	}

	public void addDelaySettings(IDelaySettings delaySettings) {
		_delaySetterHelper.addDelaySettings(delaySettings);
	}

	public void addDelaySettings(Object outerObject,
			XMLFunctionCall functionCall,
			IFunctionCallDeclaration functionCallDeclaration) {

		_delaySetterHelper.addDelaySettings(outerObject, functionCall,
				functionCallDeclaration);
	}

	public boolean containsDeclaration(IDeclaration declaration) {
		return _declarationToXMLObject.containsDeclaration(declaration);
	}

	private XMLAgentClass[] createXMLAgentClassList(
			AgentClassDeclarationList agentClassDeclarationList)
			throws XMLWriteException {
		XMLAgentClass[] xmlAgentClassList = new XMLAgentClass[agentClassDeclarationList
				.size()];
		for (int i = 0; i < agentClassDeclarationList.size(); i++) {
			xmlAgentClassList[i] = _xmlModelObjectWriter
					.getXMLAgentClass((IAgentClassDeclaration) agentClassDeclarationList
							.get(i));
		}
		return xmlAgentClassList;
	}

	private XMLAnalysis createXMLAnalysis(IAnalysisDeclaration realAnalysis)
			throws XMLWriteException {
		XMLAnalysis xmlAnalysis = new XMLAnalysis();
		setXMLIdAndNameDocumentable(xmlAnalysis, realAnalysis);

		setXMLAnalysis(xmlAnalysis, realAnalysis);

		return xmlAnalysis;
	}

	private XMLAnalysisItem createXMLAnalysisItem(
			IAnalysisItemDeclaration realAnalysisItem) throws XMLWriteException {
		XMLAnalysisItem xmlAnalysisItem = new XMLAnalysisItem();

		setXMLAnalysisItem(xmlAnalysisItem, realAnalysisItem);

		return xmlAnalysisItem;
	}

	private XMLAnalysis[] createXMLAnalysisList(
			AnalysisDeclarationList analysisDeclarationList)
			throws XMLWriteException {
		XMLAnalysis[] xmlAnalysisList = new XMLAnalysis[analysisDeclarationList
				.size()];
		for (int i = 0; i < analysisDeclarationList.size(); i++) {
			xmlAnalysisList[i] = getXMLAnalysis((IAnalysisDeclaration) analysisDeclarationList
					.get(i));
		}
		return xmlAnalysisList;
	}

	private XMLExperiment createXMLExperiment(
			IExperimentDeclaration realExperiment) {
		XMLExperiment xmlExperiment = new XMLExperiment();

		setXMLIdAndNameDocumentable(xmlExperiment, realExperiment);

		xmlExperiment.setTest(realExperiment.isTestSimulationDeclaration());

		xmlExperiment.setFunctionCall(createXMLFunctionCall(realExperiment,
				realExperiment.getSimulationActionFunctionCallDeclaration()));

		xmlExperiment.setConfigurationFile(_xmlModelTypeWriter
				.createRelativeFileStringIfPossible(new File(realExperiment
						.getConfigurationFileName())));

		return xmlExperiment;
	}

	private XMLExperiment[] createXMLExperimentList(
			ExperimentDeclarationList experimentDeclarationList) {
		XMLExperiment[] xmlExperimentList = new XMLExperiment[experimentDeclarationList
				.size()];
		for (int i = 0; i < experimentDeclarationList.size(); i++) {
			xmlExperimentList[i] = createXMLExperiment((IExperimentDeclaration) experimentDeclarationList
					.get(i));
		}
		return xmlExperimentList;
	}

	// public fr IXMLFeatureWriters
	public XMLFunctionCallInternGroup createXMLFunctionCallInternGroup(
			IFunctionCallInternDeclaration realFunctionCallIntern)
			throws XMLWriteException {
		if (realFunctionCallIntern instanceof IComposedFunctionCallInternDeclaration) {
			return _xmlModelFunctionWriter
					.createXMLComposedFunctionCallIntern((IComposedFunctionCallInternDeclaration) realFunctionCallIntern);
		}
		if (realFunctionCallIntern instanceof IAtomicFunctionCallInternDeclaration) {
			return _xmlModelFunctionWriter
					.createXMLAtomicFunctionCallIntern((IAtomicFunctionCallInternDeclaration) realFunctionCallIntern);
		}
		return null;
	}

	XMLFunctionCall createXMLFunctionCall(Object outerObject,
			IFunctionCallDeclaration functionCallDeclaration) {
		return _xmlModelFunctionWriter.createXMLFunctionCall(outerObject,
				functionCallDeclaration);
	}

	private XMLObjectInstance createXMLObjectInstance(
			IObjectInstanceDeclaration objectInstanceDeclaration)
			throws XMLWriteException {
		XMLObjectInstance xmlObjectInstance = new XMLObjectInstance();
		_xmlModelObjectWriter.setXMLObjectInstance(xmlObjectInstance,
				objectInstanceDeclaration);
		return xmlObjectInstance;
	}

	private XMLRessourceClass[] createXMLRessourceClassList(
			RessourceClassDeclarationList agentClassDeclarationList)
			throws XMLWriteException {
		XMLRessourceClass[] xmlRessourceClassList = new XMLRessourceClass[agentClassDeclarationList
				.size()];
		for (int i = 0; i < agentClassDeclarationList.size(); i++) {
			xmlRessourceClassList[i] = _xmlModelObjectWriter
					.getXMLRessourceClass((IRessourceClassDeclaration) agentClassDeclarationList
							.get(i));
		}
		return xmlRessourceClassList;
	}

	private XMLSeSAmModel createXMLSeSAmModel() throws XMLWriteException {

		XMLSeSAmModel xmlSeSAm = new XMLSeSAmModel();

		xmlSeSAm.setVersion(0.2);

		if (!"@BUILD_NUMBER@".equals(SeSAmVersion.getBuildNumber()))
			xmlSeSAm.setBuild(SeSAmVersion.getBuildNumber());

		xmlSeSAm.setGlobalAssert(Assert.ASSERTION_ON);
		xmlSeSAm
				.setCompileDebugFunction(FunctionActionDebuggingDeclarationFactory
						.isCompileDebugFunctions());
		xmlSeSAm.setRunDebugFunction(FunctionActionDebuggingDeclarationFactory
				.isRunDebugFunctions());
		xmlSeSAm.setOptimize(FunctionDeclarationCompiler.isDefaultCompileOn());
		xmlSeSAm.setInline(FunctionDeclarationCompiler.isDefaultInlineOn());
		xmlSeSAm.setClosures(AbstractSpecificFunctionType.isSupportClosures());

		xmlSeSAm.setUserType(_xmlModelTypeWriter
				.createXMLUserTypeList(getSeSAmModelDeclaration()
						.getSeSAmModelBasicsDeclaration()
						.getUserTypeDeclarationListAndFactory()
						.getUserTypeDeclarationList()));

		xmlSeSAm.setUserFunction(_xmlModelFunctionWriter
				.createXMLUserFunctionList(getSeSAmModelDeclaration()
						.getSeSAmModelBasicsDeclaration()
						.getUserFunctionDeclarationListAndFactory()
						.getUserFunctionDeclarationList()));

		xmlSeSAm
				.setUserFeatureClass(createXMLUserFeatureClassList(getSeSAmModelDeclaration()
						.getSeSAmModelBasicsDeclaration()
						.getUserFeatureClassDeclarationListAndFactory()
						.getUserFeatureClassDeclarationList()));

		xmlSeSAm
				.setRessourceClass(createXMLRessourceClassList(getSeSAmModelDeclaration()
						.getRessourceClassDeclarationListAndFactory()
						.getRessourceClassDeclarationList()));

		xmlSeSAm
				.setAgentClass(createXMLAgentClassList(getSeSAmModelDeclaration()
						.getAgentClassDeclarationListAndFactory()
						.getAgentClassDeclarationList()));

		xmlSeSAm
				.setWorldClass(createXMLWorldClassList(getSeSAmModelDeclaration()
						.getWorldClassDeclarationListAndFactory()
						.getWorldClassDeclarationList()));

		doAllDelaySettings();

		xmlSeSAm.setSituation(createXMLSituationList(getSeSAmModelDeclaration()
				.getSimulationElementsDeclaration()
				.getSituationDeclarationListAndFactory()
				.getSituationDeclarationList()));

		xmlSeSAm
				.setAnalysis(createXMLAnalysisList((AnalysisDeclarationList) getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getAnalysisDeclarationListAndFactory()
						.getDeclarationList()));

		xmlSeSAm
				.setSimulation(createXMLSimulationList(getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getSimulationDeclarationListAndFactory()
						.getSimulationDeclarationList()));

		xmlSeSAm
				.setExperiment(createXMLExperimentList(getSeSAmModelDeclaration()
						.getSimulationElementsDeclaration()
						.getExperimentDeclarationListAndFactory()
						.getExperimentDeclarationList()));

		xmlSeSAm.setUmlLayout(createXMLUmlLayoutList(getSeSAmDeclaration()
				.getUMLGraphDeclerationList()));

		doAllDelaySettings();

		return xmlSeSAm;
	}

	XMLSimulation createXMLSimulation(ISimulationDeclaration realSimulation)
			throws XMLWriteException {
		Assert
				.assertFalse(realSimulation instanceof ISimulationWithNameFunctionCallDeclDeclaration);
		XMLSimulation xmlSimulation = new XMLSimulation();

		xmlSimulation.setStartSituationID(getXMLSituation(realSimulation
				.getSituationDeclaration()));

		setXMLSimulation(realSimulation, xmlSimulation);

		return xmlSimulation;
	}

	XMLSimulation createXMLSimulation(
			ISimulationWithNameFunctionCallDeclDeclaration realSimulation)
			throws XMLWriteException {

		XMLSimulation xmlSimulation = new XMLSimulation();

		xmlSimulation.setSituation(createXMLSituation(realSimulation
				.getSituationDeclaration()));
		xmlSimulation.setNameCall(createXMLFunctionCall(realSimulation,
				realSimulation.getNameFunctionCallDeclaration()));

		setXMLSimulation(realSimulation, xmlSimulation);

		return xmlSimulation;
	}

	private XMLSimulation[] createXMLSimulationList(
			SimulationDeclarationList simulationDeclarationList)
			throws XMLWriteException {
		XMLSimulation[] xmlSimulationList = new XMLSimulation[simulationDeclarationList
				.size()];
		for (int i = 0; i < simulationDeclarationList.size(); i++) {
			xmlSimulationList[i] = getXMLSimulation((ISimulationDeclaration) simulationDeclarationList
					.get(i));
		}
		return xmlSimulationList;
	}

	private XMLSituation createXMLSituation(ISituationDeclaration realSituation)
			throws XMLWriteException {
		XMLSituation xmlSituation = new XMLSituation();
		setXMLIdAndNameDocumentable(xmlSituation, realSituation);
		xmlSituation.setWorldInstance(_xmlModelObjectWriter
				.createXMLWorldInstance(realSituation
						.getWorldInstanceDeclaration()));
		Iterator it = realSituation.getWorldInstanceDeclaration()
				.getObjectInstanceDeclarationList().iterator();
		while (it.hasNext()) {
			Object obj = it.next();
			IObjectInstanceDeclaration objectInstanceDeclaration = (IObjectInstanceDeclaration) obj;
			Assert.assertFalse(containsDeclaration(objectInstanceDeclaration));
			xmlSituation
					.addObjectInstance(getXMLObjectInstance(objectInstanceDeclaration));
		}
		return xmlSituation;
	}

	private XMLSituation[] createXMLSituationList(
			SituationDeclarationList situationDeclarationList)
			throws XMLWriteException {
		XMLSituation[] xmlSituationList = new XMLSituation[situationDeclarationList
				.size()];
		for (int i = 0; i < situationDeclarationList.size(); i++) {
			ISituationDeclaration situationDeclaration = (ISituationDeclaration) situationDeclarationList
					.get(i);
			Assert.assertFalse(containsDeclaration(situationDeclaration));
			xmlSituationList[i] = getXMLSituation(situationDeclaration);
		}
		return xmlSituationList;
	}

	public XMLSpatialInfo createXMLSpatialInfo(ISpatialInfo spatialInfo) {
		return _xmlModelTypeWriter.createXMLSpatialInfo(spatialInfo);
	}

	XMLTypeGroup createXMLTypeGroup(ISpecificType type)
			throws XMLWriteException {
		return _xmlModelTypeWriter.createXMLTypeGroup(type);
	}

	XMLTypeValueGroup createXMLTypeValueGroup(
			IArgumentDeclaration argumentDeclaration) throws XMLWriteException {

		return _xmlModelTypeWriter.createXMLTypeValueGroup(argumentDeclaration);
	}

	private XMLUMLLayout createXMLUmlLayout(UMLLayoutGraphDeclaration decl)
			throws XMLWriteException {

		XMLUMLLayout xmlLayout = new XMLUMLLayout();

		xmlLayout.setLayoutName(decl.getShortDescription());

		Iterator nodes = decl.getNodes();
		while (nodes.hasNext()) {
			IUMLLayoutGraphNode node = (IUMLLayoutGraphNode) nodes.next();
			IFigObject figNode = getFigMapper().getFigFor(node);

			XMLUMLNode xmlNode = new XMLUMLNode();
			xmlNode.setXpos(figNode.getX());
			xmlNode.setYpos(figNode.getY());
			xmlNode.setObjectID(getObject(node.getDeclaration()));
			xmlNode.setWidth(figNode.getWidth());
			xmlNode.setHeight(figNode.getHeight());
			xmlLayout.addNode(xmlNode);
		}

		return xmlLayout;
	}

	private XMLUMLLayout[] createXMLUmlLayoutList(
			UMLLayoutGraphDeclarationList list) throws XMLWriteException {

		XMLUMLLayout[] xmlLayoutList = new XMLUMLLayout[list.size()];
		for (int i = 0; i < list.size(); i++) {
			xmlLayoutList[i] = getXMLUmlLayout((UMLLayoutGraphDeclaration) list
					.get(i));
		}
		return xmlLayoutList;
	}

	private XMLUMLLayout getXMLUmlLayout(
			UMLLayoutGraphDeclaration umlLayoutDeclaration)
			throws XMLWriteException {
		if (!containsDeclaration(umlLayoutDeclaration))
			setObject(umlLayoutDeclaration,
					createXMLUmlLayout(umlLayoutDeclaration));

		return (XMLUMLLayout) getObject(umlLayoutDeclaration);
	}

	private XMLUserFeatureClass createXMLUserFeatureClass(
			IUserFeatureClassDeclaration realUserFeatureClassDeclaration)
			throws XMLWriteException {
		XMLUserFeatureClass userFeatureClass = new XMLUserFeatureClass();

		setXMLIdAndNameDocumentable(userFeatureClass,
				realUserFeatureClassDeclaration);

		Iterator it = realUserFeatureClassDeclaration
				.getFeatureVariableClassDeclarationListAndFactory()
				.getVariableClassDeclarationList().iterator();
		while (it.hasNext()) {
			userFeatureClass
					.addVariableClass(_xmlModelObjectWriter
							.getXMLVariableClass((IVariableClassDeclaration) it
									.next()));
		}
		it = realUserFeatureClassDeclaration
				.getUserFunctionDeclarationListAndFactory()
				.getUserFunctionDeclarationList().iterator();
		while (it.hasNext()) {
			userFeatureClass.addUserFunction(_xmlModelFunctionWriter
					.getXMLUserFunction((IUserFunctionDeclaration) it.next()));
		}
		return userFeatureClass;
	}

	private XMLUserFeatureClass[] createXMLUserFeatureClassList(
			IFeatureClassDeclarationList realFeatureClassDeclarationList)
			throws XMLWriteException {
		XMLUserFeatureClass[] userFeatureClassList = new XMLUserFeatureClass[realFeatureClassDeclarationList
				.size()];
		for (int i = 0; i < realFeatureClassDeclarationList.size(); i++) {
			userFeatureClassList[i] = getXMLUserFeatureClass((IUserFeatureClassDeclaration) realFeatureClassDeclarationList
					.get(i));
		}
		return userFeatureClassList;
	}

	private XMLWorldClass[] createXMLWorldClassList(
			WorldClassDeclarationList worldClassDeclarationList)
			throws XMLWriteException {
		XMLWorldClass[] xmlWorldClassList = new XMLWorldClass[worldClassDeclarationList
				.size()];
		for (int i = 0; i < worldClassDeclarationList.size(); i++) {
			xmlWorldClassList[i] = _xmlModelObjectWriter
					.getXMLWorldClass((IWorldClassDeclaration) worldClassDeclarationList
							.get(i));
		}
		return xmlWorldClassList;
	}

	private void doAllDelaySettings() throws XMLWriteException {
		_delaySetterHelper.doAllDelaySettings();
	}

	/**
	 * @return IFigMapper
	 */
	public final IFigMapper getFigMapper() {
		return _figMapper;
	}

	public XMLRootObject getObject(IDeclaration declaration)
			throws XMLWriteException {
		return _declarationToXMLObject.getObject(declaration);
	}

	public final ISeSAmDeclaration getSeSAmDeclaration() {
		return _seSAmDeclaration;
	}

	public final ISeSAmModelDeclaration getSeSAmModelDeclaration() {
		return getSeSAmDeclaration().getSeSAmModelDeclaration();
	}

	private XMLAnalysis getXMLAnalysis(IAnalysisDeclaration realAnalysis)
			throws XMLWriteException {
		if (!containsDeclaration(realAnalysis))
			setObject(realAnalysis, createXMLAnalysis(realAnalysis));
		return (XMLAnalysis) getObject(realAnalysis);
	}

	XMLObjectInstance getXMLObjectInstance(
			IObjectInstanceDeclaration objectInstanceDeclaration)
			throws XMLWriteException {
		if (!containsDeclaration(objectInstanceDeclaration))
			setObject(objectInstanceDeclaration,
					createXMLObjectInstance(objectInstanceDeclaration));
		return (XMLObjectInstance) getObject(objectInstanceDeclaration);
	}

	private XMLSimulation getXMLSimulation(ISimulationDeclaration realSimulation)
			throws XMLWriteException {
		if (!containsDeclaration(realSimulation))
			setObject(realSimulation, createXMLSimulation(realSimulation));
		return (XMLSimulation) getObject(realSimulation);
	}

	private XMLSituation getXMLSituation(ISituationDeclaration realSituation)
			throws XMLWriteException {
		if (!containsDeclaration(realSituation))
			setObject(realSituation, createXMLSituation(realSituation));
		return (XMLSituation) getObject(realSituation);
	}

	public String getXMLUniqueId(Object anObject) {
		return _uniqueIdMapper.getId(anObject);
	}

	public XMLUserFeatureClass getXMLUserFeatureClass(
			IUserFeatureClassDeclaration realUserFeatureClassDeclaration)
			throws XMLWriteException {
		if (!containsDeclaration(realUserFeatureClassDeclaration))
			setObject(realUserFeatureClassDeclaration,
					createXMLUserFeatureClass(realUserFeatureClassDeclaration));
		return (XMLUserFeatureClass) getObject(realUserFeatureClassDeclaration);
	}

	public void setObject(IDeclaration declaration, XMLRootObject object) {
		_declarationToXMLObject.setObject(declaration, object);
	}

	private void setXMLAnalysis(XMLAnalysis xmlAnalysis,
			IAnalysisDeclaration realAnalysis) throws XMLWriteException {
		setXMLIdAndNameDocumentable(xmlAnalysis, realAnalysis);

		setXMLAnalysisDeltaSettings(xmlAnalysis, realAnalysis
				.getAnalysisDeltaSettings());

		// default setttings is false
		if (realAnalysis.isDefaultAnalysis())
			xmlAnalysis.setDefault(true);

		IAnalysisTypeDeclaration analysisTypeDeclaration = realAnalysis
				.getAnalysisTypeDeclaration();

		if (analysisTypeDeclaration instanceof CSVAnalysisTypeDeclaration) {
			xmlAnalysis.setType("CSV");
			CSVAnalysisTypeDeclaration csvAnalysisTypeDeclaration = (CSVAnalysisTypeDeclaration) analysisTypeDeclaration;
			xmlAnalysis.setFile(_xmlModelTypeWriter
					.createRelativeFileStringIfPossible(new File(
							csvAnalysisTypeDeclaration.getFileString())));

			if (!CSVAnalysisTypeDeclaration.DEFAULT_SEPARATOR_STRING
					.equals(csvAnalysisTypeDeclaration.getSeparatorString())) {
				xmlAnalysis.setSeparatorString(csvAnalysisTypeDeclaration
						.getSeparatorString());
			}
			if (!CSVAnalysisTypeDeclaration.DEFAULT_TIME_COLUMN_NAME
					.equals(csvAnalysisTypeDeclaration.getTimeColumnName())) {
				xmlAnalysis.setTimeColumnName(csvAnalysisTypeDeclaration
						.getTimeColumnName());
			}

		} else if (analysisTypeDeclaration instanceof BlockChartAnalysisTypeDeclaration) {
			xmlAnalysis.setType("Block");

		} else if (analysisTypeDeclaration instanceof SeriesChartAnalysisTypeDeclaration) {
			xmlAnalysis.setType("Series");
			xmlAnalysis
					.setWindowSize(((SeriesChartAnalysisTypeDeclaration) analysisTypeDeclaration)
							.getWindowSize());

		} else if (analysisTypeDeclaration instanceof TableAnalysisTypeDeclaration) {
			xmlAnalysis.setType("Table");

		} else {
			Assert.fail("Unknown analysis type");
		}

		Iterator it = realAnalysis.getAnalysisItemDeclarationListAndFactory()
				.getAnalysisItemDeclarationList().iterator();

		while (it.hasNext()) {
			xmlAnalysis
					.addItem(createXMLAnalysisItem((IAnalysisItemDeclaration) it
							.next()));
		}

	}

	private void setXMLAnalysisDeltaSettings(XMLAnalysis xmlAnalysis,
			IAnalysisDeltaSettings analysisDeltaSettings) {
		xmlAnalysis.setDeltaTicks(analysisDeltaSettings.getDeltaTicks());

		if (analysisDeltaSettings.getStartTick() != 0)
			xmlAnalysis.setStartTick(analysisDeltaSettings.getStartTick());

		if (analysisDeltaSettings.isUsingDecisionFunctionCallDeclaration()) {
			xmlAnalysis.setUsingDecision(true);
		}

		if (!analysisDeltaSettings.getDecisionFunctionCallDeclaration()
				.isomorphicEquals(
						FunctionCallDeclarationFactory.getInstance()
								.createFunctionCallDeclaration(
										BooleanType.getInstance()))) {
			xmlAnalysis.setDecision(createXMLFunctionCall(
					analysisDeltaSettings, analysisDeltaSettings
							.getDecisionFunctionCallDeclaration()));
		}
	}

	private void setXMLAnalysisItem(XMLAnalysisItem xmlAnalysisItem,
			IAnalysisItemDeclaration realAnalysisItem) throws XMLWriteException {

		xmlAnalysisItem.setName(realAnalysisItem.getName());

		setXMLFunctionCall(realAnalysisItem, xmlAnalysisItem, realAnalysisItem
				.getValueFunctionCallDeclaration());

		if (realAnalysisItem instanceof ColorAnalysisItemDeclaration) {
			java.awt.Color color = ((ColorAnalysisItemDeclaration) realAnalysisItem)
					.getColor();

			xmlAnalysisItem.setColorBlue(color.getBlue());
			xmlAnalysisItem.setColorGreen(color.getGreen());
			xmlAnalysisItem.setColorRed(color.getRed());
		}

	}

	void setXMLFunctionCall(Object outerObject, XMLFunctionCall functionCall,
			IFunctionCallDeclaration functionCallDeclaration)
			throws XMLWriteException {
		_xmlModelFunctionWriter.setXMLFunctionCall(outerObject, functionCall,
				functionCallDeclaration);
	}

	void setXMLId(XMLAbstractIdAndNamed xmlAbstractIdAndNamed,
			IDeclaration declaration) {

		if (Assert.ASSERTION_ON)
			Assert
					.assertTrue(!(declaration instanceof IDocumentableDeclaration));

		xmlAbstractIdAndNamed.setId(getXMLUniqueId(declaration));
		// xmlAbstractIdAndNamed.setName(declaration.getName());
	}

	void setXMLIdAndNameDocumentable(
			XMLAbstractIdAndNamed xmlAbstractIdAndNamed,
			IDocumentableDeclaration documentableDeclaration) {

		xmlAbstractIdAndNamed.setId(getXMLUniqueId(documentableDeclaration));
		if (documentableDeclaration.getUsesShortDescription()) {
			xmlAbstractIdAndNamed.setName(documentableDeclaration
					.getShortDescription());
		}

		if (xmlAbstractIdAndNamed instanceof XMLAbstractIdAndDocumentable) {
			XMLAbstractIdAndDocumentable xmlAbstractIdAndDocumentable = (XMLAbstractIdAndDocumentable) xmlAbstractIdAndNamed;

			String documentationString = documentableDeclaration
					.getDocumentation();

			if (documentationString.length() > 0) {

				xmlAbstractIdAndDocumentable
						.setDocumentation(escapeDocumentationString(documentationString));
			}
		}

		if (documentableDeclaration.hasTODOFlag())
			xmlAbstractIdAndNamed.setTodoFlag(true);
	}

	void setXMLSimulation(ISimulationDeclaration realSimulation,
			XMLSimulation xmlSimulation) throws XMLWriteException {

		setXMLIdAndNameDocumentable(xmlSimulation, realSimulation);
		xmlSimulation.setTest(realSimulation.isTestSimulationDeclaration());

		Iterator analysis = realSimulation.getAnalysisDeclarationList()
				.iterator();
		while (analysis.hasNext()) {
			IAnalysisDeclaration currIAnalysisDeclaration = (IAnalysisDeclaration) analysis
					.next();

			XMLAnalysisID xmlAnalysisID = new XMLAnalysisID();

			xmlAnalysisID.setId(getXMLAnalysis(currIAnalysisDeclaration));

			xmlSimulation.addAnalysis(xmlAnalysisID);
		}

		Iterator endFunctions = realSimulation
				.getEndFunctionDeclarationListAndFactory().getDeclarationList()
				.iterator();
		while (endFunctions.hasNext()) {
			NamedFunctionCallDeclaration currNamedFunctionCallDeclaration = (NamedFunctionCallDeclaration) endFunctions
					.next();

			xmlSimulation.addEndfunction(createXMLNamedFunctionCall(
					realSimulation, currNamedFunctionCallDeclaration));
		}

		Iterator duringRuns = realSimulation
				.getDuringRunTestActionDefinitionList().iterator();
		while (duringRuns.hasNext()) {
			NamedFunctionCallDeclaration currNamedFunctionCallDeclaration = (NamedFunctionCallDeclaration) duringRuns
					.next();

			xmlSimulation.addDuringRunTest(createXMLNamedFunctionCall(
					realSimulation, currNamedFunctionCallDeclaration));
		}

		Iterator afterRuns = realSimulation
				.getAfterRunTestActionDefinitionList().iterator();
		while (afterRuns.hasNext()) {
			NamedFunctionCallDeclaration currNamedFunctionCallDeclaration = (NamedFunctionCallDeclaration) afterRuns
					.next();

			xmlSimulation.addAfterRunTest(createXMLNamedFunctionCall(
					realSimulation, currNamedFunctionCallDeclaration));
		}
	}

	private XMLNamedFunctionCall createXMLNamedFunctionCall(Object outerObject,
			NamedFunctionCallDeclaration currNamedFunctionCallDeclaration) {
		XMLNamedFunctionCall xmlNamedFunctionCall = new XMLNamedFunctionCall();

		setXMLIdAndNameDocumentable(xmlNamedFunctionCall,
				currNamedFunctionCallDeclaration);

		xmlNamedFunctionCall.setFunctionCall(createXMLFunctionCall(outerObject,
				currNamedFunctionCallDeclaration));

		return xmlNamedFunctionCall;
	}

	void setXMLUnnamedParametricFunction(Object outerObject,
			XMLUnnamedParametricFunction xmlUnnamedParametricFunction,
			IParametricFunctionCallDeclaration parametricFunctionCallDeclaration)
			throws XMLWriteException {

		_xmlModelFunctionWriter
				.setXMLUnnamedParametricFunction(outerObject,
						xmlUnnamedParametricFunction,
						parametricFunctionCallDeclaration);
	}

	public void writeXMLFile(File file) throws XMLWriteException, IOException {
		Writer writer = new BufferedWriter(new FileWriter(file));
		try {
			writeXMLFile(writer);
		} finally {
			writer.close();
		}
	}

	public void writeXMLFile(String fileString) throws XMLWriteException,
			IOException {
		writeXMLFile(new File(fileString));
	}

	public void writeXMLFile(Writer writer) throws XMLWriteException {
		try {
			createXMLSeSAmModel().marshal(writer);
		} catch (MarshalException e) {
			throw new XMLWriteException(e);

		} catch (ValidationException e) {
			throw new XMLWriteException(e);
		}
	}

	public String writeXMLString() throws XMLWriteException, IOException {
		StringWriter stringWriter = new StringWriter();
		writeXMLFile(stringWriter);
		stringWriter.close();
		return stringWriter.toString();
	}

	public void setXMLFile(XMLFile xmlFile, File file) {
		_xmlModelTypeWriter.setXMLFile(xmlFile, file);
	}
}