package de.uniwue.ki.util;
import java.awt.geom.Point2D;
import java.io.Serializable;

import junit.framework.Assert;

import de.uniwue.ki.util.math.MathHelper;

/**
 * Creation date: (28.09.2000 17:12:33)
 * @author Christoph Oechslein
 */
public final class Position2D extends Point2D.Double implements Serializable {

	public Position2D() {
		super();
	}

	public Position2D(double x, double y) {
		super(x, y);
	}

	public Position2D(Position2D position2D) {
		super(position2D.x, position2D.y);
	}

	public Position2D(java.lang.Double x, java.lang.Double y) {
		super(x.doubleValue(), y.doubleValue());
	}

	/**
	  * Adds destructivly <code>x, y</code> to <code>this</code>.
	  * @param x double
	 * @param y double
	 */
	public final void add(double x2, double y2) {
		this.x += x2;
		this.y += y2;
	}

	/**
	  * Adds destructivly <code>pos</code> to <code>this</code>.
	  * @param pos Position2D
	 */
	public final void add(Position2D pos) {
		add(pos.x, pos.y);
	}

	/**
	  * Generates a new position which is the addition of <code>pos</code> and the vector <code>x , y</code>.
	  * @param pos Position2D
	 * @param x double
	 * @param y double
	 * @return Position2D
	 */
	public static final Position2D add(Position2D pos, double x, double y) {
		Position2D newPosition2D = Position2D.createPosition(x, y);
		newPosition2D.add(pos);
		return newPosition2D;
	}

	/**
	  * Generates a new position which is the addition of <code>pos1</code> and <code>pos2</code>.
	  * @param pos1 Position2D
	 * @param pos2 Position2D
	 * @return Position2D
	 */
	public static final Position2D add(Position2D pos1, Position2D pos2) {
		return Position2D.add(pos1, pos2.x, pos2.y);
	}

	/**
	  * Adds destructivly the vector consiting of <code>directionInRad</code> and <code>length</code>.
	  * @param directionInRad double
	 * @param length double
	 */
	public final void addWithDirectionAndLength(double directionInRad, double length) {
		this.add(
			directionAndLengthToPosX(directionInRad, length),
			directionAndLengthToPosY(directionInRad, length));
	}

	/**
	  * Generates a new position which is the addition of <code>pos</code> and the vector consisting of
	  * <code>directionInRad</code> and <code>length</code>.
	  * @param pos Position2D
	 * @param directionInRad double
	 * @param length double
	 * @return Position2D
	 */
	public static final Position2D addWithDirectionAndLength(
		Position2D pos,
		double directionInRad,
		double length) {
		Position2D newPosition2D = Position2D.createPositionWithDirectionAndLength(directionInRad, length);

		newPosition2D.add(pos);

		return newPosition2D;
	}

	/**
	 * @return double
	 */
	public final double angle() {
		return Position2D.angle(0, 0, this.x, this.y);
	}

	/**
	 * @param fromPosX double
	 * @param fromPosY double
	 * @return double
	 */
	public final double angle(double fromPosX, double fromPosY) {
		return Position2D.angle(fromPosX, fromPosY, this.x, this.y);
	}

	/**
	 * @param fromPosX double
	 * @param fromPosY double
	 * @param toPosX double
	 * @param toPosY double
	 * @return double
	 */
	public static final double angle(double fromPosX, double fromPosY, double toPosX, double toPosY) {
		double resultAngle = angleIntern(fromPosX, fromPosY, toPosX, toPosY);

		if (Assert.ASSERTION_ON) {
			Assert.assertTrue("Angle must be greater than zero, was :" + resultAngle, MathHelper.doubleLowerEqual(0, resultAngle));
			Assert.assertTrue("Angle must be lower than "+Math.PI*2+", was :" + resultAngle, MathHelper.doubleLower(resultAngle, 2 * Math.PI));
		}

		return resultAngle;
	}

	/**
	 * @param fromPos Position2D
	 * @return double
	 */
	public final double angle(Position2D fromPos) {
		return Position2D.angle(fromPos, this);
	}

	/**
	 * @param fromPos Position2D
	 * @param toPosX double
	 * @param toPosY double
	 * @return double
	 */
	public static final double angle(Position2D fromPos, double toPosX, double toPosY) {
		return angle(fromPos.x, fromPos.y, toPosX, toPosY);
	}

	/**
	 * @param fromPos Position2D
	 * @param toPos Position2D
	 * @return double
	 */
	public static final double angle(Position2D fromPos, Position2D toPos) {
		return angle(fromPos.x, fromPos.y, toPos.x, toPos.y);
	}

	private static final double angleIntern(double fromPosX, double fromPosY, double toPosX, double toPosY) {
		double deltaY = toPosY - fromPosY;
		double deltaX = toPosX - fromPosX;

		//if ((deltaX == 0) && (deltaY == 0)) {
		if ((MathHelper.doubleEqual(deltaX,0)) 
				&& (MathHelper.doubleEqual(deltaY,0))) {
			return 0;
		}

		double angle = Math.atan(deltaY / deltaX);

		if(MathHelper.doubleEqual(angle,0)){
			angle=0;
		}
		// -Math.PI / 2 < angle < Math.PI / 2
		
		if ((deltaX < 0) && (deltaY >= 0)) {
			return angle + Math.PI;

		} else if ((deltaX < 0) && (deltaY < 0)) {
			return angle + Math.PI;

		} else if ((angle < 0)) {
			return angle + 2 * Math.PI;

		} else {
			return angle;
		}
	}

	/**
	  * Generates a new position which the vector consisting of
	  * <code>directionInRad</code> and <code>length</code>.
	  * @param x double
	 * @param y double
	 * @return Position2D
	 */
	public static final Position2D createPosition(double x, double y) {
		return new Position2D(x, y);
	}
	public static void main(String[] args) { 
			 double fromPosX= 112.5; 
			 double fromPosY= 147.06162237149246; 
			 double toPosX= 121.0; 
			 double toPosY= 147.0616220197303; 
           
			 System.out.println(2 * Math.PI); 
			 System.out.println(angle(fromPosX, fromPosY, toPosX, toPosY)); 
           
		}
	/**
	  * Generates a new position which the vector consisting of
	  * <code>directionInRad</code> and <code>length</code>.
	  * @param directionInRad double
	 * @param length double
	 * @return Position2D
	 */
	public static final Position2D createPositionWithDirectionAndLength(
		double directionInRad,
		double length) {
		return new Position2D(
			directionAndLengthToPosX(directionInRad, length),
			directionAndLengthToPosY(directionInRad, length));
	}

	private static final double directionAndLengthToPosX(double directionInRad, double length) {
		return Math.cos(directionInRad) * length;
	}

	private static final double directionAndLengthToPosY(double directionInRad, double length) {
		return Math.sin(directionInRad) * length;
	}

	/**
	 * @return double
	 */
	public final double distance() {
		return Math.sqrt(distanceSq());
	}

	/**
	 * @return double
	 */
	public double distanceSq() {
		double pX = this.getX();
		double pY = this.getY();
		return pX * pX + pY * pY;
	}

	/**
	 * @return java.lang.Double
	 */
	public final java.lang.Double getXDouble() {
		return new java.lang.Double(x);
	}

	/**
	 * @return java.lang.Double
	 */
	public final java.lang.Double getYDouble() {
		return new java.lang.Double(y);
	}

	/**
	  * Substracts destructivly <code>x, y</code> to <code>this</code>.
	  * @param x double
	 * @param y double
	 */
	public final void substract(double x2, double y2) {
		this.x -= x2;
		this.y -= y2;
	}

	/**
	  * Substracts destructivly <code>pos</code> to <code>this</code>.
	  * @param pos Position2D
	 */
	public final void substract(Position2D pos) {
		substract(pos.x, pos.y);
	}
}