package de.uniwue.ki.util;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.swingui.TestRunner;

import de.uniwue.ki.util.collection.TestCollectionHelper;
import de.uniwue.ki.util.file.TokenReplacerTest;
import de.uniwue.ki.util.math.TestMathHelper;/**
 * Creation date: (06.06.2001 18:55:08)
 * @author Christoph Oechslein
 */
public final class TestAllUtils extends TestCase {

	/**
	 * @param aName String
	 */
	public TestAllUtils(String aName) {
	   super(aName);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
	   TestRunner.run(TestAllUtils.class);
	}

	public static TestSuite suite() {
	   TestSuite anSuite = new TestSuite("Utils");
	
	   anSuite.addTest(TestMathHelper.suite());
	   anSuite.addTestSuite(TestCollectionHelper.class);
	   anSuite.addTest(new TestAllUtils("testPosition2D"));
	   anSuite.addTestSuite(TokenReplacerTest.class);
	
	   return anSuite;
	}

	/**
	 * @throws Exception
	 */
	public void testPosition2D() throws Exception {
	   assertEquals(45, Math.toDegrees(Position2D.angle(0, 0, 1, 1)), 0.01);
	   assertEquals(225, Math.toDegrees(Position2D.angle(1, 1, 0, 0)), 0.01);
	   assertEquals(225, Math.toDegrees(Position2D.angle(0, 0, -1, -1)), 0.01);
	   assertEquals(315, Math.toDegrees(Position2D.angle(0, 0, 1, -1)), 0.01);
	   assertEquals(135, Math.toDegrees(Position2D.angle(0, 0, -1, 1)), 0.01);
	
	   for (int x = 0; x < 10; x++) {
	      if (x != 0) {
	         assertEquals(0, Math.toDegrees(Position2D.angle(0, 0, x, 0)), 0.01);
	         assertEquals(90, Math.toDegrees(Position2D.angle(0, 0, 0, x)), 0.01);
	      }
	
	      for (int y = 1; y < 10; y++) {
	         if (x == y) {
	            assertEquals(45, Math.toDegrees(Position2D.angle(0, 0, x, y)), 0.01);
	
	         } else {
	            double resultAngle = Position2D.angle(0, 0, x, y);
	
	            assertTrue(0 <= resultAngle);
	            assertTrue(resultAngle < 2 * Math.PI);
	
	            double inverseResultAngle = Position2D.angle(x, y, 0, 0);
	
	            assertTrue(0 <= inverseResultAngle);
	            assertTrue(inverseResultAngle < 2 * Math.PI);
	
	            assertEquals((resultAngle + Math.PI) % (2 * Math.PI), inverseResultAngle, 0.01);
	         }
	      }
	   }
	}
}