package de.uniwue.ki.util;
import java.text.*;
import java.util.Locale;

import junit.framework.Assert;

import com.geocities.sbridgesgeo.debug.Inspector;

/**
 * <BR>Creation date: (23.03.00 16:50:36)
 * @author Christoph Oechslein
 */

public class Utils {

	private static int _majorJavaVersionNumber = 0;
	private static int _minorJavaVersionNumber = 0;
	private static int _bugFixJavaVersionNumber = 0;

	static {
		initializeJavaVersion();
	}

	private Utils() {
		super();
	}

	public static final boolean equals(Object anObject1, Object anObject2) {
		if (anObject1 == null)
			return anObject2 == null;
		else
			return anObject1.equals(anObject2);
	}

	public static void inspect(Object anObject) {
		Inspector.inspect(anObject);
	}

	private static void initializeJavaVersion() {
		// java.version will have form 1.1.7A, 11, 1.1., 1.1 or 1.3beta
		// It may be gibberish. It may be undefined.
		// We have do deal with all this malformed garbage.
		// Because incompetents run the world,
		// it is not nicely formatted for us in three fields.
		String ver = System.getProperty("java.version").trim();

		// chop trailing "beta"
		if (ver.endsWith("beta")) {
			ver = ver.substring(0, ver.length() - "beta".length());
		}

		if (Assert.ASSERTION_ON)
			Assert.assertTrue("length of java.version string lower than 2 : " + ver, ver.length() > 2);

		int dex = ver.indexOf('.');

		if (dex == -1) {
			// provide missing dot
			ver = ver.charAt(0) + '.' + ver.substring(1);
			dex = 1;
		}

		try {
			_majorJavaVersionNumber = Integer.parseInt(ver.substring(0, dex));
		} catch (NumberFormatException e) {
			Assert.fail("Error while parsing major java version number: " + ver.substring(0, dex));
		}

		if (dex + 1 >= ver.length()) {
			_minorJavaVersionNumber = 0;
			_bugFixJavaVersionNumber = 0;
			return;
		}

		// chop off major and first dot.
		ver = ver.substring(dex + 1);

		// chop any trailing letter as in 1.1.7A,
		// but convert 1.1.x or 1.1.X to 1.1.9
		char ch = ver.charAt(ver.length() - 1);
		if (!Character.isDigit(ch)) {
			ver = ver.substring(0, ver.length() - 1);
			if (ch == 'x' || ch == 'X')
				ver += '9';
		}
		// check minor version
		dex = ver.indexOf('.');
		if (dex == -1) {
			// provide missing BugFix number as in 1.2 or 1.0
			ver += ".0";
			dex = ver.indexOf('.');
		}

		try {
			_minorJavaVersionNumber = Integer.parseInt(ver.substring(0, dex));
		} catch (NumberFormatException e) {
			_minorJavaVersionNumber = 0;
		}

		if (dex + 1 >= ver.length()) {
			_bugFixJavaVersionNumber = 0;
			return;
		}

		// check bugfix version
		ver = ver.substring(dex + 1);

		dex = ver.indexOf('_');

		if (dex == -1) {
			try {
				_bugFixJavaVersionNumber = Integer.parseInt(ver);
			} catch (NumberFormatException e) {
				_bugFixJavaVersionNumber = 0;
			}
		} else {
			try {
				_bugFixJavaVersionNumber = Integer.parseInt(ver.substring(0, dex));
			} catch (NumberFormatException e) {
				_bugFixJavaVersionNumber = 0;
			}

			ver = ver.substring(dex + 1);
		}
	}

	/**
	 * Ensures Java runtime version e.g. 1.1.7 is sufficiently recent.
	@jps.net>
	 * @param wantedMajor java major version e.g. 1
	 * @param wantedMinor Java minor version e.g. 1
	 * @param wantedBugFix Java bugfix version e.g. 7
	 * return true if JVM version is equal to or more recent than
	 * (higher than) the level specified.
	 * Based on code by Dr. Tony Dahlman <adahlman * @return boolean
	 */
	static public boolean isJavaVersionOK(int wantedMajor, int wantedMinor, int wantedBugFix) {

		if (_majorJavaVersionNumber < wantedMajor)
			return false;
		if (_majorJavaVersionNumber > wantedMajor)
			return true;

		if (_minorJavaVersionNumber < wantedMinor)
			return false;
		if (_minorJavaVersionNumber > wantedMinor)
			return true;

		return (_bugFixJavaVersionNumber >= wantedBugFix);
	}

	public static final Object newInstanceOf(Class aClass) {
		try {
			return aClass.newInstance();
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e.getLocalizedMessage());
		} catch (InstantiationException e) {
			throw new RuntimeException(e.getLocalizedMessage());
		}
	}

	public static final Object newInstanceOf(Object obj) {
		return newInstanceOf(obj.getClass());
	}

	public static String getMemoryInfo() {
		StringBuffer buffer = new StringBuffer();
		appendMemoryInfo(buffer);
		return buffer.toString();
	}

	public static void appendMemoryInfo(StringBuffer buffer) {
		DecimalFormat numberFormat = new DecimalFormat();
		numberFormat.setMaximumFractionDigits(2);

		appendMemoryInfo(buffer, numberFormat);
	}

	public static void appendMemoryInfo(StringBuffer buffer, NumberFormat numberFormat) {
		numberFormat.format(
			((double) Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / (1 << 20),
			buffer,
			new FieldPosition(0));
		buffer.append("M (used) ");
		numberFormat.format(
			(double) Runtime.getRuntime().freeMemory() / (1 << 20),
			buffer,
			new FieldPosition(0));
		buffer.append("M (free) ");
		numberFormat.format(
			(double) Runtime.getRuntime().maxMemory() / (1 << 20),
			buffer,
			new FieldPosition(0));
		buffer.append("M (max) ");
	}

	public static void invokeLater(String threadName, Runnable runnable) {
		Thread thread = new Thread(runnable, threadName);
		thread.setPriority(Thread.MIN_PRIORITY);
		thread.start();
	}

	public static void setEnglishLocale() {
		// set to english
		Locale.setDefault(Locale.ENGLISH);
		System.setProperty("user.language", "");
	}
}