package de.uniwue.ki.util.collection;
import java.util.Iterator;
import java.util.NoSuchElementException;

import junit.framework.Assert;

import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * The SelectIterator selects elements according to the IPredicateFunction from a given Iterator. --steffen 
 * Creation date: (14.04.00 16:34:08)
 * @author Christoph Oechslein
 */

final class SelectIterator extends AbstractIterator implements Iterator {
	private Iterator _activeIterator;
	private IPredicateFunction _filterFunction;
	private Object _currentElement;

	public SelectIterator(Iterator iterator, IPredicateFunction filterFunction) throws NoSuchElementException {
		super();

		Assert.assertNotNull(iterator);
		Assert.assertNotNull(filterFunction);

		_activeIterator = iterator;
		_filterFunction = filterFunction;

		searchNextElement();
	}

	public boolean hasNext() {
		return (_activeIterator != null);
	}

	public Object next() throws NoSuchElementException {
		if (!hasNext())
			throw new NoSuchElementException();
		
		Object oldElement = _currentElement;
		searchNextElement();
		return oldElement;
	}

	public void remove() {
		throw new UnsupportedOperationException();
	}

	private void searchNextElement() throws NoSuchElementException {
		try {
			while (_activeIterator.hasNext()) {
				Object currentLocalElement = _activeIterator.next();
				if (_filterFunction.isElementOk(currentLocalElement)) {
					_currentElement = currentLocalElement;
					return;
				}
			}
		} catch (Exception e) {
			if (e instanceof RuntimeException)
				throw (RuntimeException) e;
			else
				throw new RuntimeException(e);
		}

		_currentElement = null;
		_activeIterator = null;
		_filterFunction = null;
	}
}