package de.uniwue.ki.util.collection;
import java.util.*;

import junit.framework.TestCase;

import de.uniwue.ki.util.functions.*;

/**
 * Creation date: (06.06.2001 18:56:15)
 * @author Christoph Oechslein
 */
public final class TestCollectionHelper extends TestCase {
	private List _testListEmpty;

	private List _testListNormal;

	private List _testListLong;

	private IPredicateFunction _alwaysTruePredicateFunction;
	private IPredicateFunction _alwaysFalsePredicateFunction;

	public TestCollectionHelper(String aName) {
		super(aName);
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestCollectionHelper.class);
	}

	public void setUp() throws Exception {
		_alwaysTruePredicateFunction = new IPredicateFunction() {
			public boolean isElementOk(Object anObject) {
				return true;
			}
		};

		_alwaysFalsePredicateFunction = new NotPredicateFunction(_alwaysTruePredicateFunction);

		_testListEmpty = new ArrayList();

		_testListNormal = new ArrayList();
		_testListNormal.add(new Double(1));
		_testListNormal.add(new Double(2));
		_testListNormal.add(new Double(3));
		_testListNormal.add(new Double(4));

		_testListLong = new ArrayList();
		_testListLong.add(new Double(1));
		_testListLong.add(new Double(2));
		_testListLong.add(new Double(3));
		_testListLong.add(new Double(4));
		_testListLong.add(new Double(5));
		_testListLong.add(new Double(6));
		_testListLong.add(new Double(7));
		_testListLong.add(new Double(8));
		_testListLong.add(new Double(9));
		_testListLong.add(new Double(10));
	}

	public void testCount() throws Exception {
		assertEquals(CollectionsHelper.size(_testListEmpty.iterator()), _testListEmpty.size());
		assertEquals(CollectionsHelper.size(_testListNormal.iterator()), _testListNormal.size());
		assertEquals(CollectionsHelper.size(_testListLong.iterator()), _testListLong.size());
	}

	public void testRemoveDuplicates() throws Exception {
		testRemoveDuplicates(
			Collections.nCopies(4, new Integer(1)),
			CollectionsHelper.createList(new Integer(1)));
		testRemoveDuplicates(Collections.EMPTY_LIST, Collections.EMPTY_LIST);
		testRemoveDuplicates(
			CollectionsHelper.createList(new Integer(1)),
			CollectionsHelper.createList(new Integer(1)));
	}

	private void testRemoveDuplicates(List list, List expected) {
		list = new ArrayList(list);
		CollectionsHelper.removeDuplicates(list);
		assertEquals(expected, list);
	}

	public void testFind() throws Exception {
		IPredicateFunction predicateFunction = _alwaysTruePredicateFunction;
		assertTrue(!(CollectionsHelper.find(_testListEmpty.iterator(), predicateFunction) != null));
		IPredicateFunction predicateFunction1 = _alwaysTruePredicateFunction;
		assertTrue((CollectionsHelper.find(_testListNormal.iterator(), predicateFunction1) != null));
		IPredicateFunction predicateFunction2 = _alwaysTruePredicateFunction;
		assertTrue((CollectionsHelper.find(_testListLong.iterator(), predicateFunction2) != null));
		IPredicateFunction predicateFunction3 = _alwaysFalsePredicateFunction;

		assertTrue(!(CollectionsHelper.find(_testListEmpty.iterator(), predicateFunction3) != null));
		IPredicateFunction predicateFunction4 = _alwaysFalsePredicateFunction;
		assertTrue(!(CollectionsHelper.find(_testListNormal.iterator(), predicateFunction4) != null));
		IPredicateFunction predicateFunction5 = _alwaysFalsePredicateFunction;
		assertTrue(!(CollectionsHelper.find(_testListLong.iterator(), predicateFunction5) != null));
	}

	public void testCardProduct() throws Exception {

		ITwoArgResultFunction createIteratorFunction = new ITwoArgResultFunction() {
			public Object execute(Object obj1, Object obj2) throws Exception {
				return CollectionsHelper.createIterator(obj1, obj2);
			}
		};

		assertTrue(
			CollectionsHelper.isEmpty(
				CollectionsHelper.cartesianProduct(
					createIteratorFunction,
					CollectionsHelper.EMPTY_ITERATOR_WITH_EXCEPTION,
					CollectionsHelper.createList(new Double(10.0), new Double(20.0)))));
		assertTrue(
			CollectionsHelper.isEmpty(
				CollectionsHelper.cartesianProduct(
					createIteratorFunction,
					CollectionsHelper.createIteratorWithException(new Double(1.0), new Double(2.0)),
					CollectionsHelper.EMPTY_LIST)));

		assertTrue(
			testCardProductResult(
				CollectionsHelper.createIterator(
					CollectionsHelper.createIterator(new Double(1.0), new Double(10.0)),
					CollectionsHelper.createIterator(new Double(1.0), new Double(20.0)),
					CollectionsHelper.createIterator(new Double(2.0), new Double(10.0)),
					CollectionsHelper.createIterator(new Double(2.0), new Double(20.0))),
				CollectionsHelper.cartesianProduct(
					createIteratorFunction,
					CollectionsHelper.createIteratorWithException(new Double(1.0), new Double(2.0)),
					CollectionsHelper.createList(new Double(10.0), new Double(20.0)))));

		assertTrue(
			testCardProductResult(
				CollectionsHelper.createIterator(
					CollectionsHelper.createIterator(new Double(1.0), new Double(10.0)),
					CollectionsHelper.createIterator(new Double(2.0), new Double(10.0))),
				CollectionsHelper.cartesianProduct(
					createIteratorFunction,
					CollectionsHelper.createIteratorWithException(new Double(1.0), new Double(2.0)),
					CollectionsHelper.createList(new Double(10.0)))));

		assertTrue(
			testCardProductResult(
				CollectionsHelper.createIterator(
					CollectionsHelper.createIterator(new Double(1.0), new Double(10.0)),
					CollectionsHelper.createIterator(new Double(1.0), new Double(20.0))),
				CollectionsHelper.cartesianProduct(
					createIteratorFunction,
					CollectionsHelper.createIteratorWithException(new Double(1.0)),
					CollectionsHelper.createList(new Double(10.0), new Double(20.0)))));
	}

	private boolean testCardProductResult(Iterator it1, Iterator it2) {
		while (it1.hasNext() && it2.hasNext()) {
			Iterator element1 = (Iterator) it1.next();
			Iterator element2 = (Iterator) it2.next();

			if (!CollectionsHelper.isEqual(element1, element2, true)) {
				return false;
			}
		}

		return it1.hasNext() == it2.hasNext();
	}
}