package de.uniwue.ki.util.event;
import java.io.*;
import java.lang.ref.WeakReference;
import java.util.*;

import de.uniwue.ki.util.factory.AbstractObjectFactory;

/**
 * Creation date: (30.10.2001 18:21:16)
 * @author Christoph Oechslein
 */
public abstract class AbstractListenerMulticaster implements EventListener, Serializable {
	private LinkedList _listener = new LinkedList();

	protected AbstractListenerMulticaster() {
		super();
	}

	public static abstract class ListenerMulticasterFactory extends AbstractObjectFactory {
		public abstract AbstractListenerMulticaster createAbstractListenerMulticaster();
	}

	protected static final EventListener add(
		EventListener eventListener,
		EventListener eventListenerToAdd,
		ListenerMulticasterFactory listenerMulticasterFactory) {

		if (eventListener == null) {
			return eventListenerToAdd;

		} else if (eventListener instanceof AbstractListenerMulticaster) {
			getListenerList(eventListener).add(new WeakReference(eventListenerToAdd));
			return eventListener;

		} else {
			AbstractListenerMulticaster listenerMulticaster =
				listenerMulticasterFactory.createAbstractListenerMulticaster();

			listenerMulticaster.getListenerList().add(new WeakReference(eventListener));
			listenerMulticaster.getListenerList().add(new WeakReference(eventListenerToAdd));

			return listenerMulticaster;
		}
	}

	protected static final EventListener remove(
		EventListener eventListener,
		EventListener eventListenerToRemove) {

		if (eventListener == eventListenerToRemove || eventListener == null) {
			return null;

		} else if (eventListener instanceof AbstractListenerMulticaster) {
			LinkedList listenerList = getListenerList(eventListener);

			Iterator it = listenerList.iterator();
			while (it.hasNext()) {
				WeakReference weakReference = (WeakReference)it.next();
				Object currListener = weakReference.get();
				if (currListener == null) {
					it.remove();
				} else if (currListener.equals(eventListenerToRemove)) {
					it.remove();
				}
			}

			if (listenerList.isEmpty())
				return null;
			else if (listenerList.size() == 1)
				return (EventListener) ((WeakReference)listenerList.get(0)).get();
			else
				return eventListener;

		} else {
			return eventListener;
		}
	}

	private static LinkedList getListenerList(EventListener eventListener) {
		return ((AbstractListenerMulticaster)eventListener).getListenerList();
	}

	private final LinkedList getListenerList() {
		return _listener;
	}

	protected Iterator getListeners() {
		// rainer: inserted clonde because i got concurrentModificationExceptions in some cases
		return ((List)_listener.clone()).iterator();
	}

	private void writeObject(ObjectOutputStream out) throws IOException {
		out.writeInt(getListenerList().size());
		Iterator it = getListeners();
		while (it.hasNext()) {
			WeakReference reference = (WeakReference)it.next();
			EventListener eventListener = (EventListener)reference.get();
			if (eventListener == null) {
				it.remove();
				out.writeObject(null);
			} else {
				out.writeObject(eventListener);
			}
		}
	}

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		_listener = new LinkedList();
		int size = in.readInt();
		for (int i = 0; i < size; i++) {
			Object currObj = in.readObject();
			if (currObj != null)
				getListenerList().add(new WeakReference(currObj));
		}
	}
}