package de.uniwue.ki.util.file;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

import junit.framework.Assert;
/**
 * Creation date: (30.01.2002 15:13:21)
 * @author Christoph Oechslein
 */
public class TokenReplacer {

	public static abstract class TokenReplaceConfig {
		protected final char _tokenChar;
		private final String _description;

		public TokenReplaceConfig(char tokenChar, String description) {
			_tokenChar = tokenChar;
			_description = description;
		}

		public String getDescription() {
			return _description;
		}

		public abstract String getReplaceString();
	}

	private final List _tokenReplaceConfigList;

	private TokenReplacer() {
		this(new LinkedList());

	}

	private TokenReplacer(List tokenReplaceConfigList) {
		super();

		_tokenReplaceConfigList = tokenReplaceConfigList;
	}

	public void addTokenReplaceConfig(TokenReplaceConfig tokenReplaceConfig) {
		_tokenReplaceConfigList.add(tokenReplaceConfig);
	}

	public static TokenReplacer createTokenReplacer() {
		return new TokenReplacer();
	}

	public static TokenReplacer createTokenReplacer(List TokenReplaceConigList) {
		return new TokenReplacer(TokenReplaceConigList);
	}

	private String getReplacementFor(char tokenChar) {
		String result = getTokenReplaceConfig(tokenChar).getReplaceString();

		Assert.assertNotNull(result);

		return result;
	}

	public String getTokenLegend(String example) {
		StringBuffer buff = new StringBuffer();

		buff.append("<html>" + "<h3>You can use the following tokens in the file name:</h3>" + "<ul>");

		Iterator it = _tokenReplaceConfigList.iterator();

		while (it.hasNext()) {
			TokenReplaceConfig tokenReplaceConfig = (TokenReplaceConfig) it.next();
			buff.append("<li>");
			buff.append('%');
			buff.append(tokenReplaceConfig._tokenChar);
			buff.append(" -> ");
			buff.append(tokenReplaceConfig.getDescription());
			buff.append("(E.g.: ");
			buff.append(tokenReplaceConfig.getReplaceString());
			buff.append(")");
		}

		buff.append(
			"<li>%D -> Normal Date, e.g. 22_03_2002"
				+ "<li>%T -> Normal Time, e.g. 20_15_36"
				+ "<li>%U -> User directory, e.g. c:\\Eigene Dateien"
				+ "<li>%L -> Local machine name, e.g. MyComputer"
				+ "<li>%t -> Local temp directory, e.g. c:\\temp\\"
				+ "<li>%Pproperty% -> Returns value of property, e.g. %Puser.home% is same as %U"
				+ "<li>%Y -> Year as 4-digit number, e.g.: 2002."
				+ "<li>%y -> Year as 2-digit number, e.g.: 02."
				+ "<li>%M -> Month as string, e.g.: January."
				+ "<li>%m -> Month as number, e.g.: 01."
				+ "<li>%d -> Day as number, e.g.: 01."
				+ "<li>%H -> Hour in 24 hours (0-23)."
				+ "<li>%h -> Hour in am/pm (0-11)."
				+ "<li>%S -> Minute."
				+ "<li>%s -> Seconds."
				+ "<li>%% -> %"
				+ "</ul>");

		if (example != null)
			buff.append(example);

		buff.append("</html>");

		return buff.toString();
	}

	private TokenReplaceConfig getTokenReplaceConfig(char tokenChar) {
		Iterator it = _tokenReplaceConfigList.iterator();

		while (it.hasNext()) {
			TokenReplaceConfig currTokenReplaceConfig = (TokenReplaceConfig) it.next();

			if (currTokenReplaceConfig._tokenChar == tokenChar)
				return currTokenReplaceConfig;
		}

		return null;
	}

	private boolean isAuxToken(char tokenChar) {
		return getTokenReplaceConfig(tokenChar) != null;
	}

	private void replaceApostropheAndAppend(
		String fileStringToConvert,
		int startIndex,
		int endIndex,
		StringBuffer buff,
		boolean lastWasNoToken) {

		if (startIndex < endIndex) {
			if (lastWasNoToken && (buff.length() > 0)) {
				buff.deleteCharAt(buff.length() - 1);
			} else {
				buff.append('\'');
			}

			for (int i = startIndex; i < endIndex; i++) {
				char c = fileStringToConvert.charAt(i);
				if (c != '\'')
					buff.append(c);
				else
					buff.append("''");

			}
			buff.append('\'');
		}
	}

	private void replaceApostropheAndAppend(
		String fileStringToConvert,
		int startIndex,
		StringBuffer buff,
		boolean lastWasNoToken) {
		replaceApostropheAndAppend(
			fileStringToConvert,
			startIndex,
			fileStringToConvert.length(),
			buff,
			lastWasNoToken);
	}

	private void replaceApostropheAndAppend(
		String fileStringToConvert,
		StringBuffer buff,
		boolean lastWasNoToken) {
		replaceApostropheAndAppend(fileStringToConvert, 0, buff, lastWasNoToken);
	}

	public String replaceTokens(String fileStringToConvert) {
		return replaceTokens(fileStringToConvert, new Date());
	}

	public String replaceTokens(String fileStringToConvert, Date date) {
		int index = 0;
		int oldIndex = 0;
		int size = fileStringToConvert.length();
		boolean lastWasNoToken = true;

		StringBuffer simpleDateFormatBuffer = new StringBuffer();
		while (true) {
			oldIndex = index;
			index = fileStringToConvert.indexOf('%', index);
			if (index == -1)
				break;

			replaceApostropheAndAppend(fileStringToConvert, oldIndex, index, simpleDateFormatBuffer, lastWasNoToken);

			junit.framework.Assert.assertEquals(fileStringToConvert.charAt(index), '%');
			if (index + 1 == size) {
				oldIndex = index;
				lastWasNoToken = true;
				break;

			} else {
				char tokenChar = fileStringToConvert.charAt(index + 1);

				switch (tokenChar) {
					case 'Y' :
						simpleDateFormatBuffer.append("yyyy");
						lastWasNoToken = false;
						break;
					case 'y' :
						simpleDateFormatBuffer.append("yy");
						lastWasNoToken = false;
						break;
					case 'M' :
						simpleDateFormatBuffer.append("MMMM");
						lastWasNoToken = false;
						break;
					case 'm' :
						simpleDateFormatBuffer.append("M");
						lastWasNoToken = false;
						break;
					case 'd' :
						simpleDateFormatBuffer.append("dd");
						lastWasNoToken = false;
						break;
					case 'H' :
						simpleDateFormatBuffer.append("HH");
						lastWasNoToken = false;
						break;
					case 'h' :
						simpleDateFormatBuffer.append("KK");
						lastWasNoToken = false;
						break;
					case 'S' :
						simpleDateFormatBuffer.append("mm");
						lastWasNoToken = false;
						break;
					case 's' :
						simpleDateFormatBuffer.append("ss");
						lastWasNoToken = false;
						break;
					case 'T' :
						simpleDateFormatBuffer.append("HH'_'mm'_'ss");
						lastWasNoToken = false;
						break;
					case 'D' :
						simpleDateFormatBuffer.append("dd'_'M'_'yyyy");
						lastWasNoToken = false;
						break;
					case 'L' :
						try {
							replaceApostropheAndAppend(InetAddress.getLocalHost().getHostName(), simpleDateFormatBuffer, lastWasNoToken);
						} catch(UnknownHostException e) {
						}
						lastWasNoToken = true;
						break;
					case 'U' :
						replaceApostropheAndAppend(System.getProperty("user.home"), simpleDateFormatBuffer, lastWasNoToken);
						lastWasNoToken = true;
						break;
					case 't' :
						replaceApostropheAndAppend(System.getProperty("java.io.tmpdir"), simpleDateFormatBuffer, lastWasNoToken);
						lastWasNoToken = true;
						break;
					case 'P' :
						int endPropertyIndex = fileStringToConvert.indexOf('%', index + 2);
						if (endPropertyIndex == -1) {
							lastWasNoToken = true;
						} else {
							String propertyName = fileStringToConvert.substring(index + 2, endPropertyIndex);
							replaceApostropheAndAppend(System.getProperty(propertyName), simpleDateFormatBuffer, lastWasNoToken);
							index = endPropertyIndex - 1;
							lastWasNoToken = true;
						}
						break;
					case '%' :
						replaceApostropheAndAppend("%", simpleDateFormatBuffer, lastWasNoToken);
						lastWasNoToken = true;
						break;

					default :
						if (isAuxToken(tokenChar)) {
							String replacedText = getReplacementFor(tokenChar);
							replaceApostropheAndAppend(replacedText, simpleDateFormatBuffer, lastWasNoToken);
							lastWasNoToken = true;
							break;

						} else {
							replaceApostropheAndAppend(fileStringToConvert, index, index + 2, simpleDateFormatBuffer, lastWasNoToken);
							lastWasNoToken = true;
							break;
						}
				}
			}

			index += 2;
		}

		replaceApostropheAndAppend(fileStringToConvert, oldIndex, size, simpleDateFormatBuffer, lastWasNoToken);

		String simpleDateFormatStr = simpleDateFormatBuffer.toString();

		/*
		System.out.println(simpleDateFormatStr);
		System.out.println(new java.text.SimpleDateFormat(simpleDateFormatStr).format(new java.util.Date()));
		System.out.println();
		*/

		return new java.text.SimpleDateFormat(simpleDateFormatStr).format(date);
	}

	/**
	 * @param stringToReplace String
	 * @return String
	 */
	public static String replaceTokensStatic(String stringToReplace) {
		return createTokenReplacer().replaceTokens(stringToReplace);
	}

	/**
	 * @param stringToReplace String
	 * @param date Date
	 * @return String
	 */
	public static String replaceTokensStatic(String stringToReplace, Date date) {
		return createTokenReplacer().replaceTokens(stringToReplace, date);
	}
}