package de.uniwue.ki.util.file;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;

import junit.framework.Assert;
import junit.framework.TestCase;
import junit.swingui.TestRunner;

/**
 * Creation date: 04.09.2002, 18:59:29
 * @author oechslein
 */
public class TokenReplacerTest extends TestCase {
	/**
	 * @param name String
	 */
	public TokenReplacerTest(String name) {
		super(name);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
		TestRunner.run(TokenReplacerTest.class);
	}

	public void testReplaceTokens() {
		Date currDate = new Date();

		Assert.assertEquals(
			TokenReplacer.replaceTokensStatic("c:\\mein pfad\\%H:%S.%s - %m. %M %Y", currDate),
			new java.text.SimpleDateFormat("'c:\\mein pfad\\'HH':'mm'.'ss' - 'M'. 'MMMM' 'yyyy").format(currDate));

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("%'", currDate), "%'");

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("%", currDate), "%");

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("bla foo bla%", currDate), "bla foo bla%");

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("bla %qfoo bla%q", currDate), "bla %qfoo bla%q");

		String localHostName = "";
		try {
			localHostName = InetAddress.getLocalHost().getHostName();
		} catch (UnknownHostException e) {
		}

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("%L", currDate), localHostName);

		Assert.assertEquals(TokenReplacer.replaceTokensStatic("%U", currDate), System.getProperty("user.home"));

		Iterator it = System.getProperties().entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry element = (Map.Entry) it.next();

			Assert.assertEquals(
				TokenReplacer.replaceTokensStatic("%P" + element.getKey() + "%", currDate),
				element.getValue());
		}
	}

	public void testReplaceDoku() {
		String example = "EXAMPLE";
		String legend = TokenReplacer.createTokenReplacer().getTokenLegend(example);

		Assert.assertTrue(legend.matches(".*" + example + ".*"));
	}
}