package de.uniwue.ki.util.file;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import de.uniwue.ki.util.Utils;

/**
 * Creation date: 19.09.2002, 18:47:38
 * @author oechslein
 */
public final class ZipUtilities {
	public static void main(String args[]) {
		try {
			test();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void test() throws FileNotFoundException, IOException {
		File zipFile = new File("c:\\temp\\test.zip");

		zip(zipFile, new File("c:\\temp\\"));

		File toDir = new File("c:\\temp\\foo\\");

		toDir.mkdirs();
		unzip(zipFile, toDir);
	}

	public static void zip(File zipFile, File fileOrDirToZip)
		throws FileNotFoundException, ZipException, IOException {
		zip(openZipOutputStream(zipFile), zipFile, fileOrDirToZip, null);
	}

	public static void zip(File zipFile, Iterator fileOrDirectories)
		throws FileNotFoundException, ZipException, IOException {
		zip(openZipOutputStream(zipFile), zipFile, fileOrDirectories, null);
	}

	public static void zip(ZipOutputStream zipOutputStream, File fileOrDirToZip)
		throws ZipException, IOException {
		zip(zipOutputStream, null, fileOrDirToZip, null);
	}

	public static void zip(File zipFile, File fileOrDirToZip, File rootDir)
		throws FileNotFoundException, ZipException, IOException {
		zip(openZipOutputStream(zipFile), zipFile, fileOrDirToZip, rootDir);
	}

	public static void zip(File zipFile, Iterator fileOrDirectories, File rootDir)
		throws FileNotFoundException, ZipException, IOException {
		zip(openZipOutputStream(zipFile), zipFile, fileOrDirectories, rootDir);
	}

	public static void zip(ZipOutputStream zipOutputStream, File fileOrDirToZip, File rootDir)
		throws ZipException, IOException {
		zip(zipOutputStream, null, fileOrDirToZip, rootDir);
	}

	public static void unzip(InputStream is, File pathToExtract)
		throws FileNotFoundException, ZipException, IOException {
		File tempFile = File.createTempFile("temp", ".zip");
		tempFile.deleteOnExit();

		FileUtilities.copyInputStreamToOutputStream(is, new FileOutputStream(tempFile));

		unzip(tempFile, pathToExtract);

		tempFile.delete();
	}

	public static void unzip(File zipFile, File pathToExtract) throws ZipException, IOException {

		ZipFile realZipFile = new ZipFile(zipFile);

		Enumeration enum = realZipFile.entries();
		while (enum.hasMoreElements()) {
			ZipEntry entry = (ZipEntry) enum.nextElement();
			InputStream is = realZipFile.getInputStream(entry);

			File currFile = FileUtilities.repairBadFile(new File(pathToExtract, entry.getName()));

			// looks like not always exception is thrown
			//			try {
			//				currFile = currFile.getCanonicalFile();
			//			} catch (IOException e) {
			//				// System.out.println("Repaired file from " + currFile + " to " + FileUtilities.repairBadFile(currFile));
			//				currFile = FileUtilities.repairBadFile(currFile);
			//			}

			if (currFile.getParentFile() != null)
				currFile.getParentFile().mkdirs();

			FileUtilities.copyInputStreamToOutputStream(is, new FileOutputStream(currFile));
		}

		realZipFile.close();
	}

	public static List list(File zipFile) throws FileNotFoundException, ZipException, IOException {

		List files = new LinkedList();

		ZipFile realZipFile = new ZipFile(zipFile);

		Enumeration enum = realZipFile.entries();
		while (enum.hasMoreElements()) {
			ZipEntry entry = (ZipEntry) enum.nextElement();

			files.add(new File(entry.getName()));
		}

		return files;
	}

	private static void zip(ZipOutputStream zipOutputStream, File zipFile, File fileOrDirToZip, File rootDir)
		throws ZipException, IOException {

		addZipEntry(zipFile, zipOutputStream, fileOrDirToZip, rootDir);

		zipOutputStream.finish();
		zipOutputStream.close();
	}

	private static void zip(
		ZipOutputStream zipOutputStream,
		File zipFile,
		Iterator fileOrDirectories,
		File rootDir)
		throws ZipException, IOException {

		while (fileOrDirectories.hasNext()) {
			addZipEntry(zipFile, zipOutputStream, (File) fileOrDirectories.next(), rootDir);
		}

		zipOutputStream.finish();
		zipOutputStream.close();
	}

	private static ZipOutputStream openZipOutputStream(File outputFile) throws FileNotFoundException {
		FileOutputStream fileOutputStream = new FileOutputStream(outputFile);

		ZipOutputStream zipOutputStream = new ZipOutputStream(fileOutputStream);

		return zipOutputStream;
	}

	private static void addZipEntry(
		File zipFile,
		ZipOutputStream zipOutputStream,
		File inputFile,
		File rootDir)
		throws FileNotFoundException, ZipException, IOException {

		if (Utils.equals(zipFile, inputFile))
			return;

		if (inputFile.isDirectory()) {
			File[] childs = inputFile.listFiles();
			for (int i = 0; i < childs.length; i++) {
				addZipEntry(zipFile, zipOutputStream, childs[i], rootDir);
			}

		} else {
			ZipEntry zipEntry;
			if (rootDir != null) {
				zipEntry =
					new ZipEntry(
						FileUtilities.createFileNameWithoutRoot(
							new File(FileUtilities.createRelativeFileStringIfPossible(inputFile, rootDir))));

			} else {
				zipEntry = new ZipEntry(FileUtilities.createFileNameWithoutRoot(inputFile));
			}

			zipEntry.setSize(inputFile.length());

			zipOutputStream.putNextEntry(zipEntry);

			FileInputStream fileInputStream = new FileInputStream(inputFile);
			FileUtilities.copyInputStreamToOutputStream(fileInputStream, zipOutputStream, false);

			fileInputStream.close();
			zipOutputStream.closeEntry();
		}
	}
}
