package de.uniwue.ki.util.hash;
import java.io.Serializable;
import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.util.Utils;

/**
 * Creation date: (09.02.2001 17:33:08)
 * @author Christoph Oechslein
 */
public class IsomorphicHashTableWrapper extends AbstractMap implements Map, Serializable {
	protected final Map _delegateMap;
	private Set _entrySet;
	private Set _keySet;
	private static final class IsomorphicWrapper implements Serializable {
		private IIsomorphicHashable _object;
		private static IsomorphicWrapper _cachedIsomorphicWrapper = null;

		public static final void unUseIsomorphicWrapper(IsomorphicWrapper wrapper) {
			if (wrapper != null)
				_cachedIsomorphicWrapper = wrapper;
		}

		public static final IsomorphicWrapper createIsomorphicWrapper(IIsomorphicHashable key) {
			if (_cachedIsomorphicWrapper == null) {
				return new IsomorphicWrapper(key);

			} else {
				IsomorphicWrapper isomorphicWrapper = _cachedIsomorphicWrapper;

				_cachedIsomorphicWrapper = null;

				isomorphicWrapper._object = key;

				return isomorphicWrapper;
			}
		}

		private IsomorphicWrapper(IIsomorphicHashable object) {
			super();

			_object = object;
		}

		public boolean equals(Object anObject) {
			if (anObject instanceof IsomorphicWrapper)
				return getIsomorphicHashable().isomorphicEquals(
					((IsomorphicWrapper) anObject).getIsomorphicHashable());
			else
				return false;
		}

		public int hashCode() {
			return getIsomorphicHashable().isomorphicHashCode();
		}

		public final IIsomorphicHashable getIsomorphicHashable() {
			return _object;
		}
	}

	public IsomorphicHashTableWrapper() {
		this(new HashMap());
	}

	public IsomorphicHashTableWrapper(Map delegateMap) {
		super();

		this._delegateMap = delegateMap;
	}

	public void clear() {
		Iterator it = _delegateMap.keySet().iterator();
		while (it.hasNext()) {
			IsomorphicWrapper.unUseIsomorphicWrapper((IsomorphicWrapper) it.next());
		}

		_delegateMap.clear();
	}

	public boolean containsKey(Object key) {

		IsomorphicWrapper wrapper = createWrapper(key);

		boolean result = _delegateMap.containsKey(wrapper);

		IsomorphicWrapper.unUseIsomorphicWrapper(wrapper);

		return result;
	}

	public boolean containsValue(Object value) {
		return _delegateMap.containsValue(value);
	}

	private final IsomorphicWrapper createWrapper(Object key) {
		if (key == null)
			return null;

		else if (key instanceof IIsomorphicHashable)
			return IsomorphicWrapper.createIsomorphicWrapper((IIsomorphicHashable) key);

		else {
			Assert.fail();
			return null;
		}
	}

	public Set entrySet() {
		if (_entrySet == null) {
			_entrySet = new AbstractSet() {
				public Iterator iterator() {
					return new Iterator() {
						private Iterator i = _delegateMap.entrySet().iterator();

						public boolean hasNext() {
							return i.hasNext();
						}

						public Object next() {
							final Entry currEntry = (Entry) i.next();
							final IsomorphicWrapper currWrapper = (IsomorphicWrapper) currEntry.getKey();

							return new Entry() {
								public Object getKey() {
									return currWrapper.getIsomorphicHashable();
								}
								public Object getValue() {
									return currEntry.getValue();
								}
								public Object setValue(Object newValue) {
									return currEntry.setValue(newValue);
								}
								public boolean equals(Object anObject) {
									if (!(anObject instanceof Map.Entry))
										return false;
									Map.Entry anEntry = (Map.Entry) anObject;

									return Utils.equals(getKey(), anEntry.getKey())
										&& Utils.equals(getValue(), anEntry.getValue());
								}
								public int hashCode() {
									return currEntry.hashCode();
								}
							};
						}

						public void remove() {
							i.remove();
						}
					};
				}

				public int size() {
					return size();
				}

				public boolean contains(Object k) {
					return containsKey(k);
				}
			};
		}
		return _entrySet;
	}

	public Object get(Object key) {
		if (key == null)
			return _delegateMap.get(null);
		else {
			IsomorphicWrapper wrapper = createWrapper(key);

			Object result = _delegateMap.get(wrapper);

			IsomorphicWrapper.unUseIsomorphicWrapper(wrapper);

			return result;
		}
	}

	public boolean isEmpty() {
		return _delegateMap.isEmpty();
	}

	public Set keySet() {
		if (_keySet == null) {
			_keySet = new AbstractSet() {
				public Iterator iterator() {
					return new Iterator() {
						private Iterator i = _delegateMap.entrySet().iterator();

						public boolean hasNext() {
							return i.hasNext();
						}

						public Object next() {
							final Entry currEntry = (Entry) i.next();
							final IsomorphicWrapper currWrapper = (IsomorphicWrapper) currEntry.getKey();
							return currWrapper.getIsomorphicHashable();
						}

						public void remove() {
							i.remove();
						}
					};
				}

				public int size() {
					return size();
				}

				public boolean contains(Object k) {
					return containsKey(k);
				}
			};
		}
		return _keySet;
	}

	public Object put(Object key, Object value) {
		if (key == null) {
			return _delegateMap.put(null, value);

		} else {
			// oldWrapper (if key already exists) is lost
			return _delegateMap.put(createWrapper(key), value);
		}
	}

	public Object remove(Object key) {
		IsomorphicWrapper wrapper = createWrapper(key);

		Object result = _delegateMap.remove(wrapper);

		IsomorphicWrapper.unUseIsomorphicWrapper(wrapper);

		return result;
	}

	public int size() {
		return _delegateMap.size();
	}

	public Collection values() {
		return _delegateMap.values();
	}
}