package de.uniwue.ki.util.hash;
import junit.framework.TestCase;
import junit.framework.TestSuite;/**
 * Creation date: (09.02.2001 17:53:09)
 * @author Christoph Oechslein
 */
public final class TestIsomorphicHashTableWrapper extends TestCase {
   private final TestPosition pos1 = new TestPosition(0, 0);
   private final TestPosition pos2 = new TestPosition(0, 1);
   private final TestPosition pos3 = new TestPosition(1, 0);
   private final TestPosition pos4 = new TestPosition(1, 1);   
   
   private static class TestPosition implements IIsomorphicHashable {
      private final int x;
      private final int y;

      /**
       * @param x int
       * @param y int
       */
      public TestPosition(int x, int y) {
         super();

         this.x = x;
         this.y = y;
      }

      /**
       * @param anObject IIsomorphicHashable
       * @return boolean
       * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicEquals(IIsomorphicHashable)
       */
      public boolean isomorphicEquals(IIsomorphicHashable anObject) {
         if (anObject instanceof TestPosition)
            return (x == ((TestPosition) anObject).x) && (y == ((TestPosition) anObject).y);
         else
            return false;
      }

      /**
       * @return int
       * @see de.uniwue.ki.util.hash.IIsomorphicHashable#isomorphicHashCode()
       */
      public int isomorphicHashCode() {
         return 3 * x + y + 1;
      }

      /**
       * @return String
       */
      public String toString() {
         return "[" + x + " " + y + "]";
      }
   }

	/**
	 * TestIsomorphicHashTableWrapper constructor comment.
	 * @param aName java.lang.String
	 */
	public TestIsomorphicHashTableWrapper(String aName) {
	    super(aName);
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
	   junit.swingui.TestRunner.run(TestIsomorphicHashTableWrapper.class);
	}

	/**
	 * Insert the method's description here.
	 * <BR>Creation date: (29.03.00 13:25:53)
	 * @return TestSuite
	 */
	public static TestSuite suite() {
	    TestSuite anSuite = new TestSuite("XML");
	
	    anSuite.addTest(
	        new TestIsomorphicHashTableWrapper("testHashTable"));
	
	    return anSuite;
	}

	/**
	 * @throws Exception
	 */
	public void testHashTable() throws Exception {
	    //testHashTable(new java.util.Hashtable());
	    testIsomorphicHashTable(new IsomorphicHashTableWrapper());
	}

	private void testHashTable(java.util.Map hashTable) throws Exception {
	    hashTable.put(pos1, pos1);
	    hashTable.put(pos2, pos2);
	    hashTable.put(pos3, pos3);
	    hashTable.put(pos4, pos4);
	
	    assertTrue(hashTable.containsKey(pos1));
	    assertTrue(hashTable.containsKey(pos2));
	    assertTrue(hashTable.containsKey(pos3));
	    assertTrue(hashTable.containsKey(pos4));
	    assertTrue(!hashTable.containsKey(null));
	
	    assertEquals(pos1, hashTable.get(pos1));
	    assertEquals(pos2, hashTable.get(pos2));
	    assertEquals(pos3, hashTable.get(pos3));
	    assertEquals(pos4, hashTable.get(pos4));
	    assertEquals(null, hashTable.get(new TestPosition(99, 99)));
	    assertEquals(null, hashTable.get(null));
	}

	private void testIsomorphicHashTable(java.util.Map hashTable) throws Exception {
	    testHashTable(hashTable);
	
	    assertTrue(hashTable.containsKey(new TestPosition(0, 0)));
	    assertTrue(hashTable.containsKey(new TestPosition(0, 1)));
	    assertTrue(hashTable.containsKey(new TestPosition(1, 0)));
	    assertTrue(hashTable.containsKey(new TestPosition(1, 1)));
	    assertTrue(!hashTable.containsKey(new TestPosition(99, 99)));
	
	    assertEquals(pos1, hashTable.get(new TestPosition(0, 0)));
	    assertEquals(pos2, hashTable.get(new TestPosition(0, 1)));
	    assertEquals(pos3, hashTable.get(new TestPosition(1, 0)));
	    assertEquals(pos4, hashTable.get(new TestPosition(1, 1)));
	}
}