/**
 * Created on 17.04.2003
 *
 * Kopiert von http://makeashorterlink.com/?W1C626E34 
 * 
 * @author glueckselig
 */
package de.uniwue.ki.util.jarclassloader;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
/**
 * Bietet die Mglichkeit, den System-ClassLoader dynamisch um weitere 
 * Pfade zu erweitern.
 * Der ClassLoaderExtender ist typischerweise nur was fr Desktop- oder Server-
 * Anwendungen. Im Applet-Bereich verbietet sich der Einsatz, da der 
 * SecurityManager verhindert, dass fremde URLs dem ClassLoader hinzugefgt 
 * werden knnen.
 *
 * @author Aljoscha Rittner
 */
public class ClassLoaderExtender {
	/**
	 * Sammelt aus einem Ordner alle Dateien mit der Endung jar und zip.
	 *
	 * @param extDir Ordner, in dem gesammelt werden soll.
	 * @return Array der URLs zu den Archiven oder null.
	 * @throws MalformedURLException Sollten die Archive nicht als URL zu 
	 *         reprsentieren sein, gibt es eine MalformedURLException.
	 */
	public static URL[] getArchives(File extDir) throws MalformedURLException {

		File[] jars = extDir.listFiles(new java.io.FilenameFilter() {
			public boolean accept(File dir, String name) {

				int extensionIndex = name.length() - 4;
				if (extensionIndex >= 0) {
					String extension = name.substring(extensionIndex).toLowerCase();
					return extension.equals(".jar") || extension.equals(".zip");

				} else {
					return false;
				}

			}
		});

		if ((jars != null) && (jars.length > 0)) {
			URL[] urls = new URL[jars.length];
			for (int i = 0; i < jars.length; i++) {
				urls[i] = jars[i].toURL();
			}

			return urls;

		} else {
			return new URL[] {};
		}
	}

	/**
	 * <p>
	 * Fgt das Array der URLs zu dem ClassPath des System-ClassLoaders hinzu.
	 * Es werden nur URLs hinzugefgt, die noch nicht im ClassPath existieren.
	 * </p>
	 * <p>
	 * Hinzufgen eines Ordners mit allen jar-Bibliotheken (hier der Relative 
	 * Ext-Ordner):<br>
	 * <code> addToClassPath (getArchives (new File ("ext/"));</code><br>
	 * </p>
	 * <p>
	 * Hinzufgen eines Ordners mit class-Dateien:<br>
	 * <code> addToClassPath (new URL[] {new File ("classes/").toURL()});</code><br>
	 * </p>
	 * @param  urls               Array der URLs, die dem ClassPath hinzugefgt 
	 *                            werden sollen.
	 * @throws RuntimeException   Wenn der System-ClassLoader kein URLClassLoader 
	 *                            ist, dann wird eine Runtime-Exception geworfen.
	 * @throws SecurityException  Wenn der SecurityManager den Aufruf von 
	 *                            protected Methoden verhindert, gibt es eine 
	 *                            SecurityException.
	 */
	public static void addToClassPath(URL[] urls) {

		ClassLoader loader = ClassLoaderExtender.class.getClassLoader();
		addToClassPath(urls, loader);
	}

	public static void addToClassPath(URL[] urls, ClassLoader loader) {
		Class loaderClazz = loader.getClass();
		Class urlLoaderClazz = getSuperClass(loaderClazz, "URLClassLoader");
		
		boolean extended = false;
		
		if (urlLoaderClazz != null) {
		
			try {
		
				Method addURL = urlLoaderClazz.getDeclaredMethod("addURL", new Class[] { URL.class });
				addURL.setAccessible(true);
		
				Method getURLs = urlLoaderClazz.getDeclaredMethod("getURLs", null);
				URL[] cp = (URL[]) getURLs.invoke(loader, null);
		
				for (int i = 0; i < urls.length; i++) {
		
					if (!isAlreadyInList(urls[i], cp))
						addURL.invoke(loader, new Object[] { urls[i] });
				}
		
				extended = true;
			} catch (NoSuchMethodException nsme) {
		
				nsme.printStackTrace();
			} catch (IllegalAccessException iae) {
		
				iae.printStackTrace();
			} catch (InvocationTargetException ite) {
		
				ite.printStackTrace();
			}
		}
		if (!extended) {
			throw new RuntimeException("Please extend your classpath with: " + buildCPDump(urls));
		}
	}

	private static boolean isAlreadyInList(URL url, URL[] urlList) {
		for (int j = 0; j < urlList.length; j++) {

			if (url.sameFile(urlList[j])) {
				return true;
			}
		}

		return false;
	}

	private static Class getSuperClass(Class clazz, String declaredClazz) {

		do {

			if (clazz.getName().endsWith(declaredClazz))
				return clazz;
		} while ((clazz = clazz.getSuperclass()) != null);

		return null;
	}

	private static String buildCPDump(URL[] urls) {

		int count = urls.length;
		if (count > 0) {

			StringBuffer b = new StringBuffer();
			for (int i = 0; i < (count - 1); i++) {

				b.append(urls[i].getPath()).append(';');
			}

			b.append(urls[count - 1].getPath());
			return b.toString();
		}

		return null;
	}
}
