/*
 * Created on 02.12.2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package de.uniwue.ki.util.list;

import java.util.ArrayList;

import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;
import de.uniwue.ki.util.name.INamed;

/**
 * @author herrler
 */
public class MuteableFilerableListModel extends AbstractMutableListModel {

	private ArrayList filteredList = null;

	private IPredicateFunction filter = null;
	private ListDataListener ldl = null;

	private final ArrayList _delegate;

	public MuteableFilerableListModel() {
		this(new ArrayList());
	}

	public MuteableFilerableListModel(int size) {
		this(new ArrayList(size));
	}

	public MuteableFilerableListModel(ArrayList delegate) {
		super();
		_delegate = delegate;
		filteredList = delegate;
	}

	public void add(int index, Object newObject) {
		_delegate.add(index, newObject);

		fireIntervalAdded(this, index, index);
	}

	private boolean reentrance= false;
	
	public void updateFilteredList() {
		if (reentrance) return;
		
		if (filter!= null) {
		filteredList = CollectionsHelper.asArrayList(CollectionsHelper.select(_delegate.iterator(), filter));
		}
		else filteredList=_delegate;
		
		reentrance= true;	
		fireContentsChanged(this,0,filteredList.size());
		reentrance=false;
	}

	public Object remove(int index) {
		return removeIntern(index);
	}

	public Object set(int index, Object newObject) {
		Object oldObject = get(index);

		_delegate.set(index, newObject);

		fireContentsChanged(this, index, index);

		return oldObject;
	}

	protected Object removeIntern(int index) {

		Object oldObject = get(index);

		_delegate.remove(index);

		fireIntervalRemoved(this, index, index);

		return oldObject;
	}

	public boolean accept(Object anObject) {
		return anObject instanceof INamed;
	}

	public void setFilter(IPredicateFunction function) {
		if (filter == function)
			return;

		//always just one filter
		if (filter != null)
			removeListDataListener(ldl);

		filter = function;
		ldl = new ListDataListener() {
			public void intervalAdded(ListDataEvent e) {
				updateFilteredList();
			}
			public void intervalRemoved(ListDataEvent e) {
				updateFilteredList();
			}
			public void contentsChanged(ListDataEvent e) {
				updateFilteredList();
			}
		};

		updateFilteredList();
		addListDataListener(ldl);
	}

	public Object get(int index) {
		return filteredList.get(index);
	}

	public int size() {
		return filteredList.size();
	}

}
