package de.uniwue.ki.util.math;
import java.awt.geom.Point2D;
import java.io.*;
import java.io.File;
import java.io.FileWriter;

import junit.framework.*;
/**
 * Creation date: (06.06.2001 18:56:15)
 * @author Christoph Oechslein
 */
public final class TestMathHelper extends TestCase {

	/**
	 * TestMathHelper constructor comment.
	 * @param aName java.lang.String
	 */
	public TestMathHelper(String aName) {
		super(aName);
	}

	protected static Point2D calculateIntersectionPoint(
		double lineAx1,
		double lineAy1,
		double lineAx2,
		double lineAy2,
		double lineBx1,
		double lineBy1,
		double lineBx2,
		double lineBy2) {

		Point2D point = new Point2D.Double();

		MathHelper.calculateIntersection(
			point,
			lineAx1,
			lineAy1,
			lineAx2,
			lineAy2,
			lineBx1,
			lineBy1,
			lineBx2,
			lineBy2);

		return point;
	}

	public static void main(String[] args) {
		junit.swingui.TestRunner.run(TestMathHelper.class);
	}

	public static Test suite() {
		TestSuite anSuite = new TestSuite("MathHelper");

		anSuite.addTest(new TestMathHelper("testIntersection"));
		anSuite.addTest(new TestMathHelper("testDouble"));

		return anSuite;
	}

	public void testDouble() throws Exception {
		assertTrue(MathHelper.doubleEqual(0, 0));
		assertTrue(MathHelper.doubleEqual(0, Double.MIN_VALUE));
		assertTrue(MathHelper.doubleEqual(1, 1 - Double.MIN_VALUE + Double.MIN_VALUE));

		assertFalse(MathHelper.doubleEqual(1, 0));
		assertFalse(MathHelper.doubleEqual(0.001, 0));

		assertTrue(MathHelper.doubleEqual(2.71828182845905, 2.71828182845904 + 0.00000000000001));
		assertTrue(MathHelper.doubleEqual(2.71828182845905, 2.71828182845904));
		assertTrue(MathHelper.doubleEqual(2.718281828454, 2.718281828455));

		assertFalse(MathHelper.doubleEqual(2.7182814, 2.7182815));
	}

	public void createDistributions() throws IOException {
		double lambda = 0.9;

		long AMOUNT = 1000000;
		double min = 0;
		double max = 5;
		double step = 0.1;
		
		double sum = 0;
		double sumSq = 0;

		double[] samples = new double[(int) Math.ceil((max - min) / step)];

		RandomNumberGenerator randomNumberGenerator = new RandomNumberGenerator();
		for (int i = 0; i < AMOUNT; i++) {
			double sample = randomNumberGenerator.nextExp(lambda);
			sum += sample;
			sumSq += sample * sample;
			int index = (int) Math.floor((sample - min) / step);
			if ((index >= 0) && (index < samples.length)) {
				samples[index]++;
			}
		}

		BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(new File("c:/temp/test.csv")));

		double i = min;
		
		double  mean = sum / AMOUNT;

		bufferedWriter.write(" ; " + mean + "; " + ((sumSq / AMOUNT) -  (mean * mean)) + " \n");
		bufferedWriter.write(" ; " + lambda + "; " + (lambda * lambda) + " \n");
		
		try {
			for (int index = 0; index < samples.length; index++) {
				bufferedWriter.write(i + " ; ");
				bufferedWriter.write(samples[index] + " ; ");
				bufferedWriter.write(
					(lambda * Math.exp(-lambda * i) + lambda * Math.exp(-lambda * (i + step))) / 2 + " \n");

				i += step;
			}

		} catch (ArrayIndexOutOfBoundsException e) {
		}

		bufferedWriter.close();
	}

	public void testIntersection() throws Exception {

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, 5 },
			new double[] { 0, 0 },
			new double[] { 5, 0 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, 1 },
			new double[] { 0, 0 },
			new double[] { 5, 0 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, -1 },
			new double[] { 0, 0 },
			new double[] { 99, 0 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, 2 },
			new double[] { 0, 0 },
			new double[] { -5, 0 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, -11 },
			new double[] { 0, 0 },
			new double[] { -27, 0 },
			new double[] { 0, 0 });

		testNoIntersection(
			new double[] { 0, 0 },
			new double[] { 0, -11 },
			new double[] { 0, -27 },
			new double[] { -27, 0 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 0, -11 },
			new double[] { 0, 0 },
			new double[] { -27, -27 },
			new double[] { 0, 0 });

		testIntersection(
			new double[] { 0, 0 },
			new double[] { 5, 5 },
			new double[] { 0, 5 },
			new double[] { 5, 0 },
			new double[] { 2.5, 2.5 });

		if (Assert.ASSERTION_ON)
			Assert.assertEquals(
				calculateIntersectionPoint(0, 0, 5, 5, 0, 5, 5, 0),
				new java.awt.geom.Point2D.Double(2.5, 2.5));

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.06.2001 18:59:03)
	 */
	protected void testIntersection(
		double[] x1A,
		double[] x2A,
		double[] x1B,
		double[] x2B,
		double[] intersectionPoint)
		throws Exception {

		Assert.assertEquals(
			calculateIntersectionPoint(x1A[0], x1A[1], x2A[0], x2A[1], x1B[0], x1B[1], x2B[0], x2B[1]),
			new Point2D.Double(intersectionPoint[0], intersectionPoint[1]));
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (06.06.2001 18:59:03)
	 */
	protected void testNoIntersection(
		double[] x1A,
		double[] x2A,
		double[] x1B,
		double[] x2B,
		double[] intersectionPoint)
		throws Exception {

		Assert.assertTrue(
			!calculateIntersectionPoint(
				x1A[0],
				x1A[1],
				x2A[0],
				x2A[1],
				x1B[0],
				x1B[1],
				x2B[0],
				x2B[1]).equals(
				new Point2D.Double(intersectionPoint[0], intersectionPoint[1])));
	}
}