package de.uniwue.ki.util.rectangular;

import java.io.Serializable;

import de.uniwue.ki.util.rectangular.event.*;

/**
 * Creation date: (04.12.2000 14:15:00)
 * @author Christoph Oechslein
 */
public abstract class AbstractDelegateRectangularMapModel extends AbstractRectangularMapModel {
	private IRectangularMapModel ivjRectangularMapModelIntern = null;

	private final IvjEventHandler ivjEventHandler = new IvjEventHandler();
	private final class IvjEventHandler
		implements
			RectangularMapChangedListener,
			RectangularMapObjectAddedListener,
			RectangularMapObjectRemovedListener,
			Serializable {

		public void mapChanged(RectangularMapChangedEvent event) {
			if (event.getSource() == AbstractDelegateRectangularMapModel.this.getRectangularMapModelIntern())
				fireMapChanged();
		}

		public void objectAdded(RectangularMapObjectAddedEvent event) {
			if (event.getSource() == AbstractDelegateRectangularMapModel.this.getRectangularMapModelIntern())
				rectangularMapModelInternObjectAdded(event);
		}

		public void objectRemoved(RectangularMapObjectRemovedEvent event) {
			if (event.getSource() == AbstractDelegateRectangularMapModel.this.getRectangularMapModelIntern())
				rectangularMapModelInternObjectRemoved(event);
		}
		/**
		 * @param event RectangularMapChangedEvent
		 * @see RectangularMapChangedListener#positionChanged(RectangularMapChangedEvent)
		 */
		public void positionChanged(RectangularMapChangedEvent event) {
			if (event.getSource() == AbstractDelegateRectangularMapModel.this.getRectangularMapModelIntern())
				rectangularMapModelInternPositionChanged(event);
		}
	}

	public AbstractDelegateRectangularMapModel() {
		super();
	}

	public AbstractDelegateRectangularMapModel(IRectangularMapModel rectangularMapModel) {
		super();

		setRectangularMapModel(rectangularMapModel);
	}

	public void addObject(int col, int row, Object anObject) {
		getRectangularMapModel().addObject(col, row, anObject);
	}

	public void changePosition(int oldCol, int oldRow, int newCol, int newRow, Object anObject) {
		getRectangularMapModel().changePosition(oldCol, oldRow, newCol, newRow, anObject);
	}

	public int getColsCount() {
		return getRectangularMapModel().getColsCount();
	}

	public Object getObjectAt(int col, int row, int index) {
		return getRectangularMapModel().getObjectAt(col, row, index);
	}

	public int getObjectCount(int col, int row) {
		return getRectangularMapModel().getObjectCount(col, row);
	}

	public java.util.Iterator getObjectsAt(int col, int row) {
		return getRectangularMapModel().getObjectsAt(col, row);
	}

	public IRectangularMapModel getRectangularMapModel() {
		return getRectangularMapModelIntern();
	}

	public int getRowsCount() {
		return getRectangularMapModel().getRowsCount();
	}

	public boolean isAdjustable() {
		return getRectangularMapModel().isAdjustable();
	}

	public void removeObjectAt(int col, int row, int index) {
		getRectangularMapModel().removeObjectAt(col, row, index);
	}

	public void setColsCount(int newColsCount) {
		int oldValue = getColsCount();
		getRectangularMapModel().setColsCount(newColsCount);
		firePropertyChange("colsCount", oldValue, newColsCount);
	}

	public void setRectangularMapModel(IRectangularMapModel arg1) {
		setRectangularMapModelIntern(arg1);
	}

	public void setRowsCount(int newRowsCount) {
		int oldValue = getRowsCount();
		getRectangularMapModel().setRowsCount(newRowsCount);
		firePropertyChange("rowsCount", oldValue, newRowsCount);
	}

	protected IRectangularMapModel getRectangularMapModelIntern() {
		return ivjRectangularMapModelIntern;
	}

	private void setRectangularMapModelIntern(IRectangularMapModel newValue) {
		if (ivjRectangularMapModelIntern != newValue) {
			IRectangularMapModel oldValue = getRectangularMapModelIntern();
			/* Stop listening for events from the current object */
			if (ivjRectangularMapModelIntern != null) {
				ivjRectangularMapModelIntern.removeRectangularMapChangedListener(ivjEventHandler);
				ivjRectangularMapModelIntern.removeRectangularMapObjectAddedListener(ivjEventHandler);
				ivjRectangularMapModelIntern.removeRectangularMapObjectRemovedListener(ivjEventHandler);
			}
			ivjRectangularMapModelIntern = newValue;

			/* Listen for events from the new object */
			if (ivjRectangularMapModelIntern != null) {
				ivjRectangularMapModelIntern.addRectangularMapChangedListener(ivjEventHandler);
				ivjRectangularMapModelIntern.addRectangularMapObjectAddedListener(ivjEventHandler);
				ivjRectangularMapModelIntern.addRectangularMapObjectRemovedListener(ivjEventHandler);
			}
			firePropertyChange("rectangularMapModel", oldValue, newValue);
		}
	}

	protected void rectangularMapModelInternObjectAdded(RectangularMapObjectAddedEvent event) {
		fireObjectAdded(event.getAddedObject(), event.getCol(), event.getRow());
	}

	protected void rectangularMapModelInternObjectRemoved(RectangularMapObjectRemovedEvent event) {
		fireObjectRemoved(event.getRemovedObject(), event.getCol(), event.getRow());
	}

	protected void rectangularMapModelInternPositionChanged(RectangularMapChangedEvent event) {
		firePositionChanged(event.getCol(), event.getRow());
	}
}
