package de.uniwue.ki.util.rectangular;
import java.io.Serializable;
import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: (15.11.2000 14:36:25)
 * @author Christoph Oechslein
 */
public class DefaultRectangularMapModel extends AbstractRectangularMapModel implements Serializable {

	/* List of Lists of lists */
	private final List _dataList = createNewList();

	private int _colsCount = 0;
	private int _rowsCount = 0;

	private boolean _torus = false;

	public DefaultRectangularMapModel() {
		super();
	}

	public DefaultRectangularMapModel(int colsCount, int rowsCount) {
		super();

		_colsCount = colsCount;
		_rowsCount = rowsCount;
	}


	public void addObject(int col, int row, Object anObject) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		if (addObjectIntern(col, row, anObject)) {
			fireObjectAdded(anObject, col, row);
			firePositionChanged(col, row);
			fireMapChanged();
		}
	}

	protected boolean addObjectIntern(int col, int row, Object anObject) {

		List list = getObjects(col, row, true);

		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(list);

		return list.add(anObject);
	}


	public void changePosition(int oldCol, int oldRow, int newCol, int newRow, Object anObject) {
		oldRow = getCorrectedRow(oldRow);
		oldCol = getCorrectedCol(oldCol);

		newRow = getCorrectedRow(newRow);
		newCol = getCorrectedCol(newCol);

		if ((oldRow != newRow) || (oldCol != newCol)) {

			boolean wasRemoved = removeObjectIntern(oldCol, oldRow, anObject);
			Assert.assertTrue(wasRemoved);

			boolean wasAdded = addObjectIntern(newCol, newRow, anObject);
			Assert.assertTrue(wasAdded);

			firePositionChanged(oldCol, oldRow);
			firePositionChanged(newCol, newRow);
			fireMapChanged();
		}
	}


	public Iterator getAllObjects() {
		if (_dataList == null) {
			return CollectionsHelper.EMPTY_ITERATOR;

		} else {
			return super.getAllObjects();

			/*
			Iterator objects = CollectionsHelper.EMPTY_ITERATOR;
			
			for (int i = 0; i < _dataList.size(); i++) {
			   List list = (List) _dataList.get(i);
			   if ((list == null) || (list.isEmpty())) {
			      objects = CollectionsHelper.concatenate(list.iterator(), objects);
			   }
			}
			
			return objects;
			*/

			// getAllObjects(startX,startY,endX,endY);

			/*
			IPredicateFunction isEmptyPredicateFunction = new IPredicateFunction() {
			   public boolean isElementOk(Object anObject) {
			      List list = (List) anObject;
			      return (list != null) && !list.isEmpty();
			   }
			};
			
			IMapFunction toIterator = new IMapFunction() {
			   public Object map(Object anObject) {
			      List list = (List) anObject;
			      return list.iterator();
			   }
			};
			
			return CollectionsHelper.flatten(
			   CollectionsHelper.flatten(_dataList.iterator(), isEmptyPredicateFunction, toIterator),
			   isEmptyPredicateFunction,
			   toIterator);
			}
			*/
		}
	}


	public Iterator getAllObjects(int startCol, int startRow, int endCol, int endRow) {
		if (_dataList == null) {
			return CollectionsHelper.EMPTY_ITERATOR;

		} else {
			return super.getAllObjects(startCol, startRow, endCol, endRow);
		}
	}


	public int getColsCount() {
		return _colsCount;
	}

	public Object getObjectAt(int col, int row, int index) {
		col = getCorrectedCol(col);
		row = getCorrectedRow(row);

		List list = getObjects(col, row, false);
		if (list == null)
			throw new RectangularMapIndexOutOfBoundException(col, row);
		else
			return list.get(index);
	}

	public int getObjectCount(int col, int row) {
		col = getCorrectedCol(col);
		row = getCorrectedRow(row);

		List list = getObjects(col, row, false);
		if (list == null)
			return 0;
		else
			return list.size();
	}

	private List getObjects(int rowIndex, int colIndex, boolean createIfDoesNotExist) {
		return getObjects(getRow(rowIndex, createIfDoesNotExist), colIndex, createIfDoesNotExist);
	}

	private List getObjects(List row, int colIndex, boolean createIfDoesNotExist) {
		if (row == null)
			return null;
		else
			return getObjectsFromList(row, colIndex, createIfDoesNotExist);
	}

	public Iterator getObjectsAt(int col, int row) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		List list = getObjects(col, row, false);
		if (list == null)
			return CollectionsHelper.EMPTY_ITERATOR;
		else
			return list.iterator();
	}

	private static List getObjectsFromList(List list, int index, boolean createIfDoesNotExist) {
		if (index < list.size()) {
			List currList = (List) list.get(index);
			if (currList == null && createIfDoesNotExist) {
				ArrayList newList = createNewList();
				list.set(index, newList);
				return newList;
			} else {
				return currList;
			}

		} else if (createIfDoesNotExist) {

			while (list.size() <= index) {
				list.add(null);
			}

			List newList = createNewList();
			list.set(index, newList);
			return newList;

		} else
			return null;

	}

	private final static ArrayList createNewList() {
		return new ArrayList();
	}

	private List getRow(int index, boolean createIfDoesNotExist) {
		return getObjectsFromList(_dataList, index, createIfDoesNotExist);
	}

	
	public int getRowsCount() {
		return _rowsCount;
	}


	public boolean isAdjustable() {
		return true;
	}

	public boolean isTorus() {
		return _torus;
	}

	
	public void removeObject(int col, int row, Object anObject) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		if (removeObjectIntern(col, row, anObject)) {
			fireObjectRemoved(anObject, col, row);
			firePositionChanged(col, row);
			fireMapChanged();
		}
	}

	
	public void removeObjectAt(int col, int row, int index) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		Object removedObject = removeObjectIntern(col, row, index);

		if (removedObject != null) {
			fireObjectRemoved(removedObject, col, row);
			firePositionChanged(col, row);
			fireMapChanged();
		}
	}

	private Object removeObjectIntern(int col, int row, int index) {
		List list = getObjects(col, row, false);
		if (list == null)
			return null;

		return list.remove(index);
	}

	private boolean removeObjectIntern(int col, int row, Object anObject) {
		List list = getObjects(col, row, false);
		if (list == null)
			return false;

		return list.remove(anObject);
	}

	private void removeObjectsInRect(int startCol, int startRow, int endColsCount, int endRowsCount) {
		for (int currentCol = startCol; currentCol < endColsCount; currentCol++) {
			for (int currentRow = startRow; currentRow < endRowsCount; currentRow++) {
				List list = getObjects(currentCol, currentRow, false);
				if (list != null && !list.isEmpty())
					list.clear();
			}
		}
	}

	
	public void setColsCount(int newColsCount) {
		int oldColsCount = _colsCount;

		if (newColsCount < oldColsCount)
			removeObjectsInRect(newColsCount, 0, oldColsCount, getRowsCount());

		_colsCount = newColsCount;

		firePropertyChange("colsCount", oldColsCount, newColsCount);
		fireMapChanged();
	}

	
	public void setRowsCount(int newRowsCount) {
		int oldRowsCount = _rowsCount;

		if (newRowsCount < oldRowsCount)
			removeObjectsInRect(0, newRowsCount, getColsCount(), oldRowsCount);

		_rowsCount = newRowsCount;

		firePropertyChange("rowsCount", oldRowsCount, newRowsCount);
		fireMapChanged();
	}

	
	public void setTorus(boolean torus) {
		boolean oldValue = _torus;

		_torus = torus;

		firePropertyChange("torus", Boolean.valueOf(oldValue), Boolean.valueOf(torus));
	}
	
	public void remove() {
		_dataList.clear();
				
		super.remove();
	}
}