package de.uniwue.ki.util.rectangular;
import java.util.*;

import junit.framework.Assert;

import de.uniwue.ki.util.collection.CollectionsHelper;
/**
 * Creation date: (20.12.2001 11:29:48)
 * @author Christoph Oechslein
 */
public class DefaultStaticRectangularMapModel extends AbstractRectangularMapModel {
	private static final int START_SIZE_OF_PATCH_LIST = 2;
	private final boolean _torus;
	private final int _colsCount, _rowsCount;
	private final List[][] _array;

	public DefaultStaticRectangularMapModel(int colsCount, int rowsCount, boolean torus) {
		super();

		_colsCount = colsCount;
		_rowsCount = rowsCount;
		_torus = torus;

		_array = new List[_colsCount][_rowsCount];
	}

	public void addObject(int col, int row, Object anObject) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		getList(col, row, true).add(anObject);

		fireObjectAdded(anObject, col, row);
		firePositionChanged(col, row);
		fireMapChanged();
	}

	public boolean contains(Object anObject) {
		for (int i = 0; i < getColsCount(); i++) {
			for (int j = 0; j < getRowsCount(); j++) {
				List list = getList(i, j, false);
				if ((list != null) && (list.contains(anObject))) {
					return true;
				}
			}
		}
		return false;
	}

	public void changePosition(int oldCol, int oldRow, int newCol, int newRow, Object anObject) {
		oldRow = getCorrectedRow(oldRow);
		oldCol = getCorrectedCol(oldCol);

		newRow = getCorrectedRow(newRow);
		newCol = getCorrectedCol(newCol);

		if ((oldRow != newRow) || (oldCol != newCol)) {

			boolean result = getList(oldCol, oldRow, false).remove(anObject);

			if (Assert.ASSERTION_ON && !result) {
				if (contains(anObject)) {
					Assert.fail("Object wasn't on old position!");
				} else {
					Assert.fail("Object isn't in the whole map!");
				}
			}

			getList(newCol, newRow, true).add(anObject);

			firePositionChanged(oldCol, oldRow);
			firePositionChanged(newCol, newRow);
			fireMapChanged();
		}
	}

	public final int getColsCount() {
		return _colsCount;
	}

	private List getList(int col, int row, boolean shouldCreate) {
		List list = _array[col][row];

		if (list != null) {
			return list;

		} else if (shouldCreate) {
			list = new ArrayList(START_SIZE_OF_PATCH_LIST);

			_array[col][row] = list;

			return list;

		} else {
			return null;
		}
	}

	public Object getObjectAt(int col, int row, int index) {
		return getList(col, row, false).get(index);
	}

	public int getObjectCount(int col, int row) {
		List list = getList(col, row, false);
		if (list == null)
			return 0;
		else
			return list.size();
	}

	public Iterator getObjectsAt(int col, int row) {
		List list = getList(col, row, false);
		if (list == null)
			return CollectionsHelper.EMPTY_ITERATOR;
		else
			return list.iterator();
	}

	public final int getRowsCount() {
		return _rowsCount;
	}

	public boolean isAdjustable() {
		return false;
	}

	public final boolean isTorus() {
		return _torus;
	}

	public void removeObject(int col, int row, Object anObject) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		getList(col, row, false).remove(anObject);

		fireObjectRemoved(anObject, col, row);
		firePositionChanged(col, row);
		fireMapChanged();
	}

	public void removeObjectAt(int col, int row, int index) {
		row = getCorrectedRow(row);
		col = getCorrectedCol(col);

		Object oldObject = getList(col, row, false).remove(index);

		fireObjectRemoved(oldObject, col, row);
		firePositionChanged(col, row);
		fireMapChanged();
	}

	public void setColsCount(int newColsCount) {
		throw new UnsupportedOperationException();
	}

	public void setRowsCount(int newRowsCount) {
		throw new UnsupportedOperationException();
	}

	public void setTorus(boolean newSetting) {
		throw new UnsupportedOperationException();
	}

	public void remove() {
		for (int i = 0; i < _array.length; i++) {
			for (int j = 0; j < _array[i].length; j++) {
				_array[i][j] = null;
			}
		}

		super.remove();
	}
}