package de.uniwue.ki.util.rectangular;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import de.uniwue.ki.util.Position2D;
import de.uniwue.ki.util.collection.CollectionsHelper;/**
 * Creation date: (04.12.2000 11:11:26)
 * @author Christoph Oechslein
 */
public final class TestRectangularMapModel extends TestCase {

	/**
	 * TestRectangularMapModel constructor comment.
	 * @param aName java.lang.String
	 */
	public TestRectangularMapModel(String aName) {
	   super(aName);
	}

	private void checkCount(IRectangularMapModel rectangularMapModel, int col, int row, int count) {
	   assertEquals("Check ObjectCount", count, rectangularMapModel.getObjectCount(col, row));
	
	   assertEquals(
	      "Check ObjectCount (via Iterator)",
	      count,
	      CollectionsHelper.size(rectangularMapModel.getObjectsAt(col, row)));
	}

	/**
	 * @param args String[]
	 */
	public static void main(String[] args) {
	   junit.swingui.TestRunner.run(TestRectangularMapModel.class);
	}

	/**
	 * @return TestSuite
	 */
	public static TestSuite suite() {
	   TestSuite testSuite = new TestSuite("RectangularMapModel");
	
	   testSuite.addTest(new TestRectangularMapModel("testDefaultRectangularMapModel"));
	   testSuite.addTest(new TestRectangularMapModel("testPartRectangularMapModel"));
	
	   return testSuite;
	}

	public void testDefaultRectangularMapModel() {
	   DefaultRectangularMapModel defaultRectangularMapModel = new DefaultRectangularMapModel(3, 4);
	
	   assertEquals("ColsCount", 3, defaultRectangularMapModel.getColsCount());
	   assertEquals("RowsCount", 4, defaultRectangularMapModel.getRowsCount());
	
	   testRectangularMapModel(defaultRectangularMapModel);
	}

	public void testPartRectangularMapModel() {
	   testPartRectangularMapModel(new DefaultRectangularMapModel(4, 5), 0, 0);
	   testPartRectangularMapModel(new DefaultRectangularMapModel(4, 5), 0, 1);
	   testPartRectangularMapModel(new DefaultRectangularMapModel(4, 5), 1, 0);
	   testPartRectangularMapModel(new DefaultRectangularMapModel(4, 5), 1, 1);
	}

	private void testPartRectangularMapModel(IRectangularMapModel rectangularMapModel, int startCol, int startRow) {
	
	   PartRectangularMapModel partRectangularMapModel = new PartRectangularMapModel(rectangularMapModel);
	
	   assertEquals("ColsCount", rectangularMapModel.getColsCount(), partRectangularMapModel.getColsCount());
	   assertEquals("RowsCount", rectangularMapModel.getRowsCount(), partRectangularMapModel.getRowsCount());
	
	   partRectangularMapModel.setStartCol(startCol);
	   partRectangularMapModel.setStartRow(startRow);
	
	   assertEquals("ColsCount", rectangularMapModel.getColsCount() - startCol, partRectangularMapModel.getColsCount());
	   assertEquals("RowsCount", rectangularMapModel.getRowsCount() - startRow, partRectangularMapModel.getRowsCount());
	
	   testRectangularMapModel(partRectangularMapModel);
	
	}

	private void testRectangularMapModel(IRectangularMapModel rectangularMapModel) {
	
	   Position2D position0_0 = new Position2D(0, 0);
	   Position2D position0_0_2 = new Position2D(0, 0);
	   Position2D position2_3 = new Position2D(2, 3);
	
	   rectangularMapModel.addObject(0, 0, position0_0);
	   rectangularMapModel.addObject(0, 0, position0_0_2);
	   rectangularMapModel.addObject(2, 3, position2_3);
	
	   assertTrue(
	      "All objects",
	      CollectionsHelper.isSetEqual(
	         CollectionsHelper.createIterator(position0_0, position0_0_2, position2_3),
	         rectangularMapModel.getAllObjects()));
	
	   assertTrue("Part of all objects", CollectionsHelper.isEmpty(rectangularMapModel.getAllObjects(1, 1, 2, 2)));
	
	   assertEquals("Object on place", position0_0, rectangularMapModel.getObjectAt(0, 0, 0));
	   assertEquals("Object on place", position0_0_2, rectangularMapModel.getObjectAt(0, 0, 1));
	   assertEquals("Object on place", position2_3, rectangularMapModel.getObjectAt(2, 3, 0));
	
	   for (int col = 0; col < rectangularMapModel.getColsCount(); col++) {
	      for (int row = 0; row < rectangularMapModel.getRowsCount(); row++) {
	         if (col == 0 && row == 0) {
	            checkCount(rectangularMapModel, 0, 0, 2);
	         } else if (col == 2 && row == 3)
	            checkCount(rectangularMapModel, 2, 3, 1);
	         else
	            checkCount(rectangularMapModel, col, row, 0);
	      }
	
	   }
	
	   rectangularMapModel.changePosition(0, 0, 1, 1, position0_0_2);
	
	   assertEquals("Object on place", position0_0_2, rectangularMapModel.getObjectAt(1, 1, 0));
	   checkCount(rectangularMapModel, 0, 0, 1);
	   checkCount(rectangularMapModel, 1, 1, 1);
	
	   rectangularMapModel.removeObject(1, 1, position0_0_2);
	   checkCount(rectangularMapModel, 1, 1, 0);
	
	   if (rectangularMapModel.isAdjustable()) {
	
	      int oldColsCount = rectangularMapModel.getColsCount();
	      int oldRowsCount = rectangularMapModel.getRowsCount();
	      rectangularMapModel.setColsCount(1);
	      rectangularMapModel.setRowsCount(1);
	
	      assertEquals("ColsCount", 1, rectangularMapModel.getColsCount());
	      assertEquals("RowsCount", 1, rectangularMapModel.getRowsCount());
	
	      checkCount(rectangularMapModel, 0, 0, 1);
	
	      rectangularMapModel.setColsCount(oldColsCount);
	      rectangularMapModel.setRowsCount(oldRowsCount);
	
	      assertEquals("ColsCount", oldColsCount, rectangularMapModel.getColsCount());
	      assertEquals("RowsCount", oldRowsCount, rectangularMapModel.getRowsCount());
	
	      checkCount(rectangularMapModel, 0, 0, 1);
	      checkCount(rectangularMapModel, 1, 1, 0);
	      checkCount(rectangularMapModel, 2, 3, 0);
	   }
	
	}
}