package de.uniwue.ki.util.rectangular.event;/**
 * This is the event multicaster class to support the RectangularMapChangedListenerinterface.
 * @author oechslein
 */
public class RectangularMapChangedEventMulticaster
	extends java.awt.AWTEventMulticaster
	implements RectangularMapChangedListener {

	/**
	 * Constructor to support multicast events.
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected RectangularMapChangedEventMulticaster(java.util.EventListener a, java.util.EventListener b) {
	    super(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 * @param a RectangularMapChangedListener
	 * @param b RectangularMapChangedListener
	 * @return RectangularMapChangedListener
	 */
	public static RectangularMapChangedListener add(RectangularMapChangedListener a, RectangularMapChangedListener b) {
	    return (RectangularMapChangedListener) addInternal(a, b);
	}

	/**
	 * Add new listener to support multicast events.
	 * @return java.util.EventListener
	 * @param a java.util.EventListener
	 * @param b java.util.EventListener
	 */
	protected static java.util.EventListener addInternal(java.util.EventListener a, java.util.EventListener b) {
	    if (a == null)
	        return b;
	    if (b == null)
	        return a;
	    return new RectangularMapChangedEventMulticaster(a, b);
	}

	/**
	 *
	 * @param event RectangularMapChangedEvent
	 * @see de.uniwue.ki.util.rectangular.event.RectangularMapChangedListener#mapChanged(RectangularMapChangedEvent)
	 */
	public void mapChanged(RectangularMapChangedEvent event) {
	    ((RectangularMapChangedListener) a).mapChanged(event);
	    ((RectangularMapChangedListener) b).mapChanged(event);
	}

	/**
	 *
	 * @param event RectangularMapChangedEvent
	 * @see de.uniwue.ki.util.rectangular.event.RectangularMapChangedListener#positionChanged(RectangularMapChangedEvent)
	 */
	public void positionChanged(RectangularMapChangedEvent event) {
	    ((RectangularMapChangedListener) a).positionChanged(event);
	    ((RectangularMapChangedListener) b).positionChanged(event);
	}

	/**
	 *
	 * @return java.util.EventListener
	 * @param oldl RectangularMapChangedListener
	 */
	protected java.util.EventListener remove(RectangularMapChangedListener oldl) {
	    if (oldl == a)
	        return b;
	    if (oldl == b)
	        return a;
	    java.util.EventListener a2 = removeInternal(a, oldl);
	    java.util.EventListener b2 = removeInternal(b, oldl);
	    if (a2 == a && b2 == b)
	        return this;
	    return addInternal(a2, b2);
	}

	/**
	 * Remove listener to support multicast events.
	 * @param l RectangularMapChangedListener
	 * @param oldl RectangularMapChangedListener
	 * @return RectangularMapChangedListener
	 */
	public static RectangularMapChangedListener remove(
	    RectangularMapChangedListener l,
	    RectangularMapChangedListener oldl) {
	    if (l == oldl || l == null)
	        return null;
	    if (l instanceof RectangularMapChangedEventMulticaster)
	        return (RectangularMapChangedListener) ((RectangularMapChangedEventMulticaster) l).remove(oldl);
	    return l;
	}
}