package de.uniwue.ki.util.table;

import java.io.Serializable;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.jfree.ui.SortableTableModel;

import de.uniwue.ki.util.IRemoveable;

/**
 * @author oechslein
 * 
 * Creation date: 11.06.2003
 */
public class DelegateSortableTableModel extends SortableTableModel implements IRemoveable {
	private final class MyTableModelListener implements TableModelListener, Serializable {
		public void tableChanged(TableModelEvent e) {
			delegateTableChanged(e);
		}
	}

	private final MyTableModelListener _myTableModelListener = new MyTableModelListener();

	protected final TableModel _delegate;

	public DelegateSortableTableModel(TableModel delegate) {
		_delegate = delegate;
		delegate.addTableModelListener(_myTableModelListener);
	}

	public Class getColumnClass(int columnIndex) {
		return _delegate.getColumnClass(columnIndex);
	}

	public int getColumnCount() {
		return _delegate.getColumnCount();
	}

	public String getColumnName(int columnIndex) {
		return _delegate.getColumnName(columnIndex);
	}

	public int getRowCount() {
		return _delegate.getRowCount();
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return _delegate.getValueAt(rowIndex, columnIndex);
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return _delegate.isCellEditable(rowIndex, columnIndex);
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		_delegate.setValueAt(aValue, rowIndex, columnIndex);
	}

	public void remove() {
		_delegate.removeTableModelListener(_myTableModelListener);
	}

	protected void delegateTableChanged(TableModelEvent e) {
		//			fireTableDataChanged();
		fireTableChanged(
			new TableModelEvent(
				DelegateSortableTableModel.this,
				e.getFirstRow(),
				e.getLastRow(),
				e.getColumn(),
				e.getType()));
	}

	public void sortByColumn(int column, boolean ascending) {
		if (_delegate instanceof SortableTableModel) {
			((SortableTableModel)_delegate).sortByColumn(column, ascending);
		}
	}

	public boolean isSortable(int column) {
		if (_delegate instanceof SortableTableModel)
			return ((SortableTableModel)_delegate).isSortable(column);
		else
			return false;
	}

	public int getSortingColumn() {
		if (_delegate instanceof SortableTableModel)
			return ((SortableTableModel)_delegate).getSortingColumn();
		else
			return super.getSortingColumn();
	}

	public boolean getAscending() {
		if (_delegate instanceof SortableTableModel)
			return ((SortableTableModel)_delegate).getAscending();
		else
			return super.getAscending();
	}

	public void setAscending(boolean flag) {
		if (_delegate instanceof SortableTableModel)
			 ((SortableTableModel)_delegate).setAscending(flag);
		else
			super.setAscending(flag);
	}
}
