package de.uniwue.ki.util.table;

import gnu.trove.TIntHashSet;

import javax.swing.event.TableModelEvent;
import javax.swing.table.TableModel;

import junit.framework.Assert;

import de.uniwue.ki.util.IRemoveable;

/**
 * @author oechslein
 * 
 * Creation date: 10.06.2003
 */
public class DisableColumnsTableModel extends DelegateSortableTableModel implements TableModel, IRemoveable {
	private final TIntHashSet _disabledColumns;

	public DisableColumnsTableModel(TableModel delegate) {
		super(delegate);

		_disabledColumns = new TIntHashSet(delegate.getColumnCount());
	}

	private int convertColumnIndexFromDelegate(int delegateIndex) {
		Assert.assertFalse(_disabledColumns.contains(delegateIndex));
		int columnIndex = 0;
		for (int i = 0; i < delegateIndex; i++) {
			if (!_disabledColumns.contains(i))
				columnIndex++;
		}
		return columnIndex;
	}

	private int convertColumnIndexToDelegate(int oldColumnIndex) {
		int columnIndex = 0;
		int delegateIndex = 0;
		while (columnIndex <= oldColumnIndex) {
			if (!_disabledColumns.contains(delegateIndex))
				columnIndex++;

			if (columnIndex <= oldColumnIndex)
				delegateIndex += 1;
		}

		Assert.assertEquals(oldColumnIndex, convertColumnIndexFromDelegate(delegateIndex));

		return delegateIndex;
	}

	public Class getColumnClass(int columnIndex) {
		return _delegate.getColumnClass(convertColumnIndexToDelegate(columnIndex));
	}

	public int getColumnCount() {
		return _delegate.getColumnCount() - _disabledColumns.size();
	}

	public String getColumnName(int columnIndex) {
		return _delegate.getColumnName(convertColumnIndexToDelegate(columnIndex));
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		Assert.assertTrue(columnIndex < getColumnCount());
		return _delegate.getValueAt(rowIndex, convertColumnIndexToDelegate(columnIndex));
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return _delegate.isCellEditable(rowIndex, convertColumnIndexToDelegate(columnIndex));
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		_delegate.setValueAt(aValue, rowIndex, convertColumnIndexToDelegate(columnIndex));
	}

	public void setDisabled(int delegateColumnIndex, boolean disabled) {
		if (disabled)
			_disabledColumns.add(delegateColumnIndex);
		else
			_disabledColumns.remove(delegateColumnIndex);

		fireTableDataChanged();
	}

	protected void delegateTableChanged(TableModelEvent e) {
		if (!_disabledColumns.contains(e.getColumn()))
			//			fireTableDataChanged();
			fireTableChanged(
				new TableModelEvent(
					DisableColumnsTableModel.this,
					e.getFirstRow(),
					e.getLastRow(),
					convertColumnIndexFromDelegate(e.getColumn()),
					e.getType()));
	}

	public void sortByColumn(int column, boolean ascending) {
		super.sortByColumn(convertColumnIndexToDelegate(column), ascending);
	}

	public boolean isSortable(int column) {
		return super.isSortable(convertColumnIndexToDelegate(column));
	}
}
