package de.uniwue.ki.util.tree;

import javax.swing.tree.TreeNode;

import de.uniwue.ki.util.name.AbstractNamed;
import de.uniwue.ki.util.name.INamed;

/**
 * Creation date: (07.02.2002 15:11:24)
 * @author Christoph Oechslein
 */
public abstract class AbstractObjectTreeNode extends AbstractNamed implements IObjectTreeNode, INamed {
	private final Object _userObject;
	private final IObjectTreeNode _parent;

	public AbstractObjectTreeNode(IObjectTreeNode parent, Object userObject) {
		super();

		_parent = parent;
		_userObject = userObject;
	}

	public String getName() {
		if (getUserObject() instanceof INamed)
			return ((INamed) getUserObject()).getName();
		else
			return getUserObject() + "";
	}

	public TreeNode getParent() {
		return _parent;
	}

	public Object getUserObject() {
		return _userObject;
	}


	public TreeNode[] getPath() {
		return getPathToRoot(this, 0);
	}

	protected TreeNode[] getPathToRoot(TreeNode aNode, int depth) {
		TreeNode[] retNodes;

		/* Check for null, in case someone passed in a null node, or
		   they passed in an element that isn't rooted at root. */
		if (aNode == null) {
			if (depth == 0)
				return null;
			else
				retNodes = new TreeNode[depth];
		} else {
			depth++;
			retNodes = getPathToRoot(aNode.getParent(), depth);
			retNodes[retNodes.length - depth] = aNode;
		}
		return retNodes;
	}
}