/*
 * @(#)AbstractTreeTableModel.java	1.2 98/10/27
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

package de.uniwue.ki.util.tree;
import javax.swing.tree.TreePath;

/**
 * @version 1.2 10/27/98
 * An abstract implementation of the TreeTableModel interface, handling the list 
 * of listeners. 
 * @author Philip Milne
 */

public abstract class AbstractTreeTableModel extends AbstractTreeModel implements TreeTableModel {
	private Object _root;

	private int _sortingColum = -1;
	private boolean _ascending = true;

	public AbstractTreeTableModel() {
	}

	public AbstractTreeTableModel(Object root) {
		setRoot(root);
	}

	//
	// Default implmentations for methods in the TreeModel interface. 
	//


	public Object getRoot() {
		return _root;
	}

	public void setRoot(Object root) {
		_root = root;
	}


	public boolean isLeaf(Object node) {
		return getChildCount(node) == 0;
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
	}

	// This is not called in the JTree's default mode: use a naive implementation. 
	public int getIndexOfChild(Object parent, Object child) {
		for (int i = 0; i < getChildCount(parent); i++) {
			if (getChild(parent, i) == child) {
				return i;
			}
		}
		return -1;
	}

	//
	// Default impelmentations for methods in the TreeTableModel interface. 
	//

	public Class getColumnClass(int column) {
		return Object.class;
	}

	/** By default, make the column with the Tree in it the only editable one. 
	 *  Making this column editable causes the JTable to forward mouse 
	 *  and keyboard events in the Tree column to the underlying JTree. 
	 */
	public boolean isCellEditable(Object node, int column) {
		return getColumnClass(column) == TreeTableModel.class;
	}

	public void setValueAt(Object aValue, Object node, int column) {
	}

	public boolean getAscending() {
		return _ascending;
	}

	public int getSortingColumn() {
		return _sortingColum;
	}

	public boolean isSortable(int sortable) {
		return false;
	}

	public void setAscending(boolean ascending) {
		_ascending = ascending;
	}

	public void sortByColumn(int col, boolean ascending) {
	}

	// Left to be implemented in the subclass:

	/* 
	 *   public Object getChild(Object parent, int index)
	 *   public int getChildCount(Object parent) 
	 *   public int getColumnCount() 
	 *   public String getColumnName(Object node, int column)  
	 *   public Object getValueAt(Object node, int column) 
	 */
}
