package de.uniwue.ki.util.tree;
import java.util.*;

import javax.swing.tree.TreeNode;

import de.uniwue.ki.util.collection.CollectionsHelper;

/**
 * Creation date: (07.02.2002 15:11:24)
 * @author Christoph Oechslein
 */
public class DefaultObjectTreeNode extends AbstractObjectTreeNode {
	private final List _childrenList;

	public DefaultObjectTreeNode(IObjectTreeNode parent, Object userObject) {
		this(parent, userObject, new LinkedList());
	}

	public DefaultObjectTreeNode(IObjectTreeNode parent, Object userObject, Iterator childrens) {
		this(parent, userObject, CollectionsHelper.asLinkedList(childrens));
	}

	public DefaultObjectTreeNode(IObjectTreeNode parent, Object userObject, List childrenList) {
		super(parent, userObject);

		_childrenList = childrenList;
	}

	public void addChildrens(Iterator it) {
		CollectionsHelper.addAll(_childrenList, it);
	}

	public void addChild(IObjectTreeNode child) {
		_childrenList.add(child);
	}

	public void removeChild(IObjectTreeNode child) {
		child.remove();
		_childrenList.remove(child);
	}

	public void removeChildren() {
		Iterator it = iterator();

		while (it.hasNext()) {
			IObjectTreeNode element = (IObjectTreeNode) it.next();
			element.remove();
			it.remove();
		}
	}

	public Enumeration children() {
		return CollectionsHelper.asEnumeration(_childrenList.iterator());
	}

	public Iterator iterator() {
		return _childrenList.iterator();
	}

	public boolean getAllowsChildren() {
		return true;
	}

	public TreeNode getChildAt(int childIndex) {
		return (IObjectTreeNode) _childrenList.get(childIndex);
	}

	public int getChildCount() {
		return _childrenList.size();
	}

	public int getIndex(TreeNode node) {
		return _childrenList.indexOf(node);
	}

	public boolean isLeaf() {
		return false;
	}

	public void remove() {
		removeChildren();
	}
}