package junit.framework;

public class Assert {
	public static boolean ASSERTION_ON = true;

	/**
	 * Protect constructor since it is a static only class
	 */
	protected Assert() {
	}

	/**
	 * Asserts that two bytes are equal.
	 */
	public static final void assertEquals(byte expected, byte actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two chars are equal.
	 */
	public static final void assertEquals(char expected, char actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two doubles are equal concerning a delta. If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public static final void assertEquals(double expected, double actual, double delta) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual, delta);
		}

	}

	/**
	 * Asserts that two floats are equal concerning a delta. If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public static final void assertEquals(float expected, float actual, float delta) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual, delta);
		}

	}

	/**
	 * Asserts that two ints are equal.
	 */
	public static final void assertEquals(int expected, int actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two longs are equal.
	 */
	public static final void assertEquals(long expected, long actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two objects are equal. If they are not
	 * an AssertionFailedError is thrown.
	 */
	public static final void assertEquals(Object expected, Object actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two Strings are equal. 
	 */
	static public void assertEquals(String message, String expected, String actual) {
		if (expected == null && actual == null)
			return;
		if (expected != null && expected.equals(actual))
			return;
		throw new ComparisonFailure(message, expected, actual);
	}

	/**
	 * Asserts that two Strings are equal. 
	 */
	static public void assertEquals(String expected, String actual) {
		assertEquals(null, expected, actual);
	}

	/**
	 * Asserts that two bytes are equal.
	 */
	public static final void assertEquals(String message, byte expected, byte actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, new Byte(expected), new Byte(actual));
		}
	}

	/**
	 * Asserts that two chars are equal.
	 */
	public static final void assertEquals(String message, char expected, char actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, new Character(expected), new Character(actual));
		}
	}

	/**
	 * Asserts that two doubles are equal concerning a delta. If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public static final void assertEquals(String message, double expected, double actual, double delta) {
		if (ASSERTION_ON) {
			// handle infinity specially since subtracting to infinite values gives NaN and the
			// the following test fails
			if (Double.isInfinite(expected)) {
				if (!(expected == actual))
					failNotEquals(message, new Double(expected), new Double(actual));

			} else if (
				!(Math.abs(expected - actual) <= delta)) // Because comparison with NaN always returns false
				failNotEquals(message, new Double(expected), new Double(actual));
		}
	}

	/**
	 * Asserts that two floats are equal concerning a delta. If the expected
	 * value is infinity then the delta value is ignored.
	 */
	public static final void assertEquals(String message, float expected, float actual, float delta) {
		if (ASSERTION_ON) {
			// handle infinity specially since subtracting to infinite values gives NaN and the
			// the following test fails
			if (Float.isInfinite(expected)) {
				if (!(expected == actual))
					failNotEquals(message, new Float(expected), new Float(actual));

			} else if (!(Math.abs(expected - actual) <= delta))
				failNotEquals(message, new Float(expected), new Float(actual));
		}
	}

	/**
	 * Asserts that two ints are equal.
	 */
	public static final void assertEquals(String message, int expected, int actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, new Integer(expected), new Integer(actual));
		}
	}

	/**
	 * Asserts that two longs are equal.
	 */
	public static final void assertEquals(String message, long expected, long actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, new Long(expected), new Long(actual));
		}
	}

	/**
	 * Asserts that two objects are equal. If they are not
	 * an AssertionFailedError is thrown.
	 */
	public static final void assertEquals(String message, Object expected, Object actual) {
		if (ASSERTION_ON) {
			if (expected == null && actual == null)
				return;
			if (expected != null && expected.equals(actual))
				return;
			failNotEquals(message, expected, actual);
		}

	}

	/**
	 * Asserts that two shorts are equal.
	 */
	public static final void assertEquals(String message, short expected, short actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, new Short(expected), new Short(actual));
		}
	}

	/**
	 * Asserts that two booleans are equal.
	 */
	public static final void assertEquals(String message, boolean expected, boolean actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotEquals(message, Boolean.valueOf(expected), Boolean.valueOf(actual));
		}
	}

	/**
	 * Asserts that two shorts are equal.
	 */
	public static final void assertEquals(short expected, short actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that two booleans are equal.
	 */
	public static final void assertEquals(boolean expected, boolean actual) {
		if (ASSERTION_ON) {
			assertEquals(null, expected, actual);
		}

	}

	/**
	 * Asserts that an object isn't null.
	 */
	public static final void assertNotNull(Object object) {
		if (ASSERTION_ON) {
			assertNotNull(null, object);
		}
	}

	/**
	 * Asserts that an object isn't null.
	 */
	public static final void assertNotNull(String message, Object object) {
		if (ASSERTION_ON) {
			assertTrue(message, object != null);
		}
	}

	/**
	 * Asserts that an object is null.
	 */
	public static final void assertNull(Object object) {
		if (ASSERTION_ON) {
			assertNull(null, object);
		}

	}

	/**
	 * Asserts that an object is null.
	 */
	public static final void assertNull(String message, Object object) {
		if (ASSERTION_ON) {
			assertTrue(message, object == null);
		}

	}

	/**
	 * Asserts that two objects refer to the same object. If they are not
	 * the same an AssertionFailedError is thrown.
	 */
	public static final void assertSame(Object expected, Object actual) {
		if (ASSERTION_ON) {
			assertSame(null, expected, actual);
		}

	}

	/**
	 * Asserts that two objects refer to the same object. If they are not
	 * an AssertionFailedError is thrown.
	 */
	public static final void assertSame(String message, Object expected, Object actual) {
		if (ASSERTION_ON) {
			if (expected != actual)
				failNotSame(message, expected, actual);
		}
	}

	/**
	  * Asserts that two objects refer to the same object. If they are not
	  * an AssertionFailedError is thrown with the given message.
	  */
	static public void assertNotSame(String message, Object expected, Object actual) {
		if (expected == actual)
			failSame(message, expected, actual);
	}

	/**
	 * Asserts that two objects refer to the same object. If they are not
	 * the same an AssertionFailedError is thrown.
	 */
	static public void assertNotSame(Object expected, Object actual) {
		assertNotSame(null, expected, actual);
	}

	static private void failSame(String message, Object expected, Object actual) {
		String formatted = "";
		if (message != null)
			formatted = message + " ";
		fail(formatted + "expected not same");
	}
	/**
	* Asserts that a condition is true. If it isn't it throws
	* an AssertionFailedError with the given message.
	*/
	public static final void assertTrue(String message, boolean condition) {
		if (ASSERTION_ON) {
			if (!condition)
				fail(message);
		}
	}

	/**
	 * Asserts that a condition is false. If it isn't it throws
	 * an AssertionFailedError with the given message.
	 */
	public static final void assertFalse(String message, boolean condition) {
		if (ASSERTION_ON) {
			if (condition)
				fail(message);
		}
	}

	/**
	 * Asserts that a condition is true. If it isn't it throws
	 * an AssertionFailedError.
	 */
	public static final void assertTrue(boolean condition) {
		if (ASSERTION_ON) {
			assertTrue(null, condition);
		}
	}

	/**
	 * Asserts that a condition is false. If it isn't it throws
	 * an AssertionFailedError.
	 */
	public static final void assertFalse(boolean condition) {
		if (ASSERTION_ON) {
			assertFalse(null, condition);
		}
	}

	/**
	 * Fails a test with no message.
	 */
	public static final void fail() {
		if (ASSERTION_ON) {
			fail(null);
		}
	}

	/**
	 * Fails a test with the given message.
	 */
	public static final void fail(String message) {
		if (ASSERTION_ON) {
			throw new AssertionFailedError(message);
		}
	}

	private static void failNotEquals(String message, Object expected, Object actual) {
		StringBuffer buff = new StringBuffer();
		if (message != null) {
			buff.append(message);
			buff.append(" expected:<");

		} else {
			buff.append("expected:<");
		}

		buff.append(expected);
		buff.append("> but was:<");
		buff.append(actual);
		buff.append(">");

		fail(buff.toString());
	}

	private static void failNotSame(String message, Object expected, Object actual) {
		if (message != null) {
			fail(message + " expected same");

		} else {
			fail("expected same");
		}
	}

	public static final void setAssertionFlag(boolean value) {
		ASSERTION_ON = value;
	}

	public static final boolean isAssertionFlagSettable() {
		return true;
	}
}