package mygale.mcge.cmd;



import mygale.mcge.core.*;
import mygale.mcge.fig.IFigLayered;
import mygale.mcge.layer.LayerManager;
import mygale.mcge.sel.Selection;
import mygale.mcge.throwable.MCGEInternalError;

/** Cmd to bring forward a
 *  {@link mygale.mcge.fig.Fig figure}.
 *
  */
public class CmdBringForward extends CmdMoveFigToFront {



/** This method is called each time the command wans to run itself.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
public void doIt() {
   JGraph graph = getGraph();

   LayerManager layer = graph.getLayerManager();
   Selection sel = graph.getSelection();
   boolean bInform = sel.setCmdInform(false);
   clearUndoChanges();
   if (sel.size() == 1) {
      IFigLayered figure = (IFigLayered) sel.getAllFigs().next();
      int oldDepth = layer.getDepthLevel(figure);
      layer.bringForward(figure);
      saveChange(figure, oldDepth, IFigLayered.DEPTH_FRONT);
   } else {
      throw new MCGEInternalError("try to bring to front an invalid selection.");
   }
   graph.damaged(sel);
   sel.setCmdInform(bInform);
   sel.informCmd();
   addUndoCmd();
}

/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "bring forward the selection" ;
    }

/** Reply if the given size corresponds to
     *  an valid selection size.
     *
     * @param size the size to test.
     * @return <code>true</code> if the given size is
     *         a valid size for the current command.
     */
    protected boolean isCorrectSize( int size ) { return size == 1; }


    /** Construct a new CmdBringForward.
     *
     * @param editor a reference to the current editor.
     * @param iconSize int
 */
    public CmdBringForward(JGraphEditor editor, int iconSize) {
    super(editor, "Bring Forward", iconSize, "general/MoveFront" );
    }
}
