package mygale.mcge.cmd;



import java.util.Iterator;

import mygale.mcge.core.*;
import mygale.mcge.fig.IFigLayered;
import mygale.mcge.layer.LayerManager;
import mygale.mcge.sel.Selection;
import mygale.mcge.throwable.MCGEInternalError;

/** Cmd to put to the front a Fig.
 *
  */
public class CmdBringToFront extends CmdMoveFigToFront {



/** This method is called each time the command wans to run itself.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
public void doIt() {
   JGraph graph = getGraph();

   LayerManager layer = graph.getLayerManager();
   Selection sel = graph.getSelection();
   boolean bInform = sel.setCmdInform(false);
   clearUndoChanges();
   switch (sel.size()) {
      case 0 :
         throw new MCGEInternalError("try to bring to front an empty selection.");
      case 1 :
         IFigLayered figure = (IFigLayered) sel.getAllFigs().next();
         int oldDepth = layer.getDepthLevel(figure);
         layer.bringToFront(figure);
         saveChange(figure, oldDepth, layer.getDepthLevel(figure));
         break;
      default :
         Iterator it = sel.getAllFigs();
         int depths[] = new int[sel.size()];
         for (int i = 0; it.hasNext(); i++) {
            figure = (IFigLayered) it.next();
            depths[i] = layer.getDepthLevel(figure);
         }
         layer.bringToFront(sel.getAllFigs());
         it = sel.getAllFigs();
         for (int i = 0; it.hasNext(); i++) {
            figure = (IFigLayered) it.next();
            saveChange(figure, depths[i], layer.getDepthLevel(figure));
         }
   }
   graph.damaged(sel);
   sel.setCmdInform(bInform);
   sel.informCmd();
   addUndoCmd();
}

/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "bring to front the selection" ;
    }

/** Reply if the given size corresponds to
     *  an valid selection size.
     *
     * @param size the size to test.
     * @return <code>true</code> if the given size is
     *         a valid size for the current command.
     */
    protected boolean isCorrectSize( int size ) {
    return size >= 1;
    }


    /** Construct a new CmdBringToFront.
     *
     * @param editor a reference to the current editor.
     * @param iconSize int
 */
    public CmdBringToFront(JGraphEditor editor, int iconSize) {
    super(editor, "Bring To Front", iconSize, "general/BringToFront" );
    }
}
