package mygale.mcge.cmd;

import mygale.mcge.core.JGraph;
import mygale.mcge.core.JGraphEditor;
import mygale.mcge.fig.IFigEdge;
import mygale.mcge.fig.IFigEdgeFactory;
import mygale.mcge.mode.ModeCreateEdge;
import mygale.mcge.net.INetEdge;
import mygale.mcge.net.INetEdgeFactory;

/** Editor Cmd to create a new
 *  {@link mygale.mcge.fig.FigEdge Edge}
 *  on a new
 *  {@link mygale.mcge.net.NetNode NetNode}.

 */
public class CmdCreateEdge extends CmdCreate {

	protected final INetEdgeFactory _netEdgeFactory;

	protected final IFigEdgeFactory _figEdgeFactory;

	/** This method is called each time the command wans to run itself.
	*
	* @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	*/
	public void doIt() {
		INetEdge netEdge = _netEdgeFactory.createNetEdge();
		JGraph graph = getGraph();
		IFigEdge figEdge;
		if (_figEdgeFactory == null)
			figEdge = graph.createFigEdge(netEdge);
		else
			figEdge = graph.createFigEdge(netEdge, _figEdgeFactory);
		graph.beginMode(new ModeCreateEdge(graph.getModeManager(), figEdge, netEdge));
	}

	public CmdCreateEdge(
		JGraphEditor editor,
		String name,
		String tooltips,
		int iconSize,
		String iconName,
		INetEdgeFactory netEdgeFactory) {

		this(editor, name, tooltips, iconSize, iconName, netEdgeFactory, null);
	}

	public CmdCreateEdge(
		JGraphEditor editor,
		String name,
		String tooltips,
		int iconSize,
		String iconName,
		INetEdgeFactory netEdgeFactory,
		IFigEdgeFactory figEdgeFactory) {

		super(editor, name, tooltips, iconSize, iconName);

		_netEdgeFactory = netEdgeFactory;
		_figEdgeFactory = figEdgeFactory;
	}
}
