package mygale.mcge.cmd;

import mygale.mcge.core.JGraph;
import mygale.mcge.core.JGraphEditor;
import mygale.mcge.fig.IFigNode;
import mygale.mcge.fig.IFigNodeFactory;
import mygale.mcge.mode.ModeCreateNode;
import mygale.mcge.net.INetNode;
import mygale.mcge.net.INetNodeFactory;

/** Editor Cmd to create a new FigNode on a new NetNode. When
 *  this Cmd is executed it makes the new objects as per its
 *  arguments, and then it sets the global next mode to ModePlace so
 *  that the user can place the new node in any editor window.
 *
 */
public class CmdCreateNode extends CmdCreate {

	/** Name of the Net that must be created.
	 */
	protected final INetNodeFactory _netNodeFactory;

	protected final IFigNodeFactory _figNodeFactory;

	/** Perform whatever Cmd this Cmd is meant to do. Subclasses
	 *  should override this to do whatever is intended.
	 *
	 * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	 */
	public void doIt() {
		INetNode netNode = _netNodeFactory.createNetNode();
		JGraph graph = getGraph();
		IFigNode figNode;
		if (_figNodeFactory == null)
			figNode = graph.createFigNode(netNode);
		else
			figNode = graph.createFigNode(netNode, _figNodeFactory);
		graph.beginMode(new ModeCreateNode(graph.getModeManager(), figNode, netNode));
	}

	/** Construct a new Cmd with the given arguments for Fig class.
	     *
	     * @param editor a reference to the current editor.
	     * @param name the name of this command.
	     * @param tooltip the tooltips for this command
	     * @param iconSize int
	 * @param iconName the name of the attached icon.
	     * @param netNodeFactory INetNodeFactory
	 */
	public CmdCreateNode(
		JGraphEditor editor,
		String name,
		String tooltip,
		int iconSize,
		String iconName,
		INetNodeFactory netNodeFactory) {

		this(editor, name, tooltip, iconSize, iconName, netNodeFactory, null);
	}

	/** Construct a new Cmd with the given arguments for Fig class.
	     *
	     * @param editor a reference to the current editor.
	     * @param name the name of this command.
	     * @param tooltip the tooltips for this command
	     * @param iconSize int
	 * @param iconName the name of the attached icon.
	     * @param netNodeFactory INetNodeFactory
	 * @param figNodeFactory the Fig that must be created.
	     */
	public CmdCreateNode(
		JGraphEditor editor,
		String name,
		String tooltip,
		int iconSize,
		String iconName,
		INetNodeFactory netNodeFactory,
		IFigNodeFactory figNodeFactory) {

		super(editor, name, tooltip, iconSize, iconName);

		_netNodeFactory = netNodeFactory;
		_figNodeFactory = figNodeFactory;
	}
}
