package mygale.mcge.cmd;

import mygale.mcge.core.*;

/**
 * This abstract class permits to define commands that are relationship with a
 * {@link mygale.mcge.core.JGraphEditor graph editor}.
 * <p>
 * <center>
 * <img src="doc-files/CmdGraphEditor_uml.gif" width="93" height="137" border="0" usemap="#umlclick">
 * <map name="umlclick"><area shape="Rect" coords="10,0,82,28"  href="Cmd.html">
 * <area shape="Rect" coords="0,108,92,136"  href="CmdUndoable.html"></map></center>
 * </p>
 *  
 */
public abstract class CmdGraphEditor extends Cmd {

    /**
     * A reference to the current graph editor.
     */
    private final JGraphEditor _graphEditor;

    /**
     * Construct a new CmdGraphEditor.
     * 
     * @param editor
     *            a reference to the current editor.
     * @param name
     *            the name of this command.
     */
    public CmdGraphEditor(JGraphEditor editor, String name) {
        this(editor, name, 0, null);
    }

    protected JGraphEditor getGraphEditor() {
        return _graphEditor;
    }

    protected JGraph getGraph() {
        if (getGraphEditor() == null)
            return null;
        else
            return getGraphEditor().getGraph();
    }

    /**
     * Construct a new CmdGraphEditor.
     * 
     * @param editor
     *            a reference to the current editor.
     * @param name
     *            the name of this command.
     * @param iconSize
     *            int
     * @param iconName
     *            the path to the associated icon.
     */
    public CmdGraphEditor(JGraphEditor editor, String name, int iconSize, String iconName) {
        super(name, iconSize, iconName);

        _graphEditor = editor;

        updateEnabled();
    }
}
