package mygale.mcge.cmd;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.print.PrintService;
import javax.print.attribute.HashPrintRequestAttributeSet;
import javax.print.attribute.PrintRequestAttributeSet;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.MediaSizeName;
import javax.print.attribute.standard.OrientationRequested;
import javax.swing.JOptionPane;

import mygale.mcge.core.JGraph;
import mygale.mcge.core.JGraphEditor;

/**
 * @author muehlberger
 */
public class CmdPrintGraph extends CmdGraphEditor implements Printable {

	private class DialogThread extends Thread {

		public void run() {
			try {
				_printDialog = _pJob.printDialog();
				if (_printDialog) {
					_pJob.print(attributeSet);
				}
			} catch (PrinterException p) {
				JOptionPane.showMessageDialog(
					_graph.getParent(),
					"Error during printing image!",
					"Print Activity Graph",
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	private PrintRequestAttributeSet attributeSet = new HashPrintRequestAttributeSet();
	private PrinterJob _pJob;
	private PageFormat _pFormat;
	private JGraph _graph;

	private boolean _printDialog;

	private void initialize() {
		_printDialog = false;
		attributeSet.add(new JobName("Printing SeSAm Activitiy Graph", null));
		attributeSet.add(MediaSizeName.ISO_A4);
	}

	public CmdPrintGraph(JGraphEditor editor, int iconSize) {
		super(editor, "Print", iconSize, "general/Print");
	}

	public void doIt() {
		_graph = getGraph();
		initialize();

		if (_graph.getHeight() >= _graph.getWidth()) {
			attributeSet.add(OrientationRequested.PORTRAIT);
		} else {
			attributeSet.add(OrientationRequested.LANDSCAPE);
		}

		_pJob = PrinterJob.getPrinterJob();
		_pJob.setPrintable(this);

		PrintService[] pServices = PrinterJob.lookupPrintServices();

		if (pServices.length > 0) {

			try {

				_pJob.setPrintService(pServices[0]);

				DialogThread dialogThread = new DialogThread();
				dialogThread.start();

			} catch (PrinterException p) {
				JOptionPane.showMessageDialog(
					_graph.getParent(),
					"Error during printing image!",
					"Print Activity Graph",
					JOptionPane.ERROR_MESSAGE);
			}
		} else {
			JOptionPane.showMessageDialog(
				_graph.getParent(),
				"No suitable printer found!",
				"Print Activity Graph",
				JOptionPane.ERROR_MESSAGE);
		}
	}

	public String getToolTipText() {
		return "print the activity graph";
	}

	public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
		if (pageIndex == 0) {

			Graphics2D g2d = (Graphics2D) graphics;

			double scaleFactor =
				Math.min(
					pageFormat.getImageableWidth() / getGraph().getWidth(),
					pageFormat.getImageableHeight() / getGraph().getHeight());

			g2d.setClip(0, 0, getGraph().getWidth(), getGraph().getHeight());
			g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());
			if ((getGraph().getWidth() > pageFormat.getImageableWidth())
				| (getGraph().getHeight() > pageFormat.getImageableHeight())) {
				g2d.scale(scaleFactor, scaleFactor);
			}

			getGraph().paint(g2d);
			g2d.dispose();

			return Printable.PAGE_EXISTS;
		} else {
			return Printable.NO_SUCH_PAGE;
		}
	}
}
