package mygale.mcge.cmd;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import mygale.mcge.core.JGraphEditor;

/**
 * @author muehlberger
 */
public class CmdSaveAs extends CmdGraphEditor {

	private class JPGFileFilter extends FileFilter {

		public String getExtension(File f) {
			String ext = null;
			String s = f.getName();
			int i = s.lastIndexOf('.');
			if (i > 0 && i < s.length() - 1) {
				ext = s.substring(i + 1).toLowerCase();
			}
			return ext;
		}

		public boolean isJPEGFile(File f) {
			String extension = getExtension(f);
			return ((extension != null) && (extension.equals("jpeg") || extension.equals("jpg")));
		}

		public boolean accept(File f) {
			return (f.isDirectory() || isJPEGFile(f));
		}

		public String getDescription() {
			return "SeSAm Graph Images (.jpg)";
		}
	}

	private final String name = "SaveAs";
	static private File _oldFile;
	private JFileChooser chooser;
	private JPGFileFilter filter;

	public CmdSaveAs(JGraphEditor editor, int iconSize) {
		super(editor, "SaveAs", iconSize, "general/SaveAs");
	}

	private void initialize() {
		if (chooser == null) {
			chooser = new JFileChooser();
			filter = new JPGFileFilter();
			chooser.setFileFilter(filter);
		}
	}

	public void doIt() {
		mygale.mcge.core.JGraph graph = getGraph();

		initialize();

		File target = _oldFile;

		if (_oldFile != null) {
			chooser.setCurrentDirectory(_oldFile);
		}

		int returnVal = chooser.showSaveDialog(graph.getParent());

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			target = chooser.getSelectedFile();

			if (!filter.isJPEGFile(target)) {
				File tmp = new File(target.toString() + ".jpg");
				target = tmp;
			}

			try {
				if (target != null) {
					if (!target.exists()) {
						saveToFile(graph, target);
						_oldFile = target;
					} else {

						int result =
							JOptionPane.showConfirmDialog(
								graph.getParent(),
								"Replace existing file?",
								"Save into '" + target.getName() + "'",
								JOptionPane.YES_NO_CANCEL_OPTION);
						if (result == JOptionPane.YES_OPTION) {
							saveToFile(graph, target);
							_oldFile = target;
						} else if (result == JOptionPane.NO_OPTION) {
							_oldFile = target;
							doIt();
						}
					}
				}
			} catch (IOException e) {
				JOptionPane.showMessageDialog(
					graph.getParent(),
					"Error during saving image file!",
					"Save into '" + target.getName() + "'",
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	static void saveToFile(JComponent source, File target) throws IOException {
		int width = source.getWidth();
		int height = source.getHeight();
		BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = image.createGraphics();

		g2d.setClip(0, 0, width, height);

		source.paint(g2d);
		g2d.dispose();
		ImageIO.write(image, "jpeg", target);
		image.flush();
	}

	public String getToolTipText() {
		return "save the activity graph as JPEG-image";
	}

}
