package mygale.mcge.cmd;



import mygale.mcge.core.*;
import mygale.mcge.fig.IFigLayered;
import mygale.mcge.layer.LayerManager;
import mygale.mcge.sel.Selection;
import mygale.mcge.throwable.MCGEInternalError;

/** Cmd to put to the back a Fig.

 */
public class CmdSendBackward extends CmdMoveFigToBack {



/** Perform whatever Cmd this Cmd is meant to do. Subclasses
     *  should override this to do whatever is intended.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
public void doIt() {
   JGraph graph = getGraph();

         LayerManager layer = graph.getLayerManager();
            Selection sel = graph.getSelection();
            boolean bInform = sel.setCmdInform(false);
            clearUndoChanges();
            if (sel.unlockedSize() == 1) {
               IFigLayered figure = (IFigLayered) sel.getAllFigs().next();
               int oldDepth = layer.getDepthLevel(figure);
               layer.sendBackward(figure);
               saveChange(figure, oldDepth, layer.getDepthLevel(figure));
            } else {
               throw new MCGEInternalError("try to send to back an empty selection.");
            }
            graph.damaged(sel);
            sel.setCmdInform(bInform);
            sel.informCmd();
            addUndoCmd();
}

/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "send backward the selection" ;
    }

/** Reply if the given size corresponds to
     *  an valid selection size.
     *
     * @param size the size to test.
     * @return <code>true</code> if the given size is
     *         a valid size for the current command.
     */
    protected boolean isCorrectSize( int size ) { return size == 1; }


    /** Construct a new CmdSendBackward.
     *
     * @param editor a reference to a graph editor.
     * @param iconSize int
 */
    public CmdSendBackward(JGraphEditor editor, int iconSize) {
    super(editor, "Move Back", iconSize, "general/MoveBack" );
    }
}
