package mygale.mcge.cmd;



import java.util.Iterator;

import mygale.mcge.core.*;
import mygale.mcge.fig.IFigLayered;
import mygale.mcge.layer.LayerManager;
import mygale.mcge.sel.Selection;
import mygale.mcge.throwable.MCGEInternalError;

/** Cmd to put to the back a Fig.
 *
 * @auhtor Mahdi Hannoun
 */
public class CmdSendToBack extends CmdMoveFigToBack {



/** Perform whatever Cmd this Cmd is meant to do. Subclasses
     *  should override this to do whatever is intended.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
public void doIt() {
   JGraph graph = getGraph();

         LayerManager layer = graph.getLayerManager();
            Selection sel = graph.getSelection();
            boolean bInform = sel.setCmdInform(false);
            clearUndoChanges();
            switch (sel.unlockedSize()) {
               case 0 :
                  throw new MCGEInternalError("try to send to back an empty selection.");
               case 1 :
                  IFigLayered figure = (IFigLayered) sel.getAllFigs().next();
                  int oldDepth = layer.getDepthLevel(figure);
                  layer.sendToBack(figure);
                  saveChange(figure, oldDepth, layer.getDepthLevel(figure));
                  break;
               default :
                  Iterator it = sel.getAllFigs();
                  int depths[] = new int[sel.size()];
                  for (int i = 0; it.hasNext(); i++) {
                     figure = (IFigLayered) it.next();
                     depths[i] = layer.getDepthLevel(figure);
                  }
                  layer.sendToBack(sel.getUnlockedFigs());
                  it = sel.getAllFigs();
                  for (int i = 0; it.hasNext(); i++) {
                     figure = (IFigLayered) it.next();
                     saveChange(figure, depths[i], layer.getDepthLevel(figure));
                  }
            }
            graph.damaged(sel);
            sel.setCmdInform(bInform);
            sel.informCmd();
            addUndoCmd();
}

/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "send to back the selection" ;
    }

/** Reply if the given size corresponds to
     *  an valid selection size.
     *
     * @param size the size to test.
     * @return <code>true</code> if the given size is
     *         a valid size for the current command.
     */
    protected boolean isCorrectSize( int size ) { return size >= 1; }


    /** Construct a new CmdBringToFront.
     *
     * @param editor a reference to the graph editor.
     * @param iconSize int
 */
    public CmdSendToBack(JGraphEditor editor, int iconSize) {
    super(editor, "SendToBack", iconSize, "general/SendToBack" );
    }
}
