package mygale.mcge.cmd;



import mygale.mcge.throwable.MCGEInternalError;
import mygale.mcge.undoredo.UndoRedoManager;

/** Cmd to undo the last command on the graph.
 *  <p><center>
 *    <img src="doc-files/CmdUndo_uml.gif" width="129" height="83" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="0,0,128,28"  href="CmdUndoRedoListener.html">
 *    </map>
 *  </center></p>

 */
public class CmdUndo extends CmdUndoRedoListener {



/** This method is called each time the command wants to run itself.
     *
     * throws mygale.mcge.throwable.MCGEInternalError Something wrong !
     */
    public void doIt() {
    if ( undoredoManager != null ) {
        undoredoManager.undo();
        return;
    }
    throw new MCGEInternalError( "no undo/redo manager given" );
    }

/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "undo the last action" ;
    }

/** Return <code>true</code> if this action should be available
     *  to the user. Sublass implementations of this method should
     *  always call <code>super.shouldBeEnabled()</code> first.
     *
     * @return <code>true</code> if this command is enabled,
     *         otherwise <code>false</code>.
     */
    public boolean shouldBeEnabled() {
    return ( undoredoManager != null &&
         undoredoManager.undoSize() > 0 &&
         super.shouldBeEnabled() );
    }


    /** Construct a new CmdUndo.
     *
     * @param manager a reference to an undo/redo manager.
     * @param iconSize int
 */
    public CmdUndo(UndoRedoManager manager, int iconSize) {
    super(manager, null, iconSize, "general/Undo" );
    
    register();
    }
}
