package mygale.mcge.cmd;



import mygale.mcge.throwable.MCGEInternalError;
import mygale.mcge.undoredo.*;

/** This abstract class defines the Cmd that can
 *  add and remove undoable/redoable commands
 *  from the undo/redo manager.
 *  <p><center>
 *    <img src="doc-files/CmdUndoRedoListener_uml.gif" width="173" height="137" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="50,0,127,28"  href="Cmd.html">
 *      <area shape="Rect" coords="0,108,72,136"  href="CmdUndo.html">
 *      <area shape="Rect" coords="99,108,171,136"  href="CmdRedo.html">
 *    </map>
 *  </center></p>
 *
 .
 */
public abstract class CmdUndoRedoListener extends Cmd
	implements UndoRedoListener {

	/** A reference to a undo/redo manager.
	 */
	protected UndoRedoManager undoredoManager;

/** Register this command into the undo/redo manager.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
    public void register() {
    if ( undoredoManager != null ) {
        undoredoManager.register( this );
    }
    else {
        throw new MCGEInternalError( "no UndoRedoManager given" );
    }
    }


    /** Construct a new CmdUndoRedoListener.
     *
     * @param manager a reference to the undo/redo manager.
     * @param name the name of the command.
     * @param iconName the path to the associated icon.
     */
    public CmdUndoRedoListener(UndoRedoManager manager, String name) {
    super(name);
    
    undoredoManager = manager;
    }



    /** Construct a new CmdUndoRedoListener.
     *
     * @param manager a reference to the undo/redo manager.
     * @param name the name of the command.
     * @param iconSize int
 * @param iconName the path to the associated icon.
     */
    public CmdUndoRedoListener(UndoRedoManager manager, String name, int iconSize, String iconName) {
    super(name,iconSize,iconName);
    
    undoredoManager = manager;
    }
}
