package mygale.mcge.cmd;



import mygale.mcge.core.JGraphEditor;
import mygale.mcge.undoredo.*;

/** This abstract class represents the commands that can be
 *  undoing and redoing.
 *  <p><center>
 *    <img src="doc-files/CmdUndoable_uml.gif" width="327" height="137" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="118,0,208,28"  href="CmdGraphEditor.html">
 *      <area shape="Rect" coords="127,109,199,136"  href="CmdAlign.html">
 *      <area shape="Rect" coords="0,109,109,136"  href="CmdMoveFigToFront.html">
 *      <area shape="Rect" coords="217,109,325,136"  href="CmdMoveFigToBack.html">
 *    </map>
 *  </center></p>
 *
 */
public abstract class CmdUndoable extends CmdGraphEditor {



/** Add this command to the undo list.
     *
     * @param undoCmd the new undoing command.
     */
    protected void addUndoCmd(UndoableCmd undoCmd) {
    UndoRedoManager urm = getUndoRedoManager();
    if (urm != null && undoCmd != null ) {
        urm.addUndo( undoCmd );
    }
    }

/** This method permits to clear the undo/redo list.
     */
    protected void clearUndoList() {
    UndoRedoManager urm = getUndoRedoManager();
    if ( urm != null ) {
        urm.clearUndo();
    }
    }

    // Undo/Redo API

    /** Reply the associated undo/redo manager.
     *
     * @return a reference to the associated undo/redo manager.
     *         or <code>null</code>.
     */
    protected UndoRedoManager getUndoRedoManager() {
    if ( getGraphEditor() != null ) {
        return getGraphEditor().getUndoRedoManager();
    } else
    return null;
    }

/** Construct a new CmdUndoable.
     *
     * @param editor a reference to the current editor.
     * @param name the name of this command.
     * @param iconSize int
 * @param iconName String
 */
    public CmdUndoable(JGraphEditor editor, String name, int iconSize, String iconName) {
    super( editor, name, iconSize, iconName);
    }
}
