package mygale.mcge.cmd;


import java.util.*;

import mygale.mcge.core.*;
import mygale.mcge.fig.*;
import mygale.mcge.sel.Selection;
import mygale.mcge.throwable.MCGEInternalError;
import mygale.mcge.undoredo.*;

/** Cmd to unlock Figs from view.

 */
public class CmdUnlock extends CmdUndoable {

	/** This is the set of changes.
	 */
	protected Vector _changes = new Vector();

	/** This class permits to managed the undo/redo commands.
	 *
		 */
	protected static class UndoUnlock
	implements UndoableCmd, RedoableCmd {

	/** This is the changes.
	 */
	protected Vector _changes;

	/** Construct a new UndoUnlock.
	 * @param changes Vector
	 */
	public UndoUnlock( Vector changes ) {
		_changes = changes;
	}

	/** This method was called by the undo command.
	 *  It must undo something.
	 * @see mygale.mcge.undoredo.UndoableCmd#undoIt()
	 */
	public void undoIt() {
		Enumeration enum = _changes.elements();
		while ( enum.hasMoreElements() ) {
		IFigLayered fig = (IFigLayered)enum.nextElement();
		fig.setLocked( true );
		}
	}

	/** This method was called by the redo command.
	 *  It must redo something.
	 */
	public void redoIt() {
		Enumeration enum = _changes.elements();
		while ( enum.hasMoreElements() ) {
		IFigLayered fig = (IFigLayered)enum.nextElement();
		fig.setLocked( false );
		}
	}

	} /* class UndoUnlock */

/** Add this command to the undo list.
     */
    protected void addUndoCmd() {
    if ( _changes.size() > 0 ) {
        super.addUndoCmd( new UndoUnlock( _changes ) );
    }
    }


/** Destroy the nudo/redo changes.
     */
    protected void clearUndoChanges() {
    _changes = new Vector();
    }


/** Perform whatever Cmd this Cmd is meant to do. Subclasses
     *  should override this to do whatever is intended.
     *
     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
     */
public void doIt() {
   JGraph graph = getGraph();

         Selection sel = graph.getSelection();
         if (sel.lockedSize() <= 0) {
            throw new MCGEInternalError("try to unlock an empty selection.");
         } else {
            Iterator it = sel.getLockedFigs();
            clearUndoChanges();
            while (it.hasNext()) {
               IFigLayered fig = (IFigLayered) it.next();
               saveChange(fig);
               fig.setLocked(false);
            }
            setEnabled(false);
            sel.informCmd();
            addUndoCmd();
         }
}


/** Return the tooltip text associated to the command.
     *
     * @return a string that is the tooltip of this command.
     */
    public String getToolTipText() {
    return "unlock the selection" ;
    }


//
    // Undo/Redo API

    /** This method permit to register (from doIt() for example)
     *  a change.
     *
     * @param fig a reference to a figure.
     */
    protected void saveChange(IFig fig) {
    _changes.addElement( fig );
    }


/** Return <code>true</code> if this action should be available
     *  to the user. Sublass implementations of this method should
     *  always call <code>super.shouldBeEnabled()</code> first.
     *
     * @return <code>true</code> if this command is enabled,
     *         otherwise <code>false</code>.
     */
    public boolean  shouldBeEnabled() {
   JGraph graph = getGraph();

   // at initializing
    if (getGraph() == null)
        return false;
    else {
        Selection sel = graph.getSelection();
        if ( sel.lockedSize() == 0 ) {
            return false ;
        } else
        return super.shouldBeEnabled();
        }
    }

/** Construct a new CmdUnlock.
     *
     * @param editor a reference to a graph editor.
     * @param iconSize int
 */
    public CmdUnlock(JGraphEditor editor, int iconSize) {
    super(editor, "Unlock", iconSize, "general/Unlock");
    }
}