package mygale.mcge.cmd;

import mygale.mcge.undoredo.RedoableCmd;
import mygale.mcge.undoredo.UndoableCmd;

/**
 * Insert the type's description here.
 * Creation date: (07.05.2001 10:12:22)
 
 */
public abstract class CmdZoom extends CmdUndoable {
	/** This class permits to managed the undo/redo commands.
		*
		
		*/
	protected class UndoZoom implements UndoableCmd, RedoableCmd {

		/** This is the changes.
		 */
		protected double _newScale;
		protected double _oldScale;

		/** Construct a new UndoUnlock.
		 * @param newScale double
		 * @param oldScale double
		 */
		public UndoZoom(double newScale, double oldScale) {
			_oldScale = oldScale;
			_newScale = newScale;
		}

		/** This method was called by the undo command.
		 *  It must undo something.
		 */
		public void undoIt() {
			getGraph().setScale(_oldScale);
		}

		/** This method was called by the redo command.
		 *  It must redo something.
		 * @see mygale.mcge.undoredo.RedoableCmd#redoIt()
		 */
		public void redoIt() {
			getGraph().setScale(_newScale);
		}

	} /* class UndoZoom */

	/**
	 * CmdZoomIn constructor comment.
	 * @param editor mygale.mcge.core.JGraphEditor
	 * @param name java.lang.String
	 */
	public CmdZoom(mygale.mcge.core.JGraphEditor editor, String name) {
		super(editor, name, 0, null);
	}

	/**
	 * @see mygale.mcge.cmd.Cmd
	 */
	public void doIt() {
		mygale.mcge.core.JGraph graph = getGraph();
		clearUndoChanges();
		double oldScale = graph.getScale();
		double newScale = calculateNewScale(oldScale);
		graph.setScale(newScale);
		addUndoCmd(newScale, oldScale);
	}

	/** Add this command to the undo list.
	     */
	protected void addUndoCmd(double newScale, double oldScale) {
		super.addUndoCmd(new UndoZoom(newScale, oldScale));
	}

	protected abstract double calculateNewScale(double oldScale);

	/** Destroy the nudo/redo changes.
	     */
	protected void clearUndoChanges() {
	}

	/**
	 * CmdZoomIn constructor comment.
	 * @param editor mygale.mcge.core.JGraphEditor
	 * @param name java.lang.String
	 * @param iconSize int
	 * @param iconName java.lang.String
	 */
	public CmdZoom(mygale.mcge.core.JGraphEditor editor, String name, int iconSize, String iconName) {
		super(editor, name, iconSize, iconName);
	}
}
