package mygale.mcge.demo;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.*;

import javax.swing.*;

import mygale.mcge.cmd.*;
import mygale.mcge.core.JGraph;
import mygale.mcge.fig.FigNodeEllipse;
import mygale.mcge.net.DefaultNetNode;
import mygale.mcge.net.INetNode;

/** This is a demo application of the MCGE library.
 *  <p>
 *  This demo opens a editor in a simple window and
 *  create a array of nodes.
 *
 */
public class Demo extends JApplet {

	JFrame frame;
	boolean inAnApplet;

	// Menu bars instances
	private JMenuBar jMenuBar;
	private JMenu jMenu_File;
	private CmdExit cmd_File_Quit;
	private JMenu jMenu_Edit;
	private JMenuItem jMenuItem_Edit_Preferences;
	private CmdUndo cmd_Edit_Undo;
	private CmdRedo cmd_Edit_Redo;

	// Statusbar
	private JLabel jLabel_Statusbar;

	// The graph editor
	public GraphEditor _graph;

	///
	// Constructor

	/** Creates new Demo as an applet.
	 */
	public Demo() {
		this(null);
	}

	/** Creates new Demo.
	     * @param frame JFrame
	 */
	public Demo(JFrame frame) {
		this.frame = frame;
		this.inAnApplet = (frame == null);
		if (inAnApplet) {
			getRootPane().putClientProperty("defeatSystemEventQueueCheck", Boolean.TRUE);
		}
	}

	/** Run the demo.
	     *
	     * @param args the command line arguments
	     */
	public static void main(String[] args) {
		JFrame frame = new JFrame("Mygale Connected-Graph Editor demo");

		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});

		Demo applet = new Demo(frame);
		frame.setContentPane(applet.makeContentPane());
		frame.pack();
		frame.setVisible(true);
	}

	/** Create the applet.
	     */
	public void init() {
		setContentPane(makeContentPane());
	}

	/** Create the default array of nodes.
	     */
	void createInitialGraph() {
		for (int x = 10; x <= 1000; x += 90) {
			for (int y = 10; y <= 500; y += 60) {
				createNodeRect(_graph.getGraph(), new DefaultNetNode(_graph.getGraph().getNetGraphModel()), x, y);
			}
		}
	}

	/** Create a node.
	     *  <p>
	     *  The node are a rectangular node with only
	     *  one rectangular port.
	     *  <p>
	     *  The node have only the top and bottom handles actives.
	     *  Left and right handles are inactives.
	     *
	     * @param graph the graph graph that may contains the node.
	     * @param netNode a reference to a NetNode that may be
	     *                added to the graph model.
	     * @param x x coordinate of the new node.
	     * @param y y coordinate of the new node.
	     */
	void createNodeRect(JGraph graph, INetNode netNode, int x, int y) {

		graph.getNetGraphModel().addNode(netNode);

		FigNodeEllipse figNode = (FigNodeEllipse) graph.getFigMapper().getFigFor(netNode);
		figNode.setLocation(x, y);
	}
	
	void createNodeEll(JGraph graph, INetNode netNode, int x, int y) {

		graph.getNetGraphModel().addNode(netNode);

		FigNodeEllipse figNode = (FigNodeEllipse) graph.getFigMapper().getFigFor(netNode);
		figNode.setLocation(x, y);
	}

	/** This menu create the
	     *  {@link mygale.mcge.core.JGraphEditor JGraphEditor}.
	     */
	void initEditor() {
		_graph = new GraphEditor();
	}

	/** This method create the menus.
	     */
	void initMenus() {
		jMenuBar = new JMenuBar();

		// Menu 'File'

		jMenu_File = new JMenu();
		jMenu_File.setText("File");

		cmd_File_Quit = new CmdExit("Exit");
		jMenu_File.add(cmd_File_Quit);

		// Menu 'Edit'

		jMenu_Edit = new JMenu();
		jMenu_Edit.setText("Edit");

		cmd_Edit_Undo = new CmdUndo(_graph.getUndoRedoManager(), 16);
		jMenu_Edit.add(cmd_Edit_Undo);

		cmd_Edit_Redo = new CmdRedo(_graph.getUndoRedoManager(), 16);
		jMenu_Edit.add(cmd_Edit_Redo);

		jMenu_Edit.addSeparator();

		jMenuItem_Edit_Preferences = new JMenuItem();
		jMenuItem_Edit_Preferences.setText("Preferences...");
		jMenuItem_Edit_Preferences.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				JDialogPreferences.openPreferences(_graph.getGraph(), frame);
			}
		});
		jMenu_Edit.add(jMenuItem_Edit_Preferences);

		// Menu bar

		jMenuBar.add(jMenu_File);
		jMenuBar.add(jMenu_Edit);
	}

	/** This method create the statusbar.
	     */
	void initStatusbar() {
		jLabel_Statusbar = new JLabel();
		_graph.setStatusBar(jLabel_Statusbar);
	}

	/** This method is called from within the init() to
	     *  initialize the applet.
	     */
	private Container makeContentPane() {
		initEditor();
		initMenus();
		initStatusbar();

		//Add Components to a JPanel, using the default FlowLayout.
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());
		pane.add(_graph, BorderLayout.CENTER);
		pane.add(jLabel_Statusbar, BorderLayout.SOUTH);

		if (!inAnApplet) {
			frame.setJMenuBar(jMenuBar);
		}

		createInitialGraph();

		return pane;
	}
}
