package mygale.mcge.demo;

import javax.swing.JTabbedPane;
import javax.swing.JToolBar;

import mygale.mcge.cmd.*;
import mygale.mcge.core.*;
import mygale.mcge.fig.*;
import mygale.mcge.mode.ModeMouseLocation;
import mygale.mcge.net.*;

/** This is the graph drawing area of the MCGE demo
 *
 */
class GraphArea extends JGraph {

	/// Constructors

	/** Construct a new GraphArea.
	 *
	@link GraphEditor graph editor}.
	 * @param graphEditor a reference to the enclosing
	 *        { */
	public GraphArea(JGraphEditor graphEditor) {
		super(graphEditor, new AbstractFigNodeFactory() {
			public IFigNode createFigNode(INetNode netNode) {
				return new FigNodeEllipse();
			}
		}, new AbstractFigEdgeFactory() {
			public IFigEdge createFigEdge(INetEdge netEdge) {
				return new FigEdgeLinear();
			}
		});
	}

	/** This method perform also an initialization of the ModeManager.
	     *  <p>
	     *  We assume that <a href="#_modeManager">_modeManager</a>
	     *  is set in this method.
	     */
	protected void makeModeManager() {
		super.makeModeManager();
		getModeManager().beginMode(new ModeMouseLocation(getModeManager()));
	}

	/** this method customizes the default toolbar
	     *  "Personalized".
	     *  <p>
	     *  We assume that <a href="#_tabbedToolBar>_tabbedToolBar</a>
	     *  is already set before the call to rthis method.
	     */
	protected void makePersonalizedToolBar() {
		JToolBar toolbar = new JGraphToolBar();
		CmdDelete cmdDelete = new CmdDelete(getGraphEditor(), 24);
		getSelection().addCmdToInform(cmdDelete);

		toolbar.setFloatable(false);

		toolbar.add(new CmdSelect(getGraphEditor(), 24));
		toolbar.add(cmdDelete);

		toolbar.addSeparator();

		toolbar
			.add(new CmdCreateNode(
				getGraphEditor(),
				null,
				"add a rectangular-node",
				24,
				"general/Node",
				new INetNodeFactory() {
			public INetNode createNetNode() {
				return new DefaultNetNode(getGraphEditor().getGraph().getNetGraphModel());
			}
		}));
		toolbar
			.add(new CmdCreateNode(
				getGraphEditor(),
				null,
				"add an ellipse-node",
				24,
				"general/EllipseNode",
				new INetNodeFactory() {
			public INetNode createNetNode() {
				return new DefaultNetNode(getGraphEditor().getGraph().getNetGraphModel());
			}
		}, new AbstractFigNodeFactory() {
			public IFigNode createFigNode(INetNode netnode) {
				return new FigNodeEllipse();
			}
		}));
		toolbar
			.add(new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add an linear-edge beetwen two nodes",
				24,
				"general/Edge",
				new INetEdgeFactory() {
			public INetEdge createNetEdge() {
				return new DefaultNetEdge();
			}
		}, new AbstractFigEdgeFactory() {
			public IFigEdge createFigEdge(INetEdge netEdge) {
				return new FigEdgeLinear();
			}
		}));
		toolbar
			.add(new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add an polyline-edge beetwen two nodes",
				24,
				"general/PolyEdge",
				new INetEdgeFactory() {
			public INetEdge createNetEdge() {
				return new DefaultNetEdge();
			}
		}, new AbstractFigEdgeFactory() {
			public IFigEdge createFigEdge(INetEdge netEdge) {
				return new FigEdgePoly();
			}
		}));
		toolbar
			.add(new CmdCreateEdge(
				getGraphEditor(),
				null,
				"add an spline-edge beetwen two nodes",
				24,
				"general/SplineEdge",
				new INetEdgeFactory() {
			public INetEdge createNetEdge() {
				return new DefaultNetEdge();
			}
		}, new AbstractFigEdgeFactory() {
			public IFigEdge createFigEdge(INetEdge netEdge) {
				return new FigEdgeSpline();
			}
		}));

		toolbar.addSeparator();

		toolbar
			.add(new CmdCreateFig(
				getGraphEditor(),
				null,
				"create a text area",
				24,
				"general/Text",
				new IFigFactory() {
			public IFig createFig() {
				return new FigText();
			}
		}));
		toolbar
			.add(new CmdCreateFig(
				getGraphEditor(),
				null,
				"create a rectangle",
				24,
				"general/Rect",
				new IFigFactory() {
			public IFig createFig() {
				return new FigRect();
			}
		}));
		toolbar
			.add(new CmdCreateFig(
				getGraphEditor(),
				null,
				"create an ellipse",
				24,
				"general/Circle",
				new IFigFactory() {
			public IFig createFig() {
				return new FigEllipse();
			}
		}));

		toolbar.addSeparator();

		toolbar.add(new CmdZoomIn(getGraphEditor(), 24));
		toolbar.add(new CmdZoomStandard(getGraphEditor(), 24));
		toolbar.add(new CmdZoomOut(getGraphEditor(), 24));

		((JTabbedPane) getToolBar()).add("Node/Link", toolbar);
	}
}