package mygale.mcge.demo;



import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import mygale.mcge.core.JGraph;
import mygale.mcge.util.prefs.Prefs;

/** This class is a dialog box that permits to edit
 *  some preferences.
 *
 */
public class JDialogPreferences extends JDialog {

	/** A reference to the preference BEFORE any change.
	 */
	protected Prefs _oldprefs;

	/** A reference to the preference AFTER changes.
	 */
	protected Prefs _newprefs;

	/** Buttons. */
	private JButton _buttonOk, _buttonCancel;

	/** Labels. */
	private JLabel _labels[];

	/** Color choosers. */
	private JColorButton _buttonColorBackground, _buttonColorFigBackground;

	/** Handle size chooser. */
	protected JSlider _sliderSizeSelection;

	/** This inner class defines the button that permits
	 *  to choose the background color.
	 */
	private class JColorButtonBackground extends JColorButton {

	JColorButtonBackground() {
		super( JDialogPreferences.this._newprefs.getBackgroundColor(), "Choose background color" );
	}

	public void actionPerformed (ActionEvent evt) {
		_newprefs.setBackgroundColor( this.getColor() );
	}

	}

	/** This inner class defines the button that permits
	 *  to choose the foreground color of
	 *  {@link mygale.mcge.fig.Fig figures}.
	 */
	private class JColorButtonFigBackground extends JColorButton {

	JColorButtonFigBackground() {
		super( JDialogPreferences.this._newprefs.getFigBackgroundColor(), "Choose Figure's background color" );
	}

	/**
	 * @param evt ActionEvent
	 */
	public void actionPerformed (ActionEvent evt) {
		_newprefs.setFigBackgroundColor( this.getColor() );
	}

	}

/** Construct a new JDialogPreferences.
     *
     * @param prefs old preferences.
     * @param owner a reference to the owner's appplication.
     */
    public JDialogPreferences( Prefs prefs, Frame owner ) {
    super(owner, "Preferences" );
    _oldprefs = prefs;
    try {
        _newprefs = (Prefs)prefs.clone();
    }
    catch(CloneNotSupportedException e) {
        System.out.println("Cloneable not supported");
        System.exit(0);
    }

    initComponents();
    }

/** Open the preferences' dialog box.
     *
     * @param graph a reference to the graph editor.
     * @param owner a reference to the owner's appplication.
     */
    public static void openPreferences( JGraph graph, Frame owner ) {
    Prefs prefs = graph.getPrefs();
    JDialogPreferences dialog = new JDialogPreferences( prefs, owner );
    dialog.setModal( true );
    dialog.show();
    }

/** Creation dialog's components.
     */
    protected void initComponents() {
    _labels = new JLabel[3];
    Container f = getContentPane();

    f.setLayout( new GridLayout(4, 2) );

    _labels[0] = new JLabel( "Background color" );
    _buttonColorBackground = new JColorButtonBackground();

    _labels[1] = new JLabel( "Size of selection marks" );
    _sliderSizeSelection = new JSlider( 4, 25, _newprefs.getHandleSize() );
    _sliderSizeSelection.addChangeListener
        ( new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
            _newprefs.setHandleSize( _sliderSizeSelection.getValue() );
            }
        }
          );

    _buttonOk = new JButton( "Ok" );
    _buttonOk.addActionListener
        ( new java.awt.event.ActionListener () {
            public void actionPerformed (ActionEvent evt) {
            JDialogPreferences.this._oldprefs.clone( JDialogPreferences.this._newprefs );
            JDialogPreferences.this.dispose();
            }
        }
          );

    _buttonCancel = new JButton( "Cancel" );
    _buttonCancel.addActionListener
        ( new ActionListener () {
            public void actionPerformed (ActionEvent evt) {
            JDialogPreferences.this.dispose();
            }
        }
          );

    _labels[2] = new JLabel( "Figure's background color" );
    _buttonColorFigBackground = new JColorButtonFigBackground();

    f.add( _labels[0] );
    f.add( _buttonColorBackground );
    f.add( _labels[1] );
    f.add( _sliderSizeSelection );
    f.add( _labels[2] );
    f.add( _buttonColorFigBackground );
        f.add( _buttonOk );
        f.add( _buttonCancel );

    pack();
    }
}
