package mygale.mcge.event.mouse;


/** The listener interface for receiving "interesting" mouse events
 *  (press, release, click, double click, enter, drag and move) for
 *  the different {@link mygale.mcge.mode.Mode Modes}.
 *  <P>
 *  The class that is interested in processing a mouse event
 *  either implements this interface (and all the methods it
 *  contains).
 *  <P>
 *  The listener object created from that class is then registered with a
 *  component using the component's <code>addMouseListener</code>
 *  method. A mouse event is generated when the mouse is pressed, released
 *  clicked (pressed and released). A mouse event is also generated when
 *  the mouse cursor enters or leaves a component. When a mouse event
 *  occurs the relevant method in the listener object is invoked, and
 *  the {@link mygale.mcge.event.mouse.MCGEMouseEvent MCGEMouseEvent}
 *  is passed to it.
 *  <p><center>
 *    <img src="doc-files/MCGEMouseEventListener_uml.gif" width="137" height="137" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="9,0,126,27"  href="MCGEMouseManager.html">
 *      <area shape="Rect" coords="18,108,117,135"  href="MCGEMouseEvent.html">
 *    </map>
 *  </center></p>

 *
 * @see mygale.mcge.event.mouse.MCGEMouseEvent
 * @see mygale.mcge.mode.ModeManager
 */

public interface MCGEMouseEventListener {

   /**
        * Invoked when the mouse has been clicked on a component.
        */
   void mcgeMouseClicked(MCGEMouseEvent e);

   /**
        * Invoked when the mouse has been double clicked on a component.
        */
   void mcgeMouseDoubleClicked(MCGEMouseEvent e);

   /**
        * Invoked when a mouse button is pressed on a component and then
        * dragged.  Mouse drag events will continue to be delivered to
        * the component where the first originated until the mouse button is
        * released (regardless of whether the mouse position is within the
        * bounds of the component).
        */
   void mcgeMouseDragged(MCGEMouseEvent e);

   /**
        * Invoked when the mouse enters a figure.
        */
   void mcgeMouseEntered(MCGEMouseEvent e);

   /**
        * Invoked when the mouse exits a figure.
        */
   void mcgeMouseExited(MCGEMouseEvent e);

   /**
        * Invoked when the mouse button has been moved on a component
        * (with no buttons no down).
        */
   void mcgeMouseMoved(MCGEMouseEvent e);

   /**
        * Invoked when a mouse button has been pressed on a component.
        */
   void mcgeMousePressed(MCGEMouseEvent e);

   /**
        * Invoked when a mouse button has been released on a component.
        */
   void mcgeMouseReleased(MCGEMouseEvent e);
}