package mygale.mcge.fig;

import javax.swing.event.*;

import mygale.mcge.net.INetObject;

/** This class is the base class for graphical
 *  representation of model elements.
 *  <p><center>
 *    <img src="doc-files/FigModelElement_uml.gif" width="273" height="155" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="55,0,127,28"  href="Fig.html">
 *      <area shape="Rect" coords="172,0,244,28"  href="FigObject.html">
 *      <area shape="Rect" coords="199,63,271,90"  href="../net/NetObject.html">
 *      <area shape="Rect" coords="0,126,72,154"  href="FigNode.html">
 *      <area shape="Rect" coords="109,126,181,154"  href="FigEdge.html">
 *    </map>
 *  </center></p>
 *
 */
public abstract class AbstractFigLayeredObject extends AbstractFigLayered implements IFigLayeredObject {

	/** Owners are underlying objects that "own" the graphical Fig's
	 *  that represent them. For example, {@link FigNode FigNode} and
	 *  {@link FigEdge FigEdge} keep a pointer to the net-level object
	 *  that they represent. Also, any Fig can have
	 *  {@link mygale.mcge.net.NetObject NetObject} as an owner.
	 */
	private INetObject _netobject = null;

	private final transient ChangeListener _changeListener = new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
			fireStateChanged();
		}
	};

	/** Construct a new FigModelElement.
	     */
	public AbstractFigLayeredObject() {
		super();
	}

	/**
	 * Method to support listener events.
	 * @see mygale.mcge.fig.IFigLayered#fireStateChanged()
	 */
	public void fireStateChanged() {
		super.fireStateChanged();
	}

	/** Reply the
	 *  {@link mygale.mcge.net.NetObject NetObject}
	 *  that owns this Fig.
	 *
	@link mygale.mcge.net.NetObject NetObject}.
	 * @return a { * @see mygale.mcge.fig.IFigObject#getNetObject()
	*/
	public INetObject getNetObject() {
		return _netobject;
	}

	/** Set the {@link mygale.mcge.net.NetObject NetObject}
	     *  that owns this Fig.
	     *
	    @link mygale.mcge.net.NetObject NetObject}.
	     * @param netobject the { * @see mygale.mcge.fig.IFigObject#setNetObject(INetObject)
	 */
	public void setNetObject(INetObject netobject) {
		if (_netobject != null) {
			_netobject.removeChangeListener(_changeListener);
		}

		_netobject = netobject;

		if (netobject != null) {
			netobject.addChangeListener(_changeListener);
		}
	}
}