/**
 * Created on 23.07.2003
 *
 * This class is responsible for //insert purpose//.
 * 
 * @author glueckselig
 */
package mygale.mcge.fig;

import java.awt.*;
import java.awt.geom.AffineTransform;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mygale.mcge.net.INetObject;
import mygale.mcge.util.prefs.Prefs;

public abstract class FigEdgeSuperLinear extends FigEdgePoly implements ChangeListener {

	private static final BasicStroke _stroke =
		new BasicStroke(1.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND);

	public FigEdgeSuperLinear() {
		super();
	}

	public void calcBounds() {
		super.calcBounds();

		getBounds().setSize(Math.max(getBounds().width, 30), Math.max(getBounds().height, 30));

		// Need to calculate bounds of text on arrow (need graphicsObject for this!!)
		// calcTextBounds(g2d);
	}

	public void setNetObject(INetObject netObject) {

		super.setNetObject(netObject);
	}

	public void stateChanged(ChangeEvent e) {
		fireStateChanged();
	}

	protected Stroke getStroke() {
		return _stroke;
	}

	public abstract String getName();

	/** Method to paint this FigEdge.
		 *
		 * @param g2d the graphic context.
		 */
	public void paintContent(Graphics2D g2d, Prefs prefs, int dx, int dy, boolean shadowModeOn) {

		Stroke oldStroke = g2d.getStroke();

		g2d.setStroke(getStroke());

		super.paintContent(g2d, prefs, dx, dy, shadowModeOn);

		g2d.setStroke(oldStroke);

		if (shadowModeOn
			&& (_shadowMoveMode == IFigEdge.DST_PORTS_SHADOW_MOVE)
			&& (getCtrlPointSize() > 2)) {
			return;
		}

		Point begin;
		Point end;

		if (shadowModeOn) {
			begin = _shadowHelper._first;
			end = _shadowHelper._second;

		} else {
			begin = getCtrlPointAt(0);
			end = getCtrlPointAt(1);
		}

		String str = getName();
		if (str==null) str="";

		FontMetrics fm = g2d.getFontMetrics();

		int arrowWidth = (getEndArrow() == null) ? 0 : getEndArrow().getWidth();
		int border = 1;

		int strWidth = fm.stringWidth(str);
		int distance = (int) begin.distance(end) - 2 * border - arrowWidth;

		if (strWidth > distance) {
			str = FigNodePolygon.trimToFit(str, distance, fm, "...");
			strWidth = fm.stringWidth(str);
		}

		if (strWidth <= distance) {
			// zeichne Bezeichnung auf Kante
			if (end.x < begin.x)
				arrowWidth = -arrowWidth;

			int x = (int) Math.floor((end.x + begin.x - strWidth - arrowWidth) * 0.5);
			int y = (int) Math.floor((end.y + begin.y - 5) * 0.5);

			g2d.setColor(prefs.getFigForegroundColor());

			AffineTransform oldAf = g2d.getTransform();

			if (_shadowMoveMode == IFigEdge.BOTH_PORTS_SHADOW_MOVE) {
				g2d.setTransform(createRotateTransform(oldAf, g2d, str, begin, end, dx, dy));
				g2d.drawString(str, x + dx, y + dy);
			} else {
				g2d.setTransform(createRotateTransform(oldAf, g2d, str, begin, end, 0, 0));
				g2d.drawString(str, x, y);
			}

			g2d.setTransform(oldAf);
		}
	}

	protected AffineTransform createRotateTransform(
		AffineTransform oldaf,
		Graphics2D g2d,
		String str,
		Point begin,
		Point end,
		int dx,
		int dy) {

		AffineTransform af = new AffineTransform(oldaf);

		double rotateX = (end.x + begin.x) * 0.5 + dx;
		double rotateY = (end.y + begin.y) * 0.5 + dy;

		double theta = Math.atan(((double) (end.y - begin.y)) / ((double) (end.x - begin.x)));

		af.rotate(theta, rotateX, rotateY);

		return af;
	}
}
