package mygale.mcge.fig;
import junit.framework.Assert;
import mygale.mcge.net.*;

public final class FigHelper {
	private FigHelper() {
		super();
	}

	public static void checkFigEdge(IFigMapper figMapper, INetEdge netEdge, IFigEdge figEdge) {

		checkFigObject(figMapper, netEdge, figEdge);

		if (netEdge.getBeginNode() != null) {
			if (Assert.ASSERTION_ON)
				Assert.assertSame(figEdge.getBeginFigNode(), figMapper.getFigFor(netEdge.getBeginNode()));
			Assert.assertNotNull(figEdge.getBeginFigNode());
			Assert.assertNotNull(figMapper.getFigFor(netEdge.getBeginNode()));
			Assert.assertSame(figEdge.getBeginFigNode(), figMapper.getFigFor(netEdge.getBeginNode()));
		}

		if (netEdge.getEndNode() != null) {
			Assert.assertSame(figEdge.getEndFigNode(), figMapper.getFigFor(netEdge.getEndNode()));
			Assert.assertNotNull(figEdge.getEndFigNode());
			Assert.assertNotNull(figMapper.getFigFor(netEdge.getEndNode()));
			Assert.assertSame(figEdge.getEndFigNode(), figMapper.getFigFor(netEdge.getEndNode()));
		}
	}

	public static void checkFigNode(IFigMapper figMapper, INetNode netNode, IFigNode figNode) {

		checkFigObject(figMapper, netNode, figNode);
	}

	public static void checkFigObject(IFigMapper figMapper, INetObject netObject, IFigObject figObject) {

		Assert.assertNotNull(figObject);

		Assert.assertSame(netObject, figObject.getNetObject());
		Assert.assertNotNull(figMapper.getFigFor(netObject));
		Assert.assertSame(figObject, figMapper.getFigFor(netObject));
	}

	// TODO remove	
	public static void checkFigPort(IFigMapper figMapper, INetNode netPort, IFigNode figPort) {

		checkFigObject(figMapper, netPort, figPort);

	}

	public static IFigEdge createFigEdge(
		IFigMapper figMapper,
		INetEdge netEdge,
		IFigEdgeFactory figEdgeFactory) {

		IFigEdge figEdge = (IFigEdge) figMapper.getFigFor(netEdge);

		if (figEdge == null) {
			figEdge = createFigEdgeIntern(figMapper, netEdge, figEdgeFactory);

		} else {
			figEdge.setSelected(false);
		}

		if (netEdge.getBeginNode() != null) {
			figEdge.setBeginFigNode((IFigNode) figMapper.getFigFor(netEdge.getBeginNode()));
		}

		if (netEdge.getEndNode() != null) {
			figEdge.setEndFigPort((IFigNode) figMapper.getFigFor(netEdge.getEndNode()));
		}

		if (Assert.ASSERTION_ON) {
			checkFigEdge(figMapper, netEdge, figEdge);
		}

		return figEdge;
	}

	private static IFigEdge createFigEdgeIntern(
		IFigMapper figMapper,
		INetEdge netEdge,
		IFigEdgeFactory figEdgeFactory) {

		Assert.assertNull(figMapper.getFigFor(netEdge));

		IFigEdge figEdge = figEdgeFactory.createFigEdge(netEdge);

		if (netEdge.getBeginNode() != null) {
			figEdge.setBeginFigNode((IFigNode) figMapper.getFigFor(netEdge.getBeginNode()));
		}
		if (netEdge.getEndNode() != null) {
			figEdge.setEndFigPort((IFigNode) figMapper.getFigFor(netEdge.getEndNode()));
		}

		figEdge.setNetObject(netEdge);
		figMapper.setFigFor(netEdge, figEdge);

		return figEdge;
	}

	public static IFigNode createFigNode(
		IFigMapper figMapper,
		INetNode netNode,
		IFigNodeFactory figNodeFactory) {

		IFigNode figNode = (IFigNode) figMapper.getFigFor(netNode);

		if (figNode == null) {
			figNode = createFigNodeIntern(figMapper, netNode, figNodeFactory);

		} else {
			figNode.setSelected(false);
		}

		if (Assert.ASSERTION_ON) {
			checkFigNode(figMapper, netNode, figNode);
		}

		return figNode;
	}

	private static IFigNode createFigNodeIntern(
		IFigMapper figMapper,
		INetNode netNode,
		IFigNodeFactory figNodeFactory) {

		Assert.assertNull(figMapper.getFigFor(netNode));

		IFigNode figNode = figNodeFactory.createFigNode(netNode);
		figNode.setNetObject(netNode);
		figMapper.setFigFor(netNode, figNode);

		return figNode;
	}
}