package mygale.mcge.fig;
import java.awt.*;
import java.awt.geom.Rectangle2D;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mygale.mcge.net.INetObject;
import mygale.mcge.util.prefs.Prefs;

import de.uniwue.ki.util.math.MathHelper;

/**
 * Insert the type's description here.
 * Creation date: (22.03.2001 13:28:09)
 * @author Christoph Oechslein
 */
public abstract class FigNodePolygon extends AbstractFigNode implements ChangeListener {

	public static void drawStringInRect(
		Graphics2D g2d,
		Rectangle rectForString,
		FontMetrics fm,
		String str,
		boolean centerX) {
		Rectangle2D stringRect = fm.getStringBounds(str, g2d);

		int y =
			(int)Math.round(
				rectForString.y
					+ fm.getAscent()
					+ fm.getLeading()
					+ (rectForString.getHeight() - stringRect.getHeight()) * 0.5);

		drawStringInRect(g2d, rectForString, fm, str, y, centerX);
	}

	public static void drawStringInRect(
		Graphics2D g2d,
		Rectangle rectForString,
		FontMetrics fm,
		String str,
		int y,
		boolean centerX) {

		Rectangle2D stringRect = fm.getStringBounds(str, g2d);

		if (stringRect.getWidth() > rectForString.getWidth()) {
			String trimedString = trimToFit(str, rectForString.getWidth(), fm, "...");
			Rectangle2D trimedStringRect = fm.getStringBounds(trimedString, g2d);
			drawStringInRectIntern(g2d, rectForString, y, centerX, trimedString, trimedStringRect);

		} else {
			drawStringInRectIntern(g2d, rectForString, y, centerX, str, stringRect);
		}
	}
	public static void drawStringInRectCentered(
		Graphics2D g2d,
		Rectangle rectForString,
		FontMetrics fm,
		String str) {

		drawStringInRect(g2d, rectForString, fm, str, true);
	}

	private static void drawStringInRectIntern(
		Graphics2D g2d,
		Rectangle rectForString,
		int y,
		boolean centerX,
		String trimedString,
		Rectangle2D trimedStringRect) {

		if (centerX)
			g2d.drawString(
				trimedString,
				(int)Math.round(
					rectForString.getX() + (rectForString.getWidth() - trimedStringRect.getWidth()) * 0.5),
				y);
		else
			g2d.drawString(trimedString, (int)Math.round(rectForString.getX() + 1), y);

	}

	public static String trimToFit(
		String stringToTrim,
		double width,
		FontMetrics fm,
		String stringForFilling) {
		double widthOfStringToTrim = fm.stringWidth(stringToTrim);

		if (width >= widthOfStringToTrim)
			return stringToTrim;
		else {
			double withOfStringForFilling = fm.stringWidth(stringForFilling);

			double widthOfRemovedCharacters = 0;

			int endIndex = stringToTrim.length();

			while ((width < widthOfStringToTrim + withOfStringForFilling - widthOfRemovedCharacters)
				&& (endIndex > 0)) {
				char currentChar = stringToTrim.charAt(endIndex - 1);
				widthOfRemovedCharacters += fm.charWidth(currentChar);
				--endIndex;
			}

			return stringToTrim.substring(0, endIndex) + stringForFilling;
		}
	}

	private Polygon _polygon;

	public FigNodePolygon() {
		super();

		setWidth(getMinimalWidth());
		setHeight(getMinimalHeight());
	}

	protected Rectangle calculateInteriorPaintRect() {
		final int BORDER = 2;

		Rectangle rectForString = new Rectangle();

		rectForString.setBounds(
			Math.round(getX() + BORDER),
			Math.round(getY() + BORDER),
			Math.round(getWidth() - 2 * BORDER),
			Math.round(getHeight() - 2 * BORDER));

		return rectForString;
	}

	protected Rectangle calculateRectForStringCenterLocation(
		Graphics2D g2d,
		FontMetrics fm,
		String str,
		Rectangle interiorPaintRect) {

		return new Rectangle(interiorPaintRect);
	}

	protected Rectangle calculateRectForStringTopLocation(
		Graphics2D g2d,
		FontMetrics fm,
		String str,
		Rectangle interiorPaintRect) {

		Rectangle2D stringRect = fm.getStringBounds(str, g2d);

		Rectangle rectForString = new Rectangle(interiorPaintRect);

		rectForString.height = (int)stringRect.getHeight();

		return rectForString;
	}

	public void cleanUp() {

	}

	public int getMinimalHeight() {
		return 70;
	}

	public int getMinimalWidth() {
		return 90;
	}

	public abstract String getName();

	protected Polygon getPolygon() {
		return _polygon;
	}

	public final Shape getShape() {
		return getPolygon();
	}

	public Point intersectWith(Point pt, int dx, int dy) {
		return MathHelper.intersectWith(getPolygon(), pt.x, pt.y, dx, dy);
	}

	protected Rectangle paintBottom(Graphics2D g2d, Rectangle interiorPaintRect) {
		return interiorPaintRect;
	}

	public void paintContent(Graphics2D g2d, Prefs prefs) {
		paintSymbol(g2d, prefs);

		Rectangle interiorPaintRect = calculateInteriorPaintRect();

		interiorPaintRect = paintTitle(g2d, interiorPaintRect);

		interiorPaintRect = paintBottom(g2d, interiorPaintRect);
	}

	public void paintHandle(Graphics2D g2d, int x, int y, int width, int height, Prefs prefs) {
		g2d.fill3DRect(x, y, width, height, false);
	}

	protected void paintSymbol(Graphics2D g2d, Prefs prefs) {
		// Paint the background
		g2d.setColor(getBackgroundColor(prefs));
		g2d.fill(getBounds());

		// Paint the frame
		g2d.setColor(prefs.getFigForegroundColor());
		g2d.draw(getBounds());
	}

	protected Rectangle paintTitle(Graphics2D g2d, Rectangle interiorPaintRect) {
		Rectangle rectForString = paintTitle(g2d, interiorPaintRect, true, false);

		interiorPaintRect.y += rectForString.getHeight();
		interiorPaintRect.height -= rectForString.getHeight();

		return interiorPaintRect;
	}

	protected final Rectangle paintTitle(
		Graphics2D g2d,
		Rectangle interiorPaintRect,
		boolean centerHoriz,
		boolean centerVert) {

		String caption = getName();
		FontMetrics fm = g2d.getFontMetrics();

		Rectangle rectForString;

		if (centerVert)
			rectForString = calculateRectForStringCenterLocation(g2d, fm, caption, interiorPaintRect);
		else
			rectForString = calculateRectForStringTopLocation(g2d, fm, caption, interiorPaintRect);

		if (centerHoriz)
			drawStringInRectCentered(g2d, rectForString, fm, caption);
		else
			drawStringInRect(g2d, rectForString, fm, caption, false);

		return rectForString;
	}

	public void setNetObject(INetObject netObject) {

		super.setNetObject(netObject);
	}

	protected void setPolygon(Polygon poly) {
		_polygon = poly;
	}

	public void stateChanged(ChangeEvent e) {
		updatePolygon();
		fireStateChanged();
	}

	protected void updateDimensions() {
		super.updateDimensions();
		if (getNetObject() != null) {
			updatePolygon();
		}
	}

	protected void updatePolygon() {

		setPolygon(new Polygon());
		getPolygon().addPoint(getX(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY());
		getPolygon().addPoint(getX() + getWidth(), getY() + getHeight());
		getPolygon().addPoint(getX(), getY() + getHeight());
	}
}