package mygale.mcge.fig;


import java.awt.Graphics2D;

import mygale.mcge.util.prefs.Prefs;

/** This class permits to draw a rectangle into the workspace.
 *  <p><center>
 *    <img src="doc-files/FigRect_uml.gif" width="75" height="137" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="0,0,73,27"  href="Fig.html">
 *      <area shape="Rect" coords="0,108,73,136"  href="FigText.html">
 *    </map>
 *  </center></p>
 *
 */
public class FigRect extends AbstractFigLayered {

   /** <code>true</code> if this text area must be filled,
    *  <code>false</code> otherwise.
    */
   protected boolean _filled = true;

   /** <code>true</code> if this text area must be framed,
    *  <code>false</code> otherwise.
    */
   protected boolean _framed = true;

   /** Construct a new FigText.
        */
   public FigRect() {
      super();
   }

   /** Return the filling flag.
        *
        * @return <code>true</code> if this FigText must
        *         be filled, <code>false</code> otherwise.
        */
   public boolean getFilled() {
      return _filled;
   }



   /** Change the filling flag.
    *
    * @param filled <code>true</code> if this FigText
    *               must be filled, <code>false</code> otherwise.
    */
   public void setFilled(boolean filled) {
      _filled = filled;
      fireStateChanged();
   }

   /** Return the framed flag.
        *
        * @return <code>true</code> if this FigText must
        *         be framed, <code>false</code> otherwise.
        */
   public boolean getFramed() {
      return _framed;
   }

   /** Change the framing flag.
        *
        * @param framed <code>true</code> if this FigText
        *               must be framed, <code>false</code> otherwise.
        */
   public void setFramed(boolean framed) {
      _framed = framed;
      fireStateChanged();
   }

   ////
   // Painting methods

   /** Method to paint this Fig.
    *
    * @param g2d the graphic context.
    * @param prefs Prefs
    */
   public void paintContent(Graphics2D g2d, Prefs prefs) {
      // Draw the background
      if (_filled) {
         g2d.setColor(prefs.getFigBackgroundColor());
         g2d.fillRect(getX(), getY(), getWidth(), getHeight());
      }
      // Draw the frame
      g2d.setColor(prefs.getFigForegroundColor());
      if (_framed) {
         g2d.drawRect(getX(), getY(), getWidth(), getHeight());
      }
   }
}