package mygale.mcge.fig;

import java.awt.*;

import mygale.mcge.util.prefs.Prefs;

/** This class is the base class for basic drawing objects such as
 *  rectangles, lines, text, circles, etc.
 *  <p><center>
 *    <img src="doc-files/Fig_uml.gif" width="309" height="142" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="235,0,308,28"  href="../cmd/Cmd.html">
 *      <area shape="Rect" coords="118,0,190,28"  href="../mode/Mode.html">
 *      <area shape="Rect" coords="235,54,307,82"  href="../layer/LayerContener.html">
 *      <area shape="Rect" coords="0,54,73,82"  href="../util/prefs/Prefs.html">
 *      <area shape="Rect" coords="14,113,113,139"  href="FigModelElement.html">
 *      <area shape="Rect" coords="131,113,203,139"  href="FigText.html">
 *      <area shape="Rect" coords="221,113,293,139"  href="FigEllipse.html">
 *      <area shape="Rect" coords="0,0,73,28"  href="../core/JGraphEditor.html">
 *    </map>
 *  </center></p>
 *
 */
public interface IFig {

	/** Clean this Fig.
	     */
	void cleanUp();

	/** Reply the position and the dimension of this Fig.
	     *
	     * @return position and dimension.
	     */
	Rectangle getBounds();

	/** Reply the position and the dimension of the area that
	     *  will be refresh when the Fig was damaged.
	     *
	     * @return the damaged area.
	     */
	Rectangle getDamagedBounds(Prefs prefs);

	/** Reply the height of this Fig.
	     *
	     * @return height.
	     */
	int getHeight();

	/** Reply the position of this Fig.
	     *
	     * @return the position of this Fig.
	     */
	Point getLocation();

	/** Reply the width of this Fig.
	     *
	     * @return width.
	     */
	int getWidth();



	/** Reply the horizontal position of this Fig.
	 *
	 * @return horizontal position.
	 */
	int getX();

	/** Reply the vertical position of this Fig.
	     *
	     * @return vertical position.
	     */
	int getY();


	/** Reply <code>true</code> if the given Point
	 *  is in this Fig. This is used to determine  if the user
	 *  is trying to select this Fig.
	 *
	 * @param p the Point that may contained in this Fig.
	 * @return <code>true</code> if <code>p</code> is in this Fig,
	 *         <code>false</code> otherwise.
	 */
	boolean hit(Point p, Prefs prefs);

	/** Reply <code>true</code> if the object intersects the given rectangle. Used
	     *  for selective redrawing and by ModeSelect to select all Figs
	     *  that are partly within the selection rectangle.
	     *
	     *  @param r a rectangle
	     *  @return <code>true</code> if the figure intersects
	     *          the given rectangle. otherwise <code>false</code>
	     */
	boolean intersects(Rectangle r, Prefs prefs);

	/** Change the position and the dimension of this Fig.
	     *
	     * @param x horizontal position of this Fig.
	     * @param y vertical position of this Fig.
	     * @param width width of this Fig.
	     * @param height height of this Fig.
	     */
	void setBounds(int x, int y, int width, int height);

	/** Set the height of this Fig.
	     *
	     * @param height height.
	     */
	void setHeight(int height);

	/** Change the position of this Fig.
	     *
	     * @param x the new horizontal position of this Fig.
	     * @param y the new vertical position of this Fig.
	     */
	void setLocation(int x, int y);

	/** Set the width of this Fig.
	     *
	     * @param width width.
	     */
	void setWidth(int width);

	/** Move this Fig.
	     *
	     * @param dx horizontal movement dimension.
	     * @param dy vertical movement dimension.
	     */
	void translate(int dx, int dy);

	/** Reply <code>true</code> if the entire figure is contained within
	     *  the given Rectangle.
	     *  This can be used by ModeSelect to select Figs that are totally
	     *  within the selection rectangle.
	     *
	     *  @param r a rectangle
	     *  @return <code>true</code> if the entire figure is contained within
	     *          the given rectangle. otherwise <code>false</code>
	     */
	boolean within(Rectangle r);
}