package mygale.mcge.fig;
import java.awt.*;

public interface IFigEdge extends IFigLayeredObject {
	public static final int NO_SHADOW_MOVE = 0;
	public static final int BOTH_PORTS_SHADOW_MOVE = 1;
	public static final int SRC_PORTS_SHADOW_MOVE = 2;
	public static final int DST_PORTS_SHADOW_MOVE = 3;

	/**
	 * Initiate the shadowing drawing.
	 */
	void beginShadowMoveMode(int shadowMoveMode);

	/**
	 * Stop the shadowing drawing.
	 */
	void endShadowMoveMode(int transX, int transY);

	IFigNode getBeginFigNode();

	/** return the point of the path point edge defined
	     *  by the parameter.
	     *
	     * @param index the <var>index</var> of the point from
	     *              the path point edge to get.
	     *              0 corresponds to the sourcePortPoint
	     *              and size-1 (where "size" denote the
	     *              size of the path point list)
	     *              the destination port point.
	     * @return coordinates of the control point.
	     */
	Point getCtrlPointAt(int index);

	/** Get the edge control points.
	     *
	     * @return control points of this edge.
	     */
	java.util.Iterator getCtrlPoints();

	/** Return the number of point in the path of the edge.
	     *  this number est equal to 2 when the edge is a
	     *  {@link FigEdgeLinear FigEdgeLinear} :
	     *  only the port source point and the destination port point.
	     *
	     * @return an integer that is the size of the path.
	     */
	int getCtrlPointSize();

	IFigNode getEndFigNode();

	/**
	    * return the max number of control point for the edge :
	    *  <code>Interger.MAX_VALUE</code> for a poly edge.
	    *
	    * --------------------
	    * Return the max number of control point for the edge : 2 for
	    * a linear edge.
	    *
	    * --------------------
	    * return the max number of control point for the edge. uselly two
	    *  for a linear edge, and <code>Interger.MAX_VALUE</code>
	    *  for a ply edge.
	    *
	    * @return always <code>Interger.MAX_VALUE</code>.
	    * @return always 2
	    * @return an integer thatis the max count of control points.
	    */
	int getMaxAllowedCtrlPoints();

	/**
	 * Reply the bounds of the shadowed path.
	 *
	 * @return an rectangle that is the shadow's bounds.
	 * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	 */
	Rectangle getShadowBounds(int dx, int dy);

	/** Insert a point into the path defined by the given index.
	     *
	     * @param index the index of the control point in the path.
	     * @param pt coordinates of the control point.
	     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	     */
	void insertCtrlPointAt(int index, Point pt);

	/** Remove a point at the given index from the path.
	     *
	     * @param index the index of the control point in the path.
	     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	     */
	void removeCtrlPointAt(int index);

	void setBeginFigNode(IFigNode newSourcePort);

	/** Set the point of the path defined by the given index.
	     *
	     * @param index the index of the control point in the path.
	     * @param pt new coordinates of the control point.
	     */
	void setCtrlPointAt(int index, Point pt);

	void setCtrlPoints(java.util.Iterator it);

	void setEndFigPort(IFigNode newDestPort);

	/** Try to remove control points that are in the same location.
	     *
	     * @param undoCmds a reference to a undoing commands in which
	     *                 this method store the undo actions.
	     */
	boolean tryToRemoveTheCurrentCtrlPoint(Point ctrlPoint);

	/** This method update the location of ends.
	     *
	     * @throws mygale.mcge.throwable.MCGEInternalError something wrong !
	     */
	void updateEndsLocation();
}