package mygale.mcge.fig;
import java.awt.*;

import javax.swing.JPopupMenu;

import mygale.mcge.util.prefs.Prefs;

/** This class is the base class for basic drawing objects such as
 *  rectangles, lines, text, circles, etc.
 *  <p><center>
 *    <img src="doc-files/Fig_uml.gif" width="309" height="142" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="235,0,308,28"  href="../cmd/Cmd.html">
 *      <area shape="Rect" coords="118,0,190,28"  href="../mode/Mode.html">
 *      <area shape="Rect" coords="235,54,307,82"  href="../layer/LayerContener.html">
 *      <area shape="Rect" coords="0,54,73,82"  href="../util/prefs/Prefs.html">
 *      <area shape="Rect" coords="14,113,113,139"  href="FigModelElement.html">
 *      <area shape="Rect" coords="131,113,203,139"  href="FigText.html">
 *      <area shape="Rect" coords="221,113,293,139"  href="FigEllipse.html">
 *      <area shape="Rect" coords="0,0,73,28"  href="../core/JGraphEditor.html">
 *    </map>
 *  </center></p>
 *
 */
public interface IFigLayered extends IFig {

	/**
	 * This is the depth of the front figure.
	 */
	int DEPTH_FRONT = 0;

	/**
	 * Reshape to the east *
	 */
	int RESHAPE_EAST = 5;

	/**
	 * No reshape *
	 */
	int RESHAPE_NONE = 0;

	/**
	 * Reshape to the north *
	 */
	int RESHAPE_NORTH = 2;

	/**
	 * Reshape to the north east *
	 */
	int RESHAPE_NORTHEAST = 3;

	/**
	 * Reshape to the north west *
	 */
	int RESHAPE_NORTHWEST = 1;

	/**
	 * Reshape to the south *
	 */
	int RESHAPE_SOUTH = 7;

	/**
	 * Reshape to the south east *
	 */
	int RESHAPE_SOUTHEAST = 8;

	/**
	 * Reshape to the southwest *
	 */
	int RESHAPE_SOUTHWEST = 6;

	/**
	 * Reshape to the west *
	 */
	int RESHAPE_WEST = 4;

	void addChangeListener(javax.swing.event.ChangeListener changeListener);

	/**
	 * This method permits to edit the properties of this object and if it is
	 *  a {@link FigObject FigObject}, the properties of the attached
	 *  {@link mygale.mcge.net.NetObject NetObject}.
	 *  This method must be otherwrite by subclasses.
	 */
	void editProperties(Container parent);

	void fireStateChanged();

	/**
	 * Reply the minimal height of the figure.
	 */
	int getMinimalHeight();

	/**
	 * Reply the minimal width of the figure .
	 */
	int getMinimalWidth();

	/**
	 * This method return the popup for this Fig.
	 * @param forOnlyOneFig <code>true</code> if caller of this
	 *                      method need a popupmenu for one
	 *                      Fig, <code>false</code> if
	 *                      the caller need a popup menu for
	 *                      more than one Fig.
	 */
	JPopupMenu getPopupMenu(Container parent, boolean forOnlyOneFig);

	/**
	 * Reply the direction of the reshaping if the mouse is on a reshape area.
	 *
	 * @param x the horizontal position of the mouse.
	 * @param y the vertical position of the mouse.
	 * @return the reshape direction or {@link #RESHAPE_NONE #RESHAPE_NONE}.
	 */
	int getReshapeDirection(Prefs prefs, int x, int y);

	/**
	 * Reply the lock-state of this Fig. When a Fig is locked,
	 *  it should not be moved by the user.
	 */
	boolean isLocked();

	/**
	 * Reply <code>true</code> if this Fig can be
	 *  moved around by the user
	 */
	boolean isMovable();

	/**
	 * Reply <code>true</code> if this Fig can be
	 *  reshaped by the user
	 */
	boolean isReshapable();

	/**
	 * Reply the select-state of this Fig.
	 */
	boolean isSelected();

	////
	// Painting methods

	/** Method to paint this Fig. This method call
	 *  {@link #paintContent(Graphics) paintContent()} and
	 *  {@link #paintSelection(Graphics) paintSelection()}.
	 *
	 * @param g2d the graphic context.
	 */
	void paint(Graphics2D g2d, Prefs prefs);

	/**
	 * Method to paint the shadow of this Fig. By default
	 *  this method draw a rectangle corresponding to the bounds.
	 * @param g2d the graphic context.
	 * @param dx horizontal translation of this Fig.
	 * @param dy vertical translation of this Fig.
	 */
	void paintShadow(Graphics2D g2d, int dx, int dy, Prefs prefs);

	void removeChangeListener(javax.swing.event.ChangeListener changeListener);

	/**
	 * Set the lock-state of this Fig. When a Fig is locked,
	 *  it should not be moved by the user.
	 * @param lock <code>true</code> if this Fig is locked.
	 */
	void setLocked(boolean lock);

	/**
	 * Set the select-state of this Fig.
	 * @param select <code>true</code> if this Fig was selected.
	 */
	void setSelected(boolean select);
}