package mygale.mcge.fig.arrow;



import java.awt.Graphics2D;

import mygale.mcge.util.prefs.Prefs;

/** This class represents the arrows that will
 *  be paint in the ends of edges.
 *  This arrow is orawn as an "black circle".
 *  <p><center>
 *    <img src="doc-files/package_uml.gif" width="286" height="88" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="95,0,167,27"  href="Arrow.html">
 *      <area shape="Rect" coords="212,0,285,27"  href="../FigEdge.html">
 *      <area shape="Rect" coords="0,59,99,86"  href="ArrowSimpleArrow.html">
 *      <area shape="Rect" coords="207,59,279,86"  href="ArrowCircle.html">
 *      <area shape="Rect" coords="117,59,189,86"  href="ArrowDiamond.html">
 *    </map>
 *  </center></p>
 *
 */
public class ArrowCircle extends Arrow {

	/* This is the radius of the circle.
	 */
	protected int _radius = 5;



    /** Construct a new Arrow.
     */
    public ArrowCircle() {
    super();
    }

/** Construct a new Arrow.
     *
     * @param radius the radius if the circle.
     */
    public ArrowCircle(int radius) {
    super();
    _radius = radius;
    }

/** Reply the radius of the circle.
     *
     * @return an integer that is the radius of the circle.
     */
    protected int getRadius() { return _radius; }

/** Reply the height of the arrow, before the rotation.
     *
     * @return an integer that is the height of this arrow.
     */
public int getHeight() {
   return 2 * _radius;
}



/** Reply the width of the arrow, before the rotation.
 *
 * @return in integer that is the width of this arrow.
 */
public int getWidth() {
   return 2 * _radius;
}



/** Paint the arrow.
 *  the point (<var>x</var>,<var>y</var>) is the point where
 *  the arrow will be connect to a port.
 *  You must paint the arrow on the <b>right</b> of this
 *  point to be outside the node
 *  and to the <b>left</b> to be inside the node.
 *
 * @param g the graphical context in which this arrow must be painted.
 * @param x x coordinate of the "origin".
 * @param y y coordinate of the "origin".
 * @param prefs Prefs
 */
public void paintContent(Graphics2D g, int x, int y, Prefs prefs) {
   g.setColor(getFillColor(prefs));
   g.fillOval(x, y - _radius, 2 * _radius, 2 * _radius);
   g.setColor(getShapeColor(prefs));
   g.drawOval(x, y - _radius, 2 * _radius, 2 * _radius);
}
}
