package mygale.mcge.fig.arrow;



import java.awt.Graphics2D;

import mygale.mcge.util.prefs.Prefs;

/** This class represents the arrows that will
 *  be paint in the ends of edges.
 *  This arrow is orawn as an "black diamond".
 *  <p><center>
 *    <img src="doc-files/package_uml.gif" width="286" height="88" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="95,0,167,27"  href="Arrow.html">
 *      <area shape="Rect" coords="212,0,285,27"  href="../FigEdge.html">
 *      <area shape="Rect" coords="0,59,99,86"  href="ArrowSimpleArrow.html">
 *      <area shape="Rect" coords="207,59,279,86"  href="ArrowCircle.html">
 *      <area shape="Rect" coords="117,59,189,86"  href="ArrowDiamond.html">
 *    </map>
 *  </center></p>
 *
 */
public class ArrowDiamond extends Arrow {

	/* This is the size of arrow's the diamond.
	 */
	protected int _length = 10;



    /** Construct a new Arrow.
     */
    public ArrowDiamond() {
    super();
    }

/** Construct a new Arrow.
     *
     * @param length the size of the diamond.
     */
    public ArrowDiamond(int length) {
    super();
    _length = length;
    }

/** Reply the height of the arrow, before the rotation.
     *
     * @return an integer that is the height of this arrow.
     */
public int getHeight() {
   return _length;
}



/** Reply the width of the arrow, before the rotation.
 *
 * @return an integer that is the width of this arrow.
 */
public int getWidth() {
   return (int) Math.sqrt(7 * getHeight() * getHeight());
}



/** Paint the arrow.
 *  the point (0,0) is the point where the arrow will be connect to a port.
 *  You must paint the arrow on the <b>right</b> of this
 *  point to be outside the node
 *  and to the <b>left</b> to be inside the node.
 *
 * @param g the graphical context in which this arrow must be painted.
 * @param x x coordinate of the "origin".
 * @param y y coordinate of the "origin".
 * @param prefs Prefs
 */
public void paintContent(Graphics2D g, int x, int y, Prefs prefs) {
   int halfwidth = getWidth() / 2;
   int halfheight = getHeight() / 2;
   int xcoords[] = { x, x + halfwidth, x + getWidth(), x + halfwidth };
   int ycoords[] = { y, y - halfheight, y, y + halfheight };
   g.setColor(getFillColor(prefs));
   g.fillPolygon(xcoords, ycoords, 4);
   g.setColor(getShapeColor(prefs));
   g.drawPolygon(xcoords, ycoords, 4);
}
}
