package mygale.mcge.layer;



import java.awt.*;
import java.util.*;

import mygale.mcge.core.JGraph;
import mygale.mcge.fig.IFigLayered;

import de.uniwue.ki.util.collection.CollectionsHelper;

/** A LayerDiagram is a Layer which contains
 *  {@link mygale.mcge.fig.FigLayered FigLayered} objets,
 *  ordered from back to front. Each LayerDiagram contains
 *  part of the overall picture that the user is drawing.
 *  <p><center>
 *    <img src="doc-files/LayerDiagram_uml.gif" width="241" height="147" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="63,0,135,27"  href="Layer.html">
 *      <area shape="Rect" coords="167,0,241,27"  href="LayerContener.html">
 *      <area shape="Rect" coords="0,54,72,83"  href="../core/JGraph.html">
 *      <area shape="Rect" coords="117,117,189,145"  href="../fig/fig.html">
 *    </map>
 *  </center></p>
 *
 * @see mygale.mcge.fig.FigLayered
 */
public class LayerDiagram extends Layer
	implements LayerContainer {

	/** The {@link mygale.mcge.fig.FigLayered FigLayereds} that are contained
	 *  in this Layer.
	 */
	private Vector _contents = new Vector();

   public final javax.swing.event.ChangeListener figChangeListener = new javax.swing.event.ChangeListener() {
	   public void stateChanged(javax.swing.event.ChangeEvent changeEvent) {
		   damaged( ((IFigLayered) changeEvent.getSource()) );
	   }
   };



    /** Construct a new Layer.
     *
     * @param graph the graphe that displays this Layer.
     */
    public LayerDiagram(JGraph graph) { super( graph ); }

/** Get Layer content.
     *
     * @return The content of this Layer. * @see mygale.mcge.layer.LayerContener#getContents()
 */
    public Vector getContents() { return _contents; }



/** Add a {@link mygale.mcge.fig.FigLayered FigLayered} to this Layer.
 *
@link mygale.mcge.fig.FigLayered FigLayered} to add
 *               to this Layer.
 * @param figLayered the new { * @see mygale.mcge.layer.LayerContener#add(IFigLayered)
 */
public void add(IFigLayered figLayered) {
   if (_contents == null)
      _contents = new Vector();
   if (!_contents.contains(figLayered))
      _contents.addElement(figLayered);

   getGraph().changeWorkspaceSize(figLayered.getBounds());

   damaged(figLayered);
   figLayered.addChangeListener(figChangeListener);
}

/** Move a {@link mygale.mcge.fig.FigLayered FigLayered} by one step to the front of
     *  this Layer.
     *
    @link mygale.mcge.fig.FigLayered FigLayered}, which must be
     *               in the forward.
     * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#bringForward(IFigLayered)
 */
    public void bringForward(IFigLayered FigLayeredure) {
    int i = _contents.indexOf(FigLayeredure);
    if ( i== -1 || i == _contents.size()-1 ) return;
    IFigLayered next = (IFigLayered)_contents.elementAt(i + 1);
    _contents.setElementAt(next, i);
    _contents.setElementAt(FigLayeredure, i + 1);
    }

/** Put a list of {@link mygale.mcge.fig.figLayered figLayereds} in the front of this
     *  Layer. figLayereds will appear before all other figLayereds (but in the same order).
     *
    @link mygale.mcge.fig.figLayered figLayereds}, which must be in
     *                the front. * @param figLayeredures the { * @see mygale.mcge.layer.LayerContener#bringToFront(Iterator)
 */
public void bringToFront(Iterator figLayeredures) {
   while (figLayeredures.hasNext()) {
      IFigLayered fig = (IFigLayered) figLayeredures.next();
      int index = _contents.indexOf(fig);
      if (index > 0) {
         _contents.removeElementAt(index);
         _contents.addElement(fig);
      }
   }

   /*
   int fsize = figLayeredures.size();
   int moved = 0;
   for (int index = 0; index < _contents.size(); index++) {
      FigLayered figLayered = (FigLayered) _contents.elementAt(index);
      if (figLayeredures.contains(figLayered)) {
         _contents.removeElementAt(index);
         _contents.addElement(figLayered);
         index--;
         moved++;
         if (moved >= fsize)
            return;
      }
   }
   */
}

/** Put a {@link mygale.mcge.fig.FigLayered} in the front of this Layer.
     *  FigLayered will appears before all other FigLayereds.
     *
    @link mygale.mcge.fig.FigLayered FigLayered}, which must
     *               be in the front.
     * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#bringToFront(IFigLayered)
 */
    public void bringToFront(IFigLayered FigLayeredure) {
    _contents.removeElement(FigLayeredure);
    _contents.addElement(FigLayeredure);
    }

    /** This method inform {@link mygale.mcge.core.JGraph JGraphs} that a
     *  {@link mygale.mcge.fig.FigLayered FigLayered} area must be refresh.
     *
    @link mygale.mcge.fig.FigLayered FigLayered} that it must be
     *               refresh.
     * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#damaged(IFigLayered)
 */
    public void damaged(IFigLayered FigLayeredure) {
    if (_graph == null) return;
    _graph.damaged(FigLayeredure);
    }

/** A {@link mygale.mcge.fig.FigLayered FigLayered} in this Layer has been deleted.
     *  Notify all Editors so that they can deselect the fig.
     *
    @link mygale.mcge.fig.FigLayered FigLayered}, which is deleted. * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#deleted(IFigLayered)
 */
    public void deleted(IFigLayered FigLayeredure) {
    if (_graph == null) return;
    _graph.removed(FigLayeredure);
    }

/** Get {@link mygale.mcge.fig.FigLayered FigLayereds} of this Layer.
     *
     * @return The list of elements in this Layer.
     * @see mygale.mcge.layer.LayerContainer#getAllFigs()
 */
public Iterator getAllFigs() {
   return _contents.iterator();
}

/** Reply the depth level of the given {@link mygale.mcge.fig.FigLayered FigLayered}.
     *
     * @param FigLayered a reference to the FigLayeredure.
     * @return the depth level (0 for the Front, -1 for "not found" )
     * @see mygale.mcge.layer.LayerContainer#getDepthLevel(IFigLayered)
 */
    public int getDepthLevel(IFigLayered FigLayered) {
    int depth = _contents.indexOf( FigLayered );
    if ( depth <= 0 ) return _contents.size() - depth - 1;
    else return depth;
    }

/** Return the {@link mygale.mcge.fig.FigLayered FigLayered} under the mouse.
     *
     * @param p point that contains a {@link mygale.mcge.fig.FigLayered FigLayered}.
     * @return {@link mygale.mcge.fig.FigLayered FigLayered} in <var>p</var> or
     *         <code>null<code> if not found. */
public IFigLayered hit(Point p) {
   for (int i = _contents.size() - 1; i >= 0; i--) {
      IFigLayered f = (IFigLayered) _contents.elementAt(i);
      if (f.hit(p, getPrefs()))
         return f;
   }
   return null;
}

/** Remove a {@link mygale.mcge.fig.FigLayered FigLayered} from this Layer.
     *
    @link mygale.mcge.fig.FigLayered FigLayered} that may be
     *               removed from this Layer.
     * @param figLayered the { * @see mygale.mcge.layer.LayerContener#remove(IFigLayered)
 */
public void remove(IFigLayered figLayered) {
   if (_contents == null)
      return;
   _contents.removeElement(figLayered);

   figLayered.removeChangeListener(figChangeListener);
   damaged(figLayered);
}

/** Remove all {@link mygale.mcge.fig.FigLayered FigLayered} from this Layer.
     * @see mygale.mcge.layer.LayerContainer#removeAll()
 */
public void removeAll() {
   for (int i = _contents.size() - 1; i >= 0; i--) {
      IFigLayered f = (IFigLayered) _contents.elementAt(i);
      f.removeChangeListener(figChangeListener);
   }
   _contents.removeAllElements();
}

/** Move a {@link mygale.mcge.fig.FigLayered FigLayered} by one step to the back of
     *  this Layer.
     *
    @link mygale.mcge.fig.FigLayered FigLayered}, which must be
     *               move backward.
     * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#sendBackward(IFigLayered)
 */
    public void sendBackward(IFigLayered FigLayeredure) {
    int i = _contents.indexOf(FigLayeredure);
    if ( i== -1 || i == 0 ) return;
    IFigLayered previous = (IFigLayered)_contents.elementAt(i - 1);
    _contents.setElementAt(previous, i);
    _contents.setElementAt(FigLayeredure, i - 1);
    }

/** Put a list of {@link mygale.mcge.fig.figLayered figLayereds} to the
     *  back of this Layer. figLayereds will appear backward all other
     *  figLayereds (but in the same order).
     *
    @link mygale.mcge.fig.figLayered figLayereds}, which must
     *                be in the back.
     * @param figLayeredures the { * @see mygale.mcge.layer.LayerContener#sendToBack(Iterator)
 */
public void sendToBack(Iterator figLayeredures) {
   Iterator it = CollectionsHelper.reverse(figLayeredures);
   while (it.hasNext()) {
      IFigLayered fig = (IFigLayered) it.next();
      int index = _contents.indexOf(fig);
      if (index > 0) {
         _contents.removeElementAt(index);
         _contents.insertElementAt(fig, 0);
      }
   }

   /*
   int fsize = figLayeredures.size();
   int moved = 0;
   for (int index = _contents.size() - 1; index >= moved; index--) {
      FigLayered figLayered = (FigLayered) _contents.elementAt(index);
      if (figLayeredures.contains(figLayered)) {
         if (index != 0) {
            _contents.removeElementAt(index);
            _contents.insertElementAt(figLayered, 0);
            index++;
         }
         moved++;
         if (moved >= fsize)
            return;
      }
   }
   */
}

/** Put a {@link mygale.mcge.fig.FigLayered FigLayered} to the back of this Layer.
     *  FigLayered will appears backward all other FigLayereds.
     *
    @link mygale.mcge.fig.FigLayered FigLayered}, which must be
     *               in the back.
     * @param FigLayeredure the { * @see mygale.mcge.layer.LayerContener#sendToBack(IFigLayered)
 */
    public void sendToBack(IFigLayered FigLayeredure) {
    _contents.removeElement(FigLayeredure);
    _contents.insertElementAt(FigLayeredure, 0);
    }

/** Change the depth level of this fig.
     *  This level can be changed by move to the
     *  back or to the front the FigLayeredure.
     *
     * @param FigLayered a reference to the FigLayeredure.
     * @param depth the depth level (0 for the Front, 1 ... )
     * @see mygale.mcge.layer.LayerContainer#setDepthLevel(IFigLayered, int)
 */
    public void setDepthLevel(IFigLayered FigLayered, int depth) {
    if ( _contents.removeElement( FigLayered ) ) {
        _contents.insertElementAt( FigLayered, depth );
    }
    }


// Painting

/** Method to paint the contents of this Layer.
 *
@link java.awt.Graphics graphic context} in which
 *          the Layer must be drawed.
 * @param g2d { * @see Graphics
 */
public void paintContents(Graphics2D g2d) {
   Rectangle clip = g2d.getClipBounds();
   Iterator it = getAllFigs();
   while (it.hasNext()) {
      IFigLayered f = (IFigLayered) it.next();
      if (clip == null || f.intersects(clip, getPrefs())) {
         f.paint(g2d, getPrefs());

         /*
         if (f instanceof FigEdgeImpl)
             ((FigEdgeImpl) f).calcBounds();
         
         Rectangle rect = new Rectangle(f.getBounds());
         //rect.grow(-2, -2);
         g2d.draw(rect);
         */
      }
   }
}
}
