package mygale.mcge.mode;

import java.awt.event.KeyEvent;

import mygale.mcge.cmd.CmdDelete;

/** This class implements a Mode that permits to show
 *  the location of the mouse in real time.
 *  <p><center>
 *    <img src="doc-files/ModeMouseLocation_uml.gif" width="389" height="88" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="171,0,243,28"  href="Mode.html">
 *      <area shape="Rect" coords="0,59,100,86"  href="../event/mouse/MCGEMouseEvent.html">
 *      <area shape="Rect" coords="315,59,388,86"  href="../core/JGraph.html">
 *    </map>
 *  </center></p>
 *
 */
public class ModeDelete extends Mode {

	private final CmdDelete _cmdDelete;

	/** Construct a new ModeMouseLocation with the given parent.
	 *
	 * @param modeManager a reference to the ModeManager that
	 *                    contains this Mode.
	 */
	public ModeDelete(ModeManager modeManager) {
		super(modeManager);

		_cmdDelete = new CmdDelete(modeManager.getGraph().getGraphEditor(), 24);
	}

	///
	// Accessors

	/** Always <code>false</code> because I never want
	 *  to get out of popup mode.
	 *
	 * @return <code>true</code> if this mode can be dropped from
	 *         the {@link ModeManager ModeManager} stack,
	 *         otherwise <code>false</code>.
	 */
	public boolean canExit() {
		return false;
	}

	public void keyPressed(KeyEvent ke) {
		if (ke.getKeyCode() == KeyEvent.VK_DELETE) {
			delete();
			ke.consume();
		}
	}

	private void delete() {
		if (getGraph().getSelection().unlockedSize() > 0)
			_cmdDelete.doIt();
	}
}
