package mygale.mcge.mode;

import mygale.mcge.event.mouse.MCGEMouseEvent;
import mygale.mcge.fig.IFigLayered;
import mygale.mcge.sel.Selection;

/** This class implements a Mode that permits to edit the
 *  properties of a {@link Fig figure} when the user
 *  double-click on it.
 *  <p>
 *  If the user double-click on a figure, this mode calls the
 *  method {@link mygale.mcge.fig.Fig#editProperties() editProperties()}
 *  on the figure.
 *  <p><center>
 *    <img src="doc-files/ModeEditProperties_uml.gif" width="363" height="143" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="158,0,230,28"  href="Mode.html">
 *      <area shape="Rect" coords="0,59,100,86"  href="../event/mouse/MCGEMouseEvent.html">
 *      <area shape="Rect" coords="289,59,362,86"  href="../fig/Fig.html">
 *      <area shape="Rect" coords="77,114,176,141"  href="../core/JGraph.html">
 *      <area shape="Rect" coords="212,114,311,141"  href="../sel/Selection.html">
 *    </map>
 *  </center></p>
 *
 */
public class ModeEditProperties extends Mode {

	/** Construct a new ModeEditProperties. The
	     *  {@link ModeManager ModeManager} parent
	     *  should be set before using this object.
	     */
	public ModeEditProperties() {
	}

	/** Construct a new ModeEditProperties with the given parent.
	 *
	 * @param modeManager a reference to the ModeManager that
	 *                    contains this Mode.
	 */
	public ModeEditProperties(ModeManager modeManager) {
		super(modeManager);
	}

	/** Always <code>false</code> because I never want
	 *  to get out of this mode.
	 *
	@link ModeManager ModeManager} stack,
	 *         otherwise <code>false</code>.
	 * @return always <code>true</code> if this mode can be dropped
	 *         from the { */
	public boolean canExit() {
		return false;
	}

	/** Invoked when a mouse-double-clicked event was received.
	 *  <p>
	 *  If the user double-click on a {@link Fig figure}, this
	 *  mode run its property editor and select the edited
	 *  figure.
	 *
	 * @param me description of the mouse-event.
	 * @see mygale.mcge.fig.Fig#editProperties()
	 */
	public void mcgeMouseDoubleClicked(MCGEMouseEvent me) {
		if (!me.isOnlyLeftButtonDown()) {
			return;
		}

		IFigLayered fig = me.getFigure();

		if (fig != null) {
			Selection sl = getModeManager().getGraph().getSelection();
			sl.deselectAll();
			sl.select(fig);
			fig.editProperties(getModeManager().getGraph());
			me.consume();
		}
	}
}
