package mygale.mcge.mode;

import mygale.mcge.core.JGraph;
import mygale.mcge.event.mouse.MCGEMouseEvent;

/** This class implements a Mode that permits to show
 *  the location of the mouse in real time.
 *  <p><center>
 *    <img src="doc-files/ModeMouseLocation_uml.gif" width="389" height="88" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="171,0,243,28"  href="Mode.html">
 *      <area shape="Rect" coords="0,59,100,86"  href="../event/mouse/MCGEMouseEvent.html">
 *      <area shape="Rect" coords="315,59,388,86"  href="../core/JGraph.html">
 *    </map>
 *  </center></p>
 *
 */
public class ModeMouseLocation extends Mode {

	/** Construct a new ModeMouseLocation. The
	     *  {@link ModeManager ModeManager} parent
	     *  should be set before using this object.
	     */
	public ModeMouseLocation() {
	}

	/** Construct a new ModeMouseLocation with the given parent.
	 *
	 * @param modeManager a reference to the ModeManager that
	 *                    contains this Mode.
	 */
	public ModeMouseLocation(ModeManager modeManager) {
		super(modeManager);
	}

	//

	/** Always <code>false</code> because I never want
	 *  to get out of popup mode.
	 *
	@link ModeManager ModeManager} stack,
	 *         otherwise <code>false</code>.
	 * @return <code>true</code> if this mode can be dropped from
	 *         the { */
	public boolean canExit() {
		return false;
	}

	/** Invoked when a mouse-dragged event was received.
	     *
	     * @param me description of the mouse-event.
	     * @see mygale.mcge.event.mouse.MCGEMouseEventListener#mcgeMouseDragged(MCGEMouseEvent)
	 */
	public void mcgeMouseDragged(MCGEMouseEvent me) {
		showMousePosition(me.getX(), me.getY());
	}

	/** Invoked when a mouse-moved event was received.
	 *
	 * @param me description of the mouse-event.
	 */
	public void mcgeMouseMoved(MCGEMouseEvent me) {
		showMousePosition(me.getX(), me.getY());
	}

	/** Display the given location of the mouse.
	     *
	     * @param x the location of the mouse.
	     * @param y the location of the mouse.
	     */
	protected void showMousePosition(int x, int y) {
		JGraph graph = _modeManager.getGraph();
		/*
		Rectangle workspace = graph.getWorkspaceRect();
		if (workspace.contains(x, y)) {
		*/
		graph.setStatusMsg("(" + x + "," + y + ")");
		//}
	}
}
