package mygale.mcge.mode;

import mygale.mcge.core.*;
import mygale.mcge.undoredo.*;

/** This is the abstract superclass of all undoable editor modes.
 *  A undoable Mode is defined a smal API that permits to subclasses
 *  to manage undoable actions.
 *  <p><center>
 *    <img src="doc-files/ModeUndoable_uml.gif" width="471" height="201" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="199,0,271,28"  href="Mode.html">
 *      <area shape="Rect" coords="40,59,140,86"  href="../undoredo/UndoRedoManager.html">
 *      <area shape="Rect" coords="329,0,401,28"  href="../undoredo/UndoableCmd.html">
 *      <area shape="Rect" coords="329,59,401,86"  href="../undoredo/RedoableCmd.html">
 *      <area shape="Rect" coords="0,118,100,145"  href="ModeCreateFig.html">
 *      <area shape="Rect" coords="253,118,352,145"  href="ModeCreateEdge.html">
 *      <area shape="Rect" coords="167,172,302,200"  href="ModeModifyEdgeCtrlPoint.html">
 *      <area shape="Rect" coords="118,118,217,145"  href="ModeMove.html">
 *      <area shape="Rect" coords="370,118,470,145"  href="ModeReshape.html">
 *    </map>
 *  </center></p>
 *
 */
public abstract class ModeUndoable extends Mode {

	/** Construct a new Mode.
	     */
	public ModeUndoable() {
		super();
	}

	/** Construct a new ModeUndoable.
	 *
	 * @param modeManager a reference to the ModeManager that
	 *                    contains this Mode.
	 */
	public ModeUndoable(ModeManager modeManager) {
		super(modeManager);
	}

	/** Reply the current undo/redo manager.
	 *
	 * @return a reference to the current undo/redo manager
	 *         or <code>null</code> if none.
	 */
	public UndoRedoManager getUndoRedoManager() {
		if (_modeManager != null) {
			JGraph graph = _modeManager.getGraph();
			if (graph != null) {
				JGraphEditor editor = graph.getGraphEditor();
				if (editor != null) {
					return editor.getUndoRedoManager();
				}
			}
		}
		return null;
	}

	// Undo/redo API

	/** Add an undoable command into the undo list.
	 *
	 * @param cmd a reference to the new command.
	 */
	protected void addUndoCmd(UndoableCmd cmd) {
		UndoRedoManager urm = getUndoRedoManager();
		if (urm != null) {
			urm.addUndo(cmd);
		}
	}

	/** Remove all existing commands from the
	     *  undo/redo list.
	     */
	protected void clearUndoList() {
		UndoRedoManager urm = getUndoRedoManager();
		if (urm != null) {
			urm.clearUndo();
		}
	}
}
