package mygale.mcge.net;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.uniwue.ki.util.event.StateChangeMulticaster;

/** This is the semantic object that composes a graph model.
 *  <p><center>
 *    <img src="doc-files/NetObjectImpl_uml.gif" width="353" height="151" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="149,0,221,28"  href="NetObject.html">
 *      <area shape="Rect" coords="270,59,341,86"  href="../fig/FigObject.html">
 *      <area shape="Rect" coords="0,122,99,149"  href="NetGraphModelImpl.html">
 *      <area shape="Rect" coords="194,122,267,149"  href="NetNodeImpl.html">
 *      <area shape="Rect" coords="279,122,352,149"  href="NetPortImpl.html">
 *      <area shape="Rect" coords="108,122,180,149"  href="NetEdgeImpl.html">
 *    </map>
 *  </center></p>
 *
 */
public abstract class AbstractNetObject implements INetObject {

	private transient ChangeListener _changeListener = null;

	public AbstractNetObject() {
	}

	public void addChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.add(_changeListener, newListener);
	}

	/** This method decides if the node can be deleted.
	 * @return boolean
	 * @see mygale.mcge.net.INetObject#canBeRemoved()
	 */
	public boolean canBeRemoved() {
		return true;
	}

	protected void fireStateChanged() {
		if (_changeListener != null) {
			_changeListener.stateChanged(new ChangeEvent(this));
		}
	}

	public void removeChangeListener(ChangeListener newListener) {
		_changeListener = StateChangeMulticaster.remove(_changeListener, newListener);
	}
}