package mygale.mcge.net;

import junit.framework.Assert;

/** Abstract {@link mygale.mcge.net.NetObject NetObject}
 *  classe for representing edges between
 *  {@link mygale.mcge.net.NetPort ports}.
 *  <p><center>
 *    <img src="doc-files/NetEdgeImpl_uml.gif" width="299" height="147" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="59,0,131,28"  href="NetObjectImpl.html">
 *      <area shape="Rect" coords="189,59,297,86"  href="NetEdge.html">
 *      <area shape="Rect" coords="0,118,72,145"  href="NetPortImpl.html">
 *      <area shape="Rect" coords="112,118,221,145"  href="NetGraphModelImpl.html">
 *    </map>
 *  </center></p>
 *
 */
public class DefaultNetEdge extends AbstractNetObject implements INetEdge {

	/** The begin connection port.
	 */
	protected INetNode _beginNode = null;

	/** The end connection port.
	 */
	protected INetNode _endNode = null;

	/** Contruct a new NetEdgeImpl.
	 */
	public DefaultNetEdge() {
	}

	/** Reply the begin
	     *  {@link mygale.mcge.net.NetPort connection port}.
	     *
	    @link mygale.mcge.net.NetPort connection port}.
	     * @return { * @see mygale.mcge.net.INetEdge#getBeginPort()
	 */
	public INetNode getBeginNode() {
		return _beginNode;
	}

	/** Reply the end
	     *  {@link mygale.mcge.net.NetPort connection port}.
	     *
	     * @return {@link mygale.mcge.net.NetPort connection port}.
	     */
	public INetNode getEndNode() {
		return _endNode;
	}

	/** This method is called when the node was removed from the model.
	 * @see mygale.mcge.net.INetEdge#removed()
	*/
	public void removed() {
	}

	/** Set the begin
	     *    {@link mygale.mcge.net.NetPort connection newPort}.
	     *
	    @link mygale.mcge.net.NetPort connection newPort}.
	     * @param newPort { * @see mygale.mcge.net.INetEdge#setBeginPort(INetPort)
	 */
	public void setBeginNode(INetNode newNode) {
		if (Assert.ASSERTION_ON)
			Assert.assertNotNull(newNode);

		_beginNode = newNode;
	}

	/** Set the begin
	     *    {@link mygale.mcge.net.NetPort connection newPort}.
	     *
	    @link mygale.mcge.net.NetPort connection newPort}.
	     * @param newPort { * @see mygale.mcge.net.INetEdge#setEndPort(INetPort)
	 */
	public void setEndNode(INetNode newNode) {
		Assert.assertNotNull(newNode);

		_endNode = newNode;
	}
}