package mygale.mcge.net;

import java.util.Iterator;

import de.uniwue.ki.util.collection.CollectionsHelper;
import de.uniwue.ki.util.functions.IPredicateFunction;

/**
 * This is the semantic object for nodes.
 * <p>
 * <center><img src="doc-files/NetNodeImpl_uml.gif" width="304" height="246" border="0" usemap="#umlclick">
 * <map name="umlclick"><area shape="Rect" coords="72,0,145,28"  href="NetObjectImpl.html">
 * <area shape="Rect" coords="203,0,302,28"  href="NetGraphModelImpl.html">
 * <area shape="Rect" coords="203,59,275,86"  href="NetNode.html">
 * <area shape="Rect" coords="145,118,217,145"  href="NetEdge.html">
 * <area shape="Rect" coords="14,118,86,145"  href="NetPort.html">
 * <area shape="Rect" coords="63,163,163,190"  href="NetNodeMonoPort.html">
 * <area shape="Rect" coords="0,217,100,244"  href="NetNodeDefault.html">
 * <area shape="Rect" coords="118,217,217,244"  href="NetNodeEllipse.html"></map></center>
 * </p>
 *  
 */
public class DefaultNetNode extends AbstractNetObject implements INetNode {
    private final INetGraphModel _graphModel;

    public DefaultNetNode(INetGraphModel graphModel) {
        _graphModel = graphModel;
    }

    public Iterator getIncomingNodeEdges() {
        Iterator incomingAndOutgoingEdges = getNodeEdges();
        return CollectionsHelper.select(incomingAndOutgoingEdges, new IPredicateFunction() {
            public boolean isElementOk(Object anObject) {
                return ((INetEdge) anObject).getEndNode() == DefaultNetNode.this;
            }
        });
    }

    public Iterator getOutgoingNodeEdges() {
        Iterator incomingAndOutgoingEdges = getNodeEdges();
        return CollectionsHelper.select(incomingAndOutgoingEdges, new IPredicateFunction() {
            public boolean isElementOk(Object anObject) {
                return ((INetEdge) anObject).getBeginNode() == DefaultNetNode.this;
            }
        });
    }

    public Iterator getNodeEdges() {
        return CollectionsHelper.select(getGraphModel().getEdges(), new IPredicateFunction() {
            public boolean isElementOk(Object obj) throws Exception {
                INetEdge netEdge = (INetEdge) obj;
                return (netEdge.getBeginNode() == DefaultNetNode.this)
                    || (netEdge.getEndNode() == DefaultNetNode.this);
            }
        });
    }

    public boolean canConnect(INetEdge edge) {
        return true;
    }

    public boolean canConnect(INetEdge edge, INetNode otherNode) {
        return true;
    }

    //HACK was:private -> FigUMLGraphReflectiveEdge.getPositionOfThisReflectiveEdge()
    public INetGraphModel getGraphModel() {
        return _graphModel;
    }
}
