package mygale.mcge.net;

import java.util.Iterator;

/**
 * This is the semantic object for graph models.
 * <p>
 * <center>
 * <img src="doc-files/NetGraphModel_uml.gif" width="304" height="147" border="0" usemap="#umlclick">
 * <map name="umlclick"><area shape="Rect" coords="64,0,136,28"  href="NetObject.html">
 * <area shape="Rect" coords="212,0,284,28"  href="../core/JGraph.html">
 * <area shape="Rect" coords="194,59,302,86"  href="NetGraphModelImpl.html">
 * <area shape="Rect" coords="0,118,73,146"  href="NetNode.html">
 * <area shape="Rect" coords="136,118,209,146"  href="NetEdge.html"></map></center>
 * </p>
 *  
 */
public interface INetGraphModel extends INetObject {

    /**
     * Add a {@link mygale.mcge.net.NetEdge node}in this NetGraphModel
     * 
     * @param edge
     *            the new {@link mygale.mcge.net.NetEdge NetEdge}.
     */
    void addEdge(INetEdge edge);

    void addNetGraphModelChangedListener(NetGraphModelChangedListener newListener);

    /**
     * Add a {@link mygale.mcge.net.NetNode node}in this NetGraphModel.
     * 
     * @param node
     *            the new {@link mygale.mcge.net.NetNode NetNode}.
     */
    void addNode(INetNode node);

    /**
     * Get the edge's list of this NetGraphModel.
     * 
     * @return a list of {@link mygale.mcge.net.NetEdge NetEdges}.
     */
    Iterator getEdges();

    /**
     * Get the node's list of this NetGraphModel
     * 
     * @return a list of {@link mygale.mcge.net.NetNode NetNodes}.
     */
    Iterator getNodes();

    /**
     * Remove a edge attached to this NetGraphModel.
     * 
     * @param port
     *            the {@link mygale.mcge.net.NetEdge NetEdge}to remove.
     */
    void removeEdge(INetEdge edge);

    /**
     * Remove all edges connected to this particular node. Needed for RubberGraphDeclarationWithNetModel
     * --steffen
     */
    void removeEdgesForNode(INetNode node);

    void removeNetGraphModelChangedListener(NetGraphModelChangedListener newListener);

    /**
     * Remove a node attached to this NetGraphModel
     * 
     * @param port
     *            the {@link mygale.mcge.net.NetNode NetNode}to remove.
     */
    void removeNode(INetNode node);

  
}