package mygale.mcge.net;

import java.util.Iterator;

/**
 * This is the semantic object for nodes.
 * <p>
 * <center><img src="doc-files/NetNode_uml.gif" width="358" height="147" border="0" usemap="#umlclick">
 * <map name="umlclick"><area shape="Rect" coords="0,59,73,86"  href="../core/JGraph.html">
 * <area shape="Rect" coords="118,0,190,28"  href="NetObject.html">
 * <area shape="Rect" coords="248,0,338,28"  href="NetGraphModel.html">
 * <area shape="Rect" coords="248,59,356,86"  href="NetNodeImpl.html">
 * <area shape="Rect" coords="59,118,131,146"  href="NetPort.html">
 * <area shape="Rect" coords="190,118,262,146"  href="NetEdge.html"></map></center>
 * </p>
 *  
 */
public interface INetNode extends INetObject {

    /**
     * Reply all {@link mygale.mcge.net.NetEdge NetEdges}attached to this NetNode by the way of
     * {@link mygale.mcge.net.NetPort NetPorts}.
     * 
     * @return a set of {@link mygale.mcge.net.NetEdge NetEdges}.
     */
    Iterator getNodeEdges();

    Iterator getIncomingNodeEdges();

    Iterator getOutgoingNodeEdges();

    // +++++++++++++++++++++++++++++++++++++

    /**
     * Reply if an Port can be the begining of an NetEdge. By default all Edges can be connected to this
     * Port. Subclasses must override this method to have a more restrictive managing.
     * 
     * @param edge
     *            the classname of the Edge to connect.
     * @return <code>true</code> if this Port can be connected to the given Edge.
     */
    boolean canConnect(INetEdge edge);

    /**
     * Reply if an Port can be the ending of an NetEdge. By default all Edges can be connected to this Port.
     * Subclasses must override this method to have a more restrictive managing.
     * 
     * @param edge
     *            the classname of the Edge to connect.
     * @param beginport
     *            the NetPort in the other side of the given Edge.
     * @return <code>true</code> if this Port can be connected to the given Edge.
     */
    boolean canConnect(INetEdge edge, INetNode otherNode);
}