package mygale.mcge.net;

import java.awt.AWTEventMulticaster;
import java.util.EventListener;

/**
 * This is the event multicaster class to support the mygale.mcge.net.NetGraphModelChangedListenerinterface.
 */
public class NetGraphModelChangedEventMulticaster
    extends AWTEventMulticaster
    implements NetGraphModelChangedListener {

    /**
     * Add new listener to support multicast events.
     * 
     * @param a
     *            mygale.mcge.net.NetGraphModelChangedListener
     * @param b
     *            mygale.mcge.net.NetGraphModelChangedListener
     * @return mygale.mcge.net.NetGraphModelChangedListener
     */
    public static NetGraphModelChangedListener add(
        NetGraphModelChangedListener a,
        NetGraphModelChangedListener b) {
        return (NetGraphModelChangedListener) addInternal(a, b);
    }

    /**
     * Remove listener to support multicast events.
     * 
     * @param l
     *            mygale.mcge.net.NetGraphModelChangedListener
     * @param oldl
     *            mygale.mcge.net.NetGraphModelChangedListener
     * @return mygale.mcge.net.NetGraphModelChangedListener
     */
    public static NetGraphModelChangedListener remove(
        NetGraphModelChangedListener l,
        NetGraphModelChangedListener oldl) {
        if (l == oldl || l == null)
            return null;
        if (l instanceof NetGraphModelChangedEventMulticaster)
            return (NetGraphModelChangedListener)
                ((NetGraphModelChangedEventMulticaster) l).remove(
                oldl);
        return l;
    }

    /**
     * @param event
     *            mygale.mcge.net.NetGraphModelChangedEvent
     * @see mygale.mcge.net.NetGraphModelChangedListener#addedEdge(NetGraphModelChangedEvent)
     */
    public void addedEdge(NetGraphModelChangedEvent event) {
        ((NetGraphModelChangedListener) a).addedEdge(event);
        ((NetGraphModelChangedListener) b).addedEdge(event);
    }

    /**
     * @param event
     *            mygale.mcge.net.NetGraphModelChangedEvent
     */
    public void addedNode(NetGraphModelChangedEvent event) {
        ((NetGraphModelChangedListener) a).addedNode(event);
        ((NetGraphModelChangedListener) b).addedNode(event);
    }

    /**
     * @param event
     *            mygale.mcge.net.NetGraphModelChangedEvent
     * @see mygale.mcge.net.NetGraphModelChangedListener#removedEdge(NetGraphModelChangedEvent)
     */
    public void removedEdge(NetGraphModelChangedEvent event) {
        ((NetGraphModelChangedListener) a).removedEdge(event);
        ((NetGraphModelChangedListener) b).removedEdge(event);
    }

    /**
     * @param event
     *            mygale.mcge.net.NetGraphModelChangedEvent
     * @see mygale.mcge.net.NetGraphModelChangedListener#removedNode(NetGraphModelChangedEvent)
     */
    public void removedNode(NetGraphModelChangedEvent event) {
        ((NetGraphModelChangedListener) a).removedNode(event);
        ((NetGraphModelChangedListener) b).removedNode(event);
    }

    /**
     * Constructor to support multicast events.
     * 
     * @param a
     *            java.util.EventListener
     * @param b
     *            java.util.EventListener
     */
    protected NetGraphModelChangedEventMulticaster(EventListener a, EventListener b) {
        super(a, b);
    }

    /**
     * Add new listener to support multicast events.
     * 
     * @return java.util.EventListener
     * @param a
     *            java.util.EventListener
     * @param b
     *            java.util.EventListener
     */
    protected static EventListener addInternal(
        EventListener a,
        EventListener b) {
        if (a == null)
            return b;
        if (b == null)
            return a;
        return new NetGraphModelChangedEventMulticaster(a, b);
    }

    /**
     * @return java.util.EventListener
     * @param oldl
     *            mygale.mcge.net.NetGraphModelChangedListener
     */
    protected EventListener remove(NetGraphModelChangedListener oldl) {
        if (oldl == a)
            return b;
        if (oldl == b)
            return a;
        EventListener a2 = removeInternal(a, oldl);
        EventListener b2 = removeInternal(b, oldl);
        if (a2 == a && b2 == b)
            return this;
        return addInternal(a2, b2);
    }
}
