package mygale.mcge.sel;
import java.awt.Rectangle;
import java.util.Iterator;

import mygale.mcge.cmd.Cmd;
import mygale.mcge.fig.IFigLayered;

/**
 * Insert the type's description here.
 * Creation date: (04.05.2001 14:42:21)
 
 */
public interface Selection {

   /** Add a {@link mygale.mcge.cmd.Cmd Cmd} to the Inform list.
    *
    * @param cmd a instance that this Selection must
    *            inform of changes.
    */
   void addCmdToInform(Cmd cmd);

   /** Add a new figure to the collection of
        *  {@link mygale.mcge.fig.FigLayered Figs}.
        *
        * @pazam s the new figure.
        */
   void addFig(IFigLayered s);

   /** Return <code>true</code> if the selection contains the
        *  given figure.
        */
   boolean contains(IFigLayered fig);

   /** Unselect a {@link mygale.mcge.fig.FigLayered figure}.
        *
        * @param figure the figure to unselect.
        */
   void deselect(IFigLayered figure);

   /** Unselect all the {@link mygale.mcge.fig.FigLayered Figs}.
        */
   void deselectAll();

   /** Get the {@link mygale.mcge.fig.FigLayered Figs} that are
        *  managed by this SelectionManager.
        *
        * @return a list of selections.
        */
   Iterator getAllFigs();

   /** This method return the rectangle that contains all the
    *  selected {@link mygale.mcge.fig.FigLayered FigLayered}.
    *
    * @return the bound of the selection.
    */
   Rectangle getBounds();

   /** Get the locked selections that is managed by this
        *  SelectionManager.
        *
        * @return a list of {@link mygale.mcge.fig.FigLayered figs}.
        */
   Iterator getLockedFigs();

   /** Get the unlocked selections that are managed by
        *  this SelectionManager.
        *
        * @return a list of {@link mygale.mcge.fig.FigLayered figs}.
        */
   Iterator getUnlockedFigs();

   /** This method inform all {@link mygale.mcge.cmd.Cmd Cmd}
        *  about a change in this selection.
        */
   void informCmd();

   /** Reply the number of selected
        *  {@link mygale.mcge.fig.FigLayered FigLayered's} that
        *  are locked.
        *
        * @return the number of locked figures.
        */
   int lockedSize();

   /** Remove a figure from the collection of
    *  {@link mygale.mcge.fig.FigLayered Figs}.
    *
    * @pazam s the figure to remove.
    */
   void removeFig(IFigLayered s);

   /** Select a set of {@link mygale.mcge.fig.FigLayered figures}.
        *
        * @param figs the vector of figures to select.
        */
   void select(Iterator figs);

   /** Select a {@link mygale.mcge.fig.FigLayered figure}.
   *
   * @param figure the {@link mygale.mcge.fig.FigLayered FigLayered}
   *               to select.
   */

   void select(IFigLayered figure);

   /** Enable/Disable the Cmd's information.
        *
        * @param inform <code>true</code> if this Selection must
        *               inform {@link mygale.mcge.cmd.Cmd Cmds}
        *               of any change, otherwise <code>false</code>.
        * @return the old value of the Inform flag.
        */
   boolean setCmdInform(boolean inform);

   /** Reply the number of selected
        *  {@link mygale.mcge.fig.FigLayered FigLayered's}.
        *
        * @return the size of the selection.
        */
   int size();

   /** Toggle a set of {@link mygale.mcge.fig.FigLayered FigLayered}.
        *
        * @param figs the vector of {@link mygale.mcge.fig.FigLayered FigLayered} to toggle.
        */
   void toggle(Iterator figs);

   /** Toggle the selection of a {@link mygale.mcge.fig.FigLayered FigLayered}.
        *
        * @param figure the {@link mygale.mcge.fig.FigLayered FigLayered} to toggle.
        */
   void toggle(IFigLayered figure);

   /** Reply the number of selected
        *  {@link mygale.mcge.fig.FigLayered FigLayered's} that
        *  are unlocked.
        *
        * @return the number of unlocked figures.
        */
   int unlockedSize();
}