package mygale.mcge.undoredo;



import java.util.Vector;

/** This class permits to store a set of Actions that
 *  will be undo/redo in one step.
 *  <p>
 *  The order of commands in the set should be important.
 *  The class CommandSet undoes the commands from the
 *  <b>last</b> added to the <b>first</b> one; and
 *  redoes the commands from the <b>first</b> to
 *  the <b>last</b>.
 *  <p><center>
 *    <img src="doc-files/CommandSet_uml.gif" width="191" height="165" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="0,0,72,28"  href="UndoableCmd.html">
 *      <area shape="Rect" coords="116,0,189,28"  href="RedoableCmd.html">
 *      <area shape="Rect" coords="46,136,144,163"  href="UndoRedoableCmd.html">
 *    </map>
 *  </center></p>
 *
 */
public class CommandSet
	implements UndoableCmd, RedoableCmd {

	/** This is the set of commands that will be undo/redo
	 *  when this CommandSet was undoed/redoed.
	 */
	protected Vector _commands = new Vector();


    /** Construct a new CommandSet.
     */
    public CommandSet() { }

/** Add an command to this CommandSet.
     *  We suppose that the given vector
     *  contains a set of undiable commands.
     *
     * @param set a set of undoable commands.
     */
    public void add(Vector set) {
    _commands.addAll( set );
    }


    /** Add an command to this CommandSet.
     *
     * @param cmd the command to add to this set.
     */
    public void add(UndoRedoableCmd cmd) {
    _commands.addElement( cmd );
    }

/** This method was called by the redo command.
     *  It must redo something.
     * @see mygale.mcge.undoredo.RedoableCmd#redoIt()
 */
    public void redoIt() {
    for( int i=0; i<_commands.size(); i++) {
        Object cmd = _commands.elementAt(i);
        if (cmd instanceof RedoableCmd) {
        ((RedoableCmd)cmd).redoIt();
        }
    }
    }

    /** This method was called by the undo command.
     *  It must redo something.
     * @see mygale.mcge.undoredo.UndoableCmd#undoIt()
 */
    public void undoIt() {
    for( int i=_commands.size()-1; i>=0; i--) {
        Object cmd = _commands.elementAt(i);
        if (cmd instanceof UndoableCmd) {
        ((UndoableCmd)cmd).undoIt();
        }
    }
    }
}
