package mygale.mcge.undoredo;


/** This is the manager that permits to store undo/redo commands.
 *  <p><center>
 *    <img src="doc-files/UndoRedoManager_uml.gif" width="367" height="227" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="0,0,99,28"  href="UndoRedoListener.html">
 *      <area shape="Rect" coords="131,0,203,28"  href="../core/JGraphEditor.html">
 *      <area shape="Rect" coords="248,50,365,77"  href="UndoRedoManagerImpl.html">
 *      <area shape="Rect" coords="63,140,162,167"  href="UndoRedoableCmd.html">
 *      <area shape="Rect" coords="8,198,81,226"  href="UndoableCmd.html">
 *      <area shape="Rect" coords="149,198,221,226"  href="RedoableCmd.html">
 *    </map>
 *  </center></p>
 *
 */
public interface UndoRedoManager {


   /** Add a command to the undo list.
    *
    * @param cmd the new undoable command.
    */
   void addUndo(UndoableCmd cmd);

   /** Clear the redo list.
        */
   void clearRedo();

   /** Clear the undo list.
        */
   void clearUndo();

   /** Redo the first command in the list.
        */
   void redo();


   /** Reply the count of redoable command in the list.
    *
    * @return an integer that is the number of redoable commands.
    */
   int redoSize();



   /** Add an UndoRedoListener into this Manager.
    *
    * @param listener the new listener.
    */
   void register(UndoRedoListener listener);

   /** Undo the first command in the list.
        */
   void undo();

   /** Reply the count of undoable command in the list.
        *
        * @return an integer that is the number of undoable commands.
        */
   int undoSize();

   /** Remove un UndoRedoListener into this Manager.
        *
        * @param listener the listener to remove from this manager.
        */
   void unregister(UndoRedoListener listener);
}