package mygale.mcge.util;

import java.awt.Point;

/** This is a utility class.

 */
public class MCGEGlobals {

	/** No public constructor, this is a utility class with all static
	     *  methods, so you never need an instance.
	     */
	private MCGEGlobals() {
	}

	/** Compute the distance between 2 points.
	 *
	 * @param x1 horizontal position of the first point.
	 * @param y1 vertical position of the first point.
	 * @param x2 horizontal position of the second point.
	 * @param y2 vertical position of the second point.
	 * @return the distance between given points.
	 */
	public static final double distance(int x1, int y1, int x2, int y2) {
		double dx, dy;
		dx = x1 - x2;
		dy = y1 - y2;
		return Math.sqrt(dx * dx + dy * dy);
	}

	/** Compute the distance between 2 points.
	     *
	     * @param p1 the first point.
	     * @param p2 the second point.
	     * @return the distance between given points.
	     */
	public static final double distance(Point p1, Point p2) {
		return distance(p1.x, p1.y, p2.x, p2.y);
	}

	/** Return the trofinometric angle of a vector.
	     *  The vector is from the first point to the
	     *  second point.
	     *  The point (<var>x1</var>,<var>y1</var>) is the
	     *  origin of the trigonometric space.
	     *  The resulting angle is normalized with
	     *  {@link #normalizeAngle(double) normalizeAngle()}.
	     *
	     * @param x1 x coordinate of the first point.
	     * @param y1 y coordinate of the first point.
	     * @param x2 x coordinate of the second point.
	     * @param y2 y coordinate of the seoond point.
	     * @return the troginometric angle in radians of the given vector,
	     *         or <code>Math.NaN</code> if an error occurs.
	     */
	public static final double getAngleFromVector(int x1, int y1, int x2, int y2) {
		double vector_x = x2 - x1;
		double vector_y = y2 - y1;
		double norme = Math.sqrt(vector_x * vector_x + vector_y * vector_y);
		double angle = Double.NaN;
		if (norme != 0.0) {
			vector_y = vector_y / norme;
			angle = Math.asin(vector_y);
			if (x2 < x1)
				angle = Math.PI - angle;
		}
		return normalizeAngle(angle);
	}

	/** Return the trofinometric angle of a vector.
	     *  The vector is from the first point to the
	     *  second point.
	     *  The point <var>p1</var> is the
	     *  origin of the trigonometric space.
	     *  The resulting angle is normalized with
	     *  {@link #normalizeAngle(double) normalizeAngle()}.
	     *
	     * @param p1 first point.
	     * @param p2 Point
	 * @return the troginometric angle in radians of the given vector,
	     *         or <code>Math.NaN</code> if an error occurs.
	     */
	public static final double getAngleFromVector(Point p1, Point p2) {
		return getAngleFromVector(p1.x, p1.y, p2.x, p2.y);
	}

	/** Normalize an angle.
	     *  Translate the given angle (in radians) to force it
	     *  to be between 0 and 2*PI.
	     *
	     * @param angle the angle to normalize.
	     * @return an angle (in radians) that is normalized.
	     */
	public static final double normalizeAngle(double angle) {
		if (angle >= 0.0) {
			while (angle >= (2.0 * Math.PI)) {
				angle -= 2.0 * Math.PI;
			}
		} else {
			while (angle < 0) {
				angle += 2.0 * Math.PI;
			}
		}
		return angle;
	}
}