package mygale.mcge.util.prefs;



/** This class describes an preference-changed event.
 *  <p><center>
 *    <img src="doc-files/PrefChangedEvent_uml.gif" width="250" height="97" border="0" usemap="#umlclick">
 *    <map name="umlclick">
 *      <area shape="Rect" coords="18,0,90,28"  href="Prefs.html">
 *      <area shape="Rect" coords="140,0,248,28"  href="PrefChangedListener.html">
 *    </map>
 *  </center></p>
 *
 */
public class PrefChangedEvent {

	/** This is a GUI event.
	 */
	public static int GUI = 0;

	/** This is the name of the preference that was changed.
	 */
	public final String prefName;

	/** This is the type of the event.
	 */
	public final int prefType;

	/** This is the new value of this pref.
	 */
	public final Object newValue;

	/** This is the old value of this pref.
	 */
	public final Object oldValue;

	/** This is indicate that this event was consumed.
	 */
	protected boolean _consumed=false;

//
    // Constructor

    /** Contruct a new PrefChangedEvent.
     *
     * @param prefname this is the name of the changed preference.
     * @param type this is the type of the preference.
     * @param newvalue the new value of the preference.
     * @param oldvalie the old value of the preference.
     */
    public PrefChangedEvent(String prefname, int type, Object newvalue, Object oldvalue) {
    prefName = prefname;
    prefType = type;
    newValue = newvalue;
    oldValue = oldvalue;
    }



    /** Reply the consumption state of this event.
     *
     * @return <code>true</code> if this event was consumed,
     *         <code>false</code> otherwise.
     */
    public boolean isConsumed() { return _consumed; }

/** Set the consumption state of this event.
     */
    public void consume() { _consumed = true; }
}
